package org.processmining.processprofiler3d.compviz;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohort;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;

public class DataMapProviderEdgeQuality 
{
	private PetriNetDataSource pnet;
	private IPerformanceStatisticsEnrichedNet pwait;
	private HashSet<String> visClassifiers;

	public void setVisibleClassifiers(HashSet<String> vc)
	{
		visClassifiers = vc;
	}
	
	public DataMapProviderEdgeQuality(PetriNetDataSource pnet, DataQualityDBInterface dqdi)
	{
		this.pnet = pnet;
		this.pwait = dqdi;

		//make all available classifiers visible at first
		visClassifiers = pwait.getClassifierValues();
	}

	public void setCohortClassifier(Vector<ClassificationCohort> cohorts)
	{
//		for (ClassificationCohort ccs : cohorts)
//		{
//			Vector<PlaceAlignmentInfo> paiLstx = PlaceAlignmentCalculator.Analyse(pnet, mylog, myinitmarking, myfinalmarking, ccs);
//			paiLst.put(ccs.getName(), paiLstx);
//		}
		if (pwait instanceof DataQualityDBInterface)
		{
			DataQualityDBInterface dqdi = (DataQualityDBInterface)pwait;
			dqdi.setCohortClassifier(cohorts);
		}
	}
	
	public TransitionDataMapDouble getMap(String label1, DataMapNormalization mode)
	{
		TransitionDataMapDouble map = new TransitionDataMapDouble();
		DataQualityDBInterface db = (DataQualityDBInterface)pwait;
		HashMap<String, Double> hm = db.getLabelValueGroups(label1);
		Set<String> transitions = db.getTransitionsFast();
		HashSet<String> classifiers = db.getClassifierValuesFast();	
		
		if (visClassifiers.isEmpty())
		{
			System.out.println("No visible classifiers in DataMapProviderEdge!");
		}
		
		if (mode == DataMapNormalization.LocalSum) //local sum
		{
			for (String localID : transitions)
			{
				Double max = (double)GetSumPerEdge(localID, hm, classifiers, true);
				
				for (String classifier : classifiers)
				{
					if (visClassifiers.contains(classifier))
					{
						String idx = localID + "-" + classifier;
						if (hm.containsKey(idx))
						{
							Long valL = hm.get(idx).longValue();
							if (valL == Long.MIN_VALUE)
							{
								System.out.println("DataMapProviderEdge: MinValue");
							}
							if (valL == Long.MAX_VALUE)
							{
								System.out.println("DataMapProviderEdge: MaxValue");
							}
							
							Double val = hm.get(idx).doubleValue();
							map.setValue(localID, classifier, val);
							map.setMin(localID, classifier, 0.0);
							map.setMax(localID, classifier, max);
						}
					}
				}
			}
		}
		else if (mode == DataMapNormalization.LocalMax) //local max
		{
			for (String localID : transitions)
			{
				Double min = (double)GetMinPerEdge(localID, hm, classifiers, true);
				Double max = (double)GetMaxPerEdge(localID, hm, classifiers, true);
				
				for (String classifier : classifiers)
				{
					if (visClassifiers.contains(classifier))
					{
						String idx = localID + "-" + classifier;
						if (hm.containsKey(idx))
						{
							Long valL = hm.get(idx).longValue();
							if (valL == Long.MIN_VALUE)
							{
								System.out.println("DataMapProviderEdge: MinValue");
							}
							if (valL == Long.MAX_VALUE)
							{
								System.out.println("DataMapProviderEdge: MaxValue");
							}
							
							Double val = hm.get(idx).doubleValue();
							map.setValue(localID, classifier, val);
							map.setMin(localID, classifier, min);
							map.setMax(localID, classifier, max);
//							map.setMin(localID, classifier, 0.0);
//							map.setMax(localID, classifier, 1.0);
						}
					}
				}
			}
		}
		else if (mode == DataMapNormalization.SubnetMax) //subnet max
		{
			HashMap<String, INetDataSource> hm2 = new HashMap<String, INetDataSource>();
			for (String localID : transitions)
			{
				hm2.put(localID, getSubnetForTransitionPair(localID));
			}
			for (String localID : transitions)
			{
				INetDataSource nds = hm2.get(localID);
				Double min = (double)GetMinPerSubnet(hm, transitions, classifiers, true, nds, hm2);
				Double max = (double)GetMaxPerSubnet(hm, transitions, classifiers, true, nds, hm2);
				
				for (String classifier : classifiers)
				{
					if (visClassifiers.contains(classifier))
					{
						String idx = localID + "-" + classifier;
						if (hm.containsKey(idx))
						{
							Long valL = hm.get(idx).longValue();
							if (valL == Long.MIN_VALUE)
							{
								System.out.println("DataMapProviderEdge: MinValue");
							}
							if (valL == Long.MAX_VALUE)
							{
								System.out.println("DataMapProviderEdge: MaxValue");
							}
							
							Double val = hm.get(idx).doubleValue();
							map.setValue(localID, classifier, val);
							map.setMin(localID, classifier, min);
							map.setMax(localID, classifier, max);
						}
					}
				}
			}
		}
		else if (mode == DataMapNormalization.GlobalMax) //global max
		{
			Double min = (double)GetMinPerModel(hm, transitions, classifiers, true);
			Double max = (double)GetMaxPerModel(hm, transitions, classifiers, true);
			
			for (String localID : transitions)
			{
				for (String classifier : classifiers)
				{
					if (visClassifiers.contains(classifier))
					{
						String idx = localID + "-" + classifier;
						if (hm.containsKey(idx))
						{
							Long valL = hm.get(idx).longValue();
							if (valL == Long.MIN_VALUE)
							{
								System.out.println("DataMapProviderEdge: MinValue");
							}
							if (valL == Long.MAX_VALUE)
							{
								System.out.println("DataMapProviderEdge: MaxValue");
							}
							
							Double val = hm.get(idx).doubleValue();
							map.setValue(localID, classifier, val);
							map.setMin(localID, classifier, min);
							map.setMax(localID, classifier, max);
						}
					}
				}
			}
		}
		
		return map;
	}
	
	public TransitionDataMapDouble getMap(String label1, String focusclassifier, DataMapNormalization mode)
	{
		TransitionDataMapDouble map = new TransitionDataMapDouble();
		DataQualityDBInterface db = (DataQualityDBInterface)pwait;
		HashMap<String, Double> hm = db.getLabelValueGroups(label1);
		Set<String> transitions = db.getTransitionsFast();
		HashSet<String> classifiers = db.getClassifierValuesFast();	

		if (visClassifiers.isEmpty())
		{
			System.out.println("No visible classifiers in DataMapProviderEdge!");
		}
		
		if (mode == DataMapNormalization.LocalSum) //local sum
		{
			//not a valid mode
		}
		else if (mode == DataMapNormalization.LocalMax) //local max
		{
			for (String localID : transitions)
			{
				String fidx = localID + "-" + focusclassifier;
				if (hm.containsKey(fidx))
				{
					Double focusVal = hm.get(fidx);
					Double diff = (double)GetMaxDiffPerEdge(localID, hm, classifiers, focusclassifier, true);
					Double min = focusVal - diff;
					Double max = focusVal + diff;
					
					for (String classifier : classifiers)
					{
						if (visClassifiers.contains(classifier))
						{
							String idx = localID + "-" + classifier;
							if (hm.containsKey(idx))
							{
								Long valL = hm.get(idx).longValue();
								if (valL == Long.MIN_VALUE)
								{
									System.out.println("DataMapProviderEdge: MinValue");
								}
								if (valL == Long.MAX_VALUE)
								{
									System.out.println("DataMapProviderEdge: MaxValue");
								}
								
								Double val = hm.get(idx).doubleValue();
		
								//Double res = (val.doubleValue() - min.doubleValue()) / (max.doubleValue() - min.doubleValue());
								//System.out.println("min/val/max: " + min + "/" + val + "/" + max + " - " + res);
								
								map.setValue(localID, classifier, val);
								map.setMin(localID, classifier, min);
								map.setMax(localID, classifier, max);
							}
						}
					}
				}
			}
		}
		else if (mode == DataMapNormalization.SubnetMax) //subnet max
		{
			HashMap<String, INetDataSource> hm2 = new HashMap<String, INetDataSource>();
			for (String localID : transitions)
			{
				hm2.put(localID, getSubnetForTransitionPair(localID));
			}
			for (String localID : transitions)
			{
				String fidx = localID + "-" + focusclassifier;
				if (hm.containsKey(fidx))
				{
					INetDataSource nds = hm2.get(localID);
					Double focusVal = hm.get(fidx);
					Double diff = (double)GetMaxDiffPerSubnet(hm, transitions, classifiers, focusclassifier, true, nds, hm2);
					Double min = focusVal - diff;
					Double max = focusVal + diff;
					
					for (String classifier : classifiers)
					{
						if (visClassifiers.contains(classifier))
						{
							String idx = localID + "-" + classifier;
							if (hm.containsKey(idx))
							{
								Long valL = hm.get(idx).longValue();
								if (valL == Long.MIN_VALUE)
								{
									System.out.println("DataMapProviderEdge: MinValue");
								}
								if (valL == Long.MAX_VALUE)
								{
									System.out.println("DataMapProviderEdge: MaxValue");
								}
								
								Double val = hm.get(idx).doubleValue();
		
								//Double res = (val.doubleValue() - min.doubleValue()) / (max.doubleValue() - min.doubleValue());
								//System.out.println("min/val/max: " + min + "/" + val + "/" + max + " - " + res);
								
								map.setValue(localID, classifier, val);
								map.setMin(localID, classifier, min);
								map.setMax(localID, classifier, max);
							}
						}
					}
				}
			}
		}
		else if (mode == DataMapNormalization.GlobalMax) //global max
		{			
			Double diff = (double)GetMaxDiffPerModel(hm, transitions, classifiers, focusclassifier, true);
			
			for (String localID : transitions)
			{
				String fidx = localID + "-" + focusclassifier;
				if (hm.containsKey(fidx))
				{
					Double focusVal = hm.get(fidx).doubleValue();
					Double min = focusVal - diff;
					Double max = focusVal + diff;
					
					for (String classifier : classifiers)
					{
						if (visClassifiers.contains(classifier))
						{
							String idx = localID + "-" + classifier;
							if (hm.containsKey(idx))
							{
								Long valL = hm.get(idx).longValue();
								if (valL == Long.MIN_VALUE)
								{
									System.out.println("DataMapProviderEdge: MinValue");
								}
								if (valL == Long.MAX_VALUE)
								{
									System.out.println("DataMapProviderEdge: MaxValue");
								}
								
								Double val = hm.get(idx).doubleValue();
								
								//Double res = (val.doubleValue() - min.doubleValue()) / (max.doubleValue() - min.doubleValue());
								//System.out.println("min/val/max: " + min + "/" + val + "/" + max + " - " + res);
								
								map.setValue(localID, classifier, val);
								map.setMin(localID, classifier, min);
								map.setMax(localID, classifier, max);
							}
						}
					}
				}
			}
		}
		
		return map;
	}
	
	private Double GetMaxDiffPerEdge(String localID, HashMap<String, Double> hm, Set<String> classifiers, String focusClassifier, boolean bExcludeInvis)
	{
		String fidx = localID + "-" + focusClassifier;
		Double focusVal = hm.get(fidx).doubleValue();
		
		Double max = (double)0;
		for (String classifier : classifiers)
		{
			if ((!bExcludeInvis) || visClassifiers.contains(classifier))
			{
				String idx = localID + "-" + classifier;
				if (hm.containsKey(idx))
				{
					Double val1 = hm.get(idx).doubleValue();
					Double nval = Math.abs(focusVal - val1);
					
					max = Math.max(max, nval);
				}
			}
		}
		return max;
	}
	
	private Double GetMaxDiffPerSubnet(HashMap<String, Double> hm, Set<String> transitions, Set<String> classifiers, String focusClassifier, boolean bExcludeInvis, INetDataSource nds, HashMap<String, INetDataSource> hm2)
	{
		Double max1 = (double) 0;
		for (String localID : transitions) 
		{
			INetDataSource snds = hm2.get(localID);
			if (snds == nds)
			{
				String fidx = localID + "-" + focusClassifier;
				if (hm.containsKey(fidx))
				{
					Double val1 = GetMaxDiffPerEdge(localID, hm, classifiers, focusClassifier, bExcludeInvis);
					
					max1 = Math.max(max1, val1);
				}
			}
		}
		return max1;
	}
	
	private Double GetMaxDiffPerModel(HashMap<String, Double> hm, Set<String> transitions, Set<String> classifiers, String focusClassifier, boolean bExcludeInvis)
	{
		Double max1 = (double) 0;
		for (String localID : transitions) 
		{
			String fidx = localID + "-" + focusClassifier;
			if (hm.containsKey(fidx))
			{
				Double val1 = GetMaxDiffPerEdge(localID, hm, classifiers, focusClassifier, bExcludeInvis);
				
				max1 = Math.max(max1, val1);
			}
		}
		return max1;
	}
	
	private Double GetSumPerEdge(String localID, HashMap<String, Double> hm, Set<String> classifiers, boolean bExcludeInvis)
	{
		Double sum1 = (double)0;
		for (String classifier : classifiers)
		{
			if ((!bExcludeInvis) || visClassifiers.contains(classifier))
			{
				String idx = localID + "-" + classifier;
				if (hm.containsKey(idx))
				{
					Double val1 = hm.get(idx).doubleValue();
					
					sum1 = sum1 + val1;
				}
			}
		}
		return sum1;
	}
	
	private Double GetMaxPerEdge(String localID, HashMap<String, Double> hm, Set<String> classifiers, boolean bExcludeInvis)
	{
		Double max1 = (double) 0;
		for (String classifier : classifiers)
		{		
			if ((!bExcludeInvis) || visClassifiers.contains(classifier))
			{
				String idx = localID + "-" + classifier;
				if (hm.containsKey(idx))
				{
					Double val1 = hm.get(idx).doubleValue();;
					
					max1 = Math.max(max1, val1);
				}
			}
		}
		return max1;
	}
	
	private Double GetMinPerEdge(String localID, HashMap<String, Double> hm, Set<String> classifiers, boolean bExcludeInvis)
	{
		Double min1 = Double.MAX_VALUE;
		for (String classifier : classifiers)
		{		
			if ((!bExcludeInvis) || visClassifiers.contains(classifier))
			{
				String idx = localID + "-" + classifier;
				if (hm.containsKey(idx))
				{
					Double val1 = hm.get(idx).doubleValue();
					
					min1 = Math.min(min1, val1);
				}
			}
		}
		return min1;
	}
	
	private Double GetMaxPerSubnet(HashMap<String, Double> hm, Set<String> transitions, Set<String> classifiers, boolean bExcludeInvis, INetDataSource nds, HashMap<String, INetDataSource> hm2)
	{
		Double max1 = (double) 0;
		for (String localID : transitions) 
		{
			INetDataSource snds = hm2.get(localID);
			if (snds == nds)
			{
				Double val1 = GetMaxPerEdge(localID, hm, classifiers, bExcludeInvis);
				
				max1 = Math.max(max1, val1);
			}
		}
		return max1;
	}
	
	private Double GetMinPerSubnet(HashMap<String, Double> hm, Set<String> transitions, Set<String> classifiers, boolean bExcludeInvis, INetDataSource nds, HashMap<String, INetDataSource> hm2)
	{
		Double min1 = Double.MAX_VALUE;
		for (String localID : transitions) 
		{
			INetDataSource snds = hm2.get(localID);
			if (snds == nds)
			{
				Double val1 = GetMinPerEdge(localID, hm, classifiers, bExcludeInvis);
				
				min1 = Math.min(min1, val1);
			}
		}
		return min1;
	}
	
	private Double GetMaxPerModel(HashMap<String, Double> hm, Set<String> transitions, Set<String> classifiers, boolean bExcludeInvis)
	{
		Double max1 = (double) 0;
		for (String localID : transitions) 
		{
			Double val1 = GetMaxPerEdge(localID, hm, classifiers, bExcludeInvis);
			
			max1 = Math.max(max1, val1);
		}
		return max1;
	}
	
	private Double GetMinPerModel(HashMap<String, Double> hm, Set<String> transitions, Set<String> classifiers, boolean bExcludeInvis)
	{
		Double min1 = Double.MAX_VALUE;
		for (String localID : transitions) 
		{
			Double val1 = GetMinPerEdge(localID, hm, classifiers, bExcludeInvis);
			
			min1 = Math.min(min1, val1);
		}
		return min1;
	}
	
	private INetDataSource getSubnetForTransitionPair(String tId)
	{
		String tIdStart = TransitionPair.getSourceId(tId);
		String tIdEnd = TransitionPair.getTargetId(tId);
		boolean hasStart = false;
		boolean hasEnd = false;
		
		for (Transition t : pnet.getTransitions())
		{
			if (t.getLocalID().toString().equals(tIdStart))
			{
				hasStart = true;
			}
			if (t.getLocalID().toString().equals(tIdEnd))
			{
				hasEnd = true;
			}
			if (hasStart && hasEnd)
			{
				return pnet;
			}
		}
		
		for (SubNetDataSource snds : pnet.getSubnets())
		{
			SubNetDataSource snr = getSubnetForTransitionPairHelper(snds, tIdStart, tIdEnd);
			if (snr != null)
			{
				return snr;
			}
		}
		return null;
	}
	
	private SubNetDataSource getSubnetForTransitionPairHelper(SubNetDataSource snds, String tIdStart, String tIdEnd)
	{
		boolean hasStart = false;
		boolean hasEnd = false;
		
		for (Transition t : snds.getTransitions())
		{
			if (t.getLocalID().toString().equals(tIdStart))
			{
				hasStart = true;
			}
			if (t.getLocalID().toString().equals(tIdEnd))
			{
				hasEnd = true;
			}
			if (hasStart && hasEnd)
			{
				return snds;
			}
		}
		
		for (SubNetDataSource sndsc : snds.getSubnets())
		{
			SubNetDataSource snr = getSubnetForTransitionPairHelper(sndsc, tIdStart, tIdEnd);
			if (snr != null)
			{
				return snr;
			}
		}
		return null;
	}
}
