package org.processmining.processprofiler3d.compviz;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PerformanceStatisticsDB;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;

public class DataMapProviderTransition 
{
	private PetriNetDataSource pnet;
	private IPerformanceStatisticsEnrichedNet psnet;
	private HashSet<String> visClassifiers;
	
	public void setVisibleClassifiers(HashSet<String> vc)
	{
		visClassifiers = vc;
	}
	
	public DataMapProviderTransition(PetriNetDataSource pnet, IPerformanceStatisticsEnrichedNet psnet)
	{
		this.pnet = pnet;
		this.psnet = psnet;
		
		//make all available classifiers visible at first
		visClassifiers = psnet.getClassifierValues();
	}
	
	public TransitionDataMapDouble getMap(String label1, DataMapNormalization mode)
	{
		Date startTime = new Date();
		TransitionDataMapDouble map = new TransitionDataMapDouble();
		PerformanceStatisticsDB db = (PerformanceStatisticsDB)psnet;
		HashMap<String, Double> hm = db.getLabelValueGroups(label1);
		Set<String> transitions = db.getTransitionsFast();
		HashSet<String> classifiers = db.getClassifierValuesFast();
		
		if (mode == DataMapNormalization.LocalSum) //local sum
		{
			for (String tId : transitions)
			{
				//List<double[]> classes = new ArrayList<double[]>();
				
				double max = GetSumPerTransition(tId, hm, classifiers, true);
				
				for (String classifier : classifiers)
				{
					if (visClassifiers.contains(classifier))
					{
						//FillClassValues(label1, tId, classes, classifier);
						
						String idx = tId + "-" + classifier;
						if (hm.containsKey(idx))
						{
							double val = hm.get(idx);
							map.setValue(tId, classifier, val);
							map.setMin(tId, classifier, 0.0);
							map.setMax(tId, classifier, max);
						}
					}
				}
				
				//CalcSignificance(label1, tId, classes);
			}
		}
		else if (mode == DataMapNormalization.LocalMax) //local max
		{
			for (String tId : transitions)
			{
				//List<double[]> classes = new ArrayList<double[]>();
				double max = GetMaxPerTransition(tId, hm, classifiers, true);
				
				for (String classifier : classifiers)
				{
					if (visClassifiers.contains(classifier))
					{
						//FillClassValues(label1, tId, classes, classifier);
						
						String idx = tId + "-" + classifier;
						if (hm.containsKey(idx))
						{
							double val = hm.get(idx);
							map.setValue(tId, classifier, val);
							map.setMin(tId, classifier, 0.0);
							map.setMax(tId, classifier, max);
						}
					}
				}
				
				//CalcSignificance(label1, tId, classes);
			}
		}
		else if (mode == DataMapNormalization.SubnetMax) //local max
		{
			for (String tId : transitions)
			{
				//List<double[]> classes = new ArrayList<double[]>();
				
				INetDataSource snds = getSubnetForTransition(tId);
				if (snds != null)
				{
					double max = GetMaxPerSubnet(hm, classifiers, true, snds);
					
					for (String classifier : classifiers)
					{
						if (visClassifiers.contains(classifier))
						{
							//FillClassValues(label1, tId, classes, classifier);
							
							String idx = tId + "-" + classifier;
							if (hm.containsKey(idx))
							{
								double val = hm.get(idx);
								map.setValue(tId, classifier, val);
								map.setMin(tId, classifier, 0.0);
								map.setMax(tId, classifier, max);
							}
						}
					}
				}
				
				//CalcSignificance(label1, tId, classes);
			}
		}
		else if (mode == DataMapNormalization.GlobalMax) //global max
		{
			double max = GetMaxPerModel(hm, transitions, classifiers, true);
			
			for (String tId : transitions)
			{
				//List<double[]> classes = new ArrayList<double[]>();
				
				for (String classifier : classifiers)
				{
					if (visClassifiers.contains(classifier))
					{
						//FillClassValues(label1, tId, classes, classifier);
						
						String idx = tId + "-" + classifier;
						if (hm.containsKey(idx))
						{
							double val = hm.get(idx);
							map.setValue(tId, classifier, val);
							map.setMin(tId, classifier, 0.0);
							map.setMax(tId, classifier, max);
						}
					}
				}
				
				//CalcSignificance(label1, tId, classes);
			}
		}
		else if (mode == DataMapNormalization.One) //one
		{
			for (String tId : transitions)
			{
				//List<double[]> classes = new ArrayList<double[]>();
				
				double max = GetNumPerTransition(tId, hm, classifiers, true);
				
				for (String classifier : classifiers)
				{
					if (visClassifiers.contains(classifier))
					{
						//FillClassValues(label1, tId, classes, classifier);
						
						String idx = tId + "-" + classifier;
						if (hm.containsKey(idx))
						{
							//double val = hm.get(idx);
							map.setValue(tId, classifier, 1.0);
							map.setMin(tId, classifier, 0.0);
							map.setMax(tId, classifier, max);
						}
					}
				}
				
				//CalcSignificance(label1, tId, classes);
			}
		}
		
		Date endTime = new Date();
		TimeSpan ts = new TimeSpan(startTime, endTime);
		
		System.out.println("DataMap generated (" + ts.ToString(TimeUnit.MILLISECONDS) + "ms).");
		
		return map;
	}

//	public void FillClassValues(String label1, String tId, List<double[]> classes, String classifier) 
//	{
//		Vector<Double> valList = psnet.getValues(tId, label1, classifier);
//		double[] valArray = new double[valList.size()];
//		int i = 0;
//		for (Double d : valList)
//		{
//			valArray[i] = d.doubleValue();
//			i++;
//		}
//		classes.add(valArray);
//	}
//
//	public void CalcSignificance(String label1, String tId, List<double[]> classes) 
//	{
//		try
//		{
//			boolean bSigDiff = TestUtils.oneWayAnovaTest(classes, 0.01); // true means reject null hypothesis
//			//if (bSigDiff)
//			//{
//				//String tName = getTransitionName(pnet, tId);
//				//System.out.println("Significant difference: " + tName + " - " + label1);
//			//}
//		}
//		catch (Exception ex)
//		{
//			
//		}
//	}
	
//	private String getTransitionName(INetDataSource nds, String tId)
//	{
//		for (Transition t : nds.getTransitions())
//		{
//			if (t.getLocalID().toString().equals(tId))
//			{
//				return t.getLabel();
//			}
//			
//			for (SubNetDataSource snds : nds.getSubnets())
//			{
//				String ts = getTransitionName(snds, tId);
//				if (!ts.equals(tId))
//				{
//					return ts;
//				}
//			}
//		}
//		
//		return tId;
//	}
	
	private double GetSumPerTransition(String localID, HashMap<String, Double> hm, Set<String> classifiers, boolean bExcludeInvis)
	{	
		double sum1 = 0.0;
		for (String classifier : classifiers)
		{
			if ((!bExcludeInvis) || visClassifiers.contains(classifier))
			{
				String idx = localID + "-" + classifier;
				if (hm.containsKey(idx))
				{
					double val1 = hm.get(idx);
					
					sum1 = sum1 + val1;
				}
			}
		}
		return sum1;
	}
	
	private double GetNumPerTransition(String localID, HashMap<String, Double> hm, Set<String> classifiers, boolean bExcludeInvis)
	{
		double sum1 = 0.0;
		for (String classifier : classifiers)
		{
			if ((!bExcludeInvis) || visClassifiers.contains(classifier))
			{
				String idx = localID + "-" + classifier;
				if (hm.containsKey(idx))
				{
					sum1 = sum1 + 1.0;
				}
			}
		}
		return sum1;
	}
	
	private double GetMaxPerTransition(String tp, HashMap<String, Double> hm, HashSet<String> classifiers, boolean bExcludeInvis)
	{
		double max1 = 0.0;
		for (String classifier : classifiers)
		{		
			if ((!bExcludeInvis) || visClassifiers.contains(classifier))
			{
				String idx = tp + "-" + classifier;
				if (hm.containsKey(idx))
				{
					double val1 = hm.get(idx);
				
					max1 = Math.max(max1, val1);
				}
			}
		}
		return max1;
	}
	
	private double GetMaxPerSubnet(HashMap<String, Double> hm, HashSet<String> classifiers, boolean bExcludeInvis, INetDataSource snet)
	{
		double max1 = 0.0;
		for (Transition t : snet.getTransitions()) 
		{
			double val1 = GetMaxPerTransition(t.getLocalID().toString(), hm, classifiers, bExcludeInvis);
			
			max1 = Math.max(max1, val1);
		}
		return max1;
	}
	
	private double GetMaxPerModel(HashMap<String, Double> hm, Set<String> transitions, HashSet<String> classifiers, boolean bExcludeInvis)
	{
		double max1 = 0.0;
		for (String t : transitions) 
		{
			double val1 = GetMaxPerTransition(t, hm, classifiers, bExcludeInvis);
			
			max1 = Math.max(max1, val1);
		}
		return max1;
	}
	
	private INetDataSource getSubnetForTransition(String tId)
	{
		for (Transition t : pnet.getTransitions())
		{
			if (t.getLocalID().toString().equals(tId))
			{
				return pnet;
			}
		}
		
		for (SubNetDataSource snds : pnet.getSubnets())
		{
			SubNetDataSource snr = getSubnetForTransitionHelper(snds, tId);
			if (snr != null)
			{
				return snr;
			}
		}
		return null;
	}
	
	private SubNetDataSource getSubnetForTransitionHelper(SubNetDataSource snds, String tId)
	{
		for (Transition t : snds.getTransitions())
		{
			if (t.getLocalID().toString().equals(tId))
			{
				return snds;
			}
		}
		
		for (SubNetDataSource sndsc : snds.getSubnets())
		{
			SubNetDataSource snr = getSubnetForTransitionHelper(sndsc, tId);
			if (snr != null)
			{
				return snr;
			}
		}
		return null;
	}
}
