package org.processmining.processprofiler3d.compviz;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.Vector;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.placealignment.PlaceAlignmentCalculator;
import org.processmining.processprofiler3d.placealignment.PlaceAlignmentInfo;
import org.processmining.processprofiler3d.placealignment.PlaceAlignmentInfoDetail;
import org.processmining.processprofiler3d.psen.AlignedLog;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohort;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PSENLabel;

public class DataQualityDBInterface implements IPerformanceStatisticsEnrichedNet
{
	private PetriNetDataSource pnet;
	
	private Vector<PSENLabel> labels = new Vector<PSENLabel>();
	
	private XLog mylog = null;
	private Marking myinitmarking = null;
	private Marking myfinalmarking = null;
	
	private HashMap<String, HashMap<String, PlaceAlignmentInfoDetail>> paiLst = new HashMap<String, HashMap<String, PlaceAlignmentInfoDetail>>();

	private int paidNum = 0;
	
	public DataQualityDBInterface(PetriNetDataSource pnet, XLog log)
	{
		this.pnet = pnet;

		LogAnalyser la = new LogAnalyser(log);
		//if log is already annotated with alignment information, no need to align log again for replay
		if (AlignedLog.CheckLogIsAligned(la))
		{
			mylog = log;

			Place startPlace = pnet.getStartPlace();
			Place endPlace = pnet.getStartPlace();
			
			//construct initial marking for now...
			if (startPlace != null)
			{
				myinitmarking = new Marking();
				myinitmarking.add(startPlace);
			}

			//construct final marking for now...
			if (endPlace != null)
			{
				myfinalmarking = new Marking();
				myfinalmarking.add(endPlace);
			}

//			labels.add(new PSENLabel("PEndZero", "PEndZero", PSENLabel.COUNT, false));
//			labels.add(new PSENLabel("PEndPos", "PEndPos", PSENLabel.COUNT, false));
//			labels.add(new PSENLabel("PEndNeg", "PEndNeg", PSENLabel.COUNT, false));
//			labels.add(new PSENLabel("POrder", "POrder", PSENLabel.COUNT, false));
			if (startPlace != null && endPlace != null)
			{
				labels.add(new PSENLabel("Support", "Support", PSENLabel.COUNT, false));
				labels.add(new PSENLabel("Confidence", "Confidence", PSENLabel.COUNT, false));
			}
		}
	}

	public void setCohortClassifier(Vector<ClassificationCohort> cohorts)
	{
		int num = 0;
		for (ClassificationCohort ccs : cohorts)
		{
			Vector<PlaceAlignmentInfo> paiLstx = PlaceAlignmentCalculator.Analyse(pnet, mylog, myinitmarking, myfinalmarking, ccs);
			HashMap<String, PlaceAlignmentInfoDetail> lstd = new HashMap<String, PlaceAlignmentInfoDetail>();
			for (PlaceAlignmentInfo pai : paiLstx)
			{
				for (PlaceAlignmentInfoDetail paid : pai.detailPais.values())
				{
					String pairId = paid.getTransitionPairId();
					if (pairId != null)
					{
						if (paid.pairNum > 0)
						{
							lstd.put(pairId, paid);
							num++;
						}
					}
				}
			}
			
			paiLst.put(ccs.getName(), lstd);
		}
		paidNum = num * labels.size();
	}

	public void update()
	{
		// TODO Auto-generated method stub
		
	}

	public Collection<PSENLabel> getLabels()
	{
		Vector<PSENLabel> lst = new Vector<PSENLabel>(labels);
	
		//Collections.sort(lst);
		
		return lst;
	}

	public HashSet<String> getClassifierValues()
	{
		if (paiLst != null)
		{
			return new HashSet<String>(paiLst.keySet());
		}
		return new HashSet<String>();
	}

	public HashSet<String> getClassifierValuesFast()
	{
		return getClassifierValues();
	}

	public Set<String> getTransitions()
	{
		HashSet<String> lst = new HashSet<String>();
		if (paiLst != null)
		{
			for (String key : paiLst.keySet())
			{
				HashMap<String, PlaceAlignmentInfoDetail> hm = paiLst.get(key);
				for (String transition : hm.keySet())
				{
					if (!lst.contains(transition))
					{
						lst.add(transition);
					}
				}
			}
		}
		return lst;
	}

	public Set<String> getTransitionsFast()
	{
		return getTransitions();
	}

	public Set<String> getTransitionsInclNone()
	{
		HashSet<String> lst = new HashSet<String>();
		if (paiLst != null)
		{
			for (String key : paiLst.keySet())
			{
				for (String paidId : paiLst.get(key).keySet())
				{
					if (!lst.contains(paidId))
					{
						lst.add(paidId);
					}
				}
			}
		}
		return lst;
	}

	public void addLabel(PSENLabel l)
	{
		labels.add(l);
	}

	public boolean hasLabel(String labelName)
	{
		for (PSENLabel l : labels)
		{
			if (l.getLabelName().equals(labelName))
			{
				return true;
			}
		}
		return false;
	}

	public PSENLabel getLabelByName(String label_name)
	{
		for (PSENLabel l : labels)
		{
			if (l.getLabelName().equals(label_name))
			{
				return l;
			}
		}
		return null;
	}

	public int getValueCount()
	{
		// TODO Auto-generated method stub
		return paidNum;
	}

	public boolean hasValue(String transition)
	{
		if (paiLst != null)
		{
			for (String key : paiLst.keySet())
			{
				HashMap<String, PlaceAlignmentInfoDetail> hm = paiLst.get(key);
				if (hm.containsKey(transition))
				{
					PlaceAlignmentInfoDetail paid = hm.get(transition);
					return (paid.pairNum > 0);
				}
			}
		}
		return false;
	}

	public boolean hasValue(String transition, String labelName)
	{
		if (hasValue(transition))
		{
			return hasLabel(labelName);
		}
		return false;
	}

	public boolean hasValue(String transition, String labelName, String classifierValue)
	{
		if (paiLst != null)
		{
			HashMap<String, PlaceAlignmentInfoDetail> hm = paiLst.get(classifierValue);
			if (hm.containsKey(transition))
			{
				PlaceAlignmentInfoDetail paid = hm.get(transition);
				if (paid.pairNum > 0)
				{
					return hasLabel(labelName);
				}
			}
		}
		return false;
	}

	public double getValue(String transition, String labelName, String classifierValue)
	{
		if (paiLst != null)
		{
			HashMap<String, PlaceAlignmentInfoDetail> hm = paiLst.get(classifierValue);
			if (hm.containsKey(transition))
			{
				PlaceAlignmentInfoDetail paid = hm.get(transition);
				
				Double val = 0.0;
//				if (labelName.equals("PEndZero"))
//				{
//					val = (double)paid.pairZeroNum / (double)paid.pairNum;
//				}
//				else if (labelName.equals("PEndPos"))
//				{
//					val = (double)paid.pairPosNum / (double)paid.pairNum;
//				}
//				else if (labelName.equals("PEndNeg"))
//				{
//					val = (double)paid.pairNegNum / (double)paid.pairNum;
//				}
//				else if (labelName.equals("POrder"))
//				{
//					val = (double)paid.avgOrderProb;
//				}
//				else 
					if (labelName.equals("Support"))
				{
					val = (double)paid.getSupport();
				}
				else if (labelName.equals("Confidence"))
				{
					val = (double)paid.getConfidence();
				}
				return val;
			}
		}
		return 0;
	}

	public Vector<Double> getValues(String transition, String labelName, String classifierValue)
	{
		Vector<Double> lst = new Vector<Double>();
		if (paiLst != null)
		{
			HashMap<String, PlaceAlignmentInfoDetail> hm = paiLst.get(classifierValue);
			if (hm.containsKey(transition))
			{
				PlaceAlignmentInfoDetail paid = hm.get(transition);

				Double val = 0.0;
				if (labelName.equals("PEndZero"))
				{
					val = (double)paid.pairZeroNum / (double)paid.pairNum;
				}
				else if (labelName.equals("PEndPos"))
				{
					val = (double)paid.pairPosNum / (double)paid.pairNum;
				}
				else if (labelName.equals("PEndNeg"))
				{
					val = (double)paid.pairNegNum / (double)paid.pairNum;
				}
				else if (labelName.equals("POrder"))
				{
					val = (double)paid.avgOrderProb;
				}
				lst.add(val);
				return lst;
			}
		}
		return lst;
	}

	public HashMap<String, Double> getLabelValueGroups(String labelName)
	{
		HashMap<String, Double> hm = new HashMap<String, Double>();
		for (String localId : getTransitions())
		{
			for (String classifierName : getClassifierValues())
			{
				if (hasValue(localId, labelName, classifierName))
				{
					String hashId = localId + "-" + classifierName;
					Double val = getValue(localId, labelName, classifierName);
					hm.put(hashId, val);
				}
			}
		}
		
		return hm;
	}
	
	public HashMap<String, Double> getTransitionLabelValueGroupsOptT(String trans)
	{
		HashMap<String, Double> hm = new HashMap<String, Double>();
		
		for (String classifier : paiLst.keySet())
		{
			//HashMap<String, PlaceAlignmentInfoDetail> hm2 = paiLst.get(classifier);
			
			for (PSENLabel label : getLabels())
			{
				Double val = getValue(trans, label.getLabelName(), classifier);
				String idx = classifier + "x" + trans + "x" + label.getLabelName();
				
				hm.put(idx, val);
			}
		}
		
		return hm;
	}
	
	public void addValue(String transition, String labelName, String traceId, double myValue)
	{
		// TODO Auto-generated method stub
		
	}

	public void setCohortsForTraces(Iterable<XTrace> log, CohortClassifier cc)
	{
		// TODO Auto-generated method stub
		
	}

	public boolean fitsPetrinet(PetrinetGraph pnet)
	{
		HashSet<String> idx = new HashSet<String>();
		PetriNetDataSource pnds = new PetriNetDataSource(pnet);
		for (Transition t : pnds.getAllTransitions())
		{
			idx.add(t.getLocalID().toString());
		}
		
		for (String tId : this.getTransitions())
		{
			if (tId.length() == 36) //is transition localId
			{
				boolean existsInNet = false;
				existsInNet = idx.contains(tId);
				if (!existsInNet)
				{
					return false;
				}
			}
			else //is localId of transitionPair
			{
				boolean existsInNetSource = idx.contains(TransitionPair.getSourceId(tId));
				boolean existsInNetTarget = idx.contains(TransitionPair.getTargetId(tId));
				if (!existsInNetSource || !existsInNetTarget)
				{
					return false;
				}
			}
		}
		
		return true;
	}

}
