package org.processmining.processprofiler3d.compviz;

import org.deckfour.xes.model.XLog;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.fileio.DefaultFontData;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIDepthRangeCompressor;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIExportPanel;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIFloatingLabels;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIGraphicsConfig;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JILayerManager;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JILogReplay;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIModelExport;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIMoveModelElement;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIOpenVR;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIOptionsPanelManager;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENEdgeVisDriver;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENFilter;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENSelectModelElement;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENSignificance;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENTrajectoryVisDriver;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENTransVisDriver;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENView;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENVis;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JISelection;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JITrajectoryVisUI;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIViewManager;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIViewManagerPSEN;
import org.processmining.processprofiler3d.compviz.jogl.interactions.cohortsplit.JISplitCohorts;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOCamWidget;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOCohortColorManager;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOGrid;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENEdgeVisualizationLayer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENTrajectoryVisualizationLayer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENTransitionVisualizationLayer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOReplaya;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.methods.EdgeDataVisualisationMethodCollection;
import org.processmining.processprofiler3d.compviz.methods.EdgeDataVisualizationMethod_MagneticLines;
import org.processmining.processprofiler3d.compviz.methods.EdgeDataVisualizationMethod_MagneticLinesNew;
import org.processmining.processprofiler3d.compviz.methods.EdgeDataVisualizationMethod_None;
import org.processmining.processprofiler3d.compviz.methods.EdgeDataVisualizationMethod_QualityLines;
import org.processmining.processprofiler3d.compviz.methods.TransitionDataVisualisationMethodCollection;
import org.processmining.processprofiler3d.compviz.methods.TransitionDataVisualizationMethod_BarChart1D;
import org.processmining.processprofiler3d.compviz.methods.TransitionDataVisualizationMethod_BarChart2DColor;
import org.processmining.processprofiler3d.compviz.methods.TransitionDataVisualizationMethod_BarChart2DHeight;
import org.processmining.processprofiler3d.compviz.methods.TransitionDataVisualizationMethod_None;
import org.processmining.processprofiler3d.compviz.methods.TransitionDataVisualizationMethod_TriangleGlyphs;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglModelScene;
import org.processmining.processprofiler3d.psen.AlignedLog;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.PetriNetIndexed;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.plugins.ModelComparison3;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayCollection;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLensPerspective;
import org.processmining.visualisation3d.interactions.JICameraFree;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class PP3DAppModelPerformanceVisualiser extends PP3DAppBase 
{
	private GraphLayoutConnection mylayout; 
	private XLog mylog;
	private Marking mymarking = null;
	private IPerformanceStatisticsEnrichedNet mypsen;
	private IPerformanceStatisticsEnrichedNet mywten;
	private IPerformanceStatisticsEnrichedNet myedgeq;
	private DataMapProviderTransition myMapProviderTrans;
	private DataMapProviderEdge myMapProviderEdge;
	private DataMapProviderEdgeQuality myMapProviderQuality;
	private TraceReplayCollection mytrc = null;

	private PetriNetDataSource pnds = null;
	
	public PP3DAppModelPerformanceVisualiser(PetrinetGraph pnet, GraphLayoutConnection layoutConn, XLog l, Marking initMarking, IPerformanceStatisticsEnrichedNet psen,
			IPerformanceStatisticsEnrichedNet wten, TraceReplayCollection trCol)
	{
		mylayout = layoutConn;
		mylog = l;
		mymarking = initMarking;
		mypsen = psen;
		mywten = wten;
		mytrc = trCol;
		
		pnds = new PetriNetDataSource(pnet);
		System.out.println(pnds.getNetStats());

		myedgeq = new DataQualityDBInterface(pnds, l);

		myMapProviderTrans = new DataMapProviderTransition(pnds, mypsen);
		myMapProviderEdge = new DataMapProviderEdge(pnds, mywten);
		myMapProviderQuality = new DataMapProviderEdgeQuality(pnds, (DataQualityDBInterface)myedgeq);
		
		//initialize visualisation methods
		TransitionDataVisualisationMethodCollection.TDVM_None = new TransitionDataVisualizationMethod_None();
		TransitionDataVisualisationMethodCollection.TDVM_BarChart1D = new TransitionDataVisualizationMethod_BarChart1D(mypsen, myMapProviderTrans);
		TransitionDataVisualisationMethodCollection.TDVM_BarChart2DHeight = new TransitionDataVisualizationMethod_BarChart2DHeight(mypsen, myMapProviderTrans);
		TransitionDataVisualisationMethodCollection.TDVM_BarChart2DColor = new TransitionDataVisualizationMethod_BarChart2DColor(mypsen, myMapProviderTrans);
		TransitionDataVisualisationMethodCollection.TDVM_TriangleGlyphs = new TransitionDataVisualizationMethod_TriangleGlyphs(mypsen, myMapProviderTrans);
		
		TransitionDataVisualisationMethodCollection tvmc = new TransitionDataVisualisationMethodCollection();
		tvmc.clearValues();
		tvmc.addValue(TransitionDataVisualisationMethodCollection.TDVM_None);
		tvmc.addValue(TransitionDataVisualisationMethodCollection.TDVM_BarChart1D);
		tvmc.addValue(TransitionDataVisualisationMethodCollection.TDVM_BarChart2DHeight);
		tvmc.addValue(TransitionDataVisualisationMethodCollection.TDVM_BarChart2DColor);
		tvmc.addValue(TransitionDataVisualisationMethodCollection.TDVM_TriangleGlyphs);
		
		EdgeDataVisualisationMethodCollection.EDVM_None = new EdgeDataVisualizationMethod_None();
		EdgeDataVisualisationMethodCollection.EDVM_MagneticLines = new EdgeDataVisualizationMethod_MagneticLines(mywten, myMapProviderEdge);
		EdgeDataVisualisationMethodCollection.EDVM_MagneticLinesNew = new EdgeDataVisualizationMethod_MagneticLinesNew(mywten, myMapProviderEdge);
		EdgeDataVisualisationMethodCollection.EDVM_QualityLines = new EdgeDataVisualizationMethod_QualityLines(myedgeq, myMapProviderQuality);
		
		EdgeDataVisualisationMethodCollection evmc = new EdgeDataVisualisationMethodCollection();
		evmc.clearValues();
		evmc.addValue(EdgeDataVisualisationMethodCollection.EDVM_None);
		evmc.addValue(EdgeDataVisualisationMethodCollection.EDVM_MagneticLines);
		evmc.addValue(EdgeDataVisualisationMethodCollection.EDVM_MagneticLinesNew);
		evmc.addValue(EdgeDataVisualisationMethodCollection.EDVM_QualityLines);
		
		if (mytrc == null)
		{
			//check if log is aligned log, otherwise can't generate trace replay collection
			LogAnalyser la = new LogAnalyser(mylog);
			if (AlignedLog.CheckLogIsAligned(la))
			{
				//construct initial marking for now...
				if (mymarking == null)
				{
					mymarking = new Marking();
					Place startPlace = pnds.getStartPlace();
					mymarking.add(startPlace);
				}
				
				try
				{
					mytrc = ModelComparison3.GenerateTraceReplayCollection(null, pnds, mymarking, mylog);
				}
				catch (Exception ex)
				{
					mytrc = null;
					System.out.println("Could not create trace replay collection from log!");
				}
			}
		}
		
		//HzdOpenVR.TestVRLink();
	}
	
	protected void UpdateCameraLens(JoglCamera c, double aspect)
	{
		c.setCameraLens(new JoglCameraLensPerspective(45.0, aspect));
	}

	@Override
	public void InitScene(JoglCamera cam, JoglScene scene, JoglPanel panel) 
	{
		System.out.println("Creating PSEN scene...");
		
		JoglInteractionManager iman = panel.getInteractionManager();
		
		//hack! need to load this before the first JoglTextRenderer is loaded
		JoglResourceManager.AddResource("DefaultFont", new DefaultFontData());
		
		if (panel.getWidth() > 0 && panel.getHeight() > 0)
		{
			UpdateCameraLens(cam, panel.getWidth() / panel.getHeight());
		}

		scene.addSceneObject(new JSOGrid(true, true, false, cam));
		
		JSOCohortColorManager jccm = new JSOCohortColorManager(mylog);
		scene.addSceneObject(jccm);
		
		JSOPetriNet soPnet = new JSOPetriNet(panel, scene, pnds, mylayout);
		scene.addSceneObject(soPnet);
		
		for (JSOSubNet snet : soPnet.getAllSubNets())
		{
			JSOPSENTransitionVisualizationLayer soPsenT = new JSOPSENTransitionVisualizationLayer(iman, scene, snet, jccm, panel);
			scene.addSceneObject(soPsenT);
		}

		//scene.addSceneObject(new JSOGrid(false, false, true));

		PetriNetIndexed pnetIdx = new PetriNetIndexed(pnds);
		for (JSOSubNet snet : soPnet.getAllSubNets())
		{
			JSOPSENEdgeVisualizationLayer soPsenE = new JSOPSENEdgeVisualizationLayer(scene, pnetIdx, snet, jccm, panel);
			scene.addSceneObject(soPsenE);
		}
		
		JSOPSENTrajectoryVisualizationLayer tjvis = null;
		JSOReplaya repl = null;
		if (mytrc != null)
		{
			try
			{
				tjvis = new JSOPSENTrajectoryVisualizationLayer(panel, scene, pnds, soPnet, soPnet.getRootSubNet(), jccm, mytrc, mylog);
				scene.addSceneObject(tjvis);
			}
			catch (Exception ex)
			{
				tjvis = null;
				System.out.println("Scene: Error in creating trajectory visualiser!");
			}
			
			repl = new JSOReplaya(iman);
			scene.addSceneObject(repl);
		}
		
		JSOGraphicalUserInterface jgui = new JSOGraphicalUserInterface(panel);
		
		scene.addSceneObject(jgui);

		InitInteractions(tjvis, cam, scene, iman, panel, repl, jccm, soPnet, jgui);
		
		
		scene.addSceneObject(new JSOCamWidget());

//		if (persist != null)
//		{
//			PersistLoad();
//		}
//		
//		bInitialized = true;
	}

	private void InitInteractions(JSOPSENTrajectoryVisualizationLayer tjvis, JoglCamera cam, JoglScene scene, JoglInteractionManager iman, JoglPanel panel, JSOReplaya repl, JSOCohortColorManager jccm, JSOPetriNet soPnet,
			JSOGraphicalUserInterface jgui) 
	{
		JICameraFree camInteraction = new JICameraFree(cam);
		
		//Init camera
		JoglVectord3 camPos = JoglVectord3.Add(soPnet.GetPetrinetCenter(), new JoglVectord3(0, 500, 0));
		
		cam.SetPosition(camPos);
		cam.SetPitchYawRoll(Math.PI * 0.5, Math.PI, 0);
		
		iman.addInteraction(camInteraction);

		JIViewManager jivm = new JIViewManagerPSEN(iman, (JoglModelScene)scene, jgui);
		iman.addInteraction(jivm);
		
		JIOptionsPanelManager jivc = new JIOptionsPanelManager(iman, scene, jgui);
		jivc.AddPanel(new JIExportPanel(iman, cam, scene, jgui, mypsen, mywten), "Export");
		jivc.AddPanel(new JIGraphicsConfig(panel, scene, jgui), "Graphics");
		jivc.AddPanel(new JIPSENVis(iman, scene, jgui), "Vis");
		jivc.AddPanel(new JIPSENView(iman, scene, jgui), "View");
		jivc.AddPanel(new JIPSENSignificance(iman, scene, jgui, pnds, mypsen, mywten), "Sig");
		if (mylog != null)
		{
			jivc.AddPanel(new JISplitCohorts(iman, mylog, scene, jgui, mypsen, mywten), "Split");
		}
		iman.addInteraction(jivc);
		
		JIPSENTransVisDriver jivp = new JIPSENTransVisDriver(scene, pnds, myMapProviderTrans);
		iman.addInteraction(jivp);
		
		JIPSENEdgeVisDriver jivp2 = new JIPSENEdgeVisDriver(scene, iman, myMapProviderEdge, myMapProviderQuality);
		iman.addInteraction(jivp2);
		
		if (tjvis != null)
		{
			JIPSENTrajectoryVisDriver jivp3 = new JIPSENTrajectoryVisDriver(scene, tjvis);
			iman.addInteraction(jivp3);
			
			JITrajectoryVisUI jivpui = new JITrajectoryVisUI(iman, cam, scene, jgui, tjvis);
			iman.addInteraction(jivpui);
		}

		JIPSENSelectModelElement jisme = new JIPSENSelectModelElement(iman, scene, jgui, soPnet, mypsen, mywten, myedgeq);	
		iman.addInteraction(jisme);
		
		JILayerManager jlm = new JILayerManager(scene, soPnet);
		iman.addInteraction(jlm);
		
		JIPSENFilter filterGUI = new JIPSENFilter(iman, jgui, mypsen, jccm);
		iman.addInteraction(filterGUI);
		
		if (repl != null && mytrc != null)
		{
			JILogReplay jilr = new JILogReplay(iman, mytrc, scene, jgui, repl, soPnet, mymarking);
			iman.addInteraction(jilr);
		}
		
		JIFloatingLabels jifl = new JIFloatingLabels(iman, scene, jgui, soPnet);
		iman.addInteraction(jifl);
		
		JIDepthRangeCompressor jdrc = new JIDepthRangeCompressor(cam);	
		iman.addInteraction(jdrc);

		JISelection sel = new JISelection(iman, scene, jgui, soPnet);
		iman.addInteraction(sel);
		
		JIModelExport exp = new JIModelExport(iman, cam, scene, jgui);
		iman.addInteraction(exp);
		
		JIOpenVR exp2 = new JIOpenVR(panel, scene, jgui);
		exp2.addVRModeListener(jivm);
		iman.addInteraction(exp2);
		
		JIMoveModelElement jitt = new JIMoveModelElement(panel, scene);
		iman.addInteraction(jitt);
	}
}
