package org.processmining.processprofiler3d.compviz;

import org.deckfour.xes.model.XLog;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.fileio.DefaultFontData;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIDepthRangeCompressor;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIExportPanel;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIFloatingLabels;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIGraphicsConfig;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JILayerManager;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JILogReplay;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIOpenVR;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIOptionsPanelManager;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENFilter;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENView;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JISelectModelElement;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JISelection;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIViewManager;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIViewManagerModel;
import org.processmining.processprofiler3d.compviz.jogl.interactions.camera.JICameraOrbit;
import org.processmining.processprofiler3d.compviz.jogl.interactions.cohortsplit.JISplitCohorts;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOCamWidget;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOCohortColorManager;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOGrid;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOReplaya;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglModelScene;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayCollection;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLensPerspective;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class PP3DAppModelReplayVisualiser extends PP3DAppBase 
{
	private GraphLayoutConnection mylayout = null;
	private TraceReplayCollection mytrc = null;
	private Marking mymarking = null;
	private XLog mylog = null;
	private IPerformanceStatisticsEnrichedNet mypsen = null;
	private IPerformanceStatisticsEnrichedNet mywten = null;
	private PetriNetDataSource pnds = null;

	public PP3DAppModelReplayVisualiser(PetrinetGraph pnet, GraphLayoutConnection layoutConn, TraceReplayCollection trc, Marking initMarking, XLog log, IPerformanceStatisticsEnrichedNet psen,
			IPerformanceStatisticsEnrichedNet wten)
	{
		mylayout = layoutConn;
		mytrc = trc;
		mymarking = initMarking;
		mylog = log;
		mypsen = psen;
		mywten = wten;
		
		pnds = new PetriNetDataSource(pnet);
		System.out.println(pnds.getNetStats());		
	}
	
	protected void UpdateCameraLens(JoglCamera c, double aspect)
	{
		c.setCameraLens(new JoglCameraLensPerspective(45.0, aspect));
	}

	@Override
	public void InitScene(JoglCamera cam, JoglScene scene, JoglPanel panel) 
	{
		System.out.println("Creating replay scene...");
		
		JoglInteractionManager iman = panel.getInteractionManager();
		
		//hack! need to load this before the first JoglTextRenderer is loaded
		JoglResourceManager.AddResource("DefaultFont", new DefaultFontData());
		
		if (panel.getWidth() > 0 && panel.getHeight() > 0)
		{
			UpdateCameraLens(cam, panel.getWidth() / panel.getHeight());
		}

		scene.addSceneObject(new JSOGrid(true, true, false, cam));

		JSOCohortColorManager jccm = new JSOCohortColorManager(mylog);
		scene.addSceneObject(jccm);
		
		//JSOPetriNet soPnet = new JSOPetriNet(pnet, layoutConn);
		JSOPetriNet soPnet = new JSOPetriNet(panel, scene, pnds, mylayout);
		scene.addSceneObject(soPnet);
		
		JSOGraphicalUserInterface jgui = new JSOGraphicalUserInterface(panel);	
		scene.addSceneObject(jgui);
		
		JSOReplaya repl = new JSOReplaya(iman);
		scene.addSceneObject(repl);
		
		InitInteractions(cam, scene, iman, panel, soPnet, repl, jccm, jgui);
		
		scene.addSceneObject(new JSOCamWidget());

//		if (persist != null)
//		{
//			PersistLoad();
//		}
//		
//		bInitialized = true;
	}

	private void InitInteractions(JoglCamera cam, JoglScene scene, JoglInteractionManager iman, JoglPanel panel, JSOPetriNet soPnet,
			JSOReplaya repl, JSOCohortColorManager jccm, JSOGraphicalUserInterface jgui) 
	{
		JICameraOrbit camInteraction = new JICameraOrbit(cam);
		camInteraction.setCamFocus(soPnet.GetPetrinetCenter());		
		iman.addInteraction(camInteraction);

		JIViewManager jivm = new JIViewManagerModel(iman, (JoglModelScene)scene, jgui);
		iman.addInteraction(jivm);
		
		JIOptionsPanelManager jivc = new JIOptionsPanelManager(iman, scene, jgui);
		jivc.AddPanel(new JIExportPanel(iman, cam, scene, jgui, mypsen, mywten), "Export");
		jivc.AddPanel(new JIGraphicsConfig(panel, scene, jgui), "Graphics");
		jivc.AddPanel(new JIPSENView(iman, scene, jgui), "View");
		if (mylog != null)
		{
			jivc.AddPanel(new JISplitCohorts(iman, mylog, scene, jgui, mypsen, mywten), "Split");
		}
		iman.addInteraction(jivc);
		
		JILogReplay jilr = new JILogReplay(iman, mytrc, scene, jgui, repl, soPnet, mymarking);
		jilr.setVisMode("Enabled");
		iman.addInteraction(jilr);
		
		JIFloatingLabels jifl = new JIFloatingLabels(iman, scene, jgui, soPnet);	
		iman.addInteraction(jifl);
		
		JISelectModelElement jisme = new JISelectModelElement(iman, scene, jgui, soPnet);	
		iman.addInteraction(jisme);
		
		JILayerManager jlm = new JILayerManager(scene, soPnet);
		iman.addInteraction(jlm);
		
		JIPSENFilter filterGUI = new JIPSENFilter(iman, jgui, mypsen, jccm);
		iman.addInteraction(filterGUI);
		
		JIDepthRangeCompressor jdrc = new JIDepthRangeCompressor(cam);	
		iman.addInteraction(jdrc);
		
		JISelection sel = new JISelection(iman, scene, jgui, soPnet);
		iman.addInteraction(sel);
		
		JIOpenVR exp2 = new JIOpenVR(panel, scene, jgui);
		exp2.addVRModeListener(jivm);
		iman.addInteraction(exp2);
	}
}
