package org.processmining.processprofiler3d.compviz;

import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.processprofiler3d.compviz.fileio.DefaultFontData;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIDepthRangeCompressor;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIExportPanel;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIFloatingLabels;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIGraphicsConfig;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JILayerManager;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIModelExport;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIMoveModelElement;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIOpenVR;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIOptionsPanelManager;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENView;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JISelectModelElement;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JISelection;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIViewManager;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIViewManagerModel;
import org.processmining.processprofiler3d.compviz.jogl.interactions.camera.JICameraOrbit;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOCamWidget;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOGrid;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglModelScene;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class PP3DAppModelVisualiser extends PP3DAppBase
{
	private PetriNetDataSource pnds = null;
	private GraphLayoutConnection mylayout = null;
	
	public PP3DAppModelVisualiser(PetrinetGraph pnet, GraphLayoutConnection layoutConn)
	{
		mylayout = layoutConn;
		
		pnds = new PetriNetDataSource(pnet);
		System.out.println(pnds.getNetStats());
	}
	
	@Override
	public void InitScene(JoglCamera cam, JoglScene scene, JoglPanel panel) 
	{
		System.out.println("Creating scene...");
		
		JoglInteractionManager iman = panel.getInteractionManager();
		
		//hack! need to load this before the first JoglTextRenderer is loaded
		JoglResourceManager.AddResource("DefaultFont", new DefaultFontData());

		if (panel.getWidth() > 0 && panel.getHeight() > 0)
		{
			UpdateCameraLens(cam, panel.getWidth() / panel.getHeight());
		}
		
		scene.addSceneObject(new JSOGrid(true, true, false, cam));
		
		JSOPetriNet soPnet = new JSOPetriNet(panel, scene, pnds, mylayout);
		scene.addSceneObject(soPnet);
		
		JSOGraphicalUserInterface jgui = new JSOGraphicalUserInterface(panel);
		
		scene.addSceneObject(jgui);
		
		InitInteractions(iman, cam, scene, panel, soPnet, jgui);
		
		scene.addSceneObject(new JSOCamWidget());
		
//		if (persist != null)
//		{
//			PersistLoad();
//		}
//		
//		bInitialized = true;
	}

	private void InitInteractions(JoglInteractionManager iman, JoglCamera cam, JoglScene scene, JoglPanel panel, JSOPetriNet soPnet, JSOGraphicalUserInterface jgui) 
	{
		JICameraOrbit camInteraction = new JICameraOrbit(cam);
		camInteraction.setCamFocus(soPnet.GetPetrinetCenter());		
		iman.addInteraction(camInteraction);

		JIViewManager jivm = new JIViewManagerModel(iman, (JoglModelScene)scene, jgui);
		iman.addInteraction(jivm);

		JIOptionsPanelManager jivc = new JIOptionsPanelManager(iman, scene, jgui);
		jivc.AddPanel(new JIExportPanel(iman, cam, scene, jgui, null, null), "Export");
		jivc.AddPanel(new JIGraphicsConfig(panel, scene, jgui), "Graphics");
		jivc.AddPanel(new JIPSENView(iman, scene, jgui), "View");
		iman.addInteraction(jivc);
		
		JIFloatingLabels jifl = new JIFloatingLabels(iman, scene, jgui, soPnet);	
		iman.addInteraction(jifl);
		
		JISelectModelElement jisme = new JISelectModelElement(iman, scene, jgui, soPnet);	
		iman.addInteraction(jisme);
		
		JILayerManager jlm = new JILayerManager(scene, soPnet);
		iman.addInteraction(jlm);
		
		JIDepthRangeCompressor jdrc = new JIDepthRangeCompressor(cam);	
		iman.addInteraction(jdrc);
		
		JISelection sel = new JISelection(iman, scene, jgui, soPnet);
		iman.addInteraction(sel);
		
		JIModelExport exp = new JIModelExport(iman, cam, scene, jgui);
		iman.addInteraction(exp);
		
		JIOpenVR exp2 = new JIOpenVR(panel, scene, jgui);
		exp2.addVRModeListener(jivm);
		iman.addInteraction(exp2);
		
		JIMoveModelElement jitt = new JIMoveModelElement(panel, scene);
		iman.addInteraction(jitt);
	}
}
