package org.processmining.processprofiler3d.compviz;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class TransitionDataMapDouble
{
	private Map<String, Double> value1Map = new HashMap<String, Double>();
	private Map<String, Double> value1MinMap = new HashMap<String, Double>();
	private Map<String, Double> value1MaxMap = new HashMap<String, Double>();
	
	public boolean hasValue(Transition t, String classifier)
	{
		String id = t.getLocalID().toString() + classifier;
		return (value1Map.containsKey(id));
	}
	
	public boolean hasValue(String t, String classifier)
	{
		String id = t + classifier;
		return (value1Map.containsKey(id));
	}
	
	public boolean hasValue(String id)
	{
		return (value1Map.containsKey(id));
	}
	
	public Double getValueR(Transition t, String classifier)
	{
		String id = t.getLocalID().toString() + classifier;
		if (value1Map.containsKey(id))
		{
			double val = value1Map.get(id);
			double min = value1MinMap.get(id);
			double max = value1MaxMap.get(id);

			if (max != min)
			{
				Double res = (val - min) / (max - min);
				return res;
			}
			else
			{
				if (max == 0)
				{
					return 0.0;
				}
				else
				{
					return 1.0;
				}
			}
		}
		return null;
	}
	
	public Double getValueR(String t, String classifier)
	{
		String id = t + classifier;
		if (value1Map.containsKey(id))
		{
			double val = value1Map.get(id);
			double min = value1MinMap.get(id);
			double max = value1MaxMap.get(id);

			if (max != min)
			{
				Double res = (val - min) / (max - min);
				return res;
			}
			else
			{
				if (max == 0)
				{
					return 0.0;
				}
				else
				{
					return 1.0;
				}
			}
		}
		return null;
	}
	
	public Double getValueR(String id)
	{
		if (value1Map.containsKey(id))
		{
			double val = value1Map.get(id);
			double min = value1MinMap.get(id);
			double max = value1MaxMap.get(id);

			if (max != min)
			{
				Double res = (val - min) / (max - min);
				return res;
			}
			else
			{
				if (max == 0)
				{
					return 0.0;
				}
				else
				{
					return 1.0;
				}
			}
		}
		return null;
	}
	
	public Double getValue(String id)
	{
		if (value1Map.containsKey(id))
		{
			return value1Map.get(id);
		}
		return null;
	}
	
	public Double getMin(String id)
	{
		if (value1MinMap.containsKey(id))
		{
			return value1MinMap.get(id);
		}
		return null;
	}
	
	public Double getMax(String id)
	{
		if (value1MaxMap.containsKey(id))
		{
			return value1MaxMap.get(id);
		}
		return null;
	}
	
	public void setValue(String id, Double val)
	{
		value1Map.put(id, val);
	}
	
	public void setMin(String id, Double min)
	{
		value1MinMap.put(id, min);
	}
	
	public void setMax(String id, Double max)
	{
		value1MaxMap.put(id, max);
	}
	
	public void setValue(String t, String classifier, Double val)
	{
		String id = t + classifier;
		value1Map.put(id, val);
	}
	
	public void setMin(String t, String classifier, Double min)
	{
		String id = t + classifier;
		value1MinMap.put(id, min);
	}
	
	public void setMax(String t, String classifier, Double max)
	{
		String id = t + classifier;
		value1MaxMap.put(id, max);
	}
	
	public Set<String> getEntryIds()
	{
		return value1Map.keySet();
	}
	
	public static TransitionDataMapDouble interpolate(TransitionDataMapDouble map1, TransitionDataMapDouble map2, double factor)
	{
		TransitionDataMapDouble newmap = new TransitionDataMapDouble();

		Set<String> keys = new HashSet<String>();
		keys.addAll(map1.getEntryIds());
		for (String id2 : map2.getEntryIds())
		{
			if (!keys.contains(id2))
			{
				try
				{
					keys.add(id2);
				}
				catch (Exception ex)
				{
					System.out.println("Id2:" + id2);
				}
			}
		}
		for (String id : keys)
		{
			//String name = id.substring(0, id.indexOf("cohort"));
			
			double d1 = 0;
			double d2 = 0;
			double d1min = 0;
			double d2min = 0;
			double d1max = 0;
			double d2max = 0;
			
			if (map1.hasValue(id))
			{
				d1 = map1.getValue(id);
				d1min = map1.getMin(id);
				d1max = map1.getMax(id);
			}
			if (map2.hasValue(id))
			{
				d2 = map2.getValue(id);
				d2min = map2.getMin(id);
				d2max = map2.getMax(id);
			}
			
			if ( (factor <= 0.5f && map1.hasValue(id)) || (factor > 0.5 && map2.hasValue(id)))
			{
				double d = d1 + ((d2 - d1) * factor);		
				double dmin = d1min + ((d2min - d1min) * factor);		
				double dmax = d1max + ((d2max - d1max) * factor);
	
				//System.out.println("Interp: " + id + " - " + d);
				
				newmap.setValue(id, d);
				newmap.setMin(id, dmin);
				newmap.setMax(id, dmax);
			}
		}
		
		return newmap;
	}
}
