package org.processmining.processprofiler3d.compviz.fileio;

import java.awt.geom.Rectangle2D;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Vector;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;

public class BMFontFile 
{
    public BMFFontInfo info = null;
    public BMFCommonInfo common = null;
    public Vector<BMFCharInfo> chars = new Vector<BMFCharInfo>();
    public Vector<BMFKerningInfo> kernings = new Vector<BMFKerningInfo>();
    public HashMap<Integer, BMFCharInfo> charsById = new HashMap<Integer, BMFCharInfo>();
    public HashMap<String, BMFKerningInfo> kerningsById = new HashMap<String, BMFKerningInfo>();
    public Vector<BMFPageInfo> pages = new Vector<BMFPageInfo>();

    public String dirPath = "";

    private float fontBaseScale = 1.0f;
    private HashMap<Character, Rectangle2D.Float> charRects = new HashMap<Character, Rectangle2D.Float>();
    private HashMap<Character, Rectangle2D.Float> charTexRects = new HashMap<Character, Rectangle2D.Float>();

    public static JoglTexture tex = null;
    
    public void Load(String filePath)
    {
        //dirPath = System.IO.Path.GetDirectoryName(filePath);

        Vector<Vector<String>> lines = new Vector<Vector<String>>();

        //StreamReader br = new StreamReader(File.OpenRead(filePath));
		InputStream istream = this.getClass().getResourceAsStream(filePath);

        // create new data input stream
		DataInputStream dis = new DataInputStream(istream);
		BufferedReader br = new BufferedReader(new InputStreamReader(dis));

		try
		{
	        while (br.ready())
	        {
	            String line = br.readLine();
	            if (line != null)
	            {
		            Vector<String> lst = parseLine(line);
		
		            lines.add(lst);
		
		            if (lst.get(0).equals("char"))
		            {
		                BMFCharInfo bci = new BMFCharInfo(lst);
		                chars.add(bci);
		                charsById.put(bci.id, bci);
		            }
		            else if (lst.get(0).equals("common"))
		            {
		                common = new BMFCommonInfo(lst);
		            }
		            else if (lst.get(0).equals("page"))
		            {
		                pages.add(new BMFPageInfo(lst));
		            }
		            else if (lst.get(0).equals("info"))
		            {
		                info = new BMFFontInfo(lst);
		            }
		            else if (lst.get(0).equals("kerning"))
		            {
		                BMFKerningInfo bki = new BMFKerningInfo(lst);
		                kernings.add(bki);
		                kerningsById.put("" + ((char)bki.first) + ((char)bki.second), bki);
		            }
	            }
	        }
	
	        br.close();
		}
		catch (IOException ioe)
		{
			System.out.println("Error loading bmf font data!");
		}

        fontBaseScale = 1.0f / (float)info.size;

        float invWScale = 1.0f / (float)common.scaleW;
        float invHScale = 1.0f / (float)common.scaleH;
        for (BMFCharInfo ci : chars)
        {
            charRects.put((char)(ci.id), new Rectangle2D.Float(ci.xoffset, ci.yoffset, ci.width, ci.height));
            charTexRects.put((char)(ci.id), new Rectangle2D.Float((float)ci.x * invWScale, (float)ci.y * invHScale, (float)ci.width * invWScale, (float)ci.height * invHScale));
        }

        //AddKerning('m', 'e', -8);
        //AddKerning('n', 'f', -8);
        //AddKerning('n', 'c', -8);
        //AddKerning('u', 's', -8);
        //AddKerning('i', 't', -8);
        //AddKerning('l', 'y', -8);
        //AddKerning('l', 'e', -8);
        //AddKerning('s', 't', -8);
        //AddKerning('h', 'e', -8);
        //AddKerning('n', 'd', -8);
        //AddKerning('r', 'e', -4);
        //AddKerning('j', 'e', 8);
    }

    private void AddKerning(char c1, char c2, int amount)
    {
    	Vector<String> lst = new Vector<String>();
    	lst.add("kerning");
    	lst.add("first=" + (int)c1);
    	lst.add("second=" + (int)c2);
    	lst.add("amount=" + amount);
        BMFKerningInfo ki3 = new BMFKerningInfo(lst);
        kernings.add(ki3);
        kerningsById.put("" + ((char)ki3.first) + ((char)ki3.second), ki3);
    }

    private Vector<String> parseLine(String line)
    {
        Vector<String> lst = new Vector<String>();

        boolean bInsideString = false;
        String current = "";
        for (int i = 0; i < line.length(); i++)
        {
            if (line.charAt(i) == '"')
            {
                bInsideString = !bInsideString;
                continue;
            }

            if (bInsideString)
            {
                current += line.charAt(i);
            }
            else
            {
                if (Character.isWhitespace(line.charAt(i)))
                {
                    if (current != "")
                    {
                        lst.add(current);
                        current = "";
                    }
                }
                else
                {
                    current += line.charAt(i);
                }
            }
        }

        if (current != "")
        {
            lst.add(current);
            current = "";
        }

        return lst;
    }

    public float GetLineHeight()
    {
        return common.lineHeight;
    }

    public float GetBaseHeight()
    {
        return common.baseH;
    }

    public float GetCharWidth(char c)
    {
        int cId = c;
        if (charsById.containsKey(cId))
        {
            return charsById.get(cId).width;
        }
        return 0.0f;
    }

    public float GetCharAdvance(char c)
    {
        int cId = c;
        if (charsById.containsKey(cId))
        {
            return charsById.get(cId).xadvance;
        }
        return 0.0f;
    }

    public Rectangle2D.Float GetCharRect(char c)
    {
        if (charRects.containsKey(c))
        {
            return charRects.get(c);
        }
        return new Rectangle2D.Float();
    }

    public Rectangle2D.Float GetCharTexRect(char c)
    {
        if (charTexRects.containsKey(c))
        {
            return charTexRects.get(c);
        }
        return new Rectangle2D.Float();
    }
//
//    public System.Drawing.Bitmap GetPageBitmap(int pageId)
//    {
//        string fileName = pages[0].texFile;
//
//        System.Drawing.Image img = System.Drawing.Image.FromFile(dirPath + "\\" + fileName);
//
//        return img as System.Drawing.Bitmap;
//    }
	
	public void LoadFontTexture(GraphicsPipeline pipe)
	{
		String texResLoc = "/org/processmining/processprofiler3d/compviz/jogl/resources/freesansDF.png";

		tex = JoglTexture.LoadTextureFromFile(pipe, texResLoc, false);
		JoglResourceManager.AddResource("FontTextureDF", tex);
	}

    public float GetFontBaseScale()
    {
        return fontBaseScale;
        //return info.size;
    }

    public float GetKerningFor(String s)
    {
        if (kerningsById.containsKey(s))
        {
            return kerningsById.get(s).amount;
        }
        return 0.0f;
    }

}
