package org.processmining.processprofiler3d.compviz.graphicsdatastructures;

import java.util.Vector;

import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class JoglLine3D 
{
	private JoglVectord3 startPos;
	private JoglVectord3 endPos;
	
	private JoglVectord3 dir;
	private JoglVectord3 sideDir;
	private JoglVectord3 upDir;
	
	public Vector<JoglVectord3> vertLst;
	public Vector<JoglVectord2> texCoordLst;
	public Vector<Integer> triLst;
	
	public JoglVectord3 getStartPos()
	{
		return startPos;
	}
	
	public JoglVectord3 getEndPos()
	{
		return endPos;
	}
	
	public JoglVectord3 getLineDir()
	{
		return dir;
	}
	
	public JoglVectord3 getSideDir()
	{
		return sideDir;
	}
	
	public JoglVectord3 getUpDir()
	{
		return upDir;
	}
	
	public JoglLine3D(JoglVectord3 start, JoglVectord3 end, double lineWidth, double lineHeight)
	{
		startPos = start;
		endPos = end;
		
		dir = JoglVectord3.Subtract(endPos, startPos);
		sideDir = new JoglVectord3(dir.getValue(2), 0, -dir.getValue(0));
		sideDir = JoglVectord3.Normal(sideDir);
		upDir = JoglVectord3.Multiply(JoglVectord3.Cross(sideDir, dir), -1.0);
		upDir = JoglVectord3.Normal(upDir);
		
		JoglVectord3 v0 = JoglVectord3.Add(startPos, JoglVectord3.Multiply(sideDir,lineWidth));
		JoglVectord3 v1 = JoglVectord3.Add(startPos, JoglVectord3.Multiply(sideDir,-lineWidth));
		
		JoglVectord3 v4 = JoglVectord3.Add(startPos, JoglVectord3.Multiply(upDir,lineHeight));
		JoglVectord3 v5 = JoglVectord3.Add(startPos, JoglVectord3.Multiply(upDir,-lineHeight));
		
		JoglVectord3 v2 = JoglVectord3.Add(endPos, JoglVectord3.Multiply(sideDir, lineWidth));
		JoglVectord3 v3 = JoglVectord3.Add(endPos, JoglVectord3.Multiply(sideDir,-lineWidth));
		
		JoglVectord3 v6 = JoglVectord3.Add(endPos, JoglVectord3.Multiply(upDir, lineHeight));
		JoglVectord3 v7 = JoglVectord3.Add(endPos, JoglVectord3.Multiply(upDir,-lineHeight));

		vertLst = new Vector<JoglVectord3>();
		
		vertLst.add(v0);
		vertLst.add(v1);
		vertLst.add(v2);
		vertLst.add(v3);
		vertLst.add(v4);
		vertLst.add(v5);
		vertLst.add(v6);
		vertLst.add(v7);
		
		texCoordLst = new Vector<JoglVectord2>();
		
		texCoordLst.add(new JoglVectord2(0,0));
		texCoordLst.add(new JoglVectord2(0,0));
		texCoordLst.add(new JoglVectord2(1,0));
		texCoordLst.add(new JoglVectord2(1,0));
		texCoordLst.add(new JoglVectord2(0,0));
		texCoordLst.add(new JoglVectord2(0,0));
		texCoordLst.add(new JoglVectord2(1,0));
		texCoordLst.add(new JoglVectord2(1,0));
		
		triLst = new Vector<Integer>();
		
		//draw line body			
		//upper left side
		triLst.add(0);
		triLst.add(6);
		triLst.add(2);
		
		triLst.add(0);
		triLst.add(4);
		triLst.add(6);
		
		//upper right side
		triLst.add(4);
		triLst.add(3);
		triLst.add(6);
		
		triLst.add(4);
		triLst.add(1);
		triLst.add(3);
		
		//bottom left
		triLst.add(0);
		triLst.add(2);
		triLst.add(7);
		
		triLst.add(0);
		triLst.add(7);
		triLst.add(5);
		
		//bottom right
		triLst.add(5);
		triLst.add(7);
		triLst.add(3);
		
		triLst.add(5);
		triLst.add(3);
		triLst.add(1);
	}

}
