package org.processmining.processprofiler3d.compviz.graphicsdatastructures;

import java.nio.DoubleBuffer;
import java.util.Vector;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.gldatastructures.JoglVertexArrayObjectBuffer;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshDataBufferVectord3;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;

import com.jogamp.opengl.GL2;

public class JoglMesh 
{
	protected int drawType = GL2.GL_TRIANGLES;
	
	protected JoglMeshIndexBuffer inds;
	protected JoglMeshDataBufferVectord3 vertsd;
	
	public JoglMesh(int dType)
	{
		vertsd = new JoglMeshDataBufferVectord3();
	}
	
	public JoglMesh(int dType, int vCapacity)
	{
		drawType = dType;

		vertsd = new JoglMeshDataBufferVectord3(vCapacity);
	}
	
	public int getNumVerts()
	{
		return vertsd.getNum();
	}
	
	public double[] getVerts()
	{
		return vertsd.getArray();
	}
	
	public JoglMeshDataBufferVectord3 getVertsBuffer()
	{
		return vertsd;
	}
	
	public JoglMeshIndexBuffer getIndexBuffer()
	{
		return inds;
	}
	
	public void setIndexBuffer(JoglMeshIndexBuffer b)
	{
		inds = b;
	}
	
	public void setVerts(double[] vlist)
	{
		vertsd.set(vlist);
	}
	
	public void setVerts(Vector<JoglVectord3> vlist)
	{
		vertsd.set(vlist);
	}
	
	public void setVerts(double[] vlist, JoglVectord3 offset)
	{
		vertsd.set(vlist, offset);
	}
	
	public void addVerts(double[] vlist)
	{
		vertsd.add(vlist);
	}
	
	public void addVerts(Vector<JoglVectord3> vlist)
	{
		vertsd.add(vlist);
	}
	
	public void addVerts(JoglMesh m)
	{
		vertsd.add(m.getVertsBuffer().getNativeBuffer());
	}
	
	public void addVerts(Vector<JoglVectord3> vlist, JoglVectord3 offset)
	{
		vertsd.add(vlist, offset);
	}
	
	public void addVerts(double[] vlist, JoglVectord3 offset)
	{
		vertsd.add(vlist, offset);
	}
	
	public void addVerts(JoglMesh m, JoglVectord3 offset)
	{
		vertsd.add(m.getVertsBuffer().getNativeBuffer(), offset);
	}
	
	public void addVerts(JoglMesh m, JoglMatrixd4x4 offset)
	{
		vertsd.add(m.getVertsBuffer().getNativeBuffer(), offset);
	}
	
	public void addVerts(JoglMesh m, JoglVectord3 offset, int vstart, int vend)
	{
		vertsd.add(m.getVertsBuffer().getNativeBuffer(), offset, vstart, vend);
	}
	
	public void draw(GraphicsPipeline pipe)
	{
		draw(pipe, 0, getNumVerts());
	}
	
	public void draw(GraphicsPipeline pipe, int start, int end)
	{
		GL2 gl = pipe.gl;
		
		// start drawing the mesh
		gl.glBegin(drawType);       

		vertsd.position(start);
		for (int i = start; i < end; i++)
		{
			vertsd.drawNextVertex3d(gl);
		}
		
		// done drawing the mesh
		gl.glEnd(); 
	}
	
	public Vector<JoglVertexArrayObjectBuffer> getVAOBuffers(GraphicsPipeline pipe)
	{
		Vector<JoglVertexArrayObjectBuffer> lst = new Vector<JoglVertexArrayObjectBuffer>();
		
		lst.add(vertsd.getVAOBuffer(pipe));
		
		return lst;
	}

	public JoglAABB getAABB()
	{		
		double minx = Double.POSITIVE_INFINITY;
		double miny = Double.POSITIVE_INFINITY;
		double minz = Double.POSITIVE_INFINITY;
		double maxx = Double.NEGATIVE_INFINITY;
		double maxy = Double.NEGATIVE_INFINITY;
		double maxz = Double.NEGATIVE_INFINITY;
		
		DoubleBuffer dbuffer = vertsd.getNativeBuffer();
		
		dbuffer.position(0);
		for (int i = 0; i < dbuffer.limit(); i = i + 3)
		{
			double x = dbuffer.get();
			double y = dbuffer.get();
			double z = dbuffer.get();
			
			minx = Math.min(minx, x);
			miny = Math.min(miny, y);
			minz = Math.min(minz, z);
			
			maxx = Math.max(maxx, x);
			maxy = Math.max(maxy, y);
			maxz = Math.max(maxz, z);
		}
		
		JoglVectord3 vmin = new JoglVectord3(minx, miny, minz);
		JoglVectord3 vmax = new JoglVectord3(maxx, maxy, maxz);

		JoglAABB aabb = new JoglAABB(vmin, vmax);
	
		return aabb;
	}

}