package org.processmining.processprofiler3d.compviz.jogl;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.scene.IDataVisMagnetLine;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdge;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdgeVertex;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnPlace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisMagnetLineBase;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JoglSelection 
{
	private static long selectionState = 0;
	private Vector<JoglSceneObject> elements = new Vector<JoglSceneObject>();
	private Vector<ISelectionChangeListener> selectionChangeListeners = new Vector<ISelectionChangeListener>();
	
	public boolean isEmpty()
	{
		return (elements.size() == 0);
	}
	
	public Vector<JoglSceneObject> getElements()
	{
		return elements;
	}
	
	public int getElementCount()
	{
		return elements.size();
	}
	
	public long getSelectionState()
	{
		return selectionState;
	}
	
	public JoglSelection()
	{
		
	}
	
	public boolean contains(JoglSceneObject jso)
	{
		if (jso instanceof JSODataVisMagnetLineBase)
		{
			IDataVisMagnetLine line = findInstanceOf((JSODataVisMagnetLineBase)jso);
			return (line != null);
		}
	
		return elements.contains(jso);
	}
	
	private JSODataVisMagnetLineBase findInstanceOf(JSODataVisMagnetLineBase line)
	{
		for (JoglSceneObject jso : elements)
		{
			if (jso instanceof JSODataVisMagnetLineBase)
			{
				JSODataVisMagnetLineBase oline = (JSODataVisMagnetLineBase)jso;
				if (line.getParentRef() == oline.getParentRef() && line.getCohortRef() == oline.getParentRef())
				{
					return oline;
				}
			}
		}
		return null;
	}
	
	public void add(JoglSceneObject jso)
	{
		if (jso instanceof JSOPnTransition)
		{
			((JSOPnTransition)jso).setColor(JoglModelStyle.highlightColor);
		}
		else if (jso instanceof JSOPnPlace)
		{
			((JSOPnPlace)jso).setColor(JoglModelStyle.highlightColor);
		}
		else if (jso instanceof JSOPnEdge)
		{
			((JSOPnEdge)jso).setColor(JoglModelStyle.highlightColor);
		}
		else if (jso instanceof JSOPnEdgeVertex)
		{
			((JSOPnEdgeVertex)jso).Show();
		}
//		else if (jso instanceof JSODataVisMagnetLine)
//		{
//			//JSODataVisMagnetLine line = getCurrentInstance((JSODataVisMagnetLine)jso);
//			if (line != null)
//			{
//				line.Highlight();
//			}
//		}
		elements.add(jso);
		
		onSelectionChanged();
	}
	
	public void remove(int i)
	{
		removeHelper(elements.get(i));
		
		onSelectionChanged();
	}
	
	public void remove(JoglSceneObject jso)
	{
		if (jso instanceof JSODataVisMagnetLineBase)
		{
			JSODataVisMagnetLineBase line = findInstanceOf((JSODataVisMagnetLineBase)jso);
			if (line != null)
			{
				removeHelper(line);
			}
		}
		else
		{
			removeHelper(jso);
		}
		
		onSelectionChanged();
	}
	
	private void removeHelper(JoglSceneObject jso)
	{
		if (jso instanceof JSOPnTransition)
		{
			((JSOPnTransition)jso).setColor(JoglModelStyle.defaultColor);
		}
		else if (jso instanceof JSOPnPlace)
		{
			((JSOPnPlace)jso).setColor(JoglModelStyle.defaultColor);
		}
		else if (jso instanceof JSOPnEdge)
		{
			((JSOPnEdge)jso).setColor(JoglModelStyle.defaultColor);
		}
		else if (jso instanceof JSOPnEdgeVertex)
		{
			((JSOPnEdgeVertex)jso).Hide();
		}
//		else if (jso instanceof JSODataVisMagnetLine)
//		{
//			JSODataVisMagnetLine line = getCurrentInstance((JSODataVisMagnetLine)jso);
//			if (line != null)
//			{
//				line.Unhighlight();
//			}
//		}
		elements.remove(jso);
	}
	
	public void clear()
	{
		for (int i = elements.size() - 1; i >= 0; i--)
		{
			removeHelper(elements.get(i));
		}
		
		onSelectionChanged();
	}
	
	public void onSelectionChanged()
	{
		selectionState = selectionState + 1;
		for (ISelectionChangeListener scl : new Vector<ISelectionChangeListener>(selectionChangeListeners))
		{
			scl.onSelectionChanged();
		}
	}
}
