package org.processmining.processprofiler3d.compviz.jogl.drawing;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialTextRenderer;
import org.processmining.visualisation3d.FontData;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.opengl.GL2;

public class JoglTextRenderer extends JoglSceneObject
{
	private static MaterialTextRenderer matTR = new MaterialTextRenderer();
	
	//public static Texture tex;
	private JoglShader shader;
	private JoglTexturedMesh mesh;
	
	private int charNum = 0;
	private String str;

	private FontFormat textFormat;
	private FontData font;

	private JoglVectord3 position = new JoglVectord3(0,0,0);
	private JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateIdentity();

	public JoglTextRenderer()
	{
		textFormat = new FontFormat();
		font = (FontData)JoglResourceManager.GetResource("DefaultFont");
		
		if (font == null)
		{
			System.out.println("JoglTextRenderer: no font data present!");
		}
	}
	
	public JoglTextRenderer(FontFormat ff)
	{
		textFormat = ff;
		font = (FontData)JoglResourceManager.GetResource("DefaultFont");

		if (font == null)
		{
			System.out.println("JoglTextRenderer: no font data present!");
		}
	}
	
	public void setColor(float r, float g, float b)
	{
		textFormat.setColor(r, g, b);
	}
	
	public JoglVectord3 getPosition()
	{
		return position;
	}
	
	public void setPosition(double x, double y, double z)
	{
		position = new JoglVectord3(x,y,z);
		transform = JoglMatrixd4x4.CreateTranslation(x, y, z);
	}

	public void setPosition(JoglVectord3 position) 
	{
		this.position = position;
		transform = JoglMatrixd4x4.CreateTranslation(position.getX(), position.getY(), position.getZ());
	}
	
	public void setString(String s)
	{
		str = s;
		
		JoglTexturedMesh m = new JoglTexturedMesh(GL2.GL_TRIANGLES);
		m.setVerts(GenerateGeometry(str, textFormat));
		m.setTexCoords(GenerateTextureCoords(str));
		setMesh(m);
		
		charNum = JoglTextRenderer.getCharNum(str);
	}
	
	public void setString(String s, double maxWidth)
	{
		str = s;
		
		JoglTexturedMesh m = new JoglTexturedMesh(GL2.GL_QUADS, 0);
		m.setVerts(GenerateGeometry(str, textFormat, maxWidth));
		m.setTexCoords(GenerateTextureCoords(str));
		setMesh(m);
		
		charNum = JoglTextRenderer.getCharNum(str);
	}
	
	public void setTextSize(float tx, float ty)
	{
		textFormat.width = tx;
		textFormat.height = ty;
	}
	
	public JoglTexturedMesh getMesh()
	{
		return mesh;
	}
	
	public void setMesh(JoglTexturedMesh m)
	{
		mesh = m;
	}
	
	public static int getCharNum(String s)
	{
		int cNum = 0;
		for (int i = 0; i < s.length(); i++)
		{
			char c = s.charAt(i);
			if (c != ' ')
			{
				cNum = cNum + 1;
			}
		}
		
		return cNum;
	}
	
	private double MeasureLineLength(String line, FontFormat ff)
	{
		double lineWidth = 0.0;
		for (int i = 0; i < line.length(); i++)
		{
			char c = line.charAt(i);
			double w = font.GetCharWidth(c, ff.width);
			lineWidth = lineWidth + w + (ff.tweenChars * ff.width);
		}
		lineWidth = lineWidth - (ff.tweenChars * ff.width);
		return lineWidth;
	}

	private Vector<String> BreakTextIntoLines(String text, FontFormat ff, double twidth) 
	{
		Vector<String> lines = new Vector<String>();
		double lineWidth = 0.0;
		int splitPos = -1;
		String line = text;
		int ix = 0;
		while (ix < line.length())
		{
			lineWidth = lineWidth + font.GetCharWidth(line.charAt(ix), ff.width);
			if (line.charAt(ix) == ' ')
			{
				splitPos = ix;
			}
			
			if (lineWidth > twidth)
			{
				//need to split string
				if (splitPos > -1)
				{
					lines.add(line.substring(0, splitPos).trim());
					line = line.substring(splitPos);
					splitPos = -1;
					ix = 1;
				}
				else
				{
					lines.add(line.substring(0, ix-1).trim());
					line = line.substring(ix-1);
					ix = 0;
				}
				lineWidth = 0.0;
			}
			if (lineWidth > 0.0)
			{
				lineWidth = lineWidth + (ff.tweenChars * ff.width);
			}
			ix++;
		}
		lines.add(line.trim());
		return lines;
	}
	
	public Vector<JoglVectord3> GenerateGeometry(String text, FontFormat ff) 
	{
		Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();

		GenerateLineGeometry(text, ff, 0.0, vlist);
		
		return vlist;
	}
	
	public Vector<JoglVectord3> GenerateGeometry(String text, FontFormat ff, double twidth) 
	{
		Vector<String> lines = BreakTextIntoLines(text, ff, twidth);
		
		Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();

		//center
		double yoff = ((lines.size() - 1) * ff.height * 0.5);
		for (int j = 0; j < lines.size(); j++)
		{
			String lstr = lines.get(j);
			GenerateLineGeometry(lstr, ff, yoff, vlist);
			yoff = yoff - ff.height;
		}
		
		return vlist;
	}

	private void GenerateLineGeometry(String lstr, FontFormat ff, double yoff, Vector<JoglVectord3> vlist) 
	{
		double halfwidth = ff.width * 0.5;
		double halfheight = ff.height * 0.5;
		
		//double xoff = ((lstr.length()) * ff.width * 0.5);
		double xoff = 0.0;
		double lineWidth2 = MeasureLineLength(lstr, ff);

		//center
		if (ff.align == FontFormat.ALIGN_CENTER)
		{
			xoff = lineWidth2 * 0.5;
		}
		//left
		else if (ff.align == FontFormat.ALIGN_LEFT)
		{
			xoff = 0.0;
		}
		//right
		else if (ff.align == FontFormat.ALIGN_RIGHT)
		{
			xoff = lineWidth2 * 1.0;
		}
		
		for (int i = 0; i < lstr.length(); i++)
		{
			double cwidth = font.GetCharWidth(lstr.charAt(i), ff.width);
			if (lstr.charAt(i) != ' ')
			{
				vlist.add(new JoglVectord3(xoff - cwidth, 0.0, halfheight + yoff));   // Top Left
				vlist.add(new JoglVectord3(xoff, 0.0, halfheight + yoff));   // Top Right
				vlist.add(new JoglVectord3(xoff, 0.0, -halfheight + yoff));   // Bottom Right
				vlist.add(new JoglVectord3(xoff - cwidth, 0.0, -halfheight + yoff));   // Bottom Left
			}
			xoff = xoff - (cwidth + (ff.tweenChars * ff.width));
		}
	}
	
	public Vector<JoglVectord2> GenerateTextureCoords(String text) 
	{
		double xmin = 0.0;
		double xmax = 1.0;
		double ymin = 0.0;
		double ymax = 1.0;
		
		double invx = 1.0 / 16.0f;
		
		Vector<JoglVectord2> vlist = new Vector<JoglVectord2>();
		
		for (int i = 0; i < text.length(); i++)
		{
			char c = text.charAt(i);
			if (c != ' ')
			{
				int ic = c - 32;
	
				int x = ic % 16;
				int y = (ic - x) / 16;

				float left = (float)font.GetCharLeft(c);
				float right = (float)font.GetCharRight(c);
				
				xmin = x * invx;
				xmax = xmin + invx;
				ymin = 1.0 - (y * invx);
				ymax = 1.0 - ((y * invx) + invx);
				
				vlist.add(new JoglVectord2(xmax - right, ymin));   // Top Left
				vlist.add(new JoglVectord2(xmin + left, ymin));   // Top Right
				vlist.add(new JoglVectord2(xmin + left, ymax));   // Bottom Right
				vlist.add(new JoglVectord2(xmax - right, ymax));   // Bottom Left
			}
		}
		
		return vlist;
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
//		if (tex == null)
//		{
//			//load textures
//			LoadTexture(gl);
//		}

		// Draw A Quad
		DrawWithTextureOnly(pipe);
	}

	private void DrawWithTextureOnly(GraphicsPipeline pipe) 
	{
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);

		JoglTexture tex = font.getTexture(pipe);
		tex.Bind(pipe);
		
		pipe.PushPipelineState(matTR);
		matTR.DrawColor(textFormat.colorR, textFormat.colorG, textFormat.colorB, 1.0f);
		//gl.glBegin(GL2.GL_QUADS);
		
//		int vertsOffset = 0;
//		int texCoordsOffset = 0;
//		for (int i = 0; i < charNum * 4; i++)
//		{
//			gl.glTexCoord2dv(texcsd, texCoordsOffset);
//			gl.glVertex3dv(vertsd, vertsOffset);
//			
//			vertsOffset = vertsOffset + 3;
//			texCoordsOffset = texCoordsOffset + 2;
//		}
		mesh.draw(pipe);
		
		// Done Drawing The Quad
		//gl.glEnd();
		tex.Unbind(pipe);
		pipe.PopPipelineState(matTR);

		pipe.transform.Pop();
	}
}
