package org.processmining.processprofiler3d.compviz.jogl.drawing;

import java.awt.geom.Rectangle2D;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.fileio.BMFontFile;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialTextRendererDF;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.opengl.GL2;

public class JoglTextRendererDF extends JoglSceneObject
{
	private static MaterialTextRendererDF matTR = new MaterialTextRendererDF();
	
    Vector<JoglVectord3> verts;
    Vector<JoglVectord2> texCoords;
	private JoglTexturedMesh mesh;

    private int charNum = 0;
    private String str;

    private static BMFontFile bmf = null;
    private FontFormat textFormat;

    private JoglVectord3 position = new JoglVectord3(0,0,0);
    private JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateIdentity();

    public JoglTextRendererDF(FontFormat ff)
    {
	    textFormat = ff;
        if (bmf == null)
        {
            bmf = new BMFontFile();
            bmf.Load("/org/processmining/processprofiler3d/compviz/jogl/resources/freesansDF.fnt");
            matTR.setBMF(bmf);
        }
    }

    public void setColor(float r, float g, float b)
    {
	    textFormat.setColor(r, g, b);
    }

    public JoglVectord3 getPosition()
    {
        return position;
    }

    public void setPosition(JoglVectord3 position) 
    {
	    this.position = position;
	    transform = JoglMatrixd4x4.CreateTranslation(position);
    }

    public String getString()
    {
        return str;
    }

    public void setString(String s)
    {
	    str = s;
	
        verts = GenerateGeometry(str, textFormat);
        texCoords = GenerateTextureCoords(str);

		JoglTexturedMesh m = new JoglTexturedMesh(GL2.GL_QUADS);
		m.setVerts(GenerateGeometry(str, textFormat));
		m.setTexCoords(GenerateTextureCoords(str));
		setMesh(m);
		
        charNum = JoglTextRendererDF.getCharNum(str);
    }

    public void setString(String s, double maxWidth)
    {
	    str = s;
	
        verts = GenerateGeometry(str, textFormat, maxWidth);
        texCoords = GenerateTextureCoords(str);

		JoglTexturedMesh m = new JoglTexturedMesh(GL2.GL_QUADS);
		m.setVerts(GenerateGeometry(str, textFormat));
		m.setTexCoords(GenerateTextureCoords(str));
		setMesh(m);
		
        charNum = JoglTextRendererDF.getCharNum(str);
    }

    public void setTextSize(float tx, float ty)
    {
	    textFormat.width = tx;
	    textFormat.height = ty;
    }
	
	public JoglTexturedMesh getMesh()
	{
		return mesh;
	}
	
	public void setMesh(JoglTexturedMesh m)
	{
		mesh = m;
	}

    public static int getCharNum(String s)
    {
	    int cNum = 0;
	    for (int i = 0; i < s.length(); i++)
	    {
		    char c = s.charAt(i);
		    if (c != ' ')
		    {
			    cNum = cNum + 1;
		    }
	    }
	
	    return cNum;
    }

    private double MeasureLineLength(String line, FontFormat ff)
    {
	    double lineWidth = 0.0;
        double wScale = bmf.GetFontBaseScale() * ff.width;
	    for (int i = 0; i < line.length(); i++)
	    {
            char c = line.charAt(i);
            char cn = ' ';
            if (i < line.length() - 1)
            {
                cn = line.charAt(i + 1);
            }
		    //double w = font.GetCharWidth(c, ff.width);
            //lineWidth = lineWidth + w + (ff.tweenChars * ff.width);
            double w = (bmf.GetCharAdvance(c) * wScale);
            if (cn != ' ')
            {
                lineWidth = lineWidth + w + (bmf.GetKerningFor("" + c + cn) * wScale);
            }
            else
            {
                lineWidth = lineWidth + w;
            }
	    }
	    //lineWidth = lineWidth - (ff.tweenChars * ff.width);
	    return lineWidth;
    }

    private Vector<String> BreakTextIntoLines(String text, FontFormat ff, double twidth) 
    {
    	Vector<String> lines = new Vector<String>();
	    double lineWidth = 0.0;
	    int splitPos = -1;
	    String line = text;
	    int ix = 0;
        double wScale = bmf.GetFontBaseScale() * ff.width;
	    while (ix < line.length())
	    {
            //lineWidth = lineWidth + font.GetCharWidth(line[ix], ff.width);
            lineWidth = lineWidth + (bmf.GetCharAdvance(line.charAt(ix)) * wScale);
		    if (line.charAt(ix) == ' ')
		    {
			    splitPos = ix;
		    }
		
		    if (lineWidth > twidth)
		    {
			    //need to split string
			    if (splitPos > -1)
			    {
				    lines.add(line.substring(0, splitPos).trim());
				    line = line.substring(splitPos);
				    splitPos = -1;
				    ix = 1;
			    }
			    else
			    {
				    lines.add(line.substring(0, ix-1).trim());
				    line = line.substring(ix-1);
				    ix = 0;
			    }
			    lineWidth = 0.0;
		    }
            //if (lineWidth > 0.0)
            //{
            //    lineWidth = lineWidth + (ff.tweenChars * ff.width);
            //}
		    ix++;
	    }
	    lines.add(line.trim());
	    return lines;
    }

    public Vector<JoglVectord3> GenerateGeometry(String text, FontFormat ff) 
    {
    	Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();

	    textRect = GenerateLineGeometry(text, ff, 0.0, vlist);
	
	    return vlist;
    }

    public Vector<JoglVectord3> GenerateGeometry(String text, FontFormat ff, double twidth)
    {
        float minx = Float.MAX_VALUE;
        float miny = Float.MAX_VALUE;
        float maxx = Float.MIN_VALUE;
        float maxy = Float.MIN_VALUE;

	    Vector<String> lines = BreakTextIntoLines(text, ff, twidth);
	
	    Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();

	    //center
        //double yoff = ((lines.Count - 1) * ff.height * 0.5);
        double lineHeight = bmf.GetLineHeight() * bmf.GetFontBaseScale() * ff.height;
        double yoff = ((lines.size() - 1) * lineHeight * 0.5);
	    for (int j = 0; j < lines.size(); j++)
	    {
		    String lstr = lines.get(j);
            Rectangle2D.Float lineRect = GenerateLineGeometry(lstr, ff, yoff, vlist);

            minx = (float)Math.min(minx, lineRect.getMinX());
            maxx = (float)Math.max(maxx, lineRect.getMaxX());

            miny = (float)Math.min(miny, lineRect.getMinY());
            maxy = (float)Math.max(maxy, lineRect.getMaxY());

            yoff = yoff - lineHeight;
	    }

        textRect = new Rectangle2D.Float(minx, miny, maxx - minx, maxy - miny);
	
	    return vlist;
    }

    private Rectangle2D.Float GenerateLineGeometry(String lstr, FontFormat ff, double yoff, Vector<JoglVectord3> vlist) 
    {
        float minx = Float.MAX_VALUE;
        float miny = Float.MAX_VALUE;
        float maxx = Float.MIN_VALUE;
        float maxy = Float.MIN_VALUE;

	    double halfwidth = ff.width * 0.5;
	    double halfheight = ff.height * 0.5;
	
	    //double xoff = ((lstr.length()) * ff.width * 0.5);
	    double xoff = 0.0;

	    //center
	    if (ff.align == FontFormat.ALIGN_CENTER)
        {
            double lineWidth2 = MeasureLineLength(lstr, ff);
		    xoff = lineWidth2 * -0.5;
	    }
	    //left
	    else if (ff.align == FontFormat.ALIGN_LEFT)
	    {
		    xoff = 0.0;
	    }
	    //right
	    else if (ff.align == FontFormat.ALIGN_RIGHT)
        {
            double lineWidth2 = MeasureLineLength(lstr, ff);
		    xoff = lineWidth2 * -1.0;
	    }

        float baseOffset = bmf.GetBaseHeight() * bmf.GetFontBaseScale() * (float)ff.height;
        float wScale = bmf.GetFontBaseScale() * (float)ff.width;
        float hScale = bmf.GetFontBaseScale() * (float)ff.height;
	    for (int i = 0; i < lstr.length(); i++)
	    {
            //double cwidth = font.GetCharWidth(lstr[i], ff.width);
	    	Rectangle2D.Float crect = bmf.GetCharRect(lstr.charAt(i));
	    	Rectangle2D.Float crectSc = new Rectangle2D.Float((float)(crect.getX() * wScale), (float)(crect.getY() * hScale), (float)(crect.getWidth() * wScale), (float)(crect.getHeight() * hScale));
		    if (lstr.charAt(i) != ' ')
		    {
                double xr = xoff + crectSc.getMaxX();
                double xl = xoff + crectSc.getMinX();
                double yt = crectSc.getMinY() - baseOffset - yoff;
                double yb = crectSc.getMaxY() - baseOffset - yoff;

                JoglVectord3 v1 = new JoglVectord3(xr, yt, 1.0f);   // Top Left
                JoglVectord3 v2 = new JoglVectord3(xl, yt, 1.0f);   // Top Right
                JoglVectord3 v3 = new JoglVectord3(xl, yb, 1.0f);   // Bottom Right
                JoglVectord3 v4 = new JoglVectord3(xr, yb, 1.0f);   // Bottom Left

                vlist.add(v1);   // Top Left
                vlist.add(v2);   // Top Right
                vlist.add(v3);   // Bottom Right
                vlist.add(v4);   // Bottom Left

                minx = (float)Math.min(minx, xl);
                maxx = (float)Math.max(maxx, xr);

                miny = (float)Math.min(miny, yt);
                maxy = (float)Math.max(maxy, yb);
		    }
            //xoff = xoff + (cwidth + (ff.tweenChars * ff.width));
            xoff = xoff + (bmf.GetCharAdvance(lstr.charAt(i)) * wScale);

            //kerning
            if (i < lstr.length() - 1)
            {
                char cn = lstr.charAt(i + 1);
                String s = "" + lstr.charAt(i) + cn;
                float kern = bmf.GetKerningFor(s);
                if (kern != 0.0f)
                {
                    xoff = xoff + (kern * wScale);
                }
            }
	    }

        return new Rectangle2D.Float(minx, miny, maxx - minx, maxy - miny);
    }

    public Vector<JoglVectord2> GenerateTextureCoords(String text) 
    {
        //double xmin = 0.0;
        //double xmax = 1.0;
        //double ymin = 0.0;
        //double ymax = 1.0;
	
	    //double invx = 1.0 / 16.0f;
	
	    Vector<JoglVectord2> vlist = new Vector<JoglVectord2>();
	
	    for (int i = 0; i < text.length(); i++)
	    {
		    char c = text.charAt(i);
		    if (c != ' ')
		    {
                //int ic = c - 32;

                //int x = ic % 16;
                //int y = (ic - x) / 16;

                //float left = (float)font.GetCharLeft(c);
                //float right = (float)font.GetCharRight(c);
			
                //xmin = x * invx;
                //xmax = xmin + invx;
                //ymin = (y * invx);
                //ymax = ((y * invx) + invx);
			
                //vlist.Add(new Vector2d(xmax - right, ymin));   // Top Left
                //vlist.Add(new Vector2d(xmin + left, ymin));   // Top Right
                //vlist.Add(new Vector2d(xmin + left, ymax));   // Bottom Right
                //vlist.Add(new Vector2d(xmax - right, ymax));   // Bottom Left
                Rectangle2D.Float rect = bmf.GetCharTexRect(c);

                vlist.add(new JoglVectord2(rect.getMaxX(), 1.0 - rect.getMinY()));
                vlist.add(new JoglVectord2(rect.getMinX(), 1.0 - rect.getMinY()));
                vlist.add(new JoglVectord2(rect.getMinX(), 1.0 - rect.getMaxY()));
                vlist.add(new JoglVectord2(rect.getMaxX(), 1.0 - rect.getMaxY()));
		    }
	    }
	
	    return vlist;
    }

    private Rectangle2D.Float textRect;

    public Rectangle2D.Float GetRect()
    {
        return textRect;
    }

    @Override
    public void DrawObject(GraphicsPipeline pipe)
    {
//		if (tex == null)
//		{
//			//load textures
//			LoadTexture(gl);
//		}

	    // Draw A Quad
	    DrawWithTextureOnly(pipe);
    }

    private void DrawWithTextureOnly(GraphicsPipeline pipe) 
    {
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);
		pipe.transform.MultMatrix(JoglMatrixd4x4.CreateScale(1, -1, 1));
		pipe.transform.MultMatrix(JoglMatrixd4x4.CreateTranslation(0, -10, 0));

	    pipe.PushPipelineState(matTR);
	    matTR.DrawColor(1.0f, 0.0f, 0.0f, 1.0f);

	    //int texcoord_index = gl.glGetAttribLocation(shad.getProgramHandle(), "in_uvcoord");
//        gl.glBegin(GL2.GL_QUADS);
//        for (int i = 0; i < verts.size(); i++)
//        {
//        	gl.glTexCoord2d(texCoords.get(i).getX(), texCoords.get(i).getY());
//        	gl.glVertex3d(verts.get(i).getX(), verts.get(i).getY(), verts.get(i).getZ());
//        }
//        gl.glEnd();
		mesh.draw(pipe);
	
	    // Done Drawing The Quad
	    
	    pipe.PopPipelineState(matTR);

		pipe.transform.Pop();
    }
}
