package org.processmining.processprofiler3d.compviz.jogl.drawing;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglMesh;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.gldatastructures.IJoglVertexArrayObjectBuffer;
import org.processmining.visualisation3d.gldatastructures.JoglVertexArrayObject;
import org.processmining.visualisation3d.gldatastructures.JoglVertexArrayObjectBuffer;
import org.processmining.visualisation3d.gldatastructures.JoglVertexArrayObjectIndexBuffer;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshDataBufferVectord2;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshDataBufferVectord3;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshDataBufferVectord4;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;

import com.jogamp.opengl.GL2;

public class VAODrawer implements IMeshDrawer
{
	private JoglVertexArrayObject vao;
	private JoglVertexArrayObjectIndexBuffer indbuf;
	
	public VAODrawer()
	{
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#LoadMesh(org.processmining.visualisation3d.GraphicsPipeline, org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglMesh)
	 */
	@Override
	public void LoadMesh(GraphicsPipeline pipe, JoglMesh mesh)
	{
		if (mesh.getNumVerts() == 0)
		{
			return;
		}
		
        //System.out.println("Model vert num: " + mesh.getNumVerts());
        
		JoglMeshIndexBuffer indata = mesh.getIndexBuffer();
        indbuf = indata.getVAOBuffer(pipe);
        
        vao = new JoglVertexArrayObject(pipe);

        vao.Bind(pipe);

        for (JoglVertexArrayObjectBuffer vbuf : mesh.getVAOBuffers(pipe))
        {
        	vao.AttachBuffer(vbuf); 	
        }

        vao.AttachIndexBuffer(indbuf);
        vao.UpdateBufferAttachment(pipe);

        vao.Unbind(pipe);
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#UpdateDataBufferJV2(org.processmining.visualisation3d.GraphicsPipeline, int, java.util.Vector)
	 */
	@Override
	public void UpdateDataBufferJV2(GraphicsPipeline pipe, int bufNum, Vector<JoglVectord2> vlst)
	{
		Vector<IJoglVertexArrayObjectBuffer> lst = vao.getBuffers();
		
		JoglVertexArrayObjectBuffer vbuf = (JoglVertexArrayObjectBuffer)lst.get(bufNum);
		JoglMeshDataBufferVectord2 dbuf = new JoglMeshDataBufferVectord2();
		dbuf.set(vlst);
		
		vbuf.Bind(pipe);
		vbuf.UpdateData(pipe, dbuf.getNativeBuffer());
		vbuf.Unbind(pipe);
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#UpdateDataBufferJV3(org.processmining.visualisation3d.GraphicsPipeline, int, java.util.Vector)
	 */
	@Override
	public void UpdateDataBufferJV3(GraphicsPipeline pipe, int bufNum, Vector<JoglVectord3> vlst)
	{
		Vector<IJoglVertexArrayObjectBuffer> lst = vao.getBuffers();
		
		JoglVertexArrayObjectBuffer vbuf = (JoglVertexArrayObjectBuffer)lst.get(bufNum);
		JoglMeshDataBufferVectord3 dbuf = new JoglMeshDataBufferVectord3();
		dbuf.set(vlst);
		
		vbuf.Bind(pipe);
		vbuf.UpdateData(pipe, dbuf.getNativeBuffer());
		vbuf.Unbind(pipe);
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#UpdateDataBufferJV4(org.processmining.visualisation3d.GraphicsPipeline, int, java.util.Vector)
	 */
	@Override
	public void UpdateDataBufferJV4(GraphicsPipeline pipe, int bufNum, Vector<JoglVectord4> vlst)
	{
		Vector<IJoglVertexArrayObjectBuffer> lst = vao.getBuffers();
		
		JoglVertexArrayObjectBuffer vbuf = (JoglVertexArrayObjectBuffer)lst.get(bufNum);
		JoglMeshDataBufferVectord4 dbuf = new JoglMeshDataBufferVectord4();
		dbuf.set(vlst);
		
		vbuf.Bind(pipe);
		vbuf.UpdateData(pipe, dbuf.getNativeBuffer());
		vbuf.Unbind(pipe);
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#UpdateDataBufferJV2(org.processmining.visualisation3d.GraphicsPipeline, int, java.util.Vector)
	 */
	@Override
	public void UpdateDataBufferJV2(GraphicsPipeline pipe, int bufNum, JoglMeshDataBufferVectord2 dbuf)
	{
		Vector<IJoglVertexArrayObjectBuffer> lst = vao.getBuffers();
		
		JoglVertexArrayObjectBuffer vbuf = (JoglVertexArrayObjectBuffer)lst.get(bufNum);
		
		vbuf.Bind(pipe);
		vbuf.UpdateData(pipe, dbuf.getNativeBuffer());
		vbuf.Unbind(pipe);
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#UpdateDataBufferJV3(org.processmining.visualisation3d.GraphicsPipeline, int, java.util.Vector)
	 */
	@Override
	public void UpdateDataBufferJV3(GraphicsPipeline pipe, int bufNum, JoglMeshDataBufferVectord3 dbuf)
	{
		Vector<IJoglVertexArrayObjectBuffer> lst = vao.getBuffers();
		
		JoglVertexArrayObjectBuffer vbuf = (JoglVertexArrayObjectBuffer)lst.get(bufNum);
		
		vbuf.Bind(pipe);
		vbuf.UpdateData(pipe, dbuf.getNativeBuffer());
		vbuf.Unbind(pipe);
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#UpdateDataBufferJV4(org.processmining.visualisation3d.GraphicsPipeline, int, java.util.Vector)
	 */
	@Override
	public void UpdateDataBufferJV4(GraphicsPipeline pipe, int bufNum, JoglMeshDataBufferVectord4 dbuf)
	{
		Vector<IJoglVertexArrayObjectBuffer> lst = vao.getBuffers();
		
		JoglVertexArrayObjectBuffer vbuf = (JoglVertexArrayObjectBuffer)lst.get(bufNum);
		
		vbuf.Bind(pipe);
		vbuf.UpdateData(pipe, dbuf.getNativeBuffer());
		vbuf.Unbind(pipe);
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#Dispose()
	 */
	@Override
	public void Dispose()
	{
		if (vao != null)
		{
			for (IJoglVertexArrayObjectBuffer buf : vao.getBuffers())
			{
				JoglPanel.gc.QueueForDisposal(buf);
			}
			vao.ClearBuffers();
			
			JoglPanel.gc.QueueForDisposal(indbuf);
			indbuf = null;
			JoglPanel.gc.QueueForDisposal(vao);
			vao = null;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#Bind(org.processmining.visualisation3d.GraphicsPipeline)
	 */
	@Override
	public void Bind(GraphicsPipeline pipe)
	{
		if (vao != null)
		{
			vao.Bind(pipe);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#Unbind(org.processmining.visualisation3d.GraphicsPipeline)
	 */
	@Override
	public void Unbind(GraphicsPipeline pipe)
	{
		if (vao != null)
		{
			vao.Unbind(pipe);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#DrawSlim(org.processmining.visualisation3d.GraphicsPipeline)
	 */
	@Override
	public void DrawSlim(GraphicsPipeline pipe)
	{
		if (vao != null)
		{
			indbuf.Draw(pipe.gl, GL2.GL_TRIANGLES);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#DrawSlim(org.processmining.visualisation3d.GraphicsPipeline, int, int)
	 */
	@Override
	public void DrawSlim(GraphicsPipeline pipe, int offset, int count)
	{
		if (vao != null)
		{
			indbuf.Draw(pipe.gl, GL2.GL_TRIANGLES, offset, count);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#Draw(org.processmining.visualisation3d.GraphicsPipeline)
	 */
	@Override
	public void Draw(GraphicsPipeline pipe)
	{
		if (vao != null)
		{
			vao.Bind(pipe);
			indbuf.Draw(pipe.gl, GL2.GL_TRIANGLES);
			vao.Unbind(pipe);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#Draw(org.processmining.visualisation3d.GraphicsPipeline, int, int)
	 */
	@Override
	public void Draw(GraphicsPipeline pipe, int offset, int count)
	{
		if (vao != null)
		{
	        vao.Bind(pipe);
	        indbuf.Draw(pipe.gl, GL2.GL_TRIANGLES, offset, count);
	        vao.Unbind(pipe);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#DrawLines(org.processmining.visualisation3d.GraphicsPipeline)
	 */
	@Override
	public void DrawLines(GraphicsPipeline pipe)
	{
		if (vao != null)
		{
	        vao.Bind(pipe);
	        indbuf.Draw(pipe.gl, GL2.GL_LINES);
	        vao.Unbind(pipe);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#DrawLines(org.processmining.visualisation3d.GraphicsPipeline, int, int)
	 */
	@Override
	public void DrawLines(GraphicsPipeline pipe, int offset, int count)
	{
		if (vao != null)
		{
	        vao.Bind(pipe);
	        indbuf.Draw(pipe.gl, GL2.GL_LINES, offset, count);
	        vao.Unbind(pipe);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#DrawLinesSlim(org.processmining.visualisation3d.GraphicsPipeline)
	 */
	@Override
	public void DrawLinesSlim(GraphicsPipeline pipe)
	{
		if (vao != null)
		{
			indbuf.Draw(pipe.gl, GL2.GL_LINES);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer#DrawLinesSlim(org.processmining.visualisation3d.GraphicsPipeline, int, int)
	 */
	@Override
	public void DrawLinesSlim(GraphicsPipeline pipe, int offset, int count)
	{
		if (vao != null)
		{
			indbuf.Draw(pipe.gl, GL2.GL_LINES, offset, count);
		}
	}
}
