package org.processmining.processprofiler3d.compviz.jogl.gui;

import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.visualisation3d.FontData;

public class FormattedFontCache
{
	public double[] cwidth;
	public double[] uvCache;
	public double tweenCharsScaled = 0;
	
	public FormattedFontCache(FontData font, FontFormat textFormat)
	{
		tweenCharsScaled = textFormat.tweenChars * textFormat.width;
		
		GenerateTest(font, textFormat);
		GenerateTextureCoords(font);
	}
	
	public void GenerateTest(FontData font, FontFormat textFormat)
	{
		cwidth = new double[256];
		for (int i = 0; i < 256; i++)
		{
			char c = (char)(i+32);
			double w = font.GetCharWidth(c, textFormat.width);
			cwidth[i] = w;
		}
	}

	public void GenerateTextureCoords(FontData font) 
	{
		uvCache = new double[256 * 8];
		
		double xmin = 0.0;
		double xmax = 1.0;
		double ymin = 0.0;
		double ymax = 1.0;
		
		double invx = 1.0 / 16.0f;
		
		int uvPointer = 0;
		
		for (int i = 0; i < 256; i++)
		{
			char c = (char)(i+32);
			if (c != ' ')
			{
				int ic = c - 32;
	
				int x = ic % 16;
				int y = (ic - x) / 16;
				
				float left = (float)font.GetCharLeft(c);
				float right = (float)font.GetCharRight(c);
				
				xmin = x * invx;
				xmax = xmin + invx;
				ymin = 1.0 - (y * invx);
				ymax = 1.0 - ((y * invx) + invx);
				
				double uvLeft = xmin + left;
				double uvRight = xmax - right;
				double uvTop = ymin;
				double uvBottom = ymax;
				
				uvCache[uvPointer] = uvLeft;
				uvCache[uvPointer + 1] = uvTop;

				uvCache[uvPointer + 2] = uvRight;
				uvCache[uvPointer + 3] = uvTop;

				uvCache[uvPointer + 4] = uvRight;
				uvCache[uvPointer + 5] = uvBottom;

				uvCache[uvPointer + 6] = uvLeft;
				uvCache[uvPointer + 7] = uvBottom;
				
				uvPointer = uvPointer + 8;
				
//				vlist.add(new JoglVectord2(xmin + left, ymin));   // Top Left
//				vlist.add(new JoglVectord2(xmax - right, ymin));   // Top Right
//				vlist.add(new JoglVectord2(xmax - right, ymax));   // Bottom Right
//				vlist.add(new JoglVectord2(xmin + left, ymax));   // Bottom Left
			}
		}
	}
}
