package org.processmining.processprofiler3d.compviz.jogl.gui;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglColoredMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderPipelineDefaultColored;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;

import com.jogamp.opengl.GL2;

public class JoglGuiCheckBox extends JoglGuiObject
{
	private float colorR = 0.0f;
	private float colorG = 0.0f;
	private float colorB = 0.0f;
	
	private float backcolorR = 0.0f;
	private float backcolorG = 0.0f;
	private float backcolorB = 0.0f;
	private float backcolorA = 1.0f;
	
	private JoglScreenRect rect = new JoglScreenRect(0,0,1,1);
	
	private float lineWidth = 0.01f;
	private float lineHeight = 0.01f;
	
	private boolean bChecked = false;

	private boolean bCanTouch = false;
	
	private boolean bUpdateVerts = false;
	private boolean bUpdateColors = false;

	@Override
	public boolean getCanTouch()
	{
		return bCanTouch;
	}

	@Override
	public void setCanTouch(boolean b)
	{
		bCanTouch = b;
	}
	
	public boolean getChecked()
	{
		return bChecked;
	}
	
	public void setChecked(boolean b)
	{
		bChecked = b;
	}
	
	public void setBackColor(float r, float g, float b)
	{
		backcolorR = r;
		backcolorG = g;
		backcolorB = b;

		bUpdateColors = true;
	}
	
	public void setBackColor(float r, float g, float b, float a)
	{
		backcolorR = r;
		backcolorG = g;
		backcolorB = b;
		backcolorA = a;

		bUpdateColors = true;
	}
	
	public void setBackColor(JoglVectord3 v)
	{
		backcolorR = (float)v.getX();
		backcolorG = (float)v.getY();
		backcolorB = (float)v.getZ();

		bUpdateColors = true;
	}
	
	public void setBackColor(JoglVectord4 v)
	{
		backcolorR = (float)v.getX();
		backcolorG = (float)v.getY();
		backcolorB = (float)v.getZ();
		backcolorA = (float)v.getValue(3);

		bUpdateColors = true;
	}
	
	public void setColor(float r, float g, float b)
	{
		colorR = r;
		colorG = g;
		colorB = b;

		bUpdateColors = true;
	}
	
	public void setColor(JoglVectord3 v)
	{
		colorR = (float)v.getX();
		colorG = (float)v.getY();
		colorB = (float)v.getZ();

		bUpdateColors = true;
	}
	
	public JoglVectord4 getRect()
	{
		return rect.getRect();
	}
	
	public void setRect(float x, float y, float w, float h)
	{
		rect = new JoglScreenRect(x, y, w, h);

		bUpdateVerts = true;
	}
	
	public void setRect(JoglVectord4 v)
	{
		rect = new JoglScreenRect(v);

		bUpdateVerts = true;
	}
	
	public void setLineDims(float lineWidth, float lineHeight)
	{
		this.lineWidth = lineWidth;
		this.lineHeight = lineHeight;

		bUpdateVerts = true;
	}
	
	private void updateVAOVerts(GraphicsPipeline pipe)
	{
		if (vao != null)
		{
			vao.UpdateDataBufferJV3(pipe, 0, GenerateGeometry());
		}
	}
	
	private void updateVAOColors(GraphicsPipeline pipe)
	{
		if (vao != null)
		{
			vao.UpdateDataBufferJV4(pipe, 1, GenerateColors());
		}
	}
	
	@Override
	public boolean IsInside(float x, float y)
	{
		if (bCanTouch)
		{
			if (rect.IsInside(x, y))
			{
				return true;
			}
		}
		return false;
	}
	
	private Vector<JoglVectord3> GenerateGeometry()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();

		lst.add(new JoglVectord3(rect.getLeft(), rect.getTop(), 0.0f));
		lst.add(new JoglVectord3(rect.getRight(), rect.getTop(), 0.0f));
		lst.add(new JoglVectord3(rect.getRight(), rect.getBottom(), 0.0f));
		lst.add(new JoglVectord3(rect.getLeft(), rect.getBottom(), 0.0f));

		if (rect.getHeight() < 0)
		{
			lst.add(new JoglVectord3(rect.getLeft(), rect.getTop(), 0.0f));
			lst.add(new JoglVectord3(rect.getLeft() + lineWidth, rect.getTop(), 0.0f));
			lst.add(new JoglVectord3(rect.getLeft() + lineWidth, rect.getBottom(), 0.0f));
			lst.add(new JoglVectord3(rect.getLeft(), rect.getBottom(), 0.0f));
	
			lst.add(new JoglVectord3(rect.getRight() - lineWidth, rect.getTop(), 0.0f));
			lst.add(new JoglVectord3(rect.getRight(), rect.getTop(), 0.0f));
			lst.add(new JoglVectord3(rect.getRight(), rect.getBottom(), 0.0f));
			lst.add(new JoglVectord3(rect.getRight() - lineWidth, rect.getBottom(), 0.0f));
	
			lst.add(new JoglVectord3(rect.getLeft(), rect.getTop(), 0.0f));
			lst.add(new JoglVectord3(rect.getRight(), rect.getTop(), 0.0f));
			lst.add(new JoglVectord3(rect.getRight(), rect.getTop() - lineHeight, 0.0f));
			lst.add(new JoglVectord3(rect.getLeft(), rect.getTop() - lineHeight, 0.0f));
	
			lst.add(new JoglVectord3(rect.getLeft(), rect.getBottom() + lineHeight, 0.0f));
			lst.add(new JoglVectord3(rect.getRight(), rect.getBottom() + lineHeight, 0.0f));
			lst.add(new JoglVectord3(rect.getRight(), rect.getBottom(), 0.0f));
			lst.add(new JoglVectord3(rect.getLeft(), rect.getBottom(), 0.0f));
		}
		else
		{
			lst.add(new JoglVectord3(rect.getLeft(), rect.getTop(), 0.0f));
			lst.add(new JoglVectord3(rect.getLeft() + lineWidth, rect.getTop(), 0.0f));
			lst.add(new JoglVectord3(rect.getLeft() + lineWidth, rect.getBottom(), 0.0f));
			lst.add(new JoglVectord3(rect.getLeft(), rect.getBottom(), 0.0f));
	
			lst.add(new JoglVectord3(rect.getRight() - lineWidth, rect.getTop(), 0.0f));
			lst.add(new JoglVectord3(rect.getRight(), rect.getTop(), 0.0f));
			lst.add(new JoglVectord3(rect.getRight(), rect.getBottom(), 0.0f));
			lst.add(new JoglVectord3(rect.getRight() - lineWidth, rect.getBottom(), 0.0f));
	
			lst.add(new JoglVectord3(rect.getLeft(), rect.getTop(), 0.0f));
			lst.add(new JoglVectord3(rect.getRight(), rect.getTop(), 0.0f));
			lst.add(new JoglVectord3(rect.getRight(), rect.getTop() + lineHeight, 0.0f));
			lst.add(new JoglVectord3(rect.getLeft(), rect.getTop() + lineHeight, 0.0f));
	
			lst.add(new JoglVectord3(rect.getLeft(), rect.getBottom() - lineHeight, 0.0f));
			lst.add(new JoglVectord3(rect.getRight(), rect.getBottom() - lineHeight, 0.0f));
			lst.add(new JoglVectord3(rect.getRight(), rect.getBottom(), 0.0f));
			lst.add(new JoglVectord3(rect.getLeft(), rect.getBottom(), 0.0f));
		}

		if (rect.getHeight() < 0)
		{
			lst.add(new JoglVectord3(rect.getLeft() + (lineWidth * 2.0f), rect.getTop() + (lineHeight * -2.0f), 0.0f));
			lst.add(new JoglVectord3(rect.getRight() + (lineWidth * -2.0f), rect.getTop() + (lineHeight * -2.0f), 0.0f));
			lst.add(new JoglVectord3(rect.getRight() + (lineWidth * -2.0f), rect.getBottom() + (lineHeight * 2.0f), 0.0f));
			lst.add(new JoglVectord3(rect.getLeft() + (lineWidth * 2.0f), rect.getBottom() + (lineHeight * 2.0f), 0.0f));
		}
		else
		{
			lst.add(new JoglVectord3(rect.getLeft() + (lineWidth * 2.0f), rect.getTop() + (lineHeight * 2.0f), 0.0f));
			lst.add(new JoglVectord3(rect.getRight() + (lineWidth * -2.0f), rect.getTop() + (lineHeight * 2.0f), 0.0f));
			lst.add(new JoglVectord3(rect.getRight() + (lineWidth * -2.0f), rect.getBottom() + (lineHeight * -2.0f), 0.0f));
			lst.add(new JoglVectord3(rect.getLeft() + (lineWidth * 2.0f), rect.getBottom() + (lineHeight * -2.0f), 0.0f));
		}

		return lst;
	}
	
	private Vector<JoglVectord4> GenerateColors()
	{
		Vector<JoglVectord4> lst = new Vector<JoglVectord4>();

		JoglVectord4 backcolor = new JoglVectord4(backcolorR, backcolorG, backcolorB, backcolorA);
		lst.add(backcolor);
		lst.add(backcolor);
		lst.add(backcolor);
		lst.add(backcolor);

		JoglVectord4 color = new JoglVectord4(colorR, colorG, colorB, 1.0f);
		lst.add(color);
		lst.add(color);
		lst.add(color);
		lst.add(color);

		lst.add(color);
		lst.add(color);
		lst.add(color);
		lst.add(color);

		lst.add(color);
		lst.add(color);
		lst.add(color);
		lst.add(color);

		lst.add(color);
		lst.add(color);
		lst.add(color);
		lst.add(color);

		lst.add(color);
		lst.add(color);
		lst.add(color);
		lst.add(color);
		
		return lst;
	}
	
	public JoglColoredMesh CreateMesh()
	{
		JoglColoredMesh jcm = new JoglColoredMesh(GL2.GL_QUADS);
		jcm.setVerts(GenerateGeometry());
		jcm.setColors(GenerateColors());
		jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jcm.getNumVerts()));
		
		return jcm;
	}
	
	private IMeshDrawer vao = null;
	private static JoglShader shad = null;
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		if (bVisible)
		{
//			if (backcolorA < 1.0f)
//			{
////				gl.glEnable(GL.GL_BLEND);
////				gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
//				pipe.setDrawModeAlphaBlended();
//			}
//			
//			gl.glBegin(GL2.GL_QUADS);
//			
//			gl.glColor4f(backcolorR, backcolorG, backcolorB, backcolorA);
//			gl.glVertex3f(rect.getLeft(), rect.getTop(), 0.0f);
//			gl.glVertex3f(rect.getRight(), rect.getTop(), 0.0f);
//			gl.glVertex3f(rect.getRight(), rect.getBottom(), 0.0f);
//			gl.glVertex3f(rect.getLeft(), rect.getBottom(), 0.0f);
//	
//			gl.glEnd();
//	
//			if (backcolorA < 1.0f)
//			{
////				gl.glDisable(GL.GL_BLEND);
////				gl.glBlendFunc(GL.GL_ONE, GL.GL_ZERO);
//				pipe.setDrawModeOpaque();
//			}
//	
//			gl.glBegin(GL2.GL_QUADS);
//			gl.glColor4f(colorR, colorG, colorB, 1.0f);
//	
//			if (rect.getHeight() < 0)
//			{
//				gl.glVertex3f(rect.getLeft(), rect.getTop(), 0.0f);
//				gl.glVertex3f(rect.getLeft() + lineWidth, rect.getTop(), 0.0f);
//				gl.glVertex3f(rect.getLeft() + lineWidth, rect.getBottom(), 0.0f);
//				gl.glVertex3f(rect.getLeft(), rect.getBottom(), 0.0f);
//		
//				gl.glVertex3f(rect.getRight() - lineWidth, rect.getTop(), 0.0f);
//				gl.glVertex3f(rect.getRight(), rect.getTop(), 0.0f);
//				gl.glVertex3f(rect.getRight(), rect.getBottom(), 0.0f);
//				gl.glVertex3f(rect.getRight() - lineWidth, rect.getBottom(), 0.0f);
//		
//				gl.glVertex3f(rect.getLeft(), rect.getTop(), 0.0f);
//				gl.glVertex3f(rect.getRight(), rect.getTop(), 0.0f);
//				gl.glVertex3f(rect.getRight(), rect.getTop() - lineHeight, 0.0f);
//				gl.glVertex3f(rect.getLeft(), rect.getTop() - lineHeight, 0.0f);
//		
//				gl.glVertex3f(rect.getLeft(), rect.getBottom() + lineHeight, 0.0f);
//				gl.glVertex3f(rect.getRight(), rect.getBottom() + lineHeight, 0.0f);
//				gl.glVertex3f(rect.getRight(), rect.getBottom(), 0.0f);
//				gl.glVertex3f(rect.getLeft(), rect.getBottom(), 0.0f);
//			}
//			else
//			{
//				gl.glVertex3f(rect.getLeft(), rect.getTop(), 0.0f);
//				gl.glVertex3f(rect.getLeft() + lineWidth, rect.getTop(), 0.0f);
//				gl.glVertex3f(rect.getLeft() + lineWidth, rect.getBottom(), 0.0f);
//				gl.glVertex3f(rect.getLeft(), rect.getBottom(), 0.0f);
//		
//				gl.glVertex3f(rect.getRight() - lineWidth, rect.getTop(), 0.0f);
//				gl.glVertex3f(rect.getRight(), rect.getTop(), 0.0f);
//				gl.glVertex3f(rect.getRight(), rect.getBottom(), 0.0f);
//				gl.glVertex3f(rect.getRight() - lineWidth, rect.getBottom(), 0.0f);
//		
//				gl.glVertex3f(rect.getLeft(), rect.getTop(), 0.0f);
//				gl.glVertex3f(rect.getRight(), rect.getTop(), 0.0f);
//				gl.glVertex3f(rect.getRight(), rect.getTop() + lineHeight, 0.0f);
//				gl.glVertex3f(rect.getLeft(), rect.getTop() + lineHeight, 0.0f);
//		
//				gl.glVertex3f(rect.getLeft(), rect.getBottom() - lineHeight, 0.0f);
//				gl.glVertex3f(rect.getRight(), rect.getBottom() - lineHeight, 0.0f);
//				gl.glVertex3f(rect.getRight(), rect.getBottom(), 0.0f);
//				gl.glVertex3f(rect.getLeft(), rect.getBottom(), 0.0f);
//			}
//	
//			if (bChecked)
//			{
//				if (rect.getHeight() < 0)
//				{
//					gl.glVertex3f(rect.getLeft() + (lineWidth * 2.0f), rect.getTop() + (lineHeight * -2.0f), 0.0f);
//					gl.glVertex3f(rect.getRight() + (lineWidth * -2.0f), rect.getTop() + (lineHeight * -2.0f), 0.0f);
//					gl.glVertex3f(rect.getRight() + (lineWidth * -2.0f), rect.getBottom() + (lineHeight * 2.0f), 0.0f);
//					gl.glVertex3f(rect.getLeft() + (lineWidth * 2.0f), rect.getBottom() + (lineHeight * 2.0f), 0.0f);
//				}
//				else
//				{
//					gl.glVertex3f(rect.getLeft() + (lineWidth * 2.0f), rect.getTop() + (lineHeight * 2.0f), 0.0f);
//					gl.glVertex3f(rect.getRight() + (lineWidth * -2.0f), rect.getTop() + (lineHeight * 2.0f), 0.0f);
//					gl.glVertex3f(rect.getRight() + (lineWidth * -2.0f), rect.getBottom() + (lineHeight * -2.0f), 0.0f);
//					gl.glVertex3f(rect.getLeft() + (lineWidth * 2.0f), rect.getBottom() + (lineHeight * -2.0f), 0.0f);
//				}
//			}
//			
//			gl.glEnd();
			
			if (vao == null)
			{
				vao = new VAODrawer();
				vao.LoadMesh(pipe, CreateMesh());
				
				bUpdateVerts = false;
				bUpdateColors = false;
			}
			if (shad == null)
			{
				shad = new JoglShaderPipelineDefaultColored();
				shad.Create(pipe);
			}
			
			if (bUpdateVerts)
			{
				updateVAOVerts(pipe);
				bUpdateVerts = false;
			}
			if (bUpdateColors)
			{
				updateVAOColors(pipe);
				bUpdateColors = false;
			}
			
			shad.Bind(pipe);
			if (bChecked)
			{
				vao.Draw(pipe);
			}
			else
			{
				vao.Draw(pipe, 0, 30);
			}
			shad.Unbind(pipe);
		}
	}
}
