package org.processmining.processprofiler3d.compviz.jogl.gui;

import java.util.HashMap;
import java.util.Vector;

import org.deckfour.xes.model.XLog;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.interactions.IScrollBarListener;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohort;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.visualisation3d.FontData;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.input.JoglInputMouseWheelEvent;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;

import com.jogamp.opengl.GL2;

public class JoglGuiCohortTreeBox extends JoglGuiObject implements IScrollBarListener
{
	private JSOGraphicalUserInterface gui;
	//private JoglGuiTextRenderer textRenderer;
	
	private float bgcolorR = 0.0f;
	private float bgcolorG = 0.0f;
	private float bgcolorB = 0.0f;
	private float bgcolorA = 0.3f;

	private JoglScreenRect rect = new JoglScreenRect(0,0,1,1);
	
	private float padding = 0.0f;
	private static float xoff = 0.0f;
	private static float yoff = 0.0f;

	private JoglGuiColoredRect background;
	private JoglGuiLabel lblNoCohort;
	private JoglGuiLabel lblTotal;
	
	private Vector<JoglGuiLabel> labels = new Vector<JoglGuiLabel>();
	private Vector<JoglGuiLabel> selectedLabels = new Vector<JoglGuiLabel>();
	//private JoglGuiColoredFrame scrollBarFrame;
	//private JoglGuiColoredRect scrollBar;
	
	private JoglGuiScrollBar scrollB;
	
	private Vector<JoglGuiColoredRect> highlighters = new Vector<JoglGuiColoredRect>();
	
	private float scrolldist = 0.0f;
	private float scrollmaxdist = 0.0f;
	private boolean bShowScrollbar = false;
	
	private boolean bCanTouch = false;
	private boolean bInitialized = false;
	
	private Vector<CohortEntry> dataLines = new Vector<CohortEntry>();
	
	private JoglTexturedMesh jtm = null;
	private JoglGuiMesh jgm = null;
	private JoglGuiScissorBox jgsb = null;
	private Vector<Integer> offsets = new Vector<Integer>();
	
	private boolean bMultiSelect = false;
	
//	public int getAlign()
//	{
//		return textRenderer.getAlign();
//	}
//	
//	public void setAlign(int a)
//	{
//		textRenderer.setAlign(a);
//
//		updateAlignment();
//	}
	
	public void setBackColor(float r, float g, float b)
	{
		bgcolorR = r;
		bgcolorG = g;
		bgcolorB = b;
	}
	
	public void setBackColor(float r, float g, float b, float a)
	{
		bgcolorR = r;
		bgcolorG = g;
		bgcolorB = b;
		bgcolorA = a;
	}
	
	public void setBackColor(JoglVectord3 v)
	{
		bgcolorR = (float)v.getX();
		bgcolorG = (float)v.getY();
		bgcolorB = (float)v.getZ();
	}
	
	public void setBackColor(JoglVectord4 v)
	{
		bgcolorR = (float)v.getX();
		bgcolorG = (float)v.getY();
		bgcolorB = (float)v.getZ();
		bgcolorA = (float)v.getValue(3);
	}
	
	@Override
	public boolean getCanTouch()
	{
		return bCanTouch;
	}
	
	@Override
	public void setCanTouch(boolean b)
	{
		bCanTouch = b;
	}
	
	public boolean getCanMultiSelect()
	{
		return bMultiSelect;
	}
	
	public void setCanMultiSelect(boolean b)
	{
		bMultiSelect = b;
	}
	
	public void setColor(float r, float g, float b)
	{
		//textRenderer.setColor(r, g, b);
	}
	
	public void setColor(JoglVectord3 v)
	{
		//textRenderer.setColor(v);
	}
	
	public JoglVectord4 getRect()
	{
		return rect.getRect();
	}
	
	public void setRect(float x, float y, float w, float h)
	{
		rect = new JoglScreenRect(x, y, w, h);
		
		updateAlignment();
	}
	
	public void setRect(JoglVectord4 v)
	{
		rect = new JoglScreenRect(v);
		
		updateAlignment();
	}
	
	public void setSelectedIndex(int i)
	{
		selectedLabels.clear();
		highlighters.clear();
		if (i >= 0 && i < labels.size())
		{
			selectedLabels.add(labels.get(i));
			AddHighlighterFor(labels.get(i));
		}
	}
	
	public int getSelectedIndex()
	{
		if (!selectedLabels.isEmpty())
		{
			return labels.indexOf(selectedLabels.get(0));
		}
		
		return -1;
	}
	
	public Vector<Integer> getSelectedIndices()
	{
		Vector<Integer> lst = new Vector<Integer>();
		for (JoglGuiLabel l : selectedLabels)
		{
			lst.add(labels.indexOf(l));
		}
		
		return lst;
	}
	
	public ClassificationCohort getSelectedCohort()
	{
		int idx = getSelectedIndex();
		if (idx >= 0)
		{
			return dataLines.get(idx).getCohort();
		}
		return null;
	}
	
	public Vector<ClassificationCohort> getSelectedCohorts()
	{
		Vector<ClassificationCohort> lst = new Vector<ClassificationCohort>();
		for (int idx : getSelectedIndices())
		{
			if (idx >= 0)
			{
				CohortEntry ce = dataLines.get(idx);
				lst.add(ce.getCohort());
			}
		}
		
		return lst;
	}
	
	private void updateAlignment() 
	{
//		int align = textRenderer.getAlign();
//		if (align == 0)
//		{
//			textRenderer.setPosition(rect.getLeft() + (rect.getWidth() * 0.5), rect.getTop() + (rect.getHeight() * 0.5), 0);
//		}
//		else if (align == 1)
//		{
//			textRenderer.setPosition(rect.getLeft() + padding, rect.getTop() + (rect.getHeight() * 0.5), 0);
//		}
//		else if (align == 2)
//		{
//			textRenderer.setPosition(rect.getRight() - padding, rect.getTop() + (rect.getHeight() * 0.5), 0);
//		}
	}
	
	public float getPadding()
	{
		return padding;
	}

	public void setPadding(float p)
	{
		padding = p;
		
		updateAlignment();
	}
	
//	public void setTextSize(float tx, float ty)
//	{
//		textRenderer.setTextSize(tx, ty);
//	}
	
	public JoglGuiCohortTreeBox()
	{
		//textRenderer = new JoglGuiTextRenderer(ff);
	}
	
	private int noCohortNum = 0;
	private int totalNum = 0;
	
	public void setItems(CohortClassifier cc, XLog mylog)
	{
		HashMap<Integer, Integer> histogram = cc.getCohortCaseHistogram(mylog);

		noCohortNum = histogram.get(-1);
		totalNum = histogram.get(-2);
		
		dataLines = GetCohortLabels(cc, histogram);
	}
	
	public Vector<CohortEntry> GetCohortLabels(CohortClassifier cc, HashMap<Integer, Integer> histogram)
	{
		Vector<CohortEntry> cNames = new Vector<CohortEntry>();
		cNames.add(new CohortEntry("Root", cc.getRoot()));
		
//		for (ClassificationCohort cs : cc.getCohorts())
//		{
//			if (histogram.containsKey(cs.getID()))
//			{
//				String lvlStr = "";
//				for (int i = 0; i < cs.getLevel(); i++)
//				{
//					lvlStr = lvlStr + "-";
//				}
//				String cText = lvlStr + cs.getName() + " (" + histogram.get(cs.getID()) + ")";
//				cNames.add(new CohortEntry(cText, cs));
//			}
//		}
		int lvl = 0;
		boolean bRepeat = true;
		while (bRepeat)
		{
			bRepeat = false;
			for (int i = 0; i < cNames.size(); i++)
			{
				ClassificationCohort cx = cNames.get(i).getCohort();
				if (cx.getLevel() == lvl)
				{
					bRepeat = true;
					int j = i + 1;
					for (ClassificationCohort ccx : cx.getChildren())
					{
						cNames.insertElementAt(CreateCohortEntry(histogram, ccx), j);
						j++;
					}
				}
			}
			lvl++;
		}
		
		return cNames;
	}
	
	private CohortEntry CreateCohortEntry(HashMap<Integer, Integer> histogram, ClassificationCohort cs)
	{
		if (histogram.containsKey(cs.getID()))
		{
			String lvlStr = "";
			for (int i = 0; i < cs.getLevel(); i++)
			{
				lvlStr = lvlStr + "-";
			}
			String cText = lvlStr + cs.getName() + " (" + histogram.get(cs.getID()) + ")";
			return new CohortEntry(cText, cs);
		}
		
		return new CohortEntry(cs.getName(), cs);
	}
	
	@Override
	public boolean IsInside(float x, float y)
	{
		if (bCanTouch)
		{
			if (rect.IsInside(x, y))
			{
				return true;
			}
		}
		return false;
	}
	
	@Override
	public void OnAddToGui(JSOGraphicalUserInterface g)
	{
		gui = g;
		
		float aspect = (float)gui.getAspect();
		
		FontFormat ff2 = new FontFormat();
		ff2.width = 0.05f / aspect;
		ff2.height = 0.05f;
		ff2.align = FontFormat.ALIGN_LEFT;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff3 = new FontFormat();
		ff3.width = 0.05f / aspect;
		ff3.height = 0.05f;
		ff3.align = FontFormat.ALIGN_RIGHT;
		ff3.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		float textPadding2 = 0.01f;
		
		yoff = 0;
		
		//required height
		float listHeight = (dataLines.size() + 1) * 0.05f;
		float maxHeight = rect.getHeight() - 0.05f;
		
		float perc = maxHeight / listHeight;
		scrollmaxdist = listHeight - maxHeight;
		
		background = new JoglGuiColoredRect();
		background.setRect(xoff + rect.getLeft(), yoff + rect.getTop() + 0.05f, rect.getWidth(), maxHeight);
		background.setColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(background);
		
		if (perc < 1.0f)
		{
			bShowScrollbar = true;
			
			scrollB = new JoglGuiScrollBar();
			scrollB.setCanTouch(true);
			scrollB.setPerc(perc);
			scrollB.setRect(xoff + rect.getRight() - 0.025f, yoff + rect.getTop() + 0.05f, 0.025f, maxHeight);
			//scrollB.setLine(0.005f / aspect, -0.005f);
			//scrollB.setColor(scrollbarColor);
			scrollB.addScrollbarListener(this);
			
			gui.AddGuiObject(scrollB);
		}
		
		float offy = -0.05f;
		
		//JoglTexturedMesh jtm = new JoglTexturedMesh();
		offsets.clear();
		offsets.add(0);
		
		for (CohortEntry ce : dataLines)
		{
			String ls = ce.getLabel();
			JoglGuiLabel jglv = new JoglGuiLabel(ff2.clone());
			jglv.setCanTouch(true);
			jglv.setText(ls);
			jglv.setRect(xoff + 0.01f, (yoff + offy), rect.getWidth() - 0.02f, 0.05f);
			jglv.setBackColor(JoglGuiStyle.transparentColor);
		
			labels.add(jglv);
			
			JoglTexturedMesh jtm2 = jglv.getTextMesh();
			
			if (jtm == null)
			{
				jtm = new JoglTexturedMesh(GL2.GL_QUADS, 4);
				jtm.setVerts(jtm2.getVerts(), new JoglVectord3(0, offy, 0));
				jtm.setTexCoords(jtm2.getTexCoords());
			}
			else
			{
				jtm.addVerts(jtm2.getVerts(), new JoglVectord3(0, offy, 0));
				jtm.addTexCoords(jtm2.getTexCoords());
			}
			
			offsets.add(jtm.getNumVerts());
			
			offy = offy - 0.05f;
		}
		
		if (jtm != null) //HACK! if no items in list, make things still work!
		{
			FontData fontData = (FontData)JoglResourceManager.GetResource("DefaultFont");
			jgm = new JoglGuiMesh(jtm, fontData.getTexture());
			jgm.setPosition(xoff + rect.getLeft() + 0.01f, yoff + rect.getTop() + maxHeight + 0.05f, 0);
			
			jgsb = new JoglGuiScissorBox(gui);
			jgsb.setRect(background.getRect());
			jgsb.addChild(jgm);
			gui.AddGuiObject(jgsb);
			
			//clip contents of listbox
			MoveScrollbar();
		}

		lblNoCohort = new JoglGuiLabel(ff2.clone());
		lblNoCohort.setRect(xoff + rect.getLeft(), yoff + rect.getTop(), rect.getWidth(), 0.05f);
		lblNoCohort.setText("No Cohort");
		lblNoCohort.setPadding(textPadding2);
		lblNoCohort.setBackColor(JoglGuiStyle.backColor);

		if (noCohortNum > 0)
		{
			lblNoCohort.setText("No Cohort (" + noCohortNum + ")");
		}
		
		gui.AddGuiObject(lblNoCohort);
		//controls.add(lblNoCohort);

		lblTotal = new JoglGuiLabel(ff3.clone());
		lblTotal.setRect(xoff + rect.getLeft(), yoff + rect.getTop(), rect.getWidth(), 0.05f);
		lblTotal.setText("Total");
		lblTotal.setPadding(textPadding2);
		lblTotal.setBackColor(JoglGuiStyle.transparentColor);

		if (totalNum > 0)
		{
			lblTotal.setText("Total (" + totalNum + ")");
		}
		
		gui.AddGuiObject(lblTotal);
		//controls.add(lblTotal);
		
		bInitialized = true;
	}
	
	@Override
	public void OnRemoveFromGui(JSOGraphicalUserInterface g)
	{
		labels.clear();
		if (jgm != null)
		{
			//gui.RemoveGuiObject(jgm);
			gui.RemoveGuiObject(jgsb);
			jgm.Dispose();
		}

		ClearHighlighters();
		
		if (bShowScrollbar)
		{
			gui.RemoveGuiObject(scrollB);
		}
		gui.RemoveGuiObject(background);
		gui.RemoveGuiObject(lblTotal);
		gui.RemoveGuiObject(lblNoCohort);
		
		bInitialized = false;
	}

	public void ClearHighlighters()
	{
		for (JoglGuiColoredRect highlighter : highlighters)
		{
			//gui.RemoveGuiObject(highlighter);
			jgsb.removeChild(highlighter);
		}
		highlighters.clear();
	}
	
	private void AddHighlighterFor(JoglGuiLabel targetLabel)
	{
		JoglGuiColoredRect highlighter = new JoglGuiColoredRect();
		highlighter.setRect(JoglVectord4.Add(targetLabel.getRect(), new JoglVectord4(rect.getLeft(), rect.getBottom() - 0.025f,0,0)));
		highlighter.setColor(JoglGuiStyle.highlightColor);
		highlighter.setVisible(true);
		highlighter.setZLevel(-1.0f);
		
		jgsb.addChild(highlighter);
		highlighters.add(highlighter);
	}
	
	private void RemoveHighlighterFor(JoglGuiLabel targetLabel)
	{
		int idx = selectedLabels.indexOf(targetLabel);
		jgsb.removeChild(highlighters.get(idx));
		highlighters.remove(idx);
	}
	
	//MouseListener method implementation
    public boolean mousePressed(JoglInteractionManager iman, JoglInputMouseEvent e) 
    {
		Vector<JoglGuiObject> jgo = gui.GetObjectsFromPos(e.getX(), e.getY());
		if (jgo != null && !jgo.isEmpty())
		{
			//if clicked on scrollbar
			if (jgo.contains(scrollB))
			{
				scrollB.StartMoveInteraction(iman, e);
				return true;
			}
		}
		
		//boolean bCtrlDown = (e.getModifiersEx() & InputEvent.CTRL_DOWN_MASK) != 0;
		//boolean bShiftDown = (e.getModifiersEx() & InputEvent.SHIFT_DOWN_MASK) != 0;
		boolean bCtrlDown = e.isControlDown();
		boolean bShiftDown = e.isShiftDown();
		
		//if user is not holding down the ctrl-key
		if (!bMultiSelect || !(bCtrlDown || bShiftDown))
		{
			ClearHighlighters();
			selectedLabels.clear();
		}
		
		//if clicked on a listbox item
		JoglGuiLabel targetLabel = getTargetLabel(e);
		if (targetLabel != null)
		{
			Vector<JoglGuiLabel> nlabels = new Vector<JoglGuiLabel>();
			if (bShiftDown && bMultiSelect)
			{
				getAllLabelsInBetween(nlabels, targetLabel);
			}
			else
			{
				nlabels.add(targetLabel);
			}
			
			boolean bUpdateClipping = false;
			for (JoglGuiLabel nlabel : nlabels)
			{
				if (!selectedLabels.contains(nlabel))
				{
					selectedLabels.add(nlabel);
					
					//highlight label
					//highlighter.setRect(JoglVectord4.Add(targetLabel.getRect(), new JoglVectord4(rect.getLeft(), rect.getBottom() - 0.025f,0,0)));
					AddHighlighterFor(nlabel);
					
					if (scrollB == null)
					{
						//highlighter.setVisible(true);
					}
					else
					{
						//clip contents of listbox
						bUpdateClipping = true;
					}
				}
				else if (!(bMultiSelect && bShiftDown))
				{
					RemoveHighlighterFor(nlabel);
					selectedLabels.remove(nlabel);
				}
			}
			
			if (bUpdateClipping)
			{
				MoveScrollbar();
			}
		}
		
		return false;
    }
    
    private void getAllLabelsInBetween(Vector<JoglGuiLabel> nlabels, JoglGuiLabel target)
    {
    	int idxTarget = labels.indexOf(target);
    	
    	int idx2 = -1;
    	for (JoglGuiLabel l : labels)
    	{
    		idx2++;
    		if (selectedLabels.contains(l))
    		{
    			break;
    		}
    	}
    	
    	int minId = Math.min(idxTarget, idx2);
    	int maxId = Math.max(idxTarget, idx2);
    	
    	for (int i = minId; i < maxId + 1; i++)
    	{
    		nlabels.add(labels.get(i));
    	}
    }
    
    public void mouseWheelMoved(JoglInputMouseWheelEvent e)
    {
		if (scrollB == null || background == null || gui == null)
		{
			return;
		}
		
		JoglVectord2 cursorPos = gui.GetNDCPos(e.getX(), e.getY());
		//if cursor hovers over list box
		JoglScreenRect srect = new JoglScreenRect(background.getRect());
		if (srect.IsInside((float)cursorPos.getX(), (float)cursorPos.getY()))
		{
			int delta = e.getWheelRotation();
			scrollB.setScrollPerc(scrollB.getScrollPerc() + (0.1f * delta));
			MoveScrollbar();
		}
    }
    
    public JoglGuiLabel getTargetLabel(JoglInputMouseEvent e) 
    {
    	JoglGuiLabel targetLabel = null;
		JoglVectord2 ndcPos = gui.GetNDCPos(e.getX(), e.getY());
		for (JoglGuiLabel l : labels)
		{
			if (l.IsInside((float)(ndcPos.getX() - rect.getLeft()), (float)(ndcPos.getY() - rect.getBottom()) + 0.025f))
			{
				targetLabel = l;
			}
		}
		
		return targetLabel;
    }
    
    public Vector<JoglGuiLabel> getSelectedLabels() 
    {
    	Vector<JoglGuiLabel> slst = new Vector<JoglGuiLabel>();

    	for (JoglGuiLabel l : selectedLabels)
    	{
    		slst.add(l);
    	}
    	
		return slst;
    }
	
	private void MoveWindow(JoglVectord2 d)
	{
		JoglVectord4 diff = new JoglVectord4(d.getX(), d.getY(), 0, 0);
		
		xoff = xoff + (float)diff.getX();
		yoff = yoff + (float)diff.getY();

		if (jgm != null)
		{
			jgm.setPosition(JoglVectord3.Add(jgm.getPosition(), new JoglVectord3(diff.getX(), diff.getY(),0)));
		}
		
		if (bShowScrollbar)
		{
			scrollB.setRect(JoglVectord4.Add(diff, scrollB.getRect()));
		}
		//lblHeading.setRect(JoglVectord4.Add(diff, lblHeading.getRect()));
		background.setRect(JoglVectord4.Add(diff, background.getRect()));
		//backgroundHeader.setRect(JoglVectord4.Add(diff, backgroundHeader.getRect()));
		
		for (JoglGuiLabel l : labels)
		{
			l.setRect(JoglVectord4.Add(diff, l.getRect()));
		}
	}
	
	private void MoveScrollbar()
	{
		if (scrollB == null)
			return;
		
		double oldscrolldist = scrolldist;
		scrolldist = scrollB.getScrollPerc() * scrollmaxdist;
		
		JoglVectord4 diff = new JoglVectord4(0.0f, scrolldist - oldscrolldist, 0, 0);
		
		float finalHeight = -rect.getHeight();

		//float windowTop = yoff + 0.025f;
		//float windowBottom = yoff + finalHeight + 0.025f;
		float windowTop = yoff + 0.075f;
		float windowBottom = yoff + finalHeight - 0.05f;
		
		//update selection highlight position and visibility
		for (JoglGuiColoredRect highlighter : highlighters)
		{
			highlighter.setRect(JoglVectord4.Add(diff, highlighter.getRect()));
			JoglVectord4 hrect = highlighter.getRect();
			double hrectBottom = (hrect.getY() - rect.getBottom()) + 0.025;
			double hrectTop = hrectBottom + hrect.getW();
			if (hrectBottom > windowBottom && hrectTop < windowTop)
			{
				highlighter.setVisible(true);
			}
			else
			{
				highlighter.setVisible(false);
			}
		}
		
		//update position of list text batch-mesh
		if (jgm != null)
		{
			jgm.setPosition(JoglVectord3.Add(jgm.getPosition(), new JoglVectord3(0.0f, scrolldist - oldscrolldist, 0)));
		}
		
		//update visibility of lines in list text batch-mesh
		int firstVis = -1;
		int lastVis = -1;
		int i = 0;
		for (JoglGuiLabel l : labels)
		{
			JoglVectord4 rect = JoglVectord4.Add(diff, l.getRect());
			l.setRect(rect);

			double labelBottom = rect.getY();
			double labelTop = (rect.getY() + rect.getW());

			if ((labelBottom > windowBottom) && (labelTop < windowTop))
			{
				l.setVisible(true);
				
				if (firstVis == -1)
				{
					firstVis = i;
				}
				lastVis = i;
			}
			else
			{
				l.setVisible(false);
			}
			i++;
		}
		
		int firstVert = offsets.get(firstVis);
		int lastVert = offsets.get(lastVis+1);
		int numVerts = lastVert - firstVert;

		if (jgm != null)
		{
			//jgm.setDrawRange(firstVert, lastVert);
			jgm.setDrawRange((firstVert / 4) * 6, (numVerts / 4) * 6);
		}
	}

	public void OnScrollBarMoved(JoglGuiScrollBar sb, JoglVectord2 dir) 
	{
		MoveScrollbar();
	}
}
