package org.processmining.processprofiler3d.compviz.jogl.gui;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglColoredMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialGuiHistogram;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderPipelineDefaultColored;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.maths.JoglColor3;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;

import com.jogamp.opengl.GL2;

public class JoglGuiHistogram extends JoglGuiObject
{
	private static MaterialGuiHistogram matHist = new MaterialGuiHistogram();
	
	private JSOGraphicalUserInterface gui;
	
	private float colorR = 0.0f;
	private float colorG = 0.0f;
	private float colorB = 0.0f;
	private float colorA = 1.0f;
	
	private float selcolorR = 0.0f;
	private float selcolorG = 1.0f;
	private float selcolorB = 0.0f;
	private float selcolorA = 1.0f;
	
	private float bgcolorR = 0.0f;
	private float bgcolorG = 0.0f;
	private float bgcolorB = 0.0f;
	private float bgcolorA = 0.3f;
	
	private float selbgcolorR = 0.0f;
	private float selbgcolorG = 0.0f;
	private float selbgcolorB = 0.0f;
	private float selbgcolorA = 0.5f;
	
	private JoglGuiHistogramStyle myStyle = null;

	private JoglGuiColoredRect histBg = new JoglGuiColoredRect();
	private JoglGuiColoredRect selBg = new JoglGuiColoredRect();
	private JoglGuiColoredFrame border = new JoglGuiColoredFrame();
	private JoglGuiColoredFrame selection = new JoglGuiColoredFrame();
	private JoglScreenRect rect = new JoglScreenRect(0,0,1,1);

	private Vector<Integer> hist = null;
	private int histMax = 0;
	private int startSel = 0;
	private int endSel = 0;
	
	private boolean bCanTouch = false;
	
	private float lastAspect = 1.0f;

	public JoglGuiHistogram(JSOGraphicalUserInterface gui)
	{
		this.gui = gui;
		
		histBg.setColor(bgcolorR, bgcolorG, bgcolorB, bgcolorA);
		selBg.setColor(selbgcolorR, selbgcolorG, selbgcolorB, selbgcolorA);
		border.setColor(JoglColor3.WHITE);
		selection.setColor(JoglColor3.WHITE);
	}
	
	@Override
	public boolean getCanTouch()
	{
		return bCanTouch;
	}

	@Override
	public void setCanTouch(boolean b)
	{
		bCanTouch = b;
	}
	
	public JoglGuiColoredFrame getFrame()
	{
		return border;
	}
	
	public void setBackgroundColor(float r, float g, float b)
	{
		bgcolorR = r;
		bgcolorG = g;
		bgcolorB = b;

		histBg.setColor(r, g, b);
	}
	
	public void setBackgroundColor(float r, float g, float b, float a)
	{
		bgcolorR = r;
		bgcolorG = g;
		bgcolorB = b;
		bgcolorA = a;

		histBg.setColor(r, g, b, a);
	}
	
	public void setBackgroundColor(JoglVectord3 v)
	{
		bgcolorR = (float)v.getX();
		bgcolorG = (float)v.getY();
		bgcolorB = (float)v.getZ();

		histBg.setColor(v);
	}
	
	public void setBackgroundColor(JoglVectord4 v)
	{
		bgcolorR = (float)v.getX();
		bgcolorG = (float)v.getY();
		bgcolorB = (float)v.getZ();
		bgcolorA = (float)v.getValue(3);

		histBg.setColor(v);
	}
	
	public void setColor(float r, float g, float b)
	{
		colorR = r;
		colorG = g;
		colorB = b;
		
		updateVAO();
	}
	
	public void setColor(float r, float g, float b, float a)
	{
		colorR = r;
		colorG = g;
		colorB = b;
		colorA = a;
		
		updateVAO();
	}
	
	public void setColor(JoglVectord3 v)
	{
		colorR = (float)v.getX();
		colorG = (float)v.getY();
		colorB = (float)v.getZ();
		
		updateVAO();
	}
	
	public void setColor(JoglVectord4 v)
	{
		colorR = (float)v.getX();
		colorG = (float)v.getY();
		colorB = (float)v.getZ();
		colorA = (float)v.getValue(3);
		
		updateVAO();
	}
	
	public void setStyle(JoglGuiHistogramStyle sty)
	{
		myStyle = sty;
		
		updateVAO();
	}
	
	public JoglVectord4 getRect()
	{
		return rect.getRect();
	}
	
	public void setRect(float x, float y, float w, float h)
	{
		rect = new JoglScreenRect(x, y, w, h);
		border.setRect(x, y, w, h);
		histBg.setRect(x, y, w, h);

		UpdateSelectionRectangle();
		updateVAO();
	}
	
	public void setRect(JoglVectord4 v)
	{
		rect = new JoglScreenRect(v);
		border.setRect(v);
		histBg.setRect(v);

		UpdateSelectionRectangle();
		updateVAO();
	}
	
	public void setSelection(int start, int end)
	{
		startSel = start;
		endSel = end;

		UpdateSelectionRectangle();
		updateVAO();
	}

	public void setAspect(float aspect)
	{
		border.setLine(0.0025f / aspect, 0.0025f);
		selection.setLine(0.005f / aspect, 0.005f);
		
		lastAspect = aspect;
	}
	
	public void setHistogram(Vector<Integer> h)
	{
		hist = h;

		int max = 0;
		for (int i = 0; i < hist.size(); i++)
		{
			max = Math.max(hist.get(i), max);
		}
		histMax = max;

		UpdateSelectionRectangle();
		updateVAO();
	}
	
	public void setHistogram(Vector<Integer> h, int hmax)
	{
		hist = h;
		histMax = hmax;

		UpdateSelectionRectangle();
		updateVAO();
	}
	
	private void updateVAO()
	{
		if (vao != null)
		{
			vao.Dispose();
			vao = null;
		}
	}
	
	@Override
	public boolean IsInside(float x, float y)
	{
		if (bCanTouch)
		{
			if (rect.IsInside(x, y))
			{
				return true;
			}
		}
		return false;
	}
	
	private Vector<JoglVectord3> GenerateGeometry()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();

		float totalWidth = rect.getRight() - rect.getLeft();
		float binWidth = 1.0f / (float)hist.size();
		float totalBinWidth = binWidth * totalWidth;
		float totalHeight = rect.getTop() - rect.getBottom();
		
		for (int i = 0; i < hist.size(); i++)
		{
			if (hist.get(i) != 0)
			{
				float binHeight = (float)hist.get(i) / (float)histMax;
				float left = rect.getLeft() + (i * totalBinWidth);
				float right = rect.getLeft() + ((i+1) * totalBinWidth);
				float top = rect.getBottom() + ((1.0f - binHeight) * totalHeight);
				float bottom = rect.getBottom() + totalHeight;
				
				lst.add(new JoglVectord3(left, top, 0.0f));
				lst.add(new JoglVectord3(right, top, 0.0f));
				lst.add(new JoglVectord3(right, bottom, 0.0f));
				lst.add(new JoglVectord3(left, bottom, 0.0f));
			}
		}

		return lst;
	}
	
	private Vector<JoglVectord4> GenerateColors()
	{
		Vector<JoglVectord4> lst = new Vector<JoglVectord4>();
		
		if (myStyle == null)
		{
			for (int i = 0; i < hist.size(); i++)
			{
				if (hist.get(i) != 0)
				{
					if (i >= startSel && i < endSel)
					{
						JoglVectord4 selcolor = new JoglVectord4(selcolorR, selcolorG, selcolorB, selcolorA);
						lst.add(selcolor);
						lst.add(selcolor);
						lst.add(selcolor);
						lst.add(selcolor);
					}
					else
					{
						JoglVectord4 color = new JoglVectord4(colorR, colorG, colorB, colorA);	
						lst.add(color);
						lst.add(color);
						lst.add(color);
						lst.add(color);
					}
				}
			}
		}
		else
		{
			for (int i = 0; i < hist.size(); i++)
			{
				if (hist.get(i) != 0)
				{
					if (i >= startSel && i < endSel)
					{
						JoglVectord4 selcolor = myStyle.getSelectedBarColor(i);
						lst.add(selcolor);
						lst.add(selcolor);
						lst.add(selcolor);
						lst.add(selcolor);
					}
					else
					{
						JoglVectord4 color = myStyle.getUnselectedBarColor(i);	
						lst.add(color);
						lst.add(color);
						lst.add(color);
						lst.add(color);
					}
				}
			}
		}	

		return lst;
	}
	
	public JoglColoredMesh CreateMesh()
	{
		JoglColoredMesh jcm = new JoglColoredMesh(GL2.GL_QUADS);
		jcm.setVerts(GenerateGeometry());
		jcm.setColors(GenerateColors());
		jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jcm.getNumVerts()));
		
		return jcm;
	}
	
	private IMeshDrawer vao = null;
	private static JoglShader shad = null;
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		if (bVisible && hist != null)
		{
			//adjust things if aspect ratio has changed
			if (lastAspect != (float)gui.getAspect())
			{
				setAspect((float)gui.getAspect());
			}
			
			DrawHistogramBG(pipe);
			
			DrawSelectionBG(pipe);
			
			DrawHistogramBorder(pipe);
			
			DrawHistogramBins(pipe);
			
			DrawSelectionFrame(pipe);
		}
	}

	private void UpdateSelectionRectangle()
	{
		if (hist != null && !hist.isEmpty())
		{
			float totalWidth = rect.getRight() - rect.getLeft();
			float binWidth = 1.0f / (float)hist.size();
			float totalBinWidth = binWidth * totalWidth;
			
			float left = rect.getLeft() + (startSel * totalBinWidth);
			float right = rect.getLeft() + (endSel * totalBinWidth);
			float top = rect.getTop();
			float bottom = rect.getBottom();
			JoglVectord4 selRect = new JoglVectord4(left, top, right - left, bottom - top);
	
			selBg.setRect(selRect);
			selection.setRect(selRect);
		}
	}

	private void DrawHistogramBG(GraphicsPipeline pipe) 
	{
		histBg.DrawObject(pipe);
	}

	private void DrawHistogramBorder(GraphicsPipeline pipe) 
	{
		border.DrawObject(pipe);
	}

	private void DrawHistogramBins(GraphicsPipeline pipe) 
	{
		if (hist == null || hist.isEmpty())
		{
			return;
		}
		
		if (colorA > 0.0f || selcolorA > 0.0f)
		{
			pipe.PushPipelineState(matHist);
//			if (colorA < 1.0f || selcolorA < 1.0f)
//			{
//				pipe.setDrawModeAlphaBlended();
//			}
			
//			gl.glBegin(GL2.GL_QUADS);
//	
//			//gl.glColor4f(colorR, colorG, colorB, colorA);
//			
//			for (int i = 0; i < hist.size(); i++)
//			{
//				if (i >= startSel && i < endSel)
//				{
//					gl.glColor4f(selcolorR, selcolorG, selcolorB, selcolorA);
//				}
//				else
//				{
//					gl.glColor4f(colorR, colorG, colorB, colorA);
//				}
//				
//				float left = rect.getLeft() + (i * totalBinWidth);
//				float right = rect.getLeft() + ((i+1) * totalBinWidth);
//				float binHeight = (float)hist.get(i) / (float)histMax;
//				float top = rect.getBottom() + ((1.0f - binHeight) * totalHeight);
//				float bottom = rect.getBottom() + totalHeight;
//				
//				gl.glVertex3f(left, top, 0.0f);
//				gl.glVertex3f(right, top, 0.0f);
//				gl.glVertex3f(right, bottom, 0.0f);
//				gl.glVertex3f(left, bottom, 0.0f);
//			}
//			gl.glEnd();
			if (vao == null)
			{
				vao = new VAODrawer();
				vao.LoadMesh(pipe, CreateMesh());
			}
			if (shad == null)
			{
				if (JoglResourceManager.HasResource("PipelineDefaultColoredShader"))
				{
					shad = (JoglShader)JoglResourceManager.GetResource("PipelineDefaultColoredShader");
				}
				else
				{
					shad = new JoglShaderPipelineDefaultColored();
					shad.Create(pipe);
					
					JoglResourceManager.AddResource("PipelineDefaultColoredShader", shad);
				}
			}
			
			shad.Bind(pipe);
			vao.Draw(pipe);
			shad.Unbind(pipe);
			
//			if (colorA < 1.0f || selcolorA < 1.0f)
//			{
//				pipe.setDrawModeOpaque();
//			}
			pipe.PopPipelineState(matHist);
		}
	}

	private void DrawSelectionBG(GraphicsPipeline pipe) 
	{	
		if (startSel >= endSel)
		{
			return;
		}

		selBg.DrawObject(pipe);
	}

	private void DrawSelectionFrame(GraphicsPipeline pipe) 
	{
		if (startSel >= endSel)
		{
			return;
		}
		
		//pipe.setColor(1.0f, 1.0f, 1.0f, 1.0f);
		
		selection.DrawObject(pipe);
	}
}
