package org.processmining.processprofiler3d.compviz.jogl.gui;

import java.util.HashMap;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglColoredMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderPipelineDefaultColored;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;

import com.jogamp.opengl.GL2;

public class JoglGuiHistogramMatrix extends JoglGuiObject
{
	public JoglGuiHistogramMatrix(JSOGraphicalUserInterface gui)
	{
		this.gui = gui;
	}

	private JSOGraphicalUserInterface gui;
	
	private IMeshDrawer vao = null;
	private IMeshDrawer vaoB = null;
	private static JoglShader shad = null;

	private Vector<JoglGuiHistogram> lstHist = new Vector<JoglGuiHistogram>();
	private Vector<JoglGuiHistogram> lstVisHist = new Vector<JoglGuiHistogram>();
	public JoglVectord2 off = JoglVectord2.ZERO; 
	
	private HashMap<JoglGuiHistogram, Integer> startMap = new HashMap<JoglGuiHistogram, Integer>();
	private HashMap<JoglGuiHistogram, Integer> countMap = new HashMap<JoglGuiHistogram, Integer>();
	
	private JoglColoredMesh myMesh = null;

	private HashMap<JoglGuiHistogram, Integer> startMapB = new HashMap<JoglGuiHistogram, Integer>();
	private HashMap<JoglGuiHistogram, Integer> countMapB = new HashMap<JoglGuiHistogram, Integer>();
	private JoglColoredMesh myBorderMesh = null;
	
	public void setHistograms(Vector<JoglGuiHistogram> lst)
	{
		lstHist = lst;

		updateFrameMesh(lst);		
		updateBarMesh(lst);
	}

	private void updateFrameMesh(Vector<JoglGuiHistogram> lst)
	{
		startMapB.clear();
		countMapB.clear();
		
		JoglColoredMesh jcmb = new JoglColoredMesh(GL2.GL_QUADS);
		for (JoglGuiHistogram hist : lst)
		{
			int start = jcmb.getNumVerts();
			
			JoglColoredMesh jcmc = hist.getFrame().CreateMesh();
			jcmb.addVerts(jcmc);
			jcmb.addColors(jcmc);
			
			int count = jcmb.getNumVerts() - start;
			
			startMapB.put(hist, (start / 4) * 6);
			countMapB.put(hist, (count / 4) * 6);
		}
		
		jcmb.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jcmb.getNumVerts()));
		
		myBorderMesh = jcmb;
		
		if (vaoB != null)
		{
			vaoB.Dispose();
			vaoB = null;
		}
	}

	private void updateBarMesh(Vector<JoglGuiHistogram> lst)
	{
		startMap.clear();
		countMap.clear();
		
		JoglColoredMesh jcm = new JoglColoredMesh(GL2.GL_QUADS);
		for (JoglGuiHistogram hist : lst)
		{
			int start = jcm.getNumVerts();
			
			JoglColoredMesh jcmc = hist.CreateMesh();
			jcm.addVerts(jcmc);
			jcm.addColors(jcmc);
			
			int count = jcm.getNumVerts() - start;
			
			startMap.put(hist, (start / 4) * 6);
			countMap.put(hist, (count / 4) * 6);
		}
		
		jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jcm.getNumVerts()));
		
		myMesh = jcm;
		
		if (vao != null)
		{
			vao.Dispose();
			vao = null;
		}
	}
	
	public void updateVisibleHistograms(Vector<JoglGuiHistogram> lst)
	{
		lstVisHist = lst;
	}
	
	private float lastAspect = 1.0f;
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		pipe.transform.Push();
		pipe.transform.Translated(off.getX(), -off.getY(), 0);
		
		float aspect = (float)gui.getAspect();
		if (lastAspect != aspect)
		{
			for (JoglGuiHistogram hist : lstHist)
			{
				hist.setAspect(aspect);
			}
			updateFrameMesh(lstHist);
			
			lastAspect = aspect;
		}
		
//		for (JoglGuiHistogram hist : lstVisHist)
//		{
//			hist.DrawObject(pipe);
//		}
		if (vao == null && myMesh != null)
		{
			vao = new VAODrawer();
			vao.LoadMesh(pipe, myMesh);
		}
		if (vaoB == null && myBorderMesh != null)
		{
			vaoB = new VAODrawer();
			vaoB.LoadMesh(pipe, myBorderMesh);
		}
		if (shad == null)
		{
			if (JoglResourceManager.HasResource("PipelineDefaultColoredShader"))
			{
				shad = (JoglShader)JoglResourceManager.GetResource("PipelineDefaultColoredShader");
			}
			else
			{
				shad = new JoglShaderPipelineDefaultColored();
				shad.Create(pipe);
				
				JoglResourceManager.AddResource("PipelineDefaultColoredShader", shad);
			}
		}
		
		if (vaoB != null)
		{
			shad.Bind(pipe);
			vaoB.Bind(pipe);

			for (JoglGuiHistogram hist : lstVisHist)
			{
				vaoB.DrawSlim(pipe, startMapB.get(hist), countMapB.get(hist));
			}
			vaoB.Unbind(pipe);
			shad.Unbind(pipe);
		}
		
		if (vao != null)
		{
			shad.Bind(pipe);
			vao.Bind(pipe);

			for (JoglGuiHistogram hist : lstVisHist)
			{
				vao.DrawSlim(pipe, startMap.get(hist), countMap.get(hist));
			}
			vao.Unbind(pipe);
			shad.Unbind(pipe);
		}
		
		pipe.transform.Pop();
	}
}
