package org.processmining.processprofiler3d.compviz.jogl.gui;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglColoredMesh;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialGuiColoredRect;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderPipelineDefaultColored;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;

import com.jogamp.opengl.GL2;

public class JoglGuiLabel extends JoglGuiObject
{
	private JoglGuiTextRenderer textRenderer;
	
	private float bgcolorR = 0.0f;
	private float bgcolorG = 0.0f;
	private float bgcolorB = 0.0f;
	private float bgcolorA = 1.0f;

	private JoglScreenRect rect = new JoglScreenRect(0,0,1,1);
	
	private float padding = 0.0f;
	
	private boolean bCanTouch = false;
	
	public double forceScale = 1.0;
	
	private boolean bUpdateVerts = false;
	private boolean bUpdateColors = false;
	
	public int getAlign()
	{
		return textRenderer.getAlign();
	}
	
	public void setAlign(int a)
	{
		textRenderer.setAlign(a);

		updateAlignment();

		bUpdateVerts = true;
	}
	
	public void setBackColor(float r, float g, float b)
	{
		bgcolorR = r;
		bgcolorG = g;
		bgcolorB = b;

		bUpdateColors = true;
	}
	
	public void setBackColor(float r, float g, float b, float a)
	{
		bgcolorR = r;
		bgcolorG = g;
		bgcolorB = b;
		bgcolorA = a;

		bUpdateColors = true;
	}
	
	public void setBackColor(JoglVectord3 v)
	{
		bgcolorR = (float)v.getX();
		bgcolorG = (float)v.getY();
		bgcolorB = (float)v.getZ();

		bUpdateColors = true;
	}
	
	public void setBackColor(JoglVectord4 v)
	{
		bgcolorR = (float)v.getX();
		bgcolorG = (float)v.getY();
		bgcolorB = (float)v.getZ();
		bgcolorA = (float)v.getValue(3);

		bUpdateColors = true;
	}

	@Override
	public boolean getCanTouch()
	{
		return bCanTouch;
	}
	
	@Override
	public void setCanTouch(boolean b)
	{
		bCanTouch = b;
	}
	
	public void setColor(float r, float g, float b)
	{
		textRenderer.setColor(r, g, b);
	}
	
	public void setColor(JoglVectord3 v)
	{
		textRenderer.setColor(v);
	}
	
	public JoglVectord4 getRect()
	{
		return rect.getRect();
	}
	
	public void setRect(float x, float y, float w, float h)
	{
		rect = new JoglScreenRect(x, y, w, h);
		
		updateAlignment();

		bUpdateVerts = true;
	}
	
	public void setRect(JoglVectord4 v)
	{
		rect = new JoglScreenRect(v);
		
		updateAlignment();

		bUpdateVerts = true;
	}
	
	public void setForceScale(double fs)
	{
		forceScale = fs;
		textRenderer.forceScale = fs;

		bUpdateVerts = true;
	}
	
	private void updateVAOVerts(GraphicsPipeline pipe)
	{
		if (vao != null)
		{
			vao.UpdateDataBufferJV3(pipe, 0, GenerateGeometry());
		}
	}
	
	private void updateVAOColors(GraphicsPipeline pipe)
	{
		if (vao != null)
		{
			vao.UpdateDataBufferJV4(pipe, 1, GenerateColors());
		}
	}
	
	public JoglTexturedMesh getTextMesh()
	{
		return textRenderer.getMesh();
	}
	
	private void updateAlignment() 
	{
		int align = textRenderer.getAlign();
		if (align == 0)
		{
			textRenderer.setPosition(rect.getLeft() + (rect.getWidth() * 0.5), rect.getTop() + (rect.getHeight() * 0.5), 0);
		}
		else if (align == 1)
		{
			textRenderer.setPosition(rect.getLeft() + padding, rect.getTop() + (rect.getHeight() * 0.5), 0);
		}
		else if (align == 2)
		{
			textRenderer.setPosition(rect.getRight() - padding, rect.getTop() + (rect.getHeight() * 0.5), 0);
		}
	}
	
	public float getPadding()
	{
		return padding;
	}

	public void setPadding(float p)
	{
		padding = p;
		
		updateAlignment();
		
		bUpdateVerts = true;
	}
	
	public void setTextSize(float tx, float ty)
	{
		textRenderer.setTextSize(tx, ty);
	}
	
	public JoglVectord2 MeasureText(String s)
	{
		FontFormat ff = textRenderer.getTextFormat();
		return textRenderer.MeasureText(s, ff);
	}
	
	public JoglVectord2 MeasureText(String s, double mwidth)
	{
		FontFormat ff = textRenderer.getTextFormat();
		return textRenderer.MeasureText(s, ff, mwidth);
	}
	
	public JoglGuiLabel(FontFormat ff)
	{
		textRenderer = new JoglGuiTextRenderer(ff);
	}
	
	public String getText()
	{
		return textRenderer.getString();
	}
	
	public void setText(String s)
	{
		textRenderer.setString(s);
	}
	
	public void setText(String s, double mwidth)
	{
		textRenderer.setString(s, mwidth);
	}
	
	@Override
	public boolean IsInside(float x, float y)
	{
		if (bCanTouch)
		{
			if (rect.IsInside(x, y))
			{
				return true;
			}
		}
		return false;
	}
	
	private Vector<JoglVectord3> GenerateGeometry()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();

		JoglVectord2 rcenter = rect.getCenter();
		float hwidth = (float)(rect.getWidth() * 0.5 * forceScale);
		float hheight = (float)(rect.getHeight() * 0.5 * forceScale);
		float rleft = (float)(rcenter.getX() - hwidth);
		float rright = (float)(rcenter.getX() + hwidth);
		float rtop = (float)(rcenter.getY() - hheight);
		float rbottom = (float)(rcenter.getY() + hheight);

		lst.add(new JoglVectord3(rleft, rtop, 0.0f));
		lst.add(new JoglVectord3(rright, rtop, 0.0f));
		lst.add(new JoglVectord3(rright, rbottom, 0.0f));
		lst.add(new JoglVectord3(rleft, rbottom, 0.0f));
		
		return lst;
	}
	
	private Vector<JoglVectord4> GenerateColors()
	{
		Vector<JoglVectord4> lst = new Vector<JoglVectord4>();
		
		lst.add(new JoglVectord4(bgcolorR, bgcolorG, bgcolorB, bgcolorA));
		lst.add(new JoglVectord4(bgcolorR, bgcolorG, bgcolorB, bgcolorA));
		lst.add(new JoglVectord4(bgcolorR, bgcolorG, bgcolorB, bgcolorA));
		lst.add(new JoglVectord4(bgcolorR, bgcolorG, bgcolorB, bgcolorA));
		
		return lst;
	}
	
	public JoglColoredMesh CreateMesh()
	{
		JoglColoredMesh jcm = new JoglColoredMesh(GL2.GL_QUADS);
		jcm.setVerts(GenerateGeometry());
		jcm.setColors(GenerateColors());
		jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jcm.getNumVerts()));
		
		return jcm;
	}
	
	private IMeshDrawer vao = null;
	private static JoglShader shad = null;
	private static MaterialGuiColoredRect matCR = new MaterialGuiColoredRect();
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		if (bVisible)
		{
			//draw background
			if (bgcolorA > 0.0f)
			{
				pipe.PushPipelineState(matCR);
				matCR.DrawAlpha(bgcolorA);
//				if (bgcolorA < 1.0f)
//				{
//					pipe.setDrawModeAlphaBlended();
//				}
				
//				gl.glBegin(GL2.GL_QUADS);
//				
//				JoglVectord2 rcenter = rect.getCenter();
//				float hwidth = (float)(rect.getWidth() * 0.5 * forceScale);
//				float hheight = (float)(rect.getHeight() * 0.5 * forceScale);
//				float rleft = (float)(rcenter.getX() - hwidth);
//				float rright = (float)(rcenter.getX() + hwidth);
//				float rtop = (float)(rcenter.getY() - hheight);
//				float rbottom = (float)(rcenter.getY() + hheight);
//				
//				gl.glColor4f(bgcolorR, bgcolorG, bgcolorB, bgcolorA);
//				gl.glVertex3f(rleft, rtop, 0.0f);
//				gl.glVertex3f(rright, rtop, 0.0f);
//				gl.glVertex3f(rright, rbottom, 0.0f);
//				gl.glVertex3f(rleft, rbottom, 0.0f);
//				
//				gl.glEnd();
				if (vao == null)
				{
					vao = new VAODrawer();
					vao.LoadMesh(pipe, CreateMesh());
					
					bUpdateVerts = false;
					bUpdateColors = false;
				}
				if (shad == null)
				{
					if (JoglResourceManager.HasResource("PipelineDefaultColoredShader"))
					{
						shad = (JoglShader)JoglResourceManager.GetResource("PipelineDefaultColoredShader");
					}
					else
					{
						shad = new JoglShaderPipelineDefaultColored();
						shad.Create(pipe);
						
						JoglResourceManager.AddResource("PipelineDefaultColoredShader", shad);
					}
				}
				
				if (bUpdateVerts)
				{
					updateVAOVerts(pipe);
					bUpdateVerts = false;
				}
				if (bUpdateColors)
				{
					updateVAOColors(pipe);
					bUpdateColors = false;
				}
				
				shad.Bind(pipe);
				vao.Draw(pipe);
				shad.Unbind(pipe);
		
//				if (bgcolorA < 1.0f)
//				{
//					pipe.setDrawModeOpaque();
//				}
				pipe.PopPipelineState(matCR);
			}
		
			//gl.glPushMatrix();
			//gl.glScalef(scale, scale, scale);
			textRenderer.DrawObject(pipe);
			//gl.glPopMatrix();
		}
		
	}
}
