package org.processmining.processprofiler3d.compviz.jogl.gui;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialGuiMesh;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class JoglGuiMesh extends JoglGuiObject
{	
	private JoglMesh mesh = null;
	private JoglTexture tex = null;
	
	private JoglVectord3 position = new JoglVectord3(0,0,0);
	private JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateIdentity();
	
	private int drawStart = -1;
	private int drawNum = -1;

	public JoglVectord3 getPosition()
	{
		return position;
	}
	
	public void setPosition(double x, double y, double z)
	{
		position = new JoglVectord3(x,y,z);
		transform = JoglMatrixd4x4.CreateTranslation(x, y, z);
	}
	
	public void setPosition(JoglVectord3 v)
	{
		position = v;
		transform = JoglMatrixd4x4.CreateTranslation(v.getX(), v.getY(), v.getZ());
	}
	
	public void setDrawRange(int start, int num)
	{
		drawStart = start;
		drawNum = num;
	}
	
	public JoglGuiMesh(JoglMesh m)
	{
		mesh = m;
	}
	
	public JoglGuiMesh(JoglMesh m, JoglTexture t)
	{
		mesh = m;
		tex = t;
	}
	
	public JoglMesh getMesh()
	{
		return mesh;
	}
	
	public void setMesh(JoglMesh m)
	{
		mesh = m;
	}

	private IMeshDrawer vaoDrawer;
	private static MaterialGuiMesh matGM = new MaterialGuiMesh();
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);
		
		if (vaoDrawer == null)
		{
			mesh.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(mesh.getNumVerts()));
			vaoDrawer = new VAODrawer();
			vaoDrawer.LoadMesh(pipe, mesh);
		}
		
		if (tex != null)
		{
			tex.Bind(pipe);
		}
		pipe.PushPipelineState(matGM);
		//vaoDrawer.Bind(gl);
		if (drawStart == -1 || drawNum == -1)
		{
			vaoDrawer.Draw(pipe);
			//mesh.draw(gl);
		}
		else
		{
			vaoDrawer.Draw(pipe, drawStart, drawNum);
			//mesh.draw(gl, drawStart, drawNum);
		}
		pipe.PopPipelineState(matGM);
		
		//vaoDrawer.Unbind(gl);
		if (tex != null)
		{
			tex.Unbind(pipe);
		}
		
		pipe.transform.Pop();
	}
	
	public void Dispose()
	{	
		if (vaoDrawer != null)
		{
			vaoDrawer.Dispose();
		}
	}
}
