package org.processmining.processprofiler3d.compviz.jogl.gui;

import java.util.HashSet;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.interactions.InfoPanelCategory;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOCohortColorManager;
import org.processmining.visualisation3d.FontData;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

import com.jogamp.opengl.GL2;

public class JoglGuiPerformanceInfoList extends JoglGuiObject
{
	private static float xoff = 0.0f;
	private static float yoff = 0.0f;
	
	private JSOGraphicalUserInterface gui;
	private JSOCohortColorManager jccm;
	
	private HashSet<String> visibleCats = new HashSet<String>();
	private HashSet<String> visibleCohorts = new HashSet<String>();
	
	private JoglTexturedMesh jtm = null;
	private JoglGuiMesh jgm = null;
	private Vector<Integer> offsets = new Vector<Integer>();

	private Vector<InfoPanelCategory> dataLines;
	private Vector<JoglGuiLabel> labels;
	public Vector<JoglGuiColoredRect> coloredRects = new Vector<JoglGuiColoredRect>();

	private boolean bInitializedData = false;
	
	private FontFormat ff2;
	
	private int numLines = 0;
	private float lineHeight;
	private float totalHeight = 0;
	
	private float offy = 0.0f;
	
	public static boolean bFilterVisCats = true;
	public static boolean bFilterVisCohorts = true;
	
	public float getListHeight()
	{
		return totalHeight;
	}
	
	public JoglGuiPerformanceInfoList(JSOGraphicalUserInterface gui)
	{
		this.gui = gui;
		labels = new Vector<JoglGuiLabel>();
	}
	
	public void setVisibleCats(HashSet<String> vCats)
	{
		visibleCats = vCats;
	}
	
	public void setVisibleCohorts(HashSet<String> vCohorts)
	{
		visibleCohorts = vCohorts;
	}
	
	public void setData(Vector<InfoPanelCategory> data)
	{
		dataLines = data;
	}
	
	public void InitStage1(JoglScene scene)
	{
		float aspect = (float)gui.getAspect();
		
		ff2 = new FontFormat();
		ff2.width = (0.05f / aspect) * (float)JoglGuiStyle.textScale;
		ff2.height = 0.05f * (float)JoglGuiStyle.textScale;
		ff2.align = FontFormat.ALIGN_LEFT;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		//required height
		lineHeight = 0.05f * (float)JoglGuiStyle.textScale;
		
		jccm = scene.findSceneObject(JSOCohortColorManager.class);
	}
	
	public void InitDataElements()
	{
		offy = 0.0f;
		
		offsets.clear();
		offsets.add(0);
		
		numLines = 0;
		totalHeight = lineHeight;
		
		for (InfoPanelCategory ipc : dataLines)
		{
			AddDataElements(ipc);
		}
	}
	
	public void AddDataElements(InfoPanelCategory ipc)
	{
		//if category is meant to be visible
		if ((bFilterVisCats && visibleCats.contains(ipc.catName)) || !bFilterVisCats)
		{
			//int catNumLines = ipc.getNumLines();
			
			int catNumLines = AddCategory(ff2, ipc);

			numLines = numLines + catNumLines;
			
			//required height
			totalHeight = totalHeight + (catNumLines * lineHeight);
		}
	}
	
	public JoglGuiMesh InitGuiMesh()
	{		
		if (jtm != null)
		{
			FontData fontData = (FontData)JoglResourceManager.GetResource("DefaultFont");
			jgm = new JoglGuiMesh(jtm, fontData.getTexture());
			jgm.setPosition(xoff - 0.24, yoff - (lineHeight), 0);
			
			return jgm;
		}
		return null;
	}
	
	public void InitData(JoglScene scene)
	{
		InitStage1(scene);
		InitDataElements();
		//InitGuiMesh();
		
		bInitializedData = true;
	}

	private int AddCategory(FontFormat ff2, InfoPanelCategory ipc)
	{
		int catNumLines = 0;
		if (this.bFilterVisCohorts)
		{
			catNumLines = ipc.getNumLines(visibleCohorts);
		}
		else
		{
			catNumLines = ipc.getNumLines();
		}
		
		if (catNumLines > 1)
		{
			AddCategoryHeader(ipc.catName, ff2);
			offy = offy - lineHeight; //advance yoffset by a textline
			
			for (String ls : ipc.dataLines)
			{
				String cohortName = ls.substring(0, ls.indexOf(":")).trim();

				if ((bFilterVisCohorts && visibleCohorts.contains(cohortName)) || !bFilterVisCohorts)
				{
					AddCategoryData(ls, cohortName, ff2);
					offy = offy - lineHeight; //advance yoffset by a textline
				}
			}
	
			return catNumLines;
		}
		return 0;
	}
	
	public void DisposeData()
	{
		if (bInitializedData)
		{
			labels.clear();
			coloredRects.clear();

			if (jgm != null)
			{
				jgm.Dispose();
			}
			
			offsets.clear();
			jtm = null;
			
			bInitializedData = false;
//			bResetData = false;
		}
	}
	
	private void AddCategoryHeader(String ls, FontFormat ff2)
	{
		JoglGuiLabel jglv = AddTextLine(ls, ff2);
		
		//add highlight rect
		JoglGuiColoredRect jgcr = new JoglGuiColoredRect();
		jgcr.setColor(JoglGuiStyle.disabledColor);
		jgcr.setRect(JoglVectord4.Add(jglv.getRect(), new JoglVectord4(-0.01, (lineHeight * -0.5f), 0.0, 0)));
		coloredRects.add(jgcr);

		AddLabelToBatchTextMesh(jglv, offy);
	}
	
	private void AddCategoryData(String ls, String cohortName, FontFormat ff2)
	{
		JoglGuiLabel jglv = AddTextLine(ls, ff2);
		
		JoglVectord4 cohortColor = JoglGuiStyle.disabledColor;
		if (jccm != null)
		{
			cohortColor = jccm.getColor4ForCohort(cohortName, 0.5);
		}
		
		//add cohort marker
		JoglGuiColoredRect jgcr = new JoglGuiColoredRect();
		jgcr.setColor(cohortColor);
		jgcr.setRect(JoglVectord4.Add(jglv.getRect(), new JoglVectord4(-0.01, (lineHeight * -0.5f), -0.49f, 0)));
		coloredRects.add(jgcr);
		
		AddLabelToBatchTextMesh(jglv, offy);
	}
	
	private JoglGuiLabel AddTextLine(String ls, FontFormat ff2)
	{
		JoglGuiLabel jglv = new JoglGuiLabel(ff2.clone());
		jglv.setText(ls);
		jglv.setRect(xoff - 0.24f, (yoff + offy) - lineHeight, 0.5f, lineHeight);
		jglv.setBackColor(JoglGuiStyle.transparentColor);
	
		labels.add(jglv);
		
		return jglv;
	}
	
	private void AddLabelToBatchTextMesh(JoglGuiLabel jglv, float offy)
	{
		JoglTexturedMesh jtm2 = jglv.getTextMesh();
		
		if (jtm == null)
		{
			jtm = new JoglTexturedMesh(GL2.GL_QUADS, 4);
			jtm.setVerts(jtm2.getVerts());
			jtm.setTexCoords(jtm2.getTexCoords());
		}
		else
		{
			jtm.addVerts(jtm2.getVerts(), new JoglVectord3(0, offy, 0));
			jtm.addTexCoords(jtm2.getTexCoords());
		}
		
		offsets.add(jtm.getNumVerts());
	}

	
	public void MoveWindow(JoglVectord2 d)
	{
		JoglVectord4 diff = new JoglVectord4(d.getX(), d.getY(), 0, 0);
		
		xoff = xoff + (float)diff.getX();
		yoff = yoff + (float)diff.getY();

		if (jgm != null)
		{
			jgm.setPosition(JoglVectord3.Add(jgm.getPosition(), new JoglVectord3(diff.getX(), diff.getY(), 0)));
		}
		
		for (JoglGuiLabel l : labels)
		{
			l.setRect(JoglVectord4.Add(diff, l.getRect()));
		}
		for (JoglGuiColoredRect cr : coloredRects)
		{
			cr.setRect(JoglVectord4.Add(diff, cr.getRect()));
		}
	}
	
	public void MoveScrollbar(JoglVectord4 diff, float finalHeight)
	{
		if (jgm != null)
		{
			jgm.setPosition(JoglVectord3.Add(jgm.getPosition(), new JoglVectord3(diff.getX(), diff.getY(), 0)));
		}

		//Determine draw range
		float windowTop = yoff + (lineHeight * 1.5f);
		float windowBottom = yoff + finalHeight - lineHeight;	
		
		int firstVis = -1;
		int lastVis = -1;
		int i = 0;
		for (JoglGuiLabel l : labels)
		{
			JoglVectord4 rect = JoglVectord4.Add(diff, l.getRect());
			l.setRect(rect);

			double labelBottom = rect.getY();
			double labelTop = (rect.getY() + rect.getW());

			if ((labelBottom >= windowBottom) && (labelTop <= windowTop))
			{
				l.setVisible(true);
				
				if (firstVis == -1)
				{
					firstVis = i;
				}
				lastVis = i;
			}
			else
			{
				l.setVisible(false);
			}
			i++;
		}
		for (JoglGuiColoredRect cr : coloredRects)
		{
			JoglVectord4 rect = JoglVectord4.Add(diff, cr.getRect());
			cr.setRect(rect);

			double labelBottom = rect.getY();
			double labelTop = (rect.getY() + rect.getW());

			if ((labelBottom >= windowBottom) && (labelTop <= windowTop))
			{
				cr.setVisible(true);
			}
			else
			{
				cr.setVisible(false);
			}
		}
		
		//fix indexing error in some occasions where the scrollbar is not at zero pos on reformat of info
		firstVis = Math.max(0, firstVis);
		
		int firstVert = offsets.get(firstVis);
		int lastVert = offsets.get(lastVis+1);
		int numVerts = lastVert - firstVert;

		if (jgm != null)
		{
			jgm.setDrawRange((firstVert / 4) * 6, (numVerts / 4) * 6);
		}
	}

}
