package org.processmining.processprofiler3d.compviz.jogl.gui;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTarget;
import org.processmining.visualisation3d.graphicsdatastructures.JoglScissorRect;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.gui.JoglGuiObjectCollection;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;

public class JoglGuiScissorBox extends JoglGuiObject
{
	private JSOGraphicalUserInterface gui;

	private JoglScreenRect rect = new JoglScreenRect(0,0,1,1);
	//private JoglGuiObject child = null;
	private JoglGuiObjectCollection children = new JoglGuiObjectCollection();
	
	@Override
	public boolean getCanTouch()
	{
//		if (children != null)
//		{
//			return children.getCanTouch();
//		}
		for (JoglGuiObject jgo : children)
		{
			if (jgo.getCanTouch())
			{
				return true;
			}
		}
		return false;
	}
	
	public JoglVectord4 getRect()
	{
		return rect.getRect();
	}
	
	public void setRect(float x, float y, float w, float h)
	{
		rect = new JoglScreenRect(x, y, w, h);
	}
	
	public void setRect(JoglVectord4 v)
	{
		rect = new JoglScreenRect(v);
	}
	
//	public JoglGuiObject getChild()
//	{
//		return child;
//	}
	
	public void addChild(JoglGuiObject jgo)
	{
		//child = jgo;
		children.add(jgo);
	}
	
	public void removeChild(JoglGuiObject jgo)
	{
		//child = jgo;
		children.remove(jgo);
	}
	
	public JoglGuiScissorBox(JSOGraphicalUserInterface jgui)
	{
		gui = jgui;
	}
	
	@Override
	public boolean IsInside(float x, float y)
	{
		if (getCanTouch())
		{
			if (rect.IsInside(x, y))
			{
				return true;
			}
		}
		return false;
	}
	
	@Override
	public boolean IsChildOf(JoglGuiObject jgo)
	{
		for (JoglGuiObject jgoc : children)
		{
			if (jgoc == jgo)
			{
				return true;
			}
		}
		return false;
	}
	
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		JoglRenderTarget target = pipe.target;
		
		if (children != null)
		{
			int x = (int)(((rect.getLeft() * 0.5) + 0.5) * target.getWidth());
			int y = (int)(((rect.getTop() * 0.5) + 0.5) * target.getHeight());
			int w = (int)(rect.getWidth() * 0.5 * target.getWidth());
			int h = (int)(rect.getHeight() * 0.5 * target.getHeight());
			
			JoglScissorRect srect = new JoglScissorRect(x, y, w, h);
			pipe.SetScissorRect(srect);
			
			for (JoglGuiObject jgo : children)
			{
				jgo.DrawObject(pipe);
			}
			
			pipe.SetScissorRect(null);
		}
	}
}
