package org.processmining.processprofiler3d.compviz.jogl.gui;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.interactions.IScrollBarListener;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIScrollBarMove;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;

public class JoglGuiScrollBar extends JoglGuiObject
{
	private JSOGraphicalUserInterface gui;
	
	private JoglScreenRect rect = new JoglScreenRect(0,0,1,1);
	
	private JoglGuiColoredFrame scrollBarFrame;
	private JoglGuiColoredRect scrollBar;
	
	private float lineWidthV = 0.005f;
	private float lineWidthH = 0.005f;
	
	private float minScrollBarHeight = 0.05f;
	
	private JoglVectord4 scrollbarColor = new JoglVectord4(0.5f, 0.5f, 0.5f, 1.0f);
	private JoglVectord4 scrollbarColorHi = new JoglVectord4(0.85f, 0.85f, 0.85f, 1.0f);
	
	private float perc = 0.2f;
	private float scrolloff = 0.0f;
	private float scrolloffmax = 0;
	
	private boolean bMovingScrollbar = false;
	private JoglVectord2 lastPos = null;

	private boolean bCanTouch = false;
	private boolean bHighlighted = false;
	
	private Vector<IScrollBarListener> listeners = new Vector<IScrollBarListener>();
	
	@Override
	public boolean getCanTouch()
	{
		return bCanTouch;
	}
	
	@Override
	public void setCanTouch(boolean b)
	{
		bCanTouch = b;
	}
	
	public JoglVectord4 getRect()
	{
		return rect.getRect();
	}
	
	public void setRect(float x, float y, float w, float h)
	{
		rect = new JoglScreenRect(x, y, w, h);
		
		if (gui != null)
		{
			updatePos();
		}
	}
	
	public void setRect(JoglVectord4 v)
	{
		rect = new JoglScreenRect(v);
		
		if (gui != null)
		{
			updatePos();
		}
	}
	
	private void updatePos()
	{
		JSOGraphicalUserInterface oldgui = gui;
		OnRemoveFromGui(gui);
		OnAddToGui(oldgui);
	}
	
	public void setPerc(float p)
	{
		perc = p;
	}
	
	@Override
	public void setZLevel(float z)
	{
		zlevel = z;
		
		if (scrollBarFrame != null)
		{
			scrollBarFrame.setZLevel(zlevel);
		}
		
		if (scrollBar != null)
		{
			scrollBar.setZLevel(zlevel);
		}
	}
	
	public void addScrollbarListener(IScrollBarListener sbl)
	{
		listeners.add(sbl);
	}
	
	public boolean IsScrollbarMoving()
	{
		return bMovingScrollbar;
	}
	
	@Override
	public boolean IsInside(float x, float y)
	{
		if (bCanTouch)
		{
			if (rect.IsInside(x, y))
			{
				return true;
			}
		}
		return false;
	}
	
	@Override
	public boolean IsChildOf(JoglGuiObject jgo)
	{
		if (scrollBar == jgo)
		{
			return true;
		}
		if (scrollBarFrame == jgo)
		{
			return true;
		}
		return false;
	}
	
	@Override
	public void OnAddToGui(JSOGraphicalUserInterface g)
	{
		gui = g;
		
		float aspect = (float)gui.getAspect();	
		lineWidthH = 0.005f / aspect;
		
		float scrollBarWidth = rect.getWidth() - (lineWidthH * 2.0f);
		float scrollBarHeight = Math.max(minScrollBarHeight, ((rect.getHeight() - (lineWidthV * 2.0f)) * perc));
		scrolloffmax = -((rect.getHeight() - (lineWidthV * 2.0f)) - scrollBarHeight);
		
		scrollBarFrame = new JoglGuiColoredFrame();
		scrollBarFrame.setRect(rect.getLeft(), rect.getTop(), rect.getWidth(), rect.getHeight());
		scrollBarFrame.setLine(lineWidthH, -lineWidthV);
		scrollBarFrame.setColor(scrollbarColor);
		scrollBarFrame.setZLevel(zlevel);
		
		gui.AddGuiObject(scrollBarFrame);
		
		
		scrollBar = new JoglGuiColoredRect();
		scrollBar.setRect(rect.getLeft() + lineWidthH, (rect.getTop() + scrolloff - lineWidthV + rect.getHeight()) - scrollBarHeight, scrollBarWidth, scrollBarHeight);
		scrollBar.setCanTouch(true);
		if (bHighlighted)
		{
			scrollBar.setColor(scrollbarColorHi);
		}
		else
		{
			scrollBar.setColor(scrollbarColor);
		}
		scrollBar.setZLevel(zlevel);
		
		gui.AddGuiObject(scrollBar);	
	}
	
	@Override
	public void OnRemoveFromGui(JSOGraphicalUserInterface g)
	{
		if (gui != null)
		{
			gui.RemoveGuiObject(scrollBar);
			gui.RemoveGuiObject(scrollBarFrame);
			
			gui = null;
		}
	}
	
	//MouseListener method implementation
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
    	if (gui == null)
    		return false;
    	
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == scrollBar)
			{
				bMovingScrollbar = true;
				virtualoff = scrolloff;
				lastPos = gui.GetNDCPos(e.getX(), e.getY());
				scrollBar.setColor(scrollbarColorHi);
				bHighlighted = true;
				
				return true;
			}
		}
		
		return false;
    }
	
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
    	if (gui == null)
    		return false;
    	
		if (bMovingScrollbar)
		{
			bMovingScrollbar = false;
			scrollBar.setColor(scrollbarColor);
			bHighlighted = false;
			
			return true;
		}
		return false;
    }
	
    public void mouseDragged(JoglInputMouseEvent e) 
    {
    	if (gui == null)
    		return;
    	
		if (bMovingScrollbar)
		{
			JoglVectord2 newPos = gui.GetNDCPos(e.getX(), e.getY());
			MoveScrollbar(JoglVectord2.Subtract(newPos, lastPos));
			lastPos = newPos;
		}
    }
	
    private float virtualoff = 0;
    
	private void MoveScrollbar(JoglVectord2 d)
	{
		if (scrollBar == null)
			return;
		
		float oldscrolloff = scrolloff;
		virtualoff = virtualoff + (float)d.getY();
		scrolloff = virtualoff;
		scrolloff = Math.min(0, scrolloff);
		scrolloff = Math.max(scrolloffmax, scrolloff);
		//System.out.println("ScrolloffPerc: " + (scrolloff / scrolloffmax));
		
		float diff = scrolloff - oldscrolloff;
		scrollBar.transform(new JoglVectord4(0, diff, 0, 0));
		
		if (diff != 0)
		{
			OnMoved(d);
		}
	}
	
	public void OnMoved(JoglVectord2 d)
	{
		for (IScrollBarListener sbl : listeners)
		{
			sbl.OnScrollBarMoved(this, d);
		}
	}
	
	public float getScrollPerc()
	{
		return (scrolloff / scrolloffmax);
	}
	
	public void setScrollPerc(float sperc)
	{
		sperc = Math.min(1.0f, Math.max(0.0f, sperc));
		
		float scrollBarHeight = Math.max(minScrollBarHeight, ((rect.getHeight() - (lineWidthV * 2.0f)) * perc));
		scrolloffmax = -((rect.getHeight() - (lineWidthV * 2.0f)) - scrollBarHeight);
		
		scrolloff = scrolloffmax * sperc;

		if (gui != null)
		{
			float aspect = (float)gui.getAspect();		
			lineWidthH = 0.005f / aspect;
	
			float scrollBarWidth = rect.getWidth() - (lineWidthH * 2.0f);
			scrollBar.setRect(rect.getLeft() + lineWidthH, (rect.getTop()+ scrolloff - lineWidthV + rect.getHeight()) - scrollBarHeight, scrollBarWidth, scrollBarHeight);
		}
	}
	
	public void StartMoveInteraction(JoglInteractionManager iman, JoglInputMouseEvent e)
	{
		JIScrollBarMove sbm = new JIScrollBarMove(iman, gui, this, e);
		iman.addInteraction(sbm);
	}
}
