package org.processmining.processprofiler3d.compviz.jogl.gui;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.fileio.DefaultFontData;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialGuiTextRenderer;
import org.processmining.visualisation3d.FontData;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.opengl.GL2;

public class JoglGuiTextRenderer extends JoglSceneObject
{
	private static MaterialGuiTextRenderer matTR = new MaterialGuiTextRenderer();
	
	private JoglTexturedMesh mesh;
	
	private int charNum = 0;
	private String str = "";

	private FontFormat textFormat;
	private FontData font;

	private float left = 0.0f;
	private float top = 0.0f;
	private float width = 1.0f;
	private float height = 1.0f;
	
	private double maxWidth = 0.0f;

	private JoglVectord3 position = new JoglVectord3(0,0,0);
	private JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateIdentity();

	public JoglGuiTextRenderer()
	{	
		textFormat = new FontFormat();
		font = (FontData)JoglResourceManager.GetResource("DefaultFont");

		if (font == null)
		{
			font = new DefaultFontData();
			JoglResourceManager.AddResource("DefaultFont", font);
			System.out.println("JoglGuiTextRenderer: no font data present!");
		}
	}
	
	public JoglGuiTextRenderer(FontFormat ff)
	{
		textFormat = ff;	
		font = (FontData)JoglResourceManager.GetResource("DefaultFont");
		
		if (font == null)
		{
			font = new DefaultFontData();
			JoglResourceManager.AddResource("DefaultFont", font);
			System.out.println("JoglGuiTextRenderer: no font data present!");
		}
	}
	
	public int getAlign()
	{
		return textFormat.align;
	}
	
	public void setAlign(int a)
	{
		textFormat.align = a;
	}
	
	public void setColor(float r, float g, float b)
	{
		textFormat.setColor(r, g, b);	
	}
	
	public void setColor(JoglVectord3 v)
	{
		textFormat.setColor((float)v.getX(), (float)v.getY(), (float)v.getZ());
	}
	
	public void setPosition(double x, double y, double z)
	{
		position = new JoglVectord3(x,y,z);
		transform = JoglMatrixd4x4.CreateTranslation(x, y, z);
	}
	
	public String getString()
	{
		return str;
	}
	
	public void setString(String s)
	{
		if (!str.equals(s))
		{
			str = s;
			charNum = JoglGuiTextRenderer.getCharNum(str);
			maxWidth = 0.0f;
			
			mesh = new JoglTexturedMesh(GL2.GL_QUADS);
			mesh.setVerts(GenerateGeometry(str, textFormat));
			mesh.setTexCoords(GenerateTextureCoords(str));
			mesh.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(mesh.getNumVerts()));
			
			if (vao != null)
			{
				vao.Dispose();
				vao = null;
			}
			
			UpdateBounds();
		}
	}
	
	public void setString(String s, double maxWidth)
	{
		if (!(str.equals(s) && this.maxWidth == maxWidth))
		{
			str = s;
			charNum = JoglGuiTextRenderer.getCharNum(str);
			this.maxWidth = maxWidth;

			mesh = new JoglTexturedMesh(GL2.GL_QUADS);
			mesh.setVerts(GenerateGeometry(str, textFormat, maxWidth));
			mesh.setTexCoords(GenerateTextureCoords(str));
			mesh.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(mesh.getNumVerts()));

			if (vao != null)
			{
				vao.Dispose();
				vao = null;
			}
			
			UpdateBounds();
		}
	}
	
	public FontFormat getTextFormat()
	{
		return textFormat;
	}
	
	public JoglTexturedMesh getMesh()
	{
		return mesh;
	}
	
	public JoglVectord2 getTextSize()
	{
		return new JoglVectord2(textFormat.width, textFormat.height);
	}
	
	public void setTextSize(float tx, float ty)
	{
		if (tx == textFormat.width && ty == textFormat.height)
		{
			return;
		}

		textFormat.width = tx;
		textFormat.height = ty;
		
		mesh = new JoglTexturedMesh(GL2.GL_QUADS);
		if (maxWidth > 0)
		{
			mesh.setVerts(GenerateGeometry(str, textFormat, maxWidth));
		}
		else
		{
			mesh.setVerts(GenerateGeometry(str, textFormat));		
		}
		mesh.setTexCoords(GenerateTextureCoords(str));
		mesh.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(mesh.getNumVerts()));

		if (vao != null)
		{
			vao.Dispose();
			vao = null;
		}
		
		UpdateBounds();
	}
	
	private void UpdateBounds()
	{
		double minx = Double.POSITIVE_INFINITY;
		double maxx = Double.NEGATIVE_INFINITY;
		double miny = Double.POSITIVE_INFINITY;
		double maxy = Double.NEGATIVE_INFINITY;

		double[] vertsd = mesh.getVerts();
		
		int id = 0;
		for (int i = 0; i < (vertsd.length / 3); i++)
		{
			double x = vertsd[id];
			double y = vertsd[id+1];
			//double z = vertsd[id+2];
			
			minx = Math.min(minx, x);
			miny = Math.min(miny, y);
			maxx = Math.max(maxx, x);
			maxy = Math.max(maxy, y);
			
			id = id + 3;
		}
		
		left = (float) minx;
		width = (float) (maxx - minx);
		
		top = (float) miny;
		height = (float) (maxy - miny);
		
		left = left + (float)position.getX();
		top = top + (float)position.getY();
	}
	
	public static int getCharNum(String s)
	{
		int cNum = 0;
		for (int i = 0; i < s.length(); i++)
		{
			char c = s.charAt(i);
			if (c != ' ')
			{
				cNum = cNum + 1;
			}
		}
		
		return cNum;
	}

	private Vector<String> BreakTextIntoLines(String text, FontFormat ff, double twidth) 
	{
		Vector<String> lines = new Vector<String>();
		double lineWidth = 0.0;
		int splitPos = -1;
		String line = text;
		int ix = 0;
		while (ix < line.length())
		{
			lineWidth = lineWidth + font.GetCharWidth(line.charAt(ix), ff.width);
			if (line.charAt(ix) == ' ')
			{
				splitPos = ix;
			}
			
			if (lineWidth > twidth)
			{
				//need to split string
				if (splitPos > -1)
				{
					lines.add(line.substring(0, splitPos).trim());
					line = line.substring(splitPos);
					splitPos = -1;
					ix = 1;
				}
				else
				{
					lines.add(line.substring(0, ix-1).trim());
					line = line.substring(ix-1);
					ix = 0;
				}
				lineWidth = 0.0;
			}
			if (lineWidth > 0.0) // if have not just split the line
			{
				lineWidth = lineWidth + (ff.tweenChars * ff.width);
			}
			ix++;
		}
		lines.add(line.trim());
		return lines;
	}
	
	public JoglVectord2 MeasureText(String text, FontFormat ff)
	{
		return new JoglVectord2(MeasureLineLength(text, ff), ff.height);
	}
	
	public JoglVectord2 MeasureText(String text, FontFormat ff, double twidth)
	{
		Vector<String> lines = BreakTextIntoLines(text, ff, twidth);
		
		//center
		double maxxoff = 0.0;
		double yoff = 0.0;
		for (int j = 0; j < lines.size(); j++)
		{
			String lstr = lines.get(j);
			double xoff = MeasureLineLength(lstr, ff);
			maxxoff = Math.max(xoff, maxxoff);
			yoff = yoff + ff.height;
		}
		return new JoglVectord2(maxxoff, yoff);
	}
	
	private double MeasureLineLength(String line, FontFormat ff)
	{
		double lineWidth = 0.0;
		double tweenCharsScaled = ff.tweenChars * ff.width;
		for (int i = 0; i < line.length(); i++)
		{
			char c = line.charAt(i);
			double w = font.GetCharWidth(c, ff.width);
			lineWidth = lineWidth + w + tweenCharsScaled;
		}
		lineWidth = lineWidth - tweenCharsScaled;
		return lineWidth;
	}
	
	public Vector<JoglVectord3> GenerateGeometry(String text, FontFormat ff) 
	{
		Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();
		
		GenerateLineGeometry(text, ff, 0.0, vlist);
		
		return vlist;
	}
	
	public Vector<JoglVectord3> GenerateGeometry(String text, FontFormat ff, double twidth) 
	{		
		Vector<String> lines = BreakTextIntoLines(text, ff, twidth);		
		
		Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();

		//center
		double yoff = ((lines.size() - 1) * ff.height * 0.5);
		for (int j = 0; j < lines.size(); j++)
		{
			String lstr = lines.get(j);
			
			GenerateLineGeometry(lstr, ff, yoff, vlist);
			
			yoff = yoff - ff.height;
		}
		
		return vlist;
	}

	private void GenerateLineGeometry(String lstr, FontFormat ff,
			double yoff, Vector<JoglVectord3> vlist) 
	{
		double halfheight = ff.height * 0.5;
		double xoff = 0.0;
		double lineWidth2 = MeasureLineLength(lstr, ff);
		
		//center
		if (ff.align == 0)
		{
			xoff = lineWidth2 * -0.5;
		}
		//left
		else if (ff.align == 1)
		{
			xoff = 0.0;
		}
		//right
		else if (ff.align == 2)
		{
			xoff = lineWidth2 * -1.0;
		}
		
		double tweenCharsScaled = ff.tweenChars * ff.width;
		for (int i = 0; i < lstr.length(); i++)
		{
			double cwidth = font.GetCharWidth(lstr.charAt(i), ff.width);
			if (lstr.charAt(i) != ' ')
			{
				vlist.add(new JoglVectord3(xoff, halfheight + yoff, 0.0));   // Top Left
				vlist.add(new JoglVectord3(xoff + cwidth, halfheight + yoff, 0.0));   // Top Right
				vlist.add(new JoglVectord3(xoff + cwidth, -halfheight + yoff, 0.0));   // Bottom Right
				vlist.add(new JoglVectord3(xoff, -halfheight + yoff, 0.0));   // Bottom Left
			}
			xoff = xoff + cwidth + tweenCharsScaled;
		}
	}
	
	public Vector<JoglVectord2> GenerateTextureCoords(String text) 
	{
		double xmin = 0.0;
		double xmax = 1.0;
		double ymin = 0.0;
		double ymax = 1.0;
		
		double invx = 1.0 / 16.0f;
		
		Vector<JoglVectord2> vlist = new Vector<JoglVectord2>();
		
		for (int i = 0; i < text.length(); i++)
		{
			char c = text.charAt(i);
			if (c != ' ')
			{
				int ic = c - 32;
	
				int x = ic % 16;
				int y = (ic - x) / 16;
				
				float left = (float)font.GetCharLeft(c);
				float right = (float)font.GetCharRight(c);
				
				xmin = x * invx;
				xmax = xmin + invx;
				ymin = 1.0 - (y * invx);
				ymax = 1.0 - ((y * invx) + invx);
				
				vlist.add(new JoglVectord2(xmin + left, ymin));   // Top Left
				vlist.add(new JoglVectord2(xmax - right, ymin));   // Top Right
				vlist.add(new JoglVectord2(xmax - right, ymax));   // Bottom Right
				vlist.add(new JoglVectord2(xmin + left, ymax));   // Bottom Left
			}
		}
		
		return vlist;
	}
	
	public void DrawObject(GraphicsPipeline pipe)
	{
		// Draw A Quad
		DrawWithTextureOnly(pipe);
	}

	public double forceScale = 1.0;
	private IMeshDrawer vao;

	private void DrawWithTextureOnly(GraphicsPipeline pipe) 
	{
		if (mesh != null)
		{
			pipe.transform.Push();
			pipe.transform.MultMatrix(transform);
			pipe.transform.Scaled(forceScale, forceScale, forceScale);
	
			JoglTexture tex = font.getTexture(pipe);
			tex.Bind(pipe);
			
			pipe.PushPipelineState(matTR);
			matTR.DrawColor(textFormat.colorR, textFormat.colorG, textFormat.colorB, 1.0f);
			
			if (vao == null)
			{
				vao = new VAODrawer();
				vao.LoadMesh(pipe, mesh);
			}
			vao.Draw(pipe);
	
			pipe.PopPipelineState(matTR);
			tex.Unbind(pipe);
	
			pipe.transform.Pop();
		}
	}
}
