package org.processmining.processprofiler3d.compviz.jogl.gui;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialGuiTexturedRect;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderPipelineDefaultTextured;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;

import com.jogamp.opengl.GL2;

public class JoglGuiTexturedRect extends JoglGuiObject
{
	private JoglTexture tex = null;
	private float colorR = 1.0f;
	private float colorG = 1.0f;
	private float colorB = 1.0f;
	private float colorA = 1.0f;
	
	private JoglScreenRect rect = new JoglScreenRect(0,0,1,1);

	private boolean bCanTouch = false;
	private boolean bAlpha = true;

	private boolean bUpdateVerts = false;
	
	@Override
	public boolean getCanTouch()
	{
		return bCanTouch;
	}

	@Override
	public void setCanTouch(boolean b)
	{
		bCanTouch = b;
	}
	
	public void setTex(JoglTexture t)
	{
		tex = t;
	}
	
	public void setColor(float r, float g, float b)
	{
		colorR = r;
		colorG = g;
		colorB = b;
	}
	
	public void setColor(float r, float g, float b, float a)
	{
		colorR = r;
		colorG = g;
		colorB = b;
		colorA = a;
	}
	
	public void setColor(JoglVectord3 v)
	{
		colorR = (float)v.getX();
		colorG = (float)v.getY();
		colorB = (float)v.getZ();
	}
	
	public void setColor(JoglVectord4 v)
	{
		colorR = (float)v.getX();
		colorG = (float)v.getY();
		colorB = (float)v.getZ();
		colorA = (float)v.getValue(3);
	}
	
	public JoglVectord4 getRect()
	{
		return rect.getRect();
	}
	
	public void setRect(float x, float y, float w, float h)
	{
		rect = new JoglScreenRect(x, y, w, h);
		
		bUpdateVerts = true;
	}
	
	public void setRect(JoglVectord4 v)
	{
		rect = new JoglScreenRect(v);
		
		bUpdateVerts = true;
	}
	
	private void updateVAOVerts(GraphicsPipeline pipe)
	{
		if (vao != null)
		{
			vao.UpdateDataBufferJV3(pipe, 0, GenerateGeometry());
		}
	}
	
	@Override
	public boolean IsInside(float x, float y)
	{
		if (bCanTouch)
		{
			if (rect.IsInside(x, y))
			{
				return true;
			}
		}
		return false;
	}
	
	private Vector<JoglVectord3> GenerateGeometry()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
		
		lst.add(new JoglVectord3(rect.getLeft(), rect.getTop(), 0.0f));
		lst.add(new JoglVectord3(rect.getRight(), rect.getTop(), 0.0f));
		lst.add(new JoglVectord3(rect.getRight(), rect.getBottom(), 0.0f));
		lst.add(new JoglVectord3(rect.getLeft(), rect.getBottom(), 0.0f));

		return lst;
	}
	
	private Vector<JoglVectord2> GenerateTexCoords()
	{
		Vector<JoglVectord2> lst = new Vector<JoglVectord2>();
		
		lst.add(new JoglVectord2(0.0f, 0.0f));
		lst.add(new JoglVectord2(1.0f, 0.0f));
		lst.add(new JoglVectord2(1.0f, 1.0f));
		lst.add(new JoglVectord2(0.0f, 1.0f));
		
		return lst;
	}
	
	public JoglTexturedMesh CreateMesh()
	{
		JoglTexturedMesh jcm = new JoglTexturedMesh(GL2.GL_QUADS);
		jcm.setVerts(GenerateGeometry());
		jcm.setTexCoords(GenerateTexCoords());
		jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jcm.getNumVerts()));
		
		return jcm;
	}
	
	private IMeshDrawer vao = null;
	private static JoglShader shad = null;
	private static MaterialGuiTexturedRect matTR = new MaterialGuiTexturedRect();
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		if (bVisible)
		{
			pipe.PushPipelineState(matTR);
			matTR.DrawColor(colorR, colorG, colorB, colorA, bAlpha);
//			if (bAlpha || colorA < 1.0)
//			{
//				pipe.setDrawModeAlphaBlended();
//			}

			tex.Bind(pipe);
			
//			gl.glBegin(GL2.GL_QUADS);
//
			//gl.glColor4f(colorR, colorG, colorB, colorA);
			//pipe.setColor(colorR, colorG, colorB, colorA);
//			gl.glTexCoord2f(0.0f, 0.0f);
//			gl.glVertex3f(rect.getLeft(), rect.getTop(), 0.0f);
//			gl.glTexCoord2f(1.0f, 0.0f);
//			gl.glVertex3f(rect.getRight(), rect.getTop(), 0.0f);
//			gl.glTexCoord2f(1.0f, 1.0f);
//			gl.glVertex3f(rect.getRight(), rect.getBottom(), 0.0f);
//			gl.glTexCoord2f(0.0f, 1.0f);
//			gl.glVertex3f(rect.getLeft(), rect.getBottom(), 0.0f);
//			
//			gl.glEnd();
			if (vao == null)
			{
				vao = new VAODrawer();
				vao.LoadMesh(pipe, CreateMesh());
				
				bUpdateVerts = false;
			}
			if (shad == null)
			{
				if (JoglResourceManager.HasResource("PipelineDefaultTexturedShader"))
				{
					shad = (JoglShader)JoglResourceManager.GetResource("PipelineDefaultTexturedShader");
				}
				else
				{
					shad = new JoglShaderPipelineDefaultTextured();
					shad.Create(pipe);
					
					JoglResourceManager.AddResource("PipelineDefaultTexturedShader", shad);
				}
			}
			
			if (bUpdateVerts)
			{
				updateVAOVerts(pipe);
				bUpdateVerts = false;
			}
			
			shad.Bind(pipe);
			vao.Draw(pipe);
			shad.Unbind(pipe);
			
			tex.Unbind(pipe);

//			if (bAlpha || colorA < 1.0)
//			{
//				pipe.setDrawModeOpaque();
//			}
			pipe.PopPipelineState(matTR);
		}
	}
}
