package org.processmining.processprofiler3d.compviz.jogl.gui;

import java.util.Date;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;
import org.processmining.visualisation3d.FontData;
import org.processmining.visualisation3d.JoglResourceManager;

import com.jogamp.opengl.GL2;

public class ListBoxTextMeshConstructor
{
	private FontFormat textFormat;
	private FontData font;
	private FormattedFontCache ffc;
	
	
	public Vector<JoglScreenRect> labels = new Vector<JoglScreenRect>();
	public Vector<Integer> offsets = new Vector<Integer>();
	public JoglTexturedMesh jtm = null;
	
	private static float xoff = 0.0f;
	private static float yoff = 0.0f;
	
	public ListBoxTextMeshConstructor()
	{
		font = (FontData)JoglResourceManager.GetResource("DefaultFont");
	}
	
	public void constructMesh(Vector<String> dataLines, float aspect, float rW)
	{		
		float labelXOff = xoff + 0.01f;
		float labelW = rW - 0.02f;

		FontFormat ff2 = new FontFormat();
		ff2.width = 0.05f / aspect;
		ff2.height = 0.05f;
		ff2.align = FontFormat.ALIGN_LEFT;
		ff2.setColor(1.0f, 1.0f, 1.0f);

		textFormat = ff2;
		ffc = new FormattedFontCache(font, ff2);

		offsets.clear();
		offsets.ensureCapacity(dataLines.size()+1);
		offsets.add(0);
		
		charsNum = 0;
		for (String str : dataLines)
		{
			charsNum = charsNum + JoglGuiTextRenderer.getCharNum(str);
		}

		verts = new double[charsNum * 12];
		vertPointer = 0;

		uvCoords = new double[charsNum * 8];
		uvPointer = 0;
		
		labels.ensureCapacity(dataLines.size());
		
		//do not try to create a zero length buffer, it will crash!
		if (charsNum > 0)
		{
			Date s1Time = new Date();
			
			float offy = -0.05f;
			for (String ls : dataLines)
			{
				GenerateLineGeometry(ls, offy);
				GenerateTextureCoords(ls);
				
				JoglScreenRect rect = new JoglScreenRect(labelXOff, (yoff + offy), labelW, 0.05f);
				
				labels.add(rect);
				offsets.add(vertPointer / 3);
	
				offy = offy - 0.05f;
			}
			
			if (jtm == null)
			{
				jtm = new JoglTexturedMesh(GL2.GL_QUADS, verts.length / 3);
				jtm.setVerts(verts);
				jtm.setTexCoords(uvCoords);
			}
			
			Date e1Time = new Date();
			TimeSpan ts = new TimeSpan(s1Time, e1Time);
			
			System.out.println("List label meshes generated (" + ts.ToString(TimeUnit.MILLISECONDS) + "ms).");
		}
	}

	private int charsNum = 0;
	
	private double[] verts;
	private int vertPointer = 0;
	
	
	private double[] uvCoords;
	private int uvPointer = 0;
	
	private double MeasureLineLength(String line)
	{
		double lineWidth = 0.0;
		for (int i = 0; i < line.length(); i++)
		{
			char c = line.charAt(i);
			double w = ffc.cwidth[c-32];
			lineWidth = lineWidth + w + ffc.tweenCharsScaled;
		}
		lineWidth = lineWidth - ffc.tweenCharsScaled;
		return lineWidth;
	}

	private void GenerateLineGeometry(String lstr,
			double yoff) 
	{
		double halfheight = textFormat.height * 0.5;
		double xoff = 0.0;
		double lineWidth2 = MeasureLineLength(lstr);
		
		//center
		if (textFormat.align == 0)
		{
			xoff = lineWidth2 * -0.5;
		}
		//left
		else if (textFormat.align == 1)
		{
			xoff = 0.0;
		}
		//right
		else if (textFormat.align == 2)
		{
			xoff = lineWidth2 * -1.0;
		}
		
		for (int i = 0; i < lstr.length(); i++)
		{
			//double cwidth = font.GetCharWidth(lstr.charAt(i), textFormat.width);
			double cwidth = ffc.cwidth[lstr.charAt(i)-32];
			if (lstr.charAt(i) != ' ')
			{
				double vleft = xoff;
				double vright = xoff + cwidth;
				double vtop = halfheight + yoff;
				double vbottom = -halfheight + yoff;
				
				verts[vertPointer] = vleft;
				verts[vertPointer + 1] = vtop;
				verts[vertPointer + 2] = 0.0;

				verts[vertPointer + 3] = vright;
				verts[vertPointer + 4] = vtop;
				verts[vertPointer + 5] = 0.0;

				verts[vertPointer + 6] = vright;
				verts[vertPointer + 7] = vbottom;
				verts[vertPointer + 8] = 0.0;

				verts[vertPointer + 9] = vleft;
				verts[vertPointer + 10] = vbottom;
				verts[vertPointer + 11] = 0.0;
				
				vertPointer = vertPointer + 12;
			}
			xoff = xoff + cwidth + ffc.tweenCharsScaled;
		}
	}
	
	public void GenerateTextureCoords(String text) 
	{
		for (int i = 0; i < text.length(); i++)
		{
			char c = text.charAt(i);
			if (c != ' ')
			{
				int ic = c -33;
				
				int cacheId = (ic * 8);
				for (int ix = 0; ix < 8; ix++)
				{
					uvCoords[uvPointer] = ffc.uvCache[cacheId];
					uvPointer++;
					cacheId++;
				}
			}
		}
	}

}
