package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.text.DecimalFormat;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PSENLabel;
import org.processmining.processprofiler3d.psen.datastructure.PerformanceStatisticsDB;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;

public class AsyncInfoThreadTransition extends AsyncInfoThread 
{
	private static Vector<TimeUnit> tus = null; 
	
	private IPerformanceStatisticsEnrichedNet psnet;
	private JSOPnTransition trans = null;
	
	public AsyncInfoThreadTransition(JSOPnTransition trans, IPerformanceStatisticsEnrichedNet psnet, JIPSENBaseInfo baseInfo)
	{
		super(baseInfo);
		
		this.trans = trans;
		this.psnet = psnet;

		if (tus == null)
		{
			tus = new Vector<TimeUnit>();
			tus.add(TimeUnit.DAYS);
			tus.add(TimeUnit.HOURS);
			tus.add(TimeUnit.MINUTES);
			tus.add(TimeUnit.SECONDS);
			tus.add(TimeUnit.MILLISECONDS);
		}
	}
	
    public void run() 
    {
		PerformanceStatisticsDB db = (PerformanceStatisticsDB)psnet;
		Vector<InfoPanelCategory> lines = new Vector<InfoPanelCategory>();
		
		Date startTime = new Date();
		
		Vector<String> labelNames = new Vector<String>();
		for (PSENLabel l : psnet.getLabels())
		{
			labelNames.add(l.getLabelName());
		}
		
		Collections.sort(labelNames);
		
		Vector<String> classifierValues = new Vector<String>();
		for (String s : psnet.getClassifierValues())
		{
			classifierValues.add(s);
		}
		
		Collections.sort(classifierValues);
		
		//HashMap<String, Double> tmp = db.getTransitionLabelValueGroupsOpt();
		//db.getTransitionLabelValueGroupsOptTbl();
		
		//Date endTime2 = new Date();
		//TimeSpan ts2 = new TimeSpan(startTime, endTime2);
		
		//System.out.println("Info data opt gathered in " + ts2.ToString(TimeUnit.MILLISECONDS) + "ms");

		String tId = trans.orgTransition.getLocalID().toString();
		HashMap<String, Double> tmp = db.getTransitionLabelValueGroupsOptT(tId);
		
		for (String labelName : labelNames)
		{
			//HashMap<String, Double> hm = db.getTransitionLabelValueGroups(tId, labelName);
			
			InfoPanelCategory ipc = new InfoPanelCategory(labelName);
			
			for (String classifierValue : classifierValues)
			{
				if (bAbort)
				{
					return;
				}
				
				//String idx = classifierValue;
				//if (hm.containsKey(idx))
				String idx = classifierValue + "x" + tId + "x" + labelName;
				if (tmp.containsKey(idx))
				{
					//double val = hm.get(idx);
					double val = tmp.get(idx);
					String valueStr = FormatValueString(psnet, labelName, classifierValue, val);
					
					ipc.dataLines.add("  " + classifierValue + ": " + valueStr);
				}
			}
			lines.add(ipc);
			
			baseInfo.SetData(new Vector<InfoPanelCategory>(lines));
		}

		Date endTime = new Date();
		TimeSpan ts = new TimeSpan(startTime, endTime);
		
		System.out.println("Info data gathered in " + ts.ToString(TimeUnit.MILLISECONDS) + "ms");
		bProcessingDone = true;
		
		//baseInfo.SetData(lines);
    }

	public String FormatValueString(IPerformanceStatisticsEnrichedNet psnet,
			String labelName, String classifierValue, double val) 
	{

		DecimalFormat df = new DecimalFormat("#.####");
		boolean isTimeSpan = psnet.getLabelByName(labelName).isTimeSpan();

		if (isTimeSpan)
		{
			TimeSpan ts = new TimeSpan((long)val);
			
			//return ts.ToString(tus);
			return ts.ToStringExp(tus, 4);
		}
		else
		{
			return df.format(val);
		}
	}
}
