package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.text.DecimalFormat;
import java.util.Collections;
import java.util.HashMap;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PSENLabel;
import org.processmining.processprofiler3d.psen.datastructure.PerformanceStatisticsDB;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;

public class CSVDataExporter
{
	protected Vector<Vector<String>> dataLines = new Vector<Vector<String>>();
	private Vector<TimeUnit> tus = new Vector<TimeUnit>();
	
	public CSVDataExporter(IPerformanceStatisticsEnrichedNet psnet)
	{
		tus.add(TimeUnit.DAYS);
		tus.add(TimeUnit.HOURS);
		tus.add(TimeUnit.MINUTES);
		tus.add(TimeUnit.SECONDS);
		tus.add(TimeUnit.MILLISECONDS);
		
		dataLines.add(StatsList(psnet));
		dataLines.add(StatsListCat(psnet));
	}
	
	public void addStatsFor(JSOPnTransition trans, IPerformanceStatisticsEnrichedNet psnet)
	{
		dataLines.add(StatsListVal(trans, psnet));
	}
	
	public void addStatsFor(String translabel, String transID, IPerformanceStatisticsEnrichedNet psnet)
	{
		dataLines.add(StatsListVal(translabel, transID, psnet));
	}
	
	private Vector<String> StatsList(IPerformanceStatisticsEnrichedNet psnet)
	{
		Vector<String> lines = new Vector<String>();
		lines.add("LocalID");
		lines.add("Label");
		
		Vector<String> labelNames = new Vector<String>();
		for (PSENLabel l : psnet.getLabels())
		{
			labelNames.add(l.getLabelName());
		}
		
		Collections.sort(labelNames);
		
		Vector<String> classifierValues = new Vector<String>();
		for (String s : psnet.getClassifierValues())
		{
			classifierValues.add(s);
		}
		
		Collections.sort(classifierValues);

		for (String labelName : labelNames)
		{
			for (String classifierValue : classifierValues)
			{
				lines.add(labelName);
			}
		}
		
		return lines;
	}
	
	private Vector<String> StatsListCat(IPerformanceStatisticsEnrichedNet psnet)
	{
		Vector<String> lines = new Vector<String>();
		lines.add("None");
		lines.add("Cohort");
		
		Vector<String> labelNames = new Vector<String>();
		for (PSENLabel l : psnet.getLabels())
		{
			labelNames.add(l.getLabelName());
		}
		
		Collections.sort(labelNames);
		
		Vector<String> classifierValues = new Vector<String>();
		for (String s : psnet.getClassifierValues())
		{
			classifierValues.add(s);
		}
		
		Collections.sort(classifierValues);

		for (String labelName : labelNames)
		{	
			for (String classifierValue : classifierValues)
			{
				lines.add(classifierValue);
			}
		}
		
		return lines;
	}
	
	private Vector<String> StatsListVal(String translabel, String trans, IPerformanceStatisticsEnrichedNet psnet)
	{
		Vector<String> lines = new Vector<String>();
		lines.add(trans);
		lines.add(translabel);
		
		Vector<String> labelNames = new Vector<String>();
		for (PSENLabel l : psnet.getLabels())
		{
			labelNames.add(l.getLabelName());
		}
		
		Collections.sort(labelNames);
		
		Vector<String> classifierValues = new Vector<String>();
		for (String s : psnet.getClassifierValues())
		{
			classifierValues.add(s);
		}
		
		Collections.sort(classifierValues);

		String tId = trans;
		
		for (String labelName : labelNames)
		{
			PerformanceStatisticsDB db = (PerformanceStatisticsDB)psnet;
			HashMap<String, Double> hm = db.getTransitionLabelValueGroups(tId, labelName);
			
			//InfoPanelCategory ipc = new InfoPanelCategory(labelName);
			
			for (String classifierValue : classifierValues)
			{
				String idx = classifierValue;
				if (hm.containsKey(idx))
				{
					double val = hm.get(idx);
					String valueStr = FormatValueString(psnet, labelName, classifierValue, val);
					
					//lines.add("" + classifierValue + ": " + valueStr);
					lines.add(valueStr);
				}
				else
				{
					String valueStr = "-";
					lines.add(valueStr);
				}
			}
		}
		
		return lines;
	}
	
	private Vector<String> StatsListVal(JSOPnTransition trans, IPerformanceStatisticsEnrichedNet psnet)
	{
		Vector<String> lines = new Vector<String>();
		lines.add(trans.orgTransition.getLocalID().toString());
		lines.add(trans.getLabel());
		
		Vector<String> labelNames = new Vector<String>();
		for (PSENLabel l : psnet.getLabels())
		{
			labelNames.add(l.getLabelName());
		}
		
		Collections.sort(labelNames);
		
		Vector<String> classifierValues = new Vector<String>();
		for (String s : psnet.getClassifierValues())
		{
			classifierValues.add(s);
		}
		
		Collections.sort(classifierValues);

		String tId = trans.orgTransition.getLocalID().toString();
		
		for (String labelName : labelNames)
		{
			PerformanceStatisticsDB db = (PerformanceStatisticsDB)psnet;
			HashMap<String, Double> hm = db.getTransitionLabelValueGroups(tId, labelName);
			
			//InfoPanelCategory ipc = new InfoPanelCategory(labelName);
			
			for (String classifierValue : classifierValues)
			{
				String idx = classifierValue;
				if (hm.containsKey(idx))
				{
					double val = hm.get(idx);
					String valueStr = FormatValueString(psnet, labelName, classifierValue, val);
					
					//lines.add("" + classifierValue + ": " + valueStr);
					lines.add(valueStr);
				}
				else
				{
					String valueStr = "-";
					lines.add(valueStr);
				}
			}
		}
		
		return lines;
	}

	public String FormatValueString(IPerformanceStatisticsEnrichedNet psnet,
			String labelName, String classifierValue, double val) 
	{
		DecimalFormat df = new DecimalFormat("#.####");
		boolean isTimeSpan = psnet.getLabelByName(labelName).isTimeSpan();

		if (isTimeSpan)
		{
			TimeSpan ts = new TimeSpan((long)val);
			
			//return ts.ToString(tus);
			return ts.ToStringExp(tus, 4);
		}
		else
		{
			return df.format(val);
		}
	}
	
	public void WriteToCSV(String filePath) throws IOException
	{
		FileOutputStream fos;
		Writer out;

		System.out.println("Exporting model to : " + filePath);
        fos = new FileOutputStream(filePath);
        out = new OutputStreamWriter(fos, "UTF8");
        
        //out.write(",NodeType,Label,xPos,yPos\r\n");
        for (int j = 0; j < dataLines.firstElement().size(); j++)
        {
        	String line = "";
        	for (int i = 0; i < dataLines.size(); i++)
        	{
        		String val = dataLines.get(i).get(j);
        		
        		line += "\"" + val + "\", ";
        	}
        	out.write(line + "\r\n");
        }
        
        out.close();
	}
}
