package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.awt.event.MouseEvent;

import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredFrame;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.maths.JoglColor3;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIDragSelection extends JoglInteraction
{	
	protected JoglInteractionManager iman;
	protected JoglScene scene;
	protected JSOGraphicalUserInterface gui;

	protected boolean bUpdateScene = false;
	
	private JoglVectord2 startRay;
	private JoglVectord2 endRay;
	private JoglGuiColoredFrame sFrame = null;
	
	private JISelectModelElement jise;
	
	public JIDragSelection(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui, JISelectModelElement jise, JoglInputMouseEvent e)
	{
		this.scene = scene;
		this.gui = gui;
		this.iman = iman;
		this.jise = jise;
		
		setZLevel(-2.0f);
	
		startRay = gui.GetNDCPos(e.getX(), e.getY());

		float lineWidth = 0.005f;
		sFrame = new JoglGuiColoredFrame();
		sFrame.setColor(JoglColor3.WHITE);
		sFrame.setRect((float)startRay.getX(), (float)startRay.getY(), 0.01f, 0.01f);
		sFrame.setLine(lineWidth / (float)gui.getAspect(), lineWidth);
		
		gui.AddGuiObject(sFrame);
	}
	
	public void Dispose()
	{
		gui.RemoveGuiObject(sFrame);
	}
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		bUpdateScene = false;
    }
	
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		return false;
    }

	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			endRay =  gui.GetNDCPos(e.getX(), e.getY());
			
			double swidth = Math.abs(endRay.getX() - startRay.getX()) / gui.getAspect();
			double sheight = Math.abs(endRay.getY() - startRay.getY());
			
			if (swidth > 0.005 || sheight > 0.005)
			{
				jise.SelectAllInRect(e, startRay, endRay);
			}
			else
			{
				jise.SelectAtMousePos(e);
			}
			
			Dispose();
			iman.removeInteraction(this);
			
			bUpdateScene = true;
			
			return true;
		}
		return false;
    }

	@Override
    public void mouseDragged(JoglInputMouseEvent e)
    {
		JoglVectord2 newPos = gui.GetNDCPos(e.getX(), e.getY());

		endRay = newPos;
		
		double l = Math.min(startRay.getX(), endRay.getX());
		double r = Math.max(startRay.getX(), endRay.getX());
		double t = Math.min(startRay.getY(), endRay.getY());
		double b = Math.max(startRay.getY(), endRay.getY());
		
		sFrame.setRect((float)l, (float)t, (float)(r - l), (float)(b - t));
		
		bUpdateScene = true;
    }
	
	//MouseListener method implementation
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {
		//System.out.println("Mouse click at: " + e.getX() + "/" + e.getY());
		
		return false;
    }
}
