package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.List;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredFrame;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredRect;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiScissorBox;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiScrollBar;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglScreenRect;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.input.JoglInputMouseWheelEvent;
import org.processmining.visualisation3d.interactions.JoglGuiInteraction;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIDropdownBox extends JoglGuiInteraction implements IScrollBarListener
{
	private JoglInteractionManager iman;
	
	private JoglGuiColoredRect background;
	private JoglGuiColoredRect hoverRect;
	private JoglGuiColoredFrame frame;
	private JoglGuiScrollBar scrollBar;
	//private JoglGuiColoredRect scrollBar;
	private JoglGuiScissorBox sci;
	private Vector<JoglGuiLabel> labels;
	private Vector<JoglGuiObject> controls;

	private List<String> values;
	
	private boolean bUpdateScene = false;
	private boolean bInitialized = false;
	
	private float posx = -0.25f;
	private float posy = 0.25f;
	private float width = 0.5f;
	
	private float textSize = 0.05f;
	
	private JoglGuiObject owner = null;
	
	private Vector<IDropDownListener> ddListeners = new Vector<IDropDownListener>();
	
	private JoglVectord4 backColor = new JoglVectord4(0.20f, 0.32f, 0.46f, 0.85f);
	private JoglVectord4 hoverColor = new JoglVectord4(0.45f, 0.5f, 0.6f, 0.75f);
	
	private double scrolloffset = 0.0f;
	private double maxscrolloffset = 0.0f;
	
	public void addDropDownListener(IDropDownListener ddl)
	{
		ddListeners.add(ddl);
	}
	
	public JoglGuiObject getOwner()
	{
		return owner;
	}
	
	public void setPosition(float x, float y)
	{
		posx = x;
		posy = y;
	}
	
	public void setWidth(float w)
	{
		width = w;
	}
	
	public void setTextSize(float s)
	{
		textSize = s;
	}
	
	public JIDropdownBox(JoglInteractionManager iman, JSOGraphicalUserInterface gui, JoglGuiObject owner, List<String> values)
	{
		super(gui);
		
		this.iman = iman;
		this.owner = owner;
		this.values = values;
		
		setZLevel(-1.0f);
		
		labels = new Vector<JoglGuiLabel>();
		controls = new Vector<JoglGuiObject>();
	}

	private void Init()
	{
		float aspect = (float)gui.getAspect();
		
		float textPadding = 0.005f;
		float lineHeight = 0.05f * (float)JoglGuiStyle.textScale;
		
		float bgHeight = values.size() * lineHeight;
		
		//clamp rectangle to screen
		JoglScreenRect backrect = new JoglScreenRect(posx, posy - bgHeight, width, bgHeight);
		float t = backrect.getTop();
		float b = backrect.getBottom();
		float overtop = 0.0f;
		float overbottom = 0.0f;
		
		if (b > 1.0f)
		{
			overtop = b - 1.0f;
		}
		if (t < -1.0f)
		{
			overbottom = t + 1.0f;
		}
		maxscrolloffset = Math.abs(overtop) + Math.abs(overbottom);
		JoglScreenRect backrectclamped = new JoglScreenRect(posx, posy - (bgHeight - (float)maxscrolloffset), width, bgHeight - (float)maxscrolloffset);
		
		background = new JoglGuiColoredRect();
		//background.setRect(posx, posy - bgHeight, width, bgHeight);
		background.setRect(backrectclamped.getRect());
		background.setColor(backColor);
		background.setZLevel(1.0f);

		gui.AddGuiObject(background);
		controls.add(background);
		
		frame = new JoglGuiColoredFrame();
		//frame.setRect(posx, posy - bgHeight, width, bgHeight);
		frame.setRect(backrectclamped.getRect());
		frame.setColor(1.0f, 1.0f, 1.0f, 1.0f);
		frame.setLine(0.0025f / aspect, 0.0025f);
		frame.setZLevel(3.0f);
		
		gui.AddGuiObject(frame);
		controls.add(frame);

		sci = new JoglGuiScissorBox(gui);
		sci.setRect(backrectclamped.getRect());
		sci.setZLevel(2.0f);
		
		hoverRect = new JoglGuiColoredRect();
		hoverRect.setRect(posx, posy - lineHeight, width, lineHeight);
		hoverRect.setColor(hoverColor);
		hoverRect.setZLevel(1.5f);
		
		//gui.AddGuiObject(hoverRect);
		sci.addChild(hoverRect);
		
		if (overtop != 0.0f || overbottom != 0.0f)
		{
			float perc = 0.1f;
			float sbwidth = 0.05f / (float)gui.getAspect();
			
			scrollBar = new JoglGuiScrollBar();
			scrollBar.setCanTouch(true);
			scrollBar.setPerc(perc);
			scrollBar.setRect(posx + width - sbwidth, posy - backrectclamped.getHeight(), sbwidth, backrectclamped.getHeight());
			scrollBar.setZLevel(2.5f);
			//scrollB.setLine(0.005f / aspect, -0.005f);
			//scrollBar.setColor(JoglColor3.RED);
			scrollBar.setVisible(true);
			scrollBar.addScrollbarListener(this);
			
			gui.AddGuiObject(scrollBar);
			controls.add(scrollBar);
		}
		
		FontFormat ff = new FontFormat();
		ff.width = (textSize / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = textSize * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_LEFT;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		float offy = posy;
		for (String ls : values)
		{
			JoglGuiLabel jglv = new JoglGuiLabel(ff.clone());
			jglv.setText(ls);
			jglv.setRect(posx + 0.01f, offy - lineHeight, width, lineHeight);
			jglv.setBackColor(0, 0, 0, 0);
			jglv.setCanTouch(true);
			jglv.setZLevel(2.0f);
			
			//gui.AddGuiObject(jglv);
			sci.addChild(jglv);
			labels.add(jglv);
			
			offy = offy - lineHeight;
		}
		
		gui.AddGuiObject(sci);
		controls.add(sci);
		
		bInitialized = true;
	}

	public void Dispose()
	{
		for (JoglGuiLabel l : labels)
		{
			//gui.RemoveGuiObject(l);
		}
		labels.clear();
		
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
		controls.clear();
		
		bInitialized = false;
	}
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (!bInitialized)
		{
			Init();
		}
		
		bUpdateScene = false;
    }
	
	private JoglGuiLabel getLabel(JoglInputMouseEvent e)
	{
		JoglVectord2 pos = gui.GetNDCPos(e.getX(), e.getY());
		
		for (JoglGuiLabel l : labels)
		{
			if (l.IsInside((float)pos.getX(), (float)pos.getY()))
			{
				return l;
			}
		}
		return null;
	}
	
	private JoglGuiObject getGuiObjectOrLabel(JoglInputMouseEvent e)
	{
		JoglGuiObject mylabel = getLabel(e);
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		JoglVectord2 pos = gui.GetNDCPos(e.getX(), e.getY());
		if (mylabel != null && sci.IsInside((float)pos.getX(), (float)pos.getY()))
		{
			if (jgo == null)
			{
				jgo = mylabel;
			}
			else
			{
				if (jgo.getZLevel() <= mylabel.getZLevel())
				{
					jgo = mylabel;
				}
			}
		}
		return jgo;
	}
	
	//MouseListener method implementation
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = getGuiObjectOrLabel(e);
		if (jgo != null)
		{
			if (labels.contains(jgo))
			{
				return true;
			}
			if (scrollBar != null && scrollBar.IsChildOf(jgo))
			{
				scrollBar.StartMoveInteraction(iman, e);
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = getGuiObjectOrLabel(e);
		if (jgo != null)
		{
			if (labels.contains(jgo))
			{
				return true;
			}
			if (scrollBar != null && scrollBar.IsChildOf(jgo))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = getGuiObjectOrLabel(e);
		if (jgo != null)
		{
			if (labels.contains(jgo))
			{
				Vector<JoglGuiLabel> lst = new Vector<JoglGuiLabel>();
				lst.addAll(labels);
				for (JoglGuiLabel l : lst)
				{
					if (jgo == l)
					{
						//selected item
						for (IDropDownListener ddl : ddListeners)
						{
							ddl.OnDropDownSelected(this, l.getText());
						}
						return true;
					}
				}
			}
			else if (background == jgo)
			{
				//do not close dropdown
			}
			else if (scrollBar != null && (scrollBar == jgo || scrollBar.IsChildOf(jgo)))
			{
				//do not close dropdown
			}
			else
			{
				CloseDropDown();
			}
		}
		else
		{
			//clicked somewhere else
			CloseDropDown();
		}
		return false;
    }

	public void CloseDropDown() 
	{
		for (IDropDownListener ddl : ddListeners)
		{
			ddl.OnDropDownSelected(this, null);
		}
	}
	
	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = getGuiObjectOrLabel(e);
		if (jgo != null)
		{
			if (labels.contains(jgo))
			{
				JoglGuiLabel label = (JoglGuiLabel)jgo;
				JoglVectord4 rect = label.getRect();
				hoverRect.setRect(JoglVectord4.Add(rect, new JoglVectord4(-0.01f, 0.0f, 0.0f, 0.0f)));
				
				bUpdateScene = true;
			}
			else if (background == jgo)
			{
				//do not close dropdown
			}
			else if (scrollBar != null && (scrollBar == jgo || scrollBar.IsChildOf(jgo)))
			{
				
			}
			else
			{
				CloseDropDown();
			}
		}
		else
		{
			CloseDropDown();
		}
	}
	
	@Override
	public void mouseWheelMoved(JoglInputMouseWheelEvent e)
	{
		if (scrollBar == null || background == null || gui == null)
		{
			return;
		}
		
		JoglVectord2 cursorPos = gui.GetNDCPos(e.getX(), e.getY());
		//if cursor hovers over info panel
		JoglScreenRect srect = new JoglScreenRect(background.getRect());
		if (srect.IsInside((float)cursorPos.getX(), (float)cursorPos.getY()))
		{
			int delta = e.getWheelRotation();
			scrollBar.setScrollPerc(scrollBar.getScrollPerc() + (0.1f * delta));
			MoveScrollbar();
		}
	}
	
	private double lastscrollperc = 0.0;

	public void MoveScrollbar() 
	{
		double newscrollperc = scrollBar.getScrollPerc();
		
		double delta = lastscrollperc - newscrollperc;
		JoglVectord4 diff = new JoglVectord4(0, -delta * maxscrolloffset, 0, 0);
		
		for (JoglGuiLabel l : labels)
		{
			l.setRect(JoglVectord4.Add(l.getRect(), diff));
		}
		hoverRect.setRect(JoglVectord4.Add(hoverRect.getRect(), diff));
		
		lastscrollperc = newscrollperc;
	}

	public void OnScrollBarMoved(JoglGuiScrollBar sb, JoglVectord2 dir) 
	{
		MoveScrollbar();
	}
}
