package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Vector;
import java.util.prefs.Preferences;

import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.compviz.TransitionPair;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiTextbox;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdge;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnPlace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglModelScene;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglPSENVisualizationScene;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.fileio.obj.MTLWriter;
import org.processmining.processprofiler3d.psen.fileio.obj.OBJObject;
import org.processmining.processprofiler3d.psen.fileio.obj.OBJObjectWriter;
import org.processmining.processprofiler3d.psen.fileio.obj.OBJWriter;
import org.processmining.processprofiler3d.psen.fileio.obj.SVGSceneRenderer;
import org.processmining.processprofiler3d.psen.fileio.obj.SceneExporter;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputKeyEvent;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JIExportPanel extends JIOptionsPanel implements ITextboxInputListener, IDropDownListener
{
	private JoglInteractionManager iman;
	private JoglCamera cam;
	private JoglScene scene = null;
	private IPerformanceStatisticsEnrichedNet psen;
	private IPerformanceStatisticsEnrichedNet wten;

	private JoglGuiTextbox tbxExportFilePath;
	private JoglGuiLabel btnScreenshotType;
	private JoglGuiLabel btnExportModelObj;
	private JoglGuiLabel btnExportModelCsv;
	private JoglGuiLabel btnExportModelSvg;
	private JoglGuiLabel btnExportTransitionDataCsv;
	private JoglGuiLabel btnExportTransitionPairDataCsv;
	
	private JIDropdownBox ddbox = null;

	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();

	private JoglGuiLabel highlighted = null;
	
	private String screenshotType = "Screenshot";
	
	private boolean bVisible = false;
	private boolean bUpdateScene = false;
	
	private float lastAspect = 1.0f;
	private int lastWidth = 1;
	private int lastHeight = 1;
	
	public JIExportPanel(JoglInteractionManager iman, JoglCamera cam, JoglScene s, JSOGraphicalUserInterface gui, IPerformanceStatisticsEnrichedNet psen, IPerformanceStatisticsEnrichedNet wten)
	{
		super(gui);
		
		this.iman = iman;
		this.cam = cam;
		this.scene = s;
		this.psen = psen;
		this.wten = wten;
	}
	
	@Override
	public boolean isVisible()
	{
		return bVisible;
	}
	
	@Override
	public void Show()
	{
		screenshotType = JIModelExport.screenshotType;
		
		float aspect = (float)gui.getAspect();
		lastAspect = aspect;
		lastWidth = (int)gui.getWidth();
		lastHeight = (int)gui.getHeight();
		
		FontFormat ff = new FontFormat();
		ff.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_LEFT;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff2 = new FontFormat();
		ff2.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff2.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff2.align = FontFormat.ALIGN_RIGHT;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;

		float colButtonWidth = 0.2f * (float)JoglGuiStyle.textScale;
		float colButtonX = 0.95f - colButtonWidth;

		float offy = 0.85f;
		
		String baseFilePath = "C:/Lab/Export/test";
		Preferences prefs = Preferences.userNodeForPackage(this.getClass());
		baseFilePath = prefs.get("BaseFilePath", baseFilePath);
		
		JIModelExport.exportFilePath = baseFilePath;
		
		tbxExportFilePath = new JoglGuiTextbox(ff.clone());
		tbxExportFilePath.setCanTouch(true);
		tbxExportFilePath.setRect(colButtonX, offy, colButtonWidth, 0.05f);
		tbxExportFilePath.setText(baseFilePath);
		tbxExportFilePath.setPadding(textPadding);
		tbxExportFilePath.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(tbxExportFilePath);
		controls.add(tbxExportFilePath);
		
		offy = offy - 0.1f;
		
		btnScreenshotType = new JoglGuiLabel(ff.clone());
		btnScreenshotType.setCanTouch(true);
		btnScreenshotType.setRect(colButtonX, offy, colButtonWidth, 0.05f);
		btnScreenshotType.setText(screenshotType);
		btnScreenshotType.setPadding(textPadding);
		btnScreenshotType.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnScreenshotType);
		controls.add(btnScreenshotType);
		
		offy = offy - 0.1f;
		
		btnExportModelObj = new JoglGuiLabel(ff.clone());
		btnExportModelObj.setCanTouch(true);
		btnExportModelObj.setRect(colButtonX, offy, colButtonWidth, 0.05f);
		btnExportModelObj.setText("Export Model OBJ");
		btnExportModelObj.setPadding(textPadding);
		btnExportModelObj.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnExportModelObj);
		controls.add(btnExportModelObj);
		
		offy = offy - 0.1f;
		
		btnExportModelCsv = new JoglGuiLabel(ff.clone());
		btnExportModelCsv.setCanTouch(true);
		btnExportModelCsv.setRect(colButtonX, offy, colButtonWidth, 0.05f);
		btnExportModelCsv.setText("Export Model CSV");
		btnExportModelCsv.setPadding(textPadding);
		btnExportModelCsv.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnExportModelCsv);
		controls.add(btnExportModelCsv);
		
		offy = offy - 0.1f;
		
		btnExportModelSvg = new JoglGuiLabel(ff.clone());
		btnExportModelSvg.setCanTouch(true);
		btnExportModelSvg.setRect(colButtonX, offy, colButtonWidth, 0.05f);
		btnExportModelSvg.setText("Export Model SVG");
		btnExportModelSvg.setPadding(textPadding);
		btnExportModelSvg.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnExportModelSvg);
		controls.add(btnExportModelSvg);
		
		offy = offy - 0.1f;
		
		if (psen != null)
		{
			btnExportTransitionDataCsv = new JoglGuiLabel(ff.clone());
			btnExportTransitionDataCsv.setCanTouch(true);
			btnExportTransitionDataCsv.setRect(colButtonX, offy, colButtonWidth, 0.05f);
			btnExportTransitionDataCsv.setText("Export Transition Data CSV");
			btnExportTransitionDataCsv.setPadding(textPadding);
			btnExportTransitionDataCsv.setBackColor(JoglGuiStyle.backColor);
			
			gui.AddGuiObject(btnExportTransitionDataCsv);
			controls.add(btnExportTransitionDataCsv);
			
			offy = offy - 0.1f;
		}
		
		if (wten != null)
		{
			btnExportTransitionPairDataCsv = new JoglGuiLabel(ff.clone());
			btnExportTransitionPairDataCsv.setCanTouch(true);
			btnExportTransitionPairDataCsv.setRect(colButtonX, offy, colButtonWidth, 0.05f);
			btnExportTransitionPairDataCsv.setText("Export Transition Pair Data CSV");
			btnExportTransitionPairDataCsv.setPadding(textPadding);
			btnExportTransitionPairDataCsv.setBackColor(JoglGuiStyle.backColor);
			
			gui.AddGuiObject(btnExportTransitionPairDataCsv);
			controls.add(btnExportTransitionPairDataCsv);
		}
		
		bVisible = true;
	}
	
	@Override
	public void Hide()
	{
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
		
		bVisible = false;
	}
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		bUpdateScene = false;
	}
	
	//MouseListener method implementation
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {		
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnExportModelCsv)
			{
				try 
				{
					ExportModelCSV();
				} 
				catch (IOException e1) 
				{
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == btnScreenshotType)
			{
				Vector<String> lst = new Vector<String>();
				lst.add("Screenshot");
				lst.add("Screenshot - HQ");
				lst.add("Cubemap");
				
				OpenDropDownBox(btnScreenshotType, lst);
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == btnExportModelObj)
			{
				ExportModelOBJ();
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == btnExportModelSvg)
			{
				ExportModelSVG();
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == btnExportTransitionDataCsv)
			{
				ExportDataCSV();
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == btnExportTransitionPairDataCsv)
			{
				ExportPairDataCSV();
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == tbxExportFilePath)
			{
				//viewManager.SwitchGrid();
				JITextboxInput jibt = new JITextboxInput(iman, scene, gui, tbxExportFilePath);
				jibt.listener = this;
				iman.addInteraction(jibt);
				
				bUpdateScene = true;
	        	return true;
			}
			
		}
		
		return false;
    }
	
	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnExportModelCsv)
			{
				highlight(btnExportModelCsv);
			}
			else if (jgo == btnScreenshotType)
			{
				highlight(btnScreenshotType);
			}
			else if (jgo == btnExportModelObj)
			{
				highlight(btnExportModelObj);
			}
			else if (jgo == btnExportModelSvg)
			{
				highlight(btnExportModelSvg);
			}
			else if (jgo == tbxExportFilePath)
			{
				highlight(tbxExportFilePath);
			}
			else if (jgo == btnExportTransitionDataCsv)
			{
				highlight(btnExportTransitionDataCsv);
			}
			else if (jgo == btnExportTransitionPairDataCsv)
			{
				highlight(btnExportTransitionPairDataCsv);
			}
			else
			{
				unHighlight();
			}
		}
		else
		{
			unHighlight();
		}
	}
	
	private void highlight(JoglGuiLabel label)
	{
		unHighlight();
	
		label.setBackColor(JoglGuiStyle.highlightColor);
		highlighted = label;
		
		bUpdateScene = true;
	}

	private void unHighlight() 
	{
		if (highlighted != null)
		{
			highlighted.setBackColor(JoglGuiStyle.backColor);
			highlighted = null;

			bUpdateScene = true;
		}
	}

	public void OnTextInputFinished(JITextboxInput jti) 
	{
		String txt = jti.getText();

		Preferences prefs = Preferences.userNodeForPackage(this.getClass());
		prefs.put("BaseFilePath", txt);
		
		JIModelExport.exportFilePath = txt;
		//double val = Double.parseDouble(txt);
		
		//RescaleGuiText(val);
	}
	
	//KeyListener method implementations
	@Override
	public boolean keyPressed(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt);
        
//		if (evt.getKeyCode() == PP3DKeyBindings.TakeScreenshot) //u-key
//		{
//			TakeScreenshot();
//	        
//        	bUpdateScene = true;
//		}
//		if (evt.getKeyCode() == 78) //n-key
//		{
//			PrevStep();
//        	bUpdateScene = true;
//		}
		
		return false;
	}
	
	private void OpenDropDownBox(JoglGuiLabel label, List<String> vals)
	{
		if (ddbox != null)
		{
			CloseDropDownBox();
		}

		if (!vals.isEmpty())
		{
			JoglVectord4 pos = label.getRect();
			ddbox = new JIDropdownBox(iman, gui, label, vals);
			ddbox.addDropDownListener(this);
			ddbox.setPosition((float)pos.getX(), (float)pos.getY() + 0.05f);
			ddbox.setWidth(0.2f * (float)JoglGuiStyle.textScale);
			ddbox.setTextSize(0.03f);
			iman.addInteraction(ddbox);
		}
		
    	bUpdateScene = true;	
	}

	private void CloseDropDownBox()
	{
		//close dropdown
		iman.removeInteraction(ddbox);
		ddbox.Dispose();
		ddbox = null;
	}
	
	public void OnDropDownSelected(JIDropdownBox ddb, String val) 
	{
		//System.out.println("OnDropDownSelected!");
		if (ddbox == ddb)
		{
			if (val != null)
			{
				if (ddb.getOwner() == btnScreenshotType)
				{
					screenshotType = val;
					JIModelExport.screenshotType = screenshotType;
					btnScreenshotType.setText(val);
				}
			}
			
			CloseDropDownBox();
			
			Hide();
			Show();
			
			bUpdateScene = true;
		}
	}
	
//	private void TakeScreenshot()
//	{
//		if (screenshotType.equals("Screenshot"))
//		{
//			iman.Screenshot(tbxExportFilePath.getText() + ".png");
//		}
//		else if (screenshotType.equals("Screenshot - HQ"))
//		{
//			iman.Screenshot(new JoglScreenshotCaptureHQ(tbxExportFilePath.getText() + ".png", lastWidth * 1, lastHeight * 1, 16));
//		}
//		else if (screenshotType.equals("Cubemap"))
//		{
//			iman.Screenshot(new JoglCubeMapCapture(tbxExportFilePath.getText()));
//		}
		//NEED TO MOVE TO JIModelExport!!!
//	}
	
	private void ExportDataCSV()
	{
		String filePath = tbxExportFilePath.getText() + ".csv";
		CSVDataExporter de = new CSVDataExporter(psen);
		if (scene instanceof JoglPSENVisualizationScene)
		{
			JoglPSENVisualizationScene vs = (JoglPSENVisualizationScene)scene;

			//for each transition in net
			Vector<JSOPnTransition> tlst = vs.getPetriNet().getTransitions();
			//tlst.sort(LabelComparatorAscending); //Java 1.8
			Collections.sort(tlst, LabelComparatorAscending);
			for (JSOPnTransition t : tlst)
			{
				de.addStatsFor(t, psen);
			}
		}
		
		try 
		{
			de.WriteToCSV(filePath);
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private void ExportPairDataCSV()
	{
		String filePath = tbxExportFilePath.getText() + ".csv";
		CSVDataExporter de = new CSVDataExporter(wten);
		if (scene instanceof JoglPSENVisualizationScene)
		{
			JoglPSENVisualizationScene vs = (JoglPSENVisualizationScene)scene;
			JSOPetriNet pnet = vs.getPetriNet();
			INetDataSource inds = pnet.getPetriNet();
			PetriNetDataSource pnds = null;
			if (inds instanceof PetriNetDataSource)
			{
				pnds = (PetriNetDataSource)inds;
			}

			//for each transition in net
			Vector<String> tlst = new Vector<String>(wten.getTransitions());
			//tlst.sort(LabelComparatorAscending); //Java 1.8
			//Collections.sort(tlst, LabelComparatorAscending);
			for (String t : tlst)
			{
				String sourceId = TransitionPair.getSourceId(t);
				String targetId = TransitionPair.getTargetId(t);

				if (pnds != null)
				{
					String sLabel = "";
					String tLabel = "";
					Transition tsource = getFromNet(pnds, sourceId);
					if (tsource != null)
					{
						sLabel = tsource.getLabel();
					}
					Transition ttarget = getFromNet(pnds, targetId);
					if (ttarget != null)
					{
						tLabel = ttarget.getLabel();
					}
					de.addStatsFor(sLabel + " - " + tLabel, t, wten);
				}
				else
				{
					de.addStatsFor("Arc", t, wten);			
				}
			}
		}
		
		try 
		{
			de.WriteToCSV(filePath);
		}
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private Transition getFromNet(PetriNetDataSource pnds , String tId)
	{
		for (Transition t : pnds.getAllTransitions())
		{
			if (t.getLocalID().toString().equals(tId))
			{
				return t;
			}
		}
		return null;
	}

	public static Comparator<JSOPnTransition> LabelComparatorAscending 
		= new Comparator<JSOPnTransition>() {

		public int compare(JSOPnTransition te1, JSOPnTransition te2) {
		
		//ascending order
		return te1.getLabel().compareTo(te2.getLabel());
		}
	
	};
	
	private void ExportModelSVG()
	{
		String svgFilePath = tbxExportFilePath.getText() + ".svg";
		
		//pn.Export(out);
		OBJObjectWriter oow = new OBJObjectWriter();
		
		//scene.Export(out);
		SceneExporter.Export(oow, scene);
		oow.Close();
		
		SVGSceneRenderer.RenderSceneToSVGFile(svgFilePath, cam, gui, oow);	
	}
	
	private void ExportModelOBJ()
	{
		String objFilePath = tbxExportFilePath.getText() + ".obj";
		String mtlFilePath = tbxExportFilePath.getText() + ".mtl";
		OBJWriter out = new OBJWriter();
		MTLWriter outmtl = new MTLWriter();

		System.out.println("Exporting model to : " + objFilePath);
		System.out.println("Exporting model materials to : " + mtlFilePath);
		
		out.Open(objFilePath);
		outmtl.Open(mtlFilePath);
		
		out.WriteMtlLib("test.mtl");
		
		//export base materials
		exportBaseMaterials(outmtl);
		
		//pn.Export(out);
		OBJObjectWriter oow = new OBJObjectWriter();
		
		//scene.Export(out);
		SceneExporter.Export(oow, scene);
		oow.Close();
		
		for (OBJObject oo : oow.objList)
		{
			out.WriteNewObject(oo);
		}
		
	    out.Close();
	    outmtl.Close();		
	}
	
	private void exportBaseMaterials(MTLWriter outmtl)
	{
		outmtl.WriteNewMaterial("ModelElementBlack");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(0, 0, 0);
		outmtl.WriteDiffuseColor(0, 0, 0);
		outmtl.WriteSpecularColor(0, 0, 0);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("ModelElementWhite");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(1, 1, 1);
		outmtl.WriteDiffuseColor(1, 1, 1);
		outmtl.WriteSpecularColor(1, 1, 1);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("ModelPedestal");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(1, 1, 1);
		outmtl.WriteDiffuseColor(1, 1, 1);
		outmtl.WriteSpecularColor(1, 1, 1);
		outmtl.WriteEmissiveColor(0, 0, 0);
		//outmtl.WriteColorVal("Tf", 1, 1, 1);
		outmtl.WriteTransparencyDissolve(0.3);
		outmtl.WriteIlluminationModel(9);
		
		outmtl.WriteNewMaterial("CohortWhite");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(1, 1, 1);
		outmtl.WriteDiffuseColor(1, 1, 1);
		outmtl.WriteSpecularColor(1, 1, 1);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortBlack");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(0, 0, 0);
		outmtl.WriteDiffuseColor(0, 0, 0);
		outmtl.WriteSpecularColor(0, 0, 0);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortRed");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(1, 0, 0);
		outmtl.WriteDiffuseColor(1, 0, 0);
		outmtl.WriteSpecularColor(1, 0, 0);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortGreen");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(0, 1, 0);
		outmtl.WriteDiffuseColor(0, 1, 0);
		outmtl.WriteSpecularColor(0, 1, 0);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortBlue");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(0, 0, 1);
		outmtl.WriteDiffuseColor(0, 0, 1);
		outmtl.WriteSpecularColor(0, 0, 1);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortYellow");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(1, 1, 0);
		outmtl.WriteDiffuseColor(1, 1, 0);
		outmtl.WriteSpecularColor(1, 1, 0);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortCyan");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(0, 1, 1);
		outmtl.WriteDiffuseColor(0, 1, 1);
		outmtl.WriteSpecularColor(0, 1, 1);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortMagenta");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(1, 0, 1);
		outmtl.WriteDiffuseColor(1, 0, 1);
		outmtl.WriteSpecularColor(1, 0, 1);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
	}
	
	public void ExportModelCSV() throws IOException
	{
		FileOutputStream fos;
		Writer out;
		
		String filePath = tbxExportFilePath.getText() + ".csv";
		System.out.println("Exporting model to : " + filePath);
        fos = new FileOutputStream(filePath);
        out = new OutputStreamWriter(fos, "UTF8");
        
        out.write(",NodeType,Label,xPos,yPos\r\n");
        
        int i = 1;
        
        JoglModelScene jms = (JoglModelScene)scene;
        Vector<JoglSceneObject> lst = jms.getPetriNet().getRootSubNet().getElements();
        for (JoglSceneObject jso : lst)
        {
        	if (jso instanceof JSOPnTransition)
        	{
        		JSOPnTransition t = (JSOPnTransition)jso;
        		if (t.orgTransition.isInvisible())
        		{
        			out.write("" + i + ",2," + t.getLabel() + "," + t.getPosition().getX() + "," + t.getPosition().getZ() + "\r\n");
        		}
        		else
        		{
        			out.write("" + i + ",1," + t.getLabel() + "," + t.getPosition().getX() + "," + t.getPosition().getZ() + "\r\n");
        		}
        		i++;
        	}
        	else if (jso instanceof JSOPnPlace)
        	{
        		JSOPnPlace t = (JSOPnPlace)jso;
                out.write("" + i + ",3," + t.orgPlace.getLabel() + "," + t.getPosition().getX() + "," + t.getPosition().getZ() + "\r\n");
        		i++;
        	}
        	else if (jso instanceof JSOPnEdge)
        	{
        		JSOPnEdge t = (JSOPnEdge)jso;
                out.write("" + i + ",4,Edge," + 0 + "," + 0 + "\r\n");
                i++;
                
        		for (JoglVectord3 v : t.getPoints())
        		{
                    out.write("" + i + ",5,EdgePoint," + v.getX() + "," + v.getZ() + "\r\n");
                    i++;
        		}
        	}
        }
        
        out.close();
		
	}
	
	
}
