package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.interactions.camera.JICameraVR;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.interactions.JICameraBase;
import org.processmining.visualisation3d.interactions.JoglGuiInteraction;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIFloatingLabels extends JoglGuiInteraction
{
	private JoglInteractionManager iman;
	//private JoglCamera cam;
	private JSOPetriNet jpnet;
	
	private float textSize = 0.06f;
	private float maxLabelWidthDefault = 0.2f;
	private float maxLabelWidth = 0.2f;
	private float labelMargin = 0.02f;
	
	private Vector<JoglGuiLabel> labels = new Vector<JoglGuiLabel>();
	private boolean bShowInvisibleLabels = false;
	
	public JIFloatingLabels(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui, JSOPetriNet jpnet)
	{
		super(gui);
		
		this.iman = iman;
		this.jpnet = jpnet;
		
		zlevel = 100.0f;
	}
	
	public void Dispose()
	{
		for (JoglGuiLabel jgl : labels)
		{
			gui.RemoveGuiObject(jgl);
		}
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		JICameraBase jcb = iman.findInteraction(JICameraBase.class);
		JoglCamera cam = jcb.getCamera();
		JoglCamera vrCam = null;
		if (jcb instanceof JICameraVR)
		{
			JICameraVR jcvr = (JICameraVR)jcb;
			vrCam = jcvr.getHMDCamera();
			if (vrCam != null)
			{
				cam = vrCam;
			}
		}
		
		//float aspect = (float)jgui.getAspect();
		float aspect = (float)cam.getCameraLens().getAspect();
		float labelMarginW = (float)((labelMargin / aspect) * JoglGuiStyle.textScale);
		float labelMarginH = (float)(labelMargin * JoglGuiStyle.textScale);
		float ltextSizeW = (textSize / aspect) * (float)JoglGuiStyle.textScale;
		float ltextSizeH = textSize * (float)JoglGuiStyle.textScale;
		maxLabelWidth = (float)(maxLabelWidthDefault * JoglGuiStyle.textScale);
		
		if (labels.size() == 0)
		{
			FontFormat ff = new FontFormat();
			ff.width = (textSize / aspect) * (float)JoglGuiStyle.textScale;
			ff.height = textSize * (float)JoglGuiStyle.textScale;
			ff.setColor(1.0f, 1.0f, 1.0f);
			
			//add new labels to GUI
			for (JSOSubNet model : jpnet.getAllSubNets())
			{
				for (JSOPnTransition t : model.getTransitions())
				{
					JoglVectord3 v = model.relativeToAbsolute(t.getPosition());
					
					JoglVectord3 screenPos = cam.ProjectToScreen(v, aspect);
					
					JoglGuiLabel label = new JoglGuiLabel(ff.clone());
					
					JoglVectord2 lsize = label.MeasureText(t.getLabel(), maxLabelWidth);
					float nWidth = (float)lsize.getX() + labelMarginW;
					float nHeight = (float)lsize.getY() + labelMarginH;

					double fScale = 1.0 / (screenPos.getZ() * 0.0025);
					fScale = Math.min(1.5f, fScale);
					
					//label.setColor(1.0f, 1.0f, 1.0f);
					label.setBackColor(0.0f, 0.0f, 0.0f, 0.6f);
					label.setText(t.getLabel(), maxLabelWidth);
					label.setRect((float)(-screenPos.getX()) - (nWidth * 0.5f), (float)(screenPos.getY()) - (nHeight * 0.5f), nWidth, nHeight);
					label.setZLevel((float)-screenPos.getZ());
					label.setForceScale(fScale);
	
					if (screenPos.getZ() < 0.0 || (t.orgTransition.isInvisible() && !bShowInvisibleLabels) || !model.getIsVisible())
					{
						label.setVisible(false);
					}
					
					
					gui.AddGuiObject(label);
					labels.add(label);
				}
			}
		}
		else
		{
			//update label position
			int id = 0;
			for (JSOSubNet model : jpnet.getAllSubNets())
			{
				for (JSOPnTransition t : model.getTransitions())
				{
					JoglVectord3 v = model.relativeToAbsolute(t.getPosition());
		
					JoglVectord3 screenPos = cam.ProjectToScreen(v, aspect);
					JoglGuiLabel label = labels.get(id);
					
					if (screenPos.getZ() > 0.0 && (!t.orgTransition.isInvisible() || bShowInvisibleLabels) && model.getIsVisible())
					{
						JoglVectord2 lsize = label.MeasureText(t.getLabel(), maxLabelWidth);
						float nWidth = (float)lsize.getX() + labelMarginW;
						float nHeight = (float)lsize.getY() + labelMarginH;
						
						double fScale = 1.0 / (screenPos.getZ() * 0.0025);
						fScale = Math.min(1.5f, fScale);
						
						label.setVisible(true);
						label.setText(t.getLabel(), maxLabelWidth);
						label.setTextSize(ltextSizeW, ltextSizeH);
						label.setRect((float)(-screenPos.getX()) - (nWidth * 0.5f), (float)(screenPos.getY()) - (nHeight * 0.5f), nWidth, nHeight);
						label.setZLevel((float)-screenPos.getZ());
						label.setForceScale(fScale);
		
						//remove and add object to sorted collection to update zlevel sorting
						gui.RemoveGuiObject(label);
						gui.AddGuiObject(label);
					}
					else
					{
						label.setVisible(false);
					}	
					id++;
				}
			}
		}	
	}

}
