package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.List;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.JoglTextureAnisoFilterChange;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;

public class JIGraphicsConfig extends JIOptionsPanel implements ITextboxInputListener, IDropDownListener
{
	private JoglPanel panel;
	private JoglScene scene;

	private boolean bVisible = false;
	
	private JoglGuiLabel antiAliasButton;
	private JoglGuiLabel hqGeometryToggleButton;
	private JoglGuiLabel hqShadersToggleButton;
	private JoglGuiLabel tbxAnisoFilter;

	private JoglGuiLabel lblAntiAlias;
	private JoglGuiLabel lblHQGeometry;
	private JoglGuiLabel lblHQShaders;
	private JoglGuiLabel lblAnisoFilter;

	private JoglGuiLabel highlighted = null;
	
	private JIDropdownBox ddbox = null;
	
	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();
	
	public JIGraphicsConfig(JoglPanel p, JoglScene scene, JSOGraphicalUserInterface gui)
	{
		super(gui);
		
		this.panel = p;
		this.scene = scene;
	}
	
	@Override
	public boolean isVisible()
	{
		return bVisible;
	}
	
	@Override
	public void Show()
	{
		JoglGraphicsSettings gsettings = panel.getGraphicsSettings();
		
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_LEFT;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff2 = new FontFormat();
		ff2.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff2.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff2.align = FontFormat.ALIGN_RIGHT;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		float yoff = 0.85f;

		float colLabelWidth = 0.2f * (float)JoglGuiStyle.textScale;
		float colButtonWidth = 0.2f * (float)JoglGuiStyle.textScale;
		
		float colButtonX = 0.95f - colButtonWidth;
		float colLabelX = colButtonX - colLabelWidth - 0.005f;

		antiAliasButton = new JoglGuiLabel(ff.clone());
		antiAliasButton.setCanTouch(true);
		antiAliasButton.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		antiAliasButton.setText("" + gsettings.GetAntiAliasingSamples() + "x");
		antiAliasButton.setPadding(textPadding);
		antiAliasButton.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(antiAliasButton);
		controls.add(antiAliasButton);

		lblAntiAlias = new JoglGuiLabel(ff2.clone());
		lblAntiAlias.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblAntiAlias.setText("Anti-aliasing");
		lblAntiAlias.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblAntiAlias);
		controls.add(lblAntiAlias);
		
		yoff -= 0.1f;

		tbxAnisoFilter = new JoglGuiLabel(ff.clone());
		tbxAnisoFilter.setCanTouch(true);
		tbxAnisoFilter.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		tbxAnisoFilter.setText("" + gsettings.GetAnisotropicSamples() + "x");
		tbxAnisoFilter.setPadding(textPadding);
		tbxAnisoFilter.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(tbxAnisoFilter);
		controls.add(tbxAnisoFilter);

		lblAnisoFilter = new JoglGuiLabel(ff2.clone());
		lblAnisoFilter.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblAnisoFilter.setText("Anisotropic Filter");
		lblAnisoFilter.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblAnisoFilter);
		controls.add(lblAnisoFilter);
		
		yoff -= 0.1f;
		
		hqGeometryToggleButton = new JoglGuiLabel(ff.clone());
		hqGeometryToggleButton.setCanTouch(true);
		hqGeometryToggleButton.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		if (gsettings.bHQGeometry)
		{
			hqGeometryToggleButton.setText("On");
		}
		else
		{
			hqGeometryToggleButton.setText("Off");
		}
		hqGeometryToggleButton.setPadding(textPadding);
		hqGeometryToggleButton.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(hqGeometryToggleButton);
		controls.add(hqGeometryToggleButton);
		
		lblHQGeometry = new JoglGuiLabel(ff2.clone());
		lblHQGeometry.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblHQGeometry.setText("HQ Geometry");
		lblHQGeometry.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblHQGeometry);
		controls.add(lblHQGeometry);

		yoff -= 0.1f;

		hqShadersToggleButton = new JoglGuiLabel(ff.clone());
		hqShadersToggleButton.setCanTouch(true);
		hqShadersToggleButton.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		if (gsettings.bHQShaders)
		{
			hqShadersToggleButton.setText("On");
		}
		else
		{
			hqShadersToggleButton.setText("Off");
		}
		hqShadersToggleButton.setPadding(textPadding);
		hqShadersToggleButton.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(hqShadersToggleButton);
		controls.add(hqShadersToggleButton);
		
		lblHQShaders = new JoglGuiLabel(ff2.clone());
		lblHQShaders.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblHQShaders.setText("HQ Shaders");
		lblHQShaders.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblHQShaders);
		controls.add(lblHQShaders);
		
		yoff -= 0.1f;
		
		bVisible = true;
	}
	
	@Override
	public void Hide()
	{
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
		
		bVisible = false;
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (!bVisible)
		{
			//Init();
		}
		else
		{
			JoglGraphicsSettings gsettings = panel.getGraphicsSettings();
			antiAliasButton.setText("" + gsettings.GetAntiAliasingSamples() + "x");
			tbxAnisoFilter.setText("" + gsettings.GetAnisotropicSamples() + "x");
		}
		
		bUpdateScene = false;
	}
	
	//MouseListener method implementation
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {		
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			JoglGraphicsSettings gsettings = panel.getGraphicsSettings();
			if (jgo == hqGeometryToggleButton)
			{
				gsettings.bHQGeometry = !gsettings.bHQGeometry;
				gsettings.SavePreferences();
				if (gsettings.bHQGeometry)
				{
					hqGeometryToggleButton.setText("On");
				}
				else
				{
					hqGeometryToggleButton.setText("Off");
				}

				//reset net geometry
				JSOPetriNet net = scene.findSceneObject(JSOPetriNet.class);
				if (net != null)
				{
					for (JSOSubNet jsn : net.getAllSubNets())
					{
						jsn.ReloadMeshes();
					}
				}
				
				//reset arc geometry
				JIPSENEdgeVisDriver edriver = panel.getInteractionManager().findInteraction(JIPSENEdgeVisDriver.class);
				if (edriver != null)
				{
					edriver.setEdgeVisMethod(edriver.GetEdgeVisMethodName());
				}
				
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == antiAliasButton)
			{
				Vector<String> lst = new Vector<String>();
				lst.add("1x");
				lst.add("2x");
				lst.add("4x");
				lst.add("8x");
				lst.add("16x");
				lst.add("32x");
				OpenDropDownBox(antiAliasButton, lst);
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == tbxAnisoFilter)
			{
				Vector<String> lst = new Vector<String>();
				lst.add("1x");
				lst.add("2x");
				lst.add("4x");
				lst.add("8x");
				lst.add("16x");
				OpenDropDownBox(tbxAnisoFilter, lst);
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == hqShadersToggleButton)
			{
				gsettings.bHQShaders = !gsettings.bHQShaders;
				gsettings.SavePreferences();
				if (gsettings.bHQShaders)
				{
					hqShadersToggleButton.setText("On");
				}
				else
				{
					hqShadersToggleButton.setText("Off");
				}
				
				bUpdateScene = true;
	        	return true;
			}
		}
		
		return false;
    }
	
	private void OpenDropDownBox(JoglGuiLabel label, List<String> vals)
	{
		if (ddbox != null)
		{
			CloseDropDownBox();
		}

		if (!vals.isEmpty())
		{
			JoglVectord4 pos = label.getRect();
			ddbox = new JIDropdownBox(panel.getInteractionManager(), gui, label, vals);
			ddbox.addDropDownListener(this);
			ddbox.setPosition((float)pos.getX(), (float)pos.getY() + 0.05f);
			ddbox.setWidth(0.2f * (float)JoglGuiStyle.textScale);
			ddbox.setTextSize(0.03f);
			panel.getInteractionManager().addInteraction(ddbox);
		}
		
    	bUpdateScene = true;	
	}

	private void CloseDropDownBox()
	{
		//close dropdown
		panel.getInteractionManager().removeInteraction(ddbox);
		ddbox.Dispose();
		ddbox = null;
	}
	
	public void OnDropDownSelected(JIDropdownBox ddb, String val) 
	{
		//System.out.println("OnDropDownSelected!");
		if (ddbox == ddb)
		{
			if (val != null)
			{
				JoglGraphicsSettings gsettings = panel.getGraphicsSettings();
				if (ddb.getOwner() == antiAliasButton)
				{
					String sampleNumStr = val.replace("x", "");
					int sampleNum = Integer.parseInt(sampleNumStr);
					gsettings.setMultiSamples(sampleNum);
					gsettings.SavePreferences();
					//antiAliasButton.setText(val);
				}
				if (ddb.getOwner() == tbxAnisoFilter)
				{
					String sampleNumStr = val.replace("x", "");
					int sampleNum = Integer.parseInt(sampleNumStr);
					gsettings.setAnisoSamples(sampleNum);
					gsettings.SavePreferences();
//					tbxAnisoFilter.setText(val);

					int anisoSamples = gsettings.GetAnisotropicSamples();
					for (String s : JoglResourceManager.GetResourceNames())
					{
						Object o = JoglResourceManager.GetResource(s);
						if (o instanceof JoglTexture)
						{
							JoglTexture tex = (JoglTexture)o;
							JoglTextureAnisoFilterChange fc = new JoglTextureAnisoFilterChange(tex, anisoSamples);
							panel.prerenderTasks.add(fc);
							//Texture tx = tex.getTex();
							//tx.setTexParameteri(gl, GL.GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, anisoSamples);
						}
					}
				}
			}
			
			CloseDropDownBox();
			
			Hide();
			Show();
			
			bUpdateScene = true;
		}
	}
	
	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == hqGeometryToggleButton)
			{
				highlight(hqGeometryToggleButton);
			}
			else if (jgo == antiAliasButton)
			{
				highlight(antiAliasButton);
			}
			else if (jgo == hqShadersToggleButton)
			{
				highlight(hqShadersToggleButton);
			}
			else if (jgo == tbxAnisoFilter)
			{
				highlight(tbxAnisoFilter);
			}
			else
			{
				unHighlight();
			}
		}
		else
		{
			unHighlight();
		}
	}
	
	private void highlight(JoglGuiLabel label)
	{
		unHighlight();
	
		label.setBackColor(JoglGuiStyle.highlightColor);
		highlighted = label;
		
		bUpdateScene = true;
	}

	private void unHighlight() 
	{
		if (highlighted != null)
		{
			highlighted.setBackColor(JoglGuiStyle.backColor);
			highlighted = null;

			bUpdateScene = true;
		}
	}

	public void OnTextInputFinished(JITextboxInput jti) 
	{
		String txt = jti.getText();
		double val = Double.parseDouble(txt);
		
		RescaleGuiText(val);
	}
	
	public void RescaleGuiText(double val)
	{	
		JoglGuiStyle.textScale = val;
		
		//update all interactions with gui text
		JIPSENFilter jipf = panel.getInteractionManager().findInteraction(JIPSENFilter.class);
		if (jipf != null)
		{
			jipf.UpdateFilters();
		}

		JIPSENBaseInfo jibi = panel.getInteractionManager().findInteraction(JIPSENBaseInfo.class);
		if (jibi != null)
		{
			jibi.UpdateBaseInfo(scene);
		}
		
		JIOptionsPanelManager jiopm = panel.getInteractionManager().findInteraction(JIOptionsPanelManager.class);
		if (jiopm != null)
		{
			jiopm.UpdateOptionsPanel();
		}
		
		JISelection jis = panel.getInteractionManager().findInteraction(JISelection.class);
		if (jis != null)
		{
			jis.UpdateSelectionText();
		}
	}
}
