package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiCheckBox;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredFrame;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredRect;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiPerformanceInfoList;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.interactions.JoglGuiInteraction;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIInfoOptions extends JoglGuiInteraction
{
	private JoglInteractionManager iman;
	
	private JIPSENBaseInfo baseInfo;
	private JoglGuiLabel btn;
	
	private JoglGuiLabel lblOption1;
	private JoglGuiLabel lblOption2;
	private JoglGuiColoredRect background;
	private JoglGuiColoredFrame bgFrame;
	
	private JoglGuiCheckBox checkbox1;
	private JoglGuiCheckBox checkbox2;
	
	private boolean bInitialized = false;
	private boolean bUpdateScene = false;
	
	public JIInfoOptions(JoglInteractionManager iman, JSOGraphicalUserInterface gui, JIPSENBaseInfo binfo, JoglGuiLabel btn)
	{
		super(gui);
		
		this.iman = iman;
		
		this.baseInfo = binfo;
		this.btn = btn;
		
		this.setZLevel(-2.0f);
	}
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (!bInitialized)
		{
			Init(scene);
		}
		
		bUpdateScene = false;
    }
	
	public void Init(JoglScene scene)
	{		
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_LEFT;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		float maxHeight = 0.1f * (float)JoglGuiStyle.textScale;
		
		float xoff = (float)btn.getRect().getX();
		float yoff = (float)btn.getRect().getY();
		
		background = new JoglGuiColoredRect();
		background.setCanTouch(true);
		background.setRect(xoff + (float)btn.getRect().getZ() - 0.25f, yoff - maxHeight, 0.25f, maxHeight);
		background.setColor(JoglGuiStyle.backColor);
		background.setZLevel(2.0f);
		
		gui.AddGuiObject(background);
		
		bgFrame = new JoglGuiColoredFrame();
		bgFrame.setRect(xoff + (float)btn.getRect().getZ() - 0.25f, yoff - maxHeight, 0.25f, maxHeight);
		bgFrame.setLine(0.0025f / aspect, 0.0025f);
		bgFrame.setColor(0.65f, 0.65f, 0.65f);
		bgFrame.setZLevel(2.1f);
		
		gui.AddGuiObject(bgFrame);
		
		float yoff2 = yoff - (0.1f * (float)JoglGuiStyle.textScale);
		
		lblOption1 = new JoglGuiLabel(ff.clone());
		lblOption1.setRect(xoff + (float)btn.getRect().getZ() - 0.22f, yoff - (0.05f * (float)JoglGuiStyle.textScale), 0.25f, 0.05f * (float)JoglGuiStyle.textScale);
		lblOption1.setText("Only Visible Attrs");
		lblOption1.setPadding(textPadding);
		lblOption1.setBackColor(JoglGuiStyle.transparentColor);
		lblOption1.setZLevel(2.1f);
		
		gui.AddGuiObject(lblOption1);
		
		lblOption2 = new JoglGuiLabel(ff.clone());
		lblOption2.setRect(xoff + (float)btn.getRect().getZ() - 0.22f, yoff - (0.1f * (float)JoglGuiStyle.textScale), 0.25f, 0.05f * (float)JoglGuiStyle.textScale);
		lblOption2.setText("Only Visible Cohorts");
		lblOption2.setPadding(textPadding);
		lblOption2.setBackColor(JoglGuiStyle.transparentColor);
		lblOption2.setZLevel(2.1f);
		
		gui.AddGuiObject(lblOption2);

		float cbxYOff = ((0.05f * (float)JoglGuiStyle.textScale) - 0.035f) * 0.5f;
		
		checkbox1 = new JoglGuiCheckBox();
		checkbox1.setCanTouch(true);
		checkbox1.setRect(xoff + (float)btn.getRect().getZ() - 0.25f + 0.01f, yoff - (0.05f * (float)JoglGuiStyle.textScale) + cbxYOff, 0.035f / aspect, 0.035f);
		checkbox1.setLineDims(0.0025f / aspect, 0.0025f);
		checkbox1.setColor(1.0f, 1.0f, 1.0f);
		//cbx.setColor(colors.get(i));
		checkbox1.setBackColor(JoglGuiStyle.disabledColor);
		checkbox1.setChecked(JoglGuiPerformanceInfoList.bFilterVisCats);
		checkbox1.setZLevel(2.1f);
		
		gui.AddGuiObject(checkbox1);

		checkbox2 = new JoglGuiCheckBox();
		checkbox2.setCanTouch(true);
		checkbox2.setRect(xoff + (float)btn.getRect().getZ() - 0.25f + 0.01f, yoff - (0.1f * (float)JoglGuiStyle.textScale) + cbxYOff, 0.035f / aspect, 0.035f);
		checkbox2.setLineDims(0.0025f / aspect, 0.0025f);
		checkbox2.setColor(1.0f, 1.0f, 1.0f);
		//cbx.setColor(colors.get(i));
		checkbox2.setBackColor(JoglGuiStyle.disabledColor);
		checkbox2.setChecked(JoglGuiPerformanceInfoList.bFilterVisCohorts);
		checkbox2.setZLevel(2.1f);
		
		gui.AddGuiObject(checkbox2);
		
		bInitialized = true;
	}
	
	//MouseListener method implementation
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		Vector<JoglGuiObject> jgo = gui.GetObjectsFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo.contains(btn))
			{
				Dispose();
				iman.removeInteraction(this);
				
				return true;
			}
			else if (jgo.contains(background))
			{
				return true;
			}
			else
			{
				Dispose();
				iman.removeInteraction(this);
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		Vector<JoglGuiObject> jgo = gui.GetObjectsFromPos(e.getX(), e.getY());
		if (jgo.contains(btn))
		{
			return true;
		}
		else if (jgo.contains(background))
		{
			return true;
		}
		return false;
    }
	
	@Override
	public boolean mouseClicked(JoglInputMouseEvent e)
	{
		Vector<JoglGuiObject> jgo = gui.GetObjectsFromPos(e.getX(), e.getY());

		if (jgo.contains(checkbox1))
		{
			checkbox1.setChecked(!checkbox1.getChecked());
			
			baseInfo.pil.bFilterVisCats = checkbox1.getChecked();
			baseInfo.bResetData = true;
			
		}
		else if (jgo.contains(checkbox2))
		{
			checkbox2.setChecked(!checkbox2.getChecked());

			baseInfo.pil.bFilterVisCohorts = checkbox2.getChecked();
			baseInfo.bResetData = true;
		}
		
		if (jgo.contains(btn))
		{
			return true;
		}
		else if (jgo.contains(background))
		{
			return true;
		}
		return false;
	}
	
	public void Dispose()
	{
		if (bInitialized)
		{
			gui.RemoveGuiObject(lblOption1);
			gui.RemoveGuiObject(lblOption2);
			gui.RemoveGuiObject(checkbox1);
			gui.RemoveGuiObject(checkbox2);
			gui.RemoveGuiObject(bgFrame);
			gui.RemoveGuiObject(background);
			
			bInitialized = false;
		}
	}

}
