package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.HashMap;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.scenemanagement.BSPPlane;
import org.processmining.processprofiler3d.compviz.scenemanagement.BSPTree;
import org.processmining.processprofiler3d.compviz.scenemanagement.BSPTreeNode;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JILayerManager extends JoglInteraction
{
	//private JoglScene scene = null;
	private JSOPetriNet pnet = null;
	private boolean bUpdateLayers = true;
	
	public BSPTree mytree = null;
	
    public JILayerManager(JoglScene scene, JSOPetriNet pnet)
    {
    	this.pnet = pnet;
    	
    	//make sure this is one of the last interactions in stack to execute UpdateScene
    	zlevel = 900.0f;
    }
    
    public void UpdateLayers(int level)
    {
		//for each level above root
		for (int i = level; i <= pnet.getMaxLevel(); i++)
		{
			//get all visible subnets for this layer 
			for (JSOSubNet snet : pnet.getAllSubNets())
			{
				if (snet.getLevel() == i)
				{
					//snet.dynoff = JoglVectord3.ZERO;
					snet.setDynamicOffset(JoglVectord3.ZERO);
				}
			}
		}
    	bUpdateLayers = true;
    }
    
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateLayers;
	}

	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (!bUpdateLayers)
			return;
		
		boolean bBuildSuccess = false;
		HashMap<Integer, BSPTreeNode> sceneTree = new HashMap<Integer, BSPTreeNode>();
		//for each level above root
		for (int i = 1; i <= pnet.getMaxLevel(); i++)
		{
			Vector<JSOSubNet> layer = new Vector<JSOSubNet>();
			//get all visible subnets for this layer 
			for (JSOSubNet snet : pnet.getAllSubNets())
			{
				if (snet.getLevel() == i)
				{
					if (snet.getIsVisible())
					{
						layer.add(snet);
					}
					else
					{
						//snet.dynoff = JoglVectord3.ZERO;
						snet.setDynamicOffset(JoglVectord3.ZERO);
					}
				}
			}
			
			if (layer.size() > 1)
			{
				boolean built = false;
				
				int loopCount = 0;
				while (!built && loopCount < 10)
				{
					FixLayer(layer);
					
					BSPTree xtree = new BSPTree();
					built = xtree.Build(layer);
					sceneTree.put(i, xtree.root);
					loopCount++;
				}
				if (!built)
				{
					System.out.println("Failed to split set!");
				}
				else
				{
					bBuildSuccess = true;
				}
			}
			else if (layer.size() == 1)
			{
				//layer.firstElement().dynoff = JoglVectord3.ZERO;
				layer.firstElement().setDynamicOffset(JoglVectord3.ZERO);
				
				BSPTree xtree = new BSPTree();
				xtree.Build(layer);
				sceneTree.put(i, xtree.root);
			}
		}

		//add root layer to hashmap
		BSPTreeNode xtree2 = new BSPTreeNode();
		xtree2.leafs = new Vector<JSOSubNet>();
		xtree2.leafs.add(pnet.getRootSubNet());
		sceneTree.put(0, xtree2);
		
		//build scene bsp tree
		BSPTree finaltree = new BSPTree();
		BSPTreeNode lastnode = sceneTree.get(0);
		BSPTreeNode finalnode = new BSPTreeNode();
		for (int i = 1; i <= pnet.getMaxLevel(); i++)
		{
			BSPTreeNode mynode = sceneTree.get(i);
			if (mynode != null)
			{
				finalnode.front = lastnode;
				finalnode.back = mynode;
				finalnode.plx = new BSPPlane(new JoglVectord3(0,-1,0), (i - 1) * JSOSubNet.subnetOffset.getY() * (-1));
				lastnode = finalnode;
				finalnode = new BSPTreeNode();
			}
		}
		finaltree.root = lastnode;
		mytree = finaltree;
		
		//update scenes
//		if (scene instanceof JoglModelScene)
//		{
//			JoglModelScene jms = (JoglModelScene)scene;
//			jms.setBSPTree(mytree);
//		}
		pnet.setBSPTree(mytree);
		
		if (bBuildSuccess)
		{
			bUpdateLayers = false;
		}
	}
	
	private void FixLayer(Vector<JSOSubNet> snets)
	{
		//try to fix layer
		for (int i = 0; i < snets.size(); i++)
		{
			for (int j = 0; j < snets.size(); j++)
			{
				if (i != j)
				{
					double spacing = 5;
					
					JSOSubNet netA = snets.get(i);
					JSOSubNet netB = snets.get(j);
					
					JoglVectord4 netARect = netA.ped.getPedestalMaxRect();
					JoglVectord4 netBRect = netB.ped.getPedestalMaxRect();
					
					double aleft = (netARect.getX() - netARect.getZ()) - spacing;
					double aright = (netARect.getX() + netARect.getZ()) + spacing;
					double atop = (netARect.getY() - netARect.getW()) - spacing;
					double abottom = (netARect.getY() + netARect.getW()) + spacing;
					double bleft = (netBRect.getX() - netBRect.getZ()) - spacing;
					double bright = (netBRect.getX() + netBRect.getZ()) + spacing;
					double btop = (netBRect.getY() - netBRect.getW()) - spacing;
					double bbottom = (netBRect.getY() + netBRect.getW()) + spacing;

					boolean bOverlapX = (aright > bleft) && (aleft < bright);
					boolean bOverlapY = (abottom > btop) && (atop < bbottom);
					
					if (bOverlapX && bOverlapY)
					{	
						//overlap occuring, need to resolve
						double ovx = Math.min((aright - bleft), (bright - aleft));
						double ovy = Math.min((abottom - btop), (bbottom - atop));
						
						JoglVectord3 netAOrigin = netA.getParentNet().relativeToAbsolute(netA.getOrigin());
						JoglVectord3 netBOrigin = netB.getParentNet().relativeToAbsolute(netB.getOrigin());
						double dx = netAOrigin.getX() - netBOrigin.getX();
						double dy = netAOrigin.getZ() - netBOrigin.getZ();
						double dirX = dx / Math.abs(dx);
						double dirY = dy / Math.abs(dy);
						JoglVectord3 moveA = new JoglVectord3(0,0,0);
						JoglVectord3 moveB = new JoglVectord3(0,0,0);
						
						double overlapfactor = 0.5;
						
						if (dy == 0)
						{
							double move = dirX * ovx * overlapfactor;
							moveA = new JoglVectord3(move, 0, 0);
							moveB = new JoglVectord3(-move, 0, 0);
						}
						else if (dx == 0)
						{
							double move = dirY * ovy * overlapfactor;
							moveA = new JoglVectord3(0, 0, move);
							moveB = new JoglVectord3(0, 0, -move);		
						}
						else
						{
							if (Math.abs(dx) > Math.abs(dy))
							{
								double move = dirX * ovx * overlapfactor;
								moveA = new JoglVectord3(move, 0, 0);
								moveB = new JoglVectord3(-move, 0, 0);
							}
							else
							{
								double move = dirY * ovy * overlapfactor;
								moveA = new JoglVectord3(0, 0, move);
								moveB = new JoglVectord3(0, 0, -move);		
							}
						}
						
						netA.setDynamicOffset(JoglVectord3.Add(netA.getDynamicOffset(), moveA));
						netB.setDynamicOffset(JoglVectord3.Add(netB.getDynamicOffset(), moveB));
					}
				}
			}
		}
	}
}
