package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.Date;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import javax.swing.JOptionPane;

import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.jogl.interactions.replay.ReplayProcessor;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOReplaya;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.processprofiler3d.psen.gui.SlickerOptionPane;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayCollection;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JILogReplay extends JoglInteraction 
{
	private JoglInteractionManager iman;
	private JoglScene scene = null;
	private JSOGraphicalUserInterface gui = null;
	private boolean bUpdateScene = false;

	private Marking iMarking = null;
	private JSOPetriNet jn = null;
	private JSOReplaya rpla = null;
	private TraceReplayCollection trc = null;

	private boolean bWaitForSimulation = true;
	public boolean bHasReplayConnection = false;
	public double maxLength = 0.0;
	
	private ReplayProcessor rpc;
	
	private String visMode = "None";
	private CohortClassifier mycc = null;
	
	public JILogReplay(JoglInteractionManager iman, TraceReplayCollection trc, JoglScene scene, JSOGraphicalUserInterface gui, JSOReplaya rpla, JSOPetriNet jn, Marking initMarking)
	{
		this.iman = iman;
		this.scene = scene;	
		this.gui = gui;
		this.trc = trc;
		
		this.rpla = rpla;
		this.iMarking = initMarking;
		this.jn = jn;
	}
	
	public String getVisMode()
	{
		return visMode;
	}
	
	public Vector<String> getVisModeNames()
	{
		Vector<String> lst = new Vector<String>();
		lst.add("None");
		if (mycc != null)
		{
			lst.add("Enabled");
		}
		
		return lst;
	}
	
	public void setVisMode(String val)
	{
		if (!val.equals(visMode))
		{
			visMode = val;
			bUpdateScene = true;
		}
	}
	
	public void SimulateReplay(CohortClassifier cc)
	{
		mycc = cc;
		
		//initialize state
		if (jlrp != null)
		{
			jlrp.UpdateTokenCohorts(cc);
		}
		else
		{
			if (trc != null)
			{
				System.out.println("Starting replay simulation!");
				rpc = new ReplayProcessor();
				rpc.start(trc, rpla, jn, iMarking);
				
				bWaitForSimulation = true;
				bUpdateScene = true;
			}
			else
			{
				System.out.println("No replay collection found!");
				SlickerOptionPane.showMessageDialogNonProM(null,
					    "No replay collection found!",
					    "Missing Data",
					    JOptionPane.ERROR_MESSAGE);
			}
		}
	}
	
	@Override
	public boolean ShouldAnimate()
	{
		if (rpc != null && !rpc.isFinished())
		{
			return true;
		}
		return bUpdateScene;
	}
	
	private JILogReplayPanel jlrp = null;
	
	@Override
	public void UpdateScene(JoglScene scene)
	{	
//		if (rpc != null && !rpc.isFinished())
//		{
//			if (jlrp != null)
//			{
//				HideReplayPanel();
//			}
//		}
		
		//don't do anything else until the simulation has finished!
		if (rpc == null || !rpc.isFinished())
		{
			return;
		}
		
		if (bWaitForSimulation)
		{
			if (rpc != null && rpc.isFinished())
			{
				TimeSpan ts = new TimeSpan(rpc.processStartTime, new Date());
				System.out.println("Total processing time: " + ts.ToString(TimeUnit.SECONDS) + "s.");
			}
			
			bWaitForSimulation = false;
		}
		
		if (!bWaitForSimulation)
		{		
			//System.out.println("Initializing replay panel!");
			
			if (!visMode.equals("None"))
			{
				if (jlrp == null)
				{
					ShowReplayPanel();
				}
			}
			else
			{
				if (jlrp != null)
				{
					HideReplayPanel();
				}
			}
		}
		
		bUpdateScene = false;
	}
	
	private void ShowReplayPanel()
	{
		jlrp = new JILogReplayPanel(iman, scene, gui, rpc, rpla, mycc);
		iman.addInteraction(jlrp);
		jlrp.Init();	
	}
	
	private void HideReplayPanel()
	{
		if (jlrp != null)
		{
			jlrp.Dispose();
			iman.removeInteraction(jlrp);
			jlrp = null;
		}
	}
}
