package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.awt.event.MouseEvent;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredRect;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiTexturedRect;
import org.processmining.processprofiler3d.compviz.jogl.input.PP3DKeyBindings;
import org.processmining.processprofiler3d.compviz.jogl.interactions.replay.ReplayProcessor;
import org.processmining.processprofiler3d.compviz.jogl.interactions.replay.TraceReplay;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOReplaya;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.gldatastructures.JoglTextureDelayedLoad;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputKeyEvent;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.maths.JoglColor3;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JILogReplayPanel extends JoglInteraction 
{
	private JoglScene scene = null;
	private JSOGraphicalUserInterface gui;
	private boolean bUpdateScene = false;
	
	private JSOReplaya rpla;
	private Vector<TraceReplay> replays = new Vector<TraceReplay>();

	private JoglTexture icoBtnPlay;
	
	private JoglGuiTexturedRect btnPlay;
	private JoglGuiLabel btnIncrease;
	private JoglGuiLabel btnDecrease;
	private JoglGuiLabel lblTime = null;
	private JoglGuiLabel lblSpeed = null;
	
	private JoglGuiColoredRect timeline;
	private JoglGuiColoredRect timelineMarker;
	
	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();

	private boolean bInit = true;
	public boolean bHasReplayConnection = false;
	public double maxLength = 0.0;
	
	public JILogReplayPanel(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui, ReplayProcessor rpc, JSOReplaya rpla, CohortClassifier cc)
	{
		this.scene = scene;
		this.gui = gui;

		this.rpla = rpla;
		
		setZLevel(-100.0f);
		
		replays = rpc.getReplays();
		maxLength = rpc.getMaxLength();
		
		//JISplitCohorts jsc = iman.findInteraction(JISplitCohorts.class);
		//CohortClassifier cc = jsc.mycc;
		
		UpdateTokenCohorts(cc);
		
		bUpdateScene = true;
	}

	public void UpdateTokenCohorts(CohortClassifier cc) 
	{
		rpla.ClearParticles();
		
		for (TraceReplay tr : replays)
		{
			int tokenCohort = cc.mapTrace2Cohort(tr.getTrace());
			tr.InitReplaya(tokenCohort);
		}
		
		//reset trace replay time to time previously on replay
		double lastTimeElapsed = timeElapsed;
		timeElapsed = 0.0;
		bMarkerPressed = true;
		Tick(lastTimeElapsed);
		bMarkerPressed = false;
	}
	
	public void Init()
	{
		icoBtnPlay = new JoglTextureDelayedLoad(scene, "/org/processmining/processprofiler3d/compviz/jogl/resources/IcoReplayPlay.tga");
		JoglResourceManager.AddResource("IconButtonPlay", icoBtnPlay);

		FontFormat ff = new FontFormat();
		ff.width = (0.04f / gui.getAspect()) * (float)JoglGuiStyle.textScale;
		ff.height = 0.04f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_LEFT;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff2 = new FontFormat();
		ff2.width = (0.04f / gui.getAspect()) * (float)JoglGuiStyle.textScale;
		ff2.height = 0.04f * (float)JoglGuiStyle.textScale;
		ff2.align = FontFormat.ALIGN_RIGHT;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff3 = new FontFormat();
		ff3.width = (0.06f / gui.getAspect()) * (float)JoglGuiStyle.textScale;
		ff3.height = 0.06f * (float)JoglGuiStyle.textScale;
		ff3.align = FontFormat.ALIGN_CENTER;
		ff3.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		
		lblTime = new JoglGuiLabel(ff2.clone());
		lblTime.setRect(0.6f, -0.825f, 0.2f, 0.05f);
		lblTime.setText("0.0");
		lblTime.setPadding(textPadding);
		lblTime.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblTime);
		controls.add(lblTime);
		
		lblSpeed = new JoglGuiLabel(ff.clone());
		lblSpeed.setRect(-0.7f, -0.825f, 0.2f, 0.05f);
		lblSpeed.setText("1.0x");
		lblSpeed.setPadding(textPadding);
		lblSpeed.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblSpeed);
		controls.add(lblSpeed);
		
		btnPlay = new JoglGuiTexturedRect();
		btnPlay.setRect(0.0f, -0.85f, 0.1f / (float)gui.getAspect(), 0.1f);
		btnPlay.setTex(icoBtnPlay);
		btnPlay.setColor(JoglColor3.WHITE);
		btnPlay.setCanTouch(true);	
		
		gui.AddGuiObject(btnPlay);
		controls.add(btnPlay);
		
		btnIncrease = new JoglGuiLabel(ff3.clone());
		btnIncrease.setRect(-0.8f, -0.825f, (0.05f / (float)gui.getAspect())  * (float)JoglGuiStyle.textScale, 0.05f * (float)JoglGuiStyle.textScale);
		btnIncrease.setBackColor(new JoglVectord4(0.35, 0.35, 0.35, 0.5));
		btnIncrease.setColor(JoglColor3.WHITE);
		btnIncrease.setText("+");
		btnIncrease.setCanTouch(true);	
		
		gui.AddGuiObject(btnIncrease);
		controls.add(btnIncrease);
		
		btnDecrease = new JoglGuiLabel(ff3.clone());
		btnDecrease.setRect(-0.75f, -0.825f, (0.05f / (float)gui.getAspect()) * (float)JoglGuiStyle.textScale, 0.05f * (float)JoglGuiStyle.textScale);
		btnDecrease.setBackColor(new JoglVectord4(0.35, 0.35, 0.35, 0.5));
		btnDecrease.setColor(JoglColor3.WHITE);
		btnDecrease.setText("-");
		btnDecrease.setCanTouch(true);	
		
		gui.AddGuiObject(btnDecrease);
		controls.add(btnDecrease);
		
		timeline = new JoglGuiColoredRect();
		timeline.setRect(-0.8f, -0.7f, 1.6f, 0.0125f);
		timeline.setColor(JoglColor3.WHITE);
		//timeline.setCanTouch(true);	
		
		gui.AddGuiObject(timeline);
		controls.add(timeline);
		
		timelineMarker = new JoglGuiColoredRect();
		timelineMarker.setRect(-0.8f, -0.7f, 0.025f / (float)gui.getAspect(), 0.025f);
		timelineMarker.setColor(JoglColor3.RED);
		timelineMarker.setZLevel(1.0f);
		timelineMarker.setCanTouch(true);	
		
		gui.AddGuiObject(timelineMarker);
		controls.add(timelineMarker);
		
		updateTimelineMarker(lastProgress);
		
		bInit = false;
	}
	
	public void Dispose()
	{
		rpla.ClearParticles();
		
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
	}
	
	@Override
	public boolean ShouldAnimate()
	{
		if (bPlay)// || (rpc != null && !rpc.isFinished()))
		{
			return true;
		}
		return bUpdateScene;
	}
	
	private boolean bPlay = false;
	private double defaultTimeStep = 0.02;
	private double lastProgress = 0.0;
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (bInit)
		{
			System.out.println("Initializing replay panel!");
			
			Init();
		}
		
		if (bMarkerPressed)
		{
			Tick(0.0);
		}
		else if (bPlay)
		{
			Tick(defaultTimeStep);
		}
		
		bUpdateScene = false;
	}

	private double timeElapsed = 0.0;
	
	private void Tick(double timeStep) 
	{
		//System.out.println("Tick: " + timeStep);
		double progress = 1.0;
		boolean bProgress = false;
		if (bPlay || bMarkerPressed)
		{
			//System.out.println("Tick Play!");
			for (TraceReplay replay : replays)
			{
				replay.Tick(timeStep);
				if (replay.getProgress() > 0.0)
				{
					progress = Math.min(progress, replay.getProgress());
					bProgress = true;
				}
			}
			timeElapsed = timeElapsed + timeStep;
		}
		if (!bProgress)
		{
			progress = lastProgress;
		}
		else
		{
			lastProgress = progress;
		}
		
		if (timelineMarker != null)
		{
			updateTimelineMarker(progress);
		}
	}

	private void updateTimelineMarker(double progress) 
	{
		float markerOffset = (float)Math.max(0.0, Math.min(1.0, progress));
		float timeLineLength = getTimelineTotalLength(gui);
		JoglVectord4 timelineMarkerRect = new JoglVectord4(-0.8f, -0.7f, 0.025f / (float)gui.getAspect(), 0.025f);
		timelineMarker.setRect(JoglVectord4.Add(timelineMarkerRect, new JoglVectord4(timeLineLength * markerOffset, 0, 0, 0)));
		lblTime.setText("" + String.format("%.2f", markerOffset));
	}
	
	private float getTimelineTotalLength(JSOGraphicalUserInterface gui)
	{
		return 1.6f - (0.025f / (float)gui.getAspect());
	}
	
	//KeyListener method implementations
	@Override
	public boolean keyPressed(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt);
        
		if (evt.getKeyCode() == PP3DKeyBindings.ToggleReplay) //m-key
		{
			bPlay = !bPlay;
			updatePlayButton();
        	bUpdateScene = true;
		}
//		if (evt.getKeyCode() == 78) //n-key
//		{
//			PrevStep();
//        	bUpdateScene = true;
//		}
		
		return false;
	}
	
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MousePressed");
			JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
			if (jgo == btnPlay)
			{
				return true;
			}
			if (jgo == btnIncrease)
			{
				return true;
			}
			if (jgo == btnDecrease)
			{
				return true;
			}
			if (jgo == timelineMarker)
			{
				JoglVectord2 mousePos = gui.GetNDCPos(e.getX(), e.getY());
				double yx = NDCToTimeline(mousePos.getX());
				lastYX = yx;
				bMarkerPressed = true;
				System.out.println("Marker pressed!");
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e)
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			if (bMarkerPressed)
			{
				System.out.println("Marker released!");
				bMarkerPressed = false;
			}
			
			//System.out.println("JIViewManager: MouseReleased");
			JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
			if (jgo != null)
			{
				if (jgo == btnPlay)
				{
					bPlay = !bPlay;
					updatePlayButton();
		        	bUpdateScene = true;
					
					return true;
				}
				if (jgo == btnIncrease)
				{
					defaultTimeStep = defaultTimeStep * 2.0;
					lblSpeed.setText("" + String.format("%.2f", (defaultTimeStep / 0.02)) + "x");
					
		        	bUpdateScene = true;
					
					return true;
				}
				if (jgo == btnDecrease)
				{
					defaultTimeStep = defaultTimeStep * 0.5;
					lblSpeed.setText("" + String.format("%.2f", (defaultTimeStep / 0.02)) + "x");
					
		        	bUpdateScene = true;
					
					return true;
				}
				if (jgo == timelineMarker)
				{
					return true;
				}
			}
		}
		return false;
    }

	public double lastYX = 0;
	
	//converts a normalized device coordinate X value into a value between 0 and 1 on the timeline
	private double NDCToTimeline(double screenX)
	{
		double yx = (screenX + 0.8) / getTimelineTotalLength(gui);
		
		return yx;
	}
	
	@Override
	public void mouseDragged(JoglInputMouseEvent e)
	{
		//System.out.println("Mouse dragged!");
		if (bMarkerPressed)
		{
			//System.out.println("Marker dragged!");
			JoglVectord2 mousePos = gui.GetNDCPos(e.getX(), e.getY());
			double yx = NDCToTimeline(mousePos.getX());
			double markerx = NDCToTimeline(timelineMarker.getRect().getX());
			
			double delta = yx - lastYX;			
			double fixedDelta = delta;
			if (delta > 0)
			{
				fixedDelta = Math.min(delta, 1.0 - markerx);
			}
			else if (delta < 0)
			{
				fixedDelta = -Math.min(-delta, markerx);
			}
			
			if (fixedDelta != 0)
			{
				Tick(fixedDelta * maxLength);
			}
			
			lastYX = Math.max(0, Math.min(1.0, yx));
			
			bUpdateScene = true;
		}
	}
	
	private boolean bMarkerPressed = false;
	
	private void updatePlayButton() 
	{
		if (bPlay)
		{
			btnPlay.setColor(JoglColor3.DARKGRAY);
		}
		else
		{
			btnPlay.setColor(JoglColor3.WHITE);
		}
	}
	
	public void UpdateReplayPanel()
	{
		Dispose();
		Init();
	}
}
