package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Vector;
import java.util.prefs.Preferences;

import org.processmining.processprofiler3d.compviz.jogl.input.PP3DKeyBindings;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdge;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnPlace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglModelScene;
import org.processmining.processprofiler3d.compviz.utils.JoglCubeMapCapture;
import org.processmining.processprofiler3d.psen.fileio.obj.MTLWriter;
import org.processmining.processprofiler3d.psen.fileio.obj.OBJObject;
import org.processmining.processprofiler3d.psen.fileio.obj.OBJObjectWriter;
import org.processmining.processprofiler3d.psen.fileio.obj.OBJWriter;
import org.processmining.processprofiler3d.psen.fileio.obj.SVGSceneRenderer;
import org.processmining.processprofiler3d.psen.fileio.obj.SceneExporter;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.input.JoglInputKeyEvent;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;
import org.processmining.visualisation3d.utils.JoglScreenshotCaptureHQ;

public class JIModelExport extends JoglInteraction 
{
	private JoglInteractionManager iman;
	private JoglCamera cam;
	private JoglScene scene = null;
	private JSOGraphicalUserInterface gui;
	
	public static String screenshotType = "Screenshot";
	public static String exportFilePath = "C:/Lab/Export/test";
	
	private boolean bUpdateScene = false;

	private float lastAspect = 1.0f;
	private int lastWidth = 1;
	private int lastHeight = 1;
	
	public JIModelExport(JoglInteractionManager iman, JoglCamera cam, JoglScene s, JSOGraphicalUserInterface gui)
	{
		this.iman = iman;
		this.gui = gui;
		this.cam = cam;
		scene = s;
		
		exportFilePath = "C:/Lab/Export/test";
		Preferences prefs = Preferences.userNodeForPackage(JIExportPanel.class);
		exportFilePath = prefs.get("BaseFilePath", exportFilePath);
	}
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		float aspect = (float)gui.getAspect();
		lastAspect = aspect;
		lastWidth = (int)gui.getWidth();
		lastHeight = (int)gui.getHeight();
		
		bUpdateScene = false;
	}
	
	//KeyListener method implementations
	@Override
	public boolean keyPressed(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt);
        
		if (evt.getKeyCode() == PP3DKeyBindings.TakeScreenshotSVG) //y-key
		{
			String objFilePath = exportFilePath + ".obj";
			String mtlFilePath = exportFilePath + ".mtl";
			OBJWriter out = new OBJWriter();
			MTLWriter outmtl = new MTLWriter();
			out.Open(objFilePath);
			outmtl.Open(mtlFilePath);
			
			out.WriteMtlLib("test.mtl");
			
			//export base materials
			exportBaseMaterials(outmtl);
			
			//pn.Export(out);
			OBJObjectWriter oow = new OBJObjectWriter();
			
			//scene.Export(out);
			SceneExporter.Export(oow, scene);
			oow.Close();
			
			SVGSceneRenderer.RenderSceneToSVGFile(exportFilePath + ".svg", cam, gui, oow);
			
			for (OBJObject oo : oow.objList)
			{
				out.WriteNewObject(oo);
			}
			
	        out.Close();
	        outmtl.Close();
	        
	        try 
	        {
				ExportModelCSV();
			} 
	        catch (IOException e) 
	        {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	        
        	bUpdateScene = true;
		}
        
		if (evt.getKeyCode() == PP3DKeyBindings.TakeScreenshot) //u-key
		{
			//iman.Screenshot("C:/Lab/Export/screenshot.png");
			//iman.Screenshot(new JoglCubeMapCapture("C:/Lab/Export/screenshot"));
			TakeScreenshot();
	        
        	bUpdateScene = true;
		}
//		if (evt.getKeyCode() == 78) //n-key
//		{
//			PrevStep();
//        	bUpdateScene = true;
//		}
		
		return false;
	}
	
	private void TakeScreenshot()
	{
		if (screenshotType.equals("Screenshot"))
		{
			iman.Screenshot(exportFilePath + ".png");
		}
		else if (screenshotType.equals("Screenshot - HQ"))
		{
			iman.Screenshot(new JoglScreenshotCaptureHQ(exportFilePath + ".png", lastWidth * 1, lastHeight * 1, 16));
		}
		else if (screenshotType.equals("Cubemap"))
		{
			iman.Screenshot(new JoglCubeMapCapture(exportFilePath));
		}
	}
	
	private void exportBaseMaterials(MTLWriter outmtl)
	{
		outmtl.WriteNewMaterial("ModelElementBlack");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(0, 0, 0);
		outmtl.WriteDiffuseColor(0, 0, 0);
		outmtl.WriteSpecularColor(0, 0, 0);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("ModelElementWhite");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(1, 1, 1);
		outmtl.WriteDiffuseColor(1, 1, 1);
		outmtl.WriteSpecularColor(1, 1, 1);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("ModelPedestal");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(1, 1, 1);
		outmtl.WriteDiffuseColor(1, 1, 1);
		outmtl.WriteSpecularColor(1, 1, 1);
		outmtl.WriteEmissiveColor(0, 0, 0);
		//outmtl.WriteColorVal("Tf", 1, 1, 1);
		outmtl.WriteTransparencyDissolve(0.3);
		outmtl.WriteIlluminationModel(9);
		
		outmtl.WriteNewMaterial("CohortWhite");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(1, 1, 1);
		outmtl.WriteDiffuseColor(1, 1, 1);
		outmtl.WriteSpecularColor(1, 1, 1);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortBlack");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(0, 0, 0);
		outmtl.WriteDiffuseColor(0, 0, 0);
		outmtl.WriteSpecularColor(0, 0, 0);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortRed");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(1, 0, 0);
		outmtl.WriteDiffuseColor(1, 0, 0);
		outmtl.WriteSpecularColor(1, 0, 0);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortGreen");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(0, 1, 0);
		outmtl.WriteDiffuseColor(0, 1, 0);
		outmtl.WriteSpecularColor(0, 1, 0);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortBlue");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(0, 0, 1);
		outmtl.WriteDiffuseColor(0, 0, 1);
		outmtl.WriteSpecularColor(0, 0, 1);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortYellow");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(1, 1, 0);
		outmtl.WriteDiffuseColor(1, 1, 0);
		outmtl.WriteSpecularColor(1, 1, 0);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortCyan");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(0, 1, 1);
		outmtl.WriteDiffuseColor(0, 1, 1);
		outmtl.WriteSpecularColor(0, 1, 1);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
		
		outmtl.WriteNewMaterial("CohortMagenta");
		outmtl.WriteSpecularPower(0);
		outmtl.WriteAmbientColor(1, 0, 1);
		outmtl.WriteDiffuseColor(1, 0, 1);
		outmtl.WriteSpecularColor(1, 0, 1);
		outmtl.WriteEmissiveColor(0, 0, 0);
		outmtl.WriteTransparencyDissolve(1);
		outmtl.WriteIlluminationModel(0);
	}
	
	public void ExportModelCSV() throws IOException
	{
		FileOutputStream fos;
		Writer out;
		
		String filePath = "C:/Lab/Export/test.csv";
        fos = new FileOutputStream(filePath);
        out = new OutputStreamWriter(fos, "UTF8");
        
        out.write(",NodeType,Label,xPos,yPos\r\n");
        
        int i = 1;
        
        JoglModelScene jms = (JoglModelScene)scene;
        Vector<JoglSceneObject> lst = jms.getPetriNet().getRootSubNet().getElements();
        for (JoglSceneObject jso : lst)
        {
        	if (jso instanceof JSOPnTransition)
        	{
        		JSOPnTransition t = (JSOPnTransition)jso;
        		if (t.orgTransition.isInvisible())
        		{
        			out.write("" + i + ",2," + t.getLabel() + "," + t.getPosition().getX() + "," + t.getPosition().getZ() + "\r\n");
        		}
        		else
        		{
        			out.write("" + i + ",1," + t.getLabel() + "," + t.getPosition().getX() + "," + t.getPosition().getZ() + "\r\n");
        		}
        		i++;
        	}
        	else if (jso instanceof JSOPnPlace)
        	{
        		JSOPnPlace t = (JSOPnPlace)jso;
                out.write("" + i + ",3," + t.orgPlace.getLabel() + "," + t.getPosition().getX() + "," + t.getPosition().getZ() + "\r\n");
        		i++;
        	}
        	else if (jso instanceof JSOPnEdge)
        	{
        		JSOPnEdge t = (JSOPnEdge)jso;
                out.write("" + i + ",4,Edge," + 0 + "," + 0 + "\r\n");
                i++;
                
        		for (JoglVectord3 v : t.getPoints())
        		{
                    out.write("" + i + ",5,EdgePoint," + v.getX() + "," + v.getZ() + "\r\n");
                    i++;
        		}
        	}
        }
        
        out.close();
		
	}
	
	
}
