package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.awt.event.MouseEvent;
import java.util.HashSet;
import java.util.List;

import org.processmining.processprofiler3d.compviz.PetrinetSimplifier;
import org.processmining.processprofiler3d.compviz.jogl.JoglSelection;
import org.processmining.processprofiler3d.compviz.jogl.interactions.camera.JICameraVR;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdge;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdgeVertex;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnPlace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.collision.JoglRay;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.interactions.JICameraBase;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JIMoveModelElement extends JoglInteraction
{
	private JoglInteractionManager iman;
	private JoglScene scene;
	
	private boolean bMove = false;
	
	private boolean bUpdateScene = true;
	
	private JoglVectord3 lastMousePos = null;
	private JoglVectord2 mouseDelta = JoglVectord2.ZERO;
	
	private boolean bEnabled = false;
	
	public boolean IsEnabled()
	{
		return bEnabled;
	}
	
	public void Enable(boolean bVal)
	{
		bEnabled = bVal;
	}
	
	public JIMoveModelElement(JoglPanel panel, JoglScene scene)
	{
		this.iman = panel.getInteractionManager();
		this.scene = scene;
		
		this.setZLevel(-2.1f);
	}

	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (bMove && (mouseDelta.getX() != 0 || mouseDelta.getY() != 0))
		{
			MoveTransitions(scene, mouseDelta.getX(), mouseDelta.getY());
			mouseDelta = JoglVectord2.ZERO;
		}
		
		bUpdateScene = false;
	}

	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		if (!bEnabled)
		{
			return false;
		}
		
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			JISelectModelElement jsme = iman.findInteraction(JISelectModelElement.class);
			JoglSelection selection = jsme.getSelection();
			Object hitObject = jsme.GetClosestHitObject(e);
			if (hitObject instanceof JoglSceneObject)
			{
				if (selection.contains((JoglSceneObject)hitObject))
				{
					lastMousePos = getGroundPos(e);
					getGroundPos(e);
					bMove = true;
					return true;
				}
			}
		}
		return false;
    }

	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			bMove = false;
		}
		
		return false;
    }
	
	@Override
	public void mouseDragged(JoglInputMouseEvent e)
	{
		if (bMove)
		{
			JoglVectord3 mousePos = getGroundPos(e);
			JoglVectord3 delta = JoglVectord3.Subtract(mousePos, lastMousePos);
			mouseDelta = JoglVectord2.Add(mouseDelta,  new JoglVectord2(delta.getX(), delta.getZ()));
			
			lastMousePos = mousePos;
		}
	}

	private void MoveTransitions(JoglScene scene, double dx, double dy)
	{
		HashSet<JSOSubNet> changedNets = new HashSet<JSOSubNet>();
		
		JSOPetriNet pnet = scene.findSceneObject(JSOPetriNet.class);
		JISelectModelElement sme = iman.findInteraction(JISelectModelElement.class);
		if (sme != null)
		{
			JoglSelection s = sme.selection;
			for (JoglSceneObject jso : s.getElements())
			{
				if (jso instanceof JSOPnTransition)
				{

					JSOPnTransition t = (JSOPnTransition)jso;
					if (!changedNets.contains(t.getParentNet()))
					{
						changedNets.add(t.getParentNet());
					}
					t.setPosition(JoglVectord3.Add(new JoglVectord3(dx,0,dy), t.getPosition()));
				}
				if (jso instanceof JSOPnPlace)
				{

					JSOPnPlace t = (JSOPnPlace)jso;
					if (!changedNets.contains(t.getParentNet()))
					{
						changedNets.add(t.getParentNet());
					}
					t.setPosition(JoglVectord3.Add(new JoglVectord3(dx,0,dy), t.getPosition()));
				}
				if (jso instanceof JSOPnEdgeVertex)
				{
					JSOPnEdgeVertex t = (JSOPnEdgeVertex)jso;
					if (!changedNets.contains(t.getParentNet()))
					{
						changedNets.add(t.getParentNet());
					}
					t.setPosition(JoglVectord3.Add(new JoglVectord3(dx,0,dy), t.getPosition()));
					t.UpdateAABB();
					
					//update vertex position in layout!!!
					JSOPnEdge pedge = t.myEdge;
					JSOSubNet psnet = t.getParentNet();
					PetrinetSimplifier layouter = psnet.getLayout();
					List<JoglVectord3> lst = layouter.getEdgePoints(pedge.edgeLayoutID);
					JoglVectord3 vx = lst.get(t.vertexId);
					vx = JoglVectord3.Add(new JoglVectord3(dx,0,dy), vx);
					lst.set(t.vertexId, vx);
				}
			}
			
		}
		
		if (!changedNets.isEmpty())
		{
			long start1 = System.nanoTime();
			
			for (JSOSubNet snet : pnet.getAllSubNets())
			{
				snet.ClearTransform();
				if (snet.ped != null)
				{
					snet.ped.bNeedsUpdate = true;
				}
				
				snet.Relayout();
			}
			
			long start2 = System.nanoTime();
			
			double modelTime = (start2 - start1) * 0.001 * 0.001;
			
			System.out.println("Model batch rebuild time: " + modelTime);
		}
	}
	
	public JoglCamera getCamera()
	{
		JICameraBase jcb = iman.findInteraction(JICameraBase.class);
		if (jcb instanceof JICameraVR)
		{
			JICameraVR cvr = (JICameraVR)jcb;
			return cvr.getHMDCamera();
		}
		return jcb.getCamera();
	}
	
	private JoglVectord3 getGroundPos(JoglInputMouseEvent e)
	{	
		//JoglVectord2 vx = new JoglVectord2(e.getX() / gui.getWidth(), e.getY() / gui.getHeight());
		JoglVectord2 vx = e.getPoint();		
		
		JoglCamera cam = getCamera();
		
		JoglRay r = cam.GetRay(vx);
		
		double scale = (-r.getOrigin().getY()) / r.getDir().getY();
		
		JoglVectord3 vdir = JoglVectord3.Multiply(r.getDir(), scale);
		JoglVectord3 vzero = JoglVectord3.Add(r.getOrigin(), vdir);
		
		return vzero;
	}
}
