package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.io.File;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredRect;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiTextbox;
import org.processmining.processprofiler3d.compviz.jogl.input.PP3DKeyBindings;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOVRController;
import org.processmining.processprofiler3d.compviz.vr.HzdOpenVR;
import org.processmining.processprofiler3d.compviz.vr.IVRModeListener;
import org.processmining.processprofiler3d.compviz.vr.VRSettings;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputKeyEvent;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.utils.IVRAdapter;

public class JIOpenVR extends JoglInteraction implements ITextboxInputListener
{
	private JoglPanel panel;
	private JoglInteractionManager iman;
	private JSOGraphicalUserInterface gui;
	private JoglScene scene;
	
	public static double vrScale = 0.0025;
	private VRSettings settings;

	private JoglGuiLabel highlighted = null;
	
	private boolean bUpdateScene = true;
	
	private Vector<IVRModeListener> vrModeListeners = new Vector<IVRModeListener>();
	
	public JIOpenVR(JoglPanel panel, JoglScene scene, JSOGraphicalUserInterface gui)
	{
		this.panel = panel;
		this.iman = panel.getInteractionManager();
		this.gui = gui;
		this.scene = scene;
		
		settings = new VRSettings();
		settings.LoadPreferences();
	}
	
	public void addVRModeListener(IVRModeListener l)
	{
		vrModeListeners.add(l);
	}
	
	public void removeVRModeListener(IVRModeListener l)
	{
		vrModeListeners.remove(l);
	}

	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{		
		bUpdateScene = false;
	}
	
	public boolean isVREnabled()
	{
		return (panel.getDisplay().myVR != null);
	}
	
	//KeyListener method implementations
	@Override
	public boolean keyPressed(JoglInputKeyEvent evt) 
	{
		//System.out.print("Key: " + evt.getKeyCode());
		
		if (evt.getKeyCode() == PP3DKeyBindings.InitVR) //v-key
		{
			//PrevStep();
        	//bUpdateScene = true;
			ToggleVR();
		}
		
		return false;
	}
	
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {	
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == tbxTextScale)
			{
				//viewManager.SwitchGrid();
				JITextboxInput jibt = new JITextboxInput(iman, scene, gui, tbxTextScale);
				jibt.listener = this;
				iman.addInteraction(jibt);
				
				bUpdateScene = true;
	        	return true;
			}
			if (jgo == btnAccept)
			{
				closeFileDialog();
				
				bUpdateScene = true;
				return true;
			}
		}
		
		return false;
    }
	
	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnAccept)
			{
				highlight(btnAccept);
			}
			else if (jgo == tbxTextScale)
			{
				highlight(tbxTextScale);
			}
			else
			{
				unHighlight();
			}
		}
		else
		{
			unHighlight();
		}
	}
	
	private void highlight(JoglGuiLabel label)
	{
		unHighlight();
	
		label.setBackColor(JoglGuiStyle.highlightColor);
		highlighted = label;
		
		bUpdateScene = true;
	}

	private void unHighlight() 
	{
		if (highlighted != null)
		{
			highlighted.setBackColor(JoglGuiStyle.backColor);
			highlighted = null;

			bUpdateScene = true;
		}
	}
	
	public void ToggleVR()
	{
		if (!isVREnabled())
		{
			StartVR();
		}
		else
		{
			StopVR();
		}
	}
	
	private boolean isPathValid(String txt)
	{
		File f = new File(txt);
		if(f.exists() && !f.isDirectory()) 
		{ 
			return true;
		}
		return false;
	}
	
	private void StartVR()
	{	
		if (!isPathValid(settings.getLibPath()))
		{
			openFileDialog();
		}
		
		IVRAdapter myVR = new HzdOpenVR(panel.getInteractionManager(), settings.getLibPath());
		if (myVR.Init())
		{
			for (IVRModeListener vrml : vrModeListeners)
			{
				vrml.OnVRModeActivated();
			}
//			if (scene instanceof JoglModelScene)
//			{
//				JoglModelScene modelScene = (JoglModelScene)scene;
//				JoglCamera c = panel.getCamera();
//				
//				//scale the scene to be at reasonable real-world size
//				modelScene.getPetriNet().getRootSubNet().setScale(vrScale);
//				
//				//scale camera position with the rest of the scene
//				JoglVectord3 cpos = JoglVectord3.Multiply(c.GetPosition(), vrScale);
//				JoglVectord3 clookat = JoglVectord3.Multiply(c.GetLookAt(), vrScale);	
//				
//				c.SetPosition(cpos);
//				c.SetLookAt(clookat);
//			}
//			
//			JIViewManager view = panel.getInteractionManager().findInteraction(JIViewManager.class);
//			if (view != null)
//			{
//				view.SwitchCameraMode(3); //switch to VR camera control
//			}
			
			panel.getDisplay().myVR = myVR;
			panel.InitVR();
			
			controller1 = new JSOVRController(myVR, 1);
			controller2 = new JSOVRController(myVR, 2);

			jivc1 = new JIVRControllerInteraction(iman, controller1);
			jivc2 = new JIVRControllerInteraction(iman, controller2);
			iman.addInteraction(jivc1);
			iman.addInteraction(jivc2);
			
			//add controller models to scene
			scene.addSceneObject(controller1);
			scene.addSceneObject(controller2);
		}
	}
	
	private JSOVRController controller1 = null;
	private JSOVRController controller2 = null;
	private JIVRControllerInteraction jivc1 = null;
	private JIVRControllerInteraction jivc2 = null;
	
	private void StopVR()
	{
		iman.removeInteraction(jivc1);
		iman.removeInteraction(jivc2);
		
		//remove controller models from scene
		scene.removeSceneObject(controller1);
		scene.removeSceneObject(controller2);	
		
		panel.getDisplay().myVR.Shutdown();
		panel.getDisplay().myVR = null;

		for (IVRModeListener vrml : vrModeListeners)
		{
			vrml.OnVRModeDeactivated();
		}
//		if (scene instanceof JoglModelScene)
//		{
//			JoglCamera c = panel.getCamera();
//			
//			//scale the scene to be at reasonable real-world size
//			((JoglModelScene)scene).getPetriNet().getRootSubNet().setScale(1.0);
//			
//			//scale camera position with the rest of the scene
//			JoglVectord3 cpos = JoglVectord3.Multiply(c.GetPosition(), 1.0 / vrScale);
//			JoglVectord3 clookat = JoglVectord3.Multiply(c.GetLookAt(), 1.0 / vrScale);	
//			
//			c.SetPosition(cpos);
//			c.SetLookAt(clookat);
//		}
//		
//		JIViewManager view = panel.getInteractionManager().findInteraction(JIViewManager.class);
//		if (view != null)
//		{
//			view.SwitchCameraMode(); //switch to Free camera control
//		}
	}

	private JoglGuiLabel lblMessage3;
	private JoglGuiLabel lblMessage2;
	private JoglGuiLabel lblMessage;
	private JoglGuiTextbox tbxTextScale;
	private JoglGuiLabel btnAccept;
	private JoglGuiColoredRect background;
	
	private void openFileDialog()
	{
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_LEFT;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff2 = new FontFormat();
		ff2.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff2.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff2.align = FontFormat.ALIGN_CENTER;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		float yoff = 0.475f;
		
		background = new JoglGuiColoredRect();
		background.setCanTouch(true);
		background.setColor(JoglGuiStyle.backColor);
		background.setRect(-0.2f, 0.375f, 0.4f, 0.35f);
		
		gui.AddGuiObject(background);
		
		lblMessage3 = new JoglGuiLabel(ff2.clone());
		lblMessage3.setRect(-0.175f, 0.65f, 0.35f, 0.05f);
		lblMessage3.setText("VR Mode requires OpenVR to run, but no valid");
		lblMessage3.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblMessage3);
		
		lblMessage2 = new JoglGuiLabel(ff2.clone());
		lblMessage2.setRect(-0.175f, 0.6f, 0.35f, 0.05f);
		lblMessage2.setText("OpenVR DLL was found at the specified location.");
		lblMessage2.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblMessage2);
		
		lblMessage = new JoglGuiLabel(ff2.clone());
		lblMessage.setRect(-0.175f, 0.525f, 0.35f, 0.05f);
		lblMessage.setText("Please specify the path to your OpenVR DLL:");
		lblMessage.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblMessage);

		tbxTextScale = new JoglGuiTextbox(ff.clone());
		tbxTextScale.setCanTouch(true);
		tbxTextScale.setRect(-0.15f, yoff, 0.3f, 0.05f);
		tbxTextScale.setText(settings.getLibPath());
		tbxTextScale.setPadding(textPadding);
		tbxTextScale.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(tbxTextScale);

		btnAccept = new JoglGuiTextbox(ff2.clone());
		btnAccept.setCanTouch(true);
		btnAccept.setRect(-0.075f, yoff - 0.075f, 0.15f, 0.05f);
		btnAccept.setText("Okay");
		btnAccept.setPadding(textPadding);
		btnAccept.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnAccept);
		//controls.add(tbxTextScale);
	}
	

	private void closeFileDialog()
	{
		gui.RemoveGuiObject(background);
		gui.RemoveGuiObject(lblMessage3);
		gui.RemoveGuiObject(lblMessage2);
		gui.RemoveGuiObject(lblMessage);
		gui.RemoveGuiObject(tbxTextScale);
		gui.RemoveGuiObject(btnAccept);
		
		background = null;
		lblMessage3 = null;
		lblMessage2 = null;
		lblMessage = null;
		tbxTextScale = null;
		btnAccept = null;
	}

	public void OnTextInputFinished(JITextboxInput jti)
	{
		String txt = jti.getText();
		
		File f = new File(txt);
		if(f.exists() && !f.isDirectory()) 
		{ 
			settings.setLibPath(txt);
		}
		
	}
}
