package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.HashMap;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.interactions.JoglGuiInteraction;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIOptionsPanelManager extends JoglGuiInteraction
{
	private JoglInteractionManager iman;
	
	private HashMap<JoglGuiLabel, JIOptionsPanel> panels = new HashMap<JoglGuiLabel, JIOptionsPanel>();
	
	private JoglGuiLabel highlighted = null;
	
	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();
	
	public JIOptionsPanelManager(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui)
	{
		super(gui);
		
		this.iman = iman;
	}
	
	public void AddPanel(JIOptionsPanel panel, String btnText)
	{
		JoglGuiLabel btn = CreateButton(btnText);	
		gui.AddGuiObject(btn);
		controls.add(btn);
		panels.put(btn, panel);
	}
	
	private JoglGuiLabel CreateButton(String btnText)
	{
		FontFormat ff = new FontFormat();
		//ff.width = 0.03f / aspect;
		ff.width = (0.03f / 2.0779f) * (float)JoglGuiStyle.textScale; //fixed for my standart resolution for now!
		ff.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_CENTER;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		int buttonNum = panels.size() + 1;
		
		JoglGuiLabel btn = new JoglGuiLabel(ff);
		btn.setCanTouch(true);
		btn.setRect(1.0f - (buttonNum * 0.1f), 0.95f, 0.1f, 0.05f);
		btn.setText(btnText);
		btn.setBackColor(JoglGuiStyle.backColor);
		
		return btn;
	}
	
	public void UpdateOptionsPanel()
	{
		float fwidth = (0.03f / 2.0779f) * (float)JoglGuiStyle.textScale; //fixed for my standart resolution for now!
		float fheight = 0.03f * (float)JoglGuiStyle.textScale;
		
		for (JoglGuiObject jgo : controls)
		{
			if (jgo instanceof JoglGuiLabel)
			{
				JoglGuiLabel jgl = (JoglGuiLabel)jgo;
				jgl.setTextSize(fwidth, fheight);
			}
		}
		bUpdateScene = true;
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		bUpdateScene = false;
	}
	
	//MouseListener method implementation
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {		
		//System.out.println("Mouse click at: " + e.getX() + "/" + e.getY());
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (panels.containsKey(jgo))
			{
				JoglGuiLabel clickedButton = (JoglGuiLabel)jgo;
				JIOptionsPanel clickedPanel = panels.get(clickedButton);
				if (clickedPanel.isVisible())
				{
					HidePanel(clickedPanel, clickedButton);
				}
				else
				{
					for (JoglGuiLabel jbtn : panels.keySet())
					{
						JIOptionsPanel jop = panels.get(jbtn);
						if (jop.isVisible())
						{
							HidePanel(jop, jbtn);
						}
					}
					ShowPanel(clickedPanel, clickedButton);
				}
	
	        	bUpdateScene = true;
	        	return true;
			}
		}
		
		return false;
    }

	private void ShowPanel(JIOptionsPanel panel, JoglGuiLabel btn) 
	{
		panel.Show();
		iman.addInteraction(panel);

		btn.setColor(0.5f, 0.5f, 0.5f);
		btn.setBackColor(JoglGuiStyle.disabledColor);
	}

	private void HidePanel(JIOptionsPanel panel, JoglGuiLabel btn) 
	{
		iman.removeInteraction(panel);
		panel.Hide();

		btn.setColor(1.0f, 1.0f, 1.0f);
		btn.setBackColor(JoglGuiStyle.backColor);
	}
	
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		return false;
    }

	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (panels.containsKey(jgo))
			{
				JoglGuiLabel hoverButton = (JoglGuiLabel)jgo;
				JIOptionsPanel hoverPanel = panels.get(hoverButton);
				unHighlight();
				if (!hoverPanel.isVisible())
				{
					hoverButton.setBackColor(JoglGuiStyle.highlightColor);
					highlighted = hoverButton;
				}
			}
			else
			{
				unHighlight();
			}
		}
		else
		{
			unHighlight();
		}
	}

	private void unHighlight() 
	{
		if (highlighted != null)
		{
			highlighted.setBackColor(JoglGuiStyle.backColor);
			highlighted = null;

			bUpdateScene = true;
		}
	}
}
