package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.awt.event.MouseEvent;
import java.util.HashSet;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredRect;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiMesh;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiPerformanceInfoList;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiScissorBox;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiScrollBar;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.input.JoglInputMouseWheelEvent;
import org.processmining.visualisation3d.interactions.JoglGuiInteraction;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JIPSENBaseInfo extends JoglGuiInteraction implements IScrollBarListener
{
	private JoglInteractionManager iman;
	
	private boolean bInitialized = false;
	private boolean bInitializedData = false;
	
	private JoglGuiLabel lblHeading;
	private JoglGuiColoredRect background;
	private JoglGuiColoredRect backgroundHeader;
	private JoglGuiLabel btnOptions;
	
	protected String header = "";
	protected Vector<InfoPanelCategory> dataLines;
	
	private boolean bUpdateScene = false;
	private boolean bMovingWindow = false;
	private JoglVectord2 lastPos = null;

	private static float xoff = 0.0f;
	private static float yoff = 0.0f;
	
	private float scrollperc = 0.0f;
	private float scrolldist = 0.0f;
	private float scrollmaxdist = 0.0f;
	private boolean bShowScrollbar = false;
	
	private JoglVectord4 backColorHeader = new JoglVectord4(0.0f, 0.0f, 0.0f, 0.7f);
	private JoglVectord4 backColorHeaderHi = new JoglVectord4(0.5f, 0.5f, 0.5f, 0.7f);

	private JoglGuiScrollBar scrollB;
	
	protected JoglSceneObject infoSubject = null;
	
	private HashSet<String> visibleCats = new HashSet<String>();
	private HashSet<String> visibleCohorts = new HashSet<String>();
	
	protected AsyncInfoThread dataThread = null;
	protected boolean bDataReady = false;
	protected boolean bResetData = false;
	
	public JIPSENBaseInfo(JoglInteractionManager iman, JSOGraphicalUserInterface gui)
	{
		super(gui);
		
		this.iman = iman;
	}

	public JoglSceneObject getInfoSubject()
	{
		return infoSubject;
	}
	
	public void setVisibleCats(HashSet<String> vCats)
	{
		visibleCats = vCats;
		bResetData = true;
	}
	
	public void setVisibleCohorts(HashSet<String> vCohorts)
	{
		visibleCohorts = vCohorts;
		bResetData = true;
	}
	
	private JoglGuiScissorBox jgsb = null;
	
	public void UpdateBaseInfo(JoglScene scene)
	{
		if (bInitialized)
		{
			Dispose();
			Init(scene);
		}
	}
	
	public void SetData(Vector<InfoPanelCategory> d)
	{
		bResetData = true;
		
		dataLines = d;
		bDataReady = true;
	}
	
	public void Init(JoglScene scene)
	{		
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = (0.05f / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = 0.05f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_CENTER;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		float maxHeight = 0.8f;
		
		background = new JoglGuiColoredRect();
		background.setCanTouch(true);
		background.setRect(xoff - 0.25f, yoff - maxHeight, 0.5f, maxHeight);
		background.setColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(background);
		
		backgroundHeader = new JoglGuiColoredRect();
		backgroundHeader.setRect(xoff - 0.25f, yoff, 0.5f, 0.07f * (float)JoglGuiStyle.textScale);
		backgroundHeader.setColor(backColorHeader);
		backgroundHeader.setCanTouch(true);
		
		gui.AddGuiObject(backgroundHeader);
		
		lblHeading = new JoglGuiLabel(ff.clone());
		lblHeading.setRect(xoff - 0.25f, yoff + 0.01f * (float)JoglGuiStyle.textScale, 0.5f, 0.05f * (float)JoglGuiStyle.textScale);
		lblHeading.setText(header);
		lblHeading.setPadding(textPadding);
		lblHeading.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblHeading);

		scrollB = new JoglGuiScrollBar();
		scrollB.setCanTouch(true);
		scrollB.addScrollbarListener(this);
		//InitData(scene);

		btnOptions = new JoglGuiLabel(ff.clone());
		btnOptions.setCanTouch(true);
		btnOptions.setRect(xoff - 0.25f + 0.45f, yoff, 0.05f, 0.07f * (float)JoglGuiStyle.textScale);
		btnOptions.setText("...");
		btnOptions.setPadding(textPadding);
		btnOptions.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnOptions);
		//controls.add(btnShowSigTrans);
		
		bInitialized = true;
	}
	
	public JoglGuiPerformanceInfoList pil;
	
	private void InitData(JoglScene scene)
	{
		pil = new JoglGuiPerformanceInfoList(gui);
		pil.setData(dataLines);
		pil.setVisibleCats(visibleCats);
		pil.setVisibleCohorts(visibleCohorts);
		pil.InitData(scene);
		
		float maxHeight = 0.8f;
		
		//required height
		float listHeight = pil.getListHeight();
		float finalHeight = Math.min(listHeight, maxHeight);
		float perc = maxHeight / listHeight;
		scrollmaxdist = listHeight - finalHeight;
		
		if (perc < 1.0f)
		{
			bShowScrollbar = true;
			
			scrollB.setPerc(perc);
			scrollB.setRect(xoff + 0.225f, yoff - finalHeight, 0.025f, finalHeight);
			scrollB.setScrollPerc(scrollperc);
			
			gui.AddGuiObject(scrollB);
		}
		
		JoglGuiMesh jgm = pil.InitGuiMesh();
		if (jgm != null)
		{
			jgsb = new JoglGuiScissorBox(gui);
			if (bShowScrollbar)
			{
				jgsb.setRect(xoff - 0.25f, yoff - finalHeight, 0.5f - 0.025f, finalHeight);
			}
			else
			{
				jgsb.setRect(xoff - 0.25f, yoff - finalHeight, 0.5f, finalHeight);
			}
			for (JoglGuiColoredRect cr : pil.coloredRects)
			{
				jgsb.addChild(cr);
			}
			jgsb.addChild(jgm);
			gui.AddGuiObject(jgsb);
		}
		
		//clip contents of window
		scrolldist = 0;
		MoveScrollbar();
		
		bInitializedData = true;
	}
	
	public void Close()
	{
		if (!dataThread.bProcessingDone)
		{
			dataThread.bAbort = true;
		}
		
		Dispose();
	}
	
	public void Dispose()
	{
		if (bInitialized)
		{
			DisposeData();
			
			gui.RemoveGuiObject(lblHeading);
			gui.RemoveGuiObject(backgroundHeader);
			gui.RemoveGuiObject(background);
			gui.RemoveGuiObject(btnOptions);
			
			bInitialized = false;
		}
	}
	
	private void DisposeData()
	{	
		if (bInitializedData)
		{
			if (pil != null)
			{
				pil.DisposeData();
			}
			
			if (jgsb != null)
			{
				gui.RemoveGuiObject(jgsb);
			}
	
			if (bShowScrollbar)
			{
				gui.RemoveGuiObject(scrollB);
			}
			
			bInitializedData = false;
			bResetData = false;
		}
	}
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (bInitializedData && bResetData)
		{
			DisposeData();
		}
		
		if (!bInitialized)
		{
			Init(scene);
		}
		
		if (!bInitializedData && bDataReady)
		{
			InitData(scene);
		}
		
		bUpdateScene = false;
    }
	
	//MouseListener method implementation
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		Vector<JoglGuiObject> jgo = gui.GetObjectsFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo.contains(btnOptions))
			{

				if (e.getButton() == MouseEvent.BUTTON1)
				{
					JIInfoOptions jio = new JIInfoOptions(iman, gui, this, btnOptions);
					iman.addInteraction(jio);
					
					bUpdateScene = true;
				}
				
				return true;
			}
			else if (jgo.contains(backgroundHeader))
			{

				if (e.getButton() == MouseEvent.BUTTON1)
				{
					bMovingWindow = true;
					lastPos = gui.GetNDCPos(e.getX(), e.getY());
					backgroundHeader.setColor(backColorHeaderHi);
					bUpdateScene = true;
				}
				
				return true;
			}
			else if (bShowScrollbar && jgo.contains(scrollB))
			{
				if (e.getButton() == MouseEvent.BUTTON1)
				{
					//JIScrollBarMove sbm = new JIScrollBarMove(gui, scrollB, e);
					//gui.getInteractionManager().addInteraction(sbm);
					scrollB.StartMoveInteraction(iman, e);
					bUpdateScene = true;
				}
				
				return true;
			}
			else if (jgo.contains(background))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		Vector<JoglGuiObject> jgo = gui.GetObjectsFromPos(e.getX(), e.getY());
		if (bMovingWindow)
		{
			bMovingWindow = false;
			backgroundHeader.setColor(backColorHeader);
			bUpdateScene = true;
			
			return true;
		}
		if (jgo.contains(backgroundHeader))
		{
			return true;
		}
		else if (jgo.contains(background))
		{
			return true;
		}
		return false;
    }
	
	@Override
	public boolean mouseClicked(JoglInputMouseEvent e)
	{
		Vector<JoglGuiObject> jgo = gui.GetObjectsFromPos(e.getX(), e.getY());
		if (jgo.contains(backgroundHeader))
		{
			return true;
		}
		else if (jgo.contains(background))
		{
			return true;
		}
		return false;
	}
	
	@Override
    public void mouseDragged(JoglInputMouseEvent e) 
    {
		if (bMovingWindow)
		{
			JoglVectord2 newPos = gui.GetNDCPos(e.getX(), e.getY());
			MoveWindow(JoglVectord2.Subtract(newPos, lastPos));
			lastPos = newPos;
			bUpdateScene = true;
		}
    }
	
	@Override
	public void mouseWheelMoved(JoglInputMouseWheelEvent e)
	{
		if ((!bShowScrollbar) || scrollB == null || background == null)
		{
			return;
		}
		
		JoglVectord2 cursorPos = gui.GetNDCPos(e.getX(), e.getY());
		//if cursor hovers over info panel
		if (background.IsInside((float)cursorPos.getX(), (float)cursorPos.getY()))
		{
			int delta = e.getWheelRotation();
			scrollB.setScrollPerc(scrollB.getScrollPerc() + (0.1f * delta));
			MoveScrollbar();
		}
	}
	
	private void MoveWindow(JoglVectord2 d)
	{
		JoglVectord4 diff = new JoglVectord4(d.getX(), d.getY(), 0, 0);
		
		xoff = xoff + (float)diff.getX();
		yoff = yoff + (float)diff.getY();

		lblHeading.setRect(JoglVectord4.Add(diff, lblHeading.getRect()));
		background.setRect(JoglVectord4.Add(diff, background.getRect()));
		backgroundHeader.setRect(JoglVectord4.Add(diff, backgroundHeader.getRect()));
		
		if (jgsb != null)
		{
			jgsb.setRect(JoglVectord4.Add(jgsb.getRect(), diff));
		}
		
		if (bShowScrollbar)
		{
			scrollB.setRect(JoglVectord4.Add(diff, scrollB.getRect()));
		}
		
		if (pil != null)
		{
			pil.MoveWindow(d);
		}
		
		if (btnOptions != null)
		{
			btnOptions.setRect(JoglVectord4.Add(btnOptions.getRect(), diff));
		}
	}
	
	public void OnScrollBarMoved(JoglGuiScrollBar sb, JoglVectord2 dir)
	{
		MoveScrollbar();
	}
	
	private void MoveScrollbar()
	{
		if ((!bShowScrollbar) || scrollB == null)
			return;
		
		double oldscrolldist = scrolldist;
		scrollperc = scrollB.getScrollPerc();
		scrolldist = scrollperc * scrollmaxdist;
		
		MoveScrollbar(new JoglVectord2(0.0, scrolldist - oldscrolldist));
	}
	
	private void MoveScrollbar(JoglVectord2 d)
	{
		if ((!bShowScrollbar) || scrollB == null)
			return;
		
		JoglVectord4 diff = new JoglVectord4(d.getX(), d.getY(), 0, 0);
		
		float finalHeight = -(float)background.getRect().getW();

		if (pil != null)
		{
			pil.MoveScrollbar(diff, finalHeight);
		}
	}

}
