package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.HashSet;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.DataMapNormalization;
import org.processmining.processprofiler3d.compviz.DataMapProviderEdge;
import org.processmining.processprofiler3d.compviz.DataMapProviderEdgeQuality;
import org.processmining.processprofiler3d.compviz.TransitionDataMapDouble;
import org.processmining.processprofiler3d.compviz.methods.EdgeDataVisualisationMethodCollection;
import org.processmining.processprofiler3d.compviz.methods.EdgeDataVisualizationMethod;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohort;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIPSENEdgeVisDriver extends JoglInteraction
{
	private JoglScene scene;
	private JoglInteractionManager iman;
	
	private EdgeDataVisualizationMethod edgeVisMethod = EdgeDataVisualisationMethodCollection.EDVM_None;
	private int label1Id = 0;
	private String label1Name;
	private DataMapNormalization normMode;
	
	private String scaleMode = "Linear";
	
	private HashSet<String> visClassifiers;
	
	private DataMapProviderEdge mapProvider;
	private DataMapProviderEdgeQuality mapProviderQuality;
	private JIPSENEdgeVisInterpolate interpolator = null;
	
	private String focusClassifier = null;

	public HashSet<String> getVisibleClassifiers()
	{
		return visClassifiers;
	}
	
	public void setVisibleClassifiers(HashSet<String> vc)
	{
		visClassifiers = vc;
		mapProvider.setVisibleClassifiers(vc);
		mapProviderQuality.setVisibleClassifiers(vc);
		SetVisualization();
	}
	
	public String getFocusClassifier()
	{
		return focusClassifier;
	}
	
	public void setFocusClassifier(String val)
	{
		focusClassifier = val;
		SetVisualization();
	}
	
	public String getScaleMode()
	{
		return scaleMode;
	}
	
	public void setScaleMode(String mode)
	{
		scaleMode = mode;
		
		SetVisualization();
	}
	
	public JIPSENEdgeVisDriver(JoglScene scene, JoglInteractionManager iman, DataMapProviderEdge dmpe, DataMapProviderEdgeQuality dmpq)
	{
		this.scene = scene;
		this.iman = iman;

		mapProvider = dmpe;	
		mapProviderQuality = dmpq;
	}
	
	
	public void Dispose()
	{
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		bUpdateScene = false;
	}
	
	public void setCohortClassifier(Vector<ClassificationCohort> cohorts)
	{
		mapProviderQuality.setCohortClassifier(cohorts);
	}
	
	public void setEdgeVisMethod(String methodName)
	{
		EdgeDataVisualizationMethod edvm = EdgeDataVisualisationMethodCollection.get(methodName);

		edgeVisMethod = edvm;
		normMode = edgeVisMethod.getValidNormalizationMode(1, normMode);
		
		SetVisualization();
	}
	
	public void setLabel1(String labelName)
	{
		label1Id = GetLabelNames().indexOf(labelName);
		SetVisualization();
	}
	
	public String getNormalizationMode()
	{
		return normMode.getModeName();
	}
	
	public void setNormalizationMode(String mode)
	{
		normMode = DataMapNormalization.getModeByName(mode);
		SetVisualization();
	}
	
	public String GetEdgeVisMethodName()
	{
		return edgeVisMethod.GetName();
	}
	
	public Vector<String> GetLabelNames()
	{
		return edgeVisMethod.GetLabelNames();
	}
	
	public Vector<String> GetClassifierNames()
	{
		return edgeVisMethod.GetClassifierNames();
	}
	
	public Vector<String> GetNormalizationNames()
	{
		return edgeVisMethod.GetNormalizationNames(1);
	}
	
	public String GetLabel1Name()
	{
		Vector<String> lst = GetLabelNames();
		
		if (lst.size() > label1Id)
		{
			return lst.get(label1Id);
		}
		return "Error!";
	}
	
	public boolean HasLabel1()
	{
		return edgeVisMethod.HasVariable1();
	}
	
	public void SetVisualization()
	{
		label1Name = GetLabel1Name();

		//Populate transition data
		TransitionDataMapDouble map1 = edgeVisMethod.GenerateDataMap(1, label1Name, normMode, focusClassifier);
		
		if (interpolator == null)
		{	
			interpolator = new JIPSENEdgeVisInterpolate(scene);
			interpolator.SetDataMap(edgeVisMethod, map1, map1, (focusClassifier != null), getScaleMode(), visClassifiers.isEmpty());
			
			iman.addInteraction(interpolator);
		}
		else
		{
			if (interpolator.getCurrentMap() == null)
			{
				System.out.println("GetCurrentMap null!");
			}
			
			interpolator.SetDataMap(edgeVisMethod, map1, interpolator.getCurrentMap(), (focusClassifier != null), getScaleMode(), visClassifiers.isEmpty());
		}
		
		bUpdateScene = true;
	}

}
