package org.processmining.processprofiler3d.compviz.jogl.interactions;

import org.processmining.processprofiler3d.compviz.TransitionDataMapDouble;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENEdgeVisualizationLayer;
import org.processmining.processprofiler3d.compviz.methods.EdgeDataVisualizationMethod;
import org.processmining.processprofiler3d.maths.ScaleTransforms;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIPSENEdgeVisInterpolate extends JoglInteraction
{
	private JoglScene scene;
	
	private EdgeDataVisualizationMethod edgeVisMethod;
	private TransitionDataMapDouble map1;
	private TransitionDataMapDouble map1Old;

	private TransitionDataMapDouble cmap1;
	private TransitionDataMapDouble cmap2;
	
	private float factor = 0.0f;
	private float factorStepSize = 0.02f;
	//private boolean bDoubleRange = false;
	private float DoubleRangeFactor = 0.0f;
	
	private boolean currentVis = true;
	public boolean nextVis = true;
	
	public TransitionDataMapDouble getCurrentMap()
	{
		//return currentMap;
		if (cmap1 != null && cmap2 != null)
		{
			return TransitionDataMapDouble.interpolate(cmap1, cmap2, factor);
		}
		else if (cmap2 != null)
		{
			return cmap2;
		}
		else if (cmap1 != null)
		{
			return cmap1;
		}
		return null;
	}
	
	public JIPSENEdgeVisInterpolate(JoglScene scene)
	{
		this.scene = scene;
		//this.soPsenE = soPsenE;
	}	
	
	public void Dispose()
	{
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (factor < 1.0f)
		{
			factor += factorStepSize;
			factor = Math.min(factor, 1.0f);
			UpdateVis(scene);
		}
		else
		{
			bUpdateScene = false;
		}
	}
	
	public void SetDataMap(EdgeDataVisualizationMethod mode, TransitionDataMapDouble m1, TransitionDataMapDouble m1o, boolean bDoubleRange, String smode, boolean nvis)
	{
		edgeVisMethod = mode;
		nextVis = nvis;
		
		map1 = m1;
		map1Old = m1o;
		if (m1o == null)
		{
			map1Old = m1;
		}
		
		if (bDoubleRange)
		{
			DoubleRangeFactor = 1.0f;
		}
		else
		{
			DoubleRangeFactor = 0.0f;
		}

		if (map1Old != null && map1 != null)
		{
			cmap1 = map1Old;
			cmap2 = ScaleMap(NormalizeMap(map1, DoubleRangeFactor), smode);
			
			if (cmap1 != null && cmap2 != null)
			{
				PushVisualizationData();
			}
		}
		
		factor = 0.0f;
		bUpdateScene = true;
	}
	
	public void UpdateVis(JoglScene scene)
	{
		if (factor == 1.0)
		{
			currentVis = nextVis;
		}
		if (cmap1 != null && cmap2 != null)
		{
			PushVisualizationData();
		}
		bUpdateScene = true;
	}
	
	private void PushVisualizationData()
	{
		TransitionDataMapDouble cmapf = null;
		if (factor == 0.0)
		{
			cmapf = cmap1;
		}
		else if (factor == 1.0)
		{
			cmapf = cmap2;
		}
		else
		{
			cmapf = TransitionDataMapDouble.interpolate(cmap1, cmap2, factor);
		}
		
		for (JSOPSENEdgeVisualizationLayer soPsenE : scene.findSceneObjects(JSOPSENEdgeVisualizationLayer.class))
		{
			soPsenE.SetVisualization(edgeVisMethod, cmapf, currentVis || nextVis);
			soPsenE.BuildScene();	
		}
	}
	
	private double DoubleRangeTransform(double ts, double drf)
	{
		double tsx = ts;
		if (drf > 0.0f)
		{
			Double tsDR = (ts * 2.0) - 1.0;
			tsx =  ts + ((tsDR - ts) * drf);
		}
		
		return tsx;
	}
	
	private TransitionDataMapDouble NormalizeMap(TransitionDataMapDouble cmap, double drf)
	{
		TransitionDataMapDouble newmap = new TransitionDataMapDouble();

		for (String id : cmap.getEntryIds())
		{
			double ts = 0;
			
			if (cmap.hasValue(id))
			{
				double vmin = cmap.getMin(id);
				double vmax = cmap.getMax(id);
				
				ts = cmap.getValueR(id);
				if (vmin != vmax)
				{
					ts = DoubleRangeTransform(ts, drf);
				}
				//if (ts != null)
				{
					newmap.setValue(id, ts);
					newmap.setMin(id, 0.0);
					newmap.setMax(id, 1.0);
				}
			}
		}
		
		return newmap;
	}
	
	private TransitionDataMapDouble ScaleMap(TransitionDataMapDouble cmap, String smode)
	{
		TransitionDataMapDouble newmap = new TransitionDataMapDouble();

		for (String id : cmap.getEntryIds())
		{
			if (cmap.hasValue(id))
			{
				double ts = cmap.getValueR(id);
				ts = ScaleTransforms.TransformScale(ts, smode);
				{
					newmap.setValue(id, ts);
					newmap.setMin(id, 0.0);
					newmap.setMax(id, 1.0);
				}
			}
		}
		
		return newmap;
	}

}
