package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.PP3DCohort;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiCheckBox;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredRect;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOCohortColorManager;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.interactions.JoglGuiInteraction;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIPSENFilter extends JoglGuiInteraction
{
	private JoglInteractionManager iman;
	private IPerformanceStatisticsEnrichedNet psnet;
	
	private JSOCohortColorManager cohortColors;
	
	private JIPSENTransVisDriver tdriver;
	private JIPSENEdgeVisDriver edriver;
	private JIPSENTrajectoryVisDriver tjdriver;

	private boolean bInitialized = false;
	
	private Vector<JoglGuiLabel> labels;
	private Vector<JoglGuiCheckBox> checkboxes;

	private JoglGuiLabel highlighted = null;
	private JoglGuiLabel focussed = null;

	private JoglVectord4 hoverColor = new JoglVectord4(0.75f, 0.75f, 0.75f, 0.75f);
	
	private JoglGuiColoredRect background = null;
	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();
	
	private HashMap<JoglGuiCheckBox, String> cbxClassifier;
	
//	public Vector<String> GetClassifierNames()
//	{
//		Vector<String> lst = new Vector<String>();
//
//		HashSet<String> labels = psnet.getClassifierValues();
//		for (String s : labels)
//		{
//			lst.add(s);
//		}
//		
//		Collections.sort(lst);
//		
//		return lst;
//	}
	
	public Vector<String> GetClassifierNames()
	{
		Vector<String> lst = new Vector<String>();

		//HashSet<String> labels = psnet.getClassifierValues();
		for (PP3DCohort pc : cohortColors.cohorts)
		{
			lst.add(pc.cohortName);
		}
		
		Collections.sort(lst);
		
		return lst;
	}
	
	public HashSet<String> getCheckedClassifiers()
	{
		HashSet<String> vclassifiers = new HashSet<String>();

		for (JoglGuiCheckBox cbx : checkboxes)
		{
			if (cbx.getChecked())
			{
				vclassifiers.add(cbxClassifier.get(cbx));
			}
		}
		
		//somewhat hacky!!!
		for (PP3DCohort pc : cohortColors.cohorts)
		{
			pc.bVisible = vclassifiers.contains(pc.cohortName);
		}
		
		return vclassifiers;
	}
	
	public JIPSENFilter(JoglInteractionManager iman, JSOGraphicalUserInterface gui, IPerformanceStatisticsEnrichedNet psnet, JSOCohortColorManager jccm)
	{
		super(gui);
		
		this.iman = iman;
		this.psnet = psnet;
		
		cohortColors = jccm;
	}
	
	public void UpdateFilters()
	{
		Dispose();
		Init();
	}
	
	public void Init()
	{
		tdriver = iman.findInteraction(JIPSENTransVisDriver.class);
		edriver = iman.findInteraction(JIPSENEdgeVisDriver.class);
		tjdriver = iman.findInteraction(JIPSENTrajectoryVisDriver.class);
		
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_LEFT;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		Vector<String> classifiers = GetClassifierNames();
		HashSet<String> vclassifiers = new HashSet<String>(classifiers);
		
		vclassifiers = psnet.getClassifierValues();

		background = new JoglGuiColoredRect();
		background.setCanTouch(true);
		background.setColor(JoglGuiStyle.backColor);
		background.setRect(-1.0f, 0.2175f - (0.1f * classifiers.size()), 0.05f + (0.1f * (float)JoglGuiStyle.textScale), 0.1f * classifiers.size());
		gui.AddGuiObject(background);

		labels = new Vector<JoglGuiLabel>();
		checkboxes = new Vector<JoglGuiCheckBox>();
		cbxClassifier = new HashMap<JoglGuiCheckBox, String>();
		
		float yoff = 0.0f;
		for (String s : classifiers)
		{
			JoglGuiCheckBox cbx = new JoglGuiCheckBox();
			cbx.setCanTouch(true);
			cbx.setRect(-0.975f, (0.1925f + yoff) -0.035f, 0.035f / aspect, 0.035f);
			cbx.setLineDims(0.0025f / aspect, 0.0025f);
			//cbx.setColor(1.0f, 1.0f, 1.0f);
			//cbx.setColor(colors.get(i));
			cbx.setColor(cohortColors.getColorForCohort(s));
			cbx.setBackColor(JoglGuiStyle.disabledColor);
			
			if (vclassifiers.contains(s))
			{
				cbx.setChecked(true);
			}
			
			gui.AddGuiObject(cbx);
			controls.add(cbx);
			checkboxes.add(cbx);
			cbxClassifier.put(cbx, s);
			
			JoglGuiLabel lbl = new JoglGuiLabel(ff.clone());
			lbl.setCanTouch(true);
			lbl.setRect(-0.95f, (0.2f + yoff) -0.05f, 0.1f * (float)JoglGuiStyle.textScale, 0.05f);
			lbl.setText(s);
			lbl.setBackColor(JoglGuiStyle.transparentColor);
			
			gui.AddGuiObject(lbl);
			controls.add(lbl);
			labels.add(lbl);
			
			yoff = yoff - 0.1f;
		}

		HashSet<String> visClassifiers = getCheckedClassifiers();
		if (tdriver != null)
		{
			tdriver.setVisibleClassifiers(visClassifiers);
		}
		if (edriver != null)
		{
			edriver.setVisibleClassifiers(visClassifiers);
		}
		if (tjdriver != null)
		{
			tjdriver.setVisibleClassifiers(visClassifiers);
		}
		
		bInitialized = true;
	}

	public void Dispose()
	{
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
		gui.RemoveGuiObject(background);
		
		bInitialized = false;
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (!bInitialized)
		{
			Init();
		}
		
		bUpdateScene = false;
	}
	
	//MouseListener method implementation
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
			if (jgo == background)
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
			if (jgo == background)
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {		
		//System.out.println("Mouse click at: " + e.getX() + "/" + e.getY());
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (labels.contains(jgo))
			{
				UpdateFocusCohort(jgo);
				
				bUpdateScene = true;
				return true;
			}
			if (checkboxes.contains(jgo))
			{
				for (JoglGuiCheckBox cbx : checkboxes)
				{
					if (jgo == cbx)
					{
						cbx.setChecked(!cbx.getChecked());
					}
				}
				
				HashSet<String> visClassifiers = getCheckedClassifiers();
				if (tdriver != null)
				{
					tdriver.setVisibleClassifiers(visClassifiers);
				}
				if (edriver != null)
				{
					edriver.setVisibleClassifiers(visClassifiers);
				}
				if (tjdriver != null)
				{
					tjdriver.setVisibleClassifiers(visClassifiers);
				}
				
				JIPSENBaseInfo jbi = iman.findInteraction(JIPSENBaseInfo.class);
				if (jbi != null)
				{
					jbi.setVisibleCohorts(visClassifiers);
				}
				
				bUpdateScene = true;
				return true;
			}
			if (jgo == background)
			{
				return true;
			}
		}
		
		return false;
    }

	public void UpdateFocusCohort(JoglGuiObject jgo) 
	{
		for (JoglGuiLabel l : labels)
		{
			if (jgo == l)
			{
				if (focussed != jgo)
				{
					l.setBackColor(JoglGuiStyle.highlightColor);
					focussed = l;
				}
				else if (focussed == jgo)
				{
					l.setBackColor(hoverColor);
					focussed = null;
				}
			}
		}
		//deselect all other cohorts
		for (JoglGuiLabel l : labels)
		{
			if (jgo != l)
			{
				l.setBackColor(JoglGuiStyle.transparentColor);
			}
		}
		
		//update visualizations to normalize by focussed cohort
		if (edriver != null)
		{
			if (focussed != null)
			{
				edriver.setFocusClassifier(focussed.getText());
			}
			else
			{
				edriver.setFocusClassifier(null);
			}
		}
	}

	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (labels.contains(jgo))
			{
				JoglGuiLabel la = (JoglGuiLabel)jgo;
				if (highlighted != jgo)
				{
					unHighlight();
					
					if (highlighted == focussed)
					{
						la.setBackColor(JoglGuiStyle.highlightColor);
					}
					else
					{
						la.setBackColor(hoverColor);
					}
					highlighted = la;
					
					bUpdateScene = true;
				}
			}
			else
			{
				unHighlight();
			}
			
		}
		else
		{
			unHighlight();
		}
	}

	private void unHighlight() 
	{
		if (highlighted != null)
		{
			if (highlighted == focussed)
			{
				highlighted.setBackColor(JoglGuiStyle.highlightColor);
			}
			else
			{
				highlighted.setBackColor(JoglGuiStyle.transparentColor);
			}
			highlighted = null;

			bUpdateScene = true;
		}
	}
}
