package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.HashSet;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.interactions.camera.JICameraOrbit;
import org.processmining.processprofiler3d.compviz.jogl.scene.IDataVisMagnetLine;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENEdgeVisualizationLayer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdge;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdgeVertex;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnPlace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisMagnetLineBase;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisMagnetLineGroupBase;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglFrustumClipper;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.collision.JoglRay;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JIPSENSelectModelElement extends JISelectModelElement
{
	private IPerformanceStatisticsEnrichedNet psnet = null;
	private IPerformanceStatisticsEnrichedNet pwtt = null;
	private IPerformanceStatisticsEnrichedNet pq = null;
	
	private JIPSENBaseInfo infoPanel = null;
	
	public JIPSENBaseInfo getInfoPanel()
	{
		return infoPanel;
	}
	
	public JIPSENSelectModelElement(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui, JSOPetriNet pnet, IPerformanceStatisticsEnrichedNet psen, IPerformanceStatisticsEnrichedNet pwtt, IPerformanceStatisticsEnrichedNet pq)
	{
		super(iman, scene, gui, pnet);

		this.psnet = psen;
		this.pwtt = pwtt;
		this.pq = pq;
	}

	@Override
	public void SelectAllInRect(JoglInputMouseEvent e, JoglVectord2 startRay, JoglVectord2 endRay)
	{
		JoglCamera cam = getCamera();
		
		if (!e.isShiftDown())
		{
			selection.clear();
			CloseExistingInfoPanel();
		}
		
		double l = Math.min(startRay.getX(), endRay.getX());
		double r = Math.max(startRay.getX(), endRay.getX());
		double t = Math.min(startRay.getY(), endRay.getY());
		double b = Math.max(startRay.getY(), endRay.getY());
		
		JoglFrustumClipper jfc = new JoglFrustumClipper(cam, cam.getCameraLens().getAspect(), l, r, -b, -t);

		JIMoveModelElement jmt = iman.findInteraction(JIMoveModelElement.class);
		if (jmt != null && jmt.IsEnabled())
		{
			for (JSOSubNet snet : pnet.getAllSubNets())
			{
				if (!snet.getIsVisible())
					continue;
				
				JoglVectord3 offset = snet.relativeToAbsolute(JoglVectord3.ZERO);
				for (JSOPnEdgeVertex ev : snet.getEdgeVertices())
				{
					JoglVectord3 pos = JoglVectord3.Add(ev.getPosition(), offset);
					if (jfc.IsPointInside(pos))
					{
						selection.add(ev);
					}
				}
			}
		}
		
		for (JSOSubNet snet : pnet.getAllSubNets())
		{
			if (!snet.getIsVisible())
				continue;
			
			JoglVectord3 offset = snet.relativeToAbsolute(JoglVectord3.ZERO);
			for (JSOPnPlace p : snet.getPlaces())
			{
				JoglVectord3 pos = JoglVectord3.Add(p.getPosition(), offset);
				if (jfc.IsPointInside(pos))
				{
					selection.add(p);
				}
			}
		}
		

		for (JSOSubNet snet : pnet.getAllSubNets())
		{
			if (!snet.getIsVisible())
				continue;

			JoglVectord3 offset = snet.relativeToAbsolute(JoglVectord3.ZERO);
			for (JSOPnTransition p : snet.getTransitions())
			{
				JoglVectord3 pos = JoglVectord3.Add(p.getPosition(), offset);
				if (jfc.IsPointInside(pos))
				{
					selection.add(p);
				}
			}
		}

		for (JSOSubNet snet : pnet.getAllSubNets())
		{
			if (!snet.getIsVisible())
				continue;

			JoglVectord3 offset = snet.relativeToAbsolute(JoglVectord3.ZERO);
			for (JSOPnEdge p : snet.getEdges())
			{
				boolean bInside = true;
				for (JoglVectord3 vpos : p.getPoints())
				{
					JoglVectord3 pt = JoglVectord3.Add(vpos, offset);
					if (!jfc.IsPointInside(pt))
					{
						bInside = false;
						break;
					}
				}
				if (bInside)
				{
					selection.add(p);
				}
			}
		}
		
		bUpdateScene = true;
	}

	@Override
	public void SelectAtMousePos(JoglInputMouseEvent e) 
	{
		boolean bHasInfoOpen = false;
		if (infoPanel != null)
		{
			bHasInfoOpen = true;
		}
		
		//JoglVectord2 mouseScreenPos = new JoglVectord2(e.getX() / gui.getWidth(), e.getY() / gui.getHeight());
		JoglVectord2 mouseScreenPos = e.getPoint();
		JoglCamera cam = getCamera();

		//JoglVectord3 camPos = scene.cam.GetPosition();		
		//JoglRay r = new JoglRay(camPos, scene.cam.GetRayDir(mouseScreenPos, gui.getWidth(), gui.getHeight()));
		JoglRay r = cam.GetRay(mouseScreenPos);
		
		Object hitObject = GetClosestHitObject(r);
		
		UpdateSelection(e, hitObject);

		if (hitObject instanceof JSODataVisMagnetLineBase)
		{
			JSODataVisMagnetLineBase line = (JSODataVisMagnetLineBase)hitObject;
			//PSENWaitingTimeTreeBranch transitionPair = pwtt.getTransitionPair(line.parentRef);
			//String infoTitle = pnetIdx.getTransitionPairLabel(transitionPair);
			
			//System.out.println("Hit MagnetLine: " + infoTitle + " (" + line.cohortRef + ")");
			System.out.println("Hit MagnetLine: " + line.getLabel());
			
			FocusCamOn(line);

			if (bHasInfoOpen)
			{
				OpenInfoPanel(line);
			}
		}
		else if (hitObject instanceof JSOPnTransition)
		{
			JSOPnTransition jso = (JSOPnTransition)hitObject;
			System.out.println("Hit transition: " + jso.getLabel());
			FocusCamOn(jso);

			if (bHasInfoOpen)
			{
				OpenInfoPanel(jso);
			}
		}
		else if (hitObject instanceof JSOPnPlace)
		{
			JSOPnPlace jso = (JSOPnPlace)hitObject;
			System.out.println("Hit place: ");
			FocusCamOn(jso);
		}
		else if (hitObject instanceof JSOPnEdge)
		{
			JSOPnEdge edge = (JSOPnEdge)hitObject;
			FocusCamOn(edge);
			if (pwtt.getTransitions().contains(edge.transitionPairID))
			{
				if (bHasInfoOpen)
				{
					OpenInfoPanel(edge);
				}
			}
		}
		else if (hitObject instanceof JSOPnEdgeVertex)
		{
			System.out.println("Hit Vertex: ");
			bUpdateScene = true;
		}
		
		if (hitObject == null)
		{
			FocusCamOn(pnet);
			CloseExistingInfoPanel();
		}
	}

	protected void FocusCamOn(JSODataVisMagnetLineBase line) 
	{
		JICameraOrbit jcio = iman.findInteraction(JICameraOrbit.class);	
		if (jcio != null)
		{
//			JoglVectord3 dir = JoglVectord3.Subtract(line.getEndPoint(), line.getStartPoint());
//			JoglVectord3 center = JoglVectord3.Add(line.getStartPoint(), JoglVectord3.Multiply(dir, 0.5));
//			jcio.setCamFocus(JoglVectord3.Add(center, line.getJSOPnet().getOffset()));
			jcio.setCamFocus(line.getCamFocusPoint());
			bUpdateScene = true;
		}
	}

	public void CloseExistingInfoPanel() 
	{
		if (infoPanel != null)
		{
			iman.removeInteraction(infoPanel);
			infoPanel.Close();
			infoPanel = null;
			
			bUpdateScene = true;
		}
	}
	
	public void OpenInfoPanel(JSOPnTransition jso)
	{
		CloseExistingInfoPanel();
		
		infoPanel = new JIPSENTransitionInfo(iman, gui, jso, psnet);
		
		//set visible label categories
		HashSet<String> visibleCats = new HashSet<String>();
		JIPSENTransVisDriver jtvd = iman.findInteraction(JIPSENTransVisDriver.class);
		if (jtvd != null)
		{
			DetermineVisibleCategoriesTrans(visibleCats, jtvd);
		}
		infoPanel.setVisibleCats(visibleCats);
		
		JIPSENFilter filter = iman.findInteraction(JIPSENFilter.class);
		if (filter != null)
		{
			infoPanel.setVisibleCohorts(filter.getCheckedClassifiers());
		}
		
		iman.addInteraction(infoPanel);
		
		bUpdateScene = true;
	}

	private void DetermineVisibleCategoriesTrans(HashSet<String> visibleCats, JIPSENTransVisDriver jtvd)
	{
		if (jtvd.GetTransitionVisMethodName().equals("None"))
		{
			visibleCats.addAll(jtvd.GetLabelNames());
		}
		else
		{
			if (jtvd.HasLabel1())
			{
				visibleCats.add(jtvd.GetLabel1Name());
			}
			if (jtvd.HasLabel2())
			{
				visibleCats.add(jtvd.GetLabel2Name());
			}
			if (!visibleCats.contains("aligned_frequency"))
			{
				visibleCats.add("aligned_frequency");
			}
		}
	}
	
	public void OpenInfoPanel(JSOPnEdge edge)
	{
		CloseExistingInfoPanel();
		
		//WaitingTimeTree tree = pwtt.getTransitionPair(edge.transitionPairID);
		if (edge.transitionPairID != null)
		{
			String infoTitle = pnetIdx.getTransitionPairLabel(edge.transitionPairID);

			//set visible label categories
			HashSet<String> visibleCats = new HashSet<String>();
			JIPSENEdgeVisDriver jtvd = iman.findInteraction(JIPSENEdgeVisDriver.class);
			if (jtvd != null)
			{
				DetermineVisibleCategoriesEdge(visibleCats, jtvd);

				if (jtvd.GetEdgeVisMethodName().equals("Quality Lines"))
				{
					infoPanel = new JIPSENEdgeInfo(iman, gui, edge, infoTitle, pq);			
				}
			}
			
			if (infoPanel == null)
			{
				infoPanel = new JIPSENEdgeInfo(iman, gui, edge, infoTitle, pwtt);
			}
			infoPanel.setVisibleCats(visibleCats);

			JIPSENFilter filter = iman.findInteraction(JIPSENFilter.class);
			if (filter != null)
			{
				infoPanel.setVisibleCohorts(filter.getCheckedClassifiers());
			}
			
			iman.addInteraction(infoPanel);
		}
		bUpdateScene = true;
	}
	
	public void OpenInfoPanel(JSODataVisMagnetLineBase closestLine)
	{
		CloseExistingInfoPanel();
		
		String infoTitle = closestLine.getLabel();
		infoTitle = infoTitle.replace(" (" + closestLine.getCohortRef() + ")", ""); //remove cohort reference from info title
		
		//set visible label categories
		HashSet<String> visibleCats = new HashSet<String>();
		JIPSENEdgeVisDriver jtvd = iman.findInteraction(JIPSENEdgeVisDriver.class);
		if (jtvd != null)
		{
			DetermineVisibleCategoriesEdge(visibleCats, jtvd);

			if (jtvd.GetEdgeVisMethodName().equals("Quality Lines"))
			{
				infoPanel = new JIPSENEdgeInfo(iman, gui, closestLine, infoTitle, pq);			
			}
		}

		if (infoPanel == null)
		{
			infoPanel = new JIPSENEdgeInfo(iman, gui, closestLine, infoTitle, pwtt);
		}
		
		System.out.println("Opening info for: " + closestLine.getLabel() + "(" + closestLine.getParentRef() + ")");
		
		infoPanel.setVisibleCats(visibleCats);
		
		JIPSENFilter filter = iman.findInteraction(JIPSENFilter.class);
		if (filter != null)
		{
			infoPanel.setVisibleCohorts(filter.getCheckedClassifiers());
		}
		
		iman.addInteraction(infoPanel);
		
		bUpdateScene = true;
	}

	private void DetermineVisibleCategoriesEdge(HashSet<String> visibleCats, JIPSENEdgeVisDriver jtvd)
	{
		if (jtvd.GetEdgeVisMethodName().equals("None"))
		{
			visibleCats.addAll(jtvd.GetLabelNames());
		}
		else
		{
			if (jtvd.HasLabel1())
			{
				visibleCats.add(jtvd.GetLabel1Name());
			}
			if (!visibleCats.contains("aligned_frequency"))
			{
				visibleCats.add("aligned_frequency");
			}
		}
	}
	
	@Override
	protected Object GetClosestHitObject(JoglRay r)
	{
		Object hitObject = null;
		double closestDist = Double.POSITIVE_INFINITY;

		//pick all subnets which are hit by the ray first to avoid recalculating again and again
		Vector<JSOSubNet> hitSubnets = new Vector<JSOSubNet>();
		for (JSOSubNet snet : pnet.getAllSubNets())
		{
			if (!snet.getIsVisible())
				continue;

			if (snet.getAABB().Intersects(r))
			{
				hitSubnets.add(snet);
			}
		}

		//get magnet lines hit by ray
		for (JSOPSENEdgeVisualizationLayer edgevis : scene.findSceneObjects(JSOPSENEdgeVisualizationLayer.class))
		{
			if (!edgevis.getSubNet().getIsVisible())
				continue;
			
			JoglAABB aabb = edgevis.getAABB().Translate(edgevis.getSubNet().relativeToAbsolute(JoglVectord3.ZERO));
			if (aabb.Intersects(r))
			{
				HitData hdata = GetClosestMagnetLine(r, edgevis, 5.0);
				IDataVisMagnetLine closestLine = (IDataVisMagnetLine)hdata.getHitObject();
				double dist = hdata.getHitDistance();
				
				if (closestLine != null && dist < closestDist)
				{
					hitObject = closestLine;
					closestDist = dist;
				}
			}
		}

		//get transitions hit by ray
		for (JSOSubNet snet : hitSubnets)
		{
			JoglVectord3 offset = snet.relativeToAbsolute(JoglVectord3.ZERO);
			for (JSOPnTransition jso : snet.getTransitions())
			{
				JoglAABB aabb = jso.getAABB().Translate(offset);
				if (aabb.Intersects(r))
				{
					double dist = aabb.GetIntersectionDist(r);
					if (dist < closestDist)
					{
						hitObject = jso;
						closestDist = dist;
					}
				}
			}
		}

		//get places hit by ray
		for (JSOSubNet snet : hitSubnets)
		{
			JoglVectord3 offset = snet.relativeToAbsolute(JoglVectord3.ZERO);
			for (JSOPnPlace jso : snet.getPlaces())
			{
				JoglAABB aabb = jso.getAABB().Translate(offset);
				if (aabb.Intersects(r))
				{
					double dist = aabb.GetIntersectionDist(r);
					if (dist < closestDist)
					{
						hitObject = jso;
						closestDist = dist;
					}
				}
			}
		}
		
		//get vertices hit by ray
		JIMoveModelElement jmt = iman.findInteraction(JIMoveModelElement.class);
		if (jmt != null && jmt.IsEnabled())
		{
			for (JSOSubNet snet : hitSubnets)
			{
				JoglVectord3 v = snet.relativeToAbsolute(JoglVectord3.ZERO);
				for (JSOPnEdgeVertex vtx : snet.getEdgeVertices())
				{
					JoglAABB aabb = vtx.getAABB().Translate(v);
					if (aabb.Intersects(r))
					{
						double dist = aabb.GetIntersectionDist(r);
						if (dist < closestDist)
						{
							hitObject = vtx;
							closestDist = dist;
						}
					}
				}
			}
		}

		//get edges hit by ray
		HitData hdata = GetClosestEdge(hitSubnets, r, 5.0);
		JSOPnEdge edge = (JSOPnEdge)hdata.getHitObject();
		if (edge != null && hdata.getHitDistance() < closestDist)
		{
			hitObject = edge;
			closestDist = hdata.getHitDistance();
		}
		
		return hitObject;
	}

	private HitData GetClosestMagnetLine(JoglRay r, JSOPSENEdgeVisualizationLayer edgevis,
			double mDist) 
	{
		JSOSubNet snet = edgevis.getSubNet();
		JoglVectord3 offset = snet.relativeToAbsolute(JoglVectord3.ZERO);
		
		Vector<String> selectedRefs = new Vector<String>();
		for (JoglSceneObject jso : selection.getElements())
		{
			if (jso instanceof JSOPnTransition)
			{
				String ref = ((JSOPnTransition)jso).orgTransition.getLocalID().toString();
				selectedRefs.add(ref);
			}
		}
		
		IDataVisMagnetLine closestLine = null;
		double closestDistFromRay = mDist;
		double closestDistOnRay = Double.POSITIVE_INFINITY;
		
		Vector<JSODataVisMagnetLineGroupBase> groups = edgevis.getMagnetLineGroups();
		for (JSODataVisMagnetLineGroupBase group : groups)
		{
			if (selection.isEmpty() || selectedRefs.contains(group.getSourceRef()) || selectedRefs.contains(group.getTargetRef()))
			{
				JoglAABB aabb2 = group.getAABB().Translate(offset);
				if (aabb2.Intersects(r))
				{
					Vector<JSODataVisMagnetLineBase> lines = group.getMagnetLines();
					for (JSODataVisMagnetLineBase line : lines)
					{
						JoglAABB aabb = line.getAABB().Translate(offset);
						if (aabb.Intersects(r))
						{
							double minDistFromRay = Double.POSITIVE_INFINITY;
							double minDistOnRay = Double.POSITIVE_INFINITY;
							
							Vector<JoglVectord3> pts = line.getPoints();
							if (pts.size() > 0)
							{
								JoglVectord3 end = JoglVectord3.Add(pts.get(0), offset);
								for (int i = 1; i < pts.size(); i++)
								{
									JoglVectord3 start = end;
									end = JoglVectord3.Add(pts.get(i), offset);
									double distFromRay = GetClosestDistFromLine(start, end, r);
									double distOnRay = GetClosestPointFromLine(start, end, r); //distance from ray origin to closest point on ray
									
									//mindist = Math.min(mindist, dist);
									if (distFromRay < minDistFromRay)
									{
										minDistFromRay = distFromRay;
										minDistOnRay = distOnRay;
									}
								}
								
								//if (mindist < closestDist)
								if (minDistFromRay < mDist && ((minDistFromRay * 20) + minDistOnRay) < ((closestDistFromRay * 20) + closestDistOnRay))
								{
									//System.out.println("Maxdist: " + maxdist);
									closestDistFromRay = minDistFromRay;
									closestLine = line;
									closestDistOnRay = minDistOnRay;
								}
							}			
						}
					}
				}
			}
		}
		return new HitData(closestLine, ((closestDistFromRay * 20) + closestDistOnRay));
	}
}
