package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.HashMap;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.compviz.SignificanceProviderEdge;
import org.processmining.processprofiler3d.compviz.SignificanceProviderTransition;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.JoglSelection;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiTextbox;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENEdgeVisualizationLayer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisMagnetLineBase;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglModelScene;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglPSENVisualizationScene;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIPSENSignificance extends JIOptionsPanel implements ITextboxInputListener
{
	private JoglInteractionManager iman;
	private JoglScene scene;
	private PetriNetDataSource pnet;
	private IPerformanceStatisticsEnrichedNet psnet;
	private IPerformanceStatisticsEnrichedNet pwait;
	//private JIViewManager viewManager;

	private boolean bVisible = false;
	
	private JoglGuiLabel btnShowSigTrans;
	private JoglGuiLabel btnShowSigEdge;
	private JoglGuiTextbox tbxSigLevel;

	private JoglGuiLabel lblShowSigTrans;
	private JoglGuiLabel lblShowSigEdge;
	private JoglGuiLabel lblSigLevel;

	private JoglGuiLabel highlighted = null;
	
	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();
	
	public double plevel = 0.01;
	
	public JIPSENSignificance(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui, PetriNetDataSource pnet, IPerformanceStatisticsEnrichedNet psnet, IPerformanceStatisticsEnrichedNet pwait)
	{
		super(gui);
		
		this.iman = iman;
		this.scene = scene;
		this.pnet = pnet;
		this.psnet = psnet;
		this.pwait = pwait;
		//this.viewManager = scene.findInteraction(JIViewManager.class);
	}
	
	@Override
	public boolean isVisible()
	{
		return bVisible;
	}
	
	@Override
	public void Show()
	{
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_LEFT;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff2 = new FontFormat();
		ff2.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff2.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff2.align = FontFormat.ALIGN_RIGHT;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;

		float colLabelWidth = 0.2f * (float)JoglGuiStyle.textScale;
		float colButtonWidth = 0.2f * (float)JoglGuiStyle.textScale;
		
		float colButtonX = 0.95f - colButtonWidth;
		float colLabelX = colButtonX - colLabelWidth - 0.005f;
		
		float yoff = 0.85f;

		btnShowSigTrans = new JoglGuiLabel(ff.clone());
		btnShowSigTrans.setCanTouch(true);
		btnShowSigTrans.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		btnShowSigTrans.setText("Select");
		btnShowSigTrans.setPadding(textPadding);
		btnShowSigTrans.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnShowSigTrans);
		controls.add(btnShowSigTrans);

		lblShowSigTrans = new JoglGuiLabel(ff2.clone());
		lblShowSigTrans.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblShowSigTrans.setText("Significant Transitions");
		lblShowSigTrans.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblShowSigTrans);
		controls.add(lblShowSigTrans);

		yoff -= 0.1f;
		
		btnShowSigEdge = new JoglGuiLabel(ff.clone());
		btnShowSigEdge.setCanTouch(true);
		btnShowSigEdge.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		btnShowSigEdge.setText("Select");
		btnShowSigEdge.setPadding(textPadding);
		btnShowSigEdge.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnShowSigEdge);
		controls.add(btnShowSigEdge);

		lblShowSigEdge = new JoglGuiLabel(ff2.clone());
		lblShowSigEdge.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblShowSigEdge.setText("Significant Arcs");
		lblShowSigEdge.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblShowSigEdge);
		controls.add(lblShowSigEdge);
		
		yoff -= 0.1f;
		
		tbxSigLevel = new JoglGuiTextbox(ff.clone());
		tbxSigLevel.setCanTouch(true);
		tbxSigLevel.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		tbxSigLevel.setText("" + plevel);
		tbxSigLevel.setPadding(textPadding);
		tbxSigLevel.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(tbxSigLevel);
		controls.add(tbxSigLevel);
		
		lblSigLevel = new JoglGuiLabel(ff2.clone());
		lblSigLevel.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblSigLevel.setText("p-level");
		lblSigLevel.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblSigLevel);
		controls.add(lblSigLevel);
			
		bVisible = true;
	}
	
	@Override
	public void Hide()
	{
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
		
		bVisible = false;
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (!bVisible)
		{
			//Init();
		}
		else
		{
			//camModeToggleButton.setText(viewManager.getCameraModelName());
		}
		
		bUpdateScene = false;
	}
	
	//MouseListener method implementation
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {		
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnShowSigTrans)
			{
				SelectSignificantTrans();
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == btnShowSigEdge)
			{
				SelectSignificantEdge();
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == tbxSigLevel)
			{
				//viewManager.SwitchGrid();
				JITextboxInput jibt = new JITextboxInputNumeric(iman, scene, gui, tbxSigLevel);
				jibt.listener = this;
				iman.addInteraction(jibt);
				
				bUpdateScene = true;
	        	return true;
			}
		}
		
		return false;
    }
	
	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnShowSigTrans)
			{
				highlight(btnShowSigTrans);
			}
			else if (jgo == btnShowSigEdge)
			{
				highlight(btnShowSigEdge);
			}
			else if (jgo == tbxSigLevel)
			{
				highlight(tbxSigLevel);
			}
			else
			{
				unHighlight();
			}
		}
		else
		{
			unHighlight();
		}
	}
	
	private void highlight(JoglGuiLabel label)
	{
		unHighlight();
	
		label.setBackColor(JoglGuiStyle.highlightColor);
		highlighted = label;
		
		bUpdateScene = true;
	}

	private void unHighlight() 
	{
		if (highlighted != null)
		{
			highlighted.setBackColor(JoglGuiStyle.backColor);
			highlighted = null;

			bUpdateScene = true;
		}
	}

	public void OnTextInputFinished(JITextboxInput jti) 
	{
		String txt = jti.getText();
		double val = Double.parseDouble(txt);
		
		RescaleGuiText(val);
	}
	
	public void RescaleGuiText(double val)
	{	
		plevel = val;
	}
	
	public void SelectSignificantTrans()
	{
		JIPSENTransVisDriver jtvd = iman.findInteraction(JIPSENTransVisDriver.class);
		
		SignificanceProviderTransition spt = new SignificanceProviderTransition(pnet, psnet);
		spt.pLevel = plevel;
		spt.setVisibleClassifiers(jtvd.getVisibleClassifiers());
		
		HashMap<String, Boolean> map = null;
		if (jtvd.HasLabel2())
		{
			map = spt.getMap(jtvd.GetLabel2Name());
		}
		else if (jtvd.HasLabel1())
		{
			map = spt.getMap(jtvd.GetLabel1Name());
		}

		JISelectModelElement sme = iman.findInteraction(JISelectModelElement.class);
		JoglSelection sl = sme.selection;
		sl.clear();
		
		if (map != null)
		{
			Vector<JSOPnTransition> lst = new Vector<JSOPnTransition>();
			if (scene instanceof JoglModelScene)
			{
				JoglModelScene jms = (JoglModelScene)scene;
				lst = jms.getPetriNet().getTransitions();
			}
	
			for (JSOPnTransition t : lst)
			{
				String str = t.orgTransition.getLocalID().toString();
				if (map.containsKey(str) && map.get(str))
				{
					sl.add(t);
				}
			}
		}
	}
	
	public void SelectSignificantEdge()
	{
		JIPSENEdgeVisDriver jtvd = iman.findInteraction(JIPSENEdgeVisDriver.class);
		
		SignificanceProviderEdge spt = new SignificanceProviderEdge(pnet, pwait);
		spt.pLevel = plevel;
		spt.setVisibleClassifiers(jtvd.getVisibleClassifiers());
		
		HashMap<String, Boolean> map = null;
		if (jtvd.HasLabel1())
		{
			map = spt.getMap(jtvd.GetLabel1Name());
		}

		JISelectModelElement sme = iman.findInteraction(JISelectModelElement.class);
		JoglSelection sl = sme.selection;
		sl.clear();
		
		if (map != null)
		{
			Vector<JSODataVisMagnetLineBase> lst = new Vector<JSODataVisMagnetLineBase>();
			if (scene instanceof JoglPSENVisualizationScene)
			{
				JoglPSENVisualizationScene jms = (JoglPSENVisualizationScene)scene;
				for (JSOSubNet jsn : jms.getPetriNet().getAllSubNets())
				{
					JSOPSENEdgeVisualizationLayer jev = jms.getEdgeVisualizer(jsn);
					lst.addAll(jev.getMagnetLines());
				}
			}
	
			for (JSODataVisMagnetLineBase t : lst)
			{
				String str = t.getParentRef();
				if (map.containsKey(str) && map.get(str))
				{
					sl.add(t);
				}
			}
		}
	}
}
