package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.HashSet;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.DataMapNormalization;
import org.processmining.processprofiler3d.compviz.DataMapProviderTransition;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.compviz.TransitionDataMapDouble;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENTransitionVisualizationLayer;
import org.processmining.processprofiler3d.compviz.methods.TransitionDataVisualisationMethodCollection;
import org.processmining.processprofiler3d.compviz.methods.TransitionDataVisualizationMethod;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIPSENTransVisDriver extends JoglInteraction
{
	private JoglScene scene;
	
	private TransitionDataVisualizationMethod transitionVisMethod = TransitionDataVisualisationMethodCollection.TDVM_None;
	private int label1Id = 0;
	private int label2Id = 0;
	private String label1Name;
	private String label2Name;
	private DataMapNormalization normMode1;
	private DataMapNormalization normMode2;
	private String doubleCodeMode = "None";
	
	private HashSet<String> visClassifiers;
	
	private DataMapProviderTransition mapProvider;
	private JIPSENTransVisInterpolate interpolator = null;

	public HashSet<String> getVisibleClassifiers()
	{
		return visClassifiers;
	}
	
	public void setVisibleClassifiers(HashSet<String> vc)
	{
		visClassifiers = vc;
		mapProvider.setVisibleClassifiers(vc);
		SetVisualization();
	}
	
	public String getScaleMode1()
	{
		for (JSOPSENTransitionVisualizationLayer vis : scene.findSceneObjects(JSOPSENTransitionVisualizationLayer.class))
		{
			return vis.getScaleMode1();
		}
		
		return "Linear";
	}
	
	public void setScaleMode1(String mode)
	{
		for (JSOPSENTransitionVisualizationLayer vis : scene.findSceneObjects(JSOPSENTransitionVisualizationLayer.class))
		{
			vis.setScaleMode1(mode);
		}
	}
	
	public String getScaleMode2()
	{
		for (JSOPSENTransitionVisualizationLayer vis : scene.findSceneObjects(JSOPSENTransitionVisualizationLayer.class))
		{
			return vis.getScaleMode2();
		}
		
		return "Linear";
	}
	
	public void setScaleMode2(String mode)
	{
		for (JSOPSENTransitionVisualizationLayer vis : scene.findSceneObjects(JSOPSENTransitionVisualizationLayer.class))
		{
			vis.setScaleMode2(mode);
		}
	}
	
	public JIPSENTransVisDriver(JoglScene scene, PetriNetDataSource pnet, DataMapProviderTransition mp)
	{
		this.scene = scene;

		//make all available classifiers visible at first
		visClassifiers = new HashSet<String>(GetClassifierNames());
		
		mapProvider = mp;
	}
	
	
	public void Dispose()
	{
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		bUpdateScene = false;
	}
	
	public void setTransitionVisMethod(String methodName)
	{
		transitionVisMethod = TransitionDataVisualisationMethodCollection.get(methodName);
		updateNormMode();
		SetVisualization();
	}

	public void updateNormMode() 
	{
		normMode1 = transitionVisMethod.getValidNormalizationMode(1, normMode1);
		normMode2 = transitionVisMethod.getValidNormalizationMode(2, normMode2);
	}
	
	public void setLabel1(String labelName)
	{
		label1Id = GetLabelNames().indexOf(labelName);
		SetVisualization();
	}
	
	public void setLabel2(String labelName)
	{
		label2Id = GetLabelNames().indexOf(labelName);
		SetVisualization();
	}
	
	public String getNormalizationMode1()
	{
		return normMode1.getModeName();
	}
	
	public String getNormalizationMode2()
	{
		return normMode2.getModeName();
	}
	
	public void setNormalizationMode1(String mode)
	{
		normMode1 = DataMapNormalization.getModeByName(mode);
		SetVisualization();
	}
	
	public void setNormalizationMode2(String mode)
	{
		normMode2 = DataMapNormalization.getModeByName(mode);
		SetVisualization();
	}
	
	public String getDoubleCodeMode()
	{
		return doubleCodeMode;
	}
	
	public void setDoubleCodeMode(String mode)
	{
		doubleCodeMode = mode;
		for (JSOPSENTransitionVisualizationLayer soPsenT : scene.findSceneObjects(JSOPSENTransitionVisualizationLayer.class))
		{
			soPsenT.doubleCodeMode = doubleCodeMode;
		}
	}
	
	public String GetTransitionVisMethodName()
	{
		return transitionVisMethod.GetName();
	}
	
	public Vector<String> GetLabelNames()
	{
		return transitionVisMethod.GetLabelNames();
	}
	
	public Vector<String> GetNormalizationNames1()
	{
		return transitionVisMethod.GetNormalizationNames(1);
	}
	
	public Vector<String> GetNormalizationNames2()
	{
		return transitionVisMethod.GetNormalizationNames(2);
	}
	
	public Vector<String> GetDoubleCodeNames()
	{
		Vector<String> lst = new Vector<String>();

		lst.add("None");
		lst.add("Stripes");
		
		return lst;
	}
	
	public Vector<String> GetClassifierNames()
	{
		return transitionVisMethod.GetClassifierNames();
	}
	
	public String GetLabel1Name()
	{
		Vector<String> lst = GetLabelNames();

		if (lst.size() > label1Id)
		{
			return lst.get(label1Id);
		}
		return "Error!";
	}
	
	public String GetLabel2Name()
	{
		Vector<String> lst = GetLabelNames();

		if (lst.size() > label2Id)
		{
			return lst.get(label2Id);
		}
		return "Error!";
	}
	
	public boolean HasLabel1()
	{
		return transitionVisMethod.HasVariable1();
	}
	
	public boolean HasLabel2()
	{
		return transitionVisMethod.HasVariable2();
	}
	
	public void SetVisualization()
	{
		label1Name = GetLabel1Name();
		label2Name = GetLabel2Name();
		
		//System.out.println("TDriver set visualization: " + label1Name + "/" + label2Name);
		
		TransitionDataMapDouble map1 = transitionVisMethod.GenerateDataMap(1, label1Name, normMode1, null);
		TransitionDataMapDouble map2 = transitionVisMethod.GenerateDataMap(2, label2Name, normMode2, null);
		
		for (JSOPSENTransitionVisualizationLayer soPsenT : scene.findSceneObjects(JSOPSENTransitionVisualizationLayer.class))
		{
			soPsenT.SetVisualization(transitionVisMethod, map1, map2);
			soPsenT.doubleCodeMode = doubleCodeMode;
			soPsenT.BuildScene();
		}

//		if (interpolator == null)
//		{
//			interpolator = new JIPSENTransVisInterpolate(soPsenT);
//			interpolator.SetDataMap(transitionVisMethod, map1, map1, map2, map2);
//			
//			gui.panel.addInteraction(interpolator);
//		}
//		else
//		{
//			interpolator.SetDataMap(transitionVisMethod, map1, interpolator.getCurrentMap1(), map2, interpolator.getCurrentMap2());
//			//interpolator.SetDataMap(transitionVisMethod, map1, map1, map2, map2);
//		}
		
		bUpdateScene = true;
	}

}
