package org.processmining.processprofiler3d.compviz.jogl.interactions;

import org.processmining.processprofiler3d.compviz.TransitionDataMapDouble;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENTransitionVisualizationLayer;
import org.processmining.processprofiler3d.compviz.methods.TransitionDataVisualizationMethod;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIPSENTransVisInterpolate extends JoglInteraction
{
	private JSOPSENTransitionVisualizationLayer soPsenT;
	
	private TransitionDataVisualizationMethod transitionVisMethod;
	private TransitionDataMapDouble map1;
	private TransitionDataMapDouble map1Old;
	private TransitionDataMapDouble map2;
	private TransitionDataMapDouble map2Old;
	private TransitionDataMapDouble currentMap1;
	private TransitionDataMapDouble currentMap2;
	
	private float factor = 0.0f;
	private float factorStepSize = 0.02f;
	
	public TransitionDataMapDouble getCurrentMap1()
	{
		return currentMap1;
	}
	
	public TransitionDataMapDouble getCurrentMap2()
	{
		return currentMap2;
	}
	
	public JIPSENTransVisInterpolate(JSOPSENTransitionVisualizationLayer soPsenT)
	{
		this.soPsenT = soPsenT;
	}	
	
	public void Dispose()
	{
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (factor <= 1.0f)
		{
			UpdateVis();
			factor += factorStepSize;
		}
		else
		{
			bUpdateScene = false;
		}
	}
	
	public void SetDataMap(TransitionDataVisualizationMethod mode, TransitionDataMapDouble m1, TransitionDataMapDouble m1o, TransitionDataMapDouble m2, TransitionDataMapDouble m2o)
	{
		if (mode == transitionVisMethod)
		{
			map1 = m1;
			map1Old = m1o;
			map2 = m2;
			map2Old = m2o;
			currentMap1 = m1o;
			currentMap2 = m2o;
	
			soPsenT.SetVisualization(transitionVisMethod, map1Old, map2Old);
			soPsenT.BuildScene();
			
			factor = 0.0f;
			bUpdateScene = true;
		}
		else
		{
			transitionVisMethod = mode;
			
			map1 = m1;
			map1Old = m1o;
			map2 = m2;
			map2Old = m2o;
			currentMap1 = m1;
			currentMap2 = m2;
	
			soPsenT.SetVisualization(transitionVisMethod, map1, map2);
			soPsenT.BuildScene();
			
			factor = 1.0f;
			bUpdateScene = true;
		}
	}
	
	public void UpdateVis()
	{ 
		if (factor == 0.0)
		{
			soPsenT.SetVisualization(transitionVisMethod, map1Old, map2Old);
		}
		else if (factor >= 1.0)
		{
			soPsenT.SetVisualization(transitionVisMethod, map1, map2);
		}
		else
		{
			if (map1Old != null && map1 != null)
			{
				currentMap1 = TransitionDataMapDouble.interpolate(map1Old, map1, factor);
			}
	
			if (map2Old != null && map2 != null)
			{
				currentMap2 = TransitionDataMapDouble.interpolate(map2Old, map2, factor);
			}
	
			soPsenT.SetVisualization(transitionVisMethod, currentMap1, currentMap2);
		}
		soPsenT.BuildScene();
		
		bUpdateScene = true;
	}

}
