package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.List;
import java.util.Vector;
import java.util.prefs.Preferences;

import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiTextbox;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOGrid;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.interactions.JICameraBase;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIPSENView extends JIOptionsPanel implements ITextboxInputListener, IDropDownListener
{
	private JoglInteractionManager iman;
	private JoglScene scene;
	private JIViewManagerModel viewManager;

	private boolean bVisible = false;
	
	private JoglGuiLabel btnCamModeToggle;
	private JoglGuiLabel btnCamReset;
	private JoglGuiLabel btnFloatLabelModeToggle;
	private JoglGuiLabel btnGridToggle;
	//private JoglGuiLabel levelToggleButton;
	private JoglGuiTextbox tbxTextScale;
	private JoglGuiLabel tbxProjection;
	private JoglGuiLabel btnVRToggle;

	private JoglGuiLabel lblCamMode;
	private JoglGuiLabel lblFloatingLabelState;
	private JoglGuiLabel lblGridState;
	//private JoglGuiLabel lblSimplificationLevel;
	private JoglGuiLabel lblTextScale;
	private JoglGuiLabel lblProjection;
	private JoglGuiLabel lblVRToggle;

	private JoglGuiLabel highlighted = null;
	
	private JIDropdownBox ddbox = null;
	
	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();
	
	public JIPSENView(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui)
	{
		super(gui);
		
		this.iman = iman;
		this.scene = scene;
		this.viewManager = iman.findInteraction(JIViewManagerModel.class);
	}
	
	@Override
	public boolean isVisible()
	{
		return bVisible;
	}
	
	@Override
	public void Show()
	{
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_LEFT;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff2 = new FontFormat();
		ff2.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff2.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff2.align = FontFormat.ALIGN_RIGHT;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		float yoff = 0.85f;
		
		float colLabelWidth = 0.2f * (float)JoglGuiStyle.textScale;
		float colButtonWidth = 0.2f * (float)JoglGuiStyle.textScale;
		
		float colButtonX = 0.95f - colButtonWidth;
		float colLabelX = colButtonX - colLabelWidth - 0.005f;

		btnCamModeToggle = new JoglGuiLabel(ff.clone());
		btnCamModeToggle.setCanTouch(true);
		btnCamModeToggle.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		btnCamModeToggle.setText(viewManager.getCameraModelName());
		btnCamModeToggle.setPadding(textPadding);
		btnCamModeToggle.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnCamModeToggle);
		controls.add(btnCamModeToggle);

		lblCamMode = new JoglGuiLabel(ff2.clone());
		lblCamMode.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblCamMode.setText("Camera Mode");
		lblCamMode.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblCamMode);
		controls.add(lblCamMode);
		
		yoff -= 0.1f;

		btnCamReset = new JoglGuiLabel(ff.clone());
		btnCamReset.setCanTouch(true);
		btnCamReset.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		btnCamReset.setText("Reset Camera");
		btnCamReset.setPadding(textPadding);
		btnCamReset.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnCamReset);
		controls.add(btnCamReset);

		yoff -= 0.1f;
		
		tbxProjection = new JoglGuiLabel(ff.clone());
		tbxProjection.setCanTouch(true);
		tbxProjection.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		
		JICameraBase jic = iman.findInteraction(JICameraBase.class);
		if (jic != null)
		{
			if (jic.bUseOrtho)
			{
				tbxProjection.setText("Orthographic");
			}
			else
			{
				tbxProjection.setText("Perspective");
			}
		}
		tbxProjection.setPadding(textPadding);
		tbxProjection.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(tbxProjection);
		controls.add(tbxProjection);

		lblProjection = new JoglGuiLabel(ff2.clone());
		lblProjection.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblProjection.setText("Projection");
		lblProjection.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblProjection);
		controls.add(lblProjection);
		
		yoff -= 0.1f;
		
		btnFloatLabelModeToggle = new JoglGuiLabel(ff.clone());
		btnFloatLabelModeToggle.setCanTouch(true);
		btnFloatLabelModeToggle.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		btnFloatLabelModeToggle.setText("On");
		btnFloatLabelModeToggle.setPadding(textPadding);
		btnFloatLabelModeToggle.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnFloatLabelModeToggle);
		controls.add(btnFloatLabelModeToggle);
		
		lblFloatingLabelState = new JoglGuiLabel(ff2.clone());
		lblFloatingLabelState.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblFloatingLabelState.setText("Floating Labels");
		lblFloatingLabelState.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblFloatingLabelState);
		controls.add(lblFloatingLabelState);

		yoff -= 0.1f;
		
		JSOGrid grid = scene.findSceneObject(JSOGrid.class);
		if (grid != null)
		{
			btnGridToggle = new JoglGuiLabel(ff.clone());
			btnGridToggle.setCanTouch(true);
			btnGridToggle.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
			btnGridToggle.setText("On");
			btnGridToggle.setPadding(textPadding);
			btnGridToggle.setBackColor(JoglGuiStyle.backColor);
			
			gui.AddGuiObject(btnGridToggle);
			controls.add(btnGridToggle);
			
			lblGridState = new JoglGuiLabel(ff2.clone());
			lblGridState.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
			lblGridState.setText("Grid");
			lblGridState.setBackColor(JoglGuiStyle.transparentColor);
			
			gui.AddGuiObject(lblGridState);
			controls.add(lblGridState);
			
			yoff -= 0.1f;
		}

		tbxTextScale = new JoglGuiTextbox(ff.clone());
		tbxTextScale.setCanTouch(true);
		tbxTextScale.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		tbxTextScale.setText("" + JoglGuiStyle.textScale);
		tbxTextScale.setPadding(textPadding);
		tbxTextScale.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(tbxTextScale);
		controls.add(tbxTextScale);
		
		lblTextScale = new JoglGuiLabel(ff2.clone());
		lblTextScale.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblTextScale.setText("Text Scale");
		lblTextScale.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblTextScale);
		controls.add(lblTextScale);
		
		yoff -= 0.1f;
		
		JIOpenVR jvr = iman.findInteraction(JIOpenVR.class);

		if (jvr != null)
		{
			btnVRToggle = new JoglGuiTextbox(ff.clone());
			btnVRToggle.setCanTouch(true);
			btnVRToggle.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
			if (jvr.isVREnabled())
			{
				btnVRToggle.setText("Enabled");
			}
			else
			{
				btnVRToggle.setText("Disabled");
			}
			btnVRToggle.setPadding(textPadding);
			btnVRToggle.setBackColor(JoglGuiStyle.backColor);
			
			gui.AddGuiObject(btnVRToggle);
			controls.add(btnVRToggle);
			
			lblVRToggle = new JoglGuiLabel(ff2.clone());
			lblVRToggle.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
			lblVRToggle.setText("VR Mode");
			lblVRToggle.setBackColor(JoglGuiStyle.transparentColor);
			
			gui.AddGuiObject(lblVRToggle);
			controls.add(lblVRToggle);
			
			yoff -= 0.1f;
		}

//		levelToggleButton = new JoglGuiLabel(ff.clone());
//		levelToggleButton.setCanTouch(true);
//		levelToggleButton.setRect(0.75f, 0.55f, 0.2f, 0.05f);
//		levelToggleButton.setText("0");
//		levelToggleButton.setPadding(textPadding);
//		levelToggleButton.setBackColor(JoglGuiStyle.backColor);
//		
//		gui.AddGuiObject(levelToggleButton);
//		controls.add(levelToggleButton);
//		
//		lblSimplificationLevel = new JoglGuiLabel(ff2.clone());
//		lblSimplificationLevel.setRect(0.545f, 0.55f, 0.2f, 0.05f);
//		lblSimplificationLevel.setText("Simplification");
//		lblSimplificationLevel.setBackColor(JoglGuiStyle.transparentColor);
//		
//		gui.AddGuiObject(lblSimplificationLevel);
//		controls.add(lblSimplificationLevel);
		
		bVisible = true;
	}
	
	@Override
	public void Hide()
	{
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
		
		bVisible = false;
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (!bVisible)
		{
			//Init();
		}
		else
		{
			btnFloatLabelModeToggle.setText(viewManager.getFloatingLabelState());
			btnCamModeToggle.setText(viewManager.getCameraModelName());
			btnGridToggle.setText(viewManager.getGridState());

			JIOpenVR jvr = iman.findInteraction(JIOpenVR.class);
			if (jvr != null)
			{
				if (jvr.isVREnabled())
				{
					btnVRToggle.setText("Enabled");
				}
				else
				{
					btnVRToggle.setText("Disabled");
				}
			}
		}
		
		bUpdateScene = false;
	}
	
	//MouseListener method implementation
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {		
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnFloatLabelModeToggle)
			{
				viewManager.SwitchFloatingLabels();
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == btnCamModeToggle)
			{
				viewManager.SwitchCameraMode();
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == btnCamReset)
			{
				//Reset camera to default state
				viewManager.ResetCamera();
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == tbxProjection)
			{
				Vector<String> lst = new Vector<String>();
				lst.add("Perspective");
				lst.add("Orthographic");
				OpenDropDownBox(tbxProjection, lst);
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == btnGridToggle)
			{
				viewManager.SwitchGrid();
				
				bUpdateScene = true;
	        	return true;
			}
			else if (jgo == btnVRToggle)
			{
				JIOpenVR jvr = iman.findInteraction(JIOpenVR.class);
				if (jvr != null)
				{
					jvr.ToggleVR();
				}
				
				bUpdateScene = true;
	        	return true;
			}
			else if (jgo == tbxTextScale)
			{
				//viewManager.SwitchGrid();
				JITextboxInput jibt = new JITextboxInputNumeric(iman, scene, gui, tbxTextScale);
				jibt.listener = this;
				iman.addInteraction(jibt);
				
				bUpdateScene = true;
	        	return true;
			}
//			else if (jgo == levelToggleButton)
//			{
//				JSOPetriNet pn = scene.findSceneObject(JSOPetriNet.class);
//				int lvl = (pn.getSimplificationLevel() + 1) % PetrinetSimplifier.getMaxLevel();
//				pn.setSimplificationLevel(lvl);
//				pn.BuildScene();
//				levelToggleButton.setText("" + lvl);
//				
//				bUpdateScene = true;
//				return true;
//			}
		}
		
		return false;
    }
	
	private void OpenDropDownBox(JoglGuiLabel label, List<String> vals)
	{
		if (ddbox != null)
		{
			CloseDropDownBox();
		}

		if (!vals.isEmpty())
		{
			JoglVectord4 pos = label.getRect();
			ddbox = new JIDropdownBox(iman, gui, label, vals);
			ddbox.addDropDownListener(this);
			ddbox.setPosition((float)pos.getX(), (float)pos.getY() + 0.05f);
			ddbox.setWidth(0.2f * (float)JoglGuiStyle.textScale);
			ddbox.setTextSize(0.03f);
			iman.addInteraction(ddbox);
		}
		
    	bUpdateScene = true;	
	}

	private void CloseDropDownBox()
	{
		//close dropdown
		iman.removeInteraction(ddbox);
		ddbox.Dispose();
		ddbox = null;
	}
	
	public void OnDropDownSelected(JIDropdownBox ddb, String val) 
	{
		//System.out.println("OnDropDownSelected!");
		if (ddbox == ddb)
		{
			if (val != null)
			{
				if (ddb.getOwner() == tbxProjection)
				{
					JICameraBase jic = iman.findInteraction(JICameraBase.class);
					if (jic != null)
					{
						if (val.equals("Perspective"))
						{
							jic.bUseOrtho = false;
							jic.UpdateCameraLens(gui.getAspect());
						}
						else if (val.equals("Orthographic"))
						{
							jic.bUseOrtho = true;
							jic.UpdateCameraLens(gui.getAspect());
						}
					}
					tbxProjection.setText(val);
				}
			}
			
			CloseDropDownBox();
			
			Hide();
			Show();
			
			bUpdateScene = true;
		}
	}
	
	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnFloatLabelModeToggle)
			{
				highlight(btnFloatLabelModeToggle);
			}
			else if (jgo == btnCamModeToggle)
			{
				highlight(btnCamModeToggle);
			}
			else if (jgo == btnCamReset)
			{
				highlight(btnCamReset);
			}
			else if (jgo == btnGridToggle)
			{
				highlight(btnGridToggle);
			}
			else if (jgo == btnVRToggle)
			{
				highlight(btnVRToggle);
			}
			else if (jgo == tbxTextScale)
			{
				highlight(tbxTextScale);
			}
			else if (jgo == tbxProjection)
			{
				highlight(tbxProjection);
			}
//			else if (jgo == levelToggleButton)
//			{
//				highlight(levelToggleButton);
//			}
			else
			{
				unHighlight();
			}
		}
		else
		{
			unHighlight();
		}
	}
	
	private void highlight(JoglGuiLabel label)
	{
		unHighlight();
	
		label.setBackColor(JoglGuiStyle.highlightColor);
		highlighted = label;
		
		bUpdateScene = true;
	}

	private void unHighlight() 
	{
		if (highlighted != null)
		{
			highlighted.setBackColor(JoglGuiStyle.backColor);
			highlighted = null;

			bUpdateScene = true;
		}
	}

	public void OnTextInputFinished(JITextboxInput jti) 
	{
		String txt = jti.getText();
		double val = Double.parseDouble(txt);

		Preferences prefs = Preferences.userNodeForPackage(this.getClass());
		prefs.put("TextSize", txt);
		
		RescaleGuiText(val);
	}
	
	public void RescaleGuiText(double val)
	{	
		JoglGuiStyle.textScale = val;
		
		//update all interactions with gui text
		JIPSENFilter jipf = iman.findInteraction(JIPSENFilter.class);
		if (jipf != null)
		{
			jipf.UpdateFilters();
		}

		JIPSENBaseInfo jibi = iman.findInteraction(JIPSENBaseInfo.class);
		if (jibi != null)
		{
			jibi.UpdateBaseInfo(scene);
		}
		
		JIOptionsPanelManager jiopm = iman.findInteraction(JIOptionsPanelManager.class);
		if (jiopm != null)
		{
			jiopm.UpdateOptionsPanel();
		}
		
		JISelection jis = iman.findInteraction(JISelection.class);
		if (jis != null)
		{
			jis.UpdateSelectionText();
		}
		
		JILogReplayPanel jirp = iman.findInteraction(JILogReplayPanel.class);
		if (jirp != null)
		{
			jirp.UpdateReplayPanel();
		}
	}
}
