package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.List;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiTextbox;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisMagnetLineGroup;
import org.processmining.processprofiler3d.compviz.methods.EdgeDataVisualisationMethodCollection;
import org.processmining.processprofiler3d.compviz.methods.TransitionDataVisualisationMethodCollection;
import org.processmining.processprofiler3d.maths.ScaleTransforms;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIPSENVis extends JIOptionsPanel implements IDropDownListener, ITextboxInputListener
{
	private JoglScene scene;
	private JoglInteractionManager iman;

	private JIPSENTransVisDriver tdriver = null;
	private JIPSENEdgeVisDriver edriver = null;
	private JIPSENTrajectoryVisDriver jptv = null;
	private JILogReplay jlr = null;

	private boolean bVisible = false;
	
	private JoglGuiLabel btnEditLayout;
	
	private JoglGuiLabel btnVisModeTransition;
	private JoglGuiLabel btnVisModeEdge;
	private JoglGuiLabel btnVisModeTrajectory;
	private JoglGuiLabel btnVisModeReplay;
	
	private JoglGuiLabel btnTransitionVisLabel1;
	private JoglGuiLabel btnTransitionVisLabel2;
	private JoglGuiLabel btnTransitionVisNorm1;
	private JoglGuiLabel btnTransitionVisNorm2;
	private JoglGuiLabel btnTransitionVisScale1;
	private JoglGuiLabel btnTransitionVisScale2;
	private JoglGuiLabel btnTransitionVisDoubleCode;

	private JoglGuiLabel btnEdgeVisLabel1;
	private JoglGuiLabel btnEdgeVisScale1;
	private JoglGuiLabel btnEdgeVisNorm1;

	private JoglGuiLabel btnTrajectoryVisColor1;
	private JoglGuiTextbox btnTrajectoryVisScale1;
	private JoglGuiLabel btnEdgeVisMerge;

//	private JoglGuiCheckBox cbxNormalizationMode1;
//	private JoglGuiCheckBox cbxNormalizationMode2;
//	private JoglGuiCheckBox cbxNormalizationMode3;

	private JoglGuiLabel lblEditLayout;
	private JoglGuiLabel lblVisModeTransition;
	private JoglGuiLabel lblTransitionVisLabel1;
	private JoglGuiLabel lblTransitionVisLabel2;
	private JoglGuiLabel lblVisModeEdge;
	private JoglGuiLabel lblEdgeVisLabel1;
	private JoglGuiLabel lblVisModeTrajectory;

	private JoglGuiLabel lblTransitionVisNorm1;
	private JoglGuiLabel lblTransitionVisNorm2;
	private JoglGuiLabel lblTransitionVisScale1;
	private JoglGuiLabel lblTransitionVisScale2;
	private JoglGuiLabel lblTransitionVisDoubleCodeMode;
	private JoglGuiLabel lblEdgeVisScale1;
	private JoglGuiLabel lblEdgeVisNorm1;
	private JoglGuiLabel lblTrajectoryVisColor1;
	private JoglGuiLabel lblTrajectoryVisScale1;
	private JoglGuiLabel lblEdgeVisMerge;
	private JoglGuiLabel lblVisModeReplay;
	
	private JoglGuiLabel highlighted = null;
	
	private JIDropdownBox ddbox = null;
	
	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();
	
	public JIPSENVis(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui)
	{
		super(gui);
		
		this.scene = scene;
		this.iman = iman;
	}
	
	@Override
	public boolean isVisible()
	{
		return bVisible;
	}
	
	@Override
	public void Show()
	{	
		tdriver = iman.findInteraction(JIPSENTransVisDriver.class);
		edriver = iman.findInteraction(JIPSENEdgeVisDriver.class);
		jptv = iman.findInteraction(JIPSENTrajectoryVisDriver.class);
		jlr = iman.findInteraction(JILogReplay.class);
		
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_LEFT;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff2 = new FontFormat();
		ff2.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff2.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff2.align = FontFormat.ALIGN_RIGHT;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		//int line = 0;
		float posy = 0.85f;

		float colLabelWidth = 0.2f * (float)JoglGuiStyle.textScale;
		float colButtonWidth = 0.2f * (float)JoglGuiStyle.textScale;
		
		float colButtonX = 0.95f - colButtonWidth;
		float colLabelX = colButtonX - colLabelWidth - 0.005f;

		JIMoveModelElement jmt = iman.findInteraction(JIMoveModelElement.class);
		if (jmt != null)
		{
			lblEditLayout = new JoglGuiLabel(ff2.clone());
			lblEditLayout.setRect(colLabelX, posy, colLabelWidth, 0.05f);
			lblEditLayout.setText("Edit Layout");
			lblEditLayout.setBackColor(JoglGuiStyle.transparentColor);
			
			gui.AddGuiObject(lblEditLayout);
			controls.add(lblEditLayout);
			
			btnEditLayout = new JoglGuiLabel(ff.clone());
			btnEditLayout.setCanTouch(true);
			btnEditLayout.setRect(colButtonX, posy, colButtonWidth, 0.05f);
			if (jmt.IsEnabled())
			{
				btnEditLayout.setText("Enabled");
			}
			else
			{
				btnEditLayout.setText("Disabled");
			}
			btnEditLayout.setPadding(textPadding);
			btnEditLayout.setBackColor(JoglGuiStyle.backColor);
			
			gui.AddGuiObject(btnEditLayout);
			controls.add(btnEditLayout);
			
			posy = posy - 0.1f;
		}
		
		lblVisModeTransition = new JoglGuiLabel(ff2.clone());
		lblVisModeTransition.setRect(colLabelX, posy, colLabelWidth, 0.05f);
		lblVisModeTransition.setText("Transition Visualization");
		lblVisModeTransition.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblVisModeTransition);
		controls.add(lblVisModeTransition);
		
		btnVisModeTransition = new JoglGuiLabel(ff.clone());
		btnVisModeTransition.setCanTouch(true);
		btnVisModeTransition.setRect(colButtonX, posy, colButtonWidth, 0.05f);
		btnVisModeTransition.setText(tdriver.GetTransitionVisMethodName());
		btnVisModeTransition.setPadding(textPadding);
		btnVisModeTransition.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnVisModeTransition);
		controls.add(btnVisModeTransition);
		
		posy = posy - 0.1f;

		if (!tdriver.GetTransitionVisMethodName().equals("None"))
		{
			if (tdriver.HasLabel1())
			{
				lblTransitionVisLabel1 = new JoglGuiLabel(ff2.clone());
				lblTransitionVisLabel1.setRect(colLabelX, posy, colLabelWidth, 0.05f);
				lblTransitionVisLabel1.setText("X-Axis");
				lblTransitionVisLabel1.setBackColor(JoglGuiStyle.transparentColor);
				
				gui.AddGuiObject(lblTransitionVisLabel1);
				controls.add(lblTransitionVisLabel1);
		
				btnTransitionVisLabel1 = new JoglGuiLabel(ff.clone());
				btnTransitionVisLabel1.setCanTouch(true);
				btnTransitionVisLabel1.setRect(colButtonX, posy, colButtonWidth, 0.05f);
				btnTransitionVisLabel1.setText(tdriver.GetLabel1Name());
				btnTransitionVisLabel1.setPadding(textPadding);
				btnTransitionVisLabel1.setBackColor(JoglGuiStyle.backColor);
				
				gui.AddGuiObject(btnTransitionVisLabel1);
				controls.add(btnTransitionVisLabel1);
				
				posy = posy - 0.1f;
				
				lblTransitionVisScale1 = new JoglGuiLabel(ff2.clone());
				lblTransitionVisScale1.setRect(colLabelX, posy, colLabelWidth, 0.05f);
				lblTransitionVisScale1.setText("Scale");
				lblTransitionVisScale1.setBackColor(JoglGuiStyle.transparentColor);
				
				//gui.AddGuiObject(lblTransitionVisScale1);
				//controls.add(lblTransitionVisScale1);
		
				btnTransitionVisScale1 = new JoglGuiLabel(ff.clone());
				btnTransitionVisScale1.setCanTouch(true);
				btnTransitionVisScale1.setRect(colButtonX, posy, colButtonWidth, 0.05f);
				btnTransitionVisScale1.setText(tdriver.getScaleMode1());
				btnTransitionVisScale1.setPadding(textPadding);
				btnTransitionVisScale1.setBackColor(JoglGuiStyle.backColor);
				
				//gui.AddGuiObject(btnTransitionVisScale1);
				//controls.add(btnTransitionVisScale1);
				
				//posy = posy - 0.1f;
				
				//normalization setting
				lblTransitionVisNorm1 = new JoglGuiLabel(ff2.clone());
				lblTransitionVisNorm1.setRect(colLabelX, posy, colLabelWidth, 0.05f);
				lblTransitionVisNorm1.setText("Normalization");
				lblTransitionVisNorm1.setBackColor(JoglGuiStyle.transparentColor);
				
				gui.AddGuiObject(lblTransitionVisNorm1);
				controls.add(lblTransitionVisNorm1);
		
				btnTransitionVisNorm1 = new JoglGuiLabel(ff.clone());
				btnTransitionVisNorm1.setCanTouch(true);
				btnTransitionVisNorm1.setRect(colButtonX, posy, colButtonWidth, 0.05f);
				btnTransitionVisNorm1.setText(tdriver.getNormalizationMode1());
				btnTransitionVisNorm1.setPadding(textPadding);
				btnTransitionVisNorm1.setBackColor(JoglGuiStyle.backColor);
				
				gui.AddGuiObject(btnTransitionVisNorm1);
				controls.add(btnTransitionVisNorm1);
				
				posy = posy - 0.1f;
			}

			if (tdriver.HasLabel2())
			{
				lblTransitionVisLabel2 = new JoglGuiLabel(ff2.clone());
				lblTransitionVisLabel2.setRect(colLabelX, posy, colLabelWidth, 0.05f);
				lblTransitionVisLabel2.setText("Y-Axis");
				lblTransitionVisLabel2.setBackColor(JoglGuiStyle.transparentColor);
				
				gui.AddGuiObject(lblTransitionVisLabel2);
				controls.add(lblTransitionVisLabel2);
		
				btnTransitionVisLabel2 = new JoglGuiLabel(ff.clone());
				btnTransitionVisLabel2.setCanTouch(true);
				btnTransitionVisLabel2.setRect(colButtonX, posy, colButtonWidth, 0.05f);
				btnTransitionVisLabel2.setText(tdriver.GetLabel2Name());
				btnTransitionVisLabel2.setPadding(textPadding);
				btnTransitionVisLabel2.setBackColor(JoglGuiStyle.backColor);
				
				gui.AddGuiObject(btnTransitionVisLabel2);
				controls.add(btnTransitionVisLabel2);
				
				posy = posy - 0.1f;
				
				lblTransitionVisScale2 = new JoglGuiLabel(ff2.clone());
				lblTransitionVisScale2.setRect(colLabelX, posy, colLabelWidth, 0.05f);
				lblTransitionVisScale2.setText("Scale");
				lblTransitionVisScale2.setBackColor(JoglGuiStyle.transparentColor);
				
				//gui.AddGuiObject(lblTransitionVisScale2);
				//controls.add(lblTransitionVisScale2);
		
				btnTransitionVisScale2 = new JoglGuiLabel(ff.clone());
				btnTransitionVisScale2.setCanTouch(true);
				btnTransitionVisScale2.setRect(colButtonX, posy, colButtonWidth, 0.05f);
				btnTransitionVisScale2.setText(tdriver.getScaleMode2());
				btnTransitionVisScale2.setPadding(textPadding);
				btnTransitionVisScale2.setBackColor(JoglGuiStyle.backColor);
				
				//gui.AddGuiObject(btnTransitionVisScale2);
				//controls.add(btnTransitionVisScale2);
				
				//posy = posy - 0.1f;
				
				//normalization setting
				lblTransitionVisNorm2 = new JoglGuiLabel(ff2.clone());
				lblTransitionVisNorm2.setRect(colLabelX, posy, colLabelWidth, 0.05f);
				lblTransitionVisNorm2.setText("Normalization");
				lblTransitionVisNorm2.setBackColor(JoglGuiStyle.transparentColor);
				
				gui.AddGuiObject(lblTransitionVisNorm2);
				controls.add(lblTransitionVisNorm2);
		
				btnTransitionVisNorm2 = new JoglGuiLabel(ff.clone());
				btnTransitionVisNorm2.setCanTouch(true);
				btnTransitionVisNorm2.setRect(colButtonX, posy, colButtonWidth, 0.05f);
				btnTransitionVisNorm2.setText(tdriver.getNormalizationMode2());
				btnTransitionVisNorm2.setPadding(textPadding);
				btnTransitionVisNorm2.setBackColor(JoglGuiStyle.backColor);
				
				gui.AddGuiObject(btnTransitionVisNorm2);
				controls.add(btnTransitionVisNorm2);
				
				posy = posy - 0.1f;

				lblTransitionVisDoubleCodeMode = new JoglGuiLabel(ff2.clone());
				lblTransitionVisDoubleCodeMode.setRect(colLabelX, posy, colLabelWidth, 0.05f);
				lblTransitionVisDoubleCodeMode.setText("Dual Coding");
				lblTransitionVisDoubleCodeMode.setBackColor(JoglGuiStyle.transparentColor);
				
				gui.AddGuiObject(lblTransitionVisDoubleCodeMode);
				controls.add(lblTransitionVisDoubleCodeMode);
		
				btnTransitionVisDoubleCode = new JoglGuiLabel(ff.clone());
				btnTransitionVisDoubleCode.setCanTouch(true);
				btnTransitionVisDoubleCode.setRect(colButtonX, posy, colButtonWidth, 0.05f);
				btnTransitionVisDoubleCode.setText(tdriver.getDoubleCodeMode());
				btnTransitionVisDoubleCode.setPadding(textPadding);
				btnTransitionVisDoubleCode.setBackColor(JoglGuiStyle.backColor);
				
				gui.AddGuiObject(btnTransitionVisDoubleCode);
				controls.add(btnTransitionVisDoubleCode);
				
				posy = posy - 0.1f;
			}
		}
		
		lblVisModeEdge = new JoglGuiLabel(ff2.clone());
		lblVisModeEdge.setRect(colLabelX, posy, colLabelWidth, 0.05f);
		lblVisModeEdge.setText("Edge Visualization");
		lblVisModeEdge.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblVisModeEdge);
		controls.add(lblVisModeEdge);

		btnVisModeEdge = new JoglGuiLabel(ff.clone());
		btnVisModeEdge.setCanTouch(true);
		btnVisModeEdge.setRect(colButtonX, posy, colButtonWidth, 0.05f);
		btnVisModeEdge.setText(edriver.GetEdgeVisMethodName());
		btnVisModeEdge.setPadding(textPadding);
		btnVisModeEdge.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnVisModeEdge);
		controls.add(btnVisModeEdge);
		
		posy = posy - 0.1f;
		
		if (!edriver.GetEdgeVisMethodName().equals("None"))
		{
			if (edriver.HasLabel1())
			{
				lblEdgeVisLabel1 = new JoglGuiLabel(ff2.clone());
				lblEdgeVisLabel1.setRect(colLabelX, posy, colLabelWidth, 0.05f);
				lblEdgeVisLabel1.setText("Arc Height");
				lblEdgeVisLabel1.setBackColor(JoglGuiStyle.transparentColor);
				
				gui.AddGuiObject(lblEdgeVisLabel1);
				controls.add(lblEdgeVisLabel1);
		
				btnEdgeVisLabel1 = new JoglGuiLabel(ff.clone());
				btnEdgeVisLabel1.setCanTouch(true);
				btnEdgeVisLabel1.setRect(colButtonX, posy, colButtonWidth, 0.05f);
				btnEdgeVisLabel1.setText(edriver.GetLabel1Name());
				btnEdgeVisLabel1.setPadding(textPadding);
				btnEdgeVisLabel1.setBackColor(JoglGuiStyle.backColor);
				
				gui.AddGuiObject(btnEdgeVisLabel1);
				controls.add(btnEdgeVisLabel1);
		
//				cbxNormalizationMode3 = new JoglGuiCheckBox();
//				cbxNormalizationMode3.setCanTouch(true);
//				cbxNormalizationMode3.setRect(0.955f, posy + 0.0075f, 0.035f / aspect, 0.035f);
//				cbxNormalizationMode3.setLineDims(0.0025f / aspect, 0.0025f);
//				cbxNormalizationMode3.setColor(1.0f, 1.0f, 1.0f);
//				cbxNormalizationMode3.setBackColor(JoglGuiStyle.disabledColor);
//		    	cbxNormalizationMode3.setChecked(edriver.getNormalizationMode());
//				
//				gui.AddGuiObject(cbxNormalizationMode3);
//				controls.add(cbxNormalizationMode3);
				
				posy = posy - 0.1f;
				
				lblEdgeVisScale1 = new JoglGuiLabel(ff2.clone());
				lblEdgeVisScale1.setRect(colLabelX, posy, colLabelWidth, 0.05f);
				lblEdgeVisScale1.setText("Scale");
				lblEdgeVisScale1.setBackColor(JoglGuiStyle.transparentColor);
				
				//gui.AddGuiObject(lblEdgeVisScale1);
				//controls.add(lblEdgeVisScale1);
		
				btnEdgeVisScale1 = new JoglGuiLabel(ff.clone());
				btnEdgeVisScale1.setCanTouch(true);
				btnEdgeVisScale1.setRect(colButtonX, posy, colButtonWidth, 0.05f);
				btnEdgeVisScale1.setText(edriver.getScaleMode());
				btnEdgeVisScale1.setPadding(textPadding);
				btnEdgeVisScale1.setBackColor(JoglGuiStyle.backColor);
				
				//gui.AddGuiObject(btnEdgeVisScale1);
				//controls.add(btnEdgeVisScale1);
				
				//posy = posy - 0.1f;
				
				//normalization setting
				lblEdgeVisNorm1 = new JoglGuiLabel(ff2.clone());
				lblEdgeVisNorm1.setRect(colLabelX, posy, colLabelWidth, 0.05f);
				lblEdgeVisNorm1.setText("Normalization");
				lblEdgeVisNorm1.setBackColor(JoglGuiStyle.transparentColor);
				
				gui.AddGuiObject(lblEdgeVisNorm1);
				controls.add(lblEdgeVisNorm1);
		
				btnEdgeVisNorm1 = new JoglGuiLabel(ff.clone());
				btnEdgeVisNorm1.setCanTouch(true);
				btnEdgeVisNorm1.setRect(colButtonX, posy, colButtonWidth, 0.05f);
				btnEdgeVisNorm1.setText(edriver.getNormalizationMode());
				btnEdgeVisNorm1.setPadding(textPadding);
				btnEdgeVisNorm1.setBackColor(JoglGuiStyle.backColor);
				
				gui.AddGuiObject(btnEdgeVisNorm1);
				controls.add(btnEdgeVisNorm1);
				
				posy = posy - 0.1f;
				
				//merge setting
				lblEdgeVisMerge = new JoglGuiLabel(ff2.clone());
				lblEdgeVisMerge.setRect(colLabelX, posy, colLabelWidth, 0.05f);
				lblEdgeVisMerge.setText("Merging");
				lblEdgeVisMerge.setBackColor(JoglGuiStyle.transparentColor);
				
				gui.AddGuiObject(lblEdgeVisMerge);
				controls.add(lblEdgeVisMerge);
		
				btnEdgeVisMerge = new JoglGuiLabel(ff.clone());
				btnEdgeVisMerge.setCanTouch(true);
				btnEdgeVisMerge.setRect(colButtonX, posy, colButtonWidth, 0.05f);
				if (JSODataVisMagnetLineGroup.bAllowGrouping)
				{
					btnEdgeVisMerge.setText("Enabled");
				}
				else
				{
					btnEdgeVisMerge.setText("Disabled");
				}
				btnEdgeVisMerge.setPadding(textPadding);
				btnEdgeVisMerge.setBackColor(JoglGuiStyle.backColor);
				
				gui.AddGuiObject(btnEdgeVisMerge);
				controls.add(btnEdgeVisMerge);
				
				posy = posy - 0.1f;
			}
		}

		if (jptv != null)
		{
			lblVisModeTrajectory = new JoglGuiLabel(ff2.clone());
			lblVisModeTrajectory.setRect(colLabelX, posy, colLabelWidth, 0.05f);
			lblVisModeTrajectory.setText("Trajectory Visualization");
			lblVisModeTrajectory.setBackColor(JoglGuiStyle.transparentColor);
			
			gui.AddGuiObject(lblVisModeTrajectory);
			controls.add(lblVisModeTrajectory);
	
			btnVisModeTrajectory = new JoglGuiLabel(ff.clone());
			btnVisModeTrajectory.setCanTouch(true);
			btnVisModeTrajectory.setRect(colButtonX, posy, colButtonWidth, 0.05f);
			btnVisModeTrajectory.setText(jptv.getVisMode());
			btnVisModeTrajectory.setPadding(textPadding);
			btnVisModeTrajectory.setBackColor(JoglGuiStyle.backColor);
			
			gui.AddGuiObject(btnVisModeTrajectory);
			controls.add(btnVisModeTrajectory);
	
			posy = posy - 0.1f;

			if (jptv.isVisible())
			{
				lblTrajectoryVisColor1 = new JoglGuiLabel(ff2.clone());
				lblTrajectoryVisColor1.setRect(colLabelX, posy, colLabelWidth, 0.05f);
				lblTrajectoryVisColor1.setText("Color");
				lblTrajectoryVisColor1.setBackColor(JoglGuiStyle.transparentColor);
				
				gui.AddGuiObject(lblTrajectoryVisColor1);
				controls.add(lblTrajectoryVisColor1);
		
				btnTrajectoryVisColor1 = new JoglGuiLabel(ff.clone());
				btnTrajectoryVisColor1.setCanTouch(true);
				btnTrajectoryVisColor1.setRect(colButtonX, posy, colButtonWidth, 0.05f);
				btnTrajectoryVisColor1.setText(jptv.getColorMode());
				btnTrajectoryVisColor1.setPadding(textPadding);
				btnTrajectoryVisColor1.setBackColor(JoglGuiStyle.backColor);
				
				gui.AddGuiObject(btnTrajectoryVisColor1);
				controls.add(btnTrajectoryVisColor1);
				
				posy = posy - 0.1f;
				
				lblTrajectoryVisScale1 = new JoglGuiLabel(ff2.clone());
				lblTrajectoryVisScale1.setRect(colLabelX, posy, colLabelWidth, 0.05f);
				lblTrajectoryVisScale1.setText("Scale");
				lblTrajectoryVisScale1.setBackColor(JoglGuiStyle.transparentColor);
				
				gui.AddGuiObject(lblTrajectoryVisScale1);
				controls.add(lblTrajectoryVisScale1);
		
				btnTrajectoryVisScale1 = new JoglGuiTextbox(ff.clone());
				btnTrajectoryVisScale1.setCanTouch(true);
				btnTrajectoryVisScale1.setRect(colButtonX, posy, colButtonWidth, 0.05f);
				btnTrajectoryVisScale1.setText("" + jptv.getScale());
				btnTrajectoryVisScale1.setPadding(textPadding);
				btnTrajectoryVisScale1.setBackColor(JoglGuiStyle.backColor);
				
				gui.AddGuiObject(btnTrajectoryVisScale1);
				controls.add(btnTrajectoryVisScale1);
				
				posy = posy - 0.1f;
			}
		}
		
		if (jlr != null)
		{
			lblVisModeReplay = new JoglGuiLabel(ff2.clone());
			lblVisModeReplay.setRect(colLabelX, posy, colLabelWidth, 0.05f);
			lblVisModeReplay.setText("Replay Visualization");
			lblVisModeReplay.setBackColor(JoglGuiStyle.transparentColor);
			
			gui.AddGuiObject(lblVisModeReplay);
			controls.add(lblVisModeReplay);
	
			btnVisModeReplay = new JoglGuiLabel(ff.clone());
			btnVisModeReplay.setCanTouch(true);
			btnVisModeReplay.setRect(colButtonX, posy, colButtonWidth, 0.05f);
			btnVisModeReplay.setText(jlr.getVisMode());
			btnVisModeReplay.setPadding(textPadding);
			btnVisModeReplay.setBackColor(JoglGuiStyle.backColor);
			
			gui.AddGuiObject(btnVisModeReplay);
			controls.add(btnVisModeReplay);
	
			posy = posy - 0.1f;
			
		}
		
		bVisible = true;
	}

	@Override
	public void Hide()
	{
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
		
		bVisible = false;
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (!bVisible)
		{
			//Init();
		}
		
		if (lblTransitionVisLabel1 != null && btnTransitionVisLabel1 != null)// && cbxNormalizationMode1 != null)
		{
			if (tdriver.HasLabel1())
			{
				btnTransitionVisLabel1.setCanTouch(true);
				btnTransitionVisLabel1.setColor(1.0f, 1.0f, 1.0f);
				lblTransitionVisLabel1.setColor(1.0f, 1.0f, 1.0f);
			}
			else
			{
				btnTransitionVisLabel1.setCanTouch(false);
				btnTransitionVisLabel1.setColor(0.0f, 0.0f, 0.0f);
				lblTransitionVisLabel1.setColor(0.0f, 0.0f, 0.0f);
			}
		}

		if (lblTransitionVisLabel2 != null && btnTransitionVisLabel2 != null)// && cbxNormalizationMode2 != null)
		{
			if (tdriver.HasLabel2())
			{
				btnTransitionVisLabel2.setCanTouch(true);
				btnTransitionVisLabel2.setColor(1.0f, 1.0f, 1.0f);
				lblTransitionVisLabel2.setColor(1.0f, 1.0f, 1.0f);
			}
			else
			{
				btnTransitionVisLabel2.setCanTouch(false);
				btnTransitionVisLabel2.setColor(0.0f, 0.0f, 0.0f);
				lblTransitionVisLabel2.setColor(0.0f, 0.0f, 0.0f);
			}
		}

		if (lblEdgeVisLabel1 != null && btnEdgeVisLabel1 != null)// && cbxNormalizationMode3 != null)
		{
			if (edriver.HasLabel1())
			{
				btnEdgeVisLabel1.setCanTouch(true);
				btnEdgeVisLabel1.setColor(1.0f, 1.0f, 1.0f);
				lblEdgeVisLabel1.setColor(1.0f, 1.0f, 1.0f);
				
//				cbxNormalizationMode3.setCanTouch(true);
//				cbxNormalizationMode3.setColor(1.0f, 1.0f, 1.0f);
//				cbxNormalizationMode3.setBackColor(JoglGuiStyle.backColor);
			}
			else
			{
				btnEdgeVisLabel1.setCanTouch(false);
				btnEdgeVisLabel1.setColor(0.0f, 0.0f, 0.0f);
				lblEdgeVisLabel1.setColor(0.0f, 0.0f, 0.0f);
				
//				cbxNormalizationMode3.setCanTouch(false);
//				cbxNormalizationMode3.setColor(0.5f, 0.5f, 0.5f);
//				cbxNormalizationMode3.setBackColor(JoglGuiStyle.disabledColor);
			}
		}
		
		bUpdateScene = false;
	}
	
	public void updateTrajVisScale(double s)
	{
		if (btnTrajectoryVisScale1 != null)
		{
			btnTrajectoryVisScale1.setText("" + s);
			bUpdateScene = true;
		}
	}
	
	//MouseListener method implementation
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {		
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnEditLayout)
			{
				Vector<String> lst = new Vector<String>();
				lst.add("Enabled");
				lst.add("Disabled");
				
				OpenDropDownBox(btnEditLayout, lst);
	
	        	return true;
			}
			else if (jgo == btnVisModeTransition)
			{
				OpenDropDownBox(btnVisModeTransition, TransitionDataVisualisationMethodCollection.GetNames());
	
	        	return true;
			}
			else if (jgo == btnVisModeEdge)
			{
				OpenDropDownBox(btnVisModeEdge, EdgeDataVisualisationMethodCollection.GetNames());
	
	        	return true;
			}
			else if (jgo == btnVisModeTrajectory)
			{
				OpenDropDownBox(btnVisModeTrajectory, jptv.getVisModeNames());
	
	        	return true;
			}
			else if (jgo == btnVisModeReplay)
			{
				OpenDropDownBox(btnVisModeReplay, jlr.getVisModeNames());
	
	        	return true;
			}
			else if (jgo == btnTransitionVisLabel1)
			{
				OpenDropDownBox(btnTransitionVisLabel1, tdriver.GetLabelNames());
	
	        	return true;
			}
			else if (jgo == btnTransitionVisLabel2)
			{
				OpenDropDownBox(btnTransitionVisLabel2, tdriver.GetLabelNames());
	
	        	return true;
			}
			else if (jgo == btnEdgeVisLabel1)
			{
				OpenDropDownBox(btnEdgeVisLabel1, edriver.GetLabelNames());
				
	        	return true;
			}
			else if (jgo == btnTransitionVisScale1)
			{
				OpenDropDownBox(btnTransitionVisScale1, ScaleTransforms.getScaleModes());
	
	        	return true;
			}
			else if (jgo == btnTransitionVisScale2)
			{
				OpenDropDownBox(btnTransitionVisScale2, ScaleTransforms.getScaleModes());
	
	        	bUpdateScene = true;
	        	return true;
			}
			else if (jgo == btnTransitionVisNorm1)
			{
				OpenDropDownBox(btnTransitionVisNorm1, tdriver.GetNormalizationNames1());
	
	        	return true;
			}
			else if (jgo == btnTransitionVisNorm2)
			{
				OpenDropDownBox(btnTransitionVisNorm2, tdriver.GetNormalizationNames2());
	
	        	return true;
			}
			else if (jgo == btnTransitionVisDoubleCode)
			{
				OpenDropDownBox(btnTransitionVisDoubleCode, tdriver.GetDoubleCodeNames());
	
	        	return true;
			}
			else if (jgo == btnEdgeVisScale1)
			{
				OpenDropDownBox(btnEdgeVisScale1, ScaleTransforms.getScaleModes());
				
	        	return true;
			}
			else if (jgo == btnEdgeVisNorm1)
			{
				OpenDropDownBox(btnEdgeVisNorm1, edriver.GetNormalizationNames());
	
	        	return true;
			}
			else if (jgo == btnTrajectoryVisColor1)
			{
				OpenDropDownBox(btnTrajectoryVisColor1, jptv.getColorModeNames());
	
	        	return true;
			}
			else if (jgo == btnTrajectoryVisScale1)
			{
				JITextboxInput jibt = new JITextboxInputNumeric(iman, scene, gui, btnTrajectoryVisScale1);
				jibt.listener = this;
				iman.addInteraction(jibt);
				
				bUpdateScene = true;
	
	        	return true;
			}
			else if (jgo == btnEdgeVisMerge)
			{
				Vector<String> lst = new Vector<String>();
				lst.add("Enabled");
				lst.add("Disabled");
				OpenDropDownBox(btnEdgeVisMerge, lst);
				
	        	return true;
			}
		}
		
		return false;
    }
	
	private void OpenDropDownBox(JoglGuiLabel label, List<String> vals)
	{
		if (ddbox != null)
		{
			CloseDropDownBox();
		}

		if (!vals.isEmpty())
		{
			JoglVectord4 pos = label.getRect();
			ddbox = new JIDropdownBox(iman, gui, label, vals);
			ddbox.addDropDownListener(this);
			ddbox.setPosition((float)pos.getX(), (float)pos.getY() + 0.05f);
			ddbox.setWidth(0.2f * (float)JoglGuiStyle.textScale);
			ddbox.setTextSize(0.03f);
			iman.addInteraction(ddbox);
		}
		
    	bUpdateScene = true;	
	}

	private void CloseDropDownBox()
	{
		//close dropdown
		iman.removeInteraction(ddbox);
		ddbox.Dispose();
		ddbox = null;
	}
	
	public void OnDropDownSelected(JIDropdownBox ddb, String val) 
	{
		//System.out.println("OnDropDownSelected!");
		if (ddbox == ddb)
		{
			if (val != null)
			{
				if (ddb.getOwner() == btnEditLayout)
				{
					JIMoveModelElement jmt = iman.findInteraction(JIMoveModelElement.class);
					if (jmt != null)
					{
						if (val.equals("Enabled"))
						{
							jmt.Enable(true);
						}
						else
						{
							jmt.Enable(false);
						}
						
						btnEditLayout.setText(val);
					}
				}
				else if (ddb.getOwner() == btnTransitionVisLabel1)
				{
					tdriver.setLabel1(val);
		        	btnTransitionVisLabel1.setText(tdriver.GetLabel1Name());
				}
				else if (ddb.getOwner() == btnTransitionVisLabel2)
				{
					tdriver.setLabel2(val);
		        	btnTransitionVisLabel2.setText(tdriver.GetLabel2Name());
				}
				else if (ddb.getOwner() == btnTransitionVisDoubleCode)
				{
					tdriver.setDoubleCodeMode(val);
		        	btnTransitionVisDoubleCode.setText(tdriver.getDoubleCodeMode());
				}
				else if (ddb.getOwner() == btnEdgeVisLabel1)
				{
					edriver.setLabel1(val);
					btnEdgeVisLabel1.setText(edriver.GetLabel1Name());
				}
				else if (ddb.getOwner() == btnVisModeTransition)
				{
					tdriver.setTransitionVisMethod(val);
		        	btnVisModeTransition.setText(tdriver.GetTransitionVisMethodName());
				}
				else if (ddb.getOwner() == btnVisModeEdge)
				{
					edriver.setEdgeVisMethod(val);
		        	btnVisModeEdge.setText(edriver.GetEdgeVisMethodName());
				}
				else if (ddb.getOwner() == btnTransitionVisScale1)
				{
					tdriver.setScaleMode1(val);
		        	btnTransitionVisScale1.setText(tdriver.getScaleMode1());
				}
				else if (ddb.getOwner() == btnTransitionVisScale2)
				{
					tdriver.setScaleMode2(val);
		        	btnTransitionVisScale2.setText(tdriver.getScaleMode2());
				}
				else if (ddb.getOwner() == btnTransitionVisNorm1)
				{
					tdriver.setNormalizationMode1(val);
		        	btnTransitionVisNorm1.setText(tdriver.getNormalizationMode1());
				}
				else if (ddb.getOwner() == btnTransitionVisNorm2)
				{
					tdriver.setNormalizationMode2(val);
		        	btnTransitionVisNorm2.setText(tdriver.getNormalizationMode2());
				}
				else if (ddb.getOwner() == btnEdgeVisScale1)
				{
					edriver.setScaleMode(val);
					btnEdgeVisScale1.setText(edriver.getScaleMode());
				}
				else if (ddb.getOwner() == btnEdgeVisNorm1)
				{
					edriver.setNormalizationMode(val);
					btnEdgeVisNorm1.setText(edriver.getNormalizationMode());
				}
				else if (ddb.getOwner() == btnEdgeVisMerge)
				{
					if (val.equals("Enabled"))
					{
						JSODataVisMagnetLineGroup.bAllowGrouping = true;
					}
					else
					{
						JSODataVisMagnetLineGroup.bAllowGrouping = false;
					}
					
					btnEdgeVisMerge.setText(val);
					edriver.SetVisualization();
					
				}
				else if (ddb.getOwner() == btnVisModeTrajectory)
				{
					if (jptv != null)
					{
						jptv.setVisMode(val);
					}
					
					//edriver.setNormalizationMode(val);
					btnVisModeTrajectory.setText(val);
				}
				else if (ddb.getOwner() == btnVisModeReplay)
				{
					if (jlr != null)
					{
						jlr.setVisMode(val);
					}
					
					btnVisModeReplay.setText(val);
				}
				else if (ddb.getOwner() == btnTrajectoryVisColor1)
				{
					if (jptv != null)
					{
						jptv.setColorMode(val);
					}
					btnTrajectoryVisColor1.setText(val);
				}
			}
			
			CloseDropDownBox();
			
			Hide();
			Show();
			
			bUpdateScene = true;
		}
	}
	
	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnEditLayout)
			{
				highlight(btnEditLayout);
			}
			else if (jgo == btnVisModeTransition)
			{
				highlight(btnVisModeTransition);
			}
			else if (jgo == btnVisModeEdge)
			{
				highlight(btnVisModeEdge);
			}
			else if (jgo == btnTransitionVisLabel1)
			{
				highlight(btnTransitionVisLabel1);
			}
			else if (jgo == btnTransitionVisScale1)
			{
				highlight(btnTransitionVisScale1);
			}
			else if (jgo == btnTransitionVisNorm1)
			{
				highlight(btnTransitionVisNorm1);
			}
			else if (jgo == btnTransitionVisLabel2)
			{
				highlight(btnTransitionVisLabel2);
			}
			else if (jgo == btnTransitionVisScale2)
			{
				highlight(btnTransitionVisScale2);
			}
			else if (jgo == btnTransitionVisNorm2)
			{
				highlight(btnTransitionVisNorm2);
			}
			else if (jgo == btnTransitionVisDoubleCode)
			{
				highlight(btnTransitionVisDoubleCode);
			}
			else if (jgo == btnEdgeVisLabel1)
			{
				highlight(btnEdgeVisLabel1);
			}
			else if (jgo == btnEdgeVisScale1)
			{
				highlight(btnEdgeVisScale1);
			}
			else if (jgo == btnEdgeVisNorm1)
			{
				highlight(btnEdgeVisNorm1);
			}
			else if (jgo == btnEdgeVisMerge)
			{
				highlight(btnEdgeVisMerge);
			}
			else if (jgo == btnVisModeTrajectory)
			{
				highlight(btnVisModeTrajectory);
			}
			else if (jgo == btnTrajectoryVisColor1)
			{
				highlight(btnTrajectoryVisColor1);
			}
			else if (jgo == btnTrajectoryVisScale1)
			{
				highlight(btnTrajectoryVisScale1);
			}
			else if (jgo == btnVisModeReplay)
			{
				highlight(btnVisModeReplay);
			}
			else
			{
				unHighlight();
			}
		}
		else
		{
			unHighlight();
		}
	}

	private void highlight(JoglGuiLabel label)
	{
		unHighlight();
		
		label.setBackColor(JoglGuiStyle.highlightColor);
		highlighted = label;
		
		bUpdateScene = true;
		
	}
	
	private void unHighlight() 
	{
		if (highlighted != null)
		{
			highlighted.setBackColor(JoglGuiStyle.backColor);
			highlighted = null;

			bUpdateScene = true;
		}
	}

	public void OnTextInputFinished(JITextboxInput jti) 
	{
		String txt = jti.getText();
		double val = Double.parseDouble(txt);

		if (jptv != null)
		{
			jptv.setScale(val);
		}
		
	}

}
