package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.awt.event.MouseEvent;
import java.util.List;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.JoglSelection;
import org.processmining.processprofiler3d.compviz.jogl.interactions.camera.JICameraOrbit;
import org.processmining.processprofiler3d.compviz.jogl.interactions.camera.JICameraVR;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdge;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdgeVertex;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnPlace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglFrustumClipper;
import org.processmining.processprofiler3d.psen.PetriNetIndexed;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.collision.JoglRay;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.interactions.JICameraBase;
import org.processmining.visualisation3d.maths.JoglLine;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JISelectModelElement extends JoglInteraction
{
	protected class HitData
	{
		private Object hitObject;
		private double hitDistance;
		
		public HitData(Object o, double dist)
		{
			hitObject = o;
			hitDistance = dist;
		}
		
		public Object getHitObject()
		{
			return hitObject;
		}
		
		public double getHitDistance()
		{
			return hitDistance;
		}
	}
	
	//protected JoglCamera cam;
	protected JoglInteractionManager iman;
	protected JoglScene scene;
	protected JSOPetriNet pnet;
	protected PetriNetIndexed pnetIdx;
	protected JSOGraphicalUserInterface gui;

	protected boolean bUpdateScene = false;

	protected JoglSelection selection;
	
	protected JIDragSelection jids = null;
	
	public JoglSelection getSelection()
	{
		return selection;
	}
	
	public JISelectModelElement(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui, JSOPetriNet pnet)
	{
		//this.cam = cam;
		this.iman = iman;
		this.scene = scene;
		this.gui = gui;
		this.pnet = pnet;
		this.pnetIdx = new PetriNetIndexed(pnet.getPetriNet());
		
		setZLevel(1.0f);
		
		selection = new JoglSelection();
	}

	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			if (jids != null)
			{
				jids.Dispose();
				iman.removeInteraction(jids);
			}
			
			jids = new JIDragSelection(iman, scene, gui, this, e);
			
			iman.addInteraction(jids);
			//return true;
		}
		return false;
    }

	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		return false;
    }
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		bUpdateScene = false;
    }
	
	public JoglCamera getCamera()
	{
		JICameraBase jcb = iman.findInteraction(JICameraBase.class);
		if (jcb instanceof JICameraVR)
		{
			JICameraVR cvr = (JICameraVR)jcb;
			return cvr.getHMDCamera();
		}
		return jcb.getCamera();
	}
	
	public void SelectAllInRect(JoglInputMouseEvent e, JoglVectord2 startRay, JoglVectord2 endRay)
	{
		if (!e.isShiftDown())
		{
			selection.clear();
		}
		
		Vector<JoglSceneObject> lst = GetAllInRect(startRay, endRay);
		for (JoglSceneObject jso : lst)
		{
			selection.add(jso);
		}
		
		bUpdateScene = true;
	}
	
	public Vector<JoglSceneObject> GetAllInRect(JoglVectord2 startRay, JoglVectord2 endRay)
	{
		JoglCamera cam = getCamera();
		Vector<JoglSceneObject> objs = new Vector<JoglSceneObject>();
		
		double l = Math.min(startRay.getX(), endRay.getX());
		double r = Math.max(startRay.getX(), endRay.getX());
		double t = Math.min(startRay.getY(), endRay.getY());
		double b = Math.max(startRay.getY(), endRay.getY());
		
		JoglFrustumClipper jfc = new JoglFrustumClipper(cam, cam.getCameraLens().getAspect(), l, r, -b, -t);

		JIMoveModelElement jmt = iman.findInteraction(JIMoveModelElement.class);
		if (jmt != null && jmt.IsEnabled())
		{
			for (JSOSubNet snet : pnet.getAllSubNets())
			{
				if (!snet.getIsVisible())
					continue;
				
				JoglVectord3 offset = snet.relativeToAbsolute(JoglVectord3.ZERO);
				for (JSOPnEdgeVertex ev : snet.getEdgeVertices())
				{
					JoglVectord3 pos = JoglVectord3.Add(ev.getPosition(), offset);
					if (jfc.IsPointInside(pos))
					{
						selection.add(ev);
					}
				}
			}
		}
		
		for (JSOSubNet snet : pnet.getAllSubNets())
		{
			if (!snet.getIsVisible())
				continue;
			
			JoglVectord3 offset = snet.relativeToAbsolute(JoglVectord3.ZERO);
			for (JSOPnPlace p : snet.getPlaces())
			{
				//JoglVectord3 pos = JoglVectord3.Add(p.getPosition(), snet.getOffset());
				JoglVectord3 pos = JoglVectord3.Add(p.getPosition(), offset);
				if (jfc.IsPointInside(pos))
				{
					objs.add(p);
				}
			}
		}

		for (JSOSubNet snet : pnet.getAllSubNets())
		{
			if (!snet.getIsVisible())
				continue;

			JoglVectord3 offset = snet.relativeToAbsolute(JoglVectord3.ZERO);
			for (JSOPnTransition p : snet.getTransitions())
			{
				//JoglVectord3 pos = JoglVectord3.Add(p.getPosition(), snet.getOffset());
				JoglVectord3 pos = JoglVectord3.Add(p.getPosition(), offset);
				if (jfc.IsPointInside(pos))
				{
					objs.add(p);
				}
			}
		}

		for (JSOSubNet snet : pnet.getAllSubNets())
		{
			if (!snet.getIsVisible())
				continue;

			JoglVectord3 off = snet.relativeToAbsolute(JoglVectord3.ZERO);
			for (JSOPnEdge p : snet.getEdges())
			{
				boolean bInside = true;
				for (JoglVectord3 vpos : p.getPoints())
				{
					JoglVectord3 pt = JoglVectord3.Add(vpos, off);
					if (!jfc.IsPointInside(pt))
					{
						bInside = false;
						break;
					}
				}
				if (bInside)
				{
					objs.add(p);
				}
			}
		}
		
		return objs;
	}

	public void SelectAtMousePos(JoglInputMouseEvent e) 
	{
		//JoglVectord2 vx = new JoglVectord2(e.getX() / gui.getWidth(), e.getY() / gui.getHeight());
		JoglVectord2 vx = e.getPoint();
		
		JoglCamera cam = getCamera();
		
		JoglRay r = cam.GetRay(vx);

		Object hitObject = GetClosestHitObject(r);
		
		UpdateSelection(e, hitObject);
		
		if (hitObject instanceof JSOPnTransition)
		{
			JSOPnTransition jso = (JSOPnTransition)hitObject;
			System.out.println("Hit transition: " + jso.getLabel());
			FocusCamOn(jso);
		}
		else if (hitObject instanceof JSOPnPlace)
		{
			JSOPnPlace jso = (JSOPnPlace)hitObject;
			System.out.println("Hit place: ");
			FocusCamOn(jso);
		}
		else if (hitObject instanceof JSOPnEdge)
		{
			JSOPnEdge edge = (JSOPnEdge)hitObject;
			
			FocusCamOn(edge);
		}
		else if (hitObject instanceof JSOPnEdgeVertex)
		{
			System.out.println("Hit Vertex: ");
			bUpdateScene = true;
		}
		
		if (hitObject == null)
		{
			//pick all subnets which are hit by the ray first to avoid recalculating again and again
			Vector<JSOSubNet> hitSubnets = new Vector<JSOSubNet>();
			for (JSOSubNet snet : pnet.getAllSubNets())
			{
				if (!snet.getIsVisible())
					continue;

				if (snet.getAABB().Intersects(r))
				{
					hitSubnets.add(snet);
				}
			}
			
			if (hitSubnets.isEmpty())
			{
				FocusCamOn(pnet);
			}
			else
			{
				FocusCamOn(hitSubnets.get(0));
			}
		}
	}

	protected void FocusCamOn(JSOPetriNet net) 
	{
		JICameraOrbit jcio = iman.findInteraction(JICameraOrbit.class);	
		if (jcio != null)
		{
			JoglVectord3 focusPos = net.GetPetrinetCenter();
			jcio.setCamFocus(focusPos);
			bUpdateScene = true;
		}
	}

	protected void FocusCamOn(JSOSubNet net) 
	{
		JICameraOrbit jcio = iman.findInteraction(JICameraOrbit.class);	
		if (jcio != null)
		{
			JoglVectord3 focusPos = net.GetPetrinetCenter();
			focusPos = net.relativeToAbsolute(focusPos);
			jcio.setCamFocus(focusPos);
			bUpdateScene = true;
		}
	}

	protected void FocusCamOn(JSOPnTransition jso) 
	{
		JICameraOrbit jcio = iman.findInteraction(JICameraOrbit.class);
		if (jcio != null)
		{
			jcio.setCamFocus(JoglVectord3.Add(jso.getPosition(), jso.getParentNet().getOffset()));
		}
	}

	protected void FocusCamOn(JSOPnPlace jso) 
	{
		JICameraOrbit jcio = iman.findInteraction(JICameraOrbit.class);
		if (jcio != null)
		{
			jcio.setCamFocus(JoglVectord3.Add(jso.getPosition(), jso.getParentNet().getOffset()));
			bUpdateScene = true;
		}
	}

	protected void FocusCamOn(JSOPnEdge edge) 
	{
		JICameraOrbit jcio = iman.findInteraction(JICameraOrbit.class);	
		if (jcio != null)
		{
			JoglVectord3 avg = new JoglVectord3(0,0,0);
			int num = 0;
			for (JoglVectord3 v : edge.getPoints())
			{
				avg = JoglVectord3.Add(avg, v);
				num++;
			}
			avg = JoglVectord3.Multiply(avg, 1.0 / num);
			jcio.setCamFocus(JoglVectord3.Add(avg, edge.getParentNet().getOffset()));
			bUpdateScene = true;
		}
	}

	protected void UpdateSelection(JoglInputMouseEvent e, Object hitObject) 
	{
		if (hitObject != null)
		{
			if (selection.contains((JoglSceneObject)hitObject))
			{
				if (e.isShiftDown())
				{
					selection.remove((JoglSceneObject)hitObject);
					bUpdateScene = true;
				}
				else
				{
					selection.clear();
					selection.add((JoglSceneObject)hitObject);
					bUpdateScene = true;
				}
			}
			else
			{
				if (!e.isShiftDown())
				{
					selection.clear();
				}
				selection.add((JoglSceneObject)hitObject);
				bUpdateScene = true;
			}
		}
		else
		{
			if (!e.isShiftDown())
			{
				selection.clear();
				bUpdateScene = true;
			}
		}
	}
	
	public Object GetClosestHitObject(JoglInputMouseEvent e)
	{
		//JoglVectord2 vx = new JoglVectord2(e.getX() / gui.getWidth(), e.getY() / gui.getHeight());	
		JoglVectord2 vx = e.getPoint();
		
		JoglCamera cam = getCamera();
		
		JoglRay r = cam.GetRay(vx);

		return GetClosestHitObject(r);
	}
	
	protected Object GetClosestHitObject(JoglRay r)
	{
		Object hitObject = null;
		double closestDist = Double.POSITIVE_INFINITY;
		
		//pick all subnets which are hit by the ray first to avoid recalculating again and again
		Vector<JSOSubNet> hitSubnets = new Vector<JSOSubNet>();
		for (JSOSubNet snet : pnet.getAllSubNets())
		{
			if (!snet.getIsVisible())
				continue;

			if (snet.getAABB().Intersects(r))
			{
				hitSubnets.add(snet);
			}
		}
		
		//get transitions hit by ray
		for (JSOSubNet snet : hitSubnets)
		{
			JoglVectord3 v = snet.relativeToAbsolute(JoglVectord3.ZERO);
			for (JSOPnTransition jso : snet.getTransitions())
			{
				JoglAABB aabb = jso.getAABB().Translate(v);
				if (aabb.Intersects(r))
				{
					double dist = aabb.GetIntersectionDist(r);
					if (dist < closestDist)
					{
						hitObject = jso;
						closestDist = dist;
					}
				}
			}
		}

		//get places hit by ray
		for (JSOSubNet snet : hitSubnets)
		{
			JoglVectord3 v = snet.relativeToAbsolute(JoglVectord3.ZERO);
			for (JSOPnPlace jso : snet.getPlaces())
			{
				JoglAABB aabb = jso.getAABB().Translate(v);
				if (aabb.Intersects(r))
				{
					double dist = aabb.GetIntersectionDist(r);
					if (dist < closestDist)
					{
						hitObject = jso;
						closestDist = dist;
					}
				}
			}
		}
		
		//get vertices hit by ray
		JIMoveModelElement jmt = iman.findInteraction(JIMoveModelElement.class);
		if (jmt != null && jmt.IsEnabled())
		{
			for (JSOSubNet snet : hitSubnets)
			{
				JoglVectord3 v = snet.relativeToAbsolute(JoglVectord3.ZERO);
				for (JSOPnEdgeVertex vtx : snet.getEdgeVertices())
				{
					JoglAABB aabb = vtx.getAABB().Translate(v);
					if (aabb.Intersects(r))
					{
						double dist = aabb.GetIntersectionDist(r);
						if (dist < closestDist)
						{
							hitObject = vtx;
							closestDist = dist;
						}
					}
				}
			}
		}
		
		//get edges hit by ray
		HitData hdata = GetClosestEdge(hitSubnets, r, 5.0);
		JSOPnEdge edge = (JSOPnEdge)hdata.getHitObject();
		if (edge != null && hdata.getHitDistance() < closestDist)
		{
			hitObject = edge;
			closestDist = hdata.getHitDistance();
		}
		
		return hitObject;
	}
	
	public HitData GetClosestEdge(Vector<JSOSubNet> hitSubnets, JoglRay r, double mdist)
	{
		JSOPnEdge closestEdge = null;
		double closestDistFromRay = mdist;
		double closestDistOnRay = Double.POSITIVE_INFINITY;

		for (JSOSubNet snet : hitSubnets)
		{
			JoglVectord3 offset = snet.relativeToAbsolute(JoglVectord3.ZERO);
			for (JSOPnEdge eso : snet.getEdges())
			{
				double minDistFromRay = Double.POSITIVE_INFINITY;
				double minDistOnRay = Double.POSITIVE_INFINITY;
				
				List<JoglVectord3> lst = eso.getPoints();
				for (int i = 0; i < lst.size() - 1; i++)
				{
					JoglVectord3 start = JoglVectord3.Add(lst.get(i), offset);
					JoglVectord3 end = JoglVectord3.Add(lst.get(i + 1), offset);
					double distFromRay = GetClosestDistFromLine(start, end, r); //closest distance from ray to line
					double distOnRay = GetClosestPointFromLine(start, end, r); //distance from ray origin to closest point on ray
					
					//mindist = Math.min(mindist, dist);
					if (distFromRay < minDistFromRay)
					{
						minDistFromRay = distFromRay;
						minDistOnRay = distOnRay;
					}
				}
				
				//if (mindist < closestDist)
				if (minDistFromRay < mdist && ((minDistFromRay * 20) + minDistOnRay) < ((closestDistFromRay * 20) + closestDistOnRay))
				{
					//System.out.println("Maxdist: " + maxdist);
					closestDistFromRay = minDistFromRay;
					closestEdge = eso;
					closestDistOnRay = minDistOnRay;
				}
			}
		}
		
		return new HitData(closestEdge, ((closestDistFromRay * 20) + closestDistOnRay));
	}
	
	public double GetClosestPointFromLine(JoglVectord3 start, JoglVectord3 end, JoglRay r)
	{
		JoglVectord3 dir = JoglVectord3.Subtract(end, start);
		JoglVectord3 dirN = JoglVectord3.Normal(dir);
		
		double closest = JoglLine.GetClosestPointDist(r.getOrigin(), r.getDir(), start, dirN);
		return closest;
	}
	
	public double GetClosestDistFromLine(JoglVectord3 start, JoglVectord3 end, JoglRay r)
	{
		JoglVectord3 dir = JoglVectord3.Subtract(end, start);
		JoglVectord3 dirN = JoglVectord3.Normal(dir);
		double len = dir.getMagnitude();
		
		double closest = JoglLine.GetClosestPointDist(start, dirN, r.getOrigin(), r.getDir());
		JoglVectord3 closestP2 = JoglLine.GetClosestPoint(r.getOrigin(), r.getDir(), start, dirN);
		closest = Math.min(len, Math.max(closest, 0));
		
		JoglVectord3 closestP = JoglVectord3.Add(start, JoglVectord3.Multiply(dirN, closest));
		double dist = JoglVectord3.Subtract(closestP2, closestP).getMagnitude();
		
		return dist;
	}

}
