package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.JoglSelection;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiScrollBar;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdge;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdgeVertex;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnPlace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisMagnetLineBase;
import org.processmining.processprofiler3d.psen.PetriNetIndexed;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.interactions.JoglGuiInteraction;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JISelection extends JoglGuiInteraction implements IScrollBarListener
{
	protected JoglInteractionManager iman;
	protected JoglScene scene;
	protected JSOPetriNet pnet;
	protected PetriNetIndexed pnetIdx;

	protected boolean bUpdateScene = false;

	//protected JoglSelection selection;
	
	protected JIDragSelection jids = null;
	
	protected Vector<JoglGuiLabel> labels = new Vector<JoglGuiLabel>();
	protected JoglGuiLabel extlabel = null;
	
	protected HashMap<JoglGuiLabel, JSOPnTransition> expands = new HashMap<JoglGuiLabel, JSOPnTransition>();
	protected HashMap<JoglGuiLabel, JoglSceneObject> infos = new HashMap<JoglGuiLabel, JoglSceneObject>();
	protected HashMap<JoglGuiLabel, JoglSceneObject> deselects = new HashMap<JoglGuiLabel, JoglSceneObject>();
	
	private boolean bExtended = false;
	private boolean bExtChanged = false;
	
//	public JoglSelection getSelection()
//	{
//		return selection;
//	}
	
	public JISelection(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui, JSOPetriNet pnet)
	{
		super(gui);
		
		this.iman = iman;
		this.scene = scene;
		this.pnet = pnet;
		this.pnetIdx = new PetriNetIndexed(pnet.getPetriNet());
		
		setZLevel(-100.0f);
		
		//selection = new JoglSelection();
	}
	
	public void UpdateSelectionText()
	{
		float aspect = (float)gui.getAspect();
		
		float fwidth = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		float fheight = 0.03f * (float)JoglGuiStyle.textScale;
		
		for (JoglGuiLabel l : labels)
		{
			l.setTextSize(fwidth, fheight);
		}
	}

	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			Vector<JoglGuiObject> jgos = gui.GetObjectsFromPos(e.getX(), e.getY());
			
			JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
			if (jgo != null)
			{
				if (scrollB != null && !jgos.isEmpty() && jgos.contains(scrollB))
				{
					scrollB.StartMoveInteraction(iman, e);
					bUpdateScene = true;
					
					return true;
				}
				if (labels.contains(jgo))
				{
					if (jgo == extlabel)
					{
						bExtended = !bExtended;
						bExtChanged = true;
						
						bUpdateScene = true;
					}
					if (expands.containsKey(jgo))
					{
						JIViewManagerModel viewMan = iman.findInteraction(JIViewManagerModel.class);
						if (viewMan != null)
						{
							JSOSubNet snet = FindSubnet(expands.get(jgo));
							viewMan.ToggleSubNet(snet, 0.02f);
						}
					}
					if (deselects.containsKey(jgo))
					{
						JISelectModelElement sel = iman.findInteraction(JISelectModelElement.class);
						if (sel != null)
						{
							JoglSelection s = sel.getSelection();
							s.remove(deselects.get(jgo));
							
							bUpdateScene = true;
						}
					}
					if (infos.containsKey(jgo))
					{
						JIPSENSelectModelElement psensel = iman.findInteraction(JIPSENSelectModelElement.class);
						if (psensel != null)
						{
							JIPSENBaseInfo bi = psensel.getInfoPanel();
							if (bi == null || bi.getInfoSubject() != infos.get(jgo))
							{
								JoglSceneObject infoObject = infos.get(jgo);
								if (infoObject instanceof JSOPnTransition)
								{
									psensel.OpenInfoPanel((JSOPnTransition)infoObject);
								}
								else if (infoObject instanceof JSOPnEdge)
								{
									psensel.OpenInfoPanel((JSOPnEdge)infoObject);
								}
								else if (infoObject instanceof JSODataVisMagnetLineBase)
								{
									psensel.OpenInfoPanel((JSODataVisMagnetLineBase)infoObject);
								}
							}
							else
							{
								psensel.CloseExistingInfoPanel();
							}
						}
					}
					return true;
				}
			}
		}
		return false;
    }

	private JSOSubNet FindSubnet(JSOPnTransition t)
	{
//		for (JSOSubNet snet : scene.findSceneObject(JSOPetriNet.class).getChildSubNets())
//		{
//			if (snet.parent == t)
//			{
//				return snet;
//			}
//		}
		Vector<JSOSubNet> snets = new Vector<JSOSubNet>();
		snets.addAll(scene.findSceneObject(JSOPetriNet.class).getAllSubNets());
		while (!snets.isEmpty())
		{
			JSOSubNet snet = snets.firstElement();
			snets.removeElementAt(0);
			//snets.addAll(snet.getChildSubNets());

			if (snet.getParent() == t)
			{
				return snet;
			}
		}
		return null;
	}
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
			if (jgo != null)
			{
				if (labels.contains(jgo))
				{
					return true;
				}
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {		
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
			if (jgo != null)
			{
				if (labels.contains(jgo))
				{
			        return true;
				}
			}
		}
		
		return false;
    }
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	private long lastSelectionState = -1;
	private float posx = -0.25f;
	private float posy = 0.925f;
	private float lwidth = 0.5f;
	private float lheight = 0.05f;
	private float lbtnwidth = lheight;
	private JoglVectord4 bgColor = new JoglVectord4(0, 0, 0, 0.5);
	private JoglVectord4 bgColor2 = new JoglVectord4(0, 0, 0, 0.75);
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		JISelectModelElement sme = iman.findInteraction(JISelectModelElement.class);
		if (sme != null)
		{
			JoglSelection s = sme.selection;
			if (s.getSelectionState() != lastSelectionState || bExtChanged)
			{
				bExtChanged = false;
				lastSelectionState = s.getSelectionState();
				
				InitSelectionList(s);
			}
		}
		
		bUpdateScene = false;
    }
	
	private void InitSelectionList(JoglSelection s)
	{
		//clear previous labels
		for (JoglGuiLabel lx : labels)
		{
			gui.RemoveGuiObject(lx);
		}
		labels.clear();
		extlabel = null;
		
		if (scrollB != null)
		{
			gui.RemoveGuiObject(scrollB);
		}
		scrollB = null;
		
		JIPSENSelectModelElement jipsme = iman.findInteraction(JIPSENSelectModelElement.class);
		
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_CENTER;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff2 = new FontFormat();
		ff2.width = 0.07f / aspect;
		ff2.height = 0.07f;
		ff2.align = FontFormat.ALIGN_CENTER;
		ff2.setColor(1.0f, 1.0f, 1.0f);

		JIViewManagerModel viewMan = iman.findInteraction(JIViewManagerModel.class);
		float offy = 0;
		posy = 0.925f;
		lbtnwidth = lheight / (float)gui.getAspect();
		
		int line = 0;
		for (JoglSceneObject jso : s.getElements())
		{
			
			if (line >= 3 && !bExtended)
			{
				line++;
				continue;
			}
			
			if (jso instanceof JSOPnTransition)
			{
				AddTransitionLabel((JSOPnTransition)jso, ff, offy, viewMan, jipsme);
				
				offy = offy - lheight;
			}
			else if (jso instanceof JSOPnPlace)
			{
				AddPlaceLabel((JSOPnPlace)jso, ff, offy);
				
				offy = offy - lheight;
			}
			else if (jso instanceof JSOPnEdge)
			{
				AddEdgeLabel((JSOPnEdge)jso, ff, offy, jipsme);
				
				offy = offy - lheight;
			}
			else if (jso instanceof JSOPnEdgeVertex)
			{
				AddEdgeVertexLabel((JSOPnEdgeVertex)jso, ff, offy, jipsme);
				
				offy = offy - lheight;
			}
			else if (jso instanceof JSODataVisMagnetLineBase)
			{
				AddMagnetLineLabel((JSODataVisMagnetLineBase)jso, ff, offy, jipsme);
				
				offy = offy - lheight;
			}
			
			line++;
		}
		if (line > 3)
		{
			//add extension
			JoglGuiLabel l = new JoglGuiLabel(ff.clone());
			l.setRect(posx, posy + offy, lwidth, lheight);
			if (bExtended)
			{
				l.setText("...");
			}
			else
			{
				l.setText("... (" + (line - 3) + " more)");
			}
			l.setBackColor(bgColor2);
			l.setCanTouch(true);
			
			labels.add(l);
			extlabel = l;
			gui.AddGuiObject(l);

			offy = offy - lheight;
		}
		
		if (bExtended && line > 3)
		{
			double totalHeight = (line+1) * lheight;
			double diff = (posy - totalHeight) + 1.0f;
			float sheight = (float)(totalHeight + diff);
			float perc = (float)((totalHeight + diff) / totalHeight);
			scrolldist = 0;
			scrollmaxdist = (float)diff;
			
			if (diff < 0)
			{
				
				scrollB = new JoglGuiScrollBar();
				scrollB.setCanTouch(true);
				scrollB.setPerc(perc);
				scrollB.setRect(posx + lwidth, posy - sheight + lheight, 0.025f, sheight);
				//scrollB.setScrollPerc(scrollperc);
				//scrollB.setLine(0.005f / aspect, -0.005f);
				//scrollB.setColor(scrollbarColor);
				scrollB.addScrollbarListener(this);
				
				gui.AddGuiObject(scrollB);
			}
		}
	}
	
	private JoglGuiScrollBar scrollB = null;
	private float scrolldist = 0.0f;
	private float scrollmaxdist = 0.0f;
	
	private void AddTransitionLabel(JSOPnTransition trans, FontFormat ff, float offy, JIViewManagerModel viewMan, JIPSENSelectModelElement jipsme)
	{
		int xoff = 1;
		
		JoglGuiLabel l = new JoglGuiLabel(ff.clone());
		l.setRect(posx, posy + offy, lwidth, lheight);
		l.setText(trans.getLabel());
		l.setBackColor(bgColor);
		
		labels.add(l);
		gui.AddGuiObject(l);
	
		//add deselect button
		if (true)
		{
			JoglGuiLabel lexp = new JoglGuiLabel(ff.clone());
			lexp.setRect((posx + lwidth) - (lbtnwidth * xoff), posy + offy, lbtnwidth, lheight);
			lexp.setText("X");
			lexp.setBackColor(bgColor);
			lexp.setCanTouch(true);
			
			labels.add(lexp);
			deselects.put(lexp, trans);
			gui.AddGuiObject(lexp);
			
			xoff++;
		}
		
		//if has subnet
		if (trans.hasSubnet())
		{
			JoglGuiLabel lexp = new JoglGuiLabel(ff.clone());
			lexp.setRect((posx + lwidth) - (lbtnwidth * xoff), posy + offy, lbtnwidth, lheight);
			lexp.setText("+");
			lexp.setBackColor(bgColor);
			lexp.setCanTouch(true);
	
			if (viewMan != null)
			{
				if (viewMan.IsSubNetExpanded(FindSubnet(trans)))
				{
					lexp.setText("-");
				}
				else
				{
					lexp.setText("+");
				}
			}
			
			labels.add(lexp);
			expands.put(lexp, trans);
			gui.AddGuiObject(lexp);	
			
			xoff++;
		}
		
		//add info button
		if (jipsme != null) //if has PSEN info
		{
			JoglGuiLabel lexp = new JoglGuiLabel(ff.clone());
			lexp.setRect((posx + lwidth) - (lbtnwidth * xoff), posy + offy, lbtnwidth, lheight);
			lexp.setText("i");
			lexp.setBackColor(bgColor);
			lexp.setCanTouch(true);
			
			labels.add(lexp);
			infos.put(lexp, trans);
			gui.AddGuiObject(lexp);	
			
			xoff++;
		}
	}
	
	private void AddPlaceLabel(JSOPnPlace place, FontFormat ff, float offy)
	{
		int xoff = 1;
		
		JoglGuiLabel l = new JoglGuiLabel(ff.clone());
		l.setRect(posx, posy + offy, lwidth, lheight);
		l.setText(place.orgPlace.getLabel());
		l.setBackColor(bgColor);
		
		labels.add(l);
		gui.AddGuiObject(l);

		//add deselect button
		if (true)
		{
			JoglGuiLabel lexp = new JoglGuiLabel(ff.clone());
			lexp.setRect((posx + lwidth) - (lbtnwidth * xoff), posy + offy, lbtnwidth, lheight);
			lexp.setText("X");
			lexp.setBackColor(bgColor);
			lexp.setCanTouch(true);
			
			labels.add(lexp);
			deselects.put(lexp, place);
			gui.AddGuiObject(lexp);
			
			xoff++;
		}
	}
	
	private void AddEdgeLabel(JSOPnEdge edge, FontFormat ff, float offy, JIPSENSelectModelElement jipsme)
	{
		int xoff = 1;

		JoglGuiLabel l = new JoglGuiLabel(ff.clone());
		l.setRect(posx, posy + offy, lwidth, lheight);
		String startObjLabel = edge.startObj.getOrgNode().getLabel();
		String endObjLabel = edge.endObj.getOrgNode().getLabel();
		l.setText(startObjLabel + " - " + endObjLabel);
		l.setBackColor(bgColor);
		
		labels.add(l);
		gui.AddGuiObject(l);
		
		//add deselect button
		if (true)
		{
			JoglGuiLabel lexp = new JoglGuiLabel(ff.clone());
			lexp.setRect((posx + lwidth) - (lbtnwidth * xoff), posy + offy, lbtnwidth, lheight);
			lexp.setText("X");
			lexp.setBackColor(bgColor);
			lexp.setCanTouch(true);
			
			labels.add(lexp);
			deselects.put(lexp, edge);
			gui.AddGuiObject(lexp);
			
			xoff++;
		}
	
		//add info button
		if (jipsme != null && edge.transitionPairID != null) //if has PSEN info
		{
			JoglGuiLabel lexp = new JoglGuiLabel(ff.clone());
			lexp.setRect((posx + lwidth) - (lbtnwidth * xoff), posy + offy, lbtnwidth, lheight);
			lexp.setText("i");
			lexp.setBackColor(bgColor);
			lexp.setCanTouch(true);
			
			labels.add(lexp);
			infos.put(lexp, edge);
			gui.AddGuiObject(lexp);	
			
			xoff++;
		}
	}
	
	private void AddEdgeVertexLabel(JSOPnEdgeVertex edgeV, FontFormat ff, float offy, JIPSENSelectModelElement jipsme)
	{
		int xoff = 1;

		JoglGuiLabel l = new JoglGuiLabel(ff.clone());
		l.setRect(posx, posy + offy, lwidth, lheight);
		String startObjLabel = edgeV.myEdge.startObj.getOrgNode().getLabel();
		String endObjLabel = edgeV.myEdge.endObj.getOrgNode().getLabel();
		String edgeLabel = startObjLabel + " - " + endObjLabel;
		l.setText("Edge (" + edgeLabel + ") Vertex " + edgeV.vertexId);
		l.setBackColor(bgColor);
		
		labels.add(l);
		gui.AddGuiObject(l);
		
		//add deselect button
		if (true)
		{
			JoglGuiLabel lexp = new JoglGuiLabel(ff.clone());
			lexp.setRect((posx + lwidth) - (lbtnwidth * xoff), posy + offy, lbtnwidth, lheight);
			lexp.setText("X");
			lexp.setBackColor(bgColor);
			lexp.setCanTouch(true);
			
			labels.add(lexp);
			deselects.put(lexp, edgeV);
			gui.AddGuiObject(lexp);
			
			xoff++;
		}
	}
	
	private void AddMagnetLineLabel(JSODataVisMagnetLineBase edge, FontFormat ff, float offy, JIPSENSelectModelElement jipsme)
	{
		int xoff = 1;

		JoglGuiLabel l = new JoglGuiLabel(ff.clone());
		l.setRect(posx, posy + offy, lwidth, lheight);
		l.setText(edge.getLabel());
		l.setBackColor(bgColor);
		
		labels.add(l);
		gui.AddGuiObject(l);
		
		//add deselect button
		if (true)
		{
			JoglGuiLabel lexp = new JoglGuiLabel(ff.clone());
			lexp.setRect((posx + lwidth) - (lbtnwidth * xoff), posy + offy, lbtnwidth, lheight);
			lexp.setText("X");
			lexp.setBackColor(bgColor);
			lexp.setCanTouch(true);
			
			labels.add(lexp);
			deselects.put(lexp, edge);
			gui.AddGuiObject(lexp);
			
			xoff++;
		}
		
		//add info button
		if (jipsme != null) //if has PSEN info
		{
			JoglGuiLabel lexp = new JoglGuiLabel(ff.clone());
			lexp.setRect((posx + lwidth) - (lbtnwidth * xoff), posy + offy, lbtnwidth, lheight);
			lexp.setText("i");
			lexp.setBackColor(bgColor);
			lexp.setCanTouch(true);
			
			labels.add(lexp);
			infos.put(lexp, edge);
			gui.AddGuiObject(lexp);	
			
			xoff++;
		}
	}

	public void OnScrollBarMoved(JoglGuiScrollBar sb, JoglVectord2 dir) 
	{
		double oldscrolldist = scrolldist;
		scrolldist = scrollB.getScrollPerc() * scrollmaxdist;
		
		MoveScrollbar(new JoglVectord2(0, scrolldist - oldscrolldist));
	}
	
	private void MoveScrollbar(JoglVectord2 d)
	{
		if (scrollB == null)
		{
			return;
		}
		
		JoglVectord4 diff = new JoglVectord4(0, -d.getY(), 0, 0);
		
		for (JoglGuiLabel l : labels)
		{
			l.setRect(JoglVectord4.Add(l.getRect(), diff));
		}
	}
}
