package org.processmining.processprofiler3d.compviz.jogl.interactions;

import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JISubNetInterpolate extends JoglInteraction
{
	private JoglInteractionManager iman;
	private JSOSubNet snet;
	
	private float factor = 0.0f;
	private float targetFactor = 0.0f;
	private float factorStepSize = 0.02f;
	
	public float getTargetFactor()
	{
		return targetFactor;
	}
	
	public void SetTarget(float target)
	{
		targetFactor = target;
		bUpdateScene = true;
	}
	
	public void SetSpeed(float speed)
	{
		factorStepSize = speed;
	}
	
	public JISubNetInterpolate(JoglInteractionManager iman, JSOSubNet snet)
	{
		this.iman = iman;
		this.snet = snet;
	}	
	
	public void Dispose()
	{
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{	
		if (targetFactor == 1.0f && factor <= targetFactor)
		{
			UpdateVis(scene);
			double oldfactor = factor;
			factor = Math.min(targetFactor, factor + factorStepSize);
			
			//update layer layout when factor reaches 1.0! Important so that open subnets of a currently opening subnet affect their layer...
			if (factor != oldfactor && factor == targetFactor)
			{
				JILayerManager jlm = iman.findInteraction(JILayerManager.class);
				if (jlm != null)
				{
					jlm.UpdateLayers(snet.getLevel());
				}
			}
		}
		else if (targetFactor == 0.0f && factor >= targetFactor)
		{
			UpdateVis(scene);
			factor = Math.max(targetFactor, factor - factorStepSize);
		}
		else
		{
			bUpdateScene = false;
		}
	}
	
	public void UpdateVis(JoglScene scene)
	{
		if (factor == 0.0)
		{
			if (snet.getIsVisible())
			{
				snet.setIsVisible(false);
				JILayerManager jlm = iman.findInteraction(JILayerManager.class);
				if (jlm != null)
				{
					jlm.UpdateLayers(snet.getLevel());
				}
			}
		}
		else
		{
			if (!snet.getIsVisible())
			{
				snet.setIsVisible(true);
				JILayerManager jlm = iman.findInteraction(JILayerManager.class);
				if (jlm != null)
				{
					jlm.UpdateLayers(snet.getLevel());
				}
			}
		}
		
		//calculate subnet offset given varying scale
		JoglVectord3 center = snet.GetPetrinetCenter();
		JoglVectord3 vHeight = JSOPetriNet.subnetOffset; //needs to scale with scale already, so no extra scaling needed
		JoglVectord3 morigin = JoglVectord3.Add(vHeight, JoglVectord3.Multiply(center, -1));
		JoglVectord3 total = JoglVectord3.Multiply(snet.getOrigin(), 1.0 / factor); //origin offset is in space of parent model so need to scale accordingly
		
		snet.setOffset(JoglVectord3.Add(total, morigin));
		snet.setScale(factor);
		bUpdateScene = true;
	}

}
