package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredFrame;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiTextbox;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.input.JoglInputKeyEvent;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.interactions.JoglGuiInteraction;
import org.processmining.visualisation3d.maths.JoglColor3;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JITextboxInput extends JoglGuiInteraction
{	
	protected JoglInteractionManager iman;
	protected JoglScene scene;

	protected boolean bUpdateScene = false;
	
	protected JoglGuiTextbox myLabel;
	protected JoglGuiColoredFrame frame = null;
	
	protected int counter = 0;
	protected boolean bShowCursor = true;
	protected String labelText = "";
	
	public ITextboxInputListener listener = null;
	
	public JoglGuiTextbox getLabel()
	{
		return myLabel;
	}
	
	public String getText()
	{
		return myLabel.getText();
	}
	
	public JITextboxInput(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui, JoglGuiTextbox label)
	{
		super(gui);
		
		this.scene = scene;
		this.iman = iman;
		this.myLabel = label;
		
		labelText = label.getText();
		
		setZLevel(-2000.0f);
		
		frame = new JoglGuiColoredFrame();
		frame.setRect(label.getRect());
		frame.setColor(JoglColor3.WHITE);
		frame.setLine(0.005f / (float)gui.getAspect(), 0.005f);
		
		gui.AddGuiObject(frame);
	}
	
	public void Dispose()
	{
		gui.RemoveGuiObject(frame);
	}
	
	@Override
	public boolean ShouldAnimate()
	{
		//return bUpdateScene;
		return true;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		counter++;
		if (counter > 15)
		{
			counter = 0;
			bShowCursor = !bShowCursor;
			
			updateLabelText();
		}
		
		bUpdateScene = false;
    }
	
	protected void updateLabelText()
	{
		String cursorStr = "";
		if (bShowCursor)
		{
			cursorStr = "|";
		}
		myLabel.setText(labelText + cursorStr);
		
	}

	@Override
	public boolean keyPressed(JoglInputKeyEvent evt) 
	{
		return true;
	}

	@Override
	public boolean keyReleased(JoglInputKeyEvent evt) 
	{
		if (evt.getKeyCode() == KeyEvent.VK_ENTER)
		{
			Close();
		}
		return true;
	}
	
	@Override
	public boolean keyTyped(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt); 
		
        //backspace
        if (evt.getKeyChar() == '\b')
        {
        	if (labelText.length() > 0)
        	{
        		labelText = labelText.substring(0, labelText.length()-1);
    			updateLabelText();
        	}
        }
        
        if (!isValidChar(evt.getKeyChar()))
        {
        	return true;
        }
        
		labelText = labelText + evt.getKeyChar();
		updateLabelText();
		
		return true;
	}
	
	protected boolean isValidChar(char c)
	{
		int idx = ((int)c) - 32;
		return idx > 0 && idx < 255;
	}
	
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			JoglVectord2 newPos = gui.GetNDCPos(e.getX(), e.getY());
			if (frame.IsInside((float)newPos.getX(), (float)newPos.getY()))
			{			
				return true;			
			}
			else
			{
				Close();
			}
		}
		return false;
    }

	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			JoglVectord2 newPos = gui.GetNDCPos(e.getX(), e.getY());
			if (frame.IsInside((float)newPos.getX(), (float)newPos.getY()))
			{			
				return true;			
			}
			else
			{
				Close();
			}
		}
		return false;
    }

	//MouseListener method implementation
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {
		//System.out.println("Mouse click at: " + e.getX() + "/" + e.getY());
		
		return false;
    }
	
	public void Close()
	{
		//hide cursor and commit final text to label
		bShowCursor = false;
		updateLabelText();
		
		Dispose();
		iman.removeInteraction(this);
		
		if (listener != null)
		{
			listener.OnTextInputFinished(this);
		}
	}
}
