package org.processmining.processprofiler3d.compviz.jogl.interactions;

import org.processmining.processprofiler3d.compviz.jogl.interactions.camera.JICameraVR;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOCube;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOVRController;
import org.processmining.processprofiler3d.compviz.vr.HzdVRControllerState;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.interactions.JICameraBase;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIVRControllerInteraction extends JoglInteraction
{
	private JoglInteractionManager iman;
	private JSOVRController myController;
	private JSOCube myCube = null;
	
	public JIVRControllerInteraction(JoglInteractionManager iman, JSOVRController c)
	{
		this.iman = iman;
		myController = c;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		//System.out.println("Camera update scene");
		
        double moveStep = 2 * JIOpenVR.vrScale;
        double turnStep = 0.1;
//
//        if (bShiftPressed)
//        {
//        	moveStep = 10 * JIOpenVR.vrScale;
//        }
        
//		if (bTurnLeftPressed)
//		{
//			camDx = camDx - turnStep;
//		}
//		if (bTurnRightPressed)
//		{
//			camDx = camDx + turnStep;
//		}
//		
//		if (camDx != 0 || camDy != 0)
//		{
//
//	        RotateCamera(camDx, camDy);
//	        
//	        camDx = 0.0;
//	        camDy = 0.0;
//		}
//		
//		if (bForwardPressed || bForward2Pressed)
//		{
//	    	JoglVectord3 ldir = JoglVectord3.Normal(hmdForward);
//	    	JoglVectord3 camMove = JoglVectord3.Multiply(ldir, moveStep);
//	    	
//	    	cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), camMove));
//	    	cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), camMove));
//		}
//		if (bBackwardPressed || bBackward2Pressed)
//		{
//        	JoglVectord3 ldir = JoglVectord3.Normal(hmdForward);
//	    	JoglVectord3 camMove = JoglVectord3.Multiply(ldir, -moveStep);	
//        	
//        	cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), camMove));
//        	cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), camMove));
//		}
//		if (bLeftPressed || bLeft2Pressed)
//		{
//        	JoglVectord3 leftdir = hmdSide;
//	    	JoglVectord3 camMove = JoglVectord3.Multiply(leftdir, -moveStep);
//        	
//        	cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), camMove));
//        	cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), camMove));			
//		}
//		if (bRightPressed || bRight2Pressed)
//		{
//        	JoglVectord3 leftdir = hmdSide;
//	    	JoglVectord3 camMove = JoglVectord3.Multiply(leftdir, moveStep);
//        	
//        	cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), camMove));
//        	cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), camMove));			
//		}
		
		HzdVRControllerState vcs = myController.GetControllerState();
		if (vcs != null)
		{
			int cnum = myController.controllerId;
			boolean bEnableTouchpadMove = false;
			if (vcs.IsAppMenuPressed())
			{
				System.out.println("Controller " + cnum + ": AppMenu pressed!");
				
				if (myCube == null)
				{
					myCube = new JSOCube();
					myCube.setPosition(myController.getPosition());
					
					scene.addSceneObject(myCube);
				}
				
			}
			else
			{
				if (myCube != null)
				{
					scene.removeSceneObject(myCube);
					myCube = null;
				}
			}
			if (vcs.IsGripPressed())
			{
				System.out.println("Controller " + cnum + ": Grip pressed!");

	        	bEnableTouchpadMove = true;
			}
			if (vcs.IsTriggerPressed())
			{
				System.out.println("Controller " + cnum + ": Trigger pressed!");
			}
			if (vcs.IsTouchpadPressed())
			{
				System.out.println("Controller " + cnum + ": Touchpad pressed!");
				
	        	bEnableTouchpadMove = true;
			}
			
			if (bEnableTouchpadMove)
			{
				JICameraBase icam = iman.findInteraction(JICameraVR.class);
				JoglCamera cam = icam.getCamera();
				
		        if (vcs.IsTouchpadPressed() && vcs.IsGripPressed())
		        {
		        	moveStep = 10 * JIOpenVR.vrScale;
		        }
		        
//		        JoglVectord3 ldir = JoglVectord3.Normal(hmdForward); 
//		        JoglVectord3 leftdir = hmdSide;
	        	JoglVectord3 ldir = JoglVectord3.Normal(myController.getForward()); 
	        	JoglVectord3 leftdir = myController.getLeft();
	        	
				JoglVectord3 move = JoglVectord3.Add(JoglVectord3.Multiply(ldir, moveStep * vcs.getTouchpadPos().getY()), JoglVectord3.Multiply(leftdir, moveStep * vcs.getTouchpadPos().getX()));
		        cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), move));
		        cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), move));
			}
		}
		
	}

}
