package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.awt.event.MouseEvent;
import java.util.prefs.Preferences;

import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiTexturedRect;
import org.processmining.processprofiler3d.compviz.jogl.input.PP3DKeyBindings;
import org.processmining.processprofiler3d.compviz.jogl.interactions.camera.JICameraDemo;
import org.processmining.processprofiler3d.compviz.jogl.interactions.camera.JICameraInterpolator;
import org.processmining.processprofiler3d.compviz.jogl.interactions.camera.JICameraOrbit;
import org.processmining.processprofiler3d.compviz.jogl.interactions.camera.JICameraVR;
import org.processmining.processprofiler3d.compviz.vr.IVRModeListener;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.gldatastructures.JoglTextureDelayedLoad;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputKeyEvent;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.interactions.JICameraBase;
import org.processmining.visualisation3d.interactions.JICameraFree;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIViewManager extends JoglInteraction implements IVRModeListener
{
	protected JoglInteractionManager iman;
	protected JoglScene scene;
	protected JSOGraphicalUserInterface gui;
	
	protected boolean bInit = true;

	protected JoglTexture icoCamDemo;
	protected JoglTexture icoCamFree;
	protected JoglTexture icoCamOrbit;
	//private JoglGuiColoredRect btnCam;
	protected JoglGuiTexturedRect btnCam;
	
	protected int initCameraMode = 0;
	protected JoglVectord3 initCameraPos = JoglVectord3.ZERO;
	protected JoglVectord3 initCameraDir = JoglVectord3.ZERO;
	protected JoglVectord3 initCameraDirU = JoglVectord3.ZERO;
	
	public JIViewManager(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui)
	{
		this.iman = iman;
		this.scene = scene;
		this.gui = gui;
		
		setZLevel(-200.0f);
		
		//load text size setting
		String baseTextSize = "1.25";
		Preferences prefs = Preferences.userNodeForPackage(JIPSENView.class);
		baseTextSize = prefs.get("TextSize", baseTextSize);
		JoglGuiStyle.textScale = Double.parseDouble(baseTextSize);
	}
	
	public void Init()
	{	
//		btnCam = new JoglGuiColoredRect();
//		btnCam.setRect(-0.8f, 0.8f, 0.1f / (float)gui.getAspect(), 0.1f);
//		btnCam.setColor(JoglColor3.RED);
//		btnCam.setCanTouch(true);
//		
		icoCamDemo = new JoglTextureDelayedLoad(scene, "/org/processmining/processprofiler3d/compviz/jogl/resources/IcoCameraDemoA.tga");
		icoCamFree = new JoglTextureDelayedLoad(scene, "/org/processmining/processprofiler3d/compviz/jogl/resources/IcoCameraFreeA.tga");
		icoCamOrbit = new JoglTextureDelayedLoad(scene, "/org/processmining/processprofiler3d/compviz/jogl/resources/IcoCameraOrbitA.tga");
		JoglResourceManager.AddResource("IconCameraDemo", icoCamDemo);
		JoglResourceManager.AddResource("IconCameraFree", icoCamFree);
		JoglResourceManager.AddResource("IconCameraOrbit", icoCamOrbit);
		
		btnCam = new JoglGuiTexturedRect();
		btnCam.setRect(-0.875f, 0.825f, 0.125f / (float)gui.getAspect(), 0.125f);
		btnCam.setCanTouch(true);
		
		JICameraBase camInteraction = getCameraInteraction();
		updateCamButtonTex(camInteraction);
		
		initCameraMode = getCameraModelId();
		initCameraPos = camInteraction.getCamera().GetPosition();
		initCameraDir = camInteraction.getCamera().GetLookDir();
		initCameraDirU = camInteraction.getCamera().GetUpDir();
		
		gui.AddGuiObject(btnCam);
		
		bInit = false;
	}
	
	private void updateCamButtonTex(JICameraBase camInteraction)
	{
		if (btnCam != null)
		{
			if (camInteraction instanceof JICameraOrbit)
			{
				btnCam.setTex(icoCamOrbit);
			}
			else if (camInteraction instanceof JICameraDemo)
			{
				btnCam.setTex(icoCamDemo);		
			}
			else if (camInteraction instanceof JICameraFree)
			{
				btnCam.setTex(icoCamFree);		
			}
		}
	}
	
	public void Dispose()
	{
		gui.RemoveGuiObject(btnCam);
		btnCam = null;
	}
	
	protected boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{	
		if (bInit)
		{
			Init();
		}
		
		bUpdateScene = false;
	}
	
	public JICameraBase getCameraInteraction()
	{
		JICameraOrbit jcio = iman.findInteraction(JICameraOrbit.class);
		if (jcio != null)
			return jcio;
		
		JICameraFree jio = iman.findInteraction(JICameraFree.class);
		if (jio != null)
			return jio;
		
		JICameraDemo jiod = iman.findInteraction(JICameraDemo.class);
		if (jiod != null)
			return jiod;
		
		JICameraVR jivr = iman.findInteraction(JICameraVR.class);
		if (jivr != null)
			return jivr;
		
		return null;
	}
	
	public String getCameraModelName()
	{
		JICameraOrbit jcio = iman.findInteraction(JICameraOrbit.class);
		if (jcio != null)
			return "Orbit";
		
		JICameraFree jio = iman.findInteraction(JICameraFree.class);
		if (jio != null)
			return "Free";
		
		JICameraDemo jiod = iman.findInteraction(JICameraDemo.class);
		if (jiod != null)
			return "Demo";
		
		JICameraVR jivr = iman.findInteraction(JICameraVR.class);
		if (jivr != null)
			return "VR";
		
		return "None";
	}
	
	public int getCameraModelId()
	{
		JICameraOrbit jcio = iman.findInteraction(JICameraOrbit.class);
		if (jcio != null)
			return 1;
		
		JICameraFree jio = iman.findInteraction(JICameraFree.class);
		if (jio != null)
			return 0;
		
		JICameraDemo jiod = iman.findInteraction(JICameraDemo.class);
		if (jiod != null)
			return 2;
		
		JICameraVR jivr = iman.findInteraction(JICameraVR.class);
		if (jivr != null)
			return 3;
		
		return -1;
	}
	
	//KeyListener method implementations
	@Override
	public boolean keyPressed(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt);
        
		if (evt.getKeyCode() == PP3DKeyBindings.SwitchCameraMode) //c-key
		{
			SwitchCameraMode();

        	bUpdateScene = true;
		}
		
		return false;
	}
	
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MousePressed");
			JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
			if (jgo == btnCam)
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MouseReleased");
			JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
			if (jgo == btnCam)
			{
				SwitchCameraMode();
	        	bUpdateScene = true;
				
				return true;
			}
		}
		return false;
    }
	
	public void ResetCamera()
	{
		JICameraBase camInteraction = getCameraInteraction();
		camInteraction.getCamera().SetPosition(initCameraPos);
		camInteraction.getCamera().SetLookDir(initCameraDir, initCameraDirU);
		
		SwitchCameraMode(initCameraMode);
	}
	
	public void SwitchCameraMode()
	{
		JICameraBase camInteraction = getCameraInteraction();
		if (camInteraction != null)
		{
			JICameraInterpolator jici = iman.findInteraction(JICameraInterpolator.class);
			if (jici != null)
			{
				iman.removeInteraction(jici);
			}
			
			iman.removeInteraction(camInteraction);
			
			if (camInteraction instanceof JICameraOrbit)
			{
				camInteraction = SwitchToCameraModeDemo(camInteraction.getCamera());
			}
			else if (camInteraction instanceof JICameraDemo)
			{
				camInteraction = SwitchToCameraModeFree(camInteraction.getCamera());
			}
			else if (camInteraction instanceof JICameraFree)
			{
				camInteraction = SwitchToCameraModeOrbit(camInteraction.getCamera());
			}
			else if (camInteraction instanceof JICameraVR)
			{
				camInteraction = SwitchToCameraModeFree(camInteraction.getCamera());
			}
			
			updateCamButtonTex(camInteraction);
		}
	}
	
	public void SwitchCameraMode(int mode)
	{
		JICameraBase camInteraction = getCameraInteraction();
		if (camInteraction != null)
		{
			JICameraInterpolator jici = iman.findInteraction(JICameraInterpolator.class);
			if (jici != null)
			{
				iman.removeInteraction(jici);
			}
			
			iman.removeInteraction(camInteraction);
			
			if (mode == 0)
			{
				camInteraction = SwitchToCameraModeFree(camInteraction.getCamera());
			}
			else if (mode == 1)
			{
				camInteraction = SwitchToCameraModeOrbit(camInteraction.getCamera());
			}
			else if (mode == 2)
			{
				camInteraction = SwitchToCameraModeDemo(camInteraction.getCamera());
			}
			else if (mode == 3)
			{
				camInteraction = SwitchToCameraModeVR(camInteraction.getCamera());
			}
			
			updateCamButtonTex(camInteraction);
		}
	}
	
	public JICameraBase SwitchToCameraModeFree(JoglCamera cam)
	{
		JICameraFree jcf = new JICameraFree(cam);
		iman.addInteraction(jcf);

		JoglCamera scam = cam;
		
		jcf.setPitchYawRoll(scam.GetPitch(), scam.GetYaw(), 0);
    	
    	return jcf;
	}
	
	public JICameraBase SwitchToCameraModeOrbit(JoglCamera cam)
	{
		JICameraOrbit jco = new JICameraOrbit(cam);
		iman.addInteraction(jco);

		JoglCamera scam = cam;
		
    	//calculate intersection point of current camera lookdir with ground plane
    	double diry = scam.GetLookDir().getY();
    	
    	if (diry != 0.0)
    	{
        	double disty = -scam.GetPosition().getY();
        	JoglVectord3 v = JoglVectord3.Add(scam.GetPosition(), JoglVectord3.Multiply(scam.GetLookDir(), disty / diry));
        	jco.setCamFocus(v, scam.GetPosition());
        	
        	//System.out.println("camFocusY: " + v.getY());
    	}
    	else
    	{
        	double dist = 500.0;
        	JoglVectord3 v = JoglVectord3.Add(scam.GetPosition(), JoglVectord3.Multiply(scam.GetLookDir(), dist));
        	jco.setCamFocus(v, scam.GetPosition());
    	}
		
		return jco;
	}
	
	public JICameraBase SwitchToCameraModeDemo(JoglCamera cam)
	{
		JICameraDemo jco = new JICameraDemo(iman, cam, scene);
		iman.addInteraction(jco);
		
		JoglCamera scam = cam;
    	
    	//calculate intersection point of current camera lookdir with ground plane
    	double diry = scam.GetLookDir().getY();
    	
    	if (diry != 0.0)
    	{
        	double disty = -scam.GetPosition().getY();
        	JoglVectord3 v = JoglVectord3.Add(scam.GetPosition(), JoglVectord3.Multiply(scam.GetLookDir(), disty / diry));
        	jco.setCamFocus(v, scam.GetPosition());
        	
        	//System.out.println("camFocusY: " + v.getY());
    	}
    	else
    	{
        	double dist = 500.0;
        	JoglVectord3 v = JoglVectord3.Add(scam.GetPosition(), JoglVectord3.Multiply(scam.GetLookDir(), dist));
        	jco.setCamFocus(v, scam.GetPosition());
    	}
		
		return jco;
	}
	
	public JICameraBase SwitchToCameraModeVR(JoglCamera cam)
	{
		JICameraVR jcf = new JICameraVR(cam);
		iman.addInteraction(jcf);

		JoglCamera scam = cam;
		
		jcf.setPitchYawRoll(scam.GetPitch(), scam.GetYaw(), 0);
    	
    	return jcf;
	}

	public void OnVRModeActivated()
	{	
		SwitchCameraMode(3); //switch to VR camera control
	}

	public void OnVRModeDeactivated()
	{
		SwitchCameraMode(); //switch to Free camera control
	}
}
