package org.processmining.processprofiler3d.compviz.jogl.interactions;

import java.util.HashMap;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.input.PP3DKeyBindings;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOGrid;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglModelScene;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.input.JoglInputKeyEvent;
import org.processmining.visualisation3d.interactions.JICameraBase;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;

public class JIViewManagerModel extends JIViewManager
{
	protected JoglModelScene modelScene;
	
	protected HashMap<JSOSubNet, JISubNetInterpolate> map = new HashMap<JSOSubNet, JISubNetInterpolate>();
	
	public JIViewManagerModel(JoglInteractionManager iman, JoglModelScene scene, JSOGraphicalUserInterface gui)
	{
		super(iman, scene, gui);

		modelScene = scene;
	}
	
	//KeyListener method implementations
	@Override
	public boolean keyPressed(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt);
        
		if (evt.getKeyCode() == PP3DKeyBindings.SwitchCameraMode) //c-key
		{
			SwitchCameraMode();

        	bUpdateScene = true;
		}
		if (evt.getKeyCode() == PP3DKeyBindings.ToggleFloatingLabels) //f-key
		{
			SwitchFloatingLabels();

        	bUpdateScene = true;
		}
		if (evt.getKeyCode() == PP3DKeyBindings.ToggleGrid) //g-key
		{
			SwitchGrid();

        	bUpdateScene = true;
		}
		if (evt.getKeyCode() == PP3DKeyBindings.ToggleSubprocess) //n-key
		{
			JISelectModelElement jsme = iman.findInteraction(JISelectModelElement.class);
			if (jsme != null)
			{			
				for (JSOSubNet snet : scene.findSceneObject(JSOPetriNet.class).getAllSubNets())
				{
					if (snet.getParent() != null && jsme.getSelection().contains(snet.getParent()))
					{
						ToggleSubNet(snet, 0.02f);
					}
				}
	        	bUpdateScene = true;
			}
		}
		
		return false;
	}
	
	public String getGridState()
	{
		JSOGrid grid = scene.findSceneObject(JSOGrid.class);
		if (grid != null)
		{
			if (grid.getVisible())
			{
				return "On";
			}
			else
			{
				return "Off";
			}
		}
		return "None";
	}
	
	public String getFloatingLabelState()
	{
		JIFloatingLabels jifl = iman.findInteraction(JIFloatingLabels.class);
		if (jifl != null)
		{
			return "On";
		}
		else
		{
			return "Off";
		}
	}
	
	public boolean getShowFloatingLabels()
	{
		JIFloatingLabels jifl = iman.findInteraction(JIFloatingLabels.class);
		if (jifl != null)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public void SwitchFloatingLabels()
	{
		JIFloatingLabels jifl = iman.findInteraction(JIFloatingLabels.class);
		if (jifl != null)
		{
			iman.removeInteraction(jifl);
			jifl.Dispose();
		}
		else
		{
			JSOPetriNet pnet = scene.findSceneObject(JSOPetriNet.class);
			jifl = new JIFloatingLabels(iman, scene, gui, pnet);
			
			iman.addInteraction(jifl);
		}
	}
	
	public void SwitchGrid()
	{
		Vector<JSOGrid> grids = scene.findSceneObjects(JSOGrid.class);
		for (JSOGrid grid : grids)
		{
			grid.setVisible(!grid.getVisible());
		}	
	}

	public void ToggleSubNet(JSOSubNet snet, float speed) 
	{
		//snet.setIsVisible(!snet.getIsVisible());
		if (map.containsKey(snet))
		{
			JISubNetInterpolate sni = map.get(snet);
			//panel.addInteraction(sni);
			if (sni.getTargetFactor() == 0.0f)
			{
				sni.SetTarget(1.0f);
			}
			else
			{
				sni.SetTarget(0.0f);
			}
			sni.SetSpeed(speed);
		}
		else
		{
			JISubNetInterpolate sni = new JISubNetInterpolate(iman, snet);
			iman.addInteraction(sni);
			sni.SetTarget(1.0f);
			sni.SetSpeed(speed);
			
			map.put(snet, sni);
		}
	}

	public boolean IsSubNetExpanded(JSOSubNet snet) 
	{
		//snet.setIsVisible(!snet.getIsVisible());
		if (map.containsKey(snet))
		{
			JISubNetInterpolate sni = map.get(snet);
			//panel.addInteraction(sni);
			if (sni.getTargetFactor() == 0.0f)
			{
				return false;
			}
			else
			{
				return true;
			}
		}
		else
		{
			return false;
		}
	}

	@Override
	public void OnVRModeActivated()
	{
		//JoglCamera c = panel.getCamera();
		JICameraBase camInteraction = getCameraInteraction();
		JoglCamera c = camInteraction.getCamera();
		
		//scale the scene to be at reasonable real-world size
		modelScene.getPetriNet().getRootSubNet().setScale(JIOpenVR.vrScale);
		
		//scale camera position with the rest of the scene
		JoglVectord3 cpos = JoglVectord3.Multiply(c.GetPosition(), JIOpenVR.vrScale);
		JoglVectord3 clookat = JoglVectord3.Multiply(c.GetLookAt(), JIOpenVR.vrScale);	
		
		c.SetPosition(cpos);
		c.SetLookAt(clookat);
		
		SwitchCameraMode(3); //switch to VR camera control
	}

	@Override
	public void OnVRModeDeactivated()
	{
		//JoglCamera c = panel.getCamera();
		JICameraBase camInteraction = getCameraInteraction();
		JoglCamera c = camInteraction.getCamera();
		
		//scale the scene to be at reasonable real-world size
		modelScene.getPetriNet().getRootSubNet().setScale(1.0);
		
		//scale camera position with the rest of the scene
		JoglVectord3 cpos = JoglVectord3.Multiply(c.GetPosition(), 1.0 / JIOpenVR.vrScale);
		JoglVectord3 clookat = JoglVectord3.Multiply(c.GetLookAt(), 1.0 / JIOpenVR.vrScale);	
		
		c.SetPosition(cpos);
		c.SetLookAt(clookat);
		
		SwitchCameraMode(); //switch to Free camera control
	}

}
