package org.processmining.processprofiler3d.compviz.jogl.interactions.camera;

import java.awt.event.MouseEvent;
import java.util.Collections;
import java.util.Date;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import org.processmining.processprofiler3d.compviz.jogl.input.PP3DKeyBindings;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglModelScene;
import org.processmining.processprofiler3d.netstructure.JSOPnTransitionSorter;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.input.JoglInputKeyEvent;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.interactions.JICameraBase;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JICameraDemo extends JICameraBase
{
	private JoglInteractionManager iman;
	private JoglScene scene = null;
	//public JSOPetriNet net = null;
	
	private boolean bForwardPressed = false;
	private boolean bBackwardPressed = false;
	private boolean bLeftPressed = false;
	private boolean bRightPressed = false;
	private boolean bForward2Pressed = false;
	private boolean bBackward2Pressed = false;
	private boolean bLeft2Pressed = false;
	private boolean bRight2Pressed = false;
	private boolean bZoomInPressed = false;
	private boolean bZoomOutPressed = false;
	private boolean bShiftPressed = false;

    private double minDy = 0.0;
    private double maxDy = (Math.PI * 0.5) - 0.001;
    
	private double camDx = 0.0;
	private double camDy = 0.0;

	//private double cPitch = Math.PI * 0.5;
	//private double cYaw = Math.PI;
 
    private boolean bMouseView = false;
    private JoglVectord2 lastMousePos;
    
    private JoglVectord3 camFocus = new JoglVectord3(0,0,0);
	private double camDist = 500.0;
	
	private JSOPnTransition currentTarget;
	
	private static Vector<JSOPnTransition> traversalList = null;
	
//    private double camTurnFactorX = 0.0025;
//    private double camTurnFactorY = 0.0025;
    private double camTurnFactorX = 2.775;
    private double camTurnFactorY = 2.775;
	
	public JICameraDemo(JoglInteractionManager iman, JoglCamera c, JoglScene scene)
	{
		super(c);
		
		this.iman = iman;
		this.scene = scene;

		if (traversalList == null)
		{
			Date startTime = new Date();
			
			updateTraversalList(scene);
			
			Date endTime = new Date();
			TimeSpan ts = new TimeSpan(startTime, endTime);
			
			System.out.println("Traversal sorted (" + ts.ToString(TimeUnit.MILLISECONDS) + "ms).");
			
		}
	}

	public void updateTraversalList(JoglScene scene) 
	{
		JoglModelScene jms = (JoglModelScene)scene;
		JSOPetriNet jsopn = jms.getPetriNet();
		Vector<JSOSubNet> subnets = jsopn.getAllSubNets();
		
		traversalList = new Vector<JSOPnTransition>();
		
		for (JSOSubNet sn : subnets)
		{
			Vector<JSOPnTransition> ts = sn.getTransitions();
		
			//sort somehow
			//ts.sort(new JSOPnTransitionSorter(sn));	//Java 1.8
			Collections.sort(ts, new JSOPnTransitionSorter(sn));
			
			//insert the sorted nodes of each subnet behind its parent node
			if (sn.getParent() == null)
			{
				traversalList.addAll(ts);
			}
			else
			{
				int parentidx = traversalList.indexOf(sn.getParent());
				for (int i = 0; i < ts.size(); i++)
				{
					traversalList.insertElementAt(ts.get(i), parentidx + i + 1);
				}
			}
		}
	}
	
	public void setCamFocus(JoglVectord3 focus)
	{
		JICameraInterpolator jici = iman.findInteraction(JICameraInterpolator.class);
		if (jici != null)
		{
			jici.setTargetCamFocus(focus);
		}
		else
		{
			jici = new JICameraInterpolator(iman, cam);
			jici.setZLevel(this.getZLevel()+1);
			iman.addInteraction(jici);
			
			//jici.initCamFocus(camFocus, cam.GetPosition());
			jici.initCamFocus(camFocus, camDist, camDx, camDy);
			jici.setTargetCamFocus(focus);
		}
		
		camFocus = focus;
	}
	
	public void setCamFocus(JoglVectord3 focus, JoglVectord3 pos)
	{
		JoglVectord3 dir = JoglVectord3.Subtract(focus, pos);
		JoglVectord3 dirN = JoglVectord3.Normal(dir);
		JoglVectord3 side = JoglVectord3.Cross(dirN, new JoglVectord3(0,1,0));
		
		camFocus = focus;
		camDist = dir.getMagnitude();
		
		double dotx = dirN.getY(); //JoglVectord3.Dot(dirN, new JoglVectord3(0,1,0));
		double doty = side.getZ(); //JoglVectord3.Dot(side, new JoglVectord3(0,0,1));

		camDy = -Math.asin(dotx);
		camDy = Math.max(minDy, Math.min(maxDy, camDy));
		
    	if (side.getX() > 0)
    	{
    		camDx = -Math.asin(doty);
    	}
    	else
    	{
    		camDx = Math.PI + Math.asin(doty);
    	}
	}
	
	private boolean IsKeyPressed()
	{
		if (bForwardPressed || bBackwardPressed || bLeftPressed || bRightPressed)
		{
	    	return true;
		}
		if (bForward2Pressed || bBackward2Pressed || bLeft2Pressed || bRight2Pressed)
		{
	    	return true;
		}
		if (bZoomInPressed || bZoomOutPressed)
		{
			return true;
		}
		if (bMouseView)
		{
			return true;
		}
		return false;
	}

	@Override
	public boolean ShouldAnimate()
	{
		return IsKeyPressed();
	}

	@Override
	public void UpdateScene(JoglScene scene)
	{
		//System.out.println("Camera update scene");
		
        double zoomStep = 10;
        double turnStep = 0.1;

        if (bShiftPressed)
        {
        	zoomStep = 50;
        }
        
//        if (net != null)
//        {
//        	camFocus = net.GetPetrinetCenter();
//        }
        
		if (bZoomInPressed)
		{
			camDist = camDist - zoomStep;
		}
		if (bZoomOutPressed)
		{
			camDist = camDist + zoomStep;
		}
//		if (bForwardPressed || bForward2Pressed)
//		{
//			camDy = camDy - turnStep;
//		}
//		if (bBackwardPressed || bBackward2Pressed)
//		{
//			camDy = camDy + turnStep;
//		}
        
		camDist = Math.max(0.1, camDist);
    	JICameraInterpolator jici = iman.findInteraction(JICameraInterpolator.class);
    	if (jici != null)
    	{
    		jici.setCamDist(camDist);
    	}
    	
        camDy = Math.max(minDy, Math.min(maxDy, camDy));
		
//		if (bLeftPressed || bLeft2Pressed)
//		{
//			camDx = camDx - turnStep;		
//		}
//		if (bRightPressed || bRight2Pressed)
//		{
//			camDx = camDx + turnStep;		
//		}
        
		//make camera orbit around camFocus at a distance of camDist
        CameraOrbit();
	}

	private void CameraOrbit() 
	{
        cam.SetUpDir(new JoglVectord3(0,1,0));
        
        double tx = Math.sin(camDx) * Math.cos(camDy);
        double ty = Math.cos(camDx) * Math.cos(camDy);
        double tz = Math.sin(camDy);
		JoglVectord3 cp = new JoglVectord3(tx, tz, ty);
		cp = JoglVectord3.Normal(cp);
		cp = JoglVectord3.Multiply(cp, camDist);
		cam.SetPosition(JoglVectord3.Add(camFocus, cp));
		cam.SetLookAt(camFocus);
	}

	//KeyListener method implementations
	@Override
	public boolean keyPressed(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt);
        
        if (evt.getKeyCode() == PP3DKeyBindings.Camera_Down) //s key
        {
        	bBackwardPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Down2) //down key
        {
        	bBackward2Pressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Up) //w key
        {
        	bForwardPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Up2) //up key
        {
        	bForward2Pressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Left) //a key
        {
        	bLeftPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Left2) //left key
        {
        	bLeft2Pressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Right) //d key
        {
        	bRightPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Right2) //right key
        {
        	bRight2Pressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_RotateLeft) //q key
        {
        	//RotateCamera(0.1, 0);
        	bZoomInPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_RotateRight) //e key
        {
        	//RotateCamera(-0.1, 0);
        	bZoomOutPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Run) //shift key
        {
        	bShiftPressed = true;
        }
        
		if (bLeftPressed || bLeft2Pressed)
		{
			//next target
			setNextCamTarget();
			
		}
		if (bRightPressed || bRight2Pressed)
		{
			//prev target	
			setPrevCamTarget();
		}
        
        return false;
	}

	public void setNextCamTarget() 
	{
		if (scene instanceof JoglModelScene)
		{
			Vector<JSOPnTransition> tsVis = getVisibleTransitons();
			
			int id = -1;
			if (currentTarget != null)
			{
				id = tsVis.indexOf(currentTarget);
			}
			id = (id + 1) % tsVis.size();
			
			currentTarget = tsVis.get(id);
			
			JoglVectord3 targetPos = currentTarget.getPosition();
			if (currentTarget.getParentNet() != null)
			{
				targetPos = currentTarget.getParentNet().relativeToAbsolute(targetPos);
			}
			setCamFocus(targetPos);
		}
	}

	public void setPrevCamTarget() 
	{
		if (scene instanceof JoglModelScene)
		{
			Vector<JSOPnTransition> tsVis = getVisibleTransitons();
			
			int id = -1;
			if (currentTarget != null)
			{
				id = tsVis.indexOf(currentTarget);
			}
			id = (id - 1);
			if (id < 0)
			{
				id = tsVis.size() - 1;
			}
			
			currentTarget = tsVis.get(id);

			JoglVectord3 targetPos = currentTarget.getPosition();
			if (currentTarget.getParentNet() != null)
			{
				targetPos = currentTarget.getParentNet().relativeToAbsolute(targetPos);
			}
			setCamFocus(targetPos);
		}
	}

	public Vector<JSOPnTransition> getVisibleTransitons() 
	{
		Vector<JSOPnTransition> tsVis = new Vector<JSOPnTransition>();
		
		//remove all transitions that are currently hidden from traversal
		for (JSOPnTransition t : traversalList)
		{
			if (t.getParentNet().getIsVisible())
			{
				tsVis.add(t);
			}
		}
		
		return tsVis;
	}

	@Override
	public boolean keyReleased(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt); 
        if (evt.getKeyCode() == PP3DKeyBindings.Camera_Down) //s key
        {
        	bBackwardPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Down2) //down key
        {
        	bBackward2Pressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Up) //w key
        {
        	bForwardPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Up2) //up key
        {
        	bForward2Pressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Left) //a key
        {
        	bLeftPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Left2) //left key
        {
        	bLeft2Pressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Right) //d key
        {
        	bRightPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Right2) //right key
        {
        	bRight2Pressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_RotateLeft) //q key
        {
        	bZoomInPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_RotateRight) //e key
        {
        	bZoomOutPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Run) //shift key
        {
        	bShiftPressed = false;
        }
        
        return false;
	}
	
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
      //System.out.println("mouse pressed: " + e.getButton());

      if (e.getButton() == MouseEvent.BUTTON3) 
      {
    	  lastMousePos = e.getPoint();
    	  bMouseView = true;
//    	  if (bAnimPaused)
//    	  {
//    		  anim.resume();
//    	  }
      }
      
      return false;
    }

	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
        if (e.getButton() == MouseEvent.BUTTON3) 
        {
        	JICameraInterpolator jici = iman.findInteraction(JICameraInterpolator.class);
        	if (jici != null)
        	{
        		JoglVectord3 cf = jici.getCurrentFocus();
        		jici.initCamFocus(cf, camDist, camDx, camDy);
        	}
        	
      		bMouseView = false;
        }

        return false;
    }

	@Override
    public void mouseDragged(JoglInputMouseEvent e) 
    {
    	if (bMouseView)
    	{
	        JoglVectord2 newPos = e.getPoint();
	        double dx = newPos.getX() - lastMousePos.getX();
	        double dy = newPos.getY() - lastMousePos.getY();

	        //double minDy = (-Math.PI * 0.5) + 0.001;
	        double minDy = 0.0;
	        double maxDy = (Math.PI * 0.5) - 0.001;
	        
	        double aspect = cam.getCameraLens().getAspect();
			camDx = camDx + (dx * camTurnFactorX * aspect);
	        camDy = Math.max(minDy, Math.min(maxDy, camDy - (dy * camTurnFactorY)));  

        	JICameraInterpolator jici = iman.findInteraction(JICameraInterpolator.class);
        	if (jici != null)
        	{
        		JoglVectord3 cf = jici.getCurrentFocus();
        		jici.initCamFocus(cf, camDist, camDx, camDy);
        	}
    	}
        lastMousePos = e.getPoint();
    }

}
