package org.processmining.processprofiler3d.compviz.jogl.interactions.camera;

import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JICameraInterpolator extends JoglInteraction
{
	public JoglInteractionManager iman;
	public JoglCamera cam;
	//public JSOPetriNet net = null;

    private double minDy = 0.0;
    private double maxDy = (Math.PI * 0.5) - 0.001;
    
	private double camDx = 0.0;
	private double camDy = 0.0;
    
    private JoglVectord3 camFocus = new JoglVectord3(0,0,0);
    private JoglVectord3 camDir = new JoglVectord3(1,0,0);
	private double camDist = 500.0;
	
	private double maxCamSpeed = 10.0;
	
    private JoglVectord3 targetCamFocus = new JoglVectord3(0,0,0);
	
	public JICameraInterpolator(JoglInteractionManager iman, JoglCamera c)
	{
		this.iman = iman;
		this.cam = c;
	}
	
	public JoglVectord3 getCurrentFocus()
	{
		return camFocus;
	}
	
	public void setTargetCamFocus(JoglVectord3 focus)
	{
		targetCamFocus = focus;
	}
	
	public void initCamFocus(JoglVectord3 focus, JoglVectord3 pos)
	{
		JoglVectord3 dir = JoglVectord3.Subtract(focus, pos);
		JoglVectord3 dirN = JoglVectord3.Normal(dir);
		JoglVectord3 side = JoglVectord3.Cross(dirN, new JoglVectord3(0,1,0));
		
		camFocus = focus;
		camDist = dir.getMagnitude();
		
		double dotx = dirN.getY(); //JoglVectord3.Dot(dirN, new JoglVectord3(0,1,0));
		double doty = side.getZ(); //JoglVectord3.Dot(side, new JoglVectord3(0,0,1));

		camDy = -Math.asin(dotx);
		camDy = Math.max(minDy, Math.min(maxDy, camDy));
		
    	if (side.getX() > 0)
    	{
    		camDx = -Math.asin(doty);
    	}
    	else
    	{
    		camDx = Math.PI + Math.asin(doty);
    	}

        double tx = Math.sin(camDx) * Math.cos(camDy);
        double ty = Math.cos(camDx) * Math.cos(camDy);
        double tz = Math.sin(camDy);
		JoglVectord3 cp = new JoglVectord3(tx, tz, ty);
		cp = JoglVectord3.Normal(cp);
		
		camDir = cp;
    	camFocus = focus;
		targetCamFocus = focus;
	}
	
	public void initCamFocus(JoglVectord3 focus, double cDist, double cDx, double cDy)
	{
		camFocus = focus;
		camDist = cDist;
		
		camDx = cDx;
		camDy = cDy;
		
        double tx = Math.sin(camDx) * Math.cos(camDy);
        double ty = Math.cos(camDx) * Math.cos(camDy);
        double tz = Math.sin(camDy);
		JoglVectord3 cp = new JoglVectord3(tx, tz, ty);
		cp = JoglVectord3.Normal(cp);
		
		camDir = cp;
    	camFocus = focus;
	}
	
	public void setCamDist(double cdist)
	{
		camDist = cdist;
	}

	@Override
	public boolean ShouldAnimate()
	{
		return true;
	}

	@Override
	public void UpdateScene(JoglScene scene)
	{
		//make camera orbit around camFocus at a distance of camDist
        if (camFocus != targetCamFocus)
        {
        	CameraOrbit();
        }
        else
        {
        	iman.removeInteraction(this);
        }
	}

	private void CameraOrbit() 
	{
		JoglVectord3 movedir = JoglVectord3.Subtract(targetCamFocus, camFocus);
		double tdist = movedir.getMagnitude();
		if (tdist > 0)
		{
			movedir = JoglVectord3.Normal(movedir);
			movedir = JoglVectord3.Multiply(movedir, Math.min(maxCamSpeed, tdist));
			
			camFocus = JoglVectord3.Add(camFocus, movedir);
			
	        cam.SetUpDir(new JoglVectord3(0,1,0));
	        
			JoglVectord3 cp = JoglVectord3.Multiply(camDir, camDist);
			cam.SetPosition(JoglVectord3.Add(camFocus, cp));
			cam.SetLookAt(camFocus);
		}
	}
}
