package org.processmining.processprofiler3d.compviz.jogl.interactions.camera;

import java.awt.event.MouseEvent;

import org.processmining.processprofiler3d.compviz.jogl.input.PP3DKeyBindings;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.input.JoglInputKeyEvent;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.interactions.JICameraBase;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JICameraOrbit extends JICameraBase
{
	//public JSOPetriNet net = null;
	
	private boolean bForwardPressed = false;
	private boolean bBackwardPressed = false;
	private boolean bLeftPressed = false;
	private boolean bRightPressed = false;
	private boolean bForward2Pressed = false;
	private boolean bBackward2Pressed = false;
	private boolean bLeft2Pressed = false;
	private boolean bRight2Pressed = false;
	private boolean bZoomInPressed = false;
	private boolean bZoomOutPressed = false;
	private boolean bShiftPressed = false;

    private double minDy = 0.0;
    private double maxDy = (Math.PI * 0.5) - 0.001;
    
	private double camDx = 0.0;
	private double camDy = 0.0;

	//private double cPitch = Math.PI * 0.5;
	//private double cYaw = Math.PI;
 
    private boolean bMouseView = false;
    private JoglVectord2 lastMousePos;
    
    private JoglVectord3 camFocus = new JoglVectord3(0,0,0);
	private double camDist = 500.0;
	
//    private double camTurnFactorX = 0.0025;
//    private double camTurnFactorY = 0.0025;
    private double camTurnFactorX = 2.775;
    private double camTurnFactorY = 2.775;
	
	public JICameraOrbit(JoglCamera c)
	{
		super(c);
	}
	
	public void setCamFocus(JoglVectord3 focus)
	{
		camFocus = focus;
	}
	
	public void setCamFocus(JoglVectord3 focus, JoglVectord3 pos)
	{
		JoglVectord3 dir = JoglVectord3.Subtract(focus, pos);
		JoglVectord3 dirN = JoglVectord3.Normal(dir);
		JoglVectord3 side = JoglVectord3.Cross(dirN, new JoglVectord3(0,1,0));
		
		camFocus = focus;
		camDist = dir.getMagnitude();
		
		double dotx = dirN.getY(); //JoglVectord3.Dot(dirN, new JoglVectord3(0,1,0));
		double doty = side.getZ(); //JoglVectord3.Dot(side, new JoglVectord3(0,0,1));

		camDy = -Math.asin(dotx);
		camDy = Math.max(minDy, Math.min(maxDy, camDy));
		
    	if (side.getX() > 0)
    	{
    		camDx = -Math.asin(doty);
    	}
    	else
    	{
    		camDx = Math.PI + Math.asin(doty);
    	}
		//System.out.println("CamDy: " + camDy);
		//System.out.println("CamDx: " + camDx);
	}
	
	private boolean IsKeyPressed()
	{
		if (bForwardPressed || bBackwardPressed || bLeftPressed || bRightPressed)
		{
	    	return true;
		}
		if (bForward2Pressed || bBackward2Pressed || bLeft2Pressed || bRight2Pressed)
		{
	    	return true;
		}
		if (bZoomInPressed || bZoomOutPressed)
		{
			return true;
		}
		if (bMouseView)
		{
			return true;
		}
		return false;
	}

	@Override
	public boolean ShouldAnimate()
	{
		return IsKeyPressed();
	}

	@Override
	public void UpdateScene(JoglScene scene)
	{
		//System.out.println("Camera update scene");
		
        double zoomStep = 10;
        double turnStep = 0.1;

        if (bShiftPressed)
        {
        	zoomStep = 50;
        }
        
//        if (net != null)
//        {
//        	camFocus = net.GetPetrinetCenter();
//        }
        
		if (bZoomInPressed)
		{
			camDist = camDist - zoomStep;
		}
		if (bZoomOutPressed)
		{
			camDist = camDist + zoomStep;
		}
		if (bForwardPressed || bForward2Pressed)
		{
			camDy = camDy - turnStep;
		}
		if (bBackwardPressed || bBackward2Pressed)
		{
			camDy = camDy + turnStep;
		}
        
		camDist = Math.max(0.1, camDist);
        camDy = Math.max(minDy, Math.min(maxDy, camDy));
		
		if (bLeftPressed || bLeft2Pressed)
		{
			camDx = camDx - turnStep;		
		}
		if (bRightPressed || bRight2Pressed)
		{
			camDx = camDx + turnStep;		
		}
        
		//make camera orbit around camFocus at a distance of camDist
        CameraOrbit();
	}

	private void CameraOrbit() 
	{
        cam.SetUpDir(new JoglVectord3(0,1,0));
        
        double tx = Math.sin(camDx) * Math.cos(camDy);
        double ty = Math.cos(camDx) * Math.cos(camDy);
        double tz = Math.sin(camDy);
		JoglVectord3 cp = new JoglVectord3(tx, tz, ty);
		cp = JoglVectord3.Normal(cp);
		cp = JoglVectord3.Multiply(cp, camDist);
		cam.SetPosition(JoglVectord3.Add(camFocus, cp));
		cam.SetLookAt(camFocus);
	}

	//KeyListener method implementations
	@Override
	public boolean keyPressed(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt);
        
        if (evt.getKeyCode() == PP3DKeyBindings.Camera_Down) //s key
        {
        	bBackwardPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Down2) //down key
        {
        	bBackward2Pressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Up) //w key
        {
        	bForwardPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Up2) //up key
        {
        	bForward2Pressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Left) //a key
        {
        	bLeftPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Left2) //left key
        {
        	bLeft2Pressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Right) //d key
        {
        	bRightPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Right2) //right key
        {
        	bRight2Pressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_RotateLeft) //q key
        {
        	//RotateCamera(0.1, 0);
        	bZoomInPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_RotateRight) //e key
        {
        	//RotateCamera(-0.1, 0);
        	bZoomOutPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Run) //shift key
        {
        	bShiftPressed = true;
        }
        
        return false;
	}

	@Override
	public boolean keyReleased(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt); 
        if (evt.getKeyCode() == PP3DKeyBindings.Camera_Down) //s key
        {
        	bBackwardPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Down2) //down key
        {
        	bBackward2Pressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Up) //w key
        {
        	bForwardPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Up2) //up key
        {
        	bForward2Pressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Left) //a key
        {
        	bLeftPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Left2) //left key
        {
        	bLeft2Pressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Right) //d key
        {
        	bRightPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Right2) //right key
        {
        	bRight2Pressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_RotateLeft) //q key
        {
        	bZoomInPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_RotateRight) //e key
        {
        	bZoomOutPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Run) //shift key
        {
        	bShiftPressed = false;
        }
        
        return false;
	}
	
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
      //System.out.println("mouse pressed: " + e.getButton());

      if (e.getButton() == MouseEvent.BUTTON3) 
      {
    	  lastMousePos = e.getPoint();
    	  bMouseView = true;
//    	  if (bAnimPaused)
//    	  {
//    		  anim.resume();
//    	  }
      }
      
      return false;
    }

	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
        if (e.getButton() == MouseEvent.BUTTON3) 
        {
      	  bMouseView = false;
        }

        return false;
    }

	@Override
    public void mouseDragged(JoglInputMouseEvent e) 
    {
    	if (bMouseView)
    	{
	        JoglVectord2 newPos = e.getPoint();
	        double dx = newPos.getX() - lastMousePos.getX();
	        double dy = newPos.getY() - lastMousePos.getY();

	        //double minDy = (-Math.PI * 0.5) + 0.001;
	        double minDy = 0.0;
	        double maxDy = (Math.PI * 0.5) - 0.001;
	        
	        double aspect = cam.getCameraLens().getAspect();
			camDx = camDx + (dx * camTurnFactorX * aspect);
	        camDy = Math.max(minDy, Math.min(maxDy, camDy - (dy * camTurnFactorY)));
    	}
        lastMousePos = e.getPoint();
    }

}
