package org.processmining.processprofiler3d.compviz.jogl.interactions.camera;

import java.awt.event.MouseEvent;

import org.processmining.processprofiler3d.compviz.jogl.input.PP3DKeyBindings;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIOpenVR;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.input.JoglInputKeyEvent;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.interactions.JICameraBase;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JICameraVR extends JICameraBase
{
	private boolean bForwardPressed = false;
	private boolean bBackwardPressed = false;
	private boolean bLeftPressed = false;
	private boolean bRightPressed = false;
	private boolean bForward2Pressed = false;
	private boolean bBackward2Pressed = false;
	private boolean bLeft2Pressed = false;
	private boolean bRight2Pressed = false;
	private boolean bTurnLeftPressed = false;
	private boolean bTurnRightPressed = false;
	private boolean bShiftPressed = false;
	
	private double camDx = 0.0;
	private double camDy = 0.0;

	private double cPitch = Math.PI * 0.5;
	private double cYaw = Math.PI;
 
    private boolean bMouseView = false;
    private JoglVectord2 lastMousePos;
	
    public JoglCamera hmdCamera; //right eye camera of headset view
    
    public JoglVectord3 hmdForward;
    public JoglVectord3 hmdUp;
    public JoglVectord3 hmdSide;
    
    public JoglVectord3 hmdPosAbsolute;
    
//    private double camTurnFactorX = 0.005;
//    private double camTurnFactorY = 0.005;
    private double camTurnFactorX = 5.55;
    private double camTurnFactorY = 5.55;
    
    public JICameraVR(JoglCamera c)
    {
		super(c);
		
		hmdForward = c.GetLookDir();
		hmdUp = c.GetUpDir();
		hmdSide = c.GetLeftDir();
		
		hmdPosAbsolute = c.GetPosition();
    }

    public JoglCamera getHMDCamera()
    {
    	return hmdCamera;
    }

    public void setHMDCamera(JoglCamera vrc)
    {
    	hmdCamera = vrc;
    }
    
	public void InitCamera(JSOPetriNet pnet)
	{	
		JoglVectord3 camPos = JoglVectord3.Add(pnet.GetPetrinetCenter(), new JoglVectord3(0, 500, 0));
		
		cam.SetPosition(camPos);
		cam.SetPitchYawRoll(Math.PI * 0.5, Math.PI, 0);
	}
    
	public void setPitchYawRoll(double p, double y, double r)
	{
		//cPitch = p;
		cYaw = y;

		cPitch = 0;
		cam.SetPitchYawRoll(cPitch, cYaw, 0);
	}
	
	private void RotateCamera(double x, double y)
	{
		if (x != 0)
		{
			cYaw = cYaw + x;
		}
		cPitch = 0;
		
//		JoglVectord3 diff = JoglVectord3.Subtract(cam.GetPosition(), hmdPosAbsolute);
//		cam.SetPosition(JoglVectord3.Add(hmdPosAbsolute, JoglVectord3.Multiply(diff, -1.0)));
//		cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), JoglVectord3.Multiply(diff, -1.0)));
		
		cam.SetPitchYawRoll(cPitch, cYaw, 0);
		
//		cam.SetPosition(JoglVectord3.Add(hmdPosAbsolute, diff));
//		cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), diff));
		
		//System.out.println("Cam: " + cPitch + "/" + cYaw);
	}
	
	private boolean IsKeyPressed()
	{
		if (bForwardPressed || bBackwardPressed || bLeftPressed || bRightPressed)
		{
	    	return true;
		}
		if (bForward2Pressed || bBackward2Pressed || bLeft2Pressed || bRight2Pressed)
		{
	    	return true;
		}
		if (bTurnLeftPressed || bTurnRightPressed)
		{
			return true;
		}
		if (bMouseView)
		{
			return true;
		}
		return false;
	}

	@Override
	public boolean ShouldAnimate()
	{
		return IsKeyPressed();
	}
	
	public void setHmdDir(JoglVectord3 forward, JoglVectord3 up, JoglVectord3 side)
	{
		hmdForward = forward;
		hmdUp = up;
		hmdSide = side;
	}

	@Override
	public void UpdateScene(JoglScene scene)
	{
		//System.out.println("Camera update scene");
		
        double moveStep = 2 * JIOpenVR.vrScale;
        double turnStep = 0.1;

        if (bShiftPressed)
        {
        	moveStep = 10 * JIOpenVR.vrScale;
        }
        
		if (bTurnLeftPressed)
		{
			camDx = camDx - turnStep;
		}
		if (bTurnRightPressed)
		{
			camDx = camDx + turnStep;
		}
		
		if (camDx != 0 || camDy != 0)
		{

	        RotateCamera(camDx, camDy);
	        
	        camDx = 0.0;
	        camDy = 0.0;
		}
		
		if (bForwardPressed || bForward2Pressed)
		{
	    	JoglVectord3 ldir = JoglVectord3.Normal(hmdForward);
	    	JoglVectord3 camMove = JoglVectord3.Multiply(ldir, moveStep);
	    	
	    	cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), camMove));
	    	cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), camMove));
		}
		if (bBackwardPressed || bBackward2Pressed)
		{
        	JoglVectord3 ldir = JoglVectord3.Normal(hmdForward);
	    	JoglVectord3 camMove = JoglVectord3.Multiply(ldir, -moveStep);	
        	
        	cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), camMove));
        	cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), camMove));
		}
		if (bLeftPressed || bLeft2Pressed)
		{
        	JoglVectord3 leftdir = hmdSide;
	    	JoglVectord3 camMove = JoglVectord3.Multiply(leftdir, -moveStep);
        	
        	cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), camMove));
        	cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), camMove));			
		}
		if (bRightPressed || bRight2Pressed)
		{
        	JoglVectord3 leftdir = hmdSide;
	    	JoglVectord3 camMove = JoglVectord3.Multiply(leftdir, moveStep);
        	
        	cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), camMove));
        	cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), camMove));			
		}
		
//		Vector<JSOVRController> controllers = scene.findSceneObjects(JSOVRController.class);
//		for (JSOVRController c : controllers)
//		{
//			HzdVRControllerState vcs = c.GetControllerState();
//			if (vcs != null)
//			{
//				int cnum = c.controllerId;
//				boolean bEnableTouchpadMove = false;
//				if (vcs.IsAppMenuPressed())
//				{
//					System.out.println("Controller " + cnum + ": AppMenu pressed!");
//				}
//				if (vcs.IsGripPressed())
//				{
//					System.out.println("Controller " + cnum + ": Grip pressed!");
//
//		        	bEnableTouchpadMove = true;
//				}
//				if (vcs.IsTriggerPressed())
//				{
//					System.out.println("Controller " + cnum + ": Trigger pressed!");
//				}
//				if (vcs.IsTouchpadPressed())
//				{
//					System.out.println("Controller " + cnum + ": Touchpad pressed!");
//					
//		        	bEnableTouchpadMove = true;
//				}
//				
//				if (bEnableTouchpadMove)
//				{
//			        if (vcs.IsTouchpadPressed() && vcs.IsGripPressed())
//			        {
//			        	moveStep = 10 * JIOpenVR.vrScale;
//			        }
////		        	JoglVectord3 ldir = JoglVectord3.Normal(hmdForward); 
////		        	JoglVectord3 leftdir = hmdSide;
//		        	JoglVectord3 ldir = JoglVectord3.Normal(c.getForward()); 
//		        	JoglVectord3 leftdir = c.getLeft();
//		        	
//					JoglVectord3 move = JoglVectord3.Add(JoglVectord3.Multiply(ldir, moveStep * vcs.getTouchpadPos().getY()), JoglVectord3.Multiply(leftdir, moveStep * vcs.getTouchpadPos().getX()));
//		        	cam.SetPosition(JoglVectord3.Add(cam.GetPosition(), move));
//		        	cam.SetLookAt(JoglVectord3.Add(cam.GetLookAt(), move));
//				}
//			}
//		}
		
	}

	//KeyListener method implementations
	@Override
	public boolean keyPressed(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt);
        
        if (evt.getKeyCode() == PP3DKeyBindings.Camera_Down) //s key
        {
        	bBackwardPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Down2) //down key
        {
        	bBackward2Pressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Up) //w key
        {
        	bForwardPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Up2) //up key
        {
        	bForward2Pressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Left2) //left key
        {
        	bLeft2Pressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Left) //a key
        {
        	bLeftPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Right2) //right key
        {
        	bRight2Pressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Right) //d key
        {
        	bRightPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_RotateLeft) //q key
        {
        	//RotateCamera(0.1, 0);
        	bTurnLeftPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_RotateRight) //e key
        {
        	//RotateCamera(-0.1, 0);
        	bTurnRightPressed = true;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Run) //shift key
        {
        	bShiftPressed = true;
        }
        
        return false;
	}

	@Override
	public boolean keyReleased(JoglInputKeyEvent evt) 
	{
        //System.out.println(evt); 
        if (evt.getKeyCode() == PP3DKeyBindings.Camera_Down) //s key
        {
        	bBackwardPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Down2) //down key
        {
        	bBackward2Pressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Up2) //up key
        {
        	bForward2Pressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Up) //w key
        {
        	bForwardPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Left2) //left key
        {
        	bLeft2Pressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Left) //a key
        {
        	bLeftPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Right2) //right key
        {
        	bRight2Pressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Right) //d key
        {
        	bRightPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_RotateLeft) //q key
        {
        	bTurnLeftPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_RotateRight) //e key
        {
        	bTurnRightPressed = false;
        }
        else if (evt.getKeyCode() == PP3DKeyBindings.Camera_Run) //shift key
        {
        	bShiftPressed = false;
        }
        
        return false;
	}
	
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
      //System.out.println("mouse pressed: " + e.getButton());

      if (e.getButton() == MouseEvent.BUTTON3) 
      {
    	  lastMousePos = e.getPoint();
    	  bMouseView = true;
//    	  if (bAnimPaused)
//    	  {
//    		  anim.resume();
//    	  }
      }
      
      return false;
    }

	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
        if (e.getButton() == MouseEvent.BUTTON3) 
        {
      	  bMouseView = false;
        }
        
        return false;
    }

	@Override
    public void mouseDragged(JoglInputMouseEvent e) 
    {
    	if (bMouseView)
    	{
	        JoglVectord2 newPos = e.getPoint();
	        double dx = newPos.getX() - lastMousePos.getX();
	        double dy = newPos.getY() - lastMousePos.getY();

	        double aspect = cam.getCameraLens().getAspect();
			camDx = camDx + (dx * camTurnFactorX * aspect);
	        camDy = camDy + (dy * camTurnFactorY);
	        
    	}
        lastMousePos = e.getPoint();
    }

}
