package org.processmining.processprofiler3d.compviz.jogl.interactions.cohortsplit;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.processprofiler3d.compviz.PP3DCohort;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JILogReplay;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENEdgeVisDriver;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENFilter;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENTransVisDriver;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOCohortColorManager;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENTrajectoryVisualizationLayer;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohort;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.measurements.TimeSpan;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIAsyncDataSplit extends JoglInteraction
{
	private XLog myLog = null;
	private JoglInteractionManager iman;
	private JSOGraphicalUserInterface gui;
	private CohortClassifier mycc = null;
	
	private AsyncDataSplitThread thread = null;
	private boolean bInitialized = false;
	
	private JoglGuiLabel lblProcessMsg;
	private Date splitStart;
	
	public JIAsyncDataSplit(JoglInteractionManager iman, JSOGraphicalUserInterface gui, IPerformanceStatisticsEnrichedNet psen, IPerformanceStatisticsEnrichedNet wten, CohortClassifier cc, XLog l, Vector<XTrace> tracelst)
	{
		this.iman = iman;
		this.gui = gui;
		mycc = cc;
		myLog = l;
		
		Thread.currentThread().setPriority(Thread.MAX_PRIORITY);
		
		splitStart = new Date();
		
		thread = new AsyncDataSplitThread(psen, wten, cc, tracelst);
		thread.start();
	}
	
	private void Init(JoglScene scene)
	{
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = (0.05f / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = 0.05f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_CENTER;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		
		float w = 0.2f * (float)JoglGuiStyle.textScale;
		float h = 0.05f * (float)JoglGuiStyle.textScale;
		float l = w * -0.5f;
		float t = h * 0.5f;
		
		lblProcessMsg = new JoglGuiLabel(ff.clone());
		lblProcessMsg.setRect(l, t, w, h);
		lblProcessMsg.setText("Processing data...");
		lblProcessMsg.setPadding(textPadding);
		lblProcessMsg.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(lblProcessMsg);
		//controls.add(lblVisModeTransition);
		
		bInitialized = true;
	}
	
	private void Dispose(JoglScene scene)
	{
		gui.RemoveGuiObject(lblProcessMsg);
		
		bInitialized = false;
	}
	
	@Override
	public boolean ShouldAnimate()
	{
		return true;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (!bInitialized)
		{
			Init(scene);
		}
		
		if (thread != null && thread.bProcessingDone)
		{
			System.out.println("Data split thread completed.");
			
			UpdateVis(scene);
			System.out.println("Data split vis update completed.");
			thread = null;

			Date splitEnd = new Date();
			TimeSpan ts = new TimeSpan(splitStart, splitEnd);
			System.out.println("Data split completed (" + ts.ToString(TimeUnit.SECONDS) + "s).");
			
			Dispose(scene);
			iman.removeInteraction(this);
		}
	}
	
	private void UpdateVis(JoglScene scene)
	{
		Vector<PP3DCohort> cohorts = new Vector<PP3DCohort>();
		HashMap<Integer,Integer> histogram = mycc.getCohortCaseHistogram(myLog);
		
		for (ClassificationCohort ccs : mycc.getCohorts())
		{
			int cId = ccs.getID();
			
			PP3DCohort pc = new PP3DCohort();
			pc.cohortName = ccs.getName();
			pc.cohortID = cId;
			
			//don't assign colors to cohorts for which no cases exist!
			if (histogram.containsKey(cId))
			{
				pc.caseNum = histogram.get(cId);
			}
			
			cohorts.add(pc);
		}
		
		JSOCohortColorManager jccm = scene.findSceneObject(JSOCohortColorManager.class);
		//jccm.updateColorsByCohort(mycc);
		jccm.updateColorsByCohort(cohorts);
		
		JIPSENTransVisDriver jtvd = iman.findInteraction(JIPSENTransVisDriver.class);
		if (jtvd != null)
		{
			//make all classifiers visible at first
			jtvd.setVisibleClassifiers(new HashSet<String>(jtvd.GetClassifierNames()));
		}
		
		JSOPSENTrajectoryVisualizationLayer jptv = scene.findSceneObject(JSOPSENTrajectoryVisualizationLayer.class);
		if (jptv != null)
		{
			jptv.UpdateCohorts(mycc, jtvd.GetClassifierNames());
		}
		
		JIPSENFilter f = iman.findInteraction(JIPSENFilter.class);
		if (f != null)
		{
			f.UpdateFilters();
		}
		
		if (jtvd != null)
		{
			jtvd.SetVisualization();
		}
		
		JIPSENEdgeVisDriver jevd = iman.findInteraction(JIPSENEdgeVisDriver.class);
		if (jevd != null)
		{
			jevd.setCohortClassifier(mycc.getCohorts());
			jevd.SetVisualization();
		}
		
		JILogReplay jlr = iman.findInteraction(JILogReplay.class);
		if (jlr != null)
		{
			jlr.SimulateReplay(mycc);
		}
	}
}
