package org.processmining.processprofiler3d.compviz.jogl.interactions.cohortsplit;

import java.awt.event.MouseEvent;
import java.util.Date;
import java.util.Vector;

import org.deckfour.xes.model.XLog;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiCohortTreeBox;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredRect;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohort;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.processprofiler3d.psen.classification.CohortPreset;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.input.JoglInputMouseWheelEvent;
import org.processmining.visualisation3d.interactions.JoglGuiInteraction;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JICohortConfig extends JoglGuiInteraction
{
	private JoglScene scene;
	private JoglInteractionManager iman;

	private JoglGuiLabel lblHeader;
	private JoglGuiLabel btnAdd;
	private JoglGuiLabel btnRemove;
	private JoglGuiLabel btnEdit;
	private JoglGuiLabel btnSave;
	private JoglGuiLabel btnAccept;
	private JoglGuiLabel btnCancel;
	private JoglGuiColoredRect background;
	private JoglGuiCohortTreeBox lbxCohorts;
	
	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();
	private JoglGuiLabel highlighted = null;
	
	private boolean bInit = true;
	
	private CohortClassifier mycc = null;
	private XLog mylog = null;
	
	private Date lastClick;
	private JoglGuiLabel lastTarget = null;
	private int doubleClickTime = 600;
	
	private int lastSelectedItem = -1;
	
	public JICohortConfig(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui, CohortClassifier cc, XLog log)
	{
		super(gui);
		
		this.scene = scene;
		this.iman = iman;
		
		mycc = cc;
		mylog = log;
		
		setZLevel(-200.0f);
		
		lastClick = new Date(new Date().getTime() - doubleClickTime);
	}
	
	public void Init()
	{	
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = 0.05f / aspect;
		ff.height = 0.05f;
		ff.align = FontFormat.ALIGN_CENTER;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff2 = new FontFormat();
		ff2.width = 0.05f / aspect;
		ff2.height = 0.05f;
		ff2.align = FontFormat.ALIGN_LEFT;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff3 = new FontFormat();
		ff3.width = 0.05f / aspect;
		ff3.height = 0.05f;
		ff3.align = FontFormat.ALIGN_RIGHT;
		ff3.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		float textPadding2 = 0.01f;
		
		background = new JoglGuiColoredRect();
		background.setCanTouch(true);
		background.setRect(-0.25f, -0.575f, 0.5f, 1.125f);
		background.setColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(background);
		controls.add(background);

		lblHeader = new JoglGuiLabel(ff.clone());
		lblHeader.setRect(-0.25f, 0.5f, 0.5f, 0.05f);
		lblHeader.setText("Configure Cohorts");
		lblHeader.setPadding(textPadding);
		lblHeader.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(lblHeader);
		controls.add(lblHeader);
		
		btnAdd = new JoglGuiLabel(ff.clone());
		btnAdd.setCanTouch(true);
		btnAdd.setRect(-0.225f, -0.4f, 0.2f, 0.05f);
		btnAdd.setText("Add");
		btnAdd.setPadding(textPadding);
		btnAdd.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnAdd);
		controls.add(btnAdd);
		
		btnRemove = new JoglGuiLabel(ff.clone());
		btnRemove.setCanTouch(true);
		btnRemove.setRect(0.025f, -0.4f, 0.2f, 0.05f);
		btnRemove.setText("Remove");
		btnRemove.setPadding(textPadding);
		btnRemove.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnRemove);
		controls.add(btnRemove);
		
		btnEdit = new JoglGuiLabel(ff.clone());
		btnEdit.setCanTouch(true);
		btnEdit.setRect(-0.225f, -0.475f, 0.2f, 0.05f);
		btnEdit.setText("Edit");
		btnEdit.setPadding(textPadding);
		btnEdit.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnEdit);
		controls.add(btnEdit);
		
		btnSave = new JoglGuiLabel(ff.clone());
		btnSave.setCanTouch(true);
		btnSave.setRect(0.025f, -0.475f, 0.2f, 0.05f);
		btnSave.setText("Save");
		btnSave.setPadding(textPadding);
		btnSave.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnSave);
		controls.add(btnSave);
		
		btnAccept = new JoglGuiLabel(ff.clone());
		btnAccept.setCanTouch(true);
		btnAccept.setRect(-0.225f, -0.55f, 0.2f, 0.05f);
		btnAccept.setText("Accept");
		btnAccept.setPadding(textPadding);
		btnAccept.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnAccept);
		controls.add(btnAccept);
		
		btnCancel = new JoglGuiLabel(ff.clone());
		btnCancel.setCanTouch(true);
		btnCancel.setRect(0.025f, -0.55f, 0.2f, 0.05f);
		btnCancel.setText("Cancel");
		btnCancel.setPadding(textPadding);
		btnCancel.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnCancel);
		controls.add(btnCancel);
		
		updateCohortList();
		
		lbxCohorts.setSelectedIndex(lastSelectedItem);
		
		bInit = false;
	}

	public void updateCohortList() 
	{
		if (lbxCohorts != null)
		{
			gui.RemoveGuiObject(lbxCohorts);
			controls.remove(lbxCohorts);
		}
		
		lbxCohorts = new JoglGuiCohortTreeBox();
		lbxCohorts.setCanTouch(true);
		lbxCohorts.setCanMultiSelect(true);
		lbxCohorts.setRect(-0.25f + 0.0125f, -0.325f, 0.475f, 0.8f);
		lbxCohorts.setZLevel(-200.0f);
		lbxCohorts.setItems(mycc, mylog);
		lbxCohorts.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(lbxCohorts);
		controls.add(lbxCohorts);
	}
	
	public void reopen()
	{
		bUpdateScene = true;
	}
	
	public void Dispose()
	{
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
		
		bInit = false;
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{	
		if (bInit)
		{
			Init();
		}
		
		bUpdateScene = false;
	}
	
	
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MousePressed");
			Vector<JoglGuiObject> jgos = gui.GetObjectsFromPos(e.getX(), e.getY());
			if (jgos.contains(lbxCohorts))
			{
				lbxCohorts.mousePressed(iman, e);
				return true;
			}
			if (jgos.contains(btnAdd))
			{
				return true;
			}
			if (jgos.contains(btnRemove))
			{
				return true;
			}
			if (jgos.contains(btnEdit))
			{
				return true;
			}
			if (jgos.contains(btnSave))
			{
				return true;
			}
			if (jgos.contains(btnAccept))
			{
				return true;
			}
			if (jgos.contains(btnCancel))
			{
				return true;
			}
			if (jgos.contains(background))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MouseReleased");
			Vector<JoglGuiObject> jgos = gui.GetObjectsFromPos(e.getX(), e.getY());
			
			Date nowTime = new Date();
			//check if is double click
			if (new TimeSpan(lastClick, nowTime).getMilliseconds() < doubleClickTime)
			{
				if (jgos.contains(lbxCohorts))
				{
					JoglGuiLabel cTarget = lbxCohorts.getTargetLabel(e);
					//if clicked same target as previous click in short time
					if (lastTarget == cTarget)
					{
						DoubleClick(e);
					}
				}
			}
			
			if (jgos.contains(btnAdd))
			{
				//lbxCohorts.mousePressed(e);
				OpenAddCohortDialog();
				
				return true;
			}
			if (jgos.contains(btnRemove))
			{
				RemovePressed();
				
				return true;
			}
			if (jgos.contains(btnEdit))
			{
				EditCohort();
				
				return true;
			}
			if (jgos.contains(btnSave))
			{
				SaveCohortPreset();
				return true;
			}
			if (jgos.contains(btnAccept))
			{
				JISplitCohorts jisc = iman.findInteraction(JISplitCohorts.class);
				jisc.updateCohorts(mycc);
				
				Close();
				
				return true;
			}
			if (jgos.contains(btnCancel))
			{
				//lbxCohorts.mousePressed(e);
				Close();
				
				return true;
			}
			
			if (jgos.contains(lbxCohorts))
			{
				lastTarget = lbxCohorts.getTargetLabel(e);
			}
			else
			{
				lastTarget = null;
			}

			lastClick = nowTime;
			
			if (jgos.contains(background))
			{
				return true;
			}
		}
		return false;
    }
	
	public void DoubleClick(JoglInputMouseEvent e) 
	{
		JoglVectord2 pos = gui.GetNDCPos(e.getX(), e.getY());
		if (lbxCohorts.IsInside((float)pos.getX(), (float)pos.getY()))
		{
			//double clicked lbxCohorts
			EditCohort();
		}
	}
	
	public void EditCohort()
	{
		ClassificationCohort selectedCohort = lbxCohorts.getSelectedCohort();	
		if (selectedCohort != null && selectedCohort.getLevel() > 0)
		{
			ClassificationCohort scParent = selectedCohort.getParent();
			mycc.removeCohort(selectedCohort.getName());
			
			String pName = null;
			if (scParent != null && scParent.getLevel() > 0)
			{
				pName = scParent.getName();
			}
			
			JICohortConfig2 jicc = new JICohortConfig2(iman, scene, gui, mycc, mylog, this, selectedCohort.getCohortDefinition(), pName);
			iman.addInteraction(jicc);

			Close();
		}	
	}

	public void SaveCohortPreset() 
	{
		CohortPreset cp = mycc.getPreset();
		cp.presetName = "Preset " + (CohortClassifier.customPresets.size() + 1);
		CohortClassifier.customPresets.add(cp);
	}

	public void RemovePressed() 
	{
		Vector<ClassificationCohort> cLst = lbxCohorts.getSelectedCohorts();		
		for (ClassificationCohort c : cLst)
		{
			if (c.getLevel() > 0)
			{
				mycc.removeCohort(c.getName());
			}
		}
		updateCohortList();
	}
	
	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnAdd)
			{
				highlight(btnAdd);
			}
			else if (jgo == btnRemove)
			{
				highlight(btnRemove);
			}
			else if (jgo == btnAccept)
			{
				highlight(btnAccept);
			}
			else if (jgo == btnCancel)
			{
				highlight(btnCancel);
			}
			else if (jgo == btnEdit)
			{
				highlight(btnEdit);
			}
			else if (jgo == btnSave)
			{
				highlight(btnSave);
			}
			else
			{
				unHighlight();
			}
		}
		else
		{
			unHighlight();
		}
	}
	
	private void highlight(JoglGuiLabel label)
	{
		unHighlight();
		
		label.setBackColor(JoglGuiStyle.highlightColor);
		highlighted = label;
		
		bUpdateScene = true;
	}

	private void unHighlight() 
	{
		if (highlighted != null)
		{
			highlighted.setBackColor(JoglGuiStyle.backColor);
			highlighted = null;

			bUpdateScene = true;
		}
	}

	@Override
	public void mouseWheelMoved(JoglInputMouseWheelEvent e)
	{
		if (lbxCohorts != null)
		{
			lbxCohorts.mouseWheelMoved(e);
		}
	}
	
	public void OpenAddCohortDialog()
	{	
		String pName = null;
		ClassificationCohort selectedCohort = lbxCohorts.getSelectedCohort();	
		if (selectedCohort != null && selectedCohort.getLevel() > 0)
		{
			pName = selectedCohort.getName();
		}
		JICohortConfig2 jicc = new JICohortConfig2(iman, scene, gui, mycc, mylog, this, pName);
		iman.addInteraction(jicc);
		
		lastSelectedItem = lbxCohorts.getSelectedIndex();
		
		Close();
	}
	
	public void Close()
	{
		Dispose();
		iman.removeInteraction(this);
	}
	
}
