package org.processmining.processprofiler3d.compviz.jogl.interactions.cohortsplit;

import java.awt.event.MouseEvent;
import java.util.List;
import java.util.Vector;

import org.deckfour.xes.model.XLog;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredRect;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.interactions.IDropDownListener;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIDropdownBox;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortIntRange;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortLiteral;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortTimeOfDay;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.processprofiler3d.psen.classification.CohortDefinition;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.interactions.JoglGuiInteraction;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JICohortConfig2 extends JoglGuiInteraction implements IDropDownListener
{
	private JoglScene scene;
	private JoglInteractionManager iman;

	private JoglGuiLabel lblHeader;
	private JoglGuiLabel btnClassifierType;
	private JoglGuiColoredRect background;
	
	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();
	private JoglGuiLabel highlighted = null;
	
	private boolean bInit = true;
	
	private CohortClassifier mycc = null;
	
	private JICohortConfig parent = null;
	private XLog mylog = null;
	//private LogAnalyser la = null;
	
	private JIDropdownBox ddbox = null;
	
	private CohortDefinition forcedDef = null;
	private String parentName = null;
	
	public JICohortConfig2(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui, CohortClassifier cc, XLog log, JICohortConfig jicc, String parentName)
	{
		super(gui);
		
		this.parent = jicc;
		this.scene = scene;
		this.iman = iman;
		
		mycc = cc;
		mylog = log;
		//la = new LogAnalyser(log);
		this.parentName = parentName;
		
		setZLevel(-200.0f);
	}
	
	public JICohortConfig2(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui, CohortClassifier cc, XLog log, JICohortConfig jicc, CohortDefinition def, String parentName)
	{
		super(gui);
		
		this.parent = jicc;
		this.scene = scene;
		this.iman = iman;
		
		mycc = cc;
		mylog = log;
		//la = new LogAnalyser(log);
		
		forcedDef = def;
		this.parentName = parentName;
		
		setZLevel(-200.0f);
	}
	
	public void Init()
	{	
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = 0.05f / aspect;
		ff.height = 0.05f;
		ff.align = FontFormat.ALIGN_CENTER;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		
		background = new JoglGuiColoredRect();
		background.setCanTouch(true);
		background.setRect(-0.25f, -0.5f, 0.5f, 1.05f);
		background.setColor(JoglGuiStyle.backColor);
		
		//gui.AddGuiObject(background);
		//controls.add(background);

		lblHeader = new JoglGuiLabel(ff.clone());
		lblHeader.setRect(-0.25f, 0.5f, 0.5f, 0.05f);
		lblHeader.setText("Add Generic Literal Cohort");
		lblHeader.setPadding(textPadding);
		lblHeader.setBackColor(JoglGuiStyle.backColor);
//		
//		gui.AddGuiObject(lblHeader);
//		controls.add(lblHeader);
		
		btnClassifierType = new JoglGuiLabel(ff.clone());
		btnClassifierType.setRect(-0.225f, 0.5f, 0.45f, 0.05f);
		btnClassifierType.setPadding(textPadding);
		btnClassifierType.setBackColor(JoglGuiStyle.backColor);
		btnClassifierType.setCanTouch(true);
		btnClassifierType.setZLevel(1.0f);
		if (forcedDef == null)
		{
			btnClassifierType.setText(JISplitCohorts.preferredClassifierType);	
		}
		else
		{
			btnClassifierType.setText(forcedDef.getClassifierType());	
		}
		
		gui.AddGuiObject(btnClassifierType);
		controls.add(btnClassifierType);

		bInit = false;
		
		OpenAddCohortDialog(btnClassifierType.getText());
	}
	
	public void Dispose()
	{	
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{	
		if (bInit)
		{
			Init();
		}
		
		bUpdateScene = false;
	}
	
	private JICohortConfig2Panel classifierPanel;
	
	public void OpenAddCohortDialog(String classifierType)
	{	
		if (classifierPanel != null)
		{
			classifierPanel.Dispose();
			iman.removeInteraction(classifierPanel);
			classifierPanel = null;
		}
		
		if (classifierType.equals(ClassificationCohortTimeOfDay.classifierName))
		{
			classifierPanel = new JICohortConfig2TimeOfDay(iman, scene, gui, mycc, mylog, this, forcedDef);
		}
		else if (classifierType.equals(ClassificationCohortLiteral.classifierName))
		{
			classifierPanel = new JICohortConfig2GenericLiteral(iman, scene, gui, mycc, mylog, this, forcedDef);
		}
		else if (classifierType.equals(ClassificationCohortIntRange.classifierName))
		{
			classifierPanel = new JICohortConfig2IntRange(iman, scene, gui, mycc, mylog, this, forcedDef);
		}
		
		if (classifierPanel != null)
		{
			iman.addInteraction(classifierPanel);		
		}
		
		//Close();
	}

	@Override
	public boolean mousePressed(JoglInputMouseEvent e)
	{
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MouseReleased");
			Vector<JoglGuiObject> jgos = gui.GetObjectsFromPos(e.getX(), e.getY());
			
			if (jgos.contains(btnClassifierType))
			{
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean mouseReleased(JoglInputMouseEvent e)
	{
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MouseReleased");
			Vector<JoglGuiObject> jgos = gui.GetObjectsFromPos(e.getX(), e.getY());
			
			if (jgos.contains(btnClassifierType))
			{
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean mouseClicked(JoglInputMouseEvent e)
	{
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MouseReleased");
			Vector<JoglGuiObject> jgos = gui.GetObjectsFromPos(e.getX(), e.getY());
			
			if (jgos.contains(btnClassifierType))
			{
				Vector<String> lst = new Vector<String>();
				lst.add(ClassificationCohortTimeOfDay.classifierName);
				lst.add(ClassificationCohortLiteral.classifierName);
				lst.add(ClassificationCohortIntRange.classifierName);
				OpenDropDownBox(btnClassifierType, lst);
				
				return true;
			}
		}
		return false;
	}
	
	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnClassifierType)
			{
				highlight(btnClassifierType);
			}
			else
			{
				unHighlight();
			}
		}
		else
		{
			unHighlight();
		}
	}

	private void highlight(JoglGuiLabel label)
	{
		unHighlight();
		
		label.setBackColor(JoglGuiStyle.highlightColor);
		highlighted = label;
		
		bUpdateScene = true;
	}

	private void unHighlight() 
	{
		if (highlighted != null)
		{
			highlighted.setBackColor(JoglGuiStyle.backColor);
			highlighted = null;

			bUpdateScene = true;
		}
	}
	
	private void OpenDropDownBox(JoglGuiLabel label, List<String> vals)
	{
		if (ddbox != null)
		{
			CloseDropDownBox();
		}

		if (!vals.isEmpty())
		{
			JoglVectord4 pos = label.getRect();
			ddbox = new JIDropdownBox(iman, gui, label, vals);
			ddbox.setZLevel(-201.0f);
			ddbox.addDropDownListener(this);
			ddbox.setPosition((float)pos.getX(), (float)pos.getY() + 0.05f);
			ddbox.setWidth(0.45f);
			ddbox.setTextSize(0.03f);
			iman.addInteraction(ddbox);
		}
		
    	bUpdateScene = true;	
	}

	private void CloseDropDownBox()
	{
		//close dropdown
		iman.removeInteraction(ddbox);
		ddbox.Dispose();
		ddbox = null;
	}
	
	public void OnDropDownSelected(JIDropdownBox ddb, String val) 
	{
		if (ddbox == ddb)
		{
			if (val != null)
			{
				if (ddb.getOwner() == btnClassifierType)
				{
					btnClassifierType.setText(val);
					OpenAddCohortDialog(val);
				}
			}
			CloseDropDownBox();
			
			bUpdateScene = true;
		}
	}
	
	public void Close()
	{
		if (classifierPanel != null && classifierPanel.getResultCohort() != null)
		{
			if (parentName == null)
			{
				mycc.addCohort(classifierPanel.getResultCohort());
			}
			else
			{
				mycc.addCohort(classifierPanel.getResultCohort(), parentName);
			}
		}
		
		JISplitCohorts.preferredClassifierType = btnClassifierType.getText();
		
		Dispose();
		iman.removeInteraction(this);
		
		//parent.reopen();
		iman.addInteraction(parent);
		parent.Init();
	}
	
}
