package org.processmining.processprofiler3d.compviz.jogl.interactions.cohortsplit;

import java.awt.event.MouseEvent;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Vector;

import org.deckfour.xes.model.XLog;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredRect;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiListBox;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiTextbox;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglScreenRect;
import org.processmining.processprofiler3d.compviz.jogl.interactions.IDropDownListener;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIDropdownBox;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JITextboxInput;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortLiteral;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.processprofiler3d.psen.classification.CohortDefinition;
import org.processmining.processprofiler3d.psen.classification.CohortDefinitionLiteral;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.input.JoglInputMouseWheelEvent;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JICohortConfig2GenericLiteral extends JICohortConfig2Panel implements IDropDownListener
{
	private JoglGuiLabel lblHeader;
	private JoglGuiLabel lblNoCohort;
	private JoglGuiLabel lblTotal;
	private JoglGuiLabel btnAttribute;
	private JoglGuiTextbox btnCohortName;
	private JoglGuiListBox lbxAvailable;
	private JoglGuiListBox lbxSelected;
	private JoglGuiLabel btnAdd;
	private JoglGuiLabel btnRemove;
	private JoglGuiLabel btnAccept;
	private JoglGuiLabel btnCancel;
	private JoglGuiColoredRect background;
	
	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();
	private JoglGuiLabel highlighted = null;
	private JIDropdownBox ddbox = null;
	
	private boolean bInit = true;
	
	private CohortClassifier mycc = null;
	private CohortClassifier testcc = null;
	
	private XLog mylog = null;
	private LogAnalyser la = null;

	private CohortDefinition forcedDef = null;
	
	private Vector<String> availableValues = new Vector<String>();
	private Vector<String> selectedValues = new Vector<String>();
	
	private Date lastClick;
	private JoglScreenRect lastTarget = null;
	private int doubleClickTime = 600;
	
	public JICohortConfig2GenericLiteral(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui, CohortClassifier cc, XLog log, JICohortConfig2 jicc, CohortDefinition def)
	{
		super(iman, scene, gui, jicc);
		
		mycc = cc;
		mylog = log;
		la = new LogAnalyser(log);
		
		forcedDef = def;
		
		setZLevel(-200.0f);
		
		//updatedAttribute();
		
		lastClick = new Date(new Date().getTime() - doubleClickTime);
	}
	
	private void updatedAttribute()
	{
		String attrName = btnAttribute.getText();
		testcc = new CohortClassifier();

		selectedValues.clear();
		availableValues.clear();
		
		if (forcedDef != null && forcedDef instanceof CohortDefinitionLiteral)
		{
			CohortDefinitionLiteral cdl = (CohortDefinitionLiteral)forcedDef;
			for (String s : la.getAvailableTraceAttrValuesLiteral(attrName))
			{
				if (cdl.values.contains(s))
				{
					selectedValues.add(s);
				}
				else
				{
					availableValues.add(s);
				}
			}
		}
		else
		{
			for (String s : la.getAvailableTraceAttrValuesLiteral(attrName))
			{
				availableValues.add(s);
			}
		}
		
		updateLists();
	}
	
	@Override
	public void Init()
	{	
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = 0.05f / aspect;
		ff.height = 0.05f;
		ff.align = FontFormat.ALIGN_CENTER;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff2 = new FontFormat();
		ff2.width = 0.05f / aspect;
		ff2.height = 0.05f;
		ff2.align = FontFormat.ALIGN_LEFT;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		float textPadding2 = 0.01f;
		
		background = new JoglGuiColoredRect();
		background.setCanTouch(true);
		background.setRect(-0.25f, -0.5f, 0.5f, 1.125f);
		background.setColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(background);
		controls.add(background);

		lblHeader = new JoglGuiLabel(ff.clone());
		lblHeader.setRect(-0.25f, 0.575f, 0.5f, 0.05f);
		lblHeader.setText("Add Generic Literal Cohort");
		lblHeader.setPadding(textPadding);
		lblHeader.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(lblHeader);
		controls.add(lblHeader);

		lblNoCohort = new JoglGuiLabel(ff2.clone());
		lblNoCohort.setRect(-0.225f, -0.4f, 0.2f, 0.05f);
		lblNoCohort.setText("No Cohort");
		lblNoCohort.setPadding(textPadding2);
		lblNoCohort.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(lblNoCohort);
		controls.add(lblNoCohort);

		lblTotal = new JoglGuiLabel(ff2.clone());
		lblTotal.setRect(0.025f, -0.4f, 0.2f, 0.05f);
		lblTotal.setText("Total");
		lblTotal.setPadding(textPadding2);
		lblTotal.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(lblTotal);
		controls.add(lblTotal);

		//if preferred attribute is not a literal attribute, choose a literal attribute
		String prefAttrib = getPreferredAttributeName();
		
		btnAttribute = new JoglGuiLabel(ff.clone());
		btnAttribute.setCanTouch(true);
		btnAttribute.setRect(-0.225f, 0.425f, 0.45f, 0.05f);
		btnAttribute.setText(prefAttrib);
		btnAttribute.setPadding(textPadding);
		btnAttribute.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnAttribute);
		controls.add(btnAttribute);
		
		btnCohortName = new JoglGuiTextbox(ff.clone());
		btnCohortName.setCanTouch(true);
		btnCohortName.setRect(-0.225f, 0.35f, 0.45f, 0.05f);
		if (forcedDef == null)
		{
			btnCohortName.setText("Cohort" + (mycc.getNumCohorts() + 1));
		}
		else
		{
			btnCohortName.setText(forcedDef.name);
		}
		btnCohortName.setPadding(textPadding);
		btnCohortName.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnCohortName);
		controls.add(btnCohortName);
		
		updatedAttribute();
		
		btnAdd = new JoglGuiLabel(ff.clone());
		btnAdd.setCanTouch(true);
		btnAdd.setRect(-0.0225f, 0.03f, 0.045f, 0.075f);
		btnAdd.setText(">>");
		btnAdd.setPadding(textPadding);
		btnAdd.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnAdd);
		controls.add(btnAdd);
		
		btnRemove = new JoglGuiLabel(ff.clone());
		btnRemove.setCanTouch(true);
		btnRemove.setRect(-0.0225f, -0.055f, 0.045f, 0.075f);
		btnRemove.setText("<<");
		btnRemove.setPadding(textPadding);
		btnRemove.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnRemove);
		controls.add(btnRemove);
		
		btnAccept = new JoglGuiLabel(ff.clone());
		btnAccept.setCanTouch(true);
		btnAccept.setRect(-0.225f, -0.475f, 0.2f, 0.05f);
		btnAccept.setText("Add");
		btnAccept.setPadding(textPadding);
		btnAccept.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnAccept);
		controls.add(btnAccept);
		
		btnCancel = new JoglGuiLabel(ff.clone());
		btnCancel.setCanTouch(true);
		btnCancel.setRect(0.025f, -0.475f, 0.2f, 0.05f);
		btnCancel.setText("Cancel");
		btnCancel.setPadding(textPadding);
		btnCancel.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnCancel);
		controls.add(btnCancel);
		
		bInit = false;
	}

	private String getPreferredAttributeName() 
	{
		String prefAttrib = JISplitCohorts.preferredAttribute;
		if (forcedDef != null)
		{
			prefAttrib = forcedDef.attrName;
		}
		if (!ClassificationCohortLiteral.isValidAttribute(la, prefAttrib))
		{
			prefAttrib = getAttributes().firstElement();
		}
		return prefAttrib;
	}

	private Vector<String> getAttributes()
	{
		Vector<String> lst = new Vector<String>(ClassificationCohortLiteral.getAvailableAttributes(la));
		Collections.sort(lst);
		return lst;
	}

	public void updateLists() 
	{
		if (lbxAvailable != null)
		{
			controls.remove(lbxAvailable);
			gui.RemoveGuiObject(lbxAvailable);
		}
		if (lbxSelected != null)
		{
			controls.remove(lbxSelected);
			gui.RemoveGuiObject(lbxSelected);
		}
		
		lbxAvailable = new JoglGuiListBox();
		lbxAvailable.setCanTouch(true);
		lbxAvailable.setCanMultiSelect(true);
		lbxAvailable.setRect(-0.225f, -0.35f, 0.2f, 0.675f);
		lbxAvailable.setZLevel(-200.0f);
		lbxAvailable.setTextItems(availableValues);
		lbxAvailable.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(lbxAvailable);
		controls.add(lbxAvailable);
		
		lbxSelected = new JoglGuiListBox();
		lbxSelected.setCanTouch(true);
		lbxSelected.setCanMultiSelect(true);
		lbxSelected.setRect(0.025f, -0.35f, 0.2f, 0.675f);
		lbxSelected.setZLevel(-200.0f);
		lbxSelected.setTextItems(selectedValues);
		lbxSelected.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(lbxSelected);
		controls.add(lbxSelected);

		testcc.removeCohort("Available");
		testcc.removeCohort("Selected");
		
		testcc.addCohort(new CohortDefinitionLiteral("Available", btnAttribute.getText(), new HashSet<String>(availableValues)));
		testcc.addCohort(new CohortDefinitionLiteral("Selected", btnAttribute.getText(), new HashSet<String>(selectedValues)));
		
		HashMap<Integer, Integer> histogram = testcc.getCohortCaseHistogram(mylog);
		
		lblNoCohort.setText("Cases (" + histogram.get(0) + ")");
		lblTotal.setText("Cases (" + histogram.get(1) + ")");
	}

	@Override
	public void Dispose()
	{	
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{	
		if (bInit)
		{
			Init();
		}
		
		bUpdateScene = false;
	}
	
	private String getCohortName()
	{
		return btnCohortName.getText();
	}
	
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MousePressed");
			Vector<JoglGuiObject> jgos = gui.GetObjectsFromPos(e.getX(), e.getY());
			if (jgos.contains(lbxAvailable))
			{
				lbxAvailable.mousePressed(iman, e);
				return true;
			}
			if (jgos.contains(lbxSelected))
			{
				lbxSelected.mousePressed(iman, e);
				return true;
			}
			if (jgos.contains(btnCohortName))
			{
				return true;
			}
			if (jgos.contains(btnAttribute))
			{
				return true;
			}
			if (jgos.contains(btnAdd))
			{
				return true;
			}
			if (jgos.contains(btnRemove))
			{
				return true;
			}
			if (jgos.contains(btnAccept))
			{
				return true;
			}
			if (jgos.contains(btnCancel))
			{
				return true;
			}
			if (jgos.contains(background))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MouseReleased");
			Vector<JoglGuiObject> jgos = gui.GetObjectsFromPos(e.getX(), e.getY());
			
			Date nowTime = new Date();
			//check if is double click
			if (new TimeSpan(lastClick, nowTime).getMilliseconds() < doubleClickTime)
			{
				if (jgos.contains(lbxAvailable))
				{
					JoglScreenRect cTarget = lbxAvailable.getTargetLabel(e);
					//if clicked same target as previous click in short time
					if (lastTarget == cTarget)
					{
						DoubleClick(e);
					}
				}
				else if (jgos.contains(lbxSelected))
				{
					JoglScreenRect cTarget = lbxSelected.getTargetLabel(e);
					//if clicked same target as previous click in short time
					if (lastTarget == cTarget)
					{
						DoubleClick(e);
					}
				}
			}
			
			if (jgos.contains(btnCohortName))
			{
				JITextboxInput jibt = new JITextboxInput(iman, scene, gui, btnCohortName);
				iman.addInteraction(jibt);
				
				return true;
			}
			if (jgos.contains(btnAttribute))
			{
				return true;
			}
			if (jgos.contains(btnAdd))
			{	
				AddPressed();
				
				return true;
			}
			if (jgos.contains(btnRemove))
			{	
				RemovePressed();
				
				return true;
			}
			if (jgos.contains(btnAccept))
			{
				resultCohort = new CohortDefinitionLiteral(getCohortName(), btnAttribute.getText(), new HashSet<String>(selectedValues));
				
				if (forcedDef != null)
				{
					for (CohortDefinition cd : forcedDef.children)
					{
						resultCohort.children.add(cd);
					}
				}
				
				Close();
				
				return true;
			}
			if (jgos.contains(btnCancel))
			{
				//jglb.mousePressed(e);
				//if editing cohort, re-add existing cohort
				if (forcedDef != null)
				{
					resultCohort = forcedDef;
				}
				Close();
				
				return true;
			}

			if (jgos.contains(lbxAvailable))
			{
				lastTarget = lbxAvailable.getTargetLabel(e);
			}
			else if (jgos.contains(lbxSelected))
			{
				lastTarget = lbxSelected.getTargetLabel(e);
			}
			else
			{
				lastTarget = null;
			}

			lastClick = nowTime;
			
			if (jgos.contains(background))
			{
				return true;
			}
		}
		return false;
    }

	@Override
	public boolean mouseClicked(JoglInputMouseEvent e)
	{
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MouseReleased");
			Vector<JoglGuiObject> jgos = gui.GetObjectsFromPos(e.getX(), e.getY());
			
			if (jgos.contains(btnAttribute))
			{
				//LogAnalyser la = new LogAnalyser(mylog);
				OpenDropDownBox(btnAttribute, getAttributes());
				
				return true;
			}
		}
		return false;
	}
	
	public void DoubleClick(JoglInputMouseEvent e) 
	{
		JoglVectord2 pos = gui.GetNDCPos(e.getX(), e.getY());
		if (lbxAvailable.IsInside((float)pos.getX(), (float)pos.getY()))
		{
			//double clicked lbxAvailable
			AddPressed();
		}
		if (lbxSelected.IsInside((float)pos.getX(), (float)pos.getY()))
		{
			//double clicked lbxAvailable
			RemovePressed();
		}
	}

	public void AddPressed() 
	{
		boolean bUpdateLists = false;
		
		//int idx = lbxAvailable.getSelectedIndex();
		Vector<Integer> idxLst = lbxAvailable.getSelectedIndices();
		Collections.sort(idxLst);
		Collections.reverse(idxLst);
		for (Integer idx : idxLst)
		{
			if (idx > -1)
			{
				String s = availableValues.get(idx);
				availableValues.removeElementAt(idx);
				selectedValues.add(s);
				
				bUpdateLists = true;
			}
		}

		if (bUpdateLists)
		{
			updateLists();
		}
	}

	public void RemovePressed() 
	{
		boolean bUpdateLists = false;
		
		//int idx = lbxSelected.getSelectedIndex();
		Vector<Integer> idxLst = lbxSelected.getSelectedIndices();
		Collections.sort(idxLst);
		Collections.reverse(idxLst);
		for (Integer idx : idxLst)
		{
			if (idx > -1)
			{
				String s = selectedValues.get(idx);
				selectedValues.removeElementAt(idx);
				availableValues.add(s);

				bUpdateLists = true;
			}
		}

		if (bUpdateLists)
		{
			updateLists();
		}
	}
	
	private void OpenDropDownBox(JoglGuiLabel label, List<String> vals)
	{
		if (ddbox != null)
		{
			CloseDropDownBox();
		}

		if (!vals.isEmpty())
		{
			JoglVectord4 pos = label.getRect();
			ddbox = new JIDropdownBox(iman, gui, label, vals);
			ddbox.setZLevel(-201.0f);
			ddbox.addDropDownListener(this);
			ddbox.setPosition((float)pos.getX(), (float)pos.getY() + 0.05f);
			ddbox.setWidth(0.45f);
			ddbox.setTextSize(0.03f);
			iman.addInteraction(ddbox);
		}
		
    	bUpdateScene = true;	
	}

	private void CloseDropDownBox()
	{
		//close dropdown
		iman.removeInteraction(ddbox);
		ddbox.Dispose();
		ddbox = null;
	}
	
	public void OnDropDownSelected(JIDropdownBox ddb, String val) 
	{
		if (ddbox == ddb)
		{
			if (val != null)
			{
				if (ddb.getOwner() == btnAttribute)
				{
					btnAttribute.setText(val);
					updatedAttribute();
				}
			}
			CloseDropDownBox();
			
			bUpdateScene = true;
		}
	}
	
	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnCohortName)
			{
				highlight(btnCohortName);
			}
			else if (jgo == btnAttribute)
			{
				highlight(btnAttribute);
			}
			else if (jgo == btnAdd)
			{
				highlight(btnAdd);
			}
			else if (jgo == btnRemove)
			{
				highlight(btnRemove);
			}
			else if (jgo == btnAccept)
			{
				highlight(btnAccept);
			}
			else if (jgo == btnCancel)
			{
				highlight(btnCancel);
			}
			else
			{
				unHighlight();
			}
		}
		else
		{
			unHighlight();
		}
	}

	private void highlight(JoglGuiLabel label)
	{
		unHighlight();
		
		label.setBackColor(JoglGuiStyle.highlightColor);
		highlighted = label;
		
		bUpdateScene = true;
	}
	
	private void unHighlight() 
	{
		if (highlighted != null)
		{
			highlighted.setBackColor(JoglGuiStyle.backColor);
			highlighted = null;

			bUpdateScene = true;
		}
	}

	@Override
	public void mouseWheelMoved(JoglInputMouseWheelEvent e)
	{
		if (lbxAvailable != null)
		{
			lbxAvailable.mouseWheelMoved(e);
		}
		if (lbxSelected != null)
		{
			lbxSelected.mouseWheelMoved(e);
		}
	}

	@Override
	public void Close()
	{
		JISplitCohorts.preferredAttribute = btnAttribute.getText();
		
		Dispose();
		iman.removeInteraction(this);
		
		parent.Close();
	}
	
}
