package org.processmining.processprofiler3d.compviz.jogl.interactions.cohortsplit;

import java.awt.event.MouseEvent;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.deckfour.xes.model.XLog;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiCheckBox;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiColoredRect;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiHistogram;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiTextbox;
import org.processmining.processprofiler3d.compviz.jogl.interactions.IDropDownListener;
import org.processmining.processprofiler3d.compviz.jogl.interactions.ITextboxInputListener;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIDropdownBox;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JITextboxInput;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JITextboxInputNumeric;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortTimeOfDay;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.processprofiler3d.psen.classification.CohortDefinition;
import org.processmining.processprofiler3d.psen.classification.CohortDefinitionTimeOfDay;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.maths.JoglColor3;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JICohortConfig2TimeOfDay extends JICohortConfig2Panel implements ITextboxInputListener, IDropDownListener
{
	private JoglGuiLabel lblHeader;
	private JoglGuiLabel lblNoCohort;
	private JoglGuiLabel lblTotal;
	private JoglGuiLabel btnAttribute;
	private JoglGuiTextbox btnCohortName;
	private JoglGuiLabel lblStartTime;
	private JoglGuiLabel lblEndTime;
	private JoglGuiLabel lblInclude;
	private JoglGuiTextbox btnStartTime;
	private JoglGuiTextbox btnEndTime;
	private JoglGuiLabel btnAdd;
	private JoglGuiLabel btnCancel;
	private JoglGuiColoredRect background;
	private JoglGuiCheckBox cbxStartIncl;
	private JoglGuiCheckBox cbxEndIncl;
	private JoglGuiLabel lblHist1;
	private JoglGuiLabel lblHist2;
	private JoglGuiLabel lblHist3;
	
	private JoglGuiHistogram guiHist;
	
	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();
	private JoglGuiLabel highlighted = null;
	private JIDropdownBox ddbox = null;
	
	private boolean bInit = true;
	
	private CohortClassifier mycc = null;
	private CohortClassifier testcc = null;
	
	private XLog mylog;
	private LogAnalyser la = null;
	
	private boolean bHasError = false;

	private CohortDefinition forcedDef = null;
	
	public JICohortConfig2TimeOfDay(JoglInteractionManager iman, JoglScene scene, JSOGraphicalUserInterface gui, CohortClassifier cc, XLog log, JICohortConfig2 jicc, CohortDefinition def)
	{
		super(iman, scene, gui, jicc);
		
		mycc = cc;
		
		mylog = log;
		la = new LogAnalyser(log);
		
		forcedDef = def;
		
		setZLevel(-200.0f);
	}

	@Override
	public void Init()
	{	
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = 0.05f / aspect;
		ff.height = 0.05f;
		ff.align = FontFormat.ALIGN_CENTER;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff2 = new FontFormat();
		ff2.width = 0.05f / aspect;
		ff2.height = 0.05f;
		ff2.align = FontFormat.ALIGN_LEFT;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff3 = new FontFormat();
		ff3.width = 0.05f / aspect;
		ff3.height = 0.05f;
		ff3.align = FontFormat.ALIGN_RIGHT;
		ff3.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		float textPadding2 = 0.01f;
		
		background = new JoglGuiColoredRect();
		background.setCanTouch(true);
		background.setRect(-0.25f, -0.5f, 0.5f, 1.125f);
		background.setColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(background);
		controls.add(background);

		lblHeader = new JoglGuiLabel(ff.clone());
		lblHeader.setRect(-0.25f, 0.575f, 0.5f, 0.05f);
		lblHeader.setText("Add Time Of Day Cohort");
		lblHeader.setPadding(textPadding);
		lblHeader.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(lblHeader);
		controls.add(lblHeader);

		lblNoCohort = new JoglGuiLabel(ff2.clone());
		lblNoCohort.setRect(-0.225f, -0.375f, 0.2f, 0.05f);
		lblNoCohort.setText("No Cohort");
		lblNoCohort.setPadding(textPadding2);
		lblNoCohort.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblNoCohort);
		controls.add(lblNoCohort);

		lblTotal = new JoglGuiLabel(ff2.clone());
		lblTotal.setRect(0.025f, -0.375f, 0.2f, 0.05f);
		lblTotal.setText("Total");
		lblTotal.setPadding(textPadding2);
		lblTotal.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblTotal);
		controls.add(lblTotal);
		
		//if preferred attribute is not a timestamp attribute, choose a timestamp attribute
		String prefAttrib = getPreferredAttribute();
		
		btnAttribute = new JoglGuiLabel(ff.clone());
		btnAttribute.setCanTouch(true);
		btnAttribute.setRect(-0.225f, 0.425f, 0.45f, 0.05f);
		btnAttribute.setText(prefAttrib);
		btnAttribute.setPadding(textPadding);
		btnAttribute.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnAttribute);
		controls.add(btnAttribute);
		
		btnCohortName = new JoglGuiTextbox(ff.clone());
		btnCohortName.setCanTouch(true);
		btnCohortName.setRect(-0.225f, 0.35f, 0.45f, 0.05f);
		if (forcedDef == null)
		{
			btnCohortName.setText("Cohort" + (mycc.getNumCohorts() + 1));
		}
		else
		{
			btnCohortName.setText(forcedDef.name);
		}
		btnCohortName.setPadding(textPadding);
		btnCohortName.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnCohortName);
		controls.add(btnCohortName);

		lblStartTime = new JoglGuiLabel(ff.clone());
		lblStartTime.setRect(-0.225f, 0.15f, 0.2f, 0.05f);
		lblStartTime.setText("Start Time (Hour)");
		lblStartTime.setPadding(textPadding);
		lblStartTime.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblStartTime);
		controls.add(lblStartTime);

		lblEndTime = new JoglGuiLabel(ff.clone());
		lblEndTime.setRect(-0.225f, 0.05f, 0.2f, 0.05f);
		lblEndTime.setText("End Time (Hour)");
		lblEndTime.setPadding(textPadding);
		lblEndTime.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblEndTime);
		controls.add(lblEndTime);

		lblInclude = new JoglGuiLabel(ff.clone());
		lblInclude.setRect(0.075f, 0.25f, 0.15f, 0.05f);
		lblInclude.setText("Inclusive");
		lblInclude.setPadding(textPadding);
		lblInclude.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblInclude);
		controls.add(lblInclude);
		
		btnStartTime = new JoglGuiTextbox(ff.clone());
		btnStartTime.setCanTouch(true);
		btnStartTime.setRect(-0.025f, 0.15f, 0.1f, 0.05f);
		if (forcedDef != null && forcedDef instanceof CohortDefinitionTimeOfDay)
		{
			CohortDefinitionTimeOfDay cdtod = (CohortDefinitionTimeOfDay)forcedDef;
			btnStartTime.setText("" + cdtod.startTime);
		}
		else
		{
			btnStartTime.setText("0");
		}
		btnStartTime.setPadding(textPadding);
		btnStartTime.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnStartTime);
		controls.add(btnStartTime);
		
		btnEndTime = new JoglGuiTextbox(ff.clone());
		btnEndTime.setCanTouch(true);
		btnEndTime.setRect(-0.025f, 0.05f, 0.1f, 0.05f);
		if (forcedDef != null && forcedDef instanceof CohortDefinitionTimeOfDay)
		{
			CohortDefinitionTimeOfDay cdtod = (CohortDefinitionTimeOfDay)forcedDef;
			btnEndTime.setText("" + cdtod.endTime);
		}
		else
		{
			btnEndTime.setText("1");
		}
		btnEndTime.setPadding(textPadding);
		btnEndTime.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnEndTime);
		controls.add(btnEndTime);
		
		float cbxWidth = (0.035f / aspect);
		
		cbxStartIncl = new JoglGuiCheckBox();
		cbxStartIncl.setCanTouch(true);	
		cbxStartIncl.setRect(0.15f - (cbxWidth * 0.5f), 0.1575f, cbxWidth, 0.035f);
		cbxStartIncl.setLineDims(0.0025f / aspect, 0.0025f);
		cbxStartIncl.setColor(JoglColor3.WHITE);
		cbxStartIncl.setBackColor(JoglGuiStyle.backColor);
		if (forcedDef != null && forcedDef instanceof CohortDefinitionTimeOfDay)
		{
			CohortDefinitionTimeOfDay cdtod = (CohortDefinitionTimeOfDay)forcedDef;
			cbxStartIncl.setChecked(cdtod.bInclStart);
		}
		
		gui.AddGuiObject(cbxStartIncl);
		controls.add(cbxStartIncl);
		
		cbxEndIncl = new JoglGuiCheckBox();
		cbxEndIncl.setCanTouch(true);	
		cbxEndIncl.setRect(0.15f - (cbxWidth * 0.5f), 0.0575f, cbxWidth, 0.035f);
		cbxEndIncl.setLineDims(0.0025f / aspect, 0.0025f);
		cbxEndIncl.setColor(JoglColor3.WHITE);
		cbxEndIncl.setBackColor(JoglGuiStyle.backColor);
		if (forcedDef != null && forcedDef instanceof CohortDefinitionTimeOfDay)
		{
			CohortDefinitionTimeOfDay cdtod = (CohortDefinitionTimeOfDay)forcedDef;
			cbxEndIncl.setChecked(cdtod.bInclEnd);
		}
		
		gui.AddGuiObject(cbxEndIncl);
		controls.add(cbxEndIncl);
		
		btnAdd = new JoglGuiLabel(ff.clone());
		btnAdd.setCanTouch(true);
		btnAdd.setRect(-0.225f, -0.475f, 0.2f, 0.05f);
		btnAdd.setText("Add");
		btnAdd.setPadding(textPadding);
		btnAdd.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnAdd);
		controls.add(btnAdd);
		
		btnCancel = new JoglGuiLabel(ff.clone());
		btnCancel.setCanTouch(true);
		btnCancel.setRect(0.025f, -0.475f, 0.2f, 0.05f);
		btnCancel.setText("Cancel");
		btnCancel.setPadding(textPadding);
		btnCancel.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnCancel);
		controls.add(btnCancel);

		guiHist = new JoglGuiHistogram(gui);
		guiHist.setColor(JoglColor3.RED);
		guiHist.setRect(-0.225f, -0.2f, 0.45f, 0.15f);
		//guiHist.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(guiHist);
		controls.add(guiHist);

		lblHist1 = new JoglGuiLabel(ff2.clone());
		lblHist1.setRect(-0.225f, -0.255f, 0.45f, 0.05f);
		lblHist1.setText("00:00");
		lblHist1.setPadding(textPadding);
		lblHist1.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblHist1);
		controls.add(lblHist1);

		lblHist2 = new JoglGuiLabel(ff.clone());
		lblHist2.setRect(-0.225f, -0.255f, 0.45f, 0.05f);
		lblHist2.setText("12:00");
		lblHist2.setPadding(textPadding);
		lblHist2.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblHist2);
		controls.add(lblHist2);

		lblHist3 = new JoglGuiLabel(ff3.clone());
		lblHist3.setRect(-0.225f, -0.255f, 0.45f, 0.05f);
		lblHist3.setText("24:00");
		lblHist3.setPadding(textPadding);
		lblHist3.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblHist3);
		controls.add(lblHist3);
		
		updatedAttribute();
		
		bInit = false;
		
	}

	private String getPreferredAttribute() 
	{
		String prefAttrib = JISplitCohorts.preferredAttribute;
		if (forcedDef != null)
		{
			prefAttrib = forcedDef.attrName;
		}
		if (!ClassificationCohortTimeOfDay.isValidAttribute(la, prefAttrib))
		{
			Vector<String> attrs = getAttributes();
			if (!attrs.isEmpty())
			{
				prefAttrib = attrs.firstElement();
			}
			else
			{
				prefAttrib = "";
			}
		}
		return prefAttrib;
	}

	@Override
	public void Dispose()
	{
		//jglb.Dispose();
		
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{	
		if (bInit)
		{
			Init();
		}
		
		bUpdateScene = false;
	}
	
	private String getCohortName()
	{
		return btnCohortName.getText();
	}
	
	private int getStartTime()
	{
		String startTimeStr = cleanString(btnStartTime.getText());
		return Integer.parseInt(startTimeStr);
	}
	
	private int getEndTime()
	{
		String endTimeStr = cleanString(btnEndTime.getText());
		
		return Integer.parseInt(endTimeStr);
	}

	private String cleanString(String valueStr) 
	{
		String timeStr = "";
		for (int i = 0; i < valueStr.length(); i++)
		{
			char c = valueStr.charAt(i);
			if (Character.isDigit(c))
			{
				timeStr += c;
			}
		}
		return timeStr;
	}
	
	private boolean includeStart()
	{
		return cbxStartIncl.getChecked();
	}
	
	private boolean includeEnd()
	{
		return cbxEndIncl.getChecked();
	}
	
	private boolean hasValidStartTime()
	{
		return (getStartTime() >= 0 && getStartTime() <= 24);
	}
	
	private boolean hasValidEndTime()
	{
		return (getEndTime() >= 0 && getEndTime() <= 24);
	}

	private Vector<String> getAttributes()
	{
		Vector<String> lst = new Vector<String>(ClassificationCohortTimeOfDay.getAvailableAttributes(la));
		Collections.sort(lst);
		return lst;
	}
	
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MousePressed");
			Vector<JoglGuiObject> jgos = gui.GetObjectsFromPos(e.getX(), e.getY());
//			if (jgos.contains(jglb))
//			{
//				jglb.mousePressed(e);
//				return true;
//			}
			if (jgos.contains(btnCohortName))
			{
				//jglb.mousePressed(e);
				return true;
			}
			if (jgos.contains(btnAttribute))
			{
				//jglb.mousePressed(e);
				return true;
			}
			if (jgos.contains(btnStartTime))
			{
				//jglb.mousePressed(e);
				return true;
			}
			if (jgos.contains(btnEndTime))
			{
				//jglb.mousePressed(e);
				return true;
			}
			if (jgos.contains(btnAdd))
			{
				//jglb.mousePressed(e);
				return true;
			}
			if (jgos.contains(btnCancel))
			{
				//jglb.mousePressed(e);
				return true;
			}
			if (jgos.contains(background))
			{
				return true;
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MouseReleased");
			Vector<JoglGuiObject> jgos = gui.GetObjectsFromPos(e.getX(), e.getY());
			//jglb.mouseReleased(e);
			if (jgos.contains(btnCohortName))
			{
				JITextboxInput jibt = new JITextboxInput(iman, scene, gui, btnCohortName);
				iman.addInteraction(jibt);
				
				return true;
			}
			if (jgos.contains(btnAttribute))
			{
				return true;
			}
			if (jgos.contains(btnStartTime))
			{	
				JITextboxInput jibt = new JITextboxInputNumeric(iman, scene, gui, btnStartTime);
				jibt.listener = this;
				iman.addInteraction(jibt);
				
				return true;
			}
			if (jgos.contains(btnEndTime))
			{	
				JITextboxInput jibt = new JITextboxInputNumeric(iman, scene, gui, btnEndTime);
				jibt.listener = this;
				iman.addInteraction(jibt);
				
				return true;
			}
			if (jgos.contains(btnAdd))
			{
				if (!bHasError)
				{
					AddCohort();
				
					Close();
				}
				return true;
			}
			if (jgos.contains(btnCancel))
			{
				//jglb.mousePressed(e);
				//if editing cohort, re-add existing cohort
				if (forcedDef != null)
				{
					resultCohort = forcedDef;
				}
				Close();
				
				return true;
			}
			if (jgos.contains(background))
			{
				return true;
			}
		}
		return false;
    }

	@Override
	public boolean mouseClicked(JoglInputMouseEvent e)
	{
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MouseReleased");
			Vector<JoglGuiObject> jgos = gui.GetObjectsFromPos(e.getX(), e.getY());
			
			if (jgos.contains(btnAttribute))
			{
				//LogAnalyser la = new LogAnalyser(mylog);
				OpenDropDownBox(btnAttribute, getAttributes());
				
				return true;
			}
			if (jgos.contains(cbxStartIncl))
			{
				cbxStartIncl.setChecked(!cbxStartIncl.getChecked());
				
				updateLists();
				
				bUpdateScene = true;
				return true;
			}
			if (jgos.contains(cbxEndIncl))
			{
				cbxEndIncl.setChecked(!cbxEndIncl.getChecked());
				
				updateLists();
				
				bUpdateScene = true;
				return true;
			}
		}
		return false;
	}

	public void AddCohort()
	{
		resultCohort = new CohortDefinitionTimeOfDay(getCohortName(), btnAttribute.getText(), getStartTime(), getEndTime(), includeStart(), includeEnd());

		if (forcedDef != null)
		{
			for (CohortDefinition cd : forcedDef.children)
			{
				resultCohort.children.add(cd);
			}
		}
	}
	
	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnCohortName)
			{
				highlight(btnCohortName);
			}
			else if (jgo == btnAttribute)
			{
				highlight(btnAttribute);
			}
			else if (jgo == btnStartTime)
			{
				highlight(btnStartTime);
			}
			else if (jgo == btnEndTime)
			{
				highlight(btnEndTime);
			}
			else if (jgo == btnAdd)
			{
				highlight(btnAdd);
			}
			else if (jgo == btnCancel)
			{
				highlight(btnCancel);
			}
			else
			{
				unHighlight();
			}
		}
		else
		{
			unHighlight();
		}
	}

	private void highlight(JoglGuiLabel label)
	{
		unHighlight();
		
		label.setBackColor(JoglGuiStyle.highlightColor);
		highlighted = label;
		
		bUpdateScene = true;
	}
	
	private void unHighlight() 
	{
		if (highlighted != null)
		{
			highlighted.setBackColor(JoglGuiStyle.backColor);
			if (highlighted == btnStartTime && !hasValidStartTime())
			{
				highlighted.setBackColor(JoglColor3.RED);
			}
			if (highlighted == btnEndTime && !hasValidEndTime())
			{
				highlighted.setBackColor(JoglColor3.RED);
			}
			
			highlighted = null;

			bUpdateScene = true;
		}
	}
	
	private void OpenDropDownBox(JoglGuiLabel label, List<String> vals)
	{
		if (ddbox != null)
		{
			CloseDropDownBox();
		}

		if (!vals.isEmpty())
		{
			JoglVectord4 pos = label.getRect();
			ddbox = new JIDropdownBox(iman, gui, label, vals);
			ddbox.setZLevel(-201.0f);
			ddbox.addDropDownListener(this);
			ddbox.setPosition((float)pos.getX(), (float)pos.getY() + 0.05f);
			ddbox.setWidth(0.45f);
			ddbox.setTextSize(0.03f);
			iman.addInteraction(ddbox);
		}
		
    	bUpdateScene = true;	
	}

	private void CloseDropDownBox()
	{
		//close dropdown
		iman.removeInteraction(ddbox);
		ddbox.Dispose();
		ddbox = null;
	}
	
	public void OnDropDownSelected(JIDropdownBox ddb, String val) 
	{
		if (ddbox == ddb)
		{
			if (val != null)
			{
				if (ddb.getOwner() == btnAttribute)
				{
					btnAttribute.setText(val);
					updatedAttribute();
				}
			}
			CloseDropDownBox();
			
			bUpdateScene = true;
		}
	}
	
	public void updateLists() 
	{
		if (getAttributes().isEmpty())
		{
			return;
		}
	
		//testcc.removeCohort("Available");
		testcc.removeCohort("Selected");
		
		//testcc.addCohort("Available", availableValues);
		testcc.addCohort(new CohortDefinitionTimeOfDay("Selected", btnAttribute.getText(), getStartTime(), getEndTime(), includeStart(), includeEnd()));
		
		int selStart = getStartTime();
		int selEnd = getEndTime();
		if (!includeStart())
		{
			selStart += 1;
		}
		if (includeEnd())
		{
			selEnd += 1;
		}
		guiHist.setSelection(selStart, selEnd);
		
		HashMap<Integer, Integer> histogram = testcc.getCohortCaseHistogram(mylog);
		
		lblNoCohort.setText("Excluded (" + histogram.get(-1) + ")");
		lblTotal.setText("Included (" + histogram.get(0) + ")");
	}

	@Override
	public void Close()
	{
		JISplitCohorts.preferredAttribute = btnAttribute.getText();
		
		Dispose();
		iman.removeInteraction(this);
		
		parent.Close();
	}
	
	private void updatedAttribute()
	{
		testcc = new CohortClassifier();
		
		Vector<Integer> hist = getHistogram(mylog);
		guiHist.setHistogram(hist);
			
		updateLists();
	}

	public void OnTextInputFinished(JITextboxInput jti) 
	{
		if (jti.getLabel() == btnStartTime)
		{
			if (!hasValidStartTime())
			{
				btnStartTime.setBackColor(JoglColor3.RED);
			}
			else
			{
				btnStartTime.setBackColor(JoglGuiStyle.backColor);
			}
		}
		if (jti.getLabel() == btnEndTime)
		{
			if (!hasValidEndTime())
			{
				btnEndTime.setBackColor(JoglColor3.RED);
			}
			else
			{
				btnEndTime.setBackColor(JoglGuiStyle.backColor);
			}
		}
		bHasError = !(hasValidStartTime() && hasValidEndTime());
		
		if (!bHasError)
		{
			updateLists();
		}
	}
	
	private Vector<Integer> getHistogram(XLog l)
	{
		Vector<Integer> lst = new Vector<Integer>();
		
		if (getAttributes().isEmpty())
		{
			for (int i = 0; i < 24; i++)
			{
				lst.add(0);
			}
		}
		else
		{
			CohortClassifier xcc = new CohortClassifier();
			for (int i = 0; i < 24; i++)
			{
				xcc.addCohort(new ClassificationCohortTimeOfDay("Test " + i, btnAttribute.getText(), i, i+1, true, false));
			}		
			
			HashMap<Integer, Integer> map = xcc.getCohortCaseHistogram(l);
			
			for (int i = 0; i < 24; i++)
			{
				lst.add(map.get(i));
			}
		}
		
		return lst;
	}
}
