package org.processmining.processprofiler3d.compviz.jogl.interactions.cohortsplit;

import java.awt.event.MouseEvent;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.compviz.jogl.interactions.IDropDownListener;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIDropdownBox;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIOptionsPanel;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortIntRange;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortLiteral;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortTimeOfDay;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.processprofiler3d.psen.classification.CohortPreset;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PerformanceStatisticsDB;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.measurements.TimeSpan;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JISplitCohorts extends JIOptionsPanel implements IDropDownListener
{
	private XLog log = null;
	private LogAnalyser la = null;
	
	private IPerformanceStatisticsEnrichedNet psen = null;
	private IPerformanceStatisticsEnrichedNet wten = null;
	
	private JoglScene scene;
	private JoglInteractionManager iman;
	
	private JoglGuiLabel btnClassifierType;
	private JoglGuiLabel btnAttribute;
	private JoglGuiLabel btnPreset;
	private JoglGuiLabel btnUpdate;

	private JoglGuiLabel lblClassifierType;
	private JoglGuiLabel lblAttribute;
	private JoglGuiLabel lblPreset;

	private boolean bVisible = false;
	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();
	private JoglGuiLabel highlighted = null;
	private JIDropdownBox ddbox = null;
	
	private Vector<XTrace> tracelst;
	
	public JISplitCohorts(JoglInteractionManager iman, XLog l, JoglScene scene, JSOGraphicalUserInterface gui, IPerformanceStatisticsEnrichedNet psen, IPerformanceStatisticsEnrichedNet wten)
	{
		super(gui);
		
		this.log = l;
		this.la = new LogAnalyser(l);
		this.scene = scene;
		this.iman = iman;
		
		this.psen = psen;
		this.wten = wten;

		Date startTime = new Date();
		//cache the traces we actually have data for
		tracelst = ((PerformanceStatisticsDB)psen).getValidTraces(log);
		Date endTime = new Date();
		TimeSpan ts = new TimeSpan(startTime, endTime);
		System.out.println("GetValidTraces completed (" + ts.ToString(TimeUnit.MILLISECONDS) + "ms).");
		
		setZLevel(-300.0f);
	}

	@Override
	public boolean isVisible()
	{
		return bVisible;
	}
	
	@Override
	public void Show()
	{
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_LEFT;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff2 = new FontFormat();
		ff2.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff2.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff2.align = FontFormat.ALIGN_RIGHT;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;

		float colLabelWidth = 0.2f * (float)JoglGuiStyle.textScale;
		float colButtonWidth = 0.2f * (float)JoglGuiStyle.textScale;
		
		float colButtonX = 0.95f - colButtonWidth;
		float colLabelX = colButtonX - colLabelWidth - 0.005f;
		
		float yoff = 0.85f;

		btnClassifierType = new JoglGuiLabel(ff.clone());
		btnClassifierType.setCanTouch(true);
		btnClassifierType.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		btnClassifierType.setText(getClassifierTypes().get(0));
		btnClassifierType.setPadding(textPadding);
		btnClassifierType.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnClassifierType);
		controls.add(btnClassifierType);

		lblClassifierType = new JoglGuiLabel(ff2.clone());
		lblClassifierType.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblClassifierType.setText("Classifier Type");
		lblClassifierType.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblClassifierType);
		controls.add(lblClassifierType);
		
		yoff -= 0.1f;

		Vector<String> attributesLst = getAttributes();
		btnAttribute = new JoglGuiLabel(ff.clone());
		btnAttribute.setCanTouch(true);
		btnAttribute.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		if (attributesLst.isEmpty())
		{
			btnAttribute.setText("");
		}
		else
		{
			btnAttribute.setText(attributesLst.get(0));
		}
		btnAttribute.setPadding(textPadding);
		btnAttribute.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnAttribute);
		controls.add(btnAttribute);
		
		lblAttribute = new JoglGuiLabel(ff2.clone());
		lblAttribute.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblAttribute.setText("Attribute");
		lblAttribute.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblAttribute);
		controls.add(lblAttribute);
		
		yoff -= 0.1f;

		Vector<String> presetsLst = getPresets();
		btnPreset = new JoglGuiLabel(ff.clone());
		btnPreset.setCanTouch(true);
		btnPreset.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		if (presetsLst.isEmpty())
		{
			btnPreset.setText("");
		}
		else
		{
			btnPreset.setText(presetsLst.get(0));
		}
		btnPreset.setPadding(textPadding);
		btnPreset.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnPreset);
		controls.add(btnPreset);
		
		lblPreset = new JoglGuiLabel(ff2.clone());
		lblPreset.setRect(colLabelX, yoff, colLabelWidth, 0.05f);
		lblPreset.setText("Preset");
		lblPreset.setBackColor(JoglGuiStyle.transparentColor);
		
		gui.AddGuiObject(lblPreset);
		controls.add(lblPreset);
		
		yoff -= 0.1f;
		
		btnUpdate = new JoglGuiLabel(ff.clone());
		btnUpdate.setCanTouch(true);
		btnUpdate.setRect(colButtonX, yoff, colButtonWidth, 0.05f);
		btnUpdate.setText("Update");
		btnUpdate.setPadding(textPadding);
		btnUpdate.setBackColor(JoglGuiStyle.backColor);
		
		gui.AddGuiObject(btnUpdate);
		controls.add(btnUpdate);

		bVisible = true;
	}

	@Override
	public void Hide()
	{
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
		
		bVisible = false;
	}
	
	private boolean bUpdateScene = false;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{	
//		if (bInit)
//		{
//			Init();
//		}
		
		bUpdateScene = false;
	}
	
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MousePressed");
			JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
			if (jgo != null)
			{
				if (controls.contains(jgo))
				{
					return true;
				}
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		if (e.getButton() == MouseEvent.BUTTON1)
		{
			//System.out.println("JIViewManager: MouseReleased");
			JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
			if (jgo != null)
			{
				if (controls.contains(jgo))
				{
					return true;
				}
			}
		}
		return false;
    }
	
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {		
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnClassifierType)
			{
	        	OpenDropDownBox(btnClassifierType, getClassifierTypes());
	        	return true;
			}
			else if (jgo == btnAttribute)
			{
	        	OpenDropDownBox(btnAttribute, getAttributes());
	        	return true;
			}
			else if (jgo == btnPreset)
			{
	        	OpenDropDownBox(btnPreset, getPresets());
	        	return true;
			}
			else if (jgo == btnUpdate)
			{
				if (ddbox != null)
				{
					CloseDropDownBox();
				}

				updateCohorts();
	
	        	bUpdateScene = true;
	        	return true;
			}
		}
		
		return false;
    }
	
	private void OpenDropDownBox(JoglGuiLabel label, List<String> lst)
	{
		if (ddbox != null)
		{
			CloseDropDownBox();
		}

		if (!lst.isEmpty())
		{
			JoglVectord4 pos = label.getRect();
			ddbox = new JIDropdownBox(iman, gui, label, lst);
			ddbox.addDropDownListener(this);
			ddbox.setPosition((float)pos.getX(), (float)pos.getY() + 0.05f);
			ddbox.setWidth(0.2f * (float)JoglGuiStyle.textScale);
			ddbox.setTextSize(0.03f);
			iman.addInteraction(ddbox);
		}
    	bUpdateScene = true;
	}

	private void CloseDropDownBox()
	{
		//close dropdown
		iman.removeInteraction(ddbox);
		ddbox.Dispose();
		ddbox = null;
	}
	
	public void OnDropDownSelected(JIDropdownBox ddb, String val) 
	{
		//System.out.println("OnDropDownSelected!");
		if (ddbox == ddb)
		{
			if (val != null)
			{
				if (ddb.getOwner() == btnClassifierType)
				{
					btnClassifierType.setText(val);
					setClassifierType(val);
				}
				else if (ddb.getOwner() == btnAttribute)
				{
					btnAttribute.setText(val);
				}
				else if (ddb.getOwner() == btnPreset)
				{
					btnPreset.setText(val);
				}
			}
			
			CloseDropDownBox();
			bUpdateScene = true;
		}
	}
	
	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (jgo == btnClassifierType)
			{
				highlight(btnClassifierType);
			}
			else if (jgo == btnAttribute)
			{
				highlight(btnAttribute);
			}
			else if (jgo == btnPreset)
			{
				highlight(btnPreset);
			}
			else if (jgo == btnUpdate)
			{
				highlight(btnUpdate);
			}
			else
			{
				unHighlight();
			}
		}
		else
		{
			unHighlight();
		}
	}
	
	private void highlight(JoglGuiLabel label)
	{
		unHighlight();
		
		label.setBackColor(JoglGuiStyle.highlightColor);
		highlighted = label;
		
		bUpdateScene = true;
	}

	private void unHighlight() 
	{
		if (highlighted != null)
		{
			highlighted.setBackColor(JoglGuiStyle.backColor);
			highlighted = null;

			bUpdateScene = true;
		}
	}
	
	private void setClassifierType(String val)
	{
		Vector<String> availableAttrs = getAttributes();
		if (!availableAttrs.isEmpty())
		{
			btnAttribute.setText(availableAttrs.get(0));
		}
		else
		{
			btnAttribute.setText("");
		}
		
		Vector<String> availablePresets = getPresets();
		if (!availablePresets.isEmpty())
		{
			btnPreset.setText(availablePresets.get(0));
		}
		else
		{
			btnPreset.setText("");
		}
	}
	
	private Vector<String> getClassifierTypes()
	{
		Vector<String> lst = new Vector<String>();
		lst.add(ClassificationCohortLiteral.classifierName);
		lst.add(ClassificationCohortTimeOfDay.classifierName);
		lst.add(ClassificationCohortIntRange.classifierName);
		
		return lst;
	}
	
	private Vector<String> getAttributes()
	{
		Vector<String> lst = new Vector<String>();

    	String selectedClassifierType = btnClassifierType.getText();
    	if (selectedClassifierType == ClassificationCohortTimeOfDay.classifierName)
    	{
    		lst.addAll(ClassificationCohortTimeOfDay.getAvailableAttributes(la));
    	}
    	else if (selectedClassifierType == ClassificationCohortLiteral.classifierName)
    	{
    		lst.addAll(ClassificationCohortLiteral.getAvailableAttributes(la));
    	}	
    	else if (selectedClassifierType == ClassificationCohortIntRange.classifierName)
    	{
    		lst.addAll(ClassificationCohortIntRange.getAvailableAttributes(la));
    	}
    	
    	Collections.sort(lst);
    	
		return lst;
	}
	
	private Vector<String> getPresets()
	{
		Vector<String> lst = new Vector<String>();
		String attributeName = btnAttribute.getText();
		if (attributeName != "")
		{
			lst.addAll(CohortClassifier.getPresetNames(btnAttribute.getText(), log));
		}
    	return lst;
	}
	
	public static String preferredClassifierType = "";
	public static String preferredAttribute = "";
	public CohortClassifier mycc = new CohortClassifier();
	
	public void updateCohorts()
	{
		String attributeName = btnAttribute.getText();
		
		CohortPreset preset = CohortClassifier.getPreset(btnPreset.getText(), attributeName, log);
    	preferredClassifierType = btnClassifierType.getText();
    	preferredAttribute = attributeName;

    	if (attributeName != "")
    	{
	    	if (preset != null)
	    	{
	    		mycc = new CohortClassifier();
	    		mycc.setPreset(preset);
				
				updateCohorts(mycc);
	    	}
	    	else
	    	{
	    		JICohortConfig jicc = iman.findInteraction(JICohortConfig.class);
	    		JICohortConfig2 jicc2 = iman.findInteraction(JICohortConfig2.class);
	    		
	    		//don't open a new config window if one is already open
	    		if (jicc == null && jicc2 == null)
	    		{
	    			jicc = new JICohortConfig(iman, scene, gui, mycc, log);
	    			iman.addInteraction(jicc);
	    		}
	    	}
    	}
	}

	
	public void updateCohorts(CohortClassifier cc)
	{
		JIAsyncDataSplit jiads = new JIAsyncDataSplit(iman, gui, psen, wten, cc, log, tracelst);
		iman.addInteraction(jiads);
	}
}
