package org.processmining.processprofiler3d.compviz.jogl.interactions.replay;

import java.util.Date;
import java.util.Vector;

import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOReplaya;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayCollection;

public class ReplayProcessor
{
	private Vector<TraceReplay> replays = new Vector<TraceReplay>();
	private double maxLength = 0.0;
	
	public Date processStartTime;
	
	//private int numReplays = 0;
	private int numReplaysTotal = 0;
	private int numReplaysTotalExpected = 0;
	
	public double getMaxLength()
	{
		maxLength = 0;
		for (ReplayProcessorThread rpt : rpts)
		{
			maxLength = Math.max(maxLength, rpt.maxLength);
		}
		return maxLength;
	}
	
	public Vector<TraceReplay> getReplays()
	{
		replays = new Vector<TraceReplay>();
		for (ReplayProcessorThread rpt : rpts)
		{
			replays.addAll(rpt.replays);
		}
		return replays;
	}
	
	private Vector<ReplayProcessorThread> rpts = new Vector<ReplayProcessorThread>();
	
	public void start(TraceReplayCollection trc, JSOReplaya rpla, JSOPetriNet jn, Marking initMarking)
	{	
		ReplaySimOptimizer rso = new ReplaySimOptimizer(jn);
		
		processStartTime = new Date();
		
		numReplaysTotalExpected = trc.getValidTraceReplays().size();

		// get the runtime object associated with the current Java application
		Runtime runtime = Runtime.getRuntime();
		
		// get the number of processors available to the Java virtual machine
		int numberOfProcessors = runtime.availableProcessors();
		 
		System.out.println("Number of processors available to this JVM: " + numberOfProcessors);
		
		//reset replayer tokens!
		rpla.ClearParticles();
		
		//use at least one processor, but where enough cores available leave two for UI and rendering
		int useProcessors = Math.max(1, numberOfProcessors - 2);
		int parts = (int)Math.ceil(((float)numReplaysTotalExpected) / ((float)useProcessors));
		
		int num = 1;
		int start = 0;
		int end = parts;
		while (start < numReplaysTotalExpected)
		{
			ReplayProcessorThread rpt1 = new ReplayProcessorThread("Token Processing Thread " + num, start, end);
			rpt1.start(trc, rpla, jn, initMarking, rso);	
			rpts.add(rpt1);
			
			num++;
			start = end;
			end = Math.min(numReplaysTotalExpected, end + parts);
		}
//		ReplayProcessorThread rpt1 = new ReplayProcessorThread("New 1", 0, 250);
//		rpt1.start(trc, rpla, pg, jn, initMarking);	
//		rpts.add(rpt1);
	}
	
	public float getProgress()
	{
		numReplaysTotal = 0;
		if (!rpts.isEmpty())
		{
			for (ReplayProcessorThread rpt : rpts)
			{
				numReplaysTotal = numReplaysTotal + rpt.getProcessedCount();
			}
			return ((float)numReplaysTotal) / ((float)numReplaysTotalExpected);
		}
		return 0.0f;
	}
	
	public String getProgressString()
	{
		numReplaysTotal = 0;
		if (!rpts.isEmpty())
		{
			for (ReplayProcessorThread rpt : rpts)
			{
				numReplaysTotal = numReplaysTotal + rpt.getProcessedCount();
			}
			return "" + numReplaysTotal + "/" + numReplaysTotalExpected;
		}
		return "0/0";
	}
	
	public boolean isFinished()
	{
		if (!rpts.isEmpty())
		{
			for (ReplayProcessorThread rpt : rpts)
			{
				if (!rpt.isFinished())
				{
					return false;
				}
			}
			return true;
		}
		return false;
	}

}
