package org.processmining.processprofiler3d.compviz.jogl.interactions.replay;

import java.util.Vector;

import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOReplaya;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayAnalyser;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayCollection;

public class ReplayProcessorThread implements Runnable
{
	private Thread t;
	private String threadName = "Thread";

	private TraceReplayCollection trc = null;
	private JSOReplaya rpla;
	private JSOPetriNet jpnet;
	private Marking initMarking;
	private ReplaySimOptimizer rso;
	
	public Vector<TraceReplay> replays = new Vector<TraceReplay>();
	public double maxLength = 0.0;
	
	private int numReplays = 0;
	private int numReplaysTotal = 0;
	private int numReplaysTotalExpected = 0;
	
	private int numStart = 0;
	private int numEnd = 0;
	
	public ReplayProcessorThread(String tName, int numS, int numE)
	{
		threadName = tName;
		
		numStart = numS;
		numEnd = numE;
	}
	
	public void run()
	{	
		replays.clear();
		maxLength = 0.0;
		
		numReplays = 0;
		numReplaysTotal = 0;
		numReplaysTotalExpected = trc.getValidTraceReplays().size();
		//for (TraceReplayAnalyser tra : trc.getValidTraceReplays())
		for (int i = numStart; i < numEnd; i++)
		{	
			TraceReplayAnalyser tra = trc.getValidTraceReplays().get(i);
			TraceReplay rx = new TraceReplay(rpla, jpnet, initMarking, tra);
			if (rx.Simulate(rso))
			{
				maxLength = Math.max(maxLength, rx.getTotalLength());
				replays.add(rx);
				numReplays++;
			}
			numReplaysTotal++;
			//System.out.println("Processed replays: " + numReplaysTotal + "/" + numReplaysTotalExpected);
		}
		System.out.println(threadName + " finished. Number of replays that could be simulated correctly: " + numReplays + "/" + numReplaysTotal);
	}
	   
	public void start(TraceReplayCollection trc, JSOReplaya rpla, JSOPetriNet jn, Marking initMarking, ReplaySimOptimizer rso)
	{
		this.trc = trc;
		this.rpla = rpla;
		this.jpnet = jn;
		this.initMarking = initMarking;
		this.rso = rso;
		
		System.out.println("Starting " +  threadName + "(" + numStart + "-" + numEnd + ")");
		if (t == null)
		{
			t = new Thread(this, threadName);
			t.start();
		}
	}
	
	public float getProgress()
	{
		return ((float)numReplaysTotal) / ((float)numReplaysTotalExpected);
	}
	
	public int getProcessedCount()
	{
		return numReplaysTotal;
	}
	
	public String getProgressString()
	{
		return "" + numReplaysTotal + "/" + numReplaysTotalExpected;
	}
	
	public boolean isFinished()
	{
		return (t.getState() == Thread.State.TERMINATED);
		//return !t.isAlive();
	}

}
