package org.processmining.processprofiler3d.compviz.jogl.interactions.replay;

import java.util.HashMap;
import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnPlace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;

//This class should work as a cache for the mapping operations that get repeated lots during token flow replay,
//to reduce computations and processing time
public class ReplaySimOptimizer 
{
	private JSOPetriNet jpnet;
	private HashMap<PetrinetNode, JSOPnPlace> placeMap = new HashMap<PetrinetNode, JSOPnPlace>(); 
	private HashMap<JSOPnTransition, JSOSubNet> transSubMap = new HashMap<JSOPnTransition, JSOSubNet>(); 
	private HashMap<JSOSubNet, JSOPnPlace> startPlaceMap = new HashMap<JSOSubNet, JSOPnPlace>(); 
	private HashMap<JSOSubNet, JSOPnPlace> endPlaceMap = new HashMap<JSOSubNet, JSOPnPlace>(); 

	private HashMap<JSOPnTransition, Vector<JSOPnPlace>> precMap = new HashMap<JSOPnTransition, Vector<JSOPnPlace>>();
	private HashMap<JSOPnTransition, Vector<JSOPnPlace>> follMap = new HashMap<JSOPnTransition, Vector<JSOPnPlace>>();
	
	public ReplaySimOptimizer(JSOPetriNet pnet)
	{
		jpnet = pnet;
		
		for (JSOSubNet sn : jpnet.getAllSubNets())
		{
			startPlaceMap.put(sn, getStartPlaceHelper(sn));
			endPlaceMap.put(sn, getEndPlaceHelper(sn));
			transSubMap.put(sn.getParent(), sn);
			
			for (JSOPnTransition t : sn.getTransitions())
			{
				precMap.put(t, getPreceedingPlacesHelper(t));
				follMap.put(t, getFollowingPlacesHelper(t));
			}
		}
		
		for (JSOPnPlace p : jpnet.getPlaces())
		{
			placeMap.put(p.orgPlace, p);
		}
	}
	
	public JSOPnPlace getPlaceFor(PetrinetNode p)
	{
		//return jpnet.getPlaceFor(p);
		return placeMap.get(p);
	}
	
	public JSOSubNet getSubnetForTransition(JSOPnTransition t)
	{
		if (transSubMap.containsKey(t))
		{
			return transSubMap.get(t);
		}
		return null;
	}
	
	public JSOPnPlace getStartPlace(JSOSubNet sn)
	{
		return startPlaceMap.get(sn);
	}
	
	public JSOPnPlace getEndPlace(JSOSubNet sn)
	{
		return endPlaceMap.get(sn);
	}
	
	private JSOPnPlace getStartPlaceHelper(JSOSubNet sn)
	{
		Place sp = sn.getPetriNet().getStartPlace();
		JSOPnPlace px = sn.getPlaceFor(sp);
		return px;
	}
	
	private JSOPnPlace getEndPlaceHelper(JSOSubNet sn)
	{
		Place sp = sn.getPetriNet().getEndPlace();
		JSOPnPlace px = sn.getPlaceFor(sp);
		return px;
	}
	
	public Vector<JSOPnPlace> getPreceedingPlaces(JSOPnTransition t)
	{
		return precMap.get(t);
	}
	
	public Vector<JSOPnPlace> getFollowingPlaces(JSOPnTransition t)
	{
		return follMap.get(t);
	}
	
	private Vector<JSOPnPlace> getPreceedingPlacesHelper(JSOPnTransition t)
	{

		INetDataSource net = t.getParentNet().getPetriNet();
		Vector<JSOPnPlace> tokens = new Vector<JSOPnPlace>();
		for (Arc a : net.getInEdges(t.getOrgNode()))
		{
			JSOPnPlace px = t.getParentNet().getPlaceFor(a.getSource());
			for (int i = 0; i < a.getWeight(); i++)
			{
				tokens.add(px);
			}
		}
		return tokens;
	}
	
	private Vector<JSOPnPlace> getFollowingPlacesHelper(JSOPnTransition t)
	{
		INetDataSource net = t.getParentNet().getPetriNet();
		Vector<JSOPnPlace> tokens = new Vector<JSOPnPlace>();
		for (Arc a : net.getOutEdges(t.getOrgNode()))
		{
			JSOPnPlace px = t.getParentNet().getPlaceFor(a.getSource());
			for (int i = 0; i < a.getWeight(); i++)
			{
				tokens.add(px);
			}
		}
		return tokens;
	}
}
