package org.processmining.processprofiler3d.compviz.jogl.interactions.replay;

import java.util.Vector;

import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayAnalyser;
import org.processmining.processprofiler3d.psen.replayanalysis.TransitionEvent;

public class ReplaySimulator 
{
	private JSOPetriNet jpnet = null;
	private Vector<TransitionEvent> tes = null;
	private int stepNum = 0;
	private ReplayaStateSim replx = null;
	
	public ReplaySimulator(JSOPetriNet jn, Marking initMarking, TraceReplayAnalyser lra, ReplaySimOptimizer rso)
	{
		jpnet = jn;
		
		tes = lra.getAllTransitionEvents();
		stepNum = 0;
		
		//initialize state
		replx = new ReplayaStateSim(jpnet, initMarking, rso);	
	}
	
	public boolean Simulate()
	{
		//System.out.println("Simulate trace replay.");
		
		//find max eventId
		int max = 0;
		for (TransitionEvent te : tes)
		{
			max = Math.max(max, te.getCompleteId());
		}
		
		while (stepNum <= max)
		{
			NextStep();
		}
		
		executionOrder = new Vector<ReplayaEvent>();
		
		while (!rEvts.isEmpty())
		{
			if (!executeNextEvent())
			{
				System.out.println("Could not finish trace replay simulation!");
				return false;
			}
		}
		replx.FinishReplaySim();
		return true;
	}
	
	public Vector<TokenInfo> getTokens()
	{
		return replx.getTokens();
	}
	
	public Vector<ReplayaEvent> getExecutionOrder()
	{
		return executionOrder;
	}

	private Vector<ReplayaEvent> executionOrder;
	
	private boolean executeNextEvent() 
	{
		int minStartId = 10000; //TODO: set proper init value
		for (int i = 0; i < rEvts.size(); i++)
		{
			minStartId = Math.min(minStartId, rEvts.get(i).sortEventId);
		}
		
		Vector<ReplayaEvent> executable = new Vector<ReplayaEvent>();
		for (int i = 0; i < rEvts.size(); i++)
		{
			ReplayaEvent r = rEvts.get(i);
			
			if (replx.CanTransitionEventFire(r))
			{
				executable.add(r);
			}	
		}
		
		int minExecId = 10000; //TODO: set proper init value
		for (int i = 0; i < executable.size(); i++)
		{
			minExecId = Math.min(minExecId, executable.get(i).sortEventId);
		}
		
//		if (executable.size() > 1)
//		{
//			String s = "";
//		}
		if (minExecId > minStartId)
		{
			String s = "";
		}
		
		for (ReplayaEvent r : executable)
		{
			if (r.sortEventId == minExecId)
			{
				//System.out.println("Execute: " + r.toString());
				ExecuteEvt(r);
				executionOrder.add(r);
				rEvts.remove(r);
				
				return true;
			}
		}
		
		return false;
	}

	private Vector<ReplayaEvent> rEvts = new Vector<ReplayaEvent>();
	
	public void NextStep()
	{	
		for (TransitionEvent te : tes)
		{
			if (te.getStartId() == stepNum)
			{
				//consume parent tokens
				JSOPnTransition t = jpnet.getTransitionFor(te.getTransition().getLocalID());
				
				ReplayaEvent r1 = new ReplayaEvent(ReplayaEvent.CONSUME_TOKENS);
				r1.startEventId = te.getStartId();
				r1.endEventId = te.getCompleteId();
				r1.sortEventId = stepNum;
				r1.t = t;
				r1.te = te;
				rEvts.add(r1);
				
				if (t.hasSubnet())
				{
					//produce start token
					ReplayaEvent r2 = new ReplayaEvent(ReplayaEvent.PRODUCE_STARTTOKENS);
					r2.startEventId = te.getStartId();
					r2.endEventId = te.getStartId();
					r2.sortEventId = stepNum;
					r2.t = t;
					r2.te = te;
					rEvts.add(r2);
				}			
			}
			if (te.getCompleteId() == stepNum)
			{
				//create child tokens
				JSOPnTransition t = jpnet.getTransitionFor(te.getTransition().getLocalID());
				
				ReplayaEvent r1 = new ReplayaEvent(ReplayaEvent.PRODUCE_TOKENS);
				r1.startEventId = te.getStartId();
				r1.endEventId = te.getCompleteId();
				r1.sortEventId = stepNum;
				r1.t = t;
				r1.te = te;
				rEvts.add(r1);
				
				if (t.hasSubnet())
				{
					//consume end token
					ReplayaEvent r2 = new ReplayaEvent(ReplayaEvent.CONSUME_ENDTOKENS);
					r2.startEventId = te.getCompleteId();
					r2.endEventId = te.getCompleteId();
					r2.sortEventId = stepNum;
					r2.t = t;
					r2.te = te;
					rEvts.add(r2);
				}
			}
		}

//		JSOReplaya repl = scene.findSceneObject(JSOReplaya.class);
//		repl.SetState(replx);
		
		stepNum++;
	}
	
	private void ExecuteEvt(ReplayaEvent evt)
	{
		if (evt.type == ReplayaEvent.CONSUME_TOKENS)
		{
			replx.consume(evt, evt.t);
		}
		else if (evt.type == ReplayaEvent.PRODUCE_TOKENS)
		{
			replx.produce(evt, evt.t);
		}
		else if (evt.type == ReplayaEvent.CONSUME_ENDTOKENS)
		{
			replx.consumeEndToken(evt, evt.t);
		}
		else if (evt.type == ReplayaEvent.PRODUCE_STARTTOKENS)
		{
			replx.produceStartToken(evt, evt.t);
		}
	}

}
