package org.processmining.processprofiler3d.compviz.jogl.interactions.replay;

import java.util.Date;

import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.psen.replayanalysis.TransitionEvent;

public class ReplayaEvent 
{
	public static int CONSUME_TOKENS = 0;
	public static int PRODUCE_TOKENS = 1;
	public static int CONSUME_ENDTOKENS = 2;
	public static int PRODUCE_STARTTOKENS = 3;
	public static int INITIAL_MARKING = 4;
	public static int FINAL_MARKING = 5;
	
	public int type = -1;
	public int startEventId = -1;
	public int endEventId = -1;
	public int sortEventId = -1;
	
	public TransitionEvent te = null;
	public JSOPnTransition t = null;
	
	public double maxFactor = 1.0;
	public double stepFactor = 0.0;
	public double startTime = 0.0;
	public double endTime = 1.0;
	
	public ReplayaEvent(int evtType)
	{
		type = evtType;
	}
	
	public void setMaxFactor(XTrace trace)
	{
		startTime = getStartTime(trace).getTime() / 100000.0;
		endTime = getEndTime(trace).getTime() / 100000.0;
		maxFactor = endTime - startTime;
	}
	
	public boolean isStarted(double time)
	{
		double t = time - startTime;
		if (t >= 0.0)
		{
			return true;
		}
		return false;
	}
	
	public boolean isComplete(double time)
	{
		double t = time - startTime;
		if (t >= maxFactor)
		{
			return true;
		}
		return false;
	}
	
//	public double advance(double deltaTime)
//	{
//		//stepFactor = stepFactor + deltaTime;
//		
//		if (stepFactor + deltaTime >= maxFactor)
//		{
//			double x = maxFactor - stepFactor;
//			stepFactor = maxFactor;
//			return x;
//		}
//		stepFactor = stepFactor + deltaTime;
//		return deltaTime;
//	}
	
	public double advance(double time)
	{
		double t = Math.min(maxFactor, Math.max(0, time - startTime));
		stepFactor = t;
		
		return 0;
	}
	
	public double timeScale()
	{
		return maxFactor;
	}
	
	private Date getStartTime(XTrace trace)
	{
		if (type == PRODUCE_TOKENS)
		{
			Date ds = getEventTimestamp(trace.get(startEventId));
			Date de = getEventTimestamp(trace.get(endEventId));
			long timespan = de.getTime() - ds.getTime();
			
			return new Date(ds.getTime() + (long)(timespan * 0.5));
		}
		else if (type == CONSUME_ENDTOKENS)
		{
			Date de = getEventTimestamp(trace.get(endEventId));
			
			return new Date(de.getTime());
		}
		else if (type == PRODUCE_STARTTOKENS)
		{
			Date ds = getEventTimestamp(trace.get(startEventId));
			
			return new Date(ds.getTime());
		}
		return getEventTimestamp(trace.get(startEventId));
	}
	
	private Date getEndTime(XTrace trace)
	{
		if (type == CONSUME_TOKENS)
		{
			Date ds = getEventTimestamp(trace.get(startEventId));
			Date de = getEventTimestamp(trace.get(endEventId));
			long timespan = de.getTime() - ds.getTime();
			
			return new Date(ds.getTime() + (long)(timespan * 0.5));
		}
		else if (type == CONSUME_ENDTOKENS)
		{
			Date de = getEventTimestamp(trace.get(endEventId));
			return new Date(de.getTime());
		}
		else if (type == PRODUCE_STARTTOKENS)
		{
			Date ds = getEventTimestamp(trace.get(startEventId));
			
			return new Date(ds.getTime());
		}
		return getEventTimestamp(trace.get(endEventId));
	}
	
	private Date getEventTimestamp(XEvent evt)
	{
		return getAttributeValueTimestamp(evt, XTimeExtension.KEY_TIMESTAMP);
	}
	
	private static Date getAttributeValueTimestamp(XEvent evt, String name)
	{
		XAttributeTimestamp xAtt3 = (XAttributeTimestamp)evt.getAttributes().get(name);
		if (xAtt3 != null && xAtt3.getValue() != null)
		{
			Date eventName = xAtt3.getValue();
			return eventName;
		}
		return null;
	}
	
	@Override
	public String toString()
	{
		String evtType = "";
		if (type == CONSUME_TOKENS)
		{
			evtType = "Consume";
		}
		if (type == PRODUCE_TOKENS)
		{
			evtType = "Produce";
		}
		if (type == PRODUCE_STARTTOKENS)
		{
			evtType = "Produce Start Tokens";
		}
		if (type == CONSUME_ENDTOKENS)
		{
			evtType = "Consume End Tokens";
		}
		
		return "ReplayaEvent (" + evtType + "): " + t.getLabel();
	}
}
