package org.processmining.processprofiler3d.compviz.jogl.interactions.replay;

import java.util.HashMap;
import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnPlace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOReplaya;
import org.processmining.processprofiler3d.compviz.jogl.scene.TokenCube;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class ReplayaState
{
	private JSOReplaya rpla = null;
	private JSOPetriNet jpnet = null;
	private Vector<TokenCube> objs = new Vector<TokenCube>();
	private HashMap<JSOPnPlace, Vector<TokenCube>> pobjs = new HashMap<JSOPnPlace, Vector<TokenCube>>();

	private HashMap<JSOPnTransition, Vector<Vector<TokenCube>>> activeTransitions = new HashMap<JSOPnTransition, Vector<Vector<TokenCube>>>();
	
	//private JoglVectord3 tokenColor = JoglColor3.RED;
	private int tokenCohort = -1;
	
	public ReplayaState(JSOReplaya rpla, JSOPetriNet pnet, Marking initMarking, int cohort)
	{
		this.rpla = rpla;
		jpnet = pnet;
		tokenCohort = cohort;
		
		InitializeState(pnet, initMarking);
	}

	private void InitializeState(JSOPetriNet pnet, Marking initMarking) 
	{
		for (Place p : initMarking.toList())
		{
			JSOPnPlace pl = pnet.getPlaceFor(p);
			
			if (!pobjs.containsKey(pl))
			{
				pobjs.put(pl, new Vector<TokenCube>());
			}
			
			int i = pobjs.get(pl).size();
			TokenCube c = new TokenCube();
			c.setTokenCohort(tokenCohort);
			c.place = pl;
			
			JoglVectord3 sourcePos = JoglVectord3.Add(pl.getPosition(), pl.getParentNet().getOffset());
			c.setPosition(JoglVectord3.Add(sourcePos, new JoglVectord3(0, 5 * (i + 1),0)));
			ITokenLane tl = LaneCache.getStaticLane(pl);
			c.setTokenLane(tl);
			tl.AddToken(c);
			
			objs.add(c);
			//rpla.AddToken(c);
			pobjs.get(pl).add(c);
		}
	}
	
	public void consume(double startTime, double timeSpan, JSOPnTransition t)
	{
		//consume parent tokens
		INetDataSource net = t.getParentNet().getPetriNet();
		Vector<TokenCube> tokens = new Vector<TokenCube>();
		for (Arc a : net.getInEdges(t.getOrgNode()))
		{
			JSOPnPlace px = jpnet.getPlaceFor(a.getSource());
			for (int i = 0; i < a.getWeight(); i++)
			{
				TokenCube tc = RemoveToken(px, t);
				tc.timeScale = timeSpan;
				tc.startTime = startTime;
				tc.endTime = tc.startTime + tc.timeScale;
				tokens.add(tc);
			}
		}
		
		if (!activeTransitions.containsKey(t))
		{
			activeTransitions.put(t, new Vector<Vector<TokenCube>>());
		}
		activeTransitions.get(t).add(tokens);
	}

	public void produceStartToken(JSOPnTransition t) 
	{
		if (getActivityInstances(t) > 0)
		{
			INetDataSource net = t.getParentNet().getPetriNet();
			Place sp = net.getSubnetForTransition(t.orgTransition).getStartPlace();
			JSOPnPlace px = jpnet.getPlaceFor(sp);
			AddToken(px, t);
		}
	}

	public void produce(double startTime, double timeSpan, JSOPnTransition t)
	{
		if (getActivityInstances(t) > 0)
		{
			for (TokenCube jso : activeTransitions.get(t).get(0))
			{
				if (jso.getTokenLane() != null)
				{
					jso.getTokenLane().RemoveToken(jso);
				}
				objs.remove(jso);
				//rpla.RemoveToken(jso);
			}
			activeTransitions.get(t).remove(0);
			
			//create child tokens
			INetDataSource net = t.getParentNet().getPetriNet();
			for (Arc a : net.getOutEdges(t.getOrgNode()))
			{
				JSOPnPlace px = jpnet.getPlaceFor(a.getTarget());
				for (int i = 0; i < a.getWeight(); i++)
				{
					TokenCube tc = AddToken(px, t);
					tc.timeScale = timeSpan;
					tc.startTime = startTime;
					tc.endTime = tc.startTime + tc.timeScale;
				}
			}
		}
	}

	public void consumeEndToken(JSOPnTransition t) 
	{
		//if (getActivityInstances(t) > 0)
		{
			INetDataSource net = t.getParentNet().getPetriNet();
			Place sp = net.getSubnetForTransition(t.orgTransition).getEndPlace();
			JSOPnPlace px = jpnet.getPlaceFor(sp);
			TokenCube tc = RemoveToken(px, t);
			if (tc.getTokenLane() != null)
			{
				tc.getTokenLane().RemoveToken(tc);
			}

			objs.remove(tc);
			//rpla.RemoveToken(tc);
		}
	}
	
	public int getActivityInstances(JSOPnTransition t)
	{
		if (activeTransitions.containsKey(t))
		{
			return activeTransitions.get(t).size();
		}
		return 0;
	}
	
	public void ClearTokens()
	{
//		for (TokenCube tc : objs)
//		{
//			rpla.RemoveToken(tc);
//		}
		objs.clear();
	}
	
	public int getTokenNum(JSOPnPlace source)
	{
		if (!pobjs.containsKey(source))
		{
			pobjs.put(source, new Vector<TokenCube>());
		}
		
		return pobjs.get(source).size();
	}
	
	public TokenCube AddToken(JSOPnPlace target, JSOPnTransition source)
	{
		if (!pobjs.containsKey(target))
		{
			pobjs.put(target, new Vector<TokenCube>());
		}
		
		int i = pobjs.get(target).size();
		TokenCube c = new TokenCube();
		c.setTokenCohort(tokenCohort);
		c.place = target;
		
		c.setPosition(JoglVectord3.Add(source.getPosition(), new JoglVectord3(0, 5 * (i + 1),0)));

		//add token to new lane
		if (source.getParentNet() == target.getParentNet())
		{
			ITokenLane l = LaneCache.getEdgeLane(source, target);
			c.setTokenLane(l);
			l.AddToken(c);
		}
		else //should only happen with produce_start_token events
		{
			ITokenLane l = LaneCache.getStaticLane(target);
			c.setTokenLane(l);
			l.AddToken(c);
		}
		
		objs.add(c);
		//rpla.AddToken(c);
		pobjs.get(target).add(c);
		
		return c;
	}

	public TokenCube RemoveToken(JSOPnPlace source, JSOPnTransition target)
	{
		if (!pobjs.containsKey(source))
		{
			pobjs.put(source, new Vector<TokenCube>());
		}
		
		if (!pobjs.get(source).isEmpty())
		{
			int i = pobjs.get(source).size();
			TokenCube c = pobjs.get(source).lastElement();
			
			if (c != null)
			{
				pobjs.get(source).remove(c);
				//objs.remove(c);

				c.place = source;
				c.factor = 0.0;
				
				c.setPosition(JoglVectord3.Add(source.getPosition(), new JoglVectord3(0, 5 * i, 0)));

				//remove token from previous lane if any
				if (c.getTokenLane() != null)
				{
					c.getTokenLane().RemoveToken(c);
				}
				
				//add token to new lane
				if (source.getParentNet() == target.getParentNet())
				{
					ITokenLane l = LaneCache.getEdgeLane(source, target);
					c.setTokenLane(l);
					l.AddToken(c);
				}
				else //should only happen with consume_end_tokens events
				{
					ITokenLane l = LaneCache.getStaticLane(source);
					c.setTokenLane(l);
					l.AddToken(c);
				}
				
				return c;
			}
		}
		return null;
	}
	
	public void Tick(double time)
	{
		for (TokenCube jso : objs)
		{
			jso.Tick(time);
		}
	}
}
