package org.processmining.processprofiler3d.compviz.jogl.interactions.replay;

import java.util.HashMap;
import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnNode;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnPlace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;

public class ReplayaStateSim 
{
	private JSOPetriNet jpnet = null;
	private Vector<TokenInfo> objs = new Vector<TokenInfo>();
	private HashMap<JSOPnPlace, Vector<TokenInfo>> pobjs = new HashMap<JSOPnPlace, Vector<TokenInfo>>();

	private HashMap<JSOPnTransition, Vector<Vector<TokenInfo>>> activeTransitions = new HashMap<JSOPnTransition, Vector<Vector<TokenInfo>>>();

	private ReplaySimOptimizer rsopt = null;
	
	public ReplayaStateSim(JSOPetriNet pnet, Marking initMarking, ReplaySimOptimizer rso)
	{
		jpnet = pnet;
		rsopt = rso;
		
		InitializeState(pnet, initMarking);
	}

	public Vector<TokenInfo> getTokens()
	{
		return objs;
	}
	
	private void InitializeState(JSOPetriNet pnet, Marking initMarking) 
	{
		for (Place p : initMarking.toList())
		{
			//JSOPnPlace pl = pnet.getPlaceFor(p);
			JSOPnPlace pl = rsopt.getPlaceFor(p);
			
			if (!pobjs.containsKey(pl))
			{
				pobjs.put(pl, new Vector<TokenInfo>());
			}
			
			TokenInfo ti = new TokenInfo();
			ti.place = pl;
			
			ReplayaEvent re = new ReplayaEvent(ReplayaEvent.INITIAL_MARKING);
			ti.evts.add(re);
			Vector<JSOPnNode> nodes = new Vector<JSOPnNode>();
			nodes.add(pl);
			ti.nodes.add(nodes);
			
			objs.add(ti);
			pobjs.get(pl).add(ti);
		}
	}
	
	public void FinishReplaySim()
	{
		//set tokeninfo for final marking
		for (JSOPnPlace pl : pobjs.keySet())
		{
			Vector<TokenInfo> tokens = pobjs.get(pl);
			if (!tokens.isEmpty())
			{
				for (TokenInfo ti : tokens)
				{
					ReplayaEvent re = new ReplayaEvent(ReplayaEvent.FINAL_MARKING);
					ti.evts.add(re);
					Vector<JSOPnNode> nodes = new Vector<JSOPnNode>();
					nodes.add(pl);
					ti.nodes.add(nodes);
				}
			}
		}
	}
	
	public boolean CanTransitionEventFire(ReplayaEvent evt)
	{
		if (evt.type == ReplayaEvent.CONSUME_TOKENS)
		{
			INetDataSource net = evt.t.getParentNet().getPetriNet();
			boolean bReady = true;
			for (Arc a : net.getInEdges(evt.t.getOrgNode()))
			{
				JSOPnPlace px = jpnet.getPlaceFor(a.getSource());
				if (getTokenNum(px) < a.getWeight())
				{
					bReady = false;
				}
			}
			return bReady;
		}
		else if (evt.type == ReplayaEvent.PRODUCE_TOKENS)
		{
			if (getActivityInstances(evt.t) > 0)
			{
				if (evt.t.hasSubnet())
				{
//					INetDataSource net = evt.t.getParentNet().getPetriNet().getSubnetForTransition(evt.t.orgTransition);
//					JSOPnPlace px = jpnet.getPlaceFor(net.getEndPlace());
					JSOPnPlace px = rsopt.getEndPlace(rsopt.getSubnetForTransition(evt.t));
					if (getTokenNum(px) > 0)
					{
						return false;
					}			
				}
				return true;
			}
		}
		else if (evt.type == ReplayaEvent.CONSUME_ENDTOKENS)
		{
			//if (getActivityInstances(evt.t) > 0)
			{
//				INetDataSource net = evt.t.getParentNet().getPetriNet().getSubnetForTransition(evt.t.orgTransition);
//				JSOPnPlace px = jpnet.getPlaceFor(net.getEndPlace());
				JSOPnPlace px = rsopt.getEndPlace(rsopt.getSubnetForTransition(evt.t));
				if (getTokenNum(px) > 0)
				{
					return true;
				}
				return false;
			}
		}
		else if (evt.type == ReplayaEvent.PRODUCE_STARTTOKENS)
		{
			if (getActivityInstances(evt.t) > 0)
			{
				return true;
			}
		}
		return false;
	}
	
	public void consume(ReplayaEvent evt, JSOPnTransition t)
	{
		//consume parent tokens
		//INetDataSource net = t.getParentNet().getPetriNet();
		Vector<TokenInfo> tokens = new Vector<TokenInfo>();
		for (JSOPnPlace px : rsopt.getPreceedingPlaces(t))
//		for (Arc a : net.getInEdges(t.getOrgNode()))
		{
//			JSOPnPlace px = jpnet.getPlaceFor(a.getSource());
//			for (int i = 0; i < a.getWeight(); i++)
//			{
				TokenInfo tc = RemoveToken(px, t);
				tc.evts.add(evt);
				//tc.timeScale = evt.timeScale();
				tokens.add(tc);
				
				Vector<JSOPnNode> nodes = new Vector<JSOPnNode>();
				nodes.add(px);
				nodes.add(t);
				tc.nodes.add(nodes);
//			}
		}
		
		if (!activeTransitions.containsKey(t))
		{
			activeTransitions.put(t, new Vector<Vector<TokenInfo>>());
		}
		activeTransitions.get(t).add(tokens);
	}
	
	public void produce(ReplayaEvent evt, JSOPnTransition t)
	{
		if (getActivityInstances(t) > 0)
		{
			Vector<TokenInfo> removedTokens = activeTransitions.get(t).get(0);
			for (TokenInfo ti : removedTokens)
			{
				ti.evts.add(evt);
				//objs.remove(jso);
				//pobjs.get(jso.place).remove(jso);
			}
			activeTransitions.get(t).remove(0);
			
			//create child tokens
			INetDataSource net = t.getParentNet().getPetriNet();
			//for (JSOPnPlace px : rsopt.getPreceedingPlaces(t))
			for (Arc a : net.getOutEdges(t.getOrgNode()))
			{
				//JSOPnPlace px = jpnet.getPlaceFor(a.getTarget());
				JSOPnPlace px = rsopt.getPlaceFor(a.getTarget());
				for (int i = 0; i < a.getWeight(); i++)
				{
					TokenInfo ti = AddToken(px, t);
					ti.evts.add(evt);
					
					Vector<JSOPnNode> nodes = new Vector<JSOPnNode>();
					nodes.add(t);
					nodes.add(px);
					ti.nodes.add(nodes);
					//tc.timeScale = evt.timeScale();
				}
				
				for (TokenInfo ti : removedTokens)
				{
					Vector<JSOPnNode> nodes = new Vector<JSOPnNode>();
					nodes.add(t);
					nodes.add(px);
					ti.nodes.add(nodes);
					//objs.remove(jso);
					//pobjs.get(jso.place).remove(jso);
				}
			}
		}
	}

	public void consumeEndToken(ReplayaEvent evt, JSOPnTransition t) 
	{
		//if (getActivityInstances(t) > 0)
		{
//			INetDataSource net = t.getParentNet().getPetriNet();
//			Place sp = net.getSubnetForTransition(t.orgTransition).getEndPlace();
//			JSOPnPlace px = jpnet.getPlaceFor(sp);
			JSOPnPlace px = rsopt.getEndPlace(rsopt.getSubnetForTransition(t));
			TokenInfo tc = RemoveToken(px, t);
			tc.evts.add(evt);
			Vector<JSOPnNode> nodes = new Vector<JSOPnNode>();
			nodes.add(px);
			nodes.add(px);
			tc.nodes.add(nodes);

			//objs.remove(tc);
			//pobjs.get(tc.place).remove(tc);
		}
	}

	public void produceStartToken(ReplayaEvent evt, JSOPnTransition t) 
	{
		if (getActivityInstances(t) > 0)
		{
//			INetDataSource net = t.getParentNet().getPetriNet();
//			Place sp = net.getSubnetForTransition(t.orgTransition).getStartPlace();
//			JSOPnPlace px = jpnet.getPlaceFor(sp);
			JSOPnPlace px = rsopt.getStartPlace(rsopt.getSubnetForTransition(t));
			TokenInfo tc = AddToken(px, t);
			tc.evts.add(evt);
			Vector<JSOPnNode> nodes = new Vector<JSOPnNode>();
			nodes.add(px);
			nodes.add(px);
			tc.nodes.add(nodes);
		}
	}
	
	public int getActivityInstances(JSOPnTransition t)
	{
		if (activeTransitions.containsKey(t))
		{
			return activeTransitions.get(t).size();
		}
		return 0;
	}
	
	public void ClearTokens()
	{
		objs.clear();
	}
	
	public int getTokenNum(JSOPnPlace source)
	{
		if (!pobjs.containsKey(source))
		{
			pobjs.put(source, new Vector<TokenInfo>());
		}
		
		return pobjs.get(source).size();
	}

	public TokenInfo RemoveToken(JSOPnPlace source, JSOPnTransition target)
	{
		if (!pobjs.containsKey(source))
		{
			pobjs.put(source, new Vector<TokenInfo>());
		}
		
		if (!pobjs.get(source).isEmpty())
		{
			TokenInfo ti = pobjs.get(source).lastElement();
			
			if (ti != null)
			{
				pobjs.get(source).remove(ti);
				//objs.remove(c);

				ti.place = source;
				//c.factor = 0.0;
				
				return ti;
			}
		}
		return null;
	}
	
	public TokenInfo AddToken(JSOPnPlace target, JSOPnTransition source)
	{
		if (!pobjs.containsKey(target))
		{
			pobjs.put(target, new Vector<TokenInfo>());
		}
		
		TokenInfo c = new TokenInfo();
		c.place = target;
		
		objs.add(c);
		pobjs.get(target).add(c);
		
		return c;
	}
	
	public void Tick(double deltaTime)
	{
	}
}
