package org.processmining.processprofiler3d.compviz.jogl.interactions.replay;

import java.util.List;
import java.util.Vector;

import org.deckfour.xes.model.XTrace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdge;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnNode;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.TokenCube;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class TokenLaneEdge implements ITokenLane
{
	private JSOPnNode source;
	private JSOPnNode target;
	private JSOPnEdge edge;
	private Vector<JoglVectord3> posArray = new Vector<JoglVectord3>();
	private Vector<JoglVectord3> dirArray = new Vector<JoglVectord3>();
	private Vector<Double> lenArray = new Vector<Double>();
	
	private Vector<TokenCube> tokens = new Vector<TokenCube>();
	
	public TokenLaneEdge(JSOPnNode start, JSOPnNode end)
	{
		source = start;
		target = end;
		
		if (start.getParentNet() == end.getParentNet())
		{
			for (JSOPnEdge e : start.getParentNet().getEdges())
			{
				if (e.startObj == start && e.endObj == end)
				{
					edge = e;
				}
			}
		}
		
		if (edge == null)
		{
			//throw new Exception();
			System.out.println("No edge found for TokenLaneEdge!");
		}
		
		update();
	}
	
	@Override
	public void update()
	{
		List<JoglVectord3> lst = edge.getPoints();
		
		posArray.add(source.getPosition());
		for (int i = 0; i < lst.size(); i++)
		{
			JoglVectord3 sourcePos = lst.get(i);
			posArray.add(sourcePos);
		}
		posArray.add(target.getPosition());
		
		for (int i = 0; i < posArray.size()-1; i++)
		{
			JoglVectord3 dir = JoglVectord3.Subtract(posArray.get(i + 1), posArray.get(i));
			dirArray.add(dir);
		}
		
		Vector<Double> lenArray2 = new Vector<Double>();
		double totalLen = 0;
		for (int i = 0; i < dirArray.size(); i++)
		{
			JoglVectord3 dir = dirArray.get(i);
			double len = JoglVectord3.Distance(new JoglVectord3(0,0,0), dir);
			lenArray2.add(len);
			totalLen = totalLen + len;
		}
		
		for (int i = 0; i < lenArray2.size(); i++)
		{
			lenArray.add(lenArray2.get(i) / totalLen);
		}
	}
	
	public JoglVectord3 getPosition(double factor)
	{
		int i = 1;
		JoglVectord3 tmpPos = null;
		if (factor >= 1.0)
		{
			tmpPos = JoglVectord3.Add(posArray.lastElement(), new JoglVectord3(0, 5 * i, 0));
		}
		else
		{
			double tl = 0;
			for (int ix = 0; ix < lenArray.size(); ix++)
			{
				double len = lenArray.get(ix);
				if (tl <= factor && tl + len > factor)
				{
					JoglVectord3 sourcePosX = JoglVectord3.Add(posArray.get(ix), new JoglVectord3(0, 5 * i, 0));
					tmpPos = JoglVectord3.Add(sourcePosX, JoglVectord3.Multiply(dirArray.get(ix), (factor - tl) / len));
					break;
				}
				tl = tl + len;
			}
			
			if (tmpPos == null)
			{
				tmpPos = posArray.lastElement();
			}
		}
		return tmpPos;
	}
	
	@Override
	public JoglVectord3 getOffset()
	{
		return source.getParentNet().getOffset();
	}

	@Override
	public double getScale()
	{
		return source.getParentNet().getScale();
	}
	
	@Override
	public JSOSubNet getSubnet()
	{
		return source.getParentNet();
	}
	
	public Vector<TokenParticle> generateTokenInfos(XTrace trc, double startTime, double endTime, JSOPnNode startNode, JSOPnNode endNode, int cohortId)
	{
		Vector<TokenParticle> tps = new Vector<TokenParticle>();
		
		double startTimeAcc = startTime;
		double totalTime = endTime - startTime;
		
		for (int i = 0; i < dirArray.size(); i++)
		{
			TokenParticle tp = new TokenParticle();
			tp.startPos = posArray.get(i);
			tp.dir = dirArray.get(i);
			tp.startTime = startTimeAcc;
			tp.setLifetime(lenArray.get(i) * totalTime);
			tp.trace = trc;
			tp.cohort = cohortId;
			tp.subnet = getSubnet();
			
			tps.add(tp);
			
			startTimeAcc = startTimeAcc + tp.getLifetime();
		}
		return tps;
	}
	
	public void AddToken(TokenCube tc)
	{
		tokens.add(tc);
	}
	
	public void RemoveToken(TokenCube tc)
	{
		tokens.remove(tc);
	}
	
	public Vector<TokenCube> getTokens()
	{
		return tokens;
	}
}
