package org.processmining.processprofiler3d.compviz.jogl.interactions.replay;

import java.util.Vector;

import org.deckfour.xes.model.XTrace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnNode;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnPlace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.TokenCube;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class TokenLaneStatic implements ITokenLane 
{
	private JSOPnNode source;
	private JoglVectord3 sourcePos;

	private Vector<TokenCube> tokens = new Vector<TokenCube>();
	
	public TokenLaneStatic(JSOPnNode start)
	{
		source = start;
		
		update();
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.plugins.compviz.jogl.interactions.replay.ITokenLane#update()
	 */
	@Override
	public void update()
	{
		sourcePos = source.getPosition();
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.plugins.compviz.jogl.interactions.replay.ITokenLane#getPosition(double)
	 */
	@Override
	public JoglVectord3 getPosition(double factor)
	{
		int i = 1;
		return JoglVectord3.Add(sourcePos, new JoglVectord3(0, 5 * i, 0));
	}
	
	@Override
	public JoglVectord3 getOffset()
	{
		return source.getParentNet().getOffset();
	}

	@Override
	public double getScale()
	{
		return source.getParentNet().getScale();
	}
	
	@Override
	public JSOSubNet getSubnet()
	{
		return source.getParentNet();
	}
	
	public Vector<TokenParticle> generateTokenInfos(XTrace trc, double startTime, double endTime, JSOPnNode startNode, JSOPnNode endNode, int cohortId)
	{
		TokenParticle tp = new TokenParticle();
		if (source instanceof JSOPnPlace)
		{
			double offsetDist = 4.0;
			if (cohortId == 0)
			{
				tp.startPos = JoglVectord3.Add(sourcePos, new JoglVectord3(offsetDist,0,offsetDist));
			}
			else if (cohortId == 1)
			{
				tp.startPos = JoglVectord3.Add(sourcePos, new JoglVectord3(-offsetDist,0,offsetDist));			
			}
			else if (cohortId == 2)
			{
				tp.startPos = JoglVectord3.Add(sourcePos, new JoglVectord3(offsetDist,0,-offsetDist));
			}
			else if (cohortId == 3)
			{
				tp.startPos = JoglVectord3.Add(sourcePos, new JoglVectord3(-offsetDist,0,-offsetDist));
			}
		}
		else
		{
			tp.startPos = sourcePos;
		}
		tp.dir = new JoglVectord3(0,0,0);
		tp.startTime = startTime;
		tp.setLifetime(endTime - startTime);
		tp.trace = trc;
		tp.cohort = cohortId;
		tp.subnet = getSubnet();
		
		Vector<TokenParticle> tps = new Vector<TokenParticle>();
		tps.add(tp);
		
		return tps;
	}
	
	public void AddToken(TokenCube tc)
	{
		tokens.add(tc);
	}
	
	public void RemoveToken(TokenCube tc)
	{
		tokens.remove(tc);
	}
	
	public Vector<TokenCube> getTokens()
	{
		return tokens;
	}
}
