package org.processmining.processprofiler3d.compviz.jogl.interactions.replay;

import java.util.Date;
import java.util.Vector;

import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnNode;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOReplaya;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayAnalyser;

public class TraceReplay 
{
	private JSOReplaya rpla = null;
	private JSOPetriNet jpnet = null;
	private Marking initMarking = null;
	private TraceReplayAnalyser lra;
	private XTrace trace = null;
	private int stepNum = 0;
	private ReplayaState replx = null;
	private Vector<TokenInfo> tinfos;
	
	private Date startTime;
	private Date endTime;
	private TimeSpan length;
	private double stepTime = 0;
	private double totalStepLength = 0;
	
	private Date replayTime;
	private Date lastReplayTime;
	
	private double startOffset = 0;
	
	private Vector<TokenParticle> prts = null;
	
	public XTrace getTrace()
	{
		return trace;
	}
	
	public TraceReplay(JSOReplaya rpla, JSOPetriNet jn, Marking initMarking, TraceReplayAnalyser lra)
	{
		this.rpla = rpla;
		jpnet = jn;
		this.initMarking = initMarking;
		this.lra = lra;
		
	}
	
	public boolean Simulate(ReplaySimOptimizer rso)
	{
		//simulate replay
		ReplaySimulator rs = new ReplaySimulator(jpnet, initMarking, lra, rso);
		if (!rs.Simulate())
		{
			return false;
		}
		tinfos = rs.getTokens();
		rEvts = rs.getExecutionOrder();
		
		trace = lra.getTrace();
		
		//InitReplaya();
		
		startTime = getEventTimestamp(trace.get(0));
		endTime = getEventTimestamp(trace.get(trace.size()-1));
		
		length = new TimeSpan(startTime, endTime);
		replayTime = startTime;
		lastReplayTime = startTime;
		
		stepNum = 0;
		stepTime = 0;
		totalStepLength = (endTime.getTime() - startTime.getTime()) / 100000.0;
		startOffset = startTime.getTime() / 100000.0;
		
		return true;
	}

	public void InitReplaya(int tokenCohort) 
	{
		//redundant! this is used for process steps instead of absolute time!
		//double stepLength = 0;
		for (ReplayaEvent re : rEvts)
		{
			re.setMaxFactor(trace);
			if (re.type == ReplayaEvent.CONSUME_TOKENS || re.type == ReplayaEvent.PRODUCE_TOKENS)
			{
				//stepLength = stepLength + re.timeScale();
			}
		}
		//totalStepLength = stepLength;
		
		prts = ProcessTokenInfos(tinfos, trace, tokenCohort);
		
		//initialize state
		replx = new ReplayaState(rpla, jpnet, initMarking, tokenCohort);
		
//		startTime = getEventTimestamp(trace.get(0));
//		endTime = getEventTimestamp(trace.get(trace.size()-1));
//		
//		length = new TimeSpan(startTime, endTime);
//		replayTime = startTime;
//		lastReplayTime = startTime;
//		
//		stepNum = 0;
//		stepTime = 0;
//		totalStepLength = (endTime.getTime() - startTime.getTime()) / 100000.0;
//		startOffset = startTime.getTime() / 100000.0;
		
		rpla.AddParticles(prts);
	}
	
	public double getTotalLength()
	{	
		return totalStepLength;
	}
	
	public static Vector<TokenParticle> ProcessTokenInfos(Vector<TokenInfo> tinfos, XTrace trace, int tokenCohort)
	{
		Vector<TokenParticle> parts = new Vector<TokenParticle>();
		for (TokenInfo ti : tinfos)
		{
			for (int i = 0; i < ti.evts.size() - 1; i++)
			{
				ReplayaEvent evt1 = ti.evts.get(i);
				ReplayaEvent evt2 = ti.evts.get(i+1);

				if (evt1.type == ReplayaEvent.INITIAL_MARKING && evt2.type == ReplayaEvent.CONSUME_TOKENS)
				{
					double startTime = 0;
					double lifeTime = evt2.startTime - startTime;
					
					JSOPnNode startNode = ti.nodes.get(i).get(0);
					
					parts.addAll(LaneCache.getStaticLane(startNode).generateTokenInfos(trace, startTime, startTime + lifeTime, startNode, null, tokenCohort));
				}
				else if (evt1.type == ReplayaEvent.CONSUME_TOKENS && evt2.type == ReplayaEvent.PRODUCE_TOKENS)
				{
					double startTime = evt1.startTime;
					double lifeTime = evt1.endTime - startTime;
					
					JSOPnNode startNode = ti.nodes.get(i).get(0);
					JSOPnNode endNode = ti.nodes.get(i).get(1);
					
					parts.addAll(LaneCache.getEdgeLane(startNode, endNode).generateTokenInfos(trace, startTime, startTime + lifeTime, startNode, endNode, tokenCohort));
				}
				else if (evt1.type == ReplayaEvent.PRODUCE_STARTTOKENS && evt2.type == ReplayaEvent.CONSUME_TOKENS)
				{
					double startTime = evt1.startTime;
					double lifeTime = evt2.startTime - startTime;
					
					JSOPnNode startNode = ti.nodes.get(i).get(0);
					parts.addAll(LaneCache.getStaticLane(startNode).generateTokenInfos(trace, startTime, startTime + lifeTime, startNode, null, tokenCohort));
				}
				else if (evt1.type == ReplayaEvent.PRODUCE_TOKENS && evt2.type == ReplayaEvent.CONSUME_ENDTOKENS)
				{
					//token moving to place
					if (true)
					{
						double startTime = evt1.startTime;
						double lifeTime = evt1.endTime - startTime;

						if (lifeTime > 0)
						{
							JSOPnNode startNode = ti.nodes.get(i).get(0);
							JSOPnNode endNode = ti.nodes.get(i).get(1);
							parts.addAll(LaneCache.getEdgeLane(startNode, endNode).generateTokenInfos(trace, startTime, startTime + lifeTime, startNode, endNode, tokenCohort));
						}
					}
					
					if (true)
					{
						double startTime = evt1.endTime;
						double lifeTime = evt2.endTime - startTime;
						
						if (lifeTime > 0)
						{
							JSOPnNode startNode = ti.nodes.get(i).get(1);
	
							parts.addAll(LaneCache.getStaticLane(startNode).generateTokenInfos(trace, startTime, startTime + lifeTime, startNode, null, tokenCohort));
						}
					}
				}
				else if (evt1.type == ReplayaEvent.PRODUCE_TOKENS && evt2.type == ReplayaEvent.FINAL_MARKING)
				{
					//token moving to place
					if (true)
					{
						double startTime = evt1.startTime;
						double lifeTime = evt1.endTime - startTime;

						if (lifeTime > 0)
						{
							JSOPnNode startNode = ti.nodes.get(i).get(0);
							JSOPnNode endNode = ti.nodes.get(i).get(1);
							parts.addAll(LaneCache.getEdgeLane(startNode, endNode).generateTokenInfos(trace, startTime, startTime + lifeTime, startNode, endNode, tokenCohort));
						}
					}
					
					//token staying at place
					if (true)
					{
						double startTime = evt1.endTime;
						double lifeTime = Double.POSITIVE_INFINITY;
						
						if (lifeTime > 0)
						{
							JSOPnNode startNode = ti.nodes.get(i).get(1);
	
							parts.addAll(LaneCache.getStaticLane(startNode).generateTokenInfos(trace, startTime, startTime + lifeTime, startNode, null, tokenCohort));
						}
					}
				}
				else if (evt1.type == ReplayaEvent.PRODUCE_TOKENS && evt2.type == ReplayaEvent.CONSUME_TOKENS)
				{
					//token moving to place
					if (true)
					{
						double startTime = evt1.startTime;
						double lifeTime = evt1.endTime - startTime;

						if (lifeTime > 0)
						{
							JSOPnNode startNode = ti.nodes.get(i).get(0);
							JSOPnNode endNode = ti.nodes.get(i).get(1);
							parts.addAll(LaneCache.getEdgeLane(startNode, endNode).generateTokenInfos(trace, startTime, startTime + lifeTime, startNode, endNode, tokenCohort));
						}
					}
					
					//token waiting
					if (true)
					{
						double startTime = evt1.endTime;
						double lifeTime = evt2.startTime - startTime;

						if (lifeTime > 0)
						{
							JSOPnNode startNode = ti.nodes.get(i).get(1);
							parts.addAll(LaneCache.getStaticLane(startNode).generateTokenInfos(trace, startTime, startTime + lifeTime, startNode, null, tokenCohort));
						}
					}
				}
			}
		}
		return parts;
	}
	
	public void Tick(double deltaTime)
	{
		stepTime = stepTime + deltaTime;
		double currentTime = (stepTime + startOffset);
//		Vector<ReplayaEvent> remove = new Vector<ReplayaEvent>();
//		for (ReplayaEvent re : rEvts)
//		{
//			if (re.isStarted(currentTime))
//			{
//				activeEvents.add(re);
//				ExecuteEvt(re);
//				remove.add(re);
//			}
//		}
//		
//		for (ReplayaEvent re : remove)
//		{
//			rEvts.remove(re);
//		}
//		
//		remove.clear();
//		for (ReplayaEvent re : activeEvents)
//		{
//			re.advance(currentTime);
//			if (re.isComplete(currentTime))
//			{
//				//remove
//				remove.add(re);
//			}
//		}
//		
//		for (ReplayaEvent re : remove)
//		{
//			activeEvents.remove(re);
//		}
//		
//		replx.Tick(currentTime);
		
		for (TokenParticle tpx : prts)
		{
			if (tpx.getInvLifetime() == 0)
			{
				if (currentTime >= tpx.startTime)
				{
					tpx.factor = 0.5;
				}
				else
				{
					tpx.factor = -1.0;
				}	
			}
			else
			{
				tpx.factor = (currentTime - tpx.startTime) * tpx.getInvLifetime();
			}
		}
	}
	
	//private Vector<ReplayaEvent> activeEvents = new Vector<ReplayaEvent>();
	
	public double getProgress()
	{
		double sTime = Math.max(0.0, stepTime);
		return (sTime / totalStepLength);
	}
	
	private Vector<ReplayaEvent> rEvts = new Vector<ReplayaEvent>();
	//private Vector<JSOPnTransition> lastTrans = new Vector<JSOPnTransition>();
	
//	private void ClearTransitionHighlights() 
//	{
//		for (JSOPnTransition jso : lastTrans)
//		{
//			jso.setColor(JoglColor3.BLACK);
//		}
//		lastTrans.clear();
//	}
	
//	private void ExecuteEvt(ReplayaEvent evt)
//	{
//		if (evt.type == ReplayaEvent.CONSUME_TOKENS)
//		{
//			replx.consume(evt.startTime, evt.timeScale(), evt.t);
//			//evt.t.setColor(JoglColor3.RED);
//			lastTrans.add(evt.t);
//		}
//		else if (evt.type == ReplayaEvent.PRODUCE_TOKENS)
//		{
//			replx.produce(evt.startTime, evt.timeScale(), evt.t);
//			//evt.t.setColor(JoglColor3.GREEN);
//			lastTrans.add(evt.t);
//		}
//		else if (evt.type == ReplayaEvent.CONSUME_ENDTOKENS)
//		{
//			replx.consumeEndToken(evt.t);
//		}
//		else if (evt.type == ReplayaEvent.PRODUCE_STARTTOKENS)
//		{
//			replx.produceStartToken(evt.t);
//		}
//	}
	
	private Date getEventTimestamp(XEvent evt)
	{
		return getAttributeValueTimestamp(evt, XTimeExtension.KEY_TIMESTAMP);
	}
	
	private static Date getAttributeValueTimestamp(XEvent evt, String name)
	{
		XAttributeTimestamp xAtt3 = (XAttributeTimestamp)evt.getAttributes().get(name);
		if (xAtt3 != null && xAtt3.getValue() != null)
		{
			Date eventName = xAtt3.getValue();
			return eventName;
		}
		return null;
	}

}
