package org.processmining.processprofiler3d.compviz.jogl.materials;

import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderPipelineDefaultTextured;
import org.processmining.visualisation3d.FontData;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.gldatastructures.JoglTexSampler;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.materials.JoglMaterialGL3;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;

import com.jogamp.opengl.GL2;

public class MaterialCamWidgetCubeText extends JoglMaterialGL3
{
	private BlendMode oldDrawMode = BlendMode.Opaque;
	
	private JoglShader shad = null;
	private JoglTexture boundTex = null;
	private JoglTexSampler texsampler = null;
	
	public MaterialCamWidgetCubeText()
	{
	}

	@Override
	public void Apply(GraphicsPipeline pipe)
	{
		super.Apply(pipe);

		FontData font = (FontData)JoglResourceManager.GetResource("DefaultFont");
		
		//on closing of window font might have been destroyed before final draw, so check and exit early
		if (font == null)
			return;

		boundTex = font.getTexture(pipe);
		boundTex.Bind(pipe);
		
		if (gl3 != null)
		{
			if (texsampler == null)
			{
				texsampler = new JoglTexSampler(pipe);
				texsampler.setMinFilter(gl3, GL2.GL_LINEAR_MIPMAP_LINEAR);
				texsampler.setMagFilter(gl3, GL2.GL_LINEAR);
			}
			if (texsampler != null)
			{
				texsampler.setMaxAnisotropy(gl3, pipe.getGraphicsSettings().GetAnisotropicSamples());
				texsampler.Bind(gl3, 0);
			}
		}
		
		
		if (shad == null)
		{
			InitShaders(pipe);
		}
		
		oldDrawMode = currentBlendMode;
		setDrawMode(BlendMode.AlphaBlended);
		setColor(0, 0, 0, 1);
		
		if (shad != null)
		{
			shad.Bind(pipe);
		}
	}

	@Override
	public void Unapply(GraphicsPipeline pipe)
	{
		super.Unapply(pipe);
		
		if (shad != null)
		{
			shad.Unbind(pipe);
		}
		
		setDrawMode(oldDrawMode);

		boundTex.Unbind(pipe);
		if (gl3 != null)
		{
			if (texsampler != null)
			{
				texsampler.Unbind(gl3, 0);
			}
		}
	}
	
	private void InitShaders(GraphicsPipeline pipe)
	{
		if (JoglGraphicsSettings.SupportsShaderObjects(pipe))
		{
			if (JoglResourceManager.HasResource("PipelineDefaultTexturedShader"))
			{
				shad = (JoglShader)JoglResourceManager.GetResource("PipelineDefaultTexturedShader");
			}
			else
			{
				shad = new JoglShaderPipelineDefaultTextured();
				shad.Create(pipe);
				
				JoglResourceManager.AddResource("PipelineDefaultTexturedShader", shad);
			}
		}
	}
}
