package org.processmining.processprofiler3d.compviz.jogl.materials;

import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderGenerateEpirect;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.materials.JoglMaterial;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.util.texture.Texture;

public class MaterialEpirectConversion extends JoglMaterial
{
	private boolean oldDepthTest = true;
	private MaskState oldMaskState = MaskState.DrawBoth;
	
	private JoglShader shader;
	private Texture tex;

	public MaterialEpirectConversion()
	{
	}
	
	public void SetTexture(Texture t)
	{
		tex = t;
	}
	
	@Override
	public void Apply(GraphicsPipeline pipe)
	{
		super.Apply(pipe);
		
		if (shader == null)
		{
			InitShaders(pipe);
		}
		
		if (shader != null)
		{
			shader.Bind(pipe);
		}
		if (tex != null)
		{
			tex.bind(gl);
		}
		
		oldDepthTest = bDepthTest;
		oldMaskState = currentMaskState;
		
		setDepthTest(false);
		setMaskState(MaskState.DrawColorOnly);
	}

	@Override
	public void Unapply(GraphicsPipeline pipe)
	{
		super.Unapply(pipe);

		if (shader != null)
		{
			shader.Unbind(pipe);
		}
		if (tex != null)
		{
			gl.glBindTexture(GL.GL_TEXTURE_2D, 0);
		}
		
		setMaskState(oldMaskState);
		setDepthTest(oldDepthTest);
	}
	
	public void InitShaders(GraphicsPipeline pipe)
	{
		if (JoglGraphicsSettings.SupportsShaderObjects(pipe))
		{
			shader = new JoglShaderGenerateEpirect();
			shader.Create(pipe);
		}
	}

}
