package org.processmining.processprofiler3d.compviz.jogl.materials;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;

import com.jogamp.opengl.GL;

public class MaterialGridMasked extends MaterialGridBase
{
	private boolean oldDepthTest = true;
	private MaskState oldMaskState = MaskState.DrawBoth;
	
	private JoglShader shad = null;
	private boolean bSwitchedOffDepth = false;
	private JoglTexture boundTex = null;
	
	public MaterialGridMasked()
	{
	}

	@Override
	public void Apply(GraphicsPipeline pipe)
	{
		super.Apply(pipe);
		
		InitShaders(pipe);

		JoglTexture tbo = null;
		if (JoglResourceManager.HasResource("PetriNetScreenMask"))
		{
			tbo = (JoglTexture)JoglResourceManager.GetResource("PetriNetScreenMask");
		}
		if (tbo != null)
		{
			tbo.Bind(pipe);
			boundTex = tbo;
		}
		
		if (shad != null)
		{
			shad.Bind(pipe);
		}
	}

	@Override
	public void Unapply(GraphicsPipeline pipe)
	{
		super.Unapply(pipe);

		if (boundTex != null)
		{
			gl.glBindTexture(GL.GL_TEXTURE_2D, 0);
		}
		
		if (shad != null)
		{
			shad.Unbind(pipe);
		}

		if (bSwitchedOffDepth)
		{
			setDepthTest(oldDepthTest);
			setMaskState(oldMaskState);
			bSwitchedOffDepth = false;
		}
	}
	
	@Override
	public void setDepthState(boolean bDepthOff)
	{
		if (bDepthOff && !bSwitchedOffDepth)
		{
			oldDepthTest = bDepthTest;
			oldMaskState = currentMaskState;
			
			setDepthTest(false);
			setMaskState(MaskState.DrawColorOnly);
			bSwitchedOffDepth = true;
		}
	}

	@Override
	public void DrawColor(JoglVectord4 gridColor)
	{
		setColor(gridColor);		
	}

	private void InitShaders(GraphicsPipeline pipe) 
	{
		if (JoglGraphicsSettings.SupportsShaderObjects(pipe))
		{
			if (shad == null)
			{
				if (JoglResourceManager.HasResource("GridShaderMasked"))
				{
					shad = (JoglShader)JoglResourceManager.GetResource("GridShaderMasked");
				}
				else
				{
					shad = new JoglShader("/org/processmining/processprofiler3d/compviz/jogl/resources", "gridShaderVS", "gridShaderMaskedFS");
					shad.Create(pipe);
					
					JoglResourceManager.AddResource("GridShaderMasked", shad);
				}
			}
		}
	}
}
