package org.processmining.processprofiler3d.compviz.jogl.materials;

import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderGridOrtho;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLensOrthographic;
import org.processmining.visualisation3d.materials.JoglMaterial;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;

public class MaterialGridOrtho extends JoglMaterial
{
	private boolean oldDepthTest = true;
	private MaskState oldMaskState = MaskState.DrawBoth;
	
	private JoglShaderGridOrtho shadOrtho = null;
	private boolean bSwitchedOffDepth = false;
	
	public MaterialGridOrtho()
	{
	}

	@Override
	public void Apply(GraphicsPipeline pipe)
	{
		super.Apply(pipe);
		
		InitShaders(pipe);
		
		if (shadOrtho != null)
		{
			shadOrtho.Bind(pipe);
//			shadOrtho.applyCameraLens(pipe, jclo);
//			shadOrtho.applyCamera(pipe, c);
		}
	}

	@Override
	public void Unapply(GraphicsPipeline pipe)
	{
		super.Unapply(pipe);

		if (shadOrtho != null)
		{
			shadOrtho.Unbind(pipe);
		}

		if (bSwitchedOffDepth)
		{
			setDepthTest(oldDepthTest);
			setMaskState(oldMaskState);
			bSwitchedOffDepth = false;
		}
	}
	
	public void setDepthState(boolean bDepthOff)
	{
		if (bDepthOff && !bSwitchedOffDepth)
		{
			oldDepthTest = bDepthTest;
			oldMaskState = currentMaskState;
			
			setDepthTest(false);
			setMaskState(MaskState.DrawColorOnly);
			bSwitchedOffDepth = true;
		}
	}
	
	public void DrawColor(JoglVectord4 gridColor)
	{
		setColor(gridColor);		
	}
	
	public void ApplyCamera(JoglCamera c)
	{
		if (c.getCameraLens() instanceof JoglCameraLensOrthographic)
		{
			JoglCameraLensOrthographic jclo = (JoglCameraLensOrthographic)c.getCameraLens();
			if (shadOrtho != null)
			{
				//shadOrtho.Bind(pipe);
				shadOrtho.applyCameraLens(pipe, jclo);
				shadOrtho.applyCamera(pipe, c);
			}
		}	
	}

	private void InitShaders(GraphicsPipeline pipe) 
	{
		if (JoglGraphicsSettings.SupportsShaderObjects(pipe))
		{		
			if (shadOrtho == null)
			{
				if (JoglResourceManager.HasResource("GridShaderOrtho"))
				{
					shadOrtho = (JoglShaderGridOrtho)JoglResourceManager.GetResource("GridShaderOrtho");
				}
				else
				{
					shadOrtho = new JoglShaderGridOrtho();
					shadOrtho.Create(pipe);
					
					JoglResourceManager.AddResource("GridShaderOrtho", shadOrtho);
				}
			}
		}
	}
}
