package org.processmining.processprofiler3d.compviz.jogl.materials;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.materials.JoglMaterial;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;

public class MaterialGuiMesh extends JoglMaterial
{
	private JoglShader shad;
	private JoglShader boundShad;
	private BlendMode oldBlendMode = BlendMode.Opaque;
	
	public MaterialGuiMesh()
	{
	}

	@Override
	public void Apply(GraphicsPipeline pipe)
	{
		super.Apply(pipe);

		if (shad == null)
		{
			InitShader(pipe);
		}
		
		if (shad != null)
		{
			shad.Bind(pipe);
			boundShad = shad;
		}
		
		oldBlendMode = currentBlendMode;
		setDrawMode(BlendMode.AlphaBlended);
	}

	@Override
	public void Unapply(GraphicsPipeline pipe)
	{
		super.Unapply(pipe);
		
		setDrawMode(oldBlendMode);
		
		if (boundShad != null)
		{
			boundShad.Unbind(pipe);
			boundShad = null;
		}
	}

	private void InitShader(GraphicsPipeline pipe)
	{
		if (JoglGraphicsSettings.SupportsShaderObjects(pipe))
		{
			if (JoglResourceManager.HasResource("GenericVAOShader"))
			{
				shad = (JoglShader)JoglResourceManager.GetResource("GenericVAOShader");
			}
			else
			{
				shad = new JoglShader("/org/processmining/processprofiler3d/compviz/jogl/resources", "vaoShaderVS","vaoShaderFS");
				shad.Create(pipe);
				
				JoglResourceManager.AddResource("GenericVAOShader", shad);
			}
		}
	}
}
