package org.processmining.processprofiler3d.compviz.jogl.materials;

import java.util.Vector;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.materials.JoglMaterial;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;

public class MaterialMagnetLine extends JoglMaterial
{
	private JoglShader shad = null;
	private JoglShader shadMask = null;
	private JoglShader boundShader = null;

	private boolean bInitialized = false;
	
	public MaterialMagnetLine()
	{
	}

	@Override
	public void Apply(GraphicsPipeline pipe)
	{
		super.Apply(pipe);

		if (!bInitialized)
		{	
			InitShaders(pipe);
		}
	}

	@Override
	public void Unapply(GraphicsPipeline pipe)
	{
		super.Unapply(pipe);
		
		setColor(1, 1, 1);
		
		if (boundShader != null)
		{
			boundShader.Unbind(pipe);
		}
	}
	
	public void DrawColor(JoglVectord3 col)
	{
		setColor(col);
	}
	
	public void DrawMagnetLine(double length, double height, Vector<JoglVectord3> colors)
	{
		if (shad != null)
		{
			shad.Bind(pipe);
			boundShader = shad;
		}
		
		if (boundShader != null)
		{
			int colorNum = Math.min(4, colors.size()); //shader only supports up to 4 colors!
			
			boundShader.Bind(pipe);
			boundShader.SetUniform(pipe, "below", 1.0f);
			boundShader.SetUniform(pipe, "blendFactor", (float)Math.abs(height / 50.0));
			boundShader.SetUniform(pipe, "lineLength", (float)length);
			boundShader.SetUniform(pipe, "numCohorts", colorNum);
			
			for (int i = 0; i < colorNum; i++)
			{
				boundShader.SetUniform(pipe, "cohorts" + (i + 1), colors.get(i));
			}
		}
		
		//fallback if shader not working!
		setColor(colors.get(0));
	}
	
	public void DrawMagnetLineMasked(double length, double height, Vector<JoglVectord3> colors)
	{
		if (shad != null)
		{
			shad.Bind(pipe);
			boundShader = shad;
		}
		
		if (boundShader != null)
		{
			int colorNum = Math.min(4, colors.size()); //shader only supports up to 4 colors!
			
			boundShader.Bind(pipe);
			boundShader.SetUniform(pipe, "below", 0.65f);
			boundShader.SetUniform(pipe, "blendFactor", (float)Math.abs(height / 50.0));
			boundShader.SetUniform(pipe, "lineLength", (float)length);
			boundShader.SetUniform(pipe, "numCohorts", colorNum);
			
			for (int i = 0; i < colorNum; i++)
			{
				boundShader.SetUniform(pipe, "cohorts" + (i + 1), colors.get(i));
			}
		}
		
		//fallback if shader not working!
		setColor(colors.get(0));
	}
	
	private void InitShaders(GraphicsPipeline pipe) 
	{
		if (JoglGraphicsSettings.SupportsShaderObjects(pipe))
		{
			if (shad == null)
			{
				if (JoglResourceManager.HasResource("MagnetLineShader"))
				{
					shad = (JoglShader)JoglResourceManager.GetResource("MagnetLineShader");
				}
				else
				{
					shad = new JoglShader("/org/processmining/processprofiler3d/compviz/jogl/resources", "magnetLineShaderVS", "magnetLineShaderFS");
					shad.Create(pipe);
					
					JoglResourceManager.AddResource("MagnetLineShader", shad);
				}
			}
			if (shadMask == null)
			{
				if (JoglResourceManager.HasResource("MagnetLineShaderMasked"))
				{
					shadMask = (JoglShader)JoglResourceManager.GetResource("MagnetLineShaderMasked");
				}
				else
				{
					shadMask = new JoglShader("/org/processmining/processprofiler3d/compviz/jogl/resources", "magnetLineShaderVS", "magnetLineShaderMaskedFS");
					shadMask.Create(pipe);
					
					JoglResourceManager.AddResource("MagnetLineShaderMasked", shadMask);
				}
			}
		}
		
		bInitialized = true;
	}
}
