package org.processmining.processprofiler3d.compviz.jogl.materials;

import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderTextRender;
import org.processmining.visualisation3d.FontData;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.gldatastructures.JoglTexSampler;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.materials.JoglMaterialGL3;
import org.processmining.visualisation3d.maths.JoglColor3;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;

import com.jogamp.opengl.GL2;

public class MaterialPetrinetElementBatchLabels extends JoglMaterialGL3 implements IMaterialPetrinetElementBatchLabels
{
	private boolean oldDepthTest = true;
	private BlendMode oldDrawMode = BlendMode.Opaque;
	private MaskState oldMaskState = MaskState.DrawBoth;
	
	private JoglShader textShader;
	private JoglShader boundShader;
	private JoglTexture tex;
	private JoglTexSampler texsampler = null;
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.jogl.materials.IMaterialPetrinetElementBatchLabels#IsShaderBound()
	 */
	@Override
	public boolean IsShaderBound()
	{
		return (boundShader != null);
	}
	
	public MaterialPetrinetElementBatchLabels()
	{
	}

	@Override
	public void Apply(GraphicsPipeline pipe)
	{
		super.Apply(pipe);
		
		if (textShader == null)
		{
			InitShader(pipe);
		}

		FontData fontData = (FontData)JoglResourceManager.GetResource("DefaultFont");
		tex = fontData.getTexture(pipe);
		tex.Bind(pipe);
		
		if (gl3 != null)
		{
			if (texsampler == null)
			{
				texsampler = new JoglTexSampler(pipe);
				texsampler.setMinFilter(gl3, GL2.GL_LINEAR_MIPMAP_LINEAR);
				texsampler.setMagFilter(gl3, GL2.GL_LINEAR);
			}
			if (texsampler != null)
			{
				JoglGraphicsSettings gSettings = pipe.getGraphicsSettings();
				texsampler.setMaxAnisotropy(gl3, gSettings.GetAnisotropicSamples());
				texsampler.Bind(gl3, 0);
			}
		}
		
		oldDepthTest = bDepthTest;
		oldDrawMode = currentBlendMode;
		oldMaskState = currentMaskState;
		
		setDrawMode(BlendMode.AlphaBlended);
		setDepthTest(false);
		setMaskState(MaskState.DrawColorOnly);
		setColor(0, 0, 0, 1.0f);   // set the color of the quads

		if (textShader != null)
		{
			textShader.Bind(pipe);
			textShader.SetUniform(pipe, "textColor", JoglColor3.BLACK);
			boundShader = textShader;
		}
	}

	@Override
	public void Unapply(GraphicsPipeline pipe)
	{
		super.Unapply(pipe);
		
		if (boundShader != null)
		{
			boundShader.Unbind(pipe);
			boundShader = null;
		}

		tex.Unbind(pipe);
		if (gl3 != null)
		{
			if (texsampler != null)
			{
				texsampler.Unbind(gl3, 0);
			}
		}

		setMaskState(oldMaskState);
		setDepthTest(oldDepthTest);
		setDrawMode(oldDrawMode);
		setColor(1.0f, 1.0f, 1.0f, 1.0f);   // set the color of the quads
	}

	private void InitShader(GraphicsPipeline pipe)
	{
		if (JoglGraphicsSettings.SupportsShaderObjects(pipe))
		{
			if (JoglResourceManager.HasResource("TextShader"))
			{
				textShader = (JoglShader)JoglResourceManager.GetResource("TextShader");
			}
			else
			{
				textShader = new JoglShaderTextRender();
				textShader.Create(pipe);
				
				JoglResourceManager.AddResource("TextShader", textShader);
			}
		}
	}
}
