package org.processmining.processprofiler3d.compviz.jogl.materials;

import org.processmining.processprofiler3d.compviz.fileio.BMFontFile;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderTextRenderDF;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.materials.JoglMaterial;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;

public class MaterialPetrinetElementBatchLabelsDF extends JoglMaterial implements IMaterialPetrinetElementBatchLabels
{
	private boolean oldDepthTest = true;
	private BlendMode oldDrawMode = BlendMode.Opaque;
	private MaskState oldMaskState = MaskState.DrawBoth;
	
	private JoglTexture tex;
	private JoglShader textShader;
	private JoglShader boundShader;
	
	public boolean IsShaderBound()
	{
		return (boundShader != null);
	}
	
	public MaterialPetrinetElementBatchLabelsDF()
	{
	}

	@Override
	public void Apply(GraphicsPipeline pipe)
	{
		super.Apply(pipe);
		
		if (textShader == null)
		{
			InitShader(pipe);
		}

		if (tex == null)
		{
			InitTexture(pipe);
		}
		
		tex.Bind(pipe);

		oldDepthTest = bDepthTest;
		oldDrawMode = currentBlendMode;
		oldMaskState = currentMaskState;
		
		setDrawMode(BlendMode.AlphaBlended);
		setDepthTest(false);
		setMaskState(MaskState.DrawColorOnly);
		setColor(0, 0, 0, 1.0f);   // set the color of the quads

		if (textShader != null)
		{
			textShader.Bind(pipe);
			//textShader.SetUniform(gl, "textColor", JoglColor3.BLACK);
			boundShader = textShader;
		}
	}

	@Override
	public void Unapply(GraphicsPipeline pipe)
	{
		super.Unapply(pipe);
		
		if (boundShader != null)
		{
			boundShader.Unbind(pipe);
			boundShader = null;
		}
		
		tex.Unbind(pipe);

		setMaskState(oldMaskState);
		setDepthTest(oldDepthTest);
		setDrawMode(oldDrawMode);
		setColor(1.0f, 1.0f, 1.0f, 1.0f);   // set the color of the quads
	}

	private void InitTexture(GraphicsPipeline pipe)
	{
        if (BMFontFile.tex == null)
        {
    		BMFontFile bmf = new BMFontFile();
    		bmf.LoadFontTexture(pipe);
        }
        
		tex = BMFontFile.tex;	
	}
	
	private void InitShader(GraphicsPipeline pipe)
	{
		if (JoglGraphicsSettings.SupportsShaderObjects(pipe))
		{
			if (JoglResourceManager.HasResource("TextShaderDF"))
			{
				textShader = (JoglShader)JoglResourceManager.GetResource("TextShaderDF");
			}
			else
			{
				textShader = new JoglShaderTextRenderDF();
				textShader.Create(pipe);
				
				JoglResourceManager.AddResource("TextShaderDF", textShader);
			}
		}
	}
}
