uniform float scaleW;
uniform float scaleH;
uniform float scaleD;

uniform vec3 camPos;
uniform vec3 camLook;
uniform vec3 camUp;
uniform vec3 camLeft;

//const float smoothing = 1.0/16.0;

float distRay(float x , float y)
{
	vec3 rpos = camPos + (camUp * y * scaleH) + (camLeft * x * scaleW);
	float d = (-rpos.y) / camLook.y;
	return d;
}

vec3 posRayIntersect(float x , float y)
{
	vec3 rpos = camPos + (camUp * y * scaleH) + (camLeft * x * scaleW);
	float d = (-rpos.y) / camLook.y;
	vec3 v = rpos + (camLook * d);
	return v;
}

float colorFact(float xpos, float ypos)
{
	//prevent divide by zero!
	if (camLook.y == 0)
	{
		return 0.0;
	}
	
	vec3 vpos = posRayIntersect(xpos, ypos);
	
	float gridSize = 50.0;
	float invGridSize = 1.0 / gridSize;
	float xcomp = mod(vpos.x, gridSize) * invGridSize;
	float ycomp = mod(vpos.z, gridSize) * invGridSize;
	float lineWidth = 0.1;
	float lineHalf = lineWidth * 0.5;
	
	float xf = abs(0.5 - xcomp) * 2.0;
	float yf = abs(0.5 - ycomp) * 2.0;
	float smoothingx = 0.7 * length(vec2(dFdx(xf), dFdy(xf)));
	float smoothingy = 0.7 * length(vec2(dFdx(yf), dFdy(yf)));
	float alphax = smoothstep((1.0 - lineHalf) - smoothingx, (1.0 - lineHalf) + smoothingx, xf);
	float alphay = smoothstep((1.0 - lineHalf) - smoothingy, (1.0 - lineHalf) + smoothingy, yf);
	float tc = min(1.0, alphax + alphay);
	return tc;
}

float colorFactD(float xpos, float ypos)
{
	//prevent divide by zero!
	if (camLook.y == 0)
	{
		return 1.0;
	}
	
	float tc = distRay(xpos, ypos);
	
	tc = clamp(tc / 5000.0, 0.0, 1.0);
	return tc;
}

void main( )
{
	vec4 colorBack = vec4(0.3, 0.5, 0.7, 1.0);
	vec4 colorGrid = vec4(0.75, 0.75, 0.75, 1.0);

	//colorGrid = vec4(gl_FragCoord.x / scaleW, gl_FragCoord.y / scaleH, 0.0, 1.0);
	
	//vec3 groundNorm = vec4(-camUp.x, camUp.y, -camUp.z);
	//vec3 groundPos = -camPos;

	float xpos = (((1.0 - (gl_FragCoord.x / scaleW)) -0.5) * 2.0);
	float ypos = (((gl_FragCoord.y / scaleH) -0.5) * 2.0);
	//float mfactor = clamp(distRay(xpos, ypos) / 1000.0, 0.0, 1.0);
	
	float tc = colorFact(xpos, ypos);
	
	gl_FragColor = mix(colorBack, colorGrid, tc);
}
