uniform float below;
uniform float lineLength;
uniform float numCohorts;
uniform vec3 cohorts1;
uniform vec3 cohorts2;
uniform vec3 cohorts3;
uniform vec3 cohorts4;

varying vec2 texture_coordinate;
uniform sampler2D my_color_texture;

//gamma corrected color blending...(not sure this works quite correctly yet)
vec3 mixColors(vec3 c1, vec3 c2, float f)
{
	vec3 gamma = vec3(2.2, 2.0, 1.8);
	vec3 invgamma = vec3(1.0 / gamma.r, 1.0 / gamma.g, 1.0 / gamma.b);

	vec3 cc1 = pow(c1, gamma);
	vec3 cc2 = pow(c2, gamma);

	vec3 result = mix(cc1, cc2, f);
	vec3 cresult = pow(result, invgamma);
	return cresult;	
}

vec3 stepmixColors(vec3 c1, vec3 c2, float edge0, float delta, float x)
{
    float mixFactor = clamp(0.5 * (x - edge0 + delta) / delta, 0.0, 1.0);
    return mixColors(c1, c2, mixFactor);
}

vec4 getColorStripe2()
{
	float tx = (texture_coordinate.x * lineLength * 0.05 * 0.66);
	float dist = abs(fract(tx) - 0.5) * 2.0;
	float delta = fwidth(dist);
	float tr = smoothstep(0.5 - (delta * 0.5), 0.5 + (delta * 0.5), dist);

	return vec4(mixColors(cohorts1, cohorts2, tr), 1);
}

vec4 getColorStripe3()
{
	float tx = (texture_coordinate.x * lineLength * (0.05 / 3.0));
	float dist = abs(fract(tx) - 0.5) * 2.0;
	float delta = fwidth(dist) * 0.66;
	
    const float A = 0.25;
    const float B = 0.75;

	vec3 lineColor = cohorts1;

    if (distance(dist, A) < delta) 
    	lineColor = stepmixColors(cohorts1, cohorts2, A, delta, dist);
    else if (distance(dist, B) < delta) 
    	lineColor = stepmixColors(cohorts2, cohorts3, B, delta, dist);
    else if (dist < A) 
    	lineColor = cohorts1;
    else if (dist < B) 
    	lineColor = cohorts2;
    else 
    	lineColor = cohorts3;
    	
	return vec4(lineColor, 1);
}

vec4 getColorStripe4()
{
	float tx = (texture_coordinate.x * lineLength * (0.05 * 0.25));	
	float dist = abs(fract(tx) - 0.5) * 2.0;
	float delta = fwidth(dist) * 0.5;
	
    const float A = 0.167;
    const float B = 0.5;
    const float C = 0.833;

	vec3 lineColor = cohorts1;

    if (distance(dist, A) < delta) 
    	lineColor = stepmixColors(cohorts1, cohorts2, A, delta, dist);
    else if (distance(dist, B) < delta) 
    	lineColor = stepmixColors(cohorts2, cohorts3, B, delta, dist);
    else if (distance(dist, C) < delta)
    	lineColor = stepmixColors(cohorts3, cohorts4, C, delta, dist);
    else if (dist < A) 
    	lineColor = cohorts1;
    else if (dist < B) 
    	lineColor = cohorts2;
    else if (dist < C) 
    	lineColor = cohorts3;
    else 
    	lineColor = cohorts4;
    	
	return vec4(lineColor, 1);
}

void main( )
{
	vec2 texcoord = (gl_FragCoord.xy) / textureSize(my_color_texture, 0);
	vec4 tex = texture2D(my_color_texture, texcoord);
	
	if (tex.r == 1.0)
	{
		discard;
	}
	
	if (numCohorts == 1)
	{
		gl_FragColor = vec4(cohorts1, 1);
	}
	else if (numCohorts == 2)
	{
		gl_FragColor = getColorStripe2();
	}
	else if (numCohorts == 3)
	{
		gl_FragColor = getColorStripe3();
	}
	else if (numCohorts == 4)
	{
		gl_FragColor = getColorStripe4();
	}
	
	gl_FragColor = gl_FragColor * below;
}
