package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglColoredMesh;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialCamWidgetCubeBase;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialCamWidgetCubeSides;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialCamWidgetCubeText;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.opengl.GL2;

public class CamWidgetCube extends JoglSceneObject
{
	private static MaterialCamWidgetCubeBase matCWCB = new MaterialCamWidgetCubeBase();
	private static MaterialCamWidgetCubeSides matCWCS = new MaterialCamWidgetCubeSides();
	private static MaterialCamWidgetCubeText matCWCT = new MaterialCamWidgetCubeText();
	
	private Vector<JoglVectord3> cubeVertexCoords;
	private Vector<JoglVectord2> txCoords;
	
	private JoglTexturedMesh jtm;
	private JoglColoredMesh jtm2;
	private IMeshDrawer vao;
	private IMeshDrawer vao2;
	
	public CamWidgetCube()
	{
		cubeVertexCoords = GenerateCubeVertexCoords();
		txCoords = GenerateTextureCoords("BRFLTB");
		
		jtm = new JoglTexturedMesh(GL2.GL_QUADS);
		jtm.setVerts(cubeVertexCoords);
		jtm.setTexCoords(txCoords);
		
		jtm2 = new JoglColoredMesh(GL2.GL_QUADS);
		jtm2.setVerts(cubeVertexCoords);
		
		Vector<JoglVectord3> cubeColors = new Vector<JoglVectord3>();
		JoglVectord3 cubeBlue = new JoglVectord3(0.0f, 0.0f, 1.0f);
		cubeColors.add(cubeBlue);   // set the color of the quad (blue)
		cubeColors.add(cubeBlue);   // set the color of the quad (blue)
		cubeColors.add(cubeBlue);   // set the color of the quad (blue)
		cubeColors.add(cubeBlue);   // set the color of the quad (blue)
		
		//draw right quad
		JoglVectord3 cubeRed = new JoglVectord3(1.0f, 0.0f, 0.0f);
		cubeColors.add(cubeRed);   // set the color of the quad (red)
		cubeColors.add(cubeRed);   // set the color of the quad (red)
		cubeColors.add(cubeRed);   // set the color of the quad (red)
		cubeColors.add(cubeRed);   // set the color of the quad (red)

		//draw front quad
		JoglVectord3 cubeGreen = new JoglVectord3(0.0f, 1.0f, 0.0f);
		cubeColors.add(cubeGreen);   // set the color of the quad (green)
		cubeColors.add(cubeGreen);   // set the color of the quad (green)
		cubeColors.add(cubeGreen);   // set the color of the quad (green)
		cubeColors.add(cubeGreen);   // set the color of the quad (green)

		//draw left quad
		JoglVectord3 cubeYellow = new JoglVectord3(1.0f, 1.0f, 0.0f);
		cubeColors.add(cubeYellow);   // set the color of the quad (yellow)
		cubeColors.add(cubeYellow);   // set the color of the quad (yellow)
		cubeColors.add(cubeYellow);   // set the color of the quad (yellow)
		cubeColors.add(cubeYellow);   // set the color of the quad (yellow)

		//draw top quad
		JoglVectord3 cubePurple = new JoglVectord3(1.0f, 0.0f, 1.0f);
		cubeColors.add(cubePurple);   // set the color of the quad (purple)
		cubeColors.add(cubePurple);   // set the color of the quad (purple)
		cubeColors.add(cubePurple);   // set the color of the quad (purple)
		cubeColors.add(cubePurple);   // set the color of the quad (purple)

		//draw bottom quad
		JoglVectord3 cubeCyan = new JoglVectord3(0.0f, 1.0f, 1.0f);
		cubeColors.add(cubeCyan);   // set the color of the quad (cyan)
		cubeColors.add(cubeCyan);   // set the color of the quad (cyan)
		cubeColors.add(cubeCyan);   // set the color of the quad (cyan)
		cubeColors.add(cubeCyan);   // set the color of the quad (cyan)
		
		jtm2.setColors3(cubeColors);
		
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		pipe.PushPipelineState(matCWCB);
//		pipe.SetMaskState(GraphicsPipeline.MaskState.DrawColorOnly);
//		pipe.DepthTest(false);
//		pipe.SetCullState(GraphicsPipeline.CullState.CullFront); //looks like winding order in the model is wrong, might want to fix that eventually
		
		DrawCubeSides(pipe);
		DrawCubeText(pipe);

//		pipe.DepthTest(true);
//		pipe.SetMaskState(GraphicsPipeline.MaskState.DrawBoth);
//		pipe.SetCullState(GraphicsPipeline.CullState.CullNone);
		pipe.PopPipelineState(matCWCB);
	}
	
	private void DrawCubeSides(GraphicsPipeline pipe)
	{
		pipe.PushPipelineState(matCWCS);
		
		if (vao2 == null)
		{
			vao2 = new VAODrawer();
			jtm2.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jtm2.getNumVerts()));
			vao2.LoadMesh(pipe, jtm2);
		}

		//draw a cube
		vao2.Draw(pipe);
		
		pipe.PopPipelineState(matCWCS);
	}
	
	private void DrawCubeText(GraphicsPipeline pipe)
	{	
		pipe.PushPipelineState(matCWCT);
			
		if (vao == null)
		{
			vao = new VAODrawer();
			jtm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jtm.getNumVerts()));
			vao.LoadMesh(pipe, jtm);
		}

		//draw a cube
		vao.Draw(pipe);
		
//		JoglMeshDrawState jmds = new JoglMeshDrawState();
//		jmds.drawType = GL2.GL_QUADS;
//		jmds.drawEnd = 24;
//		
//		JoglMeshDrawer jmd = new JoglMeshDrawer();
//		jmd.draw(gl, jtm, jmds);

		pipe.PopPipelineState(matCWCT);
	}
	
	public Vector<JoglVectord3> GenerateCubeVertexCoords()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
		
		//draw back quad
		lst.add(new JoglVectord3(-1.0f, 1.0f, 1.0f));   // Top Left
		lst.add(new JoglVectord3( 1.0f, 1.0f, 1.0f));   // Top Right
		lst.add(new JoglVectord3( 1.0f,-1.0f, 1.0f));   // Bottom Right
		lst.add(new JoglVectord3(-1.0f,-1.0f, 1.0f));   // Bottom Left
		
		//draw right quad
		lst.add(new JoglVectord3(-1.0f, 1.0f,-1.0f));   // Top Left
		lst.add(new JoglVectord3(-1.0f, 1.0f, 1.0f));   // Top Right
		lst.add(new JoglVectord3(-1.0f,-1.0f, 1.0f));   // Bottom Right
		lst.add(new JoglVectord3(-1.0f,-1.0f,-1.0f));   // Bottom Left

		//draw front quad
		lst.add(new JoglVectord3(1.0f, 1.0f, -1.0f));   // Top Left
		lst.add(new JoglVectord3(-1.0f, 1.0f, -1.0f));   // Top Right
		lst.add(new JoglVectord3(-1.0f,-1.0f, -1.0f));   // Bottom Right
		lst.add(new JoglVectord3(1.0f,-1.0f, -1.0f));   // Bottom Left

		//draw left quad
		lst.add(new JoglVectord3(1.0f, 1.0f,1.0f));   // Top Left
		lst.add(new JoglVectord3(1.0f, 1.0f,-1.0f));   // Top Right
		lst.add(new JoglVectord3(1.0f,-1.0f,-1.0f));   // Bottom Right
		lst.add(new JoglVectord3(1.0f,-1.0f,1.0f));   // Bottom Left

		//draw top quad
		lst.add(new JoglVectord3(-1.0f, 1.0f,-1.0f));   // Top Left
		lst.add(new JoglVectord3( 1.0f, 1.0f,-1.0f));   // Top Right
		lst.add(new JoglVectord3( 1.0f, 1.0f, 1.0f));   // Bottom Right
		lst.add(new JoglVectord3(-1.0f, 1.0f, 1.0f));   // Bottom Left

		//draw bottom quad
		lst.add(new JoglVectord3(-1.0f,-1.0f,-1.0f));   // Top Left
		lst.add(new JoglVectord3(-1.0f,-1.0f, 1.0f));   // Top Right
		lst.add(new JoglVectord3( 1.0f,-1.0f, 1.0f));   // Bottom Right
		lst.add(new JoglVectord3( 1.0f,-1.0f,-1.0f));   // Bottom Left
		
		return lst;
	}

	public Vector<JoglVectord2> GenerateTextureCoords(String text) 
	{
		//FontData font = JoglInteractor.defaultFont;
		
		double xmin = 0.0;
		double xmax = 1.0;
		double ymin = 0.0;
		double ymax = 1.0;
		
		double invx = 1.0 / 16.0f;
		
		Vector<JoglVectord2> vlist = new Vector<JoglVectord2>();
		
		for (int i = 0; i < text.length(); i++)
		{
			char c = text.charAt(i);
			if (c != ' ')
			{
				int ic = c - 32;
	
				int x = ic % 16;
				int y = (ic - x) / 16;

				float left = 0;
				float right = 0;
				//float left = (float)font.GetCharLeft(c);
				//float right = (float)font.GetCharRight(c);
				
				xmin = x * invx;
				xmax = xmin + invx;
				ymin = 1.0 - (y * invx);
				ymax = 1.0 - ((y * invx) + invx);

				vlist.add(new JoglVectord2(xmin + left, ymin));   // Top Right
				vlist.add(new JoglVectord2(xmax - right, ymin));   // Top Left
				vlist.add(new JoglVectord2(xmax - right, ymax));   // Bottom Left
				vlist.add(new JoglVectord2(xmin + left, ymax));   // Bottom Right
			}
		}
		
		return vlist;
	}
}
