package org.processmining.processprofiler3d.compviz.jogl.scene;

import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLensPerspective;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTarget;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTargetCustom;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;

public class JSOCamWidget extends JSOCamWidgetBase
{
	private JoglCamera newcam;
	
	public JSOCamWidget()
	{
		newcam = new JoglCamera();
		newcam.setCameraLens(new JoglCameraLensPerspective(10.0, 1.0, 0.1, 1000.0));
	}
	
	private CamWidgetCube cube = null;
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{	
		if (!JoglGraphicsSettings.SupportsFramebufferObjects(pipe))
			return;
		
		if (cube == null)
		{
			cube = new CamWidgetCube();
		}

		DrawCubeToTexture(pipe);
	}

	private void DrawCubeToTexture(GraphicsPipeline pipe) 
	{
		JoglCamera c = pipe.cam;
		JoglRenderTarget target = pipe.target;
		
		//calculate actual size of widget on screen in pixels
		float w = (0.25f / (float)target.getAspect()) * 0.5f * target.getWidth();
		float h = 0.25f * 0.5f * target.getHeight();

		newcam.SetLookDir(c.GetLookDir()); //copy look direction from actual camera
		
		newcam.SetProjectionMatrix(pipe);
		newcam.SetCameraMatrix(pipe);
		
		JoglRenderTarget newtarget = new JoglRenderTargetCustom(0, (int)(target.getHeight() - h), (int)w, (int)h);
		newtarget.FitViewport(pipe);
		
		double widgetDistF = 30.0;

		JoglVectord3 forwardOff = JoglVectord3.Multiply(newcam.GetLookDir(), widgetDistF);
		JoglVectord3 position = JoglVectord3.Add(newcam.GetPosition(), forwardOff);
		JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateTranslation(position);

		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);
		
		cube.DrawObject(pipe);
		
		pipe.transform.Pop();
		
		target.FitViewport(pipe);

		c.DrawCamera(pipe);
	}

	@Override
	public void Dispose(GraphicsPipeline pipe)
	{
	}
}
