package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialCamWidgetCube;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderPipelineDefaultTextured;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglFrameBufferObject;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.gldatastructures.JoglTextureBufferObject;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLensPerspective;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTarget;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTargetFramebuffer;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.settings.JoglGraphicsSettings;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;

public class JSOCamWidgetCP extends JSOCamWidgetBase
{
	private static MaterialCamWidgetCube matCWC = new MaterialCamWidgetCube();
	
	private JoglCamera cam;
	private JoglCamera newcam;
	
	public JSOCamWidgetCP(JoglCamera c)
	{
		cam = c;
		
		newcam = new JoglCamera();
		//newcam.setDepthRange(0.1f, 1000.0f);
	}
	
	private JoglFrameBufferObject fbo = null;
	private JoglRenderTargetFramebuffer fbTarget = null;
	private CamWidgetCube cube = null;
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{	
		JoglRenderTarget target = pipe.target;
		
		if (!JoglGraphicsSettings.SupportsFramebufferObjects(pipe))
			return;
		
		if (cube == null)
		{
			cube = new CamWidgetCube();
		}
		
		DrawCubeToTextureCP(pipe);
		
		target.Bind(pipe);
		target.FitViewport(pipe);
		
		JoglTextureBufferObject jtbo = (JoglTextureBufferObject)fbo.getColorBuffer0();
		jtbo.getTexture().Bind(pipe);

		DrawTextureToScreenCP(pipe);

		jtbo.getTexture().Unbind(pipe);
	}
	
	private void DrawCubeToTextureCP(GraphicsPipeline pipe) 
	{
		JoglRenderTarget target = pipe.target;

		if (fbo != null && (fbo.getWidth() != target.getWidth() || fbo.getHeight() != target.getHeight()))
		{
			fbo.Dispose(pipe);
			fbo = null;
		}
		
		if (fbo == null)
		{	
			fbo = new JoglFrameBufferObject(pipe, target.getWidth(), target.getHeight());
			fbo.Bind(pipe);
			fbo.AttachDepthBuffer(pipe, fbo.CreateRboAttachment(pipe, GL.GL_DEPTH_COMPONENT32));
			fbo.AttachColorBuffer0(pipe, fbo.CreateTboAttachment(pipe, GL.GL_RGBA8, 0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE));

			if (!fbo.isFramebufferComplete())
			{
				System.out.println("Framebuffer failed to initialize.");
			}
			
			fbo.Unbind(pipe);
			
			fbTarget = new JoglRenderTargetFramebuffer(fbo);
		}
		
		fbo.Bind(pipe);
		
		fbTarget.FitViewport(pipe);
		
		//clear fbo
		fbTarget.Clear(pipe);

		//JoglVectord2 oldDepthRange = cam.getDepthRange();
		//newcam.setDepthRange(0.1f, 100.0f);
		//cam.SetProjectionMatrix(gl, target.getSize());
		newcam.SetLookDir(cam.GetLookDir()); //copy look direction from actual camera
		
		//newcam.SetProjectionMatrix(gl, target.getAspect());
		//newcam.SetProjectionMatrix(gl, target.getAspect(), 0, 0.25, 1.0, 0.75);
		newcam.setCameraLens(new JoglCameraLensPerspective(45.0, target.getAspect(), 0.1, 1000.0, 0.0, 0.25, 1.0, 0.75));
		newcam.SetProjectionMatrix(pipe);
		newcam.SetCameraMatrix(pipe);
		
		//double widgetDist = cam.getDepthRange().getX() + ((cam.getDepthRange().getY() - cam.getDepthRange().getX()) * 0.5);
		double widgetDistF = 50.0;
		double widgetDistL = 35.0;
		double widgetDistU = 18.0;

		JoglVectord3 upOff = JoglVectord3.Multiply(newcam.GetUpDir(), widgetDistU);
		JoglVectord3 leftOff = JoglVectord3.Multiply(newcam.GetLeftDir(), widgetDistL);
		JoglVectord3 forwardOff = JoglVectord3.Multiply(newcam.GetLookDir(), widgetDistF);
		JoglVectord3 position = JoglVectord3.Add(newcam.GetPosition(), JoglVectord3.Add(JoglVectord3.Add(forwardOff, leftOff), upOff));
		JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateTranslation(position);

		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);
		
		GraphicsPipeline newPipe = new GraphicsPipeline(pipe);
		newPipe.target = fbTarget;
		
		cube.DrawObject(newPipe);
		
		pipe.transform.Pop();

		//cam.setDepthRange((float)oldDepthRange.getX(), (float)oldDepthRange.getY());
		//cam.SetProjectionMatrix(gl, target.getAspect());
		cam.DrawCamera(pipe);
		
		fbo.Unbind(pipe);
	}
	
	private Vector<JoglVectord3> GenerateGeometry()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();

		float w = 0.5f;// / (float)target.getAspect();
		float h = 0.5f;
		float left = -1.0f;
		float top = 1.0f - h;
		
		lst.add(new JoglVectord3(left, top, 0.0f));
		lst.add(new JoglVectord3(left + w, top, 0.0f));
		lst.add(new JoglVectord3(left + w, top + h, 0.0f));
		lst.add(new JoglVectord3(left, top + h, 0.0f));

		return lst;
	}
	
	private Vector<JoglVectord2> GenerateTexCoords()
	{
		Vector<JoglVectord2> lst = new Vector<JoglVectord2>();
		
		lst.add(new JoglVectord2(0.0f, 0.0f));
		lst.add(new JoglVectord2(1.0f, 0.0f));
		lst.add(new JoglVectord2(1.0f, 1.0f));
		lst.add(new JoglVectord2(0.0f, 1.0f));

		return lst;
	}
	
	public JoglTexturedMesh CreateMesh()
	{
		JoglTexturedMesh jcm = new JoglTexturedMesh(GL2.GL_QUADS);
		jcm.setVerts(GenerateGeometry());
		jcm.setTexCoords(GenerateTexCoords());
		jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jcm.getNumVerts()));
		
		return jcm;
	}
	
	private IMeshDrawer vao = null;
	private JoglShader shad = null;
	
	private void DrawTextureToScreenCP(GraphicsPipeline pipe) 
	{
//		float w = 2.0f;// / (float)target.getAspect();
//		float h = 2.0f;
//		float left = -1.0f;
//		float top = 1.0f - h;
//		float w = 0.5f;// / (float)target.getAspect();
//		float h = 0.5f;
//		float left = -1.0f;
//		float top = 1.0f - h;
		
		//revert projection and transformation
		pipe.transform.Push();
		pipe.transform.LoadIdentity();	
		pipe.projection.Push();
		pipe.projection.LoadIdentity();
		
		pipe.PushPipelineState(matCWC);
//		gl.glDisable(GL.GL_DEPTH_TEST);
//		gl.glDepthMask(false);
//
//		pipe.setDrawModeAlphaBlended();
		
//		gl.glBegin(GL2.GL_QUADS);
//		
//		gl.glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
//		gl.glTexCoord2f(0.0f, 0.0f);
//		gl.glVertex3f(left, top, 0.0f);
//		gl.glTexCoord2f(1.0f, 0.0f);
//		gl.glVertex3f(left + w, top, 0.0f);
//		gl.glTexCoord2f(1.0f, 1.0f);
//		gl.glVertex3f(left + w, top + h, 0.0f);
//		gl.glTexCoord2f(0.0f, 1.0f);
//		gl.glVertex3f(left, top + h, 0.0f);
//		
//		gl.glEnd();
		if (vao == null)
		{
			vao = new VAODrawer();
			vao.LoadMesh(pipe, CreateMesh());
		}
		if (shad == null)
		{
			shad = new JoglShaderPipelineDefaultTextured();
			shad.Create(pipe);
		}
		
		shad.Bind(pipe);
		vao.Draw(pipe);
		shad.Unbind(pipe);

//		pipe.setDrawModeOpaque();
//
//		gl.glDepthMask(true);
//		gl.glEnable(GL.GL_DEPTH_TEST);
		pipe.PopPipelineState(matCWC);

		//reinstate projection and transformation
		pipe.projection.Pop();
		pipe.transform.Pop();
	}

	@Override
	public void Dispose(GraphicsPipeline pipe)
	{
		if (vao != null)
		{
			vao.Dispose();
		}
		if (shad != null)
		{
			shad.Dispose(pipe);
		}
		if (fbo != null)
		{
			fbo.Dispose(pipe);
			fbo = null;
		}
	}
}
