package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.deckfour.xes.model.XLog;
import org.processmining.processprofiler3d.compviz.PP3DCohort;
import org.processmining.visualisation3d.maths.JoglColor3;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JSOCohortColorManager extends JoglSceneObject
{
	//private XLog myLog = null;
	private static Map<String, JoglVectord3> colorsByCohortName;
	private static Map<Integer, JoglVectord3> colorsByCohortID;
	public static Vector<PP3DCohort> cohorts = new Vector<PP3DCohort>();

	public JSOCohortColorManager(XLog log)
	{
		//myLog = log;
	}
	
	public JoglVectord3 getColorForCohort(String cohortName)
	{
		if (colorsByCohortName.containsKey(cohortName))
		{
			return colorsByCohortName.get(cohortName);
		}
		//return gray for unknown cohorts
		return JoglColor3.GRAY;
	}
	
	public JoglVectord3 getColorForCohort(int cId)
	{
		if (colorsByCohortID.containsKey(cId))
		{
			return colorsByCohortID.get(cId);
		}
		//return gray for unknown cohorts
		return JoglColor3.GRAY;
	}
	
	public JoglVectord4 getColor4ForCohort(String cohortName, double alpha)
	{
		if (colorsByCohortName.containsKey(cohortName))
		{
			JoglVectord3 col = colorsByCohortName.get(cohortName);
			return new JoglVectord4(col.getX(), col.getY(), col.getZ(), alpha);
		}
		//return gray for unknown cohorts
		return new JoglVectord4(0.5f,0.5f,0.5f, 1.0f);
	}
	
	public JoglVectord4 getColor4ForCohort(int cId, double alpha)
	{
		if (colorsByCohortID.containsKey(cId))
		{
			JoglVectord3 col = colorsByCohortID.get(cId);
			return new JoglVectord4(col.getX(), col.getY(), col.getZ(), alpha);
		}
		//return gray for unknown cohorts
		return new JoglVectord4(0.5f,0.5f,0.5f, 1.0f);
	}
	
//	public void updateColorsByCohort(CohortClassifier cc) 
//	{
//		Vector<JoglVectord3> colors = new Vector<JoglVectord3>();
//		colors.add(JoglColor3.RED);
//		colors.add(JoglColor3.GREEN);
//		colors.add(JoglColor3.BLUE);
//		colors.add(JoglColor3.YELLOW);
//		colors.add(JoglColor3.CYAN);
//		colors.add(JoglColor3.MAGENTA);
//		colors.add(JoglColor3.WHITE);
//		colors.add(JoglColor3.BLACK);
//		
//		colorsByCohortName = new HashMap<String, JoglVectord3>();
//		colorsByCohortID = new HashMap<Integer, JoglVectord3>();
////		int i = 0;
////		for (String classifier : cc.getCohortNames())
////		{
////			if (i < colors.size())
////			{
////				colorsByCohort.put(classifier, colors.get(i));
////			}
////			else
////			{
////				//use gray color once we ran out of palette colors
////				colorsByCohort.put(classifier, new JoglVectord3(0.5f,0.5f,0.5f));
////			}
////			i++;
////		}
//		
//		HashMap<Integer,Integer> histogram = cc.getCohortCaseHistogram(myLog);
//		
//		int i = 0;
//		for (ClassificationCohort ccs : cc.getCohorts())
//		{
//			int cId = ccs.getID();
//			//don't assign colors to cohorts for which no cases exist!
//			if (!histogram.containsKey(cId))
//			{
//				continue;
//			}
//			else
//			{
//				if (histogram.get(cId) == 0)
//				{
//					continue;
//				}
//			}
//			
//			if (i < colors.size())
//			{
//				colorsByCohortName.put(ccs.getName(), colors.get(i));
//				colorsByCohortID.put(cId, colors.get(i));
//			}
//			else
//			{
//				//use gray color once we ran out of palette colors
//				colorsByCohortName.put(ccs.getName(), JoglColor3.GRAY);
//				colorsByCohortID.put(cId, JoglColor3.GRAY);
//			}
//			i++;
//		}
//	}
	
	public void updateColorsByCohort(Vector<PP3DCohort> pcLst) 
	{
		Vector<JoglVectord3> colors = new Vector<JoglVectord3>();
		colors.add(JoglColor3.RED);
		colors.add(JoglColor3.GREEN);
		colors.add(JoglColor3.BLUE);
		colors.add(JoglColor3.YELLOW);
		colors.add(JoglColor3.CYAN);
		colors.add(JoglColor3.MAGENTA);
		colors.add(JoglColor3.WHITE);
		colors.add(JoglColor3.BLACK);
		
		colorsByCohortName = new HashMap<String, JoglVectord3>();
		colorsByCohortID = new HashMap<Integer, JoglVectord3>();
//		int i = 0;
//		for (String classifier : cc.getCohortNames())
//		{
//			if (i < colors.size())
//			{
//				colorsByCohort.put(classifier, colors.get(i));
//			}
//			else
//			{
//				//use gray color once we ran out of palette colors
//				colorsByCohort.put(classifier, new JoglVectord3(0.5f,0.5f,0.5f));
//			}
//			i++;
//		}
		
		cohorts.clear();
		
		int i = 0;
		for (PP3DCohort pc : pcLst)
		{
			cohorts.add(pc);
			int cId = pc.cohortID;
			//don't assign colors to cohorts for which no cases exist!
			if (pc.caseNum == 0)
			{
				pc.cohortColor = JoglColor3.GRAY;
				continue;
			}
			
			if (i < colors.size())
			{
				pc.cohortColor = colors.get(i);
				colorsByCohortName.put(pc.cohortName, colors.get(i));
				colorsByCohortID.put(cId, colors.get(i));
			}
			else
			{
				//use gray color once we ran out of palette colors
				pc.cohortColor = JoglColor3.GRAY;
				colorsByCohortName.put(pc.cohortName, JoglColor3.GRAY);
				colorsByCohortID.put(cId, JoglColor3.GRAY);
			}
			i++;
		}
	}

}
