package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglColoredMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderTrajectory;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.opengl.GL2;

/*
 * A quad to render in the scene
 */
public class JSOCube extends JoglSceneObject 
{
	private JoglVectord3 position = new JoglVectord3(0,0,0);
	private JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateIdentity();

	public void setPosition(double x, double y, double z)
	{
		position = new JoglVectord3(x,y,z);
		transform = JoglMatrixd4x4.CreateTranslation(x, y, z);
	}
	
	public void setPosition(JoglVectord3 v)
	{
		position = v;
		transform = JoglMatrixd4x4.CreateTranslation(v.getX(), v.getY(), v.getZ());
	}
	
	private Vector<JoglVectord3> GenerateGeometry()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
		
		lst.add(new JoglVectord3(-1.0f, 1.0f, 1.0f));   // Top Left
		lst.add(new JoglVectord3( 1.0f, 1.0f, 1.0f));   // Top Right
		lst.add(new JoglVectord3( 1.0f,-1.0f, 1.0f));   // Bottom Right
		lst.add(new JoglVectord3(-1.0f,-1.0f, 1.0f));   // Bottom Left
		
		// Draw Another Quad
		lst.add(new JoglVectord3(-1.0f, 1.0f,-1.0f));   // Top Left
		lst.add(new JoglVectord3(-1.0f, 1.0f, 1.0f));   // Top Right
		lst.add(new JoglVectord3(-1.0f,-1.0f, 1.0f));   // Bottom Right
		lst.add(new JoglVectord3(-1.0f,-1.0f,-1.0f));   // Bottom Left

		lst.add(new JoglVectord3(1.0f, 1.0f, -1.0f));   // Top Left
		lst.add(new JoglVectord3(-1.0f, 1.0f, -1.0f));   // Top Right
		lst.add(new JoglVectord3(-1.0f,-1.0f, -1.0f));   // Bottom Right
		lst.add(new JoglVectord3(1.0f,-1.0f, -1.0f));   // Bottom Left

		// Draw Another Quad
		lst.add(new JoglVectord3(1.0f, 1.0f,1.0f));   // Top Left
		lst.add(new JoglVectord3(1.0f, 1.0f,-1.0f));   // Top Right
		lst.add(new JoglVectord3(1.0f,-1.0f,-1.0f));   // Bottom Right
		lst.add(new JoglVectord3(1.0f,-1.0f,1.0f));   // Bottom Left

		return lst;
	}
	
	private Vector<JoglVectord3> GenerateColors()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();

		lst.add(new JoglVectord3(0.0f, 0.0f, 1.0f));   // set the color of the quad
		lst.add(new JoglVectord3(0.0f, 0.0f, 1.0f));   // set the color of the quad
		lst.add(new JoglVectord3(0.0f, 0.0f, 1.0f));   // set the color of the quad
		lst.add(new JoglVectord3(0.0f, 0.0f, 1.0f));   // set the color of the quad
		
		// Draw Another Quad
		lst.add(new JoglVectord3(1.0f, 0.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(1.0f, 0.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(1.0f, 0.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(1.0f, 0.0f, 0.0f));   // set the color of the quad

		lst.add(new JoglVectord3(0.0f, 1.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(0.0f, 1.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(0.0f, 1.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(0.0f, 1.0f, 0.0f));   // set the color of the quad

		// Draw Another Quad
		lst.add(new JoglVectord3(1.0f, 1.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(1.0f, 1.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(1.0f, 1.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(1.0f, 1.0f, 0.0f));   // set the color of the quad

		return lst;
	}
	
	public JoglColoredMesh CreateMesh()
	{
		JoglColoredMesh jcm = new JoglColoredMesh(GL2.GL_QUADS);
		jcm.setVerts(GenerateGeometry());
		jcm.setColors3(GenerateColors());
		jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jcm.getNumVerts()));
		
		return jcm;
	}
	
	private IMeshDrawer vao = null;
	private JoglShader shad = null;
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);
		
		// Draw A Quad
//		gl.glBegin(GL2.GL_QUADS);       
//		
//		gl.glColor3f(0.0f, 0.0f, 1.0f);   // set the color of the quad
//		gl.glVertex3f(-1.0f, 1.0f, 1.0f);   // Top Left
//		gl.glVertex3f( 1.0f, 1.0f, 1.0f);   // Top Right
//		gl.glVertex3f( 1.0f,-1.0f, 1.0f);   // Bottom Right
//		gl.glVertex3f(-1.0f,-1.0f, 1.0f);   // Bottom Left
//		
//		// Draw Another Quad
//		gl.glColor3f(1.0f, 0.0f, 0.0f);   // set the color of the quad
//		gl.glVertex3f(-1.0f, 1.0f,-1.0f);   // Top Left
//		gl.glVertex3f(-1.0f, 1.0f, 1.0f);   // Top Right
//		gl.glVertex3f(-1.0f,-1.0f, 1.0f);   // Bottom Right
//		gl.glVertex3f(-1.0f,-1.0f,-1.0f);   // Bottom Left
//
//		gl.glColor3f(0.0f, 1.0f, 0.0f);   // set the color of the quad
//		gl.glVertex3f(1.0f, 1.0f, -1.0f);   // Top Left
//		gl.glVertex3f(-1.0f, 1.0f, -1.0f);   // Top Right
//		gl.glVertex3f(-1.0f,-1.0f, -1.0f);   // Bottom Right
//		gl.glVertex3f(1.0f,-1.0f, -1.0f);   // Bottom Left
//
//		// Draw Another Quad
//		gl.glColor3f(1.0f, 1.0f, 0.0f);   // set the color of the quad
//		gl.glVertex3f(1.0f, 1.0f,1.0f);   // Top Left
//		gl.glVertex3f(1.0f, 1.0f,-1.0f);   // Top Right
//		gl.glVertex3f(1.0f,-1.0f,-1.0f);   // Bottom Right
//		gl.glVertex3f(1.0f,-1.0f,1.0f);   // Bottom Left
//		
//		// Done Drawing The Quad
//		gl.glEnd(); 
		if (vao == null)
		{
			vao = new VAODrawer();
			vao.LoadMesh(pipe, CreateMesh());
		}
		if (shad == null)
		{
			shad = new JoglShaderTrajectory();
			shad.Create(pipe);
		}
		
		shad.Bind(pipe);
		vao.Draw(pipe);
		shad.Unbind(pipe);
		
		pipe.transform.Pop();
	}

}
