package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglMesh;
import org.processmining.processprofiler3d.compviz.jogl.JoglModelStyle;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialGrid;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialGridBase;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialGridMasked;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialGridOrtho;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLens;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLensOrthographic;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLensPerspective;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTarget;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.opengl.GL2;

public class JSOGrid extends JoglSceneObject 
{
	private static MaterialGrid matG = new MaterialGrid();
	private static MaterialGridMasked matGMask = new MaterialGridMasked();
	private static MaterialGridOrtho matGOrtho = new MaterialGridOrtho();
	private static float width = 0.5f;
	private static float step = 20.0f;
	
	private boolean bVisible = true;
	private boolean bNoDepthInteraction = false;
	private boolean bCustomDepthRange = false;
	private boolean bUseMask = false;
	
	private JoglVectord4 gridColor = new JoglVectord4(0.75, 0.75, 0.75, 1.0);
	private JoglMesh mesh;
	
	private JoglCamera snapCam; //the camera around which the grid should be centered
								//if this is null, the grid will be centered around the world origin instead
								//the rendering camera maybe different from the camera around which the grid
								//should be centered, for example in virtual reality
	
	public boolean getVisible()
	{
		return bVisible;
	}
	
	public void setVisible(boolean b)
	{
		bVisible = b;
	}
	
	public JSOGrid(boolean bNoDepth, boolean bDepthRange, boolean bMasked, JoglCamera cam)
	{
		snapCam = cam;
		
		bNoDepthInteraction = bNoDepth;
		bCustomDepthRange = bDepthRange;
		bUseMask = bMasked;
		
		gridColor = JoglModelStyle.gridColor;
		
		mesh = new JoglMesh(GL2.GL_QUADS, 4000);
		mesh.setVerts(GenerateGeometry());
		mesh.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(mesh.getNumVerts()));
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		DrawGridWithParams(pipe, bUseMask, bCustomDepthRange, bNoDepthInteraction);
	}
	
	public void DrawGridWithParams(GraphicsPipeline pipe, boolean bMasked, boolean bCustomDepthRange, boolean bNoDepthInteraction)
	{
		JoglCamera c = pipe.cam;
		JoglCameraLens defaultLens = c.getCameraLens();
		JoglRenderTarget target = pipe.target;
		
		InitGrid(pipe);
		
		if (!bVisible)
			return;
		
		//JoglVectord2 oldDepthRange = c.getDepthRange();
		if (bCustomDepthRange)
		{
			//c.setDepthRange(0.1f, 10000.0f);
			//c.SetProjectionMatrix(gl, target.getAspect());
			if (defaultLens instanceof JoglCameraLensPerspective)
			{
				JoglCameraLensPerspective jclp = (JoglCameraLensPerspective)defaultLens;
				c.setCameraLens(new JoglCameraLensPerspective(jclp.getFieldOfView(), target.getAspect(), 0.1, 10000.0));
			}
			else
			{
				c.setCameraLens(new JoglCameraLensOrthographic(target.getAspect()));
			}
			c.SetProjectionMatrix(pipe);
		}
		
		if (bMasked)
		{
			DrawGrid(pipe, bNoDepthInteraction, matGMask);
		}
		else
		{
			DrawGrid(pipe, bNoDepthInteraction, matG);
		}

		if (bCustomDepthRange)
		{
			//c.setDepthRange((float)oldDepthRange.getX(), (float)oldDepthRange.getY());
			//c.SetProjectionMatrix(gl, target.getAspect());
			c.setCameraLens(defaultLens);
			c.SetProjectionMatrix(pipe);
		}
		
		//draw grid again in actual depth range to enable zTest with some visualization elements
		//DrawGrid(gl, false);
	}
	
	private IMeshDrawer vaoDrawer;
	private IMeshDrawer vaoDrawerOrtho;
	private boolean bInitedGrid = false;
	
	private void InitGrid(GraphicsPipeline pipe)
	{
		if (bInitedGrid)
			return;
		
		vaoDrawer = new VAODrawer();
		vaoDrawer.LoadMesh(pipe, mesh);
        
        bInitedGrid = true;
	}

	public static Vector<JoglVectord3> GenerateGeometry() 
	{
		Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();
		
		double w = width;
		double h = 5000.0;
		double depth = -5.0;
		
		int numLines = 500;
		double xoff = -numLines * step * 0.5;
		
		for (int i = 0; i < numLines; i++)
		{
			vlist.add(new JoglVectord3(-w + xoff, depth, -h));
			vlist.add(new JoglVectord3( w + xoff, depth, -h));
			vlist.add(new JoglVectord3( w + xoff, depth,  h));
			vlist.add(new JoglVectord3(-w + xoff, depth,  h));
			
			xoff = xoff + step;
		}
		
		xoff = -numLines * step * 0.5;
		for (int i = 0; i < numLines; i++)
		{
			vlist.add(new JoglVectord3(-h, depth, -w + xoff));
			vlist.add(new JoglVectord3( h, depth, -w + xoff));
			vlist.add(new JoglVectord3( h, depth,  w + xoff));
			vlist.add(new JoglVectord3(-h, depth,  w + xoff));
			
			xoff = xoff + step;
		}
		
		return vlist;
	}

	public static Vector<JoglVectord3> GenerateGeometryOrtho(double w, double h) 
	{
		Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();
		
		vlist.add(new JoglVectord3(-w, h, 0.0));
		vlist.add(new JoglVectord3(w, h, 0.0));
		vlist.add(new JoglVectord3(w, -h, 0.0));
		vlist.add(new JoglVectord3(-w, -h, 0.0));
		
		return vlist;
	}

	private void DrawGrid(GraphicsPipeline pipe, boolean bDepthOff, MaterialGridBase mat) 
	{
		JoglCamera c = pipe.cam;

		pipe.transform.Push();
		
		if (snapCam != null)
		{
			//move grid with camera
			//JoglVectord3 cpos = pipe.panel.getCamera().GetPosition();	//otherwise vr cameras get confused
			JoglVectord3 cpos = snapCam.GetPosition();	//otherwise vr cameras get confused
			double x = cpos.getX() - (cpos.getX() % step);
			double z = cpos.getZ() - (cpos.getZ() % step);
			JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateTranslation(x, 0, z);
			
			pipe.transform.MultMatrix(transform);
		}
		
		if (c.getCameraLens() instanceof JoglCameraLensOrthographic)
		{
			DrawGridOrtho(pipe, bDepthOff, c);
		}
		else
		{		
			DrawGridPersp(pipe, bDepthOff, mat);
		}
		
		pipe.transform.Pop();
	}

	private void DrawGridPersp(GraphicsPipeline pipe, boolean bDepthOff, MaterialGridBase mat) 
	{
		pipe.PushPipelineState(mat);
		mat.setDepthState(bDepthOff);
		
		mat.DrawColor(gridColor);
		//mesh.draw(gl);
    
		vaoDrawer.Draw(pipe);

		pipe.PopPipelineState(mat);
	}

	private void DrawGridOrtho(GraphicsPipeline pipe, boolean bDepthOff, JoglCamera c) 
	{
		pipe.PushPipelineState(matGOrtho);
		matGOrtho.setDepthState(bDepthOff);
		matGOrtho.ApplyCamera(c);
		
		double w = 500.0 * c.getCameraLens().getAspect();
		double h = 500.0;
		
		//draw screen rectangle for shader
		pipe.transform.Push();
		pipe.transform.LoadIdentity();
		
		if (vaoDrawerOrtho == null)
		{
			JoglMesh mOrtho = new JoglMesh(GL2.GL_QUADS);
			mOrtho.setVerts(GenerateGeometryOrtho(w,h));
			mOrtho.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(mOrtho.getNumVerts()));
			
			vaoDrawerOrtho = new VAODrawer();
			vaoDrawerOrtho.LoadMesh(pipe, mOrtho);
		}
		
		vaoDrawerOrtho.Draw(pipe);
		
		pipe.PopPipelineState(matGOrtho);

		pipe.transform.Pop();
	}
	
	public void Dispose()
	{
		if (vaoDrawer != null)
		{
			vaoDrawer.Dispose();
			vaoDrawer = null;
		}
		if (mesh != null)
		{
			mesh = null;
		}
	}

}
