package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.HashMap;
import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.processprofiler3d.compviz.TransitionDataMapDouble;
import org.processmining.processprofiler3d.compviz.TransitionPair;
import org.processmining.processprofiler3d.compviz.jogl.JoglSelection;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENSelectModelElement;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialCull;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet.ISubNetLayoutListener;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisGroup;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisMagnetLine;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisMagnetLineBase;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisMagnetLineGroupBase;
import org.processmining.processprofiler3d.compviz.methods.EdgeDataVisualisationMethodCollection;
import org.processmining.processprofiler3d.compviz.methods.EdgeDataVisualizationMethod;
import org.processmining.processprofiler3d.psen.PetriNetIndexed;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JSOPSENEdgeVisualizationLayer extends JSOPSENVisualizationLayer implements ISubNetLayoutListener
{
	private static MaterialCull matC = new MaterialCull();

	private PetriNetIndexed pnetIdx;
	
	public EdgeDataVisualizationMethod edgeVisMethod = EdgeDataVisualisationMethodCollection.EDVM_None;
	private TransitionDataMapDouble map;
	
	public HashMap<String, JSODataVisMagnetLineBase> mlines = new HashMap<String, JSODataVisMagnetLineBase>();
	public HashMap<String, JSODataVisMagnetLineGroupBase> mgroups = new HashMap<String, JSODataVisMagnetLineGroupBase>();
	
	//private float DoubleRangeFactor = 0.0f;
	public double topBottomCutoff = -5;
	public double magnetLineArcHeight = 50.0;
	
	public void SetVisualization(EdgeDataVisualizationMethod mode, TransitionDataMapDouble map, boolean bNoVisibleClassifiers)
	{	
		//this.DoubleRangeFactor = DoubleRangeFactor;
		this.edgeVisMethod = mode;
		this.map = map;

		if (bNoVisibleClassifiers)
		{
			//clear existing magnet lines
			for (JSODataVisMagnetLineGroupBase group : mgroups.values())
			{
				group.Dispose();
			}
			mgroups.clear();
		}
		else
		{
			ResetVisualization();
		}
		
		bMagnetLinesMoved = true;
	}
	
	private VisualisationDataPipeline getDataPipeline()
	{
		VisualisationDataPipeline tdp = new VisualisationDataPipeline();
		tdp.gSettings = gSettings;
		tdp.map1 = map;
		tdp.cohortColors = cohortColors;
		//tdp.scaleMode1 = scaleMode1;
		//tdp.scaleMode2 = scaleMode2;
		
		return tdp;
	}
	
	public void ResetVisualization()
	{
		edgeVisMethod.InitVisualisations(this, snet, getDataPipeline());
		
		bMagnetLinesMoved = true;
	}
	
	public JSOPSENEdgeVisualizationLayer(JoglScene scene, PetriNetIndexed pnetIdx, JSOSubNet snet, JSOCohortColorManager jccm, JoglPanel p)
	{
		super(scene, snet, jccm, p);

		this.pnetIdx = pnetIdx;
		
		BuildScene();
		
		snet.layoutListeners.add(this);
	}
	
	public Vector<JSOPnTransitionPair> getVisElements(IPerformanceStatisticsEnrichedNet pwait)
	{
		Vector<JSOPnTransitionPair> lst = new Vector<JSOPnTransitionPair>();
		for (String localIDs : pwait.getTransitionsInclNone()) //get all transitions, even those not currently in a cohort
		{
			PetrinetNode source = pnetIdx.getNode(TransitionPair.getSourceId(localIDs));
			PetrinetNode target = pnetIdx.getNode(TransitionPair.getTargetId(localIDs));
			
			JSOPnTransition sourceNode = snet.getTransitionFor(source);
			JSOPnTransition targetNode = snet.getTransitionFor(target);

			if (sourceNode != null && targetNode != null)
			{
				JSOPnTransitionPair tp = new JSOPnTransitionPair();
				tp.localIDs = localIDs;
				tp.sourceNode = sourceNode;
				tp.targetNode = targetNode;
				
				lst.add(tp);
			}
		}
		return lst;
	}
	
	@Override
	public void BuildScene()
	{	
		elements.clear();
		lastSelectionState = -1; //reset selection state check

		VisualisationDataPipeline tdp = getDataPipeline();
		
		edgeVisMethod.VisualizeData(this, tdp);
		
		boundingBox = UpdateAABB();
	}
	
	private JoglAABB UpdateAABB()
	{
		double minx = Double.POSITIVE_INFINITY;
		double miny = Double.POSITIVE_INFINITY;
		double minz = Double.POSITIVE_INFINITY;
		double maxx = Double.NEGATIVE_INFINITY;
		double maxy = Double.NEGATIVE_INFINITY;
		double maxz = Double.NEGATIVE_INFINITY;

		for (JSODataVisMagnetLineBase jso : getMagnetLines())
		{
			JoglAABB aabb = jso.getAABB();
			
			minx = Math.min(minx, aabb.getMin().getX());
			miny = Math.min(miny, aabb.getMin().getY());
			minz = Math.min(minz, aabb.getMin().getZ());
			
			maxx = Math.max(maxx, aabb.getMax().getX());
			maxy = Math.max(maxy, aabb.getMax().getY());
			maxz = Math.max(maxz, aabb.getMax().getZ());
		}
		
		JoglVectord3 vmin = new JoglVectord3(minx, miny, minz);
		JoglVectord3 vmax = new JoglVectord3(maxx, maxy, maxz);

		return new JoglAABB(vmin, vmax);
	}
	
	public Vector<JSODataVisMagnetLineBase> getMagnetLines()
	{
		Vector<JSODataVisMagnetLineBase> lines = new Vector<JSODataVisMagnetLineBase>();
		for (JSODataVisGroup vjso : elements)
		{
			for (JoglSceneObject jso : vjso.getElements())
			{
				if (jso instanceof JSODataVisMagnetLineBase)
				{
					lines.add((JSODataVisMagnetLineBase)jso);
				}
			}
		}
		return lines;
	}
	
	public Vector<JSODataVisMagnetLineGroupBase> getMagnetLineGroups()
	{
		Vector<JSODataVisMagnetLineGroupBase> lines = new Vector<JSODataVisMagnetLineGroupBase>();
		for (JSODataVisGroup vjso : elements)
		{
			if (vjso instanceof JSODataVisMagnetLineGroupBase)
			{
				lines.add((JSODataVisMagnetLineGroupBase)vjso);
			}
		}
		return lines;
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		JoglCamera c = pipe.cam;
		double camY = c.GetPosition().getY();
		
		DeterminVisibleElements(scene);
		
		DrawTop(pipe, (camY < topBottomCutoff));
		DrawBottom(pipe, (camY > topBottomCutoff));
	}

	public void setMagnetLinesUpdate()
	{
		bMagnetLinesMoved = true;
	}
	
	public boolean HasVisibleElements()
	{
		boolean bHasVisibleElements = false;

		for (JSODataVisGroup group : viselements)
		{
			if (!group.isEmpty())
			{
				bHasVisibleElements = true;
				break;
			}
		}
		
		return bHasVisibleElements;
	}
	
	private long lastSelectionState = -1;
	private boolean bMagnetLinesMoved = true;
	private Vector<JoglSceneObject> selectionObjs = new Vector<JoglSceneObject>();
	
	public void DeterminVisibleElements(JoglScene scene)
	{
		JIPSENSelectModelElement sme = iman.findInteraction(JIPSENSelectModelElement.class);
		JoglSelection selection = sme.getSelection();

		boolean bNewSelectionState = false;
		long newSelectionState = selection.getSelectionState();		
		if (newSelectionState != lastSelectionState)
		{
			bNewSelectionState = true;
		}
		
		lastSelectionState = newSelectionState;
		
		if (!selection.isEmpty())
		{
			if (bMagnetLinesMoved || bNewSelectionState)
			{	
				ClearSelectionObjs();
				
				//create selection magnetlines
				for (JoglSceneObject jso : selection.getElements())
				{
					if (jso instanceof JSODataVisMagnetLineBase)
					{
						JSODataVisMagnetLineBase line = getCurrentInstance((JSODataVisMagnetLineBase)jso);
						if (line != null)
						{
							line.Highlight();
							selectionObjs.add(line);
						}
					}
				}
				bMagnetLinesMoved = false;
			}
		}
		else
		{
			ClearSelectionObjs();
		}
		
		if (!bNewSelectionState)
		{
			return;
		}
		
		viselements.clear();
		
		if (selection.isEmpty())
		{
			viselements.addAll(elements);
		}
		else
		{	
			Vector<String> selectedRefs = new Vector<String>();
			for (JoglSceneObject jso : selection.getElements())
			{
				if (jso instanceof JSOPnTransition)
				{
					String ref = ((JSOPnTransition)jso).orgTransition.getLocalID().toString();
					selectedRefs.add(ref);
				}
				else if (jso instanceof JSODataVisMagnetLine)
				{
					selectedRefs.add(((JSODataVisMagnetLine)jso).getParentRef());
				}
			}
			
			//for each group of visualization elements
			for (int i = 0; i < elements.size(); i++)
			{
				JSODataVisGroup group = elements.get(i);
				if (group instanceof JSODataVisMagnetLineGroupBase)
				{
					JSODataVisMagnetLineGroupBase mlgroup = (JSODataVisMagnetLineGroupBase)group;
					if ((selectedRefs.contains(mlgroup.getSourceRef()) && selectedRefs.contains(mlgroup.getTargetRef())) || selectedRefs.contains(mlgroup.parentRef))
					{
						viselements.add(mlgroup);
					}
				}
				else
				{
					viselements.add(group);
				}
			}
		}
		
	}
	
	private void ClearSelectionObjs()
	{
		for (JoglSceneObject jso : selectionObjs)
		{
			if (jso instanceof JSODataVisMagnetLineBase)
			{
				JSODataVisMagnetLineBase line = getCurrentInstance((JSODataVisMagnetLineBase)jso);
				if (line != null)
				{
					line.Unhighlight();
				}
			}
		}
		selectionObjs.clear();
	}
	
	private JSODataVisMagnetLineBase getCurrentInstance(JSODataVisMagnetLineBase oldLine)
	{
		if (mlines.containsKey(oldLine.getParentRef() + oldLine.getCohortRef()))
		{
			return mlines.get(oldLine.getParentRef() + oldLine.getCohortRef());
		}
		return null;
	}

	public void DrawTop(GraphicsPipeline pipe, boolean bMasked)
	{
		pipe.PushPipelineState(matC);
		//pipe.SetCullState(GraphicsPipeline.CullState.CullBack);
		
		//for each group of visualization elements
		for (int i = 0; i < viselements.size(); i++)
		{
			JSODataVisGroup group = viselements.get(i);
			if (!group.isEmpty())
			{
				if (group instanceof JSODataVisMagnetLineGroupBase)
				{
					JSODataVisMagnetLineGroupBase mlgroup = (JSODataVisMagnetLineGroupBase)group;
					mlgroup.DrawTop(pipe, bMasked);
				}
				else
				{
					group.DrawObject(pipe);
				}
			}
		}
		
		//pipe.SetCullState(GraphicsPipeline.CullState.CullNone);
		pipe.PopPipelineState(matC);
	}

	public void DrawBottom(GraphicsPipeline pipe, boolean bMasked)
	{
		pipe.PushPipelineState(matC);
		//pipe.SetCullState(GraphicsPipeline.CullState.CullBack);
		
		//for each group of visualization elements
		for (int i = 0; i < viselements.size(); i++)
		{
			JSODataVisGroup group = viselements.get(i);
			if (!group.isEmpty())
			{
				if (group instanceof JSODataVisMagnetLineGroupBase)
				{
					JSODataVisMagnetLineGroupBase mlgroup = (JSODataVisMagnetLineGroupBase)group;
					mlgroup.DrawBottom(pipe, bMasked);
				}
				else
				{
					group.DrawObject(pipe);
				}
			}
		}
		
		//pipe.SetCullState(GraphicsPipeline.CullState.CullNone);
		pipe.PopPipelineState(matC);
	}

	@Override
	public void Dispose()
	{
		for (JSODataVisMagnetLineGroupBase group : mgroups.values())
		{
			group.Dispose();
		}
	}

	public void OnLayoutChanged(JSOSubNet snet)
	{
		//rebuild vis once layout changed
		ResetVisualization();
		BuildScene();
	}
}
