package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.TransitionDataMapDouble;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglColoredMesh;
import org.processmining.processprofiler3d.compviz.jogl.JoglSelection;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIPSENSelectModelElement;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet.ISubNetLayoutListener;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisGroup;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisGroupBarChartBase;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisTriangleGroup;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderBarTest;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderTrajectory;
import org.processmining.processprofiler3d.compviz.methods.TransitionDataVisualisationMethodCollection;
import org.processmining.processprofiler3d.compviz.methods.TransitionDataVisualizationMethod;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.opengl.GL2;

public class JSOPSENTransitionVisualizationLayer extends JSOPSENVisualizationLayer implements ISubNetLayoutListener
{
	private TransitionDataVisualizationMethod transitionVisMethod = TransitionDataVisualisationMethodCollection.TDVM_None;

	private TransitionDataMapDouble map1 = null;
	private TransitionDataMapDouble map2 = null;
	
	private String scaleMode1 = "Linear";
	private String scaleMode2 = "Linear";
	
	public String doubleCodeMode = "None";
	//private boolean bVerbose = false;
	
	private IMeshDrawer vao = null;
	private JoglShader shad = null;
	private JoglShader shadDCNone = null;
	private JoglShader shadDCStripes = null;
	private JoglColoredMesh mymesh = null;
	
	public String getScaleMode1()
	{
		return scaleMode1;
	}
	
	public void setScaleMode1(String mode)
	{
		if (!mode.equals(scaleMode1))
		{
			scaleMode1 = mode;
			BuildScene();
		}
	}
	
	public String getScaleMode2()
	{
		return scaleMode2;
	}
	
	public void setScaleMode2(String mode)
	{
		if (!mode.equals(scaleMode2))
		{
			scaleMode2 = mode;
			BuildScene();
		}
	}
	
	public Vector<JSOPnTransition> getNetElements()
	{
		Vector<JSOPnTransition> lst = new Vector<JSOPnTransition>();
		
		return lst;
	}
	
	public void SetVisualization(TransitionDataVisualizationMethod mode, TransitionDataMapDouble m1, TransitionDataMapDouble m2)
	{
		transitionVisMethod = mode;
		map1 = m1;
		map2 = m2;
	}
	
	public JSOPSENTransitionVisualizationLayer(JoglInteractionManager iman, JoglScene scene, JSOSubNet snet, JSOCohortColorManager jccm, JoglPanel p)
	{	
		super(scene, snet, jccm, p);
		
		BuildScene();
		
		snet.layoutListeners.add(this);
	}
	
	private VisualisationDataPipeline getDataPipeline()
	{
		VisualisationDataPipeline tdp = new VisualisationDataPipeline();
		tdp.gSettings = gSettings;
		tdp.map1 = map1;
		tdp.map2 = map2;
		tdp.cohortColors = cohortColors;
		tdp.scaleMode1 = scaleMode1;
		tdp.scaleMode2 = scaleMode2;
		
		return tdp;
	}
	
	@Override
	public void BuildScene()
	{	
		//dispose VAOs properly
		for (JSODataVisGroup vg : elements)
		{
			if (vg instanceof JSODataVisTriangleGroup)
			{
				JSODataVisTriangleGroup dvbc = (JSODataVisTriangleGroup)vg;
				dvbc.Dispose();
			}
			else if (vg instanceof JSODataVisGroupBarChartBase)
			{
				JSODataVisGroupBarChartBase dvbc = (JSODataVisGroupBarChartBase)vg;
				dvbc.Dispose();
			}
		}
		elements.clear();
		lastSelectionState = -1; //reset selection state check
		
		if (vao != null)
		{
			vao.Dispose();
			vao = null;
		}
		
		VisualisationDataPipeline tdp = getDataPipeline();
		
		//Visualize transition data
		transitionVisMethod.VisualizeData(this, tdp);	
	}

	public void CreateQuadMesh()
	{
		JoglColoredMesh jcm = new JoglColoredMesh(GL2.GL_QUADS);
		for (JSODataVisGroup vg : elements)
		{
			if (vg instanceof JSODataVisGroupBarChartBase)
			{
				JSODataVisGroupBarChartBase barG = (JSODataVisGroupBarChartBase)vg;
				barG.LoadMesh(jcm);
			}
		}
		if (jcm.getNumVerts() > 0)
		{
			jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jcm.getNumVerts()));
			mymesh = jcm;
		}
		else
		{
			mymesh = null;
		}
	}

	public void CreateTriMesh()
	{
		JoglColoredMesh jcm = new JoglColoredMesh(GL2.GL_QUADS);
		for (JSODataVisGroup vg : elements)
		{
			if (vg instanceof JSODataVisTriangleGroup)
			{
				JSODataVisTriangleGroup triG = (JSODataVisTriangleGroup)vg;
				triG.LoadMesh(jcm);
			}
		}
		if (jcm.getNumVerts() > 0)
		{
			jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(jcm.getNumVerts()));
			mymesh = jcm;
		}
		else
		{
			mymesh = null;
		}
	}

	private long lastSelectionState = -1;
	
	public boolean HasVisibleElements()
	{
		boolean bHasVisibleElements = false;

		for (JSODataVisGroup group : viselements)
		{
			if (!group.isEmpty())
			{
				bHasVisibleElements = true;
				break;
			}
		}
		
		return bHasVisibleElements;
	}
	
	public void DeterminVisibleElements(JoglScene scene)
	{	
		JIPSENSelectModelElement sme = iman.findInteraction(JIPSENSelectModelElement.class);
		JoglSelection selection = sme.getSelection();

		long newSelectionState = selection.getSelectionState();
		if (newSelectionState == lastSelectionState)
		{
			return;
		}
		
		lastSelectionState = newSelectionState;
		
		viselements.clear();
		
		if (selection.isEmpty())
		{
			viselements.addAll(elements);
		}
		else
		{
			Vector<String> selectedRefs = new Vector<String>();
			for (JoglSceneObject jso : selection.getElements())
			{
				if (jso instanceof JSOPnTransition)
				{
					String ref = ((JSOPnTransition)jso).orgTransition.getLocalID().toString();
					selectedRefs.add(ref);
				}
			}
			
			for (int i = 0; i < elements.size(); i++)
			{
				JSODataVisGroup group = elements.get(i);
				if (selectedRefs.contains(group.parentRef))
				{
					viselements.add(group);
				}
			}
		}
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{		
		//DeterminVisibleElements(scene);
		
		if (vao == null && mymesh != null)
		{
			vao = new VAODrawer();
			vao.LoadMesh(pipe, mymesh);	
		}
		if (shadDCNone == null)
		{
			shadDCNone = new JoglShaderTrajectory();
			shadDCNone.Create(pipe);
		}
		if (shadDCStripes == null)
		{
			shadDCStripes = new JoglShaderBarTest();
			shadDCStripes.Create(pipe);		
		}
		
		if (doubleCodeMode.equals("Stripes"))
		{
			shad = shadDCStripes;
		}
		else
		{
			shad = shadDCNone;
		}
		
		if (vao != null)
		{
			shad.Bind(pipe);
			vao.Bind(pipe);

			for (JSODataVisGroup vg : viselements)
			{
				if (vg instanceof JSODataVisTriangleGroup)
				{
					JSODataVisTriangleGroup triG = (JSODataVisTriangleGroup)vg;
					triG.DrawObjectFast(pipe, vao);
				}
				else if (vg instanceof JSODataVisGroupBarChartBase)
				{
					JSODataVisGroupBarChartBase barG = (JSODataVisGroupBarChartBase)vg;
					barG.DrawObjectFast(pipe, vao);
				}
			}
			
			vao.Unbind(pipe);
			shad.Unbind(pipe);
		}
		else
		{
			for (int i = 0; i < viselements.size(); i++)
			{
				JSODataVisGroup group = viselements.get(i);
				group.DrawObject(pipe);
			}
		}
	}
	
	public void DrawTop(GraphicsPipeline pipe, boolean bMasked)
	{
		DrawObject(pipe);
	}
	
	public void DrawBottom(GraphicsPipeline pipe, boolean bMasked)
	{
		
	}

	public void OnLayoutChanged(JSOSubNet snet)
	{
		//rebuild vis once layout changed
		BuildScene();
	}
}
