package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.awt.Dimension;
import java.util.Vector;

import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.LocalNodeID;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.ExpandableSubNet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.compviz.PetrinetSimplifier;
import org.processmining.processprofiler3d.compviz.SubNetDataSource;
import org.processmining.processprofiler3d.compviz.scenemanagement.BSPTree;
import org.processmining.processprofiler3d.psen.fileio.obj.OBJWriter;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JSOPetriNet extends JoglSceneObject implements IJSOModel
{
	private JoglPanel panel;
	private PetriNetDataSource pnds;
	private GraphLayoutConnection layout;
	
	private JoglAABB boundingBox;
	
	private int simplificationLevel = 0;
	private PetrinetSimplifier ps = null;
	private JSOSubNet sub = null;
	private Vector<JSOSubNet> subnets = new Vector<JSOSubNet>();

	protected BSPTree sceneTree;
	
	public void setBSPTree(BSPTree bspTree)
	{
		sceneTree = bspTree;
	}
	
	public boolean getIsVisible()
	{
		return true;
	}
	
	public int getMaxLevel()
	{
		int mlevel = 0;
		for (JSOSubNet snet : getAllSubNets())
		{
			mlevel = Math.max(mlevel, snet.getLevel());
		}
		return mlevel;
	}
	
	public JoglAABB getAABB()
	{
		return boundingBox;
	}
	
	public INetDataSource getPetriNet()
	{
		return pnds;
	}
	
	public int getSimplificationLevel()
	{
		return simplificationLevel;
	}
	
	public void setSimplificationLevel(int lvl)
	{
		simplificationLevel = lvl;
	}
	
	public JoglVectord3 getOffset()
	{
		return new JoglVectord3(0,0,0);
	}
	
	public double getScale()
	{
		return 1.0;
	}
	
	public JSOSubNet getRootSubNet()
	{
		return sub;
	}
	
	public Vector<JSOSubNet> getAllSubNets()
	{
		Vector<JSOSubNet> sns = new Vector<JSOSubNet>();
		sns.add(sub);
		//sns.addAll(subnets);
		Vector<JSOSubNet> cs = new Vector<JSOSubNet>();
		cs.addAll(subnets);
		while (!cs.isEmpty())
		{
			JSOSubNet sn = cs.firstElement();
			cs.removeElementAt(0);
			cs.addAll(sn.getChildSubNets());
			
			sns.add(sn);
		}
		return sns;
	}
	
	public Vector<JSOSubNet> getChildSubNets()
	{
		return subnets;
	}
	
	public JoglTexture getMaskTexture()
	{
		return sub.getMaskTexture();
	}
	
	//public JSOPetriNet(PetrinetGraph pnet, GraphLayoutConnection layoutConn)
	public JSOPetriNet(JoglPanel p, JoglScene scene, PetriNetDataSource pnds, GraphLayoutConnection layoutConn)
	{
		this.panel = p;
		//this.pnet = pnet;
		//this.pnds = new PetriNetDataSource(pnet);
		this.pnds = pnds;
		layout = layoutConn;
		
		BuildScene(scene);
	}
	
	public void BuildScene(JoglScene scene)
	{	
		ps = new PetrinetSimplifier(pnds, layout, simplificationLevel);
		BuildFrom(scene, ps);
		
		for (JSOSubNet sn : subnets)
		{
			sn.BuildScene(scene);
		}
		
		JoglVectord2 netSize = GetPetrinetSize();
		System.out.println("PetriNet size: " + netSize.getX() + "/" + netSize.getY());
		
		boundingBox = UpdateAABB();
	}

	public static JoglVectord3 subnetOffset = new JoglVectord3(0, 500, 0);
	
	private void BuildFrom(JoglScene scene, PetrinetSimplifier simp) 
	{	
		INetDataSource source = pnds;
		sub = new JSOSubNet(panel, scene, source, layout, null);
		
		for (SubNetDataSource esn : source.getSubnets())
		{		
			JoglVectord3 subnetOrigin = new JoglVectord3(0, 0, 0);
			Transition tx = source.getTransitionForSubnet(esn);
			if (tx != null)
			{
				//subnetOffset = JoglVectord3.Add(subnetOffset, ps.getPosition(tx));
				subnetOrigin = ps.getPosition(tx);
			}
			JSOPnTransition parentTrans = getTransitionFor(tx);
			JSOSubNet subN = new JSOSubNet(panel, scene, esn, layout, parentTrans);
			subN.setIsVisible(false);
			JoglVectord3 center = subN.GetPetrinetCenter();
			//subN.setOrigin(subnetOrigin);
			subN.setOffset(JoglVectord3.Add(subnetOrigin, JoglVectord3.Add(subnetOffset, center)));
			subnets.add(subN);
		}
		
		sub.setChildSubNets(subnets);
	}
	
	public JoglAABB UpdateAABB()
	{
		double minx = Double.POSITIVE_INFINITY;
		double miny = Double.POSITIVE_INFINITY;
		double minz = Double.POSITIVE_INFINITY;
		double maxx = Double.NEGATIVE_INFINITY;
		double maxy = Double.NEGATIVE_INFINITY;
		double maxz = Double.NEGATIVE_INFINITY;

		for (JSOPnTransition jso : getTransitions())
		{
			JoglAABB aabb = jso.getAABB();
			
			minx = Math.min(minx, aabb.getMin().getX());
			miny = Math.min(miny, aabb.getMin().getY());
			minz = Math.min(minz, aabb.getMin().getZ());
			
			maxx = Math.max(maxx, aabb.getMax().getX());
			maxy = Math.max(maxy, aabb.getMax().getY());
			maxz = Math.max(maxz, aabb.getMax().getZ());
		}
		
		for (JSOPnPlace jso : getPlaces())
		{
			JoglAABB aabb = jso.getAABB();
			
			minx = Math.min(minx, aabb.getMin().getX());
			miny = Math.min(miny, aabb.getMin().getY());
			minz = Math.min(minz, aabb.getMin().getZ());
			
			maxx = Math.max(maxx, aabb.getMax().getX());
			maxy = Math.max(maxy, aabb.getMax().getY());
			maxz = Math.max(maxz, aabb.getMax().getZ());
		}
		
		JoglVectord3 vmin = new JoglVectord3(minx, miny, minz);
		JoglVectord3 vmax = new JoglVectord3(maxx, maxy, maxz);

		return new JoglAABB(vmin, vmax);
	}
	
	public Vector<JoglSceneObject> getElements()
	{
		Vector<JoglSceneObject> elements = new Vector<JoglSceneObject>();
		
		elements.addAll(sub.getElements());
		
		for (JSOSubNet sn : subnets)
		{
			elements.addAll(sn.getElements());
		}
		
		return elements;
	}
	
	public JSOPnNode getElementFor(PetrinetNode pn)
	{
		JSOPnNode ele = null;
		if (pn instanceof Place)
		{
			ele = getPlaceFor(pn);
		}
		else if (pn instanceof Transition)
		{
			ele = getTransitionFor(pn);
		}
		
		return ele;
	}
	
	public JSOPnPlace getPlaceFor(PetrinetNode pn)
	{
		for (JoglSceneObject jso : getElements())
		{
			if (jso instanceof JSOPnPlace)
			{
				JSOPnPlace p = (JSOPnPlace)jso;
				if (p.orgPlace.getLocalID() == pn.getLocalID())
				{
					return p;
				}
			}
		}
		return null;
	}
	
	public JSOPnTransition getTransitionFor(PetrinetNode pn)
	{
		for (JoglSceneObject jso : getElements())
		{
			if (jso instanceof JSOPnTransition)
			{
				JSOPnTransition p = (JSOPnTransition)jso;
				if (p.orgTransition == pn)
				{
					return p;
				}
			}
		}
		return null;
	}
	
	public JSOPnTransition getTransitionFor(LocalNodeID id)
	{
		for (JoglSceneObject jso : getElements())
		{
			if (jso instanceof JSOPnTransition)
			{
				JSOPnTransition p = (JSOPnTransition)jso;
				if (p.orgTransition.getLocalID().toString().equals(id.toString()))
				{
					return p;
				}
			}
		}
		return null;
	}

	public Transition getTransitionForSubnet(ExpandableSubNet esn)
	{
		String esnId = esn.getLabel();
		for (Transition tx : ps.getTransitions())
		{
			if (tx.getAttributeMap().containsKey("PNMLSubnetId"))
			{
				if (esnId.equals(tx.getAttributeMap().get("PNMLSubnetId")))
				{
					return tx;
				}
			}
		}
		return null;
	}
	
	public Vector<JSOPnPlace> getPlaces()
	{
		Vector<JSOPnPlace> lst = new Vector<JSOPnPlace>();
		
		for (JoglSceneObject jso : getElements())
		{
			if (jso instanceof JSOPnPlace)
			{
				lst.add((JSOPnPlace)jso);
			}
		}
		
		return lst;
	}
	
	public Vector<JSOPnTransition> getTransitions()
	{
		Vector<JSOPnTransition> lst = new Vector<JSOPnTransition>();
		
		for (JoglSceneObject jso : getElements())
		{
			if (jso instanceof JSOPnTransition)
			{
				lst.add((JSOPnTransition)jso);
			}
		}
		
		return lst;
	}
	
	public Vector<JSOPnEdge> getEdges()
	{
		Vector<JSOPnEdge> lst = new Vector<JSOPnEdge>();

		for (JoglSceneObject jso : getElements())
		{
			if (jso instanceof JSOPnEdge)
			{
				lst.add((JSOPnEdge)jso);
			}
		}
		
		return lst;
	}
	
	public JoglVectord3 GetPetrinetCenter()
	{
		double minPosX = 0;
		double maxPosX = 0;
		double minPosY = 0;
		double maxPosY = 0;
		
		for (Place p : pnds.getPlaces()) 
		{
			JoglVectord3 pos = ps.getPosition(p);
			Dimension size = ps.getSize(p);
			
			minPosX = Math.min(minPosX, pos.getX() - (size.getWidth() * 0.5));
			maxPosX = Math.max(maxPosX, pos.getX() + (size.getWidth() * 0.5));
			minPosY = Math.min(minPosY, pos.getZ() - (size.getHeight() * 0.5));
			maxPosY = Math.max(maxPosY, pos.getZ() + (size.getHeight() * 0.5));
		}
		
		for (Transition t : pnds.getTransitions()) 
		{
			JoglVectord3 pos = ps.getPosition(t);
			Dimension size = ps.getSize(t);
			
			minPosX = Math.min(minPosX, pos.getX() - (size.getWidth() * 0.5));
			maxPosX = Math.max(maxPosX, pos.getX() + (size.getWidth() * 0.5));
			minPosY = Math.min(minPosY, pos.getZ() - (size.getHeight() * 0.5));
			maxPosY = Math.max(maxPosY, pos.getZ() + (size.getHeight() * 0.5));		
		}
		
		JoglVectord3 centerPos = new JoglVectord3(minPosX + ((maxPosX - minPosX) * 0.5), 0, minPosY + ((maxPosY - minPosY) * 0.5));
		
		return centerPos;
	}
	
	public JoglVectord2 GetPetrinetSize()
	{
		double minPosX = 0;
		double maxPosX = 0;
		double minPosY = 0;
		double maxPosY = 0;
		
		for (Place p : pnds.getPlaces()) 
		{
			JoglVectord3 pos = ps.getPosition(p);
			Dimension size = ps.getSize(p);
			
			minPosX = Math.min(minPosX, pos.getX() - (size.getWidth() * 0.5));
			maxPosX = Math.max(maxPosX, pos.getX() + (size.getWidth() * 0.5));
			minPosY = Math.min(minPosY, pos.getZ() - (size.getHeight() * 0.5));
			maxPosY = Math.max(maxPosY, pos.getZ() + (size.getHeight() * 0.5));
		}
		
		for (Transition t : pnds.getTransitions()) 
		{
			JoglVectord3 pos = ps.getPosition(t);
			Dimension size = ps.getSize(t);
			
			minPosX = Math.min(minPosX, pos.getX() - (size.getWidth() * 0.5));
			maxPosX = Math.max(maxPosX, pos.getX() + (size.getWidth() * 0.5));
			minPosY = Math.min(minPosY, pos.getZ() - (size.getHeight() * 0.5));
			maxPosY = Math.max(maxPosY, pos.getZ() + (size.getHeight() * 0.5));		
		}
		
		JoglVectord2 centerPos = new JoglVectord2(maxPosX - minPosX, maxPosY - minPosY);
		
		return centerPos;
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		//draw each subnet in order of BSPTree traversal
		Vector<JSOSubNet> nets = new Vector<JSOSubNet>();
		if (sceneTree != null)
		{
			sceneTree.Traverse(pipe.cam, nets);
			
			for (JSOSubNet snet : nets)
			{
				snet.DrawObject(pipe);
			}
		}
		else
		{
			System.out.println("No BSP tree available for scene! Can not guarantee correct draw order!");
		}
		//sub.DrawObject(pipe);
	}
	
	public void Dispose(GraphicsPipeline pipe)
	{
		sub.Dispose(pipe);
		
		for (JSOSubNet sn : subnets)
		{
			sn.Dispose(pipe);
		}
	}
	
	public void Export(JoglScene scene, OBJWriter out)
	{
        sub.Export(scene, out);
        for (JSOSubNet snet : subnets)
        {
        	if (snet.getIsVisible())
        	{
        		snet.Export(scene, out);
        	}
        }
        
	}
}
