package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.HashMap;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglMesh;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.JoglTextRenderer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.JoglTextRendererDF;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.interactions.JIViewManagerModel;
import org.processmining.processprofiler3d.compviz.jogl.materials.IMaterialPetrinetElementBatchLabels;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialPetrinetElementBatch;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialPetrinetElementBatchLabels;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialPetrinetElementBatchLabelsDF;
import org.processmining.processprofiler3d.psen.fileio.obj.OBJWriter;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;

public class JSOPetriNetBatch extends JoglSceneObject
{
	private static MaterialPetrinetElementBatch matPE = new MaterialPetrinetElementBatch();
	private static MaterialPetrinetElementBatch matPEMask = new MaterialPetrinetElementBatch();
	private static MaterialPetrinetElementBatchLabels matLabels = new MaterialPetrinetElementBatchLabels();
	private static MaterialPetrinetElementBatchLabelsDF matLabelsDF = new MaterialPetrinetElementBatchLabelsDF();
	
	private JoglPanel panel;
	private IJSOModel pnet = null;
	
	private JoglMesh modelBatchMesh = null;
	private JoglTexturedMesh labelBatchMesh = null;
	
	private HashMap<JoglSceneObject, Integer> startMap = new HashMap<JoglSceneObject, Integer>();
	private HashMap<JoglSceneObject, Integer> endMap = new HashMap<JoglSceneObject, Integer>();

	public JSOPetriNetBatch(JoglPanel p, IJSOModel pnet)
	{
		this.panel = p;
		this.pnet = pnet;
	}
	
	public void BuildScene()
	{
		BatchFrom();
	}

	private int whiteEnd = 0;
	
	private void BatchFrom() 
	{
		if (vaoDrawerModel != null)
		{
			vaoDrawerModel.Dispose();
			vaoDrawerModel = null;
		}
		if (vaoDrawerLabel != null)
		{
			vaoDrawerLabel.Dispose();
			vaoDrawerLabel = null;
		}
		
		//calculate number of vertices expected to avoid memory reallocations
		int reqCapacity = 0;
		int lreqCapacity = 0;
		for (JoglSceneObject jso : pnet.getElements())
		{
			if (jso instanceof JSOPnPlace)
			{
				JSOPnPlace p = (JSOPnPlace)jso;
				reqCapacity = reqCapacity + p.getMesh().getNumVerts();
			}
			else if (jso instanceof JSOPnTransition)
			{
				JSOPnTransition t = (JSOPnTransition)jso;
				reqCapacity = reqCapacity + t.getMesh().getNumVerts();
			}
			else if (jso instanceof JSOPnEdge)
			{
				JSOPnEdge e = (JSOPnEdge)jso;
				reqCapacity = reqCapacity + e.getMesh().getNumVerts();
			}
			else if (jso instanceof JoglTextRenderer)
			{
				JoglTextRenderer tr = (JoglTextRenderer)jso;
				lreqCapacity = lreqCapacity + tr.getMesh().getNumVerts();
			}
		}
		
		modelBatchMesh = new JoglMesh(GL.GL_TRIANGLES, reqCapacity);
		labelBatchMesh = new JoglTexturedMesh(GL2.GL_QUADS, lreqCapacity);
		startMap.clear();
		endMap.clear();

		//white bits
		for (JoglSceneObject jso : pnet.getElements())
		{
			if (jso instanceof JSOPnPlace)
			{
				JSOPnPlace p = (JSOPnPlace)jso;
				modelBatchMesh.addVerts(p.getMesh(), p.getPosition(), 0, p.BorderStart * 3);
			}
			else if (jso instanceof JSOPnTransition)
			{
				JSOPnTransition t = (JSOPnTransition)jso;
				if (!t.orgTransition.isInvisible())
				{
					modelBatchMesh.addVerts(t.getMesh(), t.getPosition(), JSOPnTransition.CenterStart * 3, JSOPnTransition.CenterEnd * 3);
				}
			}
		}
		
		whiteEnd = modelBatchMesh.getNumVerts();
		
		//black bits
		for (JoglSceneObject jso : pnet.getElements())
		{
			if (jso instanceof JSOPnPlace)
			{
				JSOPnPlace p = (JSOPnPlace)jso;
				startMap.put(p, modelBatchMesh.getNumVerts());
				modelBatchMesh.addVerts(p.getMesh(), p.getPosition(), p.BorderStart * 3, p.BorderEnd * 3);
				endMap.put(p, modelBatchMesh.getNumVerts());
			}
			else if (jso instanceof JSOPnTransition)
			{
				JSOPnTransition t = (JSOPnTransition)jso;
				startMap.put(t, modelBatchMesh.getNumVerts());
				if (t.orgTransition.isInvisible())
				{
					modelBatchMesh.addVerts(t.getMesh(), t.getPosition(), JSOPnTransition.CenterStart * 3, JSOPnTransition.CenterEnd * 3);
				}
				modelBatchMesh.addVerts(t.getMesh(), t.getPosition(), 0, JSOPnTransition.CenterStart * 3);
				endMap.put(t, modelBatchMesh.getNumVerts());
			}
			else if (jso instanceof JSOPnEdge)
			{
				JSOPnEdge e = (JSOPnEdge)jso;
				startMap.put(e, modelBatchMesh.getNumVerts());
				modelBatchMesh.addVerts(e.getMesh());
				endMap.put(e, modelBatchMesh.getNumVerts());
			}
			else if (jso instanceof JoglTextRenderer)
			{
				JoglTextRenderer tr = (JoglTextRenderer)jso;
				labelBatchMesh.addVerts(tr.getMesh(), tr.getPosition());
				labelBatchMesh.addTexCoords(tr.getMesh());
			}
			else if (jso instanceof JoglTextRendererDF)
			{
				JoglTextRendererDF tr = (JoglTextRendererDF)jso;

				JoglMatrixd4x4 m1 = JoglMatrixd4x4.CreateTranslation(tr.getPosition());
				JoglMatrixd4x4 m2 = JoglMatrixd4x4.CreateScale(1, -1, 1);
				JoglMatrixd4x4 m3 = JoglMatrixd4x4.CreateRotationX(-90.0 * 0.0174533);
				JoglMatrixd4x4 m4 = JoglMatrixd4x4.CreateRotationY(180.0 * 0.0174533);
				
				JoglMatrixd4x4 mx = JoglMatrixd4x4.Multiply(JoglMatrixd4x4.Multiply(JoglMatrixd4x4.Multiply(m1, m2), m3), m4);
			    
				labelBatchMesh.addVerts(tr.getMesh(), mx);
				labelBatchMesh.addTexCoords(tr.getMesh());
			}
		}
		
		modelBatchMesh.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(modelBatchMesh.getNumVerts()));
		labelBatchMesh.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(labelBatchMesh.getNumVerts()));
	}

	private IMeshDrawer vaoDrawerModel;
	private IMeshDrawer vaoDrawerLabel;
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		DrawModel(pipe);
		
		if (ShouldDrawLabels())
		{
			DrawLabels(pipe, matLabels);
			//DrawLabels(pipe, matLabelsDF);
		}
		//DrawModelMask(gl, target);
		
		//for debug purposes
		//JoglDrawState jds = new JoglDrawState();
		//jds.get(gl);
	}

	private void DrawModel(GraphicsPipeline pipe)
	{
		if (vaoDrawerModel == null && modelBatchMesh != null)
		{
			vaoDrawerModel = new VAODrawer();
			vaoDrawerModel.LoadMesh(pipe, modelBatchMesh);
		}

		pipe.PushPipelineState(matPE);
		
		if (vaoDrawerModel != null)
		{
			vaoDrawerModel.Bind(pipe);
			vaoDrawerModel.DrawSlim(pipe, 0, whiteEnd);
			for (int i = 0; i < pnet.getElements().size(); i++)
			{
				JoglSceneObject jso = pnet.getElements().get(i);
				if (jso instanceof JSOPnElement)
				{
					JSOPnElement ele = (JSOPnElement)jso;
					if (startMap.containsKey(jso))
					{
						int start = startMap.get(jso);
						int end = endMap.get(jso);
						JoglVectord3 color = ele.getColor();
						matPE.DrawColor(color);   // set the color of the quads
						vaoDrawerModel.DrawSlim(pipe, start, end - start);
					}
				}
			}
			vaoDrawerModel.Unbind(pipe);
		}
		else if (modelBatchMesh != null)
		{
			modelBatchMesh.draw(pipe, 0, whiteEnd);
			for (int i = 0; i < pnet.getElements().size(); i++)
			{
				JoglSceneObject jso = pnet.getElements().get(i);
				if (jso instanceof JSOPnElement)
				{
					JSOPnElement ele = (JSOPnElement)jso;
					if (startMap.containsKey(jso))
					{
						int start = startMap.get(jso);
						int end = endMap.get(jso);
						JoglVectord3 color = ele.getColor();
						matPE.DrawColor(color);   // set the color of the quads
						modelBatchMesh.draw(pipe, start, end - start);
					}
				}
			}
		}
		
		pipe.PopPipelineState(matPE);
	}
	
	public void DrawMask(GraphicsPipeline pipe)
	{
		if (vaoDrawerModel == null && modelBatchMesh != null)
		{
			vaoDrawerModel = new VAODrawer();
			vaoDrawerModel.LoadMesh(pipe, modelBatchMesh);
		}
		
		pipe.PushPipelineState(matPEMask);
		
		if (vaoDrawerModel != null)
		{
			vaoDrawerModel.Draw(pipe);
		}
		else if (modelBatchMesh != null)
		{
			modelBatchMesh.draw(pipe);
		}
		
		pipe.PopPipelineState(matPEMask);
	}

	private void DrawLabels(GraphicsPipeline pipe, IMaterialPetrinetElementBatchLabels mat) 
	{		
		if (vaoDrawerLabel == null && labelBatchMesh != null)
		{
			vaoDrawerLabel = new VAODrawer();
			vaoDrawerLabel.LoadMesh(pipe, labelBatchMesh);
		}
		
		pipe.PushPipelineState(matLabels);		
		
		if (mat.IsShaderBound() && vaoDrawerLabel != null)
		{
			vaoDrawerLabel.Draw(pipe);
		}
		else if (labelBatchMesh != null)
		{
			labelBatchMesh.draw(pipe);
		}
		
		pipe.PopPipelineState(matLabels);
	}

	private static int exportNum = 0;
	
	public void Export(JoglScene scene, OBJWriter out)
	{
		double[] vx = modelBatchMesh.getVerts();

		JoglVectord3 off = JoglVectord3.Multiply(pnet.getOffset(), pnet.getScale());
		
		out.WriteNewObject("Test White" + exportNum);
		Vector<Integer> verts = new Vector<Integer>();
		Vector<Integer> texcoords = new Vector<Integer>();
		int j = 0;
		for (int i = 0; i < whiteEnd; i++)
		{
			JoglVectord3 vt = new JoglVectord3(vx[j], vx[j+1], vx[j+2]);
			JoglVectord3 vtx = JoglVectord3.Add(vt, off);
			verts.add(out.WriteVertex(vtx));
			j = j + 3;
		}
		
		out.WriteUseMtl("ModelElementWhite");
		
		//write indices (starting with 1)
		for (int i = 0; i < verts.size() - 2; i = i + 3)
		{
			out.WriteFaceTri(verts.get(i), verts.get(i+1), verts.get(i+2));
		}

		out.WriteNewObject("Test Black" + exportNum);
		verts.clear();
		j = whiteEnd * 3;
		for (int i = whiteEnd; i < (vx.length / 3); i++)
		{
			JoglVectord3 vt = new JoglVectord3(vx[j], vx[j+1], vx[j+2]);
			JoglVectord3 vtx = JoglVectord3.Add(vt, off);
			verts.add(out.WriteVertex(vtx));
			j = j + 3;
		}

		out.WriteUseMtl("ModelElementBlack");
		
		//write indices (starting with 1)
		for (int i = 0; i < verts.size() - 2; i = i + 3)
		{
			out.WriteFaceTri(verts.get(i), verts.get(i+1), verts.get(i+2));
		}

		if (ShouldDrawLabels())
		{
			double[] vlx = labelBatchMesh.getVertsBuffer().getArray();
			double[] vtcx = labelBatchMesh.getTexCoords();
			
			out.WriteNewObject("Test Label" + exportNum);
			verts.clear();
			j = 0;
			int k = 0;
			for (int i = 0; i < (vlx.length / 3); i++)
			{
				JoglVectord3 vt = new JoglVectord3(vlx[j], vlx[j+1], vlx[j+2]);
				JoglVectord3 vtx = JoglVectord3.Add(vt, off);
				verts.add(out.WriteVertex(vtx));
				
				JoglVectord2 vtc = new JoglVectord2(vtcx[k], vtcx[k+1]);
				texcoords.add(out.WriteTexCoords(vtc));
				
				j = j + 3;
				k = k + 2;
			}
	
			out.WriteUseMtl("ModelElementLabel");
			
			//write indices (starting with 1)
			for (int i = 0; i < verts.size() - 3; i = i + 4)
			{
				//out.WriteFaceTri(verts.get(i), verts.get(i+1), verts.get(i+2));
				//out.WriteFaceTri(verts.get(i), verts.get(i+2), verts.get(i+3));
				out.WriteFaceTriTex(verts.get(i), verts.get(i+1), verts.get(i+2), texcoords.get(i), texcoords.get(i+1), texcoords.get(i+2));
				out.WriteFaceTriTex(verts.get(i), verts.get(i+2), verts.get(i+3), texcoords.get(i), texcoords.get(i+2), texcoords.get(i+3));
			}
		}
		else
		{
			for (JSOPnTransition t : pnet.getTransitions())
			{
				out.WriteNewObject(t.getLabel());
				JoglVectord3 vt = t.getPosition();
				JoglVectord3 vtx = JoglVectord3.Add(vt, off);
				int vertId = out.WriteVertex(vtx);
		
				out.WriteUseMtl("ModelElementFloatingLabel");
				
				//write indices (starting with 1)
				out.WriteFaceTri(vertId, vertId, vertId);
			}
		}
		exportNum++;
	}
	
	private boolean ShouldDrawLabels()
	{
		JIViewManagerModel jivm = panel.getInteractionManager().findInteraction(JIViewManagerModel.class);
		return !jivm.getShowFloatingLabels();
	}
	
	public void Dispose(GraphicsPipeline pipe)
	{
		startMap.clear();
		endMap.clear();
		
		if (vaoDrawerModel != null)
		{
			vaoDrawerModel.Dispose();
			vaoDrawerModel = null;
		}
		if (vaoDrawerLabel != null)
		{
			vaoDrawerLabel.Dispose();
			vaoDrawerModel = null;
		}
	}
}
