package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.List;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglLine3D;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglMesh;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class JSOPnEdge extends JSOPnElement 
{
	public JSOPnNode startObj;
	public JSOPnNode endObj;
	
	private List<JoglVectord3> points;
	private JoglMesh mesh;
//	private double lineWidth = 1.0;
//	private double arrowWidth = 4;
//	private double arrowLength = 8;

	public String transitionPairID;
	
	public int edgeLayoutID = -1;
	
	public List<JoglVectord3> getPoints()
	{
		return points;
	}

	public void setPoints(List<JoglVectord3> pts)
	{
		points = pts;
	}
	
	public JSOPnEdge(List<JoglVectord3> pts)
	{
		points = pts;
	}
	
	public List<JSOPnEdgeVertex> getVerts()
	{
		Vector<JSOPnEdgeVertex> verts = new Vector<JSOPnEdgeVertex>();
		
		for (int i = 1; i < (points.size() - 1); i++)
		{
			JoglVectord3 v = points.get(i);
			JSOPnEdgeVertex vert = new JSOPnEdgeVertex();
			vert.vertexId = i-1;
			vert.vertexPos = v;
			vert.myEdge = this;
			vert.setParentNet(getParentNet());
			vert.setPosition(v);
			vert.UpdateAABB();
			verts.add(vert);
		}
		
		return verts;
	}
	
	public static Vector<JoglVectord3> GenerateGeometry3D(List<JoglVectord3> pts, double depth, double lineWidth, double arrowWidth, double arrowLength) 
	{
		Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();
		
		double lineHeight = lineWidth;
		
		JoglLine3D lastLine = null;
		JoglLine3D curLine = null;

		for (int i = 0; i < pts.size() - 1; i++)
		{
			JoglVectord3 p1 = new JoglVectord3(pts.get(i).getX(), depth, pts.get(i).getZ());
			JoglVectord3 p2 = new JoglVectord3(pts.get(i+1).getX(), depth, pts.get(i+1).getZ());	
			
			//if not last line segment
			if (i < pts.size() - 2)
			{		
				JoglLine3D line = new JoglLine3D(p1, p2, lineWidth, lineHeight);
				
				for (int j = 0; j < line.triLst.size(); j++)
				{
					vlist.add(line.vertLst.get(line.triLst.get(j)));
				}
				
				lastLine = curLine;
				curLine = line;
			}
			//last line segment
			else
			{
				JoglVectord3 dir = JoglVectord3.Subtract(p2, p1);
				JoglVectord3 sideDir = new JoglVectord3(dir.getValue(2), 0, -dir.getValue(0));
				sideDir = JoglVectord3.Normal(sideDir);
				JoglVectord3 upDir = JoglVectord3.Multiply(JoglVectord3.Cross(sideDir, dir), -1.0);
				upDir = JoglVectord3.Normal(upDir);
				
				double dlen = dir.getMagnitude();
				JoglVectord3 dirn = JoglVectord3.Normal(dir);
				
				JoglVectord3 p2off = JoglVectord3.Add(p1, JoglVectord3.Multiply(dirn, dlen - arrowLength));
	
				JoglLine3D line = new JoglLine3D(p1, p2off, lineWidth, lineHeight);

				for (int j = 0; j < line.triLst.size(); j++)
				{
					vlist.add(line.vertLst.get(line.triLst.get(j)));
				}
				
				JoglVectord3 a1 = JoglVectord3.Add(p2off, JoglVectord3.Multiply(sideDir,arrowWidth));
				JoglVectord3 a2 = JoglVectord3.Add(p2off, JoglVectord3.Multiply(sideDir,-arrowWidth));
				JoglVectord3 a3 = JoglVectord3.Add(p2off, new JoglVectord3(0, arrowWidth * 0.5, 0));
				JoglVectord3 a4 = JoglVectord3.Add(p2off, new JoglVectord3(0, -arrowWidth * 0.5, 0));
				
				//draw arrow head
				vlist.add(p2);
				vlist.add(a1);
				vlist.add(a3);

				vlist.add(a2);
				vlist.add(p2);
				vlist.add(a3);
				
				vlist.add(a1);
				vlist.add(p2);
				vlist.add(a4);

				vlist.add(p2);
				vlist.add(a2);
				vlist.add(a4);
				
				//back of arrowhead
				vlist.add(a1);
				vlist.add(a4);
				vlist.add(a2);

				vlist.add(a2);
				vlist.add(a3);
				vlist.add(a1);

				lastLine = curLine;
				curLine = line;
			}
			
			//fill in bends
			if (i > 0)
			{
				//suboptimal, potential overdraw and z-issues!
				double dot = JoglVectord3.Dot(curLine.getSideDir(), lastLine.getLineDir());
				if (dot > 0)
				{
					//top
					vlist.add(curLine.vertLst.get(0));
					vlist.add(lastLine.vertLst.get(2));
					vlist.add(curLine.vertLst.get(4));
					
					//bottom
					vlist.add(curLine.vertLst.get(5));
					vlist.add(lastLine.vertLst.get(2));
					vlist.add(curLine.vertLst.get(0));
				}
				else if (dot < 0)
				{
					//top
					vlist.add(curLine.vertLst.get(1));
					vlist.add(curLine.vertLst.get(4));
					vlist.add(lastLine.vertLst.get(3));

					//bottom
					vlist.add(lastLine.vertLst.get(3));
					vlist.add(curLine.vertLst.get(5));
					vlist.add(curLine.vertLst.get(1));
				}
			}
		}
		
		return vlist;
	}
	
	public JoglMesh getMesh()
	{
		return mesh;
	}
	
	public void setMesh(JoglMesh m)
	{
		mesh = m;
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		//Draw geometry
		pipe.PushPipelineState(matPE);
		
		matPE.DrawColor(colorR, colorG, colorB);   // set the color of the triangle
		mesh.draw(pipe);
		
		// Done Drawing The line
		pipe.PopPipelineState(matPE);
	}
	
	public void DrawMask(GraphicsPipeline pipe)
	{
		//Draw geometry
		pipe.PushPipelineState(matPE);
		
		matPE.DrawMask();   // set the color of the triangle
		mesh.draw(pipe);
		
		// Done Drawing The line
		pipe.PopPipelineState(matPE);
	}
}
