package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglMesh;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTarget;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class JSOPnPlace extends JSOPnNode 
{
	private float width = 1;
	private float height = 1;
	private float halfwidth = 0.5f;
	private float halfheight = 0.5f;
	
	private double lineWidth = 2.0;
	
	private JoglMesh mesh;
	
	public Place orgPlace;

	public JSOPnPlace()
	{
		setNumVerts(32);
	}
	
	public JSOPnPlace(int numPoints)
	{
		setNumVerts(numPoints);
	}
	
	public void setNumVerts(int numPoints)
	{
		NumVerts = numPoints;
		BorderStart = numPoints * 3; //numPoints * 3;
		BorderEnd = (numPoints * 9) + (numPoints * 6); //(numPoints * 9) + (numPoints * 6);		
	}
	
	public JoglVectord2 getSize()
	{
		return new JoglVectord2(width, height);
	}
	
	public void setSize(float w, float h)
	{
		width = w;
		height = h;
		
		halfwidth = w * 0.5f;
		halfheight = h * 0.5f;
		
		//UpdateAABB();
	}
	
	public JoglMesh getMesh()
	{
		return mesh;
	}
	
	public void setMesh(JoglMesh m)
	{
		mesh = m;
		
		UpdateAABB();
	}
	
	@Override
	public PetrinetNode getOrgNode()
	{
		return orgPlace;
	}
	
	public static Vector<JoglVectord3> GenerateGeometry3D(double width, double height, double lineWidth, double depth, int numPoints) 
	{	
		double halfwidth = width * 0.5;
		//double halfheight = height * 0.5;
		
		Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();

		Vector<JoglVectord3> points = new Vector<JoglVectord3>();
		Vector<JoglVectord3> points2 = new Vector<JoglVectord3>();

		double outerRadius = halfwidth;
		double innerRadius = outerRadius - lineWidth;
		double circ = (Math.PI * 2.0) / numPoints;
		
		//generate vertex positions in a circle
		for (int i = 0; i < numPoints; i++)
		{
			JoglVectord3 vp = new JoglVectord3(Math.sin(i * circ), 0, Math.cos(i * circ));
			points.add(JoglVectord3.Multiply(vp, innerRadius));
			points2.add(JoglVectord3.Multiply(vp, outerRadius));
		}

		// generate triangles for the center of the place
		for (int i = 0; i < numPoints; i++)
		{
			vlist.add(points.get(i));
			vlist.add(points.get((i+1) % numPoints));
			vlist.add(new JoglVectord3(0, 0, 0));
		}

		// generate triangles for the border of the place
		for (int i = 0; i < numPoints; i++)
		{
			vlist.add(points2.get(i));
			vlist.add(points2.get((i+1) % numPoints));
			vlist.add(points.get(i));

			vlist.add(points2.get((i+1) % numPoints));
			vlist.add(points.get((i+1) % numPoints));
			vlist.add(points.get(i));
		}


		// generate triangles for 3D border of the place
		for (int i = 0; i < numPoints; i++)
		{
			JoglVectord3 p1 = points2.get(i);
			JoglVectord3 p2 = points2.get((i+1) % numPoints);

			vlist.add(p2);
			vlist.add(p1);
			vlist.add(JoglVectord3.Add(p1, new JoglVectord3(0, depth, 0)));

			vlist.add(JoglVectord3.Add(p2, new JoglVectord3(0, depth, 0)));
			vlist.add(p2);
			vlist.add(JoglVectord3.Add(p1, new JoglVectord3(0, depth, 0)));
		}
		
		return vlist;
	}
	
	public int NumVerts = 32;
	public int BorderStart = 96; //numPoints * 3;
	public int BorderEnd = 480; //(numPoints * 9) + (numPoints * 6);
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		//2D
		//int borderStart = numPoints * 3;
		//int borderEnd = numPoints * 9;
		
		//3D
		//int borderStart = numPoints * 3;
		//int borderEnd = (numPoints * 9) + (numPoints * 6);
		
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);
		
		// start drawing the place    
		pipe.PushPipelineState(matPE); 

		//draw white circle
		matPE.DrawWhite();   // set the color of the circle
		mesh.draw(pipe, 0, BorderStart);

		//draw black border around circle
		matPE.DrawColor(colorR, colorG, colorB);   // set the color of the border
		mesh.draw(pipe, BorderStart, BorderEnd);
		
		// done drawing the place
		pipe.PopPipelineState(matPE);
		
		pipe.transform.Pop();
	}

	public void DrawMask(GraphicsPipeline pipe, JoglRenderTarget target)
	{
		//2D
		//int borderStart = numPoints * 3;
		//int borderEnd = numPoints * 9;
		
		//3D
		//int borderStart = numPoints * 3;
		//int borderEnd = (numPoints * 9) + (numPoints * 6);
		
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);
		
		// start drawing the place      
		pipe.PushPipelineState(matPE);
		
		//draw white circle
		matPE.DrawMask();   // set the color of the circle
		mesh.draw(pipe, 0, BorderStart);

		//draw black border around circle
		mesh.draw(pipe, BorderStart, BorderEnd);
		
		// done drawing the place
		pipe.PopPipelineState(matPE);
		
		pipe.transform.Pop();
	}
	
	public void UpdateAABB()
	{
		JoglVectord3 amin = new JoglVectord3(-halfwidth, -5, -halfwidth);
		JoglVectord3 amax = new JoglVectord3(halfwidth, 0, halfwidth);
		
		JoglVectord3 vmin = new JoglVectord3(position.getX() + amin.getX(), position.getY() + amin.getY(), position.getZ() + amin.getZ());
		JoglVectord3 vmax = new JoglVectord3(position.getX() + amax.getX(), position.getY() + amax.getY(), position.getZ() + amax.getZ());

		JoglAABB aabb = new JoglAABB(vmin, vmax);
		
		boundingBox = aabb;
		//return aabb;
	}
}
