package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglMesh;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTarget;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class JSOPnTransition extends JSOPnNode 
{
	private float width = 1;
	private float height = 1;
	private float halfwidth = 0.5f;
	private float halfheight = 0.5f;

	private float lineWidth = 2.0f;
	
	private JoglMesh mesh;
	
	private String label;
	public Transition orgTransition;
	
	public String getLabel()
	{
		return label;
	}
	
	public void setLabel(String s)
	{
		label = s;
	}
	
	public JoglVectord2 getSize()
	{
		return new JoglVectord2(width, height);
	}
	
	public void setSize(float w, float h)
	{
		width = w;
		height = h;
		
		halfwidth = w * 0.5f;
		halfheight = h * 0.5f;
	}
	
	public JoglMesh getMesh()
	{
		return mesh;
	}
	
	public void setMesh(JoglMesh m)
	{
		mesh = m;
		
		UpdateAABB();
		//boundingBox = UpdateAABB();
	}
	
	@Override
	public PetrinetNode getOrgNode()
	{
		return orgTransition;
	}
	
	public boolean hasSubnet()
	{
		return getParentNet().getPetriNet().hasSubnet(orgTransition);
	}
	
	public static Vector<JoglVectord3> GenerateGeometry3D(double width, double height, double lineWidth, double depth) 
	{
		double halfwidth = width * 0.5;
		double halfheight = height * 0.5;
		
		Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();

		//left side
		vlist.add(new JoglVectord3(-halfwidth, 0.0f, halfheight));   // Top Left
		vlist.add(new JoglVectord3(-halfwidth + lineWidth, 0.0f, halfheight - lineWidth));   // Top Right
		vlist.add(new JoglVectord3(-halfwidth + lineWidth, 0.0f, -halfheight + lineWidth));   // Bottom Right

		vlist.add(new JoglVectord3(-halfwidth + lineWidth, 0.0f, -halfheight + lineWidth));   // Bottom Right
		vlist.add(new JoglVectord3(-halfwidth, 0.0f, -halfheight));   // Bottom Left
		vlist.add(new JoglVectord3(-halfwidth, 0.0f, halfheight));   // Top Left
		
		//right side
		vlist.add(new JoglVectord3( halfwidth - lineWidth, 0.0f, halfheight - lineWidth));   // Top Left
		vlist.add(new JoglVectord3( halfwidth, 0.0f, halfheight));   // Top Right
		vlist.add(new JoglVectord3( halfwidth, 0.0f, -halfheight));   // Bottom Right

		vlist.add(new JoglVectord3( halfwidth, 0.0f, -halfheight));   // Bottom Right
		vlist.add(new JoglVectord3( halfwidth - lineWidth, 0.0f, -halfheight + lineWidth));   // Bottom Left
		vlist.add(new JoglVectord3( halfwidth - lineWidth, 0.0f, halfheight - lineWidth));   // Top Left

		//top side
		vlist.add(new JoglVectord3(-halfwidth, 0.0f, halfheight));   // Top Left
		vlist.add(new JoglVectord3( halfwidth, 0.0f, halfheight));   // Top Right
		vlist.add(new JoglVectord3( halfwidth - lineWidth, 0.0f, halfheight - lineWidth));   // Bottom Right

		vlist.add(new JoglVectord3( halfwidth - lineWidth, 0.0f, halfheight - lineWidth));   // Bottom Right
		vlist.add(new JoglVectord3(-halfwidth + lineWidth, 0.0f, halfheight - lineWidth));   // Bottom Left
		vlist.add(new JoglVectord3(-halfwidth, 0.0f, halfheight));   // Top Left

		//bottom side
		vlist.add(new JoglVectord3(-halfwidth + lineWidth, 0.0f, -halfheight + lineWidth));   // Top Left
		vlist.add(new JoglVectord3( halfwidth - lineWidth, 0.0f, -halfheight + lineWidth));   // Top Right
		vlist.add(new JoglVectord3( halfwidth, 0.0f, -halfheight));   // Bottom Right

		vlist.add(new JoglVectord3( halfwidth, 0.0f, -halfheight));   // Bottom Right
		vlist.add(new JoglVectord3(-halfwidth, 0.0f, -halfheight));   // Bottom Left
		vlist.add(new JoglVectord3(-halfwidth + lineWidth, 0.0f, -halfheight + lineWidth));   // Top Left

		//3D side left side
		vlist.add(new JoglVectord3(-halfwidth, 0.0f, halfheight));   // Top Left
		vlist.add(new JoglVectord3(-halfwidth, 0.0f, -halfheight));   // Top Right
		vlist.add(new JoglVectord3(-halfwidth, depth, -halfheight));   // Bottom Right

		vlist.add(new JoglVectord3(-halfwidth, depth, -halfheight));   // Bottom Right
		vlist.add(new JoglVectord3(-halfwidth, depth, halfheight));   // Bottom Left
		vlist.add(new JoglVectord3(-halfwidth, 0.0f, halfheight));   // Top Left
		
		//3D side right side
		vlist.add(new JoglVectord3( halfwidth, 0.0f, halfheight));   // Top Left
		vlist.add(new JoglVectord3( halfwidth, depth, halfheight));   // Top Right
		vlist.add(new JoglVectord3( halfwidth, depth, -halfheight));   // Bottom Right

		vlist.add(new JoglVectord3( halfwidth, depth, -halfheight));   // Bottom Right
		vlist.add(new JoglVectord3( halfwidth, 0.0f, -halfheight));   // Bottom Left
		vlist.add(new JoglVectord3( halfwidth, 0.0f, halfheight));   // Top Left

		//3D side top side
		vlist.add(new JoglVectord3(-halfwidth, 0.0f, halfheight));   // Top Left
		vlist.add(new JoglVectord3(-halfwidth, depth, halfheight));   // Top Right
		vlist.add(new JoglVectord3( halfwidth, depth, halfheight));   // Bottom Right

		vlist.add(new JoglVectord3( halfwidth, depth, halfheight));   // Bottom Right
		vlist.add(new JoglVectord3(halfwidth, 0.0f, halfheight));   // Bottom Left
		vlist.add(new JoglVectord3(-halfwidth, 0.0f, halfheight));   // Top Left
		
		//3D side bottom
		vlist.add(new JoglVectord3(-halfwidth, 0.0f, -halfheight));   // Top Left
		vlist.add(new JoglVectord3( halfwidth, 0.0f, -halfheight));   // Top Right
		vlist.add(new JoglVectord3( halfwidth, depth, -halfheight));   // Bottom Right

		vlist.add(new JoglVectord3( halfwidth, depth, -halfheight));   // Bottom Right
		vlist.add(new JoglVectord3(-halfwidth, depth, -halfheight));   // Bottom Left
		vlist.add(new JoglVectord3(-halfwidth, 0.0f, -halfheight));   // Top Left
		
		//center
		vlist.add(new JoglVectord3(-halfwidth + lineWidth, 0.0f, halfheight - lineWidth));   // Top Left
		vlist.add(new JoglVectord3( halfwidth - lineWidth, 0.0f, halfheight - lineWidth));   // Top Right
		vlist.add(new JoglVectord3( halfwidth - lineWidth, 0.0f, -halfheight + lineWidth));   // Bottom Right

		vlist.add(new JoglVectord3( halfwidth - lineWidth, 0.0f, -halfheight + lineWidth));   // Bottom Right
		vlist.add(new JoglVectord3(-halfwidth + lineWidth, 0.0f, -halfheight + lineWidth));   // Bottom Left
		vlist.add(new JoglVectord3(-halfwidth + lineWidth, 0.0f, halfheight - lineWidth));   // Top Left
		
		return vlist;
	}
	
	public static int CenterStart = 48;
	public static int CenterEnd = 54;
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		//2D
		//int centerStart = 16;
		//int centerEnd = 20;
		
		//3D
		//int centerStart = 48;
		//int centerEnd = 54;
		
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);

		// Draw A Quad
		pipe.PushPipelineState(matPE);

		//draw black border around transition
		matPE.DrawColor(colorR, colorG, colorB);   // set the color of the quads
		mesh.draw(pipe, 0, CenterStart);
		
		//draw white inside of transition
		matPE.DrawWhite();   // set the color of the quad
		mesh.draw(pipe, CenterStart, CenterEnd);
		
		// Done Drawing The Quad
		pipe.PopPipelineState(matPE);
		
		pipe.transform.Pop();
	}

	public void DrawMask(GraphicsPipeline pipe, JoglRenderTarget target)
	{
		//2D
		//int centerStart = 16;
		//int centerEnd = 20;
		
		//3D
		int centerStart = 48;
		int centerEnd = 54;
		
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);

		// Draw A Quad
		pipe.PushPipelineState(matPE);

		matPE.DrawMask();   // set the color of the quads
		//draw black border around transition
		//mesh.draw(pipe, 0, centerStart);
		//draw white inside of transition
		//mesh.draw(pipe, centerStart, centerEnd);
		mesh.draw(pipe, 0, centerEnd);
		
		
		// Done Drawing The Quad
		pipe.PopPipelineState(matPE);
		
		pipe.transform.Pop();
	}
	
	public void UpdateAABB()
	{		
//		JoglAABB a = mesh.getAABB();
//		JoglVectord3 amin = a.getMin();
//		JoglVectord3 amax = a.getMax();
		JoglVectord3 amin = new JoglVectord3(-halfwidth, -5, -halfheight);
		JoglVectord3 amax = new JoglVectord3(halfwidth, 0, halfheight);
		
		JoglVectord3 vmin = new JoglVectord3(position.getX() + amin.getX(), position.getY() + amin.getY(), position.getZ() + amin.getZ());
		JoglVectord3 vmax = new JoglVectord3(position.getX() + amax.getX(), position.getY() + amax.getY(), position.getZ() + amax.getZ());

		JoglAABB aabb = new JoglAABB(vmin, vmax);
		
		boundingBox = aabb;
		//return aabb;
	}
}
