package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.HashMap;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.interactions.replay.TokenParticle;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.IMouseProcessor;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JSOReplaya extends JoglSceneObject
{
	private IMouseProcessor iman = null;
	private TokenDrawer tokenDrawer = null;
	private Vector<TokenParticle> tparts = new Vector<TokenParticle>();
	private HashMap<JSOSubNet, Vector<TokenParticle>> tpartsByNet = new HashMap<JSOSubNet, Vector<TokenParticle>>();
	
	public JSOReplaya(IMouseProcessor iman)
	{
		this.iman = iman;
	}
	
	public void AddParticles(Vector<TokenParticle> prts)
	{
		//make sure no particles are drawn while adding to the list
		synchronized(this)
		{
			tparts.addAll(prts);
			for (TokenParticle tp : prts)
			{
				if (!tpartsByNet.containsKey(tp.subnet))
				{
					tpartsByNet.put(tp.subnet, new Vector<TokenParticle>());
				}
				tpartsByNet.get(tp.subnet).add(tp);
			}
		}
	}
	
	public void ClearParticles()
	{
		tparts.clear();
		tpartsByNet.clear();
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
//		for (JSOSubNet snet : tokensByNet.keySet())
//		{
//			TokenCube.DrawCubesOpt2(gl, tokensByNet.get(snet));
//		}
		for (JSOSubNet snet : tpartsByNet.keySet())
		{
			pipe.transform.Push();
			JoglMatrixd4x4 transform = snet.getTransform();
			pipe.transform.MultMatrix(transform);
			
			DrawObject(pipe, snet);

			pipe.transform.Pop();
		}
	}
	
	public void DrawObject(GraphicsPipeline pipe, JSOSubNet snet)
	{
		//don't draw tokens if subnet is not visible
		if (!snet.getIsVisible())
			return;
		
		if (tokenDrawer == null)
		{
			JSOCohortColorManager cman = pipe.scene.findSceneObject(JSOCohortColorManager.class);
			tokenDrawer = new TokenDrawer(cman);
		}
		
//		Vector<ITokenLane> lanes = LaneCache.getLanesForSubnet(snet);
//		TokenCube.DrawCubesOpt3(gl, lanes);
		
		//TokenCube.DrawParticles(gl, tparts);
		
		//make sure no particles are added to the list while drawing
		synchronized(this)
		{
			if (tpartsByNet.containsKey(snet))
			{
				tokenDrawer.DrawParticles2(pipe, snet, tpartsByNet.get(snet));
			}
		}
	}
	
	public void Dispose()
	{
		if (tokenDrawer != null)
		{
			tokenDrawer.Dispose();
		}
	}
}
