package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.ArrayList;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglColoredMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderTrajectory;
import org.processmining.processprofiler3d.resourceanalysis.statics.FrequencyTree;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.opengl.GL2;

/*
 * A quad to render in the scene
 */
public class JSOResourceWheel extends JoglSceneObject 
{
	private FrequencyTree ftree = null;
	private JoglVectord3 position = new JoglVectord3(0,0,0);
	private JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateIdentity();

	private ArrayList<String> myActivities = null;
	private ArrayList<String> myResources = null;
	private ArrayList<String> myCohorts = null;
	
	public JSOResourceWheel(FrequencyTree ft)
	{
		ftree = ft;
		
		myActivities = ftree.getActivities();
		myResources = ftree.getResources();
		myCohorts = ftree.getCohorts();
	}
	
	public void setPosition(double x, double y, double z)
	{
		position = new JoglVectord3(x,y,z);
		transform = JoglMatrixd4x4.CreateTranslation(x, y, z);
	}
	
	public void setPosition(JoglVectord3 v)
	{
		position = v;
		transform = JoglMatrixd4x4.CreateTranslation(v);
	}
	
	private Vector<JoglVectord3> GenerateGeometry()
	{
		int maxFreq = 0;
		for (String sa : myActivities)
		{
			for (String sr : myResources)
			{
				for (String sc : myCohorts)
				{
					int freq = ftree.getValue(sa, sr, sc);
					maxFreq = Math.max(maxFreq, freq);
				}
			}
		}
		maxFreq++;
		
		ArrayList<String> alst = myActivities;
		int numPoints = alst.size();
		
		Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();

		Vector<JoglVectord3> points = new Vector<JoglVectord3>();
		Vector<JoglVectord3> points2 = new Vector<JoglVectord3>();

		double depth = 100.0;
		double outerRadius = 5.0;
		double outerRadius2 = 35.0;
		double circ = (Math.PI * 2.0) / numPoints;
		
		//generate vertex positions in a circle
		for (int i = 0; i < numPoints; i++)
		{
			JoglVectord3 vp = new JoglVectord3(Math.sin(i * circ), 0, Math.cos(i * circ));
			points2.add(JoglVectord3.Multiply(vp, outerRadius));
			points.add(JoglVectord3.Multiply(vp, outerRadius2));
		}

		// generate quads for 3D border of the place
		double resW = depth / (double)myResources.size();
		double resOff = 0;
		for (String res : myResources)
		{
			JoglVectord3 voff1 = new JoglVectord3(0, resOff, 0);
			JoglVectord3 voff2 = new JoglVectord3(0, resOff + resW, 0);
			for (int i = 0; i < numPoints; i++)
			{
				JoglVectord3 p1 = points2.get(i);
				JoglVectord3 p2 = points2.get((i+1) % numPoints);

				vlist.add(JoglVectord3.Add(p2, voff1));
				vlist.add(JoglVectord3.Add(p1, voff1));
				vlist.add(JoglVectord3.Add(p1, voff2));
				vlist.add(JoglVectord3.Add(p2, voff2));
			}
			resOff += resW;
		}
		
		double cohortW = depth / ((double)myResources.size() * (double)myCohorts.size());
		double cohortOff = 0;
		for (String res : myResources)
		{
			for (String cohort : myCohorts)
			{
				for (int i = 0; i < numPoints; i++)
				{
					JoglVectord3 p1 = points2.get(i);
					JoglVectord3 p2 = points.get(i);
					
					int freq = ftree.getValue(myActivities.get(i), res, cohort);
					double scale = (double)freq / (double)maxFreq;
		
					JoglVectord3 dir = JoglVectord3.Subtract(p2, p1);
					JoglVectord3 px = JoglVectord3.Add(p1, JoglVectord3.Multiply(dir, scale));
					
					vlist.add(JoglVectord3.Add(p1, new JoglVectord3(0, cohortOff, 0)));
					vlist.add(JoglVectord3.Add(px, new JoglVectord3(0, cohortOff, 0)));
					vlist.add(JoglVectord3.Add(px, new JoglVectord3(0, cohortOff + cohortW, 0)));
					vlist.add(JoglVectord3.Add(p1, new JoglVectord3(0, cohortOff + cohortW, 0)));
				}
				cohortOff += cohortW;
			}
		}
		
		return vlist;
	}
	
	private Vector<JoglVectord3> GenerateColors(int verts)
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();

		ArrayList<String> alst = myActivities;
		int numPoints = alst.size();

		// generate quads for 3D border of the place
		int ires = 0;
		for (String res : myResources)
		{
			JoglVectord3 resColor = new JoglVectord3(0.5f, 0.5f, 0.5f);
			if (ires % 2 == 0)
			{
				resColor = new JoglVectord3(0.35f, 0.35f, 0.35f);
			}
			
			for (int i = 0; i < numPoints; i++)
			{
				lst.add(resColor);   // set the color of the quad
				lst.add(resColor);   // set the color of the quad
				lst.add(resColor);   // set the color of the quad
				lst.add(resColor);   // set the color of the quad
			}
			ires++;
		}
		
		Vector<JoglVectord3> clist = new Vector<JoglVectord3>();
		clist.add(new JoglVectord3(1.0f, 0.0f, 0.0f));
		clist.add(new JoglVectord3(0.0f, 1.0f, 0.0f));
		clist.add(new JoglVectord3(0.0f, 0.0f, 1.0f));
		clist.add(new JoglVectord3(1.0f, 1.0f, 0.0f));
		clist.add(new JoglVectord3(1.0f, 0.0f, 1.0f));
		clist.add(new JoglVectord3(0.0f, 1.0f, 1.0f));
		clist.add(new JoglVectord3(1.0f, 1.0f, 1.0f));
		clist.add(new JoglVectord3(0.0f, 0.0f, 0.0f));
		
		for (String res : myResources)
		{
			int icohort = 0;
			for (String cohorts : myCohorts)
			{
				for (int i = 0; i < numPoints; i++)
				{
					JoglVectord3 cohortColor = clist.get(icohort);
					lst.add(cohortColor);   // set the color of the quad
					lst.add(cohortColor);   // set the color of the quad
					lst.add(cohortColor);   // set the color of the quad
					lst.add(cohortColor);   // set the color of the quad
				}
				icohort++;
			}
		}
//		for (int i = 0; i < verts; i++)
//		{
//			lst.add(new JoglVectord3(0.0f, 0.0f, 1.0f));   // set the color of the quad
//		}

		return lst;
	}
	
	public JoglColoredMesh CreateMesh()
	{
		JoglColoredMesh jcm = new JoglColoredMesh(GL2.GL_QUADS);
		jcm.setVerts(GenerateGeometry());
		jcm.setColors3(GenerateColors(jcm.getNumVerts()));
		jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jcm.getNumVerts()));
		
		return jcm;
	}
	
	private IMeshDrawer vao = null;
	private JoglShader shad = null;
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);
		
		if (vao == null)
		{
			vao = new VAODrawer();
			vao.LoadMesh(pipe, CreateMesh());
		}
		if (shad == null)
		{
			shad = new JoglShaderTrajectory();
			shad.Create(pipe);
		}
		
		shad.Bind(pipe);
		vao.Draw(pipe);
		shad.Unbind(pipe);
		
		pipe.transform.Pop();
	}

}
