package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.awt.Dimension;
import java.awt.geom.Point2D;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.PetrinetSimplifier;
import org.processmining.processprofiler3d.compviz.SubNetDataSource;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglMesh;
import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.drawing.JoglTextRenderer;
import org.processmining.processprofiler3d.psen.fileio.obj.OBJWriter;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.collision.CannotCreateBoundingBoxException;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.collision.JoglAABBFactory;
import org.processmining.visualisation3d.gldatastructures.JoglFrameBufferObject;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.gldatastructures.JoglTextureBufferObject;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTarget;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTargetFramebuffer;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;

public class JSOSubNet extends JoglSceneObject implements IJSOModel
{
	public interface ISubNetLayoutListener
	{
		public void OnLayoutChanged(JSOSubNet snet);
	}
	
	private JoglPanel panel;
	private INetDataSource snet;
	private GraphLayoutConnection layout;

	private JSOPnTransition parent = null;
	private Vector<JSOSubNet> ancestors = new Vector<JSOSubNet>();
	private Vector<JSOSubNet> subnets = new Vector<JSOSubNet>();
	
	private Vector<JoglSceneObject> elements;
	private HashMap<Transition, JoglSceneObject> labels = new HashMap<Transition, JoglSceneObject>();
	
	private JoglAABB boundingBox;
	
	private JoglVectord2 center = null;
	private JoglVectord2 size = null;
	
	private int simplificationLevel = 0;
	private PetrinetSimplifier ps = null;
	
	private JSOPetriNetBatch batcher = null;
	
	private JoglVectord3 offset = new JoglVectord3(0,0,0); //current position (potentially animated)
	private JoglVectord3 dynoff = new JoglVectord3(0,0,0);
	private double scale = 1.0;
	private JoglMatrixd4x4 localTransform = null;
	private JoglMatrixd4x4 globalTransform = null;
	
	private boolean bVisible = true;
	
	public JSOSubNetPedestal ped = null;
	
	public Vector<ISubNetLayoutListener> layoutListeners = new Vector<ISubNetLayoutListener>();
	
	public PetrinetSimplifier getLayout()
	{
		return ps;
	}
	
	public int getLevel()
	{
		if (parent != null)
		{
			return getParentNet().getLevel() + 1;
		}
		return 0;
	}
	
	public JoglAABB getAABB()
	{
		JoglVectord3 v = relativeToAbsolute(JoglVectord3.ZERO);
		return boundingBox.Translate(v);
	}
	
	public INetDataSource getPetriNet()
	{
		return snet;
	}
	
	public boolean getIsVisible()
	{
		if (getParentNet() != null)
		{
			return bVisible && getParentNet().getIsVisible();
		}
		return bVisible;
	}
	
	public void setIsVisible(boolean bVis)
	{
		bVisible = bVis;
	}
	
	public int getSimplificationLevel()
	{
		return simplificationLevel;
	}
	
	public void setSimplificationLevel(int lvl)
	{
		simplificationLevel = lvl;
	}
	
	public JSOPnTransition getParent()
	{
		return parent;
	}
	
	public JSOSubNet getParentNet()
	{
		if (parent != null)
		{
			return parent.getParentNet();
		}
		return null;
	}
	
	public Vector<JSOSubNet> getAncestors()
	{
		return ancestors;
	}
	
	public double getScale()
	{
		return scale;
	}
	
	public void setScale(double s)
	{
		scale = s;
		
		localTransform = null;
		ClearTransform();
	}
	
	public JoglVectord3 getOffset()
	{
		return JoglVectord3.Add(offset, dynoff);
	}
	
	public void setOffset(JoglVectord3 off)
	{
		offset = off;
		
		localTransform = null;
		ClearTransform();
	}
	
	public JoglVectord3 getDynamicOffset()
	{
		return dynoff;
	}
	
	public void setDynamicOffset(JoglVectord3 doff)
	{
		dynoff = doff;
		
		localTransform = null;
		ClearTransform();
	}
	
	public JoglVectord3 getOrigin()
	{
		//return origin;
		if (getParentNet() != null)
		{
			JoglVectord3 parentPos = parent.getPosition();
			//return getParentNet().relativeToAbsolute(parentPos);
			return parentPos;
		}
		return JoglVectord3.ZERO;
	}
	
	public double getSubnetYPos()
	{
		JoglVectord3 level = this.relativeToAbsolute(JoglVectord3.ZERO);
		return level.getY();
	}
	
	public Vector<JSOSubNet> getChildSubNets()
	{
		return subnets;
	}
	
	public void setChildSubNets(Vector<JSOSubNet> snts)
	{
		subnets.addAll(snts);
	}
	
	public JoglTexture getMaskTexture()
	{
		if (fbo != null)
		{
			JoglTextureBufferObject jtbo = (JoglTextureBufferObject)fbo.getColorBuffer0();
			return jtbo.getTexture();
		}
		return null;
	}
	
	public JSOSubNet(JoglPanel p, JoglScene scene, INetDataSource snet, GraphLayoutConnection layoutConn, JSOPnTransition parent)
	{
		this.panel = p;
		this.snet = snet;
		layout = layoutConn;
		ped = new JSOSubNetPedestal(this);
		this.parent = parent;
		
		//get ancestors
		JSOSubNet parentNet = this;
		while (parentNet != null)
		{
			ancestors.insertElementAt(parentNet, 0);
			parentNet = parentNet.getParentNet();
		}
		
		BuildScene(scene);
	}
	
	public void BuildScene(JoglScene scene)
	{	
		elements = new Vector<JoglSceneObject>();
		
		ps = new PetrinetSimplifier(snet, layout, simplificationLevel);
		BuildFrom(scene, ps);
		
		boundingBox = UpdateAABB();
	}

	public static JoglVectord3 subnetOffset = new JoglVectord3(0, 500, 0);
	
	private void BuildFrom(JoglScene scene, PetrinetSimplifier simp) 
	{		
		for (Place p : simp.getPlaces()) 
		{
			//System.out.println("Place Id: " + p.getLabel() + " - " + p.getLocalID());
			Dimension size = simp.getSize(p);
			//System.out.println("Place: " + pos.toString() + ", " + size.toString());

			JSOPnPlace pso = new JSOPnPlace(32);
			
			pso.setPosition(simp.getPosition(p));
			pso.setSize((float)size.getWidth(), (float)size.getHeight());
			pso.UpdateAABB();
			
			pso.orgPlace = p;
			pso.setParentNet(this);
			
			elements.add(pso);
		}
		
		for (Transition t : simp.getTransitions()) 
		{
			//System.out.println("Transition Id: " + t.getLabel() + " - " + t.getLocalID());
			Dimension size = simp.getSize(t);
			//System.out.println("Transition: " + pos.toString() + ", " + size.toString());

			JSOPnTransition tso = new JSOPnTransition();
			tso.setPosition(simp.getPosition(t));
			tso.setSize((float)size.getWidth(), (float)size.getHeight());
			tso.UpdateAABB();
			
			tso.setLabel(t.getLabel());
			
			tso.orgTransition = t;
			tso.setParentNet(this);
			
			elements.add(tso);
			
		}

		for (int edgeId = 0; edgeId < simp.getNumEdges(); edgeId++) 
		{
			List<JoglVectord3> lst = new Vector<JoglVectord3>(simp.getEdgePoints(edgeId));
			PetrinetNode source = simp.getEdgeSource(edgeId);
			PetrinetNode target = simp.getEdgeTarget(edgeId);
			
			List<JoglVectord3> vec = getEdgePoints(simp, lst, source, target);
			
			JSOPnEdge eso = new JSOPnEdge(vec);
			eso.startObj = getElementFor(source);
			eso.endObj = getElementFor(target);
			eso.transitionPairID = getTransitionPairIdFor(source, target);
			eso.edgeLayoutID = edgeId;
			eso.setParentNet(this);
			
			elements.add(eso);
			elements.addAll(eso.getVerts());
		}
		
		float textSize = 10.0f;
		FontFormat ff = new FontFormat();
		ff.width = textSize;
		ff.height = textSize;
		ff.setColor(0.0f, 0.0f, 0.0f);
		
		for (Transition t : simp.getTransitions()) 
		{
			if (!t.isInvisible())
			{
				Dimension size = simp.getSize(t);
				//System.out.println("Transition: " + pos.toString() + ", " + size.toString());			
	
				String text = t.getLabel();

				JoglTextRenderer tr = new JoglTextRenderer(ff.clone());
				//JoglTextRendererDF tr = new JoglTextRendererDF(ff.clone());
				tr.setString(text, size.getWidth() - 4.0); //size minus border width
				tr.setPosition(JoglVectord3.Add(simp.getPosition(t), new JoglVectord3(0,1,0)));
				
				elements.add(tr);
				labels.put(t, tr);
			}
		}	
		
		if (snet instanceof SubNetDataSource)
		{
			for (SubNetDataSource esn : snet.getSubnets())
			{		
				JoglVectord3 subnetOrigin = new JoglVectord3(0, 0, 0);
				Transition tx = snet.getTransitionForSubnet(esn);
				if (tx != null)
				{
					//subnetOffset = JoglVectord3.Add(subnetOffset, ps.getPosition(tx));
					subnetOrigin = ps.getPosition(tx);
				}
				
				JSOPnTransition parentTrans = getTransitionFor(tx);
				JSOSubNet subN = new JSOSubNet(panel, scene, esn, layout, parentTrans);
				subN.setIsVisible(false);
				JoglVectord3 center = subN.GetPetrinetCenter();
				//subN.setOrigin(subnetOrigin);
				subN.setOffset(JoglVectord3.Add(subnetOrigin, JoglVectord3.Add(subnetOffset, center)));
				subnets.add(subN);
			}
		}
		
		if (batcher == null)
		{
			batcher = new JSOPetriNetBatch(panel, this);
		}
		//batcher.BuildScene();
		bLoaded = false;
	}
	
	private boolean bLoaded = false;
	
	public void ReloadMeshes()
	{
		bLoaded = false;
	}
	
	public void Relayout()
	{
		//update label positions
		for (JSOPnTransition t : getTransitions())
		{
			JoglTextRenderer tr = (JoglTextRenderer)labels.get(t.orgTransition);
			if (tr != null) //if floating labels are used, there is no text renderer
			{
				tr.setPosition(JoglVectord3.Add(t.getPosition(), new JoglVectord3(0,1,0)));
			}
		}

		//update arc positions
		for (JSOPnEdge edge : getEdges()) 
		{
			List<JoglVectord3> lst = new Vector<JoglVectord3>(ps.getEdgePoints(edge.edgeLayoutID));
			
			List<JoglVectord3> vec = getEdgePoints2(ps, lst, edge.startObj, edge.endObj);
			edge.setPoints(vec);
		}
		
		size = null;
		center = null;
		
		boundingBox = UpdateAABB();
		
		ReloadMeshes();
		
		for (ISubNetLayoutListener snli : layoutListeners)
		{
			snli.OnLayoutChanged(this);
		}
	}
	
	private void loadMeshes(GraphicsPipeline pipe)
	{
		for (JoglSceneObject jso : elements) 
		{
			if (jso instanceof JSOPnPlace)
			{
				JSOPnPlace pso = (JSOPnPlace)jso;
				JoglVectord2 size = pso.getSize();
	
				//create mesh data
				JoglMesh m = new JoglMesh(GL2.GL_TRIANGLES);
				int numVerts = 12;
				if (pipe.getGraphicsSettings().bHQGeometry)
				{
					numVerts = 32;
				}
				pso.setNumVerts(numVerts);
				m.setVerts(JSOPnPlace.GenerateGeometry3D(size.getX(), size.getY(), 2, -5.0, pso.NumVerts));
				pso.setMesh(m);
			}
			else if (jso instanceof JSOPnTransition)
			{
				JSOPnTransition tso = (JSOPnTransition)jso;
				JoglVectord2 size = tso.getSize();

				//create mesh data
				JoglMesh m = new JoglMesh(GL2.GL_TRIANGLES);
				m.setVerts(JSOPnTransition.GenerateGeometry3D(size.getX(), size.getY(), 2, -5.0));
				
				tso.setMesh(m);
			}
			else if (jso instanceof JSOPnEdge)
			{
				JSOPnEdge eso = (JSOPnEdge)jso;
				List<JoglVectord3> vec = eso.getPoints();
				
				//create mesh data
				JoglMesh m = new JoglMesh(GL2.GL_TRIANGLES);
				m.setVerts(JSOPnEdge.GenerateGeometry3D(vec, -2.5, 1, 4, 8));
					
				eso.setMesh(m);
			}
		}

		batcher.BuildScene();
		bLoaded = true;
	}
	
	private String getTransitionPairIdFor(PetrinetNode source, PetrinetNode target)
	{
		String sourceId = null;
		if (source instanceof Transition)
		{
			sourceId = source.getLocalID().toString();
		}
		else
		{
			Collection<Arc> lst = snet.getInEdges(source);
			if (lst.size() == 1)
			{
				Arc a = lst.iterator().next();
				sourceId = a.getSource().getLocalID().toString();
			}
		}
		
		String targetId = null;
		if (target instanceof Transition)
		{
			targetId = target.getLocalID().toString();
		}
		else
		{
			Collection<Arc> lst = snet.getOutEdges(target);
			if (lst.size() == 1)
			{
				Arc a = lst.iterator().next();
				targetId = a.getTarget().getLocalID().toString();
			}
		}
		
		if (sourceId != null && targetId != null)
		{
			return sourceId + targetId;
		}
		return null;
	}
	
	public JoglAABB UpdateAABB()
	{
		JoglAABBFactory aabbFact = new JoglAABBFactory();

		for (JSOPnTransition jso : getTransitions())
		{
			JoglAABB aabb = jso.getAABB();
			
			aabbFact.addPoint(aabb.getMin());
			aabbFact.addPoint(aabb.getMax());
		}
		
		for (JSOPnPlace jso : getPlaces())
		{
			JoglAABB aabb = jso.getAABB();

			aabbFact.addPoint(aabb.getMin());
			aabbFact.addPoint(aabb.getMax());
		}

		JoglAABB myAabb = null;
		
		try
		{
			myAabb = aabbFact.produce();
		}
		catch (CannotCreateBoundingBoxException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return myAabb;
	}
	
	public JoglVectord3 relativeToAbsolute(JoglVectord3 vx)
	{
		JoglVectord3 v = vx;
		for (int i = ancestors.size()-1; i >= 0; i--)
		{
			JSOSubNet snet = ancestors.get(i);
			v = JoglVectord3.Multiply(JoglVectord3.Add(snet.getOffset(), v), snet.getScale());
		}
		
		return v;
	}
	
	public Vector<JoglSceneObject> getElements()
	{
		return elements;
	}
	
	public JSOPnNode getElementFor(PetrinetNode pn)
	{
		JSOPnNode ele = null;
		if (pn instanceof Place)
		{
			ele = getPlaceFor(pn);
		}
		else if (pn instanceof Transition)
		{
			ele = getTransitionFor(pn);
		}
		
		return ele;
	}
	
	public JSOPnPlace getPlaceFor(PetrinetNode pn)
	{
		for (JoglSceneObject jso : elements)
		{
			if (jso instanceof JSOPnPlace)
			{
				JSOPnPlace p = (JSOPnPlace)jso;
				if (p.orgPlace == pn)
				{
					return p;
				}
			}
		}
		return null;
	}
	
	public JSOPnTransition getTransitionFor(PetrinetNode pn)
	{
		for (JoglSceneObject jso : elements)
		{
			if (jso instanceof JSOPnTransition)
			{
				JSOPnTransition p = (JSOPnTransition)jso;
				if (p.orgTransition == pn)
				{
					return p;
				}
			}
		}
		return null;
	}
	
	
	public Vector<JSOPnPlace> getPlaces()
	{
		Vector<JSOPnPlace> lst = new Vector<JSOPnPlace>();
		
		for (JoglSceneObject jso : elements)
		{
			if (jso instanceof JSOPnPlace)
			{
				lst.add((JSOPnPlace)jso);
			}
		}
		
		return lst;
	}
	
	public Vector<JSOPnTransition> getTransitions()
	{
		Vector<JSOPnTransition> lst = new Vector<JSOPnTransition>();
		
		for (JoglSceneObject jso : elements)
		{
			if (jso instanceof JSOPnTransition)
			{
				lst.add((JSOPnTransition)jso);
			}
		}
		
		return lst;
	}
	
	public Vector<JSOPnEdge> getEdges()
	{
		Vector<JSOPnEdge> lst = new Vector<JSOPnEdge>();

		for (JoglSceneObject jso : elements)
		{
			if (jso instanceof JSOPnEdge)
			{
				lst.add((JSOPnEdge)jso);
			}
		}
		
		return lst;
	}
	
	public Vector<JSOPnEdgeVertex> getEdgeVertices()
	{
		Vector<JSOPnEdgeVertex> lst = new Vector<JSOPnEdgeVertex>();

		for (JoglSceneObject jso : elements)
		{
			if (jso instanceof JSOPnEdgeVertex)
			{
				lst.add((JSOPnEdgeVertex)jso);
			}
		}
		
		return lst;
	}
	
	public JoglVectord3 GetPetrinetCenter()
	{
		if (center == null)
		{
			double minPosX = 0;
			double maxPosX = 0;
			double minPosY = 0;
			double maxPosY = 0;
			
			for (JSOPnPlace p : getPlaces()) 
			{
				JoglVectord3 pos = p.getPosition();
				JoglVectord2 size = p.getSize();
				
				minPosX = Math.min(minPosX, pos.getX() - (size.getX() * 0.5));
				maxPosX = Math.max(maxPosX, pos.getX() + (size.getX() * 0.5));
				minPosY = Math.min(minPosY, pos.getZ() - (size.getY() * 0.5));
				maxPosY = Math.max(maxPosY, pos.getZ() + (size.getY() * 0.5));
			}
			
			for (JSOPnTransition t : getTransitions()) 
			{
				JoglVectord3 pos = t.getPosition();
				JoglVectord2 size = t.getSize();
				
				minPosX = Math.min(minPosX, pos.getX() - (size.getX() * 0.5));
				maxPosX = Math.max(maxPosX, pos.getX() + (size.getX() * 0.5));
				minPosY = Math.min(minPosY, pos.getZ() - (size.getY() * 0.5));
				maxPosY = Math.max(maxPosY, pos.getZ() + (size.getY() * 0.5));		
			}
			
			JoglVectord3 centerPos = new JoglVectord3(minPosX + ((maxPosX - minPosX) * 0.5), 0, minPosY + ((maxPosY - minPosY) * 0.5));
			
			center = new JoglVectord2(centerPos.getX(), centerPos.getZ());
			
			return centerPos;
		}
		else
		{
			return new JoglVectord3(center.getX(), 0, center.getY());
		}
	}
	
	public JoglVectord2 GetPetrinetSize()
	{
		if (size == null)
		{
			double minPosX = 0;
			double maxPosX = 0;
			double minPosY = 0;
			double maxPosY = 0;
			
			for (JSOPnPlace p : getPlaces()) 
			{
				JoglVectord3 pos = p.getPosition();
				JoglVectord2 size = p.getSize();
				
				minPosX = Math.min(minPosX, pos.getX() - (size.getX() * 0.5));
				maxPosX = Math.max(maxPosX, pos.getX() + (size.getX() * 0.5));
				minPosY = Math.min(minPosY, pos.getZ() - (size.getY() * 0.5));
				maxPosY = Math.max(maxPosY, pos.getZ() + (size.getY() * 0.5));
			}
			
			for (JSOPnTransition t : getTransitions()) 
			{
				JoglVectord3 pos = t.getPosition();
				JoglVectord2 size = t.getSize();
				
				minPosX = Math.min(minPosX, pos.getX() - (size.getX() * 0.5));
				maxPosX = Math.max(maxPosX, pos.getX() + (size.getX() * 0.5));
				minPosY = Math.min(minPosY, pos.getZ() - (size.getY() * 0.5));
				maxPosY = Math.max(maxPosY, pos.getZ() + (size.getY() * 0.5));		
			}
			
			JoglVectord2 dims = new JoglVectord2(maxPosX - minPosX, maxPosY - minPosY);
			
			size = dims;
			
			return dims;
		}
		else
		{
			return size;
		}
	}

	private List<JoglVectord3> getEdgePoints(GraphLayoutConnection layoutConn, List<JoglVectord3> vec, PetrinetNode source,
			PetrinetNode target) 
	{
		
		Point2D sourcePos = layoutConn.getPosition(source);
		JoglVectord3 sourcePosV = new JoglVectord3(-sourcePos.getX(), 0, sourcePos.getY());
		Point2D targetPos = layoutConn.getPosition(target);
		JoglVectord3 targetPosV = new JoglVectord3(-targetPos.getX(), 0, targetPos.getY());
		
		double placeRadius = 12.5;
		
		if (source instanceof Place)
		{
			Place sourceP = (Place)source;
			placeRadius = layoutConn.getSize(sourceP).getWidth() * 0.5;
			
			if (vec.size() > 0)
			{
				sourcePosV = ClipByPlace(vec.get(0), sourcePosV, placeRadius);
			}
			else
			{
				sourcePosV = ClipByPlace(targetPosV, sourcePosV, placeRadius);
			}
		}
		if (target instanceof Place)
		{
			Place targetP = (Place)source;
			placeRadius = layoutConn.getSize(targetP).getWidth() * 0.5;
			
			if (vec.size() > 0)
			{
				targetPosV = ClipByPlace(vec.get(vec.size()-1), targetPosV, placeRadius);
			}
			else
			{
				targetPosV = ClipByPlace(sourcePosV, targetPosV, placeRadius);
			}
		}
		if (source instanceof Transition)
		{
			Dimension sd = layoutConn.getSize(source);
			if (vec.size() > 0)
			{
				sourcePosV = ClipByTransition(vec.get(0), sourcePosV, sd.getWidth(), sd.getHeight());
			}
			else
			{
				sourcePosV = ClipByTransition(targetPosV, sourcePosV, sd.getWidth(), sd.getHeight());
			}
		}
		if (target instanceof Transition)
		{
			Dimension sd = layoutConn.getSize(target);
			if (vec.size() > 0)
			{
				targetPosV = ClipByTransition(vec.get(vec.size()-1), targetPosV, sd.getWidth(), sd.getHeight());
			}
			else
			{
				targetPosV = ClipByTransition(sourcePosV, targetPosV, sd.getWidth(), sd.getHeight());
			}
		}
		
		vec.add(0, sourcePosV);
		vec.add(targetPosV);
		
		return vec;
	}

	private List<JoglVectord3> getEdgePoints(PetrinetSimplifier simp, List<JoglVectord3> vec, PetrinetNode source,
			PetrinetNode target) 
	{
		JoglVectord3 sourcePosV = simp.getPosition(source);
		JoglVectord3 targetPosV = simp.getPosition(target);
		
		double placeRadius = 12.5;
		
		if (source instanceof Place)
		{
			Place sourceP = (Place)source;
			placeRadius = simp.getSize(sourceP).getWidth() * 0.5;
			
			if (vec.size() > 0)
			{
				sourcePosV = ClipByPlace(vec.get(0), sourcePosV, placeRadius);
			}
			else
			{
				sourcePosV = ClipByPlace(targetPosV, sourcePosV, placeRadius);
			}
		}
		else if (source instanceof Transition)
		{
			Dimension sd = simp.getSize(source);
			if (vec.size() > 0)
			{
				sourcePosV = ClipByTransition(vec.get(0), sourcePosV, sd.getWidth(), sd.getHeight());
			}
			else
			{
				sourcePosV = ClipByTransition(targetPosV, sourcePosV, sd.getWidth(), sd.getHeight());
			}
		}
		if (target instanceof Place)
		{
			Place targetP = (Place)target;
			placeRadius = simp.getSize(targetP).getWidth() * 0.5;
			
			if (vec.size() > 0)
			{
				targetPosV = ClipByPlace(vec.get(vec.size()-1), targetPosV, placeRadius);
			}
			else
			{
				targetPosV = ClipByPlace(sourcePosV, targetPosV, placeRadius);
			}
		}
		else if (target instanceof Transition)
		{
			Dimension sd = simp.getSize(target);
			if (vec.size() > 0)
			{
				targetPosV = ClipByTransition(vec.get(vec.size()-1), targetPosV, sd.getWidth(), sd.getHeight());
			}
			else
			{
				targetPosV = ClipByTransition(sourcePosV, targetPosV, sd.getWidth(), sd.getHeight());
			}
		}
		
		vec.add(0, sourcePosV);
		vec.add(targetPosV);
		
		return vec;
	}

	private List<JoglVectord3> getEdgePoints2(PetrinetSimplifier simp, List<JoglVectord3> vec, JSOPnNode source,
			JSOPnNode target) 
	{
		JoglVectord3 sourcePosV = source.getPosition();
		JoglVectord3 targetPosV = target.getPosition();
		
		double placeRadius = 12.5;
		
		if (source instanceof JSOPnPlace)
		{
			JSOPnPlace sourceP = (JSOPnPlace)source;
			placeRadius = sourceP.getSize().getX() * 0.5;
			
			if (vec.size() > 0)
			{
				sourcePosV = ClipByPlace(vec.get(0), sourcePosV, placeRadius);
			}
			else
			{
				sourcePosV = ClipByPlace(targetPosV, sourcePosV, placeRadius);
			}
		}
		else if (source instanceof JSOPnTransition)
		{
			JSOPnTransition sourceT = (JSOPnTransition)source;
			JoglVectord2 sd = sourceT.getSize();
			if (vec.size() > 0)
			{
				sourcePosV = ClipByTransition(vec.get(0), sourcePosV, sd.getX(), sd.getY());
			}
			else
			{
				sourcePosV = ClipByTransition(targetPosV, sourcePosV, sd.getX(), sd.getY());
			}
		}
		if (target instanceof JSOPnPlace)
		{
			JSOPnPlace targetP = (JSOPnPlace)target;
			placeRadius = targetP.getSize().getX() * 0.5;
			
			if (vec.size() > 0)
			{
				targetPosV = ClipByPlace(vec.get(vec.size()-1), targetPosV, placeRadius);
			}
			else
			{
				targetPosV = ClipByPlace(sourcePosV, targetPosV, placeRadius);
			}
		}
		else if (target instanceof JSOPnTransition)
		{
			JSOPnTransition targetT = (JSOPnTransition)target;
			JoglVectord2 sd = targetT.getSize();
			if (vec.size() > 0)
			{
				targetPosV = ClipByTransition(vec.get(vec.size()-1), targetPosV, sd.getX(), sd.getY());
			}
			else
			{
				targetPosV = ClipByTransition(sourcePosV, targetPosV, sd.getX(), sd.getY());
			}
		}
		
		vec.add(0, sourcePosV);
		vec.add(targetPosV);
		
		return vec;
	}
	
	//clip against place if b is center of place
	private JoglVectord3 ClipByPlace(JoglVectord3 a, JoglVectord3 b, double radius)
	{
		JoglVectord3 dir = JoglVectord3.Subtract(b, a);
		double length = dir.getMagnitude();
		dir = JoglVectord3.Normal(dir);
		
		JoglVectord3 result = JoglVectord3.Add(a, JoglVectord3.Multiply(dir, length - radius));
		
		return result;
	}
	
	//clip against transition if b is center of transition
	private JoglVectord3 ClipByTransition(JoglVectord3 a, JoglVectord3 b, double w, double h)
	{
		JoglVectord3 dir = JoglVectord3.Subtract(a, b);
		dir = JoglVectord3.Normal(dir);

		double dot1 = JoglVectord3.Dot(new JoglVectord3(1, 0, 0), dir);
		double dot2 = JoglVectord3.Dot(new JoglVectord3(0, 0, 1), dir);
		
		double l1 = Double.POSITIVE_INFINITY;
		double l2 = Double.POSITIVE_INFINITY;
		JoglVectord3 p1 = a;
		JoglVectord3 p2 = a;
		
		if (dot1 != 0)
		{
			double invdot1 = 1.0 / dot1;
			
			p1 = JoglVectord3.Multiply(dir, Math.abs(invdot1) * w * 0.5);
			
			l1 = p1.getMagnitude();
			
		}
		if (dot2 != 0)
		{
			double invdot2 = 1.0 / dot2;
			
			p2 = JoglVectord3.Multiply(dir, Math.abs(invdot2) * h * 0.5);
			
			l2 = p2.getMagnitude();
		}
		
		JoglVectord3 result = null;
		if (l1 < l2)
			result = JoglVectord3.Add(b, p1);
		else
			result = JoglVectord3.Add(b, p2);
		
		return result;
	}
	
	public JoglMatrixd4x4 getLocalTransform()
	{
		if (localTransform == null)
		{
			JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateIdentity();
	
			JoglMatrixd4x4 mscale = JoglMatrixd4x4.CreateScale(getScale());
			JoglMatrixd4x4 mtransl = JoglMatrixd4x4.CreateTranslation(getOffset());
			transform = JoglMatrixd4x4.Multiply(transform, mscale);
			transform = JoglMatrixd4x4.Multiply(transform, mtransl);
			
			localTransform = transform;
			
			return transform;
		}
		else
		{
			return localTransform;
		}
	}
	
	public JoglMatrixd4x4 getTransform()
	{
		if (globalTransform == null)
		{
			JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateIdentity();
			for (JSOSubNet snet : ancestors)
			{
				transform = JoglMatrixd4x4.Multiply(transform, snet.getLocalTransform());
			}
			
			globalTransform = transform;
			
			return transform;
		}
		else
		{
			return globalTransform;
		}
	}
	
	public void ClearTransform()
	{
		globalTransform = null;
		
		//update pedestal if any
		if (ped != null)
		{
			ped.bNeedsUpdate = true;
		}
		
		//invalidate global transforms of child subnets
		for (JSOSubNet snet : subnets)
		{
			snet.ClearTransform();
		}
	}

	private JoglFrameBufferObject fbo = null;
	private JoglRenderTargetFramebuffer fbTarget = null;
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{	
		JSOSubNetStackRenderer snr = new JSOSubNetStackRenderer(this);
		snr.DrawSubnet(pipe, pipe.scene);
		//DrawModel(pipe);
	}

	public void DrawModel(GraphicsPipeline pipe)
	{
		if (!bLoaded)
		{
			loadMeshes(pipe);
		}
		
		if (bVisible && batcher != null)
		{	
			batcher.DrawObject(pipe);
			
			for (JoglSceneObject jso : elements)
			{
				if (jso instanceof JSOPnEdgeVertex)
				{
					jso.DrawObject(pipe);
				}
			}
		}
	}

	public void DrawModelMask(GraphicsPipeline pipe) 
	{
		JoglRenderTarget target = pipe.target;
		
		if (fbo != null)
		{
			if ((fbo.getWidth() != target.getWidth()) || (fbo.getHeight() != target.getHeight()) )
			{
				fbo.Dispose(pipe);
				fbo = null;
			}
		}

		if (fbo == null)
		{	
			fbo = new JoglFrameBufferObject(pipe, target.getWidth(), target.getHeight());
			fbo.Bind(pipe);
			fbo.AttachDepthBuffer(pipe, fbo.CreateRboAttachment(pipe, GL.GL_DEPTH_COMPONENT32));
			fbo.AttachColorBuffer0(pipe, fbo.CreateTboAttachment(pipe, GL.GL_RGBA8, 0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE));

			if (!fbo.isFramebufferComplete())
			{
				System.out.println("Framebuffer failed to initialize.");
			}
			
			fbo.Unbind(pipe);
			
			fbTarget = new JoglRenderTargetFramebuffer(fbo);
		}
		
		fbo.Bind(pipe);
		
		fbTarget.FitViewport(pipe);
		
		//clear fbo
		fbTarget.Clear(pipe);
		
		GraphicsPipeline newPipe = new GraphicsPipeline(pipe);
		newPipe.target = fbTarget;
		
		DrawMask(newPipe);
		
		fbo.Unbind(pipe);
		
		JoglResourceManager.AddResource("PetriNetScreenMask", getMaskTexture());

		target.Bind(pipe);
		target.FitViewport(pipe);
	}
	
	public void DrawMask(GraphicsPipeline pipe)
	{
		if (!bLoaded)
		{
			loadMeshes(pipe);
		}
		
		if (bVisible && batcher != null)
		{
			batcher.DrawMask(pipe);
		}
	}
	
	public void Export(JoglScene scene, OBJWriter out)
	{
		if (batcher != null)
		{
			batcher.Export(scene, out);
		}
		
		if (parent != null && ped != null)
		{
			ped.ExportPedestal(out);
		}
	}
	
	public void Dispose(GraphicsPipeline pipe)
	{
		elements.clear();
		
		if (fbo != null)
		{
			fbo.Dispose(pipe);
			fbo = null;
		}

		if (batcher != null)
		{
			batcher.Dispose(pipe);
		}
	}
}
