package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.nio.ShortBuffer;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialColorOnly;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialDepthOnly;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialSubnetPedestalOutline;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialSubnetPedestalSurface;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderSubNetPedestal;
import org.processmining.processprofiler3d.psen.fileio.obj.OBJWriter;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.collision.CannotCreateBoundingBoxException;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.collision.JoglAABBFactory;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.common.nio.Buffers;
import com.jogamp.opengl.GL2;

public class JSOSubNetPedestal extends JoglSceneObject
{	
	private static MaterialDepthOnly matDO = new MaterialDepthOnly();
	private static MaterialColorOnly matCO = new MaterialColorOnly();
	private static MaterialSubnetPedestalSurface matPedSurf = new MaterialSubnetPedestalSurface();
	private static MaterialSubnetPedestalOutline matPedLine = new MaterialSubnetPedestalOutline();
	
	private static int exportNum = 0;
	private static double pedestalSpacing = 10.0;
	
	public boolean bNeedsUpdate = true;
	
	private JSOSubNet snet;
	
	public JSOSubNetPedestal(JSOSubNet snet)
	{
		this.snet = snet;
	}
	
	public JoglAABB getAABB()
	{
		JoglAABBFactory aabbFact = new JoglAABBFactory();
		
		if (verts == null)
		{
			verts = GenerateGeometry();
		}
		
		for (JoglVectord3 vx : verts)
		{
			JoglVectord3 vxz = snet.relativeToAbsolute(vx);
			aabbFact.addPoint(vxz);
		}
		
		JoglAABB aabb = null;
		try
		{
			aabb = aabbFact.produce();
		} 
		catch (CannotCreateBoundingBoxException e)
		{
			System.out.println("Could not create bounding box for subnet pedestal!");
			e.printStackTrace();
		}
		
		return aabb;
	}
	
	public JoglVectord4 getPedestalMaxRect()
	{
		JoglVectord3 origin = snet.getOrigin();
		origin = snet.getParentNet().relativeToAbsolute(origin);
		
		//calculate subnet offset given scale 1.0	
		JoglVectord2 dim = snet.GetPetrinetSize();
		double pedwidth = (dim.getX() * 0.5) + pedestalSpacing;
		double pedheight = (dim.getY() * 0.5) + pedestalSpacing;
		
		double xoff = snet.getDynamicOffset().getX();
		double zoff = snet.getDynamicOffset().getZ();
			
		return new JoglVectord4(origin.getX() + xoff, origin.getZ() + zoff, pedwidth, pedheight);	
	}
	
	public Vector<JoglVectord3> getPedestalRectPointsAbsolute2()
	{
		JoglVectord3 origin = snet.getOrigin();
		origin = snet.getParentNet().relativeToAbsolute(origin);
		
		//calculate subnet offset given scale 1.0
		
		JoglVectord2 dim = snet.GetPetrinetSize();
		double pedwidth = (dim.getX() * 0.5) + pedestalSpacing;
		double pedheight = (dim.getY() * 0.5) + pedestalSpacing;
		double peddepth = (JSOPetriNet.subnetOffset.getY() - 5.0);
		
		double xoff = snet.getDynamicOffset().getX();
		double zoff = snet.getDynamicOffset().getZ();

		JoglVectord3 pedOrigin = new JoglVectord3(origin.getX(), origin.getY(), origin.getZ());
		double pedx = origin.getX() + xoff;
		double pedy = origin.getY() + peddepth;
		double pedz = origin.getZ() + zoff;
		JoglVectord3 ped1 = new JoglVectord3(pedx + pedwidth, pedy, pedz + pedheight);
		JoglVectord3 ped2 = new JoglVectord3(pedx - pedwidth, pedy, pedz + pedheight);
		JoglVectord3 ped3 = new JoglVectord3(pedx - pedwidth, pedy, pedz - pedheight);
		JoglVectord3 ped4 = new JoglVectord3(pedx + pedwidth, pedy, pedz - pedheight);

		//transform from relative parent space into absolute
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
		lst.add(pedOrigin);
		lst.add(ped1);
		lst.add(ped2);
		lst.add(ped3);
		lst.add(ped4);
		
		return lst;
	}
	
	public void DrawColor(GraphicsPipeline pipe)
	{
		pipe.PushPipelineState(matCO);
		DrawObject(pipe);
		pipe.PopPipelineState(matCO);
	}
	
	public void DrawDepth(GraphicsPipeline pipe)
	{
		pipe.PushPipelineState(matDO);
		DrawObject(pipe);
		pipe.PopPipelineState(matDO);
	}
	
	public void UpdateGeometry(GraphicsPipeline pipe)
	{
		if (bNeedsUpdate)
		{
			verts = GenerateGeometry();
			
			if (vaoOutline != null)
			{
				vaoOutline.UpdateDataBufferJV3(pipe, 0, verts);
			}
			if (vaoSurface != null)
			{
				vaoSurface.UpdateDataBufferJV3(pipe, 0, verts);
			}
			bNeedsUpdate = false;
		}
	}
	
	private Vector<JoglVectord3> GenerateGeometry()
	{
		double scale = snet.getScale();
		JoglVectord3 origin = snet.getOrigin();
		
		JoglVectord2 dim = snet.GetPetrinetSize();
		double pedwidth = (dim.getX() * 0.5 * scale) + pedestalSpacing;
		double pedheight = (dim.getY() * 0.5 * scale) + pedestalSpacing;
		double peddepth = (JSOPetriNet.subnetOffset.getY() - 5.0) * scale;
		
		double xoff = snet.getDynamicOffset().getX() * scale;
		double zoff = snet.getDynamicOffset().getZ() * scale;		
		
		JoglVectord3 pedOrigin = new JoglVectord3(origin.getX(), origin.getY(), origin.getZ());
		
		double pedx = origin.getX() + xoff;
		double pedy = origin.getY() + peddepth;
		double pedz = origin.getZ() + zoff;
		JoglVectord3 ped1 = new JoglVectord3(pedx + pedwidth, pedy, pedz + pedheight);
		JoglVectord3 ped2 = new JoglVectord3(pedx - pedwidth, pedy, pedz + pedheight);
		JoglVectord3 ped3 = new JoglVectord3(pedx - pedwidth, pedy, pedz - pedheight);
		JoglVectord3 ped4 = new JoglVectord3(pedx + pedwidth, pedy, pedz - pedheight);

		//transform from parent space into local space
		JoglVectord3 localOffset = JoglVectord3.Multiply(snet.getOffset(), -1);
		double localScale = 1.0 / snet.getScale();
		JoglVectord3 v0 = JoglVectord3.Add(localOffset, JoglVectord3.Multiply(pedOrigin, localScale));
		JoglVectord3 v1 = JoglVectord3.Add(localOffset, JoglVectord3.Multiply(ped1, localScale));
		JoglVectord3 v2 = JoglVectord3.Add(localOffset, JoglVectord3.Multiply(ped2, localScale));
		JoglVectord3 v3 = JoglVectord3.Add(localOffset, JoglVectord3.Multiply(ped3, localScale));
		JoglVectord3 v4 = JoglVectord3.Add(localOffset, JoglVectord3.Multiply(ped4, localScale));
		
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
		lst.add(v0);
		lst.add(v1);
		lst.add(v2);
		lst.add(v3);
		lst.add(v4);

		return lst;
	}
	
	public JoglMeshIndexBuffer getIndicesOutline()
	{
        ShortBuffer indata = Buffers.newDirectShortBuffer(16);

        indata.put((short)1);
        indata.put((short)2);
        
        indata.put((short)3);
        indata.put((short)4);
        
        indata.put((short)2);
        indata.put((short)3);
        
        indata.put((short)4);
        indata.put((short)1);
        
        indata.put((short)0);
        indata.put((short)1);
        indata.put((short)0);
        indata.put((short)2);
        
        indata.put((short)0);
        indata.put((short)3);
        indata.put((short)0);
        indata.put((short)4);

        indata.rewind();
        
        return new JoglMeshIndexBuffer(indata);
	}
	
	public JoglMeshIndexBuffer getIndicesSurface()
	{
        ShortBuffer indata = Buffers.newDirectShortBuffer(18);

        indata.put((short)0);
        indata.put((short)1);
        indata.put((short)2);

        indata.put((short)0);
        indata.put((short)3);
        indata.put((short)4);
        
        indata.put((short)0);
        indata.put((short)2);
        indata.put((short)3);
        
        indata.put((short)0);
        indata.put((short)4);
        indata.put((short)1);
        
        indata.put((short)2);
        indata.put((short)1);
        indata.put((short)3);
        
        indata.put((short)4);
        indata.put((short)3);
        indata.put((short)1);

        indata.rewind();
        
        return new JoglMeshIndexBuffer(indata);
	}
	
	public JoglMesh CreateOutlineMesh()
	{
		JoglMesh jcm = new JoglMesh(GL2.GL_TRIANGLES);
		jcm.setVerts(verts);
		jcm.setIndexBuffer(getIndicesOutline());

		return jcm;
	}
	
	public JoglMesh CreateSurfaceMesh()
	{
		JoglMesh jcm = new JoglMesh(GL2.GL_TRIANGLES);
		jcm.setVerts(verts);
		jcm.setIndexBuffer(getIndicesSurface());

		return jcm;
	}
	
	private Vector<JoglVectord3> verts = null;
	private IMeshDrawer vaoOutline = null;
	private IMeshDrawer vaoSurface = null;
	private static JoglShader shad = null;
	
	@Override
	public void DrawObject(GraphicsPipeline pipe) 
	{	
		if (shad == null)
		{
			shad = new JoglShaderSubNetPedestal();
			shad.Create(pipe);
		}
		
		shad.Bind(pipe);
		
		//draw pedestal outline
		DrawOutline(pipe, verts);

		//draw pedestal surface
		DrawSurface(pipe, verts);
		
		shad.Unbind(pipe);
	}

	private void DrawOutline(GraphicsPipeline pipe, Vector<JoglVectord3> lst) 
	{	
		pipe.PushPipelineState(matPedLine);
		
		if (vaoOutline == null)
		{
			vaoOutline = new VAODrawer();
			vaoOutline.LoadMesh(pipe, CreateOutlineMesh());
		}
		
		vaoOutline.DrawLines(pipe);
		
		pipe.PopPipelineState(matPedLine);
	}

	private void DrawSurface(GraphicsPipeline pipe, Vector<JoglVectord3> lst) 
	{
		pipe.PushPipelineState(matPedSurf);
		
		if (vaoSurface == null)
		{
			vaoSurface = new VAODrawer();
			vaoSurface.LoadMesh(pipe, CreateSurfaceMesh());
		}
		
		vaoSurface.Draw(pipe);
		
		pipe.PopPipelineState(matPedSurf);
	}
	
	public void ExportPedestal(OBJWriter out)
	{
		if (verts == null)
		{
			verts = GenerateGeometry();
		}
	
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
		lst.add(snet.relativeToAbsolute(verts.get(0)));
		lst.add(snet.relativeToAbsolute(verts.get(1)));
		lst.add(snet.relativeToAbsolute(verts.get(2)));
		lst.add(snet.relativeToAbsolute(verts.get(3)));
		lst.add(snet.relativeToAbsolute(verts.get(4)));
		
		out.WriteNewObject("Pedestal " + exportNum);
		
		Vector<Integer> verts = new Vector<Integer>();
		
		//sides
		verts.add(out.WriteVertex(lst.get(0)));
		verts.add(out.WriteVertex(lst.get(1)));
		verts.add(out.WriteVertex(lst.get(2)));
		
		verts.add(out.WriteVertex(lst.get(0)));
		verts.add(out.WriteVertex(lst.get(3)));
		verts.add(out.WriteVertex(lst.get(4)));
		
		verts.add(out.WriteVertex(lst.get(0)));
		verts.add(out.WriteVertex(lst.get(2)));
		verts.add(out.WriteVertex(lst.get(3)));
		
		verts.add(out.WriteVertex(lst.get(0)));
		verts.add(out.WriteVertex(lst.get(4)));
		verts.add(out.WriteVertex(lst.get(1)));

		//top
		verts.add(out.WriteVertex(lst.get(2)));
		verts.add(out.WriteVertex(lst.get(1)));
		verts.add(out.WriteVertex(lst.get(3)));

		verts.add(out.WriteVertex(lst.get(4)));
		verts.add(out.WriteVertex(lst.get(3)));
		verts.add(out.WriteVertex(lst.get(1)));
		
		out.WriteUseMtl("ModelPedestal");
		
		//write indices (starting with 1)
		for (int i = 0; i < verts.size() - 2; i = i + 3)
		{
			out.WriteFaceTri(verts.get(i), verts.get(i+1), verts.get(i+2));
		}
		
		exportNum++;
	}

}
