package org.processmining.processprofiler3d.compviz.jogl.scene;

import org.processmining.processprofiler3d.compviz.scenemanagement.JoglModelReplayScene;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglPSENVisualizationScene;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JSOSubNetStackRenderer
{
	private JSOSubNet snet;
	private JSOGrid grid;
	private JSOPSENTransitionVisualizationLayer tvisx;
	private JSOPSENEdgeVisualizationLayer evisx;
	private JSOReplaya replaya = null;
	
	public JSOSubNetStackRenderer(JSOSubNet snet)
	{
		this.snet = snet;
	}
	
	public void DrawSubnet(GraphicsPipeline pipe, JoglScene scene) 
	{
		JoglCamera c = pipe.cam;
		
		snet.ped.UpdateGeometry(pipe);
		
		int netLevel = snet.getLevel();
		if (scene instanceof JoglPSENVisualizationScene)
		{
			tvisx = ((JoglPSENVisualizationScene)scene).getTransitionVisualizer(snet);
			evisx = ((JoglPSENVisualizationScene)scene).getEdgeVisualizer(snet);
			replaya = ((JoglPSENVisualizationScene)scene).getReplaya();
		}
		if (scene instanceof JoglModelReplayScene)
		{
			replaya = ((JoglModelReplayScene)scene).getReplaya();
		}

		pipe.transform.Push();
		JoglMatrixd4x4 transform = snet.getTransform();
		pipe.transform.MultMatrix(transform);
		
		snet.DrawModelMask(pipe);
		if (tvisx != null)
		{
			tvisx.DeterminVisibleElements(scene);
		}
		if (evisx != null)
		{
			evisx.DeterminVisibleElements(scene);
		}

		if (netLevel == 0)
		{
			grid = scene.findSceneObject(JSOGrid.class);
		}
		
		//if cam above grid
		boolean bCamAboveLayer = c.GetPosition().getY() >= snet.getSubnetYPos();
		if (bCamAboveLayer)
		{
			DrawFromAbove(pipe, netLevel);
		}
		else
		{
			DrawFromBelow(pipe, netLevel);
		}
		if (netLevel > 0)
		{
			snet.ped.DrawColor(pipe);
		}
		if (bCamAboveLayer && replaya != null)
		{
			replaya.DrawObject(pipe, snet);
		}
		
		pipe.transform.Pop();
	}

	private void DrawFromAbove(GraphicsPipeline pipe, int netLevel)
	{
		if (evisx != null && evisx.HasVisibleElements())
		{
			evisx.DrawBottom(pipe, true);
		}
		
		if (netLevel == 0 && grid != null)
		{
			grid.DrawGridWithParams(pipe, false, true, true);
		}
		snet.DrawModel(pipe);

		//replaya.DrawObject(pipe, snet);

		if (tvisx != null && tvisx.HasVisibleElements())
		{
			tvisx.DrawTop(pipe, false);
		}
		if (evisx != null && evisx.HasVisibleElements())
		{
			evisx.DrawTop(pipe, false);
		}
	}

	private void DrawFromBelow(GraphicsPipeline pipe, int netLevel)
	{
		if (replaya != null)
		{
			replaya.DrawObject(pipe, snet); //Draw before so that is visible through pedestal
		}
		
		if (evisx != null && evisx.HasVisibleElements())
		{
			evisx.DrawTop(pipe, true);
		}
		if (tvisx != null && tvisx.HasVisibleElements())
		{
			tvisx.DrawTop(pipe, true);
		}
		snet.DrawModel(pipe);
		if (netLevel == 0 && grid != null)
		{
			grid.DrawGridWithParams(pipe, false, true, true);
		}
		if (evisx != null && evisx.HasVisibleElements())
		{
			evisx.DrawBottom(pipe, false);
		}
	}

}
