package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglColoredMesh;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglOpenVRMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderOpenVRMesh;
import org.processmining.processprofiler3d.compviz.vr.HzdOpenVR;
import org.processmining.processprofiler3d.compviz.vr.HzdVRControllerState;
import org.processmining.processprofiler3d.compviz.vr.VRController;
import org.processmining.processprofiler3d.compviz.vr.jna.VRControllerState;
import org.processmining.processprofiler3d.compviz.vr.jna.VRRenderModel;
import org.processmining.processprofiler3d.compviz.vr.jna.VRRenderModelTexture;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;
import org.processmining.visualisation3d.utils.IVRAdapter;

import com.jogamp.opengl.GL2;

/*
 * A quad to render in the scene
 */
public class JSOVRController extends JoglSceneObject 
{
	private JoglVectord3 position = new JoglVectord3(0,0,0);
	private JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateIdentity();

	public int controllerId = 0;
	private IVRAdapter vrAdapter = null;
	
	public JSOVRController(IVRAdapter vra, int id)
	{
		vrAdapter = vra;
		controllerId = id;
	}
	
	public JoglVectord3 getPosition()
	{
		return position;
	}
	
	public void setPosition(double x, double y, double z)
	{
		position = new JoglVectord3(x,y,z);
		transform = JoglMatrixd4x4.CreateTranslation(x, y, z);
	}
	
	public void setPosition(JoglVectord3 v)
	{
		position = v;
		transform = JoglMatrixd4x4.CreateTranslation(v.getX(), v.getY(), v.getZ());
	}
	
	private Vector<JoglVectord3> GenerateGeometry()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
		float scale = 0.01f;
		
		lst.add(new JoglVectord3(-scale, scale, scale));   // Top Left
		lst.add(new JoglVectord3( scale, scale, scale));   // Top Right
		lst.add(new JoglVectord3( scale,-scale, scale));   // Bottom Right
		lst.add(new JoglVectord3(-scale,-scale, scale));   // Bottom Left
		
		// Draw Another Quad
		lst.add(new JoglVectord3(-scale, scale,-scale));   // Top Left
		lst.add(new JoglVectord3(-scale, scale, scale));   // Top Right
		lst.add(new JoglVectord3(-scale,-scale, scale));   // Bottom Right
		lst.add(new JoglVectord3(-scale,-scale,-scale));   // Bottom Left

		lst.add(new JoglVectord3( scale, scale, -scale));   // Top Left
		lst.add(new JoglVectord3(-scale, scale, -scale));   // Top Right
		lst.add(new JoglVectord3(-scale,-scale, -scale));   // Bottom Right
		lst.add(new JoglVectord3( scale,-scale, -scale));   // Bottom Left

		// Draw Another Quad
		lst.add(new JoglVectord3(scale, scale, scale));   // Top Left
		lst.add(new JoglVectord3(scale, scale,-scale));   // Top Right
		lst.add(new JoglVectord3(scale,-scale,-scale));   // Bottom Right
		lst.add(new JoglVectord3(scale,-scale, scale));   // Bottom Left

		return lst;
	}
	
	private Vector<JoglVectord3> GenerateColors()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();

		lst.add(new JoglVectord3(0.0f, 0.0f, 1.0f));   // set the color of the quad
		lst.add(new JoglVectord3(0.0f, 0.0f, 1.0f));   // set the color of the quad
		lst.add(new JoglVectord3(0.0f, 0.0f, 1.0f));   // set the color of the quad
		lst.add(new JoglVectord3(0.0f, 0.0f, 1.0f));   // set the color of the quad
		
		// Draw Another Quad
		lst.add(new JoglVectord3(1.0f, 0.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(1.0f, 0.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(1.0f, 0.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(1.0f, 0.0f, 0.0f));   // set the color of the quad

		lst.add(new JoglVectord3(0.0f, 1.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(0.0f, 1.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(0.0f, 1.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(0.0f, 1.0f, 0.0f));   // set the color of the quad

		// Draw Another Quad
		lst.add(new JoglVectord3(1.0f, 1.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(1.0f, 1.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(1.0f, 1.0f, 0.0f));   // set the color of the quad
		lst.add(new JoglVectord3(1.0f, 1.0f, 0.0f));   // set the color of the quad

		return lst;
	}
	
	public JoglColoredMesh CreateMesh()
	{
		JoglColoredMesh jcm = new JoglColoredMesh(GL2.GL_QUADS);
		jcm.setVerts(GenerateGeometry());
		jcm.setColors3(GenerateColors());
		jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jcm.getNumVerts()));
		
		return jcm;
	}
	
	private IMeshDrawer vao = null;
	private JoglShader shad = null;
	private JoglTexture tex = null;
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		//update position
		updateControllerPos(pipe);
		
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);
		
		if (shad == null)
		{
			shad = new JoglShaderOpenVRMesh();
			shad.Create(pipe);
		}
		
		if (vao != null && shad != null)
		{
			if (tex != null)
			{
				tex.Bind(pipe);
			}
			
			shad.Bind(pipe);
			vao.Draw(pipe);
			shad.Unbind(pipe);

			if (tex != null)
			{
				tex.Unbind(pipe);
			}
		}
		
		pipe.transform.Pop();
	}

	private int vrTexId = -1;
	
	private VRController getController()
	{
		if (vrAdapter instanceof HzdOpenVR)
		{
			HzdOpenVR hov = (HzdOpenVR)vrAdapter;
			VRController myController = null;
			if (controllerId == 1)
			{
				myController = hov.controller1;
			}
			else if (controllerId == 2)
			{
				myController = hov.controller2;
			}
			return myController;
		}
		return null;
	}
	
	public HzdVRControllerState GetControllerState()
	{
		VRController myController = getController();
		if (myController != null)
		{
			VRControllerState vcs = myController.GetControllerState();
			return new HzdVRControllerState(vcs);
		}
		return null;
	}
	
	private void updateControllerPos(GraphicsPipeline pipe)
	{
		if (vrAdapter instanceof HzdOpenVR)
		{
			HzdOpenVR hov = (HzdOpenVR)vrAdapter;
			VRController myController = getController();
			
			if (myController != null)
			{
				if (vao == null)
				{
					VRRenderModel rm = hov.LoadMesh(myController.meshName);
					if (rm != null)
					{
						JoglOpenVRMesh ovm = rm.loadMesh(); 
						if (ovm != null)
						{
							vao = new VAODrawer();
							vao.LoadMesh(pipe, ovm);
						}
						
						vrTexId = rm.diffuseTexId;
						
						hov.FreeMesh(rm);
					}
				}
				
				if (tex == null && vrTexId >= 0)
				{
					VRRenderModelTexture vrtex = hov.LoadTexture(vrTexId);
					if (vrtex != null)
					{
						tex = vrtex.loadTexture(pipe);
					}				
				}

				setPosition(myController.trans.hmdPosWS);
				
				JoglMatrixd4x4 rotationMatrix = myController.getWSRotationMat();
				//JoglVectord3 scaleVec = myController.getScale();
				
				transform = JoglMatrixd4x4.Multiply(transform, rotationMatrix);
			}
			else
			{
				return;
			}
		}
	}
	
	public JoglVectord3 getForward()
	{
		VRController myController = getController();
		if (myController != null)
		{
			return myController.trans.hmdLookDirWS;
		}
		return JoglVectord3.ZERO;
	}
	
	public JoglVectord3 getLeft()
	{
		VRController myController = getController();
		if (myController != null)
		{
			return myController.trans.hmdLeftDirWS;
		}
		return JoglVectord3.ZERO;
	}
	
	public JoglVectord3 getUp()
	{
		VRController myController = getController();
		if (myController != null)
		{
			return myController.trans.hmdUpDirWS;
		}
		return JoglVectord3.ZERO;
	}

}
