package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedInterpolationMesh;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSODataVisMagnetLine;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglVectord3;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;

public class MagnetLineMeshCreator 
{
	public String hash = "";
	
	private JoglTexturedMesh meshTop;
	private JoglTexturedMesh meshCenter;
	private JoglTexturedMesh meshBottom;

	private double[] lineTop;
	private double[] line0;
	private double[] lineBottom;
	
	public MagnetLineMeshCreator(JoglVectord3 startPos, JoglVectord3 endPos, double maxHeight, int numPoints)
	{
		hash = createHash(startPos, endPos, maxHeight, numPoints);
		
		Vector<JoglVectord3> vec1 = JSODataVisMagnetLine.getMagnetLinePoints(startPos, endPos, maxHeight, numPoints);
		Vector<JoglVectord3> vec3 = JSODataVisMagnetLine.getMagnetLinePoints(startPos, endPos, 0.0, numPoints);
		Vector<JoglVectord3> vec5 = JSODataVisMagnetLine.getMagnetLinePoints(startPos, endPos, -maxHeight, numPoints);

		//create line data
		InitLineData(vec1, vec3, vec5);
		
		//create mesh data
		InitMeshData(vec1, vec3, vec5);
	}

	private String createHash(JoglVectord3 startPos, JoglVectord3 endPos, double maxHeight, int numPoints)
	{
		String hash = "(" + startPos.getX() + "," + startPos.getY() + "," + startPos.getZ() + ")";
		hash += "(" + endPos.getX() + "," + endPos.getY() + "," + endPos.getZ() + ")";
		hash += "," + maxHeight;
		hash += "," + numPoints;
		
		return hash;
	}
	
	public boolean NeedsUpdate(JoglVectord3 startPos, JoglVectord3 endPos, double maxHeight, int numPoints)
	{
		String hashx = createHash(startPos, endPos, maxHeight, numPoints);
		return !hash.equals(hashx);
	}

	private void InitLineData(Vector<JoglVectord3> vec1, Vector<JoglVectord3> vec3, Vector<JoglVectord3> vec5) 
	{
		lineTop = new double[vec1.size() * 3];
		int idx = 0;
		for (int i = 0; i < vec1.size(); i++)
		{
			JoglVectord3 v = vec1.get(i);
			lineTop[idx] = v.getX();
			lineTop[idx+1] = v.getY();
			lineTop[idx+2] = v.getZ();
			
			idx = idx + 3;
		}

		line0 = new double[vec3.size() * 3];
		idx = 0;
		for (int i = 0; i < vec3.size(); i++)
		{
			JoglVectord3 v = vec3.get(i);
			line0[idx] = v.getX();
			line0[idx+1] = v.getY();
			line0[idx+2] = v.getZ();
			
			idx = idx + 3;
		}

		lineBottom = new double[vec5.size() * 3];
		idx = 0;
		for (int i = 0; i < vec5.size(); i++)
		{
			JoglVectord3 v = vec5.get(i);
			lineBottom[idx] = v.getX();
			lineBottom[idx+1] = v.getY();
			lineBottom[idx+2] = v.getZ();
			
			idx = idx + 3;
		}
	}

	private void InitMeshData(Vector<JoglVectord3> vec1, Vector<JoglVectord3> vec3, Vector<JoglVectord3> vec5) 
	{
		JoglTexturedMesh m1 = new JoglTexturedMesh(GL2.GL_TRIANGLES);
		m1.setVerts(JSODataVisMagnetLine.GenerateGeometry3D(vec1, 1));
		m1.setTexCoords(JSODataVisMagnetLine.GenerateTexCoords3D(vec1, 1));
		m1.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(m1.getNumVerts()));

		JoglTexturedMesh m3 = new JoglTexturedMesh(GL2.GL_TRIANGLES);
		m3.setVerts(JSODataVisMagnetLine.GenerateGeometry3D(vec3, 1));
		m3.setTexCoords(JSODataVisMagnetLine.GenerateTexCoords3D(vec3, 1));
		m3.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(m3.getNumVerts()));

		JoglTexturedMesh m5 = new JoglTexturedMesh(GL2.GL_TRIANGLES);
		m5.setVerts(JSODataVisMagnetLine.GenerateGeometry3D(vec5, 1));
		m5.setTexCoords(JSODataVisMagnetLine.GenerateTexCoords3D(vec5, 1));
		m5.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(m5.getNumVerts()));
		
		meshTop = m1;
		meshCenter = m3;
		meshBottom = m5;
	}

	public JoglTexturedInterpolationMesh getMesh(double val)
	{
		JoglTexturedInterpolationMesh m = new JoglTexturedInterpolationMesh(GL.GL_TRIANGLES, meshTop.getNumVerts());
		m.setTexCoords(meshTop.getTexCoords());
		if (val == 0.0)
		{
			m.setVerts(meshCenter.getVerts());
			m.setVerts2(meshCenter.getVerts());
			m.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(m.getNumVerts()));
			return m;
		}
		else if (val > 0.0)
		{
			if (val >= 1.0)
			{
				m.setVerts(meshTop.getVerts());
				m.setVerts2(meshTop.getVerts());
				m.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(m.getNumVerts()));
				return m;
			}
			else
			{
				m.setVerts(meshCenter.getVerts());
				m.setVerts2(meshTop.getVerts());
				m.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(m.getNumVerts()));
				return m;
			}
		}
		else
		{
			if (val <= -1.0)
			{
				m.setVerts(meshBottom.getVerts());
				m.setVerts2(meshBottom.getVerts());
				m.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(m.getNumVerts()));
				return m;
			}
			else
			{
				m.setVerts(meshCenter.getVerts());
				m.setVerts2(meshBottom.getVerts());	
				m.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(m.getNumVerts()));	
				return m;
			}
		}
	}
	
//	public JoglTexturedMesh getMesh(double val)
//	{
//		if (val == 0.0)
//		{
//			return meshCenter;
//		}
//		else if (val > 0.0)
//		{
//			if (val >= 1.0)
//			{
//				return meshTop;
//			}
//			else
//			{
//				JoglTexturedMesh m = InterpolateMesh(meshCenter, meshTop, val);
//				
//				return m;
//			}
//		}
//		else
//		{
//			if (val <= -1.0)
//			{
//				return meshBottom;
//			}
//			else
//			{
//				JoglTexturedMesh m = InterpolateMesh(meshCenter, meshBottom, val * -1);
//				
//				return m;
//			}
//		}
//	}

	private JoglTexturedMesh InterpolateMesh(JoglTexturedMesh m1, JoglTexturedMesh m2, double val) 
	{
		if (m1.getNumVerts() != m2.getNumVerts())
		{
			System.out.println("Can't interpolate meshes: vertex num differs!");
			return null;
		}
		
		//JoglTexturedMesh m = new JoglTexturedMesh(GL.GL_TRIANGLES, m1.getNumVerts());
		JoglTexturedMesh m = new JoglTexturedMesh(GL2.GL_TRIANGLES);
		
		double[] verts1 = m1.getVerts();
		double[] verts2 = m2.getVerts();
		
		double[] nverts = new double[verts1.length];
		for (int i = 0; i < verts1.length; i++)
		{
			nverts[i] = verts1[i] + ((verts2[i] - verts1[i]) * val);
		}
		m.setVerts(nverts);
		m.setTexCoords(m1.getTexCoords());
		m.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(m.getNumVerts()));
		return m;
	}
	
	public Vector<JoglVectord3> getPoints(double val)
	{
		Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();
		
		if (val == 0.0)
		{
			for (int i = 0; i < line0.length; i = i + 3)
			{
				vlist.add(new JoglVectord3(line0[i], line0[i+1], line0[i+2]));
			}
			return vlist;
		}
		else if (val > 0.0)
		{
			if (val >= 1.0)
			{
				for (int i = 0; i < lineTop.length;  i = i + 3)
				{
					vlist.add(new JoglVectord3(lineTop[i], lineTop[i+1], lineTop[i+2]));
				}
				return vlist;
			}
			else
			{
				for (int i = 0; i < lineTop.length;  i = i + 3)
				{
					double x = line0[i] + ((lineTop[i] - line0[i]) * val);
					double y = line0[i+1] + ((lineTop[i+1] - line0[i+1]) * val);
					double z = line0[i+2] + ((lineTop[i+2] - line0[i+2]) * val);
					vlist.add(new JoglVectord3(x, y, z));
				}
				return vlist;
			}
		}
		else
		{
			if (val <= -1.0)
			{
				for (int i = 0; i < lineBottom.length;  i = i + 3)
				{
					vlist.add(new JoglVectord3(lineBottom[i], lineBottom[i+1], lineBottom[i+2]));
				}
				return vlist;
			}
			else
			{
				double vali = val * -1.0;
				for (int i = 0; i < lineBottom.length;  i = i + 3)
				{
					double x = line0[i] + ((lineBottom[i] - line0[i]) * vali);
					double y = line0[i+1] + ((lineBottom[i+1] - line0[i+1]) * vali);
					double z = line0[i+2] + ((lineBottom[i+2] - line0[i+2]) * vali);
					vlist.add(new JoglVectord3(x, y, z));
				}
				return vlist;
			}
		}
	}
}
