package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.nio.ShortBuffer;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.interactions.replay.ITokenLane;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialTokenCube;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglColor3;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.common.nio.Buffers;
import com.jogamp.opengl.GL2;

public class TokenCube extends JoglSceneObject
{
	private ITokenLane lane = null;
	private JoglVectord3 position = new JoglVectord3(0,0,0);
	private JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateIdentity();
	private Vector<JoglVectord3> cubeVertexCoords;

	public double factor = 0.0;
	public JSOPnPlace place = null;
	
	private int tokenCohort = -1;
	
	public double startTime = 0;
	public double endTime = 1.0;
	public double timeScale = 1.0; //timespan from start to end
	
	public static float tokenScale = 4.0f;

	private static Vector<JoglVectord3> colors = null;
	
	public TokenCube()
	{
		if (colors == null)
		{
			colors = new Vector<JoglVectord3>();
			colors.add(JoglColor3.RED);
			colors.add(JoglColor3.GREEN);
			colors.add(JoglColor3.BLUE);
			colors.add(JoglColor3.YELLOW);
			colors.add(JoglColor3.CYAN);
			colors.add(JoglColor3.MAGENTA);
			colors.add(JoglColor3.WHITE);
			colors.add(JoglColor3.BLACK);
		}
		
		cubeVertexCoords = GenerateCubeVertexCoords(tokenScale);
	}
	
	public static JoglVectord3 getTokenColor(int cohort)
	{
		if (colors == null)
		{
			colors = new Vector<JoglVectord3>();
			colors.add(JoglColor3.RED);
			colors.add(JoglColor3.GREEN);
			colors.add(JoglColor3.BLUE);
			colors.add(JoglColor3.YELLOW);
			colors.add(JoglColor3.CYAN);
			colors.add(JoglColor3.MAGENTA);
			colors.add(JoglColor3.WHITE);
			colors.add(JoglColor3.BLACK);
		}
		return colors.get(cohort);
	}
	
	public int getTokenCohort()
	{
		return tokenCohort;
	}
	
	public void setTokenCohort(int cohortid)
	{
		tokenCohort = cohortid;
	}
	
	public ITokenLane getTokenLane()
	{
		return lane;
	}
	
	public void setTokenLane(ITokenLane tl)
	{
		lane = tl;
	}
	
	public void setPosition(double x, double y, double z)
	{
		position = new JoglVectord3(x,y,z);
		transform = JoglMatrixd4x4.CreateTranslation(x, y, z);
	}
	
	public void setPosition(JoglVectord3 v)
	{
		position = v;
		transform = JoglMatrixd4x4.CreateTranslation(v.getX(), v.getY(), v.getZ());
	}
	
	public JoglMatrixd4x4 getTransform()
	{
		return transform;
	}
	
	public void Tick(double time)
	{
		double t = Math.min(timeScale, Math.max(0, time - startTime)); //clamp time between start and end time 
		factor = t;
		
		if (timeScale > 0)
		{
			setPosition(lane.getPosition(factor / timeScale));
		}
		else
		{
			setPosition(lane.getPosition(1.0));
		}
	}
	
	private IMeshDrawer vaoDrawer;
	//private JoglShader shad;
	private static MaterialTokenCube matTC = new MaterialTokenCube();
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
//		if (shad == null)
//		{
//			shad = getShader(pipe);
//		}
		
		if (vaoDrawer == null)
		{
			getVAODrawer(pipe);
		}
		
		pipe.PushPipelineState(matTC);
		matTC.DrawColor(getTokenColor(tokenCohort));
		//gl.glDepthMask(true);
		//gl.glDisable(GL.GL_DEPTH_TEST);
//		gl.glEnable(GL.GL_CULL_FACE);
//		gl.glCullFace(GL.GL_FRONT); //looks like winding order in the model is wrong, might want to fix that eventually

		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);

		//pipe.setColor(getTokenColor(tokenCohort));   // set the color of the quad (red)

		//draw a cube
		vaoDrawer.Draw(pipe);
		
		pipe.transform.Pop();

//		gl.glDisable(GL.GL_CULL_FACE);
		//gl.glEnable(GL.GL_DEPTH_TEST);
		//gl.glDepthMask(false);
		pipe.PopPipelineState(matTC);
	}

	private void getVAODrawer(GraphicsPipeline pipe) 
	{
		JoglMesh mesh = new JoglMesh(GL2.GL_QUADS, 24);
		mesh.setVerts(cubeVertexCoords);
		mesh.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(mesh.getNumVerts()));
		
		vaoDrawer = new VAODrawer();
		vaoDrawer.LoadMesh(pipe, mesh);
	}

//	private static JoglShader getShader(GraphicsPipeline pipe) 
//	{
//		JoglShader s = null;
//		if (JoglResourceManager.HasResource("GenericVAOShader"))
//		{
//			s = (JoglShader)JoglResourceManager.GetResource("GenericVAOShader");
//		}
//		else
//		{
//			s = new JoglShader("/org/processmining/processprofiler3d/compviz/jogl/resources", "vaoShaderVS","vaoShaderFS");
//			s.Create(pipe);
//			
//			JoglResourceManager.AddResource("GenericVAOShader", s);
//		}
//		return s;
//	}
	
	public static IMeshDrawer GenerateVAO(GraphicsPipeline pipe, float scale)
	{
		JoglMesh mx = new JoglMesh(GL2.GL_TRIANGLES);
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
		
		//draw back quad
		lst.add(new JoglVectord3(-scale, scale, scale));   // Top Left 1
		lst.add(new JoglVectord3( scale, scale, scale));   // Top Right 2
		lst.add(new JoglVectord3( scale,-scale, scale));   // Bottom Right 3
		lst.add(new JoglVectord3(-scale,-scale, scale));   // Bottom Left 4
		lst.add(new JoglVectord3(-scale, scale,-scale));   // Top Left 5
		lst.add(new JoglVectord3(-scale,-scale,-scale));   // Bottom Left 6
		lst.add(new JoglVectord3( scale, scale, -scale));   // Top Left 7
		lst.add(new JoglVectord3( scale,-scale, -scale));   // Bottom Left 8

		mx.setVerts(lst);
		
		ShortBuffer indata = Buffers.newDirectShortBuffer(36);
        indata.put((short)1);
        indata.put((short)0);
        indata.put((short)2);
        indata.put((short)3);
        indata.put((short)2);
        indata.put((short)0);

        indata.put((short)0);
        indata.put((short)4);
        indata.put((short)3);
        indata.put((short)5);
        indata.put((short)3);
        indata.put((short)4);

        indata.put((short)4);
        indata.put((short)6);
        indata.put((short)5);
        indata.put((short)7);
        indata.put((short)5);
        indata.put((short)6);

        indata.put((short)6);
        indata.put((short)1);
        indata.put((short)7);
        indata.put((short)2);
        indata.put((short)7);
        indata.put((short)1);

        indata.put((short)6);
        indata.put((short)4);
        indata.put((short)1);
        indata.put((short)0);
        indata.put((short)1);
        indata.put((short)4);

        indata.put((short)3);
        indata.put((short)5);
        indata.put((short)2);
        indata.put((short)7);
        indata.put((short)2);
        indata.put((short)5);
        mx.setIndexBuffer(new JoglMeshIndexBuffer(indata));
		
        IMeshDrawer vao = new VAODrawer();
        vao.LoadMesh(pipe, mx);
        
		return vao;
	}
	
	//this data is still has wrong winding direction for the polygons, the VAO has been fixed already!
	public static Vector<JoglVectord3> GenerateCubeVertexCoords(float scale)
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
		
		//draw back quad
		lst.add(new JoglVectord3(-scale, scale, scale));   // Top Left 1
		lst.add(new JoglVectord3( scale, scale, scale));   // Top Right 2
		lst.add(new JoglVectord3( scale,-scale, scale));   // Bottom Right 3
		lst.add(new JoglVectord3(-scale,-scale, scale));   // Bottom Left 4
		
		//draw right quad
		lst.add(new JoglVectord3(-scale, scale,-scale));   // Top Left 5
		lst.add(new JoglVectord3(-scale, scale, scale));   // Top Right 1
		lst.add(new JoglVectord3(-scale,-scale, scale));   // Bottom Right 4
		lst.add(new JoglVectord3(-scale,-scale,-scale));   // Bottom Left 6

		//draw front quad
		lst.add(new JoglVectord3( scale, scale, -scale));   // Top Left 7
		lst.add(new JoglVectord3(-scale, scale, -scale));   // Top Right 5
		lst.add(new JoglVectord3(-scale,-scale, -scale));   // Bottom Right 6
		lst.add(new JoglVectord3( scale,-scale, -scale));   // Bottom Left 8

		//draw left quad
		lst.add(new JoglVectord3(scale, scale, scale));   // Top Left 2
		lst.add(new JoglVectord3(scale, scale,-scale));   // Top Right 7
		lst.add(new JoglVectord3(scale,-scale,-scale));   // Bottom Right 8
		lst.add(new JoglVectord3(scale,-scale, scale));   // Bottom Left 3

		//draw top quad
		lst.add(new JoglVectord3(-scale, scale,-scale));   // Top Left 5
		lst.add(new JoglVectord3( scale, scale,-scale));   // Top Right 7
		lst.add(new JoglVectord3( scale, scale, scale));   // Bottom Right 2
		lst.add(new JoglVectord3(-scale, scale, scale));   // Bottom Left 1

		//draw bottom quad
		lst.add(new JoglVectord3(-scale,-scale,-scale));   // Top Left 6
		lst.add(new JoglVectord3(-scale,-scale, scale));   // Top Right 4
		lst.add(new JoglVectord3( scale,-scale, scale));   // Bottom Right 3
		lst.add(new JoglVectord3( scale,-scale,-scale));   // Bottom Left 8
		
		return lst;
	}
}
