package org.processmining.processprofiler3d.compviz.jogl.scene;

import java.nio.ShortBuffer;
import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.PP3DCohort;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.interactions.replay.ITokenLane;
import org.processmining.processprofiler3d.compviz.jogl.interactions.replay.TokenParticle;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialTokenCubeOpt;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialTokenCubeOpt2;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglResourceManager;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.common.nio.Buffers;
import com.jogamp.opengl.GL2;

public class TokenDrawer extends JoglSceneObject
{	
	public IMeshDrawer vaoDrawer = null;
	public float tokenScale = 4.0f;

	private JSOCohortColorManager colorManager = null;
	//private Vector<JoglVectord3> colors = null;
	
	public TokenDrawer(JSOCohortColorManager cman)
	{
		colorManager = cman;
//		colors = new Vector<JoglVectord3>();
//		colors.add(JoglColor3.RED);
//		colors.add(JoglColor3.GREEN);
//		colors.add(JoglColor3.BLUE);
//		colors.add(JoglColor3.YELLOW);
//		colors.add(JoglColor3.CYAN);
//		colors.add(JoglColor3.MAGENTA);
//		colors.add(JoglColor3.WHITE);
//		colors.add(JoglColor3.BLACK);
	}
	
	public JoglVectord3 getTokenColor(int cohort)
	{
		//return colors.get(cohort);
		return colorManager.getColorForCohort(cohort);
	}

	private JoglShader getShader(GraphicsPipeline pipe) 
	{
		JoglShader s = null;
		if (JoglResourceManager.HasResource("GenericVAOShader"))
		{
			s = (JoglShader)JoglResourceManager.GetResource("GenericVAOShader");
		}
		else
		{
			s = new JoglShader("/org/processmining/processprofiler3d/compviz/jogl/resources", "vaoShaderVS","vaoShaderFS");
			s.Create(pipe);
			
			JoglResourceManager.AddResource("GenericVAOShader", s);
		}
		return s;
	}
	
	private static MaterialTokenCubeOpt matTCO = new MaterialTokenCubeOpt();
	private static MaterialTokenCubeOpt2 matTCO2 = new MaterialTokenCubeOpt2();
	
	public void DrawCubesOpt(GraphicsPipeline pipe, Vector<TokenCube> cubes)
	{
//		if (shad == null)
//		{
//			shad = getShader(gl);
//		}
		
		if (vaoDrawer == null)
		{
			//JoglMesh mesh = new JoglMesh(GL2.GL_QUADS, 24);
			//mesh.setVerts(GenerateCubeVertexCoords(2.0f));
			
			//vaoStatic = new VAODrawer();
			//vaoStatic.LoadQuadMesh(gl, mesh);
			vaoDrawer = GenerateVAO(pipe, 2.0f);
		}
		
		pipe.PushPipelineState(matTCO);
		
		//gl.glDepthMask(true);
		//gl.glDisable(GL.GL_DEPTH_TEST);
//		gl.glEnable(GL.GL_CULL_FACE);
//		gl.glCullFace(GL.GL_BACK);

		vaoDrawer.Bind(pipe);
		for (TokenCube tc : cubes)
		{
			ITokenLane lane = tc.getTokenLane();
			double laneScale = lane.getScale();
			JoglVectord3 laneOffset = lane.getOffset();
			
			pipe.transform.Push();
			pipe.transform.Scaled(laneScale, laneScale, laneScale);
			pipe.transform.Translated(laneOffset);
			pipe.transform.MultMatrix(tc.getTransform());
	
			//pipe.setColor(getTokenColor(tc.getTokenCohort()));   // set the color of the token
			matTCO.DrawColor(getTokenColor(tc.getTokenCohort()));
	
			//draw a cube
			vaoDrawer.DrawSlim(pipe);
			
			pipe.transform.Pop();
		}
		vaoDrawer.Unbind(pipe);
		
//		gl.glDisable(GL.GL_CULL_FACE);
		//gl.glEnable(GL.GL_DEPTH_TEST);
		//gl.glDepthMask(false);
		pipe.PopPipelineState(matTCO);
	}
	
	public void DrawCubesOpt2(GraphicsPipeline pipe, Vector<TokenCube> cubes)
	{
		if (cubes.isEmpty())
			return;
		
//		if (shad == null)
//		{
//			shad = getShader(gl);
//		}
		
		if (vaoDrawer == null)
		{
			//JoglMesh mesh = new JoglMesh(GL2.GL_QUADS, 24);
			//mesh.setVerts(GenerateCubeVertexCoords(2.0f));
			
			//vaoStatic = new VAODrawer();
			//vaoStatic.LoadQuadMesh(gl, mesh);
			vaoDrawer = GenerateVAO(pipe, tokenScale);
		}
		
		pipe.PushPipelineState(matTCO2);
		
		//gl.glDepthMask(true);
//		gl.glDisable(GL.GL_DEPTH_TEST);
//		gl.glEnable(GL.GL_CULL_FACE);
//		gl.glCullFace(GL.GL_BACK);

		ITokenLane lane = cubes.get(0).getTokenLane();
		double laneScale = lane.getScale();
		JoglVectord3 laneOffset = lane.getOffset();
		
		pipe.transform.Push();
		pipe.transform.Scaled(laneScale, laneScale, laneScale);
		pipe.transform.Translated(laneOffset);
		
		vaoDrawer.Bind(pipe);
		for (TokenCube tc : cubes)
		{
			
			pipe.transform.Push();
			pipe.transform.MultMatrix(tc.getTransform());
	
			//pipe.setColor(getTokenColor(tc.getTokenCohort()));   // set the color of the token
			matTCO2.DrawColor(getTokenColor(tc.getTokenCohort()));
	
			//draw a cube
			vaoDrawer.DrawSlim(pipe);
			
			pipe.transform.Pop();
		}
		vaoDrawer.Unbind(pipe);
		
		pipe.transform.Pop();
//		gl.glDisable(GL.GL_CULL_FACE);
//		gl.glEnable(GL.GL_DEPTH_TEST);
		//gl.glDepthMask(false);
		pipe.PopPipelineState(matTCO2);
	}
	
	public void DrawCubesOpt3(GraphicsPipeline pipe, Vector<ITokenLane> lanes)
	{
		if (lanes.isEmpty())
			return;
		
//		if (shad == null)
//		{
//			shad = getShader(gl);
//		}
		
		if (vaoDrawer == null)
		{
			//JoglMesh mesh = new JoglMesh(GL2.GL_QUADS, 24);
			//mesh.setVerts(GenerateCubeVertexCoords(2.0f));
			
			//vaoStatic = new VAODrawer();
			//vaoStatic.LoadQuadMesh(gl, mesh);
			vaoDrawer = GenerateVAO(pipe, tokenScale);
		}
		
		pipe.PushPipelineState(matTCO2);
		
		//gl.glDepthMask(true);
//		gl.glDisable(GL.GL_DEPTH_TEST);
//		gl.glEnable(GL.GL_CULL_FACE);
//		gl.glCullFace(GL.GL_BACK);

		
		ITokenLane lane = lanes.get(0);
		double laneScale = lane.getScale();
		JoglVectord3 laneOffset = lane.getOffset();
		
		pipe.transform.Push();
		pipe.transform.Scaled(laneScale, laneScale, laneScale);
		pipe.transform.Translated(laneOffset);
		
		vaoDrawer.Bind(pipe);
		for (ITokenLane l : lanes)
		{
			Vector<TokenCube> cubes = l.getTokens();
			for (TokenCube tc : cubes)
			{	
				pipe.transform.Push();
				pipe.transform.MultMatrix(tc.getTransform());
		
				//pipe.setColor(getTokenColor(tc.getTokenCohort()));   // set the color of the token
				matTCO2.DrawColor(getTokenColor(tc.getTokenCohort()));
		
				//draw a cube
				vaoDrawer.DrawSlim(pipe);
				
				pipe.transform.Pop();
			}
		}
		vaoDrawer.Unbind(pipe);
		
		pipe.transform.Pop();
//		gl.glDisable(GL.GL_CULL_FACE);
//		gl.glEnable(GL.GL_DEPTH_TEST);
		//gl.glDepthMask(false);
		pipe.PopPipelineState(matTCO2);
	}
	
	public void DrawParticles(GraphicsPipeline pipe, Vector<TokenParticle> parts)
	{
		if (parts.isEmpty())
			return;
	
//		if (shad == null)
//		{
//			shad = getShader(gl);
//		}
		
		if (vaoDrawer == null)
		{
			//JoglMesh mesh = new JoglMesh(GL2.GL_QUADS, 24);
			//mesh.setVerts(GenerateCubeVertexCoords(2.0f));
			
			//vaoStatic = new VAODrawer();
			//vaoStatic.LoadQuadMesh(gl, mesh);
			vaoDrawer = GenerateVAO(pipe, tokenScale);
		}
		
		pipe.PushPipelineState(matTCO2);
		
		//gl.glDepthMask(true);
//		gl.glDisable(GL.GL_DEPTH_TEST);
//		gl.glEnable(GL.GL_CULL_FACE);
//		gl.glCullFace(GL.GL_BACK);

		
//		ITokenLane lane = lanes.get(0);
//		double laneScale = lane.getScale();
//		JoglVectord3 laneOffset = lane.getOffset();
//		
		pipe.transform.Push();
//		
		vaoDrawer.Bind(pipe);
		for (TokenParticle tc : parts)
		{
			if (tc.startPos != null)
			{
				double factor = tc.factor;
				JoglVectord3 pos = JoglVectord3.Add(tc.startPos, JoglVectord3.Multiply(tc.dir, factor));
				pipe.transform.Push();
				pipe.transform.Translated(pos);
		
				//pipe.setColor(getTokenColor(0));   // set the color of the token
				matTCO2.DrawColor(getTokenColor(0));
		
				//draw a cube
				vaoDrawer.DrawSlim(pipe);
				
				pipe.transform.Pop();
			}
		}
		vaoDrawer.Unbind(pipe);
		
		pipe.transform.Pop();
//		gl.glDisable(GL.GL_CULL_FACE);
//		gl.glEnable(GL.GL_DEPTH_TEST);
		//gl.glDepthMask(false);
		pipe.PopPipelineState(matTCO2);
	}
	
	//public HashSet<Integer> visCohorts = null;
	
	public void DrawParticles2(GraphicsPipeline pipe, final JSOSubNet net, Vector<TokenParticle> parts)
	{
		JoglCamera c = pipe.cam;
		
		if (parts.isEmpty())
			return;
		
//		if (shad == null)
//		{
//			shad = getShader(gl);
//		}
		
		if (vaoDrawer == null)
		{
			vaoDrawer = GenerateVAO(pipe, tokenScale);
		}

		//determine visible tokens
		//to minimize processing project camera into 2D plane of subnet and then sort tokens by distance in two-dimensional space
		final JoglVectord3 camPos = c.GetPosition();
		JoglVectord2 camPosP2D = new JoglVectord2(camPos.getX(), camPos.getZ());;
		for (JSOSubNet snet : net.getAncestors())
		{
			camPosP2D = JoglVectord2.Add(JoglVectord2.Multiply(camPosP2D, 1.0 / snet.getScale()), new JoglVectord2(-snet.getOffset().getX(), -snet.getOffset().getZ()));
		}
		Vector<TokenParticle> vis = new Vector<TokenParticle>();
		for (TokenParticle tc : parts)
		{
			if (tc.startPos != null && tc.cohort >= 0)
			{	
				double factor = tc.factor;
				if (factor >= 0 && factor <= 1.0)
				{
					PP3DCohort pc = colorManager.cohorts.get(tc.cohort);
					
					if (pc.bVisible)
					{
						vis.add(tc);
						
						JoglVectord3 pos = JoglVectord3.Add(tc.startPos, JoglVectord3.Multiply(tc.dir, tc.factor));
						JoglVectord2 pos2D = new JoglVectord2(pos.getX(), pos.getZ());
			        	double dist = JoglVectord2.Subtract(camPosP2D, pos2D).getMagnitudeSquared();
			        	tc.tmpPos = pos;
			        	tc.tmpDist = dist;
					}
				}
			}
		}
		
		//TODO: issue here with "Comparison method violates its general contract!" when replay is at start of timeline
		//sort particles by distance to cam
		try
		{
			Collections.sort(vis, new Comparator<TokenParticle>() 
			{
		        @Override
		        public int compare(TokenParticle o1, TokenParticle o2) 
		        {
		            return (int)(o2.tmpDist - o1.tmpDist);
		        }
		    });
		}
		catch (Exception ex)
		{
			
		}
		
		pipe.PushPipelineState(matTCO2);

		vaoDrawer.Bind(pipe);
		for (TokenParticle tc : vis)
		{
			JoglVectord3 pos = tc.tmpPos;
			pipe.transform.Push();
			pipe.transform.Translated(pos);
	
			//pipe.setColor(getTokenColor(tc.cohort));   // set the color of the token
			matTCO2.DrawColor(getTokenColor(tc.cohort));
	
			//draw a cube
			vaoDrawer.DrawSlim(pipe);
			
			pipe.transform.Pop();
		}
		vaoDrawer.Unbind(pipe);
		
		pipe.PopPipelineState(matTCO2);
	}
	
	public void Dispose()
	{
		if (vaoDrawer != null)
		{
			vaoDrawer.Dispose();
			vaoDrawer = null;
		}
	}
	
	public static IMeshDrawer GenerateVAO(GraphicsPipeline pipe, float scale)
	{
		JoglMesh mx = new JoglMesh(GL2.GL_TRIANGLES);
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
		
		//draw back quad
		lst.add(new JoglVectord3(-scale, scale, scale));   // Top Left 1
		lst.add(new JoglVectord3( scale, scale, scale));   // Top Right 2
		lst.add(new JoglVectord3( scale,-scale, scale));   // Bottom Right 3
		lst.add(new JoglVectord3(-scale,-scale, scale));   // Bottom Left 4
		lst.add(new JoglVectord3(-scale, scale,-scale));   // Top Left 5
		lst.add(new JoglVectord3(-scale,-scale,-scale));   // Bottom Left 6
		lst.add(new JoglVectord3( scale, scale, -scale));   // Top Left 7
		lst.add(new JoglVectord3( scale,-scale, -scale));   // Bottom Left 8

		mx.setVerts(lst);
		
		ShortBuffer indata = Buffers.newDirectShortBuffer(36);
        indata.put((short)1);
        indata.put((short)0);
        indata.put((short)2);
        indata.put((short)3);
        indata.put((short)2);
        indata.put((short)0);

        indata.put((short)0);
        indata.put((short)4);
        indata.put((short)3);
        indata.put((short)5);
        indata.put((short)3);
        indata.put((short)4);

        indata.put((short)4);
        indata.put((short)6);
        indata.put((short)5);
        indata.put((short)7);
        indata.put((short)5);
        indata.put((short)6);

        indata.put((short)6);
        indata.put((short)1);
        indata.put((short)7);
        indata.put((short)2);
        indata.put((short)7);
        indata.put((short)1);

        indata.put((short)6);
        indata.put((short)4);
        indata.put((short)1);
        indata.put((short)0);
        indata.put((short)1);
        indata.put((short)4);

        indata.put((short)3);
        indata.put((short)5);
        indata.put((short)2);
        indata.put((short)7);
        indata.put((short)2);
        indata.put((short)5);
        mx.setIndexBuffer(new JoglMeshIndexBuffer(indata));
		
        IMeshDrawer vao = new VAODrawer();
        vao.LoadMesh(pipe, mx);
        
		return vao;
	}
	
	//this data is still has wrong winding direction for the polygons, the VAO has been fixed already!
	public static Vector<JoglVectord3> GenerateCubeVertexCoords(float scale)
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
		
		//draw back quad
		lst.add(new JoglVectord3(-scale, scale, scale));   // Top Left 1
		lst.add(new JoglVectord3( scale, scale, scale));   // Top Right 2
		lst.add(new JoglVectord3( scale,-scale, scale));   // Bottom Right 3
		lst.add(new JoglVectord3(-scale,-scale, scale));   // Bottom Left 4
		
		//draw right quad
		lst.add(new JoglVectord3(-scale, scale,-scale));   // Top Left 5
		lst.add(new JoglVectord3(-scale, scale, scale));   // Top Right 1
		lst.add(new JoglVectord3(-scale,-scale, scale));   // Bottom Right 4
		lst.add(new JoglVectord3(-scale,-scale,-scale));   // Bottom Left 6

		//draw front quad
		lst.add(new JoglVectord3( scale, scale, -scale));   // Top Left 7
		lst.add(new JoglVectord3(-scale, scale, -scale));   // Top Right 5
		lst.add(new JoglVectord3(-scale,-scale, -scale));   // Bottom Right 6
		lst.add(new JoglVectord3( scale,-scale, -scale));   // Bottom Left 8

		//draw left quad
		lst.add(new JoglVectord3(scale, scale, scale));   // Top Left 2
		lst.add(new JoglVectord3(scale, scale,-scale));   // Top Right 7
		lst.add(new JoglVectord3(scale,-scale,-scale));   // Bottom Right 8
		lst.add(new JoglVectord3(scale,-scale, scale));   // Bottom Left 3

		//draw top quad
		lst.add(new JoglVectord3(-scale, scale,-scale));   // Top Left 5
		lst.add(new JoglVectord3( scale, scale,-scale));   // Top Right 7
		lst.add(new JoglVectord3( scale, scale, scale));   // Bottom Right 2
		lst.add(new JoglVectord3(-scale, scale, scale));   // Bottom Left 1

		//draw bottom quad
		lst.add(new JoglVectord3(-scale,-scale,-scale));   // Top Left 6
		lst.add(new JoglVectord3(-scale,-scale, scale));   // Top Right 4
		lst.add(new JoglVectord3( scale,-scale, scale));   // Bottom Right 3
		lst.add(new JoglVectord3( scale,-scale,-scale));   // Bottom Left 8
		
		return lst;
	}
}
