package org.processmining.processprofiler3d.compviz.jogl.scene.datavis;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglColoredMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderBarTest;
import org.processmining.processprofiler3d.psen.fileio.obj.OBJWriter;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.opengl.GL2;

public class JSODataVisBarChart extends JoglSceneObject 
{	
	private IMeshDrawer vao;
	private static JoglShader shad;
	
	private float width = 100.0f;
	private float height = 100.0f;
	
	private JoglVectord3 position = new JoglVectord3(0,0,0);
	private JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateIdentity();

	private Vector<Float> valX = new Vector<Float>();
	private Vector<Float> valY = new Vector<Float>();
	private Vector<JoglVectord3> colX = new Vector<JoglVectord3>();
	
	public void setData(Vector<Float> vX, Vector<Float> vY, Vector<JoglVectord3> c)
	{
		valX = vX;
		valY = vY;
		colX = c;
		
		if (vao != null)
		{
			vao.Dispose();
		}
	}
	
	public JoglVectord3 getPosition()
	{
		return position;
	}
	
	public void setPosition(double x, double y, double z)
	{
		position = new JoglVectord3(x,y,z);
		transform = JoglMatrixd4x4.CreateTranslation(x, y, z);
	}
	
	public void setSize(float w, float h)
	{
		width = w;
		height = h;
		
		if (vao != null)
		{
			vao.Dispose();
		}
	}

	private float getMaxValX()
	{
		float max = Float.NEGATIVE_INFINITY;
		for (int i = 0; i < valX.size(); i++)
		{
			max = Math.max(max, valX.get(i));
		}
		return max;
	}

	private float getMaxValY()
	{
		float max = Float.NEGATIVE_INFINITY;
		for (int i = 0; i < valY.size(); i++)
		{
			max = Math.max(max, valY.get(i));
		}
		return max;
	}
	
	private Vector<JoglVectord3> GenerateGeometry()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();

		float scaleX = width * 0.5f;
		
		float totalWidth = 0.0f;
		for (int i = 0; i < valX.size(); i++)
		{
			float pieceWidth = 0;
			float pieceHeight = 0;
			if (getMaxValX() > 0)
			{
				pieceWidth = valX.get(i) * width;
			}
			else
			{
				pieceWidth = (1.0f / valX.size()) * width;
				//System.out.println("PieceWidth = " + pieceWidth);
			}
			
			pieceHeight = valY.get(i) * height;
			//System.out.println("BarHeight: " + pieceHeight);
			
			//gl.glColor3d(colX.get(i).getX(), colX.get(i).getY(), colX.get(i).getZ()); // set the color of the quad
			lst.add(new JoglVectord3(-scaleX + totalWidth, 0.0f, 0.0f));   // Bottom Left
			lst.add(new JoglVectord3(-scaleX + totalWidth, pieceHeight, 0.0f));   // Top Left
			lst.add(new JoglVectord3(-scaleX + totalWidth + pieceWidth, pieceHeight, 0.0f));   // Top Right
			lst.add(new JoglVectord3(-scaleX + totalWidth + pieceWidth, 0.0f, 0.0f));   // Bottom Right
			
			totalWidth = totalWidth + pieceWidth;
		}
		
		return lst;
	}
	
	private Vector<JoglVectord3> GenerateColor()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();

		for (int i = 0; i < valX.size(); i++)
		{
			//gl.glColor3d(colX.get(i).getX(), colX.get(i).getY(), colX.get(i).getZ()); // set the color of the quad
			lst.add(new JoglVectord3(colX.get(i).getX(), colX.get(i).getY(), colX.get(i).getZ()));   // Bottom Left
			lst.add(new JoglVectord3(colX.get(i).getX(), colX.get(i).getY(), colX.get(i).getZ()));   // Top Left
			lst.add(new JoglVectord3(colX.get(i).getX(), colX.get(i).getY(), colX.get(i).getZ()));   // Top Right
			lst.add(new JoglVectord3(colX.get(i).getX(), colX.get(i).getY(), colX.get(i).getZ()));   // Bottom Right
		}
		
		return lst;
	}
	
	public JoglColoredMesh CreateMesh()
	{
		JoglColoredMesh jcm = new JoglColoredMesh(GL2.GL_QUADS);
		jcm.setVerts(GenerateGeometry());
		jcm.setColors3(GenerateColor());
		jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesQuad(jcm.getNumVerts()));
		
		return jcm;
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		if (valX.isEmpty()) //if there is no bar data then don't draw anything
		{
			return;
		}
		
//		float scaleX = width * 0.5f;
		
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);
		
		//draw a quad
//		gl.glBegin(GL2.GL_QUADS);       
//		
//		float totalWidth = 0.0f;
//		for (int i = 0; i < valX.size(); i++)
//		{
//			float pieceWidth = 0;
//			float pieceHeight = 0;
//			if (getMaxValX() > 0)
//			{
//				pieceWidth = valX.get(i) * width;
//			}
//			else
//			{
//				pieceWidth = (1.0f / valX.size()) * width;
//				//System.out.println("PieceWidth = " + pieceWidth);
//			}
//			
//			pieceHeight = valY.get(i) * height;
//			//System.out.println("BarHeight: " + pieceHeight);
//			
//			gl.glColor3d(colX.get(i).getX(), colX.get(i).getY(), colX.get(i).getZ()); // set the color of the quad
//			gl.glVertex3f(-scaleX + totalWidth, 0.0f, 0.0f);   // Bottom Left
//			gl.glVertex3f(-scaleX + totalWidth, pieceHeight, 0.0f);   // Top Left
//			gl.glVertex3f(-scaleX + totalWidth + pieceWidth, pieceHeight, 0.0f);   // Top Right
//			gl.glVertex3f(-scaleX + totalWidth + pieceWidth, 0.0f, 0.0f);   // Bottom Right
//			
//			totalWidth = totalWidth + pieceWidth;
//		}
//		
//		//done drawing the quad
//		gl.glEnd(); 
		
		if (vao == null)
		{
			vao = new VAODrawer();
			vao.LoadMesh(pipe, CreateMesh());
			
		}
		if (shad == null)
		{
			//shad = new JoglShaderTrajectory();
			shad = new JoglShaderBarTest();
			shad.Create(pipe);
		}
		
		shad.Bind(pipe);
		vao.Draw(pipe);
		shad.Unbind(pipe);

		pipe.transform.Pop();
	}
	
	public void Dispose()
	{
		if (vao != null)
		{
			vao.Dispose();
			vao = null;
		}
	}

	public void DrawObjectFast(GraphicsPipeline pipe, IMeshDrawer d)
	{
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);
		
		d.DrawSlim(pipe, tStart, tCount);
		
		pipe.transform.Pop();	
	}
	
	private int tStart = 0;
	private int tCount = 0;
	
	public void LoadMesh(JoglColoredMesh jcm)
	{
		tStart = (jcm.getNumVerts() / 4) * 6;
		
		//int i = 0;
		//for (JoglVectord3 v : points)
		{
			//JoglVectord3 col = colors.get(i);
			//gl.glColor3d(col.getX(), col.getY(), col.getZ());
			jcm.addColors3(GenerateColor());
				
			//i++;
			//gl.glVertex3d(v.getX(), v.getY(), v.getZ());
			jcm.addVerts(GenerateGeometry());
		}
		
		tCount = ((jcm.getNumVerts() / 4) * 6) - tStart;
	}
	
	public void Export(JSOSubNet snet, OBJWriter out)
	{
		float scaleX = width * 0.5f;
		
		//gl.glMultMatrixd(transform.GetData(), 0);
		
		//draw a quad       
		
		float totalWidth = 0.0f;
		for (int i = 0; i < valX.size(); i++)
		{
			float pieceWidth = 0;
			float pieceHeight = 0;
			if (getMaxValX() > 0)
			{
				pieceWidth = valX.get(i) * width;
			}
			else
			{
				pieceWidth = (1.0f / valX.size()) * width;
				//System.out.println("PieceWidth = " + pieceWidth);
			}
			
			pieceHeight = valY.get(i) * height;
			//System.out.println("BarHeight: " + pieceHeight);
			
			Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
			//gl.glColor3d(colX.get(i).getX(), colX.get(i).getY(), colX.get(i).getZ()); // set the color of the quad
			lst.add(new JoglVectord3(-scaleX + totalWidth, 0.0f, 0.0f));   // Bottom Left
			lst.add(new JoglVectord3(-scaleX + totalWidth, pieceHeight, 0.0f));   // Top Left
			lst.add(new JoglVectord3(-scaleX + totalWidth + pieceWidth, pieceHeight, 0.0f));   // Top Right
			lst.add(new JoglVectord3(-scaleX + totalWidth + pieceWidth, 0.0f, 0.0f));   // Bottom Right
			
			totalWidth = totalWidth + pieceWidth;

			out.WriteNewObject("BarChart " + 0);
			
			int id1 = out.WriteVertex(snet.relativeToAbsolute(JoglVectord3.Add(position, lst.get(0))));
			int id2 = out.WriteVertex(snet.relativeToAbsolute(JoglVectord3.Add(position, lst.get(1))));
			int id3 = out.WriteVertex(snet.relativeToAbsolute(JoglVectord3.Add(position, lst.get(2))));
			int id4 = out.WriteVertex(snet.relativeToAbsolute(JoglVectord3.Add(position, lst.get(3))));

			out.WriteUseMtl(getExportMaterialNameFromColor(colX.get(i)));
			
			out.WriteFaceTri(id1, id2, id3);
			out.WriteFaceTri(id3, id4, id1);
		}
		
		//done drawing the quad
	}
	
	public String getExportMaterialNameFromColor(JoglVectord3 col)
	{
		if (col.getX() == 1 && col.getY() == 0 && col.getZ() == 0)
		{
			return "CohortRed";
		}
		if (col.getX() == 0 && col.getY() == 1 && col.getZ() == 0)
		{
			return "CohortGreen";
		}
		if (col.getX() == 0 && col.getY() == 0 && col.getZ() == 1)
		{
			return "CohortBlue";
		}
		if (col.getX() == 1 && col.getY() == 1 && col.getZ() == 0)
		{
			return "CohortYellow";
		}
		if (col.getX() == 0 && col.getY() == 1 && col.getZ() == 1)
		{
			return "CohortCyan";
		}
		if (col.getX() == 1 && col.getY() == 0 && col.getZ() == 1)
		{
			return "CohortMagenta";
		}
		if (col.getX() == 1 && col.getY() == 1 && col.getZ() == 1)
		{
			return "CohortWhite";
		}
		if (col.getX() == 1 && col.getY() == 0 && col.getZ() == 1)
		{
			return "CohortBlack";
		}
		
		return "ModelPedestal";
	}
}
