package org.processmining.processprofiler3d.compviz.jogl.scene.datavis;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglLine3D;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedInterpolationMesh;
import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialMagnetLine;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.psen.fileio.obj.OBJWriter;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglCatmullRomSpline;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;

import com.jogamp.opengl.GL2;

public class JSODataVisMagnetLine extends JSODataVisMagnetLineBase implements Comparable<JSODataVisMagnetLine>
{
	private static MaterialMagnetLine matML = new MaterialMagnetLine();
	
	private Vector<JoglVectord3> colors;
	
	private JoglTexturedMesh mesh;
	private IMeshDrawer vaoDrawer;
	private JoglTexturedMesh meshHi;
	private IMeshDrawer vaoDrawerHi;
	
//	private double lineWidth = 1.0;
//	private double arrowWidth = 4;
//	private double arrowLength = 8;
	
	private double height = 0;
	private double length = 100.0;

	public JoglAABB getAABB()
	{
		return aabb;
	}
	
	public double getHeight()
	{
		return height;
	}
	
	public void setHeight(double h)
	{
		height = h;
	}
	
	public void setColor(float r, float g, float b)
	{
		colors = new Vector<JoglVectord3>();
		colors.add(new JoglVectord3(r,g,b));
	}
	
	public void setColor(JoglVectord3 col)
	{
		colors = new Vector<JoglVectord3>();
		colors.add(new JoglVectord3((float)col.getX(), (float)col.getY(), (float)col.getZ()));
	}
	
	public void addColor(JoglVectord3 col)
	{
		colors.add(new JoglVectord3((float)col.getX(), (float)col.getY(), (float)col.getZ()));
	}
	
	public JSODataVisMagnetLine(Vector<JoglVectord3> pts)
	{
		super(pts);
	}
	
	public void Dispose()
	{
//		if (vaoDrawer != null)
//		{
//			vaoDrawer.Dispose();
//		}
		if (vaoDrawerHi != null)
		{
			vaoDrawerHi.Dispose();
		}
	}
	
	public static Vector<JoglVectord3> getMagnetLinePoints(JoglVectord3 sourcePos, JoglVectord3 targetPos, double height, int numPoints)
	{
		JoglVectord3 dir = JoglVectord3.Subtract(targetPos, sourcePos);
		
		Vector<JoglVectord3> vec = new Vector<JoglVectord3>();
		vec.add(JoglVectord3.Add(JoglVectord3.Add(sourcePos, JoglVectord3.Multiply(dir, -0.5f)), new JoglVectord3(0, -height, 0)));
		vec.add(sourcePos);
		vec.add(JoglVectord3.Add(JoglVectord3.Add(sourcePos, JoglVectord3.Multiply(dir, 0.5f)), new JoglVectord3(0, height, 0)));
		vec.add(targetPos);
		vec.add(JoglVectord3.Add(JoglVectord3.Add(targetPos, JoglVectord3.Multiply(dir, 0.5f)), new JoglVectord3(0, -height, 0)));
		
		JoglCatmullRomSpline cspline = new JoglCatmullRomSpline(vec);

		Vector<JoglVectord3> vec2 = new Vector<JoglVectord3>();
		for (int i = 0; i < (numPoints - 1); i++)
		{
			vec2.add(cspline.GetPos(1.0 + (((double)i / (double)(numPoints - 1)) * 2.0)));
		}
		vec2.add(cspline.GetPos(3.0));
		
		return vec2;
	}

	public JoglAABB updateAABB()
	{
		double minx = Double.POSITIVE_INFINITY;
		double miny = Double.POSITIVE_INFINITY;
		double minz = Double.POSITIVE_INFINITY;
		double maxx = Double.NEGATIVE_INFINITY;
		double maxy = Double.NEGATIVE_INFINITY;
		double maxz = Double.NEGATIVE_INFINITY;
		
//		JoglAABB a = mesh.getAABB();
//		JoglVectord3 amin = a.getMin();
//		JoglVectord3 amax = a.getMax();
		
		//since mesh uses run-time blend in shader for vertex positions calculate AABB from raw geometry data instead
		JoglVectord3 startP = this.getStartPoint();
		JoglVectord3 endP = this.getEndPoint();
		
		minx = Math.min(minx, startP.getX());
		miny = Math.min(miny, startP.getY());
		minz = Math.min(minz, startP.getZ());
		
		maxx = Math.max(maxx, startP.getX());
		maxy = Math.max(maxy, startP.getY());
		maxz = Math.max(maxz, startP.getZ());
		
		minx = Math.min(minx, endP.getX());
		miny = Math.min(miny, endP.getY());
		minz = Math.min(minz, endP.getZ());
		
		maxx = Math.max(maxx, endP.getX());
		maxy = Math.max(maxy, endP.getY());
		maxz = Math.max(maxz, endP.getZ());

		miny = Math.min(miny, startP.getY() + height);
		maxy = Math.max(maxy, startP.getY() + height);
		
		//line has thickness, so make sure that box at least has same thickness in any case
		if (maxx - minx == 0)
		{
			minx = minx - 1.0f;
			maxx = maxx + 1.0f;
		}
		if (maxy - miny == 0)
		{
			miny = miny - 1.0f;
			maxy = maxy + 1.0f;
		}
		if (maxz - minz == 0)
		{
			minz = minz - 1.0f;
			maxz = maxz + 1.0f;
		}
		
		JoglVectord3 vmin = new JoglVectord3(minx, miny, minz);
		JoglVectord3 vmax = new JoglVectord3(maxx, maxy, maxz);

		JoglAABB aabb = new JoglAABB(vmin, vmax);
		
		return aabb;
	}

	public static Vector<JoglVectord3> GenerateGeometry3D(Vector<JoglVectord3> pts, double lineWidth) 
	{
		Vector<JoglVectord3> vlist = new Vector<JoglVectord3>();
		
		JoglLine3D lastLine = null;
		JoglLine3D curLine = null;

		//System.out.println("Magnet line 3D!");
		for (int i = 0; i < pts.size() - 1; i++)
		{
			JoglVectord3 p1 = pts.get(i);
			JoglVectord3 p2 = pts.get(i+1);

			JoglLine3D line = new JoglLine3D(p1, p2, lineWidth, lineWidth);
			
			for (int j = 0; j < line.triLst.size(); j++)
			{
				vlist.add(line.vertLst.get(line.triLst.get(j)));
			}
			
			lastLine = curLine;
			curLine = line;
			
			//fill in bends
			if (i > 0)
			{
				//suboptimal, potential overdraw and z-issues!
				double dot = JoglVectord3.Dot(curLine.getUpDir(), lastLine.getLineDir());
				if (dot > 0)
				{
					//System.out.println("Fill bends dot+!");
					//left
					vlist.add(curLine.vertLst.get(4));
					vlist.add(curLine.vertLst.get(0));
					vlist.add(lastLine.vertLst.get(6));
					
					//right
					vlist.add(curLine.vertLst.get(1));
					vlist.add(curLine.vertLst.get(4));
					vlist.add(lastLine.vertLst.get(6));
				}
				else
				{
					//provide degenerate triangles to enable interpolation while avoiding draw issues
					JoglVectord3 center = JoglVectord3.Add(lastLine.vertLst.get(6), JoglVectord3.Multiply(JoglVectord3.Subtract(curLine.vertLst.get(4), lastLine.vertLst.get(6)), 0.5));
					vlist.add(center);
					vlist.add(curLine.vertLst.get(0));
					vlist.add(center);
					
					//right
					vlist.add(curLine.vertLst.get(1));
					vlist.add(center);
					vlist.add(center);
					
				}
				
				if (dot < 0)
				{
					//left
					vlist.add(lastLine.vertLst.get(7));
					vlist.add(curLine.vertLst.get(0));
					vlist.add(curLine.vertLst.get(5));
					
					//right
					vlist.add(curLine.vertLst.get(5));
					vlist.add(curLine.vertLst.get(1));
					vlist.add(lastLine.vertLst.get(7));
				}
				else
				{
					//provide degenerate triangles to enable interpolation while avoiding draw issues
					JoglVectord3 center = JoglVectord3.Add(lastLine.vertLst.get(7), JoglVectord3.Multiply(JoglVectord3.Subtract(curLine.vertLst.get(5), lastLine.vertLst.get(7)), 0.5));		
					//left
					vlist.add(center);
					vlist.add(curLine.vertLst.get(0));
					vlist.add(center);
					
					//right
					vlist.add(center);
					vlist.add(curLine.vertLst.get(1));
					vlist.add(center);
				}
			}
		}
		
		return vlist;
	}

	public static Vector<JoglVectord2> GenerateTexCoords3D(Vector<JoglVectord3> pts, double lineWidth) 
	{
		Vector<JoglVectord2> vlist = new Vector<JoglVectord2>();
		
		JoglLine3D lastLine = null;
		JoglLine3D curLine = null;

		//System.out.println("Magnet line 3D!");
		for (int i = 0; i < pts.size() - 1; i++)
		{
			JoglVectord3 p1 = pts.get(i);
			JoglVectord3 p2 = pts.get(i+1);
			
			double texCoordMin = (double)i / (double)pts.size();
			double texCoordMax = (double)(i + 1) / (double)pts.size();

			JoglLine3D line = new JoglLine3D(p1, p2, lineWidth, lineWidth);
			
			for (int j = 0; j < line.triLst.size(); j++)
			{
				JoglVectord2 texCoord = line.texCoordLst.get(line.triLst.get(j));
				if (texCoord.getX() == 0.0)
				{
					vlist.add(new JoglVectord2(texCoordMin, texCoord.getY()));
				}
				else
				{
					vlist.add(new JoglVectord2(texCoordMax, texCoord.getY()));
				}
			}
			
			lastLine = curLine;
			curLine = line;
			
			//fill in bends
			if (i > 0)
			{
				//suboptimal, potential overdraw and z-issues!
				//double dot = JoglVectord3.Dot(curLine.getUpDir(), lastLine.getLineDir());
				//if (dot > 0)
				{
					JoglVectord2 tc = new JoglVectord2(texCoordMin, 0.0);
					
					//left
					vlist.add(tc);
					vlist.add(tc);
					vlist.add(tc);
					
					//right
					vlist.add(tc);
					vlist.add(tc);
					vlist.add(tc);
				}
				//else if (dot < 0)
				{
					JoglVectord2 tc = new JoglVectord2(texCoordMin, 0.0);
					
					//left
					vlist.add(tc);
					vlist.add(tc);
					vlist.add(tc);
					
					//right
					vlist.add(tc);
					vlist.add(tc);
					vlist.add(tc);
				}
			}
		}
		return vlist;
	}
	
	public void SetMesh(JoglTexturedMesh m)
	{
		mesh = m;
//		if (vaoDrawer != null)
//		{
//			vaoDrawer.Dispose();
//		}
//		vaoDrawer = null;
		
		aabb = updateAABB();
	}
	
	public void SetVAODrawer(IMeshDrawer vao)
	{
		vaoDrawer = vao;
	}
	
	public void SetMeshHi(JoglTexturedMesh m)
	{
		meshHi = m;
	}
	
	@Override
	public void Highlight()
	{
		JoglTexturedMesh m = new JoglTexturedMesh(GL2.GL_TRIANGLES);
		m.setVerts(JSODataVisMagnetLine.GenerateGeometry3D(points, 1.5));
		m.setTexCoords(JSODataVisMagnetLine.GenerateTexCoords3D(points, 1.5));
		m.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(m.getNumVerts()));
		
		SetMeshHi(m);
	}

	@Override
	public void Unhighlight()
	{
		meshHi = null;
		if (vaoDrawerHi != null)
		{
			vaoDrawerHi.Dispose();
		}
		vaoDrawerHi = null;
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		//draw selection highlight
//		if (meshHi != null)
//		{
//			JoglVectord4 col = JoglModelStyle.highlightColor;
//			gl.glColor3d(col.getX(), col.getY(), col.getZ());
//			DrawTrianglesHi(gl);
//		}

		pipe.PushPipelineState(matML);
		
		matML.DrawMagnetLine(length, height, colors);
		
		//Draw geometry	
		DrawTriangles(pipe);
		
		pipe.PopPipelineState(matML);
	}

	public void DrawMasked(GraphicsPipeline pipe)
	{
		//draw selection highlight
//		if (meshHi != null)
//		{
//			JoglVectord4 col = JoglModelStyle.highlightColor;
//			gl.glColor3d(col.getX(), col.getY(), col.getZ());
//			DrawTrianglesHi(gl);
//		}

		pipe.PushPipelineState(matML);
		matML.DrawMagnetLineMasked(length, height, colors);
		
		jsoPnet.getMaskTexture().Bind(pipe);
		
		DrawTriangles(pipe);
		
		pipe.PopPipelineState(matML);
		
		jsoPnet.getMaskTexture().Unbind(pipe);
	
	}
	
	public void DrawTriangles(GraphicsPipeline pipe)
	{
		//Draw geometry
		vaoDrawer.Draw(pipe);
	}
	
	public void DrawTrianglesHi(GraphicsPipeline pipe)
	{
		//Draw geometry
		if (vaoDrawerHi == null)
		{
			vaoDrawerHi = new VAODrawer();
			vaoDrawerHi.LoadMesh(pipe, meshHi);
		}
		
		//meshHi.draw(gl);
		vaoDrawerHi.Draw(pipe);
		
		// Done Drawing The line
	}

	public int compareTo(JSODataVisMagnetLine arg0) 
	{
		return (int)(height - arg0.height);
	}
	
	@Override
	public void Export(JSOSubNet snet, OBJWriter out)
	{
		float blendFactor = (float)Math.abs(height / 50.0);
		
		if (mesh instanceof JoglTexturedInterpolationMesh)
		{
			JoglTexturedInterpolationMesh jtim = (JoglTexturedInterpolationMesh)mesh;
			double[] verts1 = jtim.getVerts();
			double[] verts2 = jtim.getVerts2();

			Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
			for (int i = 0; i < verts1.length-2; i+=3)
			{
				lst.add(JoglVectord3.Blend(new JoglVectord3(verts1[i], verts1[i+1], verts1[i+2]), new JoglVectord3(verts2[i], verts2[i+1], verts2[i+2]), blendFactor));
			}
			
			out.WriteNewObject("MagnetLine " + label);
	
			Vector<Integer> ids = new Vector<Integer>();
			for (int i = 0; i < lst.size(); i++)
			{
				ids.add(out.WriteVertex(snet.relativeToAbsolute(lst.get(i))));
			}
	
			out.WriteUseMtl(getExportMaterialNameFromColor(colors.get(0)));
			for (int i = 0; i < ids.size()-2; i+=3)
			{
				out.WriteFaceTri(ids.get(i), ids.get(i+1), ids.get(i+2));
			}
		}
	}
}
