package org.processmining.processprofiler3d.compviz.jogl.scene.datavis;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglTexturedMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialColorOnly;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialDepthOnly;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.MagnetLineMeshCreator;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JSODataVisMagnetLineGroup extends JSODataVisMagnetLineGroupBase
{
	private static MaterialColorOnly matCO = new MaterialColorOnly();
	private static MaterialDepthOnly matDO = new MaterialDepthOnly();
	
	public static boolean bAllowGrouping = true;
	private double maxHeight = 50.0;
	private double topBottomCutoff = -5;
	
	private MagnetLineMeshCreator mlmc;
	private JoglTexturedMesh meshTop;
	private JoglTexturedMesh meshBottom;
	private IMeshDrawer drawerTop;
	private IMeshDrawer drawerBottom;
	
	public JSODataVisMagnetLineGroup(JSOSubNet snet, JSOPnTransition sourceNode, JSOPnTransition targetNode, String localIDs, MagnetLineMeshCreator mlmc, double maxHeight)
	{
		super(snet, sourceNode, targetNode, localIDs);
		
		this.mlmc = mlmc;
		this.maxHeight = maxHeight;
		
		meshTop = mlmc.getMesh(0.5);
		meshBottom = mlmc.getMesh(-0.5);
	}
	
	public boolean NeedsUpdate(JoglVectord3 startPos, JoglVectord3 endPos, double maxHeight, int numPoints)
	{
		return mlmc.NeedsUpdate(startPos, endPos, maxHeight, numPoints);
	}
	
	@Override
	public void Dispose()
	{
		mlmc = null;
		
		meshTop = null;
		meshBottom = null;
		
		if (drawerTop != null)
		{
			drawerTop.Dispose();
		}
		if (drawerBottom != null)
		{
			drawerBottom.Dispose();
		}
	}

	private Vector<Double> prevValues = new Vector<Double>();
	
	@Override
	public JSODataVisMagnetLineBase AddCohortValue(double scale, JoglVectord3 color, String cohortName)
	{
		double mx = scale * maxHeight;
		
		int groupId = 0;
		boolean isGrouped = false;
		if (bAllowGrouping)
		{
			for (double prev : prevValues)
			{
				if (Math.abs(prev - mx) < 2.0)
				{
					//group
					JSODataVisMagnetLine eso = (JSODataVisMagnetLine)elements.get(groupId);
					eso.addColor(color);
					isGrouped = true;
					return eso;
				}
				groupId++;
			}
		}
		
		if (!isGrouped)
		{
			for (int i = 0; i < prevValues.size(); i++)
			{
				if (mx < prevValues.get(i))
				{
					prevValues.insertElementAt(mx, i);
					
					JSODataVisMagnetLine eso = AddMagnetLine(maxHeight, scale);
					eso.setParentRef(this.parentRef);
					eso.setCohortRef(cohortName);
					eso.setLabel(label + " (" + cohortName + ")");
					eso.setColor(color);
					elements.insertElementAt(eso, i);
					
					return eso;
				}
			}
			
			prevValues.add(mx);
			
			JSODataVisMagnetLine eso = AddMagnetLine(maxHeight, scale);
			eso.setParentRef(this.parentRef);
			eso.setCohortRef(cohortName);
			eso.setLabel(label + " (" + cohortName + ")");
			eso.setColor(color);
			elements.add(eso);
			
			return eso;
		}
		return null;
	}
	
	private JSODataVisMagnetLine AddMagnetLine(double lHeight, double ts) 
	{
		//System.out.println("MagnetLine (" + source.getLabel() + " - " + target.getLabel() + ") height (" + classifier + "): " + lh);
		Vector<JoglVectord3> vec = mlmc.getPoints(ts);
		
		JSODataVisMagnetLine eso = new JSODataVisMagnetLine(vec);
		eso.setHeight(ts * lHeight);
		if (ts >= 0.0)
		{
			eso.SetMesh(meshTop);
		}
		else
		{
			eso.SetMesh(meshBottom);
		}
		eso.setJSOPnet(snet);
		
		return eso;
	}
	
	@Override
	public void ClearMagnetLines()
	{
		for (JoglSceneObject jso : elements)
		{
			if (jso instanceof JSODataVisMagnetLine)
			{
				((JSODataVisMagnetLine)jso).Dispose();
			}
		}
		
		prevValues.clear();
		elements.clear();
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		JoglCamera c = pipe.cam;
		
		double camY = c.GetPosition().getY();
		DrawBottom(pipe, (camY > topBottomCutoff));
		DrawTop(pipe, (camY < topBottomCutoff));
	}

	@Override
	public void DrawTop(GraphicsPipeline pipe, boolean bMasked)
	{
		//for each visualization element in group
		//draw color value only to avoid issues with z-fighting
		DrawTopColor(pipe, bMasked);
		
		//write depth value for each group element
		DrawTopDepth(pipe, bMasked);
	}

	private void DrawTopDepth(GraphicsPipeline pipe, boolean bMasked)
	{
		pipe.PushPipelineState(matDO);
		//pipe.SetMaskState(GraphicsPipeline.MaskState.DrawDepthOnly);
		for (int j = 0; j < elements.size(); j++)
		{
			JoglSceneObject jso = elements.get(j);
			JSODataVisMagnetLine line = (JSODataVisMagnetLine)jso;
			//line.DrawObject(gl, target);
			if (line.getHeight() >= topBottomCutoff)
			{
				if (bMasked)
				{
					//line.DrawMasked(gl, scene, target);
					DrawLineMasked(pipe, line);
				}
				else
				{
					//line.DrawObject(gl, scene, target);
					DrawLine(pipe, line);
				}
			}
		}
		//pipe.SetMaskState(GraphicsPipeline.MaskState.DrawBoth);
		pipe.PopPipelineState(matDO);
	}

	private void DrawTopColor(GraphicsPipeline pipe, boolean bMasked)
	{
		JoglCamera c = pipe.cam;

		pipe.PushPipelineState(matCO);
		//pipe.SetMaskState(GraphicsPipeline.MaskState.DrawColorOnly);
		
		double camY = c.GetPosition().getY();
		//if seen from above draw in sorted order
		if (camY >= (1.0 + snet.getSubnetYPos()))
		{
			for (int j = 0; j < elements.size(); j++)
			{
				JoglSceneObject jso = elements.get(j);
				JSODataVisMagnetLine line = (JSODataVisMagnetLine)jso;
				//line.DrawObject(gl, target);
				if (line.getHeight() >= topBottomCutoff)
				{
					if (bMasked)
					{
						//line.DrawMasked(gl, scene, target);
						DrawLineMasked(pipe, line);
					}
					else
					{
						//line.DrawObject(gl, scene, target);
						DrawLine(pipe, line);
					}
				}
			}
		}
		//if seen from below draw in reverse order
		else
		{
			for (int j = elements.size() - 1; j >= 0; j--)
			{
				JoglSceneObject jso = elements.get(j);
				JSODataVisMagnetLine line = (JSODataVisMagnetLine)jso;
				//line.DrawObject(gl, target);
				if (line.getHeight() >= topBottomCutoff)
				{
					if (bMasked)
					{
						//line.DrawMasked(gl, scene, target);
						DrawLineMasked(pipe, line);
					}
					else
					{
						//line.DrawObject(gl, scene, target);
						DrawLine(pipe, line);
					}
				}
			}
		}

		//pipe.SetMaskState(GraphicsPipeline.MaskState.DrawBoth);
		pipe.PopPipelineState(matCO);
	}

	@Override
	public void DrawBottom(GraphicsPipeline pipe, boolean bMasked)
	{	
		//for each visualization element in group
		//draw color value only to avoid issues with z-fighting
		DrawBottomColor(pipe, bMasked);
		
		//write depth value for each group element
		DrawBottomDepth(pipe, bMasked);
	}

	private void DrawBottomDepth(GraphicsPipeline pipe, boolean bMasked)
	{
		pipe.PushPipelineState(matDO);
		//pipe.SetMaskState(GraphicsPipeline.MaskState.DrawDepthOnly);
		for (int j = 0; j < elements.size(); j++)
		{
			JoglSceneObject jso = elements.get(j);
			JSODataVisMagnetLine line = (JSODataVisMagnetLine)jso;
			//line.DrawObject(gl, target);
			if (line.getHeight() < topBottomCutoff)
			{
				if (bMasked)
				{
					//line.DrawMasked(gl, scene, target);
					DrawLineMasked(pipe, line);
				}
				else
				{
					//line.DrawObject(gl, scene, target);
					DrawLine(pipe, line);
				}
			}
		}
		//pipe.SetMaskState(GraphicsPipeline.MaskState.DrawBoth);
		pipe.PopPipelineState(matDO);
	}

	private void DrawBottomColor(GraphicsPipeline pipe, boolean bMasked)
	{
		JoglCamera c = pipe.cam;
		
		//gl.glDepthMask(false);
		//pipe.SetMaskState(GraphicsPipeline.MaskState.DrawColorOnly);
		pipe.PushPipelineState(matCO);
		
		double camY = c.GetPosition().getY();
		//if seen from above draw in sorted order
		if (camY >= (1.0 + snet.getSubnetYPos()))
		{
			for (int j = 0; j < elements.size(); j++)
			{
				JoglSceneObject jso = elements.get(j);
				JSODataVisMagnetLine line = (JSODataVisMagnetLine)jso;
				//line.DrawObject(gl, target);
				if (line.getHeight() < topBottomCutoff)
				{
					if (bMasked)
					{
						//line.DrawMasked(gl, scene, target);
						DrawLineMasked(pipe, line);
					}
					else
					{
						//line.DrawObject(gl, scene, target);
						DrawLine(pipe, line);
					}
				}
			}
		}
		//if seen from below draw in reverse order
		else
		{
			for (int j = elements.size() - 1; j >= 0; j--)
			{
				JoglSceneObject jso = elements.get(j);
				JSODataVisMagnetLine line = (JSODataVisMagnetLine)jso;
				//line.DrawObject(gl, target);
				if (line.getHeight() < topBottomCutoff)
				{
					if (bMasked)
					{
						//line.DrawMasked(gl, scene, target);
						DrawLineMasked(pipe, line);
					}
					else
					{
						//line.DrawObject(gl, scene, target);
						DrawLine(pipe, line);
					}
				}
			}
		}

		//pipe.SetMaskState(GraphicsPipeline.MaskState.DrawBoth);
		pipe.PopPipelineState(matCO);
	}
	
	private void DrawLine(GraphicsPipeline pipe, JSODataVisMagnetLine line)
	{
		if (drawerTop == null)
		{
			drawerTop = new VAODrawer();
			drawerTop.LoadMesh(pipe, meshTop);
		}
		if (drawerBottom == null)
		{
			drawerBottom = new VAODrawer();
			drawerBottom.LoadMesh(pipe, meshBottom);
		}
		
		if (line.getHeight() >= 0)
		{
			line.SetVAODrawer(drawerTop);
		}
		else
		{
			line.SetVAODrawer(drawerBottom);
		}
		line.DrawObject(pipe);
	}
	
	private void DrawLineMasked(GraphicsPipeline pipe, JSODataVisMagnetLine line)
	{
		if (drawerTop == null)
		{
			drawerTop = new VAODrawer();
			drawerTop.LoadMesh(pipe, meshTop);
		}
		if (drawerBottom == null)
		{
			drawerBottom = new VAODrawer();
			drawerBottom.LoadMesh(pipe, meshBottom);
		}
		
		if (line.getHeight() >= 0)
		{
			line.SetVAODrawer(drawerTop);
		}
		else
		{
			line.SetVAODrawer(drawerBottom);
		}
		line.DrawMasked(pipe);
	}
}
