package org.processmining.processprofiler3d.compviz.jogl.scene.datavis;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnEdge;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnPlace;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JSODataVisMagnetLineGroupNew extends JSODataVisMagnetLineGroupBase
{
	private double maxHeight = 50.0;
	private double topBottomCutoff = -5;
	
	public JSODataVisMagnetLineGroupNew(JSOSubNet snet, JSOPnTransition sourceNode, JSOPnTransition targetNode, String localIDs, double maxHeight)
	{
		super(snet, sourceNode, targetNode, localIDs);
		
		this.maxHeight = maxHeight;
	}
	
	public boolean NeedsUpdate(JoglVectord3 startPos, JoglVectord3 endPos, double maxHeight)
	{
		return true;
	}
	
	@Override
	public void Dispose()
	{
	}

	private Vector<Double> prevValues = new Vector<Double>();
	
	public JSODataVisMagnetLineBase AddCohortValue(double scale, JoglVectord3 color, String cohortName)
	{
		double mx = scale * maxHeight;
		
		int groupId = 0;
		boolean isGrouped = false;
		for (double prev : prevValues)
		{
			if (Math.abs(prev - mx) < 2.0)
			{
				//group
				JSODataVisMagnetLineNew eso = (JSODataVisMagnetLineNew)elements.get(groupId);
				eso.addColor(color);
				isGrouped = true;
				return eso;
			}
			groupId++;
		}
		
		if (!isGrouped)
		{
			for (int i = 0; i < prevValues.size(); i++)
			{
				if (mx < prevValues.get(i))
				{
					prevValues.insertElementAt(mx, i);
					
					JSODataVisMagnetLineNew eso = AddMagnetLine(maxHeight, scale);
					eso.setParentRef(this.parentRef);
					eso.setCohortRef(cohortName);
					eso.setLabel(label + " (" + cohortName + ")");
					eso.setColor(color);
					elements.insertElementAt(eso, i);
					
					return eso;
				}
			}
			
			prevValues.add(mx);
			
			JSODataVisMagnetLineNew eso = AddMagnetLine(maxHeight, scale);
			eso.setParentRef(this.parentRef);
			eso.setCohortRef(cohortName);
			eso.setLabel(label + " (" + cohortName + ")");
			eso.setColor(color);
			elements.add(eso);
			
			return eso;
		}
		return null;
	}
	
	private JSODataVisMagnetLineNew AddMagnetLine(double lHeight, double ts) 
	{
		//System.out.println("MagnetLine (" + source.getLabel() + " - " + target.getLabel() + ") height (" + classifier + "): " + lh);
		//Vector<JoglVectord3> vec = mlmc.getPoints(ts);
		Vector<JoglVectord3> vec = new Vector<JoglVectord3>();
		JoglVectord3 offset = new JoglVectord3(0,0,0);
		vec.add(JoglVectord3.Add(sourceNode.getPosition(), offset));
		
		for (JSOPnPlace p : snet.getPlaces())
		{
			JSOPnEdge bStart = null;
			JSOPnEdge bEnd = null;
			for (JSOPnEdge e : snet.getEdges())
			{
				if (e.startObj == sourceNode && e.endObj == p)
				{
					bStart = e;
				}
				if (e.endObj == targetNode && e.startObj == p)
				{
					bEnd = e;
				}
			}
			if (bStart != null && bEnd != null)
			{
				for (JoglVectord3 v1 : bStart.getPoints())
				{
					vec.add(JoglVectord3.Add(v1, offset));	
				}
				vec.add(JoglVectord3.Add(p.getPosition(), offset));
				for (JoglVectord3 v1 : bEnd.getPoints())
				{
					vec.add(JoglVectord3.Add(v1, offset));	
				}
				break;
			}
		}
		
		vec.add(JoglVectord3.Add(targetNode.getPosition(), offset));
		
		JSODataVisMagnetLineNew eso = new JSODataVisMagnetLineNew(vec);
		eso.setHeight(ts * lHeight);
//		if (ts >= 0.0)
//		{
//			eso.SetMesh(meshTop);
//		}
//		else
//		{
//			eso.SetMesh(meshBottom);
//		}
		eso.setJSOPnet(snet);
		
		return eso;
	}
	
	@Override
	public void ClearMagnetLines()
	{
		for (JoglSceneObject jso : elements)
		{
			if (jso instanceof JSODataVisMagnetLineNew)
			{
				((JSODataVisMagnetLineNew)jso).Dispose();
			}
		}
		
		prevValues.clear();
		elements.clear();
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		for (int j = 0; j < elements.size(); j++)
		{
			JoglSceneObject jso = elements.get(j);
			JSODataVisMagnetLineNew line = (JSODataVisMagnetLineNew)jso;
			DrawLine(pipe, line);
		}
	}

	@Override
	public void DrawTop(GraphicsPipeline pipe, boolean bMasked)
	{
		for (int j = 0; j < elements.size(); j++)
		{
			JoglSceneObject jso = elements.get(j);
			JSODataVisMagnetLineNew line = (JSODataVisMagnetLineNew)jso;

			if (line.getHeight() >= topBottomCutoff)
			{
				if (bMasked)
				{
					DrawLineMasked(pipe, line);
				}
				else
				{
					DrawLine(pipe, line);
				}
			}
		}
	}
	
	@Override
	public void DrawBottom(GraphicsPipeline pipe, boolean bMasked)
	{
		for (int j = 0; j < elements.size(); j++)
		{
			JoglSceneObject jso = elements.get(j);
			JSODataVisMagnetLineNew line = (JSODataVisMagnetLineNew)jso;

			if (line.getHeight() < topBottomCutoff)
			{
				if (bMasked)
				{
					DrawLineMasked(pipe, line);
				}
				else
				{
					DrawLine(pipe, line);
				}
			}
		}
	}
	
	private void DrawLine(GraphicsPipeline pipe, JSODataVisMagnetLineNew line)
	{
		line.DrawObject(pipe);
	}
	
	private void DrawLineMasked(GraphicsPipeline pipe, JSODataVisMagnetLineNew line)
	{
		line.DrawMasked(pipe);
	}
}
