package org.processmining.processprofiler3d.compviz.jogl.scene.datavis;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglColoredMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.jogl.shaders.JoglShaderTrajectory;
import org.processmining.processprofiler3d.psen.fileio.obj.OBJWriter;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglShader;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglColor3;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

import com.jogamp.opengl.GL2;

public class JSODataVisTriangle extends JoglSceneObject implements Comparable<JSODataVisTriangle>
{
	private JoglVectord3 colX = JoglColor3.GREEN;
	
	private float width = 100.0f;
	private float height = 100.0f;
	
	private JoglVectord3 position = new JoglVectord3(0,0,0);
	private JoglMatrixd4x4 transform = JoglMatrixd4x4.CreateIdentity();

	private IMeshDrawer vao;
	private static JoglShader shad;
	
	public void setColor(float r, float g, float b)
	{
//		colorR = r;
//		colorG = g;
//		colorB = b;
		colX = new JoglVectord3(r,g,b);
	}
	
	public void setColor(JoglVectord3 col)
	{
		colX = col;
//		colorR = (float)col.getX();
//		colorG = (float)col.getY();
//		colorB = (float)col.getZ();
	}
	
	public JoglVectord3 getPosition()
	{
		return position;
	}
	
	public void setPosition(double x, double y, double z)
	{
		position = new JoglVectord3(x,y,z);
		transform = JoglMatrixd4x4.CreateTranslation(x, y, z);
	}
	
	public JoglVectord2 getSize()
	{
		return new JoglVectord2(width, height);
	}
	
	public void setSize(float w, float h)
	{
		width = w;
		height = h;
	}
	
	public int compareTo(JSODataVisTriangle o)
	{
		double triH = o.getSize().getY();
		return (int)(triH - this.height);
	}
	
	private Vector<JoglVectord3> GenerateGeometry()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();

		float scaleX = width * 0.5f;
		
		lst.add(new JoglVectord3(-scaleX, 0.0f, 0.0f));   // Bottom Left
		lst.add(new JoglVectord3( 0.0f, height, 0.0f));   // Top Center
		lst.add(new JoglVectord3( scaleX, 0.0f, 0.0f));   // Bottom Right
		
		return lst;
	}
	
	private Vector<JoglVectord3> GenerateColor()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();

		lst.add(new JoglVectord3(colX.getX(), colX.getY(), colX.getZ())); // set the color of the triangle
		lst.add(new JoglVectord3(colX.getX(), colX.getY(), colX.getZ())); // set the color of the triangle
		lst.add(new JoglVectord3(colX.getX(), colX.getY(), colX.getZ())); // set the color of the triangle
		
		return lst;
	}
	
	public JoglColoredMesh CreateMesh()
	{
		JoglColoredMesh jcm = new JoglColoredMesh(GL2.GL_TRIANGLES);
		jcm.setVerts(GenerateGeometry());
		jcm.setColors3(GenerateColor());
		jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(jcm.getNumVerts()));
		
		return jcm;
	}

	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		//float scaleX = width * 0.5f;
		
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);
		
		//draw a triangle
//		gl.glBegin(GL2.GL_TRIANGLES);       
//		
//		//gl.glColor3f(colorR, colorG, colorB); // set the color of the triangle
//		gl.glColor3d(colX.getX(), colX.getY(), colX.getZ()); // set the color of the triangle
//		gl.glVertex3f(-scaleX, 0.0f, 0.0f);   // Bottom Left
//		gl.glVertex3f( 0.0f, height, 0.0f);   // Top Center
//		gl.glVertex3f( scaleX, 0.0f, 0.0f);   // Bottom Right

		if (vao == null)
		{
			vao = new VAODrawer();
			vao.LoadMesh(pipe, CreateMesh());
			
		}
		if (shad == null)
		{
			shad = new JoglShaderTrajectory();
			shad.Create(pipe);
		}
		
		shad.Bind(pipe);
		vao.Draw(pipe);
		shad.Unbind(pipe);
		
		//done drawing the triangle
		//gl.glEnd(); 

		pipe.transform.Pop();
	}

	public void DrawObjectFast(GraphicsPipeline pipe, IMeshDrawer d)
	{
		pipe.transform.Push();
		pipe.transform.MultMatrix(transform);
		
		d.DrawSlim(pipe, tStart, tCount);
		
		pipe.transform.Pop();	
	}
	
	private int tStart = 0;
	private int tCount = 0;
	
	public void LoadMesh(JoglColoredMesh jcm)
	{
		tStart = jcm.getNumVerts();
		
		//int i = 0;
		//for (JoglVectord3 v : points)
		{
			//JoglVectord3 col = colors.get(i);
			//gl.glColor3d(col.getX(), col.getY(), col.getZ());
			jcm.addColors3(GenerateColor());
				
			//i++;
			//gl.glVertex3d(v.getX(), v.getY(), v.getZ());
			jcm.addVerts(GenerateGeometry());
		}
		
		tCount = jcm.getNumVerts() - tStart;
	}
	
	public void Dispose()
	{
		if (vao != null)
		{
			vao.Dispose();
			vao = null;
		}
	}
	
	public void Export(JSOSubNet snet, OBJWriter out)
	{
		float scaleX = width * 0.5f;
		
		//gl.glPushMatrix();
		//gl.glMultMatrixd(transform.GetData(), 0);
		
		//draw a triangle
		//gl.glBegin(GL2.GL_TRIANGLES);       

		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();
		//gl.glColor3f(colorR, colorG, colorB); // set the color of the triangle
		lst.add(new JoglVectord3(-scaleX, 0.0f, 0.0f));   // Bottom Left
		lst.add(new JoglVectord3( 0.0f, height, 0.0f));   // Top Center
		lst.add(new JoglVectord3( scaleX, 0.0f, 0.0f));   // Bottom Right
		
		//done drawing the triangle
		//gl.glEnd(); 

		//gl.glPopMatrix();
		out.WriteNewObject("TriangleChart " + 0);
		
		int id1 = out.WriteVertex(snet.relativeToAbsolute(JoglVectord3.Add(position, lst.get(0))));
		int id2 = out.WriteVertex(snet.relativeToAbsolute(JoglVectord3.Add(position, lst.get(1))));
		int id3 = out.WriteVertex(snet.relativeToAbsolute(JoglVectord3.Add(position, lst.get(2))));

		out.WriteUseMtl(getExportMaterialNameFromColor(colX));
		out.WriteFaceTri(id1, id2, id3);
	}
	
	public String getExportMaterialNameFromColor(JoglVectord3 col)
	{
		if (col.getX() == 1 && col.getY() == 0 && col.getZ() == 0)
		{
			return "CohortRed";
		}
		if (col.getX() == 0 && col.getY() == 1 && col.getZ() == 0)
		{
			return "CohortGreen";
		}
		if (col.getX() == 0 && col.getY() == 0 && col.getZ() == 1)
		{
			return "CohortBlue";
		}
		if (col.getX() == 1 && col.getY() == 1 && col.getZ() == 0)
		{
			return "CohortYellow";
		}
		if (col.getX() == 0 && col.getY() == 1 && col.getZ() == 1)
		{
			return "CohortCyan";
		}
		if (col.getX() == 1 && col.getY() == 0 && col.getZ() == 1)
		{
			return "CohortMagenta";
		}
		if (col.getX() == 1 && col.getY() == 1 && col.getZ() == 1)
		{
			return "CohortWhite";
		}
		if (col.getX() == 1 && col.getY() == 0 && col.getZ() == 1)
		{
			return "CohortBlack";
		}
		
		return "ModelPedestal";
	}
}
