package org.processmining.processprofiler3d.compviz.jogl.scene.datavis;

import java.util.Collections;
import java.util.HashSet;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglColoredMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialColorOnly;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialDepthOnly;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransition;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.VisualisationDataPipeline;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JSODataVisTriangleGroup extends JSODataVisGroup
{
	private static MaterialColorOnly matCO = new MaterialColorOnly();
	private static MaterialDepthOnly matDO = new MaterialDepthOnly();
	
	private JSOSubNet snet = null;
	
	//AddDataVisTriangles
	// visualizes two label values of PSEN using triangle glyphs for each cohort for given transition in petrinet
	// label1 - label of PSEN value to visualize as width of triangle
	// label2 - label of PSEN value to visualize as height of triangle
	// max1 - value to normalize width of each triangle with
	// max2 - value to normalize height of each triangle with
	//=========================================================================================
	public JSODataVisTriangleGroup(JSOSubNet sn, JSOPnTransition t, VisualisationDataPipeline tdp, double triHeight)
	{
		this.snet = sn;
		
		HashSet<String> classifiers = tdp.classifiers;

		JoglVectord3 pos = t.getPosition();
		JoglVectord2 size = t.getSize();
		
		int i = 0;
//			if (max1 > 0.0 && max2 > 0.0)
		{
			Vector<JSODataVisTriangle> tris = new Vector<JSODataVisTriangle>();
			for (String classifier : classifiers)
			{	
				if (tdp.map1.hasValue(t.orgTransition, classifier) && tdp.map2.hasValue(t.orgTransition, classifier))
				{
					double val1 = tdp.map1.getValueR(t.orgTransition, classifier);
					double val2 = tdp.map2.getValueR(t.orgTransition, classifier);
					val1 = tdp.TransformScale1(val1);
					val2 = tdp.TransformScale2(val2);
					
					if (val2 > 0.0 && val1 > 0.0)
					{
						JSODataVisTriangle tri = AddDataVisTriangle(t, tdp, triHeight, val1, val2, i, classifier);
						tris.add(tri);
					}
					else
					{
						if (val1 < 0.0)
						{
							System.out.println(t.getLabel() + ": Triangle Width is negative. This should not happen!");
						}
						if (val2 < 0.0)
						{
							System.out.println(t.getLabel() + ": Triangle Height is negative. This should not happen!");
						}
					}
				}
				i++;
			}
			
			//sort triangles by height
			Collections.sort(tris);
			
			int id = 0;
			//JSODataVisTriangleGroup group = new JSODataVisTriangleGroup();
			this.parentRef = t.orgTransition.getLocalID().toString();
			for (JSODataVisTriangle tri : tris)
			{
				JoglVectord2 triSize = tri.getSize();

				double posXx = (pos.getX() + (size.getX() * 0.5)) - (triSize.getX() * 0.5);
				double posYx = pos.getZ() + (size.getY() * 0.5) - (id * 2.0);

				tri.setPosition(posXx, 0, posYx);
				this.addElement(tri);
				id++;
			}
			this.updateAABB();
		}
	}

	//AddDataVisTriangle
	// visualizes two label values of PSEN using triangle glyph for one cohort for given transition in petrinet
	// val1 - normalized PSEN value to visualize as width of triangle
	// val2 - normalized PSEN value to visualize as height of triangle
	// id - number of triangle in glyph
	//=========================================================================================
	private JSODataVisTriangle AddDataVisTriangle(JSOPnTransition t, VisualisationDataPipeline tdp, double tHeight, double val1, double val2, int id, String classifier)
	{
		//JoglVectord3 col = colorsByCohort.get(classifier);
		JoglVectord3 col = tdp.cohortColors.getColorForCohort(classifier);
		
		JoglVectord3 pos = t.getPosition();
		JoglVectord2 size = t.getSize();	
		
		double triWidth = (size.getX() * val1);
		double triHeight = (tHeight * val2);

		double posXx = (pos.getX() + (size.getX() * 0.5)) - (triWidth * 0.5);
		double posYx = pos.getZ() + (size.getY() * 0.5) - (id * 2.0);
		
		JSODataVisTriangle tri = new JSODataVisTriangle();
		tri.setPosition(posXx, 0, posYx);
		tri.setSize((float)triWidth, (float)triHeight);
		tri.setColor(col);
		//elements.add(tri);
		
		return tri;
	}

	@Override
	public JoglAABB updateAABB()
	{
		double minx = Double.POSITIVE_INFINITY;
		double miny = Double.POSITIVE_INFINITY;
		double minz = Double.POSITIVE_INFINITY;
		double maxx = Double.NEGATIVE_INFINITY;
		double maxy = Double.NEGATIVE_INFINITY;
		double maxz = Double.NEGATIVE_INFINITY;
		
		for (int i = 0; i < elements.size(); i++)
		{
			JoglSceneObject jso = elements.get(i);
			if (jso instanceof JSODataVisTriangle)
			{
				JSODataVisTriangle tri = (JSODataVisTriangle)jso;
				JoglVectord3 pos = tri.getPosition();
				JoglVectord2 size = tri.getSize();
				
				minx = Math.min(minx, pos.getX() + (size.getX() * 0.5));
				minx = Math.min(minx, pos.getX() + (size.getX() * -0.5));
				miny = Math.min(miny, pos.getY() + size.getY());
				miny = Math.min(miny, pos.getY());
				minz = Math.min(minz, pos.getZ());

				maxx = Math.max(maxx, pos.getX() + (size.getX() * 0.5));
				maxx = Math.max(maxx, pos.getX() + (size.getX() * -0.5));
				maxy = Math.max(maxy, pos.getY() + size.getY());
				maxy = Math.max(maxy, pos.getY());
				maxz = Math.max(maxz, pos.getZ());
			}
		}
		
		JoglVectord3 vmin = new JoglVectord3(minx, miny, minz);
		JoglVectord3 vmax = new JoglVectord3(maxx, maxy, maxz);

		aabb = new JoglAABB(vmin, vmax);
		
		return aabb;
	}

	public void LoadMesh(JoglColoredMesh jcm)
	{
		for (JoglSceneObject jso : elements)
		{
			if (jso instanceof JSODataVisTriangle)
			{
				JSODataVisTriangle dvbc = (JSODataVisTriangle)jso;
				dvbc.LoadMesh(jcm);
			}
		}
	}
	
	@Override
	public void DrawObject(GraphicsPipeline pipe)
	{
		JoglCamera c = pipe.cam;
		
		//draw color value only to avoid issues with z-fighting
		pipe.PushPipelineState(matCO);
		
		double camPosZ = c.GetPosition().getZ();
		//double triPosZ = JoglVectord3.Multiply(JoglVectord3.Add(aabb.getCenter(), subnetOffset), subnetScale).getZ();
		double triPosZ = snet.relativeToAbsolute(aabb.getCenter()).getZ();
		
		//draw sorted by view direction
		if (camPosZ < triPosZ)
		{
			for (int j = 0; j < elements.size(); j++)
			{
				elements.get(j).DrawObject(pipe);
			}
		}
		else
		{
			for (int j = elements.size() - 1; j >= 0; j--)
			{
				elements.get(j).DrawObject(pipe);
			}
		}
		pipe.PopPipelineState(matCO);
		
		//write depth value for each group element
		pipe.PushPipelineState(matDO);
		for (int j = 0; j < elements.size(); j++)
		{
			elements.get(j).DrawObject(pipe);
		}
		pipe.PopPipelineState(matDO);
	}
	
	public void DrawObjectFast(GraphicsPipeline pipe, IMeshDrawer d)
	{
		JoglCamera c = pipe.cam;
		
		//draw color value only to avoid issues with z-fighting
		pipe.PushPipelineState(matCO);
		
		double camPosZ = c.GetPosition().getZ();
		//double triPosZ = JoglVectord3.Multiply(JoglVectord3.Add(aabb.getCenter(), subnetOffset), subnetScale).getZ();
		double triPosZ = snet.relativeToAbsolute(aabb.getCenter()).getZ();
		
		//draw sorted by view direction
		if (camPosZ < triPosZ)
		{
			for (int j = 0; j < elements.size(); j++)
			{
				if (elements.get(j) instanceof JSODataVisTriangle)
				{
					JSODataVisTriangle tri = (JSODataVisTriangle)elements.get(j);
					tri.DrawObjectFast(pipe, d);
				}
			}
		}
		else
		{
			for (int j = elements.size() - 1; j >= 0; j--)
			{
				if (elements.get(j) instanceof JSODataVisTriangle)
				{
					JSODataVisTriangle tri = (JSODataVisTriangle)elements.get(j);
					tri.DrawObjectFast(pipe, d);
				}
			}
		}
		pipe.PopPipelineState(matCO);
		
		//write depth value for each group element
		pipe.PushPipelineState(matDO);
		for (int j = 0; j < elements.size(); j++)
		{
			if (elements.get(j) instanceof JSODataVisTriangle)
			{
				JSODataVisTriangle tri = (JSODataVisTriangle)elements.get(j);
				tri.DrawObjectFast(pipe, d);
			}
		}
		pipe.PopPipelineState(matDO);
	}
	
	public void Dispose()
	{
		//dispose VAOs properly
		for (JoglSceneObject jso : elements)
		{
			if (jso instanceof JSODataVisTriangle)
			{
				JSODataVisTriangle dvbc = (JSODataVisTriangle)jso;
				dvbc.Dispose();
			}
		}
	}
}
