package org.processmining.processprofiler3d.compviz.jogl.scene.datavis;

import java.util.Set;

import org.processmining.processprofiler3d.compviz.TransitionDataMapDouble;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENEdgeVisualizationLayer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPnTransitionPair;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.VisualisationDataPipeline;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class JSOPSENEdgeVisualizer_MagnetLinesNew extends JSOPSENEdgeVisualizer_Base
{
	public JSOPSENEdgeVisualizer_MagnetLinesNew(JSOPSENEdgeVisualizationLayer ev)
	{
		super(ev);
	}

	@Override
	public void InitVisualisations(JSOSubNet snet, IPerformanceStatisticsEnrichedNet wtic, VisualisationDataPipeline edp) 
	{
		JoglVectord3 offY = new JoglVectord3(0,1,0);
		for (JSOPnTransitionPair tp : ev.getVisElements(wtic))
		{
//		for (String localIDs : wtic.getTransitionsInclNone()) //get all transitions, even those not currently in a cohort
//		{
//			PetrinetNode source = edp.pnetIdx.getNode(TransitionPair.getSourceId(localIDs));
//			PetrinetNode target = edp.pnetIdx.getNode(TransitionPair.getTargetId(localIDs));
//			
//			JSOPnTransition sourceNode = snet.getTransitionFor(source);
//			JSOPnTransition targetNode = snet.getTransitionFor(target);
//
//			if (sourceNode != null && targetNode != null)
//			{
				JoglVectord3 sourcePosV = tp.sourceNode.getPosition();
				JoglVectord3 targetPosV = tp.targetNode.getPosition();
				
				//handle self-loops gracefully
				if (tp.sourceNode == tp.targetNode)
				{
					double nodeWidth = tp.sourceNode.getSize().getX() * 0.5;
					JoglVectord3 nodePos = sourcePosV;
					sourcePosV = JoglVectord3.Add(nodePos, new JoglVectord3(-nodeWidth,0,0));
					targetPosV = JoglVectord3.Add(nodePos, new JoglVectord3(nodeWidth,0,0));
				}

				//adjust mesh detail based on graphics settings
//				int numPoints = 7;
//				if (gSettings.bHQGeometry)
//				{
//					numPoints = 40;
//				}
				
				//check if stale group exists and remove
				if (ev.mgroups.containsKey(tp.localIDs))
				{
					JSODataVisMagnetLineGroupBase groupx = ev.mgroups.get(tp.localIDs);
					if (groupx instanceof JSODataVisMagnetLineGroupNew)
					{
						JSODataVisMagnetLineGroupNew groupm = (JSODataVisMagnetLineGroupNew)groupx;
						if (groupm.NeedsUpdate(JoglVectord3.Add(sourcePosV, offY), JoglVectord3.Add(targetPosV, offY), ev.magnetLineArcHeight))
						{
							ev.mgroups.remove(tp.localIDs);
							groupx.Dispose();
						}
					}
					else				
					{
						ev.mgroups.remove(tp.localIDs);
						groupx.Dispose();
					}
				}
				
				if (!ev.mgroups.containsKey(tp.localIDs))
				{
					//MagnetLineMeshCreator mlmc = new MagnetLineMeshCreator(JoglVectord3.Add(sourcePosV, offY), JoglVectord3.Add(targetPosV, offY), magnetLineArcHeight, numPoints);
					JSODataVisMagnetLineGroupNew group = new JSODataVisMagnetLineGroupNew(snet, tp.sourceNode, tp.targetNode, tp.localIDs, ev.magnetLineArcHeight);
					group.setLabel(tp.sourceNode.getLabel() + " - " + tp.targetNode.getLabel());
					ev.mgroups.put(tp.localIDs, group);
					//mlmcs.put(localIDs, mlmc);
				}
			//}
		}
	}

	@Override
	public void VisualizeEdgeData(IPerformanceStatisticsEnrichedNet wtic, VisualisationDataPipeline edp)
	{	
		Set<String> classifiers = wtic.getClassifierValues();
		
		ev.mlines.clear();

		for (JSOPnTransitionPair tp : ev.getVisElements(wtic))
//		for (String localIDs : wtic.getTransitions()) 
		{
			AddMagnetLines(edp.map1, classifiers, tp, edp);
		}
	}

	private void AddMagnetLines(TransitionDataMapDouble map, Set<String> classifiers, JSOPnTransitionPair tp, VisualisationDataPipeline edp) 
	{
		if (!ev.mgroups.containsKey(tp.localIDs))
		{
			return;
		}
		
		JSODataVisMagnetLineGroupBase group = ev.mgroups.get(tp.localIDs);
		group.ClearMagnetLines();
		
		int validValues = 0;
		for (String classifier : classifiers)
		{
			if (map.hasValue(tp.localIDs, classifier))
			{
				Double ts = map.getValueR(tp.localIDs, classifier);
				if (ts != null)
				{
					if (ts > 1.1 || ts < -1.1)
					{
						System.out.println("MagnetLine value for " + tp.getTransitionPairLabel() + " (" + classifier +" out of visualization range: " + ts);
					}
					
					JSODataVisMagnetLineBase eso = group.AddCohortValue(ts.doubleValue(), edp.cohortColors.getColorForCohort(classifier), classifier);
					ev.mlines.put(eso.getParentRef() + eso.getCohortRef(), eso);
					
					validValues++;
				}
			}
		}

		group.updateAABB();
		if (!ev.elements.contains(group))
		{
			ev.elements.add(group);
		}
	}

}
