package org.processmining.processprofiler3d.compviz.methods;

import java.util.Collections;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.DataMapNormalization;
import org.processmining.processprofiler3d.compviz.DataMapProviderEdge;
import org.processmining.processprofiler3d.compviz.TransitionDataMapDouble;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENEdgeVisualizationLayer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.VisualisationDataPipeline;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSOPSENEdgeVisualizer_MagnetLinesNew;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PSENLabel;

public class EdgeDataVisualizationMethod_MagneticLinesNew extends EdgeDataVisualizationMethod
{
//	None (0, "None", false),
//	MagneticLines (1, "Magnetic Lines", true),
//	MagneticLinesNew (2, "Magnetic Lines New", true),
//	QualityLines (3, "Quality Lines", true);
	private IPerformanceStatisticsEnrichedNet pwait;
	private DataMapProviderEdge mapProvider;
    
    public EdgeDataVisualizationMethod_MagneticLinesNew(IPerformanceStatisticsEnrichedNet psen, DataMapProviderEdge mp)
    {
    	 super("Magnetic Lines New", 1);
    	 
    	 pwait = psen;
    	 mapProvider = mp;
    }
	
    @Override
	public Vector<String> GetNormalizationNames(int dim)
	{
    	if (dim == 1)
    	{
			Vector<String> lst = new Vector<String>();
	
			lst.add(DataMapNormalization.LocalMax.getModeName());
			lst.add(DataMapNormalization.SubnetMax.getModeName());
			lst.add(DataMapNormalization.GlobalMax.getModeName());
				
			return lst;
    	}
    	return super.GetNormalizationNames(dim);
	}
	
    @Override
	public Vector<String> GetLabelNames()
	{
		Vector<String> lst = new Vector<String>();

		for (PSENLabel l : pwait.getLabels())
		{
			lst.add(l.getLabelName());
		}
		
		Collections.sort(lst);
		
		return lst;
	}
	
    @Override
	public Vector<String> GetClassifierNames()
	{
		Vector<String> lst = new Vector<String>();

		for (String s : pwait.getClassifierValues())
		{
			lst.add(s);
		}

		Collections.sort(lst);
		
		return lst;
	}

    @Override
	public TransitionDataMapDouble GenerateDataMap(int dim, String label1Name, DataMapNormalization normMode, String focusClassifier)
	{
    	if (dim == 1)
    	{
			TransitionDataMapDouble map1 = null;
			if (focusClassifier == null)
			{
				map1 = mapProvider.getMap(label1Name, normMode);
			}
			else
			{
				map1 = mapProvider.getMap(label1Name, focusClassifier, normMode);
			}
			return map1;
    	}
    	return super.GenerateDataMap(dim, label1Name, normMode, focusClassifier);
	}

    @Override
	public void InitVisualisations(JSOPSENEdgeVisualizationLayer ev, JSOSubNet snet, VisualisationDataPipeline edp) 
	{
    	JSOPSENEdgeVisualizer_MagnetLinesNew evml = new JSOPSENEdgeVisualizer_MagnetLinesNew(ev);
    	evml.InitVisualisations(snet, pwait, edp);
	}

    @Override
	public void VisualizeData(JSOPSENEdgeVisualizationLayer ev, VisualisationDataPipeline edp)
	{	
    	JSOPSENEdgeVisualizer_MagnetLinesNew evml = new JSOPSENEdgeVisualizer_MagnetLinesNew(ev);
    	evml.VisualizeEdgeData(pwait, edp);
	}
}
