package org.processmining.processprofiler3d.compviz.methods;

import java.util.Collections;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.DataMapNormalization;
import org.processmining.processprofiler3d.compviz.DataMapProviderTransition;
import org.processmining.processprofiler3d.compviz.TransitionDataMapDouble;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENTransitionVisualizationLayer;
import org.processmining.processprofiler3d.compviz.jogl.scene.VisualisationDataPipeline;
import org.processmining.processprofiler3d.compviz.jogl.scene.datavis.JSOPSENTransitionVisualizer_BarChart2DHeight;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PSENLabel;

public class TransitionDataVisualizationMethod_BarChart2DHeight extends TransitionDataVisualizationMethod
{
//	None (0, "None", false, false),
//	BarChart1D (1, "Bar Chart 1D", true, false),
//	BarChart2DHeight (2, "Bar Chart 2D Height", true, true),
//	BarChart2DColor (3, "Bar Chart 2D Color", true, true),
//	TriangleGlyphs (4, "Triangle Glyphs", true, true);
	private IPerformanceStatisticsEnrichedNet psen;
	private DataMapProviderTransition mapProvider;
    
    public TransitionDataVisualizationMethod_BarChart2DHeight(IPerformanceStatisticsEnrichedNet psen, DataMapProviderTransition mp) 
    {
    	super("Bar Chart 2D Height", 2);
    	
    	this.psen = psen;
    	this.mapProvider = mp;
    }
	
    @Override
	public Vector<String> GetLabelNames()
	{
		Vector<String> lst = new Vector<String>();

		for (PSENLabel l : psen.getLabels())
		{
			lst.add(l.getLabelName());
		}
		
		Collections.sort(lst);
		
		return lst;
	}
	
    @Override
	public Vector<String> GetClassifierNames()
	{
		Vector<String> lst = new Vector<String>();

		for (String s : psen.getClassifierValues())
		{
			lst.add(s);
		}

		Collections.sort(lst);
		
		return lst;
	}
	
    @Override
	public DataMapNormalization getValidNormalizationMode(int dim, DataMapNormalization current)
	{
    	if (dim == 1)
    	{
	    	Vector<DataMapNormalization> validModes1 = GetValidNormalizationModes(1);
			if (!validModes1.contains(current))
			{
				return validModes1.get(0);
			}
			return current;
    	}
    	else if (dim == 2)
    	{
    		Vector<DataMapNormalization> validModes2 = GetValidNormalizationModes(2);
    		if (!validModes2.contains(current))
    		{
    			return validModes2.get(0);
    		}
    		return current; 		
    	}
    	return super.getValidNormalizationMode(dim, current);
	}
	
    @Override
	public Vector<String> GetNormalizationNames(int dim)
	{
    	if (dim == 1)
    	{
			Vector<String> lst = new Vector<String>();
			lst.add(DataMapNormalization.LocalSum.getModeName());
			lst.add(DataMapNormalization.One.getModeName());
	
			return lst;
    	}
    	else if (dim == 2)
    	{
    		Vector<String> lst = new Vector<String>();
    		lst.add(DataMapNormalization.LocalMax.getModeName());
    		lst.add(DataMapNormalization.SubnetMax.getModeName());
    		lst.add(DataMapNormalization.GlobalMax.getModeName());
    		
    		return lst;
    	}
    	return super.GetNormalizationNames(dim);
	}
	
    @Override
	public Vector<DataMapNormalization> GetValidNormalizationModes(int dim)
	{
    	if (dim == 1)
    	{
			Vector<DataMapNormalization> lst = new Vector<DataMapNormalization>();
			lst.add(DataMapNormalization.LocalSum);
			lst.add(DataMapNormalization.One);
			
			return lst;
    	}
    	else if (dim == 2)
    	{
    		Vector<DataMapNormalization> lst = new Vector<DataMapNormalization>();
    		lst.add(DataMapNormalization.LocalMax);
    		lst.add(DataMapNormalization.GlobalMax);
    		
    		return lst;
    	}
    	return super.GetValidNormalizationModes(dim);
	}

    @Override
	public TransitionDataMapDouble GenerateDataMap(int dim, String label1Name, DataMapNormalization normMode, String focusClassifier)
	{
		return mapProvider.getMap(label1Name, normMode);
	}

    @Override
	public void VisualizeData(JSOPSENTransitionVisualizationLayer ev, VisualisationDataPipeline tdp)
	{	
		tdp.classifiers = psen.getClassifierValues();
		
    	JSOPSENTransitionVisualizer_BarChart2DHeight evml = new JSOPSENTransitionVisualizer_BarChart2DHeight(ev);
    	evml.VisualizeTransitionData(tdp);
	}
}
