package org.processmining.processprofiler3d.compviz.plugins;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.processprofiler3d.compviz.plugins.data.ModelViz;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */
public class ThreeDeeModel 
{
	@Plugin(
		name = "Process Profiler 3D: Visualize Model", 
		parameterLabels = { "Petri Net" }, 
		returnLabels = { "3D Model Visualisation" }, 
		returnTypes = { ModelViz.class }, 
		userAccessible = true, 
		help = "This plug-in visualizes a process model in 3D."
	)
	@UITopiaVariant(
		affiliation = "Queensland University of Technology", 
		author = "Erik Poppe", 
		email = "erik.poppe@qut.edu.au"
	)

	public ModelViz visualise(final UIPluginContext context, PetrinetGraph pnet)
	{	
		//System.out.println("threeDeeMe");

		//get graph layout connection
		GraphLayoutConnection layoutConn = null;
		try 
		{
			layoutConn = context.getConnectionManager().getFirstConnection(GraphLayoutConnection.class, context, pnet);
		} 
		catch (ConnectionCannotBeObtained exc)
		{
			// do nothing
			System.out.println("ThreeDeeModel: cannot find graph layout!");
		}

		ModelViz mviz = new ModelViz();
		mviz.pnet = pnet;
		mviz.layoutConn = layoutConn;
		
		return mviz;
	}
	
	
}
