package org.processmining.processprofiler3d.compviz.plugins;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.model.ProMResource;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.processprofiler3d.compviz.plugins.data.ModelReplayViz;
import org.processmining.processprofiler3d.psen.connections.PSENConnection;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */
public class ThreeDeeModelReplay 
{
	@Plugin(
		name = "Process Profiler 3D: Visualize Replay", 
		//parameterLabels = { "Petri Net", "Log", "Log Alignment" }, 
		parameterLabels = { "Petri Net", "Event Log", "Performance Statistics Enriched Net" }, 
		returnLabels = { "3D Model Visualisation" }, 
		returnTypes = { ModelReplayViz.class }, 
		userAccessible = true, 
		help = "This plug-in visualizes event log replays on a process model in 3D."
	)
	@UITopiaVariant(
		affiliation = "Queensland University of Technology", 
		author = "Erik Poppe", 
		email = "erik.poppe@qut.edu.au"
	)

	
	public ModelReplayViz threeDeeMe(final UIPluginContext context, PetrinetGraph pnet, XLog log, IPerformanceStatisticsEnrichedNet psnet)
	{	
		//get graph layout connection
		GraphLayoutConnection layoutConn = null;
		try 
		{
			layoutConn = context.getConnectionManager().getFirstConnection(GraphLayoutConnection.class, context, pnet);
		} 
		catch (ConnectionCannotBeObtained exc)
		{
			// do nothing
			System.out.println("ThreeDeeModelReplay: cannot find graph layout!");
		}
		
		//check if a replay connection is available, otherwise fail gracefully
//		PSENLogReplayConnection lrc = null;
//		try 
//		{
//			lrc = context.getConnectionManager().getFirstConnection(PSENLogReplayConnection.class, context, pnet);
//		} 
//		catch (ConnectionCannotBeObtained e) 
//		{
//			System.out.println("ThreeDeeModelReplay: cannot find replay connection!");
//		}

		PSENConnection wtic = null;
		try 
		{
			wtic = context.getConnectionManager().getFirstConnection(PSENConnection.class, context, psnet);
			System.out.println("Found waiting time info connection!");
		} 
		catch (ConnectionCannotBeObtained e) 
		{
			System.out.println("Could not find waiting time info connection!");
		}
		
		if (wtic != null)
		{
			boolean psenFits = psnet.fitsPetrinet(pnet);
			boolean wticFits = wtic.getWaitingTimeEnrichedNet().fitsPetrinet(pnet);
	
			if /*(lrc != null &&*/(psenFits && wticFits)
			{
				for (ProMResource<?> pr : context.getGlobalContext().getResourceManager().getAllResources())
				{
					System.out.println("Resource: " + pr.getName());
				}
				
				ModelReplayViz mviz = new ModelReplayViz();
				mviz.pnet = pnet;
				mviz.layoutConn = layoutConn;
				mviz.mylog = log;
				mviz.psen = psnet;
				mviz.wten = wtic.getWaitingTimeEnrichedNet();
				return mviz;
	//			JoglPanel panel = new JoglPanel();
	//			if (panel.isContextInitialized())
	//			{
	//				JoglScene scene = new JoglModelReplayScene(panel, pnet, layoutConn, context);
	//				panel.setScene(scene);
	//				panel.setBackground(0.3f, 0.5f, 0.7f);
	//				
	//				return panel;
	//			}
			}
		}

		context.getFutureResult(0).cancel(true);
		return null;
	}
}
