package org.processmining.processprofiler3d.compviz.plugins;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.processprofiler3d.compviz.plugins.data.ModelPSENViz;
import org.processmining.processprofiler3d.psen.connections.PSENConnection;
import org.processmining.processprofiler3d.psen.connections.PSENLogReplayConnection;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayCollection;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */
@Plugin(
	name = "Process Profiler 3D: Visualize Performance Profile", 
	parameterLabels = { "Petrinet Graph", "Performance Statistics Enriched Net", "XLog" }, 
	returnLabels = { "3D Model Visualisation" }, 
	returnTypes = { ModelPSENViz.class }, 
	userAccessible = true, 
	help = "This plug-in visualizes the differences between different event log cohorts on a process model"
)
public class ThreeDeePSEN 
{
    @PluginVariant(requiredParameterLabels = { 0,1 })
	@UITopiaVariant(
		affiliation = "Queensland University of Technology", 
		author = "Erik Poppe", 
		email = "erik.poppe@qut.edu.au"
	)
	public ModelPSENViz threeDeeMe(final UIPluginContext context, PetrinetGraph pnet, IPerformanceStatisticsEnrichedNet psnet)
	{	
		//System.out.println("threeDeeMe");

		//PetrinetGraph pnet = psnet.getPetriNet();
		
		//get graph layout connection
		GraphLayoutConnection layoutConn = null;
		try 
		{
			layoutConn = context.getConnectionManager().getFirstConnection(GraphLayoutConnection.class, context, pnet);
		} 
		catch (ConnectionCannotBeObtained exc)
		{
			// do nothing
			System.out.println("ThreeDeePSEN: cannot find graph layout!");
		}

		PSENConnection wtic = null;
		try 
		{
			wtic = context.getConnectionManager().getFirstConnection(PSENConnection.class, context, psnet);
			System.out.println("Found waiting time info connection!");
		} 
		catch (ConnectionCannotBeObtained e) 
		{
			System.out.println("Could not find waiting time info connection!");
		}
		
		if (wtic != null)
		{
			boolean psenFits = psnet.fitsPetrinet(pnet);
			boolean wticFits = wtic.getWaitingTimeEnrichedNet().fitsPetrinet(pnet);
	
			if (psenFits && wticFits)
			{
				ModelPSENViz mviz = new ModelPSENViz();
				mviz.pnet = pnet;
				mviz.layoutConn = layoutConn;
				mviz.psen = psnet;
				mviz.wten = wtic.getWaitingTimeEnrichedNet();
				mviz.trc = null; //getReplayCollection(context, pnet);
				
				return mviz;
	//			JoglPanel panel = new JoglPanel();
	//			if (panel.isContextInitialized())
	//			{
	//				JoglScene scene = new JoglPSENVisualizationScene(panel, pnet, psnet, layoutConn, wtic);
	//				panel.setScene(scene);
	//				panel.setBackground(0.3f, 0.5f, 0.7f);
	//				
	//				return panel;
	//			}
			}
			else
			{
				if (!psenFits)
				{
					System.out.println("PSEN does not fit Petrinet provided!");
				}
				if (!wticFits)
				{
					System.out.println("PSENWaitingTimeTree does not fit Petrinet provided!");
				}
			}
		}
		
		context.getFutureResult(0).cancel(true);
		return null;
	}

    @PluginVariant(requiredParameterLabels = { 0,1,2 })
	@UITopiaVariant(
			affiliation = "Queensland University of Technology", 
			author = "Erik Poppe", 
			email = "erik.poppe@qut.edu.au"
		)
	public ModelPSENViz threeDeeMe(final UIPluginContext context, PetrinetGraph pnet, IPerformanceStatisticsEnrichedNet psnet, XLog log)
	{	
		//System.out.println("threeDeeMe");

		//PetrinetGraph pnet = psnet.getPetriNet();
		
		//get graph layout connection
		GraphLayoutConnection layoutConn = null;
		try 
		{
			layoutConn = context.getConnectionManager().getFirstConnection(GraphLayoutConnection.class, context, pnet);
		} 
		catch (ConnectionCannotBeObtained exc)
		{
			// do nothing
			System.out.println("ThreeDeePSEN: cannot find graph layout!");
		}

		PSENConnection wtic = null;
		try 
		{
			wtic = context.getConnectionManager().getFirstConnection(PSENConnection.class, context, psnet);
			System.out.println("Found waiting time info connection!");
		} 
		catch (ConnectionCannotBeObtained e) 
		{
			System.out.println("Could not find waiting time info connection!");
		}
		
		if (wtic != null)
		{
			boolean psenFits = psnet.fitsPetrinet(pnet);
			boolean wticFits = wtic.getWaitingTimeEnrichedNet().fitsPetrinet(pnet);
	
			if (psenFits && wticFits)
			{
				ModelPSENViz mviz = new ModelPSENViz();
				mviz.pnet = pnet;
				mviz.layoutConn = layoutConn;
				mviz.log = log;
				mviz.psen = psnet;
				mviz.wten = wtic.getWaitingTimeEnrichedNet();
				mviz.trc = getReplayCollection(context, pnet, log);
				
				return mviz;
	//			JoglPanel panel = new JoglPanel();
	//			if (panel.isContextInitialized())
	//			{
	//				JoglScene scene = new JoglPSENVisualizationScene(panel, pnet, psnet, layoutConn, wtic);
	//				panel.setScene(scene);
	//				panel.setBackground(0.3f, 0.5f, 0.7f);
	//				
	//				return panel;
	//			}
			}
			else
			{
				if (!psenFits)
				{
					System.out.println("PSEN does not fit Petrinet provided!");
				}
				if (!wticFits)
				{
					System.out.println("PSENWaitingTimeTree does not fit Petrinet provided!");
				}
			}
		}
		
		context.getFutureResult(0).cancel(true);
		return null;
	}
    
    private TraceReplayCollection getReplayCollection(final UIPluginContext context, PetrinetGraph pnet, XLog log)
    {
    	PSENLogReplayConnection layoutConn = null;
		try 
		{
			layoutConn = context.getConnectionManager().getFirstConnection(PSENLogReplayConnection.class, context, pnet, log);
		} 
		catch (ConnectionCannotBeObtained exc)
		{
			// do nothing
			System.out.println("ThreeDeeModel: cannot find graph layout!");
		}
		
		if (layoutConn != null)
		{
			return layoutConn.getObjectWithRole(PSENLogReplayConnection.REPLAYANALYSER);
		}
		
    	return null;
    }
		
	
	
}
