package org.processmining.processprofiler3d.compviz.plugins.visualisers;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.PP3DAppModelReplayVisualiser;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.compviz.plugins.data.ModelReplayViz;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglModelReplayScene;
import org.processmining.processprofiler3d.compviz.scenemanagement.PP3DApplicationInitializer;
import org.processmining.processprofiler3d.psen.AlignedLog;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.connections.PSENLogReplayConnection;
import org.processmining.processprofiler3d.psen.gui.SlickerOptionPane;
import org.processmining.processprofiler3d.psen.plugins.ModelComparison3;
import org.processmining.processprofiler3d.psen.plugins.PP3DPluginBase;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayCollection;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.WatcherThread;
import org.processmining.visualisation3d.scenemanagement.JoglScene;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */
@Plugin(
	name = "Process Profiler 3D: Visualize Replay", 
	//parameterLabels = { "Petri Net", "Log", "Log Alignment" }, 
	parameterLabels = { "Petri Net", "canceller" }, 
	returnLabels = { "3D Model Visualisation" }, 
	returnTypes = { JComponent.class }, 
	userAccessible = true, 
	help = "This plug-in visualizes event log replays on a process model in 3D."
)
@Visualizer
public class ThreeDeeModelReplayViz extends PP3DPluginBase
{
    @PluginVariant(requiredParameterLabels = { 0, 1 })
	public JComponent threeDeeMe(final UIPluginContext context, ModelReplayViz mviz, ProMCanceller canceller)
	{
		//get initial markings
		Marking initMarking = getInitialMarking(context, mviz.pnet);
		if (initMarking == null)
		{
			//SlickerOptionPane.showMessageDialog(new JPanel(), "Initial Marking for petrinet could not be found or created! Cannot simulate replay.");
			SlickerOptionPane.showErrorMessage(context, "Initial Marking for petrinet could not be found or created! Cannot simulate replay.", "Error");
			System.out.println("Initial Marking for petrinet could not be found or created! Cannot simulate replay.");

			context.getFutureResult(0).cancel(true);
			return null;
		}		
		
		if (initMarking.isEmpty())
		{
			//SlickerOptionPane.showMessageDialog(new JPanel(), "Initial Marking for petrinet is empty! Cannot simulate replay.");
			SlickerOptionPane.showErrorMessage(context, "Initial Marking for petrinet is empty! Cannot simulate replay.", "Error");
			System.out.println("Initial Marking for petrinet is empty! Cannot simulate replay.");
			
			context.getFutureResult(0).cancel(true);
			return null;
		}
		
		TraceReplayCollection trc = null;
		
		//check if log is aligned log, otherwise can't generate trace replay collection
		LogAnalyser la = new LogAnalyser(mviz.mylog);
		if (AlignedLog.CheckLogIsAligned(la))
		{
			try
			{
				trc = ModelComparison3.GenerateTraceReplayCollection(context, new PetriNetDataSource(mviz.pnet), initMarking, mviz.mylog);
			}
			catch (Exception ex)
			{
				trc = null;
				System.out.println("Could not create trace replay collection from log!");
			}
		}

		//otherwise, check if a replay connection is available
		if (trc == null)
		{
			trc = getTraceReplayCollection(context, mviz);
		}

		if (trc != null)
		{
			JoglPanel panel = new JoglPanel();
			if (panel.isContextInitialized())
			{
				JoglScene scene = new JoglModelReplayScene();
				scene.setBackgroundColor(0.3f, 0.5f, 0.7f);

				PP3DAppModelReplayVisualiser app = new PP3DAppModelReplayVisualiser(mviz.pnet, mviz.layoutConn, trc, initMarking, mviz.mylog, mviz.psen, mviz.wten);
				
				panel.setCamera(app.InitCamera(panel));
				panel.setScene(scene);
				panel.getInteractionManager().addInteraction(new PP3DApplicationInitializer(panel, app));

				WatcherThread wthread = new WatcherThread(panel, canceller);
				wthread.start();
				
				return panel;
			}
		}
		else
		{
			//SlickerOptionPane.showMessageDialog(new JPanel(), "Could not obtain a trace replay collection! Cannot simulate replay.");
			SlickerOptionPane.showErrorMessage(context, "Could not obtain a trace replay collection! Cannot simulate replay.", "Error");
			System.out.println("Could not obtain a trace replay collection! Cannot simulate replay.");
		}

		context.getFutureResult(0).cancel(true);
		return null;
	}

	private TraceReplayCollection getTraceReplayCollection(final UIPluginContext context, ModelReplayViz mviz) 
	{
		TraceReplayCollection trc = null;
		try 
		{
			PSENLogReplayConnection lrc = context.getConnectionManager().getFirstConnection(PSENLogReplayConnection.class, context, mviz.pnet);
			trc = lrc.getObjectWithRole(PSENLogReplayConnection.REPLAYANALYSER);
		} 
		catch (ConnectionCannotBeObtained e) 
		{
			System.out.println("ThreeDeeModelReplay: cannot find replay connection!");
		}
		return trc;
	}
}
