package org.processmining.processprofiler3d.compviz.plugins.visualisers;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.processprofiler3d.compviz.PP3DAppModelVisualiser;
import org.processmining.processprofiler3d.compviz.plugins.data.ModelViz;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglModelScene;
import org.processmining.processprofiler3d.compviz.scenemanagement.PP3DApplicationInitializer;
import org.processmining.processprofiler3d.psen.plugins.PP3DPluginBase;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.WatcherThread;
import org.processmining.visualisation3d.scenemanagement.JoglScene;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */

public class ThreeDeeModelViz extends PP3DPluginBase
{
	@Plugin(
		name = "Process Profiler 3D: Visualize Model", 
		parameterLabels = { "ModelViz", "canceller"}, 
		returnLabels = { "3D Model Visualisation" }, 
		returnTypes = { JComponent.class }, 
		userAccessible = true, 
		help = "This plug-in visualizes a process model in 3D."
	)
	@Visualizer
    @PluginVariant(requiredParameterLabels = { 0, 1 })
	public JComponent visualise(final UIPluginContext context, ModelViz mviz, ProMCanceller canceller)
	{
		JoglPanel panel = new JoglPanel();
		if (panel.isContextInitialized())
		{
			JoglScene scene = new JoglModelScene();
			scene.setBackgroundColor(0.3f, 0.5f, 0.7f);
			
			PP3DAppModelVisualiser app = new PP3DAppModelVisualiser(mviz.pnet, mviz.layoutConn);

			panel.setCamera(app.InitCamera(panel));
			panel.setScene(scene);
			panel.getInteractionManager().addInteraction(new PP3DApplicationInitializer(panel, app));

			WatcherThread wthread = new WatcherThread(panel, canceller);
			wthread.start();
			
			context.getFutureResult(0).setLabel("3D Visualization");
			return panel;
		}

		context.getFutureResult(0).cancel(true);
		return null;
	}
    
    @PluginVariant(requiredParameterLabels = { 0, 1 })
	public JComponent visualise(final UIPluginContext context, Petrinet pnet, ProMCanceller canceller)
	{
    	//TestDB();
    	
		//get graph layout connection
		GraphLayoutConnection layoutConn = getGraphLayoutFor(context, pnet);
		
		JoglPanel panel = new JoglPanel();
		if (panel.isContextInitialized())
		{
			JoglScene scene = new JoglModelScene();
			scene.setBackgroundColor(0.3f, 0.5f, 0.7f);
			
			PP3DAppModelVisualiser app = new PP3DAppModelVisualiser(pnet, layoutConn);

			panel.setCamera(app.InitCamera(panel));
			panel.setScene(scene);
			panel.getInteractionManager().addInteraction(new PP3DApplicationInitializer(panel, app));

			WatcherThread wthread = new WatcherThread(panel, canceller);
			wthread.start();

			context.getFutureResult(0).setLabel("3D Visualization");
			return panel;
		}

		context.getFutureResult(0).cancel(true);
		return null;
	}
}
