package org.processmining.processprofiler3d.compviz.plugins.visualisers;

import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.contexts.uitopia.model.ProMResource;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.PP3DAppModelPerformanceVisualiser;
import org.processmining.processprofiler3d.compviz.plugins.data.ModelPSENViz;
import org.processmining.processprofiler3d.compviz.scenemanagement.JoglPSENVisualizationScene;
import org.processmining.processprofiler3d.compviz.scenemanagement.PP3DApplicationInitializer;
import org.processmining.processprofiler3d.psen.connections.PSENConnection;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.gui.SlickerOptionPane;
import org.processmining.processprofiler3d.psen.plugins.PP3DPluginBase;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.WatcherThread;
import org.processmining.visualisation3d.scenemanagement.JoglScene;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */
@Plugin(
	name = "Process Profiler 3D: Visualize Performance Profile", 
	parameterLabels = { "Petrinet Graph", "canceller"}, 
	returnLabels = { "3D Model Visualisation" }, 
	returnTypes = { JComponent.class }, 
	userAccessible = true, 
	help = "This plug-in visualizes the differences between different event log cohorts on a process model"
)
@Visualizer
public class ThreeDeePSENViz extends PP3DPluginBase
{
    @PluginVariant(requiredParameterLabels = { 0, 1 })
	public JComponent threeDeeMe(final UIPluginContext context, ModelPSENViz mviz, ProMCanceller canceller)
	{
		JoglPanel panel = new JoglPanel();
		if (panel.isContextInitialized())
		{
			JoglScene scene = new JoglPSENVisualizationScene();
			scene.setBackgroundColor(0.3f, 0.5f, 0.7f);

			//get initial markings
			Marking initMarking = getInitialMarking(context, mviz.pnet);
			
			PP3DAppModelPerformanceVisualiser app = new PP3DAppModelPerformanceVisualiser(mviz.pnet, mviz.layoutConn, mviz.log, initMarking, mviz.psen, mviz.wten, mviz.trc);

			panel.setCamera(app.InitCamera(panel));
			panel.setScene(scene);
			panel.getInteractionManager().addInteraction(new PP3DApplicationInitializer(panel, app));

			WatcherThread wthread = new WatcherThread(panel, canceller);
			wthread.start();
			
			context.getFutureResult(0).setLabel("3D Visualization");
			return panel;
		}
		
		context.getFutureResult(0).cancel(true);
		return null;
	}
		

//    @PluginVariant(requiredParameterLabels = { 0 })
//	public JComponent threeDeeMe(final UIPluginContext context, IPerformanceStatisticsEnrichedNet psen)
//	{
//    	IPerformanceStatisticsEnrichedNet wten = getWaitingTimeInfo(context, psen);
//    	
//    	Petrinet pnet = getPetrinetFor(context, psen, wten);
//    	if (pnet == null)
//    	{
//			System.out.println("ThreeDeePSENViz: no fitting petrinet found!");
//    		context.getFutureResult(0).cancel(true);
//    		return null;
//    	}
//    	
//    	GraphLayoutConnection layoutConn = getGraphLayoutFor(context, pnet);
//    	
//    	if (pnet != null && layoutConn != null && wten != null)
//    	{
//			JoglPanel panel = new JoglPanel();
//			if (panel.isContextInitialized())
//			{
//				JoglScene scene = new JoglPSENVisualizationScene(panel, pnet, layoutConn, null, psen, wten);
//				panel.setScene(scene);
//				panel.setBackground(0.3f, 0.5f, 0.7f);
//				
//				context.getFutureResult(0).setLabel("3D Visualization");
//				return panel;
//			}
//    	}
//		
//		context.getFutureResult(0).cancel(true);
//		return null;
//	}
    
    //get all fitting petri nets that currently exist in ProM and let the user choose which one to use for the visualization if more than one exist
    private Petrinet getPetrinetFor(final UIPluginContext context, IPerformanceStatisticsEnrichedNet psen, IPerformanceStatisticsEnrichedNet wten)
    {
    	Vector<ProMResource<?>> lst = new Vector<ProMResource<?>>();
		for (ProMResource<?> pr : context.getGlobalContext().getResourceManager().getAllResources())
		{
			//System.out.println("Resource: " + pr.getName());
			if (pr.getInstance() instanceof Petrinet)
			{
				Petrinet pnet = (Petrinet)pr.getInstance();
				if (psen.fitsPetrinet(pnet) && wten.fitsPetrinet(pnet))
				{
					lst.add(pr);
				}
			}
		}
		
		if (lst.size() == 1)
		{
			//if there is only one fitting net, automatically select it
			return (Petrinet)lst.get(0).getInstance();
		}
		else if (lst.size() > 0)
		{
			//if there are several fitting nets, let the user choose
			Object[] strs = new Object[lst.size()];
			int i = 0;
			for (ProMResource<?> pr : lst)
			{
				strs[i] = pr;
				i++;
			}
			Object choice = SlickerOptionPane.showInputDialog(null, "Please choose a petri net to visualize the PSEN on.", "Choose Petri Net",
			        JOptionPane.QUESTION_MESSAGE, null, strs, strs[0]);
			
			return (Petrinet)((ProMResource<?>)choice).getInstance();
		}
		return null;
    }
    
    private IPerformanceStatisticsEnrichedNet getWaitingTimeInfo(final UIPluginContext context, IPerformanceStatisticsEnrichedNet psen)
    {
		PSENConnection pConn = null;
		try 
		{
			pConn = context.getConnectionManager().getFirstConnection(PSENConnection.class, context, psen);
			return pConn.getWaitingTimeEnrichedNet();
		} 
		catch (ConnectionCannotBeObtained exc)
		{
			// do nothing
			System.out.println("ThreeDeePSENViz: cannot find waiting time info!");
		}
		catch (Exception ex)
		{
			// do nothing
			System.out.println("ThreeDeePSENViz: issue with waiting time info!");
		}
    	return null;
    }
}
