package org.processmining.processprofiler3d.compviz.scenemanagement;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class BSPTreeNode 
{
	public int axis = -1;
	public double value = -1;
	
	public BSPPlane plx = null;
	public BSPTreeNode front = null;
	public BSPTreeNode back = null;
	
	public Vector<JSOSubNet> leafs = null;
	
	public boolean Build(Vector<JSOSubNet> layer)
	{
		if (layer.size() == 1)
		{
			leafs = new Vector<JSOSubNet>();
			leafs.add(layer.firstElement());
			return true;
		}
		
		Vector<BSPPlane> planes = getSeparatingPlanes(layer);		
		Vector<BSPPlane> goodplanes = getGoodPlanes(layer, planes);
		
		if (goodplanes.size() > 0)
		{
			//System.out.println("Managed to split set!");
			Vector<JSOSubNet> inobjs = new Vector<JSOSubNet>();
			Vector<JSOSubNet> outobjs = new Vector<JSOSubNet>();
			
			SplitObjsByPlane(layer, goodplanes.firstElement(), inobjs, outobjs);

			plx = goodplanes.firstElement();
			front = new BSPTreeNode();
			back = new BSPTreeNode();
			
			boolean build1 = front.Build(outobjs);
			boolean build2 = back.Build(inobjs);
			
			return build1 && build2;
		}
		else
		{
			//System.out.println("Failed to split set!");
			leafs = new Vector<JSOSubNet>();
			leafs.addAll(layer);
			
			return false;
		}
	}

	public void SplitObjsByPlane(Vector<JSOSubNet> layer, BSPPlane splitplane, Vector<JSOSubNet> inobjs, Vector<JSOSubNet> outobjs) 
	{
		for (JSOSubNet snet : layer)
		{
			Vector<JoglVectord3> lstB = snet.ped.getPedestalRectPointsAbsolute2();
			
			int outpts = 0;
			int inpts = 0;
			
			for (int j = 0; j < 5; j++)
			{
				double res = JoglVectord3.Dot(lstB.get(j), splitplane.normal) - splitplane.offset;
				if (res > 0)
				{
					outpts++;
				}
				else
				{
					inpts++;
				}
			}
			
			if (outpts == 5)
			{
				outobjs.add(snet);
			}
			else if (inpts == 5)
			{
				inobjs.add(snet);
			}
		}
	}

	private Vector<BSPPlane> getGoodPlanes(Vector<JSOSubNet> layer, Vector<BSPPlane> planes) 
	{
		Vector<BSPPlane> goodplanes = new Vector<BSPPlane>();
		
		//for each potential plane
		for (BSPPlane pl : planes)
		{	
			for (JSOSubNet snet : layer)
			{
				Vector<JoglVectord3> lstB = snet.ped.getPedestalRectPointsAbsolute2();
				
				int outpts = 0;
				int inpts = 0;
				
				for (int j = 0; j < 5; j++)
				{
					double res = JoglVectord3.Dot(lstB.get(j), pl.normal) - pl.offset;
					if (res > 0)
					{
						outpts++;
					}
					else
					{
						inpts++;
					}
				}
				
				if (outpts == 5)
				{
					pl.outobj++;
				}
				else if (inpts == 5)
				{
					pl.inobj++;
				}
				else
				{
					pl.clipobj++;
				}
			}
			
			//plane separates some objects in space, but does not clip any objects
			if (pl.clipobj == 0 && pl.inobj != 0 && pl.outobj != 0)
			{
				goodplanes.add(pl);
			}
		}
		return goodplanes;
	}

	private Vector<BSPPlane> getSeparatingPlanes(Vector<JSOSubNet> layer) 
	{
		Vector<BSPPlane> planes = new Vector<BSPPlane>();
		
		//get potential separating planes
		for (JSOSubNet snet : layer)
		{
			Vector<JoglVectord3> lstA = snet.ped.getPedestalRectPointsAbsolute2();
	
			JoglVectord3 a = lstA.get(0);
			for (int i = 0; i < 4; i++)
			{
				int id1 = (i) % 4;
				int id2 = (i+1) % 4;
				
				JoglVectord3 b = lstA.get(id1 + 1);
				JoglVectord3 c = lstA.get(id2 + 1);
				
				//calculate normal of plane and offset
				JoglVectord3 v1 = JoglVectord3.Normal(JoglVectord3.Subtract(b, a));
				JoglVectord3 v2 = JoglVectord3.Normal(JoglVectord3.Subtract(c, a));
				JoglVectord3 n = JoglVectord3.Cross(v1, v2);
				n = JoglVectord3.Normal(n);
				double ares = JoglVectord3.Dot(a, n) + 0.5;
				
				BSPPlane pl = new BSPPlane(n, ares);		
				planes.add(pl);
			}
		}
		return planes;
	}
	
	public void Traverse(JoglCamera cam, Vector<JSOSubNet> nets)
	{
		if (leafs != null)
		{
			nets.addAll(leafs);
		}
		else
		{
			double res = JoglVectord3.Dot(cam.GetPosition(), plx.normal) - plx.offset;
			if (res > 0)
			{
				back.Traverse(cam, nets);
				front.Traverse(cam, nets);
			}
			else
			{
				front.Traverse(cam, nets);
				back.Traverse(cam, nets);
			}
		}
	}
}
