package org.processmining.processprofiler3d.compviz.scenemanagement;

import java.util.Vector;

import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLens;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLensOrthographic;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLensPerspective;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class JoglFrustumClipper 
{
        private JoglCamera cam;

        private double near;
        private double far;

        private JoglVectord3 lpos;
        private JoglVectord3 rpos;
        private JoglVectord3 tpos;
        private JoglVectord3 bpos;
        
        private JoglVectord3 lnorm;
        private JoglVectord3 rnorm;
        private JoglVectord3 tnorm;
        private JoglVectord3 bnorm;

        private boolean bDontCull = false; //can't cull for frustum of over 180 degree, so don't pretend to be able to

        public JoglFrustumClipper(JoglCamera cam, double aspect)
        {
            this.cam = cam;
            JoglCameraLens cLens = cam.getCameraLens();
            if (!(cLens instanceof JoglCameraLensPerspective))
            {
//                bDontCull = true;
//                System.out.println("Can't cull against orthogonal projection yet!");
            	InitOrtho(cam, aspect, (JoglCameraLensOrthographic)cLens);
                return;
            }

            InitPerspective(cam, aspect, (JoglCameraLensPerspective)cLens);
        }
        
        public JoglFrustumClipper(JoglCamera cam, double aspect, double l, double r, double t, double b)
        {
            this.cam = cam;
            JoglCameraLens cLens = cam.getCameraLens();
            if (!(cLens instanceof JoglCameraLensPerspective))
            {
                //bDontCull = true;
                //System.out.println("Can't cull against orthogonal projection yet!");
            	InitOrtho(cam, aspect, (JoglCameraLensOrthographic)cLens, l, r, t, b);
                return;
            }

            InitPerspective(cam, aspect, l, r, t, b, (JoglCameraLensPerspective)cLens);
        }

		private void InitOrtho(JoglCamera cam, double aspect, JoglCameraLensOrthographic cLensP) 
		{
			cLensP.GetRay(cam, new JoglVectord2(0,0));
			cLensP.GetRay(cam, new JoglVectord2(1,1));

            lnorm = cam.GetLeftDir();
            rnorm = JoglVectord3.Multiply(cam.GetLeftDir(), -1.0);
            tnorm = cam.GetUpDir();
            bnorm = JoglVectord3.Multiply(cam.GetUpDir(), -1.0);
            
            lpos = cLensP.GetRay(cam, new JoglVectord2(1,1)).getOrigin();
            rpos = cLensP.GetRay(cam, new JoglVectord2(0,1)).getOrigin();
            tpos = cLensP.GetRay(cam, new JoglVectord2(0,1)).getOrigin();
            bpos = cLensP.GetRay(cam, new JoglVectord2(1,0)).getOrigin();
		}

		private void InitOrtho(JoglCamera cam, double aspect, JoglCameraLensOrthographic cLensP, double l, double r, double t, double b) 
		{
			cLensP.GetRay(cam, new JoglVectord2(0,0));
			cLensP.GetRay(cam, new JoglVectord2(1,1));

            lnorm = cam.GetLeftDir();
            rnorm = JoglVectord3.Multiply(cam.GetLeftDir(), -1.0);
            tnorm = cam.GetUpDir();
            bnorm = JoglVectord3.Multiply(cam.GetUpDir(), -1.0);
            
            lpos = cLensP.GetRay(cam, new JoglVectord2((l * 0.5) + 0.5,1)).getOrigin();
            rpos = cLensP.GetRay(cam, new JoglVectord2((r * 0.5) + 0.5,1)).getOrigin();
            tpos = cLensP.GetRay(cam, new JoglVectord2(0,(t * 0.5) + 0.5)).getOrigin();
            bpos = cLensP.GetRay(cam, new JoglVectord2(1,(b * 0.5) + 0.5)).getOrigin();
		}

		private void InitPerspective(JoglCamera cam, double aspect, JoglCameraLensPerspective cLensP) 
		{
            if (cLensP.getFieldOfView() >= 180.0 || (cLensP.getFieldOfView() * aspect) > 180.0)
            {
                bDontCull = true;
                System.out.println("Can't cull against frustum of > 180 degree!");
            }

            double constCamFactor = (float)cLensP.getTanFoV();          

            JoglVectord3 pos = cam.GetPosition();
            JoglVectord3 lpoint = JoglVectord3.Add(pos, JoglVectord3.Add(cam.GetLookDir(), JoglVectord3.Multiply(cam.GetLeftDir(), constCamFactor * aspect)));
            JoglVectord3 rpoint = JoglVectord3.Add(pos, JoglVectord3.Add(cam.GetLookDir(), JoglVectord3.Multiply(cam.GetLeftDir(), -constCamFactor * aspect)));
            JoglVectord3 tpoint = JoglVectord3.Add(pos, JoglVectord3.Add(cam.GetLookDir(), JoglVectord3.Multiply(cam.GetUpDir(), constCamFactor)));
            JoglVectord3 bpoint = JoglVectord3.Add(pos, JoglVectord3.Add(cam.GetLookDir(), JoglVectord3.Multiply(cam.GetUpDir(), -constCamFactor)));

            JoglVectord3 loff = JoglVectord3.Subtract(lpoint, pos);
            JoglVectord3 roff = JoglVectord3.Subtract(rpoint, pos);
            JoglVectord3 toff = JoglVectord3.Subtract(tpoint, pos);
            JoglVectord3 boff = JoglVectord3.Subtract(bpoint, pos);

            lnorm = JoglVectord3.Cross(loff, JoglVectord3.Multiply(cam.GetUpDir(), -1.0));
            rnorm = JoglVectord3.Cross(roff, cam.GetUpDir());
            tnorm = JoglVectord3.Cross(toff, cam.GetLeftDir());
            bnorm = JoglVectord3.Cross(boff, JoglVectord3.Multiply(cam.GetLeftDir(), -1.0));

            lnorm = JoglVectord3.Normal(lnorm);
            rnorm = JoglVectord3.Normal(rnorm);
            tnorm = JoglVectord3.Normal(tnorm);
            bnorm = JoglVectord3.Normal(bnorm);
            
            lpos = pos;
            rpos = pos;
            tpos = pos;
            bpos = pos;
		}

		private void InitPerspective(JoglCamera cam, double aspect, double l, double r, double t, double b,
				JoglCameraLensPerspective cLensP) 
		{
            if (cLensP.getFieldOfView() >= 180.0 || (cLensP.getFieldOfView() * aspect) > 180.0)
            {
                bDontCull = true;
                System.out.println("Can't cull against frustum of > 180 degree!");
            }

            double constCamFactor = (float)cLensP.getTanFoV();       

            JoglVectord3 pos = cam.GetPosition();
            JoglVectord3 lpoint = JoglVectord3.Add(pos, JoglVectord3.Add(cam.GetLookDir(), JoglVectord3.Multiply(cam.GetLeftDir(), constCamFactor * -l  * aspect)));
            JoglVectord3 rpoint = JoglVectord3.Add(pos, JoglVectord3.Add(cam.GetLookDir(), JoglVectord3.Multiply(cam.GetLeftDir(), constCamFactor * -r  * aspect)));
            JoglVectord3 tpoint = JoglVectord3.Add(pos, JoglVectord3.Add(cam.GetLookDir(), JoglVectord3.Multiply(cam.GetUpDir(), constCamFactor * -t)));
            JoglVectord3 bpoint = JoglVectord3.Add(pos, JoglVectord3.Add(cam.GetLookDir(), JoglVectord3.Multiply(cam.GetUpDir(), constCamFactor * -b)));

            JoglVectord3 loff = JoglVectord3.Subtract(lpoint, pos);
            JoglVectord3 roff = JoglVectord3.Subtract(rpoint, pos);
            JoglVectord3 toff = JoglVectord3.Subtract(tpoint, pos);
            JoglVectord3 boff = JoglVectord3.Subtract(bpoint, pos);

            lnorm = JoglVectord3.Cross(loff, JoglVectord3.Multiply(cam.GetUpDir(), -1.0));
            rnorm = JoglVectord3.Cross(roff, cam.GetUpDir());
            tnorm = JoglVectord3.Cross(toff, cam.GetLeftDir());
            bnorm = JoglVectord3.Cross(boff, JoglVectord3.Multiply(cam.GetLeftDir(), -1.0));

            lnorm = JoglVectord3.Normal(lnorm);
            rnorm = JoglVectord3.Normal(rnorm);
            tnorm = JoglVectord3.Normal(tnorm);
            bnorm = JoglVectord3.Normal(bnorm);
            
            lpos = pos;
            rpos = pos;
            tpos = pos;
            bpos = pos;
		}
        
        public boolean IsPointInside(JoglVectord3 vpos)
        {
            if (bDontCull)
            {
                return true;
            }

            Vector<JoglVectord3> testv = new Vector<JoglVectord3>();
            testv.add(JoglVectord3.Multiply(cam.GetLookDir(), -1.0));
            testv.add(lnorm);
            testv.add(rnorm);
            testv.add(tnorm);
            testv.add(bnorm);
            
            Vector<JoglVectord3> testp = new Vector<JoglVectord3>();
            testp.add(cam.GetPosition());
            testp.add(lpos);
            testp.add(rpos);
            testp.add(tpos);
            testp.add(bpos);

            JoglVectord3 sdir = JoglVectord3.Subtract(vpos, cam.GetPosition());

            boolean bClip = false;
            for (int i = 0; i < testv.size(); i++)
            {
                double sdirdot = JoglVectord3.Dot(testv.get(i), sdir);
                //double mindist = 0; //minimum distance point needs to be from frustum to be considered "outside"
                double mindist = JoglVectord3.Dot(testv.get(i), JoglVectord3.Subtract(testp.get(i), cam.GetPosition())); //minimum distance point needs to be from frustum to be considered "outside"

                if (sdirdot > mindist)
                {
                    bClip = true;
                    break;
                }
            }

            if (!bClip)
            {
                return true;
            }

            return false;
        }
	
}
