package org.processmining.processprofiler3d.compviz.scenemanagement;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.interactions.replay.LaneCache;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENEdgeVisualizationLayer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENTrajectoryVisualizationLayer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOReplaya;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JoglModelReplayScene extends JoglModelScene 
{	
	protected JSOReplaya replaya = null;
	
	public JoglModelReplayScene()
	{
		
	}
	
	public JSOReplaya getReplaya()
	{
		return replaya;
	}
	
	@Override
	public void addSceneObject(JoglSceneObject jso)
	{
		if (jso instanceof JSOReplaya)
		{
			replaya = (JSOReplaya)jso;
			filtered.add(jso);
		}
		super.addSceneObject(jso);
	}
	
	@Override
	public Vector<JoglAABB> getBoundingBoxes()
	{
		//modify camera depth range based on scene
		Vector<JoglAABB> lst = new Vector<JoglAABB>();
		//lst.add(pnet.getAABB());
		
		JSOPetriNet pnet = findSceneObject(JSOPetriNet.class);
		//for (JSOSubNet snet : scene.findSceneObjects(JSOSubNet.class))
		for (JSOSubNet snet : pnet.getAllSubNets())
		{
			if (snet.getIsVisible())
			{
				lst.add(snet.getAABB());
				
				//use AABB for pedestals as well, to avoid oversized pedals from being clipped by the near-view-plane
				if (snet.ped != null)
				{
					lst.add(snet.ped.getAABB());
				}
			}
		}
		
		//cam.calcDepthRange(pnet.getAABB());
		
		for (JSOPSENEdgeVisualizationLayer edgeVis : findSceneObjects(JSOPSENEdgeVisualizationLayer.class))
		{
			JSOSubNet snet = edgeVis.getSubNet();
			if (snet.getIsVisible() && edgeVis.getElementCount() > 0)
			{
				JoglVectord3 offset = snet.relativeToAbsolute(JoglVectord3.ZERO);
				lst.add(edgeVis.getAABB().Translate(offset));		
			}
		}
		
		JSOPSENTrajectoryVisualizationLayer jptv = findSceneObject(JSOPSENTrajectoryVisualizationLayer.class);
		if (jptv != null)
		{
			JoglAABB jptvAABB = jptv.getAABB();
			if (jptvAABB != null)
			{
				lst.add(jptvAABB);
			}
		}
		
		lst.add(pnet.getAABB());
		
		return lst;
	}
	
	@Override
	public void Draw(GraphicsPipeline pipe)
	{
		pipe.cam.DrawCamera(pipe);

		DrawModel(pipe);

		for (int i = 0; i < objs.size(); i++)
		{
			JoglSceneObject jso = objs.get(i);
			if (!filtered.contains(jso))
			{
				jso.DrawObject(pipe);
			}
		}

		if (bDrawGUI)
		{
			DrawGui(pipe);
		}
	}

	@Override
	protected void DrawSubnet(GraphicsPipeline pipe, JSOSubNet snet) 
	{
		JoglCamera c = pipe.cam;
		
		snet.ped.UpdateGeometry(pipe);
		
		int netLevel = snet.getLevel();
		
		pipe.transform.Push();
		JoglMatrixd4x4 transform = snet.getTransform();
		pipe.transform.MultMatrix(transform);

		snet.DrawModelMask(pipe);
		
		//if cam above subnet
		boolean bCamAboveLayer = c.GetPosition().getY() >= snet.getSubnetYPos();
		if (bCamAboveLayer)
		{
			DrawFromAbove(pipe, netLevel, snet);
		}
		else
		{
			DrawFromBelow(pipe, netLevel, snet);
		}
		if (netLevel > 0)
		{
			snet.ped.DrawColor(pipe);
		}
		
		pipe.transform.Pop();
	}
	
	private void DrawFromAbove(GraphicsPipeline pipe, int netLevel, JSOSubNet snet)
	{
		if (netLevel == 0)
		{
			grid.DrawGridWithParams(pipe, false, true, true);
		}

		snet.DrawModel(pipe);

		replaya.DrawObject(pipe, snet);
		
	}
	
	private void DrawFromBelow(GraphicsPipeline pipe, int netLevel, JSOSubNet snet)
	{
		replaya.DrawObject(pipe, snet); //Draw before so that is visible through pedestal
		
		snet.DrawModel(pipe);
		if (netLevel == 0)
		{
			grid.DrawGridWithParams(pipe, false, true, true);
		}
	}

	@Override
	public void Dispose(GraphicsPipeline pipe)
	{
		//PersistSave();
		
		super.Dispose(pipe);
		
		replaya.Dispose();
		replaya = null;
		LaneCache.Clear();
	}
}
