package org.processmining.processprofiler3d.compviz.scenemanagement;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.scene.JSOCamWidgetBase;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOGrid;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JoglModelScene extends JoglScene 
{	
	protected JSOPetriNet pnet;
	protected Vector<JSOSubNet> snets = new Vector<JSOSubNet>();
	protected JSOGrid grid;
	protected JSOCamWidgetBase camWidget;
	
//	protected BSPTree sceneTree;
	
	protected Vector<JoglSceneObject> filtered = new Vector<JoglSceneObject>();
	
	public JoglModelScene()
	{		
	}
	
	public JSOPetriNet getPetriNet()
	{
		return pnet;
	}
	
//	public void setBSPTree(BSPTree bspTree)
//	{
//		sceneTree = bspTree;
//	}

	@Override
	public void addSceneObject(JoglSceneObject jso)
	{
		if (jso instanceof JSOPetriNet)
		{
			pnet = (JSOPetriNet)jso;
			filtered.add(jso);
		}
		else if (jso instanceof JSOSubNet)
		{
			snets.add((JSOSubNet)jso);
			filtered.add(jso);
		}
		else if (jso instanceof JSOGrid)
		{
			grid = (JSOGrid)jso;
			filtered.add(jso);
		}
		else if (jso instanceof JSOCamWidgetBase)
		{
			camWidget = (JSOCamWidgetBase)jso;
			filtered.add(jso);
		}
		else if (jso instanceof JSOGraphicalUserInterface)
		{
			filtered.add(jso);
		}
		super.addSceneObject(jso);
	}

	@Override
	public void removeSceneObject(JoglSceneObject jso)
	{
		if (jso == pnet)
		{
			pnet = null;
			filtered.remove(jso);
		}
		else if (jso instanceof JSOSubNet && snets.contains(jso))
		{
			snets.remove(jso);
			filtered.remove(jso);
		}
		else if (jso == grid)
		{
			grid = null;
			filtered.remove(jso);
		}
		else if (jso == camWidget)
		{
			camWidget = null;
			filtered.remove(jso);
		}
		else if (jso instanceof JSOGraphicalUserInterface)
		{
			filtered.remove(jso);
		}
		super.removeSceneObject(jso);
	}
	
	@Override
	public void Draw(GraphicsPipeline pipe)
	{
		pipe.cam.DrawCamera(pipe);

		DrawModel(pipe);

//		for (int i = 0; i < objs.size(); i++)
//		{
//			JoglSceneObject jso = objs.get(i);
//			if (jso instanceof JSOReplaya) //hack, filter out specific object to draw
//			{
//				jso.DrawObject(gl, this, target);
//			}
//		}

		for (int i = 0; i < objs.size(); i++)
		{
			JoglSceneObject jso = objs.get(i);
			if (!filtered.contains(jso))
			{
				jso.DrawObject(pipe);
			}
		}

		if (bDrawGUI)
		{
			DrawGui(pipe);
		}
	}
	
	protected void DrawModel(GraphicsPipeline pipe)
	{
		pnet.DrawObject(pipe);
		
//		//draw each subnet in order of BSPTree traversal
//		Vector<JSOSubNet> nets = new Vector<JSOSubNet>();
//		if (sceneTree != null)
//		{
//			sceneTree.Traverse(pipe.cam, nets);
//			
//			for (JSOSubNet snet : nets)
//			{
//				DrawSubnet(pipe, snet);
//			}
//		}
//		else
//		{
//			System.out.println("No BSP tree available for scene! Can not guarantee correct draw order!");
//		}
	}
	
	protected void DrawSubnet(GraphicsPipeline pipe, JSOSubNet snet) 
	{
		JoglCamera c = pipe.cam;
	
		snet.ped.UpdateGeometry(pipe);
		
		int netLevel = snet.getLevel();
		boolean bDrawGrid = false;
		if (netLevel == 0)
		{
			bDrawGrid = true;
		}
		
		pipe.transform.Push();
		JoglMatrixd4x4 transform = snet.getTransform();
		pipe.transform.MultMatrix(transform);

		//if cam above grid
		boolean bCamAboveLayer = c.GetPosition().getY() >= snet.getSubnetYPos();
		if (bCamAboveLayer)
		{
			if (bDrawGrid)
			{
				grid.DrawGridWithParams(pipe, false, true, true);
			}
			if (netLevel > 0)
			{
				//draw only to colorbuffer
				snet.ped.DrawColor(pipe);
			}
			snet.DrawObject(pipe);

			//draw only to depthbuffer
			if (netLevel > 0)
			{
				snet.ped.DrawDepth(pipe);
			}
		}
		else
		{
			snet.DrawObject(pipe);
			if (netLevel > 0)
			{
				snet.ped.DrawObject(pipe);
			}
			if (bDrawGrid)
			{
				grid.DrawGridWithParams(pipe, false, true, true);
			}
		}
		
		pipe.transform.Pop();
	}
	
	@Override
	public void DrawGui(GraphicsPipeline pipe)
	{
		gui.DrawObject(pipe);
		camWidget.DrawObject(pipe);
	}
	
	@Override
	public Vector<JoglAABB> getBoundingBoxes()
	{
		//modify camera depth range based on scene
		Vector<JoglAABB> lst = new Vector<JoglAABB>();
		//lst.add(pnet.getAABB());
		
		JSOPetriNet pnet = findSceneObject(JSOPetriNet.class);
		//for (JSOSubNet snet : scene.findSceneObjects(JSOSubNet.class))
		for (JSOSubNet snet : pnet.getAllSubNets())
		{
			if (snet.getIsVisible())
			{
				lst.add(snet.getAABB());
				
				//use AABB for pedestals as well, to avoid oversized pedals from being clipped by the near-view-plane
				if (snet.ped != null)
				{
					lst.add(snet.ped.getAABB());
				}
			}
		}
		
		lst.add(pnet.getAABB());
		
		return lst;
	}

	@Override
	public void Dispose(GraphicsPipeline pipe)
	{
		//PersistSave();
		
		super.Dispose(pipe);
		
		pnet.Dispose(pipe);
		camWidget.Dispose(pipe);
		
		pnet = null;
		snets.clear();
		grid = null;
		camWidget = null;
	}
}
