package org.processmining.processprofiler3d.compviz.scenemanagement;

import java.util.HashMap;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENEdgeVisualizationLayer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENTrajectoryVisualizationLayer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPSENTransitionVisualizationLayer;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOPetriNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOReplaya;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNet;
import org.processmining.processprofiler3d.compviz.jogl.scene.JSOSubNetStackRenderer;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglSceneObject;

public class JoglPSENVisualizationScene extends JoglModelScene 
{
	private HashMap<JSOSubNet, JSOPSENEdgeVisualizationLayer> evis = new HashMap<JSOSubNet, JSOPSENEdgeVisualizationLayer>();
	private HashMap<JSOSubNet, JSOPSENTransitionVisualizationLayer> tvis = new HashMap<JSOSubNet, JSOPSENTransitionVisualizationLayer>();
	private JSOPSENTrajectoryVisualizationLayer tjvis = null;
	private JSOReplaya replaya = null;
	
	public JoglPSENVisualizationScene()
	{
		
	}
	
	public JSOPSENEdgeVisualizationLayer getEdgeVisualizer(JSOSubNet sn)
	{
		if (evis.containsKey(sn))
		{
			return evis.get(sn);
		}
		return null;
	}

	public JSOPSENTransitionVisualizationLayer getTransitionVisualizer(JSOSubNet sn)
	{
		if (tvis.containsKey(sn))
		{
			return tvis.get(sn);
		}
		return null;
	}
	
	public JSOReplaya getReplaya()
	{
		return replaya;
	}
	
	@Override
	public Vector<JoglAABB> getBoundingBoxes()
	{
		//modify camera depth range based on scene
		Vector<JoglAABB> lst = new Vector<JoglAABB>();
		//lst.add(pnet.getAABB());
		
		JSOPetriNet pnet = findSceneObject(JSOPetriNet.class);
		//for (JSOSubNet snet : scene.findSceneObjects(JSOSubNet.class))
		for (JSOSubNet snet : pnet.getAllSubNets())
		{
			if (snet.getIsVisible())
			{
				lst.add(snet.getAABB());
				
				//use AABB for pedestals as well, to avoid oversized pedals from being clipped by the near-view-plane
				if (snet.ped != null)
				{
					lst.add(snet.ped.getAABB());
				}
			}
		}
		
		//cam.calcDepthRange(pnet.getAABB());
		
		for (JSOPSENEdgeVisualizationLayer edgeVis : findSceneObjects(JSOPSENEdgeVisualizationLayer.class))
		{
			JSOSubNet snet = edgeVis.getSubNet();
			if (snet.getIsVisible() && edgeVis.getElementCount() > 0)
			{
				JoglVectord3 offset = snet.relativeToAbsolute(JoglVectord3.ZERO);
				lst.add(edgeVis.getAABB().Translate(offset));		
			}
		}
		
		JSOPSENTrajectoryVisualizationLayer jptv = findSceneObject(JSOPSENTrajectoryVisualizationLayer.class);
		if (jptv != null)
		{
			JoglAABB jptvAABB = jptv.getAABB();
			if (jptvAABB != null)
			{
				lst.add(jptvAABB);
			}
		}
		
		lst.add(pnet.getAABB());
		
		return lst;
	}
	
	@Override
	public void addSceneObject(JoglSceneObject jso)
	{
		if (jso instanceof JSOPSENEdgeVisualizationLayer)
		{
			JSOPSENEdgeVisualizationLayer evisx  = (JSOPSENEdgeVisualizationLayer)jso;
			evis.put(evisx.getSubNet(), evisx);
			filtered.add(jso);
		}
		else if (jso instanceof JSOPSENTransitionVisualizationLayer)
		{
			JSOPSENTransitionVisualizationLayer tvisx  = (JSOPSENTransitionVisualizationLayer)jso;
			tvis.put(tvisx.getSubNet(), tvisx);
			filtered.add(jso);
		}
		else if (jso instanceof JSOPSENTrajectoryVisualizationLayer)
		{
			tjvis  = (JSOPSENTrajectoryVisualizationLayer)jso;
			filtered.add(jso);
		}
		else if (jso instanceof JSOReplaya)
		{
			replaya = (JSOReplaya)jso;
			filtered.add(jso);
		}
		super.addSceneObject(jso);
	}
	
	@Override
	public void Draw(GraphicsPipeline pipe)
	{
		pipe.cam.DrawCamera(pipe);
		
		DrawModel(pipe);

		JSOPSENTrajectoryVisualizationLayer jptv = this.findSceneObject(JSOPSENTrajectoryVisualizationLayer.class);
		if (jptv != null)
		{
			jptv.DrawObject(pipe);
		}
		
		//draw VR controllers
//		for (JoglSceneObject jso : objs)
//		{
//			if (jso instanceof JSOVRController)
//			{
//				JSOVRController vrCube = (JSOVRController)jso;
//				vrCube.DrawObject(pipe);
//			}
//		}

		for (int i = 0; i < objs.size(); i++)
		{
			JoglSceneObject jso = objs.get(i);
			if (!filtered.contains(jso))
			{
				jso.DrawObject(pipe);
			}
		}
		
		if (bDrawGUI)
		{
			DrawGui(pipe);
		}
	}

	//this code is not actually being called anymore and is subject to deprecation
	@Override
	protected void DrawSubnet(GraphicsPipeline pipe, JSOSubNet snet) 
	{
		JSOSubNetStackRenderer snr = new JSOSubNetStackRenderer(snet);
		snr.DrawSubnet(pipe, this);
		
//		JoglCamera c = pipe.cam;
//		
//		snet.ped.UpdateGeometry(pipe);
//		
//		int netLevel = snet.getLevel();
//		JSOPSENTransitionVisualizationLayer tvisx = tvis.get(snet);
//		JSOPSENEdgeVisualizationLayer evisx = evis.get(snet);
//		boolean bDrawGrid = false;
//		
//		if (netLevel == 0)
//		{
//			bDrawGrid = true;
//		}
//		
//		//double layerHeight = snet.getSubnetYPos();
//
//		pipe.transform.Push();
//		JoglMatrixd4x4 transform = snet.getTransform();
//		pipe.transform.MultMatrix(transform);
//		
//		snet.DrawModelMask(pipe);
//		evisx.DeterminVisibleElements(this);
//		
//		//if cam above grid
//		boolean bCamAboveLayer = c.GetPosition().getY() >= snet.getSubnetYPos();
//		if (bCamAboveLayer)
//		{
//			if (evisx.HasVisibleElements())
//			{
//				evisx.DrawBottom(pipe, true);
//			}
//			if (bDrawGrid)
//			{
//				grid.DrawGridWithParams(pipe, false, true, true);
//			}
//			if (netLevel > 0)
//			{
//				//draw only to colorbuffer
//				snet.ped.DrawColor(pipe);
//			}
//			snet.DrawObject(pipe);
//
//			//draw only to depthbuffer
//			if (netLevel > 0)
//			{
//				snet.ped.DrawDepth(pipe);
//			}
//			
//			tvisx.DrawObject(pipe);
//			if (evisx.HasVisibleElements())
//			{
//				evisx.DrawTop(pipe, false);
//			}
//		}
//		else
//		{
//			if (evisx.HasVisibleElements())
//			{
//				evisx.DrawTop(pipe, true);
//			}
//			tvisx.DrawObject(pipe);
//			snet.DrawObject(pipe);
//			if (bDrawGrid)
//			{
//				grid.DrawGridWithParams(pipe, false, true, true);
//			}
//			if (evisx.HasVisibleElements())
//			{
//				evisx.DrawBottom(pipe, false);
//			}
//			if (netLevel > 0)
//			{
//				snet.ped.DrawObject(pipe);
//			}
//		}
//		
//		pipe.transform.Pop();
	}
	
	@Override
	public void Dispose(GraphicsPipeline pipe)
	{
		//PersistSave();
		
		super.Dispose(pipe);
		
		for (JSOPSENEdgeVisualizationLayer edgeVis : evis.values())
		{
			edgeVis.Dispose();
		}
		
		evis.clear();
		tvis.clear();
		
		if (tjvis != null)
		{
			tjvis.Dispose(pipe);
		}
	}
}
