package org.processmining.processprofiler3d.compviz.utils;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglFrameBufferObject;
import org.processmining.visualisation3d.gldatastructures.JoglMultiSampleFrameBufferObject;
import org.processmining.visualisation3d.gldatastructures.JoglTextureBufferObject;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLensPerspective;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTargetFramebuffer;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.utils.IScreenshotCapture;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.GLException;
import com.jogamp.opengl.util.texture.Texture;
import com.jogamp.opengl.util.texture.TextureData;
import com.jogamp.opengl.util.texture.TextureIO;

public class JoglCubeMapCapture implements IScreenshotCapture
{
//	private static final int[] targets = { GL.GL_TEXTURE_CUBE_MAP_POSITIVE_X,
//          GL.GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
//          GL.GL_TEXTURE_CUBE_MAP_POSITIVE_Y,
//          GL.GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,
//          GL.GL_TEXTURE_CUBE_MAP_POSITIVE_Z,
//          GL.GL_TEXTURE_CUBE_MAP_NEGATIVE_Z };
	
	private static final int[] targets = { 
		GL.GL_TEXTURE_CUBE_MAP_POSITIVE_X,
        GL.GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
        GL.GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,
        GL.GL_TEXTURE_CUBE_MAP_POSITIVE_Y,
        GL.GL_TEXTURE_CUBE_MAP_POSITIVE_Z,
        GL.GL_TEXTURE_CUBE_MAP_NEGATIVE_Z};
	
	private int cubew = 4096;
	private int cubeh = 4096;
	private int samples = 1;
	
	private Vector<TextureData> mydata = new Vector<TextureData>();
	private String basePath = "C:/Lab/Export/cubemap_";
	
	public JoglCubeMapCapture(String path)
	{
		basePath = path;
	}
	
	public JoglCubeMapCapture(String path, int tw, int th)
	{
		basePath = path;
		
		cubew = tw;
		cubeh = th;
	}
	
	public JoglCubeMapCapture(String path, int tw, int th, int s)
	{
		basePath = path;
		
		cubew = tw;
		cubeh = th;
		samples = s;
	}
	
	public void GrabScreenshot(GraphicsPipeline pipe) 
	{
		JoglCubemapToEpirectConverter jctec = new JoglCubemapToEpirectConverter(4096, 2048, 16);
		Init(pipe);
		
		Date startTime = new Date();
		
		renderCubeMap(pipe);	
		jctec.Init(pipe); //screws up if called earlier, investigate!!!
		jctec.renderToEpirect(pipe, loadToCubemap(pipe));
		
		Date endTime = new Date();
		TimeSpan ts = new TimeSpan(startTime, endTime);	
		
		System.out.println("Cubemap generation (" + ts.ToString(TimeUnit.MILLISECONDS) + "ms).");

		writeToFile(basePath + "[token].png", "[token]");
		jctec.writeToFileEpirect(basePath + "epi.png");
		
		Dispose(pipe);
		jctec.Dispose(pipe);
	}
	
	private void Init(GraphicsPipeline pipe)
	{
		if (samples > 1)
		{
			myfboms = new JoglMultiSampleFrameBufferObject(pipe, cubew, cubeh, samples);
			myfboms.Bind(pipe);
			myfboms.AttachDepthBuffer(pipe, myfboms.CreateRboAttachment(pipe, GL.GL_DEPTH_COMPONENT32));
			myfboms.AttachColorBuffer0(pipe, myfboms.CreateRboAttachment(pipe, GL.GL_RGBA8));

			if (!myfboms.isFramebufferComplete())
			{
				System.out.println("Framebuffer failed to initialize.");
			}
			
			myfboms.Unbind(pipe);
			
			myfbo = new JoglFrameBufferObject(pipe, cubew, cubeh);
			myfbo.Bind(pipe);
			myfbo.AttachColorBuffer0(pipe, myfbo.CreateTboAttachment(pipe, GL.GL_RGB, 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE));

			if (!myfbo.isFramebufferComplete())
			{
				System.out.println("Framebuffer failed to initialize.");
			}
			
			myfbo.Unbind(pipe);
		}
		else
		{
			myfbo = new JoglFrameBufferObject(pipe, cubew, cubeh);
			myfbo.Bind(pipe);
			myfbo.AttachDepthBuffer(pipe, myfbo.CreateRboAttachment(pipe, GL.GL_DEPTH_COMPONENT32));
			myfbo.AttachColorBuffer0(pipe, myfbo.CreateTboAttachment(pipe, GL.GL_RGB, 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE));

			if (!myfbo.isFramebufferComplete())
			{
				System.out.println("Framebuffer failed to initialize.");
			}
			
			myfbo.Unbind(pipe);
		}
	}
	
	private void renderCubeMap(GraphicsPipeline pipe) 
	{
		if (samples > 1)
		{
			renderCubeMapHelperMS(pipe);
		}
		else
		{
			renderCubeMapHelper(pipe);
		}
	}
	
	private JoglFrameBufferObject myfbo;
	private JoglMultiSampleFrameBufferObject myfboms;
	
	private void renderCubeMapHelperMS(GraphicsPipeline pipe) 
	{
//		myfboms = new JoglMultiSampleFrameBufferObject(gl, cubew, cubeh, samples);
//		myfboms.Bind(gl);
//		myfboms.AttachDepthBuffer(gl, myfboms.CreateRboAttachment(gl, GL.GL_DEPTH_COMPONENT32));
//		myfboms.AttachColorBuffer0(gl, myfboms.CreateRboAttachment(gl, GL.GL_RGBA8));
//
//		if (!myfboms.isFramebufferComplete())
//		{
//			System.out.println("Framebuffer failed to initialize.");
//		}
//		
//		myfboms.Unbind(gl);
//		
//		myfbo = new JoglFrameBufferObject(gl, cubew, cubeh);
//		myfbo.Bind(gl);
//		myfbo.AttachColorBuffer0(gl, myfbo.CreateTboAttachment(gl, GL.GL_RGB, 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE));
//		JoglTextureBufferObject tbo = (JoglTextureBufferObject)myfbo.getColorBuffer0();
//
//		if (!myfbo.isFramebufferComplete())
//		{
//			System.out.println("Framebuffer failed to initialize.");
//		}
//		
//		myfbo.Unbind(gl);

		JoglTextureBufferObject tbo = (JoglTextureBufferObject)myfbo.getColorBuffer0();
		JoglRenderTargetFramebuffer jrtb = new JoglRenderTargetFramebuffer(myfboms);
		jrtb.setBackgroundColor(pipe.scene.getBackgroundColor());
		
		Vector<JoglCamera> cams = setupCameras(pipe.cam);

		for (JoglCamera c : cams)
		{
			JoglVectord2 r1 = c.calcDepthRange(pipe.scene.getBoundingBoxes());
			c.setCameraLens(new JoglCameraLensPerspective(90.0, jrtb.getAspect(), r1.getX(), r1.getY()));
		}
		
		mydata.clear();
		
		pipe.scene.setDrawGui(false);
		jrtb.Bind(pipe);
		jrtb.FitViewport(pipe);
		for (int i = 0; i < 6; i++)
		{
			jrtb.Clear(pipe);
	
			GraphicsPipeline newPipe = new GraphicsPipeline(pipe);
			newPipe.cam = cams.get(i);
			newPipe.target = jrtb;
			
			pipe.scene.Draw(newPipe);
			
			//copy from ms framebuffer!
			myfboms.BlitToFramebuffer(pipe, myfbo);
			
			try 
			{
				TextureData td = tbo.getTexture().getTextureData(pipe);
				mydata.add(td);
			} 
			catch (GLException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 
			catch (IOException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			myfboms.Bind(pipe, GL2.GL_DRAW_FRAMEBUFFER);
		}
		jrtb.Unbind(pipe);
		pipe.scene.setDrawGui(true);
		
		//System.out.println("Cubemap saved: ");
		
		//myfboms.Dispose(gl);
		//myfbo.Dispose(gl);
	}
	
	private void renderCubeMapHelper(GraphicsPipeline pipe) 
	{
//		myfbo = new JoglFrameBufferObject(gl, cubew, cubeh);
//		myfbo.Bind(gl);
//		myfbo.AttachDepthBuffer(gl, myfbo.CreateRboAttachment(gl, GL.GL_DEPTH_COMPONENT32));
//		myfbo.AttachColorBuffer0(gl, myfbo.CreateTboAttachment(gl, GL.GL_RGB, 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE));
//		JoglTextureBufferObject tbo = (JoglTextureBufferObject)myfbo.getColorBuffer0();
//
//		if (!myfbo.isFramebufferComplete())
//		{
//			System.out.println("Framebuffer failed to initialize.");
//		}
//		
//		myfbo.Unbind(gl);

		JoglTextureBufferObject tbo = (JoglTextureBufferObject)myfbo.getColorBuffer0();
		JoglRenderTargetFramebuffer jrtb = new JoglRenderTargetFramebuffer(myfbo);
		jrtb.setBackgroundColor(pipe.scene.getBackgroundColor());
		
		Vector<JoglCamera> cams = setupCameras(pipe.cam);

		for (JoglCamera c : cams)
		{
			c.calcDepthRange(pipe.scene.getBoundingBoxes());
		}
		
		mydata.clear();
		
		pipe.scene.setDrawGui(false);
		jrtb.Bind(pipe);
		jrtb.FitViewport(pipe);
		for (int i = 0; i < 6; i++)
		{
			jrtb.Clear(pipe);
			
			GraphicsPipeline newPipe = new GraphicsPipeline(pipe);
			newPipe.cam = cams.get(i);
			newPipe.target = jrtb;
			
			pipe.scene.Draw(newPipe);
			
			try 
			{
				TextureData td = tbo.getTexture().getTextureData(pipe);
				mydata.add(td);
			} 
			catch (GLException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 
			catch (IOException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		jrtb.Unbind(pipe);
		pipe.scene.setDrawGui(true);
		
		//System.out.println("Cubemap saved: ");
		
		//myfbo.Dispose(gl);
	}

	public Vector<JoglCamera> setupCameras(JoglCamera cam) 
	{
		Vector<JoglCamera> cams = new Vector<JoglCamera>();
		cams.add(new JoglCamera());
		cams.add(new JoglCamera());
		cams.add(new JoglCamera());
		cams.add(new JoglCamera());
		cams.add(new JoglCamera());
		cams.add(new JoglCamera());
		
		JoglVectord3 camPos = cam.GetPosition();
		for (JoglCamera c : cams)
		{
			c.SetPosition(camPos);
			//c.setFieldOfView(90.0f);
		}
		
		cams.get(0).SetLookAt(JoglVectord3.Add(camPos, new JoglVectord3(1,0,0)), new JoglVectord3(0,1,0));
		cams.get(1).SetLookAt(JoglVectord3.Add(camPos, new JoglVectord3(-1,0,0)), new JoglVectord3(0,1,0));
		cams.get(2).SetLookAt(JoglVectord3.Add(camPos, new JoglVectord3(0,1,0)), new JoglVectord3(0,0,-1));
		cams.get(3).SetLookAt(JoglVectord3.Add(camPos, new JoglVectord3(0,-1,0)), new JoglVectord3(0,0,-1));
		cams.get(4).SetLookAt(JoglVectord3.Add(camPos, new JoglVectord3(0,0,-1)), new JoglVectord3(0,1,0));
		cams.get(5).SetLookAt(JoglVectord3.Add(camPos, new JoglVectord3(0,0,1)), new JoglVectord3(0,1,0));
		return cams;
	}

	public Texture loadToCubemap(GraphicsPipeline pipe) throws GLException 
	{
		Texture cubemap = TextureIO.newTexture(GL.GL_TEXTURE_CUBE_MAP);
		
		for (int i = 0; i < 6; i++) 
		{
			TextureData data = mydata.get(i);
			cubemap.updateImage(pipe.gl, data, targets[i]);
		}
		
		return cubemap;
	}
	
	public void writeToFile(String fileName, String token)
	{
		Vector<String> nameLst = new Vector<String>();
		nameLst.add("posx");
		nameLst.add("negx");
		nameLst.add("posy");
		nameLst.add("negy");
		nameLst.add("posz");
		nameLst.add("negz");
		
		for (int i = 0; i < 6; i++)
		{
			String finalFileName = fileName.replace(token, nameLst.get(i));
			try 
			{
				TextureData td = mydata.get(i);
				TextureIO.write(td, new File(finalFileName));
			} 
			catch (GLException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 
			catch (IOException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		System.out.println("Cubemap saved: ");	
	}
	
	public void Dispose(GraphicsPipeline pipe)
	{
		if (myfboms != null)
		{
			myfboms.Dispose(pipe);
		}
		if (myfbo != null)
		{
			myfbo.Dispose(pipe);
		}
	}
}
