package org.processmining.processprofiler3d.compviz.utils;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglMesh;
import org.processmining.processprofiler3d.compviz.jogl.drawing.IMeshDrawer;
import org.processmining.processprofiler3d.compviz.jogl.drawing.VAODrawer;
import org.processmining.processprofiler3d.compviz.jogl.materials.MaterialEpirectConversion;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglFrameBufferObject;
import org.processmining.visualisation3d.gldatastructures.JoglMultiSampleFrameBufferObject;
import org.processmining.visualisation3d.gldatastructures.JoglTextureBufferObject;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLensPerspective;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTargetFramebuffer;
import org.processmining.visualisation3d.maths.JoglVectord3;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.GLException;
import com.jogamp.opengl.util.texture.Texture;
import com.jogamp.opengl.util.texture.TextureIO;

public class JoglCubemapToEpirectConverter 
{
	private static MaterialEpirectConversion matEC = new MaterialEpirectConversion();
	
	private int epiw = 4096;
	private int epih = 2048;
	private int samples = 1;
	
	private Texture epi = null;
	
	private JoglMultiSampleFrameBufferObject myfboms;
	private JoglFrameBufferObject myfbo;
	
	public JoglCubemapToEpirectConverter()
	{
		
	}
	
	public JoglCubemapToEpirectConverter(int w, int h)
	{
		epiw = w;
		epih = h;
	}
	
	public JoglCubemapToEpirectConverter(int w, int h, int s)
	{
		epiw = w;
		epih = h;
		samples = s;
	}

	public void Init(GraphicsPipeline pipe)
	{
		if (samples > 1)
		{
			myfboms = new JoglMultiSampleFrameBufferObject(pipe, epiw, epih, samples);
			myfboms.Bind(pipe);
			myfboms.AttachColorBuffer0(pipe, myfboms.CreateRboAttachment(pipe, GL.GL_RGBA8));

			if (!myfboms.isFramebufferComplete())
			{
				System.out.println("Framebuffer failed to initialize.");
				samples = 1;
			}
			
			myfboms.Unbind(pipe);
			
			myfbo = new JoglFrameBufferObject(pipe, epiw, epih);
			myfbo.Bind(pipe);
			myfbo.AttachColorBuffer0(pipe, myfbo.CreateTboAttachment(pipe, GL.GL_RGB, 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE));

			if (!myfbo.isFramebufferComplete())
			{
				System.out.println("Framebuffer failed to initialize.");
			}
			
			myfbo.Unbind(pipe);
		}
		else
		{
			myfbo = new JoglFrameBufferObject(pipe, epiw, epih);
			myfbo.Bind(pipe);
			myfbo.AttachColorBuffer0(pipe, myfbo.CreateTboAttachment(pipe, GL.GL_RGB, 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE));

			if (!myfbo.isFramebufferComplete())
			{
				System.out.println("Framebuffer failed to initialize.");
			}
			
			myfbo.Unbind(pipe);	
		}

//		shader = new JoglShaderGenerateEpirect();
//		shader.Create(pipe);
	}
	
	public Texture renderToEpirect(GraphicsPipeline pipe, Texture t) 
	{
		if (samples > 1)
		{
			return loadToEpirectHelperMS(pipe, t);
		}
		else
		{
			return loadToEpirectHelper(pipe, t);
		}
	}
	
	private Vector<JoglVectord3> GenerateGeometry()
	{
		Vector<JoglVectord3> lst = new Vector<JoglVectord3>();

		lst.add(new JoglVectord3(0,  2, 1));
		lst.add(new JoglVectord3(3, -1, 1));
		lst.add(new JoglVectord3(-3, -1, 1));

		return lst;
	}
	
	public JoglMesh CreateMesh()
	{
		JoglMesh jcm = new JoglMesh(GL2.GL_TRIANGLES);
		jcm.setVerts(GenerateGeometry());
		jcm.setIndexBuffer(JoglMeshIndexBuffer.GenerateIndicesTri(jcm.getNumVerts()));
		
		return jcm;
	}
	
	private IMeshDrawer vao = null;
	
	private Texture loadToEpirectHelper(GraphicsPipeline pipe, Texture t) 
	{
		JoglTextureBufferObject tbo = (JoglTextureBufferObject)myfbo.getColorBuffer0();
		JoglRenderTargetFramebuffer jrtb = new JoglRenderTargetFramebuffer(myfbo);
		
		jrtb.Bind(pipe);
		jrtb.FitViewport(pipe);
		
		matEC.SetTexture(t);
		pipe.PushPipelineState(matEC);
		
		JoglCamera c = new JoglCamera();
		c.setCameraLens(new JoglCameraLensPerspective(45.0, jrtb.getAspect()));
		//c.DrawCamera(gl, jrtb.getSize());
		c.DrawCamera(pipe);
		
		//draw triangle over screen
//	    gl.glBegin(GL.GL_TRIANGLES);
//	    gl.glVertex3f(0,  2, 1);
//	    gl.glVertex3f(3, -1, 1);
//	    gl.glVertex3f(-3, -1, 1);
//	    gl.glEnd();
		if (vao == null)
		{
			vao = new VAODrawer();
			vao.LoadMesh(pipe, CreateMesh());
		}	
		vao.Draw(pipe);

		jrtb.Unbind(pipe);

		pipe.PopPipelineState(matEC);
		
		epi = tbo.getTexture().getTex();
		
		return epi;
	}
	
	private Texture loadToEpirectHelperMS(GraphicsPipeline pipe, Texture t) 
	{
		JoglTextureBufferObject tbo = (JoglTextureBufferObject)myfbo.getColorBuffer0();
		
		JoglRenderTargetFramebuffer jrtb = new JoglRenderTargetFramebuffer(myfboms);
		
		jrtb.Bind(pipe);
		jrtb.FitViewport(pipe);

		matEC.SetTexture(t);
		pipe.PushPipelineState(matEC);
		
		JoglCamera c = new JoglCamera();
		c.setCameraLens(new JoglCameraLensPerspective(45.0, jrtb.getAspect()));
		//c.DrawCamera(gl, jrtb.getSize());
		c.DrawCamera(pipe);

		//draw triangle over screen
//	    gl.glBegin(GL.GL_TRIANGLES);
//	    gl.glVertex3f(0,  2, 1);
//	    gl.glVertex3f(3, -1, 1);
//	    gl.glVertex3f(-3, -1, 1);
//	    gl.glEnd();
		if (vao == null)
		{
			vao = new VAODrawer();
			vao.LoadMesh(pipe, CreateMesh());
		}	
		vao.Draw(pipe);

		//copy from ms framebuffer!
		myfboms.BlitToFramebuffer(pipe, myfbo);
	    
		jrtb.Unbind(pipe);

		pipe.PopPipelineState(matEC);

		epi = tbo.getTexture().getTex();
		return epi;
	}

	public void writeToFileEpirect(String fileName)
	{
		try
		{
			TextureIO.write(epi, new File(fileName));
		} 
		catch (GLException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		System.out.println("Epirectangular panorama saved: " + fileName);
	}

	public void Dispose(GraphicsPipeline pipe)
	{
		if (myfboms != null)
		{
			myfboms.Dispose(pipe);
		}
		if (myfbo != null)
		{
			myfbo.Dispose(pipe);
		}
//		if (shader != null)
//		{
//			shader.Dispose(pipe);
//		}
		
	}
	
}
