package org.processmining.processprofiler3d.compviz.vr;

import java.io.File;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.vr.jna.OpenVRInterop;
import org.processmining.processprofiler3d.compviz.vr.jna.OpenVRInterop.EVRInitError;
import org.processmining.processprofiler3d.compviz.vr.jna.OpenVRInteropImpl;
import org.processmining.processprofiler3d.compviz.vr.jna.VRRenderModel;
import org.processmining.processprofiler3d.compviz.vr.jna.VRRenderModelTexture;
import org.processmining.processprofiler3d.compviz.vr.jna.VRTrackedDevicePose;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.gldatastructures.JoglFrameBufferObject;
import org.processmining.visualisation3d.gldatastructures.JoglMultiSampleFrameBufferObject;
import org.processmining.visualisation3d.gldatastructures.JoglTextureBufferObject;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTarget;
import org.processmining.visualisation3d.graphicsdatastructures.JoglRenderTargetFramebuffer;
import org.processmining.visualisation3d.interactions.JICameraBase;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;
import org.processmining.visualisation3d.utils.IVRAdapter;

import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;
import com.sun.jna.Pointer;
import com.sun.jna.ptr.IntByReference;

public class HzdOpenVR implements IVRAdapter 
{	
	private static OpenVRInterop vrio;
	
	private JoglInteractionManager iman;
	private HzdVRSystem myVrSys;
	private HzdVRCompositor myVrComp;
	private HzdVRRenderModels myVrModels;
	private VRRenderInfo myRInfo;
	private boolean bInitialized = false;
	private boolean bDrawGUI = true;
	
	private String libPath = "";
	
	public HzdOpenVR(JoglInteractionManager iman, String libPath)
	{
		this.iman = iman;
		this.libPath = libPath;
	}
	
	private static boolean TryLoadDLL(String libPath)
	{
		File f = new File(libPath);
		if(f.exists() && !f.isDirectory()) 
		{ 
			//vrio = OpenVRInterop.INSTANCE;
			vrio = OpenVRInteropImpl.getOpenVRInteropInstance(libPath);
		}
		else
		{
			System.out.println("Could not load OpenVR DLL!");
			return false;	
		}
		
		return true;
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.vr.IVRAdapter#Init()
	 */
	@Override
	public boolean Init()
	{
		if (!TryLoadDLL(libPath))
		{
			return false;
		}
		
		System.out.println("IsHmdPresent: " + vrio.VR_IsHmdPresent());
		System.out.println("IsRuntimeInstalled: " + vrio.VR_IsRuntimeInstalled());
		System.out.println("RuntimePath: " + vrio.VR_RuntimePath());
		
		int peError = EVRInitError.None;
		IntByReference ibr = new IntByReference();
		ibr.setValue(peError);
		Pointer result = vrio.VR_InitInternal(ibr, OpenVRInterop.EVRApplicationType.VRApplication_Scene);
		peError = ibr.getValue();
		
		System.out.println("InitError: " + peError);
		String strError = vrio.VR_GetStringForHmdError(peError);
		System.out.println("InitErrorStr: " + strError);
		
		if (peError == EVRInitError.None)
		{		
			System.out.println("IsInterfaceValid (IVRSystem): " + vrio.VR_IsInterfaceVersionValid(HzdVRSystem.ivrSystemStr));
			System.out.println("IsInterfaceValid (IVRCompositor): " + vrio.VR_IsInterfaceVersionValid(HzdVRCompositor.ivrCompositorStr));
			System.out.println("IsInterfaceValid (IVRRenderModels): " + vrio.VR_IsInterfaceVersionValid(HzdVRRenderModels.ivrRenderModelsStr));
	
			myVrSys = HzdVRSystem.getInstance(vrio);
			if (myVrSys != null)
			{
				HzdVRSystem.VR_RenderTargetSize targetSize = myVrSys.GetRecommendedRenderTargetSize();
				System.out.println("Recommended rendertarget size: " + targetSize.getWidth() + "x" + targetSize.getHeight());
				
				myRInfo = new VRRenderInfo(myVrSys);
			}
	
			myVrComp = HzdVRCompositor.getInstance(vrio);
			
			myVrModels = HzdVRRenderModels.getInstance(vrio);
			
			bInitialized = true;
			
			return true;
		}
		
		return false;
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.vr.IVRAdapter#Shutdown()
	 */
	@Override
	public void Shutdown()
	{
		if (bInitialized)
		{
			vrio.VR_ShutdownInternal();
		}
	}
	
	public VRRenderInfo getRenderInfo()
	{
		return myRInfo;
	}

//	private JoglCamera[] setupCameras(JoglScene scene, VRTrackedDevicePose hmdPose)
//	{
//		JICameraBase jcb = iman.findInteraction(JICameraBase.class);
//		JoglCamera defaultCam = jcb.getCamera();
//
//		//transform camera space
//		VRTransform hmdT = new VRTransform(defaultCam);
//		hmdT.transform(hmdPose);
//		
//		//set up cameras for left and right eye
//		JoglCamera camLeft = new JoglCamera();
//		JoglCamera camRight = new JoglCamera();
//		
//		camLeft.SetPosition(hmdT.hmdPosWS);
//		camRight.SetPosition(hmdT.hmdPosWS);
//		
//		camLeft.SetLookDir(hmdT.hmdLookDirWS, hmdT.hmdUpDirWS);
//		camRight.SetLookDir(hmdT.hmdLookDirWS, hmdT.hmdUpDirWS);
//		
//		//update interaction
//		if (jcb instanceof JICameraVR)
//		{
//			JICameraVR vrCamInt = (JICameraVR)jcb;
//			if (vrCamInt != null)
//			{
//				vrCamInt.setHmdDir(hmdT.hmdLookDirWS, hmdT.hmdUpDirWS, hmdT.hmdLeftDirWS);
//				vrCamInt.hmdPosAbsolute = hmdT.hmdPosWS;
//				vrCamInt.setHMDCamera(camRight);
//			}
//		}
//		
//		//add eye to head offset to each camera
//		//need inverse of matrix, so use wrong eye. This might cause issues if transform contains more than just IPD!
//		addEyeOffset(camLeft, myVrSys.getEyeToHeadTransformRight());
//		addEyeOffset(camRight, myVrSys.getEyeToHeadTransformLeft());
//		
//		//determine scene depth range for each camera
//		Vector<JoglAABB> boundingBoxes = scene.getBoundingBoxes();
//		if (!boundingBoxes.isEmpty())
//		{
//			JoglVectord2 depthRangeLeft = camLeft.calcDepthRange(boundingBoxes);
//			JoglVectord2 depthRangeRight = camRight.calcDepthRange(boundingBoxes);
//	
//			JoglCameraLensPerspective lensL = new JoglCameraLensPerspective(myRInfo.fieldOfView, myRInfo.aspect, depthRangeLeft.getX(), depthRangeLeft.getY());
//			JoglCameraLensPerspective lensR = new JoglCameraLensPerspective(myRInfo.fieldOfView, myRInfo.aspect, depthRangeRight.getX(), depthRangeRight.getY());
//			
//			camLeft.setCameraLens(lensL);
//			camRight.setCameraLens(lensR);
//		}
//		else
//		{
//			JoglVectord2 depthRangeLeft = new JoglVectord2(0.1, 2000.0);
//			JoglVectord2 depthRangeRight = new JoglVectord2(0.1, 2000.0);;
//	
//			JoglCameraLensPerspective lensL = new JoglCameraLensPerspective(myRInfo.fieldOfView, myRInfo.aspect, depthRangeLeft.getX(), depthRangeLeft.getY());
//			JoglCameraLensPerspective lensR = new JoglCameraLensPerspective(myRInfo.fieldOfView, myRInfo.aspect, depthRangeRight.getX(), depthRangeRight.getY());
//			
//			camLeft.setCameraLens(lensL);
//			camRight.setCameraLens(lensR);
//		}
//		
//		JoglCamera[] cams = new JoglCamera[2];
//		cams[0] = camLeft;
//		cams[1] = camRight;
//		
//		return cams;
//	}
//
//	private void addEyeOffset(JoglCamera cam, JoglMatrixd4x4 offset)
//	{
//		double[] vrDataL = offset.GetData();
//		JoglVectord3 offsL = new JoglVectord3(vrDataL[12], vrDataL[13], vrDataL[14]);
//		JoglVectord3 camPosrelL = JoglVectord3.Add(JoglVectord3.Add(JoglVectord3.Multiply(cam.GetLeftDir(), offsL.getX()), JoglVectord3.Multiply(cam.GetUpDir(), offsL.getY())), JoglVectord3.Multiply(cam.GetLookDir(), offsL.getZ()));
//		cam.SetPosition(JoglVectord3.Add(camPosrelL, cam.GetPosition()));
//	}
	
	public VRController controller1 = null;
	public VRController controller2 = null;
	
	private JoglMultiSampleFrameBufferObject myfboms;
	private JoglFrameBufferObject myfbo;
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.compviz.vr.IVRAdapter#Render(com.jogamp.opengl.GL2, org.processmining.processprofiler3d.compviz.scenemanagement.JoglScene, org.processmining.processprofiler3d.compviz.jogl.drawing.JoglRenderTarget)
	 */
	@Override
	public void Render(GraphicsPipeline pipe)
	{
		JoglScene scene = pipe.scene;
		JoglRenderTarget target = pipe.target;
		
		int numSamples = pipe.getGraphicsSettings().GetAntiAliasingSamples();
		
		//sync rendering with display & tracking info and get newest tracking info
		VRTrackedDevicePose[] poses = myVrComp.WaitGetPoses();
		
		//setup cameras for left and right eye
//		JoglCamera[] cams = setupCameras(scene, poses[VR_TrackedDevice.TrackedDeviceIndex_Hmd]);
//		JoglCamera camLeft = cams[0];
//		JoglCamera camRight = cams[1];
		
		VRHmdUtil hmd = new VRHmdUtil(iman, myVrSys, myRInfo);
		hmd.setupCameras(scene, poses[VR_TrackedDevice.TrackedDeviceIndex_Hmd]);
		JoglCamera camLeft = hmd.camLeft;
		JoglCamera camRight = hmd.camRight;
		
		UpdateControllers(poses);
		
		scene.setDrawGui(false);
	
//		camLeft.logCameraMatrix();
//		camLeft.logProjectionMatrix();
		
		//Render scene for left eye
		//RenderHelper(pipe, camLeft, (int)myRInfo.sceneWidth, (int)myRInfo.sceneHeight);
		RenderHelperMS(pipe, camLeft, (int)myRInfo.sceneWidth, (int)myRInfo.sceneHeight, numSamples);

//		if (myfbo != null)
//		{
//			myVrComp.SubmitFrameLeft(pipe, (JoglRenderBufferObject)myfbo.getColorBuffer0(), myRInfo);
//		}
		if (myfbo != null)
		{
			JoglTextureBufferObject tbo = (JoglTextureBufferObject)myfbo.getColorBuffer0();
			myVrComp.SubmitFrameLeft(pipe, tbo.getTexture(), myRInfo);
			
			//myfbo.BlitToBackbuffer(gl, target.getWidth(), target.getHeight());
		}
		
		//Render scene for right eye
		//RenderHelper(pipe, camRight, (int)myRInfo.sceneWidth, (int)myRInfo.sceneHeight);
		RenderHelperMS(pipe, camRight, (int)myRInfo.sceneWidth, (int)myRInfo.sceneHeight, numSamples);

//		if (myfbo != null)
//		{
//			myVrComp.SubmitFrameRight(pipe, (JoglRenderBufferObject)myfbo.getColorBuffer0(), myRInfo);
//		}
		if (myfbo != null)
		{
			JoglTextureBufferObject tbo = (JoglTextureBufferObject)myfbo.getColorBuffer0();
			myVrComp.SubmitFrameRight(pipe, tbo.getTexture(), myRInfo);
			
			myfbo.BlitToBackbuffer(pipe, target.getWidth(), target.getHeight());
		}
		
//		if (myfbo != null)
//		{
//			JoglTextureBufferObject tbo = (JoglTextureBufferObject)myfbo.getColorBuffer0();
//			SubmitFrameDual(gl, tbo.getTextureId());
//			
//			myfbo.BlitToBackbuffer(gl, target.getWidth(), target.getHeight());
//		}
		scene.setDrawGui(true);

		if (bDrawGUI)
		{
			pipe.target.Bind(pipe);
			pipe.target.FitViewport(pipe);
			
			//scene.getGUI().DrawObject(pipe);
			scene.DrawGui(pipe);
			
			pipe.target.Unbind(pipe);
		}
	}

	private void UpdateControllers(VRTrackedDevicePose[] poses)
	{
		Vector<Integer> classLst = new Vector<Integer>();
		int cnum = 1;
		for (int i = 0; i < poses.length; i++)
		{
			int devClass = myVrSys.GetTrackedDeviceClass(i);
			classLst.add(devClass);
			
			if (devClass == OpenVRInterop.ETrackedDeviceClass.TrackedDeviceClass_Controller)
			{
				//grab controllers
				VRTrackedDevicePose cpose = poses[i];
				JICameraBase jcb = iman.findInteraction(JICameraBase.class);
				JoglCamera defaultCam = jcb.getCamera();
				
				VRTransform cT = new VRTransform(defaultCam);
				cT.transform(cpose);
				
				VR_TrackedDevice dev = myVrSys.GetTrackedDevice(i);
				
				VRController controller = new VRController(dev, cT);
//				VRControllerState vcs = controller.GetControllerState();
//				BitSet bits = vcs.toBitSet(vcs.ulButtonPressed);
//				if (bits.get(OpenVRInterop.EVRButtonId.EVRButtonId_k_EButton_ApplicationMenu))
//				{
//					System.out.println("Controller " + cnum + ": AppMenu pressed!");
//				}
//				if (bits.get(OpenVRInterop.EVRButtonId.EVRButtonId_k_EButton_Grip))
//				{
//					System.out.println("Controller " + cnum + ": Grip pressed!");
//				}
//				if (bits.get(OpenVRInterop.EVRButtonId.EVRButtonId_k_EButton_SteamVR_Trigger))
//				{
//					System.out.println("Controller " + cnum + ": Trigger pressed!");
//				}
//				if (bits.get(OpenVRInterop.EVRButtonId.EVRButtonId_k_EButton_SteamVR_Touchpad))
//				{
//					System.out.println("Controller " + cnum + ": Touchpad pressed!");
//				}
				
				if (cnum == 1)
				{
					controller1 = controller;
				}
				else
				{
					controller2 = controller;
				}
				
				cnum++;
			}
		}
	}
	
	private void RenderHelper(GraphicsPipeline pipe, JoglCamera c, int resx, int resy) 
	{
		if (myfbo == null)
		{
			myfbo = new JoglFrameBufferObject(pipe, resx, resy);
			myfbo.Bind(pipe);
			myfbo.AttachDepthBuffer(pipe, myfbo.CreateRboAttachment(pipe, GL.GL_DEPTH_COMPONENT32));
			myfbo.AttachColorBuffer0(pipe, myfbo.CreateTboAttachment(pipe, GL.GL_RGBA8, 0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE));
	
			if (!myfbo.isFramebufferComplete())
			{
				System.out.println("Framebuffer failed to initialize.");
				return;
			}
			
			myfbo.Unbind(pipe);
		}
		
		if (myfbo != null)
		{
			JoglRenderTargetFramebuffer jrtb = new JoglRenderTargetFramebuffer(myfbo);
			//set scene background
			JoglVectord3 bgColor = pipe.scene.getBackgroundColor();
			jrtb.setBackgroundColor(bgColor);
			
			pipe.scene.setDrawGui(false);
			jrtb.Bind(pipe);
			jrtb.FitViewport(pipe);
	
			jrtb.Clear(pipe);

			GraphicsPipeline newPipe = new GraphicsPipeline(pipe);
			newPipe.cam = c;
			newPipe.target = jrtb;
			
			pipe.scene.Draw(newPipe);
	
			jrtb.Unbind(pipe);
			pipe.scene.setDrawGui(true);
		}
	}
	
	private void RenderHelperMS(GraphicsPipeline pipe, JoglCamera c, int resx, int resy, int samples) 
	{
		if (myfboms == null)
		{
			myfboms = new JoglMultiSampleFrameBufferObject(pipe, resx, resy, samples);
			myfboms.Bind(pipe);
			myfboms.AttachDepthBuffer(pipe, myfboms.CreateRboAttachment(pipe, GL.GL_DEPTH_COMPONENT32));
			myfboms.AttachColorBuffer0(pipe, myfboms.CreateRboAttachment(pipe, GL.GL_RGBA8));
	
			if (!myfboms.isFramebufferComplete())
			{
				System.out.println("Framebuffer (MS) failed to initialize.");
				return;
			}
			
			myfboms.Unbind(pipe);
		}
		
		if (myfbo == null)
		{
			myfbo = new JoglFrameBufferObject(pipe, resx, resy);
			myfbo.Bind(pipe);
			myfbo.AttachColorBuffer0(pipe, myfbo.CreateTboAttachment(pipe, GL.GL_RGBA8, 0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE));
			//myfbo.AttachColorBuffer0(pipe, myfbo.CreateRboAttachment(pipe, GL.GL_RGBA8));
	
			if (!myfbo.isFramebufferComplete())
			{
				System.out.println("Framebuffer failed to initialize.");
				return;
			}
			
			myfbo.Unbind(pipe);
		}
		
		if (myfboms != null && myfbo != null)
		{
			JoglRenderTargetFramebuffer jrtb = new JoglRenderTargetFramebuffer(myfboms);
			jrtb.setBackgroundColor(pipe.scene.getBackgroundColor());
			
			pipe.scene.setDrawGui(false);
			jrtb.Bind(pipe);
			jrtb.FitViewport(pipe);
	
			jrtb.Clear(pipe);

			GraphicsPipeline newPipe = new GraphicsPipeline(pipe);
			newPipe.target = jrtb;
			newPipe.cam = c;
			
			pipe.scene.Draw(newPipe);
			
			//copy from ms framebuffer!
			myfboms.BlitToFramebuffer(pipe, myfbo);
			
			myfboms.Bind(pipe, GL2.GL_DRAW_FRAMEBUFFER);
	
			jrtb.Unbind(pipe);
			pipe.scene.setDrawGui(true);
			
			//myfbo.Dispose(gl);
			//myfboms.Dispose(gl);
		}
	}
	
	public VRRenderModel LoadMesh(String m)
	{
		return myVrModels.LoadRenderModel(m);
	}
	
	public void FreeMesh(VRRenderModel vrm)
	{
		myVrModels.FreeRenderModel(vrm);
	}
	
	public VRRenderModelTexture LoadTexture(int texture_id)
	{
		return myVrModels.LoadTexture(texture_id);
	}
}
