package org.processmining.processprofiler3d.compviz.vr;

import org.processmining.processprofiler3d.compviz.jogl.AverageTimeMeasure;
import org.processmining.processprofiler3d.compviz.vr.jna.IVRCompositor;
import org.processmining.processprofiler3d.compviz.vr.jna.OpenVRInterop;
import org.processmining.processprofiler3d.compviz.vr.jna.OpenVRInterop.EVRInitError;
import org.processmining.processprofiler3d.compviz.vr.jna.VRTexture;
import org.processmining.processprofiler3d.compviz.vr.jna.VRTextureBounds;
import org.processmining.processprofiler3d.compviz.vr.jna.VRTrackedDevicePose;
import org.processmining.visualisation3d.GraphicsPipeline;
import org.processmining.visualisation3d.gldatastructures.JoglMultiSampleRenderBufferObject;
import org.processmining.visualisation3d.gldatastructures.JoglRenderBufferObject;
import org.processmining.visualisation3d.gldatastructures.JoglTexture;

import com.sun.jna.Pointer;
import com.sun.jna.ptr.IntByReference;

public class HzdVRCompositor extends IVRCompositor
{
	public static VRTexture texFromGL(int texId)
	{
		return new VRTexture(texId, OpenVRInterop.EGraphicsAPIConvention.API_OpenGL, OpenVRInterop.EColorSpace.ColorSpace_Auto);
	}
	
	public static String ivrCompositorStr = "FnTable:IVRCompositor_015";
	
	public static HzdVRCompositor getInstance(OpenVRInterop vrio)
	{
		int peError2 = EVRInitError.None;
		IntByReference ibr = new IntByReference();
		ibr.setValue(peError2);
		Pointer ptr2 = vrio.VR_GetGenericInterface(ivrCompositorStr, ibr);
		peError2 = ibr.getValue();

		System.out.println("GetGenericInterface: " + vrio.VR_GetStringForHmdError(peError2));
		if (peError2 == EVRInitError.None)
		{
			HzdVRCompositor myVrComp = new HzdVRCompositor(ptr2);
			return myVrComp;
		}
		
		return null;
	}
	
	public HzdVRCompositor()
	{
	}

	public HzdVRCompositor(Pointer p)
	{
		super(p);
	}

	VRTexture frameTex = null;
	private AverageTimeMeasure avgSubmitTime = new AverageTimeMeasure();
	private int count = 0;
    
	public void SubmitFrameDual(GraphicsPipeline pipe, JoglTexture tex, VRRenderInfo myRInfo)
	{
		MeasureFrameTime();
		
		if (frameTex == null)
		{
			frameTex = texFromGL(tex.getTextureId());
		}
		frameTex.handle = tex.getTextureId();
		frameTex.eColorSpace = 0;
		frameTex.write();
		//System.out.println(frameTex.toString(true));
		//VRTexture frameTex = texFromGL(myfboms.getColorbufferId());
//		if (frameTex == null)
//		{
//			frameTex = texFromGL(texId);
//		}

		VRTextureBounds pBoundsL = myRInfo.textureBounds[OpenVRInterop.EVREye.Eye_Left];
		VRTextureBounds pBoundsR = myRInfo.textureBounds[OpenVRInterop.EVREye.Eye_Right];
		int nSubmitFlags = OpenVRInterop.EVRSubmitFlags.Submit_Default;// & OpenVRInterop.EVRSubmitFlags.Submit_GlRenderBuffer;
		int errorCode1 = Submit.invoke(OpenVRInterop.EVREye.Eye_Left, frameTex, pBoundsL, nSubmitFlags);
		int errorCode2 = Submit.invoke(OpenVRInterop.EVREye.Eye_Right, frameTex, pBoundsR, nSubmitFlags);
		
		if (errorCode1 != 0)
		{
			System.out.println("Submit Left Frame Error: " + errorCode1);
		}
		if (errorCode2 != 0)
		{
			System.out.println("Submit Right Frame Error: " + errorCode2);
		}
	}
	
	public void SubmitFrameLeft(GraphicsPipeline pipe, JoglTexture tex, VRRenderInfo myRInfo)
	{
		MeasureFrameTime();
		
		if (frameTex == null)
		{
			frameTex = texFromGL(tex.getTextureId());
		}
		frameTex.handle = tex.getTextureId();
		frameTex.eColorSpace = 0;
		frameTex.write();
		//System.out.println(frameTex.toString(true));
		//VRTexture frameTex = texFromGL(myfboms.getColorbufferId());
//		if (frameTex == null)
//		{
//			frameTex = texFromGL(texId);
//		}

		VRTextureBounds pBoundsL = myRInfo.textureBounds[OpenVRInterop.EVREye.Eye_Left];
		int nSubmitFlags = OpenVRInterop.EVRSubmitFlags.Submit_Default;// & OpenVRInterop.EVRSubmitFlags.Submit_GlRenderBuffer;

		int errorCode = Submit.invoke(OpenVRInterop.EVREye.Eye_Left, frameTex, pBoundsL, nSubmitFlags);
		
		if (errorCode != 0)
		{
			System.out.println("Submit Left Frame Error: " + errorCode);
		}
	}
	
	public void SubmitFrameLeft(GraphicsPipeline pipe, JoglRenderBufferObject tex, VRRenderInfo myRInfo)
	{
		MeasureFrameTime();
		
		if (frameTex == null)
		{
			frameTex = texFromGL(tex.getRenderBufferId());
		}
		frameTex.handle = tex.getRenderBufferId();
		frameTex.eColorSpace = 0;
		frameTex.write();
		//System.out.println(frameTex.toString(true));
		//VRTexture frameTex = texFromGL(myfboms.getColorbufferId());
//		if (frameTex == null)
//		{
//			frameTex = texFromGL(texId);
//		}

		VRTextureBounds pBoundsL = myRInfo.textureBounds[OpenVRInterop.EVREye.Eye_Left];
		int nSubmitFlags = OpenVRInterop.EVRSubmitFlags.Submit_Default & OpenVRInterop.EVRSubmitFlags.Submit_GlRenderBuffer;

		int errorCode = Submit.invoke(OpenVRInterop.EVREye.Eye_Left, frameTex, pBoundsL, nSubmitFlags);
		
		if (errorCode != 0)
		{
			System.out.println("Submit Left Frame Error: " + errorCode);
		}
	}
	
	public void SubmitFrameLeft(GraphicsPipeline pipe, JoglMultiSampleRenderBufferObject tex, VRRenderInfo myRInfo)
	{
		MeasureFrameTime();
		
		if (frameTex == null)
		{
			frameTex = texFromGL(tex.getRenderBufferId());
		}
		frameTex.handle = tex.getRenderBufferId();
		frameTex.eColorSpace = 0;
		frameTex.write();
		//System.out.println(frameTex.toString(true));
		//VRTexture frameTex = texFromGL(myfboms.getColorbufferId());
//		if (frameTex == null)
//		{
//			frameTex = texFromGL(texId);
//		}

		VRTextureBounds pBoundsL = myRInfo.textureBounds[OpenVRInterop.EVREye.Eye_Left];
		int nSubmitFlags = OpenVRInterop.EVRSubmitFlags.Submit_Default & OpenVRInterop.EVRSubmitFlags.Submit_GlRenderBuffer;

		int errorCode = Submit.invoke(OpenVRInterop.EVREye.Eye_Left, frameTex, pBoundsL, nSubmitFlags);
		
		if (errorCode != 0)
		{
			System.out.println("Submit Left Frame Error: " + errorCode);
		}
	}
	
	public void SubmitFrameRight(GraphicsPipeline pipe, JoglTexture tex, VRRenderInfo myRInfo)
	{
		//MeasureFrameTime();
		
		if (frameTex == null)
		{
			frameTex = texFromGL(tex.getTextureId());
		}
		frameTex.handle = tex.getTextureId();
		frameTex.eColorSpace = 0;
		frameTex.write();
		//System.out.println(frameTex.toString(true));
		//VRTexture frameTex = texFromGL(myfboms.getColorbufferId());
//		if (frameTex == null)
//		{
//			frameTex = texFromGL(texId);
//		}

		VRTextureBounds pBoundsR = myRInfo.textureBounds[OpenVRInterop.EVREye.Eye_Right];
		int nSubmitFlags = OpenVRInterop.EVRSubmitFlags.Submit_Default;// & OpenVRInterop.EVRSubmitFlags.Submit_GlRenderBuffer;

		int errorCode = Submit.invoke(OpenVRInterop.EVREye.Eye_Right, frameTex, pBoundsR, nSubmitFlags);
		
		if (errorCode != 0)
		{
			System.out.println("Submit Right Frame Error: " + errorCode);
		}
	}
	
	public void SubmitFrameRight(GraphicsPipeline pipe, JoglRenderBufferObject tex, VRRenderInfo myRInfo)
	{
		//MeasureFrameTime();
		
		if (frameTex == null)
		{
			frameTex = texFromGL(tex.getRenderBufferId());
		}
		frameTex.handle = tex.getRenderBufferId();
		frameTex.eColorSpace = 0;
		frameTex.write();
		//System.out.println(frameTex.toString(true));
		//VRTexture frameTex = texFromGL(myfboms.getColorbufferId());
//		if (frameTex == null)
//		{
//			frameTex = texFromGL(texId);
//		}

		VRTextureBounds pBoundsR = myRInfo.textureBounds[OpenVRInterop.EVREye.Eye_Right];
		int nSubmitFlags = OpenVRInterop.EVRSubmitFlags.Submit_Default & OpenVRInterop.EVRSubmitFlags.Submit_GlRenderBuffer;

		int errorCode = Submit.invoke(OpenVRInterop.EVREye.Eye_Right, frameTex, pBoundsR, nSubmitFlags);
		
		if (errorCode != 0)
		{
			System.out.println("Submit Right Frame Error: " + errorCode);
		}
	}
	
	public void SubmitFrameRight(GraphicsPipeline pipe, JoglMultiSampleRenderBufferObject tex, VRRenderInfo myRInfo)
	{
		//MeasureFrameTime();
		
		if (frameTex == null)
		{
			frameTex = texFromGL(tex.getRenderBufferId());
		}
		frameTex.handle = tex.getRenderBufferId();
		frameTex.eColorSpace = 0;
		frameTex.write();
		//System.out.println(frameTex.toString(true));
		//VRTexture frameTex = texFromGL(myfboms.getColorbufferId());
//		if (frameTex == null)
//		{
//			frameTex = texFromGL(texId);
//		}

		VRTextureBounds pBoundsR = myRInfo.textureBounds[OpenVRInterop.EVREye.Eye_Right];
		int nSubmitFlags = OpenVRInterop.EVRSubmitFlags.Submit_Default & OpenVRInterop.EVRSubmitFlags.Submit_GlRenderBuffer;

		int errorCode = Submit.invoke(OpenVRInterop.EVREye.Eye_Right, frameTex, pBoundsR, nSubmitFlags);
		
		if (errorCode != 0)
		{
			System.out.println("Submit Right Frame Error: " + errorCode);
		}
	}

	private void MeasureFrameTime() 
	{
		avgSubmitTime.stop();
		
		if (count > 90)
		{
			double avgTimeMs = avgSubmitTime.getAverageMilliSeconds();
			double avgFps = 1000.0 / avgTimeMs;
			//System.out.println("Submit: " + avgFps);
			count = 0;
		}
		count++;
		avgSubmitTime.start();
	}
	
	private final int maxPoses = 16;
	private VRTrackedDevicePose myClass = new VRTrackedDevicePose(); 
	private VRTrackedDevicePose myClass2 = new VRTrackedDevicePose(); 
	private VRTrackedDevicePose[] rPoses = (VRTrackedDevicePose[]) myClass.toArray(maxPoses);
	private VRTrackedDevicePose[] gPoses = (VRTrackedDevicePose[]) myClass2.toArray(maxPoses);
	
	public VRTrackedDevicePose[] WaitGetPoses()
	{
		WaitGetPoses.invoke(rPoses, maxPoses, gPoses, maxPoses);
		return gPoses;
	}
}
