package org.processmining.processprofiler3d.compviz.vr;

import org.processmining.processprofiler3d.compviz.vr.jna.IVRRenderModels;
import org.processmining.processprofiler3d.compviz.vr.jna.OpenVRInterop;
import org.processmining.processprofiler3d.compviz.vr.jna.OpenVRInterop.EVRInitError;
import org.processmining.processprofiler3d.compviz.vr.jna.VRRenderModel;
import org.processmining.processprofiler3d.compviz.vr.jna.VRRenderModelTexture;

import com.sun.jna.Pointer;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.ptr.PointerByReference;

public class HzdVRRenderModels
{
	public static String ivrRenderModelsStr = "FnTable:IVRRenderModels_005";

	public static HzdVRRenderModels getInstance(OpenVRInterop vrio)
	{
		if (vrio.VR_IsInterfaceVersionValid(ivrRenderModelsStr))
		{
			IntByReference ibr = new IntByReference();
			int peError1 = EVRInitError.None;
			ibr.setValue(peError1);
			Pointer ptr1 = vrio.VR_GetGenericInterface(ivrRenderModelsStr, ibr);
			peError1 = ibr.getValue();
	
			System.out.println("GetGenericInterface: " + vrio.VR_GetStringForHmdError(peError1));
			if (peError1 == EVRInitError.None)
			{
				HzdVRRenderModels vrRenderModels = new HzdVRRenderModels(vrio, ptr1);
				return vrRenderModels;
			}
		}
		
		return null;
	}
	
	private IVRRenderModels vrRenderModels;
	
	private HzdVRRenderModels(OpenVRInterop vrio, Pointer ptr)
	{
		vrRenderModels = new IVRRenderModels(ptr);
	}
	
	public VRRenderModel LoadRenderModel(String m)
	{
		PointerByReference pbr = new PointerByReference();
		
		int error = vrRenderModels.LoadRenderModel_Async.invoke(m, pbr);
		
		if (error == OpenVRInterop.EVRRenderModelError.EVRRenderModelError_VRRenderModelError_None)
		{
			VRRenderModel rm = new VRRenderModel(pbr.getValue());
//			VRRenderModelVertex vmv = new VRRenderModelVertex(rm.vertexData);
//			VRRenderModelVertex[] verts = (VRRenderModelVertex[])vmv.toArray(rm.vertexCount);
			
			return rm;
		}
		
		
		return null;
	}
	
	public void FreeRenderModel(VRRenderModel vrm)
	{
		vrRenderModels.FreeRenderModel.invoke(vrm.getPointer());
	}
	
	public VRRenderModelTexture LoadTexture(int texture_id)
	{
		PointerByReference pbr = new PointerByReference();
		
		int error = vrRenderModels.LoadTexture_Async.invoke(texture_id, pbr);
		
		if (error == OpenVRInterop.EVRRenderModelError.EVRRenderModelError_VRRenderModelError_None)
		{
			VRRenderModelTexture rm = new VRRenderModelTexture(pbr.getValue());
			
			return rm;
		}
		
		
		return null;
	}

}
