package org.processmining.processprofiler3d.compviz.vr;

import org.processmining.processprofiler3d.compviz.vr.jna.IVRSystem;
import org.processmining.processprofiler3d.compviz.vr.jna.OpenVRInterop;
import org.processmining.processprofiler3d.compviz.vr.jna.OpenVRInterop.EVRInitError;
import org.processmining.processprofiler3d.compviz.vr.jna.VRControllerState;
import org.processmining.processprofiler3d.compviz.vr.jna.VRHmdMatrix34;
import org.processmining.processprofiler3d.compviz.vr.jna.VRHmdMatrix44;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;

import com.sun.jna.Pointer;
import com.sun.jna.ptr.FloatByReference;
import com.sun.jna.ptr.IntByReference;

public class HzdVRSystem 
{
	public class VR_RenderTargetSize
	{
		private int width = 1;
		private int height = 1;
		
		public VR_RenderTargetSize(int w, int h)
		{
			width = w;
			height = h;
		}
		
		public int getWidth()
		{
			return width;
		}
		
		public int getHeight()
		{
			return height;
		}
	}
	
	public JoglMatrixd4x4 getEyeToHeadTransformLeft()
	{
		VRHmdMatrix34 p = vrSys.GetEyeToHeadTransform.invoke(OpenVRInterop.EVREye.Eye_Left);
		
		return p.toJoglMatrix4x4();
	}
	
	public JoglMatrixd4x4 getEyeToHeadTransformRight()
	{
		VRHmdMatrix34 p = vrSys.GetEyeToHeadTransform.invoke(OpenVRInterop.EVREye.Eye_Right);
		
		return p.toJoglMatrix4x4();
	}
	
	public static String ivrSystemStr = "FnTable:IVRSystem_012";
	
	private OpenVRInterop vrio; 
	private IVRSystem vrSys;

	private HzdVRSystem(OpenVRInterop vrio, Pointer ptr)
	{
		this.vrio = vrio;
		this.vrSys = new IVRSystem(ptr);
	}

	public JoglMatrixd4x4 GetProjL(double fNear, double fFar)
	{
		VRHmdMatrix44 mat = vrSys.GetProjectionMatrix.invoke(OpenVRInterop.EVREye.Eye_Left, (float)fNear, (float)fFar, OpenVRInterop.EGraphicsAPIConvention.API_OpenGL);
		return mat.toJoglMatrix4x4();
	}

	public JoglMatrixd4x4 GetProjR(double fNear, double fFar)
	{
		VRHmdMatrix44 mat = vrSys.GetProjectionMatrix.invoke(OpenVRInterop.EVREye.Eye_Right, (float)fNear, (float)fFar, OpenVRInterop.EGraphicsAPIConvention.API_OpenGL);
		return mat.toJoglMatrix4x4();
	}
	
	public VR_RenderTargetSize GetRecommendedRenderTargetSize()
	{
		IntByReference pnWidth = new IntByReference();
		IntByReference pnHeight = new IntByReference();
		vrSys.GetRecommendedRenderTargetSize.invoke(pnWidth, pnHeight);
		
		//VRHmdMatrix34 p1 = vrSys.GetEyeToHeadTransform.invoke(OpenVRInterop.EVREye.Eye_Left);
		//Pointer p2 = vrSys.GetEyeToHeadTransform.invoke(OpenVRInterop.EVREye.Eye_Right);
		
		return new VR_RenderTargetSize(pnWidth.getValue(), pnHeight.getValue());
	}
	
	public VR_TrackedDevice GetHeadMountedDisplay()
	{
		return new VR_TrackedDevice(vrSys, VR_TrackedDevice.TrackedDeviceIndex_Hmd);
	}
	
	public VR_TrackedDevice GetTrackedDevice(int deviceId)
	{
		return new VR_TrackedDevice(vrSys, deviceId);
	}
	
	public int GetTrackedDeviceClass(int deviceId)
	{
		return vrSys.GetTrackedDeviceClass.invoke(deviceId);
	}
	
	public VRControllerState GetControllerState(int deviceId)
	{
		Pointer p = new Pointer(0);
		vrSys.GetControllerState.invoke(deviceId, p);
		
		return new VRControllerState(p);
	}
	
	public void GetProjectionRaw(int eye, FloatByReference pfLeft, FloatByReference pfRight, FloatByReference pfTop, FloatByReference pfBottom)
	{
		vrSys.GetProjectionRaw.invoke(eye, pfLeft, pfRight, pfTop, pfBottom);
	}
	
	public static HzdVRSystem getInstance(OpenVRInterop vrio)
	{
		if (vrio.VR_IsInterfaceVersionValid(ivrSystemStr))
		{
			IntByReference ibr = new IntByReference();
			int peError1 = EVRInitError.None;
			ibr.setValue(peError1);
			Pointer ptr1 = vrio.VR_GetGenericInterface(ivrSystemStr, ibr);
			peError1 = ibr.getValue();
	
			System.out.println("GetGenericInterface: " + vrio.VR_GetStringForHmdError(peError1));
			if (peError1 == EVRInitError.None)
			{
				HzdVRSystem vrSys = new HzdVRSystem(vrio, ptr1);
				return vrSys;
			}
		}
		
		return null;
	}
}
