package org.processmining.processprofiler3d.compviz.vr;

import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.interactions.camera.JICameraVR;
import org.processmining.processprofiler3d.compviz.vr.jna.VRTrackedDevicePose;
import org.processmining.visualisation3d.JoglInteractionManager;
import org.processmining.visualisation3d.collision.JoglAABB;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCameraLensPerspective;
import org.processmining.visualisation3d.interactions.JICameraBase;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class VRHmdUtil
{
	private JoglInteractionManager iman;
	private HzdVRSystem myVrSys;
	private VRRenderInfo myRInfo;

	public JoglCamera camLeft;
	public JoglCamera camRight;

	public JoglVectord2 defaultDepthRange = new JoglVectord2(0.1, 2000.0);
	
	public VRHmdUtil(JoglInteractionManager iman, HzdVRSystem myVrSys, VRRenderInfo myRInfo)
	{
		this.iman = iman;
		this.myVrSys = myVrSys;
		this.myRInfo = myRInfo;
	}

	public void setupCameras(JoglScene scene, VRTrackedDevicePose hmdPose)
	{
		JICameraBase jcb = iman.findInteraction(JICameraBase.class);
		JoglCamera defaultCam = jcb.getCamera();

		//transform camera space
		VRTransform hmdT = new VRTransform(defaultCam);
		hmdT.transform(hmdPose);
		
		//set up cameras for left and right eye
		camLeft = new JoglCamera();
		camRight = new JoglCamera();
		
		camLeft.SetPosition(hmdT.hmdPosWS);
		camRight.SetPosition(hmdT.hmdPosWS);
		
		camLeft.SetLookDir(hmdT.hmdLookDirWS, hmdT.hmdUpDirWS);
		camRight.SetLookDir(hmdT.hmdLookDirWS, hmdT.hmdUpDirWS);
		
		//update interaction
		if (jcb instanceof JICameraVR)
		{
			JICameraVR vrCamInt = (JICameraVR)jcb;
			if (vrCamInt != null)
			{
				vrCamInt.setHmdDir(hmdT.hmdLookDirWS, hmdT.hmdUpDirWS, hmdT.hmdLeftDirWS);
				vrCamInt.hmdPosAbsolute = hmdT.hmdPosWS;
				vrCamInt.setHMDCamera(camRight);
			}
		}
		
		//add eye to head offset to each camera
		//need inverse of matrix, so use wrong eye. This might cause issues if transform contains more than just IPD!
		addEyeOffset(camLeft, myVrSys.getEyeToHeadTransformRight());
		addEyeOffset(camRight, myVrSys.getEyeToHeadTransformLeft());
		
		//determine scene depth range for each camera
		Vector<JoglAABB> boundingBoxes = scene.getBoundingBoxes();
		JoglVectord2 depthRangeLeft = defaultDepthRange;
		JoglVectord2 depthRangeRight = defaultDepthRange;
		if (!boundingBoxes.isEmpty())
		{
			depthRangeLeft = camLeft.calcDepthRange(boundingBoxes);
			depthRangeRight = camRight.calcDepthRange(boundingBoxes);
		}
		
		JoglCameraLensPerspective lensL = new JoglCameraLensPerspective(myRInfo.fieldOfView, myRInfo.aspect, depthRangeLeft.getX(), depthRangeLeft.getY());
		JoglCameraLensPerspective lensR = new JoglCameraLensPerspective(myRInfo.fieldOfView, myRInfo.aspect, depthRangeRight.getX(), depthRangeRight.getY());
		
		camLeft.setCameraLens(lensL);
		camRight.setCameraLens(lensR);
	}

	private void addEyeOffset(JoglCamera cam, JoglMatrixd4x4 offset)
	{
		double[] vrDataL = offset.GetData();
		JoglVectord3 offsL = new JoglVectord3(vrDataL[12], vrDataL[13], vrDataL[14]);
		JoglVectord3 camPosrelL = JoglVectord3.Add(JoglVectord3.Add(JoglVectord3.Multiply(cam.GetLeftDir(), offsL.getX()), JoglVectord3.Multiply(cam.GetUpDir(), offsL.getY())), JoglVectord3.Multiply(cam.GetLookDir(), offsL.getZ()));
		cam.SetPosition(JoglVectord3.Add(camPosrelL, cam.GetPosition()));
	}

}
