package org.processmining.processprofiler3d.compviz.vr;

import org.processmining.processprofiler3d.compviz.vr.jna.VRTextureBounds;
import org.processmining.processprofiler3d.compviz.vr.jna.OpenVRInterop.EVREye;

import com.sun.jna.ptr.FloatByReference;

public class VRRenderInfo 
{
	private float radToDeg = 57.2958f;
	
	public float sceneWidth;
	public float sceneHeight;
	
	public float aspect;
	public float fieldOfView;
	
	public float tanHalfFovX;
	public float tanHalfFovY;
	
	public VRTextureBounds textureBounds[] = new VRTextureBounds[2];
	
	public VRRenderInfo(HzdVRSystem vrSys)
	{
		HzdVRSystem.VR_RenderTargetSize targetSize = vrSys.GetRecommendedRenderTargetSize();
		sceneWidth = targetSize.getWidth();
		sceneHeight = targetSize.getHeight();
		
		float l_left = 0.0f, l_right = 0.0f, l_top = 0.0f, l_bottom = 0.0f;
		FloatByReference lfbr = new FloatByReference();
		lfbr.setValue(l_left);
		FloatByReference rfbr = new FloatByReference();
		rfbr.setValue(l_right);
		FloatByReference tfbr = new FloatByReference();
		tfbr.setValue(l_top);
		FloatByReference bfbr = new FloatByReference();
		bfbr.setValue(l_bottom);
		vrSys.GetProjectionRaw(EVREye.Eye_Left, lfbr, rfbr, tfbr, bfbr);
		l_left = lfbr.getValue();
		l_right = rfbr.getValue();
		l_top = tfbr.getValue();
		l_bottom = bfbr.getValue();

		float r_left = 0.0f, r_right = 0.0f, r_top = 0.0f, r_bottom = 0.0f;
		lfbr.setValue(r_left);
		rfbr.setValue(r_right);
		tfbr.setValue(r_top);
		bfbr.setValue(r_bottom);
		vrSys.GetProjectionRaw(EVREye.Eye_Right, lfbr, rfbr, tfbr, bfbr);
		r_left = lfbr.getValue();
		r_right = rfbr.getValue();
		r_top = tfbr.getValue();
		r_bottom = bfbr.getValue();

		tanHalfFovX = Math.max(Math.max(-l_left, l_right), Math.max(-r_left, r_right));
		tanHalfFovY = Math.max(Math.max(-l_top, l_bottom), Math.max(-r_top, r_bottom));
		
		textureBounds[0] = new VRTextureBounds();
		textureBounds[1] = new VRTextureBounds();

		textureBounds[0].uMin = 0.5f + 0.5f * l_left / tanHalfFovX;
		textureBounds[0].uMax = 0.5f + 0.5f * l_right / tanHalfFovX;
		textureBounds[0].vMin = 0.5f - 0.5f * l_bottom / tanHalfFovY;
		textureBounds[0].vMax = 0.5f - 0.5f * l_top / tanHalfFovY;

		textureBounds[1].uMin = 0.5f + 0.5f * r_left / tanHalfFovX;
		textureBounds[1].uMax = 0.5f + 0.5f * r_right / tanHalfFovX;
		textureBounds[1].vMin = 0.5f - 0.5f * r_bottom / tanHalfFovY;
		textureBounds[1].vMax = 0.5f - 0.5f * r_top / tanHalfFovY;
		
		// Grow the recommended size to account for the overlapping fov
		sceneWidth = sceneWidth / Math.max(textureBounds[0].uMax - textureBounds[0].uMin, textureBounds[1].uMax - textureBounds[1].uMin);
		sceneHeight = sceneHeight / Math.max(textureBounds[0].vMax - textureBounds[0].vMin, textureBounds[1].vMax - textureBounds[1].vMin);

		aspect = tanHalfFovX / tanHalfFovY;
		fieldOfView = (float) (2.0f * Math.atan(tanHalfFovY) * radToDeg);
	}
}
