package org.processmining.processprofiler3d.compviz.vr;

import org.processmining.processprofiler3d.compviz.vr.jna.VRTrackedDevicePose;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class VRTransform
{
	public JoglMatrixd4x4 hmdRotMat;
	public JoglVectord3 hmdPos;
	
	//transform camera space
	public JoglVectord3 hmdLeftDir; //left axis
	public JoglVectord3 hmdUpDir; //up axis
	public JoglVectord3 hmdLookDir;//look axis

	public JoglVectord3 hmdPosWSrel;
	public JoglVectord3 hmdLeftDirWS;
	public JoglVectord3 hmdUpDirWS;
	public JoglVectord3 hmdLookDirWS;

	public JoglVectord3 hmdPosWS;
	
	public JoglVectord3 scale;
	
	public JoglCamera defaultCam;

	public VRTransform(JoglCamera defaultCam)
	{
		this.defaultCam = defaultCam;
	}
	
	public void transform(VRTrackedDevicePose hmdPose)
	{
		scale = hmdPose.getScale();
		
		hmdRotMat = hmdPose.getRotationMatrix();
		hmdPos = hmdPose.getPosition();
		
		//transform camera space
		hmdLeftDir = hmdRotMat.Transformv3(new JoglVectord3(1,0,0)); //left axis
		hmdUpDir = hmdRotMat.Transformv3(new JoglVectord3(0,1,0)); //up axis
		hmdLookDir = hmdRotMat.Transformv3(new JoglVectord3(0,0,-1));//look axis

		hmdPosWSrel = JoglVectord3.Add(JoglVectord3.Add(JoglVectord3.Multiply(defaultCam.GetLeftDir(), hmdPos.getX()), JoglVectord3.Multiply(defaultCam.GetUpDir(), hmdPos.getY())), JoglVectord3.Multiply(defaultCam.GetLookDir(), hmdPos.getZ()));
		hmdLeftDirWS = JoglVectord3.Add(JoglVectord3.Add(JoglVectord3.Multiply(defaultCam.GetLeftDir(), hmdLeftDir.getX()), JoglVectord3.Multiply(defaultCam.GetUpDir(), hmdLeftDir.getY())), JoglVectord3.Multiply(defaultCam.GetLookDir(), hmdLeftDir.getZ()));
		hmdUpDirWS = JoglVectord3.Add(JoglVectord3.Add(JoglVectord3.Multiply(defaultCam.GetLeftDir(), hmdUpDir.getX()), JoglVectord3.Multiply(defaultCam.GetUpDir(), hmdUpDir.getY())), JoglVectord3.Multiply(defaultCam.GetLookDir(), hmdUpDir.getZ()));
		hmdLookDirWS = JoglVectord3.Add(JoglVectord3.Add(JoglVectord3.Multiply(defaultCam.GetLeftDir(), hmdLookDir.getX()), JoglVectord3.Multiply(defaultCam.GetUpDir(), hmdLookDir.getY())), JoglVectord3.Multiply(defaultCam.GetLookDir(), hmdLookDir.getZ()));

		hmdPosWS = JoglVectord3.Add(hmdPosWSrel, defaultCam.GetPosition());
	}

}
