package org.processmining.processprofiler3d.compviz.vr;

import org.processmining.processprofiler3d.compviz.vr.jna.IVRSystem;
import org.processmining.processprofiler3d.compviz.vr.jna.OpenVRInterop.EVRInitError;
import org.processmining.processprofiler3d.compviz.vr.jna.VRControllerState;
import org.processmining.processprofiler3d.compviz.vr.jna.VRTrackedDevicePose;
import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;

import com.sun.jna.Memory;
import com.sun.jna.Pointer;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.ptr.PointerByReference;

public class VR_TrackedDevice
{
	public static final int TrackedDeviceIndex_Hmd = 0;
	
	private IVRSystem vrSys;
	private int deviceId = 0;
	
	private VRTrackedDevicePose currentPose = null;
	
	public VR_TrackedDevice(IVRSystem vrSys, int dId)
	{
		this.vrSys = vrSys;
		deviceId = dId;
	}
	
	public boolean IsTrackedDeviceConnected()
	{
		return vrSys.IsTrackedDeviceConnected.invoke(deviceId);
	}
	
	public int GetTrackedDeviceActivityLevel()
	{
		return vrSys.GetTrackedDeviceActivityLevel.invoke(deviceId);
	}
	
	public int GetTrackedDeviceClass()
	{
		return vrSys.GetTrackedDeviceClass.invoke(deviceId);
	}

	public boolean GetBoolTrackedDeviceProperty(int prop)
	{
		int pError = EVRInitError.None;
		IntByReference ibr = new IntByReference();
		ibr.setValue(pError);
		
		boolean bVal = vrSys.GetBoolTrackedDeviceProperty.invoke(deviceId, prop, ibr);
		
		ibr.getValue();
		
		return bVal;
	}

	public float GetFloatTrackedDeviceProperty(int prop)
	{
		int pError = EVRInitError.None;
		IntByReference ibr = new IntByReference();
		ibr.setValue(pError);
		
		float fVal = vrSys.GetFloatTrackedDeviceProperty.invoke(deviceId, prop, ibr);
		
		ibr.getValue();
		
		return fVal;
	}

	public int GetInt32TrackedDeviceProperty(int prop)
	{
		int pError = EVRInitError.None;
		IntByReference ibr = new IntByReference();
		ibr.setValue(pError);
		
		int iVal = vrSys.GetInt32TrackedDeviceProperty.invoke(deviceId, prop, ibr);
		
		ibr.getValue();
		
		return iVal;
	}

	public long GetUint64TrackedDeviceProperty(int prop)
	{
		int pError = EVRInitError.None;
		IntByReference ibr = new IntByReference();
		ibr.setValue(pError);
		
		long lVal = vrSys.GetUint64TrackedDeviceProperty.invoke(deviceId, prop, ibr);
		
		ibr.getValue();
		
		return lVal;
	}

	public String GetStringTrackedDeviceProperty(int prop)
	{
		int pError = EVRInitError.None;
		IntByReference ibr = new IntByReference();
		ibr.setValue(pError);
		
		String val = "";
		PointerByReference pbr = new PointerByReference();
		Memory mem = new Memory(250);
		pbr.setPointer(mem);
		
		int lVal = vrSys.GetStringTrackedDeviceProperty.invoke(deviceId, prop, pbr, 250, ibr);
		
		int errorInt = ibr.getValue();
		Pointer ptr = pbr.getValue();
		//val = pbr.getValue().getString(0);
		String s = mem.getString(0);
		
		val = s;
		
		return val;
	}
	
	public VRControllerState GetControllerState()
	{
		Pointer p = new VRControllerState().getPointer();
		vrSys.GetControllerState.invoke(deviceId, p);
		
		return new VRControllerState(p);
	}
	
//    public IGetMatrix34TrackedDeviceProperty GetMatrix34TrackedDeviceProperty;
//    public IGetStringTrackedDeviceProperty GetStringTrackedDeviceProperty;

	public void setPose(VRTrackedDevicePose p)
	{
		currentPose = p;
	}
	
    public JoglMatrixd4x4 getAbsoluteTransform()
    {
    	if (currentPose != null)
    	{
    		return currentPose.getAbsoluteTransform();
    	}
    	return JoglMatrixd4x4.CreateIdentity();
    }
	
    public JoglMatrixd4x4 getRotationMatrix()
    {
    	if (currentPose != null)
    	{
    		return currentPose.getRotationMatrix();
    	}
    	return JoglMatrixd4x4.CreateIdentity();
    }
	
    public JoglVectord3 getPosition()
    {
    	if (currentPose != null)
    	{
    		return currentPose.getPosition();
    	}
    	return JoglVectord3.ZERO;
    }
}
