package org.processmining.processprofiler3d.compviz.vr.jna;

import java.nio.ShortBuffer;
import java.util.List;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.graphicsdatastructures.JoglOpenVRMesh;
import org.processmining.visualisation3d.graphicsdatastructures.JoglMeshIndexBuffer;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;

import com.jogamp.common.nio.Buffers;
import com.jogamp.opengl.GL2;
import com.sun.jna.Pointer;
import com.sun.jna.Structure;

public class VRRenderModel extends Structure
{	
	//public Pointer handle; // void *
	//public IntByReference handle;
	//public long handle;
	public Pointer vertexData;
	public int vertexCount;
	public Pointer indexData;
	public int triangleCount;
	public int diffuseTexId;

	public VRRenderModel()
	{
		super();
		
		//this.setAlignType(this.ALIGN_NONE);
	}
	
	public VRRenderModel(Pointer peer)
	{
		super(peer);
		read();
	}
	
	protected List<String> getFieldOrder() 
	{
		Vector<String> lst = new Vector<String>();
		//lst.add("handle");
		lst.add("vertexData");
		lst.add("vertexCount");
		lst.add("indexData");
		lst.add("triangleCount");
		lst.add("diffuseTexId");
		
		return lst;
	}


	public JoglOpenVRMesh loadMesh()
	{
		VRRenderModelVertex vmv = new VRRenderModelVertex(vertexData);
		VRRenderModelVertex[] verts = (VRRenderModelVertex[])vmv.toArray(vertexCount);

		Vector<JoglVectord3> lstVerts = new Vector<JoglVectord3>();
		Vector<JoglVectord3> lstNorms = new Vector<JoglVectord3>();
		Vector<JoglVectord2> lstUVs = new Vector<JoglVectord2>();
		for (VRRenderModelVertex rmv : verts)
		{
			lstVerts.add(new JoglVectord3(rmv.vPos.v[0],rmv.vPos.v[1],rmv.vPos.v[2]));
			lstNorms.add(new JoglVectord3(rmv.vNorm.v[0],rmv.vNorm.v[1],rmv.vNorm.v[2]));
			lstUVs.add(new JoglVectord2(rmv.rfTextureCoord[0],rmv.rfTextureCoord[1]));
		}
		
		short[] indices = indexData.getShortArray(0, triangleCount * 3);

        ShortBuffer indata = Buffers.newDirectShortBuffer(triangleCount * 3);
        for (int i = 0; i < (triangleCount * 3); i++)
        {
            indata.put(indices[i]);
        }
        indata.rewind();

		JoglOpenVRMesh jcm = new JoglOpenVRMesh(GL2.GL_TRIANGLES);
		jcm.setVerts(lstVerts);
		jcm.setTexCoords(lstUVs);
		jcm.setNormals(lstNorms);
		JoglMeshIndexBuffer idxBuf = new JoglMeshIndexBuffer(indata);
		jcm.setIndexBuffer(idxBuf);		
		
		return jcm;
		
	}
	
	public static class ByReference extends VRRenderModel implements Structure.ByReference {
		
	};
	public static class ByValue extends VRRenderModel implements Structure.ByValue {
		
	};

}
