package org.processmining.processprofiler3d.compviz.vr.jna;

import java.util.List;
import java.util.Vector;

import org.processmining.visualisation3d.maths.JoglMatrixd4x4;
import org.processmining.visualisation3d.maths.JoglVectord3;

import com.sun.jna.Pointer;
import com.sun.jna.Structure;

public class VRTrackedDevicePose extends Structure 
{
	public VRHmdMatrix34 mDeviceToAbsoluteTracking;
	/** velocity in tracker space in m/s */
	public VRHmdVector3 vVelocity;
	/** angular velocity in radians/s (?) */
	public VRHmdVector3 vAngularVelocity;
	/** @see HmdTrackingResult */
	public int eTrackingResult;
	public byte bPoseIsValid;
	public byte bDeviceIsConnected;
	
	public VRTrackedDevicePose() 
	{
		super();
	}
	
	protected List<String> getFieldOrder() 
	{
		Vector<String> lst = new Vector<String>();
		lst.add("mDeviceToAbsoluteTracking");
		lst.add("vVelocity");
		lst.add("vAngularVelocity");
		lst.add("eTrackingResult");
		lst.add("bPoseIsValid");
		lst.add("bDeviceIsConnected");
		
		return lst;
	}
	
	public VRTrackedDevicePose(VRHmdMatrix34 mDeviceToAbsoluteTracking, VRHmdVector3 vVelocity, VRHmdVector3 vAngularVelocity, int eTrackingResult, byte bPoseIsValid, byte bDeviceIsConnected) 
	{
		super();
		
		this.mDeviceToAbsoluteTracking = mDeviceToAbsoluteTracking;
		this.vVelocity = vVelocity;
		this.vAngularVelocity = vAngularVelocity;
		this.eTrackingResult = eTrackingResult;
		this.bPoseIsValid = bPoseIsValid;
		this.bDeviceIsConnected = bDeviceIsConnected;
	}
	
	public VRTrackedDevicePose(Pointer peer) 
	{
		super(peer);
	}
	
	public JoglMatrixd4x4 getAbsoluteTransform()
	{
		return mDeviceToAbsoluteTracking.toJoglMatrix4x4();
	}
	
	public JoglMatrixd4x4 getRotationMatrix()
	{
    	JoglMatrixd4x4 mat = JoglMatrixd4x4.CreateIdentity();
        
        mat.SetValue(0,0, mDeviceToAbsoluteTracking.m[0]);
        mat.SetValue(0,1, mDeviceToAbsoluteTracking.m[1]);
        mat.SetValue(0,2, mDeviceToAbsoluteTracking.m[2]);
        mat.SetValue(0,3, 0);

        mat.SetValue(1,0, mDeviceToAbsoluteTracking.m[4]);
        mat.SetValue(1,1, mDeviceToAbsoluteTracking.m[5]);
        mat.SetValue(1,2, mDeviceToAbsoluteTracking.m[6]);
        mat.SetValue(1,3, 0);

        mat.SetValue(2,0, mDeviceToAbsoluteTracking.m[8]);
        mat.SetValue(2,1, mDeviceToAbsoluteTracking.m[9]);
        mat.SetValue(2,2, mDeviceToAbsoluteTracking.m[10]);
        mat.SetValue(2,3, 0);
        
		return mat;
	}
	
	public JoglVectord3 getPosition()
	{
		JoglVectord3 hmdPos = new JoglVectord3(mDeviceToAbsoluteTracking.m[3], mDeviceToAbsoluteTracking.m[7], mDeviceToAbsoluteTracking.m[11]);
		
		return hmdPos;
	}
	
	public JoglVectord3 getScale()
	{
		JoglVectord3 xvec = new JoglVectord3(mDeviceToAbsoluteTracking.m[0], mDeviceToAbsoluteTracking.m[1], mDeviceToAbsoluteTracking.m[2]);
		JoglVectord3 yvec = new JoglVectord3(mDeviceToAbsoluteTracking.m[4], mDeviceToAbsoluteTracking.m[5], mDeviceToAbsoluteTracking.m[6]);
		JoglVectord3 zvec = new JoglVectord3(mDeviceToAbsoluteTracking.m[8], mDeviceToAbsoluteTracking.m[9], mDeviceToAbsoluteTracking.m[10]);
		
		return new JoglVectord3(xvec.getMagnitude(), yvec.getMagnitude(), zvec.getMagnitude());
	}
	
	public static class ByReference extends VRTrackedDevicePose implements Structure.ByReference 
	{
	};
	
	public static class ByValue extends VRTrackedDevicePose implements Structure.ByValue 
	{
	};
}
