package org.processmining.processprofiler3d.maths.stats;

import java.util.List;

import org.apache.commons.math3.distribution.FDistribution;
import org.apache.commons.math3.stat.descriptive.rank.Median;

public class LeveneTest 
{
	public boolean test(List<double[]> classes, double pLevel)
	{
		double w = 0; //result of test
		double N = 0; //total number of cases
		double k = classes.size(); // number groups
		
		for (double[] ds : classes)
		{
			N = N + ds.length;
		}
		
		double w1 = (N - k) / (k - 1);
		
		double zdotdot = calcZdotdot(classes, N);
		double[] yidotlst = new double[classes.size()];
		double[] zidotlst = new double[classes.size()];
		
		double w2top = 0;
		for (int i = 0; i < k; i++)
		{
			//Mean mn = new Mean();
			Median m = new Median();
			m.setData(classes.get(i));
			double Yidot = m.evaluate();
			yidotlst[i] = Yidot;
			
			double zidot = calcZidot(classes, i, Yidot);
			zidotlst[i] = zidot;
			
			double term2 = (zidot - zdotdot);
			double term = classes.get(i).length * term2 * term2;
			w2top = w2top + term;
		}
		
		double w2bottom = 0;
		for (int i = 0; i < k; i++)
		{
			for (int j = 0; j < classes.get(i).length; j++)
			{
				double term = calcZij(classes, i,j, yidotlst[i]) - zidotlst[i];
				w2bottom = w2bottom + (term * term);
			}
		}
		
		double w2 = w2top / w2bottom;
		
		w = w1 * w2;
		
		FDistribution fdist = new FDistribution(k-1, N-k);
		
		//double pw1 = fdist.cumulativeProbability(0.05);
		//fdist.
		//double pw2 = fdist.inverseCumulativeProbability(pLevel);
		double pw = 1.0 - fdist.cumulativeProbability(w);
		//double pw2 = fdist.inverseCumulativeProbability(w);
		
		//boolean bVariancesEqual = w <= pw2;
		boolean bVariancesEqual = pw < pLevel;
		
		return bVariancesEqual;
	}
	
	private double calcZdotdot(List<double[]> classes, double N)
	{
		double Zij = 0;
		for (int i = 0; i < classes.size(); i++)
		{
			//Mean mn = new Mean();
			Median m = new Median();
			m.setData(classes.get(i));
			double Yidot = m.evaluate();
			
			for (int j = 0; j < classes.get(i).length; j++)
			{
				Zij = Zij + calcZij(classes, i, j, Yidot);
			}
		}
		
		return (1.0 / N) * Zij;
	}
	
	private double calcZidot(List<double[]> classes, int i, double Yidot)
	{
		double Zidot = 0;
		for (int j = 0; j < classes.get(i).length; j++)
		{
			Zidot = Zidot + calcZij(classes, i, j, Yidot);
		}
		
		return (1.0 / classes.get(i).length) * Zidot;
	}
	
	private double calcZij(List<double[]> classes, int i, int j, double Yidot)
	{
		double[] groupI = classes.get(i);
		
		double Yij = groupI[j];
		
		return Math.abs(Yij - Yidot);
	}
}
