package org.processmining.processprofiler3d.netstructure;

import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class PathFragmentCollection 
{
	private Vector<PetrinetNode> mynodes;
	private Vector<PathFragment> myfrags;
	
	public PathFragmentCollection(Vector<PetrinetNode> nodes, Vector<PathFragment> frags)
	{
		mynodes = nodes;
		myfrags = frags;
		
		int j = 1;
		while (j > 0)
		{
			j = 0;
		
			int i = 1;
			while (i > 0)
			{
				i = findSequence();
				if (i > 0)
				{
					j++;
				}
			}
			
			i = 1;
			while (i > 0)
			{
				i = findParallel();
				if (i > 0)
				{
					j++;
				}
			}
		}
		
		//String s = "test";
	}
	
	public int findSequence()
	{
		int i = 0;
		for (PetrinetNode pn : new Vector<PetrinetNode>(mynodes))
		{
			if (findSequenceForNode(pn))
			{
				i++;
			}
		}
		return i;
	}
	
	public boolean findSequenceForNode(PetrinetNode pn)
	{
		Vector<PathFragment> fragsIn = new Vector<PathFragment>();
		Vector<PathFragment> fragsOut = new Vector<PathFragment>();
		Vector<PathFragment> fragsLoop = new Vector<PathFragment>();
		
		for (PathFragment pf : myfrags)
		{
			//loop fragment!
			if (pf.getSource() == pf.getTarget() && pf.getSource() == pn)
			{
				fragsLoop.add(pf);
			}
			else
			{		
				if (pf.getTarget() == pn)
				{
					fragsIn.add(pf);
				}
				if (pf.getSource() == pn)
				{
					fragsOut.add(pf);
				}
			}
		}
		
		if (fragsIn.size() == 1 && fragsOut.size() == 1)
		{
			if (fragsLoop.size() > 1)
			{
				return false; //come back later, after parallel flowers have been merged
			}
			
			if (fragsLoop.size() == 1)
			{
				//merge fragments into sequence
				PathFragment pf = new PathFragmentFlower(fragsIn.get(0), fragsOut.get(0), fragsLoop.get(0));
				myfrags.remove(fragsIn.get(0));
				myfrags.remove(fragsOut.get(0));
				myfrags.remove(fragsLoop.get(0));
				mynodes.remove(pn);
				
				myfrags.add(pf);
				
				return true;
			}
			else
			{		
				//merge fragments into sequence
				PathFragment pf = new PathFragmentSequence(fragsIn.get(0), fragsOut.get(0));
				myfrags.remove(fragsIn.get(0));
				myfrags.remove(fragsOut.get(0));
				mynodes.remove(pn);
				
				myfrags.add(pf);
				
				return true;
			}
		}
		
		return false;
	}

	
	public int findParallel()
	{
		for (PathFragment pf : new Vector<PathFragment>(myfrags))
		{
			if (findParallelForFrag(pf))
			{
				return 1;
			}
		}
		return 0;
	}
	
	public boolean findParallelForFrag(PathFragment pf)
	{
		for (PathFragment pf2 : new Vector<PathFragment>(myfrags))
		{
			if (pf != pf2)
			{
				if (pf.getSource() == pf2.getSource() && pf.getTarget() == pf2.getTarget())
				{
					//merge
					PathFragment pfres = new PathFragmentParallel(pf, pf2);
					myfrags.remove(pf);
					myfrags.remove(pf2);
					myfrags.add(pfres);
					
					return true;
				}
			}
		}
		
		return false;
	}
	
	public Vector<PetrinetNode> getMaxSortedNodes()
	{
		Vector<PetrinetNode> lst = new Vector<PetrinetNode>();
		Vector<PathFragmentSimple> fs = getSimpleFragments();
		for (PathFragmentSimple pfs : fs)
		{
			if (pfs.getTarget() instanceof Transition)
			{
				lst.add(pfs.getTarget());
			}
		}
		
		return lst;
	}
	
	public Vector<PathFragmentSimple> getSimpleFragments()
	{
		Vector<PathFragmentSimple> fs = new Vector<PathFragmentSimple>();
		for (PathFragment pf : myfrags)
		{
			pf.getSimpleFragments(fs);
		}
		return fs;
	}
	
}
