package org.processmining.processprofiler3d.netstructure;

import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.processprofiler3d.compviz.INetDataSource;

public class PetriNetStructureUtil 
{
	public INetDataSource pn = null;
	public Place start = null;
	
	public PetriNetStructureUtil(INetDataSource pn)
	{
		this.pn = pn;
	}

	//get maximum path distance right now, not optimal
	private int getMinDist(PetrinetNode o1, PetrinetNode o2)
	{
		int dist = Integer.MAX_VALUE;
		
		Vector<ArcPath> lst = new Vector<ArcPath>();
		for (Arc a : pn.getOutEdges(o1))
		{
			ArcPath ap = new ArcPath();
			ap.addArc(a);
			lst.add(ap);
		}

		for (ArcPath ap : lst)
		{
			if (ap.getDest() == o2)
			{
				dist = 1;
				return dist;
			}
		}
		
		int i = 1;
		while (!lst.isEmpty())
		{
			Vector<ArcPath> successors = new Vector<ArcPath>();
			for (ArcPath pnode : lst)
			{
				successors.addAll(getSuccessorPaths(pnode));
			}
			
			lst = successors;
			i++;
			
			for (ArcPath ap : lst)
			{
				if (ap.getDest() == o2)
				{
					dist = i;
					break;
				}
			}
		}
		
		return dist;
	}
	
	//get maximum path distance right now, not optimal
	private int getMaxDist(PetrinetNode o1, PetrinetNode o2)
	{
		int dist = Integer.MAX_VALUE;
		
		Vector<ArcPath> lst = new Vector<ArcPath>();
		for (Arc a : pn.getOutEdges(o1))
		{
			ArcPath ap = new ArcPath();
			ap.addArc(a);
			lst.add(ap);
		}

		for (ArcPath ap : lst)
		{
			if (ap.getDest() == o2)
			{
				dist = 1;
			}
		}
		
		int i = 1;
		while (!lst.isEmpty())
		{
			Vector<ArcPath> successors = new Vector<ArcPath>();
			for (ArcPath pnode : lst)
			{
				successors.addAll(getSuccessorPaths(pnode));
			}
			
			lst = successors;
			i++;
			
			for (ArcPath ap : lst)
			{
				if (ap.getDest() == o2)
				{
					dist = i;
				}
			}
		}
		
		return dist;
	}
	
	private Vector<PetrinetNode> getSuccessors(PetrinetNode t1)
	{
		Vector<PetrinetNode> lst = new Vector<PetrinetNode>();
		
		for (PetrinetEdge<?, ?> e : pn.getOutEdges(t1))
		{
			lst.add(e.getTarget());
		}
		
		return lst;
	}
	
	private Vector<ArcPath> getSuccessorPaths(ArcPath p)
	{
		Vector<ArcPath> lst = new Vector<ArcPath>();
		
		for (Arc a : pn.getOutEdges(p.getDest()))
		{
			if (!p.containsArc(a))
			{
				ArcPath np = new ArcPath(p);
				np.addArc(a);
				lst.add(np);
			}
		}
		
		return lst;
	}
	
	public ArcPath getMinDistPath(PetrinetNode o1, PetrinetNode o2)
	{
		Vector<ArcPath> lst = new Vector<ArcPath>();
		for (Arc a : pn.getOutEdges(o1))
		{
			ArcPath ap = new ArcPath();
			ap.addArc(a);
			lst.add(ap);
		}

		for (ArcPath ap : lst)
		{
			if (ap.getDest() == o2)
			{
				return ap;
			}
		}
		
		while (!lst.isEmpty())
		{
			Vector<ArcPath> successors = new Vector<ArcPath>();
			for (ArcPath pnode : lst)
			{
				successors.addAll(getSuccessorPaths(pnode));
			}
			
			lst = successors;
			
			for (ArcPath ap : lst)
			{
				if (ap.getDest() == o2)
				{
					return ap;
				}
			}
		}
		
		return null;
	}
	
	public ArcPath getMaxDistPath(PetrinetNode o1, PetrinetNode o2)
	{
		ArcPath mPath = null;
		
		Vector<ArcPath> lst = new Vector<ArcPath>();
		for (Arc a : pn.getOutEdges(o1))
		{
			ArcPath ap = new ArcPath();
			ap.addArc(a);
			lst.add(ap);
		}

		for (ArcPath ap : lst)
		{
			if (ap.getDest() == o2)
			{
				mPath = ap;
			}
		}
		
		while (!lst.isEmpty())
		{
			Vector<ArcPath> successors = new Vector<ArcPath>();
			for (ArcPath pnode : lst)
			{
				successors.addAll(getSuccessorPaths(pnode));
			}
			
			lst = successors;
			
			for (ArcPath ap : lst)
			{
				if (ap.getDest() == o2)
				{
					mPath = ap;
				}
			}
		}
		
		return mPath;
	}
	
	public Vector<ArcPath> getAllPaths(PetrinetNode o1, PetrinetNode o2)
	{
		Vector<ArcPath> mPaths = new Vector<ArcPath>();
		
		Vector<ArcPath> lst = new Vector<ArcPath>();
		for (Arc a : pn.getOutEdges(o1))
		{
			ArcPath ap = new ArcPath();
			ap.addArc(a);
			lst.add(ap);
		}

		for (ArcPath ap : lst)
		{
			if (ap.getDest() == o2)
			{
				mPaths.add(ap);
			}
		}
		
		while (!lst.isEmpty())
		{
			Vector<ArcPath> successors = new Vector<ArcPath>();
			for (ArcPath pnode : lst)
			{
				successors.addAll(getSuccessorPaths(pnode));
			}
			
			lst = successors;
			
			for (ArcPath ap : lst)
			{
				if (ap.getDest() == o2)
				{
					mPaths.add(ap);
				}
			}
		}
		
		return mPaths;
	}
	
	public ArcPathCollection getAllPathsForNet()
	{
		Place pstart = pn.getStartPlace();
		Place pend = pn.getEndPlace();
		
		Vector<ArcPath> allPaths = getAllPaths(pstart, pend);
		
		return new ArcPathCollection(allPaths);
	}
	
	public PathFragmentCollection getPathFragmentCollection()
	{
		Vector<PetrinetNode> nodes = new Vector<PetrinetNode>();
		Vector<Arc> arcs = new Vector<Arc>();
		Vector<PathFragment> frags = new Vector<PathFragment>();
		nodes.addAll(pn.getTransitions());
		nodes.addAll(pn.getPlaces());
		
		for (PetrinetNode pnode : nodes)
		{
			for (Arc a : pn.getOutEdges(pnode))
			{
				if (!arcs.contains(a))
				{
					arcs.add(a);
				}
			}
		}
		
		for (Arc a : arcs)
		{
			frags.add(new PathFragmentSimple(a));
		}
		
		PathFragmentCollection pfc = new PathFragmentCollection(nodes, frags);
		
		return pfc;
	}
}
