package org.processmining.processprofiler3d.placealignment;

import java.util.Date;
import java.util.HashMap;
import java.util.Vector;

import org.apache.commons.lang3.tuple.Pair;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.psen.AlignedLog;
import org.processmining.processprofiler3d.psen.PetriNetIndexed;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohort;

public class PlaceAlignmentInfo
{
	public Place myPlace = null;
	
	public HashMap<String, PlaceAlignmentInfoDetail> detailPais = new HashMap<String, PlaceAlignmentInfoDetail>();
	public Vector<String> detailStrs = new Vector<String>();
	
	public int numberOfCases = 0;

	public int numberOfActivatedCases = 0;
	public int numberOfActivatedCasesPStaysPos = 0;
	public int numberOfActivatedCasesPGoesNeg = 0;
	
	public int numberOfCasesEndPZero = 0;
	public int numberOfCasesEndPPos = 0;
	public int numberOfCasesEndPNeg = 0;
	
	public int numberOfActivatedCasesEndPZero = 0;
	public int numberOfActivatedCasesEndPPos = 0;
	public int numberOfActivatedCasesEndPNeg = 0;
	
	public int numOrderChecks = 0;
	public double orderProb = 0;
	public double avgOrderProb = 0;
	
	public PlaceAlignmentInfo(Place p)
	{
		myPlace = p;
	}
	
	public int getSupport()
	{
		int pairNum = 0;
		
		for (PlaceAlignmentInfoDetail paid : detailPais.values())
		{
			pairNum += paid.getSupport();
		}
		
		return pairNum;
	}
	
	public float getConfidence()
	{
		int pairNum = 0;
		int pairZeroNum = 0;
		
		for (PlaceAlignmentInfoDetail paid : detailPais.values())
		{
			pairNum += paid.getSupport();
			pairZeroNum += paid.pairZeroNum;
		}
		
		if (pairNum > 0)
		{
			return (float)pairZeroNum / (float)pairNum;
		}
		return 0;
	}
	
	public void Calculate(XLog log, Marking initMarking, Marking finalMarking, INetDataSource net, PetriNetIndexed pidx)
	{
		if (initMarking != null && finalMarking != null)
		{
			PlaceAlignmentVisitor pav = new PlaceAlignmentVisitor(myPlace, net, pidx, initMarking, finalMarking);
			
			for (XTrace trc : log)
			{
				pav.ProcessTrace(trc);
				
				Score(pav.activations, pav.tokens, pav.bWentNeg);
	
				ProcessPairs(pidx, pav.evtPairs);
			}
		}
	}
	
	public void Calculate(XLog log, Marking initMarking, Marking finalMarking, INetDataSource net, PetriNetIndexed pidx, ClassificationCohort cohort)
	{
		if (initMarking != null && finalMarking != null)
		{
			PlaceAlignmentVisitor pav = new PlaceAlignmentVisitor(myPlace, net, pidx, initMarking, finalMarking);
			
			for (XTrace trc : log)
			{
				if (!cohort.isPartOfCohort(trc))
				{
					continue;
				}
				
				pav.ProcessTrace(trc);
				
				Score(pav.activations, pav.tokens, pav.bWentNeg);
				
				ProcessPairs(pidx, pav.evtPairs);
			}
		}
	}

	private void ProcessPairs(PetriNetIndexed pidx, Vector<Pair<XEvent, XEvent>> evtPairs)
	{
		for (Pair<XEvent,XEvent> pair : evtPairs)
		{
			double prob = 1;
			
			Transition leftT = null;
			Transition rightT = null;
			String leftName = "Null";
			String rightName = "Null";
			
			if (pair.getLeft() != null)
			{
				String leftId = AlignedLog.getLocalNodeId(pair.getLeft());
				if (leftId != null)
				{
					if (leftId.equals("-1"))
					{
						leftName = XConceptExtension.instance().extractName(pair.getLeft());
					}
					else
					{
						leftT = pidx.getTransition(leftId);
						leftName = leftT.getLabel();
					}
				}
			}
			if (pair.getRight() != null)
			{
				String rightId = AlignedLog.getLocalNodeId(pair.getRight());
				if (rightId != null)
				{
					if (rightId.equals("-1"))
					{
						leftName = XConceptExtension.instance().extractName(pair.getRight());
					}
					else
					{
						rightT = pidx.getTransition(rightId);
						rightName = rightT.getLabel();
					}
				}
			}
			String pairName = leftName + " - " + rightName;
			detailStrs.add(pairName);
			
			if (!detailPais.containsKey(pairName))
			{
				PlaceAlignmentInfoDetail pad = new PlaceAlignmentInfoDetail(myPlace, leftT, rightT);
				
				pad.addPair(pair.getLeft(), pair.getRight());
				
				detailPais.put(pairName, pad);
			}
			else
			{
				PlaceAlignmentInfoDetail pad = detailPais.get(pairName);
				pad.addPair(pair.getLeft(), pair.getRight());
			}	
			
			if (pair.getLeft() != null && pair.getRight() != null)
			{
				prob = calcIncorrectOrderProb(pair.getLeft(), pair.getRight());
				orderProb += prob;
				numOrderChecks++;
				avgOrderProb = orderProb / (double)numOrderChecks;
			}
		}
	}

	private void Score(int activations, int tokens, boolean bWentNeg)
	{
		numberOfCases++;
		
		if (tokens == 0)
		{
			numberOfCasesEndPZero++;
		}
		else if (tokens > 0)
		{
			numberOfCasesEndPPos++;
		}
		else
		{
			numberOfCasesEndPNeg++;
		}
		
		if (activations > 0)
		{
			numberOfActivatedCases++;
			if (bWentNeg)
			{
				numberOfActivatedCasesPGoesNeg++;
			}
			else
			{
				numberOfActivatedCasesPStaysPos++;
			}
			
			if (tokens == 0)
			{
				numberOfActivatedCasesEndPZero++;
			}
			else if (tokens > 0)
			{
				numberOfActivatedCasesEndPPos++;
			}
			else
			{
				numberOfActivatedCasesEndPNeg++;
			}
		}
	}
	
	public static double calcIncorrectOrderProb(XEvent evtA, XEvent evtB)
	{
		String lifecycleTrans = XLifecycleExtension.instance().extractTransition(evtA);
		Date evtATime = XTimeExtension.instance().extractTimestamp(evtA);
		Date evtBTime = XTimeExtension.instance().extractTimestamp(evtB);
		
		if (evtATime == null || evtBTime == null)
		{
			return 0.5;
		}
		
		//long deviationA = 1000 * 60 * 60; //1 hour
		//long deviationB = 1000 * 60 * 60;
		long deviationA = 1000 * 60 * 10; //10 minutes
		long deviationB = 1000 * 60 * 10;
		double aMin = evtATime.getTime() - deviationA;
		double aMax = evtATime.getTime() + deviationA;
		double bMin = evtBTime.getTime() - deviationB;
		double bMax = evtBTime.getTime() + deviationB;
		
		double spanA = aMax - aMin;
		double spanB = bMax - bMin;
		double spanOverlap = 0;
		double spanAAfterB = 0;
		double spanBAfterA = 0;
		
		if (aMax > bMin && aMin < bMax)
		{
			spanOverlap = Math.min(aMax, bMax) - Math.max(aMin, bMin);
		}
		
		if (aMax > bMax)
		{
			spanAAfterB = aMax - Math.max(aMin, bMax);
		}
		if (bMax > aMax)
		{
			spanBAfterA = bMax - Math.max(bMin, aMax);
		}
		
		if (spanA == 0 || spanB == 0)
		{
			if (spanA == 0 && spanB == 0)
			{
				if (aMin < bMin)
				{
					return 0;
				}
				else if (aMin == bMin)
				{
					return 0.5;
				}
				else
				{
					return 1;
				}
			}
			else if (spanA == 0)
			{
				if (aMin < bMin)
				{
					return 0;
				}
				else if (aMin < bMax)
				{
					//what to do?
					return (spanB - spanBAfterA) / spanB;
				}
				else
				{
					return 1;
				}
			}
			else if (spanB == 0)
			{
				if (bMin < aMin)
				{
					return 1;
				}
				else if (bMin < aMax)
				{
					//what to do?
					return spanAAfterB / spanA;
				}
				else
				{
					return 0;
				}
			}
		}
		else
		{
			
			double factorA = spanOverlap * spanOverlap * 0.5;
			double factorB = spanAAfterB * spanB;
			
			double divFactor = spanA * spanB;
			
			return (factorA + factorB) / divFactor;
		}
		
		return 0;
	}
	
	// :)
	public PlaceAlignmentInfoDetail getPAID(String tpairId)
	{
		for (PlaceAlignmentInfoDetail paid : detailPais.values())
		{
			String paidId = paid.getTransitionPairId();
			if (paidId != null && paidId.equals(tpairId))
			{
				return paid;
			}
		}
		
		return null;
	}

}
