package org.processmining.processprofiler3d.placealignment;

import org.deckfour.xes.model.XEvent;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.processprofiler3d.psen.AlignedLog;

public class PlaceAlignmentInfoDetail
{
	public Place myPlace = null;
	public Transition myStart = null;
	public Transition myEnd = null;
	
	public int numOrderChecks = 0;
	public double orderProb = 0;
	public double avgOrderProb = 0;
	
	public int pairNum = 0;
	public int pairZeroNum = 0;
	public int pairNegNum = 0;
	public int pairPosNum = 0;
	
	private boolean bInit = false;
	private boolean bFinish = false; 
	
	public PlaceAlignmentInfoDetail(Place p, Transition sid, Transition eid)
	{
		myPlace = p;
		myStart = sid;
		myEnd = eid;
	}
	
	public int getSupport()
	{
		return pairNum;
	}
	
	public float getConfidence()
	{
		if (pairNum > 0)
		{
			return (float)pairZeroNum / (float)pairNum;
		}
		return 0;
	}
	
	public String getLabel()
	{
		String startLabel = "Null";
		if (myStart != null)
		{
			startLabel = myStart.getLabel();
		}
		if (bInit)
		{
			startLabel = "InitialMarking";
		}
		
		String endLabel = "Null";
		if (myEnd != null)
		{
			endLabel = myEnd.getLabel();
		}
		if (bFinish)
		{
			endLabel = "FinalMarking";
		}
		
		return startLabel + " - " + endLabel;
	}
	
	public String getTransitionPairId()
	{
		if (myStart != null && myEnd != null)
		{
			return myStart.getLocalID().toString() + myEnd.getLocalID().toString();
		}
		return null;
	}
	
	public void addPair(XEvent left, XEvent right)
	{
		XEvent l1 = null;
		XEvent r1 = null;
		
		int initMarking = 0;
		int finalMarking = 0;
		
		if (left != null)
		{
			//if left event is aligned
			if (AlignedLog.getStepType(left) == StepTypes.LMGOOD || AlignedLog.getStepType(left) == StepTypes.MINVI)
			{
				l1 = left;
			}
			
			//if left event is initial marking
			String leftId = AlignedLog.getLocalNodeId(left);
			if (leftId != null && leftId.equals("-1"))
			{
				if (AlignedLog.getStepType(left) == StepTypes.MINVI)
				{
					initMarking = 1;
				}
				l1 = null;
				bInit = true;
			}
		}
		if (right != null)
		{
			//if right event is aligned
			if (AlignedLog.getStepType(right) == StepTypes.LMGOOD || AlignedLog.getStepType(right) == StepTypes.MINVI)
			{
				r1 = right;
			}

			//if right event is final marking
			String rightId = AlignedLog.getLocalNodeId(right);
			if (rightId != null && rightId.equals("-1"))
			{
				if (AlignedLog.getStepType(right) == StepTypes.MINVI)
				{
					finalMarking = 1;
				}
				r1 = null;
				bFinish = true;
			}
		}
		
		
		pairNum++;
		
		int tokens = initMarking;
		if (l1 != null)
		{
			tokens++;
		}
		if (r1 != null)
		{
			tokens--;
		}
		tokens = tokens - finalMarking;
		
		//if both events are unaligned, stop here
		if (l1 == null && r1 == null)
		{
			return;
		}

		if (tokens == 0)
		{
			pairZeroNum++;
			
			if (l1 != null && r1 != null)
			{
				double prob = PlaceAlignmentInfo.calcIncorrectOrderProb(l1, r1);
				
				orderProb += prob;
				numOrderChecks++;
				avgOrderProb = orderProb / (double)numOrderChecks;
			}
		}
		else if (tokens > 0)
		{
			pairPosNum++;
		}
		else if (tokens < 0)
		{
			pairNegNum++;
		}
		
	}
}
