package org.processmining.processprofiler3d.placealignment;

import java.util.Vector;

import javax.swing.table.AbstractTableModel;

public class PlaceAlignmentTableModel extends AbstractTableModel implements IPlaceAlignmentTableModel
{
    private String[] columnNames = {"Place",
            "Activated #",
            "Activated P+ #",
            "Activated P- #",
            "End P0 #",
            "End P+ #",
            "End P- #"};
    
    private Vector<PlaceAlignmentInfo> data = null;

    public PlaceAlignmentTableModel(Vector<PlaceAlignmentInfo> lst)
    {
    	data = lst;
    }
    
    public int getNumCases()
    {
    	if (data.isEmpty())
    	{
    		return 0;
    	}
    	return data.firstElement().numberOfCases;
    }
    
    public int getColumnCount() 
    {
        return columnNames.length;
    }

    public int getRowCount() 
    {
        return data.size();
    }

    public String getColumnName(int col) 
    {
        return columnNames[col];
    }

    public Object getValueAt(int row, int col) 
    {
    	PlaceAlignmentInfo pai = data.get(row);
    	
    	if (col == 0)
    	{
    		return pai.myPlace.getLabel();
    	}
    	else if (col == 1)
    	{
    		return pai.numberOfActivatedCases;
    	}
    	else if (col == 2)
    	{
    		return pai.numberOfActivatedCasesPStaysPos;
    	}
    	else if (col == 3)
    	{
    		return pai.numberOfActivatedCasesPGoesNeg;
    	}
    	else if (col == 4)
    	{
    		return pai.numberOfActivatedCasesEndPZero;
    	}
    	else if (col == 5)
    	{
    		return pai.numberOfActivatedCasesEndPPos;
    	}
    	else if (col == 6)
    	{
    		return pai.numberOfActivatedCasesEndPNeg;
    	}
		
        return null;
    }

    public float getCellStatus(int row, int col) 
    {
    	PlaceAlignmentInfo pai = data.get(row);
    	
    	if (col == 0)
    	{
    		return 0;
    	}
    	else if (col == 1)
    	{
    		return 0;
    	}
    	else if (col == 2)
    	{
    		float perc = (float)pai.numberOfActivatedCasesPStaysPos / (float)pai.numberOfActivatedCases;
//    		if (perc == 1.0f)
//    		{
//    			return 0;
//    		}
//    		else if (perc > 0.0f)
//    		{
//    			return 1;
//    		}
//    		else
//    		{
//    			return 2;
//    		}
    		return (1.0f-perc) * 2.0f;
    	}
    	else if (col == 3)
    	{
    		float perc = (float)pai.numberOfActivatedCasesPGoesNeg / (float)pai.numberOfActivatedCases;
//    		if (perc == 0.0f)
//    		{
//    			return 0;
//    		}
//    		else if (perc < 1.0f)
//    		{
//    			return 1;
//    		}
//    		else
//    		{
//    			return 2;
//    		}
    		return perc * 2.0f;
    	}
    	else if (col == 4)
    	{
    		float perc = (float)pai.numberOfActivatedCasesEndPZero / (float)pai.numberOfActivatedCases;
//    		if (perc == 1.0f)
//    		{
//    			return 0;
//    		}
//    		else if (perc > 0.0f)
//    		{
//    			return 1;
//    		}
//    		else
//    		{
//    			return 2;
//    		}
    		return (1.0f-perc) * 2.0f;
    	}
    	else if (col == 5)
    	{
    		float perc = (float)pai.numberOfActivatedCasesEndPPos / (float)pai.numberOfActivatedCases;
//    		if (perc == 0.0f)
//    		{
//    			return 0;
//    		}
//    		else if (perc < 1.0f)
//    		{
//    			return 1;
//    		}
//    		else
//    		{
//    			return 2;
//    		}
    		return perc * 2.0f;
    	}
    	else if (col == 6)
    	{
    		float perc = (float)pai.numberOfActivatedCasesEndPNeg / (float)pai.numberOfActivatedCases;
//    		if (perc == 0.0f)
//    		{
//    			return 0;
//    		}
//    		else if (perc < 1.0f)
//    		{
//    			return 1;
//    		}
//    		else
//    		{
//    			return 2;
//    		}
    		return perc * 2.0f;
    	}
		
        return 0;
    }

    public Class getColumnClass(int c) 
    {
        return getValueAt(0, c).getClass();
    }

    /*
     * Don't need to implement this method unless your table's
     * editable.
     */
//    public boolean isCellEditable(int row, int col) {
//        //Note that the data/cell address is constant,
//        //no matter where the cell appears onscreen.
//        if (col < 2) {
//            return false;
//        } else {
//            return true;
//        }
//    }

    /*
     * Don't need to implement this method unless your table's
     * data can change.
     */
//    public void setValueAt(Object value, int row, int col) {
//        data[row][col] = value;
//        fireTableCellUpdated(row, col);
//    }
//    ...
}
