package org.processmining.processprofiler3d.placealignment;

import java.util.HashSet;
import java.util.Vector;

import org.apache.commons.lang3.tuple.MutablePair;
import org.apache.commons.lang3.tuple.Pair;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryNaiveImpl;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.psen.AlignedLog;
import org.processmining.processprofiler3d.psen.PetriNetIndexed;

public class PlaceAlignmentVisitor
{
	public int activations = 0;
	private int initTokens = 0;
	private int finalTokens = 0;
	public int tokens = 0;
	public boolean bWentNeg = false;
	
	private HashSet<String> producers = new HashSet<String>();
	private HashSet<String> consumers = new HashSet<String>();

	private Vector<XEvent> evtProducers = new Vector<XEvent>();
	//Vector<XEvent> evtConsumers = new Vector<XEvent>();
	public Vector<Pair<XEvent, XEvent>> evtPairs = new Vector<Pair<XEvent, XEvent>>();
	
	public PlaceAlignmentVisitor(Place myPlace, INetDataSource net, PetriNetIndexed pidx, Marking initMarking, Marking finalMarking)
	{
		//get in edges
		for (Arc a : net.getInEdges(myPlace))
		{
			producers.add(a.getSource().getLocalID().toString());
		}
		
		//get out edges
		for (Arc a : net.getOutEdges(myPlace))
		{
			consumers.add(a.getTarget().getLocalID().toString());
		}
		
		initTokens = initMarking.occurrences(myPlace);
		finalTokens = finalMarking.occurrences(myPlace);
	}
	
	public void ProcessTrace(XTrace trc)
	{
		activations = 0;
		tokens = initTokens;
		bWentNeg = false;
		
		evtProducers = new Vector<XEvent>();
		//evtConsumers = new Vector<XEvent>();
		evtPairs = new Vector<Pair<XEvent, XEvent>>();
		
		int evtNum = 0;
		for (XEvent evt : trc)
		{
			XLifecycleExtension.StandardModel lifecycleTrans = XLifecycleExtension.instance().extractStandardTransition(evt);
			
			if (lifecycleTrans != XLifecycleExtension.StandardModel.COMPLETE)
			{
				continue;
			}
			
			ProcessEvent(evtNum, evt);
			evtNum++;
		}
		
		//process left-over producer events
		int finalCount = 0;
		while (evtProducers.size() > 0)
		{
			XEvent prodEvt = evtProducers.firstElement();
			evtProducers.remove(0);
			
			if (consumers.isEmpty())
			{
				evtPairs.add(new MutablePair<>(prodEvt, CreateFinalMarkingEvent(finalCount == 0)));
			}
			else
			{
				evtPairs.add(new MutablePair<>(prodEvt, (XEvent)null));
			}
			finalCount++;
		}
		
		tokens -= finalTokens;
	}

	private void ProcessEvent(int evtNum, XEvent evt)
	{
		String lid = AlignedLog.getLocalNodeId(evt);
		StepTypes stype = AlignedLog.getStepType(evt);
		
		if (stype == StepTypes.LMGOOD || stype == StepTypes.MINVI)
		{
			
			if (consumers.contains(lid))
			{
				tokens--;
				activations++;
				
				if (tokens < 0)
				{
					bWentNeg = true;
				}

				//if (!consumers.isEmpty() && !producers.isEmpty())
				{
					//evtConsumers.add(evt);
					if (evtProducers.isEmpty())
					{
						if (producers.isEmpty())
						{
							evtPairs.add(new MutablePair<>(CreateInitMarkingEvent(evtNum == 0), evt));
						}
						else
						{
							evtPairs.add(new MutablePair<>((XEvent)null, evt));
						}
						
					}
					else
					{
						XEvent prodEvt = evtProducers.firstElement();
						evtProducers.remove(0);
						evtPairs.add(new MutablePair<>(prodEvt, evt));
						
					}
				}
			}
			
			if (producers.contains(lid))
			{
				tokens++;
				activations++;

				//if (!consumers.isEmpty() && !producers.isEmpty())
				{
					evtProducers.add(evt);
				}
			}
		}
		else if (stype == StepTypes.L)
		{
			if (producers.contains(lid))
			{
				evtPairs.add(new MutablePair<>(evt, (XEvent)null));
			}
			if (consumers.contains(lid))
			{
				evtPairs.add(new MutablePair<>((XEvent)null, evt));
			}
		}
		else if (stype == StepTypes.MREAL)
		{
			//is not a start event or end event
			//if (!consumers.isEmpty() && !producers.isEmpty())
			{
				if (consumers.contains(lid))
				{
					//evtConsumers.add(evt);
					if (evtProducers.isEmpty())
					{
						if (producers.isEmpty())
						{
							evtPairs.add(new MutablePair<>(CreateInitMarkingEvent(evtNum == 0), evt));
						}
						else
						{
							evtPairs.add(new MutablePair<>((XEvent)null, evt));
							
						}
					}
					else
					{
						XEvent prodEvt = evtProducers.firstElement();
						evtProducers.remove(0);
						evtPairs.add(new MutablePair<>(prodEvt, evt));
					}
				}
				
				if (producers.contains(lid))
				{
					evtProducers.add(evt);
				}
			}
			
		}
	}

	private XEvent CreateInitMarkingEvent(boolean bAligned)
	{
		XEvent initEvt;
		XFactory xfactor = new XFactoryNaiveImpl();
		initEvt = xfactor.createEvent();
		
		XConceptExtension.instance().assignName(initEvt, "$InitialMarking$");

		XAttribute att = null;
		if (bAligned)
		{
			att = xfactor.createAttributeLiteral(AlignedLog.ATT_NAME_STEP_TYPE, StepTypes.MINVI.toString(), null);
		}
		else
		{
			att = xfactor.createAttributeLiteral(AlignedLog.ATT_NAME_STEP_TYPE, StepTypes.MREAL.toString(), null);
		}
		initEvt.getAttributes().putIfAbsent(AlignedLog.ATT_NAME_STEP_TYPE, att);
		
		XAttribute att2 = xfactor.createAttributeLiteral(AlignedLog.ATT_NAME_NODE_ID, "-1", null);
		initEvt.getAttributes().putIfAbsent(AlignedLog.ATT_NAME_NODE_ID, att2);
		
		return initEvt;
	}

	private XEvent CreateFinalMarkingEvent(boolean bAligned)
	{
		XEvent finalEvt;
		XFactory xfactor = new XFactoryNaiveImpl();
		finalEvt = xfactor.createEvent();
		
		XConceptExtension.instance().assignName(finalEvt, "$FinalMarking$");
		
		XAttribute att = null;
		if (bAligned)
		{
			att = xfactor.createAttributeLiteral(AlignedLog.ATT_NAME_STEP_TYPE, StepTypes.MINVI.toString(), null);
		}
		else
		{
			att = xfactor.createAttributeLiteral(AlignedLog.ATT_NAME_STEP_TYPE, StepTypes.MREAL.toString(), null);
		}
		finalEvt.getAttributes().putIfAbsent(AlignedLog.ATT_NAME_STEP_TYPE, att);
		
		XAttribute att2 = xfactor.createAttributeLiteral(AlignedLog.ATT_NAME_NODE_ID, "-1", null);
		finalEvt.getAttributes().putIfAbsent(AlignedLog.ATT_NAME_NODE_ID, att2);
		
		return finalEvt;
	}

}
