package org.processmining.processprofiler3d.psen;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;

public class LogAnalyser 
{
	private XLog log;
	
	public LogAnalyser(XLog l)
	{
		log = l;
	}

	public boolean areTraceNamesUnique()
	{
		HashSet<String> lst = new HashSet<String>();
		for (XTrace trc : log)
		{
			String val = XConceptExtension.instance().extractName(trc);
			if (val != null)
			{
				if (lst.contains(val))
				{
					return false;
				}
				else
				{
					lst.add(val);
				}
			}
		}
		
		return true;
	}
	
	public boolean HasLifecycleData()
	{
		if (HasGlobalTraceAttributes())
		{
			HashSet<String> lst = getAvailableEventAttrsLiteral();
		
			return lst.contains(XLifecycleExtension.KEY_TRANSITION);
		}
		else
		{
			HashSet<String> lst = calcAvailableEventAttrsLiteral();
		
			return lst.contains(XLifecycleExtension.KEY_TRANSITION);
		}
	}
	
	public int getLifecycleDataType()
	{
		if (!HasLifecycleData())
			return 0;
		
		HashSet<String> vals = getAvailableEventAttrValuesLiteral(XLifecycleExtension.KEY_TRANSITION);
		int code = 0;
		if (vals.contains("assign"))
		{
			code = code + 1;
		}
		if (vals.contains("schedule"))
		{
			code = code + 2;
		}
		if (vals.contains("start"))
		{
			code = code + 4;
		}
		if (vals.contains("complete"))
		{
			code = code + 8;
		}
		return code;
	}
	
	public HashMap<String, HashSet<XLifecycleExtension.StandardModel>> getLifecyclePerAlignedTransition()
	{
		HashMap<String, HashSet<XLifecycleExtension.StandardModel>> lst = new HashMap<String, HashSet<XLifecycleExtension.StandardModel>>();

		for (XTrace trace : log)
		{
			for (XEvent myEvent : trace)
			{
				String localID = AlignedLog.getLocalNodeId(myEvent);
				if (localID != null)
				{
					XLifecycleExtension.StandardModel lifecycleValue = XLifecycleExtension.instance().extractStandardTransition(myEvent);
					if (lifecycleValue != null)
					{
						if (!lst.containsKey(localID))
						{
							lst.put(localID, new HashSet<XLifecycleExtension.StandardModel>());
						}	
						if (!lst.get(localID).contains(lifecycleValue))
						{
							lst.get(localID).add(lifecycleValue);
						}
					}
				}
			}
		}
		
		return lst;
	}
	
	public HashMap<String, HashSet<XLifecycleExtension.StandardModel>> getLifecyclePerEventClass(TransEvClassMapping xmap)
	{
		HashMap<String, HashSet<XLifecycleExtension.StandardModel>> lst = new HashMap<String, HashSet<XLifecycleExtension.StandardModel>>();
		XEventClassifier xec = xmap.getEventClassifier();

		for (XTrace trace : log)
		{
			for (XEvent myEvent : trace)
			{
				String evclass = xec.getClassIdentity(myEvent);
				XLifecycleExtension.StandardModel lifecycleValue = XLifecycleExtension.instance().extractStandardTransition(myEvent);
				if (lifecycleValue != null)
				{
					if (!lst.containsKey(evclass))
					{
						lst.put(evclass, new HashSet<XLifecycleExtension.StandardModel>());
					}	
					if (!lst.get(evclass).contains(lifecycleValue))
					{
						lst.get(evclass).add(lifecycleValue);
					}
				}
			}
		}
		
		return lst;
	}
	
	public boolean HasGlobalTraceAttributes()
	{
		List<XAttribute> attrs = log.getGlobalTraceAttributes();
		return !attrs.isEmpty();
	}
	
	public boolean HasGlobalEventAttributes()
	{
		List<XAttribute> attrs = log.getGlobalEventAttributes();
		return !attrs.isEmpty();
	}
	
	public HashSet<String> getAvailableTraceAttrsLiteral()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		if (HasGlobalTraceAttributes())
		{
			List<XAttribute> attrs = log.getGlobalTraceAttributes();
			for (XAttribute xatt : attrs)
			{
				//System.out.println("XattClass: " + xatt.getClass());
				if (xatt instanceof XAttributeLiteral)
				{
					availableAttrs.add(xatt.getKey());
				}
			}
		}
		else
		{
			availableAttrs = calcAvailableTraceAttrsLiteral();
		}
		return availableAttrs;
	}
	
	public HashSet<String> calcAvailableTraceAttrsLiteral()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		for (XTrace trace : log)
		{
			for (String s : trace.getAttributes().keySet())
			{
				XAttribute xatt = trace.getAttributes().get(s);
				//System.out.println("XattClass: " + xatt.getClass());
				if (xatt instanceof XAttributeLiteral)
				{
					if (!availableAttrs.contains(s))
					{
						availableAttrs.add(s);
					}
				}
			}
		}
		return availableAttrs;
	}
	
	public boolean isTraceAttrLiteral(String aName)
	{
		return getAvailableTraceAttrsLiteral().contains(aName);
	}
	
	public boolean isTraceAttrTimestamp(String aName)
	{
		return getAvailableTraceAttrsTimestamp().contains(aName);
	}
	
	public boolean isTraceAttrDiscrete(String aName)
	{
		return getAvailableTraceAttrsDiscrete().contains(aName);
	}
	
	public boolean isTraceAttrContinuous(String aName)
	{
		return getAvailableTraceAttrsContinuous().contains(aName);
	}
	
	public boolean isTraceAttrBoolean(String aName)
	{
		return getAvailableTraceAttrsBoolean().contains(aName);
	}
	
	public HashSet<String> calcAvailableTraceAttrs()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		for (XTrace trace : log)
		{
			for (String s : trace.getAttributes().keySet())
			{
				if (!availableAttrs.contains(s))
				{
					availableAttrs.add(s);
				}
			}
		}
		return availableAttrs;
	}
	
	public HashSet<String> getAvailableTraceAttrsTimestamp()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		if (HasGlobalTraceAttributes())
		{
			List<XAttribute> attrs = log.getGlobalTraceAttributes();
			for (XAttribute xatt : attrs)
			{
				//System.out.println("XattClass: " + xatt.getClass());
				if (xatt instanceof XAttributeTimestamp)
				{
					availableAttrs.add(xatt.getKey());
				}
			}
		}
		else
		{
			availableAttrs = calcAvailableTraceAttrsTimestamp();
		}
		return availableAttrs;
	}
	
	public HashSet<String> calcAvailableTraceAttrsTimestamp()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		for (XTrace trace : log)
		{
			for (String s : trace.getAttributes().keySet())
			{
				XAttribute xatt = trace.getAttributes().get(s);
				//System.out.println("XattClass: " + xatt.getClass());
				if (xatt instanceof XAttributeTimestamp)
				{
					if (!availableAttrs.contains(s))
					{
						availableAttrs.add(s);
					}
				}
			}
		}
		return availableAttrs;
	}
	
	public HashSet<String> getAvailableTraceAttrsDiscrete()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		if (HasGlobalTraceAttributes())
		{
			List<XAttribute> attrs = log.getGlobalTraceAttributes();
			for (XAttribute xatt : attrs)
			{
				//System.out.println("XattClass: " + xatt.getClass());
				if (xatt instanceof XAttributeDiscrete)
				{
					availableAttrs.add(xatt.getKey());
				}
			}
		}
		else
		{
			availableAttrs = calcAvailableTraceAttrsDiscrete();
		}
		return availableAttrs;
	}
	
	public HashSet<String> calcAvailableTraceAttrsDiscrete()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		for (XTrace trace : log)
		{
			for (String s : trace.getAttributes().keySet())
			{
				XAttribute xatt = trace.getAttributes().get(s);
				//System.out.println("XattClass: " + xatt.getClass());
				if (xatt instanceof XAttributeDiscrete)
				{
					if (!availableAttrs.contains(s))
					{
						availableAttrs.add(s);
					}
				}
			}
		}
		return availableAttrs;
	}
	
	public HashSet<String> getAvailableTraceAttrsContinuous()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		if (HasGlobalTraceAttributes())
		{
			List<XAttribute> attrs = log.getGlobalTraceAttributes();
			for (XAttribute xatt : attrs)
			{
				//System.out.println("XattClass: " + xatt.getClass());
				if (xatt instanceof XAttributeContinuous)
				{
					availableAttrs.add(xatt.getKey());
				}
			}
		}
		else
		{
			availableAttrs = calcAvailableTraceAttrsContinuous();
		}
		return availableAttrs;
	}
	
	public HashSet<String> calcAvailableTraceAttrsContinuous()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		for (XTrace trace : log)
		{
			for (String s : trace.getAttributes().keySet())
			{
				XAttribute xatt = trace.getAttributes().get(s);
				//System.out.println("XattClass: " + xatt.getClass());
				if (xatt instanceof XAttributeContinuous)
				{
					if (!availableAttrs.contains(s))
					{
						availableAttrs.add(s);
					}
				}
			}
		}
		return availableAttrs;
	}
	
	public HashSet<String> getAvailableTraceAttrsBoolean()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		if (HasGlobalTraceAttributes())
		{
			List<XAttribute> attrs = log.getGlobalTraceAttributes();
			for (XAttribute xatt : attrs)
			{
				//System.out.println("XattClass: " + xatt.getClass());
				if (xatt instanceof XAttributeBoolean)
				{
					availableAttrs.add(xatt.getKey());
				}
			}
		}
		else
		{
			availableAttrs = calcAvailableTraceAttrsBoolean();
		}
		return availableAttrs;
	}
	
	public HashSet<String> calcAvailableTraceAttrsBoolean()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		for (XTrace trace : log)
		{
			for (String s : trace.getAttributes().keySet())
			{
				XAttribute xatt = trace.getAttributes().get(s);
				//System.out.println("XattClass: " + xatt.getClass());
				if (xatt instanceof XAttributeBoolean)
				{
					if (!availableAttrs.contains(s))
					{
						availableAttrs.add(s);
					}
				}
			}
		}
		return availableAttrs;
	}
	
	public HashSet<String> getAvailableEventAttrsLiteral()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		List<XAttribute> attrs = log.getGlobalEventAttributes();
		for (XAttribute xatt : attrs)
		{
			//System.out.println("XattClass: " + xatt.getClass());
			if (xatt instanceof XAttributeLiteral)
			{
				availableAttrs.add(xatt.getKey());
			}
		}
		return availableAttrs;
	}
	
	public HashSet<String> calcAvailableEventAttrs()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		for (XTrace trace : log)
		{
			for (XEvent event : trace)
			{
				for (String s : event.getAttributes().keySet())
				{
					if (!availableAttrs.contains(s))
					{
						availableAttrs.add(s);
					}
				}
			}
		}
		return availableAttrs;
	}
	
	public HashSet<String> calcAvailableEventAttrsLiteral()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		for (XTrace trace : log)
		{
			for (XEvent event : trace)
			{
				for (String s : event.getAttributes().keySet())
				{
					XAttribute xatt = event.getAttributes().get(s);
					//System.out.println("XattClass: " + xatt.getClass());
					if (xatt instanceof XAttributeLiteral)
					{
						if (!availableAttrs.contains(s))
						{
							availableAttrs.add(s);
						}
					}
				}
			}
		}
		return availableAttrs;
	}
	
	public HashSet<String> getAvailableEventAttrsTimestamp()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		List<XAttribute> attrs = log.getGlobalEventAttributes();
		for (XAttribute xatt : attrs)
		{
			//System.out.println("XattClass: " + xatt.getClass());
			if (xatt instanceof XAttributeTimestamp)
			{
				availableAttrs.add(xatt.getKey());
			}
		}
		return availableAttrs;
	}
	
	public HashSet<String> calcAvailableEventAttrsTimestamp()
	{
		HashSet<String> availableAttrs = new HashSet<String>();
		for (XTrace trace : log)
		{
			for (XEvent event : trace)
			{
				for (String s : event.getAttributes().keySet())
				{
					XAttribute xatt = event.getAttributes().get(s);
					//System.out.println("XattClass: " + xatt.getClass());
					if (xatt instanceof XAttributeTimestamp)
					{
						if (!availableAttrs.contains(s))
						{
							availableAttrs.add(s);
						}
					}
				}
			}
		}
		return availableAttrs;
	}

	public HashSet<String> getAvailableTraceAttrValuesLiteral(String aName)
	{
		HashSet<String> availableValues = new HashSet<String>();
		for (XTrace myTrace : log) // in the event log, each element is a case
		{
			XAttribute myAttribute = myTrace.getAttributes().get(aName);
			if (myAttribute instanceof XAttributeLiteral)
			{
				XAttributeLiteral myAttributeL = (XAttributeLiteral)myAttribute;
				if (myAttributeL.getValue() != null)
				{
					String val = myAttributeL.getValue();
					if (!availableValues.contains(val))
					{
						availableValues.add(val);
					}
				}
			}
		}
		return availableValues;
	}
	
	public HashSet<Date> getAvailableTraceAttrValuesTimestamp(String aName)
	{
		HashSet<Date> availableValues = new HashSet<Date>();
		for (XTrace myTrace : log) // in the event log, each element is a case
		{
			XAttribute myAttribute = myTrace.getAttributes().get(aName);
			if (myAttribute instanceof XAttributeTimestamp)
			{
				XAttributeTimestamp myAttributeL = (XAttributeTimestamp)myAttribute;
				if (myAttributeL.getValue() != null)
				{
					Date val = myAttributeL.getValue();
					if (!availableValues.contains(val))
					{
						availableValues.add(val);
					}
				}
			}
		}
		return availableValues;
	}

	public HashSet<String> getAvailableEventAttrValuesLiteral(String aName)
	{
		HashSet<String> availableValues = new HashSet<String>();
		for (int i=0; i < log.size(); i++) // in the event log, each element is a case
		{
			for (int j=0; j < log.get(i).size(); j++) // in each case, each element is an event
			{
				XEvent myEvent = log.get(i).get(j);
				
				XAttribute myAttribute = myEvent.getAttributes().get(aName);
				if (myAttribute instanceof XAttributeLiteral)
				{
					XAttributeLiteral myAttributeL = (XAttributeLiteral)myAttribute;
					if (myAttributeL.getValue() != null)
					{
						String val = myAttributeL.getValue();
						if (!availableValues.contains(val))
						{
							availableValues.add(val);
						}
					}
				}
			}
		}
		return availableValues;
	}
	
	public HashSet<Date> getAvailableEventAttrValuesTimestamp(String aName)
	{
		HashSet<Date> availableValues = new HashSet<Date>();
		for (int i=0; i < log.size(); i++) // in the event log, each element is a case
		{
			for (int j=0; j < log.get(i).size(); j++) // in each case, each element is an event
			{
				XEvent myEvent = log.get(i).get(j);
				
				XAttribute myAttribute = myEvent.getAttributes().get(aName);
				if (myAttribute instanceof XAttributeTimestamp)
				{
					XAttributeTimestamp myAttributeL = (XAttributeTimestamp)myAttribute;
					if (myAttributeL.getValue() != null)
					{
						Date val = myAttributeL.getValue();
						if (!availableValues.contains(val))
						{
							availableValues.add(val);
						}
					}
				}
			}
		}
		return availableValues;
	}
	
	public String getEventLiteralAttribute(XEvent ev, String aName)
	{
		XAttribute myAttribute = ev.getAttributes().get(aName);
		if (myAttribute instanceof XAttributeLiteral)
		{
			XAttributeLiteral myAttributeL = (XAttributeLiteral)myAttribute;
			if (myAttributeL.getValue() != null)
			{
				String val = myAttributeL.getValue();
				return val;
			}
		}
		return null;
	}
	
	public <T> HashSet<T> getAvailableTraceAttrValues(Class<T> c, String aName)
	{
		HashSet<T> availableValues = new HashSet<T>();
		for (int i=0; i < log.size(); i++) // in the event log, each element is a case
		{
			XTrace myTrace = log.get(i);
			
			XAttribute myAttribute = myTrace.getAttributes().get(aName);
			if (myAttribute instanceof XAttributeLiteral)
			{
				XAttributeLiteral myAttributeL = (XAttributeLiteral)myAttribute;
				if (myAttributeL.getValue() != null)
				{
					if (c.isInstance(myAttributeL.getValue()))
					{
						T val = (T)myAttributeL.getValue();
						if (!availableValues.contains(val))
						{
							availableValues.add(val);
						}
					}
				}
			}
			else if (myAttribute instanceof XAttributeTimestamp)
			{
				XAttributeTimestamp myAttributeL = (XAttributeTimestamp)myAttribute;
				if (myAttributeL.getValue() != null)
				{
					if (c.isInstance(myAttributeL.getValue()))
					{
						T val = (T)myAttributeL.getValue();
						if (!availableValues.contains(val))
						{
							availableValues.add(val);
						}
					}
				}
			}
		}
		return availableValues;
	}
	
	public <T> HashSet<T> getAvailableEventAttrValues(Class<T> c, String aName)
	{
		HashSet<T> availableValues = new HashSet<T>();
		for (int i=0; i < log.size(); i++) // in the event log, each element is a case
		{
			for (int j=0; j < log.get(i).size(); j++) // in each case, each element is an event
			{
				XEvent myEvent = log.get(i).get(j);
				
				XAttribute myAttribute = myEvent.getAttributes().get(aName);
				if (myAttribute instanceof XAttributeLiteral)
				{
					XAttributeLiteral myAttributeL = (XAttributeLiteral)myAttribute;
					if (myAttributeL.getValue() != null)
					{
						if (c.isInstance(myAttributeL.getValue()))
						{
							T val = (T)myAttributeL.getValue();
							if (!availableValues.contains(val))
							{
								availableValues.add(val);
							}
						}
					}
				}
				else if (myAttribute instanceof XAttributeTimestamp)
				{
					XAttributeTimestamp myAttributeL = (XAttributeTimestamp)myAttribute;
					if (myAttributeL.getValue() != null)
					{
						if (c.isInstance(myAttributeL.getValue()))
						{
							T val = (T)myAttributeL.getValue();
							if (!availableValues.contains(val))
							{
								availableValues.add(val);
							}
						}
					}
				}
			}
		}
		return availableValues;
	}
}
