package org.processmining.processprofiler3d.psen;

import java.text.NumberFormat;

import javax.swing.JLabel;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.connections.petrinets.EvClassLogPetrinetConnection;
import org.processmining.models.connections.petrinets.PNRepResultAllRequiredParamConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayer.PNLogReplayer;
import org.processmining.plugins.petrinet.replayer.algorithms.IPNReplayAlgorithm;
import org.processmining.plugins.petrinet.replayer.algorithms.IPNReplayParameter;
import org.processmining.plugins.petrinet.replayer.ui.PNReplayerUI;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;

import nl.tue.astar.AStarException;

public class MyPNLogReplayer extends PNLogReplayer implements ILogReplayer
{
	private TransEvClassMapping mapping = null;
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.psen.ILogReplayer#replayLog(org.processmining.contexts.uitopia.UIPluginContext, org.processmining.models.graphbased.directed.petrinet.PetrinetGraph, org.deckfour.xes.model.XLog)
	 */
	@Override
	public PNRepResult replayLog(final UIPluginContext context, PetrinetGraph net, XLog log)
			throws ConnectionCannotBeObtained, AStarException 
	{
		return replayLogGUI(context, net, log);
	}
	
	public PNRepResult replayLogGUI(final UIPluginContext context, PetrinetGraph net, XLog log)
			throws ConnectionCannotBeObtained, AStarException 
	{
		if (net.getTransitions().isEmpty()) 
		{
			context.showConfiguration("Error", new JLabel("Cannot replay on a Petri net that does not contain transitions. Select Cancel or Continue to continue."));
			context.getFutureResult(0).cancel(true);
			return null;
		}
		PNReplayerUI pnReplayerUI = new PNReplayerUI();
		Object[] resultConfiguration = pnReplayerUI.getConfiguration(context, net, log);
		if (resultConfiguration == null) 
		{
			context.getFutureResult(0).cancel(true);
			return null;
		}

		// if all parameters are set, replay log
		if (resultConfiguration[PNReplayerUI.MAPPING] != null) 
		{
			context.log("replay is performed. All parameters are set.");

			// This connection MUST exists, as it is constructed by the configuration if necessary
			context.getConnectionManager().getFirstConnection(EvClassLogPetrinetConnection.class, context, net, log);

			// get all parameters
			IPNReplayAlgorithm selectedAlg = (IPNReplayAlgorithm) resultConfiguration[PNReplayerUI.ALGORITHM];
			IPNReplayParameter algParameters = (IPNReplayParameter) resultConfiguration[PNReplayerUI.PARAMETERS];

			// since based on GUI, create connection
			algParameters.setCreateConn(true);
			algParameters.setGUIMode(true);
			
			this.mapping = (TransEvClassMapping) resultConfiguration[PNReplayerUI.MAPPING];

			PNRepResult res = replayLogPrivate(context, net, log, selectedAlg, algParameters);

			context.getFutureResult(0).setLabel("Performance Statistics for " + XConceptExtension.instance().extractName(log) + " on " + net.getLabel());

			return res;

		} 
		else 
		{
			context.log("replay is not performed because not enough parameters were submitted");
			context.getFutureResult(0).cancel(true);
			return null;
		}
	}
	
	/**
	 * This method is from the PNLogReplayer class in the replay plug-in 
	 * We repeat this method here as the method is with a "private" type.
	 * The main change is regarding to the internal variable of the mapping.  
	 * 
	 * Main method to replay log.
	 * 
	 * @param context
	 * @param net
	 * @param log
	 * @param mapping
	 * @param selectedAlg
	 * @param parameters
	 * @return
	 * @throws AStarException
	 */
	private PNRepResult replayLogPrivate(PluginContext context, PetrinetGraph net, XLog log,
			IPNReplayAlgorithm selectedAlg, IPNReplayParameter parameters) throws AStarException 
	{	
		if (selectedAlg.isAllReqSatisfied(context, net, log, this.mapping, parameters)) 
		{
			// for each trace, replay according to the algorithm. Only returns two objects
			PNRepResult replayRes = null;

			if (parameters.isGUIMode()) 
			{
				long start = System.nanoTime();

				replayRes = selectedAlg.replayLog(context, net, log, this.mapping, parameters);

				long period = System.nanoTime() - start;
				NumberFormat nf = NumberFormat.getInstance();
				nf.setMinimumFractionDigits(2);
				nf.setMaximumFractionDigits(2);

				context.log("Replay is finished in " + nf.format(period / 1000000000) + " seconds");
			} 
			else 
			{
				replayRes = selectedAlg.replayLog(context, net, log, this.mapping, parameters);
			}

			// add connection
			if (replayRes != null) 
			{
				if (parameters.isCreatingConn()) 
				{
					context.addConnection(new PNRepResultAllRequiredParamConnection(
							"Connection between replay result, " + XConceptExtension.instance().extractName(log)
									+ ", and " + net.getLabel(), net, log, this.mapping, selectedAlg, parameters, replayRes));
				}
			}

			return replayRes;
		} 
		else 
		{
			if (context != null) 
			{
				context.log("The provided parameters are not valid for the selected algorithm.");
				context.getFutureResult(0).cancel(true);
			}
			return null;
		}
	}
	
	public boolean mappingIsAvailable()
	{
		if (this.mapping == null)
			return false;
		else 
			return true;
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.processprofiler3d.psen.ILogReplayer#getMapping()
	 */
	@Override
	public TransEvClassMapping getMapping()
	{
		return this.mapping;
	}


}
