package org.processmining.processprofiler3d.psen;

import java.util.Date;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.processprofiler3d.psen.datastructure.H2MemoryDatabase;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PerformanceStatisticsDB;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayCollection;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculator;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorLogFrequency;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorLogFrequencyFiltered;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorTransitionAlignedFrequency;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorTransitionDuration;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorTransitionPairAlignedFrequency;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorTransitionPairSoujournTime;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorTransitionPairWaitingTime;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorTransitionQueueTime;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorTransitionRunTime;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorTransitionServiceTime;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorTransitionSoujournTime;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorTransitionSyncTime;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorTransitionThroughputTime;
import org.processmining.processprofiler3d.psen.replayanalysis.stats.StatCalculatorTransitionWaitingTime;

public class PSENProvider 
{
	private PetrinetGraph hNet;
	
	private IPerformanceStatisticsEnrichedNet psen;
	private IPerformanceStatisticsEnrichedNet wten;
	
	private Vector<StatCalculator> statsCalc = new Vector<StatCalculator>();
	
	public IPerformanceStatisticsEnrichedNet getPSEnet()
	{
		return this.psen;
	}
	
	public PSENProvider(final UIPluginContext context, PetrinetGraph hNet, XLog log, TraceReplayCollection provider)
	{
		this.hNet = hNet;
	    
		statsCalc.add(new StatCalculatorLogFrequency(log));
		statsCalc.add(new StatCalculatorLogFrequencyFiltered());
		statsCalc.add(new StatCalculatorTransitionAlignedFrequency());
		statsCalc.add(new StatCalculatorTransitionDuration());
		statsCalc.add(new StatCalculatorTransitionQueueTime());
		statsCalc.add(new StatCalculatorTransitionServiceTime());
		statsCalc.add(new StatCalculatorTransitionSoujournTime());
		statsCalc.add(new StatCalculatorTransitionSyncTime());
		statsCalc.add(new StatCalculatorTransitionThroughputTime());
		statsCalc.add(new StatCalculatorTransitionWaitingTime());
		statsCalc.add(new StatCalculatorTransitionRunTime());
		statsCalc.add(new StatCalculatorTransitionPairAlignedFrequency());
		statsCalc.add(new StatCalculatorTransitionPairWaitingTime());
		statsCalc.add(new StatCalculatorTransitionPairSoujournTime());
		
		if (constructPSEnet())
		{
			Date startTime = new Date();
			context.log("Calculating performance statistics...");
			caculateStatistics(provider);
			Date endTime = new Date();
			TimeSpan ts = new TimeSpan(startTime, endTime);
			
			context.log("Performance statistics calculation complete (" + ts.ToString(TimeUnit.SECONDS) + "s).");

			context.log("Creating performance statistics indices...");
			//psen.update();
			context.log("PSEN index created.");
			//wten.update();
			context.log("WTEN index created.");
		}
	}

	public IPerformanceStatisticsEnrichedNet getWaitingTimeTree()
	{
		return wten;
	}
	
	public boolean constructPSEnet()
	{
		//this.PSEnet = new PerformanceStatisticsEnrichedNet();
		H2MemoryDatabase h2db = new H2MemoryDatabase();
		
		if (h2db.isDriverAvailable())
		{	
			PerformanceStatisticsDB db = new PerformanceStatisticsDB(h2db);
			db.InitPSEN();
			this.psen = db;
			
			//waitC = new WaitingTimeEnrichedNet();
			PerformanceStatisticsDB db2 = new PerformanceStatisticsDB(h2db);
			db2.InitWTEN();
			wten = db2;
			
			for (StatCalculator sCalc : statsCalc)
			{
				sCalc.addLabelsToPSEN(this.psen, wten);
			}
			return true;
		}
		
		return false;
	}
	
    //calculate stats from log to fill in PSEN later
	public void caculateStatistics(TraceReplayCollection provider)
	{	
		//LogAnalyser la = new LogAnalyser(log);
		
		//if (la.HasLifecycleData())
		{
			for (StatCalculator sCalc : statsCalc)
			{
				sCalc.calculate(hNet, provider, this.psen, this.wten);
			}
		}
	}

}
