package org.processmining.processprofiler3d.psen;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;

public class PetriNet1toManyMapping 
{
	private Map<PetrinetNode, Vector<PetrinetNode>> mapOrig2ViewNode = new HashMap<PetrinetNode, Vector<PetrinetNode>>();
	private Map<PetrinetNode, PetrinetNode> mapView2OrigNode = new HashMap<PetrinetNode, PetrinetNode>();

	public void addPair(PetrinetNode orig, PetrinetNode view)
	{
		Vector<PetrinetNode> viewNodes = new Vector<PetrinetNode>();
		viewNodes.add(view);
		mapOrig2ViewNode.put(orig, viewNodes);
		
		mapView2OrigNode.put(view, orig);
	}
	
	public void addPair(PetrinetNode orig, Vector<PetrinetNode> view)
	{
		mapOrig2ViewNode.put(orig, view);
		
		for (PetrinetNode viewNode : view)
		{
			if (!mapView2OrigNode.containsKey(viewNode))
			{
				mapView2OrigNode.put(viewNode, orig);
			}
		}
	}
	
	public boolean hasOrigNode(PetrinetNode pn)
	{
		return mapOrig2ViewNode.containsKey(pn);
	}
	
	public boolean hasViewNode(PetrinetNode pn)
	{
		return mapView2OrigNode.containsKey(pn);
	}
	
	public Collection<PetrinetNode> getOrigValues()
	{
		return mapOrig2ViewNode.keySet();
	}
	
	public Collection<PetrinetNode> getViewValues()
	{
		return mapView2OrigNode.keySet();
	}
	
	public Vector<PetrinetNode> mapToView(PetrinetNode pn)
	{
		return mapOrig2ViewNode.get(pn);
	}
	
	public PetrinetNode mapToOrig(PetrinetNode pn)
	{
		return mapView2OrigNode.get(pn);
	}
}
