package org.processmining.processprofiler3d.psen;

import java.util.HashMap;

import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.compviz.SubNetDataSource;
import org.processmining.processprofiler3d.compviz.TransitionPair;

public class PetriNetIndexed 
{
	private HashMap<String, Transition> transitionsById = new HashMap<String, Transition>();
	private HashMap<String, Place> placesById = new HashMap<String, Place>();
	
	public PetriNetIndexed(PetrinetGraph pnet)
	{
		IndexFromSource(new PetriNetDataSource(pnet));
	}
	
	public PetriNetIndexed(INetDataSource pnet)
	{
		IndexFromSource(pnet);
	}

	private void IndexFromSource(INetDataSource pnet) 
	{
		for (Transition t : pnet.getTransitions())
		{
			transitionsById.put(t.getLocalID().toString(), t);
		}
		
		for (Place p : pnet.getPlaces())
		{
			placesById.put(p.getLocalID().toString(), p);
		}
		
		for (SubNetDataSource esn : pnet.getSubnets())
		{
			IndexFromSource(esn);
		}
	}
	
	public Place getPlace(String localID)
	{
		if (placesById.containsKey(localID))
		{
			return placesById.get(localID);
		}
		return null;
	}
	
	public Transition getTransition(String localID)
	{
		if (transitionsById.containsKey(localID))
		{
			return transitionsById.get(localID);
		}
		return null;
	}
	
	public PetrinetNode getNode(String localID)
	{
		if (transitionsById.containsKey(localID))
		{
			return transitionsById.get(localID);
		}
		if (placesById.containsKey(localID))
		{
			return placesById.get(localID);
		}
		return null;
	}
	
//	public String getTransitionPairLabel(WaitingTimeTree pair)
//	{
//		return getTransitionPairLabel(pair.getSourceId(), pair.getTargetId());
//	}
	
	public String getTransitionPairLabel(String localIDs)
	{
		return getTransitionPairLabel(TransitionPair.getSourceId(localIDs), TransitionPair.getTargetId(localIDs));
	}
	
	public String getTransitionPairLabel(String sourceId, String targetId)
	{
		Transition source = getTransition(sourceId);
		Transition target = getTransition(targetId);
		
		String header = "";
		
		String sourceLabel = "";
		String targetLabel = "";
		
		if (source != null)
		{
			sourceLabel = source.getLabel();
		}
		if (target != null)
		{
			targetLabel =  target.getLabel();
		}
		header = sourceLabel + " - " + targetLabel;
		
		return header;
	}
}
