package org.processmining.processprofiler3d.psen;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.geom.Point2D;
import java.util.HashMap;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.classification.XEventNameClassifier;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.ResetInhibitorNet;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.InhibitorArc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.ResetArc;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetImpl;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.compviz.SubNetDataSource;

//Project hierarchical petri net to a flat fully expanded petri net
public class PetrinetGraphProjector 
{
	public TransEvClassMapping xxmap = null;
	
	public PetrinetGraphProjector()
	{
		XEventClassifier transitionClassifier = new XEventNameClassifier();
		//xxmap = new TransEvClassMapping(transitionClassifier, null);
		xxmap = new TransEvClassMapping(transitionClassifier, new XEventClass("", 1));
	}
	
	public Petrinet projectPetrinet(PetrinetGraph net,
			PetriNet1to1Mapping map)
	{
		//PetrinetSuper newNet = new PetrinetSuper("Flattened " + net.getLabel());
		Petrinet newNet = new PetrinetImpl("Flattened " + net.getLabel());
		
		assert(map != null);

		PetriNetDataSource src = new PetriNetDataSource(net);
		CopyNodes(newNet, src, null, null, map);
		CopyEdges(newNet,src, null, null, map);
		
		return newNet;
	}
	
	public Petrinet projectPetrinet(PetrinetGraph net,
			PetriNet1to1Mapping map, XLog log, TransEvClassMapping xmap)
	{
		//PetrinetSuper newNet = new PetrinetSuper("Flattened " + net.getLabel());
		Petrinet newNet = new PetrinetImpl("Flattened " + net.getLabel());
		
		assert(map != null);

		PetriNetDataSource src = new PetriNetDataSource(net);
		CopyNodes(newNet, src, null, null, map);
		CopyEdges(newNet,src, null, null, map);

		CreateNewNetLogMapping(newNet, map, log, xmap);
		
		return newNet;
	}
	
	//create new eventlog to petrinet mapping for new net based on eventlog to petrinet mapping for old net
	private void CreateNewNetLogMapping(PetrinetGraph net, PetriNet1to1Mapping map, XLog log,
			TransEvClassMapping xmap) 
	{
		PetriNetIndexed pidx = new PetriNetIndexed(net);
		XLogInfo summary = XLogInfoFactory.createLogInfo(log, xxmap.getEventClassifier());
		XEventClasses eventClasses = summary.getEventClasses();

		// sort event class
		HashMap<String, XEventClass> strlst = new HashMap<String, XEventClass>();
		for (XEventClass xc : eventClasses.getClasses())
		{
			strlst.put(xc.toString(), xc);
		}
		
		for (Transition t : xmap.keySet()) 
		{
			String evtClassStr = xmap.get(t).toString();
			if (strlst.containsKey(evtClassStr))
			{
				Transition tx = pidx.getTransition(t.getLocalID().toString());
				XEventClass xevc = strlst.get(evtClassStr);
				xxmap.put(tx, xevc);
			}
		}
	}
	
	public static void CopyNodes(PetrinetGraph newNet, INetDataSource net, INetDataSource parentNet, Transition parent,
			PetriNet1to1Mapping map)
	{
		//HashMap<Transition, INetDataSource> subnets = new HashMap<Transition, INetDataSource>();
		
		for (SubNetDataSource esn : net.getSubnets())
		{
			Transition tx = net.getTransitionForSubnet(esn);
			
			//subnets.put(tx, esn);
			
			//add subnet elements
			CopyNodes(newNet, esn, net, tx, map);
		}
		
		Place startP = net.getStartPlace();
		Place endP = net.getEndPlace();
		
		// copy entire petrinet and visualize
		for (Place p : net.getPlaces()) 
		{
			if (parent == null)
			{
				CopyPlace(newNet, map, p);
			}
			else
			{
				if (p != startP && p != endP)
				{
					CopyPlace(newNet, map, p);
				}
			}
		}
		
		Color inviTransColor = new Color(230,230,230);
		for (Transition t : net.getTransitions()) 
		{
			if (!net.hasSubnet(t))
			{
				CopyTransition(newNet, map, inviTransColor, t);
			}
		}
	}

	private static void CopyEdges(PetrinetGraph newNet, INetDataSource net, INetDataSource parentNet,
			Transition parent, PetriNet1to1Mapping map) 
	{
		//HashMap<Transition, INetDataSource> subnets = new HashMap<Transition, INetDataSource>();
		
		for (SubNetDataSource esn : net.getSubnets())
		{
			Transition tx = net.getTransitionForSubnet(esn);
			
			//subnets.put(tx, esn);
			
			//add subnet elements
			CopyEdges(newNet, esn, net, tx, map);
		}
		
		Place startP = net.getStartPlace();
		Place endP = net.getEndPlace();	
		
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : net.getEdges()) 
		{
			PetrinetNode source = edge.getSource();
			PetrinetNode target = edge.getTarget();
			
			boolean bAdded = false;
			
			if (source instanceof Transition)
			{
				if (net.hasSubnet((Transition)source))
				{
					continue;
				}
			}
			if (target instanceof Transition)
			{
				if (net.hasSubnet((Transition)target))
				{
					continue;
				}
			}
			
			if (parentNet != null)
			{
				if (source == startP)
				{
					PetrinetNode pn = parentNet.getInEdges(parent).iterator().next().getSource();
					Place px = (Place)pn;

					//CopyArc(newNet, net, edge, mapOrig2ViewNode.get(px), mapOrig2ViewNode.get(target));
					CopyArc(newNet, net, edge, map.mapToView(px), map.mapToView(target));
					bAdded = true;
				}
				else if (target == endP)
				{
					PetrinetNode pn = parentNet.getOutEdges(parent).iterator().next().getTarget();
//					if (!(pn instanceof Place))
//					{
//						String s = "";
//					}
					
					Place px = (Place)pn;
					
					CopyArc(newNet, net, edge, map.mapToView(source), map.mapToView(px));
					bAdded = true;
				}
			}
			
			if (!bAdded)
			{
				CopyArc(newNet, net, edge, map.mapToView(source), map.mapToView(target));
			}
			
		}
	}

	private static void CopyArc(PetrinetGraph newNet, INetDataSource net,
			PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge, PetrinetNode source, PetrinetNode target) 
	{
		if (newNet instanceof ResetInhibitorNet)
		{
			ResetInhibitorNet rNetP = (ResetInhibitorNet)newNet;
			if (edge instanceof ResetArc) 
			{
				ResetArc nArc = rNetP.addResetArc((Place)source, (Transition)target, edge.getLabel());
				nArc.setLocalID(edge.getLocalID());
			} 
			else if (edge instanceof InhibitorArc) 
			{
				InhibitorArc nArc = rNetP.addInhibitorArc((Place)source, (Transition)target, edge.getLabel());
				nArc.setLocalID(edge.getLocalID());
			} 
			else 
			{
				//Arc oArc = net.getArc(edge.getSource(), edge.getTarget());
				Arc oArc = (Arc)edge;
				Arc nArc = null;
				if (source instanceof Transition) 
				{
					nArc = rNetP.addArc((Transition)source, (Place)target, oArc.getWeight());
				} 
				else 
				{
					nArc = rNetP.addArc((Place)source, (Transition)target, oArc.getWeight());
				}
				nArc.setLocalID(edge.getLocalID());
			}
		}
		else
		{
			//Arc oArc = net.getArc(edge.getSource(), edge.getTarget());
			Arc oArc = (Arc)edge;
			Arc nArc = null;
			if (source instanceof Transition) 
			{
				nArc = newNet.addArc((Transition)source, (Place)target, oArc.getWeight());
			} 
			else 
			{
				nArc = newNet.addArc((Place)source, (Transition)target, oArc.getWeight());
			}
			nArc.setLocalID(edge.getLocalID());		
		}
	}

	private static void CopyTransition(PetrinetGraph newNet, PetriNet1to1Mapping map, Color inviTransColor, Transition t) 
	{
		Dimension newD = new Dimension();
		newD.setSize((Dimension) t.getAttributeMap().get(AttributeMap.SIZE));
//	newD.setSize(newD.getWidth(), newD.getHeight());
		//TransitionP newTrans = newNet.addTransition(t.getLabel(), newD);
		Transition newTrans = newNet.addTransition(t.getLabel());
		newTrans.setLocalID(t.getLocalID());
		newTrans.setInvisible(t.isInvisible());
		newTrans.getAttributeMap().put(AttributeMap.SHOWLABEL, false);
		map.addPair(t, newTrans);
//		mapOrig2ViewNode.put(t, newTrans);
//		if (mapView2OrigNode != null) 
//		{
//			mapView2OrigNode.put(newTrans, t);
//		}
		if (newTrans.isInvisible())
		{
			newTrans.getAttributeMap().put(AttributeMap.FILLCOLOR, inviTransColor);
			newTrans.getAttributeMap().put(AttributeMap.SHOWLABEL, true);
		}
	}

	private static void CopyPlace(PetrinetGraph newNet, PetriNet1to1Mapping map, Place p) 
	{
		Place newPlace = newNet.addPlace(p.getLabel());
		newPlace.setLocalID(p.getLocalID());
		
		map.addPair(p, newPlace);
//		mapOrig2ViewNode.put(p, newPlace);
//		if (mapView2OrigNode != null) 
//		{
//			mapView2OrigNode.put(newPlace, p);
//		}
	}

	
	public static Marking projectMarking(Marking initMarking, PetriNet1to1Mapping map) 
	{
		Marking newMarking = new Marking();
		// copy initial marking
		for (Place p : initMarking.baseSet()) 
		{
			PetrinetNode pn = map.mapToView(p);
			Place newPlace = (Place)pn;
			if (newPlace == null)
			{
				PetrinetNode pxx = null;
				for (PetrinetNode px : map.getViewValues())
				{
					if (p.getLabel().equals(px.getLabel()))
					{
						pxx = px;
					}
				}
				if (pxx != null)
				{
					newMarking.add((Place)pxx, initMarking.occurrences(p));
				}
				String s = "";
			}
			else
			{
				newMarking.add(newPlace, initMarking.occurrences(p));
			}
		}

		return newMarking;
	}
	
	public static GraphLayoutConnection projectGraphLayout(GraphLayoutConnection oldLayoutConn, PetrinetGraph net,
			PetrinetGraph newNet, PetriNet1to1Mapping map)
	{

		// copy layout
		GraphLayoutConnection newLayoutConn = null;
		if (oldLayoutConn != null) 
		{
			newLayoutConn = new GraphLayoutConnection(newNet);
			for (Place p : net.getPlaces()) 
			{
				newLayoutConn.setPosition(map.mapToView(p), oldLayoutConn.getPosition(p));
			}
			for (Transition t : net.getTransitions()) 
			{
				//newLayoutConn.setPosition(mapOrig2ViewNode.get(t), new Point2D.Double(oldLayoutConn.getPosition(t).getX(),oldLayoutConn.getPosition(t).getY()-oldLayoutConn.getSize(t).getHeight()*2));
				newLayoutConn.setPosition(map.mapToView(t), new Point2D.Double(oldLayoutConn.getPosition(t).getX(),oldLayoutConn.getPosition(t).getY()));
			}
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : net.getEdges()) 
			{
				newLayoutConn.setEdgePoints(
						newNet.getArc(map.mapToView(edge.getSource()), map.mapToView(edge.getTarget())),
						oldLayoutConn.getEdgePoints(edge));
			}
		}
		return newLayoutConn;
	}
}
