package org.processmining.processprofiler3d.psen;

import java.awt.Color;
import java.awt.geom.Point2D;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Vector;

import org.deckfour.xes.classification.XEventAndClassifier;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.classification.XEventLifeTransClassifier;
import org.deckfour.xes.classification.XEventNameClassifier;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.LocalNodeID;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetImpl;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;

//project petri net to petri net extended with lifecylce events
public class PetrinetGraphProjector2 
{
	private static final Color inviTransColor = new Color(230,230,230);
	//private LogAnalyser logA = null;
	
	public TransEvClassMapping xxmap = null;
	
	public PetrinetGraphProjector2()
	{
		XEventClassifier transitionClassifier = new XEventAndClassifier(new XEventNameClassifier(), new XEventLifeTransClassifier());
		//xxmap = new TransEvClassMapping(transitionClassifier, null);
		xxmap = new TransEvClassMapping(transitionClassifier, new XEventClass("", 1));
	}
//	
//	public static Petrinet projectPetrinet(PetrinetGraph net,
//			PetriNet1toManyMapping map)
//	{
//		PetrinetGraphProjector2 pgp = new PetrinetGraphProjector2();
//		return pgp.projectPetrinet2(net, map);
//	}
	
	private HashMap<Transition, HashSet<XLifecycleExtension.StandardModel>> tlifecycleData = null;
	
	public Petrinet projectPetrinet2(PetrinetGraph net, PetriNet1toManyMapping map, XLog log)
	{
		LogAnalyser la = new LogAnalyser(log);
		
		HashMap<String, HashSet<XLifecycleExtension.StandardModel>> lifecycleData = la.getLifecyclePerAlignedTransition();
		
		tlifecycleData = new HashMap<Transition, HashSet<XLifecycleExtension.StandardModel>>();
		for (Transition t : net.getTransitions()) 
		{
			if (lifecycleData.containsKey(t.getLocalID().toString()))
			{
				HashSet<XLifecycleExtension.StandardModel> lst = lifecycleData.get(t.getLocalID().toString());
				if (lst != null)
				{
					tlifecycleData.put(t, lst);
				}
			}
		}
		
		//PetrinetSuper newNet = new PetrinetSuper("Expanded " + net.getLabel());
		Petrinet newNet = new PetrinetImpl("Expanded " + net.getLabel());
		
		assert(map != null);

		PetriNetDataSource src = new PetriNetDataSource(net);
		CopyNodes(newNet, src, null, null, map);
	
		return newNet;
	}
	
	public Petrinet projectPetrinet2(PetrinetGraph net, PetriNet1toManyMapping map, XLog log, TransEvClassMapping xmap)
	{
		LogAnalyser la = new LogAnalyser(log);
		
		HashMap<String, HashSet<XLifecycleExtension.StandardModel>> lifecycleData = la.getLifecyclePerEventClass(xmap);
		tlifecycleData = new HashMap<Transition, HashSet<XLifecycleExtension.StandardModel>>();
		for (Transition t : net.getTransitions()) 
		{
			if (xmap.containsKey(t))
			{
				String evtClassStr = xmap.get(t).toString();
				HashSet<XLifecycleExtension.StandardModel> lst = lifecycleData.get(evtClassStr);
				if (lst != null)
				{
					tlifecycleData.put(t, lst);
				}
			}
		}
		
		//PetrinetSuper newNet = new PetrinetSuper("Expanded " + net.getLabel());
		Petrinet newNet = new PetrinetImpl("Expanded " + net.getLabel());
		
		assert(map != null);

		PetriNetDataSource src = new PetriNetDataSource(net);
		CopyNodes(newNet, src, null, null, map);
		
		CreateNewNetLogMapping(net, map, log, xmap, lifecycleData);
	
		return newNet;
	}

	//create new eventlog to petrinet mapping for new net based on eventlog to petrinet mapping for old net
	private void CreateNewNetLogMapping(PetrinetGraph net, PetriNet1toManyMapping map, XLog log,
			TransEvClassMapping xmap, HashMap<String, HashSet<XLifecycleExtension.StandardModel>> lifecycleData) 
	{
		XLogInfo summary = XLogInfoFactory.createLogInfo(log, xxmap.getEventClassifier());
		XEventClasses eventClasses = summary.getEventClasses();

		// sort event class
		HashMap<String, XEventClass> strlst = new HashMap<String, XEventClass>();
		for (XEventClass xc : eventClasses.getClasses())
		{
			strlst.put(xc.toString(), xc);
		}
		for (Transition t : net.getTransitions()) 
		{
			if (!xmap.containsKey(t))
			{
				continue;
			}
			
			String evtClassStr = xmap.get(t).toString();
			HashSet<XLifecycleExtension.StandardModel> lst = lifecycleData.get(evtClassStr);
			Vector<PetrinetNode> pnds = map.mapToView(t);
			if (lst != null)
			{
				if (lst.contains(XLifecycleExtension.StandardModel.SCHEDULE) && strlst.containsKey(evtClassStr+"+schedule"))
				{
					Transition tx = (Transition)pnds.get(0);
					XEventClass xevc = strlst.get(evtClassStr+"+schedule");
					xxmap.put(tx, xevc);
				}
				if (lst.contains(XLifecycleExtension.StandardModel.ASSIGN) && strlst.containsKey(evtClassStr+"+assign"))
				{
					Transition tx = (Transition)pnds.get(1);
					XEventClass xevc = strlst.get(evtClassStr+"+assign");
					xxmap.put(tx, xevc);
				}
				if (lst.contains(XLifecycleExtension.StandardModel.START) && strlst.containsKey(evtClassStr+"+start"))
				{
					Transition tx = (Transition)pnds.get(2);
					XEventClass xevc = strlst.get(evtClassStr+"+start");
					xxmap.put(tx, xevc);
				}
				if (lst.contains(XLifecycleExtension.StandardModel.COMPLETE) && strlst.containsKey(evtClassStr+"+complete"))
				{
					Transition tx = (Transition)pnds.get(3);
					XEventClass xevc = strlst.get(evtClassStr+"+complete");
					xxmap.put(tx, xevc);
				}
			}
			else
			{
				String s = "Something!";
				Transition tx = (Transition)pnds.get(0);
				xxmap.put(tx, xxmap.getDummyEventClass());				
			}
		}
	}
	
	private void CopyNodes(PetrinetGraph newNet, INetDataSource net, INetDataSource parentNet, Transition parent,
			PetriNet1toManyMapping map)
	{
		// copy entire petrinet and visualize
		for (Place p : net.getPlaces()) 
		{
			CopyPlace(newNet, map, p);
		}
		
		for (Transition t : net.getTransitions()) 
		{
			CopyTransition(newNet, net, map, t);
		}
	}

	private void CopyPlace(PetrinetGraph newNet, PetriNet1toManyMapping map, Place p) 
	{
		Place newPlace = newNet.addPlace(p.getLabel());
		newPlace.setLocalID(p.getLocalID());
		
		map.addPair(p, newPlace);
	}

	private void CopyTransition(PetrinetGraph newNet, INetDataSource net, PetriNet1toManyMapping map, Transition t) 
	{
		//Dimension newD = new Dimension();
		//newD.setSize((Dimension) t.getAttributeMap().get(AttributeMap.SIZE));

		if (t.isInvisible())
		{
			CopyTransitionInvisible(newNet, net, map, t);		
		}
		else
		{
			if (tlifecycleData.containsKey(t))
			{
				if (!tlifecycleData.get(t).isEmpty())
				{
					CopyTransitionHelper(newNet, net, map, t);
				}
				else
				{
					CopyTransitionInvisible(newNet, net, map, t);	
				}
			}
			else
			{
				CopyTransitionInvisible(newNet, net, map, t);	
			}
		}
	}

	private void CopyTransitionInvisible(PetrinetGraph newNet, INetDataSource net, PetriNet1toManyMapping map, Transition t) 
	{
		Transition newTrans1 = newNet.addTransition(t.getLabel());
		newTrans1.setLocalID(t.getLocalID());
		newTrans1.setInvisible(t.isInvisible());
		newTrans1.getAttributeMap().put(AttributeMap.SHOWLABEL, false);
		Vector<PetrinetNode> viewNodes = new Vector<PetrinetNode>();
		viewNodes.add(newTrans1); //schedule
		viewNodes.add(newTrans1); //assign
		viewNodes.add(newTrans1); //start
		viewNodes.add(newTrans1); //complete
		map.addPair(t, viewNodes);
		if (newTrans1.isInvisible())
		{
			newTrans1.getAttributeMap().put(AttributeMap.FILLCOLOR, inviTransColor);
			newTrans1.getAttributeMap().put(AttributeMap.SHOWLABEL, true);
		}
		
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : net.getInEdges(t))
		{
			PetrinetNode source = map.mapToView(edge.getSource()).get(0);

			Arc oArc = (Arc)edge;
			Arc nArc = newNet.addArc((Place)source, newTrans1, oArc.getWeight());
				
			nArc.setLocalID(edge.getLocalID());
		}
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : net.getOutEdges(t))
		{
			PetrinetNode target = map.mapToView(edge.getTarget()).get(0);

			Arc oArc = (Arc)edge;
			Arc nArc = newNet.addArc(newTrans1, (Place)target, oArc.getWeight());
			nArc.setLocalID(edge.getLocalID());
		}
	}

	private void CopyTransitionHelper(PetrinetGraph newNet, INetDataSource net, PetriNet1toManyMapping map, Transition t) 
	{
		HashSet<XLifecycleExtension.StandardModel> lData = tlifecycleData.get(t);
		Vector<Transition> newNodes = new Vector<Transition>();

		Transition newTrans1 = null;
		Transition newTrans2 = null;
		Transition newTrans3 = null;
		Transition newTrans4 = null;
		if (lData.contains(XLifecycleExtension.StandardModel.SCHEDULE))
		{
			newTrans1 = newNet.addTransition(t.getLabel() + "+schedule");
			//xxmap.put(newTrans1, arg1)
			newNodes.add(newTrans1);
		}
		if (lData.contains(XLifecycleExtension.StandardModel.ASSIGN))
		{
			newTrans2 = newNet.addTransition(t.getLabel() + "+assign");
			newNodes.add(newTrans2);
		}
		if (lData.contains(XLifecycleExtension.StandardModel.START))
		{
			newTrans3 = newNet.addTransition(t.getLabel() + "+start");
			newTrans3.setLocalID(t.getLocalID());
			newTrans3.getAttributeMap().put(AttributeMap.SHOWLABEL, false);
			newNodes.add(newTrans3);
		}
		if (lData.contains(XLifecycleExtension.StandardModel.COMPLETE))
		{
			newTrans4 = newNet.addTransition(t.getLabel() + "+complete");
			newNodes.add(newTrans4);
		}
		
//		if (newNodes.isEmpty())
//		{
//			String s = "";
//		}
		
		//add connection between net and first transition
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : net.getInEdges(t))
		{
			PetrinetNode source = map.mapToView(edge.getSource()).get(0);

			Arc oArc = (Arc)edge;
			Arc nArc = newNet.addArc((Place)source, newNodes.firstElement(), oArc.getWeight());
				
			nArc.setLocalID(edge.getLocalID());
		}
		
		//add bridge between transition pairs
		for (int i = 0; i < newNodes.size()-1; i++)
		{
			Place pconn = newNet.addPlace("pconn" + t.getLabel());
			pconn.setLocalID(new LocalNodeID());
			newNet.addArc(newNodes.get(i), pconn);
			newNet.addArc(pconn, newNodes.get(i+1));
		}

		//add connection between last transition and net
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : net.getOutEdges(t))
		{
			PetrinetNode target = map.mapToView(edge.getTarget()).get(0);

			Arc oArc = (Arc)edge;
			Arc nArc = newNet.addArc(newNodes.lastElement(), (Place)target, oArc.getWeight());
			nArc.setLocalID(edge.getLocalID());
		}
		
		//save mapping of nodes
		Vector<PetrinetNode> viewNodes = new Vector<PetrinetNode>();
		
		if (newTrans1 == null)
		{
			if (newTrans2 != null)
			{
				newTrans1 = newTrans2;
			}
			else if (newTrans3 != null)
			{
				newTrans1 = newTrans3;
			}
			else if (newTrans4 != null)
			{
				newTrans1 = newTrans4;
			}
		}
		if (newTrans2 == null)
		{
			if (newTrans3 != null)
			{
				newTrans2 = newTrans3;
			}
			else if (newTrans1 != null)
			{
				newTrans2 = newTrans1;
			}
			else if (newTrans4 != null)
			{
				newTrans2 = newTrans4;
			}
		}
		if (newTrans3 == null)
		{
			if (newTrans1 != null)
			{
				newTrans3 = newTrans1;
			}
			else if (newTrans2 != null)
			{
				newTrans3 = newTrans2;
			}
			else if (newTrans4 != null)
			{
				newTrans3 = newTrans4;
			}
		}
		if (newTrans4 == null)
		{
			if (newTrans3 != null)
			{
				newTrans4 = newTrans3;
			}
			else if (newTrans2 != null)
			{
				newTrans4 = newTrans2;
			}
			else if (newTrans1 != null)
			{
				newTrans4 = newTrans1;
			}
		}
		
		viewNodes.add(newTrans1); //schedule
		viewNodes.add(newTrans2); //assign
		viewNodes.add(newTrans3); //start
		viewNodes.add(newTrans4); //complete
		
		map.addPair(t, viewNodes);
	}

	
	public static Marking projectMarking(Marking initMarking, PetriNet1toManyMapping map) 
	{
		Marking newMarking = new Marking();
		// copy initial marking
		for (Place p : initMarking.baseSet()) 
		{
			PetrinetNode pn = map.mapToView(p).get(0);
			Place newPlace = (Place)pn;
			if (newPlace == null)
			{
				PetrinetNode pxx = null;
				for (PetrinetNode px : map.getViewValues())
				{
					if (p.getLabel().equals(px.getLabel()))
					{
						pxx = px;
					}
				}
				if (pxx != null)
				{
					newMarking.add((Place)pxx, initMarking.occurrences(p));
				}
				String s = "";
			}
			else
			{
				newMarking.add(newPlace, initMarking.occurrences(p));
			}
		}

		return newMarking;
	}
	
	public static GraphLayoutConnection projectGraphLayout(GraphLayoutConnection oldLayoutConn, PetrinetGraph net,
			PetrinetGraph newNet,
			Map<PetrinetNode, PetrinetNode> mapOrig2ViewNode)
	{

		// copy layout
		GraphLayoutConnection newLayoutConn = null;
		if (oldLayoutConn != null) 
		{
			newLayoutConn = new GraphLayoutConnection(newNet);
			for (Place p : net.getPlaces()) 
			{
				newLayoutConn.setPosition(mapOrig2ViewNode.get(p), oldLayoutConn.getPosition(p));
			}
			for (Transition t : net.getTransitions()) 
			{
				newLayoutConn.setPosition(mapOrig2ViewNode.get(t), new Point2D.Double(oldLayoutConn.getPosition(t).getX(),oldLayoutConn.getPosition(t).getY()));
			}
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : net.getEdges()) 
			{
				newLayoutConn.setEdgePoints(
						newNet.getArc(mapOrig2ViewNode.get(edge.getSource()), mapOrig2ViewNode.get(edge.getTarget())),
						oldLayoutConn.getEdgePoints(edge));
			}
		}
		return newLayoutConn;
	}
}
