package org.processmining.processprofiler3d.psen.classification;

import java.util.HashSet;
import java.util.Vector;

import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.processprofiler3d.psen.LogAnalyser;

public class ClassificationCohortIntRange extends ClassificationCohort 
{
	public static String classifierName = "Integer Range";
	
	public String attrName = "";
	private int startValue = 0;
	private int endValue = 0;
	
	private boolean bInclStart = true;
	private boolean bInclEnd = true;
	
	public ClassificationCohortIntRange(String n, String aName, int st, int et)
	{
		super(n);
		
		attrName = aName;
		startValue = st;
		endValue = et;
	}
	
	public ClassificationCohortIntRange(CohortDefinitionIntRange cdtod)
	{
		super(cdtod.name);
		
		attrName = cdtod.attrName;
		startValue = cdtod.startValue;
		endValue = cdtod.endValue;
		bInclStart = cdtod.bInclStart;
		bInclEnd = cdtod.bInclEnd;
	}
	
	@Override
	public CohortDefinition getCohortDefinition()
	{
		CohortDefinition cd = new CohortDefinitionIntRange(name, attrName, startValue, endValue, bInclStart, bInclEnd);

		for (ClassificationCohort cc : children)
		{
			cd.children.add(cc.getCohortDefinition());
		}
		
		return cd;
	}
	
	@Override
	public boolean isPartOfCohort(XTrace trc)
	{
		XAttributeDiscrete myTimeAttr = (XAttributeDiscrete)trc.getAttributes().get(attrName);
		
		//skip events without discrete attribute
		if (myTimeAttr == null)
			return false;
		
		Long mytime = myTimeAttr.getValue();

//		if (mytime >= startValue && mytime <= endValue)
//		{
//			return true;
//		}
//		
//		return false;
		boolean bStartMet = false;
		boolean bEndMet = false;
		
		if (bInclStart)
		{
			bStartMet = (mytime >= startValue);
		}
		else
		{
			bStartMet = (mytime > startValue);
		}
		
		if (bInclEnd)
		{
			bEndMet = (mytime <= endValue);
		}
		else
		{
			bEndMet = (mytime < endValue);
		}
		
		return (bStartMet && bEndMet);
	}
	
	public static Vector<String> getPresetNames()
	{
		Vector<String> presets = new Vector<String>();
		//presets.add("Quarter Day Periods");
		
		return presets;
	}
	
	public static CohortPreset getPreset(String presetName, String selectedAttribute, XLog l)
	{
//		if (presetName.equals("Quarter Day Periods"))
//		{
//			CohortPreset cp = new CohortPreset();
//			cp.cohortDefs.add(new CohortDefinitionTimeOfDay("cohort1", selectedAttribute, 0, 6));
//			cp.cohortDefs.add(new CohortDefinitionTimeOfDay("cohort2", selectedAttribute, 6, 12));
//			cp.cohortDefs.add(new CohortDefinitionTimeOfDay("cohort3", selectedAttribute, 12, 18));
//			cp.cohortDefs.add(new CohortDefinitionTimeOfDay("cohort4", selectedAttribute, 18, 24));
//			
//			return cp;
//		}
		
		return null;
	}
	
	public static boolean isValidAttribute(LogAnalyser la, String s)
	{
		return getAvailableAttributes(la).contains(s);
	}
	
	public static boolean isValidAttribute(XLog log, String s)
	{
		LogAnalyser la = new LogAnalyser(log);
		return getAvailableAttributes(la).contains(s);
	}
	
	public static HashSet<String> getAvailableAttributes(LogAnalyser la)
	{
		return la.getAvailableTraceAttrsDiscrete();
	}
	
	public static HashSet<String> getAvailableAttributes(XLog log)
	{
		LogAnalyser la = new LogAnalyser(log);
		return la.getAvailableTraceAttrsDiscrete();
	}
}
