package org.processmining.processprofiler3d.psen.classification;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Vector;

import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.processprofiler3d.psen.LogAnalyser;

public class ClassificationCohortLiteral extends ClassificationCohort 
{
	public static String classifierName = "Generic Literal Attribute";
	
	public String attrName = "";
	private HashSet<String> cValues = new HashSet<String>();
	
	public ClassificationCohortLiteral(String n, String aName)
	{
		super(n);
		attrName = aName;
	}
	
	public ClassificationCohortLiteral(CohortDefinitionLiteral cdl)
	{
		super(cdl.name);
		
		attrName = cdl.attrName;
		addValues(cdl.values);
	}
	
	@Override
	public CohortDefinition getCohortDefinition()
	{
		CohortDefinitionLiteral cdl = new CohortDefinitionLiteral(name, attrName, cValues);

		for (ClassificationCohort cc : children)
		{
			cdl.children.add(cc.getCohortDefinition());
		}
		
		return cdl;
	}
	
	public void addValue(String s)
	{
		cValues.add(s);
	}
	
	public void addValues(Collection<String> s)
	{
		cValues.addAll(s);
	}
	
	public boolean containsValue(String v)
	{
		return cValues.contains(v);
	}
	
	@Override
	public boolean isPartOfCohort(XTrace trc)
	{
		XAttributeLiteral myLiteralAttr = (XAttributeLiteral)trc.getAttributes().get(attrName);
		
		//skip events without specified attribute
		if (myLiteralAttr == null || myLiteralAttr.getValue() == null)
			return false;
		
		String str = myLiteralAttr.getValue();

		if (cValues.contains(str))
		{
			return true;
		}
		
		return false;
	}
	
	public static Vector<String> getPresetNames()
	{
		Vector<String> presets = new Vector<String>();
		presets.add("Split in half (alphabetically)");
		presets.add("Split in half (randomnly)");
		presets.add("Split in individual values");
		
		return presets;
	}
	
	public static CohortPreset getPreset(String presetName, String selectedAttribute, XLog l)
	{
		if (presetName.equals("Split in half (alphabetically)"))
		{
			LogAnalyser la = new LogAnalyser(l);
			CohortPreset cp = new CohortPreset();
			Vector<String> vs = new Vector<String>(la.getAvailableTraceAttrValuesLiteral(selectedAttribute));
			Collections.sort(vs);
			
			HashSet<String> vs1 = new HashSet<String>();
			HashSet<String> vs2 = new HashSet<String>();
			int halfSize = (int)(vs.size() * 0.5);
			for (int i = 0; i < vs.size(); i++)
			{
				if (i < halfSize)
				{
					vs1.add(vs.get(i));
				}
				else
				{
					vs2.add(vs.get(i));
				}
			}
			
			cp.cohortDefs.add(new CohortDefinitionLiteral("cohort1", selectedAttribute, vs1));
			cp.cohortDefs.add(new CohortDefinitionLiteral("cohort2", selectedAttribute, vs2));
			
			return cp;
		}
		else if (presetName.equals("Split in half (randomnly)"))
		{
			LogAnalyser la = new LogAnalyser(l);
			CohortPreset cp = new CohortPreset();
			Vector<String> vs = new Vector<String>(la.getAvailableTraceAttrValuesLiteral(selectedAttribute));
			Collections.shuffle(vs);
			
			HashSet<String> vs1 = new HashSet<String>();
			HashSet<String> vs2 = new HashSet<String>();
			int halfSize = (int)(vs.size() * 0.5);
			for (int i = 0; i < vs.size(); i++)
			{
				if (i < halfSize)
				{
					vs1.add(vs.get(i));
				}
				else
				{
					vs2.add(vs.get(i));
				}
			}
			
			cp.cohortDefs.add(new CohortDefinitionLiteral("cohort1", selectedAttribute, vs1));
			cp.cohortDefs.add(new CohortDefinitionLiteral("cohort2", selectedAttribute, vs2));
			
			return cp;
		}
		else if (presetName.equals("Split in individual values"))
		{
			LogAnalyser la = new LogAnalyser(l);
			CohortPreset cp = new CohortPreset();
			HashSet<String> vs = la.getAvailableTraceAttrValuesLiteral(selectedAttribute);
			
			int cohortId = 1;
			for (String str : vs)
			{
				HashSet<String> lst = new HashSet<String>();
				lst.add(str);
				//cp.cohortDefs.add(new CohortDefinitionLiteral("cohort" + cohortId, selectedAttribute, lst));
				cp.cohortDefs.add(new CohortDefinitionLiteral(str, selectedAttribute, lst));
				cohortId++;
			}
			
			return cp;
		}
		
		return null;
	}
	
	public static boolean isValidAttribute(LogAnalyser la, String s)
	{
		return getAvailableAttributes(la).contains(s);
	}
	
	public static boolean isValidAttribute(XLog log, String s)
	{
		LogAnalyser la = new LogAnalyser(log);
		return getAvailableAttributes(la).contains(s);
	}
	
	public static HashSet<String> getAvailableAttributes(LogAnalyser la)
	{
		return la.getAvailableTraceAttrsLiteral();
	}
	
	public static HashSet<String> getAvailableAttributes(XLog log)
	{
		LogAnalyser la = new LogAnalyser(log);
		return la.getAvailableTraceAttrsLiteral();
	}
}
