package org.processmining.processprofiler3d.psen.classification;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.Vector;

import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.processprofiler3d.psen.LogAnalyser;

public class ClassificationCohortTimeOfDay extends ClassificationCohort 
{
	public static String classifierName = "Time of Day";
	
	public String attrName = "";
	private int startTime = 0;
	private int endTime = 0;
	
	private boolean bInclStart = false;
	private boolean bInclEnd = true;
	
	public ClassificationCohortTimeOfDay(String n, String aName, int st, int et)
	{
		super(n);
		
		attrName = aName;
		startTime = st;
		endTime = et;
	}
	
	public ClassificationCohortTimeOfDay(String n, String aName, int st, int et, boolean bStartIn, boolean bEndIn)
	{
		super(n);
		
		attrName = aName;
		startTime = st;
		endTime = et;
		
		bInclStart = bStartIn;
		bInclEnd = bEndIn;
	}
	
	public ClassificationCohortTimeOfDay(CohortDefinitionTimeOfDay cdtod)
	{
		super(cdtod.name);
		
		attrName = cdtod.attrName;
		startTime = cdtod.startTime;
		endTime = cdtod.endTime;
		bInclStart = cdtod.bInclStart;
		bInclEnd = cdtod.bInclEnd;
	}
	
	@Override
	public CohortDefinition getCohortDefinition()
	{
		CohortDefinition cd = new CohortDefinitionTimeOfDay(name, attrName, startTime, endTime, bInclStart, bInclEnd);
		
		for (ClassificationCohort cc : children)
		{
			cd.children.add(cc.getCohortDefinition());
		}
		
		return cd;
	}
	
	@Override
	public boolean isPartOfCohort(XTrace trc)
	{
		XAttributeTimestamp myTimeAttr = (XAttributeTimestamp)trc.getAttributes().get(attrName);
		
		//skip events without timestamp attribute
		if (myTimeAttr == null || myTimeAttr.getValue() == null)
			return false;
		
		Date mytime = myTimeAttr.getValue();
		int mytimeHours = getHours(mytime);

		boolean bStartMet = false;
		boolean bEndMet = false;
		
		if (bInclStart)
		{
			bStartMet = (mytimeHours >= startTime);
		}
		else
		{
			bStartMet = (mytimeHours > startTime);
		}
		
		if (bInclEnd)
		{
			bEndMet = (mytimeHours <= endTime);
		}
		else
		{
			bEndMet = (mytimeHours < endTime);
		}
		
		return (bStartMet && bEndMet);
	}
	
	private static int getHours(Date date)
	{
		Calendar calendar = GregorianCalendar.getInstance(); // creates a new calendar instance
		calendar.setTime(date);   // assigns calendar to given date 
		return calendar.get(Calendar.HOUR_OF_DAY); // gets hour in 24h format
	}
	
	public static Vector<String> getPresetNames()
	{
		Vector<String> presets = new Vector<String>();
		presets.add("Quarter Day Periods");
		
		return presets;
	}
	
	public static CohortPreset getPreset(String presetName, String selectedAttribute, XLog l)
	{
		if (presetName.equals("Quarter Day Periods"))
		{
			CohortPreset cp = new CohortPreset();
			cp.cohortDefs.add(new CohortDefinitionTimeOfDay("cohort1", selectedAttribute, 0, 6));
			cp.cohortDefs.add(new CohortDefinitionTimeOfDay("cohort2", selectedAttribute, 6, 12));
			cp.cohortDefs.add(new CohortDefinitionTimeOfDay("cohort3", selectedAttribute, 12, 18));
			cp.cohortDefs.add(new CohortDefinitionTimeOfDay("cohort4", selectedAttribute, 18, 24));
			
			return cp;
		}
		
		return null;
	}
	
	public static boolean isValidAttribute(LogAnalyser la, String s)
	{
		return getAvailableAttributes(la).contains(s);
	}
	
	public static boolean isValidAttribute(XLog log, String s)
	{
		LogAnalyser la = new LogAnalyser(log);
		return getAvailableAttributes(la).contains(s);
	}
	
	public static HashSet<String> getAvailableAttributes(LogAnalyser la)
	{
		return la.getAvailableTraceAttrsTimestamp();
	}
	
	public static HashSet<String> getAvailableAttributes(XLog log)
	{
		LogAnalyser la = new LogAnalyser(log);
		return la.getAvailableTraceAttrsTimestamp();
	}
}
