package org.processmining.processprofiler3d.psen.classification;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Vector;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.util.ui.wizard.ListWizard;
import org.processmining.framework.util.ui.wizard.ProMWizardDisplay;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.gui.CohortConfigurationStep;
import org.processmining.processprofiler3d.psen.gui.CohortConfigurationStep2;

public class CohortClassifier 
{
	public static Vector<CohortPreset> customPresets = new Vector<CohortPreset>();
	
	protected Vector<String> cohortLst =  new Vector<String>();
	protected Vector<ClassificationCohort> cohorts = new Vector<ClassificationCohort>();
	protected ClassificationCohort root = new ClassificationCohort("Root");
	
	public ClassificationCohort getRoot()
	{
		return root;
	}
	
	public int getNumCohorts()
	{
		return getCohortsHelper().size();
		//return cohortLst.size();
	}
	
	public Vector<ClassificationCohort> getCohorts()
	{
		return new Vector<ClassificationCohort>(getCohortsHelper());
	}
	
	protected Vector<ClassificationCohort> getCohortsHelper()
	{
		if (cohorts == null)
		{
			Vector<ClassificationCohort> lst = new Vector<ClassificationCohort>();
			Vector<ClassificationCohort> lstFinal = new Vector<ClassificationCohort>();
			
			lst.addAll(root.children);
			
			boolean bRepeat = true;
			while (bRepeat)
			{
				bRepeat = false;
				for (int i = 0; i < lst.size(); i++)
				{
					ClassificationCohort cc = lst.get(i);
					if (!cc.children.isEmpty())
					{
						lstFinal.add(cc);
						lst.remove(cc);
						lst.addAll(cc.children);
						
						bRepeat = true;
						break;
					}
				}
			}
			
			lstFinal.addAll(lst);
			
			cohorts = lstFinal;
		}
		return cohorts;
	}
	
	public Vector<ClassificationCohort> getLeafCohorts()
	{
		Vector<ClassificationCohort> lst = new Vector<ClassificationCohort>();
		
		lst.addAll(root.children);
		
		boolean bRepeat = true;
		while (bRepeat)
		{
			bRepeat = false;
			for (int i = 0; i < lst.size(); i++)
			{
				ClassificationCohort cc = lst.get(i);
				if (!cc.children.isEmpty())
				{
					lst.remove(cc);
					lst.addAll(cc.children);
					
					bRepeat = true;
					break;
				}
			}
		}
		
		return lst;
	}
	
	public Vector<String> getCohortNames()
	{
		if (cohortLst == null)
		{
			//sort cohorts by id to make sure ids can be used as index into name list
			Vector<ClassificationCohort> lst = new Vector<ClassificationCohort>(getCohortsHelper());
			Collections.sort(lst , new Comparator<ClassificationCohort>() {
		        public int compare(ClassificationCohort cc1, ClassificationCohort cc2) 
		        {
		        	return cc1.getID() - cc2.getID();
		          }
		        });
			
			Vector<String> lst2 = new Vector<String>();
			for (ClassificationCohort cc : lst)
			{
				lst2.add(cc.getName());
			}
			cohortLst = lst2;
		}
		return new Vector<String>(cohortLst);
	}
	
	public int mapTrace2Cohort(XTrace trc)
	{
//		//int i = 0;
//		for (ClassificationCohort cc : cohorts)
//		{
//			if (cc.isPartOfCohort(trc))
//			{
//				return cc.getID();
//			}
//			
//			//i++;
//		}
		return root.mapTraceToCohortID(trc);
		
		//return -1;
	}
	
	public boolean cohortOverlap(XTrace trc)
	{
		int numCohorts = 0;
		for (ClassificationCohort cc : getCohortsHelper())
		{
			if (cc.isPartOfCohort(trc))
			{
				numCohorts++;
			}
		}
		
		return numCohorts > 1;
	}
	
	public HashMap<Integer, Integer> getCohortCaseHistogram(XLog log)
	{
		HashMap<Integer, Integer> map = new HashMap<Integer, Integer>();
		
		int[] caseCount = new int[getNumCohorts()];
		int unclassifiedCount = 0;
		int totalCount = 0;
		
		for (XTrace xt : log)
		{
			int cohort = mapTrace2Cohort(xt);
			if (cohort >= 0)
			{
				caseCount[cohort] = caseCount[cohort] + 1;
			}
			else
			{
				unclassifiedCount++;
			}
			totalCount++;
		}
		
		map.put(-2, totalCount);
		map.put(-1, unclassifiedCount);
		
		for (int i = 0; i < getNumCohorts(); i++)
		{
			map.put(i, caseCount[i]);
		}
		
		return map;
	}
	
	public static CohortClassifier Create(final UIPluginContext context, XLog log)
	{
		ListWizard<CohortClassifier> wizard =
			      new ListWizard<CohortClassifier>(
			        new CohortConfigurationStep(log),
			        new CohortConfigurationStep2(log));
			    return ProMWizardDisplay.show(context, wizard, null);	
	}
	
	public CohortPreset getPreset()
	{
		CohortPreset cp = new CohortPreset();

		for (int i = 0; i < getNumCohorts(); i++)
		{
			ClassificationCohort ccl = getCohortsHelper().get(i);
			cp.cohortDefs.add(ccl.getCohortDefinition());
		}
		
		return cp;
	}
	
	public void addCohort(CohortDefinition cd)
	{
		addCohort(cd.createClassificationCohort());
	}
	
	public void addCohort(ClassificationCohort cc)
	{
		root.addChild(cc);

		updateCohortData();
	}
	
	public void addCohort(CohortDefinition cd, String parentName)
	{
		addCohort(cd.createClassificationCohort(), parentName);
	}
	
	public void addCohort(ClassificationCohort cc, String parentName)
	{
		for (ClassificationCohort cp : getCohorts())
		{
			if (cp.getName().equals(parentName))
			{
				cp.addChild(cc);
			}
		}

		updateCohortData();
	}
	
	public void removeCohort(String cohortName)
	{
		for (int i = 0; i < getNumCohorts(); i++)
		{
			if (getCohortsHelper().get(i).getName() == cohortName)
			{
				ClassificationCohort cc = getCohortsHelper().get(i);
				if (cc.parent != null)
				{
					cc.parent.removeChild(cc);
				}
				break;
			}
		}
		
		updateCohortData();
	}
	
	private void updateCohortData()
	{
		cohorts = null;
		cohortLst = null;
		
		//update cohortIDs
		generateCohortIDs();		
	}
	
	public static Vector<String> getPresetNames(String attributeName, XLog l)
	{
		Vector<String> presets = new Vector<String>();
		presets.add("None");

		LogAnalyser la = new LogAnalyser(l);
		if (la.isTraceAttrLiteral(attributeName))
		{
    		presets.addAll(ClassificationCohortLiteral.getPresetNames());
		}
		else if (la.isTraceAttrTimestamp(attributeName))
		{
    		presets.addAll(ClassificationCohortTimeOfDay.getPresetNames());
		}
		else if (la.isTraceAttrDiscrete(attributeName))
		{
    		presets.addAll(ClassificationCohortIntRange.getPresetNames());
		}
		
		for (CohortPreset cp : customPresets)
		{
			presets.add(cp.presetName);
		}
		
		return presets;
	}
	
	public static CohortPreset getPreset(String presetName, String selectedAttribute, XLog l)
	{
		for (CohortPreset cp : customPresets)
		{
			if (cp.presetName.equals(presetName))
			{
				return cp;
			}
		}
		
		LogAnalyser la = new LogAnalyser(l);
		if (la.isTraceAttrLiteral(selectedAttribute))
		{
			return ClassificationCohortLiteral.getPreset(presetName, selectedAttribute, l);
		}
		else if (la.isTraceAttrTimestamp(selectedAttribute))
		{
			return ClassificationCohortTimeOfDay.getPreset(presetName, selectedAttribute, l);
		}
		else if (la.isTraceAttrDiscrete(selectedAttribute))
		{
			return ClassificationCohortIntRange.getPreset(presetName, selectedAttribute, l);
		}
		
		return null;
	}
	
	public void setPreset(CohortPreset cp)
	{
		for (CohortDefinition cd : cp.cohortDefs)
		{
			addCohort(cd);
		}
	}
	
	private void generateCohortIDs()
	{
		Vector<ClassificationCohort> lst = new Vector<ClassificationCohort>(getCohortsHelper());
		Collections.sort(lst , new Comparator<ClassificationCohort>() {
	        public int compare(ClassificationCohort cc1, ClassificationCohort cc2) 
	        {
	        	return cc1.getLevel() - cc2.getLevel();
	          }
	        });
		
		for (int i = 0; i < lst.size(); i++)
		{
			ClassificationCohort ccl = lst.get(i);
			ccl.setID(i);
		}
	}
}
