package org.processmining.processprofiler3d.psen.datastructure;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;

public class PSDBStatementSet
{
	public PreparedStatement addValueTraceStatement = null;
	public PreparedStatement getValueStatement = null;
	public PreparedStatement hasValueStatementT = null;
	public PreparedStatement hasValueStatementTL = null;
	public PreparedStatement hasValueStatementTLC = null;
	public PreparedStatement hasValueStatementTIN = null;
	public PreparedStatement hasValueStatementTLIN = null;
	public PreparedStatement hasValueStatementTLCIN = null;
	public PreparedStatement getClassifiersStatement = null;
	public PreparedStatement getTransitionsStatement = null;
	public PreparedStatement getTransitionsINStatement = null;

	public PSDBStatementSet(Connection con, String tblPSENName, String tblTempName)
	{
		try
		{	
			hasValueStatementT = con.prepareStatement("SELECT COUNT(value) As result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=? AND classifier1Name<>-1");
			hasValueStatementTL = con.prepareStatement("SELECT COUNT(value) As result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=? AND labelName=? AND classifier1Name<>-1");
			hasValueStatementTLC = con.prepareStatement("SELECT COUNT(value) As result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=? AND labelName=? AND classifier1Name=?");
			hasValueStatementTIN = con.prepareStatement("SELECT COUNT(value) As result FROM " + tblPSENName + " WHERE transitionID=?");
			hasValueStatementTLIN = con.prepareStatement("SELECT COUNT(value) As result FROM " + tblPSENName + " WHERE transitionID=? AND labelName=?");
			hasValueStatementTLCIN = con.prepareStatement("SELECT COUNT(value) As result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=? AND labelName=? AND classifier1Name=?");
			getValueStatement = con.prepareStatement("SELECT value FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=? AND labelName=? AND classifier1Name=?");
			addValueTraceStatement = con.prepareStatement("INSERT INTO " + tblPSENName + " ( transitionID, labelName, traceID, value ) VALUES (?,?,?,?)");
			
			getClassifiersStatement = con.prepareStatement("SELECT DISTINCT classifier1Name FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE classifier1Name<>-1");
			getTransitionsStatement = con.prepareStatement("SELECT DISTINCT transitionID FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE classifier1Name<>-1");
			getTransitionsINStatement = con.prepareStatement("SELECT DISTINCT transitionID FROM " + tblPSENName);
		}
		catch(SQLException se)
		{
			System.out.println( se.getMessage() );
			se.printStackTrace();
		}
		catch( Exception e )
		{
			System.out.println( e.getMessage() );
			e.printStackTrace();
		}
	}
	
	public void Close(Connection con)
	{
		if (con != null)
		{
			try 
			{
				hasValueStatementT.close();
				hasValueStatementTL.close();
				hasValueStatementTLC.close();
				getValueStatement.close();
				getClassifiersStatement.close();
				getTransitionsStatement.close();
				con.close();
			} 
			catch (SQLException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public boolean hasValue(int tid)
	{
		try
		{
	        hasValueStatementT.setInt(1, tid);
	        ResultSet rs = hasValueStatementT.executeQuery();
	        rs.next();
	        int rowNum = rs.getInt("result");
			
			return rowNum > 0;
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	public boolean hasValue(int tid, PSENLabel label)
	{
		try
		{
	        hasValueStatementTL.setInt(1, tid);
	        hasValueStatementTL.setInt(2, label.lid);
	        ResultSet rs = hasValueStatementTL.executeQuery();
	        rs.next();
	        int rowNum = rs.getInt("result");
			
			return rowNum > 0;
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	public boolean hasValue(int tid, PSENLabel label, int cid)
	{
		try
		{
			hasValueStatementTLC.setInt(1, tid);
	        hasValueStatementTLC.setInt(2, label.lid);
	        hasValueStatementTLC.setInt(3, cid);
	        ResultSet rs = hasValueStatementTLC.executeQuery();
	        rs.next();
	        int rowNum = rs.getInt("result");
			
			return rowNum > 0;
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	public boolean hasValue2(int tid)
	{
		try
		{
	        hasValueStatementTIN.setInt(1, tid);
	        ResultSet rs = hasValueStatementTIN.executeQuery();
	        rs.next();
	        int rowNum = rs.getInt("result");
			
			return rowNum > 0;
		} 
		catch (SQLException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	public boolean hasValue2(int tid, PSENLabel label)
	{
		try
		{
	        hasValueStatementTLIN.setInt(1, tid);
	        hasValueStatementTLIN.setInt(2, label.lid);
	        ResultSet rs = hasValueStatementTLIN.executeQuery();
	        rs.next();
	        int rowNum = rs.getInt("result");
			
			return rowNum > 0;
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	public boolean hasValue2(int tid, PSENLabel label, int cid)
	{
		try
		{
	        hasValueStatementTLCIN.setInt(1, tid);
	        hasValueStatementTLCIN.setInt(2, label.lid);
	        hasValueStatementTLCIN.setInt(3, cid);
	        ResultSet rs = hasValueStatementTLCIN.executeQuery();
	        rs.next();
	        int rowNum = rs.getInt("result");
			
			return rowNum > 0;
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
//	public double getValue(int tid, PSENLabel label, int cid)
//	{
//		try
//		{
//			String aggOperator = label.getSQLAggOperator();
//			
//			if (aggOperator != "")
//			{
//				Statement stmt = con.createStatement();
//				ResultSet rs = stmt.executeQuery("SELECT " + aggOperator + "(value) AS result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=" + tid + " AND labelName=" + label.lid + " AND classifier1Name=" + cid);
//
//				if (rs.next())
//				{
//					double result = rs.getDouble("result");
//					return result;
//				}
//				
//				stmt.close();
//			}
//		} 
//		catch (SQLException e) 
//		{
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//		
//		return 0.0;
//	}
	
	public Vector<Double> getValues(int tid, PSENLabel label, int cid)
	{
		Vector<Double> lst = new Vector<Double>();
		try
		{
	        getValueStatement.setInt(1, tid);
	        getValueStatement.setInt(2, label.lid);
	        getValueStatement.setInt(3, cid);
			ResultSet rs = getValueStatement.executeQuery();

			while (rs.next())
			{
				lst.add(rs.getDouble("value"));
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return lst;
	}
	
//	public HashMap<String, Double> getTransitionLabelValueGroups(int tid, PSENLabel label)
//	{
//		HashMap<String, Double> lst = new HashMap<String, Double>();
//		try
//		{
//			String aggOperator = label.getSQLAggOperator();
//			
//			if (aggOperator != "")
//			{
//		        Statement stmt = con.createStatement();
//		        ResultSet rs = stmt.executeQuery("SELECT classifier1Name, " + aggOperator + "(value) AS result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=" + tid + " AND labelName=" + label.lid + " GROUP BY classifier1Name");
//		    	
//				while (rs.next())
//				{
//					String s = numToClass.get(rs.getInt("classifier1Name"));
//					lst.put(s, rs.getDouble("result"));
//					//System.out.println(s);
//				}
//				stmt.close();
//			}
//		} 
//		catch (SQLException e) 
//		{
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//		return lst;
//	}
	
//	public HashMap<String, Double> getLabelValueGroups(PSENLabel label)
//	{
//		HashMap<String, Double> lst = new HashMap<String, Double>();
//		try
//		{
//			String aggOperator = label.getSQLAggOperator();
//			
//			if (aggOperator != "")
//			{
//		        Statement stmt = con.createStatement();
//		        ResultSet rs = stmt.executeQuery("SELECT transitionID, classifier1Name, " + aggOperator + "(value) AS result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE labelName=" + label.lid + " GROUP BY transitionID, classifier1Name");
//		    	
//				while (rs.next())
//				{
//					String s = numToTid.get(rs.getInt("transitionID")) + "-" + numToClass.get(rs.getInt("classifier1Name"));
//					lst.put(s, rs.getDouble("result"));
//					//System.out.println(s);
//				}
//				stmt.close();
//			}
//		} 
//		catch (SQLException e) 
//		{
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//		return lst;
//	}
	
//	public void getLabelValueGroups2(int tid, PSENLabel label, Vector<String> vst, Vector<Double> dst)
//	{
//		try
//		{
//	        Statement stmt = con.createStatement();
//	        ResultSet rs = stmt.executeQuery("SELECT traceID, value AS result FROM " + tblPSENName + " WHERE transitionID=" + tid + " AND labelName=" + label.lid);
//	    	
//			while (rs.next())
//			{
//				int traceId = rs.getInt("traceID");
//				String traceName = numToTrace.get(traceId);
//				vst.add(traceName);
//				dst.add(rs.getDouble("result"));
//			}
//			stmt.close();
//		} 
//		catch (SQLException e) 
//		{
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//	}

}
