package org.processmining.processprofiler3d.psen.datastructure;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.Vector;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.compviz.TransitionPair;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.processprofiler3d.psen.preprocess.LPPAction;

public class PerformanceStatisticsDB implements IPerformanceStatisticsEnrichedNet
{
	private static int tempNum = 1;
	
    private String tblPSENName = "table1";
    public String tblTempName = "temp1";
    public String idxName = "traceIdx1";
    
    private Connection con;
	
    public PerformanceDBLabelStore labels = new PerformanceDBLabelStore();
	public PerformanceDBKeyStore keyStore = new PerformanceDBKeyStore();

	private PreparedStatement addValueTraceStatement = null;
	private PreparedStatement getValueStatement = null;
	private PreparedStatement hasValueStatementT = null;
	private PreparedStatement hasValueStatementTL = null;
	private PreparedStatement hasValueStatementTLC = null;
	private PreparedStatement hasValueStatementTIN = null;
	private PreparedStatement hasValueStatementTLIN = null;
	private PreparedStatement hasValueStatementTLCIN = null;
	private PreparedStatement getClassifiersStatement = null;
	private PreparedStatement getTransitionsStatement = null;
	private PreparedStatement getTransitionsINStatement = null;
	
	public Collection<PSENLabel> getLabels()
	{
		return this.labels.values();
	}

	public void addLabel(PSENLabel l)
	{
		labels.addLabel(l);
	}

	public boolean hasLabel(String labelName)
	{
		return labels.hasLabel(labelName);
	}

	public PSENLabel getLabelById(int labelId)
	{
		return labels.getLabelById(labelId);
	}

	public PSENLabel getLabelByName(String labelName)
	{
		return labels.getLabelByName(labelName);
	}
	
	public PerformanceStatisticsDB(H2MemoryDatabase h2db)
	{
		con = h2db.getConnection();
		//DB_CONNECTION = "jdbc:h2:mem:test;DB_CLOSE_DELAY=-1";
		//dbNum++;
	}
	
	public Connection getConnection()
	{
		return con;
	}
	
	public int getValueCount()
	{
		try 
		{
	        Statement stmt = con.createStatement();
	        ResultSet rs = stmt.executeQuery("SELECT COUNT(value) AS result FROM " + tblPSENName);
	        int count = 0;
	        while (rs.next())
	        {
	        	count = rs.getInt("result");
	        }
			return count;
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return 0;
	}
	
	public void InitPSEN()
	{
		tblPSENName = "tablePSEN";
		//tblTempName = "temp" + tempNum;
		tblTempName = "tableTrace";
		idxName = "traceIdx" + tempNum;
		tempNum++;
		Init();
	}
	
	public void InitWTEN()
	{
		tblPSENName = "tableWTEN";
		//tblTempName = "temp" + tempNum;
		tblTempName = "tableTrace";
		idxName = "traceIdx" + tempNum;
		tempNum++;
		Init();
	}
	
	public void Init()
	{	
		try
		{
	        //create PSEN table in database
	        Statement stmt = con.createStatement();
	        stmt.executeUpdate( "CREATE TABLE " + tblPSENName + " ( rowID int NOT NULL PRIMARY KEY AUTO_INCREMENT, transitionID SMALLINT, labelName SMALLINT, traceID int, value double )" );
	        //register custom median aggregator function
	        stmt.execute("CREATE AGGREGATE IF NOT EXISTS MEDIAN FOR \"" + MedianTest.class.getName() + "\"");

	        //create trace table in database
			stmt.executeUpdate( "CREATE TABLE IF NOT EXISTS " + tblTempName + " ( traceID int NOT NULL PRIMARY KEY, classifier1Name SMALLINT )" );
			stmt.close();
			
			hasValueStatementT = con.prepareStatement("SELECT COUNT(value) As result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=? AND classifier1Name<>-1");
			hasValueStatementTL = con.prepareStatement("SELECT COUNT(value) As result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=? AND labelName=? AND classifier1Name<>-1");
			hasValueStatementTLC = con.prepareStatement("SELECT COUNT(value) As result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=? AND labelName=? AND classifier1Name=?");
			hasValueStatementTIN = con.prepareStatement("SELECT COUNT(value) As result FROM " + tblPSENName + " WHERE transitionID=?");
			hasValueStatementTLIN = con.prepareStatement("SELECT COUNT(value) As result FROM " + tblPSENName + " WHERE transitionID=? AND labelName=?");
			hasValueStatementTLCIN = con.prepareStatement("SELECT COUNT(value) As result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=? AND labelName=? AND classifier1Name=?");
			getValueStatement = con.prepareStatement("SELECT value FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=? AND labelName=? AND classifier1Name=?");
			addValueTraceStatement = con.prepareStatement("INSERT INTO " + tblPSENName + " ( transitionID, labelName, traceID, value ) VALUES (?,?,?,?)");
			
			getClassifiersStatement = con.prepareStatement("SELECT DISTINCT classifier1Name FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE classifier1Name<>-1");
			getTransitionsStatement = con.prepareStatement("SELECT DISTINCT transitionID FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE classifier1Name<>-1");
			getTransitionsINStatement = con.prepareStatement("SELECT DISTINCT transitionID FROM " + tblPSENName);
		}
		catch(SQLException se)
		{
			System.out.println( se.getMessage() );
			se.printStackTrace();
		}
		catch( Exception e )
		{
			System.out.println( e.getMessage() );
			e.printStackTrace();
		}
	}
	
	public void Close()
	{
		if (con != null)
		{
			try 
			{
				hasValueStatementT.close();
				hasValueStatementTL.close();
				hasValueStatementTLC.close();
				hasValueStatementTIN.close();
				hasValueStatementTLIN.close();
				hasValueStatementTLCIN.close();
				getValueStatement.close();
				addValueTraceStatement.close();
				
				getClassifiersStatement.close();
				getTransitionsStatement.close();
				getTransitionsINStatement.close();
				con.close();
			} 
			catch (SQLException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public boolean hasValue(String transition)
	{
		try
		{
			if (keyStore.tidToNum.containsKey(transition))
			{
				int tid = keyStore.tidToNum.get(transition);
		        hasValueStatementT.setInt(1, tid);
		        ResultSet rs = hasValueStatementT.executeQuery();
		        rs.next();
		        int rowNum = rs.getInt("result");
				
				return rowNum > 0;
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	public boolean hasValue(String transition, String labelName)
	{
		PSENLabel label = getLabelByName(labelName);
		try
		{
			if (keyStore.tidToNum.containsKey(transition))
			{
				int tid = keyStore.tidToNum.get(transition);
		        hasValueStatementTL.setInt(1, tid);
		        hasValueStatementTL.setInt(2, label.lid);
		        ResultSet rs = hasValueStatementTL.executeQuery();
		        rs.next();
		        int rowNum = rs.getInt("result");
				
				return rowNum > 0;
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	public boolean hasValue(String transition, String labelName, String classifierValue)
	{
		PSENLabel label = getLabelByName(labelName);
		try
		{
			if (keyStore.tidToNum.containsKey(transition))
			{
				int tid = keyStore.tidToNum.get(transition);
				int cid = keyStore.classToNum.get(classifierValue);
				hasValueStatementTLC.setInt(1, tid);
		        hasValueStatementTLC.setInt(2, label.lid);
		        hasValueStatementTLC.setInt(3, cid);
		        ResultSet rs = hasValueStatementTLC.executeQuery();
		        rs.next();
		        int rowNum = rs.getInt("result");
				
				return rowNum > 0;
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	public boolean hasValue2(String transition)
	{
		try
		{
			if (keyStore.tidToNum.containsKey(transition))
			{
				int tid = keyStore.tidToNum.get(transition);
		        hasValueStatementTIN.setInt(1, tid);
		        ResultSet rs = hasValueStatementTIN.executeQuery();
		        rs.next();
		        int rowNum = rs.getInt("result");
				
				return rowNum > 0;
			}
		} 
		catch (SQLException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	public boolean hasValue2(String transition, String labelName)
	{
		PSENLabel label = getLabelByName(labelName);
		try
		{
			if (keyStore.tidToNum.containsKey(transition))
			{
				int tid = keyStore.tidToNum.get(transition);
		        hasValueStatementTLIN.setInt(1, tid);
		        hasValueStatementTLIN.setInt(2, label.lid);
		        ResultSet rs = hasValueStatementTLIN.executeQuery();
		        rs.next();
		        int rowNum = rs.getInt("result");
				
				return rowNum > 0;
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	public boolean hasValue2(String transition, String labelName, String classifierValue)
	{
		PSENLabel label = getLabelByName(labelName);
		try
		{
			if (keyStore.tidToNum.containsKey(transition))
			{
				int tid = keyStore.tidToNum.get(transition);
				int cid = keyStore.classToNum.get(classifierValue);
		        hasValueStatementTLCIN.setInt(1, tid);
		        hasValueStatementTLCIN.setInt(2, label.lid);
		        hasValueStatementTLCIN.setInt(3, cid);
		        ResultSet rs = hasValueStatementTLCIN.executeQuery();
		        rs.next();
		        int rowNum = rs.getInt("result");
				
				return rowNum > 0;
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	public double getValue(String transition, String labelName, String classifierValue)
	{
		PSENLabel label = getLabelByName(labelName);
		try
		{
			String aggOperator = label.getSQLAggOperator();
			
			if (aggOperator != "")
			{
				if (keyStore.tidToNum.containsKey(transition))
				{
					int tid = keyStore.tidToNum.get(transition);
					int cid = keyStore.classToNum.get(classifierValue);
					Statement stmt = con.createStatement();
					ResultSet rs = stmt.executeQuery("SELECT " + aggOperator + "(value) AS result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=" + tid + " AND labelName=" + label.lid + " AND classifier1Name=" + cid);
	
					if (rs.next())
					{
						double result = rs.getDouble("result");
						return result;
					}
					
					stmt.close();
				}
				else
				{
					System.out.println("No database ID found for transition (" + transition + ")!");
				}
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return 0.0;
	}
	
	public Vector<Double> getValues(String transition, String labelName, String classifierValue)
	{
		Vector<Double> lst = new Vector<Double>();
		PSENLabel label = getLabelByName(labelName);
		try
		{
			if (keyStore.tidToNum.containsKey(transition))
			{
				int tid = keyStore.tidToNum.get(transition);
				int cid = keyStore.classToNum.get(classifierValue);
		        getValueStatement.setInt(1, tid);
		        getValueStatement.setInt(2, label.lid);
		        getValueStatement.setInt(3, cid);
				ResultSet rs = getValueStatement.executeQuery();
	
				while (rs.next())
				{
					lst.add(rs.getDouble("value"));
				}
			}
			else
			{
				System.out.println("No database ID found for transition (" + transition + ")!");
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return lst;
	}
	
	public HashMap<String, Double> getTransitionLabelValueGroups(String tId, String labelName)
	{
		if (tblSummary != null)
		{
			return tblSummary.getTransitionLabelValueGroups(tId, labelName);
		}
		return new HashMap<String, Double>();
	}
	
	public HashMap<String, Double> getTransitionLabelValueGroupsOptT(String tId)
	{
		if (tblSummary != null)
		{
			return tblSummary.getTransitionLabelValueGroupsOptT(tId);
		}
		return new HashMap<String, Double>();
	}
	
	public void getTransitionLabelValueGroupsOptTbl(String tblName)
	{
		tblSummary = new PerformanceSummaryTable(tblName, con);
		tblSummary.Init(tblPSENName, tblTempName);
		
		tblSummary.keyStore = keyStore;
		tblSummary.labels = labels;
	}
	
	private PerformanceSummaryTable tblSummary;
	
	public HashMap<String, Double> getLabelValueGroups(String labelName)
	{
		if (tblSummary != null)
		{
			return tblSummary.getLabelValueGroups(labelName);
		}
		return new HashMap<String, Double>();
	}
	
	public void getLabelValueGroups2(String transitionId, String labelName, Vector<String> vst, Vector<Double> dst)
	{
		PSENLabel label = getLabelByName(labelName);
		try
		{
			if (keyStore.tidToNum.containsKey(transitionId))
			{
				int tid = keyStore.tidToNum.get(transitionId);
		        Statement stmt = con.createStatement();
		        ResultSet rs = stmt.executeQuery("SELECT traceID, value AS result FROM " + tblPSENName + " WHERE transitionID=" + tid + " AND labelName=" + label.lid);
		    	
				while (rs.next())
				{
					int traceId = rs.getInt("traceID");
					String traceName = keyStore.numToTrace.get(traceId);
					vst.add(traceName);
					dst.add(rs.getDouble("result"));
				}
				stmt.close();
			}
			else
			{
				System.out.println("No database ID found for transition (" + transitionId + ")!");
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void addValue(String transition, String labelName, String traceId, double myValue)
	{
		PSENLabel label = getLabelByName(labelName);
		try 
		{
			int tid = keyStore.regTransition(transition);
			int trid = keyStore.regTrace(traceId);
			addValueTraceStatement.setInt(1, tid);
			addValueTraceStatement.setInt(2, label.lid);
			addValueTraceStatement.setInt(3, trid);
			addValueTraceStatement.setDouble(4, myValue);
			addValueTraceStatement.executeUpdate();
		}
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public PreparedStatement getBatchAddFor(String transition, String labelName)
	{

		PSENLabel label = getLabelByName(labelName);
		try 
		{
			con.setAutoCommit(false);
			
			int tid = keyStore.regTransition(transition);
			addValueTraceStatement.setInt(1, tid);
			addValueTraceStatement.setInt(2, label.lid);
		}
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return addValueTraceStatement;
	}

	public void commitBatchAdd()
	{
		try
		{
			addValueTraceStatement.executeBatch();
			addValueTraceStatement.clearBatch();
			
			con.commit();
			con.setAutoCommit(true);
		}
		catch (SQLException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public int regTracePublic(String traceId)
	{
		return keyStore.regTrace(traceId);
	}
	
	public Vector<XTrace> getValidTraces(XLog log)
	{
		Vector<XTrace> lst = null;
		HashSet<String> lstStr = new HashSet<String>();
		try 
		{
			Statement stmt4 = con.createStatement();
			ResultSet rs = stmt4.executeQuery( "SELECT DISTINCT traceID FROM " + tblPSENName );
			
			while (rs.next())
			{
				String trName = keyStore.numToTrace.get(rs.getInt("traceID"));
				lstStr.add(trName);
			}
			
			stmt4.close();

			lst = new Vector<XTrace>(lstStr.size());
			for (XTrace tr : log)
			{
				String tName = LPPAction.getAttributeValueLiteral(tr, XConceptExtension.KEY_NAME);
				if (lstStr.contains(tName))
				{
					lst.add(tr);
				}
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if (lst == null)
		{
			lst = new Vector<XTrace>(lstStr.size());
		}
		
		System.out.println("Trace list (" + lst.size() + "/" + log.size() + ")");
		
		return lst;
	}
	
	public void setCohortsForTraces(Iterable<XTrace> log, CohortClassifier cc)
	{
		try 
		{
			keyStore.classToNum.clear();
			keyStore.numToClass.clear();
			
			Statement stmt4 = con.createStatement();
			stmt4.executeUpdate( "DELETE FROM " + tblTempName );
			stmt4.close();
			
	        PreparedStatement stmt = con.prepareStatement("INSERT INTO " + tblTempName + " ( traceID, classifier1Name ) VALUES (?,?)");
	        for (XTrace tr : log)
			{
				String traceID = LPPAction.getAttributeValueLiteral(tr, XConceptExtension.KEY_NAME);
				int trid = keyStore.regTrace(traceID);
				int cohortId = cc.mapTrace2Cohort(tr);
				String cohortName = "None";
				if (cohortId != -1)
				{
					cohortName = cc.getCohortNames().get(cohortId);
				}

				stmt.setInt(1, trid);
				stmt.setInt(2, keyStore.regClass(cohortName));
				stmt.addBatch();
			}
			stmt.executeBatch();
			stmt.close();
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void update() 
	{
		//not needed anymore, so keep empty for now
		createIndex();
	}
	
	public void createIndex()
	{
		try 
		{
	        Statement stmt = con.createStatement();
	        //stmt.executeUpdate("CREATE INDEX " + idxName + " ON " + tblPSENName + " (traceID)");
	        //stmt.executeUpdate("CREATE INDEX " + idxName + " ON " + tblPSENName + " (transitionID, labelName, classifier1Name)");
	        stmt.executeUpdate("CREATE INDEX " + idxName + " ON " + tblPSENName + " (transitionID, labelName)");
	        //stmt.executeUpdate("CREATE TABLE " + tblTempName + " AS (SELECT DISTINCT traceID, classifier1Name FROM " + tblPSENName + ")");
			stmt.close();
		}
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void dropIndex()
	{
		try 
		{
	        Statement stmt = con.createStatement();
	        stmt.executeUpdate("DROP INDEX " + idxName);
			stmt.close();
		}
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public HashSet<String> getClassifierValues() 
	{
		HashSet<String> lst = new HashSet<String>();
		try 
		{
	        ResultSet rs = getClassifiersStatement.executeQuery();
			while( rs.next() )
			{
				lst.add(keyStore.numToClass.get(rs.getInt("classifier1Name")));
			    //System.out.println( name );
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return lst;
	}

	public HashSet<String> getClassifierValuesFast() 
	{
		if (tblSummary != null)
		{
			return tblSummary.getClassifierValuesFast();
		}
		return new HashSet<String>();
	}

	public Set<String> getTransitions() 
	{
		HashSet<String> lst = new HashSet<String>();
		try 
		{
	        ResultSet rs = getTransitionsStatement.executeQuery();
			while( rs.next() )
			{
			    lst.add(keyStore.numToTid.get(rs.getInt("transitionID")));
			    //System.out.println( name );
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return lst;
	}

	public Set<String> getTransitionsFast() 
	{
		if (tblSummary != null)
		{
			return tblSummary.getTransitionsFast();
		}
		return new HashSet<String>();
	}

	public Set<String> getTransitionsInclNone() 
	{
		HashSet<String> lst = new HashSet<String>();
		try 
		{
	        ResultSet rs = getTransitionsINStatement.executeQuery();
			while( rs.next() )
			{
			    lst.add(keyStore.numToTid.get(rs.getInt("transitionID")));
			    //System.out.println( name );
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return lst;
	}

	public void MergeDB(PerformanceStatisticsDB db2)
	{
		try 
		{
			Statement stmt = db2.getConnection().createStatement();
			ResultSet rs = stmt.executeQuery("SELECT * FROM " + tblPSENName);
			while( rs.next() )
			{
//				String labelNodeName = rs.getString("labelName");
//				PSENLabel myLabel = null;
//				for (PSENLabel l : labels.values())
//				{
//					if (l.getNodeName().equals(labelNodeName))
//					{
//						myLabel = l;
//						break;
//					}
//				}
				int labelId = rs.getInt("labelName");
				PSENLabel myLabel = db2.getLabelById(labelId);
				if (myLabel != null)
				{
					addValue(db2.keyStore.numToTid.get(rs.getInt("transitionID")), myLabel.getLabelName(), db2.keyStore.numToTrace.get(rs.getInt("traceID")), rs.getDouble("value"));
				}
				else
				{
					System.out.println("Error! Label not found!");
				}
			}
			stmt.close();
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public boolean fitsPetrinet(PetrinetGraph pnet) 
	{
		HashSet<String> idx = new HashSet<String>();
		PetriNetDataSource pnds = new PetriNetDataSource(pnet);
		for (Transition t : pnds.getAllTransitions())
		{
			idx.add(t.getLocalID().toString());
		}
		
		for (String tId : this.getTransitions())
		{
			if (tId.length() == 36) //is transition localId
			{
				boolean existsInNet = false;
				//existsInNet = pnds.existsInNet(tId);
				existsInNet = idx.contains(tId);
				if (!existsInNet)
				{
					return false;
				}
			}
			else //is localId of transitionPair
			{
				//boolean existsInNetSource = pnds.existsInNet(TransitionPair.getSourceId(tId));
				//boolean existsInNetTarget = pnds.existsInNet(TransitionPair.getTargetId(tId));
				boolean existsInNetSource = idx.contains(TransitionPair.getSourceId(tId));
				boolean existsInNetTarget = idx.contains(TransitionPair.getTargetId(tId));
				if (!existsInNetSource || !existsInNetTarget)
				{
					return false;
				}
			}
		}
		
		return true;
	}
}
