package org.processmining.processprofiler3d.psen.datastructure;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

public class PerformanceSummaryTable
{
	private Connection con;
	private String tblCalcTmpName = "tableCalcTmp";
	
	public PerformanceDBLabelStore labels;	
	public PerformanceDBKeyStore keyStore = null;

	private PreparedStatement getClassifiersStatementFast = null;
	private PreparedStatement getTransitionsStatementFast = null;
	
	public PerformanceSummaryTable(String tblName, Connection con)
	{
		tblCalcTmpName = tblName;
		
		this.con = con;
	}
	
	public void Init(String tblPSENName, String tblTempName)
	{
		//check if table already exists
		if (TableExists())
		{
			//if table exists, delete it
	        Dispose();
		}
			
		try
		{
	        Statement stmt = con.createStatement();
	        stmt.execute("CREATE TABLE " + tblCalcTmpName + " AS SELECT transitionID, labelName, classifier1Name, MIN(value) AS resultMIN, MAX(value) AS resultMAX, AVG(value) AS resultAVG, COUNT(value) AS resultCOUNT, SUM(value) AS resultSUM, MEDIAN(value) AS resultMEDIAN FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " GROUP BY labelName, transitionID, classifier1Name");
			stmt.close();
			
			getClassifiersStatementFast = con.prepareStatement("SELECT DISTINCT classifier1Name FROM " + tblCalcTmpName + " WHERE classifier1Name<>-1");
			getTransitionsStatementFast = con.prepareStatement("SELECT DISTINCT transitionID FROM " + tblCalcTmpName + " WHERE classifier1Name<>-1");
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public boolean TableExists()
	{
		boolean tableExists = false;

		try
		{
			ResultSet rset = con.getMetaData().getTables(null, null, tblCalcTmpName.toUpperCase(), null);
			if (rset.next())
			{
				tableExists = true;
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return tableExists;
	}
	
	public void Dispose()
	{
		try
		{
	        Statement stmt = con.createStatement();
	        stmt.execute("DROP TABLE " + tblCalcTmpName);
			stmt.close();
			
			if (getClassifiersStatementFast != null)
			{
				getClassifiersStatementFast.close();
				getClassifiersStatementFast = null;
			}
			if (getTransitionsStatementFast != null)
			{
				getTransitionsStatementFast.close();
				getTransitionsStatementFast = null;
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public HashSet<String> getClassifierValuesFast() 
	{
		HashSet<String> lst = new HashSet<String>();
		try 
		{
	        ResultSet rs = getClassifiersStatementFast.executeQuery();
			while( rs.next() )
			{
				lst.add(keyStore.numToClass.get(rs.getInt("classifier1Name")));
			    //System.out.println( name );
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return lst;
	}

	public Set<String> getTransitionsFast() 
	{
		HashSet<String> lst = new HashSet<String>();
		try 
		{
	        ResultSet rs = getTransitionsStatementFast.executeQuery();
			while( rs.next() )
			{
			    lst.add(keyStore.numToTid.get(rs.getInt("transitionID")));
			    //System.out.println( name );
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return lst;
	}
	
	public HashMap<String, Double> getTransitionLabelValueGroups(String tId, String labelName)
	{
//		if (queryCache.containsKey(tId + labelName))
//		{
//			return (HashMap<String, Double>)queryCache.get(tId + labelName);
//		}
		
		HashMap<String, Double> lst = new HashMap<String, Double>();
		PSENLabel label = labels.getLabelByName(labelName);
		try
		{
			String aggOperator = label.getSQLAggOperator();
			
			if (aggOperator != "")
			{
				if (keyStore.tidToNum.containsKey(tId))
				{
					int tid = keyStore.tidToNum.get(tId);
			        Statement stmt = con.createStatement();
			        //ResultSet rs = stmt.executeQuery("SELECT classifier1Name, " + aggOperator + "(value) AS result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE transitionID=" + tid + " AND labelName=" + label.lid + " GROUP BY classifier1Name");
			        ResultSet rs = stmt.executeQuery("SELECT classifier1Name, result" + aggOperator + " AS result FROM " + tblCalcTmpName + " WHERE transitionID=" + tid + " AND labelName=" + label.lid);
			    	
					while (rs.next())
					{
						String s = keyStore.numToClass.get(rs.getInt("classifier1Name"));
						lst.put(s, rs.getDouble("result"));
						//System.out.println(s);
					}
					stmt.close();
				}
				else
				{
					System.out.println("No database ID found for transition (" + tId + ")!");
				}
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		//queryCache.put(tId + labelName, lst);
		return lst;
	}
	
	public HashMap<String, Double> getTransitionLabelValueGroupsOptT(String tId)
	{
		HashMap<String, Double> lst = new HashMap<String, Double>();
		try
		{
			if (keyStore.tidToNum.containsKey(tId))
			{
				int tid = keyStore.tidToNum.get(tId);
		        Statement stmt = con.createStatement();
		        ResultSet rs = stmt.executeQuery("SELECT labelName, classifier1Name, resultMIN AS result0, resultMAX AS result2, resultAVG AS result1, resultCOUNT AS result3, resultSUM AS result4, resultMEDIAN AS result5 FROM " + tblCalcTmpName + " WHERE transitionID=" + tid + " GROUP BY labelName, transitionID, classifier1Name");
		    	
				while (rs.next())
				{
					//String s = "";
					String sclass = keyStore.numToClass.get(rs.getInt("classifier1Name"));
					String strans = tId;
					int labelId = rs.getInt("labelName");
					for (PSENLabel l : labels.values())
					{
						if (l.lid == labelId)
						{
							lst.put(sclass + "x" + strans + "x" + l.getLabelName(), rs.getDouble("result" + l.getAggregationType()));
						}
					}
					//System.out.println(s);
				}
				stmt.close();
			}
			else
			{
				System.out.println("No database ID found for transition (" + tId + ")!");
			}
		} 
		catch (SQLException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return lst;
	}
	
	public HashMap<String, Double> getLabelValueGroups(String labelName)
	{
		HashMap<String, Double> lst = new HashMap<String, Double>();
		PSENLabel label = labels.getLabelByName(labelName);
		if (label != null)
		{
			try
			{
				String aggOperator = label.getSQLAggOperator();
				
				if (aggOperator != "")
				{
			        Statement stmt = con.createStatement();
			        //ResultSet rs = stmt.executeQuery("SELECT transitionID, classifier1Name, " + aggOperator + "(value) AS result FROM " + tblPSENName + " JOIN " + tblTempName + " ON " + tblPSENName + ".traceID=" + tblTempName + ".traceID" + " WHERE labelName=" + label.lid + " GROUP BY transitionID, classifier1Name");
			        ResultSet rs = stmt.executeQuery("SELECT transitionID, classifier1Name, result" + aggOperator + " AS result FROM " + tblCalcTmpName + " WHERE labelName=" + label.lid + " GROUP BY transitionID, classifier1Name");
			    	
					while (rs.next())
					{
						String s = keyStore.numToTid.get(rs.getInt("transitionID")) + "-" + keyStore.numToClass.get(rs.getInt("classifier1Name"));
						lst.put(s, rs.getDouble("result"));
						//System.out.println(s);
					}
					stmt.close();
				}
			} 
			catch (SQLException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return lst;
	}

}
