package org.processmining.processprofiler3d.psen.fileio;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;
import java.util.Vector;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.providedobjects.ProvidedObjectDeletedException;
import org.processmining.framework.providedobjects.ProvidedObjectID;
import org.processmining.processprofiler3d.psen.connections.PSENConnection;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PSENLabel;
import org.processmining.processprofiler3d.psen.datastructure.PerformanceStatisticsDB;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */

@Plugin(name = "Export Performance Comparison",
parameterLabels = { "PNReplayLogModel", "File" },
returnLabels = {},
returnTypes = {},
userAccessible = true)
@UIExportPlugin(description = "Performance Comparison file",
        extension = "psen")

public class ModelComparisonExport 
{
	  @UITopiaVariant(affiliation = "Queensland University of Technology",
              author = "Erik Poppe",
              email = "erik.poppe@qut.edu.au")

	@PluginVariant(variantLabel = "Export PSEN", requiredParameterLabels = { 0, 1 })
	public void export(final UIPluginContext context, IPerformanceStatisticsEnrichedNet psnet, File file)
	throws IOException 
	{	
		FileWriter writer = new FileWriter(file);
		PrintWriter pwriter = new PrintWriter(writer);

		pwriter.println("<PSENData>");
		List<ProvidedObjectID> pids = context.getProvidedObjectManager().getProvidedObjects();
		String label = "Performance Statistics Enriched Net";
		try 
		{
			label = context.getProvidedObjectManager().getProvidedObjectLabel(pids.get(0));
		} 
		catch (ProvidedObjectDeletedException e1) 
		{
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		WritePSENToFile(pwriter, psnet, label);

		try 
		{
			PSENConnection wtic = context.getConnectionManager().getFirstConnection(PSENConnection.class, context, psnet);
			System.out.println("Found waiting time info connection!");

			WriteWaitingTimeTreeToFile(pwriter, wtic.getWaitingTimeEnrichedNet());
		} 
		catch (ConnectionCannotBeObtained e) 
		{
			System.out.println("Could not find waiting time info connection!");
		}

		pwriter.println("</PSENData>");
		pwriter.close();
	}
	
	public void WritePSENToFile(PrintWriter pw, IPerformanceStatisticsEnrichedNet psnet, String name)
	{
		PerformanceStatisticsDB db = (PerformanceStatisticsDB)psnet;
		  pw.println("<PSEN name=\"" + name + "\">");

		  for (PSENLabel l : psnet.getLabels())
		  {
			  String typeStr = "";
			  if (l.isTimeSpan())
			  {
				  typeStr = "1";
			  }
			  else
			  {
				  typeStr = "0";
			  }
			  pw.println("<Label name=\"" + l.getLabelName() + "\" node=\"" + l.getNodeName() + "\" aggtype=\"" + l.getAggregationType() + "\" type=\"" + typeStr + "\">");	  
			  
			  pw.println("</Label>");
		  }

		  Vector<String> vst = new Vector<String>();
		  Vector<Double> dst = new Vector<Double>();
		  for (String tId : psnet.getTransitionsInclNone())
		  {
			  if (db.hasValue2(tId))
			  {
				  pw.println("<Transition id=\"" + tId + "\">");
				  for (PSENLabel pl : getUniqueLabelNodes(psnet))
				  {
					  if (db.hasValue2(tId, pl.getLabelName()))
					  {
						  pw.println("<LabelNode name=\"" + pl.getNodeName() + "\">");
						  db.getLabelValueGroups2(tId, pl.getLabelName(), vst, dst);
						  for (int i = 0; i < vst.size(); i++)
						  {
							  pw.println("<Value trace=\"" + vst.get(i) + "\">" + dst.get(i) + "</Value>");
						  }
						  vst.clear();
						  dst.clear();
						  pw.println("</LabelNode>");
					  }
				  }
				  pw.println("</Transition>");
			  }
		  }
		  
		  pw.println("</PSEN>");
	}
	
	public void WriteWaitingTimeTreeToFile(PrintWriter pw, IPerformanceStatisticsEnrichedNet psnet)
	{
		PerformanceStatisticsDB db = (PerformanceStatisticsDB)psnet;
		  pw.println("<PSENWaitingTimeTree>");

		  for (PSENLabel l : psnet.getLabels())
		  {
			  String typeStr = "";
			  if (l.isTimeSpan())
			  {
				  typeStr = "1";
			  }
			  else
			  {
				  typeStr = "0";
			  }
			  pw.println("<Label name=\"" + l.getLabelName() + "\" node=\"" + l.getNodeName() + "\" aggtype=\"" + l.getAggregationType() + "\" type=\"" + typeStr + "\">");	  
			  
			  pw.println("</Label>");
		  }

		  Vector<String> vst = new Vector<String>();
		  Vector<Double> dst = new Vector<Double>();
		  for (String tId : psnet.getTransitionsInclNone())
		  {
			  if (db.hasValue2(tId))
			  {
				  pw.println("<TransitionPair id=\"" + tId + "\">");
				  for (PSENLabel pl : getUniqueLabelNodes(psnet))
				  {
					  if (db.hasValue2(tId, pl.getLabelName()))
					  {
						  pw.println("<LabelNode name=\"" + pl.getNodeName() + "\">");
						  db.getLabelValueGroups2(tId, pl.getLabelName(), vst, dst);
						  for (int i = 0; i < vst.size(); i++)
						  {
							  pw.println("<Value trace=\"" + vst.get(i) + "\">" + dst.get(i) + "</Value>");
						  }
						  vst.clear();
						  dst.clear();
						  
						  pw.println("</LabelNode>");
					  }
				  }
				  pw.println("</TransitionPair>");
			  }
		  }
		  
		  pw.println("</PSENWaitingTimeTree>");
	}
	
	private Vector<PSENLabel> getUniqueLabelNodes(IPerformanceStatisticsEnrichedNet psnet)
	{
		Vector<PSENLabel> lbs = new Vector<PSENLabel>();
		Vector<String> lblNodes = new Vector<String>();
		for (PSENLabel pl : psnet.getLabels())
		{
			if (!lblNodes.contains(pl.getNodeName()))
			{
				lblNodes.add(pl.getNodeName());
				lbs.add(pl);
			}
		}
		return lbs;
	}
	
	
}
