package org.processmining.processprofiler3d.psen.fileio;

import java.awt.Dimension;
import java.awt.geom.Point2D;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.providedobjects.ProvidedObjectDeletedException;
import org.processmining.framework.providedobjects.ProvidedObjectID;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.compviz.SubNetDataSource;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */

@Plugin(name = "Export PNML Hierarchical Net",
parameterLabels = { "Petri net", "File" },
returnLabels = {},
returnTypes = {},
userAccessible = true)
@UIExportPlugin(description = "Hierarchical Petrinet file",
extension = "pnml")

public class PNMLHierarchicalExport 
{
	@UITopiaVariant(affiliation = "Queensland University of Technology",
              author = "Erik Poppe",
              email = "erik.poppe@qut.edu.au")
	
	@PluginVariant(variantLabel = "Export Hierarchical PNML", requiredParameterLabels = { 0, 1 })
	public void export(final UIPluginContext context, Petrinet pnet, File file)
			throws IOException 
	{	
		//get graph layout connection
		try 
		{
			layout = context.getConnectionManager().getFirstConnection(GraphLayoutConnection.class, context, pnet);
		} 
		catch (ConnectionCannotBeObtained exc)
		{
			// do nothing
			System.out.println("PNMLHierarchicalExport: cannot find graph layout!");
			//SlickerOptionPane.showErrorMessage(context, "Model could not be exported because no GraphLayout could be found.", "Error");
			context.getFutureResult(0).cancel(true);

			return;
		}
		
		//get initial markings	
		try 
		{
			// get initial marking of the net
			InitialMarkingConnection initMarkingConn = context.getConnectionManager().getFirstConnection(InitialMarkingConnection.class, context, pnet);
			initMarking = initMarkingConn.getObjectWithRole(InitialMarkingConnection.MARKING);
		} 
		catch (Exception exc)
		{
			// it's okay not to have any initial marking
		}
		
		//get final markings	
		try 
		{
			// get initial marking of the net
			FinalMarkingConnection finalMarkingConn = context.getConnectionManager().getFirstConnection(FinalMarkingConnection.class, context, pnet);
			finalMarking = finalMarkingConn.getObjectWithRole(FinalMarkingConnection.MARKING);
		} 
		catch (Exception exc)
		{
			// it's okay not to have any initial marking
		}

		FileWriter writer = new FileWriter(file);
		PrintWriter pwriter = new PrintWriter(writer);

		snNum = 1;
		pNum = 1;
		tNum = 1;
		aNum = 1;
		
		//find name of petri net to be exported
		String netName = "Petri Net";
		List<ProvidedObjectID> lst = context.getProvidedObjectManager().getProvidedObjects();
		Vector<String> lst2 = new Vector<String>();
		for (ProvidedObjectID pid : lst)
		{
			try 
			{
				lst2.add(context.getProvidedObjectManager().getProvidedObjectLabel(pid));
			} 
			catch (ProvidedObjectDeletedException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		if (!lst2.isEmpty())
		{
			netName = lst2.get(0);
		}
		
		pwriter.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
		pwriter.println("<pnml>");
		WritePetriNet(pwriter, new PetriNetDataSource(pnet), netName);
		pwriter.println("</pnml>");
		pwriter.close();
	}

	private int snNum = 1;
	private int pNum = 1;
	private int tNum = 1;
	private int aNum = 1;
	
	private GraphLayoutConnection layout = null;
	private Marking initMarking = null;
	private Marking finalMarking = null;
	
	private HashMap<PetrinetNode, String> nodeIds = new HashMap<PetrinetNode, String>();
	
	public void WritePetriNet(PrintWriter pwriter, PetriNetDataSource pnet, String netName)
	{
		//String netName = "Projected ED_Overall";
		pwriter.println("<net id=\"net1\" type=\"http://www.informatik.hu-berlin.de/top/pntd/ptNetb\">");
		pwriter.println("<name><text>" + netName + "</text></name>");

		for (Place p : pnet.getPlaces())
		{
			WritePlace(pwriter, p);
		}
		
		for (Transition t : pnet.getTransitions())
		{
			WriteTransition(pwriter, t);
		}
		
		for (Arc a : pnet.getEdges())
		{
			WriteArc(pwriter, a);
		}
		
		for (SubNetDataSource esn : pnet.getSubnets())
		{
			WritePetriNet(pwriter, esn);
		}
		
		pwriter.println("</net>");
	}
	
	public void WritePetriNet(PrintWriter pwriter, SubNetDataSource pnet)
	{
		Transition tx = pnet.getParent().getTransitionForSubnet(pnet);
		String pageId = nodeIds.get(tx);//"sub" + snNum;
		pwriter.println("<page id=\"" + pageId + "\">");
		pwriter.println("<net>");

		for (Place p : pnet.getPlaces())
		{
			WritePlace(pwriter, p);
		}
		
		for (Transition t : pnet.getTransitions())
		{
			WriteTransition(pwriter, t);
		}
		
		for (Arc a : pnet.getEdges())
		{
			WriteArc(pwriter, a);
		}
		
		for (SubNetDataSource esn : pnet.getSubnets())
		{
			//not tested!
			WritePetriNet(pwriter, esn);
		}
		
		pwriter.println("</net>");
		pwriter.println("</page>");
		
		snNum++;
	}
	
	public void WritePlace(PrintWriter pwriter, Place p)
	{
		String placeId = "p" + pNum;
		pwriter.println("<place id=\"" + placeId + "\">");
		pwriter.println("<name><text>" + p.getLabel() + "</text></name>");
		
		WriteNodeGraphics(pwriter, p);

		if (initMarking != null)
		{
			int tokenNum = initMarking.occurrences(p);
			if (tokenNum > 0)
			{
				pwriter.println("<initialMarking>");
				pwriter.println("<text>" + tokenNum + "</text>");
				pwriter.println("</initialMarking>");
			}
		}

		if (finalMarking != null)
		{
			int tokenNum = finalMarking.occurrences(p);
			if (tokenNum > 0)
			{
				pwriter.println("<finalMarking>");
				pwriter.println("<text>" + tokenNum + "</text>");
				pwriter.println("</finalMarking>");
			}
		}
		
		pwriter.println("<toolspecific tool=\"ProM\" version=\"6.4\" localNodeID=\"" + p.getLocalID().toString() + "\"/>");
		pwriter.println("</place>");
		
		nodeIds.put(p, placeId);
		pNum++;
	}
	
	public void WriteTransition(PrintWriter pwriter, Transition t)
	{
		String transId = "t" + tNum;
		pwriter.println("<transition id=\"" + transId + "\">");
		pwriter.println("<name><text>" + t.getLabel() + "</text></name>");
		
		WriteNodeGraphics(pwriter, t);
		
		if (t.isInvisible())
		{
			pwriter.println("<toolspecific tool=\"ProM\" version=\"6.4\" activity=\"$invisible$\" localNodeID=\"" + t.getLocalID().toString() + "\"/>");
		}
		else
		{
			pwriter.println("<toolspecific tool=\"ProM\" version=\"6.4\" localNodeID=\"" + t.getLocalID().toString() + "\"/>");
		}
		pwriter.println("</transition>");

		nodeIds.put(t, transId);
		tNum++;
	}
	
	public void WriteNodeGraphics(PrintWriter pwriter, PetrinetNode pn)
	{
		pwriter.println("<graphics>");
		int px = 0;
		int py = 0;
		int pw = 40;
		int ph = 40;
		
		if (layout != null)
		{
			Point2D pt = layout.getPosition(pn);
			px = (int)pt.getX();
			py = (int)pt.getY();
			
			Dimension s = layout.getSize(pn);
			pw = s.width;
			ph = s.height;
		}
		
		pwriter.println("<position x=\"" + px + "\" y=\"" + py + "\"/>");
		pwriter.println("<dimension x=\"" + pw + "\" y=\"" + ph + "\"/>");
		pwriter.println("</graphics>");		
	}
	
	public void WriteArc(PrintWriter pwriter, Arc a)
	{
		String arcId = "a" + aNum;
		String sourceId = nodeIds.get(a.getSource());
		String targetId = nodeIds.get(a.getTarget());
		pwriter.println("<arc id=\"" + arcId + "\" source=\"" + sourceId + "\" target=\"" + targetId + "\">");
		pwriter.println("<inscription><text>" + a.getLabel() + "</text></inscription>");
		List<Point2D> pts = layout.getEdgePoints(a);
		if (pts.isEmpty())
		{
			pwriter.println("<graphics/>");
		}
		else
		{
			pwriter.println("<graphics>");

			for (Point2D pt : pts)
			{
				int px = (int)pt.getX();
				int py = (int)pt.getY();
				pwriter.println("<position x=\"" + px + "\" y=\"" + py + "\"/>");
			}
			
			pwriter.println("</graphics>");
		}
		pwriter.println("<toolspecific tool=\"ProM\" version=\"6.4\" localNodeID=\"" + a.getLocalID().toString() + "\"/>");
		pwriter.println("</arc>");
		
		aNum++;
	}
}
