package org.processmining.processprofiler3d.psen.fileio;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.abstractplugins.AbstractImportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetImpl;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.psen.fileio.xml.XMLNode;
import org.processmining.processprofiler3d.psen.fileio.xml.XMLTag;
import org.processmining.processprofiler3d.psen.fileio.xml.XMLTagNode;
import org.processmining.processprofiler3d.psen.fileio.xml.XMLTree;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */

@Plugin(name = "Import PNML Hierarchical Net",
parameterLabels = { "File" },
//returnLabels = { "Petri Net", "Initial Marking", "Final Marking" },
returnLabels = { "Petri Net" },
//returnTypes = {Petrinet.class, Marking.class, Marking.class})
returnTypes = {Petrinet.class})
@UIImportPlugin(description = "PNML Hierarchical Net file",
        extensions = "pnml")

public class PNMLHierarchicalImport extends AbstractImportPlugin 
{
	@UITopiaVariant(affiliation = "Queensland University of Technology",
              author = "Erik Poppe",
              email = "erik.poppe@qut.edu.au")
	@Override
	protected Petrinet importFromStream(final PluginContext context,
	                                    final InputStream input,
	                                    final String filename,
	                                    final long fileSizeInBytes)
	{

		Petrinet pnetResult = null;
		
	    // Fill in object from input
		BufferedReader br = new BufferedReader(new InputStreamReader(input));
		
		XMLTree tree = null;
		
		while (tree == null)
		{
			try 
			{
				br.mark(0);
			} 
			catch (IOException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			XMLTag xmltag = XMLTag.ReadFrom(br);
			if (xmltag != null)
			{
				if (xmltag.isTagName("pnml"))
				{
					try 
					{
						br.reset();
					} 
					catch (IOException e) 
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					tree = XMLTree.ReadFrom(br);
				}
			}
			else
			{
				//would be nice to show a good error message here!
				context.getFutureResult(0).cancel(true);
			}	
		}
		
		if (tree != null)
		{
			XMLNode xn = tree.root.children.get(0);
			
			if (xn instanceof XMLTagNode)
			{
				XMLTagNode xtn = (XMLTagNode)xn;
				
				Petrinet newNet = new PetrinetImpl("Imported Hierarchical Net");
				GraphLayoutConnection newLayoutConn = new GraphLayoutConnection(newNet);
				Marking initMarking = new Marking();
				Marking finalMarking = new Marking();

				PNMLNet pnmlNet = null;
				if (xtn.hasAttribute("type"))
				{
					if (xtn.isAttributeValue("type", "http://www.pnml.org/version-2009/grammar/pnmlcoremodel"))
					{
						pnmlNet = new PNMLNet();
					}
					else if (xtn.isAttributeValue("type", "http://www.informatik.hu-berlin.de/top/pntd/ptNetb"))
					{
						pnmlNet = new PNMLNetWopedHierarchical();
					}
					else
					{
						pnmlNet = new PNMLNet();
					}
				}
				else
				{
					pnmlNet = new PNMLNet();
				}
				pnmlNet.ReadPNMLNet(newNet, newLayoutConn, initMarking, finalMarking, (XMLTagNode)xn);
				context.addConnection(newLayoutConn);
				
				if (!initMarking.isEmpty())
				{
					//register initial marking with object manager
					context.getProvidedObjectManager().createProvidedObject(pnmlNet.netName + " Initial Marking", initMarking, context);

					context.addConnection(new InitialMarkingConnection(newNet, initMarking));
				}
				if (!finalMarking.isEmpty())
				{
					//register final marking with object manager
					context.getProvidedObjectManager().createProvidedObject(pnmlNet.netName + " Final Marking", initMarking, context);

					context.addConnection(new FinalMarkingConnection(newNet, finalMarking));
				}
		
				pnetResult = newNet;
				
				//try to set petri net name correctly
				if (pnmlNet != null)
				{
					context.getFutureResult(0).setLabel(pnmlNet.netName);
				}
			}
		}
		
		//Return the net and the marking
		return pnetResult;
	}
}
