package org.processmining.processprofiler3d.psen.fileio;

import java.awt.Dimension;
import java.awt.geom.Dimension2D;
import java.awt.geom.Point2D;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;
import java.util.Vector;

import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.LocalNodeID;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.psen.fileio.xml.XMLNode;
import org.processmining.processprofiler3d.psen.fileio.xml.XMLTagNode;
import org.processmining.processprofiler3d.psen.fileio.xml.XMLTextNode;

public class PNMLNet 
{
	public String netName = "Petri Net";
	
	public PetrinetGraph ReadPNMLNet(Petrinet newNet, GraphLayoutConnection newLayoutConn, Marking initMarking, Marking finalMarking, XMLTagNode netNode)
	{	
		//read name of net
		for (XMLNode c : netNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("name"))
				{
					netName = ReadPNMLName(tn);
				}
			}
		}
		
		for (XMLNode c : netNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("page"))
				{
					ReadPNMLPage(newNet, newLayoutConn, initMarking, finalMarking, tn);
				}
				else if (tn.isTagName("place"))
				{
					ReadPNMLPlace(newNet, newLayoutConn, initMarking, finalMarking, tn);
				}
				else if (tn.isTagName("transition"))
				{
					ReadPNMLTransition(newNet, newLayoutConn, tn);
				}
				else if (tn.isTagName("arc"))
				{
					ReadPNMLArc(newNet, newLayoutConn, tn);
				}
			}
		}
		
		return newNet;
	}
	
	public void ReadPNMLPage(Petrinet newNet, GraphLayoutConnection newLayoutConn, Marking initMarking, Marking finalMarking, XMLTagNode pageNode)
	{
		for (XMLNode c : pageNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("place"))
				{
					ReadPNMLPlace(newNet, newLayoutConn, initMarking, finalMarking, tn);
				}
				else if (tn.isTagName("transition"))
				{
					ReadPNMLTransition(newNet, newLayoutConn, tn);
				}
				else if (tn.isTagName("arc"))
				{
					ReadPNMLArc(newNet, newLayoutConn, tn);
				}
			}
		}
	}
	
	private HashMap<String, Object> objsById = new HashMap<String, Object>();
	
	public void ReadPNMLPlace(Petrinet newNet, GraphLayoutConnection newLayoutConn, Marking initMarking, Marking finalMarking, XMLTagNode placeNode)
	{
		String placeLabel = "";
		String placeInitialMarking = "";
		String placeFinalMarking = "";
		Point2D placePosition = null;
		Dimension2D placeSize = null;
		LocalNodeID lNodeId = null;
		for (XMLNode c : placeNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("name"))
				{
					placeLabel = ReadPNMLName(tn);
				}
				else if (tn.isTagName("toolspecific"))
				{
					//ReadPNMLName(tn);
					lNodeId = ReadToolspecificLocalNodeID(tn);
				}
				else if (tn.isTagName("graphics"))
				{
					//ReadPNMLName(tn);
					placePosition = ReadPNMLGraphicsPosition(tn);;
					placeSize = ReadPNMLGraphicsDimension(tn);
				}
				else if (tn.isTagName("initialMarking"))
				{
					placeInitialMarking = ReadPNMLName(tn);
				}
				else if (tn.isTagName("finalMarking"))
				{
					placeFinalMarking = ReadPNMLName(tn);
				}
			}
		}
		Place p = newNet.addPlace(placeLabel);
		if (lNodeId != null)
		{
			p.setLocalID(lNodeId);
		}
		newLayoutConn.setPosition(p, placePosition);
		newLayoutConn.setSize(p, placeSize);
		
		if (placeNode.hasAttribute("id"))
		{
			objsById.put(placeNode.getAttribute("id"), p);
		}
		
		if (placeInitialMarking != "")
		{
			int weight = Integer.parseInt(placeInitialMarking);
			initMarking.add(p, weight);
		}
		if (placeFinalMarking != "")
		{
			int weight = Integer.parseInt(placeFinalMarking);
			finalMarking.add(p, weight);
		}
	}
	
	public void ReadPNMLTransition(Petrinet newNet, GraphLayoutConnection newLayoutConn, XMLTagNode transitionNode)
	{
		String transitionLabel = "";
		Point2D transitionPosition = null;
		Dimension2D transitionSize = null;
		LocalNodeID lNodeId = null;
		boolean isVisible = true;
		for (XMLNode c : transitionNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("name"))
				{
					transitionLabel = ReadPNMLName(tn);
				}
				else if (tn.isTagName("toolspecific"))
				{
					//ReadPNMLName(tn);
					lNodeId = ReadToolspecificLocalNodeID(tn);
					isVisible = ReadToolspecificIsVisible(tn);
				}
				else if (tn.isTagName("graphics"))
				{
					//ReadPNMLName(tn);
					transitionPosition = ReadPNMLGraphicsPosition(tn);;
					transitionSize = ReadPNMLGraphicsDimension(tn);
				}
			}
		}
		Transition t = newNet.addTransition(transitionLabel);
		if (lNodeId != null)
		{
			t.setLocalID(lNodeId);
		}
		t.setInvisible(!isVisible);
		newLayoutConn.setPosition(t, transitionPosition);
		newLayoutConn.setSize(t, transitionSize);
		
		if (transitionNode.hasAttribute("id"))
		{
			objsById.put(transitionNode.getAttribute("id"), t);
		}
	}
	
	public void ReadPNMLArc(Petrinet newNet, GraphLayoutConnection newLayoutConn, XMLTagNode arcNode)
	{
		String arcLabel = "1";
		String arcType = "normal";
		LocalNodeID lNodeId = null;
		List<Point2D> points = null;
		for (XMLNode c : arcNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("name"))
				{
					arcLabel = ReadPNMLName(tn);
				}
				else if (tn.isTagName("toolspecific"))
				{
					//ReadPNMLName(tn);
					lNodeId = ReadToolspecificLocalNodeID(tn);
				}
				else if (tn.isTagName("arctype"))
				{
					//ReadPNMLName(tn);
					arcType = ReadPNMLName(tn);
				}
				else if (tn.isTagName("graphics"))
				{
					points = ReadPNMLGraphicsPositionList(tn);
				}
			}
		}
		
		if (!arcType.equals("normal"))
		{
			System.out.println("Arc type: " + arcType + " not supported!");
		}
		
		Arc a = null;
		PetrinetNode source = null;
		PetrinetNode target = null;
		
		if (arcNode.hasAttribute("source"))
		{
			Object src = objsById.get(arcNode.getAttribute("source"));
			if (src instanceof PetrinetNode)
			{
				source = (PetrinetNode)src;
			}
		}
		if (arcNode.hasAttribute("target"))
		{
			Object src = objsById.get(arcNode.getAttribute("target"));
			if (src instanceof PetrinetNode)
			{
				target = (PetrinetNode)src;
			}
		}
		
		int weight = Integer.parseInt(arcLabel);
		
		if (source instanceof Place && target instanceof Transition)
		{
			a = newNet.addArc((Place)source, (Transition)target, weight);
		}
		else if (source instanceof Transition && target instanceof Place)
		{
			a = newNet.addArc((Transition)source, (Place)target, weight);
		}
		if (a != null && lNodeId != null)
		{
			a.setLocalID(lNodeId);
		}
		
		if (points != null && !points.isEmpty())
		{
			newLayoutConn.setEdgePoints(a, points);
		}
		
		if (arcNode.hasAttribute("id"))
		{
			objsById.put(arcNode.getAttribute("id"), a);
		}
	}
	
	public LocalNodeID ReadToolspecificLocalNodeID(XMLTagNode tn)
	{
		if (tn.hasAttribute("tool") && tn.isAttributeValue("tool", "ProM") && tn.hasAttribute("version") && tn.isAttributeValue("version", "6.4"))
		{
			if (tn.hasAttribute("localNodeID"))
			{
				String lNodeId = tn.getAttribute("localNodeID");
				return new LocalNodeID(UUID.fromString(lNodeId));
			}
		}
		return null;
	}
	
	public boolean ReadToolspecificIsVisible(XMLTagNode tn)
	{
		if (tn.hasAttribute("tool") && tn.isAttributeValue("tool", "ProM") && tn.hasAttribute("version") && tn.isAttributeValue("version", "6.4"))
		{
			if (tn.hasAttribute("activity"))
			{
				String activityLabel = tn.getAttribute("activity");
				return !activityLabel.equals("$invisible$");
			}
		}
		return true;
	}
	
	public String ReadPNMLName(XMLTagNode nameNode)
	{
		for (XMLNode c : nameNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("text"))
				{
					return ReadPNMLText(tn);
				}
			}
		}
		return "";
	}
	
	public String ReadPNMLText(XMLTagNode textNode)
	{
		for (XMLNode c : textNode.children)
		{
			if (c instanceof XMLTextNode)
			{
				XMLTextNode tn = (XMLTextNode)c;			
				return HtmlUtil.decodeHtmlChars(tn.text);
			}
		}
		return "";
	}
	
	public Point2D ReadPNMLGraphicsPosition(XMLTagNode graphicsNode)
	{
		for (XMLNode c : graphicsNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("position"))
				{
					double x = 0;
					double y = 0;
					if (tn.hasAttribute("x"))
					{
						x = Double.parseDouble(tn.getAttribute("x"));
					}
					if (tn.hasAttribute("y"))
					{
						y = Double.parseDouble(tn.getAttribute("y"));
					}
					Point2D pt = new Point2D.Double(x, y);
					return pt;
				}
			}
		}
		return null;
	}
	
	public List<Point2D> ReadPNMLGraphicsPositionList(XMLTagNode graphicsNode)
	{
		List<Point2D> points = new Vector<Point2D>();
		for (XMLNode c : graphicsNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("position"))
				{
					double x = 0;
					double y = 0;
					if (tn.hasAttribute("x"))
					{
						x = Double.parseDouble(tn.getAttribute("x"));
					}
					if (tn.hasAttribute("y"))
					{
						y = Double.parseDouble(tn.getAttribute("y"));
					}
					Point2D pt = new Point2D.Double(x, y);
					points.add(pt);
				}
			}
		}
		return points;
	}
	
	public Dimension2D ReadPNMLGraphicsDimension(XMLTagNode graphicsNode)
	{
		for (XMLNode c : graphicsNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("dimension"))
				{

					double x = 0;
					double y = 0;
					if (tn.hasAttribute("x"))
					{
						x = Double.parseDouble(tn.getAttribute("x"));
					}
					if (tn.hasAttribute("y"))
					{
						y = Double.parseDouble(tn.getAttribute("y"));
					}
					Dimension2D pt = new Dimension();
					pt.setSize(x, y);
					return pt;
				}
			}
		}
		return null;
	}
}
