package org.processmining.processprofiler3d.psen.fileio;

import java.awt.geom.Dimension2D;
import java.awt.geom.Point2D;
import java.util.HashMap;
import java.util.List;

import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.LocalNodeID;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.ExpandableSubNet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.psen.fileio.xml.XMLNode;
import org.processmining.processprofiler3d.psen.fileio.xml.XMLTagNode;

public class PNMLNetWopedHierarchical extends PNMLNet
{	
	@Override
	public PetrinetGraph ReadPNMLNet(Petrinet newNet, GraphLayoutConnection newLayoutConn, Marking initMarking, Marking finalMarking, XMLTagNode netNode)
	{	
		//read name of net
		for (XMLNode c : netNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("name"))
				{
					netName = ReadPNMLName(tn);
				}
			}
		}
		
		for (XMLNode c : netNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("page"))
				{
					ReadPNMLPage(newNet, newLayoutConn, initMarking, finalMarking, tn);
				}
			}
		}
		
		for (XMLNode c : netNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("place"))
				{
					ReadPNMLPlace(newNet, newLayoutConn, initMarking, finalMarking, tn);
				}
				else if (tn.isTagName("transition"))
				{
					ReadPNMLTransition(newNet, newLayoutConn, tn);
				}
				else if (tn.isTagName("arc"))
				{
					ReadPNMLArc(newNet, newLayoutConn, tn);
				}
			}
		}
		
		return newNet;
	}

	@Override
	public void ReadPNMLPage(Petrinet newNet, GraphLayoutConnection newLayoutConn, Marking initMarking, Marking finalMarking, XMLTagNode pageNode)
	{
		String pageId = "Subnet";
		if (pageNode.hasAttribute("id"))
		{
			pageId = pageNode.getAttribute("id");
		}
		
		ExpandableSubNet esn = newNet.addGroup(pageId);
		//esn.

		XMLTagNode subnetNode = null;
		for (XMLNode c : pageNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("net"))
				{
					subnetNode = tn;
				}
			}
		}
		
		if (subnetNode != null)
		{
			for (XMLNode c : subnetNode.children)
			{
				if (c instanceof XMLTagNode)
				{
					XMLTagNode tn = (XMLTagNode)c;
					if (tn.isTagName("page"))
					{
						ReadPNMLPage(newNet, esn, newLayoutConn, initMarking, finalMarking, tn);
					}
				}
			}
			
			for (XMLNode c : subnetNode.children)
			{
				if (c instanceof XMLTagNode)
				{
					XMLTagNode tn = (XMLTagNode)c;
					if (tn.isTagName("place"))
					{
						ReadPNMLPlace(newNet, esn, newLayoutConn, initMarking, finalMarking, tn);
					}
					else if (tn.isTagName("transition"))
					{
						ReadPNMLTransition(newNet, esn, newLayoutConn, tn);
					}
					else if (tn.isTagName("arc"))
					{
						ReadPNMLArc(newNet, esn, newLayoutConn, tn);
					}
				}
			}
		}
		objsById.put(pageId, esn);
	}
	
	public void ReadPNMLPage(Petrinet newNet, ExpandableSubNet subNet, GraphLayoutConnection newLayoutConn, Marking initMarking, Marking finalMarking, XMLTagNode pageNode)
	{
		String pageId = "Subnet";
		if (pageNode.hasAttribute("id"))
		{
			pageId = pageNode.getAttribute("id");
		}
		
		ExpandableSubNet esn = newNet.addGroup(pageId, subNet);

		XMLTagNode subnetNode = null;
		for (XMLNode c : pageNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("net"))
				{
					subnetNode = tn;
				}
			}
		}
		
		if (subnetNode != null)
		{
			for (XMLNode c : subnetNode.children)
			{
				if (c instanceof XMLTagNode)
				{
					XMLTagNode tn = (XMLTagNode)c;
					if (tn.isTagName("page"))
					{
						ReadPNMLPage(newNet, esn, newLayoutConn, initMarking, finalMarking, tn);
					}
				}
			}
			
			for (XMLNode c : subnetNode.children)
			{
				if (c instanceof XMLTagNode)
				{
					XMLTagNode tn = (XMLTagNode)c;
					if (tn.isTagName("place"))
					{
						ReadPNMLPlace(newNet, esn, newLayoutConn, initMarking, finalMarking, tn);
					}
					else if (tn.isTagName("transition"))
					{
						ReadPNMLTransition(newNet, esn, newLayoutConn, tn);
					}
					else if (tn.isTagName("arc"))
					{
						ReadPNMLArc(newNet, esn, newLayoutConn, tn);
					}
				}
			}
		}
		objsById.put(pageId, esn);
	}
	
	private HashMap<String, Object> objsById = new HashMap<String, Object>();

	@Override
	public void ReadPNMLPlace(Petrinet newNet, GraphLayoutConnection newLayoutConn, Marking initMarking, Marking finalMarking, XMLTagNode placeNode)
	{
		String placeLabel = "";
		String placeInitialMarking = "";
		String placeFinalMarking = "";
		Point2D placePosition = null;
		Dimension2D placeSize = null;
		LocalNodeID lNodeId = null;
		
		for (XMLNode c : placeNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("name"))
				{
					placeLabel = ReadPNMLName(tn);
				}
				else if (tn.isTagName("toolspecific"))
				{
					//ReadPNMLName(tn);
					if (tn.isAttributeValue("tool", "ProM"))
					{
						lNodeId = ReadToolspecificLocalNodeID(tn);
					}
				}
				else if (tn.isTagName("graphics"))
				{
					//ReadPNMLName(tn);
					placePosition = ReadPNMLGraphicsPosition(tn);;
					placeSize = ReadPNMLGraphicsDimension(tn);
				}
				else if (tn.isTagName("initialMarking"))
				{
					placeInitialMarking = ReadPNMLName(tn);
				}
				else if (tn.isTagName("finalMarking"))
				{
					placeFinalMarking = ReadPNMLName(tn);
				}
			}
		}
		Place p = newNet.addPlace(placeLabel);
		if (lNodeId != null)
		{
			p.setLocalID(lNodeId);
		}
		else
		{
			System.out.println("Place has no localNodeId: " + p.getLabel());
		}
		
		newLayoutConn.setPosition(p, placePosition);
		newLayoutConn.setSize(p, placeSize);
		
		if (placeNode.hasAttribute("id"))
		{
			objsById.put(placeNode.getAttribute("id"), p);
		}
		
		if (placeInitialMarking != "")
		{
			int weight = Integer.parseInt(placeInitialMarking);
			initMarking.add(p, weight);
		}
		
		if (placeFinalMarking != "")
		{
			int weight = Integer.parseInt(placeFinalMarking);
			finalMarking.add(p, weight);
		}
	}
	
	public void ReadPNMLPlace(Petrinet newNet, ExpandableSubNet esn, GraphLayoutConnection newLayoutConn, Marking initMarking, Marking finalMarking, XMLTagNode placeNode)
	{
		String placeLabel = "";
		String placeInitialMarking = "";
		String placeFinalMarking = "";
		Point2D placePosition = null;
		Dimension2D placeSize = null;
		LocalNodeID lNodeId = null;
		for (XMLNode c : placeNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("name"))
				{
					placeLabel = ReadPNMLName(tn);
				}
				else if (tn.isTagName("toolspecific"))
				{
					//ReadPNMLName(tn);
					if (tn.isAttributeValue("tool", "ProM"))
					{
						lNodeId = ReadToolspecificLocalNodeID(tn);
					}
				}
				else if (tn.isTagName("graphics"))
				{
					//ReadPNMLName(tn);
					placePosition = ReadPNMLGraphicsPosition(tn);;
					placeSize = ReadPNMLGraphicsDimension(tn);
				}
				else if (tn.isTagName("initialMarking"))
				{
					placeInitialMarking = ReadPNMLName(tn);
				}
				else if (tn.isTagName("finalMarking"))
				{
					placeFinalMarking = ReadPNMLName(tn);
				}
			}
		}
		//Place p = newNet.addPlace(placeLabel);
		Place p = new Place(placeLabel, (AbstractDirectedGraph<PetrinetNode, PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>>) newNet, esn);
		esn.addChild(p);
		if (lNodeId != null)
		{
			p.setLocalID(lNodeId);
		}
		else
		{
			System.out.println("Place has no localNodeId: " + p.getLabel());
		}
		
		newLayoutConn.setPosition(p, placePosition);
		newLayoutConn.setSize(p, placeSize);
		
		if (placeNode.hasAttribute("id"))
		{
			objsById.put(placeNode.getAttribute("id"), p);
		}
		
		if (placeInitialMarking != "")
		{
			int weight = Integer.parseInt(placeInitialMarking);
			initMarking.add(p, weight);
		}
		
		if (placeFinalMarking != "")
		{
			int weight = Integer.parseInt(placeFinalMarking);
			finalMarking.add(p, weight);
		}
	}

	@Override
	public void ReadPNMLTransition(Petrinet newNet, GraphLayoutConnection newLayoutConn, XMLTagNode transitionNode)
	{
		String transitionLabel = "";
		Point2D transitionPosition = null;
		Dimension2D transitionSize = null;
		LocalNodeID lNodeId = null;
		boolean isVisible = true;
		for (XMLNode c : transitionNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("name"))
				{
					transitionLabel = ReadPNMLName(tn);
				}
				else if (tn.isTagName("toolspecific"))
				{
					//ReadPNMLName(tn);
					if (tn.isAttributeValue("tool", "ProM"))
					{
						lNodeId = ReadToolspecificLocalNodeID(tn);
						isVisible = ReadToolspecificIsVisible(tn);
					}
				}
				else if (tn.isTagName("graphics"))
				{
					//ReadPNMLName(tn);
					transitionPosition = ReadPNMLGraphicsPosition(tn);;
					transitionSize = ReadPNMLGraphicsDimension(tn);
				}
			}
		}
		
		ExpandableSubNet esn = null;
		if (objsById.containsKey(transitionNode.getAttribute("id")))
		{
			Object obj = objsById.get(transitionNode.getAttribute("id"));
			if (obj instanceof ExpandableSubNet)
			{
				esn = (ExpandableSubNet)obj;
			}
		}
		
		//if (esn == null)
		{
			Transition t = newNet.addTransition(transitionLabel);
			if (lNodeId != null)
			{
				t.setLocalID(lNodeId);
			}
			else
			{
				System.out.println("Transition has no localNodeId: " + t.getLabel());
			}
			
			t.setInvisible(!isVisible);
			newLayoutConn.setPosition(t, transitionPosition);
			newLayoutConn.setSize(t, transitionSize);
			
			if (transitionNode.hasAttribute("id"))
			{
				objsById.put(transitionNode.getAttribute("id"), t);
			}
			if (esn != null)
			{
				t.getAttributeMap().put("PNMLSubnetId", transitionNode.getAttribute("id"));
			}
		}
//		else
//		{
//			if (lNodeId != null)
//			{
//				esn.setLocalID(lNodeId);
//			}
//			newLayoutConn.setPosition(esn, transitionPosition);
//			newLayoutConn.setSize(esn, transitionSize);		
//		}
	}
	
	public void ReadPNMLTransition(Petrinet newNet, ExpandableSubNet esn, GraphLayoutConnection newLayoutConn, XMLTagNode transitionNode)
	{
		String transitionLabel = "";
		Point2D transitionPosition = null;
		Dimension2D transitionSize = null;
		LocalNodeID lNodeId = null;
		boolean isVisible = true;
		for (XMLNode c : transitionNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("name"))
				{
					transitionLabel = ReadPNMLName(tn);
				}
				else if (tn.isTagName("toolspecific"))
				{
					//ReadPNMLName(tn);
					if (tn.isAttributeValue("tool", "ProM"))
					{
						lNodeId = ReadToolspecificLocalNodeID(tn);
						isVisible = ReadToolspecificIsVisible(tn);
					}
				}
				else if (tn.isTagName("graphics"))
				{
					//ReadPNMLName(tn);
					transitionPosition = ReadPNMLGraphicsPosition(tn);;
					transitionSize = ReadPNMLGraphicsDimension(tn);
				}
			}
		}

		ExpandableSubNet myesn = null;
		if (objsById.containsKey(transitionNode.getAttribute("id")))
		{
			Object obj = objsById.get(transitionNode.getAttribute("id"));
			if (obj instanceof ExpandableSubNet)
			{
				myesn = (ExpandableSubNet)obj;
			}
		}
		
		//Transition t = newNet.addTransition(transitionLabel);
		Transition t = new Transition(transitionLabel, (AbstractDirectedGraph<PetrinetNode, PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>>) newNet, esn);
		esn.addChild(t);
		if (lNodeId != null)
		{
			t.setLocalID(lNodeId);
		}
		else
		{
			System.out.println("Transition has no localNodeId: " + t.getLabel());
		}
		
		t.setInvisible(!isVisible);
		newLayoutConn.setPosition(t, transitionPosition);
		newLayoutConn.setSize(t, transitionSize);
		
		if (transitionNode.hasAttribute("id"))
		{
			objsById.put(transitionNode.getAttribute("id"), t);
		}
		if (myesn != null)
		{
			t.getAttributeMap().put("PNMLSubnetId", transitionNode.getAttribute("id"));
		}
	}

	@Override
	public void ReadPNMLArc(Petrinet newNet, GraphLayoutConnection newLayoutConn, XMLTagNode arcNode)
	{
		String arcLabel = "1";
		String arcType = "normal";
		LocalNodeID lNodeId = null;
		List<Point2D> points = null;
		for (XMLNode c : arcNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("name"))
				{
					arcLabel = ReadPNMLName(tn);
				}
				else if (tn.isTagName("toolspecific"))
				{
					//ReadPNMLName(tn);
					if (tn.isAttributeValue("tool", "ProM"))
					{
						lNodeId = ReadToolspecificLocalNodeID(tn);
					}
				}
				else if (tn.isTagName("arctype"))
				{
					//ReadPNMLName(tn);
					arcType = ReadPNMLName(tn);
				}
				else if (tn.isTagName("graphics"))
				{
					points = ReadPNMLGraphicsPositionList(tn);
				}
			}
		}
		
		if (!arcType.equals("normal"))
		{
			System.out.println("Arc type: " + arcType + " not supported!");
		}
		
		Arc a = null;
		PetrinetNode source = null;
		PetrinetNode target = null;
		
		if (arcNode.hasAttribute("source"))
		{
			Object src = objsById.get(arcNode.getAttribute("source"));
			if (src instanceof PetrinetNode)
			{
				source = (PetrinetNode)src;
			}
		}
		if (arcNode.hasAttribute("target"))
		{
			Object src = objsById.get(arcNode.getAttribute("target"));
			if (src instanceof PetrinetNode)
			{
				target = (PetrinetNode)src;
			}
		}
		
		int weight = Integer.parseInt(arcLabel);
		
		if (source instanceof Place && target instanceof Transition)
		{
			a = newNet.addArc((Place)source, (Transition)target, weight);
		}
		else if (source instanceof Transition && target instanceof Place)
		{
			a = newNet.addArc((Transition)source, (Place)target, weight);
		}
		if (a != null && lNodeId != null)
		{
			a.setLocalID(lNodeId);
		}
		
		if (points != null && !points.isEmpty())
		{
			newLayoutConn.setEdgePoints(a, points);
		}
		
		if (arcNode.hasAttribute("id"))
		{
			objsById.put(arcNode.getAttribute("id"), a);
		}
	}
	
	public void ReadPNMLArc(Petrinet newNet, ExpandableSubNet esn, GraphLayoutConnection newLayoutConn, XMLTagNode arcNode)
	{
		String arcLabel = "1";
		String arcType = "normal";
		LocalNodeID lNodeId = null;
		List<Point2D> points = null;
		for (XMLNode c : arcNode.children)
		{
			if (c instanceof XMLTagNode)
			{
				XMLTagNode tn = (XMLTagNode)c;
				if (tn.isTagName("name"))
				{
					arcLabel = ReadPNMLName(tn);
				}
				else if (tn.isTagName("toolspecific"))
				{
					//ReadPNMLName(tn);
					if (tn.isAttributeValue("tool", "ProM"))
					{
						lNodeId = ReadToolspecificLocalNodeID(tn);
					}
				}
				else if (tn.isTagName("arctype"))
				{
					//ReadPNMLName(tn);
					arcType = ReadPNMLName(tn);
				}
				else if (tn.isTagName("graphics"))
				{
					points = ReadPNMLGraphicsPositionList(tn);
				}
			}
		}
		
		if (!arcType.equals("normal"))
		{
			System.out.println("Arc type: " + arcType + " not supported!");
		}
		
		Arc a = null;
		PetrinetNode source = null;
		PetrinetNode target = null;
		
		if (arcNode.hasAttribute("source"))
		{
			Object src = objsById.get(arcNode.getAttribute("source"));
			if (src instanceof PetrinetNode)
			{
				source = (PetrinetNode)src;
			}
		}
		if (arcNode.hasAttribute("target"))
		{
			Object src = objsById.get(arcNode.getAttribute("target"));
			if (src instanceof PetrinetNode)
			{
				target = (PetrinetNode)src;
			}
		}
		
		int weight = Integer.parseInt(arcLabel);
		
		a = new Arc(source, target, weight, esn);
		esn.addChild(a);
		if (a != null && lNodeId != null)
		{
			a.setLocalID(lNodeId);
		}
		
		if (points != null && !points.isEmpty())
		{
			newLayoutConn.setEdgePoints(a, points);
		}
		
		if (arcNode.hasAttribute("id"))
		{
			objsById.put(arcNode.getAttribute("id"), a);
		}
	}
}
