package org.processmining.processprofiler3d.psen.fileio.obj;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;

import org.processmining.visualisation3d.maths.JoglVectord3;

public class MTLWriter 
{
	private FileOutputStream fos;
	private Writer out;
	
	public MTLWriter()
	{
	}
	
	public void Open(String filePath)
	{
		 try 
		 {
			//String filePath = "C:/Lab/Export/test.obj";
	        fos = new FileOutputStream(filePath);
	        out = new OutputStreamWriter(fos, "UTF8");
	        out.write("# ProM - Process Profiler 3D - Model Export\r\n");	        
		 } 
		 catch (IOException e) 
		 {
		    e.printStackTrace();
		 }
	}
	
	public void Write(String s)
	{
		try 
		{
			out.write(s);
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteLine(String s)
	{
		try 
		{
			out.write(s + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteNewMaterial(String name)
	{
		try 
		{
			out.write("newmtl " + name + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteColorVal(String t, JoglVectord3 v)
	{
		try 
		{
			out.write(t + " " + v.getX() + " " + v.getY() + " " + v.getZ() + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteColorVal(String t, double x, double y, double z)
	{
		try 
		{
			out.write(t + " " + x + " " + y + " " + z + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteAmbientColor(JoglVectord3 v)
	{
		try 
		{
			out.write("Ka " + v.getX() + " " + v.getY() + " " + v.getZ() + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteAmbientColor(double x, double y, double z)
	{
		try 
		{
			out.write("Ka " + x + " " + y + " " + z + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteDiffuseColor(JoglVectord3 v)
	{
		try 
		{
			out.write("Kd " + v.getX() + " " + v.getY() + " " + v.getZ() + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteDiffuseColor(double x, double y, double z)
	{
		try 
		{
			out.write("Kd " + x + " " + y + " " + z + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteSpecularColor(JoglVectord3 v)
	{
		try 
		{
			out.write("Ks " + v.getX() + " " + v.getY() + " " + v.getZ() + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteSpecularColor(double x, double y, double z)
	{
		try 
		{
			out.write("Ks " + x + " " + y + " " + z + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteEmissiveColor(JoglVectord3 v)
	{
		try 
		{
			out.write("Ke " + v.getX() + " " + v.getY() + " " + v.getZ() + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteEmissiveColor(double x, double y, double z)
	{
		try 
		{
			out.write("Ke " + x + " " + y + " " + z + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteSpecularPower(double x)
	{
		try 
		{
			out.write("Ns " + x + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteTransparencyDissolve(double x)
	{
		try 
		{
			out.write("d " + x + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteIlluminationModel(int x)
	{
		try 
		{
			out.write("illum " + x + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void Close()
	{
        try 
        {
			out.close();
		} 
        catch (IOException e) 
        {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
