package org.processmining.processprofiler3d.psen.fileio.obj;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Vector;

import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;

public class OBJWriter 
{
	private FileOutputStream fos;
	private Writer out;

	private int vertNum = 0;
	private int texcoordNum = 0;
	
	public OBJWriter()
	{
	}
	
	public void Open(String filePath)
	{
		 try
		 {
			//String filePath = "C:/Lab/Export/test.obj";
	        fos = new FileOutputStream(filePath);
	        out = new OutputStreamWriter(fos, "UTF8");
	        out.write("# ProM - Process Profiler 3D - Model Export\r\n");	        
		 } 
		 catch (IOException e) 
		 {
		    e.printStackTrace();
		 }
	}
	
	public void Write(String s)
	{
		try 
		{
			out.write(s);
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteLine(String s)
	{
		try 
		{
			out.write(s + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteMtlLib(String name)
	{
		try 
		{
			out.write("mtllib " + name + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteNewObject(String name)
	{
		try 
		{
			out.write("o " + name + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteNewObject(OBJObject o)
	{
		WriteNewObject(o.objName);
		
		Vector<Integer> ids = new Vector<Integer>();
		Vector<Integer> ids2 = new Vector<Integer>();
		for (JoglVectord3 v : o.verts)
		{
			ids.add(WriteVertex(v));
		}
		
		if (o.texcoords != null)
		{
			for (JoglVectord2 v : o.texcoords)
			{
				ids2.add(WriteTexCoords(v));
			}
		}
		
		if (o.material != null)
		{
			WriteUseMtl(o.material);
		}
		
		for (OBJFace f : o.faces)
		{
			if (f instanceof OBJFaceTextured)
			{
				OBJFaceTextured ft = (OBJFaceTextured)f;
				WriteFaceTriTex(ids.get(f.v1), ids.get(f.v2), ids.get(f.v3), ids2.get(ft.vt1), ids2.get(ft.vt2), ids2.get(ft.vt3));
			}
			else
			{
				WriteFaceTri(ids.get(f.v1), ids.get(f.v2), ids.get(f.v3));
			}
				
		}
	}
	
	public int WriteVertex(JoglVectord3 v)
	{
		try 
		{
			out.write("v " + v.getX() + " " + v.getY() + " " + v.getZ() + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		vertNum++;
		
		return vertNum;
	}
	
	public int WriteVertex(double x, double y, double z)
	{
		try 
		{
			out.write("v " + x + " " + y + " " + z + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		vertNum++;
		
		return vertNum;
	}
	
	public int WriteTexCoords(JoglVectord2 v)
	{
		try 
		{
			out.write("vt " + v.getX() + " " + v.getY() + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		texcoordNum++;
		
		return texcoordNum;
	}
	
	public int WriteTexCoords(double u, double v)
	{
		try 
		{
			out.write("vt " + u + " " + v + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		texcoordNum++;
		
		return texcoordNum;
	}
	
	public void WriteUseMtl(String mtlName)
	{
		try
		{
			out.write("usemtl " + mtlName + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteFaceTri(int v1, int v2, int v3)
	{
		try 
		{
			out.write("f " + v1 + " " + v2 + " " + v3 + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteFaceTriTex(int v1, int v2, int v3, int vt1, int vt2, int vt3)
	{
		try 
		{
			out.write("f " + v1 + "/" + vt1 + " " + v2 + "/" + vt2 + " " + v3 + "/" + vt3 + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void Close()
	{
        try 
        {
			out.close();
		} 
        catch (IOException e) 
        {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
