package org.processmining.processprofiler3d.psen.fileio.obj;

import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;

import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord3;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;

public class SVGSceneRenderer 
{
	public static void RenderSceneToSVGFile(String fileName, JoglCamera cam, JSOGraphicalUserInterface gui, OBJObjectWriter oow)
	{
		SVGScene svgs = new SVGScene((int)gui.getWidth(), (int)gui.getHeight());

		for (OBJObject oo : oow.objList)
		{
			String fillColor = GetFillColor(oo);
			
			//project vertices
			Vector<JoglVectord3> verts = projectVerts(cam, gui, oo.verts);
			
			//compose triangles
			for (OBJFace f : oo.faces)
			{
				if (isBackfacingTri(verts.get(f.v1), verts.get(f.v2), verts.get(f.v3)))
				{
					//backfacing triangle
					continue;
				}
				
				SVGPath tri = projectTri(verts.get(f.v1), verts.get(f.v2), verts.get(f.v3));
				if (tri != null)
				{
					tri.fillColor = fillColor;
					svgs.paths.add(tri);
				}
			}
		}
		
		//sort polygons by distance of center from camera
		Collections.sort(svgs.paths, new Comparator<SVGPath>() 
		{
	        @Override
	        public int compare(SVGPath o1, SVGPath o2) 
	        {
	        	Double d1 = new Double((o1.verts.get(0).getZ() + o1.verts.get(1).getZ() + o1.verts.get(2).getZ()) / 3.0);
	        	Double d2 = new Double((o2.verts.get(0).getZ() + o2.verts.get(1).getZ() + o2.verts.get(2).getZ()) / 3.0);
	            return d2.compareTo(d1);
	        }           
	    });
		
		svgs.WriteToFile(fileName);
	}
	
	private static Vector<JoglVectord3> projectVerts(JoglCamera cam, JSOGraphicalUserInterface gui, Vector<JoglVectord3> vts)
	{
		float aspect = (float)gui.getAspect();
		double halfW = 0.5 * gui.getWidth();
		double halfH = 0.5 * gui.getHeight();

		Vector<JoglVectord3> verts = new Vector<JoglVectord3>();
		for (JoglVectord3 vtx : vts)
		{
			JoglVectord3 screenPosNDC = cam.ProjectToScreen(vtx, aspect);
			JoglVectord2 screenPos = new JoglVectord2((screenPosNDC.getX() + 1) * halfW, (screenPosNDC.getY() + 1) * halfH);
		
			verts.add(new JoglVectord3(gui.getWidth() - screenPos.getX(), gui.getHeight() - screenPos.getY(), screenPosNDC.getZ()));
		}
		
		return verts;
	}
	
	private static SVGPath projectTri(JoglVectord3 v1, JoglVectord3 v2, JoglVectord3 v3)
	{
		SVGPath path = new SVGPath();
		if (v1.getZ() > 0.0)
		{
			path.verts.addElement(v1);
		}
		if (v2.getZ() > 0.0)
		{
			path.verts.addElement(v2);
		}
		if (v3.getZ() > 0.0)
		{
			path.verts.addElement(v3);
		}
		
		if (path.verts.size() == 3)
		{
			return path;
		}
		return null;
	}
	
	private static boolean isBackfacingTri(JoglVectord3 v1, JoglVectord3 v2, JoglVectord3 v3)
	{
		JoglVectord2 a = JoglVectord2.Normal(new JoglVectord2(v2.getX() - v1.getX(), v2.getY() - v1.getY()));
		JoglVectord2 b = JoglVectord2.Normal(new JoglVectord2(v3.getX() - v2.getX(), v3.getY() - v2.getY()));
		
		double dp = JoglVectord2.Dot(new JoglVectord2(a.getY(), -a.getX()), b);
		return (dp <= 0);	
	}

	private static String GetFillColor(OBJObject oo) 
	{
		String fillColor = "red";
		if (oo.material.equals("ModelElementBlack"))
		{
			fillColor = "black";
		}
		else if (oo.material.equals("ModelElementWhite"))
		{
			fillColor = "white";
		}
		else if (oo.material.equals("CohortWhite"))
		{
			fillColor = "white";
		}
		else if (oo.material.equals("CohortBlack"))
		{
			fillColor = "black";
		}
		else if (oo.material.equals("CohortRed"))
		{
			fillColor = "red";
		}
		else if (oo.material.equals("CohortGreen"))
		{
			fillColor = "green";
		}
		else if (oo.material.equals("CohortBlue"))
		{
			fillColor = "blue";
		}
		else if (oo.material.equals("CohortYellow"))
		{
			fillColor = "yellow";
		}
		else if (oo.material.equals("CohortCyan"))
		{
			fillColor = "cyan";
		}
		else if (oo.material.equals("CohortMagenta"))
		{
			fillColor = "magenta";
		}
		return fillColor;
	}

}
