package org.processmining.processprofiler3d.psen.fileio.obj;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.text.DecimalFormat;
import java.util.Vector;

import org.processmining.visualisation3d.maths.JoglVectord3;

public class SVGWriter 
{
	private FileOutputStream fos;
	private Writer out;
	
	public SVGWriter()
	{
	}
	
	public void Open(String filePath)
	{
		 try
		 {
			//String filePath = "C:/Lab/Export/test.obj";
			System.out.println("Exporting model to : " + filePath);
	        fos = new FileOutputStream(filePath);
	        out = new OutputStreamWriter(fos, "UTF8");
	        out.write("<?xml version='1.0' encoding='ascii'?>\r\n");
	        //out.write("<?xml version='1.0' encoding='ascii'?>\r\n");	        
		 } 
		 catch (IOException e) 
		 {
		    e.printStackTrace();
		 }
	}
	
	public void Write(String s)
	{
		try 
		{
			out.write(s);
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteLine(String s)
	{
		try 
		{
			out.write(s + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteOpenSVG(int w, int h)
	{
		try 
		{
			out.write("<svg xmlns=\"http://www.w3.org/2000/svg\" height=\"" + h + "\" version=\"1.1\" width=\"" + w + "\">" + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteOpenGroup(String name)
	{
		try 
		{
			out.write("<g id=\"" + name + "\">" + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void WritePath(Vector<JoglVectord3> lst)
	{
		try 
		{
			//String pts = "808.505, 428.592 808.938, 425.801";
			String pts = "";
			for (JoglVectord3 v : lst)
			{
				pts = pts + customFormat("###.###", v.getX()) + ", " + customFormat("###.###", v.getY()) + " ";
			}
			out.write("<path d=\" M " + pts + "\" fill=\"none\" stroke=\"rgb(0, 0, 0)\" stroke-linecap=\"butt\" stroke-linejoin=\"bevel\" stroke-opacity=\"1.0\" stroke-width=\"3.0\" />" + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void WriteFilledPath(Vector<JoglVectord3> lst, String fillColor)
	{
		try 
		{
			//String pts = "808.505, 428.592 808.938, 425.801";
			String pts = "";
			for (JoglVectord3 v : lst)
			{
				pts = pts + customFormat("###.###", v.getX()) + ", " + customFormat("###.###", v.getY()) + " ";
			}
			out.write("<path d=\" M " + pts + "\" fill=\"" + fillColor + "\" stroke=\"none\" stroke-linecap=\"butt\" stroke-linejoin=\"bevel\" stroke-opacity=\"1.0\" stroke-width=\"3.0\" />" + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	static public String customFormat(String pattern, double value) 
	{
	      DecimalFormat myFormatter = new DecimalFormat(pattern);
	      return myFormatter.format(value);
	}
	
	public void WriteCloseGroup()
	{
		try 
		{
			out.write("</g>" + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void WriteCloseSVG()
	{
		try 
		{
			out.write("</svg>" + "\r\n");
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void Close()
	{
        try 
        {
			out.close();
		} 
        catch (IOException e) 
        {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
