package org.processmining.processprofiler3d.psen.gui;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.processprofiler3d.placealignment.RunnablePAA;
import org.processmining.processprofiler3d.psen.AlignedLog;
import org.processmining.processprofiler3d.psen.preprocess.LPPAction;

import com.fluxicon.slickerbox.components.SlickerButton;

public class AnalysisReport extends ProMUIOverlay
{
	private Petrinet net;
	private XLog log;
	private XLog annotatedLog;
	
	public AnalysisReport(Petrinet hierarchicalNet, XLog log, Object[] results)
	{
		this.net = hierarchicalNet;
		this.log = log;
		this.annotatedLog = (XLog)results[1];
		this.overlayTitle = "Analysis Results:";
	}
	
	@Override
	protected JComponent CreateContentPanel()
	{
		JPanel listPane = new JPanel();
		listPane.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
		listPane.setBackground(ProMUIColors.BACKGROUND_GRAY);
		listPane.setLayout(new BoxLayout(listPane, BoxLayout.PAGE_AXIS));
		listPane.add(new JLabel("Traces: " + annotatedLog.size() + "/" + log.size()));
		listPane.add(Box.createVerticalStrut(10));
		
		float minTraceFitness = 1;
		float maxTraceFitness = 0;
		float avgTraceFitness = 0;
		int fitTraces = 0;
		int numTraces = 0;
		for (XTrace trc : annotatedLog)
		{
			Double val = LPPAction.getAttributeValueContinuous(trc, "Trace Fitness");
			if (val != null)
			{
				float traceFitness = val.floatValue();
				minTraceFitness = Math.min(minTraceFitness, traceFitness);
				maxTraceFitness = Math.max(maxTraceFitness, traceFitness);
				avgTraceFitness = avgTraceFitness + traceFitness;
				fitTraces++;
			}
			numTraces++;
		}
		
		if (numTraces > 0)
		{
			int totalEvents = 0;
			int correctEvents = 0;
			int incorrectEvents = 0;
			int missingEvents = 0;
			int invisibleEvents = 0;
	
			for (XTrace trc : annotatedLog)
			{
				for (XEvent evt : trc)
				{
					StepTypes st = AlignedLog.getStepType(evt);
					if (st == StepTypes.LMGOOD)
					{
						correctEvents++;
						totalEvents++;
					}
					else if (st == StepTypes.MINVI)
					{
						invisibleEvents++;
						totalEvents++;
					}
					else if (st == StepTypes.L)
					{
						incorrectEvents++;
						totalEvents++;
					}
					else if (st == StepTypes.MREAL)
					{
						missingEvents++;
						totalEvents++;
					}
				}
			}
			
			if (totalEvents > 0)
			{
				listPane.add(new JLabel("# fitting events: " + correctEvents));
				listPane.add(new JLabel("# invisible events: " + invisibleEvents));
				listPane.add(new JLabel("# incorrect events: " + incorrectEvents));
				listPane.add(new JLabel("# missing events: " + missingEvents));
			}
			else
			{
				listPane.add(new JLabel("# fitting events: N/A"));
				listPane.add(new JLabel("# invisible events: N/A"));
				listPane.add(new JLabel("# incorrect events: N/A"));
				listPane.add(new JLabel("# missing events: N/A"));
			}
		}
		else
		{
			listPane.add(new JLabel("# fitting events: N/A"));
			listPane.add(new JLabel("# invisible events: N/A"));
			listPane.add(new JLabel("# incorrect events: N/A"));
			listPane.add(new JLabel("# missing events: N/A"));
		}
		
		listPane.add(Box.createVerticalStrut(10));
		
		if (fitTraces > 0)
		{
			avgTraceFitness = avgTraceFitness / fitTraces;
			
			listPane.add(new JLabel("Min Trace Fitness: " + minTraceFitness));
			listPane.add(new JLabel("Avg Trace Fitness: " + avgTraceFitness));
			listPane.add(new JLabel("Max Trace Fitness: " + maxTraceFitness));
		}
		else
		{
			listPane.add(new JLabel("Min Trace Fitness: N/A"));
			listPane.add(new JLabel("Avg Trace Fitness: N/A"));
			listPane.add(new JLabel("Max Trace Fitness: N/A"));
		}
		
		Dimension minSize = new Dimension(5, 10);
		Dimension prefSize = new Dimension(5, 100);
		Dimension maxSize = new Dimension(5, Short.MAX_VALUE);
		listPane.add(new Box.Filler(minSize, prefSize, maxSize));
		
		JButton btn = new SlickerButton("Show Place Alignment");
		btn.addActionListener(new ActionListener()
				{
					@Override
					public void actionPerformed(ActionEvent e)
					{
						test();
					}
				});
		//btn.setEnabled(false);
		listPane.add(btn);
		
		return listPane;
	}
	
	private RunnablePAA paa = null;
	
	private void test()
	{
		paa = new RunnablePAA();
		paa.ctxt = uis.getContext().createChildContext("Test");
		paa.net = net;
		paa.annotatedLog = annotatedLog;
		paa.ctxt.getExecutor().execute(paa);
		
//		PlaceAlignmentAnalysis paa = new PlaceAlignmentAnalysis();
//		paa.replayALog(uis.getContext(), net, annotatedLog);
	}

}
