package org.processmining.processprofiler3d.psen.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.framework.util.ui.widgets.ProMList;
import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;
import org.processmining.framework.util.ui.widgets.ProMScrollPane;
import org.processmining.framework.util.ui.widgets.ProMScrollablePanel;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohort;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortIntRange;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortLiteral;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortTimeOfDay;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.processprofiler3d.psen.classification.CohortDefinition;
import org.processmining.processprofiler3d.psen.classification.CohortDefinitionIntRange;
import org.processmining.processprofiler3d.psen.classification.CohortDefinitionLiteral;
import org.processmining.processprofiler3d.psen.classification.CohortDefinitionTimeOfDay;

import com.fluxicon.slickerbox.components.SlickerButton;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class PSENAddGenericCohortPanel extends ProMPropertiesPanel implements ActionListener
{
	private static final long serialVersionUID = 2643942608122666179L;

	private static String panelTitle = "Add Classifier Cohort";
	
	private CohortClassifier mycc;
	private XLog myLog = null;
	private LogAnalyser la = null;
	
	private ProMTextField name;
	
	private ProMList<String> lstAvailableElements;
	private DefaultListModel<String> lmAvailableElements;
	
	private ProMList<String> lstSelectedElements;
	private DefaultListModel<String> lmSelectedElements;
	
	private SlickerButton btnAdd;
	private SlickerButton btnRemove;
	
	private JCheckBox cbxOnlyUnassigned;
	private ProMComboBox<String> cbxType;
	private ProMComboBox<String> cbxAttribute;

	private ProMTextField start;
	private ProMTextField end;
	private JCheckBox cbxInclStart;
	private JCheckBox cbxInclEnd;
	
	private JPanel wrapPanel;
	
	private Vector<String> allAvailableValues = null;

	public static void showDialog(CohortClassifier cc, XLog l)
	{
		int id = cc.getNumCohorts();
		
		PSENAddGenericCohortPanel panel = new PSENAddGenericCohortPanel(cc, l, id);

	    int result = AddCohortPane.showDialog(panel);
		
	    if (result == JOptionPane.OK_OPTION)
	    {	
	    	if (panel.isDataValid(cc))
	    	{
		    	panel.addCohortTo(cc);
	    	}
	    }
	}
	
	public String getCohortName()
	{
		return name.getText();
	}
	
	public HashSet<String> getSelectedValues()
	{	
		HashSet<String> vals = new HashSet<String>();
    	for (int i = 0; i < lmSelectedElements.size(); i++)
    	{
    		vals.add(lmSelectedElements.get(i));
    	}
    	
		return vals;
	}
	
	public int getStartHour()
	{
		//return Integer.parseUnsignedInt(start.getText()); //Java 1.8
		return Integer.parseInt(start.getText());
	}
	
	public int getEndHour()
	{
		//return Integer.parseUnsignedInt(end.getText()); //Java 1.8
		return Integer.parseInt(end.getText());
	}
	
	public String getSelectedAttribute()
	{
		return (String)cbxAttribute.getSelectedItem();
	}
	
	public Vector<String> getAvailableValues()
	{
		Vector<String> lst = new Vector<String>(la.getAvailableTraceAttrValuesLiteral(getSelectedAttribute()));
		Collections.sort(lst);
		return lst;
	}
	
	public Vector<String> getAttributes()
	{
		if (getSelectedClassifierType().equals(ClassificationCohortLiteral.classifierName))
		{
			Vector<String> lst = new Vector<String>(ClassificationCohortLiteral.getAvailableAttributes(la));
			Collections.sort(lst);
			return lst;
		}
		else if (getSelectedClassifierType().equals(ClassificationCohortTimeOfDay.classifierName))
		{
			Vector<String> lst = new Vector<String>(ClassificationCohortTimeOfDay.getAvailableAttributes(la));
			Collections.sort(lst);
			return lst;
		}
		else if (getSelectedClassifierType().equals(ClassificationCohortIntRange.classifierName))
		{
			Vector<String> lst = new Vector<String>(ClassificationCohortIntRange.getAvailableAttributes(la));
			Collections.sort(lst);
			return lst;
		}
		return new Vector<String>();
	}
	
	public String getSelectedClassifierType()
	{
		return (String)cbxType.getSelectedItem();
	}
	
	public Vector<String> getClassifierTypes()
	{
		Vector<String> cohortTypes = new Vector<String>();
		cohortTypes.add(ClassificationCohortLiteral.classifierName);
		cohortTypes.add(ClassificationCohortTimeOfDay.classifierName);
		cohortTypes.add(ClassificationCohortIntRange.classifierName);
		return cohortTypes;
	}
	
	public PSENAddGenericCohortPanel(CohortClassifier cc, XLog l, int id, String title) 
	{
		super(title);
		
		mycc = cc;
		myLog = l;
		la = new LogAnalyser(myLog);
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBackground(this.getBackground());
		setAlignmentX(LEFT_ALIGNMENT);
		
	    cbxType = addComboBox("Classifier Type", getClassifierTypes());
	    cbxType.addActionListener(this);
		
	    cbxAttribute = addComboBox("Attribute", getAttributes());
	    cbxAttribute.addActionListener(this);
		
	    //name = addTextField("Name", "cohort" + (id + 1));
		name = new ProMTextField();
		name.setText("cohort" + (id + 1));
	    name.setAlignmentX(LEFT_ALIGNMENT);
	    
	    addProperty("Cohort Name", name);
		
		wrapPanel = new JPanel();
		wrapPanel.setMinimumSize(new Dimension(3200,420));
		wrapPanel.setPreferredSize(new Dimension(3200,420));
		wrapPanel.setMaximumSize(new Dimension(3200,420));
		
		BoxLayout layout = new BoxLayout(wrapPanel, BoxLayout.X_AXIS);
		
		wrapPanel.setLayout(layout);
		wrapPanel.setBackground(this.getBackground());
		wrapPanel.setAlignmentX(LEFT_ALIGNMENT);
		
		//wrapPanel.add(Box.createGlue());
		wrapPanel.add(CreateTypeDependentPanel());
		//wrapPanel.add(Box.createGlue());
		wrapPanel.setBackground(Color.RED);
		
	    //add(panel);
		ProMScrollablePanel propPanel = getPropertiesPanel();
		propPanel.add(wrapPanel);
	    
	    setPreferredSize(new Dimension(800,620));
	}
	
	public PSENAddGenericCohortPanel(CohortClassifier cc, XLog l, int id) 
	{
		this(cc, l, id, panelTitle);
	}

	private JPanel CreateTypeDependentPanel()
	{
		if (getSelectedClassifierType().equals(ClassificationCohortLiteral.classifierName))
		{
			return CreateLiteralClassifierPanel();
		}
		else if (getSelectedClassifierType().equals(ClassificationCohortTimeOfDay.classifierName))
		{
			return CreateTimeOfDayClassifierPanel();
		}
		else if (getSelectedClassifierType().equals(ClassificationCohortIntRange.classifierName))
		{
			return CreateIntegerRangeClassifierPanel();
		}
		return null;
	}

	private JPanel CreateLiteralClassifierPanel()
	{
		JPanel outerPanel = new JPanel();
		
		BoxLayout layout3 = new BoxLayout(outerPanel, BoxLayout.Y_AXIS);
		
		outerPanel.setLayout(layout3);
		outerPanel.setBackground(this.getBackground());
		//outerPanel.setBackground(Color.RED);
		
		JPanel panel = new JPanel();
		
		BoxLayout layout = new BoxLayout(panel, BoxLayout.X_AXIS);
		
		panel.setLayout(layout);
		panel.setBackground(this.getBackground());

		allAvailableValues = getAvailableValues();
	    
		lmAvailableElements = new DefaultListModel<String>();
		for (String s : allAvailableValues)
		{
			lmAvailableElements.addElement(s);
		}
		
		lstAvailableElements = new ProMList<String>("Available Values", lmAvailableElements);
		lstAvailableElements.setPreferredSize(new Dimension(200, 400));
		
		panel.add(lstAvailableElements);

		//create column of buttons between lists
		JPanel panel2 = new JPanel();
		BoxLayout layout2 = new BoxLayout(panel2, BoxLayout.Y_AXIS);
		panel2.setLayout(layout2);
		panel2.setBackground(this.getBackground());
		
	    btnAdd = new SlickerButton("Add");
	    btnAdd.setAlignmentX(CENTER_ALIGNMENT);
	    btnAdd.addActionListener(this);
	    panel2.add(btnAdd);
	    
	    btnRemove = new SlickerButton("Remove");
	    btnRemove.setAlignmentX(CENTER_ALIGNMENT);
	    btnRemove.addActionListener(this);
	    panel2.add(btnRemove);
	    
	    panel.add(panel2);

		lmSelectedElements = new DefaultListModel<String>();
		
		lstSelectedElements = new ProMList<String>("Selected Values", lmSelectedElements);
		lstSelectedElements.setPreferredSize(new Dimension(200, 400));
		
		panel.add(lstSelectedElements);	
		panel.setAlignmentX(LEFT_ALIGNMENT);
		
		outerPanel.add(panel);
		
	    cbxOnlyUnassigned = new JCheckBox("Only show unassigned");
	    cbxOnlyUnassigned.setAlignmentX(LEFT_ALIGNMENT);
	    cbxOnlyUnassigned.addActionListener(this);
	    cbxOnlyUnassigned.setEnabled(mycc != null);
	    //add(cbxOnlyUnassigned);
	    outerPanel.add(cbxOnlyUnassigned);
		
		return outerPanel;
	}

	private JPanel CreateTimeOfDayClassifierPanel()
	{		
		JPanel panel = new JPanel();
		
		BoxLayout layout = new BoxLayout(panel, BoxLayout.Y_AXIS);
		
		panel.setLayout(layout);
		panel.setBackground(this.getBackground());

		final JPanel panelA = new JPanel();
		panelA.setOpaque(false);
		panelA.setBackground(new Color(60, 60, 60, 0));
		BoxLayout layout3 = new BoxLayout(panelA, BoxLayout.X_AXIS);
		panelA.setLayout(layout3);

		final JPanel panelB = new JPanel();
		panelB.setOpaque(false);
		panelB.setBackground(new Color(60, 60, 60, 0));
		BoxLayout layout2 = new BoxLayout(panelB, BoxLayout.X_AXIS);
		panelB.setLayout(layout2);

		Dimension labelSize = new Dimension(75,25);
		
		JLabel lblStart = new JLabel("Start Time");
		lblStart.setMinimumSize(labelSize);
		lblStart.setPreferredSize(labelSize);
		lblStart.setMaximumSize(labelSize);
		panelA.add(lblStart);

		JLabel lblEnd = new JLabel("End Time");
		lblEnd.setMinimumSize(labelSize);
		lblEnd.setPreferredSize(labelSize);
		lblEnd.setMaximumSize(labelSize);
		panelB.add(lblEnd);

	    start = new ProMTextField();
	    start.setText("0");
	    start.setAlignmentX(LEFT_ALIGNMENT);
	    
	    panelA.add(start);

	    end = new ProMTextField();
	    end.setText("1");
	    end.setAlignmentX(LEFT_ALIGNMENT);
	    
	    panelB.add(end);
	    
	    cbxInclStart = SlickerFactory.instance().createCheckBox("Inclusive", false);
	    //cbxInclStart = new JCheckBox("Inclusive");
	    cbxInclStart.setOpaque(false);
	    cbxInclStart.setForeground(WidgetColors.TEXT_COLOR);
	    cbxInclStart.setAlignmentX(LEFT_ALIGNMENT);
	    cbxInclStart.setBackground(panel.getBackground());
	    cbxInclStart.addActionListener(this);
	    panelA.add(cbxInclStart);
	    
	    //cbxInclEnd = new JCheckBox("Inclusive");
	    cbxInclEnd = SlickerFactory.instance().createCheckBox("Inclusive", false);
	    cbxInclEnd.setOpaque(false);
	    cbxInclEnd.setForeground(WidgetColors.TEXT_COLOR);
	    cbxInclEnd.setAlignmentX(LEFT_ALIGNMENT);
	    cbxInclEnd.setBackground(panelB.getBackground());
	    cbxInclEnd.addActionListener(this);
	    panelB.add(cbxInclEnd);
	    
	    panel.add(panelA);
	    panel.add(panelB);
		panel.add(Box.createGlue());
	    
		return panel;
	}

	private JPanel CreateIntegerRangeClassifierPanel()
	{
		JPanel panel = new JPanel();
		
		BoxLayout layout = new BoxLayout(panel, BoxLayout.Y_AXIS);
		
		panel.setLayout(layout);
		panel.setBackground(this.getBackground());

		final JPanel panelA = new JPanel();
		panelA.setOpaque(false);
		panelA.setBackground(new Color(60, 60, 60, 0));
		BoxLayout layout3 = new BoxLayout(panelA, BoxLayout.X_AXIS);
		panelA.setLayout(layout3);

		final JPanel panelB = new JPanel();
		panelB.setOpaque(false);
		panelB.setBackground(new Color(60, 60, 60, 0));
		BoxLayout layout2 = new BoxLayout(panelB, BoxLayout.X_AXIS);
		panelB.setLayout(layout2);

		Dimension labelSize = new Dimension(75,25);
		
		JLabel lblStart = new JLabel("Start");
		lblStart.setMinimumSize(labelSize);
		lblStart.setPreferredSize(labelSize);
		lblStart.setMaximumSize(labelSize);
		panelA.add(lblStart);

		JLabel lblEnd = new JLabel("End");
		lblEnd.setMinimumSize(labelSize);
		lblEnd.setPreferredSize(labelSize);
		lblEnd.setMaximumSize(labelSize);
		panelB.add(lblEnd);
		
	    start = new ProMTextField();
	    start.setText("0");
	    start.setAlignmentX(LEFT_ALIGNMENT);
	    
	    panelA.add(start);

	    end = new ProMTextField();
	    end.setText("1");
	    end.setAlignmentX(LEFT_ALIGNMENT);
	    
	    panelB.add(end);
	    
	    cbxInclStart = SlickerFactory.instance().createCheckBox("Inclusive", false);
	    //cbxInclStart = new JCheckBox("Inclusive");
	    cbxInclStart.setOpaque(false);
	    cbxInclStart.setForeground(WidgetColors.TEXT_COLOR);
	    cbxInclStart.setAlignmentX(LEFT_ALIGNMENT);
	    cbxInclStart.setBackground(panel.getBackground());
	    cbxInclStart.addActionListener(this);
	    panelA.add(cbxInclStart);
	    
	    //cbxInclEnd = new JCheckBox("Inclusive");
	    cbxInclEnd = SlickerFactory.instance().createCheckBox("Inclusive", false);
	    cbxInclEnd.setOpaque(false);
	    cbxInclEnd.setForeground(WidgetColors.TEXT_COLOR);
	    cbxInclEnd.setAlignmentX(LEFT_ALIGNMENT);
	    cbxInclEnd.setBackground(panelB.getBackground());
	    cbxInclEnd.addActionListener(this);
	    panelB.add(cbxInclEnd);
	    
	    panel.add(panelA);
	    panel.add(panelB);
		panel.add(Box.createGlue());
	    
		return panel;
	}
	
	private ProMScrollablePanel getPropertiesPanel()
	{
		for (Component c : this.getComponents())
		{
			if (c instanceof ProMScrollPane)
			{
				JComponent ccx = (JComponent)((ProMScrollPane)c).getComponent(0);
				return (ProMScrollablePanel)ccx.getComponent(0);
			}
		}
		
		return null;
	}
	
	public boolean isDataValid(CohortClassifier cc)
	{
    	String cname = getCohortName();
    	
    	boolean bDataIsValid = true;
    	if (cc.getCohortNames().contains(cname))
    	{
    		bDataIsValid = false;
    	}  	
    	
		return bDataIsValid;
	}
	
	public CohortDefinition getCohortDefinition()
	{
    	String cname = getCohortName();
		if (getSelectedClassifierType().equals(ClassificationCohortLiteral.classifierName))
		{
	    	HashSet<String> vals = getSelectedValues();
	    	
			return new CohortDefinitionLiteral(cname, getSelectedAttribute(), vals);
		}
		else if (getSelectedClassifierType().equals(ClassificationCohortTimeOfDay.classifierName))
		{
	    	int startHrs = getStartHour();
	    	int endHrs = getEndHour();
	    	
	    	return new CohortDefinitionTimeOfDay(cname, getSelectedAttribute(), startHrs, endHrs, cbxInclStart.isSelected(), cbxInclEnd.isSelected());
			
		}
		else if (getSelectedClassifierType().equals(ClassificationCohortIntRange.classifierName))
		{
	    	int startHrs = getStartHour();
	    	int endHrs = getEndHour();
	    	
	    	return new CohortDefinitionIntRange(cname, getSelectedAttribute(), startHrs, endHrs, cbxInclStart.isSelected(), cbxInclEnd.isSelected());
		}
		return null;
	}
	
	public void addCohortTo(CohortClassifier cc)
	{
		CohortDefinition cdef = getCohortDefinition();
		cc.addCohort(cdef);
	}

	public void actionPerformed(ActionEvent arg0) 
	{
		//add button pressed
		if (arg0.getSource() == btnAdd)
		{
			addPressed();
		}
		//remove button pressed
		else if (arg0.getSource() == btnRemove)
		{
			removePressed();
		}
		else if (arg0.getSource() == cbxOnlyUnassigned)
		{
			checkboxToggled();
		}
		else if (arg0.getSource() == cbxAttribute)
		{
			attributeChanged();
		}
		else if (arg0.getSource() == cbxType)
		{
			typeChanged();
		}
	}
	
	private void addPressed()
	{
		List<String> vals = lstAvailableElements.getSelectedValuesList();

		for (String s : vals)
		{
			lmAvailableElements.removeElement(s);
		}
		lstAvailableElements.invalidate();
		lstAvailableElements.validate();
		lstAvailableElements.repaint();
		
		//update contents of list
		//lm2.removeAllElements();
		for (String s : vals)
		{
			lmSelectedElements.addElement(s);
		}
		lstSelectedElements.invalidate();
		lstSelectedElements.validate();
		lstSelectedElements.repaint();
	}
	
	private void removePressed()
	{
		List<String> vals = lstSelectedElements.getSelectedValuesList();

		for (String s : vals)
		{
			lmSelectedElements.removeElement(s);
		}
		lstSelectedElements.invalidate();
		lstSelectedElements.validate();
		lstSelectedElements.repaint();
		
		//update contents of list
		//lm.removeAllElements();
		for (String s : vals)
		{
			lmAvailableElements.addElement(s);
		}
		lstAvailableElements.invalidate();
		lstAvailableElements.validate();
		lstAvailableElements.repaint();
	}
	
	private void checkboxToggled()
	{
		lmAvailableElements.removeAllElements();
		if (cbxOnlyUnassigned.isSelected())
		{
			for (String s : getUnassignedValues())
			{
				if (!lmSelectedElements.contains(s))
				{
					lmAvailableElements.addElement(s);
				}
			}
		}
		else
		{
			for (String s : allAvailableValues)
			{
				if (!lmSelectedElements.contains(s))
				{
					lmAvailableElements.addElement(s);
				}
			}
		}
		lstAvailableElements.invalidate();
		lstAvailableElements.validate();
		lstAvailableElements.repaint();
		
		//System.out.println("Cbx toggled.");
	}
	
	private void attributeChanged()
	{
		//System.out.println("Attribute: " + getAttribute());
		allAvailableValues = getAvailableValues();
		
		lmAvailableElements.clear();
		for (String s : allAvailableValues)
		{
			lmAvailableElements.addElement(s);
		}
		lstAvailableElements.invalidate();
		lstAvailableElements.validate();
		lstAvailableElements.repaint();
		
		lmSelectedElements.clear();
		lstSelectedElements.invalidate();
		lstSelectedElements.validate();
		lstSelectedElements.repaint();		
	}
	
	private void typeChanged()
	{
		cbxAttribute.setModel(new DefaultComboBoxModel<>(getAttributes()));
		
		JPanel p = CreateTypeDependentPanel();
		
		wrapPanel.removeAll();
		wrapPanel.add(p);
	}
	
	public Vector<String> getUnassignedValues()
	{
		Vector<String> avalues = new Vector<String>(allAvailableValues);
		if (mycc != null)
		{
			//remove all values that are already covered by existing cohorts from avalues
			//CAN EASILY BE MORE OPTIMIZED!
			for (ClassificationCohort cc : mycc.getCohorts())
			{
				if (cc instanceof ClassificationCohortLiteral)
				{
					ClassificationCohortLiteral ccl = (ClassificationCohortLiteral)cc;
					
					if (ccl.attrName.equals(getSelectedAttribute()))
					{
						int i = 0;
						while (i < avalues.size())
						{
							if (ccl.containsValue(avalues.get(i)))
							{
								avalues.remove(i);
							}
							else
							{
								i++;
							}
						}
					}
				}
			}
		}
		
		return avalues;
	}

}
