package org.processmining.processprofiler3d.psen.gui;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.framework.util.ui.widgets.ProMList;
import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohort;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortLiteral;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.processprofiler3d.psen.classification.CohortDefinitionLiteral;

import com.fluxicon.slickerbox.components.SlickerButton;

public class PSENAddGenericLiteralCohortPanel extends ProMPropertiesPanel implements ActionListener
{
	private static final long serialVersionUID = 2643942608122666179L;

	private CohortClassifier mycc;
	private XLog myLog = null;
	private LogAnalyser la = null;
	
	private ProMTextField name;
	
	private ProMList<String> lstAvailableElements;
	private DefaultListModel<String> lmAvailableElements;
	
	private ProMList<String> lstSelectedElements;
	private DefaultListModel<String> lmSelectedElements;
	
	private SlickerButton btnAdd;
	private SlickerButton btnRemove;
	
	private JCheckBox cbxOnlyUnassigned;
	private ProMComboBox<String> cbxAttribute;
	
	private Vector<String> allAvailableValues = null;

	public static void showDialog(CohortClassifier cc, XLog l)
	{
		int id = cc.getNumCohorts();
		
		PSENAddGenericLiteralCohortPanel panel = new PSENAddGenericLiteralCohortPanel(cc, l, id);

	    int result = AddCohortPane.showDialog(panel);
		
	    if (result == JOptionPane.OK_OPTION)
	    {	
	    	if (panel.isDataValid(cc))
	    	{
		    	panel.addCohortTo(cc);
	    	}
	    }
	}
	
	public String getCohortName()
	{
		return name.getText();
	}
	
	public HashSet<String> getSelectedValues()
	{	
		HashSet<String> vals = new HashSet<String>();
    	for (int i = 0; i < lmSelectedElements.size(); i++)
    	{
    		vals.add(lmSelectedElements.get(i));
    	}
    	
		return vals;
	}
	
	public String getAttribute()
	{
		return (String)cbxAttribute.getSelectedItem();
	}
	
	public Vector<String> getAvailableValues()
	{
		Vector<String> lst = new Vector<String>(la.getAvailableTraceAttrValuesLiteral(getAttribute()));
		Collections.sort(lst);
		return lst;
	}
	
	public Vector<String> getAttributes()
	{
		Vector<String> lst = new Vector<String>(ClassificationCohortLiteral.getAvailableAttributes(la));
		Collections.sort(lst);
		return lst;
	}
	
	public PSENAddGenericLiteralCohortPanel(CohortClassifier cc, XLog l, int id) 
	{
		super("Add Classifier Cohort");
		
		mycc = cc;
		myLog = l;
		la = new LogAnalyser(myLog);
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBackground(this.getBackground());
		setAlignmentX(LEFT_ALIGNMENT);
		
		JPanel panel = new JPanel();
		
		BoxLayout layout = new BoxLayout(panel, BoxLayout.X_AXIS);
		
		panel.setLayout(layout);
		panel.setBackground(this.getBackground());
		
	    cbxAttribute = addComboBox("Attribute", getAttributes());
	    cbxAttribute.addActionListener(this);
		
	    //name = addTextField("Name", "cohort" + (id + 1));
		name = new ProMTextField();
		name.setText("cohort" + (id + 1));
	    name.setAlignmentX(LEFT_ALIGNMENT);
	    
	    add(name);

		allAvailableValues = getAvailableValues();
	    
		lmAvailableElements = new DefaultListModel<String>();
		for (String s : allAvailableValues)
		{
			lmAvailableElements.addElement(s);
		}
		
		lstAvailableElements = new ProMList<String>("Available Values", lmAvailableElements);
		lstAvailableElements.setPreferredSize(new Dimension(200, 500));
		
		panel.add(lstAvailableElements);

		//create column of buttons between lists
		JPanel panel2 = new JPanel();
		BoxLayout layout2 = new BoxLayout(panel2, BoxLayout.Y_AXIS);
		panel2.setLayout(layout2);
		panel2.setBackground(this.getBackground());
		
	    btnAdd = new SlickerButton("Add");
	    btnAdd.setAlignmentX(CENTER_ALIGNMENT);
	    btnAdd.addActionListener(this);
	    panel2.add(btnAdd);
	    
	    btnRemove = new SlickerButton("Remove");
	    btnRemove.setAlignmentX(CENTER_ALIGNMENT);
	    btnRemove.addActionListener(this);
	    panel2.add(btnRemove);
	    
	    panel.add(panel2);

		lmSelectedElements = new DefaultListModel<String>();
		
		lstSelectedElements = new ProMList<String>("Selected Values", lmSelectedElements);
		lstSelectedElements.setPreferredSize(new Dimension(200, 500));
		
		panel.add(lstSelectedElements);
		panel.setAlignmentX(LEFT_ALIGNMENT);
		
	    add(panel);

	    cbxOnlyUnassigned = new JCheckBox("Only show unassigned");
	    cbxOnlyUnassigned.setAlignmentX(LEFT_ALIGNMENT);
	    cbxOnlyUnassigned.addActionListener(this);
	    add(cbxOnlyUnassigned);
	    
	    setPreferredSize(new Dimension(800,620));
	}
	
	public boolean isDataValid(CohortClassifier cc)
	{
    	String cname = getCohortName();
    	
    	boolean bDataIsValid = true;
    	if (cc.getCohortNames().contains(cname))
    	{
    		bDataIsValid = false;
    	}  	
    	
		return bDataIsValid;
	}
	
	public void addCohortTo(CohortClassifier cc)
	{
    	String cname = getCohortName();
    	HashSet<String> vals = getSelectedValues();
    	
		cc.addCohort(new CohortDefinitionLiteral(cname, getAttribute(), vals));
	}

	public void actionPerformed(ActionEvent arg0) 
	{
		//add button pressed
		if (arg0.getSource() == btnAdd)
		{
			addPressed();
		}
		//remove button pressed
		else if (arg0.getSource() == btnRemove)
		{
			removePressed();
		}
		else if (arg0.getSource() == cbxOnlyUnassigned)
		{
			checkboxToggled();
		}
		else if (arg0.getSource() == cbxAttribute)
		{
			attributeChanged();
		}
	}
	
	private void addPressed()
	{
		List<String> vals = lstAvailableElements.getSelectedValuesList();

		for (String s : vals)
		{
			lmAvailableElements.removeElement(s);
		}
		lstAvailableElements.invalidate();
		lstAvailableElements.validate();
		lstAvailableElements.repaint();
		
		//update contents of list
		//lm2.removeAllElements();
		for (String s : vals)
		{
			lmSelectedElements.addElement(s);
		}
		lstSelectedElements.invalidate();
		lstSelectedElements.validate();
		lstSelectedElements.repaint();
	}
	
	private void removePressed()
	{
		List<String> vals = lstSelectedElements.getSelectedValuesList();

		for (String s : vals)
		{
			lmSelectedElements.removeElement(s);
		}
		lstSelectedElements.invalidate();
		lstSelectedElements.validate();
		lstSelectedElements.repaint();
		
		//update contents of list
		//lm.removeAllElements();
		for (String s : vals)
		{
			lmAvailableElements.addElement(s);
		}
		lstAvailableElements.invalidate();
		lstAvailableElements.validate();
		lstAvailableElements.repaint();
	}
	
	private void checkboxToggled()
	{
		lmAvailableElements.removeAllElements();
		if (cbxOnlyUnassigned.isSelected())
		{
			for (String s : getUnassignedValues())
			{
				if (!lmSelectedElements.contains(s))
				{
					lmAvailableElements.addElement(s);
				}
			}
		}
		else
		{
			for (String s : allAvailableValues)
			{
				if (!lmSelectedElements.contains(s))
				{
					lmAvailableElements.addElement(s);
				}
			}
		}
		lstAvailableElements.invalidate();
		lstAvailableElements.validate();
		lstAvailableElements.repaint();
		
		//System.out.println("Cbx toggled.");
	}
	
	private void attributeChanged()
	{
		//System.out.println("Attribute: " + getAttribute());
		allAvailableValues = getAvailableValues();
		
		lmAvailableElements.clear();
		for (String s : allAvailableValues)
		{
			lmAvailableElements.addElement(s);
		}
		lstAvailableElements.invalidate();
		lstAvailableElements.validate();
		lstAvailableElements.repaint();
		
		lmSelectedElements.clear();
		lstSelectedElements.invalidate();
		lstSelectedElements.validate();
		lstSelectedElements.repaint();		
	}
	
	public Vector<String> getUnassignedValues()
	{
		Vector<String> avalues = new Vector<String>(allAvailableValues);
		
		//remove all values that are already covered by existing cohorts from avalues
		//CAN EASILY BE MORE OPTIMIZED!
		for (ClassificationCohort cc : mycc.getCohorts())
		{
			if (cc instanceof ClassificationCohortLiteral)
			{
				ClassificationCohortLiteral ccl = (ClassificationCohortLiteral)cc;
				
				if (ccl.attrName.equals(getAttribute()))
				{
					int i = 0;
					while (i < avalues.size())
					{
						if (ccl.containsValue(avalues.get(i)))
						{
							avalues.remove(i);
						}
						else
						{
							i++;
						}
					}
				}
			}
		}
		
		return avalues;
	}

}
