package org.processmining.processprofiler3d.psen.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortTimeOfDay;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;
import org.processmining.processprofiler3d.psen.classification.CohortDefinitionTimeOfDay;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class PSENAddTimeOfDayCohortPanel extends ProMPropertiesPanel implements ActionListener
{
	private static final long serialVersionUID = 5888441569703132935L;

	private ProMComboBox<String> cbxAttribute;
	private ProMTextField name;
	private ProMTextField start;
	private ProMTextField end;

	private CohortClassifier mycc;
	private XLog myLog = null;
	private LogAnalyser la = null;
	
	private JCheckBox cbxInclStart;
	private JCheckBox cbxInclEnd;

	public static void showDialog(CohortClassifier cc, XLog l)
	{
		int id = cc.getNumCohorts();
		
		PSENAddTimeOfDayCohortPanel todcp = new PSENAddTimeOfDayCohortPanel(cc, l, id);

		int result = AddCohortPane.showDialog(todcp);
		
	    //System.out.println(result);
	    
	    if (result == JOptionPane.OK_OPTION)
	    {	
	    	if (todcp.isDataValid(cc))
	    	{
	    		todcp.addCohortTo(cc);
	    	}
	    }
	}
	
	public String getAttribute()
	{
		return (String)cbxAttribute.getSelectedItem();
	}
	
	public String getCohortName()
	{
		return name.getText();
	}
	
	public int getStartHour()
	{
		//return Integer.parseUnsignedInt(start.getText()); //Java 1.8
		return Integer.parseInt(start.getText());
	}
	
	public int getEndHour()
	{
		//return Integer.parseUnsignedInt(end.getText()); //Java 1.8
		return Integer.parseInt(end.getText());
	}
	
	public Vector<String> getAttributes()
	{
		Vector<String> lst = new Vector<String>(ClassificationCohortTimeOfDay.getAvailableAttributes(la));
		Collections.sort(lst);
		return lst;
	}
	
	public PSENAddTimeOfDayCohortPanel(CohortClassifier cc, XLog l, int id)
	{	
		super("Add Classifier Cohort");

		mycc = cc;
		myLog = l;
		la = new LogAnalyser(myLog);
		
		BoxLayout layout = new BoxLayout(this, BoxLayout.Y_AXIS);
		setLayout(layout);
		this.setAlignmentY(TOP_ALIGNMENT);

	    cbxAttribute = addComboBox("Attribute", getAttributes());
	    //cbxAttribute.addActionListener(this);
	    
	    name = addTextField("Name", "cohort" + (id + 1));
	    //start = addTextField("Start Time", "0");
	    //end = addTextField("End Time", "1");

		final JPanel panel = new JPanel();
		panel.setOpaque(false);
		panel.setBackground(new Color(60, 60, 60, 0));
		BoxLayout layout3 = new BoxLayout(panel, BoxLayout.X_AXIS);
		panel.setLayout(layout3);

		final JPanel panel2 = new JPanel();
		panel2.setOpaque(false);
		panel2.setBackground(new Color(60, 60, 60, 0));
		BoxLayout layout2 = new BoxLayout(panel2, BoxLayout.X_AXIS);
		panel2.setLayout(layout2);

	    start = new ProMTextField();
	    start.setText("0");
	    start.setAlignmentX(LEFT_ALIGNMENT);
	    
	    panel.add(start);

	    end = new ProMTextField();
	    end.setText("1");
	    end.setAlignmentX(LEFT_ALIGNMENT);
	    
	    panel2.add(end);
	    
	    cbxInclStart = SlickerFactory.instance().createCheckBox("Inclusive", false);
	    //cbxInclStart = new JCheckBox("Inclusive");
	    cbxInclStart.setOpaque(false);
	    cbxInclStart.setForeground(WidgetColors.TEXT_COLOR);
	    cbxInclStart.setAlignmentX(LEFT_ALIGNMENT);
	    cbxInclStart.setBackground(panel.getBackground());
	    cbxInclStart.addActionListener(this);
	    panel.add(cbxInclStart);
	    
	    //cbxInclEnd = new JCheckBox("Inclusive");
	    cbxInclEnd = SlickerFactory.instance().createCheckBox("Inclusive", false);
	    cbxInclEnd.setOpaque(false);
	    cbxInclEnd.setForeground(WidgetColors.TEXT_COLOR);
	    cbxInclEnd.setAlignmentX(LEFT_ALIGNMENT);
	    cbxInclEnd.setBackground(panel2.getBackground());
	    cbxInclEnd.addActionListener(this);
	    panel2.add(cbxInclEnd);
	    
	    addProperty("Start Time", panel);
	    addProperty("End Time", panel2);
//	    add(Box.createVerticalStrut(3));
//	    add(panel);
//	    add(Box.createVerticalStrut(3));
//	    add(panel2);
	    
	    //setBounds(0, 0, 3000, 1000);
	    setPreferredSize(new Dimension(1000,300));
	}
	
	public boolean isDataValid(CohortClassifier cc)
	{
    	String cname = getCohortName();
    	int startHrs = getStartHour();
    	int endHrs = getEndHour();
    	
    	//validate data provided by user
    	boolean bDataIsValid = true;
    	if (startHrs < 0 || startHrs > 24)
    	{
    		bDataIsValid = false;
    	}
    	if (endHrs < 0 || endHrs > 24)
    	{
    		bDataIsValid = false;
    	}
    	if (endHrs < startHrs)
    	{
    		bDataIsValid = false;
    	}
    	if (cc.getCohortNames().contains(cname))
    	{
    		bDataIsValid = false;
    	}	
    	
		return bDataIsValid;
	}

	public void actionPerformed(ActionEvent arg0) 
	{
		if (arg0.getSource() == cbxInclStart)
		{
			//checkboxToggled();
		}
		else if (arg0.getSource() == cbxInclEnd)
		{
			//attributeChanged();
		}
	}
	
	public void addCohortTo(CohortClassifier cc)
	{
    	String cname = getCohortName();
    	int startHrs = getStartHour();
    	int endHrs = getEndHour();
    	
		cc.addCohort(new CohortDefinitionTimeOfDay(cname, getAttribute(), startHrs, endHrs, cbxInclStart.isSelected(), cbxInclEnd.isSelected()));
	}
	
}
