package org.processmining.processprofiler3d.psen.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.Vector;

import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortIntRange;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortLiteral;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohortTimeOfDay;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;

public class PSENClassifierConfigPanel extends ProMPropertiesPanel implements ActionListener
{
	private static final long serialVersionUID = -6472747647763402268L;
	
	private ProMComboBox<String> cbxClassifierType;
	private ProMComboBox<String> cbxAttribute;
	private ProMComboBox<String> cbxPreset;

	private XLog log;
	private LogAnalyser la;
	
	public PSENClassifierConfigPanel(XLog log) 
	{
		super("Classifier Configuration");

		this.log = log;
		la = new LogAnalyser(log);
		
		Vector<String> cohortTypes = new Vector<String>();
		cohortTypes.add(ClassificationCohortLiteral.classifierName);
		cohortTypes.add(ClassificationCohortTimeOfDay.classifierName);
		cohortTypes.add(ClassificationCohortIntRange.classifierName);
		
	    cbxClassifierType = addComboBox("Classifier type", cohortTypes);
	    Vector<String> attrs = null;
	    if (la.HasGlobalTraceAttributes())
	    {
	    	attrs = new Vector<String>(la.getAvailableTraceAttrsLiteral());
	    }
	    else
	    {
	    	attrs = new Vector<String>(la.calcAvailableTraceAttrsLiteral());
	    }
	    Collections.sort(attrs);
    	cbxAttribute = addComboBox("Attribute", attrs);
	    cbxPreset = addComboBox("Preset", CohortClassifier.getPresetNames(getSelectedAttribute(), log));
	    
	    cbxClassifierType.addActionListener(this);
	}
	
	public String getSelectedClassifierType()
	{
		return (String)cbxClassifierType.getSelectedItem();
	}
	
	public String getSelectedAttribute()
	{
		return (String)cbxAttribute.getSelectedItem();
	}
	
	public String getSelectedPreset()
	{
		return (String)cbxPreset.getSelectedItem();
	}
	
	public CohortClassifier getClassifier()
	{
    	CohortClassifier cc = new CohortClassifier();
    	return cc;
	}

	public void actionPerformed(ActionEvent arg0) 
	{
		//classifier type has been changed
		if (arg0.getSource() == cbxClassifierType)
		{
			cbxAttribute.removeAllItems();
			cbxPreset.removeAllItems();
			
			Vector<String> attrs = null;
			Vector<String> presets = null;
	    	String selectedClassifierType = getSelectedClassifierType();
	    	if (selectedClassifierType.equals(ClassificationCohortTimeOfDay.classifierName))
	    	{
	    		attrs = new Vector<String>(ClassificationCohortTimeOfDay.getAvailableAttributes(la));
	    		presets = new Vector<String>(CohortClassifier.getPresetNames(getSelectedAttribute(), log));
	    	}
	    	else if (selectedClassifierType.equals(ClassificationCohortLiteral.classifierName))
	    	{
	    		attrs = new Vector<String>(ClassificationCohortLiteral.getAvailableAttributes(la));
	    		presets = new Vector<String>(CohortClassifier.getPresetNames(getSelectedAttribute(), log));
	    	}	
	    	else if (selectedClassifierType.equals(ClassificationCohortIntRange.classifierName))
	    	{
	    		attrs = new Vector<String>(ClassificationCohortIntRange.getAvailableAttributes(la));
	    		presets = new Vector<String>(CohortClassifier.getPresetNames(getSelectedAttribute(), log));
	    	}
	    	
	    	Collections.sort(attrs);
    		cbxAttribute.addAllItems(attrs);
			cbxPreset.addAllItems(presets);
		}
	}

}
