package org.processmining.processprofiler3d.psen.gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JViewport;

import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;
import org.processmining.framework.util.ui.widgets.ProMScrollPane;
import org.processmining.framework.util.ui.widgets.ProMScrollablePanel;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;

import com.fluxicon.slickerbox.components.RoundedPanel;

public class TransitionVisConfigPanel extends ProMPropertiesPanel implements ActionListener
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -3609968130649266671L;

	public TransitionVisConfigPanel(Petrinet hierarchicalNet)
	{
		super("Transitions");
		//this.
		//check net structure
		PetriNetDataSource pnds = new PetriNetDataSource(hierarchicalNet);

		BoxLayout layout = new BoxLayout(this, BoxLayout.Y_AXIS);
		setLayout(layout);
		Helper(pnds, this);
		
		FixLabelLayouts();
	}

	public void FixLabelLayouts() 
	{
		for (Component c : this.getComponents())
		{
			if (c instanceof ProMScrollPane)
			{
				ProMScrollPane psp = (ProMScrollPane)c;

				for (Component cc : psp.getComponents())
				{
					if (cc instanceof JViewport)
					{
						JViewport vp = (JViewport)cc;
						for (Component ccc : vp.getComponents())
						{
							if (ccc instanceof ProMScrollablePanel)
							{
								ProMScrollablePanel pspp = (ProMScrollablePanel)ccc;
								for (Component cccc : pspp.getComponents())
								{
									if (cccc instanceof RoundedPanel)
									{
										RoundedPanel rp = (RoundedPanel)cccc;
										for (Component ccccc : rp.getComponents())
										{
											if (ccccc instanceof JLabel)
											{
												JLabel l = (JLabel)ccccc;
												l.setMaximumSize(new Dimension(400,30));
												l.setPreferredSize(new Dimension(400,30));
											}
										}
										
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	private HashMap<ProMComboBox<String>, Transition> tmap = new HashMap<ProMComboBox<String>, Transition>();

	public Vector<Transition> getAllTransitions(INetDataSource inds)
	{
		Vector<Transition> lst = new Vector<Transition>();
		
		lst.addAll(inds.getTransitions());
		
		for (INetDataSource indsc : inds.getSubnets())
		{
			lst.addAll(getAllTransitions(indsc));
		}
		
		return lst;
	}
	
	private void Helper(INetDataSource src, JPanel panel)
	{	
		Vector<String> items = new Vector<String>();
		items.add("Visible");
		items.add("Invisible");
		
		Vector<Transition> allTransitions = getAllTransitions(src);
		//sort list alphabetically
		Collections.sort(allTransitions, new Comparator<Transition>()
		{
		     public int compare(Transition o1, Transition o2)
		     {
		         return o1.getLabel().compareTo(o2.getLabel());
		     }
		});
		
		for (Transition t : allTransitions)
		{
			
			ProMComboBox<String> jcb = this.addComboBox(t.getLabel(), items);
			jcb.setMaximumSize(new Dimension(200,30));
			jcb.setPreferredSize(new Dimension(200,30));
			
			if (t.isInvisible())
			{
				jcb.setSelectedItem("Invisible");
			}
			else
			{
				jcb.setSelectedItem("Visible");
			}
			
			jcb.addActionListener(this);
			
			tmap.put(jcb, t);
		}
	}

	public void actionPerformed(ActionEvent e) 
	{
		if (e.getSource() instanceof ProMComboBox<?>)
		{
			ProMComboBox<String> jcb = (ProMComboBox<String>)e.getSource();
			Transition t = tmap.get(jcb);
			boolean bVis = jcb.getSelectedItem().equals("Visible");
			t.setInvisible(!bVis);
		}
	}

}
