package org.processmining.processprofiler3d.psen.gui;

import java.awt.EventQueue;
import java.util.Stack;

import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JRootPane;

import org.deckfour.uitopia.ui.UITopiaFrame;
import org.deckfour.uitopia.ui.main.MainView;
import org.deckfour.uitopia.ui.overlay.OverlayEnclosure;
import org.processmining.contexts.uitopia.UIPluginContext;

public class UIInjectionStack
{
	private UITopiaFrame uf = null;
	private UIPluginContext context = null;
	private MainView mw = null;
	
	private Stack<UIInjectionStackEntry> dialogStack = new Stack<UIInjectionStackEntry>();
	
	public UIInjectionStack(final UIPluginContext context)
	{
		uf = context.getGlobalContext().getUI();
		this.context = context;
		mw = FindMainView(uf);
	}
	
	public UIPluginContext getContext()
	{
		return context;
	}
	
	public void ShowDialog(final UIPluginContext context, OverlayEnclosure overlay)
	{
		if (EventQueue.isDispatchThread())
		{
			System.out.println("Cannot show dialog in this thread!");
		}
		
		UIInjectionStackEntry stackEntry = new UIInjectionStackEntry(context, overlay);
		if (dialogStack.isEmpty())
		{
//			System.out.println("ShowDialog Current thread:" + Thread.currentThread().getId());
		}
		else
		{
//			System.out.println("ShowDialog Current thread:" + Thread.currentThread().getId());
			
			synchronized(this.context)
			{
				UIInjectionStackEntry current = dialogStack.pop();
				
				dialogStack.push(current);		
				dialogStack.push(stackEntry);
				dialogStack.push(current);

				stackEntry.thread = Thread.currentThread();
				
				this.context.notify();
			}
			
			stackEntry.threadwait();
			
			return;
		}
		
		ShowDialogHelper(stackEntry);
	}

	private void ShowDialogHelper(UIInjectionStackEntry stackEntry)
	{
		dialogStack.push(stackEntry);
		mw.showOverlay(stackEntry.overlay);
		
		try
		{
			synchronized(context)
			{
				context.wait();
			}
		}
		catch (InterruptedException e1)
		{
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		mw.hideOverlay(stackEntry.overlay);
		
		stackEntry.threadcontinue();
		
		dialogStack.pop();
		
		if (!dialogStack.isEmpty())
		{
			ShowDialogHelper(dialogStack.pop());
		}
	}

	protected MainView FindMainView(UITopiaFrame uf)
	{
		JRootPane jrp = (JRootPane)uf.getComponent(0);
		JLayeredPane jlp = (JLayeredPane)jrp.getComponent(1);
		JPanel jp1 = (JPanel)jlp.getComponent(0);		
		
		MainView mw = (MainView)jp1.getComponent(0);
		return mw;
	}
	
	public void CloseDialog()
	{
		synchronized(context)
		{
			context.notify();
		}
	}

}
