package org.processmining.processprofiler3d.psen.plugins;


import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.processprofiler3d.psen.connections.PSENConnection;
import org.processmining.processprofiler3d.psen.datastructure.H2MemoryDatabase;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PSENLabel;
import org.processmining.processprofiler3d.psen.datastructure.PerformanceStatisticsDB;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */
public class MergePSENs 
{
	@Plugin(
		name = "Process Profiler 3D: Merge Performance Profiles", 
		parameterLabels = { "Performance Statistics Enriched Net", "Performance Statistics Enriched Net"}, 
		returnLabels = { "Performance Statistics Enriched Net" }, 
		returnTypes = { IPerformanceStatisticsEnrichedNet.class}, 
		userAccessible = true, 
		help = "This plug-in merges two performance statistics databases."
	)
	@UITopiaVariant(
		affiliation = "Queensland University of Technology", 
		author = "Erik Poppe", 
		email = "erik.poppe@qut.edu.au"
	)

	public Object replayALog(final UIPluginContext context, IPerformanceStatisticsEnrichedNet psen1, IPerformanceStatisticsEnrichedNet psen2) 
	{
		//Object results = psen1;
		PerformanceStatisticsDB psennew = null;
		
		H2MemoryDatabase h2db = new H2MemoryDatabase();
		if (h2db.isDriverAvailable())
		{
			psennew = new PerformanceStatisticsDB(h2db);
			psennew.InitPSEN();
			for (PSENLabel l : psen1.getLabels())
			{
				psennew.addLabel(l);
			}
			
			System.out.println("PSEN1 value count: " + psen1.getValueCount());
			System.out.println("PSEN2 value count: " + psen2.getValueCount());
			
			psennew.MergeDB((PerformanceStatisticsDB)psen1);
			psennew.MergeDB((PerformanceStatisticsDB)psen2);
			
			System.out.println("PSENNew value count: " + psennew.getValueCount());
	
			PSENConnection wtic1 = null;
			try
			{
				wtic1 = context.getConnectionManager().getFirstConnection(PSENConnection.class, context, psen1);
				System.out.println("Found waiting time info connection!");
			} 
			catch (ConnectionCannotBeObtained e) 
			{
				System.out.println("Could not find waiting time info connection!");
			}
			
			PSENConnection wtic2 = null;
			try 
			{
				wtic2 = context.getConnectionManager().getFirstConnection(PSENConnection.class, context, psen2);
				System.out.println("Found waiting time info connection!");
			} 
			catch (ConnectionCannotBeObtained e) 
			{
				System.out.println("Could not find waiting time info connection!");
			}
	
			if (wtic1 != null && wtic2 != null)
			{
				IPerformanceStatisticsEnrichedNet wten1 = wtic1.getWaitingTimeEnrichedNet();
				IPerformanceStatisticsEnrichedNet wten2 = wtic2.getWaitingTimeEnrichedNet();
		
				System.out.println("WTEN1 value count: " + wten1.getValueCount());
				System.out.println("WTEN2 value count: " + wten2.getValueCount());
				
				PerformanceStatisticsDB wtennew = new PerformanceStatisticsDB(h2db);
				wtennew.InitWTEN();
				for (PSENLabel l : wten1.getLabels())
				{
					wtennew.addLabel(l);
				}
				
				wtennew.MergeDB((PerformanceStatisticsDB)wten1);
				wtennew.MergeDB((PerformanceStatisticsDB)wten2);
				
				System.out.println("WTENNew value count: " + wtennew.getValueCount());
		
				//add waitingtimeinfoconnection to context
				context.addConnection(new PSENConnection(psennew, wtennew));
				
		//		if (results == null)
		//		{
		//			context.getFutureResult(0).cancel(true);
		//		}
				context.getFutureResult(0).setLabel("Merged PSEN");
			}
			else
			{
				context.getFutureResult(0).cancel(true);
			}
		}
		else
		{
			context.getFutureResult(0).cancel(true);
		}
		return psennew;
	}
}
