package org.processmining.processprofiler3d.psen.plugins;


import java.util.HashSet;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.hub.ProMResourceManager;
import org.processmining.contexts.uitopia.model.ProMResource;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.providedobjects.ProvidedObjectID;
import org.processmining.framework.providedobjects.ProvidedObjectManager;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.processprofiler3d.psen.AlignedLog;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.gui.AnalysisReport;
import org.processmining.processprofiler3d.psen.gui.MarkingWarningDialog;
import org.processmining.processprofiler3d.psen.gui.SlickerOptionPane;
import org.processmining.processprofiler3d.psen.preprocess.LPPRemoveEventAttributes;
import org.processmining.processprofiler3d.psen.preprocess.LPPRemoveTraceAttributes;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */
public class ModelComparisonBase 
{
	@Plugin(
		name = "Process Profiler 3D: Generate Performance Profile", 
		parameterLabels = { "Petri Net", "Event Log"}, 
		returnLabels = { "Performance Statistics Enriched Net", "ConvertedLog" }, 
		returnTypes = { IPerformanceStatisticsEnrichedNet.class, XLog.class}, 
		userAccessible = true, 
		help = "This plug-in compares the differences between cohorts in a process log."
	)
	@UITopiaVariant(
		affiliation = "Queensland University of Technology", 
		author = "Erik Poppe", 
		email = "erik.poppe@qut.edu.au"
	)

	public Object[] replayALog(final UIPluginContext context, Petrinet hierarchicalNet, XLog log) 
	{
		LogAnalyser la = new LogAnalyser(log);

		Object[] results = null;

		//check if trace names are unique
		if (!la.areTraceNamesUnique())
		{
			//SlickerOptionPane.showMessageDialog(new JPanel(), "Trace names in log are not unique! Cannot analyse.");
			SlickerOptionPane.showErrorMessage(context, "Trace names in log are not unique! Cannot analyse.", "Error");
			System.out.println("Trace names in log are not unique! Cannot analyse.");
		}
		else
		{
			boolean bUseAligned = false;
			
			//if log is already annotated with alignment information, no need to align log again for replay
			if (AlignedLog.CheckLogIsAligned(la))
			{
				MarkingWarningDialog mwd = new MarkingWarningDialog("Would you like to skip alignment?", "The selected log contains alignment information.");
				int n = mwd.ShowDialog(context);
				
				if (n == JOptionPane.YES_OPTION)
				{
					bUseAligned = true;
				}
				else
				{
					//remove attributes
					log = RemoveAlignmentInfo(la, log);
				}
			}
			
			if (bUseAligned)
			{
				ModelComparison3 mc = new ModelComparison3();
				IPerformanceStatisticsEnrichedNet psen = mc.replayALog(context, hierarchicalNet, log);
				
				results = new Object[2];
				results[0] = psen;
				results[1] = log;
			}
			//if log is not annotated with alignment information, align log for replay first
			else
			{
				ModelComparison mc = new ModelComparison();
				Object[] results2 = mc.replayALog(context, hierarchicalNet, log);
				
				if (results2 != null)
				{
					results = new Object[2];
					results[0] = results2[0];
					results[1] = results2[2];
				}
				
			}
		}
		
		
		if (results == null)
		{
			context.getFutureResult(0).cancel(true);
		}
		else
		{
			AnalysisReport report = new AnalysisReport(hierarchicalNet, log, results);
			report.ShowDialog(context);
			
			String orgLogName = XConceptExtension.instance().extractName(log);
			context.getFutureResult(1).setLabel(orgLogName + " Annotated");
			
			exportPSEN(context, (IPerformanceStatisticsEnrichedNet)results[0], "Performance Statistics Enriched Net");
			exportAnnotatedLog(context, (XLog)results[1], orgLogName + " Annotated");
			
			context.getFutureResult(0).cancel(true);
		}
		
		return results;
	}

	private XLog RemoveAlignmentInfo(LogAnalyser la, XLog log)
	{
		HashSet<String> gTraceAttrs = la.calcAvailableTraceAttrs();
		HashSet<String> gEventAttrs = la.calcAvailableEventAttrs();
		
		LPPRemoveTraceAttributes lrta = new LPPRemoveTraceAttributes();
		Vector<String> remAttrs = new Vector<String>();
		if (gTraceAttrs.contains(AlignedLog.ATT_NAME_IS_BROKEN))
		{
			remAttrs.add(AlignedLog.ATT_NAME_IS_BROKEN);
		}
		if (gTraceAttrs.contains(AlignedLog.ATT_NAME_IS_PERFECT_FIT))
		{
			remAttrs.add(AlignedLog.ATT_NAME_IS_PERFECT_FIT);
		}
		if (gTraceAttrs.contains(AlignedLog.ATT_NAME_IS_RELIABLE))
		{
			remAttrs.add(AlignedLog.ATT_NAME_IS_RELIABLE);
		}
		if (gTraceAttrs.contains(AlignedLog.ATT_NAME_REPLAY_GROUP_ID))
		{
			remAttrs.add(AlignedLog.ATT_NAME_REPLAY_GROUP_ID);
		}
		if (gTraceAttrs.contains("Calculation Time (ms)"))
		{
			remAttrs.add("Calculation Time (ms)");
		}
		if (gTraceAttrs.contains("Max Fitness Cost"))
		{
			remAttrs.add("Max Fitness Cost");
		}
		if (gTraceAttrs.contains("Max Move-Log Cost"))
		{
			remAttrs.add("Max Move-Log Cost");
		}
		if (gTraceAttrs.contains("Move-Log Fitness"))
		{
			remAttrs.add("Move-Log Fitness");
		}
		if (gTraceAttrs.contains("Move-Model Fitness"))
		{
			remAttrs.add("Move-Model Fitness");
		}
		if (gTraceAttrs.contains("Num. States"))
		{
			remAttrs.add("Num. States");
		}
		if (gTraceAttrs.contains("Queued States"))
		{
			remAttrs.add("Queued States");
		}
		if (gTraceAttrs.contains("Raw Fitness Cost"))
		{
			remAttrs.add("Raw Fitness Cost");
		}
		if (gTraceAttrs.contains("Trace Fitness"))
		{
			remAttrs.add("Trace Fitness");
		}
		if (gTraceAttrs.contains("Trace Length"))
		{
			remAttrs.add("Trace Length");
		}
		if (gTraceAttrs.contains("Traversed Arcs"))
		{
			remAttrs.add("Traversed Arcs");
		}
				
		log = lrta.execute(log, remAttrs);

		LPPRemoveEventAttributes lrea = new LPPRemoveEventAttributes();
		Vector<String> remAttrs2 = new Vector<String>();
		if (gEventAttrs.contains(AlignedLog.ATT_NAME_NODE_ID))
		{
			remAttrs2.add(AlignedLog.ATT_NAME_NODE_ID);
		}
		if (gEventAttrs.contains(AlignedLog.ATT_NAME_STEP_TYPE))
		{
			remAttrs2.add(AlignedLog.ATT_NAME_STEP_TYPE);
		}
		if (gEventAttrs.contains("emitId"))
		{
			remAttrs2.add("emitId");
		}
				
		log = lrea.execute(log, remAttrs2);
		return log;
	}
	
	private void exportPSEN(final UIPluginContext context, IPerformanceStatisticsEnrichedNet obj, String objName)
	{
		ProvidedObjectManager pom = context.getGlobalContext().getProvidedObjectManager();
		ProMResourceManager rm = context.getGlobalContext().getResourceManager();
		ProvidedObjectID poid = pom.createProvidedObject(objName, obj, IPerformanceStatisticsEnrichedNet.class, context);
		ProMResource<?> res = rm.getResourceForInstance(obj);
		
		//rm.addResource(poid, obj);
		res.setFavorite(true);
	}
	
	private void exportAnnotatedLog(final UIPluginContext context, XLog obj, String objName)
	{
		ProvidedObjectManager pom = context.getGlobalContext().getProvidedObjectManager();
		ProMResourceManager rm = context.getGlobalContext().getResourceManager();
		ProvidedObjectID poid = pom.createProvidedObject(objName, obj, XLog.class, context);
		ProMResource<?> res = rm.getResourceForInstance(obj);
		
		//rm.addResource(poid, obj);
		res.setFavorite(true);
	}
}
