package org.processmining.processprofiler3d.psen.plugins;


import java.util.Vector;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */
public class ModelCopyLocalIDs 
{
	@Plugin(
		name = "Process Profiler 3D: Copy LocalNodeIDs", 
		parameterLabels = {"Petri Net", "Petri Net"}, 
		returnLabels = { "Flattened Petri Net" }, 
		returnTypes = { Petrinet.class}, 
		userAccessible = true, 
		help = "This plug-in copies localNodeIDs of one process model to another one, matching nodes by label."
	)
	@UITopiaVariant(
		affiliation = "Queensland University of Technology", 
		author = "Erik Poppe", 
		email = "erik.poppe@qut.edu.au"
	)

	
	public Petrinet replayALog(final UIPluginContext context, Petrinet source, Petrinet target) 
	{
		PetriNetDataSource pndsSrc = new PetriNetDataSource(source);
		PetriNetDataSource pndsTgt = new PetriNetDataSource(target);

		Vector<Transition> allTransitionsSrc = getAllTransitions(pndsSrc);
		Vector<Transition> allTransitionsTgt = getAllTransitions(pndsTgt);
		
		//match transitions based on label of transition
		for (Transition tSrc : allTransitionsSrc)
		{
			for (Transition tTgt : allTransitionsTgt)
			{
				if (tSrc.getLabel().equals(tTgt.getLabel()))
				{
					//found match! copy localNodeID
					tTgt.setLocalID(tSrc.getLocalID());
					
					break;
				}
			}
		}

		Vector<Place> allPlacesSrc = getAllPlaces(pndsSrc);
		Vector<Place> allPlacesTgt = getAllPlaces(pndsTgt);
		
		//match places based on label of place
		for (Place pSrc : allPlacesSrc)
		{
			for (Place pTgt : allPlacesTgt)
			{
				if (pSrc.getLabel().equals(pTgt.getLabel()))
				{
					//found match! copy localNodeID
					pTgt.setLocalID(pSrc.getLocalID());
					
					break;
				}
			}
		}

		Vector<Arc> allEdgesSrc = getAllEdges(pndsSrc);
		Vector<Arc> allEdgesTgt = getAllEdges(pndsTgt);
		
		//match arcs based on LocalNodeID of source an target
		for (Arc aSrc : allEdgesSrc)
		{
			for (Arc aTgt : allEdgesTgt)
			{
				if (aSrc.getSource().getLocalID().equals(aTgt.getSource().getLocalID()) && aSrc.getTarget().getLocalID().equals(aTgt.getTarget().getLocalID()))
				{
					//found match! copy localNodeID
					aTgt.setLocalID(aSrc.getLocalID());
					
					break;
				}
			}
		}
		
		return null;
	}
	
	public Vector<Transition> getAllTransitions(INetDataSource inds)
	{
		Vector<Transition> lst = new Vector<Transition>();
		
		lst.addAll(inds.getTransitions());
		
		for (INetDataSource indsc : inds.getSubnets())
		{
			lst.addAll(getAllTransitions(indsc));
		}
		
		return lst;
	}
	
	public Vector<Place> getAllPlaces(INetDataSource inds)
	{
		Vector<Place> lst = new Vector<Place>();
		
		lst.addAll(inds.getPlaces());
		
		for (INetDataSource indsc : inds.getSubnets())
		{
			lst.addAll(getAllPlaces(indsc));
		}
		
		return lst;
	}
	
	public Vector<Arc> getAllEdges(INetDataSource inds)
	{
		Vector<Arc> lst = new Vector<Arc>();
		
		lst.addAll(inds.getEdges());
		
		for (INetDataSource indsc : inds.getSubnets())
		{
			lst.addAll(getAllEdges(indsc));
		}
		
		return lst;
	}
		
	
	
}
