package org.processmining.processprofiler3d.psen.plugins;


import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.psen.PetriNet1to1Mapping;
import org.processmining.processprofiler3d.psen.PetrinetGraphProjector;
import org.processmining.processprofiler3d.psen.connections.PetrinetFlattenConnection;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */
public class ModelFlatten 
{
	@Plugin(
		name = "Process Profiler 3D: Flatten Hierarchical Model", 
		parameterLabels = {"Petri Net"}, 
		returnLabels = { "Flattened Petri Net" }, 
		returnTypes = { Petrinet.class}, 
		userAccessible = true, 
		help = "This plug-in flattens a hierarchical process model"
	)
	@UITopiaVariant(
		affiliation = "Queensland University of Technology", 
		author = "Erik Poppe", 
		email = "erik.poppe@qut.edu.au"
	)

	
	public Petrinet replayALog(final UIPluginContext context, Petrinet net) 
	{	

		//System.out.println("Starting replay");
		//context.log("Starting replay...");
		
		PetrinetGraphProjector proj = new PetrinetGraphProjector();
		PetriNet1to1Mapping map = new PetriNet1to1Mapping();
		Petrinet pg = proj.projectPetrinet(net, map);
		
		PetrinetFlattenConnection pfc = new PetrinetFlattenConnection(net, pg, map);
		context.addConnection(pfc);

		//get initial markings
		Marking initMarking = null;
		
		try 
		{
			// get initial marking of the net
			InitialMarkingConnection initMarkingConn = context.getConnectionManager().getFirstConnection(InitialMarkingConnection.class, context, net);
			initMarking = initMarkingConn.getObjectWithRole(InitialMarkingConnection.MARKING);
		} 
		catch (Exception exc)
		{
			// it's okay not to have any initial marking
			initMarking = new Marking();
		}
		
		initMarking = PetrinetGraphProjector.projectMarking(initMarking, map);
		
		//register initial marking with object manager
		context.getProvidedObjectManager().createProvidedObject("Initial Marking", initMarking, Marking.class, context);
		
		context.addConnection(new InitialMarkingConnection(pg, initMarking));

		return pg;
	}
		
	
	
}
