package org.processmining.processprofiler3d.psen.plugins;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.models.connections.petrinets.EvClassLogPetrinetConnection;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.processprofiler3d.psen.PetriNet1to1Mapping;
import org.processmining.processprofiler3d.psen.PetrinetGraphProjector;
import org.processmining.processprofiler3d.psen.connections.PetrinetFlattenConnection;
import org.processmining.processprofiler3d.psen.gui.SlickerOptionPane;

public class ModelFlattenLayer 
{
	private Petrinet pgflat;
	private PetriNet1to1Mapping map;
	private Marking initMarking = null;
	private Marking finalMarking = null;
	
	public Petrinet getNet()
	{
		return pgflat;
	}
	
	public PetriNet1to1Mapping getMapping()
	{
		return map;
	}
	
	public Marking getInitMarking()
	{
		return initMarking;
	}
	
	public Marking getFinalMarking()
	{
		return finalMarking;
	}
	
	public ModelFlattenLayer(final UIPluginContext context, Petrinet hierarchicalNet, Marking iMarking, Marking fMarking)
	{	
		//get a flattened petri net from the hierarchical petri net
		PetrinetFlattenConnection pfc = getFlattenedNet(context, hierarchicalNet);
		Init(context, iMarking, fMarking, pfc);
	}
	
	public ModelFlattenLayer(final UIPluginContext context, Petrinet hierarchicalNet, Marking iMarking, Marking fMarking, XLog log)
	{	
		//get a flattened petri net from the hierarchical petri net
		PetrinetFlattenConnection pfc = getFlattenedNet(context, hierarchicalNet, log);
		Init(context, iMarking, fMarking, pfc);
	}

	private void Init(final UIPluginContext context, Marking iMarking, Marking fMarking, PetrinetFlattenConnection pfc)
	{
		pgflat = pfc.getFlattenedNet();
		map = pfc.getNetMapping();
		
		//obtain an initial marking for flattened petri net
		try
		{
			//check if initial marking for flattened net already exists
			InitialMarkingConnection initMarkingConn = context.getConnectionManager().getFirstConnection(InitialMarkingConnection.class, context, pgflat);
			initMarking = initMarkingConn.getObjectWithRole(InitialMarkingConnection.MARKING);
		}
		catch (Exception exc)
		{
			//otherwise create initial marking for flattened net by projecting initial marking from hierarchical net
			if (iMarking != null)
			{
				initMarking = PetrinetGraphProjector.projectMarking(iMarking, map);

				//register flattened net initial marking with object manager
				context.getProvidedObjectManager().createProvidedObject("Flattened Net Initial Marking", initMarking, Marking.class, context);			
				
				context.addConnection(new InitialMarkingConnection(pgflat, initMarking));
			}
		}
		
		//obtain an final marking for flattened petri net
		try
		{
			//check if final marking for flattened net already exists
			FinalMarkingConnection finalMarkingConn = context.getConnectionManager().getFirstConnection(FinalMarkingConnection.class, context, pgflat);
			finalMarking = finalMarkingConn.getObjectWithRole(FinalMarkingConnection.MARKING);
		}
		catch (Exception exc)
		{
			//otherwise create final marking for flattened net by projecting initial marking from hierarchical net
			if (fMarking != null)
			{
				finalMarking = PetrinetGraphProjector.projectMarking(fMarking, map);
				
				//register flattened net final marking with object manager
				context.getProvidedObjectManager().createProvidedObject("Flattened Net Final Marking", finalMarking, Marking.class, context);
				
				context.addConnection(new FinalMarkingConnection(pgflat, finalMarking));
			}
		}
		
		//register flattened net with object manager
		context.getProvidedObjectManager().createProvidedObject("Flattened Net", pgflat, Petrinet.class, context);
	}
	
	private PetrinetFlattenConnection getFlattenedNet(final UIPluginContext context, Petrinet hierarchicalNet)
	{
		Petrinet pgflat = null;
		try
		{
			//try to get a flattened petri net previously created by looking for existing connection
			PetrinetFlattenConnection pfc = context.getConnectionManager().getFirstConnection(PetrinetFlattenConnection.class, context, hierarchicalNet);
			pgflat = pfc.getObjectWithRole(PetrinetFlattenConnection.FLATTENED_NET);
			
			System.out.println("Found flatten connection.");
			context.log("Found flatten connectiond.");
			
			return pfc;
		}
		catch (Exception exc)
		{
			//otherwise flatten hierarchical petri net
			PetrinetGraphProjector proj = new PetrinetGraphProjector();
			PetriNet1to1Mapping map = new PetriNet1to1Mapping();
			pgflat = proj.projectPetrinet(hierarchicalNet, map);
	
			PetrinetFlattenConnection pfc = new PetrinetFlattenConnection(hierarchicalNet, pgflat, map);
			context.addConnection(pfc);
			
			System.out.println("Created flattened petri net.");
			context.log("Created flattened petri net.");
			
			return pfc;
		}
	}
	
	private PetrinetFlattenConnection getFlattenedNet(final UIPluginContext context, Petrinet hierarchicalNet, XLog log)
	{
		Petrinet pgflat = null;
		try
		{
			//try to get a flattened petri net previously created by looking for existing connection
			PetrinetFlattenConnection pfc = context.getConnectionManager().getFirstConnection(PetrinetFlattenConnection.class, context, hierarchicalNet);
			pgflat = pfc.getObjectWithRole(PetrinetFlattenConnection.FLATTENED_NET);
			
			System.out.println("Found flatten connection.");
			context.log("Found flatten connectiond.");
			
			return pfc;
		}
		catch (Exception exc)
		{
			TransEvClassMapping xmap = getTransEvClassMapping(context, hierarchicalNet, log);
			
			//otherwise flatten hierarchical petri net
			PetrinetGraphProjector proj = new PetrinetGraphProjector();
			PetriNet1to1Mapping map = new PetriNet1to1Mapping();
			pgflat = proj.projectPetrinet(hierarchicalNet, map, log, xmap);
	
			PetrinetFlattenConnection pfc = new PetrinetFlattenConnection(hierarchicalNet, pgflat, map);
			context.addConnection(pfc);
			
			EvClassLogPetrinetConnection eclpc = new EvClassLogPetrinetConnection("evc", pgflat, log, proj.xxmap.getEventClassifier(), proj.xxmap);
			context.addConnection(eclpc);
			
			System.out.println("Created flattened petri net.");
			context.log("Created flattened petri net.");
			
			return pfc;
		}
	}

	private TransEvClassMapping getTransEvClassMapping(final UIPluginContext context, Petrinet net, XLog log)
	{
		EvClassLogPetrinetConnection conn = null;
		// check connection in order to determine whether mapping step is needed or not
		try 
		{
			// connection is found, no need for mapping step
			// connection is not found, another plugin to create such connection
			// is automatically executed
			conn = context.getConnectionManager().getFirstConnection(EvClassLogPetrinetConnection.class, context, net, log);
		} 
		catch (Exception e) 
		{
			//SlickerOptionPane.showMessageDialog(new JPanel(), "No mapping can be constructed between the net and the log!");
			SlickerOptionPane.showErrorMessage(context, "No mapping can be constructed between the net and the log!", "Error");
			return null;
		}

		// init gui for each step
		TransEvClassMapping mapping = (TransEvClassMapping)conn.getObjectWithRole(EvClassLogPetrinetConnection.TRANS2EVCLASSMAPPING);
		
		return mapping;
	}
}
