package org.processmining.processprofiler3d.psen.plugins;


import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.psen.PetriNet1toManyMapping;
import org.processmining.processprofiler3d.psen.PetrinetGraphProjector2;
import org.processmining.processprofiler3d.psen.connections.PetrinetLifecycleExpansionConnection;


/*
 * This is the entry class of the comparative visualisation plug-in.
 * */
public class ModelLifecycleExpand 
{
	@Plugin(
		name = "Process Profiler 3D: Expand Model with Event Lifecycle", 
		parameterLabels = {"Petri Net"}, 
		returnLabels = { "Expanded Petri Net" }, 
		returnTypes = { Petrinet.class}, 
		userAccessible = true, 
		help = "This plug-in expands a process model for event lifecycle data"
	)
	@UITopiaVariant(
		affiliation = "Queensland University of Technology", 
		author = "Erik Poppe", 
		email = "erik.poppe@qut.edu.au"
	)

	
	public Petrinet expandLog(final UIPluginContext context, Petrinet net) 
	{	
		//System.out.println("Starting replay");
		//context.log("Starting replay...");
		
		//Map<PetrinetNode, PetrinetNode> mapOrig2ViewNode = new HashMap<PetrinetNode, PetrinetNode>();
		//Map<PetrinetNode, PetrinetNode> mapView2OrigNode = new HashMap<PetrinetNode, PetrinetNode>();
		PetriNet1toManyMapping map = new PetriNet1toManyMapping();
		PetrinetGraphProjector2 proj = new PetrinetGraphProjector2();
		Petrinet pg = proj.projectPetrinet2(net, map, null, null);

		PetrinetLifecycleExpansionConnection plec = new PetrinetLifecycleExpansionConnection(net, pg, map);
		context.addConnection(plec);
		
		//get initial markings
		Marking initMarking = null;
		
		try 
		{
			// get initial marking of the net
			InitialMarkingConnection initMarkingConn = context.getConnectionManager().getFirstConnection(InitialMarkingConnection.class, context, net);
			initMarking = initMarkingConn.getObjectWithRole(InitialMarkingConnection.MARKING);
		} 
		catch (Exception exc)
		{
			// it's okay not to have any initial marking
			initMarking = new Marking();
		}
		
		initMarking = PetrinetGraphProjector2.projectMarking(initMarking, map);

		//register initial marking with object manager
		context.getProvidedObjectManager().createProvidedObject("Initial Marking", initMarking, Marking.class, context);
		
		
		context.addConnection(new InitialMarkingConnection(pg, initMarking));

		return pg;
	}
		
	
	
}
