package org.processmining.processprofiler3d.psen.plugins;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.models.connections.petrinets.EvClassLogPetrinetConnection;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.processprofiler3d.psen.PetriNet1toManyMapping;
import org.processmining.processprofiler3d.psen.PetrinetGraphProjector2;
import org.processmining.processprofiler3d.psen.connections.PetrinetLifecycleExpansionConnection;
import org.processmining.processprofiler3d.psen.gui.SlickerOptionPane;

public class ModelLifecycleExpansionLayer 
{
	private Petrinet pgexp = null;
	private PetriNet1toManyMapping map2 = null;
	private Marking myInitMarking = null;
	private Marking myFinalMarking = null;
	
	public Petrinet getNet()
	{
		return pgexp;
	}
	
	public PetriNet1toManyMapping getMapping()
	{
		return map2;
	}
	
	public Marking getInitMarking()
	{
		return myInitMarking;
	}
	
	public Marking getFinalMarking()
	{
		return myFinalMarking;
	}
	
	public ModelLifecycleExpansionLayer(final UIPluginContext context, Petrinet pgflat, Marking iMarking, Marking fMarking, XLog log)
	{		
		PetrinetLifecycleExpansionConnection plec = getExpandedNet(context, pgflat, log);
		pgexp = plec.getExpandedNet();
		map2 = plec.getNetMapping();
		
		//get initial marking for lifecycle expanded petri net
		try
		{
			//check if there is an existing initial marking by checking connections
			InitialMarkingConnection initMarkingConn = context.getConnectionManager().getFirstConnection(InitialMarkingConnection.class, context, pgexp);
			myInitMarking = initMarkingConn.getObjectWithRole(InitialMarkingConnection.MARKING);	
		}
		catch (Exception exc)
		{	
			//otherwise create initial marking for expanded net by projecting initial marking of flattened net
			if (iMarking != null)
			{
				myInitMarking = PetrinetGraphProjector2.projectMarking(iMarking, map2);

				//register expanded net initial marking with object manager
				context.getProvidedObjectManager().createProvidedObject("Expanded Net Initial Marking", myInitMarking, Marking.class, context);
				
				context.addConnection(new InitialMarkingConnection(pgexp, myInitMarking));
			}
		}
		
		//get final marking for lifecycle expanded petri net
		try
		{
			//check if there is an existing final marking by checking connections
			FinalMarkingConnection finalMarkingConn = context.getConnectionManager().getFirstConnection(FinalMarkingConnection.class, context, pgexp);
			myFinalMarking = finalMarkingConn.getObjectWithRole(FinalMarkingConnection.MARKING);	
		}
		catch (Exception exc)
		{	
			//otherwise create initial marking for expanded net by projecting initial marking of flattened net
			if (fMarking != null)
			{
				myFinalMarking = PetrinetGraphProjector2.projectMarking(fMarking, map2);

				//register expanded net final marking with object manager
				context.getProvidedObjectManager().createProvidedObject("Expanded Net Final Marking", myFinalMarking, Marking.class, context);
				
				context.addConnection(new FinalMarkingConnection(pgexp, myFinalMarking));
			}
		}
		
		//register expanded net with object manager
		context.getProvidedObjectManager().createProvidedObject("Expanded Net", pgexp, Petrinet.class, context);
	}
	
	public ModelLifecycleExpansionLayer(final UIPluginContext context, Petrinet pgflat, Marking iMarking, Marking fMarking, XLog log, boolean bNoTransEvMapping)
	{		
		PetrinetLifecycleExpansionConnection plec = getExpandedNet(context, pgflat, log, bNoTransEvMapping);
		pgexp = plec.getExpandedNet();
		map2 = plec.getNetMapping();
		
		//get initial marking for lifecycle expanded petri net
		try
		{
			//check if there is an existing initial marking by checking connections
			InitialMarkingConnection initMarkingConn = context.getConnectionManager().getFirstConnection(InitialMarkingConnection.class, context, pgexp);
			myInitMarking = initMarkingConn.getObjectWithRole(InitialMarkingConnection.MARKING);	
		}
		catch (Exception exc)
		{	
			//otherwise create initial marking for expanded net by projecting initial marking of flattened net
			if (iMarking != null)
			{
				myInitMarking = PetrinetGraphProjector2.projectMarking(iMarking, map2);

				//register expanded net initial with object manager
				context.getProvidedObjectManager().createProvidedObject("Expanded Net Initial Marking", myInitMarking, Marking.class, context);
				
				//add new connection
				context.addConnection(new InitialMarkingConnection(pgexp, myInitMarking));
			}
		}
		
		//get final marking for lifecycle expanded petri net
		try
		{
			//check if there is an existing final marking by checking connections
			FinalMarkingConnection finalMarkingConn = context.getConnectionManager().getFirstConnection(FinalMarkingConnection.class, context, pgexp);
			myFinalMarking = finalMarkingConn.getObjectWithRole(FinalMarkingConnection.MARKING);	
		}
		catch (Exception exc)
		{	
			//otherwise create initial marking for expanded net by projecting initial marking of flattened net
			if (fMarking != null)
			{
				myFinalMarking = PetrinetGraphProjector2.projectMarking(fMarking, map2);

				//register expanded net final marking with object manager
				context.getProvidedObjectManager().createProvidedObject("Expanded Net Final Marking", myFinalMarking, Marking.class, context);
				
				
				context.addConnection(new FinalMarkingConnection(pgexp, myFinalMarking));
			}
		}
		
		//register expanded net with object manager
		context.getProvidedObjectManager().createProvidedObject("Expanded Net", pgexp, Petrinet.class, context);
	}
	
	private PetrinetLifecycleExpansionConnection getExpandedNet(final UIPluginContext context, Petrinet pgflat, XLog log, boolean bNoTransEvMapping)
	{
		Petrinet pgexp = null;
		PetriNet1toManyMapping map2 = null;
		try
		{
			PetrinetLifecycleExpansionConnection plec = context.getConnectionManager().getFirstConnection(PetrinetLifecycleExpansionConnection.class, context, pgflat);
			pgexp = plec.getObjectWithRole(PetrinetLifecycleExpansionConnection.EXPANDED_NET);
			map2 = plec.getObjectWithRole(PetrinetLifecycleExpansionConnection.MAPPING);
			
			System.out.println("Found lifecycle expansion connection.");
			context.log("Found lifecycle expansion connectiond.");
			
			return plec;
		}
		catch (Exception exc)
		{	
			//expand petri net for lifecycle events
			//TransEvClassMapping xmap = getTransEvClassMapping(context, pgflat, log);

			PetrinetGraphProjector2 proj = new PetrinetGraphProjector2();
			map2 = new PetriNet1toManyMapping();
			pgexp = proj.projectPetrinet2(pgflat, map2, log);
	
			//EvClassLogPetrinetConnection eclpc = new EvClassLogPetrinetConnection("evc", pgexp, log, proj.xxmap.getEventClassifier(), proj.xxmap);
			PetrinetLifecycleExpansionConnection plec = new PetrinetLifecycleExpansionConnection(pgflat, pgexp, map2);
			//context.addConnection(eclpc);
			context.addConnection(plec);
			
			System.out.println("Created lifecycle expanded petri net.");
			context.log("Created lifecycle expanded petri net.");
			
			return plec;
		}	
	}
	
	private PetrinetLifecycleExpansionConnection getExpandedNet(final UIPluginContext context, Petrinet pgflat, XLog log)
	{
		Petrinet pgexp = null;
		PetriNet1toManyMapping map2 = null;
		try
		{
			PetrinetLifecycleExpansionConnection plec = context.getConnectionManager().getFirstConnection(PetrinetLifecycleExpansionConnection.class, context, pgflat);
			pgexp = plec.getObjectWithRole(PetrinetLifecycleExpansionConnection.EXPANDED_NET);
			map2 = plec.getObjectWithRole(PetrinetLifecycleExpansionConnection.MAPPING);
			
			System.out.println("Found lifecycle expansion connection.");
			context.log("Found lifecycle expansion connectiond.");
			
			return plec;
		}
		catch (Exception exc)
		{	
			//expand petri net for lifecycle events
			TransEvClassMapping xmap = getTransEvClassMapping(context, pgflat, log);

			PetrinetGraphProjector2 proj = new PetrinetGraphProjector2();
			map2 = new PetriNet1toManyMapping();
			pgexp = proj.projectPetrinet2(pgflat, map2, log, xmap);

			EvClassLogPetrinetConnection eclpc = new EvClassLogPetrinetConnection("evc", pgexp, log, proj.xxmap.getEventClassifier(), proj.xxmap);
			PetrinetLifecycleExpansionConnection plec = new PetrinetLifecycleExpansionConnection(pgflat, pgexp, map2);
			context.addConnection(eclpc);
			context.addConnection(plec);
			
			System.out.println("Created lifecycle expanded petri net.");
			context.log("Created lifecycle expanded petri net.");
			
			return plec;
		}	
	}
	
	private TransEvClassMapping getTransEvClassMapping(final UIPluginContext context, Petrinet net, XLog log)
	{
		EvClassLogPetrinetConnection conn = null;
		// check connection in order to determine whether mapping step is needed or not
		try 
		{
			// connection is found, no need for mapping step
			// connection is not found, another plugin to create such connection
			// is automatically executed
			conn = context.getConnectionManager().getFirstConnection(EvClassLogPetrinetConnection.class, context, net, log);
		} 
		catch (Exception e) 
		{
			//SlickerOptionPane.showMessageDialog(new JPanel(), "No mapping can be constructed between the net and the log");
			SlickerOptionPane.showErrorMessage(context, "No mapping can be constructed between the net and the log!", "Error");
			return null;
		}

		// init gui for each step
		TransEvClassMapping mapping = (TransEvClassMapping)conn.getObjectWithRole(EvClassLogPetrinetConnection.TRANS2EVCLASSMAPPING);
		
		return mapping;
	}

}
