package org.processmining.processprofiler3d.psen.plugins;

import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.processprofiler3d.compviz.INetDataSource;

public class ModelStructureChecker
{
	private boolean bStructureComplies = true;
	public Vector<String> problemList = new Vector<String>();
	
	public ModelStructureChecker(INetDataSource esn)
	{
		bStructureComplies = CheckSubnetStructure(esn);
	}
	
	public boolean isStructureCompliant()
	{
		return bStructureComplies;
	}
	
	private boolean CheckSubnetStructure(INetDataSource esn)
	{
		boolean bCompliant = true;
		if (CheckSubnetStructureHelper(esn))
		{
			for (INetDataSource inds : esn.getSubnets())
			{
				if (!CheckSubnetStructure(inds))
				{
					bCompliant = false;
				}
			}
		}
		else
		{
			bCompliant = false;
		}
		return bCompliant;
	}
	
	private boolean CheckSubnetStructureHelper(INetDataSource esn)
	{
		String netName = "Subnet";
		if (esn.getParent() == null)
		{
			netName = "Root net";
		}
		else
		{
			netName = "Subnet '" + esn.getParentTrans().getLabel() + "'";
		}
		
		//check if subnet has single entry and single exit (transitions)!
		int startPlacesNum = 0;
		for (Place p : esn.getPlaces())
		{
			if (esn.getInEdges(p).isEmpty())
			{
				startPlacesNum++;
			}
		}
		
		int endPlacesNum = 0;
		for (Place p : esn.getPlaces())
		{
			if (esn.getOutEdges(p).isEmpty())
			{
				endPlacesNum++;
			}
		}
		
		if (startPlacesNum > 1 || endPlacesNum > 1)
		{	
			if (startPlacesNum > 1)
			{
				problemList.add(netName + " has " + startPlacesNum + " start places.");
			}
			if (endPlacesNum > 1)
			{
				problemList.add(netName + " has " + endPlacesNum + " end places.");
			}

			return false;
		}
		
		Place ps = esn.getStartPlace();
		Place pe = esn.getEndPlace();
		
		int startTransitionsNum = esn.getOutEdges(ps).size();
		int endTransitionsNum = esn.getInEdges(pe).size();
		
		if (startTransitionsNum > 1 || endTransitionsNum > 1)
		{	
			
			if (startTransitionsNum > 1)
			{
				problemList.add(netName + " has " + startTransitionsNum + " transitions following the start place.");
			}
			if (endTransitionsNum > 1)
			{
				problemList.add(netName + " has " + endTransitionsNum + " transitions preceeding the end place.");
			}
			
			return false;
		}	
		return true;
	}

}
