package org.processmining.processprofiler3d.psen.plugins;

import javax.swing.JOptionPane;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.psen.gui.MarkingWarningDialog;

public class PP3DPluginBase
{
	public PP3DPluginBase()
	{
	}

	protected GraphLayoutConnection getGraphLayoutFor(final UIPluginContext context, Petrinet pnet) 
	{
		GraphLayoutConnection layoutConn = null;
		try 
		{
			layoutConn = context.getConnectionManager().getFirstConnection(GraphLayoutConnection.class, context, pnet);
		} 
		catch (ConnectionCannotBeObtained exc)
		{
			// do nothing
			System.out.println("ThreeDeeModelViz: cannot find graph layout!");
		}
		catch (Exception ex)
		{
			// do nothing
			System.out.println("ThreeDeePSENViz: issue with graph layout!");
		}
		return layoutConn;
	}
    
	protected Marking getInitialMarking(final UIPluginContext context, PetrinetGraph pnet) 
	{
		Marking initMarking;
		try 
		{
			// get initial marking of the net
			InitialMarkingConnection initMarkingConn = context.getConnectionManager().getFirstConnection(InitialMarkingConnection.class, context, pnet);
			initMarking = initMarkingConn.getObjectWithRole(InitialMarkingConnection.MARKING);
		} 
		catch (Exception exc)
		{
			//create automatic initial marking to suggest to user
			initMarking = new Marking();
			
			PetriNetDataSource pnds = new PetriNetDataSource(pnet);
			Place startPlace = pnds.getStartPlace();
			initMarking.add(startPlace, 1);
			
			MarkingWarningDialog mwd = new MarkingWarningDialog("Would you like to automatically create a marking? " + initMarking.toString(), "Initial Marking for petrinet is empty!");
			int n = mwd.ShowDialog(context);
			
			if (n == JOptionPane.YES_OPTION)
			{
				System.out.println("Created initial marking: " + initMarking.toString());
				
				InitialMarkingConnection imc = new InitialMarkingConnection(pnet, initMarking);

				//register initial marking with object manager
				context.getProvidedObjectManager().createProvidedObject("Initial Marking", initMarking, Marking.class, context);
				
				context.addConnection(imc);
			}
			else
			{
				return null;
			}
		}
		return initMarking;
	}

	protected Marking getFinalMarking(final UIPluginContext context, Petrinet hierarchicalNet) 
	{
		Marking finalMarking;
		try 
		{
			// get final marking of the net
			FinalMarkingConnection finalMarkingConn = context.getConnectionManager().getFirstConnection(FinalMarkingConnection.class, context, hierarchicalNet);
			finalMarking = finalMarkingConn.getObjectWithRole(FinalMarkingConnection.MARKING);
			
			System.out.println("Found final marking: " + finalMarking.toString());
		} 
		catch (Exception exc)
		{
			//create automatic final marking to suggest to user
			finalMarking = new Marking();
			
			PetriNetDataSource pnds = new PetriNetDataSource(hierarchicalNet);
			Place endPlace = pnds.getEndPlace();
			finalMarking.add(endPlace, 1);
			
			MarkingWarningDialog mwd = new MarkingWarningDialog("Would you like to automatically create a marking? " + finalMarking.toString(), "Final Marking for petrinet is empty!");
			int n = mwd.ShowDialog(context);
			
			if (n == JOptionPane.YES_OPTION)
			{		
				System.out.println("Created final marking: " + finalMarking.toString());
				
				FinalMarkingConnection imc = new FinalMarkingConnection(hierarchicalNet, finalMarking);
				
				//register final marking with object manager
				context.getProvidedObjectManager().createProvidedObject("Final Marking", finalMarking, Marking.class, context);
				
				context.addConnection(imc);
			}
			else
			{
				return null;
			}
		}
		return finalMarking;
	}

}
