package org.processmining.processprofiler3d.psen.preprocess;

import java.util.Date;
import java.util.Vector;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeLiteralImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.log.utils.XLogBuilder;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;

public class LPPAction 
{	
	public XLog execute(final UIPluginContext context, XLog log)
	{
		return log;
	}

	public static void CopyExtensions(XLog source, XLog target)
	{
		for (XExtension ext : source.getExtensions())
		{
			target.getExtensions().add(ext);
		}
	}

	public static void CopyGlobalEventAttributes(XLog source, XLog target)
	{
		for (XAttribute gatt : source.getGlobalEventAttributes())
		{
			target.getGlobalEventAttributes().add(gatt);
		}
	}

	public static void CopyGlobalTraceAttributes(XLog source, XLog target)
	{
		for (XAttribute gatt : source.getGlobalTraceAttributes())
		{
			target.getGlobalTraceAttributes().add(gatt);
		}
	}

	public static void CopyClassifiers(XLog source, XLog target)
	{
		for (XEventClassifier xec : source.getClassifiers())
		{
			target.getClassifiers().add(xec);
		}
	}
	
	protected void RemoveEmptyTraces(XLog log)
	{
		for (int i = 0; i < log.size(); i++)
		{
			if (log.get(i).isEmpty())
			{
				log.remove(i);
				i--;
			}
		}
	}
	
	public static void AddGenericAttribute(XLogBuilder newLog, String attrName, Object attrValue)
	{
		if (attrValue instanceof String)
		{
			newLog.addAttribute(attrName, (String)attrValue);
		}
		else if (attrValue instanceof Date)
		{
			newLog.addAttribute(attrName, (Date)attrValue);
		}
		else if (attrValue instanceof Long)
		{
			newLog.addAttribute(attrName, (Long)attrValue);
		}
		else if (attrValue instanceof Double)
		{
			newLog.addAttribute(attrName, (Double)attrValue);
		}
		else if (attrValue instanceof Boolean)
		{
			newLog.addAttribute(attrName, (boolean)attrValue);
		}
	}
	
	public static Object getAttributeValue(XEvent evt, String name)
	{
		XAttribute xAtt = evt.getAttributes().get(name);
		if (xAtt instanceof XAttributeLiteral)
		{
			XAttributeLiteral xAtt3 = (XAttributeLiteral)xAtt;
			if (xAtt3 != null && xAtt3.getValue() != null)
			{
				String eventName = xAtt3.getValue();
				return eventName;
			}
		}
		else if (xAtt instanceof XAttributeTimestamp)
		{
			XAttributeTimestamp xAtt3 = (XAttributeTimestamp)xAtt;
			if (xAtt3 != null && xAtt3.getValue() != null)
			{
				Date eventName = xAtt3.getValue();
				return eventName;
			}
		}
		else if (xAtt instanceof XAttributeBoolean)
		{
			XAttributeBoolean xAtt3 = (XAttributeBoolean)xAtt;
			if (xAtt3 != null)
			{
				Boolean eventName = xAtt3.getValue();
				return eventName;
			}
		}
		else if (xAtt instanceof XAttributeDiscrete)
		{
			XAttributeDiscrete xAtt3 = (XAttributeDiscrete)xAtt;
			if (xAtt3 != null)
			{
				Long eventName = xAtt3.getValue();
				return eventName;
			}
		}
		else if (xAtt instanceof XAttributeContinuous)
		{
			XAttributeContinuous xAtt3 = (XAttributeContinuous)xAtt;
			if (xAtt3 != null)
			{
				Double eventName = xAtt3.getValue();
				return eventName;
			}
		}
		return null;
	}
	
	public static String getAttributeValueLiteral(XEvent evt, String name)
	{
		XAttributeLiteral xAtt3 = (XAttributeLiteral)evt.getAttributes().get(name);
		if (xAtt3 != null && xAtt3.getValue() != null)
		{
			String eventName = xAtt3.getValue();
			return eventName;
		}
		return null;
	}
	
	public static Date getAttributeValueTimestamp(XEvent evt, String name)
	{
		XAttributeTimestamp xAtt3 = (XAttributeTimestamp)evt.getAttributes().get(name);
		if (xAtt3 != null && xAtt3.getValue() != null)
		{
			Date eventName = xAtt3.getValue();
			return eventName;
		}
		return null;
	}
	
	public static Boolean getAttributeValueBoolean(XEvent evt, String name)
	{
		XAttributeBoolean xAtt3 = (XAttributeBoolean)evt.getAttributes().get(name);
		if (xAtt3 != null)
		{
			Boolean eventName = xAtt3.getValue();
			return eventName;
		}
		return null;
	}
	
	public static Long getAttributeValueDiscrete(XEvent evt, String name)
	{
		XAttributeDiscrete xAtt3 = (XAttributeDiscrete)evt.getAttributes().get(name);
		if (xAtt3 != null)
		{
			Long eventName = xAtt3.getValue();
			return eventName;
		}
		return null;
	}
	
	public static Double getAttributeValueContinuous(XEvent evt, String name)
	{
		XAttributeContinuous xAtt3 = (XAttributeContinuous)evt.getAttributes().get(name);
		if (xAtt3 != null)
		{
			Double eventName = xAtt3.getValue();
			return eventName;
		}
		return null;
	}
	
	public static Object getAttributeValue(XTrace trc, String name)
	{
		XAttribute xAtt = trc.getAttributes().get(name);
		if (xAtt instanceof XAttributeLiteral)
		{
			XAttributeLiteral xAtt3 = (XAttributeLiteral)xAtt;
			if (xAtt3 != null && xAtt3.getValue() != null)
			{
				String eventName = xAtt3.getValue();
				return eventName;
			}
		}
		else if (xAtt instanceof XAttributeTimestamp)
		{
			XAttributeTimestamp xAtt3 = (XAttributeTimestamp)xAtt;
			if (xAtt3 != null && xAtt3.getValue() != null)
			{
				Date eventName = xAtt3.getValue();
				return eventName;
			}
		}
		else if (xAtt instanceof XAttributeBoolean)
		{
			XAttributeBoolean xAtt3 = (XAttributeBoolean)xAtt;
			if (xAtt3 != null)
			{
				Boolean eventName = xAtt3.getValue();
				return eventName;
			}
		}
		else if (xAtt instanceof XAttributeDiscrete)
		{
			XAttributeDiscrete xAtt3 = (XAttributeDiscrete)xAtt;
			if (xAtt3 != null)
			{
				Long eventName = xAtt3.getValue();
				return eventName;
			}
		}
		else if (xAtt instanceof XAttributeContinuous)
		{
			XAttributeContinuous xAtt3 = (XAttributeContinuous)xAtt;
			if (xAtt3 != null)
			{
				Double eventName = xAtt3.getValue();
				return eventName;
			}
		}
		return null;
	}
	
	public static String getAttributeValueLiteral(XTrace trc, String name)
	{
		XAttributeLiteral xAtt3 = (XAttributeLiteral)trc.getAttributes().get(name);
		if (xAtt3 != null && xAtt3.getValue() != null)
		{
			String eventName = xAtt3.getValue();
			return eventName;
		}
		return null;
	}
	
	public static Date getAttributeValueTimestamp(XTrace trc, String name)
	{
		XAttributeTimestamp xAtt3 = (XAttributeTimestamp)trc.getAttributes().get(name);
		if (xAtt3 != null && xAtt3.getValue() != null)
		{
			Date eventName = xAtt3.getValue();
			return eventName;
		}
		return null;
	}
	
	public static Boolean getAttributeValueBoolean(XTrace trc, String name)
	{
		XAttributeBoolean xAtt3 = (XAttributeBoolean)trc.getAttributes().get(name);
		if (xAtt3 != null)
		{
			Boolean eventName = xAtt3.getValue();
			return eventName;
		}
		return null;
	}
	
	public static Long getAttributeValueDiscrete(XTrace trc, String name)
	{
		XAttributeDiscrete xAtt3 = (XAttributeDiscrete)trc.getAttributes().get(name);
		if (xAtt3 != null)
		{
			Long eventName = xAtt3.getValue();
			return eventName;
		}
		return null;
	}
	
	public static Double getAttributeValueContinuous(XTrace trc, String name)
	{
		XAttributeContinuous xAtt3 = (XAttributeContinuous)trc.getAttributes().get(name);
		if (xAtt3 != null)
		{
			Double eventName = xAtt3.getValue();
			return eventName;
		}
		return null;
	}
	
	public static String getAttributeValueLiteral(XLog log, String name)
	{
		XAttributeLiteral xAtt3 = (XAttributeLiteral)log.getAttributes().get(name);
		if (xAtt3 != null && xAtt3.getValue() != null)
		{
			String eventName = xAtt3.getValue();
			return eventName;
		}
		return null;
	}
	
	protected static Vector<String> getAvailableLiteralValuesForTraces(XLog log, String attrName)
	{
		Vector<String> vals = new Vector<String>();
		
		//for each event in trace
		for (XTrace trace : log)
		{
			XAttribute xAtt = trace.getAttributes().get(attrName);
			if (xAtt instanceof XAttributeLiteral)
			{
				String attrValue = getAttributeValueLiteral(trace, attrName);
				if (attrValue != null)
				{
					if (!vals.contains(attrValue))
					{
						vals.add(attrValue);
					}
				}
			}
		}
		return vals;
	}

	protected static Vector<Object> getAvailableValuesForEvents(XTrace trace, String attrName)
	{
		Vector<Object> vals = new Vector<Object>();
		
		//for each event in trace
		for (XEvent event : trace)
		{
			XAttribute xAtt = event.getAttributes().get(attrName);
			if (xAtt instanceof XAttributeLiteral)
			{
				String attrValue = getAttributeValueLiteral(event, attrName);
				if (attrValue != null)
				{
					if (!vals.contains(attrValue))
					{
						vals.add(attrValue);
					}
				}
			}
			else if (xAtt instanceof XAttributeTimestamp)
			{
				Date attrValue = getAttributeValueTimestamp(event, attrName);
				if (attrValue != null)
				{
					if (!vals.contains(attrValue))
					{
						vals.add(attrValue);
					}
				}
			}
			else if (xAtt instanceof XAttributeBoolean)
			{
				Boolean attrValue = getAttributeValueBoolean(event, attrName);
				if (attrValue != null)
				{
					if (!vals.contains(attrValue))
					{
						vals.add(attrValue);
					}
				}
			}
			else if (xAtt instanceof XAttributeDiscrete)
			{
				Long attrValue = getAttributeValueDiscrete(event, attrName);
				if (attrValue != null)
				{
					if (!vals.contains(attrValue))
					{
						vals.add(attrValue);
					}
				}
			}
			else if (xAtt instanceof XAttributeContinuous)
			{
				Double attrValue = getAttributeValueContinuous(event, attrName);
				if (attrValue != null)
				{
					if (!vals.contains(attrValue))
					{
						vals.add(attrValue);
					}
				}
			}
		}
		return vals;
	}
	
	protected static Vector<String> getAvailableLiteralValuesForEvents(XTrace trace, String attrName)
	{
		Vector<String> vals = new Vector<String>();
		
		//for each event in trace
		for (XEvent event : trace)
		{
			XAttribute xAtt = event.getAttributes().get(attrName);
			if (xAtt instanceof XAttributeLiteral)
			{
				String attrValue = getAttributeValueLiteral(event, attrName);
				if (attrValue != null)
				{
					if (!vals.contains(attrValue))
					{
						vals.add(attrValue);
					}
				}
			}
		}
		return vals;
	}
	
	protected static Vector<Date> getAvailableTimestampValuesForEvents(XTrace trace, String attrName)
	{
		Vector<Date> vals = new Vector<Date>();
		
		//for each event in trace
		for (XEvent event : trace)
		{
			XAttribute xAtt = event.getAttributes().get(attrName);
			if (xAtt instanceof XAttributeTimestamp)
			{
				Date attrValue = getAttributeValueTimestamp(event, attrName);
				if (attrValue != null)
				{
					if (!vals.contains(attrValue))
					{
						vals.add(attrValue);
					}
				}
			}
		}
		return vals;
	}
	
	protected static Date getEarliestTimestampValue(XTrace trace, String attrName)
	{
		long min = Long.MAX_VALUE;
		
		//for each event in trace
		for (XEvent event : trace)
		{
			XAttribute xAtt = event.getAttributes().get(attrName);
			if (xAtt instanceof XAttributeTimestamp)
			{
				Date attrValue = getAttributeValueTimestamp(event, attrName);
				if (attrValue != null)
				{
					min = Math.min(min, attrValue.getTime());
				}
			}
		}
		return new Date(min);
	}
	
	protected static Date getLatestTimestampValue(XTrace trace, String attrName)
	{
		long max = Long.MIN_VALUE;
		
		//for each event in trace
		for (XEvent event : trace)
		{
			XAttribute xAtt = event.getAttributes().get(attrName);
			if (xAtt instanceof XAttributeTimestamp)
			{
				Date attrValue = getAttributeValueTimestamp(event, attrName);
				if (attrValue != null)
				{
					max = Math.max(max, attrValue.getTime());
				}
			}
		}
		return new Date(max);
	}
	
	protected static TimeSpan getDuration(XTrace trace, String attrName)
	{
		long min = Long.MAX_VALUE;
		long max = Long.MIN_VALUE;
		
		//for each event in trace
		for (XEvent event : trace)
		{
			XAttribute xAtt = event.getAttributes().get(attrName);
			if (xAtt instanceof XAttributeTimestamp)
			{
				Date attrValue = getAttributeValueTimestamp(event, attrName);
				if (attrValue != null)
				{
					min = Math.min(min, attrValue.getTime());
					max = Math.max(max, attrValue.getTime());
				}
			}
		}
		return new TimeSpan(new Date(min), new Date(max));
	}
	
	protected static void setAttributeValueLiteral(XEvent evt, String name, String value)
	{
		evt.getAttributes().put(name, new XAttributeLiteralImpl(name, value));
	}
	
	protected static void setAttributeValueLiteral(XTrace trc, String name, String value)
	{
		trc.getAttributes().put(name, new XAttributeLiteralImpl(name, value));
	}
	
	protected static void setAttributeValueLiteral(XLog log, String name, String value)
	{
		log.getAttributes().put(name, new XAttributeLiteralImpl(name, value));
	}

}
