package org.processmining.processprofiler3d.psen.preprocess;

import java.util.Vector;

import javax.swing.JOptionPane;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeLiteralImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.log.utils.XLogBuilder;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.classification.CohortClassifier;

public class LPPAddCaseCohortAttribute extends LPPAction
{
	private Vector<String> gTraceAttrs;
	private Vector<String> gEventAttrs;
	private CohortClassifier cc;
	private String cohortAttrName = "Cohort";

	@Override
	public XLog execute(final UIPluginContext context, XLog log)
	{	
		LogAnalyser la = new LogAnalyser(log);
		
		gTraceAttrs = new Vector<String>(la.calcAvailableTraceAttrs());
		gEventAttrs = new Vector<String>(la.calcAvailableEventAttrs());

		cc = CohortClassifier.Create(context, log);
		
		if (cc == null)
		{
			return log;
		}
		
	    // prompt the user to enter their name
	    cohortAttrName = JOptionPane.showInputDialog(null, "What should the new attribute be called?");
		
		return CreateModifiedLog(log, "XLog new Test");
	}

	public XLog CreateModifiedLog(XLog log, String logName) 
	{
		XLogBuilder newLog = XLogBuilder.newInstance();
		newLog.startLog(logName);

		//for each trace
		for (XTrace trace : log)
		{
			String traceName = getAttributeValueLiteral(trace, XConceptExtension.KEY_NAME);

			newLog.addTrace(traceName);

			//copy all trace attributes
			for (String s : gTraceAttrs)
			{
				Object attrValue = getAttributeValue(trace, s);
				if (attrValue != null)
				{
					AddGenericAttribute(newLog, s, attrValue);
				}
			}
			
			//add cohort attribute
			int cNum = cc.mapTrace2Cohort(trace);
			String cName = "NULL";
			if (cNum >= 0)
			{
				cName = cc.getCohortNames().get(cNum);
			}
			newLog.addAttribute(cohortAttrName, cName);
			
			
			//for each event in trace
			for (XEvent event : trace)
			{
				String eventName = getAttributeValueLiteral(event, XConceptExtension.KEY_NAME);
				
				newLog.addEvent(eventName);

				//copy all event attributes
				for (String s : gEventAttrs)
				{
					Object attrValue = getAttributeValue(event, s);
					if (attrValue != null)
					{
						AddGenericAttribute(newLog, s, attrValue);
					}
				}
			}
		}
		
		XLog resLog = newLog.build();
		
		CopyClassifiers(log, resLog);	
		CopyGlobalTraceAttributes(log, resLog);		
		CopyGlobalEventAttributes(log, resLog);	
		CopyExtensions(log, resLog);
		
		//should add global trace attribute "cohortAttrName"
		resLog.getGlobalTraceAttributes().add(new XAttributeLiteralImpl(cohortAttrName, "None"));
		
		return resLog;
	}
}
