package org.processmining.processprofiler3d.psen.preprocess;

import java.util.Date;
import java.util.Vector;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeDiscreteImpl;
import org.deckfour.xes.model.impl.XAttributeLiteralImpl;
import org.deckfour.xes.model.impl.XAttributeTimestampImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.log.utils.XLogBuilder;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;

public class LPPAddCaseTimingAttributes extends LPPAction
{
	private Vector<String> gTraceAttrs;
	private Vector<String> gEventAttrs;

	@Override
	public XLog execute(final UIPluginContext context, XLog log)
	{	
		LogAnalyser la = new LogAnalyser(log);
		
		gTraceAttrs = new Vector<String>(la.calcAvailableTraceAttrs());
		gEventAttrs = new Vector<String>(la.calcAvailableEventAttrs());
		
		return CreateModifiedLog(log, "XLog new Test");
	}

	public XLog CreateModifiedLog(XLog log, String logName) 
	{
		XLogBuilder newLog = XLogBuilder.newInstance();
		newLog.startLog(logName);

		Vector<Double> durations = new Vector<Double>();
		//for each trace
		for (XTrace trace : log)
		{
			Date arrivalTime = getEarliestTimestampValue(trace, XTimeExtension.KEY_TIMESTAMP);
			Date completeTime = getLatestTimestampValue(trace, XTimeExtension.KEY_TIMESTAMP);
			TimeSpan duration = new TimeSpan(arrivalTime, completeTime);
			durations.add((double)duration.getMilliseconds());
		}
		
		QuartilesClassifier qc = new QuartilesClassifier(durations);
		
		//for each trace
		for (XTrace trace : log)
		{
			String traceName = getAttributeValueLiteral(trace, XConceptExtension.KEY_NAME);

			newLog.addTrace(traceName);

			//copy all trace attributes
			for (String s : gTraceAttrs)
			{
				Object attrValue = getAttributeValue(trace, s);
				if (attrValue != null)
				{
					AddGenericAttribute(newLog, s, attrValue);
				}
			}
			
			//add startTime
			Date arrivalTime = getEarliestTimestampValue(trace, XTimeExtension.KEY_TIMESTAMP);
			Date completeTime = getLatestTimestampValue(trace, XTimeExtension.KEY_TIMESTAMP);
			TimeSpan duration = new TimeSpan(arrivalTime, completeTime);
			int quartile = qc.ClassifyValue(duration.getMilliseconds());

			newLog.addAttribute("arrivalTime", arrivalTime);
			newLog.addAttribute("completeTime", completeTime);
			newLog.addAttribute("caseDuration", duration.getMilliseconds());
			newLog.addAttribute("caseQuartile", quartileToText(quartile));
			
			//for each event in trace
			for (XEvent event : trace)
			{
				String eventName = getAttributeValueLiteral(event, XConceptExtension.KEY_NAME);
				
				newLog.addEvent(eventName);

				//copy all event attributes
				for (String s : gEventAttrs)
				{
					Object attrValue = getAttributeValue(event, s);
					if (attrValue != null)
					{
						AddGenericAttribute(newLog, s, attrValue);
					}
				}
			}
		}
		
		XLog resLog = newLog.build();
		
		CopyClassifiers(log, resLog);	
		CopyGlobalTraceAttributes(log, resLog);		
		CopyGlobalEventAttributes(log, resLog);	
		CopyExtensions(log, resLog);
		
		//should add global trace attributes "arrivalTime" and "completeTime"
		resLog.getGlobalTraceAttributes().add(new XAttributeTimestampImpl("arrivalTime", 0));
		resLog.getGlobalTraceAttributes().add(new XAttributeTimestampImpl("completeTime", 0));
		resLog.getGlobalTraceAttributes().add(new XAttributeDiscreteImpl("caseDuration", 0));
		resLog.getGlobalTraceAttributes().add(new XAttributeLiteralImpl("caseQuartile", ""));
		
		return resLog;
	}
	
	private String quartileToText(int q)
	{
        if (q == 0)
        {
            return "Outlier (Low)";
        }
        else if (q == 1)
        {
            return "1st Quartile";
        }
        else if (q == 2)
        {
            return "2nd Quartile";
        }
        else if (q == 3)
        {
            return "3rd Quartile";
        }
        else if (q == 4)
        {
            return "4th Quartile";
        }
        else if (q == 5)
        {
            return "Outlier (High)";
        }
        return "";
	}
}
