package org.processmining.processprofiler3d.psen.preprocess;

import java.util.Vector;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.log.utils.XLogBuilder;
import org.processmining.processprofiler3d.psen.LogAnalyser;

public class LPPMergeLifecycleEvents extends LPPAction
{
	private Vector<String> gTraceAttrs;
	private Vector<String> gEventAttrs;

	@Override
	public XLog execute(final UIPluginContext context, XLog log)
	{	
		LogAnalyser la = new LogAnalyser(log);

		gTraceAttrs = new Vector<String>(la.calcAvailableTraceAttrs());
		gEventAttrs = new Vector<String>(la.calcAvailableEventAttrs());

		Vector<String> availableAttrs = new Vector<String>();
		availableAttrs.add("schedule");
		availableAttrs.add("assign");
		availableAttrs.add("start");
		availableAttrs.add("complete");
		
		Vector<String> remAttrs = PreprocessSelectAttributesPanel.showDialog(context, "Select which lifecycle events to merge:", availableAttrs);
		
		XLogBuilder newLog = XLogBuilder.newInstance();
		newLog.startLog("XLog new Test");

		//for each trace
		for (XTrace trace : log)
		{
			String traceName = getAttributeValueLiteral(trace, XConceptExtension.KEY_NAME);

			newLog.addTrace(traceName);

			//copy all trace attributes
			for (String s : gTraceAttrs)
			{
				Object attrValue = getAttributeValue(trace, s);
				if (attrValue != null)
				{
					AddGenericAttribute(newLog, s, attrValue);
				}
			}
			
			//for each event in trace
			for (XEvent event : trace)
			{
				String eventName = getAttributeValueLiteral(event, "concept:name");

				String lifecycleStage = getAttributeValueLiteral(event, XLifecycleExtension.KEY_TRANSITION);
				
				//if event has lifecycle attribute and value is on remove list, don't add event to new trace
				if (lifecycleStage != null)
				{
					if (remAttrs.contains(lifecycleStage))
					{
						continue;
					}
				}
				
				newLog.addEvent(eventName);

				//copy all event attributes
				for (String s : gEventAttrs)
				{
					Object attrValue = getAttributeValue(event, s);
					if (attrValue != null)
					{
						AddGenericAttribute(newLog, s, attrValue);
					}
				}
			}
		}
		
		XLog resLog = newLog.build();
		
		CopyClassifiers(log, resLog);	
		CopyGlobalTraceAttributes(log, resLog);		
		CopyGlobalEventAttributes(log, resLog);	
		CopyExtensions(log, resLog);
		
		return resLog;
	}
}
