package org.processmining.processprofiler3d.psen.preprocess;

import java.util.Vector;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.log.utils.XLogBuilder;
import org.processmining.processprofiler3d.psen.LogAnalyser;

public class LPPRemoveEventAttributes extends LPPAction 
{
	private Vector<String> gTraceAttrs;
	private Vector<String> gEventAttrs;
	private Vector<String> remAttrs;

	@Override
	public XLog execute(final UIPluginContext context, XLog log)
	{	
		LogAnalyser la = new LogAnalyser(log);
		
		gTraceAttrs = new Vector<String>(la.calcAvailableTraceAttrs());
		gEventAttrs = new Vector<String>(la.calcAvailableEventAttrs());
		
		remAttrs = PreprocessSelectAttributesPanel.showDialog(context, "Select which attributes to remove:", gEventAttrs);
		
		return CreateModifiedLog(log);
	}
	
	public XLog execute(final UIPluginContext context, XLog log, Vector<String> remAttrs)
	{	
		LogAnalyser la = new LogAnalyser(log);
		
		gTraceAttrs = new Vector<String>(la.calcAvailableTraceAttrs());
		gEventAttrs = new Vector<String>(la.calcAvailableEventAttrs());
		
		this.remAttrs = remAttrs;
		
		return CreateModifiedLog(log);
	}
	
	public XLog execute(XLog log, Vector<String> remAttrs)
	{	
		LogAnalyser la = new LogAnalyser(log);
		
		gTraceAttrs = new Vector<String>(la.calcAvailableTraceAttrs());
		gEventAttrs = new Vector<String>(la.calcAvailableEventAttrs());
		
		this.remAttrs = remAttrs;
		
		return CreateModifiedLog(log);
	}

	public XLog CreateModifiedLog(XLog log) 
	{
		XLogBuilder newLog = XLogBuilder.newInstance();
		newLog.startLog("XLog new Test");

		//for each trace
		for (XTrace trace : log)
		{
			String traceName = getAttributeValueLiteral(trace, XConceptExtension.KEY_NAME);

			newLog.addTrace(traceName);

			//copy all trace attributes
			for (String s : gTraceAttrs)
			{
				Object attrValue = getAttributeValue(trace, s);
				if (attrValue != null)
				{
					AddGenericAttribute(newLog, s, attrValue);
				}
			}
			
			//for each event in trace
			for (XEvent event : trace)
			{
				String eventName = getAttributeValueLiteral(event, XConceptExtension.KEY_NAME);
				
				newLog.addEvent(eventName);

				//copy all event attributes
				for (String s : gEventAttrs)
				{
					if (!remAttrs.contains(s))
					{
						Object attrValue = getAttributeValue(event, s);
						if (attrValue != null)
						{
							AddGenericAttribute(newLog, s, attrValue);
						}
					}
				}
			}
		}
		
		XLog resLog = newLog.build();
		
		CopyClassifiers(log, resLog);	
		CopyGlobalTraceAttributes(log, resLog);		
		CopyGlobalEventAttributesFiltered(log, resLog);	
		CopyExtensions(log, resLog);
		
		return resLog;
	}

	protected void CopyGlobalEventAttributesFiltered(XLog source, XLog target)
	{
		for (XAttribute gatt : source.getGlobalEventAttributes())
		{
			if (!remAttrs.contains(gatt.getKey()))
			{
				target.getGlobalEventAttributes().add(gatt);
			}
		}
	}
}
