package org.processmining.processprofiler3d.psen.preprocess;

import java.util.Vector;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.log.utils.XLogBuilder;
import org.processmining.processprofiler3d.psen.LogAnalyser;

public class LPPRemoveEventsByAttribute extends LPPAction 
{
	private Vector<String> gTraceAttrs;
	private Vector<String> gEventAttrs;
	private Vector<String> remAttrs;
	private String filterAttribute;

	@Override
	public XLog execute(final UIPluginContext context, XLog log)
	{	
		LogAnalyser la = new LogAnalyser(log);

		gTraceAttrs = new Vector<String>(la.calcAvailableTraceAttrs());
		gEventAttrs = new Vector<String>(la.calcAvailableEventAttrs());

		Vector<String> remAttrs2 = PreprocessSelectAttributesPanel.showDialog(context, "Select an attribute to filter on:", gEventAttrs);
		filterAttribute = remAttrs2.firstElement();
		
		Vector<String> availableAttrs = new Vector<String>();

		for (XTrace trace : log)
		{
			Vector<String> lst = getAvailableLiteralValuesForEvents(trace, filterAttribute);
			for (String v : lst)
			{
				if (!availableAttrs.contains(v))
				{
					availableAttrs.add(v);
				}
			}
		}	
		
		remAttrs = PreprocessSelectAttributesPanel.showDialog(context, "Select which values to filter out:", availableAttrs);
		
		return CreateModifiedLog(log, "XLog new Test");
	}

	public XLog CreateModifiedLog(XLog log, String logName) 
	{	
		XLogBuilder newLog = XLogBuilder.newInstance();
		newLog.startLog(logName);

		//for each trace
		for (XTrace trace : log)
		{
			String traceName = getAttributeValueLiteral(trace, XConceptExtension.KEY_NAME);
			newLog.addTrace(traceName);

			//copy all trace attributes
			for (String s : gTraceAttrs)
			{
				Object attrValue = getAttributeValue(trace, s);
				if (attrValue != null)
				{
					AddGenericAttribute(newLog, s, attrValue);
				}
			}
			
			//for each event in trace
			for (XEvent event : trace)
			{
				String eventName = getAttributeValueLiteral(event, XConceptExtension.KEY_NAME);
				String eventLifecycle = getAttributeValueLiteral(event, filterAttribute);
				//boolean bRemove = !eventLifecycle.equals("start");
				boolean bRemove = remAttrs.contains(eventLifecycle);
				
				if (!bRemove)
				{
					newLog.addEvent(eventName);

					//copy all event attributes
					for (String s : gEventAttrs)
					{
						Object attrValue = getAttributeValue(event, s);
						if (attrValue != null)
						{
							AddGenericAttribute(newLog, s, attrValue);
						}
					}
				}
			}
		}
		
		XLog resLog = newLog.build();
		
		RemoveEmptyTraces(resLog);
		
		CopyClassifiers(log, resLog);	
		CopyGlobalTraceAttributes(log, resLog);		
		CopyGlobalEventAttributes(log, resLog);	
		CopyExtensions(log, resLog);
		
		return resLog;
	}
}
