package org.processmining.processprofiler3d.psen.preprocess;

import java.util.Vector;

import javax.swing.JOptionPane;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.log.utils.XLogBuilder;
import org.processmining.processprofiler3d.psen.LogAnalyser;
import org.processmining.processprofiler3d.psen.classification.ClassificationCohort;
import org.processmining.processprofiler3d.psen.gui.AddCohortPane;
import org.processmining.processprofiler3d.psen.gui.PSENAddGenericCohortPanel;

public class LPPRemoveTracesByAttribute extends LPPAction 
{
	private Vector<String> gTraceAttrs;
	private Vector<String> gEventAttrs;
	private ClassificationCohort ccx;

	@Override
	public XLog execute(final UIPluginContext context, XLog log)
	{	
		LogAnalyser la = new LogAnalyser(log);

		gTraceAttrs = new Vector<String>(la.calcAvailableTraceAttrs());
		gEventAttrs = new Vector<String>(la.calcAvailableEventAttrs());
		
		PSENAddGenericCohortPanel panel = new PSENAddGenericCohortPanel(null, log, 0, "Select traces to keep");

	    int result = AddCohortPane.showDialog(panel, "Select traces to keep");
		
	    if (result == JOptionPane.OK_OPTION)
	    {	
	    	//if (panel.isDataValid(cc))
	    	{
	    		ccx = panel.getCohortDefinition().createClassificationCohort();
	    	}
	    }
	    else
	    {
	    	return log;
	    }

		return CreateModifiedLog(log, "XLog new Test");
	}

	public XLog CreateModifiedLog(XLog log, String logName) 
	{
		XLogBuilder newLog = XLogBuilder.newInstance();
		newLog.startLog(logName);

		//for each trace
		for (XTrace trace : log)
		{
//			int cNum = cc.mapTrace2Cohort(trace);
//			boolean bRemove = (cNum != 0);
			
			boolean bRemove = !ccx.isPartOfCohort(trace);
			
			if (!bRemove)
			{
				String traceName = getAttributeValueLiteral(trace, XConceptExtension.KEY_NAME);
				newLog.addTrace(traceName);

				//copy all trace attributes
				for (String s : gTraceAttrs)
				{
					Object attrValue = getAttributeValue(trace, s);
					if (attrValue != null)
					{
						AddGenericAttribute(newLog, s, attrValue);
					}
				}
				
				//for each event in trace
				for (XEvent event : trace)
				{
					String eventName = getAttributeValueLiteral(event, XConceptExtension.KEY_NAME);
					
					newLog.addEvent(eventName);

					//copy all event attributes
					for (String s : gEventAttrs)
					{
						Object attrValue = getAttributeValue(event, s);
						if (attrValue != null)
						{
							AddGenericAttribute(newLog, s, attrValue);
						}
					}
				}
			}
		}
		
		XLog resLog = newLog.build();
		
		CopyClassifiers(log, resLog);	
		CopyGlobalTraceAttributes(log, resLog);		
		CopyGlobalEventAttributes(log, resLog);	
		CopyExtensions(log, resLog);
		
		return resLog;
	}
}
