package org.processmining.processprofiler3d.psen.preprocess;

import java.util.Vector;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.log.utils.XLogBuilder;
import org.processmining.processprofiler3d.psen.LogAnalyser;

public class LPPRenameEvents extends LPPAction 
{
	private Vector<String> gTraceAttrs;
	private Vector<String> gEventAttrs;
	private Vector<String> promoted;

	@Override
	public XLog execute(final UIPluginContext context, XLog log)
	{	
		LogAnalyser la = new LogAnalyser(log);

		gTraceAttrs = new Vector<String>(la.calcAvailableTraceAttrs());
		gEventAttrs = new Vector<String>(la.calcAvailableEventAttrs());

		promoted = new Vector<String>();
		
		//for each trace
		for (XTrace trace : log)
		{
			//promote all attributes that have the same value across all events to trace attributes
			for (String s : gEventAttrs)
			{
				Vector<Object> vals = getAvailableValuesForEvents(trace, s);
				if (vals.size() == 1)
				{
					if (!promoted.contains(s))
					{
						promoted.add(s);
					}
				}
			}			
		}	
		
		if (promoted.size() > 0)
		{
			promoted = PreprocessSelectAttributesPanel.showDialog(context, "Select:", promoted);
		}
		
		return CreateModifiedLog(log);
	}

	public XLog CreateModifiedLog(XLog log) 
	{
		XLogBuilder newLog = XLogBuilder.newInstance();
		newLog.startLog("XLog new Test");

		//for each trace
		for (XTrace trace : log)
		{
			XAttributeLiteral xAtt = (XAttributeLiteral)trace.getAttributes().get(XConceptExtension.KEY_NAME);
			String traceName = xAtt.getValue();

			newLog.addTrace(traceName);

			//copy all trace attributes
			for (String s : gTraceAttrs)
			{
				Object attrValue = getAttributeValue(trace, s);
				if (attrValue != null)
				{
					AddGenericAttribute(newLog, s, attrValue);
				}
			}
			
			//promote all attributes that have the same value across all events to trace attributes
			for (String s : gEventAttrs)
			{
				if (promoted.contains(s))
				{
					Vector<Object> vals = getAvailableValuesForEvents(trace, s);
					if (vals.size() == 1)
					{
						AddGenericAttribute(newLog, s, vals.get(0));
						System.out.println("Promoted attribute: " + s);
					}
				}
			}
			
			//for each event in trace
			for (XEvent event : trace)
			{
				String eventName = getAttributeValueLiteral(event, XConceptExtension.KEY_NAME);
				
				newLog.addEvent(eventName);

				//copy all event attributes
				for (String s : gEventAttrs)
				{
					Object attrValue = getAttributeValue(event, s);
					if (attrValue != null)
					{
						AddGenericAttribute(newLog, s, attrValue);
					}
				}
			}
		}	
		
		XLog resLog = newLog.build();
		
		CopyClassifiers(log, resLog);	
		CopyGlobalTraceAttributes(log, resLog);		
		CopyGlobalEventAttributes(log, resLog);	
		CopyExtensions(log, resLog);
		
		return resLog;
	}

}
