package org.processmining.processprofiler3d.psen.preprocess;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Vector;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.processprofiler3d.psen.LogAnalyser;

public class LPPSortEvents extends LPPAction
{

	public LPPSortEvents()
	{
	}
	
	@Override
	public XLog execute(final UIPluginContext context, XLog log)
	{	
		LogAnalyser la = new LogAnalyser(log);
		
		Vector<String> gEventAttrs = new Vector<String>(la.calcAvailableEventAttrs());
		
		Vector<String> remAttrs = PreprocessSelectAttributesPanel.showDialog(context, "Select attribute to sort events by:", gEventAttrs);
		
		if (remAttrs.isEmpty())
		{
			return log;
		}
		else
		{
			XLog nlog = (XLog)log.clone();
			for (XTrace trc : nlog)
			{
				sortEvents(remAttrs.get(0), trc);
			}
			
			return nlog;
		}
	}
	
	public static void sortEvents(final String field, List<XEvent> itemLocationList) 
	{
	    Collections.sort(itemLocationList, new Comparator<XEvent>() 
	    {
	        @Override
	        public int compare(XEvent o1, XEvent o2) 
	        {
	        	XAttribute attr1 = o1.getAttributes().get(field);
	        	XAttribute attr2 = o2.getAttributes().get(field);
	            if(attr1 instanceof XAttributeDiscrete && attr2 instanceof XAttributeDiscrete) 
	            {
	            	XAttributeDiscrete attrD1 = (XAttributeDiscrete)attr1;
	            	XAttributeDiscrete attrD2 = (XAttributeDiscrete)attr2;
	                return Long.compare(attrD1.getValue(), attrD2.getValue());
	            }
	            if(attr1 instanceof XAttributeContinuous && attr2 instanceof XAttributeContinuous) 
	            {
	            	XAttributeContinuous attrD1 = (XAttributeContinuous)attr1;
	            	XAttributeContinuous attrD2 = (XAttributeContinuous)attr2;
	                return Double.compare(attrD1.getValue(), attrD2.getValue());
	            }
	            if(attr1 instanceof XAttributeTimestamp && attr2 instanceof XAttributeTimestamp) 
	            {
	            	XAttributeTimestamp attrD1 = (XAttributeTimestamp)attr1;
	            	XAttributeTimestamp attrD2 = (XAttributeTimestamp)attr2;
	                return attrD1.getValue().compareTo(attrD2.getValue());
	            }
	            
	            return 0;
	        }           
	    });
	}

}
