package org.processmining.processprofiler3d.psen.preprocess;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JViewport;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.util.ui.widgets.ProMList;
import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;
import org.processmining.framework.util.ui.widgets.ProMScrollPane;
import org.processmining.framework.util.ui.widgets.ProMScrollablePanel;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.processprofiler3d.psen.LogAnalyser;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.components.SlickerButton;

public class PreprocessMenuPanel extends ProMPropertiesPanel implements ActionListener
{
	//action types
	private static final int NO_ACTION = -1;
	private static final int PROMOTE_EVENT_ATTRIBUTES = 0;
	private static final int MERGE_LIFECYCLE_EVENTS = 1;
	private static final int REMOVE_EVENT_ATTRIBUTES = 2;
	private static final int REMOVE_TRACE_ATTRIBUTES = 3;
	private static final int REMOVE_TRACES_BY_ATTRIBUTE = 4;
	private static final int CALCULATE_TRACE_TIMING_ATTRIBUTES = 5;
	private static final int CHECK_EVENT_ORDERING = 6;
	private static final int REMOVE_EVENTS_BY_ATTRIBUTE = 7;
	private static final int ADD_COHORT_ATTR = 8;
	private static final int SORT_EVENTS_BY_ATTR = 9;

	private static final long serialVersionUID = 2643942608122666179L;

	private SlickerButton btnPromoteAttrs;
	private SlickerButton btnMergeLifecycleEvents;
	private SlickerButton btnRemoveEvtAttrs;
	private SlickerButton btnTraceAttrs;
	private SlickerButton btnRemoveEventsByAttr;
	private SlickerButton btnRemoveTracesByAttr;
	private SlickerButton btnAddTimingAttrs;
	private SlickerButton btnAddCohortAttr;
	private SlickerButton btnCheckEvtOrder;
	private SlickerButton btnSortEvts;
	private SlickerButton btn7;
	
	private ProMList<String> traceAttrList;
	private ProMList<String> eventAttrList;
	
	private XLog finalLog = null;
	
	private int selectedAction = NO_ACTION;
	
	private boolean bFinished = true;

	public static XLog showDialog(final UIPluginContext context, XLog log)
	{
		boolean bRepeat = true;
		XLog finalLog = log;
		
		while (bRepeat)
		{
			PreprocessMenuPanel glcp = new PreprocessMenuPanel(finalLog);
			
			//int result = AddCohortPane.showDialog(glcp);
			int result = new PreprocessMenuDialog("Preprocess Event Log", glcp).ShowDialog(context);
	
		    if (result == JOptionPane.OK_OPTION)
		    {
				finalLog = ExecuteAction(context, finalLog, glcp.selectedAction, glcp);
				System.out.println("Preprocessing action completed.");
				LPPAction.setAttributeValueLiteral(finalLog, XConceptExtension.KEY_NAME, glcp.getLogName());
				if (glcp.bFinished)
				{
					break;
				}
		    }
		    else
		    {
		    	finalLog = null;
		    	break;
		    }
		}
	    return finalLog;
	}
	
	private ProMTextField tbxLogName;
	
	public PreprocessMenuPanel(XLog log) 
	{
		super("Choose Preprocessing Action");
		
		finalLog = log;
		

		String logName = LPPAction.getAttributeValueLiteral(log, XConceptExtension.KEY_NAME);
		if (logName == null)
		{
			logName = "Preprocessed Event Log";
		}
		
		tbxLogName = this.addTextField("New Log Name");
		tbxLogName.setText(logName);

		JPanel myProp = addRoundedPanel(addTraceInfoSection(log));
		JPanel myProp2 = addRoundedPanel(addEventInfoSection(log));
		JPanel myProp3 = addRoundedPanel(addTraceAttrSection(log));
		JPanel myProp4 = addRoundedPanel(addEventAttrSection(log));
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBackground(this.getBackground());    
	    setPreferredSize(new Dimension(800,600));
	}
	
	public JPanel addTraceInfoSection(XLog log)
	{	
		JPanel panelInfo = new JPanel();
		panelInfo.setLayout(new BoxLayout(panelInfo, BoxLayout.X_AXIS));	
		panelInfo.setBackground(new Color(0,0,0,0));
		panelInfo.setOpaque(false);
		
		JPanel infoPanel = addTraceInfoPanel(log);	
		JPanel btnPanel = addButtonPanelTraceInfo(log);

		panelInfo.add(Box.createHorizontalStrut(5));
		panelInfo.add(infoPanel);
		panelInfo.add(Box.createHorizontalStrut(5));
		panelInfo.add(btnPanel);
		panelInfo.add(Box.createHorizontalStrut(5));

		return panelInfo;
	}
	
	public JPanel addEventInfoSection(XLog log)
	{	
		JPanel panelInfo = new JPanel();
		panelInfo.setLayout(new BoxLayout(panelInfo, BoxLayout.X_AXIS));	
		panelInfo.setBackground(new Color(0,0,0,0));
		panelInfo.setOpaque(false);
		
		JPanel infoPanel = addEventInfoPanel(log);	
		JPanel btnPanel = addButtonPanelEventInfo(log);

		panelInfo.add(Box.createHorizontalStrut(5));
		panelInfo.add(infoPanel);
		panelInfo.add(Box.createHorizontalStrut(5));
		panelInfo.add(btnPanel);
		panelInfo.add(Box.createHorizontalStrut(5));

		return panelInfo;
	}

	private JPanel addTraceAttrSection(XLog log) 
	{
		JPanel panelTraceAttr = new JPanel();
		panelTraceAttr.setLayout(new BoxLayout(panelTraceAttr, BoxLayout.X_AXIS));	
		panelTraceAttr.setBackground(new Color(0,0,0,0));
		panelTraceAttr.setOpaque(false);

		ProMList<String> lst1 = addTraceAttributeList(log);
		panelTraceAttr.add(lst1);
		panelTraceAttr.add(addButtonPanelTraceAttr(log));
		return panelTraceAttr;
	}

	private JPanel addEventAttrSection(XLog log) 
	{
		ProMList<String> lst = addEventAttributeList(log);	

		JPanel panelEventAttr = new JPanel();
		panelEventAttr.setLayout(new BoxLayout(panelEventAttr, BoxLayout.X_AXIS));	
		panelEventAttr.setBackground(new Color(0,0,0,0));
		panelEventAttr.setOpaque(false);
		
		panelEventAttr.add(lst);
		panelEventAttr.add(addButtonPanelEventAttr(log));
		return panelEventAttr;
	}
	
	public JPanel addRoundedPanel(JComponent jc)
	{
		addProperty("", jc);
		
		ProMScrollablePanel pspx = getPropertiesPanel();
		RoundedPanel rp = null;
		for (Component jcc : pspx.getComponents())
		{
			if (jcc instanceof RoundedPanel)
			{
				rp = (RoundedPanel)jcc;
			}
		}
		rp.remove(0);
		rp.remove(0);
		
		return rp;
	}
	
	public Vector<String> getSelectedEventAttrs()
	{
		return new Vector<String>(eventAttrList.getSelectedValuesList());
	}
	
	public Vector<String> getSelectedTraceAttrs()
	{
		return new Vector<String>(traceAttrList.getSelectedValuesList());
	}
	
	public ProMScrollablePanel getPropertiesPanel()
	{
		for (Component jcc : getComponents())
		{
			if (jcc instanceof ProMScrollPane)
			{
				ProMScrollPane psp = (ProMScrollPane)jcc;
				RoundedPanel rp = null;
				for (Component jccx : psp.getComponents())
				{
					if (jccx instanceof JViewport)
					{
						for (Component jccxx : ((JViewport)jccx).getComponents())
						{
							if (jccxx instanceof ProMScrollablePanel)
							{
								ProMScrollablePanel pspx = (ProMScrollablePanel)jccxx;
								return pspx;
							}
						}
					}
				}
			}
		}
		
		return null;
	}

	private JPanel addButtonPanelTraceAttr(XLog log) 
	{
		LogAnalyser la = new LogAnalyser(log);
		
		JPanel btnPanel = new JPanel();
		btnPanel.setLayout(new BoxLayout(btnPanel, BoxLayout.Y_AXIS));
		btnPanel.setBackground(new Color(0,0,0,0));
		btnPanel.setOpaque(false);
	    
	    boolean bHasEventTimestamp = la.calcAvailableEventAttrsTimestamp().contains(XTimeExtension.KEY_TIMESTAMP);
	    
	    btnTraceAttrs = new SlickerButton("Remove Trace Attributes");
	    btnTraceAttrs.addActionListener(this);
	    btnPanel.add(btnTraceAttrs);
	    
	    btnAddTimingAttrs = new SlickerButton("Add Timing Attributes");
	    btnAddTimingAttrs.addActionListener(this);
	    btnAddTimingAttrs.setEnabled(bHasEventTimestamp);
	    btnPanel.add(btnAddTimingAttrs);
	    
	    btnAddCohortAttr = new SlickerButton("Add Cohort Attribute");
	    btnAddCohortAttr.addActionListener(this);
	    btnPanel.add(btnAddCohortAttr);
	    
		return btnPanel;
	}

	private JPanel addButtonPanelEventAttr(XLog log) 
	{
		LogAnalyser la = new LogAnalyser(log);
		
		JPanel btnPanel = new JPanel();
		btnPanel.setLayout(new BoxLayout(btnPanel, BoxLayout.Y_AXIS));
		btnPanel.setBackground(new Color(0,0,0,0));
		btnPanel.setOpaque(false);

		boolean bCanPromote = !getPromotableEventAttributes(la, log).isEmpty();
		
	    btnPromoteAttrs = new SlickerButton("Promote Attributes");
	    btnPromoteAttrs.addActionListener(this);
	    btnPromoteAttrs.setEnabled(bCanPromote);
	    btnPanel.add(btnPromoteAttrs);
	    
	    btnRemoveEvtAttrs = new SlickerButton("Remove Event Attributes");
	    btnRemoveEvtAttrs.addActionListener(this);
	    btnPanel.add(btnRemoveEvtAttrs);
		return btnPanel;
	}

	private JPanel addButtonPanelTraceInfo(XLog log) 
	{
		//LogAnalyser la = new LogAnalyser(log);
		
		JPanel btnPanel = new JPanel();
		btnPanel.setLayout(new BoxLayout(btnPanel, BoxLayout.Y_AXIS));
		btnPanel.setBackground(new Color(0,0,0,0));
		btnPanel.setOpaque(false);

	    btnRemoveTracesByAttr = new SlickerButton("Remove Traces by Attribute");
	    btnRemoveTracesByAttr.addActionListener(this);
	    btnPanel.add(btnRemoveTracesByAttr);
	    
		return btnPanel;
	}

	private JPanel addButtonPanelEventInfo(XLog log) 
	{
		LogAnalyser la = new LogAnalyser(log);
		
		JPanel btnPanel = new JPanel();
		btnPanel.setLayout(new BoxLayout(btnPanel, BoxLayout.Y_AXIS));
		btnPanel.setBackground(new Color(0,0,0,0));
		btnPanel.setOpaque(false);
	    
	    boolean bCanMerge = la.calcAvailableEventAttrsLiteral().contains(XLifecycleExtension.KEY_TRANSITION);
	    boolean bHasEventTimestamp = la.calcAvailableEventAttrsTimestamp().contains(XTimeExtension.KEY_TIMESTAMP);
	    
	    btnMergeLifecycleEvents = new SlickerButton("Merge Lifecycle Events");
	    btnMergeLifecycleEvents.addActionListener(this);
	    btnMergeLifecycleEvents.setEnabled(bCanMerge);
	    btnPanel.add(btnMergeLifecycleEvents);

	    btnRemoveEventsByAttr = new SlickerButton("Remove Events by Attribute");
	    btnRemoveEventsByAttr.addActionListener(this);
	    btnPanel.add(btnRemoveEventsByAttr);
	    
	    btnCheckEvtOrder = new SlickerButton("Check Event Time Ordering");
	    btnCheckEvtOrder.addActionListener(this);
	    btnCheckEvtOrder.setEnabled(bHasEventTimestamp);
	    btnPanel.add(btnCheckEvtOrder);
	    
	    btnSortEvts = new SlickerButton("Sort Events");
	    btnSortEvts.addActionListener(this);
	    //btnSortEvts.setEnabled(bHasEventTimestamp);
	    btnPanel.add(btnSortEvts);
	    
	    btn7 = new SlickerButton("Rename Events");
	    btn7.addActionListener(this);
	    btnPanel.add(btn7);
		return btnPanel;
	}

	private JPanel addTraceInfoPanel(XLog log) 
	{
		JPanel infoPanel = new JPanel();
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.Y_AXIS));
		infoPanel.setBackground(new Color(0,0,0,0));
		infoPanel.setOpaque(false);
		
		JLabel lblNumTraces = new JLabel();
		lblNumTraces.setForeground(WidgetColors.TEXT_COLOR);
		lblNumTraces.setText("" + log.size() + " traces");
		
		infoPanel.add(lblNumTraces);
		return infoPanel;
	}

	private JPanel addEventInfoPanel(XLog log) 
	{
		JPanel infoPanel = new JPanel();
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.Y_AXIS));
		infoPanel.setBackground(new Color(0,0,0,0));
		infoPanel.setOpaque(false);
		
		JLabel lblNumEvents = new JLabel();
		lblNumEvents.setForeground(WidgetColors.TEXT_COLOR);
		int numEvents = 0;
		for (XTrace trc : log)
		{
			numEvents = numEvents + trc.size();
		}
		lblNumEvents.setText("" + numEvents + " events");
		
		infoPanel.add(lblNumEvents);
		return infoPanel;
	}

	private ProMList<String> addEventAttributeList(XLog log) 
	{
		Vector<String> l2 = getEventAttributes(log);
		
		DefaultListModel<String> lm2 = new DefaultListModel<String>();
		for (String s : l2)
		{
			lm2.addElement(s);
		}
		
		eventAttrList = new ProMList<String>("Event Attributes", lm2);
		eventAttrList.setPreferredSize(new Dimension(800, 200));
		//eventAttrList.addMouseListener(this);
		
		return eventAttrList;
	}

	private ProMList<String> addTraceAttributeList(XLog log) 
	{
		Vector<String> l1 = getTraceAttributes(log);
		
		DefaultListModel<String> lm1 = new DefaultListModel<String>();
		for (String s : l1)
		{
			lm1.addElement(s);
		}
		
		traceAttrList = new ProMList<String>("Trace Attributes", lm1);
		traceAttrList.setPreferredSize(new Dimension(800, 200));
		//traceAttrList.addMouseListener(this);

		return traceAttrList;
	}

	public String getLogName()
	{
		return tbxLogName.getText();
		//return "Preprocessed Event Log";
	}
	
	public void actionPerformed(ActionEvent evt) 
	{
//		JOptionPane pane = AddCohortPane.getOptionPane(btnCheckEvtOrder);
		
		//add button pressed
		int actionId = NO_ACTION;
		if (evt.getSource() == btnPromoteAttrs)
		{
			actionId = PROMOTE_EVENT_ATTRIBUTES;
		}
		//remove button pressed
		else if (evt.getSource() == btnMergeLifecycleEvents)
		{
			actionId = MERGE_LIFECYCLE_EVENTS;
		}
		else if (evt.getSource() == btnRemoveEvtAttrs)
		{
			actionId = REMOVE_EVENT_ATTRIBUTES;
		}
		else if (evt.getSource() == btnTraceAttrs)
		{
			actionId = REMOVE_TRACE_ATTRIBUTES;
		}
		else if (evt.getSource() == btnRemoveTracesByAttr)
		{
			actionId = REMOVE_TRACES_BY_ATTRIBUTE;
		}
		else if (evt.getSource() == btnAddTimingAttrs)
		{
			actionId = CALCULATE_TRACE_TIMING_ATTRIBUTES;
		}
		else if (evt.getSource() == btnCheckEvtOrder)
		{
			actionId = CHECK_EVENT_ORDERING;
		}
		else if (evt.getSource() == btnRemoveEventsByAttr)
		{
			actionId = REMOVE_EVENTS_BY_ATTRIBUTE;
		}
		else if (evt.getSource() == btnAddCohortAttr)
		{
			actionId = ADD_COHORT_ATTR;
		}
		else if (evt.getSource() == btnSortEvts)
		{
			actionId = SORT_EVENTS_BY_ATTR;
		}
		bFinished = (actionId == NO_ACTION);
		selectedAction = actionId;
		
//		if (pane != null)
//		{
//			pane.setValue(AddCohortPane.getOKButton(pane));
//		}
	}
	
	public static XLog ExecuteAction(final UIPluginContext ctx, XLog finalLog, int actionId, PreprocessMenuPanel pmp)
	{
		XLog result = finalLog;
		if (actionId == PROMOTE_EVENT_ATTRIBUTES)
		{
			result = LogPreprocessor.promoteEventAttributes(ctx, finalLog);
		}
		//remove button pressed
		else if (actionId == MERGE_LIFECYCLE_EVENTS)
		{
			result = LogPreprocessor.mergeLifecycleEvents(ctx, finalLog);
		}
		else if (actionId == REMOVE_EVENT_ATTRIBUTES)
		{
			result = LogPreprocessor.removeEventAttributes(ctx, finalLog, pmp.getSelectedEventAttrs());
		}
		else if (actionId == REMOVE_TRACE_ATTRIBUTES)
		{
			result = LogPreprocessor.removeTraceAttributes(ctx, finalLog, pmp.getSelectedTraceAttrs());
		}
		else if (actionId == REMOVE_TRACES_BY_ATTRIBUTE)
		{
			result = LogPreprocessor.removeTracesByAttribute(ctx, finalLog);
			int numRemoved = finalLog.size() - result.size();
			System.out.println("Removed " + numRemoved + " traces of " + finalLog.size() + " from log. " + result.size() + " traces remaining.");
		}
		else if (actionId == CALCULATE_TRACE_TIMING_ATTRIBUTES)
		{
			result = LogPreprocessor.addTimingAttributes(ctx, finalLog);
		}
		else if (actionId == CHECK_EVENT_ORDERING)
		{
			result = LogPreprocessor.checkEventOrdering(ctx, finalLog);
		}
		else if (actionId == REMOVE_EVENTS_BY_ATTRIBUTE)
		{
			result = LogPreprocessor.removeEventsByAttribute(ctx, finalLog);
		}
		else if (actionId == ADD_COHORT_ATTR)
		{
			result = LogPreprocessor.addCohortAttribute(ctx, finalLog);
		}
		else if (actionId == SORT_EVENTS_BY_ATTR)
		{
			result = LogPreprocessor.sortEvents(ctx, finalLog);
		}
		return result;
	}
	
	public XLog getFinalLog()
	{
		return finalLog;
	}
	
	public Vector<String> getPromotableEventAttributes(LogAnalyser la, XLog log) 
	{
		Vector<String> gEventAttrs = new Vector<String>(la.calcAvailableEventAttrs());

		HashSet<String> promoted = new HashSet<String>();
		
		//for each trace
		for (XTrace trace : log)
		{
			//promote all attributes that have the same value across all events to trace attributes
			for (String s : gEventAttrs)
			{
				Vector<Object> vals = LPPAction.getAvailableValuesForEvents(trace, s);
				
				if (vals.size() == 1)
				{
					if (!promoted.contains(s))
					{
						promoted.add(s);
					}
				}
			}		
		}	
		
		return new Vector<String>(promoted);
	}
	
	public Vector<String> getPromotableEventAttributes(XLog log) 
	{
		LogAnalyser la = new LogAnalyser(log);
		
		Vector<String> gEventAttrs = new Vector<String>(la.calcAvailableEventAttrs());

		HashSet<String> promoted = new HashSet<String>();
		
		//for each trace
		for (XTrace trace : log)
		{
			//promote all attributes that have the same value across all events to trace attributes
			for (String s : gEventAttrs)
			{
				Vector<Object> vals = LPPAction.getAvailableValuesForEvents(trace, s);
				
				if (vals.size() == 1)
				{
					if (!promoted.contains(s))
					{
						promoted.add(s);
					}
				}
			}		
		}	
		
		return new Vector<String>(promoted);
	}
	
	public Vector<String> getEventAttributes(XLog log) 
	{
		LogAnalyser la = new LogAnalyser(log);
		
		Vector<String> gEventAttrs = new Vector<String>(la.calcAvailableEventAttrs());
		
		return gEventAttrs;
	}
	
	public Vector<String> getTraceAttributes(XLog log) 
	{
		LogAnalyser la = new LogAnalyser(log);
		
		Vector<String> gTraceAttrs = new Vector<String>(la.calcAvailableTraceAttrs());
		
		return gTraceAttrs;
	}
	
	public Vector<JButton> getButtons()
	{
		Vector<JButton> lst = new Vector<JButton>();
		
		lst.add(btnPromoteAttrs);
		lst.add(btnMergeLifecycleEvents);
		lst.add(btnRemoveEvtAttrs);
		lst.add(btnTraceAttrs);
		lst.add(btnRemoveEventsByAttr);
		lst.add(btnRemoveTracesByAttr);
		lst.add(btnAddTimingAttrs);
		lst.add(btnAddCohortAttr);
		lst.add(btnCheckEvtOrder);
		lst.add(btnSortEvts);
		lst.add(btn7);
		
		return lst;
	}

}
