package org.processmining.processprofiler3d.psen.preprocess;

import java.util.Collections;
import java.util.List;
import java.util.Vector;

public class QuartilesClassifier
{
    public double min = 0;
    public double avg = 0;
    public double median = 0;
    public double max = 0;
    public double stdDev = 0;

    public double q0;
    public double q1;
    public double q2; //median
    public double q3;
    public double q4;
    public double iqr = 0;

	public QuartilesClassifier(Vector<Double> data)
	{
        if (data.size() > 0)
        {
        	Collections.sort(data);

            min = data.firstElement();
            avg = GetAverage(data);
            median = GetMedian(data);
            max = data.lastElement();
            stdDev = GetStandardDeviation(data, avg);

            List<Double> data1;
            List<Double> data2;
            if (data.size() == 1)
            {
                data1 = new Vector<Double>(data);
                data2 = new Vector<Double>(data);
            }
            else if (data.size() % 2 == 1)
            {
                data1 = data.subList(0, (data.size() / 2));
                data2 = data.subList(data.size() / 2, (data.size() / 2) + data.size() / 2);
            }
            else
            {
                double half = (double)data.size() / (double)2;
                data1 = data.subList(0, (int)Math.floor(half));
                data2 = data.subList((int)Math.ceil(half), (int)Math.ceil(half) + (int)Math.floor(half));
            }

            q1 = GetMedian(data1);
            q3 = GetMedian(data2);

            iqr = q3 - q1;
        }

        q0 = q1 - (iqr * 1.5);
        q2 = median;
        q4 = q3 + (iqr * 1.5);

        //find minimum/maximum non-outlier values
        double minNO = max;
        double maxNO = min;

        for (double d : data)
        {
            if (d < q0 || d > q4)
            {
                //outliers.Add(d);
            }
            else
            {
                minNO = Math.min(minNO, d);
                maxNO = Math.max(maxNO, d);
            }
        }

        q0 = Math.max(minNO, q0);
        q4 = Math.min(maxNO, q4);
	}

    protected double GetAverage(List<Double> lst)
    {
        if (lst.size() == 0)
        {
            return 0;
        }

        double total = 0;
        int count = 0;
        for (double d : lst)
        {
            total += d;
            count++;
        }

        return total / (double)count;
    }

    protected double GetMedian(List<Double> lst)
    {
        if (lst.size() == 0)
        {
            return 0;
        }

        int mod = lst.size() % 2;
        if (mod == 0)
        {
            int half = lst.size() / 2;
            int id1 = half - 1;
            int id2 = half;
            double ldiff = lst.get(id2) - lst.get(id1);

            return lst.get(id1) + (ldiff / 2);
        }
        else
        {
            int id = (int)(Math.floor((double)lst.size() * 0.5));
            return lst.get(id);
        }
    }

    protected double GetStandardDeviation(List<Double> lst, double avg)
    {
        double std = 0;
        if (lst.size() > 1)
        {
            //checked
            {
                double sum = 0;
                for (double ts : lst)
                {
                    sum += ((ts - avg) * (ts - avg));
                }

                std = Math.sqrt((double)sum / (double)(lst.size() - 1));
            }
        }

        return std;
    }

    public int ClassifyValue(double val)
    {
        if (val < q0)
        {
            return 0;
        }
        else if (val < q1)
        {
            return 1;
        }
        else if (val < q2)
        {
            return 2;
        }
        else if (val < q3)
        {
            return 3;
        }
        else if (val < q4)
        {
            return 4;
        }
        else
        {
            return 5;
        }
    }

}
