package org.processmining.processprofiler3d.psen.replayanalysis;

import java.util.Date;
import java.util.HashMap;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.Progress;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.psen.AlignedLog;
import org.processmining.processprofiler3d.psen.PetriNetIndexed;
import org.processmining.processprofiler3d.psen.preprocess.LPPAction;
import org.processmining.processprofiler3d.psen.replayanalysis.tokenflow.TokenFlowSimulation;

public class LogReplayAnalyser 
{
	private PetriNetDataSource petriNet;
	private Marking initMarking;
	private XLog rlog;
	private boolean bNetExpanded = false;

	public LogReplayAnalyser(final PetriNetDataSource newNet, boolean bNetExpanded, Marking initMarking, XLog log) 
	{
		this.rlog = log;
		this.petriNet = newNet;
		this.initMarking = initMarking;
		this.bNetExpanded = bNetExpanded;
	}

	public LogReplayAnalysisResults AnalyseLogReplay(final PluginContext context) 
	{	
		TraceReplayCollection trc = new TraceReplayCollection();
		TokenFlowSimulation rinitMarking = new TokenFlowSimulation(petriNet, initMarking);
		
		int brokenTraces = 0;
		int skippedTraces = 0;
		int totalTraces = 0;
		int totalTracesAnalysed = 0;
		
		System.out.println("Analysing traces...");
		
		//calculate total number of traces
		for (XTrace rtrc : rlog) 
		{
			ReplayResultWrapper2 rrw = new ReplayResultWrapper2(rtrc, null);
			// check if this result is reliable
			if (rrw.isReliable()) 
			{	
				totalTraces = totalTraces + 1;
				
			} // end of reliable sequence
			else
			{
				skippedTraces = skippedTraces + 1;

				XTrace tr = rtrc;
				String str = LPPAction.getAttributeValueLiteral(tr, XConceptExtension.KEY_NAME);
				System.out.println("Skipped analysis of " + str + ".");
			}
		}
		
		Progress prg = null;
		if (context != null)
		{
			prg = context.getProgress();
			//prg.setCaption("Analysing traces...");
			context.log("Analysing traces...");
			prg.setMaximum(100);
			prg.setValue(0);
		}
		Date startTime = new Date();
		
		PetriNetIndexed netIdx = new PetriNetIndexed(petriNet);
		
		HashMap<Long, ITraceAlignment> tamap = new HashMap<Long, ITraceAlignment>();
		
		//analyse traces
		for (XTrace rtrc : rlog) 
		{
			ReplayResultWrapper2 rrw = new ReplayResultWrapper2(rtrc, netIdx);
			// check if this result is reliable
			if (rrw.isReliable()) 
			{
				ITraceAlignment ta = null;
				
				Long groupId = LPPAction.getAttributeValueDiscrete(rtrc, AlignedLog.ATT_NAME_REPLAY_GROUP_ID);
				if (groupId != null)
				{
					if (tamap.containsKey(groupId))
					{
						ta = tamap.get(groupId);
					}
					else
					{
						if (bNetExpanded)
						{
							ta = new TraceAlignment(rinitMarking, rtrc, rrw);
						}
						else
						{
							ta = new TraceAlignmentOpt(rinitMarking, rtrc, rrw);
						}
						tamap.put(groupId, ta);
					}
					//tas.add(ta);
				}
				else
				{
					if (bNetExpanded)
					{
						ta = new TraceAlignment(rinitMarking, rtrc, rrw);
					}
					else
					{
						ta = new TraceAlignmentOpt(rinitMarking, rtrc, rrw);
					}		
				}
				
				//this is probably suboptimal!
				//for (int i : traceIndexes)
				{
					TraceReplayAnalyser tra = new TraceReplayAnalyser(ta, rtrc);
					
					trc.addTraceReplay(tra);				
					if (!tra.isValidTrace())
					{
						brokenTraces = brokenTraces + 1;
					}
					
					totalTracesAnalysed = totalTracesAnalysed + 1;
					int percAnalysed = (int)(((double)totalTracesAnalysed / (double)totalTraces) * 100.0);
					if (prg != null)
					{
						prg.setValue(percAnalysed);
					}
				}
				
				//totalTracesAnalysed = totalTracesAnalysed + traceIndexes.size();
				//int percAnalysed = (int)(((double)totalTracesAnalysed / (double)totalTraces) * 100.0);
				//prg.setValue(percAnalysed);
				//System.out.println("Analysed " + percAnalysed + "%");
				
			} // end of reliable sequence
		}
		
		Date endTime = new Date();
		
		System.out.println("Broken traces: " + brokenTraces);
		System.out.println("Skipped traces: " + skippedTraces);
		System.out.println("Total traces analysed: " + totalTracesAnalysed);
		
		TimeSpan ts = new TimeSpan(startTime, endTime);

		Vector<TimeUnit> tus = new Vector<TimeUnit>();
		tus.add(TimeUnit.MINUTES);
		tus.add(TimeUnit.SECONDS);
		tus.add(TimeUnit.MILLISECONDS);
		
		String timeStr = ts.ToString(tus);
		System.out.println("Time taken for analysis: " + timeStr);
		if (context != null)
		{
			context.log("Traces analysis complete (" + timeStr + ").");
		}
		
		LogReplayAnalysisResults lrar = new LogReplayAnalysisResults();
		lrar.trc = trc;
		lrar.totalTraces = totalTraces;
		lrar.brokenTraces = brokenTraces;
		lrar.skippedTraces = skippedTraces;
		lrar.totalTracesAnalysed = totalTracesAnalysed;
		
		return lrar;
		//return trc;
	}
}
