package org.processmining.processprofiler3d.psen.replayanalysis;

import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.SubNetDataSource;

public class MultiLevelCluster
{

	public MultiLevelCluster()
	{
		// TODO Auto-generated constructor stub
	}
	
	public static Comparator<TransitionEvent> TransitionEventStartTimeComparator 
		= new Comparator<TransitionEvent>() {

		public int compare(TransitionEvent te1, TransitionEvent te2) {
		
		//ascending order
		//return te1.getStartId() - te2.getStartId();
		
		//descending order
		return te2.getStartId() - te1.getStartId();
		}
	
	};

	//assuming one call at a time of subnet per trace, probably not correct!
	public Vector<TransitionEvent> Cluster(Transition t, Vector<TransitionEvent> evts, SubNetDataSource esn)
	{
		Vector<TransitionEvent> cluster = new Vector<TransitionEvent>();
		
		//sort events by start time (descending)
		//evts.sort(TransitionEventStartTimeComparator);
		
		//Vector<Vector<TransitionEvent>> clusters = getClusters(t, evts);
		Place ps = esn.getStartPlace();
		Place pe = esn.getEndPlace();
		
		int startTransitionsNum = esn.getOutEdges(ps).size();
		int endTransitionsNum = esn.getInEdges(pe).size();
		
		if (startTransitionsNum == 1 && endTransitionsNum == 1)
		{	
			Transition ts = (Transition)esn.getOutEdges(ps).iterator().next().getTarget();
			Transition tend = (Transition)esn.getInEdges(pe).iterator().next().getSource();
			
			Vector<TransitionEvent> startEvents = new Vector<TransitionEvent>();
			Vector<TransitionEvent> endEvents = new Vector<TransitionEvent>();
	
			for (TransitionEvent te : evts)
			{
				if (te.isTransition(ts))
				{
					startEvents.add(te);
				}
				if (te.isTransition(tend))
				{
					endEvents.add(te);
				}
			}
			
			if (startEvents.size() == 0)
			{
				cluster.addAll(evts);
				System.out.println("No start events for hierarchy! Clustering will be faulty!");
				return cluster;
			}
			if (endEvents.size() == 0)
			{
				cluster.addAll(evts);
				System.out.println("No end events for hierarchy! Clustering will be faulty!");
				return cluster;
			}
			
			if (startEvents.size() != endEvents.size())
			{
				System.out.println("Number of start events and end events for hierarchy does not match! Clustering will be faulty!");
			}
	
			//NEED TO SORT START AND END EVENTS!
			//startEvents.sort(TransitionEventStartTimeComparator); //Java 1.8
			//endEvents.sort(TransitionEventStartTimeComparator);	//Java 1.8
			Collections.sort(startEvents, TransitionEventStartTimeComparator);
			Collections.sort(endEvents, TransitionEventStartTimeComparator);
			
			int j = 0;
			int numEventsClustered = 0;
			for (TransitionEvent tes : startEvents)
			{
				TransitionEvent first = tes;
				TransitionEvent last = endEvents.get(j);
				
				TransitionEvent nEvt = new TransitionEvent(t);
				nEvt.setScheduleEvent(first.getScheduleId());
				nEvt.setAssignEvent(first.getAssignId());
				nEvt.setStartEvent(first.getStartId());
				nEvt.setCompleteEvent(last.getCompleteId());
				
				for (TransitionEvent te : evts)
				{
					if (te.getStartId() >= first.getStartId() && te.getCompleteId() <= last.getCompleteId())
					{
						te.levelUp = nEvt;
						numEventsClustered++;
					}
				}
				//add parent events from next level (as they should be the same)
				for (TransitionEvent tx : tes.getParents())
				{
					nEvt.addParent(tx);
				}
				cluster.add(nEvt);
				
				j++;
			}
			
			if (numEventsClustered != evts.size())
			{
				System.out.println("Some subnet events have not been clustered correctly!");
			}
			
			return cluster;
		}
		else
		{
			System.out.println("Number of start or end transitions for subnet is not one! Clustering will be faulty!");

			Vector<TransitionEvent> startEvents = new Vector<TransitionEvent>();
			for (Arc a : esn.getOutEdges(ps))
			{
				Transition ts = (Transition)a.getTarget();
				for (TransitionEvent te : evts)
				{
					if (te.isTransition(ts))
					{
						startEvents.add(te);
					}
				}
				
			}
			
			Vector<TransitionEvent> endEvents = new Vector<TransitionEvent>();
			for (Arc a : esn.getInEdges(pe))
			{
				Transition tend = (Transition)a.getSource();		
	
				for (TransitionEvent te : evts)
				{
					if (te.isTransition(tend))
					{
						endEvents.add(te);
					}
				}
			}
			
			if (startEvents.isEmpty() || endEvents.isEmpty())
			{
				System.out.println("Error when clustering subnet events!");
				return cluster;
			}

			//NEED TO SORT START AND END EVENTS!
			//startEvents.sort(TransitionEventStartTimeComparator); //Java 1.8
			//endEvents.sort(TransitionEventStartTimeComparator);	//Java 1.8
			Collections.sort(startEvents, TransitionEventStartTimeComparator);
			Collections.sort(endEvents, TransitionEventStartTimeComparator);
			
			TransitionEvent first = startEvents.firstElement();
			TransitionEvent last = endEvents.lastElement();
			
			TransitionEvent nEvt = new TransitionEvent(t);
			nEvt.setScheduleEvent(first.getScheduleId());
			nEvt.setAssignEvent(first.getAssignId());
			nEvt.setStartEvent(first.getStartId());
			nEvt.setCompleteEvent(last.getCompleteId());
			
			for (TransitionEvent te : evts)
			{
				if (te.getStartId() >= first.getStartId() && te.getCompleteId() <= last.getCompleteId())
				{
					te.levelUp = nEvt;
				}
			}
			cluster.add(nEvt);
			
			return cluster;
		}
	}

}
