package org.processmining.processprofiler3d.psen.replayanalysis;

import java.util.HashMap;
import java.util.Map;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.processprofiler3d.psen.AlignedLog;
import org.processmining.processprofiler3d.psen.PetriNetIndexed;
import org.processmining.processprofiler3d.psen.preprocess.LPPAction;

public class ReplayResultWrapper2 implements IReplayResultWrapper 
{
	private XTrace syncRepRes;
	private PetriNetIndexed net;
	
	public ReplayResultWrapper2(XTrace syncRepRes, PetriNetIndexed net)
	{
		this.syncRepRes = syncRepRes;
		this.net = net;
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.plugins.psen.replayanalysis.IReplayResultWrapper#isReliable()
	 */
	@Override
	public boolean isReliable()
	{
		String val = LPPAction.getAttributeValueLiteral(syncRepRes, AlignedLog.ATT_NAME_IS_RELIABLE);
		if (val != null)
		{
			return val.toLowerCase().equals("true");
		}
		return false;
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.plugins.psen.replayanalysis.IReplayResultWrapper#getInfo()
	 */
	@Override
	public Map<String, Double> getInfo()
	{
		return new HashMap<String, Double>();
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.plugins.psen.replayanalysis.IReplayResultWrapper#getStepCount()
	 */
	@Override
	public int getStepCount()
	{
		return syncRepRes.size();
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.plugins.psen.replayanalysis.IReplayResultWrapper#getStepType(int)
	 */
	@Override
	public StepTypes getStepType(int i)
	{
		//return syncRepRes.getStepTypes().get(i);
		XEvent evt = syncRepRes.get(i);
		StepTypes st = AlignedLog.getStepType(evt);
		if (st == null)
		{
			return StepTypes.LMNOGOOD;
		}
		return st;
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.plugins.psen.replayanalysis.IReplayResultWrapper#getNodeInstance(int)
	 */
	@Override
	public Object getNodeInstance(int i)
	{
		//return syncRepRes.getNodeInstance().get(i);
		XEvent evt = syncRepRes.get(i);
		String val = AlignedLog.getLocalNodeId(evt);
		
		Transition t = net.getTransition(val);
		if (t == null)
		{
			String trcName = XConceptExtension.instance().extractName(syncRepRes);
			String evtName = XConceptExtension.instance().extractName(evt);
			String evtLifecycle = XLifecycleExtension.instance().extractTransition(evt);
			String evtInstance = "-";
			if (evt.getAttributes().get("concept:instance") instanceof XAttributeLiteral)
			{
				evtInstance = XConceptExtension.instance().extractInstance(evt);
			}
			String evtStepType = AlignedLog.getStepType(evt).toString();
			String evtInfo = "Trace - Name: " + trcName + " Event - Name: " + evtName + " Lifecycle: " + evtLifecycle + " Instance: " + evtInstance + " StepType: " + evtStepType;
			System.out.println("Could not find transition for LocalNodeID: " + val + " (" + evtInfo + ")");
		}
		
		return t;
	}
}
