package org.processmining.processprofiler3d.psen.replayanalysis;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension.StandardModel;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryNaiveImpl;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeBooleanImpl;
import org.deckfour.xes.model.impl.XAttributeDiscreteImpl;
import org.processmining.log.utils.XLogBuilder;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.psen.AlignedLog;
import org.processmining.processprofiler3d.psen.PetriNet1toManyMapping;
import org.processmining.processprofiler3d.psen.preprocess.LPPAction;
import org.processmining.processprofiler3d.psen.replayanalysis.tokenflow.TokenFlowSimulation;

public class ReplayToLogConverter 
{
	private PetrinetGraph petriNet;
	private Marking initMarking;
	private XLog log;
	private Set<SyncReplayResult> logReplayResult;
	
	public ReplayToLogConverter(final PetrinetGraph newNet, Marking initMarking, XLog log, Set<SyncReplayResult> logReplayResult)
	{
		this.log = log;
		this.petriNet = newNet;
		this.initMarking = initMarking;
		this.logReplayResult = logReplayResult;
	}
	
	public XLog ConvertLogReplay() 
	{
		TokenFlowSimulation rinitMarking = new TokenFlowSimulation(new PetriNetDataSource(petriNet), initMarking);

		XLogBuilder newLog = XLogBuilder.newInstance();
		String oldLogName = XConceptExtension.instance().extractName(log);
		newLog.startLog(oldLogName + " with Alignment");
		
		System.out.println("Analysing traces...");
		
		//calculate total number of traces
		int index = 0;
		
		//analyse traces
		for (SyncReplayResult syncRepRes : logReplayResult) 
		{
			// check if this result is reliable
			//if (syncRepRes.isReliable()) 
			{
				// filter out traces/keep reliable traces
				Set<Integer> traceIndexes = new HashSet<Integer>(syncRepRes.getTraceIndex());

				short traceSize = (short)traceIndexes.size();
				if (traceSize == 0) 
				{
					continue;
				}

				TraceAlignment ta = new TraceAlignment(rinitMarking, log.get(traceIndexes.iterator().next()), new ReplayResultWrapper(syncRepRes));
				
				//this is probably suboptimal!
				for (int i : traceIndexes)
				{
					XTrace trace = log.get(i);
					String traceName = XConceptExtension.instance().extractName(trace);
					
					newLog.addTrace(traceName);
					//add trace attribs
					newLog.addAttribute(AlignedLog.ATT_NAME_REPLAY_GROUP_ID, index);
					newLog.addAttribute(AlignedLog.ATT_NAME_IS_BROKEN, "" + (!ta.isValidTrace()));
					newLog.addAttribute(AlignedLog.ATT_NAME_IS_PERFECT_FIT, "" + ta.isPerfectFit());
					newLog.addAttribute(AlignedLog.ATT_NAME_IS_RELIABLE, "" + ta.isReliable());
					
					Map<String, Double> replayInfo = ta.getReplayInfo();
					for (String skey : replayInfo.keySet())
					{
						newLog.addAttribute(skey, replayInfo.get(skey));
					}
					
					//copy all attributes
					CopyTraceAttributes(newLog, trace);
					
					TraceAlignmentReplayStateBase tars = ta.getReplayState(trace);
					tars.Init();
					TraceReplayStepData trsd = new TraceReplayStepData();
					while (tars.NextStep(trsd))
					{
						CreateLogEvent(newLog, trsd);
					}
					CreateLogEvent(newLog, trsd);
				}
				
			} // end of reliable sequence

			index++;
		}
		
		XLog convertedLog = newLog.build();
				
		LPPAction.CopyClassifiers(log, convertedLog);	
		LPPAction.CopyGlobalTraceAttributes(log, convertedLog);		
		LPPAction.CopyGlobalEventAttributes(log, convertedLog);	
		LPPAction.CopyExtensions(log, convertedLog);
		
		SupplementGlobalTraceAttributes(convertedLog);

		//fixup data issues with invisible steps!!! HACK!
		fixupInvisibleEvents(convertedLog);
		
		return convertedLog;
	}
	
	public XLog ConvertLogReplay(boolean[] filter, Set<Integer> preservedIndexes) 
	{
		TokenFlowSimulation rinitMarking = new TokenFlowSimulation(new PetriNetDataSource(petriNet), initMarking);

		XLogBuilder newLog = XLogBuilder.newInstance();
		String oldLogName = XConceptExtension.instance().extractName(log);
		newLog.startLog(oldLogName + " with Alignment");
		
		System.out.println("Analysing traces...");
		
		//calculate total number of traces
		int index = 0;
		
		//analyse traces
		for (SyncReplayResult syncRepRes : logReplayResult) 
		{
			// check if this is filtered out
			// Erik: Skip to first not filtered out result?
			if (filter != null) 
			{
				if (!filter[index]) 
				{
					index++;
					continue;
				}
			}
			// check if this result is reliable
			//if (syncRepRes.isReliable()) 
			{
				// filter out traces/keep reliable traces
				Set<Integer> traceIndexes = new HashSet<Integer>(syncRepRes.getTraceIndex());
				if (preservedIndexes != null) 
				{
					traceIndexes.retainAll(preservedIndexes);
				}

				short traceSize = (short)traceIndexes.size();
				if (traceSize == 0) 
				{
					continue;
				}

				TraceAlignment ta = new TraceAlignment(rinitMarking, log.get(traceIndexes.iterator().next()), new ReplayResultWrapper(syncRepRes));
				
				//this is probably suboptimal!
				for (int i : traceIndexes)
				{
					XTrace trace = log.get(i);
					String traceName = XConceptExtension.instance().extractName(trace);
					
					newLog.addTrace(traceName);
					//add trace attribs
					newLog.addAttribute(AlignedLog.ATT_NAME_REPLAY_GROUP_ID, index);
					newLog.addAttribute(AlignedLog.ATT_NAME_IS_BROKEN, "" + (!ta.isValidTrace()));
					newLog.addAttribute(AlignedLog.ATT_NAME_IS_PERFECT_FIT, "" + ta.isPerfectFit());
					newLog.addAttribute(AlignedLog.ATT_NAME_IS_RELIABLE, "" + ta.isReliable());
					
					Map<String, Double> replayInfo = ta.getReplayInfo();
					for (String skey : replayInfo.keySet())
					{
						newLog.addAttribute(skey, replayInfo.get(skey));
					}
					
					//copy all attributes
					CopyTraceAttributes(newLog, trace);
					
					TraceAlignmentReplayStateBase tars = ta.getReplayState(trace);
					tars.Init();
					TraceReplayStepData trsd = new TraceReplayStepData();
					while (tars.NextStep(trsd))
					{
						CreateLogEvent(newLog, trsd);
					}
					CreateLogEvent(newLog, trsd);
				}
				
			} // end of reliable sequence

			index++;
		}
		
		XLog convertedLog = newLog.build();
		
		LPPAction.CopyClassifiers(log, convertedLog);	
		LPPAction.CopyGlobalTraceAttributes(log, convertedLog);		
		LPPAction.CopyGlobalEventAttributes(log, convertedLog);	
		LPPAction.CopyExtensions(log, convertedLog);
		
		SupplementGlobalTraceAttributes(convertedLog);
		
		//fixup data issues with invisible steps!!! HACK!
		fixupInvisibleEvents(convertedLog);

		return convertedLog;
	}

	private void fixupInvisibleEvents(XLog nlog)
	{
		for (XTrace trace : nlog)
		{
			for (XEvent event : trace)
			{
				StepTypes stepType = AlignedLog.getStepType(event);
				if (stepType == StepTypes.MINVI)
				{
					XLifecycleExtension.instance().assignStandardTransition(event, XLifecycleExtension.StandardModel.COMPLETE);
				}
			}
		}
	}

	public void CreateLogEvent(XLogBuilder newLog, TraceReplayStepData trsd) 
	{
		if (trsd.type == null)
		{
			return;
		}
		
		XEvent event = trsd.lastEvent;
		String eventName = XConceptExtension.instance().extractName(event);
		if (trsd.trans != null)
		{
			eventName = trsd.trans.getLabel();
		}
		newLog.addEvent(eventName);
		newLog.addAttribute(AlignedLog.ATT_NAME_STEP_TYPE, trsd.type.toString());
		if (trsd.trans != null)
		{
			newLog.addAttribute(AlignedLog.ATT_NAME_NODE_ID, trsd.trans.getLocalID().toString());
		}

		if (trsd.type == StepTypes.L || trsd.type == StepTypes.LMGOOD)
		{
			//copy all attributes
			CopyEventAttributes(newLog, event);
		}
		else
		{
			//copy all attributes
			for (XAttribute att : log.getGlobalEventAttributes())
			{
				if (att instanceof XAttributeLiteral)
				{
					if (att.getKey().equals(XConceptExtension.KEY_NAME))
					{ 
						//skip event name
					}
					else if (att.getKey().equals(XLifecycleExtension.KEY_TRANSITION))
					{
						//preserve test behaviour for now, this might still be wrong however!
						XAttributeLiteral attl = (XAttributeLiteral)att;
						newLog.addAttribute(attl.getKey(), XLifecycleExtension.instance().extractTransition(event));		
					}
					else
					{
						XAttributeLiteral attl = (XAttributeLiteral)att;
						newLog.addAttribute(attl.getKey(), attl.getValue());
					}	
				}
				else if (att instanceof XAttributeTimestamp)
				{
					if (att.getKey().equals(XTimeExtension.KEY_TIMESTAMP)) //use last available timestamp
					{
						XAttributeTimestamp attl = (XAttributeTimestamp)att;
						newLog.addAttribute(attl.getKey(), XTimeExtension.instance().extractTimestamp(event));
					}
					else
					{
						XAttributeTimestamp attts = (XAttributeTimestamp)att;
						newLog.addAttribute(attts.getKey(), attts.getValue());
					}
				}
				else if (att instanceof XAttributeBoolean)
				{
					XAttributeBoolean attts = (XAttributeBoolean)att;
					newLog.addAttribute(attts.getKey(), attts.getValue());
				}
				else if (att instanceof XAttributeDiscrete)
				{
					XAttributeDiscrete attts = (XAttributeDiscrete)att;
					newLog.addAttribute(attts.getKey(), attts.getValue());
				}
				else if (att instanceof XAttributeContinuous)
				{
					XAttributeContinuous attts = (XAttributeContinuous)att;
					newLog.addAttribute(attts.getKey(), attts.getValue());
				}
			}
		}
	}
	
	public static void SupplementGlobalTraceAttributes(XLog convertedLog)
	{
		HashSet<String> tattrs = new HashSet<String>();
		for (XAttribute tattr : convertedLog.getGlobalTraceAttributes())
		{
			tattrs.add(tattr.getKey());
		}
		
		if (!tattrs.contains(AlignedLog.ATT_NAME_IS_BROKEN))
		{
			convertedLog.getGlobalTraceAttributes().add(new XAttributeBooleanImpl(AlignedLog.ATT_NAME_IS_BROKEN, false));
		}
		if (!tattrs.contains(AlignedLog.ATT_NAME_IS_PERFECT_FIT))
		{
			convertedLog.getGlobalTraceAttributes().add(new XAttributeBooleanImpl(AlignedLog.ATT_NAME_IS_PERFECT_FIT, true));
		}
		if (!tattrs.contains(AlignedLog.ATT_NAME_IS_RELIABLE))
		{
			convertedLog.getGlobalTraceAttributes().add(new XAttributeBooleanImpl(AlignedLog.ATT_NAME_IS_RELIABLE, true));
		}
		if (!tattrs.contains(AlignedLog.ATT_NAME_REPLAY_GROUP_ID))
		{
			convertedLog.getGlobalTraceAttributes().add(new XAttributeDiscreteImpl(AlignedLog.ATT_NAME_REPLAY_GROUP_ID, 0));
		}
	}

	public static void CopyTraceAttributes(XLogBuilder newLog, XTrace trace) 
	{
		for (XAttribute att : trace.getAttributes().values())
		{
			if (att instanceof XAttributeLiteral)
			{
				XAttributeLiteral attl = (XAttributeLiteral)att;
				newLog.addAttribute(attl.getKey(), attl.getValue());
			}
			else if (att instanceof XAttributeTimestamp)
			{
				XAttributeTimestamp attts = (XAttributeTimestamp)att;
				newLog.addAttribute(attts.getKey(), attts.getValue());
			}
			else if (att instanceof XAttributeBoolean)
			{
				XAttributeBoolean attts = (XAttributeBoolean)att;
				newLog.addAttribute(attts.getKey(), attts.getValue());
			}
			else if (att instanceof XAttributeDiscrete)
			{
				XAttributeDiscrete attts = (XAttributeDiscrete)att;
				newLog.addAttribute(attts.getKey(), attts.getValue());
			}
			else if (att instanceof XAttributeContinuous)
			{
				XAttributeContinuous attts = (XAttributeContinuous)att;
				newLog.addAttribute(attts.getKey(), attts.getValue());
			}
		}
	}

	public static void CopyEventAttributes(XLogBuilder newLog, XEvent event) 
	{
		for (XAttribute att : event.getAttributes().values())
		{
			if (att instanceof XAttributeLiteral)
			{
				if (!att.getKey().equals(XConceptExtension.KEY_NAME)) //skip event name
				{
					XAttributeLiteral attl = (XAttributeLiteral)att;
					newLog.addAttribute(attl.getKey(), attl.getValue());
				}
			}
			else if (att instanceof XAttributeTimestamp)
			{
				XAttributeTimestamp attts = (XAttributeTimestamp)att;
				newLog.addAttribute(attts.getKey(), attts.getValue());
			}
			else if (att instanceof XAttributeBoolean)
			{
				XAttributeBoolean attts = (XAttributeBoolean)att;
				newLog.addAttribute(attts.getKey(), attts.getValue());
			}
			else if (att instanceof XAttributeDiscrete)
			{
				XAttributeDiscrete attts = (XAttributeDiscrete)att;
				newLog.addAttribute(attts.getKey(), attts.getValue());
			}
			else if (att instanceof XAttributeContinuous)
			{
				XAttributeContinuous attts = (XAttributeContinuous)att;
				newLog.addAttribute(attts.getKey(), attts.getValue());
			}
		}
	}
	
	public static XLog Project(XLog log, PetriNet1toManyMapping map2)
	{
		XLogBuilder newLog = XLogBuilder.newInstance();
		newLog.startLog("XLog new Test");
		
		for (XTrace trace : log)
		{
			String traceName = XConceptExtension.instance().extractName(trace);
			
			newLog.addTrace(traceName);
			
			//copy all attributes
			CopyTraceAttributes(newLog, trace);
			
			for (XEvent event : trace)
			{
				String eventName = XConceptExtension.instance().extractName(event);
				String localNodeID = AlignedLog.getLocalNodeId(event);
				Vector<PetrinetNode> pnsView = null;
				if (localNodeID != null)
				{
					PetrinetNode pnOrig = getOrigNode(map2, localNodeID);
					if (map2.hasOrigNode(pnOrig))
					{
						pnsView = map2.mapToView(pnOrig);
					}
					//eventName = trsd.trans.getLabel();
				}
				if (pnsView == null)
				{
					newLog.addEvent(eventName);

					//copy all attributes
					CopyEventAttributes(newLog, event);
				}
				else
				{
					StandardModel lifecycleName = XLifecycleExtension.instance().extractStandardTransition(event);
					
					PetrinetNode pnView = pnsView.get(0);
					if (lifecycleName == StandardModel.SCHEDULE)
					{
						pnView = pnsView.get(0);
					}
					else if (lifecycleName == StandardModel.ASSIGN)
					{
						pnView = pnsView.get(1);
					}
					else if (lifecycleName == StandardModel.START)
					{
						pnView = pnsView.get(2);
					}
					else if (lifecycleName == StandardModel.COMPLETE)
					{
						pnView = pnsView.get(3);
					}
					
					newLog.addEvent(pnView.getLabel());

					//copy all attributes
					CopyEventAttributes(newLog, event);
					
					//override aligned local node id
					newLog.addAttribute(AlignedLog.ATT_NAME_NODE_ID, pnView.getLocalID().toString());
					//override lifecycle value
					newLog.addAttribute(XLifecycleExtension.KEY_TRANSITION, "complete");
				}
			}
		}
		
		XLog pconvertedLog = newLog.build();
		
		LPPAction.CopyClassifiers(log, pconvertedLog);	
		LPPAction.CopyGlobalTraceAttributes(log, pconvertedLog);		
		LPPAction.CopyGlobalEventAttributes(log, pconvertedLog);	
		LPPAction.CopyExtensions(log, pconvertedLog);
		
		SupplementGlobalTraceAttributes(pconvertedLog);
		
		return pconvertedLog;
	}
	
	public static XLog ProjectBack(XLog log, PetriNet1toManyMapping map2)
	{
		XLogBuilder newLog = XLogBuilder.newInstance();
		newLog.startLog("XLog new Test");
		
		for (XTrace trace : log)
		{
			String traceName = XConceptExtension.instance().extractName(trace);
			
			newLog.addTrace(traceName);
			
			//copy all attributes
			CopyTraceAttributes(newLog, trace);
			
			for (XEvent event : trace)
			{
				String eventName = XConceptExtension.instance().extractName(event);
				String localNodeID = AlignedLog.getLocalNodeId(event);
				PetrinetNode pnOrig = null;
				if (localNodeID != null)
				{
					PetrinetNode pnView = getViewNode(map2, localNodeID);
					if (map2.hasViewNode(pnView))
					{
						pnOrig = map2.mapToOrig(pnView);
					}
					//eventName = trsd.trans.getLabel();
				}
				if (pnOrig == null)
				{
					newLog.addEvent(eventName);

					//copy all attributes
					CopyEventAttributes(newLog, event);
				}
				else
				{
					newLog.addEvent(pnOrig.getLabel());

					//copy all attributes
					CopyEventAttributes(newLog, event);
					
					//override aligned local node id
					newLog.addAttribute(AlignedLog.ATT_NAME_NODE_ID, pnOrig.getLocalID().toString());
					//override lifecycle value
					if (eventName.endsWith("+schedule"))
					{
						newLog.addAttribute(XLifecycleExtension.KEY_TRANSITION, "schedule");
					}
					else if (eventName.endsWith("+assign"))
					{
						newLog.addAttribute(XLifecycleExtension.KEY_TRANSITION, "assign");
					}
					else if (eventName.endsWith("+start"))
					{
						newLog.addAttribute(XLifecycleExtension.KEY_TRANSITION, "start");
					}
					else if (eventName.endsWith("+complete"))
					{
						newLog.addAttribute(XLifecycleExtension.KEY_TRANSITION, "complete");
					}
				}
			}
		}
		
		XLog pconvertedLog = newLog.build();
		
		LPPAction.CopyClassifiers(log, pconvertedLog);	
		LPPAction.CopyGlobalTraceAttributes(log, pconvertedLog);		
		LPPAction.CopyGlobalEventAttributes(log, pconvertedLog);	
		LPPAction.CopyExtensions(log, pconvertedLog);
		
		SupplementGlobalTraceAttributes(pconvertedLog);
		
		return pconvertedLog;
	}
	
	private static PetrinetNode getOrigNode(PetriNet1toManyMapping map2, String localID)
	{
		for (PetrinetNode pn : map2.getOrigValues())
		{
			if (pn.getLocalID().toString().equals(localID))
			{
				return pn;
			}
		}
		return null;
	}
	
	private static PetrinetNode getViewNode(PetriNet1toManyMapping map2, String localID)
	{
		for (PetrinetNode pn : map2.getViewValues())
		{
			if (pn.getLocalID().toString().equals(localID))
			{
				return pn;
			}
		}
		return null;
	}
	
	public static void fixupLog(XLog log, TransEvClassMapping tmap, PetriNetDataSource pnds)
	{
		XFactory xfactor = new XFactoryNaiveImpl();
		HashMap<String, String> classToLocalNodeID = new HashMap<String, String>();
		
		for (XTrace trace : log)
		{
			for (XEvent event : trace)
			{
				String att = AlignedLog.getLocalNodeId(event);
				if (att == null)
				{
					//if has no AlignedLocalNodeID try to find a mapping
					String tname = tmap.getEventClassifier().getClassIdentity(event);
					if (classToLocalNodeID.containsKey(tname))
					{
						String localID = classToLocalNodeID.get(tname);
						setLocalNodeId(xfactor, event, localID);
					}
					else
					{
						for (Transition t : pnds.getAllTransitions())
						{
							if (t.getLabel().equals(tname))
							{
								String localID = t.getLocalID().toString();
								setLocalNodeId(xfactor, event, localID);

								classToLocalNodeID.put(tname, localID);
								break;
							}
						}
					}
				}
			}
		}
	}
	
	public static void fixupLog(XLog log, TransEvClassMapping tmap, PetriNet1toManyMapping pmap, PetriNetDataSource pnds)
	{
		XFactory xfactor = new XFactoryNaiveImpl();
		HashMap<String, String> classToLocalNodeID = new HashMap<String, String>();
		
		for (XTrace trace : log)
		{
			for (XEvent event : trace)
			{
				String att = AlignedLog.getLocalNodeId(event);
				if (att == null)
				{
					//if has no AlignedLocalNodeID try to find a mapping
					String tname = tmap.getEventClassifier().getClassIdentity(event);
					if (classToLocalNodeID.containsKey(tname))
					{
						String localID = classToLocalNodeID.get(tname);
						setLocalNodeId(xfactor, event, localID);
					}
					else
					{
						for (Transition t : pnds.getAllTransitions())
						{
							if (t.getLabel().equals(tname))
							{
								if (pmap.hasViewNode(t))
								{
									Transition tx = (Transition)pmap.mapToOrig(t);
									String localID = tx.getLocalID().toString();
									setLocalNodeId(xfactor, event, localID);
									
									classToLocalNodeID.put(tname, localID);
									break;
								}
							}
						}
					}
				}
			}
		}
	}
	
	private static void setLocalNodeId(XFactory xfactor, XEvent event, String localID)
	{
		event.getAttributes().put(AlignedLog.ATT_NAME_NODE_ID, xfactor.createAttributeLiteral(AlignedLog.ATT_NAME_NODE_ID, localID, null));
	}
}
