package org.processmining.processprofiler3d.psen.replayanalysis;

import java.util.Collection;
import java.util.Date;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

public class TimeSpan 
{
	private long timeSpan = 0; //in milliseconds
	
	public long getMilliseconds()
	{
		return timeSpan;
	}
	
	public TimeSpan(long ts)
	{
		timeSpan = ts;
	}
	
	public TimeSpan(Date d1, Date d2)
	{
	    long diffInMillies = d2.getTime() - d1.getTime();
		timeSpan = diffInMillies;
	}
	
	public TimeSpan(long d1, long d2)
	{
	    long diffInMillies = d2 - d1;
		timeSpan = diffInMillies;
	}
	
	public static TimeSpan getMean(Collection<TimeSpan> vts)
	{
		long total = 0;
		if (vts.size() > 0)
		{
			for (TimeSpan ts : vts)
			{
				total = total + ts.getMilliseconds();
			}
			total = (long)((double)total / (double)vts.size());
			return new TimeSpan(total);
		}
		return null;
	}
	
	public static TimeSpan getMin(Collection<TimeSpan> vts)
	{
		long min = Long.MAX_VALUE;
		if (vts.size() > 0)
		{
			for (TimeSpan ts : vts)
			{
				min = Math.min(min, ts.getMilliseconds());
			}
			return new TimeSpan(min);
		}
		return null;
	}
	
	public static TimeSpan getMax(Collection<TimeSpan> vts)
	{
		long max = Long.MIN_VALUE;
		if (vts.size() > 0)
		{
			for (TimeSpan ts : vts)
			{
				max = Math.max(max, ts.getMilliseconds());
			}
			return new TimeSpan(max);
		}
		return null;
	}
	
	public static boolean containsNulls(Collection<TimeSpan> vts)
	{
		if (vts.size() > 0)
		{
			for (TimeSpan ts : vts)
			{
				if (ts == null)
					return true;
			}
		}
		return false;
	}
	
	public String ToString(TimeUnit tu)
	{
		return "" + tu.convert(timeSpan, TimeUnit.MILLISECONDS);
	}
	
	public String ToString(Vector<TimeUnit> tus)
	{
		Vector<Number> vals = new Vector<Number>();
		
		if (timeSpan == Long.MIN_VALUE)
			return "Long.MIN_VALUE";
		if (timeSpan == Long.MAX_VALUE)
			return "Long.MAX_VALUE";
		
		long span = Math.abs(timeSpan);

		for (TimeUnit tu : tus)
		{
			long val = tu.convert(span, TimeUnit.MILLISECONDS);
			vals.add(val);
			
			span = span - TimeUnit.MILLISECONDS.convert(val, tu);
		}
		
		String s = "";
		int i = 0;
		for (Number n : vals)
		{
			String formatted = "";
			if (i == 0)
			{
				formatted = "" + n.longValue();		
			}
			else if (tus.get(i) == TimeUnit.MILLISECONDS)
			{
				formatted = String.format("%03d", n.longValue());
			}
			else
			{
				formatted = String.format("%02d", n.longValue());
			}
			
			if (i == 0)
			{
				s = s + formatted;
			}
			else
			{
				s = s + ":" + formatted;
			}
			
			i++;
		}
		
		if (timeSpan < 0)
		{
			return "-" + s;
		}
		return s;
	}
	
	public String ToStringExp(Vector<TimeUnit> tus, int maxCats)
	{
		Vector<Number> vals = new Vector<Number>();
		
		if (timeSpan == Long.MIN_VALUE)
			return "Long.MIN_VALUE";
		if (timeSpan == Long.MAX_VALUE)
			return "Long.MAX_VALUE";
		
		long span = Math.abs(timeSpan);

		for (TimeUnit tu : tus)
		{
			long val = tu.convert(span, TimeUnit.MILLISECONDS);
			vals.add(val);
			
			span = span - TimeUnit.MILLISECONDS.convert(val, tu);
		}
		
		//find earliest value that has none 0
		int start = 0;
		int j = 0;
		for (Number n : vals)
		{
			if (n.doubleValue() == 0 && j == start && j < (vals.size() - 1))
			{
				start++;
			}
			j++;
		}
		
		//find last value that has none 0
		int end = vals.size() - 1;
		for (int jx = vals.size() - 1; jx > start; jx--)
		{
			Number n = vals.get(jx);
			if (n.doubleValue() == 0 && jx == end)
			{
				end--;
			}
		}
		
		//only print the [maxCats] largest values
		while (end - start > (maxCats - 1))
		{
			end--;
		}
		
		String s = "";
		int i = 0;
		for (Number n : vals)
		{
			if (i < start || i > end)
			{
				i++;
				continue;
			}
			
			String formatted = "";
			if (i == start)
			{
				formatted = "" + n.longValue();		
			}
			else if (tus.get(i) == TimeUnit.MILLISECONDS)
			{
				formatted = String.format("%03d", n.longValue());
			}
			else
			{
				formatted = String.format("%02d", n.longValue());
			}
			
			if (i == start)
			{
				s = s + formatted + getUnitShort(tus.get(i));
			}
			else
			{
				s = s + " " + formatted + getUnitShort(tus.get(i));
			}
			
			i++;
		}
		
		if (timeSpan < 0)
		{
			return "-" + s;
		}
		return s;
	}
	
	public String getUnitShort(TimeUnit tu)
	{
		if (tu == TimeUnit.MILLISECONDS)
		{
			return "ms";
		}
		if (tu == TimeUnit.SECONDS)
		{
			return "s";
		}
		if (tu == TimeUnit.MINUTES)
		{
			return "min";
		}
		if (tu == TimeUnit.HOURS)
		{
			return "h";
		}
		if (tu == TimeUnit.DAYS)
		{
			return "d";
		}
		
		return "";
	}
	
}
