package org.processmining.processprofiler3d.psen.replayanalysis;

import java.util.Map;
import java.util.Vector;

import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.SubNetDataSource;
import org.processmining.processprofiler3d.psen.replayanalysis.tokenflow.TokenFlowResults;
import org.processmining.processprofiler3d.psen.replayanalysis.tokenflow.TokenFlowSimulation;

public class TraceAlignment implements ITraceAlignment 
{
	private TokenFlowSimulation tokenFlowSim;
	private IReplayResultWrapper traceReplayResult;
	private TokenFlowResults flowResults = null;
	
	private boolean isPerfectFitTrace = false;
	private boolean isBrokenTrace = false;
	
	/* (non-Javadoc)
	 * @see org.processmining.plugins.psen.replayanalysis.ITraceAlignment#isValidTrace()
	 */
	@Override
	public boolean isValidTrace()
	{
		return !isBrokenTrace;
	}
	
	public boolean isPerfectFit()
	{
		return isPerfectFitTrace;
	}
	
	public boolean isReliable()
	{
		return traceReplayResult.isReliable();
	}
	
	public Map<String, Double> getReplayInfo()
	{
		return traceReplayResult.getInfo();
	}
	
	@SuppressWarnings("unused")
	private TraceAlignment() 
	{
	};

	public TraceAlignment(TokenFlowSimulation flowSim, XTrace trc, IReplayResultWrapper traceReplayResult) 
	{
		this.tokenFlowSim = flowSim;
		
		this.traceReplayResult = traceReplayResult;
		
		if (trc.size() > 0)
		{
			TraceAlignmentReplayStateBase tars = getReplayState(trc);
			tars.Init();
			while (tars.NextStep(null))
			{
				
			}
			isPerfectFitTrace = tars.isPerfectFitTrace;
			isBrokenTrace = tars.isBrokenTrace;
			flowResults = tars.getTransitionEvents();
		}
	}
	
	public TraceAlignmentReplayStateBase getReplayState(XTrace trc)
	{
		return new TraceAlignmentReplayState(this, tokenFlowSim, trc, traceReplayResult);
	}

	@Override
	public void addSubnetEvents(INetDataSource source, Vector<SubNetDataSource> subnets) 
	{
		flowResults.addSubnetEvents(source, subnets);
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.plugins.psen.replayanalysis.ITraceAlignment#getTransitionEventsFor(org.processmining.models.graphbased.directed.petrinet.elements.Transition)
	 */
	@Override
	public Vector<TransitionEvent> getTransitionEventsFor(Transition t)
	{
		return flowResults.getTransitionEventsFor(t);
	}
	
	/* (non-Javadoc)
	 * @see org.processmining.plugins.psen.replayanalysis.ITraceAlignment#getAllTransitionEvents()
	 */
	@Override
	public Vector<TransitionEvent> getAllTransitionEvents()
	{
		return flowResults.getAllTransitionEvents();
	}

	@Override
	public TokenFlowResults getFlowResults()
	{
		return flowResults;
	}
}
