package org.processmining.processprofiler3d.psen.replayanalysis;

import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.SubNetDataSource;
import org.processmining.processprofiler3d.psen.PetriNet1toManyMapping;
import org.processmining.processprofiler3d.psen.replayanalysis.tokenflow.TokenFlowResults;

public class TraceAlignmentProjection implements ITraceAlignment
{
	private ITraceAlignment origAlignment;
	private boolean bValid = true;
	private TokenFlowResults flowResults = null;
	
	public TraceAlignmentProjection(PetrinetGraph newNet, ITraceAlignment ta, PetriNet1toManyMapping map)
	{
		origAlignment = ta;
		bValid = ta.isValidTrace();

		flowResults = new TokenFlowResults();
		
		//copy data
		for (Transition t : newNet.getTransitions())
		{
			flowResults.ProjectTransition(ta, map, t);
		}
		
		CompareTFRs(newNet, flowResults, ta.getFlowResults());
		
		//add dependency(/parent) connections
		for (Transition t : newNet.getTransitions())
		{
			PetrinetNode nodes = map.mapToView(t).get(0); //get first events
			Vector<TransitionEvent> evts = ta.getTransitionEventsFor((Transition)nodes);  //start event should have same local id in projected net
			for (TransitionEvent evt : evts)
			{
				TransitionEvent evtLvlUp = evt.levelUp;
				for (TransitionEvent evtParent : evt.getParents())
				{
					evtLvlUp.addParent(evtParent.levelUp);
				}
			}
		}
		
		CompareTFRs(newNet, flowResults, ta.getFlowResults());
	}
	
	private void CompareTFRs(PetrinetGraph newNet, TokenFlowResults tfr1, TokenFlowResults tfr2)
	{
		for (Transition t : newNet.getTransitions())
		{
			Vector<TransitionEvent> lst1 = tfr1.getTransitionEventsFor(t);
			Vector<TransitionEvent> lst2 = tfr2.getTransitionEventsFor(t);
			
			if (lst1.size() != lst2.size())
			{
				String s = "Error!";
			}
		}
	}
	
	public ITraceAlignment getOriginalAlignment()
	{
		return origAlignment;
	}
	
	public boolean isValidTrace()
	{
		return bValid;
	}

	@Override
	public void addSubnetEvents(INetDataSource source, Vector<SubNetDataSource> subnets) 
	{
		flowResults.addSubnetEvents(source, subnets);
	}

	@Override
	public Vector<TransitionEvent> getTransitionEventsFor(Transition t)
	{
		return flowResults.getTransitionEventsFor(t);
	}

	@Override
	public Vector<TransitionEvent> getAllTransitionEvents()
	{
		return flowResults.getAllTransitionEvents();
	}
	
	@Override
	public TokenFlowResults getFlowResults()
	{
		return flowResults;
	}
}
