package org.processmining.processprofiler3d.psen.replayanalysis;

import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.processprofiler3d.psen.replayanalysis.tokenflow.TokenFlowResults;
import org.processmining.processprofiler3d.psen.replayanalysis.tokenflow.TokenFlowSimulation;

public class TraceAlignmentReplayState extends TraceAlignmentReplayStateBase 
{
	private ITraceAlignment ta;
	
	private XTrace trace;
	private IReplayResultWrapper resWrapper;
	
	private TokenFlowSimulation initMarking;
	private TokenFlowSimulation mapTokens;
	
	private int stepId = 0;
	
	private int logIndex = 0;
	private XEvent lastEvent = null;
	private int lastEventId = 0;
	
	public TraceAlignmentReplayState(ITraceAlignment ta, TokenFlowSimulation initMarking, XTrace tr, IReplayResultWrapper rw)
	{
		this.ta = ta;
		this.initMarking = initMarking;
		this.trace = tr;
		resWrapper = rw;
	}
	
	@Override
	public TokenFlowResults getTransitionEvents()
	{
		return mapTokens.getTransitionEvents();
	}

	@Override
	public TokenFlowSimulation getMarking()
	{
		return mapTokens;
	}
	
	@Override
	public void Init()
	{
		stepId = 0;

		// information accumulator
		isPerfectFitTrace = true;
		isBrokenTrace = false;
		
		// keep on track on the marking for constructing event class transitions
		mapTokens = initMarking.clone();
		mapTokens.InitResults();
		
		//perform each step of the replay
		logIndex = 0;
		lastEvent = null;
		lastEventId = 0;
	}

	@Override
	public int getStepCount()
	{
		//return syncRepRes.getStepTypes().size();
		return resWrapper.getStepCount();
	}

	@Override
	public boolean NextStep(TraceReplayStepData trs)
	{
		if (stepId >= resWrapper.getStepCount())
		{
			return false;
		}
		
		StepTypes type = resWrapper.getStepType(stepId);
		Transition trans = null;
		mapTokens.bTransitionFired = false;
		if (trs != null)
		{
			trs.bTransitionFired = false;
		}
		
		Object currentNodeInstance = resWrapper.getNodeInstance(stepId);
		switch (type) 
		{
			//step is move on log only
			case L :
				lastEvent = trace.get(logIndex);
				lastEventId = logIndex;

				trans = null;
				//trans = EventClass2Transition((XEventClass)currentNodeInstance);
				//System.out.print("L: ");
				//LogEventLifecycle(lastEvent);
				
				//System.out.println("L");

				//should probably not handle this, as it does not actually fit the model
				//Handle(lastEvent, lastEventId, trans);
				
				isPerfectFitTrace = false;
				logIndex++;
				break;
			//step is move on both log and model
			case LMGOOD :
				trans = (Transition)currentNodeInstance;
				lastEvent = trace.get(logIndex);
				lastEventId = logIndex;
				
				//System.out.println("LMGood");

				//Handle(lastEvent, lastEventId, trans);

				if (trans == null)
				{
					System.out.println("Transition == null...this should not happen!");
					//return;
				}
				
				TransitionEvent tEvt2 = null;	
				tEvt2 = HandleTransitionFire(trans, trs);
				tEvt2.setAllEvents(lastEventId);
				
				//System.out.print("LMGood: ");
				//LogEventLifecycle(lastEvent);		
				
				logIndex++;
				break;
			//step is move on model only (either real or invisible transition is fired)
			case MINVI :
			case MREAL :
				//invi is the same as real
				trans = (Transition)currentNodeInstance;
				if (lastEvent == null || resWrapper instanceof ReplayResultWrapper2)
				{
					//aligned log does contain events for these steptypes
					lastEvent = trace.get(logIndex);
					lastEventId = logIndex;
				}

				//System.out.println("MReal or MInvi");

				TransitionEvent tEvt = HandleTransitionFire(trans, trs);
				tEvt.setAllEvents(lastEventId);

				if (type.equals(StepTypes.MREAL))
				{
					isPerfectFitTrace = false;
				}
				
				if (resWrapper instanceof ReplayResultWrapper2)
				{
					//aligned log does contain events for these steptypes
					logIndex++;
				}
				break;
			case LMNOGOOD :
			case LMREPLACED:
			case LMSWAPPED:
				// unable to project this case, stop
				trans = null;
				isPerfectFitTrace = false;
				isBrokenTrace = true;
				
				if (resWrapper instanceof ReplayResultWrapper2)
				{
					//aligned log does contain events for these steptypes
					logIndex++;
				}
				
				return false;
		}
		
		if (trs != null)
		{
			//extract relevant data for analysis/visualization
			trs.type = type;
			trs.lastEvent = lastEvent;
			trs.lastEventId = lastEventId;
			trs.logIndex = logIndex;
			trs.trans = trans;
			//trs.bTransitionFired = mapTokens.bTransitionFired;
			//trs.bTokenMismatch = mapTokens.bTokenMismatch;
			//trs.parentTokens = mapTokens.parentTokens;
			//trs.childTokens = mapTokens.childTokens;
		}
		
		stepId++;
		return stepId < resWrapper.getStepCount();
	}

	private TransitionEvent HandleTransitionFire(Transition trans, TraceReplayStepData trs)
	{
		TransitionEvent tEvt;
		if (trs != null)
		{
			//tEvt = mapTokens.fireTransition(trans, ta, trs);
			tEvt = mapTokens.fireTransition(trans, ta, trs);
		}
		else
		{
			//tEvt = mapTokens.fireTransition(trans, ta);
			tEvt = mapTokens.fireTransition(trans, ta);
		}
		return tEvt;
	}
}
