package org.processmining.processprofiler3d.psen.replayanalysis;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension.StandardModel;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.processprofiler3d.psen.replayanalysis.tokenflow.TokenFlowResults;
import org.processmining.processprofiler3d.psen.replayanalysis.tokenflow.TokenFlowSimulation;

//========================================================================
// For use when we already have an aligned log and use that for replay (for token flow analysis)
//========================================================================

public class TraceAlignmentReplayStateOpt extends TraceAlignmentReplayStateBase
{
	private ITraceAlignment ta;
	
	private XTrace trace;
	private IReplayResultWrapper resWrapper;
	
	private TokenFlowSimulation initMarking;
	private TokenFlowSimulation mapTokens;
	
	private int stepId = 0;
	
	private int logIndex = 0;
	private XEvent lastEvent = null;
	private int lastEventId = 0;
	
	private Map<Transition,Vector<XEvent>> register = new HashMap<Transition, Vector<XEvent>>();
	
	public TraceAlignmentReplayStateOpt(ITraceAlignment ta, TokenFlowSimulation initMarking, XTrace tr, IReplayResultWrapper rw)
	{
		this.ta = ta;
		this.initMarking = initMarking;
		this.trace = tr;
		resWrapper = rw;
	}

	@Override
	public TokenFlowResults getTransitionEvents()
	{
		return mapTokens.getTransitionEvents();
	}

	@Override
	public TokenFlowSimulation getMarking()
	{
		return mapTokens;
	}
	
	@Override
	public void Init()
	{
		stepId = 0;

		// information accumulator
		isPerfectFitTrace = true;
		isBrokenTrace = false;
		
		// keep on track on the marking for constructing event class transitions
		mapTokens = initMarking.clone();
		mapTokens.InitResults();
		
		//perform each step of the replay
		logIndex = 0;
		lastEvent = null;
		lastEventId = 0;
	}

	@Override
	public int getStepCount()
	{
		return resWrapper.getStepCount();
	}

	@Override
	public boolean NextStep(TraceReplayStepData trs)
	{
		if (stepId >= resWrapper.getStepCount())
		{
			return false;
		}
		
		StepTypes type = resWrapper.getStepType(stepId);
		Transition trans = null;
		mapTokens.bTransitionFired = false;
		if (trs != null)
		{
			trs.bTransitionFired = false;
		}
		
//		System.out.println("TokenState:" + mapTokens.getStateDesc());
		
		Object currentNodeInstance = resWrapper.getNodeInstance(stepId);
		switch (type) 
		{
			//step is move on log only
			case L :
				lastEvent = trace.get(logIndex);
				lastEventId = logIndex;

				trans = null;
				
//				System.out.println("Move (L):" + XConceptExtension.instance().extractName(lastEvent));
				
				isPerfectFitTrace = false;
				logIndex++;
				break;
			//step is move on both log and model
			case LMGOOD :
				trans = (Transition)currentNodeInstance;
				lastEvent = trace.get(logIndex);
				lastEventId = logIndex;
				
				if (trans == null)
				{
					System.out.println("Transition == null...this should not happen!");
					//return;
				}

//				System.out.println("Move (LMGOOD):" + XConceptExtension.instance().extractName(lastEvent));
//				System.out.println("Fire:" + trans.getLabel());
				HandleTransitionFire(trans, trs);
				
				logIndex++;
				break;
			//step is move on model only (either real or invisible transition is fired)
			case MINVI :
			case MREAL :
				//invi is the same as real
				trans = (Transition)currentNodeInstance;
				if (lastEvent == null || resWrapper instanceof ReplayResultWrapper2)
				{
					//aligned log does contain events for these steptypes
					lastEvent = trace.get(logIndex);
					lastEventId = logIndex;
				}

//				if (type.equals(StepTypes.MINVI))
//				{
//					System.out.println("Move (MINVI):" + XConceptExtension.instance().extractName(lastEvent));
//				}
//				if (type.equals(StepTypes.MREAL))
//				{
//					System.out.println("Move (MREAL):" + XConceptExtension.instance().extractName(lastEvent));
//				}
//				System.out.println("Fire:" + trans.getLabel());
				
				//System.out.println("MReal or MInvi");
				HandleTransitionFire(trans, trs);
				
				if (type.equals(StepTypes.MREAL))
				{
					isPerfectFitTrace = false;
				}
				
				if (resWrapper instanceof ReplayResultWrapper2)
				{
					//aligned log does contain events for these steptypes
					logIndex++;
				}
				break;
			case LMNOGOOD :
			case LMREPLACED:
			case LMSWAPPED:
				// unable to project this case, stop
				trans = null;
				isPerfectFitTrace = false;
				isBrokenTrace = true;

//				System.out.println("Move (Other)!");
				
				if (resWrapper instanceof ReplayResultWrapper2)
				{
					//aligned log does contain events for these steptypes
					logIndex++;
				}
				
				return false;
		}
		
		if (trs != null)
		{
			//extract relevant data for analysis/visualization
			trs.type = type;
			trs.lastEvent = lastEvent;
			trs.lastEventId = lastEventId;
			trs.logIndex = logIndex;
			trs.trans = trans;
			//trs.bTransitionFired = mapTokens.bTransitionFired;
			//trs.bTokenMismatch = mapTokens.bTokenMismatch;
			//trs.parentTokens = mapTokens.parentTokens;
			//trs.childTokens = mapTokens.childTokens;
		}
		
		stepId++;
		//return stepTypes.hasNext();
		return stepId < resWrapper.getStepCount();
	}

	public void HandleTransitionFire(Transition trans, TraceReplayStepData trs) 
	{
		StandardModel lifecycle = XLifecycleExtension.instance().extractStandardTransition(lastEvent);
		if (lifecycle == null)
		{
			//assume complete if info is missing
			lifecycle = StandardModel.COMPLETE;
		}
		
		if (lifecycle != StandardModel.COMPLETE)
		{
			AddToRegister(trans);
		}
		else
		{	
			TransitionEvent tEvt = null;
			if (trs != null)
			{
				tEvt = mapTokens.fireTransition(trans, ta, trs);
			}
			else
			{
				tEvt = mapTokens.fireTransition(trans, ta);
			}
			tEvt.setAllEvents(lastEventId);
			
			ProcessRegisterEvents(trans, tEvt);
		}
	}

	public void AddToRegister(Transition trans) 
	{
		if (!register.containsKey(trans))
		{
			register.put(trans, new Vector<XEvent>());
		}
		register.get(trans).add(lastEvent);
	}

	public void ProcessRegisterEvents(Transition trans, TransitionEvent tEvt2) 
	{
		if (register.containsKey(trans))
		{
			//current approach is problematic for concurrent instances of an activity!
			Vector<XEvent> rmv = new Vector<XEvent>();
			Vector<XEvent> lst = new Vector<XEvent>(register.get(trans));
			Collections.reverse(lst);
			for (XEvent pastEvt : lst)
			{
				StandardModel pastLc = XLifecycleExtension.instance().extractStandardTransition(pastEvt);
				if (pastLc != null)
				{
					int pevtId = trace.indexOf(pastEvt);
					if (pastLc == StandardModel.SCHEDULE)
					{
						tEvt2.setScheduleEvent(pevtId);
						rmv.add(pastEvt);
					}
					else if (pastLc == StandardModel.ASSIGN)
					{
						if (tEvt2.getScheduleId() > pevtId)
						{
							tEvt2.setScheduleEvent(pevtId);		
						}
						tEvt2.setAssignEvent(pevtId);
						rmv.add(pastEvt);
					}
					else if (pastLc == StandardModel.START)
					{
						if (tEvt2.getScheduleId() > pevtId)
						{
							tEvt2.setScheduleEvent(pevtId);		
						}
						if (tEvt2.getAssignId() > pevtId)
						{
							tEvt2.setAssignEvent(pevtId);		
						}
						tEvt2.setStartEvent(pevtId);
						rmv.add(pastEvt);			
					}
				}
			}
			
			//remove handled events
			for (XEvent rEvt : rmv)
			{
				register.get(trans).remove(rEvt);
			}
		}
	}
}
