package org.processmining.processprofiler3d.psen.replayanalysis;

import java.util.Date;
import java.util.Vector;

import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.psen.preprocess.LPPAction;

public class TraceReplayAnalyser 
{
	private ITraceAlignment alignment;
	private XTrace trace;
	
	public Date startTime;
	public Date[] timestamps;
	
	public XTrace getTrace()
	{
		return trace;
	}
	
	public boolean isValidTrace()
	{
		return alignment.isValidTrace();
	}

	public TraceReplayAnalyser(ITraceAlignment ta, XTrace tr) 
	{
		this.alignment = ta;
		trace = tr;
		
		//init timing info
		timestamps = new Date[trace.size()];
		int i = 0;
		int j = 0;
		for (XEvent evt : trace)
		{
			timestamps[i] = LPPAction.getAttributeValueTimestamp(evt, XTimeExtension.KEY_TIMESTAMP);
			
			//startTime = new Date(0L); //minimum date
			if (timestamps[i] != null)
			{
				if (j == 0)
				{
					startTime = timestamps[i];
				}
				else if (timestamps[i].before(startTime))
				{
					startTime = timestamps[i];
				}
				j++;
			}
			
			i++;
		}
	}
	
	//get timestamp for event at pos (id) in trace, or last valid timestamp
	public Date getEventTimeFixed(int id)
	{
		if (id == -1)
		{
			return null;
		}
		for (int i = id; i > -1; i--)
		{
			if (timestamps[i] != null)
			{
				return timestamps[i];
			}
		}
		return startTime;
	}
	
	public ITraceAlignment getTraceAlignment()
	{
		return alignment;
	}
	
	public Vector<TransitionEvent> getAllTransitionEvents()
	{
		return alignment.getAllTransitionEvents();
	}

	public Date getTraceStartTime()
	{
		return startTime;
	}
	
//	public TransitionEvent getFirstEvent(PetriNetDataSource pnds)
//	{
//		Place startPlace = pnds.getStartPlace();
//		Transition startTransition = null;
//		for (Arc a : pnds.getOutEdges(startPlace))
//		{
//			startTransition = (Transition)a.getTarget();
//		}
//		Vector<TransitionEvent> tevt = alignment.getTransitionEventsFor(startTransition);
//		
//		return tevt.get(0);
//	}
//	
//	public TransitionEvent getLastEvent(PetriNetDataSource pnds)
//	{
//		Place endPlace = pnds.getEndPlace();
//		Transition endTransition = null;
//		for (Arc a : pnds.getInEdges(endPlace))
//		{
//			endTransition = (Transition)a.getSource();
//		}
//		Vector<TransitionEvent> tevt = alignment.getTransitionEventsFor(endTransition);
//		
//		return tevt.get(0);
//	}
	
	public TransitionEvent getLastEvent(Transition endTransition)
	{
		Vector<TransitionEvent> tevt = alignment.getTransitionEventsFor(endTransition);
		
		return tevt.get(0);
	}


}
