package org.processmining.processprofiler3d.psen.replayanalysis;

import java.util.HashMap;
import java.util.Vector;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XLog;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.SubNetDataSource;
import org.processmining.processprofiler3d.psen.PetriNet1toManyMapping;
import org.processmining.processprofiler3d.psen.preprocess.LPPAction;

public class TraceReplayCollection 
{
	private Vector<ITraceAlignment> tas = new Vector<ITraceAlignment>();
	private Vector<TraceReplayAnalyser> trs = new Vector<TraceReplayAnalyser>();
	private Vector<TraceReplayAnalyser> validtrs = new Vector<TraceReplayAnalyser>();
	private HashMap<ITraceAlignment, Vector<TraceReplayAnalyser>> tasmap = new HashMap<ITraceAlignment, Vector<TraceReplayAnalyser>>();
	
	public void addTraceReplay(TraceReplayAnalyser tra)
	{
		if (!tas.contains(tra.getTraceAlignment()))
		{
			tas.add(tra.getTraceAlignment());
		}
		
		trs.add(tra);
		if (tra.isValidTrace())
		{
			validtrs.add(tra);
		}
		
		if (!tasmap.containsKey(tra.getTraceAlignment()))
		{
			tasmap.put(tra.getTraceAlignment(), new Vector<TraceReplayAnalyser>());
		}
		tasmap.get(tra.getTraceAlignment()).add(tra);
	}
	
	public TraceReplayCollection ProjectBack(PetrinetGraph png, PetriNet1toManyMapping map)
	{
		HashMap<ITraceAlignment, TraceAlignmentProjection> proj = new HashMap<ITraceAlignment, TraceAlignmentProjection>();
		
		for (ITraceAlignment ta : tas)
		{
			TraceAlignmentProjection tap = new TraceAlignmentProjection(png, ta, map);
			proj.put(ta, tap);
		}
		
		TraceReplayCollection trc = new TraceReplayCollection();
		
		for (TraceReplayAnalyser tra : trs)
		{
			TraceReplayAnalyser trx = new TraceReplayAnalyser(proj.get(tra.getTraceAlignment()), tra.getTrace());
			
			trc.addTraceReplay(trx);
		}
		
		return trc;
	}
	
	public TraceReplayCollection ProjectBack(PetrinetGraph png, PetriNet1toManyMapping map, XLog prlog)
	{	
		HashMap<String, Integer> logmap = new HashMap<String, Integer>();
		for (int li = 0; li < prlog.size(); li++)
		{
			String nn = LPPAction.getAttributeValueLiteral(prlog.get(li), XConceptExtension.KEY_NAME);
			logmap.put(nn, li);
		}
		
		HashMap<ITraceAlignment, TraceAlignmentProjection> proj = new HashMap<ITraceAlignment, TraceAlignmentProjection>();
		
		for (ITraceAlignment ta : tas)
		{
			TraceAlignmentProjection tap = new TraceAlignmentProjection(png, ta, map);
			proj.put(ta, tap);
		}
		
		TraceReplayCollection trc = new TraceReplayCollection();
		
		for (TraceReplayAnalyser tra : trs)
		{
			String oName = LPPAction.getAttributeValueLiteral(tra.getTrace(), XConceptExtension.KEY_NAME);
			TraceReplayAnalyser trx = new TraceReplayAnalyser(proj.get(tra.getTraceAlignment()), prlog.get(logmap.get(oName)));
			
			trc.addTraceReplay(trx);
		}
		
		return trc;
	}
	
	public Vector<TraceReplayAnalyser> getTraceReplays() 
	{
		return trs;
	}

	public Vector<TraceReplayAnalyser> getValidTraceReplays() 
	{
		return validtrs;
	}
	
	public Vector<ITraceAlignment> getTraceAlignments()
	{
		return tas;
	}
	
	public Vector<TraceReplayAnalyser> getValidTraceReplaysFor(ITraceAlignment ta)
	{
//		Vector<TraceReplayAnalyser> lst = new Vector<TraceReplayAnalyser>();
//		for (TraceReplayAnalyser tra : validtrs)
//		{
//			if (tra.getTraceAlignment() == ta)
//			{
//				lst.add(tra);
//			}
//		}
//		
//		return lst;
		if (tasmap.containsKey(ta))
		{
			return tasmap.get(ta);
		}
		return null;
	}
	
	public void addSubNetEvents(INetDataSource source)
	{
		//get all subnets in order bottom to top
		int maxDepth = source.getMaxSubnetDepth();
		Vector<SubNetDataSource> subnets = new Vector<SubNetDataSource>();
		for (int i = maxDepth; i >= 0; i--)
		{
			source.getAllSubnetsAtLevel(i, subnets);
		}
		
		//construct accumulated transition events
		for (ITraceAlignment ta : tas)
		{
			ta.addSubnetEvents(source, subnets);
		}		
	}
}
