package org.processmining.processprofiler3d.psen.replayanalysis;

import java.util.Comparator;
import java.util.Date;
import java.util.Vector;

import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class TransitionEvent 
{
	private Vector<TransitionEvent> parents = new Vector<TransitionEvent>();
	
	public TransitionEvent levelUp = null;
	private Transition trans;

	private int scheduleId = -1;
	private int assignId = -1;
	private int startId = -1;
	private int completeId = -1;
	
	public void setScheduleEvent(int id)
	{
		scheduleId = id;
	}
	
	public void setAssignEvent(int id)
	{
		assignId = id;
	}
	
	public void setStartEvent(int id)
	{
		startId = id;
	}
	
	public void setCompleteEvent(int id)
	{
		completeId = id;
	}
	
	public void setAllEvents(int id)
	{
		scheduleId = id;
		assignId = id;
		startId = id;
		completeId = id;
	}
	
	public TransitionEvent(Transition t)
	{
		trans = t;
	}
	
	public boolean isTransition(Transition t)
	{
		return t.getLocalID().toString().equals(trans.getLocalID().toString());
	}
	
	public boolean isTransitionParent(Transition t)
	{
		String tIdStr = t.getLocalID().toString();
		TransitionEvent levelUpEvt = levelUp;
		while (levelUpEvt != null)
		{
			if (tIdStr.equals(levelUpEvt.trans.getLocalID().toString()))
			{
				return true;
			}
			levelUpEvt = levelUpEvt.levelUp;
		}
		
		return false;
	}
	
	public String getTransitionLabel()
	{
		return trans.getLabel();
	}
	
	public Transition getTransition()
	{
		return trans;
	}
	
	public String getTransitionID()
	{
		return trans.getLocalID().toString();
	}
	
	public void addParent(TransitionEvent t)
	{
		if (!parents.contains(t))
		{
			parents.add(t);
		}
	}
	
	public Vector<TransitionEvent> getParents()
	{
		return new Vector<TransitionEvent>(parents);
	}
	
	public boolean isParent(TransitionEvent t)
	{
		return parents.contains(t);
	}
	
	public int getScheduleId()
	{
		return scheduleId;
	}
	
	public int getAssignId()
	{
		return assignId;
	}
	
	public int getStartId()
	{
		return startId;
	}
	
	public int getCompleteId()
	{
		return completeId;
	}
	
	public Date getScheduleTime(XTrace t)
	{
		if (scheduleId > -1)
		{
			return getTime(t.get(scheduleId));
		}
		return null;
	}

	public Date getAssignTime(XTrace t)
	{
		if (assignId > -1)
		{
			return getTime(t.get(assignId));
		}
		return null;
	}
	
	public Date getStartTime(XTrace t)
	{
		if (startId > -1)
		{
			return getTime(t.get(startId));
		}
		return null;
	}
	
	public Date getCompleteTime(XTrace t)
	{
		if (completeId > -1)
		{
			return getTime(t.get(completeId));
		}
		return null;
	}
	
	private Date getTime(XEvent evt)
	{
		if (evt == null)
		{
			return null;
		}
		
		XAttributeTimestamp timeStamp = (XAttributeTimestamp)evt.getAttributes().get(XTimeExtension.KEY_TIMESTAMP);

		if (timeStamp != null && timeStamp.getValue() != null)
		{
			Date mytime = timeStamp.getValue();		
			return mytime;
		}
		return null;
	}
	
	public TimeSpan getDuration(XTrace t)
	{
		Date sTime = getStartTime(t);
		Date eTime = getCompleteTime(t);
		
		if (sTime != null && eTime != null)
		{
			TimeSpan ts = new TimeSpan(sTime, eTime);
			return ts;
		}
		return null;
	}
	
	public TimeSpan getThroughputTime(XTrace t)
	{
		Date sTime = getScheduleTime(t);
		Date eTime = getCompleteTime(t);
		
		if (sTime != null && eTime != null)
		{
			TimeSpan ts = new TimeSpan(sTime, eTime);
			return ts;
		}
		return null;
	}
	
	public TimeSpan getServiceTime(XTrace t)
	{
		Date sTime = getAssignTime(t);
		Date eTime = getCompleteTime(t);
		
		if (sTime != null && eTime != null)
		{
			TimeSpan ts = new TimeSpan(sTime, eTime);
			return ts;
		}
		return null;
	}
	
	public TimeSpan getQueueTime(XTrace t)
	{
		Date sTime = getScheduleTime(t);
		Date eTime = getAssignTime(t);
		
		if (sTime != null && eTime != null)
		{
			TimeSpan ts = new TimeSpan(sTime, eTime);
			return ts;
		}
		return null;
	}

	public Vector<Integer> getPathsFrom(Transition t1) 
	{
		Vector<Integer> times = new Vector<Integer>();
		Vector<TransitionEvent> teParents = getParents();
		for (TransitionEvent tep : teParents)
		{
			if (tep.isTransition(t1))
			{
				getPathsFromHelper(times, tep);
			}
			else if (tep.isTransitionParent(t1))
			{
				TransitionEvent levelUpEvt = tep.levelUp;
				while (levelUpEvt != null)
				{
					if (levelUpEvt.isTransition(t1))
					{
						getPathsFromHelper(times, levelUpEvt);
						break;
					}
					levelUpEvt = levelUpEvt.levelUp;
				}
				
			}
		}
		return times;
	}

	public void getPathsFromHelper(Vector<Integer> times, TransitionEvent tep) 
	{
		int sTime = tep.getCompleteId();
		//int eTime = getStartId();
		
		times.add(sTime);
	}

	public Vector<TimeSpan> getWaitingTimesFrom(XTrace tr, Transition t1) 
	{
		Vector<TimeSpan> times = new Vector<TimeSpan>();
		Vector<TransitionEvent> teParents = getParents();
		for (TransitionEvent tep : teParents)
		{
			if (tep.isTransition(t1))
			{
				getWaitingTimesHelper(tr, times, tep);
			}
			else if (tep.isTransitionParent(t1))
			{
				TransitionEvent levelUpEvt = tep.levelUp;
				while (levelUpEvt != null)
				{
					if (levelUpEvt.isTransition(t1))
					{
						getWaitingTimesHelper(tr, times, levelUpEvt);
						break;
					}
					levelUpEvt = levelUpEvt.levelUp;
				}
				
			}
		}
		return times;
	}

	public void getWaitingTimesHelper(XTrace tr, Vector<TimeSpan> times, TransitionEvent tep) 
	{
		Date sTime = tep.getCompleteTime(tr);
		Date eTime = getStartTime(tr);
		
		if (sTime != null && eTime != null)
		{
			TimeSpan ts = new TimeSpan(sTime, eTime);
			if (ts.getMilliseconds() < 0)
			{
				System.out.println("Waiting time: " + tep.getTransitionLabel() + " - " + getTransitionLabel() + " : less than 0!");
			}
			times.add(ts);
		}
	}

	public TimeSpan getWaitingTime(XTrace tr) 
	{
		Vector<TransitionEvent> teParents = getParents();
		if (teParents.size() > 0)
		{
			Long time = Long.MAX_VALUE;
			for (TransitionEvent tep : teParents)
			{
				Date sTime = tep.getCompleteTime(tr);
				Date eTime = getStartTime(tr);
				
				if (sTime != null && eTime != null)
				{
					TimeSpan ts = new TimeSpan(sTime, eTime);
					if (ts.getMilliseconds() < 0)
					{
						System.out.println("Waiting time: " + tep.getTransitionLabel() + " - " + getTransitionLabel() + " : less than 0!");
					}
					time = Math.min(time, ts.getMilliseconds());
				}
			}
			if (time < Long.MAX_VALUE)
			{
				return new TimeSpan(time);
			}
		}
		return null;
	}

	public Vector<Integer> getSoujournTimeIdsFrom(Transition t1) 
	{
		Vector<Integer> times = new Vector<Integer>();
		Vector<TransitionEvent> teParents = getParents();
		for (TransitionEvent tep : teParents)
		{
			if (tep.isTransition(t1))
			{
				getSoujournTimeIdsHelper(times, tep);
			}
			else if (tep.isTransitionParent(t1))
			{
				TransitionEvent levelUpEvt = tep.levelUp;
				while (levelUpEvt != null)
				{
					if (levelUpEvt.isTransition(t1))
					{
						getSoujournTimeIdsHelper(times, levelUpEvt);
						break;
					}
					levelUpEvt = levelUpEvt.levelUp;
				}
				
			}
		}
		return times;
	}

	public void getSoujournTimeIdsHelper(Vector<Integer> times, TransitionEvent tep) 
	{
		int sTime = tep.getStartId();	
		times.add(sTime);
	}

	public Vector<TimeSpan> getSoujournTimesFrom(XTrace tr, Transition t1) 
	{
		Vector<TimeSpan> times = new Vector<TimeSpan>();
		Vector<TransitionEvent> teParents = getParents();
		for (TransitionEvent tep : teParents)
		{
			if (tep.isTransition(t1))
			{
				getSoujournTimesHelper(tr, times, tep);
			}
			else if (tep.isTransitionParent(t1))
			{
				TransitionEvent levelUpEvt = tep.levelUp;
				while (levelUpEvt != null)
				{
					if (levelUpEvt.isTransition(t1))
					{
						getSoujournTimesHelper(tr, times, levelUpEvt);
						break;
					}
					levelUpEvt = levelUpEvt.levelUp;
				}
				
			}
		}
		return times;
	}

	public TimeSpan getSoujournTime(XTrace tr) 
	{
		Vector<TimeSpan> times = new Vector<TimeSpan>();
		Vector<TransitionEvent> teParents = getParents();
		for (TransitionEvent tep : teParents)
		{
			//if (tep.isTransition(t1))
//			{
				getSoujournTimesHelper(tr, times, tep);
//			}
//			else if (tep.isTransitionParent(t1))
//			{
//				TransitionEvent levelUpEvt = tep.levelUp;
//				while (levelUpEvt != null)
//				{
//					if (levelUpEvt.isTransition(t1))
//					{
//						getSoujournTimesHelper(tr, times, levelUpEvt);
//						break;
//					}
//					levelUpEvt = levelUpEvt.levelUp;
//				}
//				
//			}
		}
		
		return TimeSpan.getMin(times);
	}

	public void getSoujournTimesHelper(XTrace tr, Vector<TimeSpan> times, TransitionEvent tep) 
	{
		Date sTime = tep.getStartTime(tr);
		Date eTime = getStartTime(tr);
		
		if (sTime != null && eTime != null)
		{
			TimeSpan ts = new TimeSpan(sTime, eTime);
			if (ts.getMilliseconds() < 0)
			{
				System.out.println("Soujourn time: " + tep.getTransitionLabel() + " - " + getTransitionLabel() + " : less than 0!");
			}
			times.add(ts);
		}
	}

	public TimeSpan getSyncTime(XTrace tr) 
	{
		TimeSpan wtimets = getWaitingTime(tr);
		if (wtimets != null)
		{
			Long wtime = wtimets.getMilliseconds();
			Long time = (long)0;
			Vector<TransitionEvent> teParents = getParents();
			if (teParents.size() > 0)
			{
				for (TransitionEvent tep : teParents)
				{
					Date sTime = tep.getCompleteTime(tr);
					Date eTime = getStartTime(tr);
					
					if (sTime != null && eTime != null)
					{
						TimeSpan ts = new TimeSpan(sTime, eTime);
						if (ts.getMilliseconds() < 0)
						{
							System.out.println("Sync time: " + tep.getTransitionLabel() + " - " + getTransitionLabel() + " : less than 0!");
						}
						time = Math.max(time, ts.getMilliseconds() - wtime);
					}
				}
				return new TimeSpan(time);
			}
		}
		return null;
	}
	
	public boolean isComplete()
	{
		return (completeId > -1);
	}
	
	public int completeState()
	{
		int state = 0;
		if (scheduleId > -1)
		{
			state = 1;
		}
		if (assignId > -1)
		{
			state = 2;
		}
		if (startId > -1)
		{
			state = 3;
		}
		if (completeId > -1)
		{
			state = 4;
		}
		return state;
	}

	public static Comparator<TransitionEvent> StartTimeComparatorAscending 
		= new Comparator<TransitionEvent>() {

		public int compare(TransitionEvent te1, TransitionEvent te2) {
		
		//ascending order
		return te1.getStartId() - te2.getStartId();
		}
	
	};

	public static Comparator<TransitionEvent> StartTimeComparatorDescending 
		= new Comparator<TransitionEvent>() {

		public int compare(TransitionEvent te1, TransitionEvent te2) {
		
		//descending order
		return te2.getStartId() - te1.getStartId();
		}
	
	};
}
