package org.processmining.processprofiler3d.psen.replayanalysis.stats;

import java.util.Date;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.processprofiler3d.psen.AlignedLog;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PSENLabel;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayAnalyser;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayCollection;

public class StatCalculatorLogFrequencyFiltered extends StatCalculator
{	
	private String outputLabel = "";
	
	public Vector<PSENLabel> myLabels = new Vector<PSENLabel>();
	public String myLabelNode = "frequency_filtered";
	
	public StatCalculatorLogFrequencyFiltered()
	{
		outputLabel = "Frequency Filtered";
	}
	
	@Override
	public void addLabelsToPSEN(IPerformanceStatisticsEnrichedNet psnet, IPerformanceStatisticsEnrichedNet wtt)
	{
		PSENLabel frequency = new PSENLabel("frequency_filtered", "frequency_filtered", PSENLabel.SUM, false);
		myLabels.add(frequency);
		psnet.addLabel(frequency);
	}
	
	//provider level
	@Override
	public void calculate(PetrinetGraph pnet, TraceReplayCollection provider, IPerformanceStatisticsEnrichedNet psen, IPerformanceStatisticsEnrichedNet wten)
	{	
		System.out.print("Calculating " + outputLabel + "...");

		Vector<TimeUnit> tus = new Vector<TimeUnit>();
		tus.add(TimeUnit.DAYS);
		tus.add(TimeUnit.HOURS);
		tus.add(TimeUnit.MINUTES);
		tus.add(TimeUnit.SECONDS);
		tus.add(TimeUnit.MILLISECONDS);
		
		Date startTime = new Date();
		
		//for (XTrace tr : log) // in the event log, each element is a case
		for (TraceReplayAnalyser tra : provider.getTraceReplays())
		{
			XTrace tr = tra.getTrace();
			String traceID = XConceptExtension.instance().extractName(tr);
			for (XEvent myEvent : tr) // in each case, each element is an event
			{
//				XAttributeLiteral myAttribute = (XAttributeLiteral)myEvent.getAttributes().get(XConceptExtension.KEY_NAME);
//				
//				//skip events without name
//				if (myAttribute.getValue() == null)
//					continue;

				//skip events in aligned log that do not represent events in the original log
				StepTypes stepType = AlignedLog.getStepType(myEvent);
				if (stepType != null)
				{
					if (!stepExistsInLog(stepType))
					{
						continue;
					}
				}

				String localID = AlignedLog.getLocalNodeId(myEvent);
				//if event has aligned local node id use this to match to transition
				if (localID != null)
				{
					psen.addValue(localID, myLabels.firstElement().getLabelName(), traceID, 1);
				}
			}
		}

		Date endTime = new Date();
		TimeSpan duration = new TimeSpan(startTime, endTime);

		System.out.println("" + duration.ToString(tus) + "");
	}
	
	private boolean stepExistsInLog(StepTypes stepType)
	{
		if (stepType == StepTypes.L)
		{
			return true;
		}
		if (stepType == StepTypes.LMGOOD)
		{
			return true;
		}
		if (stepType == StepTypes.LMNOGOOD)
		{
			return true;
		}
		if (stepType == StepTypes.LMSWAPPED)
		{
			return true;
		}
		if (stepType == StepTypes.LMREPLACED)
		{
			return true;
		}
		return false;
	}
}
