package org.processmining.processprofiler3d.psen.replayanalysis.stats;

import java.util.Date;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.compviz.SubNetDataSource;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PSENLabel;
import org.processmining.processprofiler3d.psen.preprocess.LPPAction;
import org.processmining.processprofiler3d.psen.replayanalysis.ITraceAlignment;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayAnalyser;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayCollection;
import org.processmining.processprofiler3d.psen.replayanalysis.TransitionEvent;

public class StatCalculatorTransitionCounts extends StatCalculator
{
	private String outputLabel = "";
	
	protected Vector<PSENLabel> myLabels = new Vector<PSENLabel>();
	protected String myLabelNode = "";
	
	public StatCalculatorTransitionCounts(String l)
	{
		outputLabel = l;
	}

	@Override
	public void addLabelsToPSEN(IPerformanceStatisticsEnrichedNet psnet, IPerformanceStatisticsEnrichedNet wtt)
	{
		
	}
	
//	//trace alignment level
//	public void getTimesFromTraceAlignment(ITraceAlignment ta, Transition t, XTrace tr, IPerformanceStatisticsEnrichedNet psen)
//	{
//		String tId = t.getLocalID().toString();
//		String psenLabel = myLabels.firstElement().getLabelName();
//		String traceID = LPPAction.getAttributeValueLiteral(tr, XConceptExtension.KEY_NAME);
//		Vector<TransitionEvent> evts = ta.getTransitionEventsFor(t);
//		for (TransitionEvent te : evts)
//		{
//			psen.addValue(tId, psenLabel, traceID, 1.0);
//		}
//	}
	
	//trace alignment level
	public void getTimesFromTraceAlignmentBatch(ITraceAlignment ta, Transition t, Vector<TraceReplayAnalyser> tras, IPerformanceStatisticsEnrichedNet psen)
	{
		String tId = t.getLocalID().toString();
		String psenLabel = myLabels.firstElement().getLabelName();
		for (TraceReplayAnalyser tra : tras)
		{
			String traceID = LPPAction.getAttributeValueLiteral(tra.getTrace(), XConceptExtension.KEY_NAME);
			Vector<TransitionEvent> evts = ta.getTransitionEventsFor(t);
			for (TransitionEvent te : evts)
			{
				psen.addValue(tId, psenLabel, traceID, 1.0);
			}
		}
	}
//	
//	//trace replay level
//	public void getTimesFromTraceReplay(TraceReplayAnalyser tra, Transition t, IPerformanceStatisticsEnrichedNet psen)
//	{
//		getTimesFromTraceAlignment(tra.getTraceAlignment(), t, tra.getTrace(), psen);
//	}
//	
//	//log replay level	
//	public void getTimesFromLogReplay(TraceReplayCollection lra, Transition t, IPerformanceStatisticsEnrichedNet psen)
//	{
//		int diff = lra.getTraceReplays().size() - lra.getValidTraceReplays().size();
//		if (diff > 0)
//		{
//			System.out.println("Aligned Frequency: dropped " + diff + " invalid traces!");
//		}
//		
//		for (TraceReplayAnalyser tra : lra.getValidTraceReplays())
//		{
//			getTimesFromTraceReplay(tra, t, psen);
//		}
//	}
	
	public void getTimesFromLogReplay(TraceReplayCollection lra, Transition t, IPerformanceStatisticsEnrichedNet psen)
	{
		for (ITraceAlignment ta : lra.getTraceAlignments())
		{
			getTimesFromTraceAlignmentBatch(ta, t, lra.getValidTraceReplaysFor(ta), psen);
		}
	}
	
	//provider level
	@Override
	public void calculate(PetrinetGraph pnet, TraceReplayCollection provider, IPerformanceStatisticsEnrichedNet psen, IPerformanceStatisticsEnrichedNet wten)
	{
		//initialize data structures
		INetDataSource source = new PetriNetDataSource(pnet);

		Vector<TimeUnit> tus = new Vector<TimeUnit>();
		tus.add(TimeUnit.DAYS);
		tus.add(TimeUnit.HOURS);
		tus.add(TimeUnit.MINUTES);
		tus.add(TimeUnit.SECONDS);
		tus.add(TimeUnit.MILLISECONDS);
		
		System.out.print("Calculating " + outputLabel + "...");

		Date startTime = new Date();
		
		CalculateHelper(provider, source, psen);

		Date endTime = new Date();
		TimeSpan duration = new TimeSpan(startTime, endTime);

		System.out.println("" + duration.ToString(tus) + "");
	}

	private void CalculateHelper(TraceReplayCollection provider, INetDataSource source, IPerformanceStatisticsEnrichedNet psen) 
	{
		for (SubNetDataSource esn : source.getSubnets())
		{
			CalculateHelper(provider, esn, psen);
		}
		
		for (Transition t : source.getTransitions())
		{
			getTimesFromLogReplay(provider, t, psen);
		}
	}
}
