package org.processmining.processprofiler3d.psen.replayanalysis.stats;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Vector;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PSENLabel;
import org.processmining.processprofiler3d.psen.datastructure.PerformanceStatisticsDB;
import org.processmining.processprofiler3d.psen.preprocess.LPPAction;
import org.processmining.processprofiler3d.psen.replayanalysis.ITraceAlignment;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayAnalyser;
import org.processmining.processprofiler3d.psen.replayanalysis.TransitionEvent;

public class StatCalculatorTransitionPairSoujournTime extends StatCalculatorTransitionPairTimes
{
	
	public StatCalculatorTransitionPairSoujournTime()
	{
		super("soujourn_time", "Soujourn time");
	}
	
	@Override
	public void addLabelsToPSEN(IPerformanceStatisticsEnrichedNet psnet, IPerformanceStatisticsEnrichedNet wtt)
	{
		myLabelNode = "soujourn_time";
		myLabels.add(new PSENLabel("soujourn_time_min", myLabelNode, PSENLabel.MIN, true));
		myLabels.add(new PSENLabel("soujourn_time_mean", myLabelNode, PSENLabel.MEAN, true));
		myLabels.add(new PSENLabel("soujourn_time_med", myLabelNode, PSENLabel.MEDIAN, true));
		myLabels.add(new PSENLabel("soujourn_time_max", myLabelNode, PSENLabel.MAX, true));
		
		for (PSENLabel myLabel : myLabels)
		{
			wtt.addLabel(myLabel);
		}
	}
	
//	//trace aligment level
//	@Override
//	public void getTimesFromTraceAlignment(ITraceAlignment ta, Transition t1, Transition t2, XTrace tr, IPerformanceStatisticsEnrichedNet wten)
//	{
//		String traceID = LPPAction.getAttributeValueLiteral(tr, XConceptExtension.KEY_NAME);
//		Vector<TransitionEvent> evts = ta.getTransitionEventsFor(t2);
//		for (TransitionEvent te : evts)
//		{
//			for (TimeSpan ts : te.getSoujournTimesFrom(tr, t1))
//			{
//				//wten.addValue(t1.getLocalID().toString() + t2.getLocalID().toString(), myLabels.firstElement().getLabelName(), cohortName, ts.getMilliseconds());
//				wten.addValue(t1.getLocalID().toString() + t2.getLocalID().toString(), myLabels.firstElement().getLabelName(), traceID, ts.getMilliseconds());
//			}
//		}
//	}
	
	//trace aligment level
	@Override
	public void getTimesFromTraceAlignmentBatch(ITraceAlignment ta, Transition t1, Transition t2, Vector<TraceReplayAnalyser> tras, IPerformanceStatisticsEnrichedNet wten)
	{
		String psenLabel = myLabels.firstElement().getLabelName();
		String tpairID = t1.getLocalID().toString() + t2.getLocalID().toString();

		PerformanceStatisticsDB pdb = (PerformanceStatisticsDB)wten;
		PreparedStatement ps = pdb.getBatchAddFor(tpairID, psenLabel);

		try
		{	
			Vector<TransitionEvent> evts = ta.getTransitionEventsFor(t2);
			for (TransitionEvent te : evts)
			{
				int endID = te.getStartId();
				for (Integer startID : te.getSoujournTimeIdsFrom(t1))
				{
					for (TraceReplayAnalyser tra : tras)
					{
						String traceID = LPPAction.getAttributeValueLiteral(tra.getTrace(), XConceptExtension.KEY_NAME);
						int trid = pdb.regTracePublic(traceID);
						if (tra.timestamps[startID] != null && tra.timestamps[endID] != null)
						{
							TimeSpan tsx = new TimeSpan(tra.timestamps[startID], tra.timestamps[endID]);
							
							ps.setInt(3, trid);
							ps.setDouble(4, tsx.getMilliseconds());
							ps.addBatch();
						}
					}
				}
			}
			pdb.commitBatchAdd();
		}
		catch (SQLException ex)
		{
			// TODO Auto-generated catch block
			ex.printStackTrace();
		}
	}
}
