package org.processmining.processprofiler3d.psen.replayanalysis.stats;

import java.util.Collection;
import java.util.Date;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.compviz.SubNetDataSource;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PSENLabel;
import org.processmining.processprofiler3d.psen.replayanalysis.ITraceAlignment;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayAnalyser;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayCollection;

public class StatCalculatorTransitionPairTimes extends StatCalculator
{
	private String outputLabel = "";
	private String label = "";
	private IPerformanceStatisticsEnrichedNet waitC;
	
	protected Vector<PSENLabel> myLabels = new Vector<PSENLabel>();
	protected String myLabelNode = "";
	
	public StatCalculatorTransitionPairTimes(String l, String ol)
	{
		label = l;
		outputLabel = ol;
	}
	
	@Override
	public void addLabelsToPSEN(IPerformanceStatisticsEnrichedNet psnet, IPerformanceStatisticsEnrichedNet wtt)
	{
		waitC = wtt;
	}
	
//	//trace alignment level
//	public void getTimesFromTraceAlignment(ITraceAlignment ta, Transition t1, Transition t2, XTrace tr, IPerformanceStatisticsEnrichedNet wten)
//	{
//	}
	
	public void getTimesFromTraceAlignmentBatch(ITraceAlignment ta, Transition t1, Transition t2, Vector<TraceReplayAnalyser> tras, IPerformanceStatisticsEnrichedNet wten)
	{
		
	}
	
//	//trace replay level
//	public void getTimesFromTraceReplay(TraceReplayAnalyser tra, Transition t1, Transition t2, IPerformanceStatisticsEnrichedNet wten)
//	{
//		getTimesFromTraceAlignment(tra.getTraceAlignment(), t1, t2, tra.getTrace(), wten);
//	}
	
	//log replay level
//	public void getTimesFromLogReplay(TraceReplayCollection lra, Transition t1, Transition t2, IPerformanceStatisticsEnrichedNet wten)
//	{
//		for (TraceReplayAnalyser tra : lra.getValidTraceReplays())
//		{
//			getTimesFromTraceReplay(tra, t1, t2, wten);
//		}
//	}
	
	public void getTimesFromLogReplay(TraceReplayCollection lra, Transition t1, Transition t2, IPerformanceStatisticsEnrichedNet wten)
	{
		for (ITraceAlignment ta : lra.getTraceAlignments())
		{
			getTimesFromTraceAlignmentBatch(ta, t1, t2, lra.getValidTraceReplaysFor(ta), wten);
		}
	}
	
	//provider level
	@Override
	public void calculate(PetrinetGraph pnet, TraceReplayCollection provider, IPerformanceStatisticsEnrichedNet psen, IPerformanceStatisticsEnrichedNet wten)
	{
		INetDataSource source = new PetriNetDataSource(pnet);
		
		Vector<TimeUnit> tus = new Vector<TimeUnit>();
		tus.add(TimeUnit.DAYS);
		tus.add(TimeUnit.HOURS);
		tus.add(TimeUnit.MINUTES);
		tus.add(TimeUnit.SECONDS);
		tus.add(TimeUnit.MILLISECONDS);

		System.out.print("Calculating " + outputLabel + "...");

		Date startTime = new Date();
		
		calculateHelper(source, provider, tus, wten);

		Date endTime = new Date();
		TimeSpan duration = new TimeSpan(startTime, endTime);

		System.out.println("" + duration.ToString(tus) + "");
	}

	private void calculateHelper(INetDataSource source, TraceReplayCollection provider,
			Vector<TimeUnit> tus, IPerformanceStatisticsEnrichedNet wten) 
	{
		for (SubNetDataSource esn : source.getSubnets())
		{
			calculateHelper(esn, provider, tus, wten);
		}
		
		for (Place p : source.getPlaces())
		{
			Collection<Arc> inEdges = source.getInEdges(p);
			Collection<Arc> outEdges = source.getOutEdges(p);
			
			if (inEdges.isEmpty() || outEdges.isEmpty())
				continue;
			
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> iEdge : inEdges)
			{
				Transition t1 = (Transition)iEdge.getSource();
				
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> oEdge : outEdges)
				{
					Transition t2 = (Transition)oEdge.getTarget();
					
					CalculateTransitionPairCohortStats(provider, tus, t1, t2, wten);			    
				}
			}
		}
	}

	private void CalculateTransitionPairCohortStats(TraceReplayCollection provider, Vector<TimeUnit> tus, Transition t1,
			Transition t2, IPerformanceStatisticsEnrichedNet wten) 
	{
		String localIDs = t1.getLocalID().toString() + t2.getLocalID().toString();
//		PSENTreeNode ptnl = new PSENTreeNode(myLabelNode);
//		WaitingTimeTree pst = psen.getTransitionPair(localIDs);
//		if (pst == null)
//		{
//			pst = new WaitingTimeTree(localIDs);
//			psen.addTree(pst);
//		}
		
		//String fulllabel = t1.getLabel() + " - " + t2.getLabel();
		getTimesFromLogReplay(provider, t1, t2, wten);

//		for (PSENLabel myLabel : myLabels)
//		{
//			pst.addLabelNode(myLabel, ptnl);
//		}
		
	}
}
