package org.processmining.processprofiler3d.psen.replayanalysis.stats;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Date;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.compviz.SubNetDataSource;
import org.processmining.processprofiler3d.psen.datastructure.IPerformanceStatisticsEnrichedNet;
import org.processmining.processprofiler3d.psen.datastructure.PSENLabel;
import org.processmining.processprofiler3d.psen.datastructure.PerformanceStatisticsDB;
import org.processmining.processprofiler3d.psen.preprocess.LPPAction;
import org.processmining.processprofiler3d.psen.replayanalysis.ITraceAlignment;
import org.processmining.processprofiler3d.psen.replayanalysis.TimeSpan;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayAnalyser;
import org.processmining.processprofiler3d.psen.replayanalysis.TraceReplayCollection;
import org.processmining.processprofiler3d.psen.replayanalysis.TransitionEvent;

public class StatCalculatorTransitionTimes extends StatCalculator
{
	private String outputLabel = "";
	
	protected Vector<PSENLabel> myLabels = new Vector<PSENLabel>();
	protected String myLabelNode = "";

	protected static Vector<TimeUnit> tus = null;
	
	public StatCalculatorTransitionTimes(String l)
	{
		outputLabel = l;
		
		//init timeSpan display format
		if (tus == null)
		{
			tus = new Vector<TimeUnit>();
			tus.add(TimeUnit.DAYS);
			tus.add(TimeUnit.HOURS);
			tus.add(TimeUnit.MINUTES);
			tus.add(TimeUnit.SECONDS);
			tus.add(TimeUnit.MILLISECONDS);
		}
	}

	@Override
	public void addLabelsToPSEN(IPerformanceStatisticsEnrichedNet psnet, IPerformanceStatisticsEnrichedNet wtt)
	{
		
	}
	
//	public TimeSpan getTimeForTransitionEvent(TransitionEvent te, XTrace tr)
//	{
//		return null;
//	}
	
	public TimeSpan getTimeForTransitionEvent(TraceReplayAnalyser tra, TransitionEvent te, int sId, int cId)
	{
		Date startTime = null;
		if (sId > -1)
		{
			startTime = tra.timestamps[sId];
		}
		else if (sId == -2) //HACK!!! used as signifier for trace start time
		{
			startTime = tra.startTime;
		}
		Date endTime = null;
		if (cId > -1)
		{
			endTime = tra.timestamps[cId];
		}
		
		if (startTime != null && endTime != null)
		{
			return new TimeSpan(startTime, endTime);
		}
		return null;
	}
	
	public int getStartId(TransitionEvent te)
	{
		return 0;
	}
	
	public int getEndId(TransitionEvent te)
	{
		return 0;
	}
	
	//trace alignment level
//	public void getTimesFromTraceAlignment(ITraceAlignment ta, Transition t, XTrace tr, IPerformanceStatisticsEnrichedNet psen)
//	{
//		String traceID = LPPAction.getAttributeValueLiteral(tr, XConceptExtension.KEY_NAME);
//		Vector<TransitionEvent> evts = ta.getTransitionEventsFor(t);
//		for (TransitionEvent te : evts)
//		{
//			TimeSpan ts = getTimeForTransitionEvent(te, tr);
////			if (ts.getMilliseconds() < 0)
////			{
////				System.out.println("Waiting time less than 0!");
////			}
//			if (ts != null)
//			{
//				psen.addValue(t.getLocalID().toString(), myLabels.firstElement().getLabelName(), traceID, ts.getMilliseconds());
//			}
//		}
//	}
	
	//trace alignment level
	public void getTimesFromTraceAlignmentBatch(ITraceAlignment ta, Transition t, Vector<TraceReplayAnalyser> tras, IPerformanceStatisticsEnrichedNet psen)
	{
		String tId = t.getLocalID().toString();
		String psenLabel = myLabels.firstElement().getLabelName();
		
		PerformanceStatisticsDB pdb = (PerformanceStatisticsDB)psen;
		PreparedStatement ps = pdb.getBatchAddFor(tId, psenLabel);
		
		Vector<TransitionEvent> evts = ta.getTransitionEventsFor(t);
		
		try
		{
			for (TransitionEvent te : evts)
			{
				int sid = getStartId(te);
				int cid = getEndId(te);
				
				for (TraceReplayAnalyser tra : tras)
				{
					XTrace tr = tra.getTrace();
					String traceID = LPPAction.getAttributeValueLiteral(tr, XConceptExtension.KEY_NAME);
					int trid = pdb.regTracePublic(traceID);
					//TimeSpan ts = getTimeForTransitionEvent(te, tr);
					TimeSpan ts = getTimeForTransitionEvent(tra, te, sid, cid);
		//			if (ts.getMilliseconds() < 0)
		//			{
		//				System.out.println("Waiting time less than 0!");
		//			}
					if (ts != null)
					{
						ps.setInt(3, trid);
						ps.setDouble(4, (double)ts.getMilliseconds());
						ps.addBatch();
//						psen.addValue(tId, psenLabel, traceID, ts.getMilliseconds());
					}
				}
			}
			pdb.commitBatchAdd();
		}
		catch (SQLException ex)
		{
			// TODO Auto-generated catch block
			ex.printStackTrace();
		}
	}
	
	//trace replay level
//	public void getTimesFromTraceReplay(TraceReplayAnalyser tra, Transition t, IPerformanceStatisticsEnrichedNet psen)
//	{
//		getTimesFromTraceAlignment(tra.getTraceAlignment(), t, tra.getTrace(), psen);
//	}
	
	//log replay level	
//	public void getTimesFromLogReplay(TraceReplayCollection lra, Transition t, IPerformanceStatisticsEnrichedNet psen)
//	{
//		for (TraceReplayAnalyser tra : lra.getValidTraceReplays())
//		{
//			getTimesFromTraceReplay(tra, t, psen);
//		}
//	}
	
	//log replay level	
	public void getTimesFromLogReplay(TraceReplayCollection lra, Transition t, IPerformanceStatisticsEnrichedNet psen)
	{
		for (ITraceAlignment ta : lra.getTraceAlignments())
		{
			getTimesFromTraceAlignmentBatch(ta, t, lra.getValidTraceReplaysFor(ta), psen);
		}
	}
	
	//provider level
	@Override
	public void calculate(PetrinetGraph pnet, TraceReplayCollection provider, IPerformanceStatisticsEnrichedNet psen, IPerformanceStatisticsEnrichedNet wten)
	{
		//initialize data structures
		INetDataSource source = new PetriNetDataSource(pnet);
		
		System.out.print("Calculating " + outputLabel + "...");
		
		Date startTime = new Date();
		
		CalculateHelper(provider, source, psen);

		Date endTime = new Date();
		TimeSpan duration = new TimeSpan(startTime, endTime);

		System.out.println("" + duration.ToString(tus) + "");
	}

	private void CalculateHelper(TraceReplayCollection provider, INetDataSource source, IPerformanceStatisticsEnrichedNet psen) 
	{
		for (SubNetDataSource esn : source.getSubnets())
		{
			CalculateHelper(provider, esn, psen);
		}
		
		for (Transition t : source.getTransitions())
		{	
			getTimesFromLogReplay(provider, t, psen);
		}
	}
}
