package org.processmining.processprofiler3d.psen.replayanalysis.tokenflow;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.processmining.models.graphbased.LocalNodeID;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.processprofiler3d.compviz.INetDataSource;
import org.processmining.processprofiler3d.compviz.PetriNetDataSource;
import org.processmining.processprofiler3d.compviz.SubNetDataSource;
import org.processmining.processprofiler3d.psen.PetriNet1toManyMapping;
import org.processmining.processprofiler3d.psen.replayanalysis.ITraceAlignment;
import org.processmining.processprofiler3d.psen.replayanalysis.MultiLevel;
import org.processmining.processprofiler3d.psen.replayanalysis.TransitionEvent;

public class TokenFlowResults 
{
	private MultiLevel mlcache = new MultiLevel();
	private Map<LocalNodeID, Vector<TransitionEvent>> tEvents = null;

	public TokenFlowResults()
	{
		tEvents = new HashMap<LocalNodeID, Vector<TransitionEvent>>();
	}
	
	public TokenFlowResults(Map<LocalNodeID, Vector<TransitionEvent>> tEvents)
	{
		this.tEvents = tEvents;
	}
	
	public void Init(final PetrinetGraph newNet)
	{
		//intialize transitionevent map
//		for (Transition t : newNet.getTransitions())
//		{
//			tEvents.put(t.getLocalID(), new Vector<TransitionEvent>());
//		}
		InitResultsHelper(new PetriNetDataSource(newNet));
	}
	
	private void InitResultsHelper(INetDataSource nds)
	{
		for (Transition t : nds.getTransitions())
		{
			//tEvents.put(t.getLocalID(), new Vector<TransitionEvent>());
			//don't add transitions that have a subnet, as they should go into multilevelcache instead!
			//if (!nds.hasSubnet(t))
			{
				tEvents.put(t.getLocalID(), new Vector<TransitionEvent>());
			}
		}
		
		for (SubNetDataSource snds : nds.getSubnets())
		{
			InitResultsHelper(snds);
		}
	}
	
	//ASSUMPTION: assuming at the moment that linked TransitionEvents will already be stored in the same order for each lifecycle transition!
	//should improve implementation to deal with other cases, or at least check and fail early
	public void ProjectTransition(ITraceAlignment ta, PetriNet1toManyMapping map, Transition t) 
	{
		Vector<TransitionEvent> newEvts = new Vector<TransitionEvent>();
		Vector<PetrinetNode> nodes = map.mapToView(t);
		int i = 0;
		for (PetrinetNode tn : nodes)
		{
			Vector<TransitionEvent> evts = ta.getTransitionEventsFor((Transition)tn);  //start event should have same local id in projected net
			
			int j = 0;
			for (TransitionEvent evt : evts)
			{
				//assign right events (incomplete!)
				//evtx.setAllEvents(evt.getStartId());
				if (i == 0) //schedule
				{
					TransitionEvent evtx = new TransitionEvent(t);
					evtx.setScheduleEvent(evt.getScheduleId());
					newEvts.add(evtx);
					
					evt.levelUp = evtx;
				}
				else if (i == 1) //assign
				{
					TransitionEvent evtx = newEvts.get(j);
					evtx.setAssignEvent(evt.getAssignId());
					
					evt.levelUp = evtx;
				}
				else if (i == 2) //start
				{
					TransitionEvent evtx = newEvts.get(j);
					evtx.setStartEvent(evt.getStartId());
					
					evt.levelUp = evtx;
				}
				else if (i == 3) //complete
				{
					TransitionEvent evtx = newEvts.get(j);
					evtx.setCompleteEvent(evt.getCompleteId());
					
					evt.levelUp = evtx;
				}
				j++;
			}
			i++;
		}
		tEvents.put(t.getLocalID(), newEvts);
	}
	
	public Vector<TransitionEvent> getTransitionEventsFor(Transition t)
	{
		Vector<TransitionEvent> evts = new Vector<TransitionEvent>();
//		if (tEvents.containsKey(t.getLocalID()))
//		{
//			evts = new Vector<TransitionEvent>(tEvents.get(t.getLocalID()));
//			//System.out.println("Test" + evts.size());
//		}
//		else if (mlcache.dataContains(t))
//		{
//			evts = mlcache.getTransitionEventsFromTraceAlignment(t);
//			//System.out.println("Test" + evts.size());
//		}
		if (tEvents.containsKey(t.getLocalID()))
		{
			evts.addAll(tEvents.get(t.getLocalID()));
			//System.out.println("Test" + evts.size());
		}
		if (mlcache.dataContains(t))
		{
			evts.addAll(mlcache.getTransitionEventsFromTraceAlignment(t));
			//System.out.println("Test" + evts.size());
		}
		
		return evts;
	}
	
	public void addSubnetEvents(INetDataSource source, Vector<SubNetDataSource> subnets) 
	{
		//check for missing parent connections! (there seem to be issues in the replay construction, but better fix late than never)
		ParentCheck(source);
		
		for (SubNetDataSource esn : subnets)
		{
			Transition tx = esn.getParent().getTransitionForSubnet(esn);
			
			addTransitionEventsFor(tx, esn);
		}
		
		//construct parent connections
		for (SubNetDataSource esn : subnets)
		{
			Transition tx = esn.getParent().getTransitionForSubnet(esn);
			for (Transition tp : esn.getParent().getPreceedingTransitions(tx))
			{
				addTransitionEventParents(tx, tp, esn);
			}
			
			//int lvl = getTransitionLevel(source, 0, tx);
			//System.out.println("Transition level: " + lvl);
		}
	}
	
	private void startTransHelper(INetDataSource inds, Vector<String> lst)
	{
		Place startPlace = inds.getStartPlace();
		for (Arc a : inds.getOutEdges(startPlace))
		{
			Transition t = (Transition)a.getTarget();
			lst.add(t.getLocalID().toString());
			
			if (inds.hasSubnet(t))
			{
				startTransHelper(inds.getSubnetForTransition(t), lst);
			}
		}
	}
	
	private void ParentCheck(INetDataSource source)
	{	
		Vector<String> excludedId = new Vector<String>();
		startTransHelper(source, excludedId);
		
		for (LocalNodeID lid : tEvents.keySet())
		{
			for (TransitionEvent te : tEvents.get(lid))
			{
				if (te.getParents().isEmpty())
				{
					Transition t = te.getTransition();
					boolean bShouldHavePreceeding = !excludedId.contains(t.getLocalID().toString());
					if (bShouldHavePreceeding)
					{
						System.out.println(te.getTransitionLabel() + " has no parent event!");
					}
				}
				//source.getStartPlace();
			}
		}
	}
	
	private void addTransitionEventsFor(Transition t, SubNetDataSource esn)
	{
		if (!mlcache.dataContains(t))
		{
			//Vector<Transition> transitions = new Vector<Transition>();
			Vector<TransitionEvent> evts = new Vector<TransitionEvent>();
			for (Transition childTrans : esn.getTransitions())
			{
					//transitions.add((Transition)cdge);
					evts.addAll(getTransitionEventsFor(childTrans));
			}
			mlcache.addTransitionEventsFromTraceAlignment(evts, t, esn);
		}
	}
	
	private void addTransitionEventParents(Transition tx, Transition tp, SubNetDataSource esn)
	{
		Vector<TransitionEvent> tes = mlcache.getTransitionEventsFromTraceAlignment(tx);
		//tes.sort(TransitionEvent.StartTimeComparatorAscending);	//Java 1.8
		Collections.sort(tes, TransitionEvent.StartTimeComparatorAscending);
		
		Vector<TransitionEvent> tpes = getTransitionEventsFor(tp);
		//tpes.sort(TransitionEvent.StartTimeComparatorAscending);	//Java 1.8
		Collections.sort(tpes, TransitionEvent.StartTimeComparatorAscending);
		
		while (tes.size() > 0)
		{
			TransitionEvent te = tes.remove(0);
			while (tpes.size() > 0)
			{
				TransitionEvent tpe = tpes.get(0);
				//can only be parent if did not start later than other transitionevent
				if (tpe.getStartId() <= te.getStartId())
				{
					te.addParent(tpe);
					tpes.remove(0);
				}
				else
				{
					break;
				}
			}
		}
	}
	
	public Vector<TransitionEvent> getAllTransitionEvents()
	{
		Vector<TransitionEvent> evts = new Vector<TransitionEvent>();
		for (Vector<TransitionEvent> v : tEvents.values())
		{
			evts.addAll(v);
		}
		
		evts.addAll(mlcache.getAllTransitionEvents());
		return evts;
	}
}
