package org.processmining.processprofiler3d.resourceanalysis.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;

import org.processmining.processprofiler3d.resourceanalysis.statics.AdvancedStatisticsProvider;
import org.processmining.processprofiler3d.resourceanalysis.statics.BasicLogStatisticsProvider;
import org.processmining.processprofiler3d.resourceanalysis.statics.FrequencyTree;

public class DisplayPanel extends JPanel {
	
	//JPanel jpCaseAnalysis;
	private JScrollPane jspChart;

	
	private JButton jbUpdate = new JButton("Update Chart");
	private JComboBox<String> jcbSort;
	private JLabel jlCountSlider;
	private JList<String> lsItems;

	
	private AdvancedStatisticsProvider myAdvancedProvider;
	private BasicLogStatisticsProvider myBasicProvider;

	private FrequencyTree myFrequencyTree;
	
	/**
	 * Create the panel.
	 */
	public DisplayPanel(BasicLogStatisticsProvider basicProvider) {
		
		this.myBasicProvider = basicProvider;
		this.myAdvancedProvider = new AdvancedStatisticsProvider(basicProvider);
		this.myFrequencyTree = myAdvancedProvider.getFrequencyTree();
		
		//sort by
		jcbSort = new JComboBox<String>(new String[] {"Case No", "Case Time Shift ASC", "Case Time Shift DESC"});
		jcbSort.setPreferredSize(new Dimension(170, 25));
		
		//case count slider
		jlCountSlider = new JLabel("Case Count : ");
		jlCountSlider.setText("Case Count : " + 10);
	
		//list filter
		ArrayList<String> list = basicProvider.getListOfActivities();
		String[] strFilterList = new String[list.size()];
		strFilterList = list.toArray(strFilterList);
		
		lsItems = new JList<String>(strFilterList);
		lsItems.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		lsItems.setLayoutOrientation(JList.VERTICAL);
		lsItems.addSelectionInterval(0, lsItems.getModel().getSize()-1);
		lsItems.setPreferredSize(new Dimension(125, strFilterList.length * 18));
		
		JScrollPane jspJListScroller = new JScrollPane(lsItems);
		jspJListScroller.setPreferredSize(new Dimension(125, 175));
		//putting together the option panel
		JPanel jpOptionListPanel = new JPanel(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();
        
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        c.gridy = 0;
        jpOptionListPanel.add(new JLabel("Resources :"), c);
        
        c.gridy = 1;
        jpOptionListPanel.add(jcbSort, c);
        
        c.gridy = 2;
        jpOptionListPanel.add(new JLabel(" "), c);
        
    	c.gridy = 3;
        jpOptionListPanel.add(jlCountSlider, c);
 
        c.gridy = 8;
        jpOptionListPanel.add(new JLabel(" "), c);
        
        c.gridy = 9;
        jpOptionListPanel.add(new JLabel("List Tasks as :"), c);
        
        c.gridy = 10;
        jpOptionListPanel.add(jspJListScroller, c);
        
        c.gridy = 14;
        jpOptionListPanel.add(new JLabel(" "), c);
        
        c.gridy = 15;
        jpOptionListPanel.add(jbUpdate, c);
        
        JPanel jpOptionPanel = new JPanel();
        jpOptionPanel.setLayout(new FlowLayout());
        jpOptionPanel.add(jpOptionListPanel);
        jpOptionPanel.setPreferredSize(new Dimension(180, 250));
        
        Vector<String> mylist = new Vector<String>();
       
        for (String str : this.myBasicProvider.getListOfActivities()){
        	mylist.add(str);
        }
        
        DefaultTableModel model = new DefaultTableModel(mylist, this.myBasicProvider.getListOfResources().size());
        
        for (int i = 0; i < model.getRowCount(); i++){
        	for (int j = 0; j < model.getColumnCount(); j++){
        		String myText= "";
        		String myActivity = this.myAdvancedProvider.getListOfActivities().get(j);
        		String myResource = this.myAdvancedProvider.getListOfResources().get(i);
        		for (String str : this.myAdvancedProvider.getListOfCohorts()){
        			myText = myText+str+":"+this.myFrequencyTree.getValue(myActivity, myResource, str)+" \n";
        		}
        		model.setValueAt(myText, i, j);
        	}
        }
        
        JTable mainTable = new JTable(model);  
		jspChart = new JScrollPane(mainTable);		
		JTable rowTable = new HeaderTable(mainTable, this.myBasicProvider.getListOfResources());
		rowTable.setFillsViewportHeight(true);
        jspChart.setRowHeaderView(rowTable);
        jspChart.setCorner(JScrollPane.UPPER_LEFT_CORNER, rowTable.getTableHeader());
		jspChart.setPreferredSize(new Dimension(1250, 250));
		this.setLayout(new BorderLayout());
		this.add(jpOptionPanel, BorderLayout.WEST);
		this.add(jspChart, BorderLayout.CENTER);

	}

}
