package org.processmining.processprofiler3d.resourceanalysis.interactions;

import java.util.ArrayList;
import java.util.Vector;

import org.processmining.processprofiler3d.compviz.jogl.FontFormat;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiHistogram;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiHistogramMatrix;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiHistogramStyleResourceCohorts;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiLabel;
import org.processmining.processprofiler3d.compviz.jogl.gui.JoglGuiStyle;
import org.processmining.processprofiler3d.resourceanalysis.statics.FrequencyTree;
import org.processmining.visualisation3d.IMouseProcessor;
import org.processmining.visualisation3d.JoglInteraction;
import org.processmining.visualisation3d.gui.JoglGuiObject;
import org.processmining.visualisation3d.input.JoglInputMouseEvent;
import org.processmining.visualisation3d.maths.JoglColor3;
import org.processmining.visualisation3d.maths.JoglVectord2;
import org.processmining.visualisation3d.maths.JoglVectord4;
import org.processmining.visualisation3d.scene.JSOGraphicalUserInterface;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

public class JIResourceVis extends JoglInteraction
{
	private IMouseProcessor iman;
	private JoglScene scene;
	private JSOGraphicalUserInterface gui;
	private FrequencyTree freqTree;
	
	private boolean bVisible = false;

	//private JoglGuiLabel lblCamMode;

	private JoglGuiLabel highlighted = null;
	
	private Vector<JoglGuiObject> controls = new Vector<JoglGuiObject>();
	//private Vector<JoglGuiHistogram> visHist = new Vector<JoglGuiHistogram>();
	
	private JoglGuiLabel[] resourceLabels;
	private JoglGuiLabel[] activityLabels;
	private JoglGuiHistogram[][] histMap;
	private JoglGuiHistogramMatrix histMat = null;
	
	private float cellSizeX = 0.2f;
	private float cellSizeY = 0.2f;
	
	private JoglVectord2 viewOffset = JoglVectord2.ZERO;
	
	public JIResourceVis(IMouseProcessor iman, JoglScene scene, JSOGraphicalUserInterface gui, FrequencyTree ftree)
	{
		this.iman = iman;
		this.scene = scene;
		this.gui = gui;
		this.freqTree = ftree;
	}
	
	public boolean isVisible()
	{
		return bVisible;
	}
	
	public void Show()
	{
		float aspect = (float)gui.getAspect();
		
		FontFormat ff = new FontFormat();
		ff.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff.align = FontFormat.ALIGN_LEFT;
		ff.setColor(1.0f, 1.0f, 1.0f);
		
		FontFormat ff2 = new FontFormat();
		ff2.width = (0.03f / aspect) * (float)JoglGuiStyle.textScale;
		ff2.height = 0.03f * (float)JoglGuiStyle.textScale;
		ff2.align = FontFormat.ALIGN_RIGHT;
		ff2.setColor(1.0f, 1.0f, 1.0f);
		
		float textPadding = 0.005f;
		float yoff = 0.85f;
		float xoff = -0.8f;

		ArrayList<String> myActivities = freqTree.getActivities();
		ArrayList<String> myResources = freqTree.getResources();	
		ArrayList<String> myCohorts = freqTree.getCohorts();	

		activityLabels = new JoglGuiLabel[myActivities.size()];
		resourceLabels = new JoglGuiLabel[myResources.size()];
		
		int i = 0;
		for (String s : myResources)
		{
			JoglGuiLabel lbl = new JoglGuiLabel(ff2.clone());
			lbl.setRect(-1.0f, yoff, cellSizeX, 0.05f);
			lbl.setPadding(textPadding);
			lbl.setText(s);
			lbl.setBackColor(JoglGuiStyle.transparentColor);
			
			//gui.AddGuiObject(lbl);
			resourceLabels[i] = lbl;
			//controls.add(lbl);
			
			yoff -= cellSizeY;
			i++;
		}
		
		i = 0;
		for (String s : myActivities)
		{
			JoglGuiLabel lbl = new JoglGuiLabel(ff.clone());
			lbl.setRect(xoff, 0.95f, cellSizeX, 0.05f);
			lbl.setPadding(textPadding);
			lbl.setText(s);
			lbl.setBackColor(JoglGuiStyle.transparentColor);
			
			//gui.AddGuiObject(lbl);
			activityLabels[i] = lbl;
			//controls.add(lbl);
			
			xoff += cellSizeX;
			i++;
		}
		
		int maxFreq = 0;
		for (String sa : myActivities)
		{
			for (String sr : myResources)
			{
				for (String sc : myCohorts)
				{
					int freq = freqTree.getValue(sa, sr, sc);
					maxFreq = Math.max(maxFreq, freq);
				}
			}
		}
		maxFreq++;
		
		histMap = new JoglGuiHistogram[myActivities.size()][myResources.size()];
		
		yoff = 0.75f;
		xoff = -0.8f;
		
		int aId = 0;

		Vector<JoglGuiHistogram> lst = new Vector<JoglGuiHistogram>();
		for (String sa : myActivities)
		{
			yoff = 0.75f;

			int rId = 0;
			for (String sr : myResources)
			{
				JoglGuiHistogram ghist = new JoglGuiHistogram(gui);
				ghist.setColor(JoglColor3.RED);
				ghist.setStyle(new JoglGuiHistogramStyleResourceCohorts());
				ghist.setRect(xoff, yoff, cellSizeX, cellSizeY);
				//ghist.setRect(getCellRect(aId, rId));

				Vector<Integer> hist = new Vector<Integer>();
				hist.add(0);
//				hist.add(1);
//				hist.add(2);
//				hist.add(3);
//				hist.add(4);
//				hist.add(5);
				
				for (String sc : myCohorts)
				{
					int freq = freqTree.getValue(sa, sr, sc);
					hist.add(freq);
				}
				
				hist.add(0);
				
				ghist.setHistogram(hist, maxFreq);
				//ghist.setHistogram(hist, 6);
				
				//gui.AddGuiObject(ghist);
				//controls.add(ghist);
				lst.add(ghist);
				histMap[aId][rId] = ghist;

				yoff -= cellSizeY;
				rId++;
			}
			
			xoff += cellSizeX;
			aId++;
		}
		
		histMat = new JoglGuiHistogramMatrix(gui);
		histMat.setHistograms(lst);
		gui.AddGuiObject(histMat);
		
		bVisible = true;
	}
	
	public JoglVectord4 getCellRect(int x, int y)
	{
		float yoff = 0.75f - (float)viewOffset.getY();
		float xoff = -0.8f + (float)viewOffset.getX();

		return new JoglVectord4(xoff + (x * cellSizeX), yoff - (y * cellSizeY), cellSizeX, cellSizeY);
	}
	
	public void Hide()
	{
		for (JoglGuiObject jgo : controls)
		{
			gui.RemoveGuiObject(jgo);
		}
		
		bVisible = false;
	}
	
	private boolean bUpdateScene = true;
	
	@Override
	public boolean ShouldAnimate()
	{
		return bUpdateScene;
	}
	
	@Override
	public void UpdateScene(JoglScene scene)
	{
		if (bUpdateScene)
		{
			if (!bVisible)
			{
				Show();
			}
			
			determineVisible();
		}
		
		bUpdateScene = false;
	}
	
	private boolean bMouseDown = false;
	private JoglVectord2 lastMousePos;
	
	//MouseListener method implementation
	@Override
    public boolean mousePressed(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		
		bMouseDown = true;
		//lastMousePos = new JoglVectord2((2.0 * (e.getX() / gui.getWidth())) - 1.0, (2.0 * (e.getY() / gui.getHeight())) - 1.0);
		lastMousePos = new JoglVectord2((2.0 * e.getX()) - 1.0, (2.0 * e.getY()) - 1.0);
		
		return false;
    }
	
	@Override
    public boolean mouseReleased(JoglInputMouseEvent e) 
    {
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
			if (controls.contains(jgo))
			{
				return true;
			}
		}
		
		bMouseDown = false;
		return false;
    }
	
	@Override
    public boolean mouseClicked(JoglInputMouseEvent e) 
    {		
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
//			if (jgo == btnCamModeToggle)
//			{
//				//viewManager.SwitchCameraMode();
//	
//	        	bUpdateScene = true;
//	        	return true;
//			}
		}
		
		return false;
    }
	
	@Override
	public void mouseMoved(JoglInputMouseEvent e)
	{
		JoglGuiObject jgo = gui.GetObjectFromPos(e.getX(), e.getY());
		if (jgo != null)
		{
//			if (jgo == btnCamModeToggle)
//			{
//				highlight(btnCamModeToggle);
//			}
//			else
//			{
//				unHighlight();
//			}
		}
		else
		{
			unHighlight();
		}
	}
	
	@Override
	public void mouseDragged(JoglInputMouseEvent e)
	{
		if (bMouseDown)
		{
			//JoglVectord2 newMousePos = new JoglVectord2((2.0 * (e.getX() / gui.getWidth())) - 1.0, (2.0 * (e.getY() / gui.getHeight())) - 1.0);
			JoglVectord2 newMousePos = new JoglVectord2((2.0 * e.getX()) - 1.0, (2.0 * e.getY()) - 1.0);
			JoglVectord2 diff = JoglVectord2.Subtract(newMousePos, lastMousePos);
			viewOffset = JoglVectord2.Add(viewOffset, diff);
			
			moveView(diff.getX(), -diff.getY());
			
			lastMousePos = newMousePos;
			
			bUpdateScene = true;
		}
	}
	
	public void moveView(double dx, double dy)
	{
		histMat.off = viewOffset;
		//System.out.println("Move view: " + dx + "/" + dy);
		
//		for (JoglGuiObject jgo : controls)
//		{
//			if (jgo instanceof JoglGuiLabel)
//			{
//				JoglGuiLabel lbl = (JoglGuiLabel)jgo;
//				
//				JoglVectord4 rect = lbl.getRect();
//				lbl.setRect(JoglVectord4.Add(rect, new JoglVectord4(dx, dy, 0, 0)));
//			}
//			else if (jgo instanceof JoglGuiHistogram)
//			{
////				JoglGuiHistogram lbl = (JoglGuiHistogram)jgo;
////				
////				JoglVectord4 rect = lbl.getRect();
////				lbl.setRect(JoglVectord4.Add(rect, new JoglVectord4(dx, dy, 0, 0)));
//			}
//		}
	}
	
//	private void determineVisible()
//	{
//		//hide old gui elements
//		for (JoglGuiObject jhist : controls)
//		{
//			gui.RemoveGuiObject(jhist);
//		}
//		
//		ArrayList<String> myActivities = freqTree.getActivities();
//		ArrayList<String> myResources = freqTree.getResources();
//		
//		boolean bUpdateActivityLabels = false;
//		
//		float yoff = 0.75f - (float)viewOffset.getY();
//		float xoff = -0.8f + (float)viewOffset.getX();
//		
//		//update positions of the cells that are visible only
//		for (int y = 0; y < myResources.size(); y++)
//		{
//			if (yoff + cellSizeY >= -1.0f && yoff <= 1.0f)
//			{
//				JoglGuiLabel resLabel = resourceLabels[y];
//				resLabel.setRect(-1.0f, yoff, cellSizeX, 0.05f);
//				controls.add(resLabel);
//				gui.AddGuiObject(resLabel);
//				
//				xoff = -0.8f + (float)viewOffset.getX();
//				
//				for (int x = 0; x < myActivities.size(); x++)
//				{
//					if (xoff + cellSizeX >= -1.0f && xoff <= 1.0f)
//					{
//						//only update activity labels once
//						if (!bUpdateActivityLabels)
//						{
//							JoglGuiLabel actLabel = activityLabels[x];
//							actLabel.setRect(xoff, 0.95f, cellSizeX, 0.05f);
//							controls.add(actLabel);
//							gui.AddGuiObject(actLabel);
//						}
//						
//						JoglGuiHistogram lbl = histMap[x][y];
//						lbl.setStyle(new JoglGuiHistogramStyleResourceCohorts());
//						lbl.setRect(xoff, yoff, cellSizeX, cellSizeY);
//						
//						controls.add(lbl);
//						gui.AddGuiObject(lbl);
//					}
//					
//					xoff += cellSizeX;
//				}
//
//				//only update activity labels once
//				bUpdateActivityLabels = true;
//			}
//			
//			yoff -= cellSizeY;
//		}
//	}
	
	private void determineVisible()
	{
		//hide old gui elements
		for (JoglGuiObject jhist : controls)
		{
			gui.RemoveGuiObject(jhist);
		}
		
		ArrayList<String> myActivities = freqTree.getActivities();
		ArrayList<String> myResources = freqTree.getResources();
		Vector<JoglGuiHistogram> lst = new Vector<JoglGuiHistogram>();
		
		boolean bUpdateActivityLabels = false;
		
		float yoff = 0.75f - (float)viewOffset.getY();
		float xoff = -0.8f + (float)viewOffset.getX();
		
		//update positions of the cells that are visible only
		for (int y = 0; y < myResources.size(); y++)
		{
			if (yoff + cellSizeY >= -1.0f && yoff <= 1.0f)
			{
				JoglGuiLabel resLabel = resourceLabels[y];
				resLabel.setRect(-1.0f, yoff, cellSizeX, 0.05f);
				controls.add(resLabel);
				gui.AddGuiObject(resLabel);
				
				xoff = -0.8f + (float)viewOffset.getX();
				
				for (int x = 0; x < myActivities.size(); x++)
				{
					if (xoff + cellSizeX >= -1.0f && xoff <= 1.0f)
					{
						//only update activity labels once
						if (!bUpdateActivityLabels)
						{
							JoglGuiLabel actLabel = activityLabels[x];
							actLabel.setRect(xoff, 0.95f, cellSizeX, 0.05f);
							controls.add(actLabel);
							gui.AddGuiObject(actLabel);
						}
						
						JoglGuiHistogram lbl = histMap[x][y];
						//lbl.setRect(xoff, yoff, cellSizeX, cellSizeY);
						
						lst.add(lbl);
						//controls.add(lbl);
						//gui.AddGuiObject(lbl);
					}
					
					xoff += cellSizeX;
				}

				//only update activity labels once
				bUpdateActivityLabels = true;
			}
			
			yoff -= cellSizeY;
		}
		
		histMat.updateVisibleHistograms(lst);
	}
	
	private void highlight(JoglGuiLabel label)
	{
		unHighlight();
	
		label.setBackColor(JoglGuiStyle.highlightColor);
		highlighted = label;
		
		bUpdateScene = true;
	}

	private void unHighlight() 
	{
		if (highlighted != null)
		{
			highlighted.setBackColor(JoglGuiStyle.backColor);
			highlighted = null;

			bUpdateScene = true;
		}
	}
}
