package org.processmining.processprofiler3d.resourceanalysis.plugins;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.processprofiler3d.compviz.PP3DAppResourceVisualiser2;
import org.processmining.processprofiler3d.compviz.scenemanagement.PP3DApplicationInitializer;
import org.processmining.processprofiler3d.resourceanalysis.statics.FrequencyTree;
import org.processmining.visualisation3d.JoglPanel;
import org.processmining.visualisation3d.graphicsdatastructures.JoglCamera;
import org.processmining.visualisation3d.scenemanagement.JoglScene;

@Plugin(name = "Process Profiler 3D: Mining Resource Patterns from Event Log v2", 
		parameterLabels = { "Frequency Tree" }, 
		returnLabels = { "3D Resource Wheel Visualisation" }, 
		returnTypes = { JComponent.class }, 
		userAccessible = true, 
		help = "This plug-in mines the resource behaviour patterns from an event log.")

@Visualizer
public class ResourceAnalysisVisualiserNew2 
{

	@UITopiaVariant(
			affiliation = "Queensland University of Technology", 
			author = "Erik Poppe", 
			email = "erik.poppe@qut.edu.au"
		)
    @PluginVariant(requiredParameterLabels = { 0 })
    public JComponent Visualizer(final UIPluginContext context, FrequencyTree result) 
	{
		PP3DAppResourceVisualiser2 app = new PP3DAppResourceVisualiser2(result);
		JoglScene scene = new JoglScene();
		
		JoglPanel panel = new JoglPanel();
		panel.setScene(scene);

		JoglCamera cam = app.InitCamera(panel);
		panel.setCamera(cam);
		
		
		panel.getInteractionManager().addInteraction(new PP3DApplicationInitializer(panel, app));
		
		return panel;
    }
}
