package org.processmining.processprofiler3d.resourceanalysis.statics;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;

public class BasicLogStatisticsProvider
{

	private XLog mylog;
	private HashSet<String> mylistOfActivities = new HashSet<String>();
	private HashSet<String> mylistOfResources = new HashSet<String>();
	private HashSet<String> mylistOfLifeCycleTransitions = new HashSet<String>();
	private HashSet<String> mylistOfAttributes = new HashSet<String>();
	private HashSet<String> mylistOfCohorts = new HashSet<String>();


	public ArrayList<String> getListOfActivities()
	{
		return new ArrayList<String>(this.mylistOfActivities);
	}

	public ArrayList<String> getListOfResources()
	{
		return new ArrayList<String>(this.mylistOfResources);
	}

	public ArrayList<String> getListOfLifecycleTransitions()
	{
		return new ArrayList<String>(this.mylistOfLifeCycleTransitions);
	}

	public ArrayList<String> getListOfAttributes()
	{
		return new ArrayList<String>(this.mylistOfAttributes);
	}

	public ArrayList<String> getListOfCohorts()
	{
		return new ArrayList<String>(this.mylistOfCohorts);
	}

	public XLog getEventLog()
	{
		return this.mylog;
	}
	
	public BasicLogStatisticsProvider(XLog log)
	{
		this.mylog = log;
		this.generateListOfActivities();
		this.generateListOfResources();
		this.generateListOfLifeCycleTransitions();
		this.generateListOfEventAttributes();
		this.generateListOfCohorts();
	}

	public void generateListOfCohorts()
	{
		for (XTrace tr : mylog)
		{
			XAttributeLiteral myAttribute = (XAttributeLiteral) tr.getAttributes().get("Cohort");
			if (myAttribute == null || myAttribute.getValue() == null)
			{
				continue;
			} 
			else if (myAttribute.getValue() instanceof String)
			{
				if (!this.mylistOfCohorts.contains(myAttribute.getValue()))
				{
					this.mylistOfCohorts.add(myAttribute.getValue());
				}
			} 
			else
			{
				String temp = myAttribute.getValue().toString();
				if (!this.mylistOfCohorts.contains(temp))
				{
					this.mylistOfCohorts.add(temp);
				}
			}
		}
	}

	public void generateListOfLifeCycleTransitions()
	{
		for (XTrace tr : mylog) // in the event log, each element is a case
		{
			for (XEvent myEvent : tr) // in each case, each element is an event
			{
				XAttributeLiteral myAttribute = (XAttributeLiteral) myEvent.getAttributes().get(XLifecycleExtension.KEY_TRANSITION);
				if (myAttribute == null || myAttribute.getValue() == null)
				{
					continue;
				} 
				else if (myAttribute.getValue() instanceof String)
				{
					if (!this.mylistOfLifeCycleTransitions.contains(myAttribute.getValue()))
					{
						this.mylistOfLifeCycleTransitions.add(myAttribute.getValue());
					}
				} 
				else
				{
					String temp = myAttribute.getValue().toString();
					if (!this.mylistOfLifeCycleTransitions.contains(temp))
					{
						this.mylistOfLifeCycleTransitions.add(temp);
					}
				}
			}
		}
	}

	public void generateListOfActivities()
	{
		for (XTrace tr : mylog) // in the event log, each element is a case
		{
			for (XEvent myEvent : tr) // in each case, each element is an event
			{
				XAttributeLiteral myAttribute = (XAttributeLiteral) myEvent.getAttributes().get(XConceptExtension.KEY_NAME);
				if (myAttribute == null || myAttribute.getValue() == null)
				{
					continue;
				} 
				else if (myAttribute.getValue() instanceof String)
				{
					if (!mylistOfActivities.contains(myAttribute.getValue()))
					{
						mylistOfActivities.add(myAttribute.getValue());
					}
				} 
				else
				{
					String temp = myAttribute.getValue().toString();
					if (!mylistOfActivities.contains(temp))
					{
						mylistOfActivities.add(temp);
					}
				}
			}
		}
	}

	public void generateListOfResources()
	{
		for (XTrace tr : mylog) // in the event log, each element is a case
		{
			for (XEvent myEvent : tr) // in each case, each element is an event
			{
				XAttributeLiteral myAttribute = (XAttributeLiteral) myEvent.getAttributes().get("org:resource");
				if (myAttribute == null || myAttribute.getValue() == null)
				{
					continue;
				} 
				else if (myAttribute.getValue() instanceof String)
				{
					if (!mylistOfResources.contains(myAttribute.getValue()))
					{
						mylistOfResources.add(myAttribute.getValue());
					}
				} 
				else
				{
					String temp = myAttribute.getValue().toString();
					if (!mylistOfResources.contains(temp))
					{
						mylistOfResources.add(temp);
					}
				}
			}
		}
	}

	public void generateListOfEventAttributes()
	{
		for (XTrace tr : mylog) // in the event log, each element is a case
		{
			for (XEvent myEvent : tr) // in each case, each element is an event
			{
				Set<String> myAttribute = myEvent.getAttributes().keySet();
				for (String str : myAttribute)
				{
					if (!this.mylistOfAttributes.contains(str))
					{
						this.mylistOfAttributes.add(str);
					}
				}
			}
		}
	}
}
