package org.processmining.processprofiler3d.resourceanalysis.statics;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension.StandardModel;
import org.deckfour.xes.extension.std.XOrganizationalExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.processprofiler3d.psen.datastructure.H2MemoryDatabase;


public class LogToDatabase
{

	private Connection con;
	private XLog mylog;

	public LogToDatabase(H2MemoryDatabase h2db, XLog log)
	{
		con = h2db.getConnection();
		this.mylog = log;
	}

	public void Init()
	{
		try
		{
			//create PSEN table in database
			Statement stmt = con.createStatement();
			String sqlstatement = "Create Table dataset " + "(RowID int not null auto_increment,"
					+ "CaseID varchar(255)," + "Activity varchar(255)," + "Resource varchar(255),"
					+ "BeginTime Datetime," + "EndTime Datetime," + "Duration float," + "Cohort varchar(255),"
					+ "primary key (rowID))";
			stmt.executeUpdate(sqlstatement);
			stmt.close();

		} 
		catch (SQLException se)
		{
			System.out.println(se.getMessage());
			se.printStackTrace();
		} 
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	public void updatetable()
	{
		try
		{
			//create PSEN table in database
			Statement stmt = con.createStatement();

			for (XTrace tr : mylog) // in the event log, each element is a case
			{
				String caseId = "";
				String cohortId = "";
				
				caseId = XConceptExtension.instance().extractName(tr);
				
				XAttribute myCohortAttribute = tr.getAttributes().get("Cohort");
				if (myCohortAttribute != null && myCohortAttribute instanceof XAttributeLiteral)
				{
					XAttributeLiteral myCohortAttributeL = (XAttributeLiteral)myCohortAttribute;
					cohortId = myCohortAttributeL.getValue();
				}
				
				for (XEvent myEvent : tr) // in each case, each element is an event
				{
					String activityName = "";
					String resourceName = "";
					
					StandardModel lifecycle = XLifecycleExtension.instance().extractStandardTransition(myEvent);
					if (lifecycle != StandardModel.COMPLETE)
					{
						continue;
					}
					
					
					activityName = XConceptExtension.instance().extractName(myEvent);
					if (activityName != null && activityName.contains("'"))
					{
						activityName = activityName.replace("'", "");
					}
					
					resourceName = XOrganizationalExtension.instance().extractResource(myEvent);

					String sqlstatement = "INSERT INTO dataset(CaseID, Activity, Resource, Cohort) " + "VALUES(" + "'"
							+ caseId + "'" + "," + "'" + activityName + "'" + ","
							+ "'" + resourceName + "'" + "," + "'" + cohortId
							+ "'" + ")";
					
					stmt.executeUpdate(sqlstatement);
				}
			}

			stmt.close();

		} 
		catch (SQLException se)
		{
			System.out.println(se.getMessage());
			se.printStackTrace();
		} 
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	public void printquery()
	{
		try
		{
			//create PSEN table in database
			Statement stmt = con.createStatement();
			String query = "Select Activity, Resource, Cohort, count(*) as Frequency from dataset group by Activity, Resource, Cohort";
			ResultSet rs = stmt.executeQuery(query);
			while (rs.next())
			{
				System.out.println(
						rs.getString(1) + "  " + rs.getString(2) + "  " + rs.getString(3) + "  " + rs.getInt(4));
			}
			stmt.close();
		} 
		catch (SQLException se)
		{
			System.out.println(se.getMessage());
			se.printStackTrace();
		} 
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	public FrequencyTree setFrequency(FrequencyTree mytree)
	{
		try
		{
			//create PSEN table in database
			FrequencyTree theTree = mytree;
			Statement stmt = con.createStatement();
			String query = "Select Activity, Resource, Cohort, count(*) as Frequency from dataset group by Activity, Resource, Cohort";
			ResultSet rs = stmt.executeQuery(query);
			while (rs.next())
			{
				theTree.setValue(rs.getString(1), rs.getString(2), rs.getString(3), rs.getInt(4));
			}
			stmt.close();
			return theTree;
		} 
		catch (SQLException se)
		{
			System.out.println(se.getMessage());
			se.printStackTrace();
			return null;
		} 
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
			return null;
		}
	}
	

}
